/***************************************************************************
    file	         : kb_csv.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qdir.h>
#include	<qfileinfo.h>


#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_databuffer.h"
#include	"kb_database.h"
#include	"kb_serverinfo.h"

#include	"kb_libloader.h"

#include	"kb_basequery.h"
#include	"kb_copybase.h"
#include	"kb_copyfile.h"

#include	"std.h"
#include	"eli.h"
#include	"interp.h"
#include	"syn.h"
#include	"code.h"


#define		__DEBUG_CSV	(0)
#if		__DEBUG_CSV
#include	"kb_dprintf.h"
#endif

#include	"kb_csvadvanced.h"

/*  KBCSVType								*/
/*  -----------								*/

class	KBCSVType : public KBType
{
public	:

	KBCSVType			() ;

	virtual	bool    isValid      	(const QString &, KBError  &, const QString & = QString::null) ;
	virtual	void	getQueryText 	(KBDataArray   *, KBShared *, KBDataBuffer  &) ;
}	;



/*  KBCSV								*/
/*  -------								*/
/*  Implementation class for insertface to the CSV database		*/

class	KBCSV : public KBServer
{
	QString		m_delimiter		;
	QString		m_qualifier		;
	bool		m_hasHeaders		;

	bool			doListTables 	(KBTableDetailsList &, uint) ;

	virtual	bool	 	doConnect    	(KBServerInfo  *) ;
	virtual	bool		doListFields 	(KBTableSpec   &) ;
	virtual	bool		doCreateTable	(KBTableSpec   &,  bool, bool = false) ;
	virtual	bool		doRenameTable	(cchar *, cchar *, bool) ;
	virtual	bool		doDropTable  	(cchar *,	   bool) ;

public	:

	KBCSV	() ;
virtual~KBCSV	() ;


	virtual	KBSQLSelect	*qrySelect 	(bool, const QString &, bool) ;
	virtual	KBSQLUpdate	*qryUpdate 	(bool, const QString &, const QString &) ;
	virtual	KBSQLInsert	*qryInsert 	(bool, const QString &, const QString &) ;
	virtual	KBSQLDelete	*qryDelete 	(bool, const QString &, const QString &) ;

	virtual	bool		command		(bool, const QString &, uint, KBValue *, KBSQLSelect ** = 0) ;


	virtual	QString		listTypes	() ;
	virtual	uint		optionFlags	() ;
	virtual	bool		tableExists	(const QString &, bool &) ;

	const QString		&database	()
	{
		return	m_database ;
	}

	friend	class	KBCSVQrySelect	;
}	;


/*  KBCSVQrySelect							*/
/*  ----------------							*/
/*  Implementation class for select queries on the CSV database		*/

class	KBCSVQrySelect : public KBSQLSelect
{
	KBCSV		*m_server	;
	QString		m_table		;
	QStringList	m_fields	;
	uint		m_sequence	;


public	:
	KBCSVQrySelect (KBCSV *, bool, const QString &) ;
virtual~KBCSVQrySelect () ;

	virtual	bool	execute		(uint, const KBValue * ) ;
	virtual	KBValue	getField	(uint, uint, KBValue::VTrans) ;
	virtual	QString	getFieldName	(uint) ;
}	;


/*  KBCSVQryUpdate							*/
/*  ----------------							*/
/*  Implementation class for update queries on the CSV database		*/

class	KBCSVQryUpdate : public KBSQLUpdate
{
	KBCSV	*m_server	;

public	:
	KBCSVQryUpdate (KBCSV *, bool, const QString &, const QString &) ;
virtual~KBCSVQryUpdate () ;

	virtual	bool	execute	 (uint, const KBValue * ) ;

}	;

/*  KBCSVQryInsert							*/
/*  ----------------							*/
/*  Implementation class for insert queries on the CSV database		*/

class	KBCSVQryInsert : public KBSQLInsert
{
	KBCSV		*m_server	;
	QString		m_autocol	;
	KBValue		m_newKey	;

public	:
	KBCSVQryInsert (KBCSV *, bool, const QString &, const QString &) ;
virtual~KBCSVQryInsert () ;

	virtual	bool	execute	  (uint, const KBValue *) ;
	virtual	bool	getNewKey (const QString &, KBValue &, bool) ;
}	;

/*  KBCSVQryDelete							*/
/*  ----------------							*/
/*  Implementation class for delete queries on the CSV database		*/

class	KBCSVQryDelete : public KBSQLDelete
{
	KBCSV		*m_server	;

public	:
	KBCSVQryDelete (KBCSV *, bool, const QString &, const QString &) ;
virtual~KBCSVQryDelete () ;

	virtual	bool	execute	 (uint, const KBValue * ) ;
}	;


/*  ------------------------------------------------------------------  */

QString	paramSub
	(	const QString		&value,
		const QDict<QString>	&paramDict
	)
{
	if (value.isNull() || (value.find ("${") < 0))
		return value ;

	QString	res	= "" 	;
	int	pos1	= 0	;
	int	pos2	;

	while ((pos2 = value.find ("${", pos1)) >= 0)
	{
		res	+= value.mid (pos1, pos2 - pos1) ;
		pos1	 = pos2 + 2 ;

		if ((pos2 = value.find ("}", pos1)) < 0)
		{	res	+= "${" ;
			break	;
		}

		QString		param	= value.mid (pos1, pos2 - pos1) ;
		QStringList	bits	= QStringList::split (':', param) ;
		QString		*val	= paramDict.find (bits[0]) ;

		if	(val 	      != 0) res	+= *val		;
		else if (bits.count()  > 1) res += bits[1]	;

		pos1	 = pos2 + 1 ;
	}

	res	+= value.mid (pos1) ;
	return	res ;
}

/*  ------------------------------------------------------------------  */


/*  KBCSVType								*/
/*  KBCSVType	: Constructor for CSV type object			*/
/*  (returns)	: KBCSVType	 :					*/

KBCSVType::KBCSVType ()
	:
	KBType
	(	"CSV",
		KB::ITString,
		0,
		0,
		true
	)
{
}

/*  KBCSVType								*/
/*  isValid	: Test if value is valid for type			*/
/*  value	: const QString & : Value to check			*/
/*  pError	: KBValue &	  : Error return			*/
/*  where	: const QString & : Caller				*/
/*  (returns)	: bool		  : Valid				*/

bool	KBCSVType::isValid
	(	const QString	&,
		KBError		&,
		const QString	&
	)
{
	return	true	;
}

/*  KBCSVType								*/
/*  getQueryText							*/
/*  value	: KBDataArray  * : Raw text of value to convert		*/
/*  d		: KBShared     * : Decoded representation		*/
/*  buffer	: KBDataBuffer & : Results buffer			*/
/*  (returns)	: void		 :					*/

void	KBCSVType::getQueryText
	(	KBDataArray	*value,
		KBShared	*d,
		KBDataBuffer	&buffer
	)
{
	/* **** Stuff like binary escaping				*/
	KBType::getQueryText (value, d, buffer) ;
}





/*  ------------------------------------------------------------------  */

/*  KBCSV								*/
/*  KBCSV	: Constructor for CSV database connection class		*/
/*  (returns)	: KBServer	:					*/

KBCSV::KBCSV ()
	:
	KBServer ()
{
	m_hasHeaders	= false	;
}

/*  KBCSV								*/
/*  ~KBCSV	: Destructor for CSV database connection class		*/
/*  (returns)	:		:					*/

KBCSV::~KBCSV ()
{
}

/*  KBCSV								*/
/*  optionFlags	: Get server option flags				*/
/*  (returns)	: uint		: Flags					*/

uint	KBCSV::optionFlags ()
{
	return	0	;
}

/*  KBCSV								*/
/*  doConnect	: Open connection to database				*/
/*  svInfo	: KBServerInfo *: Server information			*/
/*  (returns)	: bool		: Success				*/

bool	KBCSV::doConnect
	(	KBServerInfo	*svInfo
	)
{
	DPRINTF
	((	"KBCSV::doConnect: [%s]\n",
		(cchar *)m_database
	))	;

	m_delimiter	= QString::null	;
	m_qualifier	= QString::null	;
	m_hasHeaders	= false	;

	if (svInfo->advanced() != 0)
		if (svInfo->advanced()->isType("csv"))
		{
			KBCSVAdvanced *a = (KBCSVAdvanced *)svInfo->advanced() ;
			m_delimiter	= a->m_delimiter  ;
			m_qualifier	= a->m_qualifier  ;
			m_hasHeaders	= a->m_hasHeaders ;
		}
		else	/* If the type is wrong then show a warning.	*/
			/* This should never happen unless someone	*/
			/* hand edits the XML database file.		*/
			KBError::EError
			(	TR("Driver error"),
				TR("Invalid advanced options, ignoring"),
				__ERRLOCN
			)	;

	if (m_delimiter.isEmpty()) m_delimiter	= ","	;
	if (m_qualifier.isEmpty()) m_qualifier	= "'"	;

	return	true	;
}

/*  KBCSV								*/
/*  qrySelect	: Open a select query					*/
/*  data	: bool		  : Query for data			*/
/*  select	: const QString&: Select query				*/
/*  (returns)	: KBSQLSelect * : Select query class or NULL on error	*/

KBSQLSelect
	*KBCSV::qrySelect
	(	bool		data,
		const QString	&select,
		bool
	)
{
	return	new KBCSVQrySelect (this, data, select) ;
}


/*  KBCSV								*/
/*  qryUpdate	: Open an update query					*/
/*  data	: bool		  : Query for data			*/
/*  update	: const QString & : Update query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KNQryUpdate *   : Update query class or NULL on error	*/

KBSQLUpdate
	*KBCSV::qryUpdate
	(	bool		data,
		const QString	&update,
		const QString	&tabName
	)
{
	return	new KBCSVQryUpdate (this, data, update, tabName) ;
}

/*  KBCSV								*/
/*  qryInsert	: Open an insert query					*/
/*  data	: bool		  : Query for data			*/
/*  insert	: const QString & : Insert query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KNQryInsert *   : Insert query class or NULL on error	*/

KBSQLInsert
	*KBCSV::qryInsert
	(	bool		data,
		const QString	&insert,
		const QString	&tabName
	)
{
	return	new KBCSVQryInsert (this, data, insert, tabName) ;
}

/*  KBCSV								*/
/*  qryDelete	: Open an delete query					*/
/*  data	: bool		  : Query for data			*/
/*  _delete	: const QString & : Delete query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KNQryDelete *   : Delete query class or NULL on error	*/

KBSQLDelete
	*KBCSV::qryDelete
	(	bool		data,
		const QString	&_delete,
		const QString	&tabName
	)
{
	return	new KBCSVQryDelete (this, data, _delete, tabName) ;
}

/*  KBCSV								*/
/*  command	: Execute arbitrary SQL					*/
/*  data	: bool		  : Querying for data			*/
/*  rawqry	: const QString & : Query text				*/
/*  nvals	: uint		  : Number of substitution values	*/
/*  values	: KBValue *	  : Substitution values			*/
/*  select	: KBSQLSelect **  : Return for result queries		*/
/*  (returns)	: bool		  : Success				*/

bool	KBCSV::command
	(	bool		,
		const QString	&,
		uint		,
		KBValue		*,
		KBSQLSelect	**
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Driver command API Not implemented",
			QString::null,
			__ERRLOCN
		   )	;

	return	false	;
}

/*  KBCSV								*/
/*  listTypes	: Get list of types with information flags		*/
/*  (returns)	: QString	: List as bar-separated string		*/

QString	KBCSV::listTypes ()
{
	return	"Primary Key,0|Foreign Key,0|String,0" ;
}

/*  KBCSV								*/
/*  tableExists	: See if named table exists				*/
/*  table	: const QString & : Table name				*/
/*  exists	: bool &	  : True if table exists		*/
/*  (returns)	: bool		  : Success				*/

bool	KBCSV::tableExists
	(	const QString	&table,
		bool		&exists
	)
{
	QString	path	= QString("%1/%2.csv").arg(m_database).arg(table) ;

	DPRINTF
	((	"KBCSV::tableExists: [%s]\n",
		path.latin1()
	))	;

	exists	= QFileInfo(path).exists() && QFileInfo(path).isReadable() ;
	return	 true	;
}

/*  KBCSV								*/
/*  doListTables: List tables in database				*/
/*  tabList	: KBTableDetailsList &					*/
/*				: Result list				*/
/*  type	: uint		: Type flags				*/
/*  (returns)	: bool		: Success				*/

bool	KBCSV::doListTables
	(	KBTableDetailsList	&tabList,
		uint			type
	)
{
	QDir	dir	(m_database, "*.csv") ;
	for (uint idx = 0 ; idx < dir.count() ; idx += 1)
		tabList.append
		(	KBTableDetails
			(	dir[idx],
				KB::IsTable,
				QP_SELECT|QP_INSERT|QP_UPDATE|QP_DELETE
		)	)	;

	return	true	;
}

/*  KBCSV								*/
/*  doListFields: List fields in table					*/
/*  tabSpec	: KBTableSpec &	: Table specification			*/
/*  (returns)	: bool		: Success				*/

bool	KBCSV::doListFields
	(	KBTableSpec	&tabSpec
	)
{
	QString	  path	= QString("%1/%2.csv").arg(m_database).arg(tabSpec.m_name) ;
	if (!QFileInfo(path).isFile() || !QFileInfo(path).isReadable())
	{
		m_lError = KBError
			   (	KBError::Error,
				"CSV file does not exists or is not readable",
				path,
				__ERRLOCN
			   )	;
		return	false	;
	}

	KBLocation location ;
	KBCopyFile copyFile (true, location) ;
	KBError    error    ;

	copyFile.setWhich   (KBCopyFile::UseDelimited) ;
	copyFile.setErrOpt  (KBCopyFile::ErrAbort) ;
	copyFile.setDelim   (m_delimiter) ;
	copyFile.setQualif  (m_qualifier) ;
	copyFile.setFile    (path) ;
	copyFile.setHeader  (false, 0) ;

	if (!copyFile.valid (m_lError))
		return	false	;

	QDict<QString>	dummy	;
	if (!copyFile.prepare(dummy, 0))
	{
		m_lError = KBError
			   (	KBError::Error,
				"Error preparing CSV file",
				path,
				__ERRLOCN
			   )	;
		return	false	;
	}

	KBValue		values	[501] ;
	bool		ok	;
	int		cols	;

	if (((cols = copyFile.getRow (values, 500, ok)) >= 0) && ok)
	{
		for (int idx = 0 ; idx < cols ; idx += 1)
		{
			KBFieldSpec *s = new KBFieldSpec
					 (	(uint)idx,
						m_hasHeaders ?
							values[idx].getRawText() :
							QString("column%1").arg(idx),
						"String",
						KB::ITString,
						0, 
						0,
						0
					 )	;

			s->m_dbType    = new KBCSVType () ;
			tabSpec.m_fldList.append (s) ;
		}

		return	true	;
	}

	m_lError = KBError
		   (	KBError::Error,
			"Error reading header from CSV file",
			path,
			__ERRLOCN
		   )	;
	return	true ;
}

/*  KBCSV								*/
/*  doCreateTable: Create a new table					*/
/*  tabSpec	 : KBTableSpec &: Table specification			*/
/*  assoc	 : bool		: Create associated objects		*/
/*  best	 : bool		: Use best match			*/
/*  (returns)	 : bool		: Success				*/

bool	KBCSV::doCreateTable
	(	KBTableSpec	&,
		bool		,
		bool
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"create",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBCSV								*/
/*  doRenameTable: Rename a  table					*/
/*  oldName	 : cchar *	: Current table name			*/
/*  newName	 : cchar *	: New table name			*/
/*  assoc	 : bool		: Rename associated stuff - none here	*/
/*  (returns)	 : bool		: Success				*/

bool	KBCSV::doRenameTable
	(	cchar	*,
		cchar	*,
		bool
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"rename",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBCSV								*/
/*  doDropTable	: Drop a table						*/
/*  table	: cchar *	: Table name				*/
/*  assoc	: bool		: Drop associated stuff - none here	*/
/*  (returns)	: bool		: Success				*/

bool	KBCSV::doDropTable
	(	cchar	*,
		bool
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"drop",
			__ERRLOCN
		   )	;
	return	false	;
}


/*  ------------------------------------------------------------------- */

static	KBCSVQrySelect	*qrySelectList[64] ;

/*  KBCSVQrySelect							*/
/*  KBCSVQrySelect							*/
/*		: Constructor for select query object			*/
/*  server	: KBCSV *	  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  query	: const QString & : Select query			*/
/*  (returns)	: KBCSVQrySelect:					*/

KBCSVQrySelect::KBCSVQrySelect
	(	KBCSV		*server,
		bool		data,
		const QString	&query
	)	
	:
	KBSQLSelect	(server, data, query),
	m_server	(server)
{
	m_nRows		= 0 ;
	m_nFields	= 0 ;
	m_sequence	= sizeof(qrySelectList)/sizeof(KBCSVQrySelect *) ;

	for (uint idx = 0 ; idx < sizeof(qrySelectList)/sizeof(KBCSVQrySelect *) ; idx += 1)
		if (qrySelectList[idx] == 0)
		{	qrySelectList[idx] = this ;
			m_sequence	   = idx  ;
			break	;
		}
}

/*  KBCSVQrySelect							*/
/*  ~KBCSVQrySelect							*/
/*		: Destructor for select query object			*/
/*  (returns)	:		:					*/

KBCSVQrySelect::~KBCSVQrySelect ()
{
	for (uint idx = 0 ; idx < sizeof(qrySelectList)/sizeof(KBCSVQrySelect *) ; idx += 1)
		if (qrySelectList[idx] == this)
		{	qrySelectList[idx] = 0 ;
			break	;
		}
}

static	bool	loadExpression
		(	const QString	&skeleton,
			cchar 		*which,
			uint		seq,
			uint		idx,
			const QString	&expression,
			KBError		&pError
		)
{
	QString	func	= QString(skeleton).arg(which).arg(seq).arg(idx).arg(expression) ;
	func		= func.replace("=", "==") ;
	func		= func.replace("'", "\"") ;

	DPRINTF
	((	"loadExpression:\n%s\n",
		func.latin1()
	))	;

	CBUFF	   *cbuff  = el_compile (0, 0, 0, func.latin1(), 0) ;
	if (cbuff == 0)
	{
		pError	= KBError
			   (	KBError::Error,
				"Expression compile error",
				expression,
				__ERRLOCN
			   )	;
		return	false	;
	}
	if (!el_loadtext (cbuff))
	{
		pError	= KBError
			   (	KBError::Error,
				"Expression load error",
				expression,
				__ERRLOCN
			   )	;
		return	false	;
	}

	return	true	;
}

static	KBValue	evalExpression
	(	cchar		*which,
		uint		seq,
		uint		idx,
		uint		cols,
		VALUE		*args,
		KBType		*&type
	)
{
	QString	fn	= QString("%1_%2_%3").arg(which).arg(seq).arg(idx) ;
        VALUE   res     = el_vexec ("", fn.latin1(), cols, args) ;
	KBValue	value	;

	switch	(res.tag->tag)
	{
		case V_NUM	:
			value	= QString::number (res.val.num) ;
			type	= &_kbFixed	;
			break	;

		case V_DBL	:
			value	= QString::number (res.val.dbl) ;
			type	= &_kbFloat	;
			break	;

		case V_STR	:
			value	= res.val.str->text ;
			type	= &_kbString	;
			break	;

		default		:
			DPRINTF
			((	"evalExpression: unexpected tag [%d]\n",
				res.tag->tag
			))	;
			value	= QString::null ;
			type	= &_kbString	;
					break	;
	}

	DPRINTF
	((	"evalExpression: [%s]\n",
		value.getRawText().latin1()
	))	;
	return	value	;
}

/*  KBCSVQrySelect							*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBCSVQrySelect::execute
	(	uint		nvals,
		const KBValue	*values
	)
{
	KBBaseSelect	select	;
	KBDataBuffer	exeSQL	;

	if (!m_server->subPlaceList
		(	m_rawQuery,
			nvals,
			values,
			exeSQL,
			m_codec,
			m_lError
		))
		return false	;

	if (!select.parse (QString(exeSQL.data()), 0))
	{
		m_lError = select.lastError() ;
		return	 false	;
	}

	if
	(	(select.orderList ().count() > 0) ||
		(select.groupList ().count() > 0) ||
		(select.havingList().count() > 0)
	)
	{
		m_lError = KBError
			   (	KBError::Error,
				"CSV driver does not support 'order', 'group', or 'having'",
				exeSQL.data(),
				__ERRLOCN
			   )	;
	}
	if (select.tableList().count() > 1)
	{
		m_lError = KBError
			   (	KBError::Error,
				"CSV driver does not support multi-table selects",
				exeSQL.data(),
				__ERRLOCN
			   )	;
	}

	m_table	= select.tableList()[0].tableName() ;

	QString	  path	= QString("%1/%2.csv").arg(m_server->database()).arg(m_table) ;
	if (!QFileInfo(path).isFile() || !QFileInfo(path).isReadable())
	{
		m_lError = KBError
			   (	KBError::Error,
				"CSV file does not exists or is not readable",
				path,
				__ERRLOCN
			   )	;
		return	false	;
	}

	KBLocation location ;
	KBCopyFile copyFile (true, location) ;
	KBError    error    ;

	copyFile.setWhich   (KBCopyFile::UseDelimited) ;
	copyFile.setErrOpt  (KBCopyFile::ErrAbort) ;
	copyFile.setDelim   (m_server->m_delimiter) ;
	copyFile.setQualif  (m_server->m_qualifier) ;
	copyFile.setFile    (path) ;
	copyFile.setHeader  (false, 0) ;

	if (!copyFile.valid (m_lError))
		return	false	;

	QDict<QString>	dummy	;
	if (!copyFile.prepare(dummy, 0))
	{
		m_lError = KBError
			   (	KBError::Error,
				"Error preparing CSV file",
				path,
				__ERRLOCN
			   )	;
		return	false	;
	}

	KBValue		colvals	[501]	;
	bool		ok	;
	int		cols	;

	if (((cols = copyFile.getRow (colvals, 500, ok)) < 0) || !ok)
	{
		m_lError = KBError
			   (	KBError::Error,
				"Error reading header line from file",
				path,
				__ERRLOCN
			   )	;
		return	false	;
	}

	QStringList	colNames;
	cchar		*sep		= ""	;
	QString		skeleton	=
					"global toint ;\n"
					"global todouble ;\n"
					"global tochar ;\n"
					"public %1_%2_%3 ("
					;

	for (int idx = 0 ; idx < cols ; idx += 1)
	{
		QString	name = m_server->m_hasHeaders ?
					colvals[idx].getRawText() :
					QString("column%1").arg(idx)
					;
		skeleton  += sep 	;
		skeleton  += name	;
		sep	   =  "," 	;
		colNames.append (name)	;
	}

	skeleton +=
		")\n"
		"{\n"
		"	return %4 ;\n"
		"}\n"
		;

	DPRINTF
	((	"KBCSVQrySelect::execute:\n%s\n",
		skeleton.latin1()
	))	;

	uint	place	= 0 ;

	for (uint idx = 0 ; idx < select.fetchList().count() ; idx += 1)
	{
		QString	expr	= select.fetchList()[idx].expr() ;

		if (!loadExpression
			(	skeleton,
				"fetch",
				m_sequence,
				idx,
				expr.replace(m_table + ".", ""),
				m_lError
			))
		{
			return	false	;
		}

		m_fields.append (expr)	;
	}

	for (uint idx = 0 ; idx < select.whereList().count() ; idx += 1)
	{
		if (!loadExpression
			(	skeleton,
				"where",
				m_sequence,
				idx,
				select.whereList()[idx].expr(0, place).replace(m_table + ".", ""),
				m_lError
			))
		{
			return	false	;
		}
	}

	m_nFields	= select.fetchList().count()	;

	KBType		**types	= 0	;
	VALUE		args	[501]	;
	int		row	= 0	;
	bool		skip	= m_server->m_hasHeaders;

	do
	{
		DPRINTF
		((	"KBCSVQrySelect::execute: got row: skip=%d\n",
			skip
		))	;

		if (skip)
		{	skip	= false	;
			continue	;
		}

		for (int idx = 0 ; idx < cols ; idx += 1)
			args[idx] = new STRING(colvals[idx].getRawText().latin1()) ;

		bool	include	= true	;

		for (uint where = 0 ; where < select.whereList().count() ; where += 1)
		{
			KBType	*type	;
			KBValue	value	;
			value	= evalExpression
					(	"where",
						m_sequence,
						where,
						cols,
						args,
						type
					)	;
			if (!value.isTrue())
			{	include	= false ;
				break	;
			}
		}

		if (!include)
			continue ;

		if (m_types == 0)
			m_types	= types	= new KBType *[m_nFields] ;

		for (uint col = 0 ; col < select.fetchList().count() ; col += 1)
		{
			KBType	*type	;
			KBValue	value	;

			value	= evalExpression
					(	"fetch",
						m_sequence,
						col,
						cols,
						args,
						type
					)	;

			if (types != 0) types[col] = type ;
			putInCache (row, col, value) ;
		}

		row	+= 1 ;
	}
	while ((cols = copyFile.getRow (colvals, 500, ok)) >= 0) ;

	m_nRows	= row	;

	if (m_types == 0)
	{
		m_types	= new KBType *[m_nFields] ;
		for (uint col = 0 ; col < m_nFields ; col += 1)
			m_types[col] = &_kbString ;
	}

	return	true	;
}

/*  KBCSVQrySelect							*/
/*  getField	: Get a specified field from the query results		*/
/*  qrow	: uint		: Row number				*/
/*  qcol	: uint		: Column number				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCSVQrySelect::getField
	(	uint		qrow,
		uint		qcol,
		KBValue::VTrans
	)
{
	KBValue	value	;
	getFromCache	(qrow, qcol, value) ;
	return	value	;
}

/*  KBCSVQrySelect							*/
/*  getFieldname: Get a specified field name from the query results	*/
/*  qcol	: uint		: Column number				*/
/*  (returns)	: QString	: Name					*/

QString	KBCSVQrySelect::getFieldName
	(	uint	qcol
	)
{
	/* First check that the request is within the range of rows	*/
	/* and fields returned by the query.				*/
	if (qcol >= m_nFields) return QString() ;

	return	m_fields[qcol] ;
}


/*  KBCSVQryUpdate							*/
/*  KBCSVQryUpdate							*/
/*		: Constructor for update query object			*/
/*  server	: KBCSV *	  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  query	: const QString & : Update query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KBCSVQryUpdate:					*/

KBCSVQryUpdate::KBCSVQryUpdate
	(	KBCSV		*server,
		bool		data,
		const QString	&query,
		const QString	&tabName
	)	
	:
	KBSQLUpdate (server, data, query, tabName),
	m_server    (server)
{
	m_nRows	= 0 ;
}

/*  KBCSVQryUpdate							*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBCSVQryUpdate::execute
	(	uint		,
		const KBValue	*
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"CSV driver is read-only, 'update' not supported",
			QString::null,
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBCSVQryUpdate							*/
/*  ~KBCSVQryUpdate							*/
/*		: Destructor for update query object			*/
/*  (returns)	:		:					*/

KBCSVQryUpdate::~KBCSVQryUpdate ()
{
}


/*  KBCSVQryInsert							*/
/*  KBCSVQryInsert							*/
/*		: Constructor for insert query object			*/
/*  server	: KBCSV *	  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  tabName	: const QString & : Table being updated			*/
/*  query	: const QString & : Insert query			*/
/*  (returns)	: KBCSVQryInsert:					*/

KBCSVQryInsert::KBCSVQryInsert
	(	KBCSV		*server,
		bool		data,
		const QString	&query,
		const QString	&tabName
	)	
	:
	KBSQLInsert (server, data, query, tabName),
	m_server    (server)
{
	m_nRows	= 0 ;
}

/*  KBCSVQryInsert							*/
/*  ~KBCSVQryInsert							*/
/*		: Destructor for insert query object			*/
/*  (returns)	:		:					*/

KBCSVQryInsert::~KBCSVQryInsert ()
{
}

/*  KBCSVQryInsert							*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBCSVQryInsert::execute
	(	uint		,
		const KBValue	*
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"CSV driver is read-only, 'insert' not supported",
			QString::null,
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBCSVQryInsert							*/
/*  getNewKey	: Get new insert key					*/
/*  primary	: const QString & : Key column name			*/
/*  _newKey	: KBValue &	  : New key				*/
/*  prior	: bool		  : Pre-insert call			*/
/*  (returns)	: bool		  : Success				*/

bool	KBCSVQryInsert::getNewKey
	(	const QString	&,
		KBValue		&,
		bool		
	)
{
	return	false	;
}

/*  KBCSVQryDelete							*/
/*  KBCSVQryDelete							*/
/*		: Constructor for delete query object			*/
/*  server	: KBCSV *	  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  query	: const QString & : Delete query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KBCSVQryDelete:					*/

KBCSVQryDelete::KBCSVQryDelete
	(	KBCSV		*server,
		bool		data,
		const QString	&query,
		const QString	&tabName
	)	
	:
	KBSQLDelete (server, data, query, tabName),
	m_server    (server)
{
	m_nRows	= 0 ;
}

/*  KBCSVQryDelete							*/
/*  ~KBCSVQryDelete							*/
/*		: Destructor for delete query object			*/
/*  (returns)	:		:					*/

KBCSVQryDelete::~KBCSVQryDelete ()
{
}

/*  KBCSVQryDelete							*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBCSVQryDelete::execute
	(	uint		,
		const KBValue	*
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"CSV driver is read-only, 'delete' not supported",
			QString::null,
			__ERRLOCN
		   )	;
	return	false	;
}


KBFACTORY
(	KBCSVFactory,
	"driver_csv"
)

KBFACTORYIMPL
(	KBCSVFactory,
	driver_csv
)

QObject	*KBCSVFactory::create
	(	QObject		  *parent,
		cchar		  *object,
		cchar		  *,
		const QStringList &
	)
{
	extern	void	el_initialize (uint, uint, bool) ;
	el_initialize (1024 * 64, 4096, false) ;

	if ((parent != 0) && !parent->inherits ("QWidget"))
	{
		fprintf	(stderr, "KBCSVFactory: parent does not inherit QWidget\n") ;
		return	0  ;
	}

	if (strcmp (object, "driver"  ) == 0) return new KBCSV () ;

	if (strcmp (object, "advanced") == 0) return new KBCSVAdvanced () ;

	return	0 ;
}

QString	KBCSVFactory::ident ()
{
	return	QString("CSV: %1%2 %3")
			.arg(__KB_BUILD_RELEASE)
			.arg(__KB_BUILD_BUILD  )
			.arg(__KB_BUILD_TIME   )
			;
}
