/***************************************************************************
    file	         : kb_csvadvanced.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#ifdef		__KB_AUTOMOC
#include	"kb_csvadvanced.moc"
#else
#include	"kb_csvadvanced.h"
#endif

#include	"rk_tabwidget.h"
#include	"rk_gridbox.h"

#include	<qlabel.h>


/*  KBCSVAdvanced							*/
/*  KBCSVAdvanced							*/
/*		: Constructor for CSV advanced options object		*/
/*  (returns)	: KBCSVAdvanced	:					*/

KBCSVAdvanced::KBCSVAdvanced  ()
	:
	KBDBAdvanced ("csv")
{
	m_delimiter	= QString::null	;
	m_qualifier	= QString::null	;
	m_hasHeaders	= true		;
}

/*  KBCSVAdvanced							*/
/*  ~KBCSVAdvanced							*/
/*		: Destructor for CSV advanced options object		*/
/*  (returns)	:		:					*/

KBCSVAdvanced::~KBCSVAdvanced ()
{
}

/*  KBCSVAdvanced							*/
/*  load	: Load settings from DOM element			*/
/*  driverElem	: QDomElement &	: DOM element				*/
/*  (returns)	: void		:					*/

void	KBCSVAdvanced::load
	(	const QDomElement &driverElem
	)
{
	m_delimiter	= driverElem.attribute ("delimiter" )	;
	m_qualifier	= driverElem.attribute ("qualifier" )	;
	m_hasHeaders	= driverElem.attribute ("hasheaders").toUInt() ;
}

/*  KBCSVAdvanced							*/
/*  save	: Save settings from DOM element			*/
/*  driverElem	: QDomElement &	: DOM element				*/
/*  (returns)	: void		:					*/

void	KBCSVAdvanced::save
	(	QDomElement	&driverElem
	)
{
	driverElem.setAttribute ("delimiter" , m_delimiter	       ) ;
	driverElem.setAttribute ("qualifier" , m_qualifier	       ) ;
	driverElem.setAttribute ("hasheaders", m_hasHeaders ? "1" : "0") ;
}

/*  KBCSVAdvanced							*/
/*  setupDialog	: Set up advanced options dialog			*/
/*  tabWidget	: RKTabWidget *	: Tab widget to which to attach pages	*/
/*  (returns)	: void		:					*/

void	KBCSVAdvanced::setupDialog
	(	RKTabWidget	*tabWidget
	)
{
	RKGridBox	*advLayout	= new RKGridBox (2, tabWidget) ;

	tabWidget->addTab	(advLayout, "CSV") ;

	new QLabel (TR("Delimiter"), 		 advLayout) ;
	m_cbDelimiter	   	= new QComboBox (advLayout) ;
	new QLabel (TR("Qualifier"), 		 advLayout) ;
	m_cbQualifier 		= new QComboBox (advLayout) ;
	new QLabel (TR("Column Headers"), 	 advLayout) ;
	m_cbHasHeaders		= new QCheckBox (advLayout) ;

	m_cbDelimiter ->insertItem  	("," ) ;
	m_cbDelimiter ->insertItem  	("|" ) ;

	m_cbQualifier ->insertItem  	("'" ) ;
	m_cbQualifier ->insertItem  	("\"") ;

	m_cbDelimiter ->setEditable 	(true) ;
	m_cbQualifier ->setEditable	(true) ;

	m_cbDelimiter ->setCurrentText	(m_delimiter ) ;
	m_cbQualifier ->setCurrentText 	(m_qualifier ) ;
	m_cbHasHeaders->setChecked	(m_hasHeaders) ;

	advLayout->addFillerRow ()	;
}

/*  KBCSVAdvanced							*/
/*  saveDialog	: Save results from dialog				*/
/*  (returns)	: void		:					*/

void	KBCSVAdvanced::saveDialog ()
{
	m_delimiter	= m_cbDelimiter ->currentText () ;
	m_qualifier	= m_cbQualifier ->currentText () ;
	m_hasHeaders	= m_cbHasHeaders->isChecked   () ;
}

/*  KBCSVAdvanced							*/
/*  copy	: Replicate this object					*/
/*  (returns)	: KBCSVAdvanced * : Replicated object			*/

KBDBAdvanced
	*KBCSVAdvanced::copy ()
{
	KBCSVAdvanced	*copy	= new KBCSVAdvanced () ;

	m_delimiter	= copy->m_delimiter	;
	m_qualifier	= copy->m_qualifier	;
	m_hasHeaders	= copy->m_hasHeaders	;

	return	copy	;
}
