/***************************************************************************
    file	         : kb_dummy.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<time.h>
#include	<sys/types.h>


#include	<qstring.h>
#include	<qdict.h>
#include	<qintdict.h>


#include	"mdbtools.h"

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_databuffer.h"
#include	"kb_database.h"
#include	"kb_serverinfo.h"

#include	"kb_libloader.h"

#include	"kb_basequery.h"

#include	"std.h"
#include	"eli.h"
#include	"interp.h"
#include	"syn.h"
#include	"code.h"


#define		__DEBUG_MDB	(0)
#if		__DEBUG_MDB
#include	"kb_dprintf.h"
#endif



struct	MDBTypeMap
{	int		ident		;	/* MDB identifier	*/
	KB::IType	itype		;	/* Internal type	*/
	char		ftype	[16]	;	/* MDB type name	*/
	uint		flags		;	/* Various flags	*/
}	;



#define	FIELD_TYPE_UNSET	(-1)

static	MDBTypeMap	typeMap[] =
{
/*	ident			itype		ftype		flags		*/
{	MDB_BOOL,		KB::ITBool,	"Boolean",	0,			},
{	MDB_BYTE,		KB::ITFixed,	"Byte",		0,			},
{	MDB_INT,		KB::ITFixed,	"Integer",	0,			},
{	MDB_LONGINT,		KB::ITFixed,	"Long",		0,			},
{	MDB_MONEY,		KB::ITFloat,	"Money",	0,			},
{	MDB_FLOAT,		KB::ITFloat,	"Float",	0,			},
{	MDB_DOUBLE,		KB::ITFloat,	"Double",	0,			},
{	MDB_SDATETIME,		KB::ITDateTime,	"DateTime",	0,			},
{	MDB_TEXT,		KB::ITString,	"Text",		0,			},
{	MDB_OLE,		KB::ITString,	"OLE",		0,			},
{	MDB_MEMO,		KB::ITBinary,	"Memo",		0,			},
{	MDB_REPID,		KB::ITString,	"RepID",	0,			},
{	MDB_NUMERIC,		KB::ITFloat,	"Numeric",	FF_LENGTH|FF_PREC	}
}	;




/*  KBMDBType								*/
/*  -----------								*/

class	KBMDBType : public KBType
{
	MDBTypeMap	*m_typeInfo	;

public	:

	KBMDBType  (MDBTypeMap *, uint, uint, bool) ;

	virtual	bool    isValid      (const QString &, KBError  &, const QString & = QString::null) ;
	virtual	void	getQueryText (KBDataArray   *, KBShared *, KBDataBuffer  &) ;
}	;



/*  KBMDB								*/
/*  -------								*/
/*  Implementation class for insertface to the MDB database		*/

class	KBMDB : public KBServer
{
	bool			doListTables 	(KBTableDetailsList &, uint) ;

	virtual	bool	 	doConnect    	(KBServerInfo  *) ;
	virtual	bool		doListFields 	(KBTableSpec   &) ;
	virtual	bool		doCreateTable	(KBTableSpec   &,  bool, bool = false) ;
	virtual	bool		doRenameTable	(cchar *, cchar *, bool) ;
	virtual	bool		doDropTable  	(cchar *,	   bool) ;

	bool			m_readOnly	;
	MdbHandle		*m_handle	;

public	:

	KBMDB	() ;
virtual~KBMDB () ;


	virtual	KBSQLSelect	*qrySelect 	(bool, const QString &, bool) ;
	virtual	KBSQLUpdate	*qryUpdate 	(bool, const QString &, const QString &) ;
	virtual	KBSQLInsert	*qryInsert 	(bool, const QString &, const QString &) ;
	virtual	KBSQLDelete	*qryDelete 	(bool, const QString &, const QString &) ;

	virtual	bool		command		(bool, const QString &, uint, KBValue *, KBSQLSelect ** = 0) ;


	virtual	QString		listTypes	() ;
	virtual	uint		optionFlags	() ;
	virtual	bool		tableExists	(const QString &, bool &) ;

	inline	MdbHandle	*handle		()
	{
		return	m_handle	;
	}
}	;


/*  KBMDBQrySelect							*/
/*  ----------------							*/
/*  Implementation class for select queries on the MDB database		*/

class	KBMDBQrySelect : public KBSQLSelect
{
	KBMDB		*m_server	;
	QString		m_table		;
	QStringList	m_fields	;
	uint		m_sequence	;


public	:
	KBMDBQrySelect (KBMDB *, bool, const QString &) ;
virtual~KBMDBQrySelect () ;

	virtual	bool	execute		(uint, const KBValue * ) ;
	virtual	KBValue	getField	(uint, uint, KBValue::VTrans) ;
	virtual	QString	getFieldName	(uint) ;
}	;


/*  KBMDBQryUpdate							*/
/*  ----------------							*/
/*  Implementation class for update queries on the MDB database		*/

class	KBMDBQryUpdate : public KBSQLUpdate
{
	KBMDB	*m_server	;

public	:
	KBMDBQryUpdate (KBMDB *, bool, const QString &, const QString &) ;
virtual~KBMDBQryUpdate () ;

	virtual	bool	execute	 (uint, const KBValue * ) ;

}	;

/*  KBMDBQryInsert							*/
/*  ----------------							*/
/*  Implementation class for insert queries on the MDB database		*/

class	KBMDBQryInsert : public KBSQLInsert
{
	KBMDB		*m_server	;
	QString		m_autocol	;
	KBValue		m_newKey	;

public	:
	KBMDBQryInsert (KBMDB *, bool, const QString &, const QString &) ;
virtual~KBMDBQryInsert () ;

	virtual	bool	execute	  (uint, const KBValue *) ;
	virtual	bool	getNewKey (const QString &, KBValue &, bool) ;
}	;

/*  KBMDBQryDelete							*/
/*  ----------------							*/
/*  Implementation class for delete queries on the MDB database		*/

class	KBMDBQryDelete : public KBSQLDelete
{
	KBMDB		*m_server	;

public	:
	KBMDBQryDelete (KBMDB *, bool, const QString &, const QString &) ;
virtual~KBMDBQryDelete () ;

	virtual	bool	execute	 (uint, const KBValue * ) ;
}	;


static	QIntDict<MDBTypeMap>	dIdentToType	;
static	QDict	<KBMDBType >	typesDict	;

/*  ------------------------------------------------------------------  */

/*  KBMDBType								*/
/*  KBMDBType	: Constructor for MDB type object			*/
/*  typeInfo	: MDBTypeMap * : Type information			*/
/*  length	: uint		 : Underlying database length		*/
/*  prec	: uint		 : Underlying database precision	*/
/*  nullOK	: bool		 : True if null is OK			*/
/*  (returns)	: KBMDBType	 :					*/

KBMDBType::KBMDBType
	(	MDBTypeMap	*typeInfo,
		uint	  	length,
		uint		prec,
		bool	  	nullOK
	)
	:
	KBType
	(	"MDB",
		typeInfo == 0 ? KB::ITUnknown : typeInfo->itype,
		length,
		prec,
		nullOK
	),
	m_typeInfo	(typeInfo)
{
}

/*  KBMDBType								*/
/*  isValid	: Test if value is valid for type			*/
/*  value	: const QString & : Value to check			*/
/*  pError	: KBValue &	  : Error return			*/
/*  where	: const QString & : Caller				*/
/*  (returns)	: bool		  : Valid				*/

bool	KBMDBType::isValid
	(	const QString	 &value,
		KBError		 &pError,
		const QString	&where
	)
{
	/* *** Specific type checking					*/
	return	KBType::isValid (value, pError, where) ;
}

/*  KBMDBType								*/
/*  getQueryText							*/
/*  value	: KBDataArray  * : Raw text of value to convert		*/
/*  d		: KBShared     * : Decoded representation		*/
/*  buffer	: KBDataBuffer & : Results buffer			*/
/*  (returns)	: void		 :					*/

void	KBMDBType::getQueryText
	(	KBDataArray	*value,
		KBShared	*d,
		KBDataBuffer	&buffer
	)
{
	/* **** Stuff like binary escaping				*/
	KBType::getQueryText (value, d, buffer) ;
}





/*  ------------------------------------------------------------------  */

/*  KBMDB								*/
/*  KBMDB	: Constructor for MDB database connection class	*/
/*  (returns)	: KBServer	:					*/

KBMDB::KBMDB ()
	:
	KBServer ()
{
	m_readOnly	= false	;
	m_handle	= 0	;
}

/*  KBMDB								*/
/*  ~KBMDB	: Destructor for MDB database connection class	*/
/*  (returns)	:		:					*/

KBMDB::~KBMDB ()
{
	if (m_handle != 0) mdb_close (m_handle) ;
}

/*  KBMDB								*/
/*  optionFlags	: Get server option flags				*/
/*  (returns)	: uint		: Flags					*/

uint	KBMDB::optionFlags ()
{
	return	0	;
}

/*  KBMDB								*/
/*  doConnect	: Open connection to database				*/
/*  svInfo	: KBServerInfo *: Server information			*/
/*  (returns)	: bool		: Success				*/

bool	KBMDB::doConnect
	(	KBServerInfo	*svInfo
	)
{
	DPRINTF
	((	"KBMDB::doConnect: [%s]\n",
		(cchar *)m_database
	))	;

	m_readOnly = svInfo->readOnly () ;

	QString	database = m_database ;

	if (database.isEmpty() || (database == "."))
		database = QString("%1/%2.mdb").arg(svInfo->getDBPath()).arg(svInfo->serverName()) ;

	if (database[0] == '$')
	{
		if (getenv(database.mid(1)) == 0)
		{
			m_lError = KBError
				   (	KBError::Error,
					QString("No '%1' environment parameter").arg(database),
					QString::null,
					__ERRLOCN
				   )	;
			return	false	;
		}

		database = getenv(database.mid(1)) ;
	}

	DPRINTF
	((	"KBMDB::doConnect: [%s]\n",
		database.latin1()
	))	;

	/* Check that we are not already connected. Note that the	*/
	/* notConnected routine sets an error message.			*/
	if (m_handle != 0)
	{	m_lError = KBError
			   (	KBError::Error,
				"Already connected to MDB database",
				QString::null,
				__ERRLOCN
			   )	;
		return	false	;
	}

	m_handle = mdb_open (database.latin1(), m_readOnly ? MDB_NOFLAGS : MDB_WRITABLE) ;
	if (m_handle == 0)
	{	m_lError = KBError
			   (	KBError::Error,
				"Cannot open MDB database",
				database,
				__ERRLOCN
			   )	;
		return	false	;
	}

	return	true	;
}

/*  KBMDB								*/
/*  qrySelect	: Open a select query					*/
/*  data	: bool		  : Query for data			*/
/*  select	: const QString&: Select query				*/
/*  (returns)	: KBSQLSelect * : Select query class or NULL on error	*/

KBSQLSelect
	*KBMDB::qrySelect
	(	bool		data,
		const QString	&select,
		bool
	)
{
	return	new KBMDBQrySelect (this, data, select) ;
}


/*  KBMDB								*/
/*  qryUpdate	: Open an update query					*/
/*  data	: bool		  : Query for data			*/
/*  update	: const QString & : Update query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KNQryUpdate *   : Update query class or NULL on error	*/

KBSQLUpdate
	*KBMDB::qryUpdate
	(	bool		data,
		const QString	&update,
		const QString	&tabName
	)
{
	return	new KBMDBQryUpdate (this, data, update, tabName) ;
}

/*  KBMDB								*/
/*  qryInsert	: Open an insert query					*/
/*  data	: bool		  : Query for data			*/
/*  insert	: const QString & : Insert query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KNQryInsert *   : Insert query class or NULL on error	*/

KBSQLInsert
	*KBMDB::qryInsert
	(	bool		data,
		const QString	&insert,
		const QString	&tabName
	)
{
	return	new KBMDBQryInsert (this, data, insert, tabName) ;
}

/*  KBMDB								*/
/*  qryDelete	: Open an delete query					*/
/*  data	: bool		  : Query for data			*/
/*  _delete	: const QString & : Delete query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KNQryDelete *   : Delete query class or NULL on error	*/

KBSQLDelete
	*KBMDB::qryDelete
	(	bool		data,
		const QString	&_delete,
		const QString	&tabName
	)
{
	return	new KBMDBQryDelete (this, data, _delete, tabName) ;
}

/*  KBMDB								*/
/*  command	: Execute arbitrary SQL					*/
/*  data	: bool		  : Querying for data			*/
/*  rawqry	: const QString & : Query text				*/
/*  nvals	: uint		  : Number of substitution values	*/
/*  values	: KBValue *	  : Substitution values			*/
/*  select	: KBSQLSelect **  : Return for result queries		*/
/*  (returns)	: bool		  : Success				*/

bool	KBMDB::command
	(	bool		,
		const QString	&,
		uint		,
		KBValue		*,
		KBSQLSelect	**
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"create",
			__ERRLOCN
		   )	;

	return	false	;
}

/*  KBMDB								*/
/*  listTypes	: Get list of types with information flags		*/
/*  (returns)	: QString	: List as bar-separated string		*/

QString	KBMDB::listTypes ()
{
	static	QString	typeList ;

	if (typeList.isNull())
	{
		typeList = "Primary Key,0|Foreign Key,0" ;

		for (uint idx = 0 ; idx < sizeof(typeMap)/sizeof(MDBTypeMap) ; idx += 1)
		{
			MDBTypeMap *m = &typeMap[idx] ;

			if ((m->flags & FF_NOCREATE) == 0)
				typeList += QString("|%1,%2").arg(m->ftype).arg(m->flags) ;
		}
	}

	return	typeList ;
}

/*  KBMDB								*/
/*  tableExists	: See if named table exists				*/
/*  table	: const QString & : Table name				*/
/*  exists	: bool &	  : True if table exists		*/
/*  (returns)	: bool		  : Success				*/

bool	KBMDB::tableExists
	(	const QString	&table,
		bool		&exists
	)
{
	MdbTableDef *tableDef = mdb_read_table_by_name
				(	m_handle,
					(char *)table.latin1(),
					MDB_TABLE
				)	;
	exists = tableDef != 0  	;
	mdb_free_tabledef (tableDef)	;

	return	 true	;
}

/*  KBMDB								*/
/*  doListTables: List tables in database				*/
/*  tabList	: KBTableDetailsList &					*/
/*				: Result list				*/
/*  type	: uint		: Type flags				*/
/*  (returns)	: bool		: Success				*/

bool	KBMDB::doListTables
	(	KBTableDetailsList	&tabList,
		uint			type
	)
{
	DPRINTF
	((	"KBMDB::doListTables: type=%04x KB::IsTable=%04x\n",
		type,
		KB::IsTable
	))	;

	if ((type & KB::IsTable) == 0)
		return true ;

	mdb_read_catalog (m_handle, MDB_TABLE) ;

	for (uint idx = 0 ; idx < m_handle->num_catalog ; idx += 1)
	{
		MdbCatalogEntry *entry = (MdbCatalogEntry *)g_ptr_array_index (m_handle->catalog, idx) ;

	        if (entry->object_type != MDB_TABLE)
			continue ;
		if (!m_showAllTables && (qstrncmp (entry->object_name, "MSys", 4) == 0))
			continue ;

		tabList.append
		(	KBTableDetails
			(	entry->object_name,
				KB::IsTable,
				QP_SELECT|QP_INSERT|QP_UPDATE|QP_DELETE
		)	)	;
        }

	return	true	;
}

/*  KBMDB								*/
/*  doListFields: List fields in table					*/
/*  tabSpec	: KBTableSpec &	: Table specification			*/
/*  (returns)	: bool		: Success				*/

bool	KBMDB::doListFields
	(	KBTableSpec	&tabSpec
	)
{
	MdbTableDef *tableDef = mdb_read_table_by_name
				(	m_handle,
					(char *)tabSpec.m_name.latin1(),
					MDB_TABLE
				)	;
	if (tableDef == 0)
	{	m_lError = KBError
			   (	KBError::Error,
				"Cannot get table details",
				tabSpec.m_name,
				__ERRLOCN
			   )	;
		return	false	;
	}

	GPtrArray   *columns  = mdb_read_columns(tableDef) ;
	if (columns  == 0)
	{	m_lError = KBError
			   (	KBError::Error,
				"Cannot get table columns",
				tabSpec.m_name,
				__ERRLOCN
			   )	;
		mdb_free_tabledef (tableDef) ;
		return	false	;
	}

	for (uint idx = 0 ; idx < tableDef->num_cols ; idx += 1)
	{
		MdbColumn   *column = (MdbColumn *)g_ptr_array_index (columns, idx) ;

		MDBTypeMap  *ptr    = dIdentToType.find(column->col_type) ;
		cchar	    *ftype  = ptr == 0 ? "<Unknown>" : ptr->ftype ;

		KBFieldSpec *s = new KBFieldSpec
				 (	(uint)idx,
					column->name,
					ftype,
					ptr == 0 ? KB::ITUnknown : ptr->itype,
					0, //flags,
					column->col_size,
					column->col_prec
				 )	;

		s->m_dbType    = new KBMDBType
				 (	ptr,
					column->col_size,
					column->col_prec,
					true
				 )	;

		tabSpec.m_fldList.append (s) ;
	}

	mdb_free_tabledef (tableDef) ;
	return	true ;
}

/*  KBMDB								*/
/*  doCreateTable: Create a new table					*/
/*  tabSpec	 : KBTableSpec &: Table specification			*/
/*  assoc	 : bool		: Create associated objects		*/
/*  best	 : bool		: Use best match			*/
/*  (returns)	 : bool		: Success				*/

bool	KBMDB::doCreateTable
	(	KBTableSpec	&,
		bool		,
		bool
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"create",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBMDB								*/
/*  doRenameTable: Rename a  table					*/
/*  oldName	 : cchar *	: Current table name			*/
/*  newName	 : cchar *	: New table name			*/
/*  assoc	 : bool		: Rename associated stuff - none here	*/
/*  (returns)	 : bool		: Success				*/

bool	KBMDB::doRenameTable
	(	cchar	*,
		cchar	*,
		bool
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"rename",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBMDB								*/
/*  doDropTable	: Drop a table						*/
/*  table	: cchar *	: Table name				*/
/*  assoc	: bool		: Drop associated stuff - none here	*/
/*  (returns)	: bool		: Success				*/

bool	KBMDB::doDropTable
	(	cchar	*,
		bool
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"drop",
			__ERRLOCN
		   )	;
	return	false	;
}


/*  ------------------------------------------------------------------- */

static	KBMDBQrySelect	*qrySelectList[64] ;

/*  KBMDBQrySelect							*/
/*  KBMDBQrySelect							*/
/*		: Constructor for select query object			*/
/*  server	: KBMDB *	  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  query	: const QString & : Select query			*/
/*  (returns)	: KBMDBQrySelect:					*/

KBMDBQrySelect::KBMDBQrySelect
	(	KBMDB		*server,
		bool		data,
		const QString	&query
	)	
	:
	KBSQLSelect	(server, data, query),
	m_server	(server)
{
	m_nRows		= 0 ;
	m_nFields	= 0 ;
	m_sequence	= sizeof(qrySelectList)/sizeof(KBMDBQrySelect *) ;

	for (uint idx = 0 ; idx < sizeof(qrySelectList)/sizeof(KBMDBQrySelect *) ; idx += 1)
		if (qrySelectList[idx] == 0)
		{	qrySelectList[idx] = this ;
			m_sequence	   = idx  ;
			break	;
		}
}

/*  KBMDBQrySelect							*/
/*  ~KBMDBQrySelect							*/
/*		: Destructor for select query object			*/
/*  (returns)	:		:					*/

KBMDBQrySelect::~KBMDBQrySelect ()
{
	for (uint idx = 0 ; idx < sizeof(qrySelectList)/sizeof(KBMDBQrySelect *) ; idx += 1)
		if (qrySelectList[idx] == this)
		{	qrySelectList[idx] = 0 ;
			break	;
		}
}

static	bool	loadExpression
		(	const QString	&skeleton,
			cchar 		*which,
			uint		seq,
			uint		idx,
			const QString	&expression,
			KBError		&pError
		)
{
	QString	func	= QString(skeleton).arg(which).arg(seq).arg(idx).arg(expression) ;
	func		= func.replace("=", "==") ;
	func		= func.replace("'", "\"") ;

	DPRINTF
	((	"loadExpression:\n%s\n",
		func.latin1()
	))	;

	CBUFF	   *cbuff  = el_compile (0, 0, 0, func.latin1(), 0) ;
	if (cbuff == 0)
	{
		pError	= KBError
			   (	KBError::Error,
				"Expression compile error",
				expression,
				__ERRLOCN
			   )	;
		return	false	;
	}
	if (!el_loadtext (cbuff))
	{
		pError	= KBError
			   (	KBError::Error,
				"Expression load error",
				expression,
				__ERRLOCN
			   )	;
		return	false	;
	}

	return	true	;
}

static	KBValue	evalExpression
	(	cchar		*which,
		uint		seq,
		uint		idx,
		uint		cols,
		VALUE		*args,
		KBType		*&type
	)
{
	QString	fn	= QString("%1_%2_%3").arg(which).arg(seq).arg(idx) ;
        VALUE   res     = el_vexec ("", fn.latin1(), cols, args) ;
	KBValue	value	;

	switch	(res.tag->tag)
	{
		case V_NUM	:
			value	= QString::number (res.val.num) ;
			type	= &_kbFixed	;
			break	;

		case V_DBL	:
			value	= QString::number (res.val.dbl) ;
			type	= &_kbFloat	;
			break	;

		case V_STR	:
			value	= res.val.str->text ;
			type	= &_kbString	;
			break	;

		default		:
			DPRINTF
			((	"evalExpression: unexpected tag [%d]\n",
				res.tag->tag
			))	;
			value	= QString::null ;
			type	= &_kbString	;
					break	;
	}

	DPRINTF
	((	"evalExpression: %3d: [%s]\n",
		col,
		value.getRawText().latin1()
	))	;
	return	value	;
}

/*  KBMDBQrySelect							*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBMDBQrySelect::execute
	(	uint		nvals,
		const KBValue	*values
	)
{
	KBBaseSelect	select	;
	KBDataBuffer	exeSQL	;

	if (!m_server->subPlaceList
		(	m_rawQuery,
			nvals,
			values,
			exeSQL,
			m_codec,
			m_lError
		))
		return false	;

	if (!select.parse (QString(exeSQL.data()), 0))
	{
		m_lError = select.lastError() ;
		return	 false	;
	}

	if
	(	(select.orderList ().count() > 0) ||
		(select.groupList ().count() > 0) ||
		(select.havingList().count() > 0)
	)
	{
		m_lError = KBError
			   (	KBError::Error,
				"MDB driver does not support 'order', 'group', or 'having'",
				exeSQL.data(),
				__ERRLOCN
			   )	;
	}
	if (select.tableList().count() > 1)
	{
		m_lError = KBError
			   (	KBError::Error,
				"MDB driver does not support multi-table selects",
				exeSQL.data(),
				__ERRLOCN
			   )	;
	}

	m_table	= select.tableList()[0].tableName() ;

	MdbTableDef *tableDef = mdb_read_table_by_name
				(	m_server->handle(),
					(char *)m_table.latin1(),
					MDB_TABLE
				)	;
	if (tableDef == 0)
	{	m_lError = KBError
			   (	KBError::Error,
				"Cannot get table details",
				m_table,
				__ERRLOCN
			   )	;
		return	false	;
	}

	GPtrArray   *columns  = mdb_read_columns(tableDef) ;
	if (columns  == 0)
	{	m_lError = KBError
			   (	KBError::Error,
				"Cannot get table columns",
				m_table,
				__ERRLOCN
			   )	;
		mdb_free_tabledef (tableDef) ;
		return	false	;
	}

	QStringList	colNames;
	cchar		*sep		= ""	;
	QString		skeleton	=
					"global ntos ;\n"
					"global ston ;\n"
					"public %1_%2_%3 ("
					;

	for (uint idx = 0 ; idx < tableDef->num_cols ; idx += 1)
	{
		MdbColumn *column = (MdbColumn *)g_ptr_array_index (columns, idx) ;
		skeleton  += sep ;
		skeleton  += column->name ;
		sep	   =  "," ;
		colNames.append (column->name) ;
	}

	skeleton +=
		")\n"
		"{\n"
		"	return %4 ;\n"
		"}\n"
		;

	DPRINTF
	((	"KBMDBQrySelect::execute:\n%s\n",
		skeleton.latin1()
	))	;

	uint	place	= 0 ;

	for (uint idx = 0 ; idx < select.fetchList().count() ; idx += 1)
	{
		QString	expr	= select.fetchList()[idx].expr() ;

		if (!loadExpression
			(	skeleton,
				"fetch",
				m_sequence,
				idx,
				expr.replace(m_table + ".", ""),
				m_lError
			))
		{
			mdb_free_tabledef(tableDef) ;
			return	false	;
		}

		m_fields.append (expr)	;
	}

	for (uint idx = 0 ; idx < select.whereList().count() ; idx += 1)
	{
		if (!loadExpression
			(	skeleton,
				"where",
				m_sequence,
				idx,
				select.whereList()[idx].expr(0, place).replace(m_table + ".", ""),
				m_lError
			))
		{
			mdb_free_tabledef(tableDef) ;
			return	false	;
		}
	}

	mdb_rewind_table (tableDef) ;

	char 	**bound_values	= (char **)calloc(tableDef->num_cols, sizeof(char *)) ;
	int 	*bound_lengths	= (int   *)calloc(tableDef->num_cols, sizeof(int   )) ;

	for (uint col = 0 ; col < tableDef->num_cols ; col += 1)
	{	bound_values[col] = (char *)calloc(1, MDB_BIND_SIZE) ;
		mdb_bind_column (tableDef, col + 1, bound_values[col], &bound_lengths[col]) ;
	}

	VALUE	*args	= new VALUE[tableDef->num_cols] ;
	uint	row	= 0 ;
	KBType	**types	= 0 ;

	m_nFields	= select.fetchList().count()	;

	while (mdb_fetch_row(tableDef))
	{
		DPRINTF
		((	"KBMDBQrySelect::execute: got row\n"
		))	;

		for (uint idx = 0 ; idx < tableDef->num_cols ; idx += 1)
		{
			MdbColumn *col	= (MdbColumn *)g_ptr_array_index(tableDef->columns, idx) ;

			switch (col->col_type)
			{
				case MDB_BOOL	  :
				case MDB_BYTE	  :
				case MDB_INT      :
				case MDB_LONGINT  :
					args[idx] = QString(bound_values[idx]).toInt   () ;
					break	  ;

				case MDB_FLOAT	  :
				case MDB_DOUBLE	  :
				case MDB_MONEY	  :
					args[idx] = QString(bound_values[idx]).toDouble() ;
					break	  ;

				case MDB_TEXT	  :
				case MDB_SDATETIME:
				case MDB_MEMO	  :
				case MDB_OLE	  :
					args[idx] = new STRING(bound_values[idx]) ;
					break	  ;

				default	:
					args[idx] = new STRING("") ;
					break	  ;
			}
		}

		bool	include	= true	;

		for (uint where = 0 ; where < select.whereList().count() ; where += 1)
		{
			KBType	*type	;
			KBValue	value	;
			value	= evalExpression
					(	"where",
						m_sequence,
						where,
						tableDef->num_cols,
						args,
						type
					)	;
			if (!value.isTrue())
			{	include	= false ;
				break	;
			}
		}

		if (!include)
			continue ;

		if (m_types == 0)
			m_types	= types	= new KBType *[m_nFields] ;

		for (uint col = 0 ; col < select.fetchList().count() ; col += 1)
		{
			KBType	*type	;
			KBValue	value	;

			value	= evalExpression
					(	"fetch",
						m_sequence,
						col,
						tableDef->num_cols,
						args,
						type
					)	;

			if (types != 0) types[col] = type ;
			putInCache (row, col, value) ;
		}

		row	+= 1 ;
	}

	m_nRows	= row	;

	if (m_types == 0)
	{
		m_types	= new KBType *[m_nFields] ;
		for (uint col = 0 ; col < m_nFields ; col += 1)
			m_types[col] = &_kbString ;
	}

	delete [] args	;
	free		 (bound_values ) ;
	free		 (bound_lengths) ;
	mdb_free_tabledef(tableDef     ) ;

	return	true	;
}

/*  KBMDBQrySelect							*/
/*  getField	: Get a specified field from the query results		*/
/*  qrow	: uint		: Row number				*/
/*  qcol	: uint		: Column number				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBMDBQrySelect::getField
	(	uint		qrow,
		uint		qcol,
		KBValue::VTrans
	)
{
	KBValue	value	;
	getFromCache	(qrow, qcol, value) ;
	return	value	;
}

/*  KBMDBQrySelect							*/
/*  getFieldname: Get a specified field name from the query results	*/
/*  qcol	: uint		: Column number				*/
/*  (returns)	: QString	: Name					*/

QString	KBMDBQrySelect::getFieldName
	(	uint	qcol
	)
{
	/* First check that the request is within the range of rows	*/
	/* and fields returned by the query.				*/
	if (qcol >= m_nFields) return QString() ;

	return	m_fields[qcol] ;
}


/*  KBMDBQryUpdate							*/
/*  KBMDBQryUpdate							*/
/*		: Constructor for update query object			*/
/*  server	: KBMDB *	  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  query	: const QString & : Update query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KBMDBQryUpdate:					*/

KBMDBQryUpdate::KBMDBQryUpdate
	(	KBMDB		*server,
		bool		data,
		const QString	&query,
		const QString	&tabName
	)	
	:
	KBSQLUpdate (server, data, query, tabName),
	m_server    (server)
{
	m_nRows	= 0 ;
}

/*  KBMDBQryUpdate							*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBMDBQryUpdate::execute
	(	uint		,
		const KBValue	*
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"MDB driver is read-only, 'update' not supported",
			QString::null,
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBMDBQryUpdate							*/
/*  ~KBMDBQryUpdate							*/
/*		: Destructor for update query object			*/
/*  (returns)	:		:					*/

KBMDBQryUpdate::~KBMDBQryUpdate ()
{
}


/*  KBMDBQryInsert							*/
/*  KBMDBQryInsert							*/
/*		: Constructor for insert query object			*/
/*  server	: KBMDB *	  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  tabName	: const QString & : Table being updated			*/
/*  query	: const QString & : Insert query			*/
/*  (returns)	: KBMDBQryInsert:					*/

KBMDBQryInsert::KBMDBQryInsert
	(	KBMDB		*server,
		bool		data,
		const QString	&query,
		const QString	&tabName
	)	
	:
	KBSQLInsert (server, data, query, tabName),
	m_server    (server)
{
	m_nRows	= 0 ;
}

/*  KBMDBQryInsert							*/
/*  ~KBMDBQryInsert							*/
/*		: Destructor for insert query object			*/
/*  (returns)	:		:					*/

KBMDBQryInsert::~KBMDBQryInsert ()
{
}

/*  KBMDBQryInsert							*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBMDBQryInsert::execute
	(	uint		,
		const KBValue	*
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"MDB driver is read-only, 'insert' not supported",
			QString::null,
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBMDBQryInsert							*/
/*  getNewKey	: Get new insert key					*/
/*  primary	: const QString & : Key column name			*/
/*  _newKey	: KBValue &	  : New key				*/
/*  prior	: bool		  : Pre-insert call			*/
/*  (returns)	: bool		  : Success				*/

bool	KBMDBQryInsert::getNewKey
	(	const QString	&,
		KBValue		&,
		bool		
	)
{
	return	false	;
}

/*  KBMDBQryDelete							*/
/*  KBMDBQryDelete							*/
/*		: Constructor for delete query object			*/
/*  server	: KBMDB *	  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  query	: const QString & : Delete query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KBMDBQryDelete:					*/

KBMDBQryDelete::KBMDBQryDelete
	(	KBMDB		*server,
		bool		data,
		const QString	&query,
		const QString	&tabName
	)	
	:
	KBSQLDelete (server, data, query, tabName),
	m_server    (server)
{
	m_nRows	= 0 ;
}

/*  KBMDBQryDelete							*/
/*  ~KBMDBQryDelete							*/
/*		: Destructor for delete query object			*/
/*  (returns)	:		:					*/

KBMDBQryDelete::~KBMDBQryDelete ()
{
}

/*  KBMDBQryDelete							*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBMDBQryDelete::execute
	(	uint		,
		const KBValue	*
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"MDB driver is read-only, 'delete' not supported",
			QString::null,
			__ERRLOCN
		   )	;
	return	false	;
}


KBFACTORY
(	KBMDBFactory,
	"driver_mdb"
)

KBFACTORYIMPL
(	KBMDBFactory,
	driver_mdb
)

QObject	*KBMDBFactory::create
	(	QObject		  *parent,
		cchar		  *object,
		cchar		  *,
		const QStringList &
	)
{
	extern	void	el_initialize (uint, uint, bool) ;
	el_initialize (1024 * 64, 4096, false) ;

	mdb_init	() ;
	mdb_set_date_fmt("%F %T") ;

	if (dIdentToType.count() == 0)
		for (uint idx = 0 ; idx < sizeof(typeMap)/sizeof(MDBTypeMap) ; idx += 1)
			if (typeMap[idx].ident != FIELD_TYPE_UNSET)
			{
				MDBTypeMap *m = &typeMap[idx] ;
				dIdentToType.insert (m->ident, m) ;
			}

	if ((parent != 0) && !parent->inherits ("QWidget"))
	{
		fprintf	(stderr, "KBMDBFactory: parent does not inherit QWidget\n") ;
		return	0  ;
	}

	if (strcmp (object, "driver"  ) == 0) return new KBMDB () ;

	if (strcmp (object, "advanced") == 0) return 0 ;

	return	0 ;
}

QString	KBMDBFactory::ident ()
{
	return	QString("MDB: %1%2 %3")
			.arg(__KB_BUILD_RELEASE)
			.arg(__KB_BUILD_BUILD  )
			.arg(__KB_BUILD_TIME   )
			;
}
