/***************************************************************************
    file	         : kb_myadvanced.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/




#include	"kb_classes.h"
#include	"kb_myadvanced.h"

#include	"rk_tabwidget.h"

#define		__DEBUG_MYADVANCED	(0)
#if		__DEBUG_MYADVANCED
#include	"kb_dprintf.h"
#endif

/*  KBMyAdvanced							*/
/*  KBMyAdvanced: Constructor for MySQL advanced options object		*/
/*  (returns)	: KBMyAdvanced	:					*/

KBMyAdvanced::KBMyAdvanced  ()
	:
	KBDBAdvanced ("mysql")
{
	DPRINTF
	((	"KBMyAdvanced::KBMyAdvanced: called\n"
	))	;

	m_ignoreCharset	= false	;
	m_foundRows	= false	;
	m_compressed	= false ;
}

/*  KBMyAdvanced							*/
/*  ~KBMyAdvanced: Destructor for MySQL advanced options object		*/
/*  (returns)	 :		:					*/

KBMyAdvanced::~KBMyAdvanced ()
{
}

/*  KBMyAdvanced							*/
/*  load	: Load settings from DOM element			*/
/*  driverElem	: QDomElement &	: DOM element				*/
/*  (returns)	: void		:					*/

void	KBMyAdvanced::load
	(	const QDomElement &driverElem
	)
{
	m_ignoreCharset	= driverElem.attribute ("ignorecharset").toUInt() ;
	m_foundRows	= driverElem.attribute ("foundrows"    ).toUInt() ;
	m_compressed	= driverElem.attribute ("compressed"   ).toUInt() ;
}

/*  KBMyAdvanced							*/
/*  save	: Save settings from DOM element			*/
/*  driverElem	: QDomElement &	: DOM element				*/
/*  (returns)	: void		:					*/

void	KBMyAdvanced::save
	(	QDomElement	&driverElem
	)
{
	driverElem.setAttribute ("ignorecharset",  m_ignoreCharset) ;
	driverElem.setAttribute ("foundrows",      m_foundRows    ) ;
	driverElem.setAttribute ("compressed",     m_compressed   ) ;
}

/*  KBMyAdvanced							*/
/*  setupDialog	: Set up advanced options dialog			*/
/*  tabWidget	: RKTabWidget *	: Tab widget to which to attach pages	*/
/*  (returns)	: void		:					*/

void	KBMyAdvanced::setupDialog
	(	RKTabWidget	*tabWidget
	)
{
	DPRINTF
	((	"KBMyAdvanced::setupDialog: called\n"
	))	;

	QWidget		*advGeneral	= new QWidget	  (tabWidget ) ;
	QVBoxLayout	*layGeneral	= new QVBoxLayout (advGeneral) ;

	tabWidget->addTab (advGeneral, "MySQL") ;

	m_cbIgnoreCharset	= new QCheckBox (advGeneral) ;
	m_cbIgnoreCharset	->setText	(TR("Ignore MySQL character set")) ;
	m_cbIgnoreCharset	->setChecked	(m_ignoreCharset) ;

	m_cbFoundRows		= new QCheckBox (advGeneral) ;
	m_cbFoundRows		->setText	(TR("Return found row count in update")) ;
	m_cbFoundRows		->setChecked	(m_foundRows	) ;

	m_cbCompressed		= new QCheckBox (advGeneral) ;
	m_cbCompressed		->setText	(TR("Request compressed connection")) ;
	m_cbCompressed		->setChecked	(m_compressed   ) ;

	layGeneral->addWidget (m_cbIgnoreCharset) ;
	layGeneral->addWidget (m_cbFoundRows	) ;
	layGeneral->addWidget (m_cbCompressed	) ;
	layGeneral->addStretch() ;
}

/*  KBMyAdvanced							*/
/*  saveDialog	: Save results from dialog				*/
/*  (returns)	: void		:					*/

void	KBMyAdvanced::saveDialog ()
{
	m_ignoreCharset = m_cbIgnoreCharset ->isChecked() ;
	m_foundRows	= m_cbFoundRows	    ->isChecked() ;
	m_compressed	= m_cbCompressed    ->isChecked() ;
}

/*  KBMyAdvanced							*/
/*  copy	: Replicate this object					*/
/*  (returns)	: KBMyAdvanced * : Replicated object			*/

KBDBAdvanced
	*KBMyAdvanced::copy ()
{
	KBMyAdvanced	*copy	= new KBMyAdvanced ()	;

	copy->m_ignoreCharset	= m_ignoreCharset	;
	copy->m_foundRows	= m_foundRows		;
	copy->m_compressed	= m_compressed		;

	return	copy	;
}
