/***************************************************************************
    file		: kb_sqlite3.h
    copyright		: (C) 1999,2000,2001,2002,2003 by Mike Richardson
			  (C) 2001,2002,2003 by John Dean
			: (C) 2004 by Brian Harring
    license             : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email               : mike@quaking.demon.co.uk 
			  john@totalrekall.co.uk
 ***************************************************************************/

#ifndef		__KB_SQLITE3_H
#define		__KB_SQLITE3_H

#ifdef 		_WIN32
#include 	<windows.h>
#include	<io.h>
#endif

#include	<qstring.h>
#include	<qcstring.h>
#include	<qdict.h>
#include	<qtextcodec.h>


#include	<sqlite3.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_databuffer.h"
#include	"kb_database.h"
#include	"kb_serverinfo.h"

#include	"kb_libloader.h"


#define	FIELD_TYPE_UNSET	(-1)

#define SQLITE_ROWID_NAME	"ROWID"

struct	SQLite3TypeMap
{	
	int		ident	  ;	/* SQLite identifier		*/
	KB::IType	itype	  ;	/* Internal type		*/
	char		mtype[16] ;	/* Sqlite type name		*/
	uint		flags	  ;	/* Various flags		*/
}	;

/*  KBSQLite3Type							*/
/*  ------------							*/
/*  This is the derived type for yhe SQLite interface. It stores	*/
/*  the SQLite type as well as the internal type.			*/

class	KBSQLite3Type : public KBType
{
	SQLite3TypeMap*	m_typeInfo	;

public:

	KBSQLite3Type	(SQLite3TypeMap *, uint, uint, bool) ;

	virtual	bool	isValid		(const QString &, KBError  &, const QString & = QString::null ) ;
	virtual	void	getQueryText	(KBDataArray   *, KBShared *, KBDataBuffer  &, QTextCodec* = 0) ;
	virtual	void	escapeText	(KBDataArray   *,	      KBDataBuffer &) ;
	virtual	void	escapeText	(QCString      &,	      KBDataBuffer &) ;

	void		retype		(int) ;
}	;


/*  KBSQLite3								*/
/*  ---------								*/
/*  Implementation class for the interface to the SQLite database	*/

class	KBSQLite3 : public KBServer
{
	sqlite3		*m_sqlite3	;
	bool		m_readOnly	;

	bool		doListTables	(KBTableDetailsList &, uint)	;
	bool		tblCreateSQL	(QList<KBFieldSpec> &, const QString &, QString &, bool) ;

	virtual	bool	doConnect	(KBServerInfo *)		;
	virtual	bool	doListFields	(KBTableSpec  &)		;
	virtual	bool	doCreateTable	(KBTableSpec  &,  bool, bool = false) ;
	virtual	bool	doRenameTable	(cchar *, cchar *, bool )	;
	virtual	bool	doDropTable	(cchar *, bool		)	;

public	:


	KBSQLite3		()	;
	virtual	~KBSQLite3	()	;


	virtual	KBSQLSelect	*qrySelect	(bool, const QString &, bool = false   ) ;
	virtual	KBSQLUpdate	*qryUpdate	(bool, const QString &, const QString &) ;
	virtual	KBSQLInsert	*qryInsert	(bool, const QString &, const QString &) ;
	virtual	KBSQLDelete	*qryDelete	(bool, const QString &, const QString &) ;

	virtual	bool 		command		(bool, const QString &, uint, KBValue *, KBSQLSelect ** = 0) ;

	bool 			bindParameters	(sqlite3_stmt  *, uint, const KBValue *, KBError &) ;
	bool 			execSQL		(const QString &, const QString &, QString &, uint, const KBValue *, QTextCodec *, cchar *, KBError &) ;

	virtual void		getQueryText	(const KBValue	*value,
						 KBDataBuffer	&buffer,
						 QTextCodec	*codec
						)	;

	virtual	QString		listTypes	()	;
	virtual	bool 		tableExists	(const QString &, bool &) ;
	virtual	uint 		optionFlags	()	;

	inline	sqlite3  *handle ()
	{
		return	 m_sqlite3 ;
	}
}	;


/*  KBSQLite3QrySelect							*/
/*  ------------------							*/
/*  Implementation class for select queries on the SQLite database	*/

class	KBSQLite3QrySelect : public KBSQLSelect
{
	KBSQLite3	*m_server	;
	QStringList	m_colnames	;
	QValueList<int>	m_coltypes	;

public	:

	KBSQLite3QrySelect		( KBSQLite3 *, bool, const QString & );
	virtual ~KBSQLite3QrySelect	();

	virtual	bool	execute		(uint, const KBValue *) ;
	virtual	QString	getFieldName	(uint) ;
	virtual	KBValue	getField	(uint, uint, KBValue::VTrans) ;
}	;


/*  KBSQLite3QryUpdate							*/
/*  ------------------							*/
/*  Implementation class for update queries on the SQLite database	*/

class	KBSQLite3QryUpdate : public KBSQLUpdate
{
	KBSQLite3	*m_server	;

public:

	KBSQLite3QryUpdate 		(KBSQLite3 *, bool, const QString &, const QString &) ;
	virtual~KBSQLite3QryUpdate	() ;

	virtual	bool	execute		( uint, const KBValue * ) ;

};

/*  KBSQLite3QryInsert							*/
/*  ------------------							*/
/*  Implementation class for insert queries on the SQLite database	*/

class	KBSQLite3QryInsert : public KBSQLInsert
{
	KBSQLite3	*m_server	;
	QString		m_autocol	;
	int		m_rowid		;

public:

	KBSQLite3QryInsert		(KBSQLite3 *, bool, const QString &, const QString &) ;
	virtual ~KBSQLite3QryInsert	() ;

	virtual	bool	execute		(uint, const KBValue *) ;
	virtual	bool	getNewKey	(const QString &, KBValue &, bool) ;
}	;

/*  KBSQLite3QryDelete							*/
/*  ------------------							*/
/*  Implementation class for delete queries on the SQLite database	*/

class	KBSQLite3QryDelete : public KBSQLDelete
{
	KBSQLite3		*m_server	  ;
	bool			m_whereHackNeeded ;

public	:

	KBSQLite3QryDelete		(KBSQLite3 *, bool, const QString &, const QString &) ;
	virtual~KBSQLite3QryDelete	() ;

	virtual bool	execute		(uint, const KBValue *) ;
}	;


#endif	// __KB_SQLITE3_H
