/***************************************************************************
    file	         : kb_about.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdarg.h>
#include	<stdlib.h>
#include	<errno.h>
#include	<time.h>

#include	<qptrlist.h>
#include	<qdict.h>
#include	<qevent.h>
#include	<qwidget.h>
#include	<qwidgetlist.h>
#include	<qguardedptr.h>
#include	<qapplication.h>
#include 	<qpalette.h>
#include 	<qtimer.h>


#include	"kb_classes.h"
#include	"kb_gui.h"
#include	"kb_appptr.h"
#include	"kb_callback.h"
#include	"kb_director.h"
#include	"kb_manual.h"

#include	"kb_dbdlg.h"
#include	"kb_serverinfo.h"
#include	"kb_script.h"

#include	"kbase.h"

static	cchar	*licenseText	=

	"<qt>"
	"<p><b>Copyright</b></p>"
	"<p>"
	"<table>"
	"<tr><td width='50'></td><td>(C) 1999,2000,2001,2002,2003,2004,2005 by Mike Richardson</td></tr>"
	"<tr><td width='50'></td><td>(C) 2001,2002,2003,2004,2005 by John Dean</td></tr>"
	"</table>"
	"</p>"
	"<p>"
	"Except for the parts listed below, this program is released "
	"under the terms of the GNU General Public License, version 2. "
	"The copyright holders retain the right to release this code "
	"under different non-exclusive licences. "
	"</p>"
	"<p>"
	"If included, the KJS interpreter is licensed under the terms of "
	"the GNU Lesser General Public License, version 2."
	"</p>"
	"<p>"
	"EMail: mike@quaking.demon.co.uk"
	"</p>"
	"</qt>"
	;

static	QString	pluginsText ()
{
	return	QString
		(	"<qt>"
			"<p>"
			"<table>"
			"<tr><td>Program<t/d><td><b>%1</b></td><td>%2%3 %4</td></tr>"
			"%5"
			"%6"
			"%7"
			"</table>"
			"</p>"
			"</qt>"
		)
#if	__KB_RUNTIME
		.arg("RekallRT")
#else
		.arg("Rekall")
#endif
		.arg(__KB_BUILD_RELEASE)
		.arg(__KB_BUILD_BUILD  )
		.arg(__KB_BUILD_TIME   )
		.arg(KBDBaseDlg  ::getIdentStrings())
		.arg(KBServerInfo::getIdentStrings())
		.arg(KBScriptIF  ::getIdentStrings()) ;
}

void	KBaseApp::showAboutApplication ()
{
	showAboutDialog (false) ;
}

#if	__KB_KDE

#include	<kmenubar.h>
#include	<kpopupmenu.h>
#include	<kaboutapplication.h>

void	KBaseApp::fixHelpMenu
	(	KBDirector	&director
	)
{
	KMenuBar *mb = menuBar() ;
	int	 mid ;

	for (uint idx = 0 ; idx < mb->count() ; idx += 1)
	    if ((mid = mb->idAt (idx)) != -1)
		if (mb->text(mid) == i18n("&Help"))
		{	mb->removeItem (mid) ;
			mb->insertItem (i18n("&Help"), customHelpMenu()) ;
			customHelpMenu()->insertSeparator () ;
			customHelpMenu()->insertItem  (getSmallIcon  ("rekall"), TR("Rekall Manual" ), &director, SLOT(showManual	())) ;
			customHelpMenu()->insertItem  (getSmallIcon  ("rekall"), TR("Rekall Plugins"), &director, SLOT(showHelpPlugins	())) ;
			break	;
		}


}

void	KBaseApp::showAboutDialog
	(	bool	showPlugins
	)
{
	KAboutApplication aboutApp
			   (	instance()->aboutData(),
				this,
				"about",
				true
			   )	;

	aboutApp.addLicensePage
	(	TR("License"),
		licenseText
	)	;

	QWidget	*plugins = aboutApp.addTextPage
			   (	TR("Plugins"),
			   	pluginsText(),
			   	true
			   )	;


	if (showPlugins)
		aboutApp.showPage (aboutApp.pageIndex (plugins)) ;
	aboutApp.exec ()	;
}

#endif	// __KB_KDE

#if	__KB_TKC

#include	<qtabdialog.h>
#include	<qtextview.h>
#include	<qtextbrowser.h>

static	cchar	*aboutText	=
	"<qt>"
	"<p><b>RekallQT</b>: A QT database front end</p>"
	"<p>Release %1</p>"
	"<p>(C) 1999,2000,2001,2002,2003,2004,2005</p>"
	"</qt>"
	;

static	cchar	*authorsText	=
	"<qt>"
	"<table>"
	"<tr><td><b>Authors</b></td><td></td></tr>"
	"<tr><td>Mike Richardson</td><td><a href=\"mailto:mike@quaking.demon.co.uk\">mike@quaking.demon.co.uk</a></td></tr>"
	"<tr><td>John Dean</td><td><a href=\"mailto:john@totalrekall.co.uk\">john@totalrekall.co.uk</a></td></tr>"
	"<tr><td><b>Major Contributors</b></td><td></td></tr>"
	"<tr><td>Abel Deuring</td><td></td></tr>"
	"<tr><td>Brian Harring</td><td></td></tr>"
	"<tr><td>Richard Shaw</td><td></td></tr>"
	"<tr><td>Tim Southerwood</td><td></td></tr>"
	"</table>"
	"<p>"
	"The KJS interpreter included with acknowledgement to Harri Porten (porten@kde.org) "
	"and Apple Computer, Inc."
	"</p>"
	"<p>"
	"</p>"
	"</qt>"
	;


class	KBAboutDlg : public QTabDialog
{
public	:

	inline	KBAboutDlg ()
		:
		QTabDialog (0, 0, true)
	{
	}

	void	show ()	;
}	;

void	KBAboutDlg::show ()
{
	QTabDialog::show()	;
}



void	KBaseApp::fixHelpMenu
	(	KBDirector	&
	)
{
}

void	KBaseApp::showAboutDialog
	(	bool	showPlugins
	)
{
	KBAboutDlg	aboutApp ;
	QTextView	*plugins ;

	plugins 	= new QTextView	(&aboutApp) ;

	plugins->setTextFormat(Qt::RichText ) ;
	plugins->setText      (pluginsText()) ;

	QTextView	*about	 ;
	QTextView	*authors ;
	QTextView	*license ;

	about		= new QTextView	(&aboutApp) ;
	authors 	= new QTextView	(&aboutApp) ;
	license 	= new QTextView (&aboutApp) ;

	about	->setTextFormat	(Qt::RichText)	;
	about   ->setText	(QString(aboutText).arg(__KB_BUILD_RELEASE)) ;
	authors ->setTextFormat	(Qt::RichText)	;
	authors ->setText	(authorsText )	;

	license ->setTextFormat	(Qt::RichText)	;
	license ->setText	(licenseText )	;

	aboutApp.addTab   (about,   TR("About"  )) ;
	aboutApp.addTab   (authors, TR("Authors")) ;
	aboutApp.addTab   (license, TR("License")) ;
	aboutApp.addTab   (plugins, TR("Plugins")) ;

	aboutApp.showPage (showPlugins ? (QWidget *)plugins : (QWidget *)about) ;

	plugins ->setMinimumSize (500, 250) ;

	aboutApp.exec () ;
}

#endif	// __KB_TKC

void	KBaseApp::showManual ()
{
	KBManual::self()->showContents() ;
}
