/***************************************************************************
    file	         : kb_dbaseviewer.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_gui.h"
#include	"kb_dbdlg.h"
#include	"kb_viewer.h"
#include	"kbase.h"

#if		__KB_AUTOMOC
#include	"kb_dbaseviewer.moc"
#else
#include	"kb_dbaseviewer.h"
#endif

#include	"tk_icons.h"


#define		__DEBUG_DBASEVIEWER	(0)
#if		__DEBUG_DBASEVIEWER
#include	"kb_dprintf.h"
#endif

static	QPtrList<KBDBaseViewer>	dbViewers	;

#if	! __KB_RUNTIME
extern	TKActionMenu		*actDatabases	;
#endif

extern	void		setActiveWindow	(QWidget *) ;

/*  KBDBaseViewer							*/
/*  findKBaseApp: Find top level window for specified location		*/
/*  location	: KBLocation &	: Location				*/
/*  (returns)	: KBaseApp *	: Top level window			*/

KBaseApp*KBDBaseViewer::findKBaseApp
	(	const KBLocation	&location
	)
{
	LITER
	(	KBDBaseViewer,
		dbViewers,
		dbViewer,

		if (dbViewer->m_dbaseDlg->isShowing (location))
			return	dbViewer->m_kbaseApp ;
	)

	return	dbViewers.at(0)->m_kbaseApp ;
}


/*  ------------------------------------------------------------------  */

/*  KBDBaseViewer							*/
/*  KBDBaseViewer: Constructor for KBDBaseViewer class			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  app		: KBaseApp *	: Parent application widget		*/
/*  dbName	: const QString&: Database file name			*/
/*  create	: bool		: True to create a new database		*/
/*  (returns)	: KBDBaseViewer	:					*/

KBDBaseViewer::KBDBaseViewer
	(	QWidget		*parent,
		KBaseApp	*app,
		const QString	&dbName,
		bool		create
	)
	:
	KBasePart	(0, parent, WStyle_NormalBorder),
	m_parent	(parent),
	m_kbaseApp	(app)
{
	m_dbaseGUI  = new KBaseGUI   (this, this, "rekallui_dbase.gui") ;
	setGUI  (m_dbaseGUI)	 ;
	m_dbaseDlg  = new KBDBaseDlg (m_partWidget, app, dbName, create, m_dbaseGUI) ;
	m_topWidget = m_dbaseDlg ;

	QString	name	= m_dbaseDlg->getDBName() ;
	int	i1	= name.findRev ('/')	;
	if (i1 >= 0) name = name.mid   (i1 + 1) ;
	int	i2	= name.findRev ('.')	;
	if (i2 >= 0) name = name.left  (i2)	;

	m_topWidget->show () ;

	QSize	topSize = m_topWidget->minimumSize() ;
	m_partWidget->resize	     (topSize.width(), topSize.height(), true, false) ;
	m_partWidget->setMinimumSize (topSize.width(), topSize.height()) ;
	m_partWidget->show	     () ;

	m_partWidget->setCaption(name) ;
	m_partWidget->setIcon   (getSmallIcon("rekall")) ;

	if (parent == app)
	{	app->setCaption (name) ;
		app->setIcon	(getSmallIcon("rekall")) ;
	}

	dbViewers.append (this) ;

	m_showDatabase = new TKAction
		   	 (	name,
				"database",
				0,
				this,
				SLOT(showDatabase()),
				this,
				name
		   	 )	;
#if	! __KB_RUNTIME
	actDatabases->insert (m_showDatabase) ;
#endif
}

/*  KBDBaseViewer							*/
/*  ~KBDBaseViewer: Destructor for KBDBaseViewer class			*/
/*  (returns)	  :							*/

KBDBaseViewer::~KBDBaseViewer ()
{
	ACTION	*actObjects = action ("KB_showObjects"  ) ;
	ACTION	*actClose   = action ("KB_closeDatabase") ;

	if (actObjects != 0) actObjects->unplugAll() ;
	if (actClose   != 0) actClose  ->unplugAll() ;

	DPRINTF
	((	"KBDBaseViewer::~KBDBaseViewer: %d objects remain\n",
		m_objects.count()
	))	;

	/* Close all objects associated with this viewer. The loop is	*/
	/* constructed this way so that it is not sensitive to removal	*/
	/* of entries, or deletion of other objects, while within the	*/
	/* delete call.							*/
	while (m_objects.count() > 0)
		delete	m_objects.take(0) ;

#if	! __KB_RUNTIME
	actDatabases->remove  (m_showDatabase) ;
#endif
	dbViewers.remove (this) ;

	if (m_partWidget)
		m_partWidget->setCaption("") ;

	if (m_parent == m_kbaseApp)
		m_kbaseApp  ->setCaption("") ;

	DELOBJ	(m_dbaseDlg) ;
}

/*  KBDBaseViewer							*/
/*  queryClose	: Check for close					*/
/*  (returns)	: void		:					*/

bool	KBDBaseViewer::queryClose ()
{
	return	true	;
}

/*  KBDBaseViewer							*/
/*  addObject	: Potentially add child object to this viewer		*/
/*  location	: KBLocation &	: Location being displayed		*/
/*  object	: KBObjBase *	: New obect				*/
/*  (returns)	: bool		: Success				*/

bool	KBDBaseViewer::addObject
	(	const KBLocation	&location,
		KBObjBase		*object
	)
{
	if (m_dbaseDlg->isShowing (location))
	{
		DPRINTF
		((	"KBDBaseViewer::add [%p]\n",
			(void *)object
		))	;

		m_objects.append (object) ;

		connect
		(	object,
			SIGNAL	(destroyed   ()),
			SLOT	(removeObject())
		)	;

		return	true ;
	}

	return	false	;
}

/*  KBDBaseViewer							*/
/*  removeObject: Remove deceased object				*/
/*  (returns)	: void		:					*/

void	KBDBaseViewer::removeObject ()
{
	extern	bool	useMDI	;
	const	QObject *object = sender () ;

	m_objects.remove ((KBObjBase *)(object)) ;

	DPRINTF
	((	"KBDBaseViewer::removeObject [%p] useMDI=%d count=%d vis=%d\n",
		(void *)object,
		useMDI,
		m_objects.count(),
		m_kbaseApp->isVisible()
	))	;

	/* If the main application window is available, we are not in	*/
	/* MDI mode, and there are no objects left, then close the	*/
	/* application down. This means that the user can exit by	*/
	/* closing the last form (or report, ...)			*/
	if (m_kbaseApp->isAvailable())
		return ;

	if (useMDI || (m_objects.count() > 0))
		return	;

	m_kbaseApp->close () ;
}

/*  KBDBaseViewer							*/
/*  showDatabase: Make database viewer visible				*/
/*  (returns)	: void		:					*/

void	KBDBaseViewer::showDatabase ()
{
	extern	bool	useMDI ;

	if (useMDI)
	{	show () ;
		return	;
	}

	m_kbaseApp->show	     	() ;
	m_kbaseApp->raise		() ;
	m_kbaseApp->setActiveWindow	() ;
	::setActiveWindow		(m_kbaseApp) ;
	m_kbaseApp->setAvailable	(true) ;
}

void	KBDBaseViewer::closeDatabase ()
{
	m_kbaseApp->closeDatabase (this) ;
}


/*  KBDBaseViewer							*/
/*  showObjects	: Check quick objects dialog				*/
/*  (returns)	: bool		:					*/

void	KBDBaseViewer::showObjects ()
{
#if	! __KB_RUNTIME
	m_dbaseDlg->showObjects () ;
#endif
}

/*  KBDBaseViewer							*/
/*  showRawSQL	: Show raw SQL window					*/
/*  (returns)	: bool		:					*/

void	KBDBaseViewer::showRawSQL ()
{
#if	! __KB_RUNTIME
	m_dbaseDlg->showRawSQL  () ;
#endif
}

/*  KBDBaseViewer							*/
/*  dumpDatabase: Dump database						*/
/*  (returns)	: bool		:					*/

void	KBDBaseViewer::dumpDatabase ()
{
#if	! __KB_RUNTIME
	m_dbaseDlg->dumpDatabase  () ;
#endif
}

/*  KBDBaseViewer							*/
/*  loadDatabase: Dump database						*/
/*  (returns)	: bool		:					*/

void	KBDBaseViewer::loadDatabase ()
{
#if	! __KB_RUNTIME
	m_dbaseDlg->loadDatabase  () ;
#endif
}

/*  KBDBaseViewer							*/
/*  autoStart	: Check for auto-start forms				*/
/*  autoLoc	: KBLocation &	: Return started location if any	*/
/*  (returns)	: bool		:					*/

bool	KBDBaseViewer::autoStart
	(	KBLocation	&autoLoc
	)
{
	return	m_dbaseDlg->autoStart (autoLoc) ;
}

KBDBInfo*KBDBaseViewer::getDBInfo ()
{
	return	m_dbaseDlg->getDBInfo () ;
}
