/***************************************************************************
    file	         : kb_dbdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qiconset.h>

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_listitem.h"
#include	"kb_script.h"
#include	"kb_scriptlist.h"
#include	"kb_viewer.h"
#include	"kb_notifier.h"
#include	"kb_dom.h"
#include	"kb_prompt.h"

#include	"kb_svrchooser.h"
#include	"kb_dbdociter.h"
#include	"kb_ascii.h"

#include	"kb_skin.h"
#include	"kb_skindlg.h"

#if		__KB_AUTOMOC
#include	"kb_dbdlg.moc"
#else
#include	"kb_dbdlg.h"
#endif

#include	"rk_vbox.h"
#include	"rk_tabwidget.h"

#include	"tk_print.h"
#include	"tk_icons.h"


#define		__DEBUG_DBDLG	(0)
#if		__DEBUG_DBDLG
#include	"kb_dprintf.h"
#endif



/*  KBDBaseDlg								*/
/*  KBDBaseDlg	: Constructor for main database dialog			*/
/*  parent	: QWidget *	  : Parent widget			*/
/*  app		: KBaseApp *	  : Application root			*/
/*  dbName	: const QString&  : Database name			*/
/*  create	: bool		  : True to create new database		*/
/*  gui		: KBaseGUI *	  : GUI for this viewer			*/
/*  (returns)	: KBDBaseDlg	  :					*/

KBDBaseDlg::KBDBaseDlg
	(	QWidget		*parent,
		KBaseApp	*app,
		const QString	&dname,
		bool		create,
		KBaseGUI	*gui
	)
	:
	RKHBox		(parent),
	m_kbaseApp	(app),
	m_dirWatch	(2000),
	m_gui		(gui)
{
	setTracking ()	;
	m_window	= new KBListWidget (this) ;

	/* Create a database information structure appropriate to the	*/
	/* database, and then create the server dialog, which always	*/
	/* appears under the first tab. Then load the rest of the parts	*/
	m_dbInfo    = new KBDBInfoData   (dname) ;
	m_dbInfo->init (create) ;

	m_serverTab = new KBServerTab    (m_window, this, m_dbInfo, gui) ;

	m_window->addPage
	(	m_serverTab,
		TR("Servers"),
		getSmallIcon ("rekall")
	)	;

	loadParts () ;

	m_allParts.append
	(	KBDBDlgPluginItem
		(	0,
			"graphic",
			QString::null,
			"Graphics",
			QString::null,
			0
	)	)	;
	m_allParts.append
	(	KBDBDlgPluginItem
		(	0,
			"print",
			"prn",
			"Printers",
			QString::null,
			0
	)	)	;
	m_allParts.append
	(	KBDBDlgPluginItem
		(	0,
			"skin",
			"skn",
			"Skins",
			QString::null,
			0
	)	)	;

	KBLocation::registerType
	(	"print",
		"prn",
		"Printer",
		0
	)	;
	KBLocation::registerType
	(	"graphic",
		QString::null,
		"Graphic",
		0
	)	;
	KBLocation::registerType
	(	"skin",
		"skn",
		"Skin",
		0
	)	;


	connect
	(	&m_dirWatch,
		SIGNAL (dirty   (const QString &)),
		SLOT   (dirDirty(const QString &))
	)	;
	connect
	(	m_window,
		SIGNAL(currentChanged(QWidget *)),
		SLOT  (pageChanged   (QWidget *))
	)	;

	m_dirWatch.addDir (m_dbInfo->getDBPath ()) ;
}

/*  KBDBaseDlg								*/
/*  ~KBDBaseDlg	: Destructor for main database dialog			*/
/*  (returns)	:		:					*/

KBDBaseDlg::~KBDBaseDlg ()
{
	delete	m_dbInfo     ;
}

/*  KBDBaseDlg								*/
/*  dirDirty	: Signal that directory has changed			*/
/*  dname	: const QString & : Directory name			*/
/*  (returns)	: void		  :					*/

void	KBDBaseDlg::dirDirty
	(	const QString	&
	)
{
	KBLocation location (m_dbInfo, "all", KBLocation::m_pFile, "") ;
	for (uint idx = 0 ; idx < m_allParts.count() ; idx += 1)
		if (m_allParts[idx].m_list != 0)
			m_allParts[idx].m_list->objChange (location) ;
}

/*  KBDBaseDlg								*/
/*  autoStart	: Check for auto-start forms				*/
/*  autoLoc	: KBLocation &	: Return started location if any	*/
/*  (returns)	: bool		:					*/

bool	KBDBaseDlg::autoStart
	(	KBLocation	&autoLoc
	)
{
	return	m_serverTab->autoStart (autoLoc) ;
}

/*  KBDBaseDlg								*/
/*  addPage	: Add page to tabber					*/
/*  item	: KBDBDlgPluginItem &					*/
/*				: Page item information			*/ 
/*  (returns)	: void		:					*/

void	KBDBaseDlg::addPage
	(	KBDBDlgPluginItem	&item
	)
{
	m_window->addPage
	(	item.m_list,
		item.m_name,
		getSmallIcon (item.m_icon.isEmpty() ? QString("rekall") : item.m_icon)
	)	;
}

void	KBDBaseDlg::pageChanged
	(	QWidget		*page
	)
{
	m_gui->setEnabled ("KB_showRawSQL", page == m_serverTab) ;
}

#if	! __KB_RUNTIME

/*  KBDBaseDlg								*/
/*  showOneObject							*/
/*		: Show quick objects dialog				*/
/*  (returns)	: bool		: Repeat				*/

bool	KBDBaseDlg::showOneObject ()
{
	KBQuickDlg	   qDlg	  (m_dbInfo, m_allParts) ;
	KBQuickDlg::Option option = (KBQuickDlg::Option)qDlg.exec() ;

	if (option == KBQuickDlg::Cancelled)
		return	false	;

	QString		server		;
	QString		object		;

	const KBDBDlgPluginItem	&info	= qDlg.getResults (server, object) ;
	QString			type	= info.m_type	;
	QString			extn	= info.m_extn	;
	KBFileList		*objList= info.m_list	;

	DPRINTF
	((	"KBDBaseDlg::showOneObject: n=%s e=%s t=%s s=%s o=%s l=%p\n",
		info.m_name.latin1(),
		extn  .latin1(),
		type  .latin1(),
		server.latin1(),
		object.latin1(),
		(void *)objList
	))	;

	/* First check on the immediate results to sort out the object	*/
	/* type and extension, and where possible the part lists.	*/

	if	(type == "print"  )
	{
		/* Printers are handled specially, since there is no	*/
		/* associated part.					*/
		extn	= ""	  ;
	}
	else if	(type == "skin"   )
	{
		/* Ditto skins ...					*/
		extn	= ""	  ;
	}
	else if	(type == "graphic")
	{
		/* Graphics are also handled specially, again there is	*/
		/* no part associated with them, however we need to	*/
		/* get the extension.					*/
		int dotp = object.findRev ('.') ;

		if (dotp >= 0)
		{	extn	= object.mid  (dotp + 1) ;
			object	= object.left (dotp) ;
		}
		else	extn	= ""  ;
	}
	else
	{
		/* ... while all other objects types are handled via	*/
		/* their part.						*/
		if (objList == 0)
		{
			DPRINTF
			((	"KBDBDlg::showObjects type [%s] not found\n",
				type.latin1()
			))	;
			return	true	;
		}

		type	= objList->getTabType     () ;
		extn	= objList->getDocExtension() ;
	}


	/* Construct the location for the object. If the object		*/
	/* selection is "all" the object name will actually be "(All)"	*/
	/* translated.							*/
	KBLocation location
		(	m_dbInfo,
			type,
			server,
			object,
			extn
		)	;

	switch (option)
	{
		case KBQuickDlg::Copy	:
			KBQuickDlg::copyObjects  (location) ;
			return	true	;

		case KBQuickDlg::Delete	:
			KBQuickDlg::deleteObject (location) ;
			return	true	;

		case KBQuickDlg::Rename	:
			KBQuickDlg::renameObject (location) ;
			return	true	;

		case KBQuickDlg::SaveToFile	:
			if (object == TR("(All)"))
				KBQuickDlg::saveAllToFile (location) ;
			else	KBQuickDlg::saveObjToFile (location) ;
			return	true	;

		case KBQuickDlg::SaveToWeb	:
			if (object == TR("(All)"))
				KBQuickDlg::saveAllToWeb  (location, objList) ;
			else	KBQuickDlg::saveObjToWeb  (location, objList) ;
			return	true	;

		default	:
			break	;
	}

	KBError	   	error	;
	QDict<QString>	dummy	;

	/* Create and open are handled specially for printers, skins	*/
	/* and graphics.						*/
	if (type == "print"  )
	{
		showPrinters (option, location) ;
		return	true ;
	}
	if (type == "skin"   )
	{
		showSkins    (option, location) ;
		return	true ;
	}
	if (type == "graphic")
	{
		showGraphics (option, location) ;
		return	true ;
	}

	if (option == KBQuickDlg::Create)
	{
		if (!m_kbaseApp->newObject (location, error))
		{	error.DISPLAY() ;
			return	true	;
		}

		return	false ;
	}

	if (m_kbaseApp->openObject
		(	0,
			location,
			KB::ShowAsData,
			dummy,
			error,
			KBValue(),
			0
		) == KB::ShowRCError)
	{
		error.DISPLAY () ;
		return	true	 ;
	}

	return	false	;
}

/*  KBDBaseDlg								*/
/*  showObjects	: Show quick objects dialog				*/
/*  (returns)	: void		:					*/

void	KBDBaseDlg::showObjects ()
{
	while (showOneObject())
		;
}

void	KBDBaseDlg::showRawSQL ()
{
	m_serverTab->showRawSQL (m_kbaseApp) ;
}


void	KBDBaseDlg::dumpDatabase ()
{
	m_serverTab->dumpDatabase (m_kbaseApp) ;
}

void	KBDBaseDlg::loadDatabase ()
{
	m_serverTab->loadDatabase (m_kbaseApp) ;
}


/*  KBDBaseDlg								*/
/*  showGraphics: Handle graphics configuration				*/
/*  option	: KBQuickDlg::Option	: Quick dialog option		*/
/*  location	: KBLocation &		: Graphic location		*/
/*  (returns)	: void			:				*/

void	KBDBaseDlg::showGraphics
	(	KBQuickDlg::Option	option,
		KBLocation		&location
	)
{
	KBError	error	;

	/* First case is request to create a new graphic. This is just	*/
	/* treated as a load into the system.				*/
	if (option == KBQuickDlg::Create)
	{
		extern	bool	importImages (KBDBInfo *, const QString &, KBError &) ;

		if (!importImages
			(	location.dbInfo(),
				location.server(),
				error
			))
		{
			error.DISPLAY() ;
			return	;
		}

		return	;
	}

	TKMessageBox::sorry
	(	0,
		QString(TR("Edit/Open option not available for graphics")),
		QString(TR("Edit/Open graphic ..."))
	)	;
}
#endif

#if	! __KB_RUNTIME

/*  KBDBaseDlg								*/
/*  showPrinters: Handle printer configuration				*/
/*  option	: KBQuickDlg::Option	: Quick dialog option		*/
/*  location	: KBLocation &		: Printer location		*/
/*  (returns)	: void			:				*/

void	KBDBaseDlg::showPrinters
	(	KBQuickDlg::Option	option,
		KBLocation		&location
	)
{
	KBError	error ;

	/* First case is request to create a new printer definition.	*/
	/* Run the printer setup, and if the user OKs then prompt for	*/
	/* a location and save.						*/
	if (option == KBQuickDlg::Create)
	{
		TKPrinter printer (TKPrinter::ResPrinter) ;
		if (!printer.setup()) return ;

		QString	name 	;
		QString	locn	= location.server() ;

		if (!doPromptSave
			(	TR("Save printer as ..."),
				TR("Enter printer name" ),
				name,
				locn,
				m_dbInfo,
				true
			))
			return	;

		KBDomDocument	pXML	("printer") ;
		QDomElement	root	= pXML.documentElement() ;
		printer.saveSettings(root) ;

		if (!location.save (locn, name, pXML.toString(), error))
			error.DISPLAY() ;

		return	;
	}

	TKPrinter printer (TKPrinter::ResPrinter) ;

	{
		KBError		error	;
		QDomDocument	pXML	= location.contentsAsDom (TR("Printer"), error) ;
		if (pXML.isNull())
		{	error.DISPLAY()	;
			return	;
		}
		QDomElement	root	= pXML.documentElement() ;

		printer.loadSettings  (root) ;
	}

	/* Run the printer setup dialog, and give up if the used does	*/
	/* not OK it. Otherwise, we can just save the settings back	*/
	/* into the XML document and save it as above.			*/
	if (!printer.setup()) return ;

	QString	name 	= location.name  () ;
	QString	locn	= location.server() ;

	if (!doPromptSave
		(	TR("Save printer as ..."),
			TR("Enter printer name" ),
			name,
			locn,
			m_dbInfo,
			true
		))
		return	;

	if ((name != location.name()) || (locn != location.server()))
	{
		KBLocation check (m_dbInfo, "print", locn, name, "") ;
		if (check.exists())
			if (TKMessageBox::questionYesNo
				(	0,
					QString(TR("Printer %1 on server %2 exists, overwrite?"))
						.arg(name)
						.arg(locn),
					TR("Overwrite printer ...")
				)
				!= TKMessageBox::Yes) return ;
	}

	KBDomDocument	pXML	("printer") ;
	QDomElement	root	= pXML.documentElement() ;

	printer.saveSettings  (root) ;

	if (!location.save (locn, name, pXML.toString(), error))
		error.DISPLAY() ;
}

/*  KBDBaseDlg								*/
/*  showSkins	: Handle skin configuration				*/
/*  option	: KBQuickDlg::Option	: Quick dialog option		*/
/*  location	: KBLocation &		: Skin location			*/
/*  (returns)	: void			:				*/

void	KBDBaseDlg::showSkins
	(	KBQuickDlg::Option	option,
		KBLocation		&location
	)
{
	if (option == KBQuickDlg::Create)
	{
		KBLocation newLoc
			   (	location.dbInfo(),
				"skin",
				location.server(),
				QString::null,
				"skn"
			   )	;
		KBSkinDlg  (this, newLoc, false, true).exec() ;
		return	   ;
	}

	KBSkinDlg (this, location, true, true).exec() ;
}

#endif

/*  ------------------------------------------------------------------  */

static	QDict<QString>	identStrings	;

void	KBDBaseDlg::addIdentString
	(	const QString	&part,
		cchar		*ident
	)
{
	identStrings.setAutoDelete (true) ;
	identStrings.replace	   (part, new QString(ident)) ;
}

QString	KBDBaseDlg::getIdentStrings ()
{
	QString			res  ;
	QDictIterator<QString>	iter (identStrings) ;

	while (iter.current() != 0)
	{
		res	+= QString("<tr><td>Part</td><td><b>%1</b></td><td><nobr>%2</nobr></td></tr>")
				  .arg( iter.currentKey())
				  .arg(*iter.current   ()) ;

		iter	+= 1 ;
	}

	return	res	;
}


