/***************************************************************************
    file	         : kb_dbdlg.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_DBDLG_H
#define	_KB_DBDLG_H

#include	<qtabbar.h>
#include	<qdict.h>


#include	"kb_classes.h"
#include	"kb_database.h"
#include	"kb_dbinfo.h"
#include	"kb_serverdata.h"
#include	"kb_servertab.h"
#include	"kb_node.h"
#include	"kb_quickdlg.h"
#include	"kb_listwidget.h"
#include	"kbase.h"


#include	"tk_dirwatch.h"
#include	"rk_hbox.h"

class	RKTabWidget	;

class	KBDBDlgPluginItem
{
public	:

	int		m_slot	;
	QString		m_type	;	/* form, report, ...		*/
	QString		m_extn	;	/* frm, rep, ...		*/
	QString		m_name	;	/* Form, Report, ...		*/
	QString		m_icon	;
	KBFileList	*m_list	;

	inline	KBDBDlgPluginItem ()
	{
		m_slot	= 0	;
		m_list	= 0	;
	}
	inline	KBDBDlgPluginItem
		(	int		slot,
			const QString	&type,
			const QString	&extn,
			const QString	&name,
			const QString	&icon,
			KBFileList	*list
		)
		:
		m_slot	(slot),
		m_type	(type),
		m_extn	(extn),
		m_name	(name),
		m_icon	(icon),
		m_list	(list)
	{
	}

	bool	operator <  (const KBDBDlgPluginItem &other) const
	{
		return	m_slot < other.m_slot ;
	}
}	;

/*  KBDBDlg								*/
/*  -------								*/
/*  This is the main database configuration and access dialog. It	*/
/*  contains pages for:							*/
/*	Server setup							*/
/*	List of tables (by server)					*/
/*	Forms								*/
/*	.... etc							*/

class	KBDBaseDlg : public RKHBox
{
	Q_OBJECT

	KBaseApp				*m_kbaseApp	;
	KBDBInfoData				*m_dbInfo	;
	KBServerTab				*m_serverTab 	;
	KBListWidget				*m_window	;
	QValueList<KBDBDlgPluginItem>		m_allParts	;
	QPtrList<KBasePart>			m_children	;
	TKDirWatch				m_dirWatch	;
	KBaseGUI				*m_gui		;

	void	loadParts	()		;
#if	! __KB_RUNTIME
	void	showPrinters	(KBQuickDlg::Option, KBLocation &) ;
	void	showSkins	(KBQuickDlg::Option, KBLocation &) ;
	void	showGraphics	(KBQuickDlg::Option, KBLocation &) ;
#endif
	static	void	addIdentString		(const QString &, cchar *) ;

protected :

	void	addPage		(KBDBDlgPluginItem &) ;

private slots :

	void	dirDirty	(const QString &) ;
	void	pageChanged	(QWidget *)	  ;

public	:

	KBDBaseDlg (QWidget *, KBaseApp *, const QString &, bool, KBaseGUI *) ;
       ~KBDBaseDlg () ;

	inline	bool	isShowing
		(	const KBLocation 	&location
		)
	{
		return	location.dbInfo() == m_dbInfo ;
	}
	inline	KBDBInfo*getDBInfo	()
	{
		return	m_dbInfo	;
	}

#if	! __KB_RUNTIME
	bool		showOneObject	() ;
	void		showObjects	() ;
	void		showRawSQL	() ;
	void		dumpDatabase	() ;
	void		loadDatabase	() ;
#endif
	bool		autoStart	(KBLocation &) ;

	inline	QString	getDBName 	() { return m_dbInfo->getDBName() ; }
	inline	QString	getDBPath 	() { return m_dbInfo->getDBPath() ; }

	static	QString	getIdentStrings	() ;
}	;

#endif	// _KB_DBDLG_H
