/***************************************************************************
    file	         : kb_dbdlg_desktop.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qlistview.h>

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_listitem.h"
#include	"kb_dbdlg.h"
#include	"kb_dbinfo.h"
#include	"kb_serverdlg.h"
#include	"kb_script.h"
#include	"kb_scriptlist.h"
#include	"kb_viewer.h"
#include	"kb_desktop.h"

#include	"kb_libloader.h"

#define		__DEBUG_DBLG_DESKTOP	(0)
#if		__DEBUG_DBLG_DESKTOP
#include	"kb_dprintf.h"
#endif


//#define	MAXPLUG	64
//#define	NUMPLUG	128

typedef	void	*(*IFPROC)()	;

extern	void	registerLanguage (const QString &) ;

#if		__KB_RUNTIME
#define		DTPREFIX	"rekallrt"
#define		DTTYPE		"RekallRT/Part"
#else
#define		DTPREFIX	"rekall"
#define		DTTYPE		"Rekall/Part"
#endif

/*  KBDBaseDlg								*/
/*  loadParts	: Load component parts					*/
/*  (returns)	: void		:					*/

void	KBDBaseDlg::loadParts ()
{
	KBLibLoader	*loader	= KBLibLoader::self () ;

	/* Scan the local desktop files which descrive the rekall	*/
	/* "services". These are the same files as used for the KTrader	*/
	/* version, but the decoding in KBDesktop is much simplified.	*/
	QString		 dtDir	= locateDir
				  (	"appdata",
					"services/rekall_dummy.desktop"
				  )	;
	uint idx;

	QPtrList<KBDesktop> dtDefs ;
	KBDesktop::scan (dtDir + "/services", DTPREFIX "_", dtDefs) ;

	for (idx = 0 ; idx < dtDefs.count() ; idx += 1)
	{
		KBDesktop *desktop = dtDefs.at(idx) ;

#if	__DEBUG_DBLG_DESKTOP
		desktop->print () ;
#endif
		if (desktop->property ("ServiceTypes") != DTTYPE)
			continue	;


		QString	libname	 = desktop->property("X-KDE-Library") ;
		QString	partType = desktop->property("X-KDE-RekallPart-PartType"   ) ;
		QString	partExtn = desktop->property("X-KDE-RekallPart-PartExtn"   ) ;
		QString	objType	 = desktop->property("X-KDE-RekallPart-ObjectType" ) ;

		if (libname.isEmpty())
		{
			/* If there is no library name then register	*/
			/* part information anyway. This handle objects	*/
			/* (such as copiers) than are used in the	*/
			/* runtime version but accessed as parts.	*/
			KBLocation::registerType
			(	partType,
				partExtn,
				objType,
				0
			)	;

			continue ;
		}

		KBLibrary *lib 	 = loader->getGlobalLibrary (libname) ;
		if (lib == 0)
		{
			XPRINTF
			((	"KBDBDlg::loadParts: %s: %s\n",
				(cchar *)libname,
				(cchar *)loader->lastErrorMessage()
			))	;
			continue;
		}

		KBPartFactory	*factory = (KBPartFactory *)lib->factory() ;
		if (factory == 0)
		{	
			fprintf
			(	kbDPrintfGetStream(),
				"KBDBaseDlg::loadParts: cannot create factory %s\n",
				(cchar *)libname
			)	;
			continue;
		}

		addIdentString (objType, factory->ident()) ;

		/* Register the type and extension, along with the part	*/
		/* factory, with the location class.			*/
		KBLocation::registerType
		(	partType,
			partExtn,
			objType,
			factory
		)	;

//		/* In the runtime, we do not show tabs for parts such	*/
//		/* as tables or for queries, although the parts are	*/
//		/* still registered in order for the openTable and	*/
//		/* openQuery methods to work. This is handled by the	*/
//		/* hidden property.					*/
//		QString	hidden	 = desktop->property("X-KDE-RekallPart-Hidden"	   ) ;
//		if (hidden == "Yes") continue ;


		KBFileList *list = (KBFileList *)factory->create(m_window, (cchar *)m_dbInfo, "browser") ;
		if (list == 0)
		{
			fprintf
			(	kbDPrintfGetStream(),
				"KBDBaseDlg::loadParts: cannot create browser %s\n",
				(cchar *)libname
			)	;
			continue;
		}

		QString	name	= desktop->property("X-KDE-RekallPart-DisplayName") ;

		m_allParts.append
		(	KBDBDlgPluginItem
			(	desktop->property("X-KDE-RekallPart-DisplaySlot").toUInt(),
				partType,
				partExtn,
				name,
				desktop->property("Icon"),
				list
		)	)	;
	}

	qHeapSort (m_allParts)	;

	/* Now similarly look for all scripting languages. There is no	*/
	/* ordering on these (maybe is should be done by language name)	*/
	/* Here we use a fixed list type for all languages.		*/
	for (idx = 0 ; idx < dtDefs.count() ; idx += 1)
	{
		KBDesktop *desktop = dtDefs.at(idx) ;

#if	__DEBUG_DBLG_DESKTOP
		desktop->print () ;
#endif

		if (desktop->property ("ServiceTypes") != "Rekall/Script")
			continue	;

		QString	libname	 = desktop->property("X-KDE-Library") ;
		if (libname.isEmpty()) continue ;

		KBLibrary *lib 	 = loader->getGlobalLibrary (libname) ;
		if (lib == 0)
		{
			DPRINTF
			((	"KBDBDlg::loadParts: %s: %s\n",
				(cchar *)libname,
				(cchar *)loader->lastErrorMessage()
			))	;
			continue;
		}

		KBFactory *factory = (KBFactory *)lib->factory () ;
		if (factory ==  0)
		{	
			fprintf
			(	kbDPrintfGetStream(),
				"KBDBaseDlg::loadParts: cannot create %s\n",
				(cchar *)libname
			)	;
			continue;
		}

		QString	lang	= desktop->property("X-KDE-RekallPart-Language") ;
		QString	options	= desktop->property("X-KDE-RekallPart-Options" ) ;

		DPRINTF
		((	"KBDBaseDlg::loadParts: got language [%s] f=%p\n",
			(cchar *)lang,
			(void  *)factory
		))	;

#if	! __KB_RUNTIME
		QString	     legend  = QString(TR("Scripts: %1")).arg(lang) ;
		KBScriptList *slst   = new KBScriptList (m_window, m_dbInfo, lang) ;

		m_allParts.append
		(	KBDBDlgPluginItem
			(	1000,
				"script",
				lang,
				legend,
				desktop->property("Icon"),
				slst
		)	)	;
#endif
		registerLanguage (lang) ;
		KBScriptIF::registerScript (lang, factory, options) ;
	}

	/* Accumulated all parts, create pages for each of them ...		*/
	for (idx = 0 ; idx < m_allParts.count() ; idx += 1)
	{	KBDBDlgPluginItem &item = m_allParts[idx] ;
		addPage		  (item) ;
	}
}
