/***************************************************************************
    file		 : kb_main_qt3.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<time.h>
#include	<unistd.h>

#include	"rk_application.h"

#include	<qglobal.h>
#include	<qfileinfo.h>
#include	<qlibrary.h>
#include	<qpaintdevice.h>

#include	<qwindowsstyle.h>
#include	<qplatinumstyle.h>
#include	<qmotifstyle.h>
#include	<qmotifplusstyle.h>
#include	<qcdestyle.h>
#include	<qsgistyle.h>

#ifdef		Q_OS_MACX
#include	<qmacstyle_mac.h>
#endif

#ifdef		Q_OS_WIN32
#include	<io.h>
#endif

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"
#include	"kb_splash.h"
#include	"kb_optionsdlg.h"
#include	"kb_options.h"
#include	"kb_dprintf.h"
#include	"kbase.h"


#include	"rk_dcopbase.h"


#include	<qsettings.h>

#if		__KB_RUNTIME
#define		APPTITLE	"RekallRT"
#else
#define		APPTITLE	"Rekall"
#endif


#ifdef		__KB_RKLITE
#define		SPLASHIMG	"rekall_lite"
#else
#define		SPLASHIMG	"rekall"
#endif



extern	KBaseApp*kbaseApp	;
extern	bool	__useMDI	;
extern	bool	__useSDI	;
extern	bool	__noNoteDB	;
extern	bool	__noLastDB	;
extern	QString	__initDEB	;
static	bool	__noSplash	;
static	bool	__noSetupWiz	;
static	QString	__language	;
static	QString	__stderr	;
static	cchar	*__errFile	;

#ifdef	Q_OS_WIN32
static	bool	__console	;
#endif

extern	void	setRootDir	(const QString& _rootdir) ;

/*  setActiveWindow							*/
/*		: Make a specifed widget the active one			*/
/*  widget	: QWidget *	: Widget in question			*/
/*  (returns)	: void		:					*/

void	setActiveWindow
	(	QWidget		*widget
	)
{
	/* Dummy routine under QT3					*/
}

/*  dropActiveWindow							*/
/*		: Drop the currently active window if it closes		*/
/*  widget	: QWidget *	: Widget in question			*/
/*  (returns)	: void		:					*/

void	dropActiveWindow
	(	QWidget		*widget
	)
{
	/* Dummy routine under QT3					*/
}

/*  rekallMain	: Main routine						*/
/*  argc	: int		: Argument count			*/
/*  argv	: char *[]	: Argument vector			*/
/*  (returns)	: int		: Exit code				*/

int	rekallMain
	(	int	argc,
		char	*argv[]
	)
{
	RKApplication	app (argc, argv) ;

	cchar		*cmd	= argv[0];
	QString		rootDir = QFileInfo(cmd).dirPath () ;
	TKConfig	*config	= TKConfig::getConfig () ;

	config->setApplication	(__KB_APPNAME) ;

	int		ptr1	= 1	;
	QString		initDB	;

	while (ptr1 < argc)
	{
		if (strcmp (argv[ptr1], "--useMDI"  ) == 0)
		{	__useMDI = true	;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--useSDI"  ) == 0)
		{	__useSDI = true	;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--skipNoteDB") == 0)
		{	__noNoteDB=true	;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--skipLastDB") == 0)
		{	__noLastDB=true	;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--errLocn" ) == 0)
		{	KBError::errDebug = 1 ;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--errTrace") == 0)
		{	KBError::errDebug = 2 ;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--skipSplash") == 0)
		{	__noSplash = true ;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--skipSetup") == 0)
		{	__noSetupWiz = true ;
			ptr1	+= 1    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--errFile") == 0)
		{	__errFile = argv[ptr1+1] ;
			ptr1	+= 2    ;
			continue	;
		}
		if (strcmp (argv[ptr1], "--rkdcop") == 0)
		{
			RKDCOPBase::setAccess
			(	atoi(argv[ptr1+1]),
				argv[ptr1+1][0] == '+'
			)	;
			ptr1	+= 2    ;
			continue	;
		}
#ifdef	Q_OS_WIN32
		if (strcmp (argv[ptr1], "--showConsole") == 0)
		{	__console  = true ;
			ptr1	+= 1    ;
			continue	;
		}
#endif	// Q_OS_WIN32

		if (strncmp (argv[ptr1], "--", 2) == 0)
		{
			ptr1	  += 1 ;
			continue       ;
		}

		break	;
	}

	fprintf
	(	stderr,
		"rekallMain: argc=%d ptr1=%d argv[ptr1]=[%s]\n",
		argc,
		ptr1,
		argv[ptr1]
	)	;

	if (ptr1 < argc)
		initDB = argv[ptr1] ;

	if (__errFile != 0)
		kbDPrintfSetStream (__errFile) ;

	/* If the REKALL_STDERR environment parameter is set then	*/
	/* redirect stderr and stdout to the named file. Mainly for	*/
	/* Windows and MAC debugging.					*/
	__stderr = getenv("REKALL_STDERR") ;
	if (!__stderr.isEmpty())
	{
		freopen	(__stderr, "w", stderr)	;
		dup2	(fileno(stderr), fileno(stdout)) ;
	}

#ifdef 	Q_OS_WIN32
	if (__console)
		if (AllocConsole())
		{
			freopen	("CON", "w", stdout) ;
			freopen	("CON", "w", stderr) ;
			fprintf	(stderr, "Starting Rekall ......\n") ;
			fprintf	(stderr, "%s (%s)\n", cmd, (cchar *)rootDir) ;
		}

	if (!rootDir.isEmpty()) setRootDir (rootDir) ;

	WSADATA wsa ;
	WSAStartup  (MAKEWORD(1,1), &wsa) ;

#endif	// Q_OS_WIN32

#ifdef	Q_OS_MACX
	if (!rootDir.isEmpty())
		setRootDir (rootDir.left(rootDir.findRev('/'))) ;
#endif	// Q_OS_MACX

	extern	void	setAppdataDir (const QString &) ;
	setAppdataDir	(locateDir ("appdata", "gui/rekallui_null.gui")) ;

#if	! __KB_RUNTIME
	if (!__noSetupWiz)
		KBOptionsDlg::setupWizard (config) ;
#endif	// __KB_RUNTIME

	if (!__noSplash)
		showSlashScreen (SPLASHIMG, 7000, Qt::white) ;

	extern	void	loadRekallPlugins () ;
	loadRekallPlugins () ;

#if	__KB_PAGESWITCHLOADER
	extern	void	pageSwitchLoader  () ;
	pageSwitchLoader  () ;
#endif

	/* Check if rekall is runnable. This handles things like the	*/
	/* run-time version with nothing to open up.			*/
	new KBaseApp (initDB) ;

	if (!kbaseApp->isRunable())
		return	1 ;

	QString	style	= KBOptions::getStyle() ;
	if (!style.isEmpty()) QApplication::setStyle (style) ;

	return	app.exec() ;
}  

