/***************************************************************************
    file	         : kb_quickact.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"tk_messagebox.h"
#include	"tk_filedialog.h"

#include	"kb_classes.h"

#include	"kb_dbinfo.h"
#include	"kb_location.h"
#include	"kb_svrchooser.h"
#include	"kb_dbdociter.h"
#include	"kb_notifier.h"
#include	"kb_file.h"
#include	"kb_filelist.h"

#include	"kb_quickdlg.h"

#define		__DEBUG_QUICKACT	(1)
#if		__DEBUG_QUICKACT
#include	"kb_dprintf.h"
#endif

#include	<qapplication.h>


static	bool	loadChooser
	(	KBSvrChooserDlg	&chooser,
		KBLocation	&location
	)
{
	KBDBDocIter docIter (false) ;
	KBError	    error   ;
	QString	    name    ;
	QString	    stamp   ;
	QString	    extn    ;

	DPRINTF
	((	"loadChooser: %s extn=[%s]\n",
		location.ident    ().latin1(),
		location.extension().latin1()
	))	;

	if (!docIter.init
		(	location.dbInfo	  (),
			location.server	  (),
			location.type  	  (),
			location.extension(),
			error
		))
	{
		error.DISPLAY() ;
		return	false	;
	}

	while (docIter.getNextDoc (name, stamp, &extn))
	{
		DPRINTF
		((	"loadChooser: add [%s][%s]\n",
			(cchar *)name,
			(cchar *)extn
		))	;
		chooser.addEntry (name, extn) ;
	}

	if (location.name() == TR("(All)"))
		chooser.clickAddAll () ;
	else	chooser.selectEntry (location.name()) ;

	return	true ;
}

bool	KBQuickDlg::copyObjects
	(	KBLocation	&location
	)
{
	KBSvrChooserDlg chooser
			(	location.dbInfo(),
				location.server(),
				true,
				true
			)	;

	if (!loadChooser (chooser, location))
		return	false	;

	if (!chooser.exec())
		return	false	;

	QString			target	;
	QStringList		forms	;
	bool			over	;

	chooser.getInfo (target, forms, over) ;

	KBMessageBoxYNAC::ActRC	actRC	= over ?
						KBMessageBoxYNAC::ActYesAll :
						KBMessageBoxYNAC::ActYes    ;

	for (uint idx = 0 ; idx < forms.count() ; idx += 1)
	{
		QString		name	= forms[idx]	;
		KBLocation	srce
				(	location.dbInfo(),
					location.type  (),
					location.server(),
					name,
					chooser.extensionFor(name)
				)	;
		KBLocation	dest
				(	location.dbInfo(),
					location.type  (),
					target,
					name,
					chooser.extensionFor(name)
				)	;

		if (dest.exists())
		{
			actRC	= KBMessageBoxYNAC::query
				  (	0,
					QString (TR("%1 exists on server %2: overwrite?"))
						.arg(name  )
						.arg(target),
					actRC,
					TR("Overwrite document ...")
				  )	;
			if (actRC == KBMessageBoxYNAC::ActCancel)
				return	false	;
		}

		KBError		error	;
		QByteArray 	data	;

		if (!srce.contents (data, error))
		{
			error.DISPLAY() ;
			return	false	;
		}

		if (!dest.save 
			(	QString::null,
				QString::null,
				data.data(),
				data.size(),
				error
			))
		{
			error.DISPLAY() ;
			return	false	;
		}
	}

	return	true	;
}

bool	KBQuickDlg::renameObject
	(	KBLocation	&location
	)
{
	QString	newName	= location.name() ;
	KBError	error	;

	if (!doPrompt
		(	QString (TR("Rename %1 ..."		  )).arg(location.type()),
	    		QString (TR("Please enter the new %1 name")).arg(location.type()),
			newName
		))
		return	false	;

	if (!location.rename (newName, error))
	{
		error.DISPLAY() ;
		return	false	;
	}

	KBNotifier::self()->nObjectChanged (location) ;
	return	true	;
}

bool	KBQuickDlg::deleteObject
	(	KBLocation	&location
	)
{
	if (TKMessageBox::questionYesNo
		(	0,
			QString	(TR("Definitely delete %1 on server %2"))
				.arg (location.name  ())
				.arg (location.server()),
			QString	(TR("Delete Object ..."))
		)
		!= TKMessageBox::Yes) return false ;

	KBError	   error ;
	if (!location.remove (error))
	{
		error.DISPLAY() ;
		return	false	;
	}

	KBNotifier::self()->nObjectChanged (location) ;
	return	true	;
}

KBMessageBoxYNAC::ActRC
	KBQuickDlg::saveObjToFile
	(	KBLocation		&location,
		const QString		&path,
		KBMessageBoxYNAC::ActRC	actRC
	)
{
	/* Check what to do if the file already exists. This depends	*/
	/* on the action return code passed in.				*/
	if (QFileInfo(path).exists())
	{
		actRC	= KBMessageBoxYNAC::query
			  (	0,
				QString (TR("%1 already exists: overwrite?")).arg(path),
				actRC,
				TR("Save to file ....")
			  )	;
		if (actRC == KBMessageBoxYNAC::ActCancel)
			return	actRC ;
	}

	QByteArray data  ;
	KBError	   error ;
	if (!location.contents (data, error))
	{
		error.DISPLAY() ;
		return	 KBMessageBoxYNAC::ActCancel ;
	}

	KBFile	file	(path)	;
	if (!file.open(IO_WriteOnly))
	{
		file.lastError().DISPLAY() ;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	file.writeBlock (data)	;
	file.close	()	;
	return	actRC	;
}


bool	KBQuickDlg::saveObjToFile
	(	KBLocation	&location
	)
{
	KBFileDialog fDlg 
		     (	".",
			location.extension(),
			qApp->activeWindow(),
			"saveobject",
			true
		     )	;

	fDlg.setOperationMode	(KBFileDialog::Saving )	;
	fDlg.setMode		(KBFileDialog::AnyFile) ;
	fDlg.setSelection	(location.filename  ()) ;
	fDlg.setCaption		(TR("Save object ...")) ;

	if (!fDlg.exec  ()) return false ;

	return	saveObjToFile
		(	location,
			fDlg.selectedFile(),
			KBMessageBoxYNAC::ActYes
		)	;
}

bool	KBQuickDlg::saveAllToFile
	(	KBLocation	&location
	)
{
	KBSvrChooserDlg chooser
			(	location.dbInfo(),
				location.server(),
				false,
				false
			)	;

	if (!loadChooser (chooser, location))
		return	false	;

	if (!chooser.exec())
		return	false	;


	QString	dir	= KBFileDialog::getExistingDirectory
			  (	QString::null,
				TR("Save to files in .....")
			  )	;
	if (dir.isNull())
		return  false	;

	QString			target	;
	QStringList		forms	;
	bool			over	;

	chooser.getInfo (target, forms, over) ;

	KBMessageBoxYNAC::ActRC	actRC	= over ?
						KBMessageBoxYNAC::ActYesAll :
						KBMessageBoxYNAC::ActYes    ;

	for (uint idx = 0 ; idx < forms.count() ; idx += 1)
	{
		QString		name	= forms[idx]	;
		KBLocation	srce
				(	location.dbInfo(),
					location.type  (),
					location.server(),
					name,
					chooser.extensionFor(name)
				)	;
		KBLocation	dest
				(	location.dbInfo(),
					location.type  (),
					target,
					name,
					chooser.extensionFor(name)
				)	;

		actRC	= KBQuickDlg::saveObjToFile
				(	srce,
					dir + "/" + dest.filename(),
					actRC
				)	;
		if (actRC == KBMessageBoxYNAC::ActCancel)
			return	false	;
	}

	return	true	;
}


KBMessageBoxYNAC::ActRC
	KBQuickDlg::saveObjToWeb
	(	KBLocation		&location,
		KBFileList		*list,
		const QString		&dir,
		KBMessageBoxYNAC::ActRC	actRC
	)
{
	if (list != 0)
		return	list->saveObjToWeb (location, dir, actRC) ;

	return	KBQuickDlg::saveObjToFile
		(	location,
			dir + "/" + location.filename(),
			actRC
		)	;
}


bool	KBQuickDlg::saveObjToWeb
	(	KBLocation		&location,
		KBFileList		*list
	)
{
	if (list != 0)
		return	list->saveObjToWeb (location) ;

	return	KBQuickDlg::saveObjToFile (location) ;
}

bool	KBQuickDlg::saveAllToWeb
	(	KBLocation	&location,
		KBFileList	*list
	)
{
	KBSvrChooserDlg chooser
			(	location.dbInfo(),
				location.server(),
				false,
				false
			)	;

	if (!loadChooser (chooser, location))
		return	false	;

	if (!chooser.exec())
		return	false	;


	QString	dir	= KBFileDialog::getExistingDirectory
			  (	QString::null,
				TR("Save to files in .....")
			  )	;
	if (dir.isNull())
		return  false	;

	QString			target	;
	QStringList		forms	;
	bool			over	;

	chooser.getInfo (target, forms, over) ;

	KBMessageBoxYNAC::ActRC	actRC	= over ?
						KBMessageBoxYNAC::ActYesAll :
						KBMessageBoxYNAC::ActYes    ;

	for (uint idx = 0 ; idx < forms.count() ; idx += 1)
	{
		QString		name	= forms[idx]	;
		KBLocation	srce
				(	location.dbInfo(),
					location.type  (),
					location.server(),
					name,
					chooser.extensionFor(name)
				)	;

		actRC	= KBQuickDlg::saveObjToWeb
				(	srce,
					list,
					dir,
					actRC
				)	;
		if (actRC == KBMessageBoxYNAC::ActCancel)
			return	false	;
	}

	return	true	;
}


