/***************************************************************************
    file	         : kb_quickdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#include	<qlabel.h>
#include	<qtextview.h>

#if		__KB_AUTOMOC
#include	"kb_quickdlg.moc"
#else
#include 	"kb_quickdlg.h"
#endif

#include	"kb_dbinfo.h"
#include	"kb_serverinfo.h"
#include	"kb_filelist.h"
#include	"kb_scriptlist.h"
#include	"kb_dbdociter.h"
#include	"kb_dbdlg.h"
#include	"kb_sidepanel.h"

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"


#define		__DEBUG_QUICKDLG	(1)
#if		__DEBUG_QUICKDLG
#include	"kb_dprintf.h"
#endif


static	QString	lastType	;
static	QString	lastServer	;
static	QString	lastObject	;


static	void	setLastValue
	(	QComboBox	*combo,
		const QString	&value
	)
{
	for (int idx = 0 ; idx < combo->count() ; idx += 1)
		if (combo->text (idx) == value)
		{	combo->setCurrentItem (idx) ;
			break	;
		}
}

/*  KBQuickDlg								*/
/*  KBQuickDlg	: Constructor for quick selection dialog		*/
/*  dbInfo	: KBDBInfo *		: Database information		*/
/*  allParts	: QStringList &		: List of all part names	*/
/*  (returns)	: KBQuickDlg		:				*/

KBQuickDlg::KBQuickDlg
	(	KBDBInfo			*dbInfo,
		QValueList<KBDBDlgPluginItem>	&allParts
	)
	:
	KBDialog	(TR("Select Object"), true),
	m_dbInfo	(dbInfo),
	m_allParts	(allParts)
{
	RKVBox		*layTop		= new RKVBox	(this) ;
	layTop->setTracking () ;

	RKHBox		*layPanel	= new RKHBox	(layTop) ;

	new KBSidePanel	(layPanel, TR("Select Object")) ;

	RKVBox		*layMain	= new RKVBox	(layPanel) ;
	RKGridBox	*laySelect	= new RKGridBox (2, layMain) ;
	RKGridBox	*layButt	= new RKGridBox	(4, layMain) ;

	m_graphic	= new QLabel	 (layPanel)  ;
	layPanel ->setStretchFactor (m_graphic, 1)   ;
	m_graphic->setMinimumSize   (120, 150)	     ;
	m_graphic->setAlignment	    (Qt::AlignHCenter|Qt::AlignVCenter) ;

	new QLabel (TR("Object Type"),    laySelect) ;
	m_cbType	= new RKComboBox (laySelect) ;

	new QLabel (TR("Server Name"),    laySelect) ;
	m_cbServer	= new RKComboBox (laySelect) ;

	new QLabel (TR("Object Name"),    laySelect) ;
	m_cbObject	= new RKComboBox (laySelect) ;

	laySelect->addFillerRow() ;

	m_bCreate	= new RKPushButton (TR("New"   ),	layButt) ;
	m_bOpen		= new RKPushButton (TR("Open"  ),	layButt) ;
	m_bRename	= new RKPushButton (TR("Rename"),	layButt) ;
	m_bDelete	= new RKPushButton (TR("Delete"),	layButt) ;
	m_bCopy		= new RKPushButton (TR("Copy"  ),	layButt) ;
	m_bSaveToFile	= new RKPushButton (TR("Save to file"),	layButt) ;
	m_bSaveToWeb	= new RKPushButton (TR("Save to web" ),	layButt) ;
	new QWidget (layButt) ;

	RKHBox	*layDone= new RKHBox (layTop) ;
	layDone->addFiller () ;
	m_bCancel	= new RKPushButton (TR("Finish"),	layDone) ;

	connect	(m_bDelete,	SIGNAL(clicked()),	SLOT(clickDelete    ())) ;
	connect	(m_bCreate,	SIGNAL(clicked()),	SLOT(clickCreate    ())) ;
	connect	(m_bRename,	SIGNAL(clicked()),	SLOT(clickRename    ())) ;
	connect	(m_bCancel,	SIGNAL(clicked()),	SLOT(clickCancel    ())) ;
	connect	(m_bOpen,	SIGNAL(clicked()),	SLOT(clickOpen      ())) ;
	connect	(m_bSaveToFile,	SIGNAL(clicked()),	SLOT(clickSaveToFile())) ;
	connect	(m_bSaveToWeb,	SIGNAL(clicked()),	SLOT(clickSaveToWeb ())) ;
	connect	(m_bCopy,	SIGNAL(clicked()),	SLOT(clickCopy      ())) ;

	connect	(m_cbType,	SIGNAL(activated(int)),	SLOT(userChange     ())) ;
	connect	(m_cbServer,	SIGNAL(activated(int)),	SLOT(userChange     ())) ;
	connect	(m_cbObject,	SIGNAL(activated(int)),	SLOT(objectChange   ())) ;

	QStringList	servers	;
	QPtrListIterator<KBServerInfo> *dIter = dbInfo->getServerIter () ;
	while (dIter->current() != 0)
	{
		servers.append (dIter->current()->serverName()) ;
		++*dIter ;
	}
	delete	dIter ;

	servers.sort () ;

	for (uint slot = 0 ; slot < m_allParts.count() ; slot += 1)
		m_cbType->insertItem (m_allParts[slot].m_name) ;

	m_cbServer->insertItem	     (KBLocation::m_pFile) ;
	m_cbServer->insertStringList (servers ) ;

	setLastValue (m_cbType,	  lastType   ) ;
	setLastValue (m_cbServer, lastServer ) ;
	userChange   () ;
	setLastValue (m_cbObject, lastObject ) ;
	objectChange () ;
}

/*  KBQuickDlg								*/
/*  clickDelete	: User clicks delete button				*/
/*  (returns)	: void		:					*/

void	KBQuickDlg::clickDelete ()
{
	done (Delete) ;
}

/*  KBQuickDlg								*/
/*  clickRename	: User clicks rename button				*/
/*  (returns)	: void		:					*/

void	KBQuickDlg::clickRename ()
{
	done (Rename) ;
}

/*  KBQuickDlg								*/
/*  clickCreate	: User clicks create button				*/
/*  (returns)	: void		:					*/

void	KBQuickDlg::clickCreate ()
{
	done (Create) ;
}

/*  KBQuickDlg								*/
/*  clickCancel	: User clicks cancel button				*/
/*  (returns)	: void		:					*/

void	KBQuickDlg::clickCancel ()
{
	done (Cancelled) ;
}

/*  KBQuickDlg								*/
/*  clickOpen	: User clicks open button				*/
/*  (returns)	: void		:					*/

void	KBQuickDlg::clickOpen ()
{
	if (m_cbObject->currentItem() > 0) done (Open) ;
}

/*  KBQuickDlg								*/
/*  clickSaveToFile							*/
/*		: User clicks save to file button			*/
/*  (returns)	: void		:					*/

void	KBQuickDlg::clickSaveToFile ()
{
	if (m_cbObject->count() > 0) done (SaveToFile) ;
}

/*  KBQuickDlg								*/
/*  clickSaveToWeb							*/
/*		: User clicks save to web button			*/
/*  (returns)	: void		:					*/

void	KBQuickDlg::clickSaveToWeb ()
{
	if (m_cbObject->count() > 0) done (SaveToWeb) ;
}

/*  KBQuickDlg								*/
/*  clickCopy	: User clicks copy button				*/
/*  (returns)	: void		:					*/

void	KBQuickDlg::clickCopy ()
{
	if (m_cbObject->count() > 0) done (Copy) ;
}

/*  KBQuickDlg								*/
/*  userChange	: User changes selections				*/
/*  (returns)	: void		:					*/

void	KBQuickDlg::userChange ()
{
	QString			svrName	= m_cbServer ->currentText () ;
	const KBDBDlgPluginItem &item	= m_allParts[m_cbType->currentItem()] ;
	KBFileList		*oList	= item.m_list ;
	KBServerInfo		*server	= m_dbInfo->findServer (svrName) ;

	DPRINTF
	((	"KBQuickDlg::userChange: typ=[%s] svr=[%s] pl=%p\n",
		item.m_type.latin1(),
		(cchar *)svrName,
		(void  *)oList
	))	;


	m_cbObject   ->clear () ;
	m_bCreate    ->setEnabled (false) ;
	m_bOpen      ->setEnabled (false) ;
	m_bRename    ->setEnabled (false) ;
	m_bDelete    ->setEnabled (false) ;
	m_bSaveToFile->setEnabled (false) ;
	m_bSaveToWeb ->setEnabled (false) ;
	m_bCopy      ->setEnabled (false) ;

	m_graphic    ->clear	  ()	  ;

	if (server == 0)
	{
		DPRINTF
		((	"KBQuickDlg::userChange: server [%s] not found\n",
			(cchar *)m_cbServer->currentText ()
		))	;
		return	;
	}

	if (oList != 0)
	{
		QStringList names = oList->getObjectNames (server) ;

		if (names.count() > 0)
		{
			m_cbObject->insertItem	     (TR("(All)")) ;
			m_cbObject->insertStringList (names) ;
		}

		m_bCreate    ->setEnabled (true) ;
		m_bCopy	     ->setEnabled ((names.count() > 0)) ;
		m_bSaveToFile->setEnabled ((names.count() > 0)) ;
		m_bSaveToWeb ->setEnabled ((names.count() > 0) && oList->hasWeb()) ;
	        m_bOpen      ->setText    (TR("Open")) ;
		m_bCancel    ->setFocus   () ;

		return	;
	}


	KBDBDocIter docIter (item.m_extn.isEmpty()) ;
	KBError	    error   ;
	QString	    name    ;
	QString	    stamp   ;
	bool	    first   = true ;

	if (!docIter.init
		(	m_dbInfo,
			svrName,
			item.m_type,
			item.m_extn.isEmpty() ? QString("*") : item.m_extn,
			error
		))
	{
		error.DISPLAY() ;
		return	;
	}

	while (docIter.getNextDoc (name, stamp))
	{
		if (first)
		{	m_cbObject->insertItem(TR("(All)")) ;
			first	= false	;
		}

		m_cbObject->insertItem (name) ;
	}

	m_bCreate    ->setEnabled (true ) ;
	m_bCopy	     ->setEnabled (m_cbObject->count() > 0) ;
	m_bSaveToFile->setEnabled (m_cbObject->count() > 0) ;
	m_bSaveToWeb ->setEnabled (m_cbObject->count() > 0) ;
        m_bOpen      ->setText    (TR("Edit")) ;
        m_bCancel    ->setFocus   () ;
}

/*  KBQuickDlg								*/
/*  objectChange: User changes selections				*/
/*  (returns)	: void		:					*/

void	KBQuickDlg::objectChange ()
{
	const KBDBDlgPluginItem &item	= m_allParts[m_cbType->currentItem()] ;
	bool			enable	= m_cbObject->currentItem() > 0 ;

	DPRINTF
	((	"KBQuickDlg::objectChange: e=%d t=%s\n",
		enable,
		item.m_type.latin1()
	))	;

	m_bOpen	 ->setEnabled (enable)	;
	m_bDelete->setEnabled (enable)	;
	m_bRename->setEnabled (enable)	;
	m_graphic->clear      () 	;

	if (enable && (item.m_type == "graphic"))
	{
		QString	svrName	= m_cbServer->currentText () ;
		QString	objName	= m_cbObject->currentText () ;

		QStringList iparts	= QStringList::split ('.', objName) ;
		KBLocation  location
		(	m_dbInfo,
			"graphic",
			svrName,
			iparts[0],
			iparts[1]
		)	;

		KBError	   error	;
		QByteArray image	;

		if (!location.contents (image, error))
		{
			error.DISPLAY()	;
			return	;
		}

		DPRINTF	(("KBQuickDlg::objectChange: set pixmap\n")) ;
		m_graphic->setPixmap (QPixmap(image)) ;
	}
}

/*  KBQuickDlg								*/
/*  getResults	: Get result values					*/
/*  item	: KBDBDlgPluginItem &					*/
/*				: Selected object type information	*/
/*  server	: QString &	: Server result				*/
/*  object	: QString &	: Object name				*/
/*  (returns)	: void;		:					*/

const	KBDBDlgPluginItem
	&KBQuickDlg::getResults
	(	QString			&server,
		QString			&object
	)
{
	lastType		= m_cbType  ->currentText () ;
	server	= lastServer	= m_cbServer->currentText () ;
	object	= lastObject	= m_cbObject->currentText () ;

	return	m_allParts[m_cbType->currentItem()]	     ;
}
