/***************************************************************************
    file	         : kb_rawsql.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_RAWSQL_H
#define	__KB_RAWSQL_H

#include	<qsplitter.h>
#include	<qwidgetstack.h>
#include	<qlayout.h>
#include	<qtextview.h>
#include	<qlistview.h>

#include	"kb_classes.h"
#include	"kb_database.h"
#include	"kb_dbinfo.h"
#include	"kb_dblink.h"
#include	"kb_part.h"

#include	"kb_textedit.h"
#include	"tk_config.h"


class	KBRawSQL	;
class	KBRawSQLPage	;
class	RKPushButton	;
class	RKTabWidget	;

/*  KBRawSQLPage							*/
/*  ------------							*/
/*  This class is used for each query page displayed. It handles both	*/
/*  the text editor used to enter a query (and which is a tab of the	*/
/*  tab widget) and the widget used to display results).		*/

class	KBRawSQLPage : public KBTextEdit
{
	Q_OBJECT

	KBRawSQL	*m_rawSQL	;
	QString		m_caption	;

	QTextView	*m_textView	;
	QListView	*m_listView	;

	void		cleanPage	() ;
	void		showText	(const QString &) ;
	void		showSelect	(KBSQLSelect   *) ;
#ifdef	__NOTDEF
	void		executeCommand	(const QString &) ;
	void		executeSelect	(const QString &) ;
	void		executeUpdate	(const QString &, const QString &) ;
	void		executeInsert	(const QString &, const QString &) ;
	void		executeDelete	(const QString &, const QString &) ;
#endif

protected slots :

	void		slotTabChanged	(QWidget   *) ;

public	:

	KBRawSQLPage
	(	KBRawSQL	*,
		const QString	&,
		const QString	& = QString::null
	)	;
       ~KBRawSQLPage	() ;

	void		execute		()	;
	void		save		(TKConfig *, uint) ;
	void		setCaption	(const QString  &) ;
}	;


/*  KBRawSQL								*/
/*  --------								*/
/*  Raw SQL execution and display class.				*/

class	KBRawSQL : public KBasePart
{
	Q_OBJECT

	QSplitter	*m_splitter	;
	QWidgetStack	*m_stack	;
	RKTabWidget	*m_tabber	;
	QLineEdit	*m_name		;
	RKPushButton	*m_bAdd		;
	RKPushButton	*m_bCopy	;
	RKPushButton	*m_bRename	;
	RKPushButton	*m_bExecute	;
	RKPushButton	*m_bRemove	;

	QPtrList<KBRawSQLPage>
			m_pages		;

	KBDBInfo	*m_dbInfo	;
	QString		m_server	;
	KBDBLink	m_dbLink	;

protected slots :

	void		slotClickAdd	 () ;
	void		slotClickCopy	 () ;
	void		slotClickRename	 () ;
	void		slotClickExecute () ;
	void		slotClickRemove	 () ;

public	:

	KBRawSQL
	(	QWidget		*,
		KBDBInfo	*,
		const QString	&,
		bool		&
	)	;

	virtual	bool	queryClose	 () ;

	inline	RKTabWidget	*tabWidget ()
	{
		return	m_tabber ;
	}
	inline	QWidgetStack	*stack ()
	{
		return	m_stack  ;
	}
	inline	KBDBLink	&dbLink ()
	{
		return	m_dbLink  ;
	}

	void		setCaption	(const QString &) ;
}	;

#endif	// __KB_RAWSQL_H
