/***************************************************************************
    file	         : kb_scriptlist.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qcursor.h>

#include	"kb_classes.h"
#include	"kb_attrdict.h"
#include	"kb_location.h"
#include	"kb_dbinfo.h"
#include	"kb_dbdlg.h"
#include	"kb_script.h"
#include	"kb_prompt.h"
#include	"kb_viewer.h"
#include	"kb_appptr.h"
#include	"kb_callback.h"
#include 	"kb_popupmenu.h"
#include 	"kb_file.h"

#include	"kbase.h"

#if		__KB_AUTOMOC
#include	"kb_scriptlist.moc"
#else
#include 	"kb_scriptlist.h"
#endif

#include	"tk_messagebox.h"
#include	"tk_icons.h"


/*  KBScriptList							*/
/*  KBScriptList: Constructor for script list widget			*/
/*  parent	: QWidget *	  : Parent widget			*/
/*  dbInfo	: KBDBInfo *	  : Database information object		*/
/*  language	: const QString & : Language extension			*/
/*  (returns)	: KBScriptList	  :					*/

KBScriptList::KBScriptList
	(	QWidget		*parent,
		KBDBInfo	*dbInfo,
		const QString	&language
	)
	:
	KBFileList
	(	parent,
		dbInfo,
		"Create new script",
		"scriptlog",
		"script",
		"Script name"
	),
	m_language	(language)
{
	enableWeb    () ;
}

/*  KBScriptList							*/
/*  ~KBScriptList: Destructor for form list widget			*/
/*  (returns)	 :		:					*/

KBScriptList::~KBScriptList ()
{
}

/*  KBScriptList							*/
/*  showServerMenu: Show popup menu for server entries			*/
/*  (returns)	  : void	  :					*/

void	KBScriptList::showServerMenu
	(	int
	)
{
	KBPopupMenu pop (0) ;

	if ((m_curItem != 0) && (m_curItem->type() == KBListItem::Server))
		pop.setTitle   (QString(TR("Scripts: %1")).arg(m_curItem->text(0))) ;
	else	pop.setTitle   (TR("Scripts")) ;

	pop.insertItem
	(	getSmallIcon("reload"),
		TR("&Reload script list"),
		this,
		SLOT(reloadServer ())
	)	;
	pop.insertItem
	(	getSmallIcon("editcopy"),
		TR("&Copy to server ..."),
		this,
		SLOT(copyToServer ())
	)	;
	pop.insertItem
	(	getSmallIcon("filenew"),
		TR("&Load from file..."),
		this,
		SLOT(loadFromFile ())
	)	;
	pop.insertItem
	(	getSmallIcon("filesave"),
		TR("Save all to &files ..."),
		this,
		SLOT(saveAllToFile())
	)	;
	pop.insertItem
	(	getSmallIcon("www"),
		TR("Save all to &web ..."),
		this,
		SLOT(saveAllToWeb ())
	)	;

	pop.exec (QCursor::pos()) ;
}

/*  KBScriptList							*/
/*  showObjectMenu: Show popup menu for object entries			*/
/*  (returns)	  : void	  :					*/

void	KBScriptList::showObjectMenu
	(	int
	)
{
	KBPopupMenu pop (0) ;

	if ((m_curItem != 0) && (m_curItem->type() == KBListItem::Object))
		pop.setTitle   (QString(TR("Scripts: %1")).arg(m_curItem->text(0))) ;
	else	pop.setTitle   (TR("Script")) ;

	pop.insertItem
	(	getSmallIcon("designview"),
		TR("&Edit script"),
		this,
		SLOT(showAsData  ())
	)	;

	if (KBaseApp::getDebugIface (m_language) != 0)
		pop.insertItem
		(	TR("&Load into debugger"),
			this,
			SLOT(loadDebugger())
		)	;

	pop.insertItem
	(	TR("&Rename script"),
		this,
		SLOT(renameScript ())
	)	;
	pop.insertItem
	(	getSmallIcon("editdelete"),
		TR("De&lete script"),
		this,
		SLOT(deleteScript ())
	)	;
	pop.insertItem
	(	getSmallIcon("filesave"),
		TR("&Save to file"),
		this,
		SLOT(saveObjToFile())
	)	;
	pop.insertItem
	(	getSmallIcon("www"),
		TR("Save to &web" ),
		this,
		SLOT(saveObjToWeb ())
	)	;

	pop.exec (QCursor::pos()) ;
}

/*  KBScriptList							*/
/*  renameScript: Rename a script					*/
/*  (returns)	: void		:					*/

void	KBScriptList::renameScript ()
{
	KBError	   error    ;
	KBLocation location ;

	if (!itemToLocation (m_curItem, location)) return ;
	if (!canOperate     (location,  "rename")) return ;


	/* Prompt the user for a new name for the script, with the	*/
	/* usual getout.						*/
	QString	newName	;
	if (!doPrompt (TR("Rename script"),
		       TR("Please enter the new script name"), newName)) return ;


	/* OK, go ahead and delete it. For files, this is handled	*/
	/* through the scripting interface so that we			*/
	/* don't need to know about things like compiled script files.	*/
	if (location.server() == KBLocation::m_pFile)
	{
		KBError	   error     ;
		KBScriptIF *scrIface = LinkKBScript (m_language, error) ;

		if (scrIface == 0)
		{
			error.DISPLAY () ;
			return	;
		}

		if (!scrIface->rename (location, newName, error))
		{
			error.DISPLAY () ;
		}
	}
	else
	{
		/* Database rename is handled in the normal way ...	*/
		if (!location.rename (newName, error))
			error.DISPLAY () ;
	}

	/* Reload the script list. Do this whether or not there was an	*/
	/* error; this ensures that the displayed list is correct.	*/
	objChange (location) ;
}

/*  KBScriptList							*/
/*  deleteScript: Delete a script					*/
/*  (returns)	: void		:					*/

void	KBScriptList::deleteScript ()
{
	KBError	   error    ;
	KBLocation location ;

	if (!itemToLocation (m_curItem,  location)) return ;
	if (!canOperate     (location,   "rename")) return ;


	if (TKMessageBox::questionYesNo
	   	(	0,
			QString (TR("Definitely delete script %1?")).arg(location.name()),
			TR("Delete script")
		)
		!= TKMessageBox::Yes) return ;


	/* OK, go ahead and delete it. For files, this is handled	*/
	/* through the scripting interface so that we don't need to	*/
	/* know about things like compiled script files.		*/
	if (location.server() == KBLocation::m_pFile)
	{
		KBError	   error     ;
		KBScriptIF *scrIface = LinkKBScript (m_language, error) ;

		if (scrIface == 0)
		{
			error.DISPLAY () ;
			return	;
		}
		if (!scrIface->unlink (location, error))
		{
			error.DISPLAY () ;
		}
	}
	else
	{
		/* Scripts store in the database are handled in the	*/
		/* default manner.					*/
		if (!location.remove (error))
			error.DISPLAY () ;
	}

	/* Reload the script list. Do this whether or not there was an	*/
	/* error; this ensures that the displayed list is correct.	*/
	objChange (location) ;
}

/*  KBScriptList							*/
/*  loadDebugger: Load script into debugger				*/
/*  (returns)	: void		:					*/

void	KBScriptList::loadDebugger ()
{
	KBError	   error    ;
	KBLocation location ;
	if (!itemToLocation (m_curItem, location)) return ;

	KBScriptIF *scrIface = KBaseApp::getDebugIface (m_language) ;
	if (scrIface == 0) return ;

	if (!scrIface->debugScript (location, error))
		error.DISPLAY () ;
}

void	KBScriptList::loadFromFile ()
{
	static	QString	lastDir	     ;
	if (lastDir.isEmpty()) lastDir = "."	;

	KBFileDialog fDlg 
		     (	lastDir,
			"*.*",
			qApp->activeWindow(),
			"loadscript",
			true
		     )	;

	fDlg.setMode	  (KBFileDialog::ExistingFile) ;
	fDlg.setCaption   ("Load script ....") ;

	if (!fDlg.exec()) return ;

	KBFile	scriptFile(fDlg.selectedFile ()) ;
	if (!scriptFile.open (IO_ReadOnly))
	{
		scriptFile.lastError().DISPLAY() ;
		return	;
	}

	QByteArray  data = scriptFile.readAll() ;

	/* We will store the script using the final part of the path	*/
	/* with the extension split off ...				*/
	QFileInfo  info	 (fDlg.selectedFile ()) ;
	KBError	   error ;
	KBLocation dest
		   (	m_dbInfo,
			"script",
			m_curItem->text(0),
			info.baseName(),
			m_language
		   )	;

	if (dest.exists())
		if (TKMessageBox::questionYesNo
		   	(	0,
				QString (TR("Script %1 already exists: overwrite?")).arg(dest.name()),
				TR("Overwrite script")
			)
			!= TKMessageBox::Yes) return ;


	if (!dest.save
		(	QString::null,
			QString::null,
			data.data(),
			data.size(),
			error
		))
		error.DISPLAY() ;

	reloadServer ()	;
}


/*  KBScriptList								*/
/*  getDocExtension							*/
/*		: Get document name extension				*/
/*  (returns)	: QString	: Extension				*/

QString	KBScriptList::getDocExtension ()
{
	return	m_language ;
}
