/***************************************************************************
    file	         : kb_serverdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qregexp.h>
#include	<qspinbox.h>

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_database.h"
#include	"kb_dbinfo.h"
#include	"kb_options.h"
#include	"kb_tableinfo.h"
#include	"kb_sidepanel.h"
#include	"kb_manual.h"
#include	"kb_font.h"

#if		__KB_AUTOMOC
#include	"kb_serverdlg.moc"
#else
#include 	"kb_serverdlg.h"
#endif

#include	"tk_filedialog.h"

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_combobox.h"
#include	"rk_listbox.h"
#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"
#include	"rk_tabwidget.h"
#include	"tk_fontdialog.h"



static	cchar	*codecNames[] =
{
	"UTF8",
	"BIG5",
	"CP 874",
	"CP 1250",
	"CP 1251",
	"CP 1252",
	"CP 1253",
	"CP 1254",
	"CP 1255",
	"CP 1256",
	"CP 1257",
	"CP 1258",
	"EUCJP",
	"EUCKR",
	"GBK",
	"JIS7",
	"KOI8-R",
	"KOI8-U",
	"SHIFT-JIS",
	"ISO8859-1",
	"ISO8859-2",
	"ISO8859-3",
	"ISO8859-4",
	"ISO8859-5",
	"ISO8859-6",
	"ISO8859-7",
	"ISO8859-8",
	"ISO8859-8-I",
	"ISO8859-9",
	"ISO8859-10",
	"ISO8859-13",
	"ISO8859-14",
	"ISO8859-15",
	"TIS-620",
	"TSCII",
	0
}	;

static	bool	checkCodec
	(	const QString	&encoding,
		const QString	&msg
	)
{
	if (!encoding.isEmpty())
	{
		QTextCodec *codec = QTextCodec::codecForName  (encoding) ;

		if (codec == 0)
		{
			TKMessageBox::sorry
			(	0,
				QString	(TR("Cannot find %1 codec for '%1'"))
					.arg(msg)
					.arg(encoding),
				TR("Encoding")
			)	;
			return	false	;
		}
	}

	return	true	;
}


/*  ------------------------------------------------------------------  */
/*  KBServerDlg								*/
/*  KBServerDlg	: Constructor for database server dialog		*/
/*  serverData	: KBServerData *  : COnnection data			*/
/*  (returns)	: KBServerDlg	  :					*/

KBServerDlg::KBServerDlg
	(	KBServerData		*serverData,
		KBDriverDetailsList	&drvList
	)
	:
	KBDialog	(TR("Database Connection"), true, "kbserverdlg"),
	m_drvList	(drvList),
	m_serverData	(serverData)
{
	RKVBox		*layTop		= new RKVBox	  (this) ;
	layTop->setTracking ()	 ;

	RKHBox		*layMain	= new RKHBox	  (layTop ) ;
	new KBSidePanel (layMain, "Database Connection") ;

	m_tabber			= new RKTabWidget (layMain) ;
	addOKCancel	(layTop, 0, 0, KBHELPKEY("Chap3FilesEntry")) ;

	RKGridBox	*laySettings	= new RKGridBox   (2, m_tabber) ;
	RKVBox		*layComment	= new RKVBox	  (m_tabber)	;
	RKGridBox	*layAdvanced	= new RKGridBox	  (2, m_tabber) ;
	RKVBox		*layInitSQL	= new RKVBox	  (m_tabber)	;

	m_tabber->addTab (laySettings, TR("Database"	)) ;
	m_tabber->addTab (layComment,  TR("Comments"	)) ;
	m_tabber->addTab (layAdvanced, TR("Advanced"	)) ;
	m_tabber->addTab (layInitSQL,  TR("Initial SQL"	)) ;
	n_fixedTabs = m_tabber->count() ;

	QPtrList<QWidget> tabOrder1 ;
	QPtrList<QWidget> tabOrder2 ;

	/*  MAIN DATABASE PAGE						*/
	/*  ------------------						*/
	new QLabel(TR("Name"), laySettings) ;
	m_serverName	= new RKLineEdit (laySettings) ;
	tabOrder1.append (m_serverName) ;

	new QLabel(TR("Type"), laySettings) ;
	m_dbType	= new RKComboBox (laySettings) ;
	tabOrder1.append (m_dbType) ;

	new QLabel(TR("Host"), laySettings) ;
	m_hostName	= new RKLineEdit (laySettings) ;
	tabOrder1.append (m_hostName) ;

	new QLabel(TR("User"), laySettings) ;
	m_userName	= new RKLineEdit (laySettings) ;
	tabOrder1.append (m_userName) ;

	new QLabel(TR("Password"), laySettings) ;
	RKHBox	*layPW  = new RKHBox	   (laySettings) ;
	m_password	= new RKLineEdit   (layPW) ;
	m_showPwd	= new QCheckBox    (TR("Show"), layPW) ;
	layPW->setSpacing (5) ;
	tabOrder1.append (m_password) ;
	tabOrder1.append (m_showPwd ) ;

	new QLabel(TR("Database"), laySettings) ;
	RKHBox	*layDB  = new RKHBox	   (laySettings) ;
	m_dbName	= new RKComboBox   (layDB) ;
	m_bDatabases	= new RKPushButton (TR(""),     layDB) ;
	layDB->setSpacing (5) ;
	tabOrder1.append (m_dbName    ) ;
	tabOrder1.append (m_bDatabases) ;


	new QLabel(TR("Port/Socket"), laySettings) ;
	RKHBox	*layPS  = new RKHBox (laySettings) ;
	m_portNumber	= new RKLineEdit (layPS) ;
	m_socketName	= new RKLineEdit (layPS) ;
	tabOrder2.append (m_portNumber) ;
	tabOrder2.append (m_socketName) ;

	int	fw	= m_showPwd->sizeHint().width() ;
	m_bDatabases->setFixedWidth   (fw) ;
	m_showPwd   ->setFixedWidth   (fw) ;
	m_portNumber->setMaximumWidth (fw) ;

	new QLabel(TR("Flags"), laySettings) ;
	m_flags		= new RKLineEdit (laySettings) ;
	tabOrder2.append (m_flags) ;

	new QWidget (laySettings) ;
	RKHBox	*layFlags = new RKHBox (laySettings) ;
	m_disabled	= new QCheckBox	(TR("Disabled"),  layFlags) ;
	m_autostart	= new QCheckBox	(TR("AutoStart"), layFlags) ;
	tabOrder2.append (m_disabled ) ;
	tabOrder2.append (m_autostart) ;


	/*  ADVANCED SETTINGS PAGE					*/
	/*  ----------------------					*/
	new QWidget (layAdvanced) ;
	m_noRekallTables	= new QCheckBox	(TR("Do not create rekall-specific tables"),	layAdvanced) ;
	new QWidget (layAdvanced) ;
	m_showAllTables		= new QCheckBox	(TR("Show all tables"),				layAdvanced) ;
	new QWidget (layAdvanced) ;
	m_cacheTables		= new QCheckBox	(TR("Cache table information"),			layAdvanced) ;
	new QWidget (layAdvanced) ;
	m_nullUserPwd		= new QCheckBox	(TR("Accept empty user name/password"),		layAdvanced) ;
	new QWidget (layAdvanced) ;
	m_pkReadOnly		= new QCheckBox	(TR("Primary keys are read only"),		layAdvanced) ;
	new QWidget (layAdvanced) ;
	m_fakeKeys		= new QCheckBox	(TR("Fake unique keys for insertion"),		layAdvanced) ;
	new QWidget (layAdvanced) ;
	m_readOnly		= new QCheckBox	(TR("Database is read-only"),			layAdvanced) ;

	new QLabel  (TR("Show tests/test suites"), layAdvanced) ;
	m_showTests		= new RKComboBox  (layAdvanced) ;
	m_showTests->insertItem (TR("Off"			 )) ;
	m_showTests->insertItem (TR("Tests only"		 )) ;
	m_showTests->insertItem (TR("Tests and test suites"	 )) ;
	m_showTests->insertItem (TR("Tests, suites and recording")) ;

	new QLabel (TR("Application font"), layAdvanced) ;
	RKHBox *hb	 = new RKHBox (layAdvanced) ;
	m_appFont	 = new RKLineEdit   (hb) ;
	RKPushButton *bf = new RKPushButton ("...",  hb) ;

	connect
	(	bf,
		SIGNAL	(clicked   ()),
		SLOT	(setAppFont())
	)	;

	new QLabel (TR("Skin suffix"),	   layAdvanced) ;
	m_skinSuffix		= new RKLineEdit(layAdvanced) ;
 
	new QLabel (TR("AutoStart form"),  layAdvanced) ;
	m_autoForm		= new RKLineEdit(layAdvanced) ;

	new QLabel (TR("Data Encoding"),   layAdvanced) ;
	m_dataEncoding		= new RKComboBox(layAdvanced) ;

	new QLabel (TR("Object Encoding"), layAdvanced) ;
	m_objEncoding		= new RKComboBox(layAdvanced) ;

	new QLabel (TR("Web Directory"),   layAdvanced) ;
	RKHBox	*wb		= new RKHBox	   (layAdvanced) ;
	m_webDirectory		= new RKLineEdit   (wb) ;
	m_webBrowse		= new RKPushButton ("...", wb) ;

	connect
	(	m_webBrowse,
		SIGNAL	(clicked  ()),
		SLOT	(webBrowse())
	)	;

#ifdef	Q_OS_LINUX
	new QLabel (TR("SSH Tunneling"), layAdvanced) ;
	m_sshTarget		= new RKLineEdit(layAdvanced) ;
#endif

	/*  COMMENT PAGE						*/
	m_comment		= new RKTextEdit(layComment) ;

	/*  INITIAL SQL PAGE						*/
	/*  ----------------						*/
	m_initSQL		= new RKTextEdit(layInitSQL) ;


	laySettings->addFillerRow() ;
	layAdvanced->addFillerRow() ;

	m_bDatabases->setText     ("...") ;
	m_password  ->setEchoMode (QLineEdit::Password) ;



	connect	(m_dbType,    SIGNAL(activated(int)),	SLOT(typeChange   ())) ;
	connect	(m_bDatabases,SIGNAL(clicked  ()),	SLOT(showDatabases())) ;
	connect (m_showPwd,   SIGNAL(toggled  (bool)) , SLOT(showPassword (bool))) ;

	m_dbName->setEditable  (true) ;


	for (uint idx0 = 0 ; idx0 < m_drvList.count() ; idx0 += 1)
		m_dbType->insertItem (m_drvList[idx0].comment()) ;

	for (uint idx1 = 0 ; idx1 < tabOrder1.count() - 1 ; idx1 += 1)
		QWidget::setTabOrder (tabOrder1.at(idx1), tabOrder1.at(idx1+1)) ;
	for (uint idx2 = 0 ; idx2 < tabOrder1.count() - 1 ; idx2 += 1)
		QWidget::setTabOrder (tabOrder1.at(idx2), tabOrder1.at(idx2+1)) ;

}

int	KBServerDlg::exec
	(	bool		isNew
	)
{

	/* Show the server settings and, if there is an advanced	*/
	/* configuration for the selected driver, add this into the	*/
	/* dialog.							*/
	if (!showServerData (isNew))
		return	false	;

	if (m_serverData->m_advanced != 0)
		m_serverData->m_advanced->setupDialog (m_tabber) ;

	if (isNew)
	{
		if (m_dbType->count() == 2)
			m_dbType->setCurrentItem (1) ;
		else	m_dbType->setCurrentItem (0) ;

		m_serverName->setSelection(0, 9) ;
        	m_serverName->setFocus    ()     ;
	}

	return	KBDialog::exec () ;
}

/*  KBServerDlg								*/
/*  ~KBServerDlg: Destructor for database server dialog			*/
/*  (returns)	:		:					*/

KBServerDlg::~KBServerDlg ()
{
}

/*  KBServerDlg								*/
/*  showServerInfo							*/
/*		: Show server information				*/
/*  (returns)	: bool		 : Success				*/

bool	KBServerDlg::showServerData
	(	bool		isNew
	)
{
	m_serverName	->setText    (m_serverData->m_serverName    ) ;
	m_hostName  	->setText    (m_serverData->m_hostName      ) ;
	m_dbName    	->setEditText(m_serverData->m_dbName        ) ;
	m_userName  	->setText    (m_serverData->m_userName      ) ;
	m_password  	->setText    (m_serverData->m_password      ) ;
	m_portNumber	->setText    (m_serverData->m_portNumber    ) ;
	m_socketName	->setText    (m_serverData->m_socketName    ) ;
	m_flags     	->setText    (m_serverData->m_flags         ) ;
	m_disabled  	->setChecked (m_serverData->m_isDisabled    ) ;
	m_autostart 	->setChecked (m_serverData->m_autoStart     ) ;

	m_noRekallTables->setChecked (m_serverData->m_noRekallTables) ;
	m_showAllTables ->setChecked (m_serverData->m_showAllTables ) ; 
	m_cacheTables   ->setChecked (m_serverData->m_cacheTables   ) ;
	m_nullUserPwd   ->setChecked (m_serverData->m_nullUserPwd   ) ;
	m_pkReadOnly    ->setChecked (m_serverData->m_pkReadOnly    ) ;
	m_fakeKeys      ->setChecked (m_serverData->m_fakeKeys      ) ;
	m_readOnly      ->setChecked (m_serverData->m_readOnly      ) ;

	m_autoForm	->setText    (m_serverData->m_autoForm	    ) ;
	m_initSQL	->setText    (m_serverData->m_initSQL	    ) ;
	m_appFont	->setText    (m_serverData->m_appFont	    ) ;
	m_skinSuffix	->setText    (m_serverData->m_skinSuffix    ) ;
	m_comment	->setText    (m_serverData->m_comment	    ) ;

#ifdef	Q_OS_LINUX
	m_sshTarget	->setText    (m_serverData->m_sshTarget	    ) ; 
#endif
	m_webDirectory	->setText    (m_serverData->m_webDirectory  ) ;

	m_dataEncoding->insertItem("") ; 
	m_objEncoding ->insertItem("") ; 

	int atd = -1 ;
	int ato = -1 ;
	for (cchar **cnp = &codecNames[0] ; *cnp != 0 ; cnp += 1)
	{	
		if (*cnp == m_serverData->m_dataEncoding) atd = m_dataEncoding->count() ;
		if (*cnp == m_serverData->m_objEncoding ) ato = m_objEncoding ->count() ;

		m_dataEncoding->insertItem(*cnp) ; 
		m_objEncoding ->insertItem(*cnp) ; 

		QTextCodec *tc = QTextCodec::codecForName(*cnp) ;
		if (tc == 0)
			fprintf
			(	stderr,
				"KBDBAdvancedDlg::KBDBAdvancedDlg: no codec [%s]\n",
				*cnp
			)	;
	}

	m_dataEncoding->setEditable (true) ;
	m_objEncoding ->setEditable (true) ;

	if (atd >= 0)
		m_dataEncoding->setCurrentItem(atd) ;
	else
		if (!m_serverData->m_dataEncoding.isEmpty())
			m_dataEncoding->setEditText (m_serverData->m_dataEncoding) ;

	if (ato >= 0)
		m_objEncoding ->setCurrentItem(ato) ;
	else
		if (!m_serverData->m_objEncoding .isEmpty())
			m_objEncoding ->setEditText (m_serverData->m_objEncoding ) ;

	m_showTests->setCurrentItem (m_serverData->m_showTests) ;

	/* Some of the controls are enabled or disabled depending on	*/
	/* whether this is !Files, and on driver capabilities.		*/
	QString	 dbType	 = m_serverData->m_dbType ;
	KBServer *server = dbType.isEmpty() ? 0 : getDriverServer(dbType) ;
	bool	 isFile	 = m_serverData->m_serverName == KBLocation::m_pFile   ;

	setEnabledAll
	(	!isFile,
		server == 0 ? 0xffffffff : server->optionFlags()
	)	;

	uint	dbIdx	= 0 ;

	if (!m_serverData->m_dbType.isEmpty())
	{
		/* Scan the driver type list for the driver type for	*/
		/* this server. If it is not found (either someone	*/
		/* edited the database file, or a driver is missing)	*/
		/* then prompt if we want to continue.			*/
		for (dbIdx = 1 ; dbIdx < m_drvList.count() ; dbIdx += 1)
			if (m_drvList[dbIdx].tag() == m_serverData->m_dbType)
				break	;

		if (dbIdx >= m_drvList.count())
		{
			if (TKMessageBox::questionYesNo
				(	0,
					QString(TR("Server \"%1\" has an unrecognised type \"%2\": edit anyway?"))
							.arg(m_serverData->m_serverName)
							.arg(m_serverData->m_dbType),
					TR("Database Connection")
				)
				!=TKMessageBox::Yes
			   )
				return	false	;
			   
			dbIdx	= 0	;
		}
	}

	m_dbType->setCurrentItem (dbIdx) ;
	return	true	;
}

/*  KBServerDlg								*/
/*  setEnabledAll: Perform setEnabled on all controls			*/
/*  server	 : bool		: Enable/disable server name		*/
/*  optionFlags	 : uint		: Option enable flags			*/
/*  (returns)	 : void		:					*/

void	KBServerDlg::setEnabledAll
	(	bool	server,
		uint	opts
	)
{
	m_serverName->setEnabled (server) ;
	m_hostName  ->setEnabled ((opts & AF_HOST        ) != 0) ;
	m_userName  ->setEnabled ((opts & AF_USERPASSWORD) != 0) ;
	m_password  ->setEnabled ((opts & AF_USERPASSWORD) != 0) ;
	m_portNumber->setEnabled ((opts & AF_PORTNUMBER  ) != 0) ;
	m_socketName->setEnabled ((opts & AF_SOCKETNAME  ) != 0) ;
	m_flags     ->setEnabled ((opts & AF_FLAGS       ) != 0) ;
#ifdef	Q_OS_LINUX
	m_sshTarget ->setEnabled ((opts & AF_SSHTUNNEL   ) != 0) ;
#endif
}


void	KBServerDlg::showPassword
	(	bool		show
	)
{
	m_password->setEchoMode (show ? RKLineEdit::Normal : RKLineEdit::Password) ;
}

/*  KBServerDlg								*/
/*  showDatabases							*/
/*		: Load database list into database combo		*/
/*  (returns)	: void		:					*/

void	KBServerDlg::showDatabases ()
{
	if (m_dbType->currentItem() > 0)
	{
		const QString	tag	= m_drvList[m_dbType->currentItem()].tag() ;
		KBServer	*server	= tag.isEmpty() ? 0 : getDriverServer(tag) ;

		if (server == 0) return ;

		KBServerInfo info
			     (		0,
					m_serverName->text(),
					tag,
					m_hostName  ->text(),
					0,
					m_userName  ->text(),
					m_password  ->text(),
					m_portNumber->text(),
					m_socketName->text()
			     )	;

		if (!server->connect (&info))
		{	server->lastError().DISPLAY() ;
			delete	server	;
			return	;
		}

		QStringList dbList ;
		if (!server->listDatabases (dbList))
		{	server->lastError().DISPLAY() ;
			delete	server	;
			return	;
		}

		QString	current	= m_dbName->currentText() ;

		m_dbName->clear () ;
		m_dbName->insertStringList (dbList) ;

		for (int idx = 0 ; idx < m_dbName->count() ; idx += 1)
			if (m_dbName->text(idx) == current)
			{	m_dbName->setCurrentItem (idx) ;
				break	;
			}

		delete	server	;
	}	
}

/*  KBServerDlg	:							*/
/*  typeChange	: User changes driver tyoe				*/
/*  (returns)	: void		:					*/

void	KBServerDlg::typeChange ()
{
	if (m_dbType->currentItem() > 0)
	{
		/* Drop the server advanced settings and remove all	*/
		/* tabber pages after the fixed set.			*/
		DELOBJ(m_serverData->m_advanced) ;
		while (m_tabber->count() > n_fixedTabs)
			delete m_tabber->page(n_fixedTabs) ;

		QString	 type	 = m_drvList[m_dbType->currentItem()].tag()   ;
		KBServer *server = type.isEmpty() ? 0 : getDriverServer(type) ;

		/* Unless the user has selected the empty entry, see	*/
		/* if the server has any advanced settings and if so	*/
		/* set up the dialog (create additional tabs).		*/
		if (!type.isEmpty())
			m_serverData->m_advanced = getDriverAdvanced (type)  ;
		if (m_serverData->m_advanced != 0)
			m_serverData->m_advanced->setupDialog (m_tabber) ;

		/* Pick up additional flags which can disable some of	*/
		/* the controls.					*/
		bool	 isFile	 = m_serverData->m_serverName == KBLocation::m_pFile ;

		setEnabledAll
		(	!isFile,
			server == 0 ? 0xffffffff : server->optionFlags()
		)	;

		DELOBJ	(server) ;
	}
}

void	KBServerDlg::webBrowse ()
{
	QString	dir	= KBFileDialog::getExistingDirectory
			  (	m_webDirectory->text(),
				TR("Save to web directory")
			  )	;

	if (!dir.isEmpty())
		m_webDirectory->setText (dir) ;
}

void KBServerDlg::setAppFont ()
{
	TKFontDialog  fDialog (this, TR("Application font"), false, true) ;
	fDialog.setFont (KBFont::specToFont (m_appFont->text())) ;
	if (fDialog.exec())
		m_appFont->setText(KBFont::fontToSpec (fDialog.font())) ;
}

void	KBServerDlg::accept ()
{
	/* Ignore the operation if the user has not set a server name	*/
	/* or a database server type.					*/
	if (m_serverName->text().isEmpty())
	{
		TKMessageBox::information
		(	0,
			TR("Please specify a connection name"),
			TR("Cannot save server")
		)	;
		return	;
	}

	if (m_serverName->text() != KBLocation::m_pFile)
		if (m_dbType->currentItem () == 0)
		{
			TKMessageBox::information
			(	0,
				TR("Please specify a server type"),
				TR("Cannot save server")
			)	;
			return	;
		}

	KBDialog::accept ()	;
}

void	KBServerDlg::save ()
{
	m_serverData->m_serverName = m_serverName->text () ;
	m_serverData->m_hostName   = m_hostName  ->text () ;
	m_serverData->m_dbName     = m_dbName    ->currentText () ;
	m_serverData->m_userName   = m_userName  ->text () ;
	m_serverData->m_password   = m_password  ->text () ;
	m_serverData->m_portNumber = m_portNumber->text () ;
	m_serverData->m_socketName = m_socketName->text () ;
	m_serverData->m_flags      = m_flags     ->text () ;
	m_serverData->m_isDisabled = m_disabled  ->isChecked() ;

	if (m_dbType->currentItem() > 0)
		m_serverData->m_dbType  = m_drvList[m_dbType->currentItem()].tag() ;
	else	m_serverData->m_dbType	= QString::null ;

	m_serverData->m_autoStart   = m_autostart ->isChecked() ;
	m_serverData->m_useUserName = m_serverData->m_userName  ;
	m_serverData->m_usePassword = m_serverData->m_password  ;

	QString	   dataEnc    = m_dataEncoding->currentText() ;
	QString	   objEnc     = m_objEncoding ->currentText() ;

	if (!checkCodec (dataEnc, TR("data  "))) return ;
	if (!checkCodec (objEnc,  TR("object"))) return ;

	m_serverData->m_showTests	= (KBServerInfo::TestsEnable)m_showTests->currentItem () ;

	m_serverData->m_noRekallTables	= m_noRekallTables->isChecked  () ;
	m_serverData->m_showAllTables	= m_showAllTables ->isChecked  () ;
	m_serverData->m_cacheTables	= m_cacheTables   ->isChecked  () ;
	m_serverData->m_nullUserPwd	= m_nullUserPwd   ->isChecked  () ;
	m_serverData->m_pkReadOnly	= m_pkReadOnly    ->isChecked  () ;
	m_serverData->m_fakeKeys	= m_fakeKeys      ->isChecked  () ;
	m_serverData->m_readOnly	= m_readOnly      ->isChecked  () ;
	m_serverData->m_autoForm	= m_autoForm      ->text () ;
	m_serverData->m_initSQL		= m_initSQL	  ->text () ;
	m_serverData->m_appFont		= m_appFont	  ->text () ;
	m_serverData->m_skinSuffix	= m_skinSuffix	  ->text () ;
	m_serverData->m_comment		= m_comment	  ->text () ;
	m_serverData->m_webDirectory	= m_webDirectory  ->text () ;

	m_serverData->m_dataEncoding	= dataEnc ;
	m_serverData->m_objEncoding	= objEnc  ;

#ifdef	Q_OS_LINUX
	m_serverData->m_sshTarget	= m_sshTarget	  ->text () ;
#endif
	if (m_serverData->m_advanced != 0)
		m_serverData->m_advanced->saveDialog() ;
}
