/***************************************************************************
    file	         : kb_servertab.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qregexp.h>
#include	<qtextbrowser.h>
#include	<qheader.h>
#include	<qstyle.h>

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_database.h"
#include	"kb_dbinfo.h"
#include	"kb_dbdociter.h"
#include	"kb_notifier.h"
#include	"kb_callback.h"
#include	"kb_appptr.h"
#include	"kb_attrdict.h"
#include	"kb_gui.h"
#include	"kb_options.h"
#include	"kb_tableinfo.h"
#include	"kb_rawsql.h"
#include	"kb_ascii.h"
#include	"kb_dbdlg.h"
#include	"kb_manual.h"

#if		__KB_AUTOMOC
#include	"kb_servertab.moc"
#else
#include 	"kb_servertab.h"
#endif

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_combobox.h"
#include	"rk_listbox.h"
#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"
#include	"kb_serverdlg.h"
#include	"rk_dcop.h"


#define		__DEBUG_SERVERTAB	(0)
#if		__DEBUG_SERVERTAB
#include	"kb_classes.h"
#endif

static	QString	yesNo
	(	bool		yes
	)
{
	return	yes ? TR("Yes") : TR("No") ;
}

static	void	addUnlessEmpty
	(	QString		&text,
		const QString	&legend,
		const QString	&value
	)
{
	if (!value.isEmpty())
		text	+= QString(TR("<tr><td><b>%1</b></td><td>%2</td></tr>")).arg(legend).arg(value) ;
}

/*  ------------------------------------------------------------------	*/

class	KBServerTabIface : public RKDCOPObject
{
	K_DCOP

	KBServerTab	*m_serverTab	;

public	:

	KBServerTabIface
		(	KBServerTab	*serverTab
		)
		;
}	;


KBServerTabIface::KBServerTabIface
	(	KBServerTab	*serverTab
	)
	:
	RKDCOPObject	("rekall-server-list"),
	m_serverTab	(serverTab)
{
}

bool	KBServerTabIface::process
	(	const QCString		&func,
		const QByteArray	&data,
		QCString		&replyType,
		QByteArray		&replyData
	)
{
	DPRINTF
	((	"KBServerTabIface::process: [%s]\n",
		func.data()
	))	;

	if (func == "listServers()")
	{
		QDataStream	reply	(replyData, IO_WriteOnly) ;
		replyType	= "QStringList"	;
		reply	<< m_serverTab->listServers() ;
		return	true	;
	}

	return	RKDCOPObject::process (func, data, replyType, replyData) ;
}

QCStringList
	KBServerTabIface::interfaces()
{
	QCStringList ifaces = RKDCOPObject::interfaces() ;
	ifaces += "ServerList" ;
	return ifaces ;
}

QCStringList
	KBServerTabIface::functions ()
{
	QCStringList funcs = RKDCOPObject::functions() ;

	static	cchar	*f[]	=
	{
		"QStringList listServers()",
		0
	}	;

	for (cchar **fp = &f[0] ; *fp != 0 ; fp += 1)
		funcs += normalizeFunctionSignature(*fp) ;

	return	funcs	;
}

/*  ------------------------------------------------------------------	*/

/*  KBServerTab								*/
/*  KBServerTab	: Constructor for database server dialog		*/
/*  parent	: KBDBaseDlg *	  : Parent database dialog		*/
/*  dbInfo	: KBDBInfoData *  : Database information		*/
/*  gui		: KBaseGUI *	  : GUI for this viewer			*/
/*  (returns)	: KBServerTab	  :					*/

KBServerTab::KBServerTab
	(	QWidget		*parent,
		KBDBaseDlg	*dbdlg,
		KBDBInfoData	*dbInfo,
		KBaseGUI	*
	)
	:
	RKVBox		(parent),
	m_dbaseDlg	(dbdlg ),
	m_dbInfo	(dbInfo)
{
	setTracking	() ;

	RKHBox	*layMain = new RKHBox		(this)	;

	RKVBox	*layServ = new RKVBox		(layMain) ;
	RKVBox	*layDet	 = new RKVBox		(layMain) ;

	QHeader	*sh	 = new QHeader		(layServ) ;
	m_serverList	 = new RKListBox	(layServ) ;

	QHeader	*dh	 = new QHeader		(layDet ) ;
	m_details	 = new QTextBrowser	(layDet ) ;

	layServ->setNoMargin    (false) ;
	layServ->setFrameStyle	(QFrame::StyledPanel|QFrame::Sunken)	;
	layServ->setLineWidth	(style().pixelMetric(QStyle::PM_DefaultFrameWidth)) ;
	layDet ->setNoMargin    (false) ;
	layDet ->setFrameStyle	(QFrame::StyledPanel|QFrame::Sunken)	;
	layDet ->setLineWidth	(style().pixelMetric(QStyle::PM_DefaultFrameWidth)) ;

	sh->addLabel		(TR("Server"  )) ;
	dh->addLabel		(TR("Settings")) ;
	sh->setStretchEnabled	(true)		 ;
	dh->setStretchEnabled	(true)		 ;

	layMain->setStretchFactor (layServ, 0) ;
	layMain->setStretchFactor (layDet,  1) ;

	RKHBox *layButt = new RKHBox (this) ;

	layButt->addFiller () ;
	QPushButton *h	= new KBManualPushButton(layButt, KBHELPKEY("Chap3ServerDialog")) ;
	m_bNew		= new RKPushButton	(TR("New"   ), layButt) ;
	m_bEdit		= new RKPushButton	(TR("Edit"  ), layButt) ;
	m_bDelete	= new RKPushButton	(TR("Delete"), layButt) ;

	layButt->setNoMargin(false) ;
	KBDialog::setupLayout (layButt) ;
//	layButt->setMargin  (KBOptions::getDlgMargin  ()) ;
//	layButt->setSpacing (KBOptions::getDlgSpacing ()) ;

	KBDialog::sameSize
	(
		h,
		m_bNew,
		m_bEdit,
		m_bDelete,
		0
	)	;

	connect (m_bNew,       SIGNAL(clicked	 ()), 	 SLOT(clickNew   ())) ;
	connect (m_bEdit,      SIGNAL(clicked	 ()), 	 SLOT(clickEdit  ())) ;
	connect (m_bDelete,    SIGNAL(clicked  	 ()), 	 SLOT(clickDelete())) ;
	connect	(m_serverList, SIGNAL(selected	(int)),  SLOT(clickEdit	 ())) ;
	connect	(m_serverList, SIGNAL(highlighted(int)), SLOT(showServer ())) ;

	/* Load the driver list. Note that we insert a null first	*/
	/* entry into the driver list so that we can have an empty	*/
	/* first entry in the combo boc.				*/
	KBError			error	;
	if (KBServer::listDrivers (m_drvList, error))
	{
		DPRINTF
		((	"KBServerTab::KBServerTab: Got %d drivers\n",
			m_drvList.count()
		))	;

		m_drvList.insert (m_drvList.begin(), KBDriverDetails()) ;
	}

	loadDBInfo    () ;

	int	w = m_serverList->sizeHint().width() ;
	if	(w < 100) w = 100 ;
	else if (w > 150) w = 150 ;
	m_serverList ->setMinimumWidth (w) ;

	m_dcopIface	= new KBServerTabIface (this) ;
}

/*  KBServerTab								*/
/*  ~KBServerTab: Destructor for database server dialog			*/
/*  (returns)	:		:					*/

KBServerTab::~KBServerTab ()
{
	delete	m_dcopIface	;
}

/*  KBServerTab								*/
/*  loadDBInfo	: Load database information				*/
/*  (returns)	: void		:					*/

void	KBServerTab::loadDBInfo ()
{
	KBServerInfo  *svInfo  ;

	m_serverList->clear () ;

	/* Insert the !Files entry first so that it appears in front of	*/
	/* all other server entries.					*/
	svInfo	= m_dbInfo->getFilesServer() ;
	m_serverList->insertItem (svInfo->m_serverName) ;

	QPtrListIterator<KBServerInfo> *svIter ;

	for
	(
		svIter  = m_dbInfo->getServerIter() ;
		(svInfo = svIter->current()) != 0   ;
		(*svIter) += 1
	)
		if (svInfo != m_dbInfo->getFilesServer())
		{
			KBDBLink      dbLink  ;
			m_serverList->insertItem (svInfo->m_serverName) ;

			if (!svInfo->disabled())
				if (!dbLink.connect (m_dbInfo, svInfo->m_serverName, true))
				{
					dbLink.lastError().DISPLAY() ;
					svInfo->m_isDisabled = true  ;
				}
		}

	delete	svIter	;

	showServer (0)	;
	m_serverList->setCurrentItem (0) ;
}

/*  KBServerTab								*/
/*  showServerInfo							*/
/*		: Show server information				*/
/*  svInfo	: KBServerData * : Server				*/
/*  enable	: bool		 : True to enable update		*/
/*  (returns)	: void		 :					*/

void	KBServerTab::showServerInfo
	(	KBServerData	*svInfo
	)
{
	QString	text	;
	QString	driver	;

	if (svInfo->m_serverName == KBLocation::m_pFile)
		if (svInfo->m_dbType.isEmpty())
		{
			m_details->setText (TR("<qt><b>No database</b></qt>")) ;
			return	;
		}

	for (uint idx = 1 ; idx < m_drvList.count() ; idx += 1)
		if (m_drvList[idx].tag() == svInfo->m_dbType)
		{	driver	= m_drvList[idx].comment() ;
			break	;
		}

	if (driver.isEmpty())
		if (!svInfo->m_dbType.isEmpty())
			driver	+ QString(TR("Unknown driver tag: %1")).arg(svInfo->m_dbType) ;

	text	+= "<qt>"	;
	text	+= "<table>"	;

	addUnlessEmpty (text, TR("Driver"), 		driver) ;
	addUnlessEmpty (text, TR("Host name"),		svInfo->m_hostName) ;
	addUnlessEmpty (text, TR("Database name"),	svInfo->m_dbName  ) ;
	addUnlessEmpty (text, TR("User name"),		svInfo->m_userName) ;
	addUnlessEmpty (text, TR("Disabled"),		yesNo(svInfo->m_isDisabled)) ;
	addUnlessEmpty (text, TR("AutoStart"),		yesNo(svInfo->m_autoStart )) ;

	text	+= "</table>"	;
	text	+= "</qt>"	;

	m_details->setText (text) ;
}

/*  KBServerTab								*/
/*  showServer	: Show server values					*/
/*  serverNo	: int		: Server index in list			*/
/*  (returns)	: void		:					*/

void	KBServerTab::showServer
	(	int	serverNo
	)
{
	KBServerData *svInfo = m_dbInfo->findServer (m_serverList->text (serverNo)) ;
	if (svInfo == 0) return ; 

	showServerInfo (svInfo) ;
}

/*  KBServerTab								*/
/*  showServer	: Show server values					*/
/*  (returns)	: void		:					*/

void	KBServerTab::showServer ()
{
	showServer (m_serverList->currentItem()) ;
}

void	KBServerTab::editServer
	(	bool		isNew
	)
{
	int		serverNo   = 0	;
	KBServerData	*svCurrent = 0	;
	KBServerData	*svCopy 	;

	if (isNew)
	{
		svCopy	= new KBServerData (m_dbInfo)  ;
		svCopy->m_serverName = TR("<unnamed>") ;
	}
	else
	{
		serverNo  = m_serverList->currentItem() ;
		svCurrent = m_dbInfo->findServer (m_serverList->currentText()) ;

		if (svCurrent == 0)
			return	; 

		if (svCurrent->linkIsOpen ())
		{
			TKMessageBox::sorry
			(	0,
				QString(TR("Server %1 has an open connection")).arg(svCurrent->m_serverName),
				TR("Cannot edit server")
			)	;
			return	;
		}

		svCopy	= new KBServerData (m_dbInfo, svCurrent) ;
	}

	KBServerDlg	svDlg	(svCopy, m_drvList) ;
	if (!svDlg.exec(isNew))
	{
		delete	svCopy	;
		return	;
	}

	svDlg.save ()	;

	/* At this point, unless the server has been marked as		*/
	/* disabled or there is no server type (in which case it must	*/
	/* be the !Files entry), we connect to the server. This will	*/
	/* disable the server if no connection can be made.		*/
	if (!svCopy->m_isDisabled && !svCopy->m_dbType.isEmpty())
	{
		KBError	error	;
		if (svCopy->getServer(error) == 0) error.DISPLAY() ;
	}

	QString	oldName	;

	if (isNew)
	{
		m_serverList->insertItem (svCopy->m_serverName) ;
	}
	else
	{
		oldName	= svCurrent->m_serverName ;
		m_serverList->changeItem (svCopy->m_serverName, serverNo) ;
		m_dbInfo->remove (svCurrent, false) ;
		delete	svCurrent ;
	}

	m_dbInfo->insert (svCopy) ;
	m_dbInfo->save   () ;

	showServerInfo	 (svCopy) ;

	/* Issue a notification that the server (may have) changed, so	*/
	/* that such things as the table lists are kept up to date.	*/
	KBLocation location
		   (
			m_dbInfo,
			"unknown",
			oldName,
			svCopy->m_serverName
		   )	;
	KBNotifier::self()->nServerChanged (location) ;
}

/*  KBServerTab								*/
/*  clickEdit	: Handle edit operation					*/
/*  (returns)	: void		:					*/

void	KBServerTab::clickEdit ()
{
	editServer (false) ;
}

/*  KBServerTab								*/
/*  clickNew	: Handle new operation					*/
/*  (returns)	: void		:					*/

void	KBServerTab::clickNew ()
{
	editServer (true) ;
}

/*  KBServerTab								*/
/*  clickDelete	: Handle delete operation				*/
/*  (returns)	: void		:					*/

void	KBServerTab::clickDelete ()
{
	uint		svNo	= m_serverList->currentItem () ;
	KBServerData	*svInfo = m_dbInfo->findServer (m_serverList->text (svNo)) ;

	if ((svInfo == 0) || (svInfo->m_serverName == KBLocation::m_pFile))
		return	;

	QString		oldName	= svInfo->m_serverName ;

	m_dbInfo->remove (svInfo) ;
	m_dbInfo->save   () ;

	/* Issue a notification then the server (may have) changed, so	*/
	/* that such things as the table lists are kept up to date.	*/
	KBLocation location
		   (
			m_dbInfo,
			"unknown",
			oldName,
			""
		   )	;
	KBNotifier::self()->nServerChanged (location) ;

	m_serverList->removeItem     (svNo) ;
	m_serverList->setCurrentItem (0)	 ;
	showServer  (0) ;
}

/*  KBServerTab								*/
/*  autoStart	: Check for auto-start forms				*/
/*  svInfo	: KBServerInfo * : Server to check			*/
/*  autoLoc	: KBLocation &	 : Return started location if any	*/
/*  (returns)	: bool		 : Form opened				*/

bool	KBServerTab::autoStartForm
	(	KBServerInfo		*svInfo,
		KBLocation		&autoLoc
	)
{
	QDict<QString> empty	;
	KBError	   error	;

#if	! __KB_RUNTIME
	/* The autostart option is ignored in the runtime version, that	*/
	/* is, we always act as if it is set.				*/
	if (!svInfo->m_autoStart)
		return	false	;
#endif

	autoLoc	= KBLocation
		  (	m_dbInfo,
			"form",
			svInfo->serverName(),
			svInfo->autoForm  ().isEmpty() ?
				QString("MainForm") :
				svInfo->autoForm ()
		  )	;

	DPRINTF
	((	"KBServerTab::autoStartForm: check [%s][%s]\n",
		(cchar *)autoLoc.server(),
		(cchar *)autoLoc.name  ()
	))	;

	QString	doc	= autoLoc.contents (error) ;
	if (doc.isEmpty())
		return	false	;

	DPRINTF
	((	"KBServerTab::autoStartForm: got [%s]\n",
		(cchar *)autoLoc.ident()
	))	;

	return	true ;
}

/*  KBServerTab								*/
/*  autoStart	: Check for auto-start forms				*/
/*  autoLoc	: KBLocation &	: Return started location if any	*/
/*  (returns)	: bool		: Form opened				*/

bool	KBServerTab::autoStart
	(	KBLocation	&autoLoc
	)
{
	QPtrListIterator<KBServerInfo> *svIter = m_dbInfo->getServerIter() ;
	KBServerInfo   *svInfo	;
	KBError	       error	;
	bool	       rc	= false	;

	while ((svInfo = svIter->current()) != 0)
	{
		if (autoStartForm (svInfo, autoLoc))
		{	rc = true ;
			break	  ;
		}

		(*svIter) += 1	;
	}

	delete	svIter	;

	if (!rc)
		if ((svInfo = m_dbInfo->getFilesServer()) != 0)
			if (autoStartForm (svInfo, autoLoc))
				rc	= true	;

	return	rc	;
}

void	KBServerTab::showRawSQL
	(	KBaseApp	*kbaseApp
	)
{
	QString		server	= m_serverList->currentText()   ;
	KBServerData	*svInfo	= m_dbInfo->findServer(server) ;

	if (svInfo != 0)
		svInfo->showRawSQL (kbaseApp) ;
}

QStringList
	KBServerTab::listServers ()
{
	QStringList servers	;

	QPtrListIterator<KBServerInfo> *svIter = m_dbInfo->getServerIter() ;
	KBServerInfo   *svInfo	;

	while ((svInfo = svIter->current()) != 0)
	{
		servers.append (svInfo->serverName()) ;
		(*svIter) += 1	;
	}

	delete	svIter	;
	return	servers	;
}

#if	! __KB_RUNTIME

void	KBServerTab::dumpDatabase
	(	KBaseApp	*
	)
{
	extern	void	dumpDatabase
			(	KBDBInfo	*,
				const QString	&,
				const QString	&
			)	;

	KBError	error	;

	QString	target	= KBFileDialog::getExistingDirectory
			  (	QString::null,
				TR("Dump database to .....")
			  )	;
	if (target.isNull()) return ;

	dumpDatabase (m_dbInfo, m_serverList->currentText(), target) ;
}

void	KBServerTab::loadDatabase
	(	KBaseApp	*
	)
{
	extern	void	loadDatabase
			(	KBDBInfo	*,
				const QString	&,
				const QString	&
			)	;

	KBError	error	;

	QString	target	= KBFileDialog::getExistingDirectory
			  (	QString::null,
				TR("Load database from .....")
			  )	;
	if (target.isNull()) return ;

	loadDatabase (m_dbInfo, m_serverList->currentText(), target) ;
}
#endif
