/***************************************************************************
    file	         : kb_splash.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<stdlib.h>

#include	<qapplication.h>
#include	<qpixmap.h>
#include	<qimage.h>
#include	<qpainter.h>


#include	"kb_classes.h"
#include	"kb_dialog.h"
#include	"kb_locator.h"

#if		__KB_AUTOMOC
#include	"kb_splash.moc"
#else
#include	"kb_splash.h"
#endif

#ifdef		Q_OS_WIN32
#define		sleep(n)	_sleep(n)
#endif

#ifndef		Q_OS_WIN32
#include	<unistd.h>
#endif

/*  KBSplash								*/
/*  KBSplash	: Constructor for splash screen class			*/
/*  screen	: const QString & : Splash screen file name		*/
/*  showFor	: uint		  : Display time in milliseconds	*/
/*  color	: QColor	  : Text colour				*/
/*  (returns)	: KBSplash	  :					*/

KBSplash::KBSplash
	(	const QString	&screen,
		uint		showFor,
		QColor		color
	)
	:
	QLabel		(0, 0, WStyle_Customize|WStyle_StaysOnTop|WStyle_NoBorder|WDestructiveClose),
#ifdef	__KB_RKLITE
	m_release	(QString("%1: Click anywhere to close splash screen")
				.arg(__KB_BUILD_RELEASE)
			),
#else
	m_release	(QString("%1%2 [%2]")
				.arg(__KB_BUILD_RELEASE)
				.arg(__KB_BUILD_BUILD  )
				.arg(__KB_BUILD_TIME   )
			),
#endif
	m_color		(color)
{
#if	__KB_RUNTIME
	m_release	+= " Runtime" ;
#endif
//	fprintf
//	(	stderr,
//		"KBSplash::KBSplash: [%s]\n",
//		(cchar *)m_release
//	)	;


        QRect	dRect	= QApplication::desktop()->geometry() ;
	QString	path	= locateFile ("appdata", QString("%1.png").arg(screen)) ;
	QPixmap	pm	;
	pm.convertFromImage (QImage(path)) ;

        setFrameStyle	(QFrame::WinPanel|QFrame::Raised) ;
        setPixmap	(pm)	;
        adjustSize	()	;
        setCaption	(TR("Rekall")) ;

        move		(dRect.center() - rect().center()) ;
        show		()	;
        repaint		(false) ;
	QApplication::flush ()	;

#ifndef	__KB_RKLITE
	QTimer::singleShot (showFor, this, SLOT(slotSplosh    ())) ;
#endif
}

/*  KBSplash								*/
/*  paintEvent	: Handle paint events					*/
/*  p		: QPaintEvent *	: Paint event				*/
/*  (returns)	: void		:					*/

void	KBSplash::paintEvent
	(	QPaintEvent	*p
	)
{
	QLabel::paintEvent (p) ;

	QPainter paint	(this) ;
	int	 offset	= fontMetrics().height() ;

	paint.setPen	(m_color) ;
	paint.drawText	(offset, offset * 2, m_release) ;

	QApplication::flush () ;
	QApplication::syncX () ;
}

/*  KBSplash								*/
/*  loadHighlights: Load text editor highlights				*/
/*  (returns)	  : void		:				*/

void	KBSplash::loadHighlights ()
{
}

/*  KBSplash								*/
/*  slotSplosh	: Close the screen					*/
/*  (returns)	: void		:					*/

void	KBSplash::slotSplosh ()
{
	close	(true)	;
}

/*  KBSplash								*/
/*  mousePressEvent							*/
/*		: Handle mouse press					*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBSplash::mousePressEvent
	(	QMouseEvent	*
	)
{
	close	(true)	;
}

/*  ------------------------------------------------------------------	*/

/*  showSlashScreen							*/
/*		: Show splash screen					*/
/*  screen	: const QString & : Splash screen file name		*/
/*  showFor	: uint		  : Display time in milliseconds	*/
/*  color	: QColor	  : Text colour				*/
/*  (returns)	: KBSplash	  :					*/

void	showSlashScreen
	(	const QString	&screen,
		uint		showFor,
		QColor		color
	)
{
	QGuardedPtr<KBSplash> splash = new KBSplash (screen, showFor, color) ;
	while (splash)
	{
		sleep (1) ;
		qApp->processEvents () ;
	}
}
