/***************************************************************************
    file	         : kb_filelist.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<errno.h>

#include	<qdatetime.h>
#include	<qtabwidget.h>


#include	"kb_classes.h"
#include	"kb_dbinfo.h"
#include	"kb_dbdociter.h"
#include	"kb_serverinfo.h"
#include	"kb_notifier.h"
#include	"kb_svrchooser.h"
#include	"kb_prompt.h"
#include	"kb_file.h"

#if		__KB_AUTOMOC
#include	"kb_filelist.moc"
#else
#include 	"kb_filelist.h"
#endif

#include	"kb_callback.h"
#include	"kb_appptr.h"

#include	"tk_icons.h"
#include	"tk_messagebox.h"
#include	"tk_filedialog.h"
#include	"rk_dcop.h"


#define		__DEBUG_FILELIST	(0)
#if		__DEBUG_FILELIST
#include	"kb_dprintf.h"
#endif

static	QString	lHead	("0_")	;
static	QString	lSort	("1_")	;
static	QString	lTail	("2_")	;
static	QString	webDir		;


static	cchar	*makeDCOPName
	(	const QString	&tabType
	)
{
	return	qstrdup (QString("rekall-part-" + tabType).latin1()) ;
}


/*  ------------------------------------------------------------------	*/

class	KBFileListIface : public RKDCOPObject
{
	K_DCOP

	KBFileList	*m_fileList	;

public	:

	KBFileListIface
		(	KBFileList	*fileList,
			cchar		*name
		)
		;
}	;


KBFileListIface::KBFileListIface
	(	KBFileList	*fileList,
		cchar		*name
	)
	:
	RKDCOPObject	(name),
	m_fileList	(fileList)
{
}

bool	KBFileListIface::process
	(	const QCString		&func,
		const QByteArray	&data,
		QCString		&replyType,
		QByteArray		&replyData
	)
{
	DPRINTF
	((	"KBFileListIface::process: [%s]\n",
		func.data()
	))	;

	if (func == "openObject(QString,QString,int)")
	{
		QString		server	;
		QString		name	;
		int		mode	;
		QDataStream	args	(data,	    IO_ReadOnly ) ;
		QDataStream	reply	(replyData, IO_WriteOnly) ;

		if (args.atEnd()) return false;
		args >> server	;
		if (args.atEnd()) return false;
		args >> name	;
		if (args.atEnd()) return false;
		args >> mode	;

		replyType	= "bool"	;
		reply	<< m_fileList->dcopOpenObject (server, name, mode) ;
		return	true	;
	}

	if (func == "openTextObject(QString,QString,int)")
	{
		QString		name	;
		QString		text	;
		int		mode	;
		QDataStream	args	(data,	    IO_ReadOnly ) ;
		QDataStream	reply	(replyData, IO_WriteOnly) ;

		if (args.atEnd()) return false;
		args >> name	;
		if (args.atEnd()) return false;
		args >> text	;
		if (args.atEnd()) return false;
		args >> mode	;

		replyType	= "bool"	;
		reply	<< m_fileList->dcopOpenTextObj (name, text, mode) ;
		return	true	;
	}

	if (func == "listObjects(QString)")
	{
		QString		server	;
		QDataStream	args	(data,	    IO_ReadOnly ) ;
		QDataStream	reply	(replyData, IO_WriteOnly) ;

		if (args.atEnd()) return false;
		args >> server	;

		replyType	= "QStringList"	;
		reply	<<  m_fileList->dcopListObjects (server) ;
		return	true	;
	}

	return	RKDCOPObject::process (func, data, replyType, replyData) ;
}

QCStringList
	KBFileListIface::interfaces()
{
	QCStringList ifaces = RKDCOPObject::interfaces() ;
	ifaces += "ObjectList" ;
	return ifaces ;
}

QCStringList
	KBFileListIface::functions ()
{
	QCStringList funcs = RKDCOPObject::functions() ;

	static	cchar	*f[]	=
	{
		"bool openObject(QString server, QString name, int mode)",
		"bool openTextObject(QString server, QString name, int mode)",
		"QStringList listObjects(QString server)",
		0
	}	;

	for (cchar **fp = &f[0] ; *fp != 0 ; fp += 1)
		funcs += normalizeFunctionSignature(*fp) ;

	return	funcs	;
}

/*  ------------------------------------------------------------------	*/

/*  KBFileList								*/
/*  KBFileList	: Constructor for file list				*/
/*  parent	: QWidget *	  : Parent widget			*/
/*  dbInfo	: KBDBInfo *	  : Database information object		*/
/*  newtext	: cchar *	  : Text for new object entry		*/
/*  icon	: cchar *	  : Icon for entries			*/
/*  col0	: cchar *	  : Column zero title			*/
/*  col2	: cchar *	  : Column two title or null		*/
/*  col3	: cchar *	  : Column three title or null		*/
/*  (returns)	: KBFileList	  :					*/

KBFileList::KBFileList
	(	QWidget		*parent,
		KBDBInfo	*dbInfo,
		cchar		*newtext,
		cchar		*icon,
		cchar		*tabType,
		cchar		*col0,
		cchar		*col2,
		cchar		*col3
	)
	:
	QListView	(parent),
	m_parent	(parent),
	m_dbInfo	(dbInfo),
	m_newtext	(newtext),
	m_icon		(icon),
	m_tabType	(tabType),
	m_wizard	(false),
	m_web		(false)
{
	m_dcopIface	= new KBFileListIface (this, makeDCOPName (tabType)) ;

	if (col0 != 0)
	{
		addColumn	   (col0,           200) ;
		addColumn	   (TR("Modified"), 200) ;
		setColumnWidthMode (0, QListView::Maximum) ;
		setColumnWidthMode (1, QListView::Maximum) ;
		if (col2 != 0)
		{	addColumn	   (col2) ;
			setColumnWidthMode (columns() - 1, QListView::Maximum) ;
		}
		if (col3 != 0)
		{	addColumn	   (col3) ;
			setColumnWidthMode (columns() - 1, QListView::Maximum) ;
		}

		setResizeMode      (QListView::LastColumn) ;
	}

	setRootIsDecorated (true)  ;
	setSorting	   (0)	   ;


	connect
	(	this,
		SIGNAL(doubleClicked    (QListViewItem *)),
		SLOT  (showDefault      (QListViewItem *))
	)	;
	connect
	(	this,
		SIGNAL(returnPressed    (QListViewItem *)),
		SLOT  (showDefault      (QListViewItem *))
	)	;
	connect
	(	this,
		SIGNAL(contextMenuRequested
					(QListViewItem *, const QPoint &, int)),
		SLOT  (showMenu         (QListViewItem *, const QPoint &, int))
	)	;

	connect
	(	KBNotifier::self(),
		SIGNAL (sServerChanged(const KBLocation &)),
		SLOT   (serverChanged (const KBLocation &))
	)	;

	connect
	(	KBNotifier::self(),
		SIGNAL (sObjectChanged(const KBLocation &)),
		SLOT	 (objChange	(const KBLocation &))
	)	;


	KBListItem *item ;

	item	= new KBServerItem (this, lHead, KBLocation::m_pFile) ;
	item->setPixmap (0, getSmallIcon(m_tabType == "table" ? "database" : "folder_open")) ;

	QPtrListIterator<KBServerInfo> *svIter = dbInfo->getServerIter() ;
	KBServerInfo		    *svInfo ;

	while ((svInfo = svIter->current()) != 0)
	{
		if (!svInfo->disabled())
		{
			item = new KBServerItem (this, lSort, svInfo->serverName()) ;
			item->setPixmap (0, getSmallIcon("database")) ;
		}

		(*svIter) += 1 ;
	}

	delete	svIter	;
	m_curState = 0	;
}

/*  KBFileList								*/
/*  ~KBFileList	: Destructor for form list widget			*/
/*  (returns)	:		:					*/

KBFileList::~KBFileList ()
{
	delete	m_dcopIface	;
}

/*  KBFileList								*/
/*  dcopOpenObject							*/
/*		: Open object in response to a DCOP request		*/
/*  server	: const QString & : Server name				*/
/*  name	: const QString & : Object name				*/
/*  mode	: int		  : Open mode (actually KB::ShowAs)	*/
/*  (returns)	: bool		  : Success				*/

bool	KBFileList::dcopOpenObject
	(	const QString	&server,
		const QString	&name,
		int		mode
	)
{
	DPRINTF
	((	"KBFileList::dcopOpenObject: [%s][%s]\n",
		server.latin1(),
		name  .latin1()
	))	;

	KBLocation	location
			(	m_dbInfo,
				m_tabType,
				server,
				name,
				getDocExtension()
			)	;

	KBCallback	*cb	 = KBAppPtr::getCallback() ;
	KBError		error 	 ;
	QDict<QString>	empty	 ;

	return	cb->openObject (0, location, (KB::ShowAs)mode, empty, error) != KB::ShowRCError ;
}

/*  KBFileList								*/
/*  dcopOpenTextObj							*/
/*		: Open object in response to a DCOP request		*/
/*  name	: const QString & : Name to use for object		*/
/*  text	: const QString & : Object definition			*/
/*  mode	: int		  : Open mode (actually KB::ShowAs)	*/
/*  (returns)	: bool		  : Success				*/

bool	KBFileList::dcopOpenTextObj
	(	const QString	&name,
		const QString	&text,
		int		mode
	)
{
	/* This is the same as the ::dcopOpenObject method above	*/
	/* except that the document definition is passed rather than	*/
	/* the name.							*/
	DPRINTF
	((	"KBFileList::dcopOpenTextObj: [%s][%24.24s....]\n",
		name  .latin1(),
		text  .latin1()
	))	;

	KBLocation	location
			(	m_dbInfo,
				m_tabType,
				KBLocation::m_pInline,
				name,
				text
			)	;

	KBCallback	*cb	 = KBAppPtr::getCallback() ;
	KBError		error 	 ;
	QDict<QString>	empty	 ;

	return	cb->openObject (0, location, (KB::ShowAs)mode, empty, error) != KB::ShowRCError ;
}

/*  KBFileList								*/
/*  dcopListObjects							*/
/*		: List objects in response to a DCOP request		*/
/*  (returns)	: QStringList	  : Object list				*/

QStringList
	KBFileList::dcopListObjects
	(	const QString	&server
	)
{

	/* Nothing to do if the server is disabled ....			*/
	KBServerInfo *s = m_dbInfo->findServer (server) ;
	if ((s != 0) && s->disabled()) return QStringList() ;

	QStringList	objects	;
	KBError		error	;
	KBDBDocIter	docIter ;

	/* Initialise the document iterator. Here we do not consider it	*/
	/* an error if there is no objects table, we simply won't	*/
	/* display anything.						*/
	if (!docIter.init
		(	m_dbInfo,
			server,
			m_tabType,
			KBLocation::extnForType (m_dbInfo, m_tabType, getDocExtension()),
			error,
			false
		))
	{
		return	QStringList () ;
	}

	QString	name	;
	QString	stamp	;

	/* Now iterate through and add entries for each document. We	*/
	/* convert the timestamp to something more readable.		*/
	while (docIter.getNextDoc (name, stamp))
		objects.append (name) ;

	return	objects	;
}

/*  KBFileList								*/
/*  itemToLocation							*/
/*		: Build location object for specified item		*/
/*  item	: KBListItem *	: The item				*/
/*  location	: KBLocation &	: Return location			*/
/*  (returns)	: bool		: Success				*/

bool	KBFileList::itemToLocation
	(	KBListItem	*item,
		KBLocation	&location
	)
{
	if (item->type() != KBListItem::Object) return false ;


	QString	docLocn  = item->parent()->text (0) ;
	QString	docName  = item->	   text (0) ;

	location = KBLocation
		   (	m_dbInfo,
			m_tabType,
			docLocn,
			docName,
			getDocExtension()
		   )	;
	return	true	;
}


/*  KBFileList								*/
/*  reloadServer: Load/reload lists					*/
/*  server	: KBServerItem * : Server to reload			*/
/*  (returns)	: void		 :					*/

void	KBFileList::reloadServer
	(	KBServerItem	*server
	)
{
	KBError	    error   ;
	KBDBDocIter docIter ;

	/* Clear the existing entries. There doesn't seem to be a Qt	*/
	/* call to do this, so do it by hand. Then create the new-entry	*/
	/* item or items.						*/
	KBListItem  *t	;
	while ((t = (KBListItem *)server->firstChild()) != 0)
		delete t ;

#if	! __KB_RUNTIME
	t = new KBListItem (server, lHead, m_newtext) ;
	t->setPixmap (0, getSmallIcon("filenew")) ;
	t->setType   (KBListItem::Create) ;

	if (m_wizard)
	{
		t = new KBListItem (server, lHead, QString(TR("%1 with wizard")).arg(m_newtext)) ;
		t->setPixmap (0, getSmallIcon("filenew")) ;
		t->setType   (KBListItem::Wizard) ;
	}
#endif

	/* Nothing to do if the server is disabled ....			*/
	KBServerInfo *s = m_dbInfo->findServer (server->text(0)) ;
	if ((s != 0) && s->disabled()) return ;


	/* Initialise the document iterator. Here we do not consider it	*/
	/* an error if there is no objects table, we simply won't	*/
	/* display anything.						*/
	if (!docIter.init
		(	m_dbInfo,
			server->text (0),
			m_tabType,
			KBLocation::extnForType (m_dbInfo, m_tabType, getDocExtension()),
			error,
			false
		))
	{
		error.DISPLAY() ;
		return	;
	}

	DPRINTF
	((	"KBFileList::reloadServer: scan ....\n"
	))	;

	QString	name	;
	QString	stamp	;

	/* Now iterate through and add entries for each document. We	*/
	/* convert the timestamp to something more readable.		*/
	while (docIter.getNextDoc (name, stamp))
	{
		int	tm_year	;
		int	tm_mon	;
		int	tm_mday	;
		int	tm_hour	;
		int	tm_min	;
		int	tm_sec	;

		DPRINTF
		((	"KBFileList::reloadServer: scan [%s][%s]\n",
			name .latin1(),
			stamp.latin1()
		))	;

		if (!stamp.isNull())
			sscanf	  ((cchar *)stamp, "%4d%2d%2d%2d%2d%2d",
						   &tm_year, &tm_mon,
						   &tm_mday, &tm_hour,
						   &tm_min,  &tm_sec
				  )	;

		t = new KBObjectItem
		    (	server,
			lSort,
			name,
			QDateTime
			(	QDate(tm_year, tm_mon, tm_mday),
				QTime(tm_hour, tm_min, tm_sec )
			)	.toString()
		    )	;

		t->setPixmap (0, getSmallIcon(m_icon)) ;
	}
}

/*  KBFileList								*/
/*  reloadServer: Load/reload lists					*/
/*  (returns)	: void		  :					*/

void	KBFileList::reloadServer ()
{
	if ((m_curItem != 0) && (m_curItem->type() == KBListItem::Server))
		reloadServer ((KBServerItem *)m_curItem) ;
}

/*  KBFileList								*/
/*  showObjecAs: Handle request to open an object in specified mode	*/
/*  item	: KBListItem *	: Selected item				*/
/*  showAs	: KB::ShowAs	: Show in design mode			*/
/*  (returns)	: void		:					*/

void	KBFileList::showObjectAs
	(	KBListItem	*item,
		KB::ShowAs	showAs
	)
{
	KBLocation	location ;
	KBError		error 	 ;
	QDict<QString>	empty	 ;
	KBCallback	*cb	 = KBAppPtr::getCallback() ;

	if (!itemToLocation (item, location))
		return ;

	if (cb->openObject (0, location, showAs, empty, error) == KB::ShowRCError)
		error.DISPLAY () ;
}

/*  KBFileList								*/
/*  createByWizard							*/
/*		: Handle request to create object using a wizard	*/
/*  item	: KBListItem *	: Selected item				*/
/*  (returns)	: void		:					*/

void	KBFileList::createByWizard
	(	KBListItem	*
	)
{
	/* This is just a place holder, and will be implemented by	*/
	/* object types that have creation wizards.			*/
}

/*  KBFileList								*/
/*  showDefault	: Default double-click/return operation			*/
/*  item	: QListViewItem *: Selected item			*/
/*  (returns)	: void		 :					*/

void	KBFileList::showDefault
	(	QListViewItem *item
	)
{
	if ((m_curItem = (KBListItem *)item) == 0)
		return ;

	if (m_curItem->type() == KBListItem::Create)
	{
		KBError		error	 ;
		KBCallback	*cb	= KBAppPtr::getCallback() ;

		KBLocation	location
				(	m_dbInfo,
					m_tabType,
					m_curItem->parent()->text(0),
					"",
					getDocExtension()
				)	;

		if (! cb->newObject (location, error)) error.DISPLAY() ;
		return	;
	}

	if (m_curItem->type() == KBListItem::Wizard)
	{
		createByWizard (m_curItem) ;
		return	;
	}

	if (m_curItem->type() == KBListItem::Object)
	{
		showObjectAs   (m_curItem, KB::ShowAsData) ;
		return	;
	}
}

/*  KBFileList								*/
/*  showAsData	: Show popup item in data view				*/
/*  (returns)	: void		 :					*/

void	KBFileList::showAsData ()
{
	showObjectAs (m_curItem, KB::ShowAsData  ) ;
}

/*  KBFileList								*/
/*  showAsDesign: Show popup item in design view			*/
/*  (returns)	: void		 :					*/

void	KBFileList::showAsDesign ()
{
#if	! __KB_RUNTIME
	showObjectAs (m_curItem, KB::ShowAsDesign) ;
#endif
}

/*  KBFileList								*/
/*  canOperate	: Check if we can operate on a specified object type	*/
/*  location	: KBLocation &	: Location for operation		*/
/*  op		: cchar *	: Putative operation			*/
/*  (returns)	: bool		: Operation allowed			*/

bool	KBFileList::canOperate
	(	const KBLocation	&location,
		cchar			*op
	)
{
	if (KBAppPtr::getCallback()->showingObj (location) != 0)
	{
		TKMessageBox::sorry
		(	0,
			QString (TR("%1 %2 is currently open")).arg(m_tabType).arg(location.name()),
			QString	(TR("Unable to %1")).arg(op)
		)	;
		return	false	;
	}

	return	true	;
}

/*  KBFileList								*/
/*  canOperate	: Check if we can operate on a specified object type	*/
/*  item	: KBListItem *	: Item selected for operation		*/
/*  op		: cchar *	: Putative operation			*/
/*  (returns)	: bool		: Operation allowed			*/

bool	KBFileList::canOperate
	(	KBListItem	*item,
		cchar		*op
	)
{
	KBLocation location ;

	if (!itemToLocation (item, location))
		return false ;

	return	canOperate (location, op) ;
}

/*  KBFileList								*/
/*  rename	: Rename an object					*/
/*  item	: KBListItem *	: Item selected for operation		*/
/*  (returns)	: void		:					*/

void	KBFileList::rename
	(	KBListItem	*item
	)
{
	KBLocation location ;
	KBError	   error    ;

	if (!itemToLocation (item, location)) return ;
	if (!canOperate (location, "rename")) return ;

	/* Prompt the user for a new name for the object, with the	*/
	/* usual get-out.						*/
	QString	newName	;
	if (!doPrompt
	    (	QString (TR("Rename %1 ...")).arg(m_tabType),
	    	QString (TR("Please enter the new %1 name")).arg(m_tabType),
		newName
	    ))	return  ;


	if (!location.rename (newName, error)) error.DISPLAY() ;

	/* Reload the form list. Do this whether or not there was an	*/
	/* error; this ensures that the displayed list is correct.	*/
	reloadServer ((KBServerItem *)item->parent()) ;
}

/*  KBFileList								*/
/*  delobj	: Delete an object					*/
/*  item	: KBListItem *	: Item selected for operation		*/
/*  (returns)	: void		:					*/

void	KBFileList::delobj
	(	KBListItem	*item
	)
{
	KBLocation location ;
	KBError	   error    ;

	if (!itemToLocation (item, location)) return ;
	if (!canOperate (location, "delete")) return ;

	if (TKMessageBox::questionYesNo
	   	(	0,
			QString (TR("Definitely delete %1?")).arg(location.name()),
			TR("Delete document")
		)
		!= TKMessageBox::Yes) return ;


	if (!location.remove (error)) error.DISPLAY() ;

	/* Reload the entire list. Do this whether or not there was an	*/
	/* error; this ensures that the displayed list is correct.	*/
	reloadServer ((KBServerItem *)item->parent()) ;
}

/*  KBFileList								*/
/*  serverChange: Notification of server changes			*/
/*  location	: const KBLocation & : Location changed			*/
/*  (returns)	: void		     :					*/

void	KBFileList::serverChanged
	(	const KBLocation &location
	)
{
	KBServerItem *item = (KBServerItem *)firstChild() ;

	/* Scan looking for the corresponding entry. When found, update	*/
	/* the name (may or may not have changed), and drop all the	*/
	/* children documents (the server may now point somewhere else)	*/
	while (item != 0)
	{
		if (location.server() == item->text(0))
		{
			item->setText (0, location.name()) ;
			reloadServer (item) ;
			return	;
		}

		item	= (KBServerItem *)item->nextSibling() ;
	}

	/* Should only get here for new servers ....			*/
	KBServerInfo *svInfo = m_dbInfo->findServer (location.name()) ;
	if ((svInfo != 0) && !svInfo->disabled())
	{
		item = new KBServerItem (this, lSort, svInfo->serverName()) ;
		item->setPixmap (0, getSmallIcon("database")) ;
	}
}

/*  KBFileList								*/
/*  objChange	: Notification of object changes			*/
/*  location	: const KBLocation & : Location changed			*/
/*  (returns)	: void		     :					*/

void	KBFileList::objChange
	(	const KBLocation &location
	)
{
	KBServerItem *item = (KBServerItem *)firstChild() ;

	while (item != 0)
	{
		if (location.server() == item->text(0))
		{
			reloadServer (item) ;
			break	;
		}

		item	= (KBServerItem *)item->nextSibling() ;
	}
}

#if	! __KB_RUNTIME

/*  KBFileList								*/
/*  saveObjToFile							*/
/*		: Save object to a file					*/
/*  location	: KBLocation &	  : Document location			*/
/*  name	: const QString & : Target file name			*/
/*  actRC	: ActRC		  : Continutation code			*/
/*  (returns)	: ActRC		  : Continutation code			*/

KBMessageBoxYNAC::ActRC
	KBFileList::saveObjToFile
	(	const KBLocation	&location,
		const QString		&name,
		KBMessageBoxYNAC::ActRC	actRC
	)
{
	DPRINTF
	((	"KBFileList::saveObjToFile: [%s]->[%s]\n",
		(cchar *)location.name(),
		(cchar *)name
	))	;

	/* This is the underlying save-object-to-file method, saving	*/
	/* the object identifed by "location" to the file "name".	*/
	QByteArray data	;
	KBError	   error;
	if (!location.contents (data, error))
	{	error.DISPLAY() ;
		return	KBMessageBoxYNAC::ActCancel ;
	}


	/* Check what to do if the file already exists. This depends	*/
	/* on the action return code passed in.				*/
	if (QFileInfo(name).exists())
		switch (actRC)
		{
			case KBMessageBoxYNAC::ActYes	:
			case KBMessageBoxYNAC::ActNo	:
				/* Already proceeding to the next	*/
				/* object, so repeat the options.	*/
				actRC = KBMessageBoxYNAC::query
				   	(	0,
						QString (TR("%1 already exists: overwrite?")).arg(name),
						TR("Save to file ....")
					)	;
				if (actRC == KBMessageBoxYNAC::ActCancel) return actRC ;
				if (actRC == KBMessageBoxYNAC::ActNo    ) return actRC ;
				break	;

			case KBMessageBoxYNAC::ActSingle:
				/* This is a single file case, so its a	*/
				/* simple yes/no decision.		*/
				if (TKMessageBox::questionYesNo
				   	(	0,
						QString (TR("%1 already exists: overwrite?")).arg(name),
						TR("Save to file ....")
					)
					!= TKMessageBox::Yes)
					return	KBMessageBoxYNAC::ActCancel ;
				break	;

			default	:
				/* Presumably allowing overwrite of all	*/
				/* objects, so just proceed.		*/
				break	;
		}

	KBFile	file	(name) ;
	if (!file.open (IO_WriteOnly|IO_Truncate))
	{
		file.lastError().DISPLAY() ;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	file.writeBlock (data)	;
	return	actRC	;
}

/*  KBFileList								*/
/*  saveObjToWeb: Save object for web access				*/
/*  location	: KBLocation &	  : Form location			*/
/*  dir		: const QString & : Target directory			*/
/*  actRC	: ActRC		  : Continutation code			*/
/*  (returns)	: ActRC		  : Continutation code			*/

KBMessageBoxYNAC::ActRC
	KBFileList::saveObjToWeb
	(	const KBLocation	&location,
		const QString		&dir,
		KBMessageBoxYNAC::ActRC	actRC
	)
{
	DPRINTF
	((	"KBFileList::saveObjToWeb: [%s]->[%s]\n",
		(cchar *)location.name(),
		(cchar *)dir
	))	;

	/* This is the underlying save-object-to-web method, saving	*/
	/* the object identifed by "location" into the directory "dir".	*/
	/* The base method defaults to saving the object document as-is	*/
	return	saveObjToFile (location, dir + "/" + location.filename(), actRC) ;
}

/*  KBFileList								*/
/*  saveObjToFile							*/
/*		: Save object to a file					*/
/*  location	: KBLocation &	: Object location			*/
/*  (returns)	: bool		: Success					*/

bool	KBFileList::saveObjToFile
	(	const KBLocation	&location
	)
{
	if (!canOperate (location, " save"))
		return	false	;

	KBFileDialog	fDlg
			(	".",
				"*.*|All file types",
				qApp->activeWindow(),
				"saveobject",
				true
			)	;

	fDlg.setSelection	(location.filename())	  ;
	fDlg.setOperationMode	(KBFileDialog::Saving )	  ;
	fDlg.setMode		(KBFileDialog::AnyFile)	  ;
	fDlg.setCaption		(TR("Save to file ....")) ;

	if (!fDlg.exec ())
		return	false	;

	return	saveObjToFile
		(	location,
			fDlg.selectedFile (),
			KBMessageBoxYNAC::ActSingle
		)
		!= KBMessageBoxYNAC::ActCancel ;
}

/*  KBFileList								*/
/*  saveObjToWeb: Save current object for web access			*/
/*  location	: KBLocation &	: Object location			*/
/*  (returns)	: bool		: Success				*/

bool	KBFileList::saveObjToWeb
	(	const KBLocation	&location
	)
{
	if (webDir.isEmpty())
		webDir	= location.dbInfo()->findServer(location.server())->webDirectory() ;

	QString	dir	= KBFileDialog::getExistingDirectory
			  (	webDir,
				TR("Save to web in .....")
			  )	;

	if (dir.isEmpty())
		return	false	;

	if (saveObjToWeb
		(	location,
			dir,
			KBMessageBoxYNAC::ActSingle
		)
		== KBMessageBoxYNAC::ActCancel) return false ;

	webDir	= dir 	;
	return	true	;
}
#endif

/*  KBFileList								*/
/*  saveObjToFile							*/
/*		: Save current object to a file				*/
/*  (returns)	: void		:					*/

void	KBFileList::saveObjToFile ()
{
#if	! __KB_RUNTIME
	/* This is the slot invoked when the user asks to save the	*/
	/* object to a file, in which case we pop up a file dialog to	*/
	/* determine the file into which to save the object.		*/
	KBLocation location ;

	if (!itemToLocation (m_curItem, location))
		return	;

	saveObjToFile (location) ;
#endif
}

/*  KBFileList								*/
/*  saveObjToWeb: Save current object for web access			*/
/*  (returns)	: void		:					*/

void	KBFileList::saveObjToWeb ()
{
#if	! __KB_RUNTIME
	/* This is the slot invoked when the user asks to save the	*/
	/* object for the web, in which case we pop up a file dialog to	*/
	/* determine the directory into which the object is saved.	*/
	KBLocation location	;

	if (!itemToLocation (m_curItem, location))
		return	;

	saveObjToWeb (location)	;
#endif
}

/*  KBFileList								*/
/*  saveAllToFile							*/
/*		: Save selected objects to file				*/
/*  (returns)	: void		:					*/

void	KBFileList::saveAllToFile ()
{
#if	! __KB_RUNTIME
	/* If there is no child then the branch has not been opened	*/
	/* so do it now. The check that follows is then for the sake of	*/
	/* paranoia.							*/
	if (m_curItem->firstChild() == 0)
		reloadServer () ;

	if (m_curItem->firstChild() == 0)
		return ;

	QString		docLocn = m_curItem->text(0) ;
	QListViewItem   *item   = m_curItem->firstChild() ;
	KBSvrChooserDlg chooser (m_dbInfo, docLocn, true, false) ;

	while (item != 0)
	{
		if (((KBListItem *)item)->type() == KBListItem::Object)
			chooser.addEntry (item->text(0)) ;

		item = item->nextSibling() ;
	}

	if (!chooser.exec ()) return ;

	QString	dir	= KBFileDialog::getExistingDirectory
			  (	QString::null,
				TR("Save to file in .....")
			  )	  ;
	if (dir.isNull()) return  ;

	QString			target	;
	QStringList		forms	;
	bool			over	;
	KBMessageBoxYNAC::ActRC	actRC	= KBMessageBoxYNAC::ActYes ;

	chooser.getInfo (target, forms, over) ;

	for (uint idx = 0 ; idx < forms.count() ; idx += 1)
	{
		QString		name	= forms[idx] ;
		KBLocation	srce	(m_dbInfo, m_tabType, docLocn, name, getDocExtension()) ;

		actRC	= saveObjToFile (srce, dir + "/" + srce.filename(), actRC) ;
		if (actRC == KBMessageBoxYNAC::ActCancel) return ;
	}
#endif
}
/*  KBFileList								*/
/*  saveAllToWeb: Save selected objects for web access			*/
/*  (returns)	: void		:					*/

void	KBFileList::saveAllToWeb ()
{
#if	! __KB_RUNTIME
	/* If there is no child then the branch has not been opened	*/
	/* so do it now. The check that follows is then for the sake of	*/
	/* paranoia.							*/
	if (m_curItem->firstChild() == 0)
		reloadServer () ;

	if (m_curItem->firstChild() == 0)
		return ;

	QString		docLocn = m_curItem->text(0) ;
	QListViewItem   *item   = m_curItem->firstChild() ;
	KBSvrChooserDlg chooser (m_dbInfo, docLocn, true, false) ;

	while (item != 0)
	{
		if (((KBListItem *)item)->type() == KBListItem::Object)
			chooser.addEntry (item->text(0)) ;

		item = item->nextSibling() ;
	}

	if (!chooser.exec ()) return ;

	if (webDir.isEmpty())
		webDir	= m_dbInfo->findServer(docLocn)->webDirectory() ;

	QString	dir	= KBFileDialog::getExistingDirectory
			  (	webDir,
				TR("Save to web in .....")
			  )	  ;
	if (dir.isNull()) return  ;

	QString			target	;
	QStringList		forms	;
	bool			over	;
	KBMessageBoxYNAC::ActRC	actRC	= KBMessageBoxYNAC::ActYes ;

	chooser.getInfo (target, forms, over) ;

	for (uint idx = 0 ; idx < forms.count() ; idx += 1)
	{
		QString		name	= forms[idx] ;
		KBLocation	srce	(m_dbInfo, m_tabType, docLocn, name, getDocExtension()) ;

		actRC	= saveObjToWeb (srce, dir, actRC) ;
		if (actRC == KBMessageBoxYNAC::ActCancel) return ;
	}

	webDir	= dir	;
#endif
}



/*  KBFileList								*/
/*  copyToServer: Copy objects to specified server			*/
/*  (returns)	: void		:					*/

void	KBFileList::copyToServer ()
{
	/* If there is no child then the branch has not been opened	*/
	/* so do it now. The check that follows is then for the sake of	*/
	/* paranoia.							*/
	if (m_curItem->firstChild() == 0)
		reloadServer () ;

	if (m_curItem->firstChild() == 0)
		return ;

	QString		docLocn = m_curItem->text(0) ;
	QListViewItem   *item   = m_curItem->firstChild() ; //->nextSibling() ;
	KBSvrChooserDlg chooser (m_dbInfo, docLocn, true) ;

	while (item != 0)
	{
		if (((KBListItem *)item)->type() == KBListItem::Object)
			chooser.addEntry (item->text(0)) ;

		item = item->nextSibling() ;
	}

	if (!chooser.exec ()) return ;

	QString		target	;
	QStringList	forms	;
	bool		over	;

	chooser.getInfo (target, forms, over) ;

	for (uint idx = 0 ; idx < forms.count() ; idx += 1)
	{
		QString		name	= forms[idx] ;
		KBLocation	srce	(m_dbInfo, m_tabType, docLocn, name, getDocExtension()) ;
		KBLocation	dest	(m_dbInfo, m_tabType, target,  name, getDocExtension()) ;

		if (!over && dest.exists())
			if (TKMessageBox::questionYesNo
			   	(	0,
					QString (TR("%1 exists on server %2: overwrite?"))
						.arg(name  )
						.arg(target),
					TR("Overwrite document ...")
				)
				!= TKMessageBox::Yes) continue ;

		KBError		error	;
		QString		text	= srce.contents (error) ;

		if (text.isNull())
		{
			error.DISPLAY() ;
			continue	;
		}

		if (!dest.save (QString::null, QString::null, text, error))
		{
			error.DISPLAY() ;
			continue	;
		}
	}
}

/*  KBFileList								*/
/*  getDocExtension							*/
/*		: Get document name extension				*/
/*  (returns)	: QString	: Extension				*/

QString	KBFileList::getDocExtension ()
{
	/* This method is overridden for object types where there is	*/
	/* an extension which is independant of Rekall itself, eg., for	*/
	/* scripts where the entension is the script file extension	*/
	/* (such as "py" for python). In all other cases it is empty.	*/
	return	""	;
}

/*  KBFileList								*/
/*  getTabType	: Return the type string as used in the objects table	*/
/*  (returns)	: QString	: Type					*/

QString	KBFileList::getTabType ()
{
	return	m_tabType	;
}

/*  KBFileList								*/
/*  getObjectNames							*/
/*		: Get list of all objects on specified server		*/
/*  server	: KBServerInfo * : Server				*/
/*  (returns)	: QStringList	 : List of names			*/

QStringList
	KBFileList::getObjectNames
	(	KBServerInfo	*server
	)
{
	KBError		error   ;
	KBDBDocIter	docIter ;

	if (!docIter.init
		(	m_dbInfo,
			server->serverName(),
			m_tabType,
			KBLocation::extnForType (m_dbInfo, m_tabType, getDocExtension()),
			error,
			false
		))
	{
		error.DISPLAY() ;
		return	QStringList () ;
	}


	QString		name	;
	QString		stamp	;
	QStringList	nameList;

	while (docIter.getNextDoc (name, stamp))
	{
		nameList.append (name) ;
	}

	return	nameList ;
}


/*  showAsCode	: Convert show-as value from strings to code		*/
/*  text	: const QString & : String value			*/
/*  deflt	: KB::ShowAs	  : Default value			*/
/*  (returns)	: KB::ShowAs	  : Valuu as code			*/

LIBAPP_API
	KB::ShowAs showAsCode
	(	const QString	&text,
		KB::ShowAs	deflt
	)
{
	if (text == "ShowAsData"   ) return KB::ShowAsData	;
	if (text == "ShowAsPrint"  ) return KB::ShowAsPrint	;
	if (text == "ShowAsPreview") return KB::ShowAsPreview	;
	if (text == "ShowAsReport" ) return KB::ShowAsReport	;
	if (text == "ShowAsDesign" ) return KB::ShowAsDesign	;

	TKMessageBox::sorry
	(	0,
		QString(TR("Unknown show-as code: %1")).arg(text),
		TR("ShowAs error")
	)	;

	return	deflt	;
}

void	KBFileList::showServerMenu (int)
{
}

void	KBFileList::showCreateMenu (int)
{
}

void	KBFileList::showObjectMenu (int)
{
}

void	KBFileList::showMenu
	(	QListViewItem	*item,
		const QPoint	&,
		int
	)
{
	if ((m_curItem = (KBListItem *)item) == 0)
		return	;

	switch (m_curItem->type())
	{
		case KBListItem::Server	:
			showServerMenu	(m_curState) ;
			return	;

		case KBListItem::Create	:
			showCreateMenu	(m_curState) ;
			return	;

		case KBListItem::Object	:
			showObjectMenu	(m_curState) ;
			return	;

		default	:
			break	;
	}
}

QString	KBFileList::getWebDir
	(	const KBLocation	&location
	)
{
	if (webDir.isEmpty())
		return	location.dbInfo()->findServer(location.server())->webDirectory() ;

	return	webDir	;
}


void	KBFileList::contentsMousePressEvent
	(	QMouseEvent	*m
	)
{
	m_curState = m->state() ;
	QListView::contentsMousePressEvent (m) ;
}
