/***************************************************************************
    file	         : kb_filelist.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_FILELIST_H
#define	_KB_FILELIST_H

#include	<qlistview.h>


#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_listitem.h"
#include	"kb_serverinfo.h"
#include	"kb_prompt.h"

#include	"libapp_exports.h"

class	KBFileList	;
class	KBFileListIface	;



/*  KBFileList								*/
/*  ----------								*/
/*  Widget used to list a subset of the files in the database		*/

class	LIBAPP_API KBFileList : public QListView
{
	Q_OBJECT

	KBFileListIface	*m_dcopIface	;

protected :

	QWidget		*m_parent	;  /* Notional parent		*/
	KBDBInfo	*m_dbInfo	;  /* Database information	*/
	cchar		*m_newtext	;  /* Text for create new item	*/
	cchar		*m_icon		;  /* Icon for contents		*/
	QString		m_tabType	;  /* Type being listed		*/

	bool		m_wizard	;  /* Wizard creation support	*/
	bool		m_web		;  /* Can export for web	*/

	KBListItem	*m_curItem 	;  /* Currently selected item	*/
	int		m_curState	;  /* Current key state		*/

	bool		canOperate   	(const KBLocation    &, cchar *) ;
	bool		canOperate   	(KBListItem    *, cchar *) ;

	void		rename		(KBListItem    *) ;
	void		delobj		(KBListItem    *) ;

	bool		itemToLocation	(KBListItem    *, KBLocation &) ;

	virtual	void	reloadServer	(KBServerItem  *) ;
	virtual	void	showObjectAs	(KBListItem    *, KB::ShowAs  ) ;
	virtual	void	createByWizard	(KBListItem    *) ;

	virtual	void	showServerMenu	(int)	;
	virtual	void	showCreateMenu	(int)	;
	virtual	void	showObjectMenu	(int)	;

	virtual	bool	dcopOpenObject	(const QString &, const QString &, int) ;
	virtual	bool	dcopOpenTextObj	(const QString &, const QString &, int) ;
	virtual	QStringList
			dcopListObjects	(const QString &) ;

	virtual	void	contentsMousePressEvent	(QMouseEvent *) ;

protected slots :

	virtual	void	showAsData	()	;
	virtual	void	showAsDesign	()	;
	virtual	void	showDefault	(QListViewItem *) ;
	virtual	void	showMenu	(QListViewItem *, const QPoint &, int) ;
	virtual	void	copyToServer	()	;
	virtual	void	reloadServer 	()	;

	virtual	void	saveObjToFile	()	;
	virtual void	saveObjToWeb	()	;
	virtual void	saveAllToFile	()	;
	virtual void	saveAllToWeb	()	;

	void		serverChanged	(const KBLocation &) ;

public	 slots :

	void		objChange	(const KBLocation &) ;

public	:

	KBFileList  (QWidget *, KBDBInfo *, cchar *,
					    cchar *,
					    cchar *,
					    cchar *,
					    cchar * = 0, cchar * = 0) ;
       ~KBFileList  () ;

	virtual	QStringList	getObjectNames	(KBServerInfo *) ;
	virtual	QString		getDocExtension	()	;
	virtual	QString		getTabType	()	;

	inline	void		enableWizard	() { m_wizard	= true	; }
	inline	void		enableWeb	() { m_web	= true	; }
	inline	bool		hasWizard	() { return m_wizard	; }
	inline	bool		hasWeb		() { return m_web	; }

	bool			saveObjToWeb	(const KBLocation &) ;
	bool			saveObjToFile	(const KBLocation &) ;
	static	QString		getWebDir	(const KBLocation &) ;

#if	! __KB_RUNTIME
	virtual KBMessageBoxYNAC::ActRC
				saveObjToWeb	(const KBLocation &, const QString &, KBMessageBoxYNAC::ActRC = KBMessageBoxYNAC::ActYes) ;
	virtual	KBMessageBoxYNAC::ActRC
				saveObjToFile	(const KBLocation &, const QString &, KBMessageBoxYNAC::ActRC = KBMessageBoxYNAC::ActYes) ;
#endif

	friend	class	KBServerItem	;
	friend	class	KBObjectItem	;
	friend	class	KBFileListIface	;
}	;

extern	LIBAPP_API	KB::ShowAs	showAsCode (const QString &, KB::ShowAs) ;

#endif	// _KB_FILELIST_H
