/***************************************************************************
    file	         : kb_svrchooser.cpp
    copyright            : (C) 1999,2000,2001,2002,2003,2004 by Mike Richardson
			   (C) 2001,2002,2003,2004 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdlib.h>
#include	<qlabel.h>
#include	<qcheckbox.h>
#include	<qlabel.h>

#if		__KB_AUTOMOC
#include	"kb_svrchooser.moc"
#else
#include	"kb_svrchooser.h"
#endif

#include	"kb_serverinfo.h"
#include	"kb_sidepanel.h"


#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_combobox.h"
#include	"rk_listbox.h"
#include	"rk_pushbutton.h"


/*  KBSvrChooserDlg							*/
/*  KBSvrChooserDlg							*/
/*		: Simple server selection dialog			*/
/*  dbInfo	: KBDBInfo *	  : Database information		*/
/*  myserver	: const QString & : My server name			*/
/*  withFiles	: bool		  : Include "!Files"			*/
/*  withServer	: bool		  : Show server selector		*/
/*  (returns)	: KBSvrChooserDlg :					*/

KBSvrChooserDlg::KBSvrChooserDlg
	(	KBDBInfo	*dbInfo,
		const QString	&myserver,
		bool		withFiles,
		bool		withServer
	)
	:
	KBDialog
	(	withServer ?
			TR("Select server and objects") :
			TR("Select objects"),
		true
	)
{
	RKVBox	*layMain	= new RKVBox	   (this)	;
	layMain->setTracking ()	;

	RKHBox	*laySelect	= new RKHBox	   (layMain  )	;
	RKHBox	*layButt	= new RKHBox	   (layMain  )	;

	new KBSidePanel (laySelect, caption()) ;

	m_lbOut			= new RKListBox	   (laySelect)	;
	RKVBox	*layAddRem	= new RKVBox	   (laySelect)	;
	m_lbIn			= new RKListBox	   (laySelect)	;

	m_bAdd			= new RKPushButton (TR("Add >>"),	layAddRem) ;
	m_bAddAll		= new RKPushButton (TR("Add All >>"),	layAddRem) ;
	m_bRemove		= new RKPushButton (TR("<< Remove"),	layAddRem) ;
	layAddRem->addFiller () ;

	m_lbIn ->setSelectionMode (QListBox::Extended) ;
	m_lbOut->setSelectionMode (QListBox::Extended) ;

	if (withServer)
	{
		new QLabel	  (TR("Copy to server: "), layButt) ;
		m_cbServer	= new RKComboBox  (layButt) ;
	}

	else	m_cbServer	= 0 ;

	m_cOver			= new QCheckBox   (TR("Overwrite"), layButt) ;
	layButt  ->addFiller () ;

	m_bOK			= new RKPushButton(layButt, "ok") ;
	m_bCancel		= new RKPushButton(layButt, "cancel") ;

	connect	 (m_bAdd,    SIGNAL(clicked()), SLOT(clickAdd   ())) ;
	connect	 (m_bAddAll, SIGNAL(clicked()), SLOT(clickAddAll())) ;
	connect	 (m_bRemove, SIGNAL(clicked()), SLOT(clickRemove())) ;

	connect	 (m_lbOut,   SIGNAL(doubleClicked(QListBoxItem *)), this, SLOT(clickAdd   ())) ;
	connect  (m_lbIn,    SIGNAL(doubleClicked(QListBoxItem *)), this, SLOT(clickRemove())) ;

	connect	 (m_lbIn,    SIGNAL(highlighted(int)), SLOT( inHighlight())) ;
	connect	 (m_lbOut,   SIGNAL(highlighted(int)), SLOT(outHighlight())) ;

	m_lbOut	 ->setMinimumWidth (150) ;
	m_lbIn	 ->setMinimumWidth (150) ;

	m_bAdd	 ->setEnabled  (false) ;
	m_bAddAll->setEnabled  (false) ;
	m_bRemove->setEnabled  (false) ;

	QPtrListIterator<KBServerInfo> *iter = dbInfo->getServerIter () ;
	KBServerInfo *server ;

	if (withServer)
	{
		if (withFiles && (myserver != KBLocation::m_pFile))
			m_cbServer->insertItem (KBLocation::m_pFile) ;

		while ((server = iter->current()) != 0)
		{
			(*iter) += 1 ;

			QString	svName	= server->serverName() ;
			if (svName == myserver) continue ;

			KBDBLink dbLink ;
			if (!dbLink.connect (dbInfo, svName, true) )
			{
				if (!dbLink.disabled()) dbLink.lastError().DISPLAY() ;
				continue ;
			}

			m_cbServer->insertItem (svName) ;
		}
	}

	delete	iter	;
}

/*  KBSvrChooserDlg							*/
/*  addEntry	: Add an entry to the out list				*/
/*  entry	: const QString & : Entry				*/
/*  entry	: const QString & : Possible extension			*/
/*  (returns)	: void		  :					*/

void	KBSvrChooserDlg::addEntry
	(	const QString	&entry,
		const QString	&extn
	)
{
	m_lbOut	 ->insertItem (entry) ;
	m_bAddAll->setEnabled (true ) ;

	m_extnMap.insert(entry, extn) ;
}

/*  KBSvrChooserDlg							*/
/*  selectEntry	: Select entry into the in list				*/
/*  entru	: const QString & : Entry				*/
/*  (returns)	: void		  :					*/

void	KBSvrChooserDlg::selectEntry
	(	const QString	&entry
	)
{
	int item = -1 ;

	for (uint idx = 0 ; idx < m_lbOut->count() ; idx += 1)
		if (m_lbOut->text(idx) == entry)
		{	item	= idx	;
			break	;
		}

	if (item >= 0)
	{	m_lbIn ->insertItem (m_lbOut->text(item)) ;
		m_lbOut->removeItem (item) ;
	}

	m_bAdd   ->setEnabled (m_lbOut->currentItem() >= 0) ;
	m_bAddAll->setEnabled (m_lbOut->count      () >  0) ;
}

/*  KBSvrChooserDlg							*/
/*  clickAdd	: User clicks add button				*/
/*  (returns)	: void		:					*/

void	KBSvrChooserDlg::clickAdd ()
{
	for (int idx = 0 ; idx < (int)m_lbOut->count() ; idx += 1)
		if (m_lbOut->isSelected(idx))
			m_lbIn ->insertItem (m_lbOut->text(idx)) ;

	for (int idx = (int)m_lbOut->count() - 1 ; idx >= 0 ; idx -= 1)
		if (m_lbOut->isSelected(idx))
			m_lbOut->removeItem (idx) ;


	m_bAdd   ->setEnabled (m_lbOut->currentItem() >= 0) ;
	m_bAddAll->setEnabled (m_lbOut->count      () >  0) ;
}

/*  KBSvrChooserDlg							*/
/*  clickAddAll	: User clicks add all button				*/
/*  (returns)	: void		:					*/

void	KBSvrChooserDlg::clickAddAll ()
{
	while (m_lbOut->count() > 0)
	{
		m_lbIn ->insertItem (m_lbOut->text(0)) ;
		m_lbOut->removeItem (0) ;
	}

	m_bAdd	 ->setEnabled (false) ;
	m_bAddAll->setEnabled (false) ;
}

/*  KBSvrChooserDlg							*/
/*  clickRemove	: User clicks remove button				*/
/*  (returns)	: void		:					*/

void	KBSvrChooserDlg::clickRemove ()
{
	for (int idx = 0 ; idx < (int)m_lbIn->count() ; idx += 1)
		if (m_lbIn->isSelected(idx))
			m_lbOut ->insertItem (m_lbIn->text(idx)) ;

	for (int idx = (int)m_lbIn->count() - 1 ; idx >= 0 ; idx -= 1)
		if (m_lbIn->isSelected(idx))
			m_lbIn->removeItem (idx) ;


	m_bRemove->setEnabled (m_lbIn->currentItem() >= 0) ;
	m_bAddAll->setEnabled (true) ;
}

/*  KBSvrChooserDlg							*/
/*  inHighlight	: User clicks on item in "in" list			*/
/*  (returns)	: void		:					*/

void	KBSvrChooserDlg::inHighlight ()
{
	m_bRemove->setEnabled (true) ;
}

/*  KBSvrChooserDlg							*/
/*  inHighlight	: User clicks on item in "out" list			*/
/*  (returns)	: void		:					*/

void	KBSvrChooserDlg::outHighlight ()
{
	m_bAdd->setEnabled (true) ;
}

/*  KBSvrChooserDlg							*/
/*  getInfo	: Get results						*/
/*  server	: QString &	: Selected server			*/
/*  entries	: QStringList &	: List of entries from "in" list	*/
/*  over	: bool &	: Overwrite option checked		*/
/*  (returns)	: void		:					*/

void	KBSvrChooserDlg::getInfo
	(	QString		&server,
		QStringList	&entries,
		bool		&over
	)
{
	server	= m_cbServer == 0 ? QString::null : m_cbServer->currentText() ;
	over	= m_cOver->isChecked  () ;

	for (uint idx = 0 ; idx < m_lbIn->count() ; idx += 1)
		entries.append (m_lbIn->text (idx)) ;
}

QString	KBSvrChooserDlg::extensionFor
	(	const QString	&entry
	)
{
	return	m_extnMap[entry] ;
}
