/***************************************************************************
    file	         : kb_base64.cpp
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/***************************************************************************
    The Base64 code is taken from LibTomCrypt, see
        http://libtomcrypt.org/index.html
 ***************************************************************************/



#include 	"kb_databuffer.h"



static const char *base64Codes = 
	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/" ;

static const unsigned char base64Map[256] =
{
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255,  62, 255, 255, 255,  63,
	 52,  53,  54,  55,  56,  57,  58,  59,  60,  61, 255, 255,
	255, 254, 255, 255, 255,   0,   1,   2,   3,   4,   5,   6,
	  7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
	 19,  20,  21,  22,  23,  24,  25, 255, 255, 255, 255, 255,
	255,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
	 37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
	 49,  50,  51, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
	255, 255, 255, 255
}	;

bool	LIBCOMMON_API	kbB64Needed
	(	const unsigned char	*data,
		unsigned int		dlen
	)
{
	while (dlen > 0)
	{	if ((*data < 0x20) || (*data >= 0x7f))
			return	true ;
		dlen	-= 1 ;
		data	+= 1 ;
	}

	return	false	;
}


LIBCOMMON_API
	void	kbB64Encode
	(	const unsigned char	*data,
		unsigned int		dlen,
		KBDataBuffer		&dest
	)
{
	unsigned int	i	;
	unsigned int	leven	= 3 * (dlen / 3) ;

	for (i = 0; i < leven; i += 3)
	{
		dest.append (base64Codes[data[0] >> 2]) ;
		dest.append (base64Codes[((data[0] & 0x3) << 4) + (data[1] >> 4)]) ;
		dest.append (base64Codes[((data[1] & 0xf) << 2) + (data[2] >> 6)]) ;
		dest.append (base64Codes[data[2] & 0x3f]) ;
		data += 3;
	}

	if (i < dlen)
	{
		unsigned int	a = data[0] ;
		unsigned int	b = (i + 1 < dlen) ? data[1] : 0 ;
		unsigned int	c = 0 ;

		dest.append (base64Codes[a >> 2]) ;
		dest.append (base64Codes[((a & 3) << 4) + (b >> 4)]) ;
		dest.append ((i + 1 < dlen) ? base64Codes[((b & 0xf) << 2) + (c >> 6)] : '=') ;
		dest.append ('=') ;
	}
}

LIBCOMMON_API
	void	kbB64Decode
	(	const unsigned char	*data, 
		unsigned int		dlen,
		KBDataBuffer		&dest
	)
{
	unsigned long t, x, y	;
	int 	      g = 3	;

	for (x = y = t = 0; x < dlen; x += 1, data += 1)
	{
		unsigned char c = base64Map[*data] ;

		if (c == 255)
			continue ;

		if (c == 254)
		{
			c  = 0 ;
			g -= 1 ;
		}

		t = (t<<6) | c ;

		if ((y += 1) == 4)
		{
			dest.append ((unsigned char)((t>>16)&255)) ;
			if (g > 1)
				dest.append ((unsigned char)((t>>8)&255)) ;
			if (g > 2)
				dest.append ((unsigned char)(t&255)) ;

			y = 0 ;
			t = 0 ;
		}
	}
}

void	kbB64Decode
	(	const QString		&data,
		KBDataBuffer		&dest
	)
{
	kbB64Decode ((const unsigned char *)data.latin1(), data.length(), dest) ;
}

