/***************************************************************************
    file	         : kb_desktop.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qdir.h>
#include	<qfile.h>
#include	<qfileinfo.h>
#include	<qtextstream.h>
#include	<qstringlist.h>

#include	"kb_classes.h"
#include	"kb_desktop.h"


#define		__DEBUG_DESKTOP	(0)
#if		__DEBUG_DESKTOP
#include	"kb_dprintf.h"
#endif


/*  KBDesktop								*/
/*  KBDesktop	: Rekall-specific .desktop file handler			*/
/*  file	: QFile &	: Source file				*/
/*  (returns)	: KBDesktop	:					*/

KBDesktop::KBDesktop
	(	QFile		&file
	)
	:
	m_path	(QFileInfo(file).fileName())
{
	init	(file)	;
}

/*  KBDesktop								*/
/*  KBDesktop	: Rekall-specific .desktop file handler			*/
/*  file	: const QString & : Source file name			*/
/*  (returns)	: KBDesktop	  :					*/

KBDesktop::KBDesktop
	(	const QString	&path
	)
	:
	m_path	(path)
{
	QFile	file(path) ;
	if (file.open(IO_ReadOnly)) init (file) ;
}

/*  KBDesktop								*/
/*  init	: Initialise						*/
/*  file	: QFile &	: Source file				*/
/*  (returns)	: void		:					*/

void	KBDesktop::init
	(	QFile		&file
	)
{
	QTextStream stream (&file) ;
	QString	    text   ;

	while (!(text = stream.readLine()).isNull())
	{
		QStringList bits = QStringList::split('=', text) ;
		if (bits.count() == 2)
			insert (bits[0], bits[1]) ;
	}
}

/*  ------------------------------------------------------------------	*/

/*  KBDesktop								*/
/*  scan	: Scan for, and decode, desktop files			*/
/*  path	: const QString &    : Directory path			*/
/*  prefix	: const QString &    : Desktop file prefix		*/
/*  desktops	: QDict<KBDesktop> & : Result dictionary		*/
/*  (returns)	: void		     :					*/

void	KBDesktop::scan
	(	const QString		&path,
		const QString		&prefix,
		QPtrList<KBDesktop>	&desktops
	)
{
	QDir	dir	;

	dir.setPath		(path)	;
	dir.setFilter		(QDir::Files) ;
	dir.setNameFilter	(QString("%1*.desktop").arg(prefix)) ;

	DPRINTF
	((	"KBDesktop::scan: scanning [%s]->[%s/%s]\n",
		(cchar *)path,
		(cchar *)dir.absPath   (),
		(cchar *)dir.nameFilter()
	))	;

	const QFileInfoList *list = dir.entryInfoList() ;
	if (list == 0) return ;

	QFileInfoListIterator iter (*list) ;
	QFileInfo *fi ;

	while ((fi = iter.current()) != 0)
	{
		DPRINTF
		((	"KBDesktop::scan: [%s]\n",
			(cchar *)fi->fileName()
		))	;

		iter += 1 ;

		QFile	file(fi->absFilePath()) ;
		if (!file.open(IO_ReadOnly)) continue ;

		KBDesktop   *dtop = new KBDesktop (file) ;

//		QTextStream stream (&file) ;
//		QString	    text   ;
//
//		while (!(text = stream.readLine()).isNull())
//		{
//			QStringList bits = QStringList::split('=', text) ;
//			if (bits.count() == 2)
//				dtop->insert (bits[0], bits[1]) ;
//		}

		desktops.append (dtop) ;
	}
}


/*  KBDesktop								*/
/*  print	: Print desktop definition for debugging		*/
/*  (returns)	: void		:					*/

void	KBDesktop::print ()
{
	QDictIterator<QString>	iter	(m_entries) ;
	QString			*value	;

	fprintf
	(	stderr,
		"%s\n",
		(cchar *)m_path
	)	;

	while ((value = iter.current()) != 0)
	{
		fprintf	(stderr, "   %s=%s\n", (cchar *)iter.currentKey(), (cchar *)*value) ;
		iter += 1 ;
	}
}
