/***************************************************************************
    file	         : kb_dom.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	"kb_classes.h"
#include	"kb_dom.h"
#include	"kb_file.h"
#include	"kb_locator.h"

#include	<qobject.h>

#define		__DEBUG_DOM	(0)
#if		__DEBUG_DOM
#include	"kb_dprintf.h"
#endif

/*  KBDomDocument							*/
/*  KBDomDocument: Constructor for document				*/
/*  (returns)	 : KBDomDocument	:				*/

KBDomDocument::KBDomDocument ()
	:
	QDomDocument ()
{
}

/*  KBDomDocument							*/
/*  KBDomDocument: Constructor for document				*/
/*  rootName	 : const QString & : Root element tag			*/
/*  (returns)	 : KBDomDocument   :					*/

KBDomDocument::KBDomDocument
	(	const QString	&rootName
	)
	:
	QDomDocument ()
{
	/* This version of the constructor creates the named root	*/
	/* element, and attaches the XML version and encoding.		*/
	QDomElement root = createElement (rootName) ;
	appendChild (root) ;

	createProcessingInstruction
	(	"xml",
		"version=\"1.0\" encoding=\"UTF-8\""
	)	;
}

/*  KBDomDocument							*/
/*  loadFile	: Load XML from file into document			*/
/*  name	: const QString & : File name				*/
/*  dir		: const char *	  : Data directory			*/
/*  location	: const char *	  : Optional location			*/
/*  (returns)	: bool		  : Success				*/

bool	KBDomDocument::loadFile
	(	const QString	&name,
		const char	*dir,
		const char	*location
	)
{
        QString path    = name	;

	if (dir != 0)
		if (location == 0)
			path	= locateFile("appdata", QString("%1/%2").arg(dir).arg(name)) ;
		else	path	= QString("%1/%2/%2").arg(location).arg(dir).arg(name) ;

	DPRINTF
	((	"KBDomDocument::loadFile: [%s][%s][%s]->[%s]\n",
		name.latin1(),
		dir,
		location,
		path.latin1()
	))	;

	KBFile	file	(path)	;
        if (!file.open (IO_ReadOnly))
	{
		m_error	= file.lastError () ;
		return	false	;
	}

        if (!setContent (&file))
        {
		m_error	= KBError
			  (	KBError::Error,
				QString(TR("Cannot parse file '%1' as XML")).arg(path),
			  	QString::null,
			  	__ERRLOCN
			  )	;
		return	false	;
	}

	return	true	;
}
