/***************************************************************************
    file	         : kb_error.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_ERROR_H
#define	__KB_ERROR_H


#include	<stdio.h>

#include	<qglobal.h>
#include	<qdict.h>
#include	<qstring.h>
#include	<qvaluelist.h>

#include 	"libcommon_exports.h"

#if		defined(_WIN32) && !defined(__MINGW32__)
#pragma		warning(push)
#pragma 	warning(disable : 4251)
#endif


/*  KBErrorInfo								*/
/*  -----------								*/
/*  Information for one error.						*/

class	LIBCOMMON_API	KBErrorInfo
{
public	:

	int		m_etype	    ;	/* General error type		*/
	QString		m_message   ;	/* Error message		*/
	QString		m_details   ;	/* Optional error details	*/
	cchar		*m_file	    ;	/* Source code file		*/
	uint		m_lno	    ;	/* Source code line number	*/
	int		m_errno	    ;	/* System error code		*/

	KBErrorInfo	()	;

	KBErrorInfo
	(	int,
		const QString	&,
		const QString	&,
		cchar		*,
		uint
	)	;
}	;

/*  ------------------------------------------------------------------	*/

/*  KBError								*/
/*  -------								*/
/*  General error class. This class is used to pass error information	*/
/*  about. It can contain one or more error, depending on how error	*/
/*  display is being blockec.						*/

class	LIBCOMMON_API	KBError
{
public	:

	enum	EType
	{	None,		/* No error				*/
		Query,		/* User query				*/
		Info,		/* Eg., previous row at first row	*/
		Warning,	/* Eg., field contents invalid		*/
		Error,		/* Eg., SQL statement syntax error	*/
		Fault		/* Eg., internal bugs			*/
	}	;

private	:

	QValueList<KBErrorInfo>	m_errors	;

public	:

	KBError		(EType, const QString &, const QString &, cchar *, uint) ;
	KBError		(const KBError &)	;
	KBError		()			;

	void			setErrno	(int)	;
	void			setDetails	(const QString &) ;

	void			append	 	(const KBError &) ;
	void			operator = 	(const KBError &) ;

	KBError::EType		getEType   	() const ;
	const QString		&getMessage	() const ;
	const QString		&getDetails	() const ;
	cchar			*getFile	() const ;
	uint			getLineno	() const ;
	int			getErrno	() const ;

	static	int		EInfo      	(const QString &, const QString &, cchar *, uint) ;
	static	int		EWarning   	(const QString &, const QString &, cchar *, uint) ;
	static	int		EError	   	(const QString &, const QString &, cchar *, uint) ;
	static	int		EFault	   	(const QString &, const QString &, cchar *, uint) ;
	static	void		EFatal	   	(const QString &, const QString &, cchar *, uint) ;

	bool			display	   	(const QString & = QString::null, cchar * = 0, uint = 0) const ;
	void			display	   	(FILE *) const ;

	static	uint		errDebug   	;
	friend	class		KBErrorBlockBase;
	friend	class		KBErrorDlg	;
}	;


/*  KBErrorBlockBase							*/
/*  ----------------							*/
/*  This class is used to allow "error blocking". For instance, there	*/
/*  are circumstances where repeated identical errors can occur, but	*/
/*  we only want to anoy the user with the first.			*/

class	LIBCOMMON_API	KBErrorBlockBase
{
public	:

	enum	EBlock
	{	All,		/* No blocking, show all		*/
		Distinct,	/* Show distinct errors			*/
		Single,		/* Show only one error			*/
		Accrue,		/* Accrue (assumulate) errors		*/
		AccrueDistinct,	/* Accrue distinct errors		*/
		None		/* Show no errors			*/
	}	;

protected	:

	KBErrorBlockBase	*m_prior	;
	KBError			*m_accrue	;
	EBlock			m_block		;
	uint			m_errorCount	;
	QDict<int>		m_occured	;

	void		display ()		;
	
public	:

	KBErrorBlockBase	(EBlock)	;
	~KBErrorBlockBase	()		;

	bool		showError	(const KBError *)	;
	uint		errorCount	()			;
}	;

#define	__ERRLOCN		__FILE__, __LINE__


#define	DISPLAY()		display	   (QString::null, __ERRLOCN)
#define	DISPLAYCAP(cap)		display	   (cap, 	   __ERRLOCN)
#define	DISPLAYFD(fd)		display	   (fd) ;

#if		defined(_WIN32) && !defined(__MINGW32__)
#pragma		warning(pop)
#endif

#endif	// __KB_CLASSES_H
