
/***************************************************************************
    file	         : kb_location.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_LOCATION_H
#define	__KB_LOCATION_H

#include "kb_classes.h"
#include "kb_dom.h"
#include "libcommon_exports.h"


class 	KBLocnTypeMap	;
class	KBServerInfo	;
class	KBBaseInsert	;
class	KBBaseUpdate	;

struct	RKColumnTypes	;

/*  KBLocation								*/
/*  ----------								*/
/*  The location of a document (a form, query, script, ...) is		*/
/*  specified by three things						*/
/*	(a) the database it is located in				*/
/*	(b) location in that database 					*/
/*	(c) the name within that location				*/
/*  The location object also provides other information, such as the	*/
/*  part factory for the object.					*/

class	LIBCOMMON_API	KBLocation
{
	KBDBInfo		*m_dbInfo	;
	QString			m_type		;
	QString			m_server	;
	QString			m_name		;
	QString			m_extension	;

	QString			m_dataServer	;

	bool			getData 	(cchar *, KBError &, QByteArray &) const ;

	bool			saveToFile	(const QString &, const QString &, cchar *, uint, KBError &) ;
	bool			saveToDB	(const QString &, const QString &, cchar *, uint, KBError &) ;
	bool			renameFile	(const QString &, KBError &) ;
	bool			renameDB	(const QString &, KBError &) ;
	bool			removeFile	(KBError &) ;
	bool			removeDB	(KBError &) ;

	QString			buildInfoQuery	(KBDBLink &) 			const ;
	QString			buildDataQuery	(KBDBLink &, cchar *, bool)	const ;
	QString			buildInsertQuery(KBDBLink &, bool   )		const ;
	QString			buildUpdateQuery(KBDBLink &)			const ;
	QString			buildDeleteQuery(KBDBLink &)			const ;

static	KBLocnTypeMap		*findByType	(const QString &) ;
static	KBLocnTypeMap		*findByExtn	(const QString &) ;

public	:

	static	const char* 	m_pLocal	;
	static	const char* 	m_pFile		;
	static	const char* 	m_pInline	;
	static	const char* 	m_pStock	;

	enum	CacheOption
	{
		CacheNone	= 0,
		CacheAll	= 1,
		CacheGraphics	= 2
	}	;

	const RKColumnTypes	*columnTypes	(KBDBLink &, KBError &) ;

	bool			operator== 	(const KBLocation &) ;

	QString			path		(const QString & = QString::null) const ;
	QString			filename	(const QString & = QString::null) const ;

	KBDBInfo		*dbInfo	  	() const ;
	const QString		&type	  	() const ;
	const QString		&server	  	() const ;
	const QString		&dataServer 	() const ;
	const QString		&name	  	() const ;
	const QString		&extension	() const ;

	void			setServer	(const QString	&) ;
	void			setName		(const QString	&) ;
	void			setDataServer	(const QString	&) ;

	QString			ident		() const ;
	QString			title		() const ;
	bool			exists		() const ;

	bool			contents	(QByteArray    &, KBError &) const ;
	QString			contents	(		  KBError &) const ;
	QDomDocument		contentsAsDom	(const QString &, KBError &) const ;
	QString			timestamp 	(		  KBError &) const ;

	bool			save		(const QString &, const QString &, const QString &, KBError &) ;
	bool			save		(const QString &, const QString &, cchar *,   uint, KBError &) ;
	bool			rename		(const QString &, KBError &) ;
	bool			remove		(KBError    &) ;

	/* The following methods check on the object location.		*/
	/* NOTE:							*/
	/* 	For historical reasons) "isFile" means "is a rekall	*/
	/*	object stored in a file" (as opposed to in an RDBMS).	*/
	/*	The "isLocal" method indicates if the object is stored	*/
	/*	in any other "local" file.				*/
	bool			isFile  	() const ;
	bool			isInline	() const ;
	bool			isStock 	() const ;
	bool			isInDB  	() const ;
	bool			isLocal  	() const ;

	KBServerInfo		*getServerInfo	() const ;
	KBPartFactory		*getFactory	() const ;

	QString			samePlace	(const QString &) const ;

	static	void		registerType	(cchar *, cchar *, const QString &, KBPartFactory *) ;
	static	QString		extnForType	(KBDBInfo *,       const QString &, const QString &) ;
	static	void		setCacheSize	(uint, CacheOption) ;
	static	void		setCacheAll	(bool)	;
	static	uint		getCacheUsed	()	;


	KBLocation		(KBDBInfo *, cchar *, const QString &, const QString & = "", const QString & = "") ;
	KBLocation		(const KBLocation &) ;
	KBLocation		()	;

	friend	class		KBLocationTest	;
}	;

#endif	// __KB_LOCATION_H
