/***************************************************************************
    file	         : kb_locator_qt3.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

/*  ------------------------------------------------------------------  */
/*  QT3/Linux								*/
/*  ------------------------------------------------------------------  */


#if		__KB_TKC

#include	<qstring.h>
#include 	<qdir.h>
#include	<qfileinfo.h>

#include	"kb_classes.h"

#define		__DEBUG_LOCATOR		(0)
#if		__DEBUG_LOCATOR	
#include 	"kb_dprintf.h"
#endif

#include	"tk_messagebox.h"


static	QString	rootdir	= ROOTDIR ;

#define	DATAPATH	"/share/apps/rekallqt/"
#define	DOCPATH		"/share/doc/HTML/en/rekallqt/"

/*  locateFile	: Locate a file of specified type			*/
/*  type	: const char *	  : File type				*/
/*  name	: const QString & : File name				*/
/*  (returns)	: QString	  : Path				*/

QString	locateFile
	(	const char	*type,
		const QString	&name
	)
{
	DPRINTF
	((	"locateFile: %s: %s (with %s)\n",
		type,
		name   .latin1(),
		rootdir.latin1()
	))	;

	QString	path	;

	if	(qstrcmp (type, "appdata") == 0) path = rootdir + DATAPATH + name ;
	else if (qstrcmp (type, "doc"    ) == 0) path = rootdir + DOCPATH  + name ;

	if (path != 0)
		if (!QFileInfo(path).exists())
			path	= QString::null ;

	return	path	;
}

/*  locateDir	: Locate a directory of specified type			*/
/*  type	: const char *	  : Directory type			*/
/*  name	: const QString & : Directory name			*/
/*  (returns)	: QString	  : Path				*/

QString	locateDir
	(	const char	*type,
		const QString	&
	)
{
	/* The name is never used here. The argument exists since under	*/
	/* KDE there seems to be no way to locate an application	*/
	/* directory without naming a file that will be found in it.	*/
	return	locateFile   (type, "") ;
}

/*  setRootDir	: Set different root directory				*/
/*  _rootdir	: const QString & : New root directory name		*/
/*  (returns)	: void		  :					*/

void	setRootDir
	(	const QString	&_rootdir
	)
{
	DPRINTF
	((	"setRootDir(%s)\n",
		_rootdir.latin1()
	))	;

	rootdir	= _rootdir ;

	if (locateDir ("appdata", "services").isNull())
		TKMessageBox::sorry
		(	0,
			"Rekall has set a root directory that does not "
			"appear to have the right contents. Rekall will "
			"probably not run",
			"Root Directory Error"
		)	;
}

/*  getRootDir	: Get root directory					*/
/*  (returns)	: QString &	: Directory name			*/

QString	getRootDir ()
{
	fprintf	(stderr, "getRootDir(%s)\n", (const char *)rootdir) ;
}

#endif
