/***************************************************************************
    file	         : kb_serverinfo.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#ifndef	__KB_SERVERINFO_H
#define	__KB_SERVERINFO_H

#if	defined(_WIN32) && !defined(__MINGW32__)
#pragma warning(push)
#pragma warning(disable : 4251)
#endif

#include	<qdict.h>
#include	<qlabel.h>
#include	<qlistbox.h>
#include	<qlineedit.h>
#include	<qcheckbox.h>
#include	<qpushbutton.h>
#include	<qdom.h>

#include	"libcommon_exports.h"

#include	"kb_classes.h"
#include	"kb_database.h"
#include	"kb_tableinfo.h"
#include	"kb_dblink.h"


/*  KBServerInfo							*/
/*  ------------							*/
/*  KBase accesses data stored in "servers", where a server is an SQL	*/
/*  engine, such as MySQL or PostgreSQL. Note that on KBase, the term	*/
/*  "database" is used to refer to a set of server connections, forms,	*/
/*  reports, and so on, which the end user views as a database.		*/

class	LIBCOMMON_API	KBServerInfo
{
public	:

	enum	TestsEnable
	{	TestsOff	= 0,
		TestsOnly	= 1,
		TestsSuite	= 2,
		TestsRecord	= 3
	}	;

protected :

	enum	SysState
	{
		SysUnknown,
		SysPresent,
		SysMissing
	}	;


	KBDBInfo	*m_dbInfo	;	/* Parent information	*/

	QString		m_serverName	;	/* Identifying name	*/
	QString		m_dbType	;	/* Server type		*/
	QString		m_hostName	;	/* Host running server	*/
	QString		m_dbName	;	/* Server database name	*/
	QString		m_userName	;	/* Server login name	*/
	QString		m_password	;	/* Server password	*/
	QString		m_portNumber	;	/* Optional port number	*/
	QString		m_socketName	;	/* Optional socket name	*/
	QString		m_flags		;	/* Optional flags	*/
	bool		m_isDisabled	;	/* Connection disabled	*/
	bool		m_autoStart	;	/* Autostart "MainForm"	*/
	QString		m_autoForm	;	/* .... or this one	*/
	bool		m_printQueries	;	/* Debug printing	*/

	bool		m_noRekallTables;	/* No rekall tables	*/
	bool		m_showAllTables	;	/* Show all tables	*/
	bool		m_cacheTables	;	/* Cache table info	*/
	bool		m_nullUserPwd	;	/* Null user/pwd OK	*/
	bool		m_pkReadOnly	;	/* Primary key r/only	*/
	bool		m_fakeKeys	;	/* Fake insert keys	*/
	bool		m_readOnly	;	/* Read-only database	*/
	TestsEnable	m_showTests	;	/* Shows tests/suites	*/
	QString		m_appFont	;	/* Application font	*/
	QString		m_initSQL	;	/* Initial SQL		*/
	QString		m_dataEncoding	;	/* Data encoding	*/
	QString		m_objEncoding	;	/* Object encoding	*/
	QString		m_webDirectory	;	/* Default web dir.	*/
	QString		m_skinSuffix	;	/* Skin suffix		*/
	QString		m_comment	;

#ifdef	Q_OS_LINUX
	QString		m_sshTarget	;
#endif

	SysState	m_objState	;	/* Object table state	*/

	KBServer	*m_server	;	/* Server connection	*/
	QPtrList<KBDBLink>
			m_links		;	/* Current links	*/

	KBTableInfoSet	*m_tableInfoSet	;	/* Table information	*/
	KBDBAdvanced	*m_advanced	;	/* Advanced settings	*/


	QString		m_useUserName	;
	QString		m_usePassword	;

	void		makeObjTable		() ;
	void		checkForTables		() ;

	virtual	bool	promptUserPassword	(KBError &) ;
	virtual	bool	promptMakeObjTable	() ;

public	:

	KBServerInfo (KBDBInfo *)	;
	KBServerInfo (KBDBInfo *, const KBServerInfo *) ;
	KBServerInfo (KBDBInfo *, cchar  *) ;
	KBServerInfo (KBDBInfo *, const QDomElement  &) ;

	KBServerInfo
	(	KBDBInfo *,
		const QString &, const QString &, const QString &,
		const QString &, const QString &, const QString &,
		const QString & = QString::null,
		const QString & = QString::null
	)	;

	virtual	~KBServerInfo () ;

	void			saveTableInfo	()		;
	void			buildSpecElement(QDomElement &) ;

	inline	KBDBInfo	*dbInfo		() const { return m_dbInfo	  ; }

	inline	const QString	&serverName	() const { return m_serverName    ; }
	inline	const QString	&dbType		() const { return m_dbType        ; }
	inline	const QString	&userName	() const { return m_useUserName   ; }
	inline	const QString	&password	() const { return m_usePassword   ; }
 	inline	const QString	&hostName	() const { return m_hostName	  ; }
 	inline	const QString	&database	() const { return m_dbName	  ; }
 	inline	const QString	&portNumber	() const { return m_portNumber    ; }
 	inline	const QString	&socketName	() const { return m_socketName    ; }
 	inline	const QString	&flags		() const { return m_flags	  ; }

 	inline	bool		showAllTables	() const { return m_showAllTables ; }
 	inline	bool		cacheTables	() const { return m_cacheTables   ; }
 	inline	bool		printQueries	() const { return m_printQueries  ; }
 	inline	bool		pkReadOnly	() const { return m_pkReadOnly    ; }
 	inline	bool		fakeKeys	() const { return m_fakeKeys	  ; }
 	inline	bool		readOnly	() const { return m_readOnly	  ; }
 	inline	TestsEnable	showTests	() const { return m_showTests	  ; }
	inline	const QString	&autoForm	() const { return m_autoForm	  ; }
	inline	const QString	&appFont	() const { return m_appFont	  ; }
	inline	const QString	&skinSuffix	() const { return m_skinSuffix	  ; }
	inline	const QString	&initSQL	() const { return m_initSQL	  ; }
 	inline	const QString	&dataEncoding	() const { return m_dataEncoding  ; }
 	inline	const QString	&objEncoding	() const { return m_objEncoding   ; }
 	inline	const QString	&webDirectory	() const { return m_webDirectory  ; }
 	inline	const QString	&comment	() const { return m_comment	  ; }
#ifdef	Q_OS_LINUX
	inline	const QString	&sshTarget	() const { return m_sshTarget     ; }
#endif
	const QString		&getDBPath	() const ;

	inline	void	setPort
		(	const QString	&port
		)
	{
		m_portNumber	= port	;
	}

	void			attachLink	(KBDBLink	*) ;
	void			detachLink	(KBDBLink	*) ;
	KBServer		*getServer	(KBError  	&) ;

	bool			linkIsOpen	() ;
	KBTableInfoSet		*tableInfoSet	() ;

	inline	bool		disabled     	() const { return m_isDisabled		; }
	inline	bool		hasObjectTable	() const { return m_objState  == SysPresent	; }
	inline	KBDBAdvanced	*advanced	() const { return m_advanced			; }

	/* The "setServer" method is used by the testing code and	*/
	/* should not be called in normal use.				*/
	void	setServer	(const QString &, KBServer *) ;


	static	QString		getIdentStrings	() ;

	friend	class		KBServerDlg	;
	friend	class		KBServerTab	;
	friend	class		KBDBInfo	;
	friend	class		KBDBAdvancedDlg	;
}	;

LIBCOMMON_API KBFactory		*getDriverFactory	(const QString &) ;
LIBCOMMON_API KBServer		*getDriverServer	(const QString &) ;
LIBCOMMON_API KBDBAdvanced	*getDriverAdvanced	(const QString &) ;

#if	defined(_WIN32) && !defined(__MINGW32__)
#pragma warning(pop)
#endif

#endif	// __KB_SERVERINFO_H
