/***************************************************************************
    file	         : kb_sidepanel.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qpainter.h>
#include	<qfontmetrics.h>

#include	"kb_sidepanel.h"

KBSidePanel::KBSidePanel
	(	QWidget		*parent,
		const QString	&extra,
		const QString	&legend
	)
	:
	QFrame		(parent),
	m_legend	(legend),
	m_extra		(extra),
	m_legendFont	("arial", 17, QFont::Bold,   false),
	m_extraFont	("arial", 12, QFont::Normal, false)
{
	if (m_legend.isNull()) m_legend = "REKALL" ;

	m_legendHeight = QFontMetrics(m_legendFont).height() ;
	m_extraHeight  = QFontMetrics(m_extraFont ).height() ;

	setMinimumWidth	(m_legendHeight + m_extraHeight * 2) ;

	int	ll	= QFontMetrics(m_legendFont).width(m_legend) ;
	int	el	= QFontMetrics(m_legendFont).width(m_extra ) ;
	setMinimumHeight(QMAX(ll, el) + m_legendHeight) ;
}

void	KBSidePanel::paintEvent
	(	QPaintEvent	*p
	)
{
	QPainter paint     (this)   ;
	paint.setBackgroundColor (QColor(0,0,255)) ;
	paint.eraseRect (p->rect()) ;

	int	w	= width ()  ;
	int	h	= height()  ;
	int	lastY	= 0	    ;
	int	lastB	= 128	    ;

	for (int y = 4 ; y < h ; y += 4)
	{
		int	b = 128 + (128 * y) / h ;

		if (b != lastB)
		{
			paint.fillRect	(0, lastY, w, y - lastY, QBrush(QColor(0, 0, b))) ;
			lastY	= y	;
			lastB	= b	;
		}
	}

	paint.fillRect	(0, lastY, w, h - lastY, QBrush(QColor(0, 0, 255))) ;

	paint.setPen 	(QPen(QColor(255,255,255))) ;
	paint.rotate	(270) ;

	paint.setFont	(m_legendFont) ;
//	paint.drawText	(-p->rect().height() + m_legendHeight / 2, m_legendHeight, m_legend) ;
	paint.drawText	(-height() + m_legendHeight / 2, m_legendHeight, m_legend) ;

	paint.setFont	(m_extraFont ) ;
//	paint.drawText	(-p->rect().height() + m_legendHeight / 2, m_legendHeight + (m_extraHeight * 4 / 3), m_extra) ;
	paint.drawText	(-height() + m_legendHeight / 2, m_legendHeight + (m_extraHeight * 4 / 3), m_extra) ;
}

void	KBSidePanel::setExtra
	(	const QString	&extra
	)
{
	m_extra	= extra	;
	update	()	;	
}
