/***************************************************************************
    file	         : kb_macrosql.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_appptr.h"
#include	"kb_callback.h"
#include	"kb_macrosql.h"
#include	"kb_node.h"
#include	"kb_dbinfo.h"


/*  KBMacroSQL								*/
/*  KBMacroSQL								*/
/* 		: Macro to open a query					*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroSQL::KBMacroSQL
	(	KBMacroExec	*exec
	)
	:
	KBMacroInstr (exec, "SQL")
{
}

/*  KBMacroSQL								*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroSQL::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 1, 1, pError) ;
}

/*  KBMacroSQL								*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroSQL::execute
	(	KBError		&pError
	)
{
	extern	KBSQLQuery	*executeSQLQuery
				(	KBDBLink	&dbLink,
					const QString	&query,
					bool		&ok,
					const KBValue	*argv,
					uint		argc
				)	;

	if (m_exec->getDBInfo() == 0)
	{
		KBError::EError
		(	TR("Executing SQL macro, no database link"),
			QString::null,
			__ERRLOCN
		)	;
		return	false	;
	}

	bool	   ok	;
	KBDBLink   dbl	;

	if (!dbl.connect(m_exec->getDBInfo(), m_exec->getServer()))
	{
		dbl.lastError().DISPLAY() ;
		return	false ;
	}

	KBSQLQuery *qry = executeSQLQuery
			  (	dbl,
				m_args[0],
				ok,
				0,
				0
			  )	;

	if (qry == 0)
		return	true	;

	if (!ok)
		qry->lastError().DISPLAY() ;

	delete	qry	;
	return	ok	;
}

/*  ------------------------------------------------------------------  */


MACROREG(SQL )

/*  ------------------------------------------------------------------  */

#ifdef	__DOCS
  <macro name="SQL">
    <arg type="text" legend="SQL statement"></arg>
      <![CDATA[
        <h3>Execute SQL</h3>
        Executes the SQL statement.
      ]]>
  </macro>
#endif	// __DOCS
