/***************************************************************************
    file	         : kb_macrotest.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qregexp.h>

#include	"kb_classes.h"
#include	"kb_macrotest.h"
#include	"kb_framer.h"
#include	"kb_form.h"
#include	"kb_button.h"
#include	"kb_label.h"
#include	"kb_richtext.h"
#include	"kb_stack.h"
#include	"kb_tabber.h"
#include	"kb_ctrl.h"
#include	"kb_test.h"
#include	"kb_prompt.h"

#include	"tk_messagebox.h"
#include	"rk_combobox.h"

#include	"kb_testerrordlg.h"

#define		__DEBUG_MACROTEST	(0)
#if		__DEBUG_MACROTEST
#include	"kb_dprintf.h"
#endif


/*  KBMacroTest								*/
/*  KBMacroTest								*/
/* 		: Macro to verify a control value			*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  action	: cchar *	   : Instruction action			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroTest::KBMacroTest
	(	KBMacroExec	*exec,
		cchar		*action
	)
	:
	KBMacroInstr (exec, action)
{
}

KBItem	*KBMacroTest::getItem
	(	QString		&error
	)
{
	KBNode	*invoker = m_exec->getInvoker() ;
	if ((invoker == 0) || (invoker->isForm() == 0))
	{	error	= "Cannot locate form" ;
		return	0 ;
	}
	KBNode   *node	= invoker->isForm()->getNamedNode(m_args[0]) ;
	if ((node    == 0) || (node   ->isItem() == 0))
	{	error	= QString(TR("Cannot locate data control: %1")).arg(m_args[0]) ;
		return	0 ;
	}
	return	node->isItem() ;
}

KBObject
	*KBMacroTest::getObject
	(	QString		&error
	)
{
	KBNode	*invoker = m_exec->getInvoker() ;
	if ((invoker == 0) || (invoker->isForm() == 0))
	{	error	= "Cannot locate form" ;
		return	0 ;
	}
	KBNode   *node	= invoker->isForm()->getNamedNode(m_args[0]) ;
	if ((node    == 0) || (node   ->isObject() == 0))
	{	error	= QString(TR("Cannot locate static control: %1")).arg(m_args[0]) ;
		return	0 ;
	}
	return	node->isObject() ;
}

KBValue	KBMacroTest::getValue ()
{
	KB::IType	type	= KB::ITUnknown	  ;
	QString		text	= m_args[3]	  ;
	int		coff	= text.find (':') ;

	if (coff >= 0)
	{	type	= (KB::IType)text.left(coff).toInt() ;
		text	= text.mid(coff+1) ;
	}

	switch (type)
	{
		case KB::ITUnknown	: return KBValue (text, &_kbUnknown ) ;
		case KB::ITRaw		: return KBValue (text, &_kbRaw     ) ;
		case KB::ITFixed	: return KBValue (text, &_kbFixed   ) ;
		case KB::ITFloat	: return KBValue (text, &_kbFloat   ) ;
		case KB::ITDate		: return KBValue (text, &_kbDate    ) ;
		case KB::ITTime		: return KBValue (text, &_kbTime    ) ;
		case KB::ITDateTime	: return KBValue (text, &_kbDateTime) ;
		case KB::ITString	: return KBValue (text, &_kbString  ) ;
		case KB::ITBinary	: return KBValue (text, &_kbBinary  ) ;
		case KB::ITBool		: return KBValue (text, &_kbBool    ) ;
		default			: break	 ;
	}

	return KBValue (text, &_kbUnknown ) ;
}

KBControl
	*KBMacroTest::ctrlAtDRow
	(	KBItem		*item,
		uint		drow
	)
{
	return	item->ctrlAtQRow(item->getBlock()->getCurDRow() + drow) ;
}

KBControl
	*KBMacroTest::control
	(	KBObject	*object
	)
{
	return	object->m_control ;
}

QString	KBMacroTest::emptyToNull
	(	const QString	&text
	)
{
	return	text.isNull() ? QString("") : text ;
}


bool	KBMacroTest::isFixable ()
{
	return	false	;
}

void	KBMacroTest::fix ()
{
}

/*  KBMacroTest								*/
/*  testFailed	: Called when a test has failed				*/
/*  pError	: KBError &	  : Error return			*/
/*  what	: const QString & : Test				*/
/*  error	: const QString & : Error message			*/
/*  path	: const QString & : Path to object if any		*/
/*  (returns)	: bool		  : Returned as instruction success	*/

bool	KBMacroTest::testFailed
	(	KBError		&pError,
		const QString	&what,
		const QString	&error,
		const QString	&path
	)
{
#if	! __KB_RUNTIME
	KBScriptTestResult	str
				(	m_exec->getName   (),
					m_exec->getPC     () - 1,
					m_exec->getComment(),
					KBScriptTestResult::testFailed,
					error,
					"macro",
					path
				)	;

	KBTest::appendTestResult(str)	;

	pError	= KBError
		  (	KBError::Error,
			QString(TR("%1 failed at instruction %1"))
				.arg(what)
				.arg(m_exec->getPC() - 1),
			error,
			__ERRLOCN
		  )	;

	/* If running in test suite mode then we may need to show the	*/
	/* test error dialog; this in turn may change the way the	*/
	/* error is handled (eg., the user can abort the suite or abort	*/
	/* all tests).							*/
	if (KBTest::getTestMode() == KBScriptIF::testSuite)
	{
		KBTest::ErrorOpt errorOpt = KBTest::getErrorOpt () ;

		DPRINTF
		((	"KBMacroTest::testFailed: errorOpt=[%d]\n",
			errorOpt
		))	;

		if ((errorOpt == KBTest::ErrorPauseTest ) || (errorOpt == KBTest::ErrorUpdateTest))
			errorOpt = (KBTest::ErrorOpt)KBTestErrorDlg(str, errorOpt, isFixable()).exec() ;

		KBTest::setErrorOpt (errorOpt) ;

		/* NOTE:						*/
		/* KBTest::ErrorAutoUpdate is a special case, used only	*/
		/* by the unittest code so that we can trigged update	*/
		/* without getting a dialog.				*/
		if ((errorOpt == KBTest::ErrorUpdateTest) || (errorOpt == KBTest::ErrorAutoUpdate))
		{
			fix	()	;
			KBTest::setObjectFixed (true) ;
			return	true	;
		}

		if (errorOpt == KBTest::ErrorPauseTest )
			return	true	;

		return	false	;
	}

	pError.DISPLAY();
#endif
	return	true	;
}

/*  ------------------------------------------------------------------	*/

/*  KBMacroVerifyValue							*/
/*  KBMacroVerifyValue							*/
/* 		: Macro to verify a control value			*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroVerifyValue::KBMacroVerifyValue
	(	KBMacroExec	*exec
	)
	:
	KBMacroTest (exec, "VerifyValue")
{
}

/*  KBMacroVerifyValue							*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroVerifyValue::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 4, 4, pError) ;
}

/*  KBMacroVerifyValue							*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroVerifyValue::execute
	(	KBError		&pError
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBMacroVerifyValue::execute: path=[%s] name=[%s] drow=[%s]\n",
		m_args[0].latin1(),
		m_args[1].latin1(),
		m_args[2].latin1()
	))	;

	QString		error	;
	KBItem		*item	;
	uint		drow	= 0 ;
	KBValue		expect	= getValue () ;
	KBControl	*ctrl	;

	m_fixable	= false		;
	m_value		= KBValue ()	;

	if ((item = getItem (error)) == 0)
		goto	errExit	;

	if (item->isHidden() == 0)
	{
		ctrl	= ctrlAtDRow(item, m_args[2].toInt()) ;
		if (ctrl == 0)
		{	error	= QString(TR("No control at display row %1")).arg(m_args[2].toInt()) ;
			goto	errExit	;
		}
	}

	drow	  = item->getBlock()->getCurDRow() ;
	m_value	  = item->getValue(drow + m_args[2].toInt()) ;
	m_fixable = true ;

	if (emptyToNull(m_value.getRawText()) != emptyToNull(expect.getRawText()))
	{
		error	= QString(TR("Value error: got '%1' expected '%2'"))
				.arg(m_value.getRawText())
				.arg(expect .getRawText())
				;
		goto	errExit	;
	}
	if (m_value.getType()->getIType() != expect.getType()->getIType())
	{	
		error	= QString(TR("Type error: got %1 expected %2"))
				.arg(m_value.getType()->getDescrip())
				.arg(expect .getType()->getDescrip())
				;
		goto	errExit	;
	}

	return	true ;

	errExit	:
		return	testFailed
			(	pError,
				TR("Value test"),
				error,
				QString("%1:%2").arg(m_args[0]).arg(m_args[2])
			)	;
#else
	return	true	;
#endif
}

bool	KBMacroVerifyValue::isFixable ()
{
	return	m_fixable	;
}

void	KBMacroVerifyValue::fix ()
{
	m_args[3] = QString("%1:%2").arg(m_value.getType()->getIType()).arg(m_value.getRawText()) ;
}

/*  ------------------------------------------------------------------	*/

/*  KBMacroVerifyRegexp							*/
/*  KBMacroVerifyRegexp							*/
/* 		: Macro to verify control value with regular expression	*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroVerifyRegexp::KBMacroVerifyRegexp
	(	KBMacroExec	*exec
	)
	:
	KBMacroTest (exec, "VerifyRegexp")
{
}

/*  KBMacroVerifyRegexp							*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroVerifyRegexp::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 4, 4, pError) ;
}

/*  KBMacroVerifyRegexp							*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroVerifyRegexp::execute
	(	KBError		&pError
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBMacroVerifyRegexp::execute: path=[%s] name=[%s] drow=[%s] r=[%s]\n",
		m_args[0].latin1(),
		m_args[1].latin1(),
		m_args[2].latin1(),
		m_args[3].latin1()
	))	;

	QString		error	;
	KBItem		*item	;
	uint		drow	= 0 ;
	KBControl	*ctrl	;

	m_fixable	= false		;
	m_value		= QString::null	;

	if ((item = getItem (error)) == 0)
		goto	errExit	;

	if (item->isHidden() == 0)
	{
		ctrl	= ctrlAtDRow(item, m_args[2].toInt()) ;
		if (ctrl == 0)
		{	error	= QString(TR("No control at display row %1")).arg(m_args[2].toInt()) ;
			goto	errExit	;
		}
	}

	drow	  = item->getBlock()->getCurDRow() ;
	m_value	  = item->getValue(drow + m_args[2].toInt()).getRawText() ;
	m_fixable = true ;

	if (m_value.find (QRegExp(m_args[3])) < 0)
	{
		error	= QString(TR("Regular expression '%1' not matched: got %2"))
					.arg(m_args[3])
					.arg(m_value)
					;
		goto	errExit	;
	}

	return	true ;

	errExit	:
		return	testFailed
			(	pError,
				TR("Regexp test"),
				error,
				QString("%1:%2").arg(m_args[0]).arg(m_args[2])
			)	;
#else
	return	true	;
#endif
}

bool	KBMacroVerifyRegexp::isFixable ()
{
	return	m_fixable	;
}

void	KBMacroVerifyRegexp::fix ()
{
	QString		  regexp ;
	KBPromptRegexpDlg reDlg
		(	TR("Verify Regular Expression"),
			TR("Enter regular expression" ),
			m_value,
			regexp
		)	;
	if (reDlg.exec())
		m_args[3] = regexp	;
}

/*  ------------------------------------------------------------------  */

/*  KBMacroVerifyChoices						*/
/*  KBMacroVerifyChoices						*/
/* 		: Macro to verify choices in a choice or link control	*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroVerifyChoices::KBMacroVerifyChoices
	(	KBMacroExec	*exec
	)
	:
	KBMacroTest (exec, "VerifyChoices")
{
}

/*  KBMacroVerifyChoices						*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroVerifyChoices::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 4, 4, pError) ;
}

/*  KBMacroVerifyChoices						*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroVerifyChoices::execute
	(	KBError		&pError
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBMacroVerifyChoices::execute: path=[%s] name=[%s] drow=[%s]: [%s]\n",
		m_args[0].latin1(),
		m_args[1].latin1(),
		m_args[2].latin1(),
		m_args[3].latin1()
	))	;

	QString	 	error	;
	KBItem	 	*item	;
	uint	 	drow	= m_args[2].toInt() ;
	KBValue	 	value	;
	KBControl	*ctrl	;
	RKComboBox	*combo	;
	QStringList	got	;

	m_fixable	= false		;
	m_choices	= QString::null	;

	if ((item = getItem (error)) == 0)
		goto	errExit	;

	if (!item->isA("KBChoice") && !item->isA("KBLink"))
	{	error	= QString(TR("Expected KBChoice or KBLink, got '%1'")).arg(item->className()) ;
		goto	errExit	;
	}

	ctrl	= ctrlAtDRow(item, m_args[2].toInt()) ;
	if (ctrl == 0)
	{	error	= QString(TR("No control at display row %1")).arg(drow) ;
		goto	errExit	;
	}

	combo	= (RKComboBox *)ctrl->getWidget() ;
	if (combo == 0)
	{	error	= QString(TR("No widget at display row %1" )).arg(drow) ;
		goto	errExit	;
	}

	for (int idx = 0 ; idx < combo->count() ; idx += 1)
		got.append (combo->text(idx)) ;

	m_choices = got.join("|") ;
	m_fixable = true	  ;

	if (m_choices != m_args[3])
	{
		DPRINTF
		((	"KBMacroVerifyChoices::execute: %p: [%s]!=[%s]\n",
			(void *)this,
			m_args[3].latin1(),
			m_choices.latin1()
		))	;
		error	= QString(TR("Choices error: Got '%1' expected '%2'")).arg(got.join("|")).arg(m_args[3]) ;
		goto	errExit	;
	}

	return	true ;

	errExit	:
		return	testFailed
			(	pError,
				TR("Choices test"),
				error,
				QString("%1:%2").arg(m_args[0]).arg(m_args[2])
			)	;
#else
		return	true	;
#endif
}

bool	KBMacroVerifyChoices::isFixable ()
{
	return	m_fixable	;
}

void	KBMacroVerifyChoices::fix ()
{
	DPRINTF
	((	"KBMacroVerifyChoices::fix: %p: [%s]->[%s]\n",
			(void *)this,
		m_args[3].latin1(),
		m_choices.latin1()
	))	;
	m_args[3] = m_choices	;
}

/*  ------------------------------------------------------------------  */

/*  KBMacroVerifyText							*/
/*  KBMacroVerifyText							*/
/* 		: Macro to verify text in label or button		*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroVerifyText::KBMacroVerifyText
	(	KBMacroExec	*exec
	)
	:
	KBMacroTest (exec, "VerifyText")
{
}

/*  KBMacroVerifyText							*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroVerifyText::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 4, 4, pError) ;
}

/*  KBMacroVerifyText						*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroVerifyText::execute
	(	KBError		&pError
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBMacroVerifyText::execute: path=[%s] name=[%s] drow=[%s]: [%s]\n",
		m_args[0].latin1(),
		m_args[1].latin1(),
		m_args[2].latin1(),
		m_args[3].latin1()
	))	;

	QString	 	error	;
	KBObject 	*object	;

	m_fixable	= false		;
	m_text		= QString::null	;

	if ((object = getObject (error)) == 0)
		goto	errExit	;

	if (!object->isA("KBButton") && !object->isA("KBLabel"))
	{	error	= QString(TR("Expected KBButton or KBLable, got '%1'")).arg(object->className()) ;
		goto	errExit	;
	}

	if (object->isA("KBLabel" ))
	{
		m_fixable = true ;
		m_text	  = ((KBLabel  *)object)->getText() ;

		if (m_text != m_args[3])
		{	error	= QString(TR("Expected '%1', got '%2'"))
					.arg(m_args[3])
					.arg(m_text)
					;
			goto	errExit	;
		}
	}

	if (object->isA("KBButton"))
	{
		m_fixable = true ;
		m_text	  = ((KBButton *)object)->getText() ;

		if (m_text != m_args[3])
		{	error	= QString(TR("Expected '%1', got '%2'"))
					.arg(m_args[3])
					.arg(m_text)
					;
			goto	errExit	;
		}
	}

	return	true ;

	errExit	:
		return	testFailed
			(	pError,
				TR("Text test"),
				error,
				m_args[0]
			)	;
#else
		return	true	;
#endif
}

bool	KBMacroVerifyText::isFixable ()
{
	return	m_fixable	;
}

void	KBMacroVerifyText::fix ()
{
	m_args[3] = m_text	;
}

/*  ------------------------------------------------------------------  */

/*  KBMacroVerifyState							*/
/*  KBMacroVerifyState							*/
/* 		: Macro to verify control state				*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroVerifyState::KBMacroVerifyState
	(	KBMacroExec	*exec
	)
	:
	KBMacroTest (exec, "VerifyState")
{
}

/*  KBMacroVerifyState							*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroVerifyState::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 5, 5, pError) ;
}

/*  KBMacroVerifyState							*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroVerifyState::execute
	(	KBError		&pError
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBMacroVerifyState::execute: path=[%s] name=[%s] drow=[%s]: e=%s v=%d\n",
		m_args[0].latin1(),
		m_args[1].latin1(),
		m_args[2].latin1(),
		m_args[3].latin1(),
		m_args[4].latin1()
	))	;

	QString	 	error	;
	KBObject	*object	;
	KBItem	 	*item	;
	uint	 	drow	= m_args[2].toInt() ;
	KBControl	*ctrl	;

	m_fixable	= false	;
	m_enabled	= false	;
	m_visible	= false	;

	if ((object = getObject (error)) == 0)
		goto	errExit	;

	if ((item = object->isItem()) != 0)
		ctrl	= ctrlAtDRow(item, m_args[2].toInt()) ;
	else	ctrl	= control   (object) ;

	if (ctrl == 0)
	{	error	= QString(TR("No control at display row %1"   )).arg(drow) ;
		goto	errExit	;
	}

	m_fixable = true ;
	m_enabled = (bool)ctrl->isEnabled() ;
	m_visible = (bool)ctrl->isVisible() ;

	if ((bool)m_enabled != (bool)m_args[3].toInt())
	{	error	= QString(TR("Control enable error at row %1" )).arg(drow) ;
		goto	errExit	;
	}
	if ((bool)m_visible != (bool)m_args[4].toInt())
	{	error	= QString(TR("Control visible error at row %1")).arg(drow) ;
		goto	errExit	;
	}

	return	true ;

	errExit	:
		return	testFailed
			(	pError,
				TR("State test"),
				error,
				QString("%1:%2").arg(m_args[0]).arg(m_args[2])
			)	;
#else
		return	true	;
#endif
}

bool	KBMacroVerifyState::isFixable ()
{
	return	m_fixable	;
}

void	KBMacroVerifyState::fix ()
{
	m_args[3] = m_enabled ? "1" : "0" ;
	m_args[4] = m_visible ? "1" : "0" ;
}

/*  ------------------------------------------------------------------  */

/*  KBMacroVerifyStack							*/
/*  KBMacroVerifyStack							*/
/* 		: Macro to verify stack/tabber state			*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroVerifyStack::KBMacroVerifyStack
	(	KBMacroExec	*exec
	)
	:
	KBMacroTest (exec, "VerifyStack")
{
}

/*  KBMacroVerifyStack							*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroVerifyStack::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 3, 3, pError) ;
}

/*  KBMacroVerifyStack							*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroVerifyStack::execute
	(	KBError		&pError
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBMacroVerifyStack::execute: path=[%s] name=[%s] page=[%s]\n",
		m_args[0].latin1(),
		m_args[1].latin1(),
		m_args[2].latin1()
	))	;

	QString	 	error	;
	KBObject	*object	;
	KBObject	*page	;

	if ((object = getObject (error)) == 0)
		goto	errExit	;

	if	(object->isStack () != 0)
	{
		page	= object->isStack ()->currentPage() ;
	}
	else if	(object->isTabber() != 0)
	{
		page	= object->isTabber()->currentPage() ;
	}
	else
	{
		error	= QString(TR("Control neither stack/tabber: got %1")).arg(object->getElement()) ;
		goto	errExit	;
	}

	if (page != 0)
		if (page->getName() != m_args[2])
		{
			error	= QString(TR("Expected page %1, got %2")).arg(m_args[2]).arg(page->getName()) ;
			goto	errExit	;
		}
	
	return	true ;

	errExit	:
		return	testFailed
			(	pError,
				TR("Stack.tabber test"),
				error,
				m_args[0]
			)	;
#else
		return	true	;
#endif
}

/*  ------------------------------------------------------------------	*/

/*  KBMacroUpdateValue							*/
/*  KBMacroUpdateValue							*/
/* 		: Macro to verify a control value			*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroUpdateValue::KBMacroUpdateValue
	(	KBMacroExec	*exec
	)
	:
	KBMacroTest (exec, "UpdateValue")
{
}

/*  KBMacroUpdateValue							*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroUpdateValue::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 4, 4, pError) ;
}

/*  KBMacroUpdateValue							*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroUpdateValue::execute
	(	KBError		&pError
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBMacroUpdateValue::execute: path=[%s] name=[%s] drow=[%s]\n",
		m_args[0].latin1(),
		m_args[1].latin1(),
		m_args[2].latin1()
	))	;

	QString		error	;
	KBItem		*item	;
	KBControl	*ctrl	;
	uint		qrow	;
	KBValue		value	;

	if ((item = getItem (error)) == 0)
		goto	errExit	;

	ctrl	= ctrlAtDRow(item, m_args[2].toInt()) ;
	if (ctrl == 0)
	{	error	= QString(TR("No control at display row %1")).arg(m_args[2]) ;
		goto	errExit	;
	}

	qrow	= item->getBlock()->getCurDRow() + m_args[2].toInt() ;
	value	= getValue() ;

	if (!item->setValue (qrow, value))
	{	error	= item->lastError().getMessage() ;
		goto	errExit	;
	}
	item->userChange (qrow, value, true) ;
	item->doLeave	 (qrow) ;

	return	true ;

	errExit	:
		return	testFailed
			(	pError,
				TR("Value update"),
				error,
				QString("%1:%2").arg(m_args[0]).arg(m_args[2])
			)	;
#else
		return	true	;
#endif
}

/*  ------------------------------------------------------------------	*/

/*  KBMacroKeyNavigation						*/
/*  KBMacroKeyNavigation						*/
/* 		: Macro to handle (keystroke) navigation		*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroKeyNavigation::KBMacroKeyNavigation
	(	KBMacroExec	*exec
	)
	:
	KBMacroTest (exec, "KeyNavigation")
{
}

/*  KBMacroKeyNavigation						*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroKeyNavigation::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 4, 4, pError) ;
}

/*  KBMacroKeyNavigation						*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroKeyNavigation::execute
	(	KBError		&pError
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBMacroKeyNavigation::execute: path=[%s] name=[%s] drow=[%s] key=[%s]\n",
		m_args[0].latin1(),
		m_args[1].latin1(),
		m_args[2].latin1(),
		m_args[3].latin1()
	))	;

	QString		error	;
	KBItem		*item	;
	KBNode		*parent	;
	KBBlock		*block	;
	KBNavigator	*navi	= 0 ;
	int		drow	;

	if ((item = getItem (error)) == 0)
		goto	errExit	;

	block	= item ->getBlock  () ;
	if (block->isFormBlock() != 0)
		if (block->isFormBlock()->getCurItem() != item)
		{
			error	= QString(TR("Currently at item %1, expected %2"))
						.arg(block->isFormBlock()->getCurItem()->getName())
						.arg(item->getName()) ;
			goto	errExit	;
		}

	drow	= block->getCurQRow() - block->getCurDRow() ;
	if (drow != m_args[2].toInt())
	{
		error	= QString(TR("Currently at row %1, expected %2")).arg(drow).arg(m_args[2].toInt()) ;
		goto	errExit	;
	}

	parent	= item->getParent() ;
	while (parent != 0)
	{
		if (parent->isFramer   () != 0)
		{	navi	= parent->isFramer   () ;
			break	;
		}
		if (parent->isFormBlock() != 0)
		{	navi	= parent->isFormBlock() ;
			break	;
		}
		parent	= parent->getParent() ;
	}

	if (navi == 0)
	{	error	= TR("Failed to find navigation object for control") ;
		goto	errExit	;
	}

	{
	QStringList	bits	= QStringList::split (":", m_args[3]) ;
	QKeyEvent	key	(QEvent::KeyRelease, bits[0].toInt(), bits[1].toInt(), bits[2].toInt()) ;
	navi->keyStroke (item, &key) ;
	}

	return	true	;

	errExit	:
		return	testFailed
			(	pError,
				TR("Key navigation"),
				error,
				QString("%1:%2").arg(m_args[0]).arg(m_args[2])
			)	;
#else
		return	true	;
#endif
}

/*  ------------------------------------------------------------------	*/

/*  KBMacroMouseNavigation						*/
/*  KBMacroMouseNavigation						*/
/* 		: Macro to handle (keystroke) navigation		*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroMouseNavigation::KBMacroMouseNavigation
	(	KBMacroExec	*exec
	)
	:
	KBMacroTest (exec, "MouseNavigation")
{
}

/*  KBMacroMouseNavigation						*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroMouseNavigation::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 3, 3, pError) ;
}

/*  KBMacroMouseNavigation						*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroMouseNavigation::execute
	(	KBError		&pError
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBMacroMouseNavigation::execute: path=[%s] name=[%s] drow=[%s]\n",
		m_args[0].latin1(),
		m_args[1].latin1(),
		m_args[2].latin1()
	))	;

	QString		error	;
	KBItem		*item	;
	KBControl	*ctrl	;

	if ((item = getItem (error)) == 0)
		goto	errExit	;

	ctrl	= ctrlAtDRow(item, m_args[2].toInt()) ;
	if (ctrl == 0)
	{	error	= QString(TR("No control at display row %1")).arg(m_args[2].toInt()) ;
		goto	errExit	;
	}

	/* TRICKY CODE							*/
	/* -----------							*/
	/* It would be nice to use "setFocus()" but this is fraught	*/
	/* with problems (see places in the testing code) since the	*/
	/* form under test might not have focus. So force the issue	*/
	/* by fabricating the appopriate event. We should get away with	*/
	/* this since the "UpdateValue" instruction corresponds to	*/
	/* focus leaving the prior control, and will trigger "onLeave"	*/
	/* when it executes.						*/
	item->focusInEvent (m_args[2].toInt(), QFocusEvent::Other) ;

	return	true ;

	errExit	:
		return	testFailed
			(	pError,
				TR("Mouse navigation"),
				error,
				QString("%1:%2").arg(m_args[0]).arg(m_args[2])
			)	;
#else
		return	true	;
#endif
}

/*  ------------------------------------------------------------------	*/

/*  KBMacroMouseClick							*/
/*  KBMacroMouseClick							*/
/* 		: Macro to handle (keystroke) navigation		*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroMouseClick::KBMacroMouseClick
	(	KBMacroExec	*exec
	)
	:
	KBMacroTest (exec, "MouseClick")
{
}

/*  KBMacroMouseClick							*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroMouseClick::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 4, 4, pError) ;
}

/*  KBMacroMouseClick							*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroMouseClick::execute
	(	KBError		&pError
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBMacroMouseClick::execute: path=[%s] name=[%s] drow=[%s] link=[%s]\n",
		m_args[0].latin1(),
		m_args[1].latin1(),
		m_args[2].latin1(),
		m_args[3].latin1()
	))	;

	QString		error	;
	KBObject	*object	;
	KBItem		*item	;
	KBControl	*ctrl	;

	if ((object = getObject (error)) == 0)
		goto	errExit	;

	if (object->isA("KBButton"))
	{
		((KBButton *)object)->clicked () ;
		return	true	;
	}

	item	= object->isItem() ;

	if (object->isA("KBRichText"))
	{
		ctrl	= ctrlAtDRow(item, m_args[2].toInt()) ;
		if (ctrl == 0)
		{	error	= QString(TR("No control at display row %1")).arg(m_args[2].toInt()) ;
			goto	errExit	;
		}

		((KBRichText *)object)->linkClicked
		(	item->getBlock()->getCurDRow() + ctrl->getDRow(),
			m_args[3]
		)	;

		return	true	;
	}

	error	= QString(TR("Object %1 not handled by MouseClick")).arg(object->getElement()) ;
	goto	errExit	;

	errExit	:
		return	testFailed
			(	pError,
				TR("Mouse click"),
				error,
				QString("%1:%2").arg(m_args[0]).arg(m_args[2])
			)	;
#else
		return	true	;
#endif
}

/*  ------------------------------------------------------------------	*/

/*  KBMacroPopupResult							*/
/*  KBMacroPopupResult							*/
/* 		: Macro to handle popup dialog results			*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroPopupResult::KBMacroPopupResult
	(	KBMacroExec	*exec
	)
	:
	KBMacroTest (exec, "PopupResult")
{
}

/*  KBMacroPopupResult							*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroPopupResult::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 3, 3, pError) ;
}

/*  KBMacroPopupResult							*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroPopupResult::execute
	(	KBError		&pError
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBMacroPopupResult::execute: t=%s r=%s v=[%s]\n",
		m_args[0].latin1(),
		m_args[1].latin1(),
		m_args[2].latin1()
	))	;

	/* It is always an error if this instruction is explicitely	*/
	/* executed, since if a popup had appeared then the instruction	*/
	/* would have been skipped over in KBMacroExec::nextPopupResult	*/
	return	testFailed
		(	pError,
			TR("Popup result"  ),
			TR("Popup expected"),
			QString::null
		)	;
#else
	return	true	;
#endif
}

/*  ------------------------------------------------------------------  */

/*  KBMacroRaisePage							*/
/*  KBMacroRaisePage							*/
/* 		: Macro to handle tabber page raising			*/
/*  exec	: KBMacroExec *	   : Macro executor			*/
/*  (returns)	: KBMacroPromptBox :					*/

KBMacroRaisePage::KBMacroRaisePage
	(	KBMacroExec	*exec
	)
	:
	KBMacroTest (exec, "RaisePage")
{
}

/*  KBMacroRaisePage							*/
/*  init	: Initialise macro instruction				*/
/*  args	: const QStringList &	: Macro arguments		*/
/*  comment	: const QString &	: Macro comment			*/
/*  pError	: KBError &		: Error return			*/
/*  (returns)	: bool			: Success			*/

bool	KBMacroRaisePage::init
	(	const QStringList	&args,
		const QString		&comment,
		KBError			&pError
	)
{
	return	KBMacroInstr::init (args, comment, 3, 3, pError) ;
}

/*  KBMacroRaisePage							*/
/*  execute	: Execute instruction					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBMacroRaisePage::execute
	(	KBError		&pError
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBMacroRaisePage::execute: path=[%s] name=[%s] page=[%s]\n",
		m_args[0].latin1(),
		m_args[1].latin1(),
		m_args[2].latin1()
	))	;

	QString		error	;
	KBObject	*object	;

	if ((object = getObject (error)) == 0)
		goto	errExit	;

	if (object->isTabber() == 0)
	{
		error	= QString(TR("Expected tabber, got %2")).arg(object->getElement()) ;
		goto	errExit	;
	}

	LITER
	(	KBNode,
		object->getChildren(),
		child,

		if (child->getName() == m_args[2])
			if (child->isTabberPage())
			{
				object->isTabber()->setCurrentPage(child->isTabberPage()) ;
				return	true	;
			}
	)

	error	= QString(TR("Failed to find tabber page %1").arg(m_args[2])) ; ;

	errExit	:
		return	testFailed
			(	pError,
				TR("Raise page"),
				error,
				m_args[0]
			)	;
#else
		return	true	;
#endif
}

/*  ------------------------------------------------------------------	*/


MACROREG(VerifyValue	)
MACROREG(VerifyRegexp	)
MACROREG(VerifyChoices	)
MACROREG(VerifyText	)
MACROREG(VerifyState	)
MACROREG(VerifyStack	)
MACROREG(UpdateValue	)
MACROREG(KeyNavigation	)
MACROREG(MouseNavigation)
MACROREG(MouseClick	)
MACROREG(PopupResult	)
MACROREG(RaisePage	)
