/***************************************************************************
    file	         : kb_macrotest.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#ifndef	__KB_MACROTEST_H
#define	__KB_MACROTEST_H

#include	"kb_macro.h"
#include	"libextra_exports.h"

class		KBControl	;
class		KBObject	;
class		KBItem		;


/*  KBMacroTest								*/
/*  -----------								*/
/*  Base class for test/verification macros				*/

class	LIBEXTRA_API	KBMacroTest : public KBMacroInstr
{
protected :

	KBObject	*getObject	(QString &)	;
	KBItem		*getItem	(QString &)	;
	KBValue		getValue	()		;
	KBControl	*ctrlAtDRow	(KBItem  *, uint) ;
	KBControl	*control	(KBObject *)	;

	QString		emptyToNull	(const QString &) ;
	bool		testFailed	(KBError &, const QString &, const QString &, const QString &) ;

	virtual	bool	isFixable	()		;
	virtual void	fix		()		;

public	:

	KBMacroTest	(KBMacroExec *, cchar *)	;
}	;

/*  KBMacroVerifyValue							*/
/*  ------------------							*/
/*  Macro used to verify a value in a control				*/

class	LIBEXTRA_API	KBMacroVerifyValue : public KBMacroTest
{
	bool		m_fixable	;
	KBValue		m_value		;

public	:

	KBMacroVerifyValue	(KBMacroExec *) ;

	virtual	bool	init
		(	const QStringList	&,
			const QString		&,
			KBError			&
		)	;

	bool	execute	(KBError &) ;

	virtual	bool	isFixable	()	;
	virtual void	fix		()	;
}	;

/*  KBMacroVerifyRegexp							*/
/*  ------------------							*/
/*  Macro used to verify a value in a control using regular expression	*/

class	LIBEXTRA_API	KBMacroVerifyRegexp : public KBMacroTest
{
	bool		m_fixable	;
	QString		m_value		;

public	:

	KBMacroVerifyRegexp	(KBMacroExec *) ;

	virtual	bool	init
		(	const QStringList	&,
			const QString		&,
			KBError			&
		)	;

	bool	execute	(KBError &) ;

	virtual	bool	isFixable	()	;
	virtual void	fix		()	;
}	;

/*  KBMacroVerifyChoices						*/
/*  --------------------						*/
/*  Macro used to verify choices in controls (choices and links)	*/

class	LIBEXTRA_API	KBMacroVerifyChoices : public KBMacroTest
{
	bool		m_fixable	;
	QString		m_choices	;

public	:

	KBMacroVerifyChoices	(KBMacroExec *) ;

	virtual	bool	init
		(	const QStringList	&,
			const QString		&,
			KBError			&
		)	;

	bool	execute	(KBError &) ;

	virtual	bool	isFixable	()	;
	virtual void	fix		()	;
}	;

/*  KBMacroVerifyText							*/
/*  -----------------							*/
/*  Macro used to verifu text in buttons and labels			*/

class	LIBEXTRA_API	KBMacroVerifyText : public KBMacroTest
{
	bool		m_fixable	;
	QString		m_text		;

public	:

	KBMacroVerifyText	(KBMacroExec *) ;

	virtual	bool	init
		(	const QStringList	&,
			const QString		&,
			KBError			&
		)	;

	bool	execute	(KBError &) ;

	virtual	bool	isFixable	()	;
	virtual void	fix		()	;
}	;

/*  KBMacroVerifyState							*/
/*  ------------------							*/
/*  Macro used to verify control state					*/

class	LIBEXTRA_API	KBMacroVerifyState : public KBMacroTest
{
	bool		m_fixable	;
	bool		m_enabled	;
	bool		m_visible	;

public	:

	KBMacroVerifyState	(KBMacroExec *) ;

	virtual	bool	init
		(	const QStringList	&,
			const QString		&,
			KBError			&
		)	;

	bool	execute	(KBError &) ;

	virtual	bool	isFixable	()	;
	virtual void	fix		()	;
}	;

/*  KBMacroVerifyStack							*/
/*  ------------------							*/
/*  Macro used to verify stack/tabber state				*/

class	LIBEXTRA_API	KBMacroVerifyStack : public KBMacroTest
{
public	:

	KBMacroVerifyStack	(KBMacroExec *) ;

	virtual	bool	init
		(	const QStringList	&,
			const QString		&,
			KBError			&
		)	;

	bool	execute	(KBError &) ;
}	;

/*  KBMacroUpdateValue							*/
/*  ------------------							*/
/*  Macro used to update control values					*/

class	LIBEXTRA_API	KBMacroUpdateValue : public KBMacroTest
{
public	:

	KBMacroUpdateValue	(KBMacroExec *) ;

	virtual	bool	init
		(	const QStringList	&,
			const QString		&,
			KBError			&
		)	;

	bool	execute	(KBError &) ;
}	;

/*  KBMacroKeyNavigation						*/
/*  --------------------						*/
/*  Macro used to handle form (keystroke) navigation			*/

class	LIBEXTRA_API	KBMacroKeyNavigation : public KBMacroTest
{
public	:

	KBMacroKeyNavigation	(KBMacroExec *) ;

	virtual	bool	init
		(	const QStringList	&,
			const QString		&,
			KBError			&
		)	;

	bool	execute	(KBError &) ;
}	;

/*  KBMacroMouseNavigation						*/
/*  ----------------------						*/
/*  Macro used to handle form (mouse) navigation			*/

class	LIBEXTRA_API	KBMacroMouseNavigation : public KBMacroTest
{
public	:

	KBMacroMouseNavigation	(KBMacroExec *) ;

	virtual	bool	init
		(	const QStringList	&,
			const QString		&,
			KBError			&
		)	;

	bool	execute	(KBError &) ;
}	;

/*  KBMacroMouseClick							*/
/*  -----------------							*/
/*  Macro used to handle general mouse clicks				*/

class	LIBEXTRA_API	KBMacroMouseClick : public KBMacroTest
{
public	:

	KBMacroMouseClick	(KBMacroExec *) ;

	virtual	bool	init
		(	const QStringList	&,
			const QString		&,
			KBError			&
		)	;

	bool	execute	(KBError &) ;
}	;

/*  KBMacroPopupResult							*/
/*  ------------------							*/
/*  Macro used to handle popup dialog responses				*/

class	LIBEXTRA_API	KBMacroPopupResult : public KBMacroTest
{
public	:

	KBMacroPopupResult	(KBMacroExec *) ;

	virtual	bool	init
		(	const QStringList	&,
			const QString		&,
			KBError			&
		)	;

	bool	execute	(KBError &) ;
}	;

/*  KBMacroRaisePage							*/
/*  ------------------							*/
/*  Macro used to handle page raise in a tabber				*/

class	LIBEXTRA_API	KBMacroRaisePage : public KBMacroTest
{
public	:

	KBMacroRaisePage	(KBMacroExec *) ;

	virtual	bool	init
		(	const QStringList	&,
			const QString		&,
			KBError			&
		)	;

	bool	execute	(KBError &) ;
}	;

#endif	// __KB_MACROTEST_H
