/***************************************************************************
    file	         : kb_ascii.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qstring.h>
#include	<qcolor.h>
#include	<qsize.h>
#include	<qpoint.h>
#include	<qrect.h>
#include	<qdatetime.h>

#include	"kb_ascii.h"

QString	KBAscii::text
	(	const	QColor	&c
	)
{
	return	c.name() ;
}

QString	KBAscii::text
	(	const	QPoint	&p
	)
{
	return	QString().sprintf ("(%d,%d)", p.x(), p.y()) ;
}

QString	KBAscii::text
	(	const	QSize	&s
	)
{
	return	QString().sprintf ("(%d,%d)", s.width(), s.height()) ;
}

QString	KBAscii::text
	(	const	QRect	&r
	)
{
	return	QString().sprintf ("(%d,%d,%d,%d)", r.left(), r.top(), r.width(), r.height()) ;
}

QString	KBAscii::text
	(	const	QDate	&d
	)
{
	return	d.toString() ;
}

QString	KBAscii::text
	(	const	QTime	&t
	)
{
	return	t.toString() ;
}

QString	KBAscii::text
	(	cchar 		*data, 
		uint		len
	)
{
	QString	res	;
	uint	offset	;

	for (offset = 0 ; (offset < len) && (offset < 16) ; offset += 1)
		res += QString().sprintf("%02x ", data[offset] & 0xff) ;

	if (offset <= len) res += "...." ;
	return	res	;
}

QString	KBAscii::text
	(	QEvent::Type	et
	)
{
#if	! __KB_RUNTIME
	switch (et)
	{
		case QEvent::None			: return "None"			;
		case QEvent::Timer			: return "Timer"		;
		case QEvent::MouseButtonPress		: return "MouseButtonPress"	;
		case QEvent::MouseButtonRelease		: return "MouseButtonRelease"	;
		case QEvent::MouseButtonDblClick	: return "MouseButtonDblClick"	;
		case QEvent::MouseMove			: return "MouseMove"		;
		case QEvent::KeyPress			: return "KeyPress"		;
		case QEvent::KeyRelease			: return "KeyRelease"		;
		case QEvent::FocusIn			: return "FocusIn"		;
		case QEvent::FocusOut			: return "FocusOut"		;
		case QEvent::Enter			: return "Enter"		;
		case QEvent::Leave			: return "Leave"		;
		case QEvent::Paint			: return "Paint"		;
		case QEvent::Move			: return "Move"			;
		case QEvent::Resize			: return "Resize"		;
		case QEvent::Create			: return "Create"		;
		case QEvent::Destroy			: return "Destroy"		;
		case QEvent::Show			: return "Show"			;
		case QEvent::Hide			: return "Hide"			;
		case QEvent::Close			: return "Close"		;
		case QEvent::Quit			: return "Quit"			;
		case QEvent::Reparent			: return "Reparent"		;
		case QEvent::ShowMinimized		: return "ShowMinimized"	;
		case QEvent::ShowNormal			: return "ShowNormal"		;
		case QEvent::WindowActivate		: return "WindowActivate"	;
		case QEvent::WindowDeactivate		: return "WindowDeactivate"	;
		case QEvent::ShowToParent		: return "ShowToParent"		;
		case QEvent::HideToParent		: return "HideToParent"		;
		case QEvent::ShowMaximized		: return "ShowMaximized"	;
		case QEvent::ShowFullScreen		: return "ShowFullScreen"	;
		case QEvent::Accel			: return "Accel"		;
		case QEvent::Wheel			: return "Wheel"		;
		case QEvent::AccelAvailable		: return "AccelAvailable"	;
		case QEvent::CaptionChange		: return "CaptionChange"	;
		case QEvent::IconChange			: return "IconChange"		;
		case QEvent::ParentFontChange		: return "ParentFontChange"	;
		case QEvent::ApplicationFontChange	: return "ApplicationFontChange";
		case QEvent::ParentPaletteChange	: return "ParentPaletteChange"	;
		case QEvent::ApplicationPaletteChange	: return "ApplicationPaletteChange"
											;
		case QEvent::PaletteChange		: return "PaletteChange"	;
		case QEvent::Clipboard			: return "Clipboard"		;
		case QEvent::Speech			: return "Speech"		;
		case QEvent::SockAct			: return "SockAct"		;
		case QEvent::AccelOverride		: return "AccelOverride"	;
		case QEvent::DeferredDelete		: return "DeferredDelete"	;
		case QEvent::DragEnter			: return "DragEnter"		;
		case QEvent::DragMove			: return "DragMove"		;
		case QEvent::DragLeave			: return "DragLeave"		;
		case QEvent::Drop			: return "Drop"			;
		case QEvent::DragResponse		: return "DragResponse"		;
		case QEvent::ChildInserted		: return "ChildInserted"	;
		case QEvent::ChildRemoved		: return "ChildRemoved"		;
		case QEvent::LayoutHint			: return "LayoutHint"		;
		case QEvent::ShowWindowRequest		: return "ShowWindowRequest"	;
		case QEvent::WindowBlocked		: return "WindowBlocked"	;
		case QEvent::WindowUnblocked		: return "WindowUnblocked"	;
		case QEvent::ActivateControl		: return "ActivateControl"	;
		case QEvent::DeactivateControl		: return "DeactivateControl"	;
		case QEvent::ContextMenu		: return "ContextMenu"		;
		case QEvent::IMStart			: return "IMStart"		;
		case QEvent::IMCompose			: return "IMCompose"		;
		case QEvent::IMEnd			: return "IMEnd"		;
		case QEvent::Accessibility		: return "Accessibility"	;
		case QEvent::TabletMove			: return "TabletMove"		;
		case QEvent::LocaleChange		: return "LocaleChange"		;
		case QEvent::LanguageChange		: return "LanguageChange"	;
		case QEvent::LayoutDirectionChange	: return "LayoutDirectionChange";
		case QEvent::Style			: return "Style"		;
		case QEvent::TabletPress		: return "TabletPress"		;
		case QEvent::TabletRelease		: return "TabletRelease"	;
		case QEvent::OkRequest			: return "OkRequest"		;
		case QEvent::HelpRequest		: return "HelpRequest"		;
		case QEvent::WindowStateChange		: return "WindowStateChange"	;
		case QEvent::IconDrag			: return "IconDrag"		;
		default					: break	 ;
	}
#endif
	return	QString::number ((int)et) ;
}

