/***************************************************************************
    file	         : kb_attr.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_ATTR_H
#define	_KB_ATTR_H

#include 	"libkbase_exports.h"

#include	<stdio.h>
#include	<stdlib.h>

#include	<qstring.h>
#include	<qdict.h>
#include	<qlistview.h>

#include	"kb_classes.h"

/*  The first bunch of flags are set where an attribute only appears	*/
/*  in some circumstances. For instance, KNF_FORM means the attribute	*/
/*  only appears in a form (for instance, control morphing does not	*/
/*  apply to reports).							*/
/*									*/
#define	KNF_FLAGS	0x000007ff
#define	KNF_FORM	0x00000001	/* Node appears in form		*/
#define	KNF_QUERY	0x00000002	/* .... query			*/
#define	KNF_REPORT	0x00000004	/* .... report			*/
#define	KNF_STATIC	0x00000010	/* Static control		*/
#define	KNF_BLOCK	0x00000020	/* Block control		*/
#define	KNF_DATA	0x00000040	/* Data control			*/
#define	KNF_FRAMER	0x00000080	/* Control in framer only	*/
#define	KNF_EXTRA	0x00000100	/* Add to extra menu		*/

#define	KAF_FORM	KNF_FORM	/* Attribute appears in form	*/
#define	KAF_QUERY	KNF_QUERY	/* .... query			*/
#define	KAF_REPORT	KNF_REPORT	/* .... report			*/


/*  The next group control which group of controls that the attribute	*/
/*  appears amongst in the properties dialogs.				*/
/*									*/
#define	KAF_GRPMASK	0x000ff000
#define	KAF_GRPFORMAT	0x00001000	/* Formatting property		*/
#define	KAF_GRPDATA	0x00002000	/* Data property		*/
#define	KAF_GRPOTHER	0x00004000	/* Other property		*/
#define	KAF_GRPEVENT	0x00008000	/* Event property		*/
#define	KAF_GRPNOTES	0x00010000	/* Special, used for notes	*/
#define	KAF_GRPWEB	0x00020000	/* Used for web only		*/


/*  Following flags relate to displaying and updating attributes in the	*/
/*  property dialog.							*/
/*									*/
#define	KAF_PROPMASK	0xfffff800
#define	KAF_REQD	0x00000800	/* Value must be set		*/
#define	KAF_CLEAR	0x00100000	/* Enable clear button		*/
#define	KAF_EDITOR	0x00200000	/* Display editor		*/
#define	KAF_SINGLE	0x00400000	/* Display single line		*/
#define	KAF_COMMON	0x01000000	/* Display in common properties	*/
#define	KAF_HIDDEN	0x02000000	/* Attribute is always hidden	*/

/*  Dummy attributes are those that are constructed for use only inside	*/
/*  property dialogs (for instance "blktype") to provide the user with	*/
/*  a handle to change a setting that is not actually a property.	*/
/*  Synthetic attributes (which are always also dummies) are used where	*/
/*  child nodes (for instance, hidden and parameter nodes) can only	*/
/*  be accessed via the parent nodes properties dialog.			*/
/*									*/
#define	KAF_DUMMY	0x08000000	/* Dummy attribute		*/
#define	KAF_SYNTHETIC	0x0c000000	/* Synthetic attribute		*/

#define	KAF_LOOSE	0x10000000	/* Attribute not attached	*/
#define	KAF_EVCS	0x20000000	/* Client side RekallWeb event	*/
#define	KAF_USER	0x40000000	/* User attribute		*/
#define	KAF_CUSTOM	0x80000000	/* Custom flags set		*/


#define	KAF_NOTES	(KAF_GRPNOTES|KAF_EDITOR|KAF_CLEAR|KAF_CUSTOM)
#define	KAF_CONFIG	(KAF_GRPOTHER|KAF_CUSTOM|KAF_HIDDEN)
#define	KAF_SLOTS	(KAF_GRPEVENT|KAF_CUSTOM|KAF_HIDDEN|KAF_SYNTHETIC)


class	KBAttrItem	;
class	KBAttrDlg	;
class	KBPropDict	;
class	KBPropDictEntry	;

/*  KBAttr								*/
/*  ------								*/
/*  The KBAttr(ibute) class, and its derived classes are used to	*/
/*  automate attribute setting and output.				*/

class	LIBKBASE_API	KBAttr
{
	void		attach ()	;

public	:

	enum	AType
	{	Base,
		String,
		Int,
		UInt,
		Bool
	}	;

protected :

	KBNode		*m_owner	;

	AType		m_type		;
	QString		m_name		;
	KBPropDictEntry	*m_dictEntry	;

private	  :

	QString		m_value		;
	QString		m_saved		;

protected :

	uint		m_nFlags	;
	uint		m_order		;
	KB::ShowAs	m_showing	;

	KBNodeMonitor	*m_monitor	;
	cchar		*m_ownerName	;

	QString		substitute	(const QString &) const ;

public	:

	KBAttr (KBNode *, AType, const QString&, const QDict<QString> &, uint = 0) ;
	KBAttr (KBNode *, AType, const QString&, const QString&, uint = 0) ;
	KBAttr (KBNode *, 	 const QString&, KBNode *, uint = 0) ;
	virtual	~KBAttr () ;

	virtual	bool		setExtraAttribs	(const QDict<QString> &);

	virtual	bool		showAs		(KB::ShowAs)	  ;
	virtual	void		printAttr 	(QString &, QString &, int, bool) ;
	virtual	void		setValue  	(const QString &) ;
	virtual	bool		valueOK	  	(const QString &) ;

	virtual	KBAttr		*replicate	(KBNode  * ) ;

	inline	AType		getType   	() const { return m_type  ; }
	inline	const QString	&getName        () const { return m_name  ; }

	virtual QString		getLegend       () const ;
	virtual QString		getDescription  () const ;
	virtual	QString		getHelpTag	() const ;
	virtual QString		getNullcheck	() const ;
	virtual	QString		getExtra	(const QString &) const ;

	virtual	bool		isHidden	() const ;

	uint			getFlags	() const ;
	uint			getOrder	() const ;
	virtual	QString 	getValue  	() const ;

#if	! __KB_RUNTIME
	virtual	KBAttrItem	*getAttrItem	() ;
	virtual	KBAttrDlg	*getAttrDlg
				(	QWidget			*,
					KBAttrItem		*,
					QDict<KBAttrItem>	&
				)	;
	virtual	QString		displayValue	(const QString &) ;
#endif
	virtual	KBEvent		*isEvent	() ;
	virtual	KBTest		*isTest		() ;

	void			showMonitor	(QListViewItem *) ;

	inline	void	setFlags
		(	uint	flags
		)
	{
		m_nFlags |= flags ;
	}
	inline	bool	required () const
	{
		return	(m_nFlags & KAF_REQD) != 0 ;
	}
	inline	cchar	*getOwnerName () const
	{
		return	m_ownerName ;
	}
	inline	KBNode	*getOwner () const
	{
		return	m_owner	  ;
	}

	KBPropDictEntry	*dictEntry	(const QString &) const ;
	KBPropDictEntry	*dictEntry	()		  const ;

	static	QString		escapeText	(const QString &, bool = true) ;
	static	void		addAttrText	(QString &, const QString &, const QString &, bool = false) ;
	static	void		addAttrText	(QString &, const QString &, int, 	      bool = false) ;
	static	QString		substitute	(const QString &, KBDocRoot *) ;

	static	KBPropDict	*getAttrDict	() ;
	static	int		getAttrValue	(const QDict<QString> &, const QString&, int) ;
	static	QString		getAttrValue	(const QDict<QString> &, const QString&, const QString &) ;
}	;

class	LIBKBASE_API	KBAttrStr : public KBAttr
{
public	:

	KBAttrStr (KBNode *, const QString&, const QDict<QString> &, uint = 0) ;
	KBAttrStr (KBNode *, const QString&, const QString&, uint = 0) ;
	KBAttrStr (KBNode *, const QString&, KBNode *, uint = 0) ;

	virtual	  ~KBAttrStr		() ;

	virtual	KBAttr	*replicate	(KBNode  *) ;
}	;

class	LIBKBASE_API	KBAttrInt : public KBAttr
{
public	:

	KBAttrInt (KBNode *, const QString&, const QDict<QString> &, uint = 0) ;
	KBAttrInt (KBNode *, const QString&, const QString&, uint = 0) ;
	KBAttrInt (KBNode *, const QString&, int     , uint = 0) ;
	KBAttrInt (KBNode *, const QString&, KBNode *, uint = 0) ;

	virtual	  ~KBAttrInt	    () ;

	virtual	void	setValue    (int) ;
	virtual	bool	valueOK	    (const QString &) ;
	virtual	KBAttr	*replicate  (KBNode  *) ;

	inline	int	getIntValue ()
	{
		return	getValue().isEmpty() ? 0 : getValue().toInt() ;
	}
}	;

class LIBKBASE_API	KBAttrUInt : public KBAttr
{
public	:

	KBAttrUInt (KBNode *, const QString&, const QDict<QString> &, uint = 0) ;
	KBAttrUInt (KBNode *, const QString&, const QString&, uint = 0) ;
	KBAttrUInt (KBNode *, const QString&, uint    , uint = 0) ;
	KBAttrUInt (KBNode *, const QString&, KBNode *, uint = 0) ;

	virtual	  ~KBAttrUInt		() ;

	virtual	void	setValue	(uint) ;
	virtual	bool	valueOK		(const QString &) ;
	virtual	KBAttr	*replicate	(KBNode  *) ;

	inline	int	getIntValue ()
	{
		return	getValue().isEmpty() ? 0 : getValue().toInt() ;
	}
}	;

class	LIBKBASE_API	KBAttrBool : public KBAttr
{
public	:

	KBAttrBool(KBNode *, const QString&, const QDict<QString> &, uint = 0) ;
	KBAttrBool(KBNode *, const QString&, const QString&, uint = 0) ;
	KBAttrBool(KBNode *, const QString&, const char   *, uint = 0) ;
	KBAttrBool(KBNode *, const QString&, bool    , uint = 0) ;
	KBAttrBool(KBNode *, const QString&, KBNode *, uint = 0) ;

	virtual	  ~KBAttrBool	    	() ;

	virtual	void	setValue    	(bool) ;
	virtual	KBAttr	*replicate	(KBNode  *) ;

	bool		getBoolValue	() ;
}	;




/*  KBATTRCTRDEF							*/
/*  ------------							*/
/*  These are convenience macros, used to generate a set of default	*/
/*  constructors for a derived attribute, where "type" is the derived	*/
/*  class name, and "base" is the base class name.			*/


#define	KBATTRCTRDEF(type,base)	\
	inline	type							\
		(	KBNode			*node,			\
			const QString		&name,			\
			const QDict<QString>	&aList,			\
			uint			flags = 0		\
		)							\
		:							\
		base	(node, name, aList, flags)			\
	{								\
	}								\
	inline	type							\
		(	KBNode			*node,			\
			const QString		&name,			\
			cchar			*value,			\
			uint			flags = 0		\
		)							\
		:							\
		base	(node, name, value, flags)			\
	{								\
	}								\
	inline	type							\
		(	KBNode			*node,			\
			const QString		&name,			\
			KBNode			*extant,		\
			uint			flags = 0		\
		)							\
		:							\
		base	(node, name, extant, flags)			\
	{								\
	}								\
									\
	int	dummy	()	;

/*  KBATTRCTRINT							*/
/*  ------------							*/
/*  As above, but generates a constructor where the attribute is	*/
/*  derived from a KBAttrInt-derived class.				*/


#define	KBATTRINTDEF(type,base)	\
	inline	type							\
		(	KBNode			*node,			\
			const QString		&name,			\
			int			value,			\
			uint			flags = 0		\
		)							\
		:							\
		base	(node, name, value, flags)			\
	{								\
	}

/*  KBATTRDLGDEF							*/
/*  ------------							*/
/*  Related macro used to generate the "getAttrDlg" definition in the	*/
/*  class definition. In runtime it expands to nothing.			*/

#if	__KB_RUNTIME
#define	KBATTRDLGDEF
#else
#define	KBATTRDLGDEF		\
	virtual	KBAttrDlg	*getAttrDlg				\
				(	QWidget			*,	\
					KBAttrItem		*,	\
					QDict<KBAttrItem>	&	\
				)	;
#endif

#endif	// _KB_ATTR_H
