/***************************************************************************
    file	         : kb_attrdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_attrdlg.moc"
#else 
#include 	"kb_attrdlg.h"
#endif

#include	"kb_attr.h"
#include	"rk_vbox.h"

/*  KBAttrDlg								*/
/*  KBAttrDlg	: Constructor for attribute dialog base			*/
/*  parent	: QWidget *		: Parent widget			*/
/*  attr	: KBAttr *		: Associated attribute		*/
/*  item	: KBAttrItem *		: Item being edited		*/
/*  attrDict	: QDict<KBAttrItem> &	: Distcionary of all attributes	*/
/*  (returns)	: KBAttrDlg		:				*/

KBAttrDlg::KBAttrDlg
	(	QWidget			*parent,
		KBAttr			*attr,
		KBAttrItem		*item,
		QDict<KBAttrItem>	&attrDict
	)
	:
	QObject		(parent),
	m_attr		(attr),
	m_item		(item),
	m_attrDict	(attrDict)
{
}

/*  KBAttrDlg								*/
/*  ~KBAttrDlg	: Destructor for attribute dialog base			*/
/*  (returns)	:		:					*/

KBAttrDlg::~KBAttrDlg ()
{
	DELOBJ	(m_topWidget) ;
}

/*  KBAttrDlg								*/
/*  getAttrVal	: Get value of named attribute				*/
/*  name	: const QString &	: Name of attribute		*/
/*  (returns)	: QString		: Value				*/

QString	KBAttrDlg::getAttrVal
	(	const QString	&name
	)
{
	/* Note that this returns the current value in the context of	*/
	/* the property editor, which will be different from the node's	*/
	/* attribute if the user has edited it.				*/
	KBAttrItem *item   = m_attrDict.find (name) ;
	return	item == 0 ? QString::null : item->value() ;
}

void	KBAttrDlg::verify ()
{
}

bool	KBAttrDlg::init ()
{
	return	init (m_item == 0 ? QString::null : m_item->value()) ;
}

void	KBAttrDlg::save ()
{
	if (m_item != 0) m_item->setValue (value()) ;
}

QString	KBAttrDlg::displayValue ()
{
	return	m_attr->displayValue (value()) ;
}

/*  ------------------------------------------------------------------  */

KBAttrDlgDialog::KBAttrDlgDialog
	(	KBAttrDlg	*attrDlg,
		const QString	&caption
	)
	:
	KBDialog	(caption, true),
	m_attrDlg	(attrDlg)
{
	RKVBox	*layMain = new RKVBox (this) ;
	layMain->setTracking () ;

	attrDlg->topWidget()->reparent (layMain, QPoint(0,0)) ;
	addOKCancel (layMain) ;
}

KBAttrDlgDialog::~KBAttrDlgDialog ()
{
	m_attrDlg->topWidget()->reparent (0, QPoint(0,0)) ;
}

void	KBAttrDlgDialog::accept ()
{
	m_attrDlg->save () ;
	done	(1) ;
}

void	KBAttrDlgDialog::reject ()
{
	done	(0) ;
}

int	KBAttrDlgDialog::exec ()
{
	return	KBDialog::exec() ;
}

