/***************************************************************************
    file	         : kb_attrdlg.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_ATTRDLG_H
#define	__KB_ATTRDLG_H


#include	"kb_classes.h"
#include	"kb_attritem.h"
#include	"kb_dialog.h"


/*  KBAttrDlg								*/
/*  ---------								*/
/*  This is an abstract base class for property dialogs (strinctly,	*/
/*  plugins to dialogs such as the main properties dialog) used when	*/
/*  editing a property.							*/

class	KBAttrDlg : public QObject
{
	Q_OBJECT

	QWidget			*m_topWidget	;

signals	:

	void			setDescription	(const QString &) ;

protected :

	KBAttr			*m_attr		;
	KBAttrItem		*m_item		;
	QDict<KBAttrItem>	&m_attrDict	;

	QString			getAttrVal	(const QString &) ;

public	:

	KBAttrDlg
	(	QWidget			*,
		KBAttr			*,
		KBAttrItem		*,
		QDict<KBAttrItem>	&
	)	;
	virtual	~KBAttrDlg	()		;

	virtual	void		verify		() ;
	virtual	bool		init		() ;
	virtual	void		save		() ;

	virtual	QString		value		() = 0 ;
	virtual	bool		init		(const QString &) = 0 ;

	QString			displayValue	() ;

	inline	void	setTopWidget
		(	QWidget	*widget
		)
	{
		m_topWidget = widget ;
	}
	inline	QWidget	*topWidget	()
	{
		return	m_topWidget	;
	}
}	;

/*  KBAttrDlgDialog							*/
/*  ---------------							*/
/*  Wrapper dialog so that KBAttrDlg derived classes can actually run	*/
/*  as real dialogs.							*/

class	KBAttrDlgDialog : public KBDialog
{
	Q_OBJECT

	KBAttrDlg	*m_attrDlg	;

protected slots :

	void		accept	()	;
	void		reject	()	;

public	:

	KBAttrDlgDialog	(KBAttrDlg *, const QString &) 	;
	~KBAttrDlgDialog()		;

	int		exec	()	;

	inline	bool	init
		(	const QString	&value
		)
	{
		return	m_attrDlg->init  (value) ;
	}
	inline	QString	value	()
	{
		return	m_attrDlg->value 	() ;
	}
	inline	QString	displayValue	()
	{
		return	m_attrDlg->displayValue () ;
	}
	inline	void	verify	()
	{
		return	m_attrDlg->verify	() ;
	}
}	;

#endif	// __KB_ATTRDLG_H
