/***************************************************************************
    file	         : kb_attrgeom.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_ATTRGEOM_H
#define	__KB_ATTRGEOM_H

#include	"libkbase_exports.h"
#include	"kb_attr.h"

#if		! __KB_RUNTIME
#include	"kb_attritem.h"
#endif


class	QFont	;

class	LIBKBASE_API	KBGridSetup
{
public	:

	int		m_spacing	;
	int		m_stretch	;

	KBGridSetup
		(	int	spacing,
			int	stretch
		)
		:
		m_spacing (spacing),
		m_stretch (stretch)
	{
	}

	KBGridSetup ()
	{
		m_spacing	= 0	;
		m_stretch	= 0	;
	}
}	;

typedef	QValueList<KBGridSetup>	KBGridSetupList	;



/*  KBAttrGeom								*/
/*  -----------								*/
/*  This class handles the geometry related attributes in a single	*/
/*  class.								*/

class	LIBKBASE_API	KBAttrGeom : public KBAttr
{
public	:

	/*  FloatMode							*/
	/*  These are used in "static" geometry mode to specify how	*/
	/*  the object resizes as its parent is resized.		*/
	enum	FloatMode
	{
		FMFixed			= 0,
		FMFloat			= 1,
		FMStretch		= 2,
		FMNoChange		= 0x80000000
	}	;

	/*  FloatMode							*/
	/*  Used to specify how child geometry is managed, either	*/
	/*  "static" (the original Rekall method) or "dynamic" using	*/
	/*  QT's layout engine.						*/
	enum	MgmtMode
	{
		MgmtUnknown		= 0,
		MgmtStatic		= 1,
		MgmtDynamic		= 2
	}	;

	/*  OverFlow							*/
	/*  Currently used only in RekallWeb. Allows containers which	*/
	/*  have a virtual size that may be larger than their real	*/
	/*  size, and are scrolled.					*/
	enum	OverFlow
	{	OverFlowOff		= 0,
		OverFlowAuto		= 1
	}	;

	/*  Mask							*/
	/*  The mask bits are used to control hiding of the separate	*/
	/*  settings according to circumstances.			*/
	enum	Mask
	{	HideX			= 0x0001,
		HideY			= 0x0002,
		HideW			= 0x0004,
		HideH			= 0x0008,
		HideXMode		= 0x0010,
		HideYMode		= 0x0020,
		HideMgmt		= 0x0040,
		HideRange		= 0x0080,
		HideAll			= 0x00ff
	}	;

	/*  Values are bundled up inside a KBAttrGeomValues structure	*/
	/*  so that they can easily be accessed en-masse.		*/
	struct	KBAttrGeomValues
	{
		int			m_x		;
		int			m_y		;
		int			m_w		;
		int			m_h		;

		int			m_minw		;
		int			m_maxw		;
		int			m_minh		;
		int			m_maxh		;
		bool			m_byChars	;

		FloatMode		m_xmode		;
		FloatMode		m_ymode		;

		Qt::AlignmentFlags	m_align		;

		MgmtMode		m_manage	;
		int			m_nRows		;
		int			m_nCols		;
		int			m_margin	;
		int			m_spacing	;

		uint			m_mask		;
		OverFlow		m_overflow	;
	}	;

	/*  XYNoChange can be passed to the set(x,y,w,h) method to	*/
	/*  indicate that the corresponding value should be left alone.	*/
	static	const	int XYNoChange	= 0x80000000 ;


private	:

	KBObject		*m_owner	;
	KBAttrGeomValues	m_d		;

	QSize			m_ps		;

	/*  When specifying minimum and maximum sizes in terms of	*/
	/*  characters, we cache the character width and height else	*/
	/*  calculations would get seriously time-consuming.		*/
	uint			m_charWidth	;
	uint			m_charHeight	;


	/* Values set then the geometry is associated with a grid	*/
	/* layout, specifying spacing and stretch values.		*/
	KBGridSetupList		m_rowSetup	;
	KBGridSetupList		m_colSetup	;


	void		setupRowColSetup()		;

	uint		hidden 		() const	;
	uint		fontWidth	() const	;
	uint		fontHeight	() const	;

public	:

	KBAttrGeom
	(	KBObject		*,
		const QDict<QString>	&,
		uint			= 0
	)	;
	KBAttrGeom
	(	KBObject		*,
		int			,
		int			,
		int			,
		int			,
		FloatMode		,
		FloatMode		,
		uint			= 0
	)	;
	KBAttrGeom
	(	KBObject		*,
		MgmtMode		,
		uint			,
		uint			,
		uint			= 0
	)	;
	KBAttrGeom
	(	KBObject		*,
		KBObject		*
	)	;
	~KBAttrGeom	() ;

	inline	int		x		() const { return m_d.m_x	; }
	inline	int		y		() const { return m_d.m_y	; }
	inline	int		w		() const { return m_d.m_w	; }
	inline	int		h		() const { return m_d.m_h	; }

	inline	int		minw		() const { return m_d.m_minw	; }
	inline	int		maxw		() const { return m_d.m_maxw	; }
	inline	int		minh		() const { return m_d.m_minh	; }
	inline	int		maxh		() const { return m_d.m_maxh	; }
	inline	bool		byChars		() const { return m_d.m_byChars	; }
	inline	FloatMode	xmode		() const { return m_d.m_xmode	; }
	inline	FloatMode	ymode		() const { return m_d.m_ymode	; }
	inline	Qt::AlignmentFlags
				alignment	() const { return m_d.m_align	; }

	inline	MgmtMode	manage		() const { return m_d.m_manage  ; }
	uint			numRows		(bool) const 	;
	uint			numCols		(bool) const	;
	inline	int		margin		() const { return m_d.m_margin  ; }
	inline	int		spacing		() const { return m_d.m_spacing ; }

	void			insertRow	(uint)		;
	void			insertCol	(uint)		;
	void			removeRow	(uint)		;
	void			removeCol	(uint)		;
	void			extendGrid	(uint, uint)	;

	uint			mask		() const	;
	OverFlow		overflow	() const	;

	QRect			geometry	(bool = false)	;
	QRect			geometry	(const QPoint &, bool = false) ;

	uint			minimumWidth	() const	;
	uint			maximumWidth	() const	;
	uint			minimumHeight	() const	;
	uint			maximumHeight	() const	;

	void			setMask		(uint)		;
	void			set		(int, int, int, int  )	;
	void			set		(FloatMode, FloatMode)	;
	void			set		(MgmtMode, uint, uint)	;
	void			set		(MgmtMode, uint, uint, int, int) ;
	void			setLimits	(int, int, int, int, bool)  ;
	void			setAlignment	(Qt::AlignmentFlags  )	;

	void			setGeometry	(const QRect &) ;
	void			setFont		(const QFont &)	;

	void			setOverFlow	(OverFlow)	;

	QRect			convGeometry	(int, int, int, int, bool) const ;

	virtual	QString 	getValue  	() const ;
	virtual	bool		isHidden	() const ;
	virtual	void		printAttr 	(QString &, QString &, int, bool) ;

	void			setRowSetup	(uint, int, int) ;
	void			setColSetup	(uint, int, int) ;

	const KBGridSetupList	&rowSetup ()	const	{ return m_rowSetup ; }
	const KBGridSetupList	&colSetup ()	const	{ return m_colSetup ; }
				
	QRect			mapGeometry	(const QRect &, const QRect &) const	;
	QRect			mapGeometry	(const QRect &		     ) const	;

	void			saveParentSize	(const QSize &) ;

	virtual	bool		setExtraAttribs	(const QDict<QString> &);

#if	! __KB_RUNTIME
	virtual	KBAttrItem	*getAttrItem	() ;
#endif

	static	QString		asText		(const KBAttrGeomValues &, uint) ;
}	;

#endif	// __KB_ATTRGEOM_H
