/***************************************************************************
    file	         : kb_attrgeomdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qspinbox.h>
#include	<qcheckbox.h>
#include	<qlabel.h>

#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"

#include	"kb_classes.h"
#include	"kb_attr.h"
#include	"kb_attrgeom.h"
#include	"kb_object.h"
#include	"kb_display.h"


#if		__KB_AUTOMOC
#include 	"kb_attrgeomdlg.moc"
#else
#include 	"kb_attrgeomdlg.h"
#endif


#define		__DEBUG_ATTRGEOMDLG	(0)
#if		__DEBUG_ATTRGEOMDLG
#include	"kb_printf.h"
#endif

#define		STATIC_HIDE	\
		(	KBAttrGeom::HideX	|KBAttrGeom::HideY	|	\
			KBAttrGeom::HideW	|KBAttrGeom::HideH	|	\
			KBAttrGeom::HideXMode	|KBAttrGeom::HideYMode		\
		)


KBAttrGeomItem::KBAttrGeomItem
	(	KBAttrGeom	*attrGeom
	)
	:
	KBAttrItem	(attrGeom),
	m_attrGeom	(attrGeom)
{
	KBNode	*pnode	= m_attr->getOwner()->getParent() ;
	if (pnode != 0)
		m_parent = pnode->isObject() ;
	else	m_parent = 0 ;

	QRect	r	= convGeometry
			  (	m_attrGeom->x(),
				m_attrGeom->y(),
				m_attrGeom->w(),
				m_attrGeom->h(),
				m_attrGeom->xmode(),
				m_attrGeom->ymode()
			  )	;

	m_d.m_x		= r.x	  () ;
	m_d.m_y		= r.y	  () ;
	m_d.m_w		= r.width () ;
	m_d.m_h		= r.height() ;
	m_d.m_xmode	= m_attrGeom->xmode	() ;
	m_d.m_ymode	= m_attrGeom->ymode	() ;

	m_d.m_minw	= m_attrGeom->minw	() ;
	m_d.m_maxw	= m_attrGeom->maxw	() ;
	m_d.m_minh	= m_attrGeom->minh	() ;
	m_d.m_maxh	= m_attrGeom->maxh	() ;
	m_d.m_byChars	= m_attrGeom->byChars	() ;

	m_d.m_align	= m_attrGeom->alignment	() ;

	m_d.m_manage	= m_attrGeom->manage	() ;
	m_d.m_nRows	= m_attrGeom->numRows	(true) ;
	m_d.m_nCols	= m_attrGeom->numCols	(true) ;
	m_d.m_margin	= m_attrGeom->margin	() ;
	m_d.m_spacing	= m_attrGeom->spacing	() ;

	m_d.m_mask	= m_attrGeom->mask	() ;
	m_d.m_overflow	= m_attrGeom->overflow	() ;
}

KBAttrGeomItem::~KBAttrGeomItem ()
{
}

QRect	KBAttrGeomItem::convGeometry
	(	int			x,
		int			y,
		int			w,
		int			h,
		KBAttrGeom::FloatMode	fmx,
		KBAttrGeom::FloatMode	fmy
	)
{
	/* The geometry is converted only if (a) our owning object has	*/
	/* a parent and (b) said parent is not dynamic.			*/
	if ((m_parent != 0) && !m_parent->isDynamic())
	{
		/* Get the parent geometry. If the parent is displayed	*/
		/* then use the screen geometry, otherwise fall back on	*/
		/* its attribute geometry.				*/
		QSize	ps ;
		if (m_parent->getContainer() == 0)
			ps	= m_parent->sizeHint     () ;
		else	ps	= m_parent->getContainer()->getTopSize() ;

		switch (fmx)
		{
			case KBAttrGeom::FMFloat   :
				x = ps.width () - x	;
				break	;

			case KBAttrGeom::FMStretch :
				w = ps.width () - x - w ;
				break	;

			default		:
				break	;
		}
		switch (fmy)
		{
			case KBAttrGeom::FMFloat   :
				y = ps.height() - y	;
				break	;

			case KBAttrGeom::FMStretch :
				h = ps.height() - y - h ;
				break	;

			default		:
				break	;
		}
	}

	return	QRect (x, y, w, h) ;
}

/*  KBAttrGeomItem							*/
/*  getAttrDlg	: Get geometry dialog					*/
/*  parent	: parent	      : Parent widget			*/
/*  attrDict	: QDict<KBAttrItem> & : Attribute disctionary		*/
/*  (returns)	: KBAttrDlg *	      : Dialog				*/

KBAttrDlg
	*KBAttrGeomItem::getAttrDlg
	(	QWidget			*parent,
		QDict<KBAttrItem>	&attrDict
	)
{
	return	new KBAttrGeomDlg (parent, m_attrGeom, this, attrDict) ;
}

void	KBAttrGeomItem::clear ()
{
}

void	KBAttrGeomItem::save ()
{
	QRect	r	= convGeometry
			  (	m_d.m_x,
				m_d.m_y,
				m_d.m_w,
				m_d.m_h,
				m_d.m_xmode,
				m_d.m_ymode
			  )	;

	m_attrGeom->set 	(r.x(), r.y(), r.width(), r.height()) ;
	m_attrGeom->set 	(m_d.m_xmode,  m_d.m_ymode) ;
	m_attrGeom->set		(m_d.m_manage, m_d.m_nRows, m_d.m_nCols, m_d.m_margin, m_d.m_spacing) ;
	m_attrGeom->setLimits	(m_d.m_minw,   m_d.m_maxw,  m_d.m_minh,  m_d.m_maxh,   m_d.m_byChars) ;
	m_attrGeom->setAlignment(m_d.m_align   )  ;
	m_attrGeom->setOverFlow	(m_d.m_overflow) ;
}

QString	KBAttrGeomItem::displayValue ()
{
	return	KBAttrGeom::asText (m_d, m_attrGeom->mask()) ;
}



/*  KBAttrGeomDlg							*/
/*  KBAttrGeomDlg							*/
/*		: Constructor for geometry dialog			*/
/*  parent	: QWidget *		: Parent widget			*/
/*  attr	: KBAttr *		: Associated attribute		*/
/*  item	: KBAttrGeomItem *	: Associated item		*/
/*  attrDict	: QDict<KBAttrItem> &	: Attribute dictionary		*/
/*  (returns)	: KBAttrGeomDlg		:				*/

KBAttrGeomDlg::KBAttrGeomDlg
	(	QWidget			*parent,
		KBAttrGeom		*attr,
		KBAttrGeomItem		*item,
		QDict<KBAttrItem>	&attrDict
	)
	:
	KBAttrDlg	(parent, attr, item, attrDict),
	m_attrItem	(item)
{
	RKHBox		*layMain = new RKHBox	(parent	   ) ;
	RKGridBox	*layGeom = new RKGridBox(2, layMain) ;
	RKGridBox	*layMgmt = new RKGridBox(2, layMain) ;
	KBNode		*node	 = m_attr->getOwner()	     ;
	KBObject	*object	 = node  ->isObject()	     ;
	setTopWidget    (layMain) ;

	m_pid	= object->parentIsDynamic() ;
	m_mask	= attr  ->mask () ;

	m_locked= false	;
	const QPtrList<KBNode> &cList = node->getChildren() ;
	LITER
	(	KBNode,
		cList,
		child,
		if (child->isObject())
		{	m_locked = true	;
			break	 ;
		}
	)

	/* The main setup difference relates to whether the parent	*/
	/* container is dynamic or static ...				*/
	if (m_pid)
	{
		m_x	  = 0	;
		m_y	  = 0	;
		m_w	  = 0	;
		m_h	  = 0	;
		m_xmode	  = 0	;
		m_ymode	  = 0	;

		m_halign  = makeComboBox (0, TR("Horizontal"), layGeom) ;
		m_valign  = makeComboBox (0, TR("Vertical"  ), layGeom) ;

		m_halign->insertItem (TR("Auto"  )) ;
		m_halign->insertItem (TR("Left"  )) ;
		m_halign->insertItem (TR("Centre")) ;
		m_halign->insertItem (TR("Right" )) ;

		m_valign->insertItem (TR("Auto"  )) ;
		m_valign->insertItem (TR("Top"   )) ;
		m_valign->insertItem (TR("Centre")) ;
		m_valign->insertItem (TR("Bottom")) ;

		m_minw	  = makeSpinBox  (0, TR("Min Width" ), layGeom, 0, 5000) ;
		m_maxw	  = makeSpinBox  (0, TR("Max Width" ), layGeom, 0, 5000) ;
		m_minh	  = makeSpinBox  (0, TR("Min Height"), layGeom, 0, 5000) ;
		m_maxh	  = makeSpinBox  (0, TR("Max Height"), layGeom, 0, 5000) ;

		new QLabel (TR("Size by characters"), layGeom) ;
		m_byChars = new QCheckBox(layGeom) ;
	}
	else
	{
		m_x	  = makeSpinBox  (KBAttrGeom::HideX,	 TR("&X"     ), layGeom, 0, 5000) ;
		m_y	  = makeSpinBox  (KBAttrGeom::HideY,	 TR("&Y"     ), layGeom, 0, 5000) ;
		m_w	  = makeSpinBox  (KBAttrGeom::HideW,	 TR("Wi&dth" ), layGeom, 0, 5000) ;
		m_h	  = makeSpinBox  (KBAttrGeom::HideH,	 TR("Hei&ght"), layGeom, 0, 5000) ;
		m_xmode	  = makeComboBox (KBAttrGeom::HideXMode, TR("X Mode" ), layGeom) ;
		m_ymode	  = makeComboBox (KBAttrGeom::HideYMode, TR("Y Mode" ), layGeom) ;

		if (m_xmode != 0)
		{
			m_xmode->insertItem (TR("Fixed"  )) ;
			m_xmode->insertItem (TR("Float"  )) ;
			m_xmode->insertItem (TR("Stretch")) ;
		}
		if (m_ymode != 0)
		{
			m_ymode->insertItem (TR("Fixed"  )) ;
			m_ymode->insertItem (TR("Float"  )) ;
			m_ymode->insertItem (TR("Stretch")) ;
		}

		if ((m_mask & STATIC_HIDE) == 0)
			if (object->getParent() != 0)
			{
				new QLabel (QString::null, layGeom) ;
				m_fill	= new RKPushButton (TR("Fill parent"), layGeom) ;
				connect	(m_fill, SIGNAL(clicked()), SLOT(fillParent())) ;
			}

		m_minw	  = 0	;
		m_maxw	  = 0	;
		m_minh	  = 0	;
		m_maxh	  = 0	;
		m_byChars = 0	;
		m_halign  = 0	;
		m_valign  = 0	;
	}

	if ((m_mask & KBAttrGeom::HideMgmt) == 0)
	{
		/* If the parent is dynamic, we want to stop the user	*/
		/* specifying a row or column count that is too small	*/
		/* to encompass any children.				*/
		int	mincols	= 0	;
		int	minrows	= 0	;

		if (m_attrItem->m_d.m_manage == KBAttrGeom::MgmtDynamic)
		{
			TITER
			(
				Object,
				node->getChildren(),
				o,

				QRect g = o->geometry() ;
				if (g.right () > mincols) mincols = g.right () ;
				if (g.bottom() > minrows) minrows = g.bottom() ;
			)

			DPRINTF
			((	"KBAttrGeomDlg::KBAttrGeomDlg: parent: min: %d,%d\n",
				minrows,
				mincols
			))	;

		}

		m_manage	= makeComboBox (0, TR("Management"), layMgmt) ;
		m_manage->insertItem (TR("Static" )) ;
		m_manage->insertItem (TR("Dynamic")) ;

		m_nRows		= makeSpinBox (0,  TR("Rows"),	  layMgmt, minrows + 1, 100) ;
		m_nCols		= makeSpinBox (0,  TR("Columns"), layMgmt, mincols + 1, 100) ;
		m_margin	= makeSpinBox (0,  TR("Margin"),  layMgmt, -1, 100) ;
		m_spacing	= makeSpinBox (0,  TR("Spacing"), layMgmt, -1, 100) ;


		if (m_locked)
		{
			m_manage ->setEnabled (false) ;
		}
		else
			connect
			(	m_manage,
				SIGNAL	(activated (int)),
				SLOT	(manageChanged())
			)	;
	}
	else
	{
		m_manage	= 0 ;
		m_nRows		= 0 ;
		m_nCols		= 0 ;
		m_margin	= 0 ;
		m_spacing	= 0 ;
	}

	if
	(	(node->isBlock    () != 0) ||
		(node->isContainer() != 0) ||
		(node->isCompLink () != 0)
		
	)
	{
		m_overflow	= makeComboBox (0, TR("Overflow"), layGeom) ;
		m_overflow->insertItem (TR("Off" )) ;
		m_overflow->insertItem (TR("Auto")) ;
	}
	else	m_overflow	= 0 ;

	layGeom->addFillerRow () ;
	layMgmt->addFillerRow () ;
}

QSpinBox
	*KBAttrGeomDlg::makeSpinBox
	(	uint		mask,
		const QString	&legend,
		RKGridBox	*layout,
		int		min,
		int		max
	)
{
	if ((m_mask & mask) == 0)
	{
		QLabel   *l = new QLabel   (legend, layout) ;
		QSpinBox *s = new QSpinBox (min, max, 1, layout) ;

		if (legend.find('&') >= 0) l->setBuddy (s) ;

		return	s ;
	}

	return	0 ;
}


RKComboBox
	*KBAttrGeomDlg::makeComboBox
	(	uint		mask,
		const QString	&legend,
		RKGridBox	*layout
	)
{
	if ((m_mask & mask) == 0)
	{
		new QLabel	(legend, layout) ;
		return	new RKComboBox  (layout) ;
	}

	return	0 ;
}


/*  KBAttrGeomDlg							*/
/*  init	: Initialise for editing				*/
/*  value	: const QString & : Value				*/
/*  (returns)	: bool		  : Always false, no modal dialog	*/

bool	KBAttrGeomDlg::init
	(	const QString	&value
	)
{
	return	false	;
}

/*  KBAttrGeomDlg							*/
/*  value	: Get value						*/
/*  (returns)	: QString	: Value					*/

QString	KBAttrGeomDlg::value ()
{
	return	QString("") ;
}

/*  KBAttrGeomDlg							*/
/*  init	: Initialise for editing				*/
/*  (returns)	: bool		: True if modal dialog executed		*/

bool	KBAttrGeomDlg::init()
{
	if (m_x       != 0) m_x     ->setValue	     (m_attrItem->m_d.m_x) ;
	if (m_y       != 0) m_y     ->setValue	     (m_attrItem->m_d.m_y) ;
	if (m_w       != 0) m_w     ->setValue	     (m_attrItem->m_d.m_w) ;
	if (m_h	      != 0) m_h     ->setValue	     (m_attrItem->m_d.m_h) ;
	if (m_xmode   != 0) m_xmode ->setCurrentItem (m_attrItem->m_d.m_xmode) ;
	if (m_ymode   != 0) m_ymode ->setCurrentItem (m_attrItem->m_d.m_ymode) ;

	if (m_byChars != 0)
	{
		m_minw	 ->setValue   (m_attrItem->m_d.m_minw) ;
		m_maxw	 ->setValue   (m_attrItem->m_d.m_maxw) ;
		m_minh	 ->setValue   (m_attrItem->m_d.m_minh) ;
		m_maxh	 ->setValue   (m_attrItem->m_d.m_maxh) ;
		m_byChars->setChecked (m_attrItem->m_d.m_byChars) ;

		switch (m_attrItem->m_d.m_align & Qt::AlignHorizontal_Mask)
		{
			case Qt::AlignLeft    :
				m_halign->setCurrentItem (1) ;
				break	;

			case Qt::AlignHCenter :
				m_halign->setCurrentItem (2) ;
				break	;

			case Qt::AlignRight   :
				m_halign->setCurrentItem (3) ;
				break	;

			default	:
				m_halign->setCurrentItem (0) ;
				break	;
		}
		switch (m_attrItem->m_d.m_align & Qt::AlignVertical_Mask)
		{
			case Qt::AlignTop     :
				m_valign->setCurrentItem (1) ;
				break	;

			case Qt::AlignVCenter :
				m_valign->setCurrentItem (2) ;
				break	;

			case Qt::AlignBottom  :
				m_valign->setCurrentItem (3) ;
				break	;

			default	:
				m_valign->setCurrentItem (0) ;
				break	;
		}
	}

	if (m_manage  != 0)
	{
		m_manage ->setCurrentItem (m_attrItem->m_d.m_manage == KBAttrGeom::MgmtDynamic? 1 : 0) ;
		m_nRows  ->setValue	  (m_attrItem->m_d.m_nRows  ) ;
		m_nCols  ->setValue	  (m_attrItem->m_d.m_nCols  ) ;
		m_margin ->setValue	  (m_attrItem->m_d.m_margin ) ;
		m_spacing->setValue	  (m_attrItem->m_d.m_spacing) ;

		if (!m_locked) manageChanged () ;
	}

	if (m_overflow != 0)
		m_overflow->setCurrentItem (m_attrItem->m_d.m_overflow) ;

	return	false	 ;
}

/*  KBAttrGeomDlg							*/
/*  saveSpinBox	: Save spinbox value if spinbox present			*/
/*  spinBox	: QSpinBox *	: Putative spinbox			*/
/*  value	: int &		: Result location			*/
/*  (returns)	: void		:					*/

void	KBAttrGeomDlg::saveSpinBox
	(	QSpinBox	*spinBox,
		int		&value
	)
{
	if (spinBox != 0) value = spinBox->value() ;
}

/*  KBAttrGeomDlg							*/
/*  save	: Save values back to associated attribute item		*/
/*  (returns)	: bool		: True if modal dialog executed		*/

void	KBAttrGeomDlg::save ()
{
	saveSpinBox (m_x,	m_attrItem->m_d.m_x   ) ;
	saveSpinBox (m_y,	m_attrItem->m_d.m_y   ) ;
	saveSpinBox (m_w,	m_attrItem->m_d.m_w   ) ;
	saveSpinBox (m_h,	m_attrItem->m_d.m_h   ) ;

	if (m_xmode  != 0)
		m_attrItem->m_d.m_xmode = (KBAttrGeom::FloatMode)m_xmode->currentItem() ;
	if (m_ymode  != 0)
		m_attrItem->m_d.m_ymode = (KBAttrGeom::FloatMode)m_ymode->currentItem() ;

	saveSpinBox (m_minw,	m_attrItem->m_d.m_minw) ;
	saveSpinBox (m_maxw,	m_attrItem->m_d.m_maxw) ;
	saveSpinBox (m_minh,	m_attrItem->m_d.m_minh) ;
	saveSpinBox (m_maxh,	m_attrItem->m_d.m_maxh) ;

	if (m_manage != 0)
		m_attrItem->m_d.m_manage   = m_manage->currentItem() != 0 ?
						KBAttrGeom::MgmtDynamic :
						KBAttrGeom::MgmtStatic  ;
	if (m_overflow != 0)
		m_attrItem->m_d.m_overflow = (KBAttrGeom::OverFlow)m_overflow->currentItem () ;

	if (m_byChars != 0)
	{
		m_attrItem->m_d.m_byChars = m_byChars->isChecked () ;

		static	int	hmap[] =
		{	Qt::AlignAuto,
			Qt::AlignLeft,
			Qt::AlignHCenter,
			Qt::AlignRight
		}	;
		static	int	vmap[] =
		{	Qt::AlignAuto,
			Qt::AlignTop,
			Qt::AlignVCenter,
			Qt::AlignBottom
		}	;

		m_attrItem->m_d.m_align   =
			(Qt::AlignmentFlags)
			(	hmap[m_halign->currentItem()] |
				vmap[m_valign->currentItem()]
			)	;
	}

	saveSpinBox (m_nRows,	m_attrItem->m_d.m_nRows  ) ;
	saveSpinBox (m_nCols,	m_attrItem->m_d.m_nCols  ) ;
	saveSpinBox (m_margin,  m_attrItem->m_d.m_margin ) ;
	saveSpinBox (m_spacing, m_attrItem->m_d.m_spacing) ;
}

/*  KBAttrGeomDlg							*/
/*  manageChanged: Management mode changed				*/
/*  (returns)	 : void		:					*/


void	KBAttrGeomDlg::manageChanged ()
{
	bool	enable	= m_manage->currentItem() == 1 ;

	m_nRows  ->setEnabled (enable) ;
	m_nCols  ->setEnabled (enable) ;
	m_margin ->setEnabled (enable) ;
	m_spacing->setEnabled (enable) ;
}

/*  KBAttrGeomDlg							*/
/*  fillParent	: Expand object to fill parent				*/
/*  (returns)	: void		:					*/

void	KBAttrGeomDlg::fillParent ()
{
	/* The object must have a parent else this slot would not have	*/
	/* been wired up, so the folling won't bomb!			*/
	QRect	rect	= m_attr->getOwner()->getParent()->isObject()->geometry() ;

	m_x	  ->setValue	   (0) ;
	m_y	  ->setValue	   (0) ;
	m_w	  ->setValue	   (rect.width ()) ;
	m_h	  ->setValue	   (rect.height()) ;
	m_xmode	  ->setCurrentItem (2) ;
	m_ymode	  ->setCurrentItem (2) ;
}
