/***************************************************************************
    file	         : kb_attrhelperdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_lineedit.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"

#include	"kb_classes.h"
#include	"kb_attr.h"
#include	"kb_node.h"
#include	"kb_docroot.h"
#include	"kb_dbinfo.h"
#include	"kb_dbdociter.h"
#include	"kb_helper.h"

#if		__KB_AUTOMOC
#include 	"kb_attrhelperdlg.moc"
#else
#include 	"kb_attrhelperdlg.h"
#endif


/*  KBAttrHelperDlg							*/
/*  KBAttrHelperDlg							*/
/*		: Constructor for helper dialog				*/
/*  parent	: QWidget *		: Parent widget			*/
/*  attr	: KBAttr *		: Associated attribute		*/
/*  item	: KBAttrItem *		: Associated item		*/
/*  attrDict	: QDict<KBAttrItem> &	: Attribute dictionary		*/	
/*  (returns)	: KBAttrHelperDlg:					*/

KBAttrHelperDlg::KBAttrHelperDlg
	(	QWidget			*parent,
		KBAttr			*attr,
		KBAttrItem		*item,
		QDict<KBAttrItem>	&attrDict
	)
	:
	KBAttrImageBaseDlg (parent, attr, item, attrDict)
{
	RKVBox	*layMain = new RKVBox	(parent ) ;
	setTopWidget (layMain) ;

	RKHBox	*layHelp = new RKHBox	    (layMain) ;
	m_helper	 = new RKComboBox   (layHelp) ;
	m_showme	 = new RKPushButton (TR("Show ..."), layHelp) ;

	extern	QStringList *getHelperSet() ;
	m_helper->clear			 () ;
	m_helper->insertItem		 ("") ;
	m_helper->insertStringList	 (*getHelperSet()) ;

	static	const char *legends [] =
	{	"Normal",
		"Rollover",
		"Disable"
	}	;

	makeSelectors	(layMain, 3, legends) ;

	layMain->addFiller () ;

	connect
	(	m_helper,
		SIGNAL	(activated	(int)),
		SLOT	(helperChanged	())
	)	;
	connect
	(	m_showme,
		SIGNAL	(clicked	()),
		SLOT	(showMe		())
	)	;
}

/*  KBAttrHelperDlg							*/
/*  init	: Initialise for editing				*/
/*  value	: const QString & : Value				*/
/*  (returns)	: bool		  : Always false, no modal dialog	*/

bool	KBAttrHelperDlg::init
	(	const QString	&value
	)
{
	QStringList hbits = QStringList::split (';', value) ;
	while (hbits.size() < m_count + 1) hbits.append (QString::null) ;

	for (int idx = 0 ; idx < m_helper->count() ; idx += 1)
		if (m_helper->text(idx) == hbits[0])
		{	m_helper->setCurrentItem (idx) ;
			break	;
		}

	loadImageList() ;

	for (uint idx1 = 0 ; idx1 < m_count ; idx1 += 1)
	{
		QString	name = hbits[idx1 + 1] ;
		m_images.at(idx1)->setText	  (name) ;
		m_lists .at(idx1)->setCurrentText (name) ;
	}

	helperChanged()	;
	return	false	;
}

void	KBAttrHelperDlg::helperChanged ()
{
	m_showme->setEnabled (m_helper->currentItem() > 0) ;
}

void	KBAttrHelperDlg::showMe ()
{
	if (m_helper->currentItem() > 0)
		KBHelperDlg::run
		(	m_helper->currentText(),
			QString::null,
			KBLocation()
		)	;
}


/*  KBAttrHelperDlg							*/
/*  value	: Get value						*/
/*  (returns)	: void		:					*/

QString	KBAttrHelperDlg::value ()
{
	QStringList list ;

	list.append (m_helper->currentText()) ;

	for (uint idx = 0 ; idx < m_count ; idx += 1)
		list.append (m_images.at(idx)->text()) ;

	return	list.join (";") ;
}

