/***************************************************************************
    file	         : kb_attrserverdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	<qvaluelist.h>

#include	"rk_vbox.h"
#include	"rk_combobox.h"

#include	"kb_classes.h"
#include	"kb_attr.h"
#include	"kb_node.h"
#include	"kb_docroot.h"
#include	"kb_dbinfo.h"
#include	"kb_desktop.h"
#include	"kb_locator.h"

#if		__KB_AUTOMOC
#include 	"kb_attrlanguagedlg.moc"
#else
#include 	"kb_attrlanguagedlg.h"
#endif


class	KBAttrLanguageMap
{
public	:

	QString		m_display	;
	QString		m_language	;

	inline	KBAttrLanguageMap
		(	const QString	&display,
			const QString	&language
		)
		:
		m_display	(display),
		m_language	(language)
	{
	}
	inline	KBAttrLanguageMap ()
	{
	}
}	;


/*  getAttrLanguageMap							*/
/*		: Get the lanage map					*/
/*  (returns)	:		*					*/

static	QValueList<KBAttrLanguageMap>	*getAttrLanguageMap ()
{
	/* This map is loaded on demand from the scripting desktop	*/
	/* files.							*/
	static	QValueList<KBAttrLanguageMap> *map	;

	if (map == 0)
	{
		QString	 	dtDir	= locateDir
					  (	"appdata",
						"services/rekall_dummy.desktop"
					  )	;
		QPtrList<KBDesktop> dtDefs	  ;
		KBDesktop::scan (dtDir + "/services", "rekall_", dtDefs) ;

		map = new QValueList<KBAttrLanguageMap> ;
		map->append (KBAttrLanguageMap(QString::null, QString::null)) ;

		for (uint idx = 0 ; idx < dtDefs.count() ; idx += 1)
		{
			KBDesktop *desktop = dtDefs.at(idx) ;
			if (desktop->property("ServiceTypes") != "Rekall/Script")
				continue ;

			QString	language = desktop->property("X-KDE-RekallPart-Language"   ) ;
			QString	display  = desktop->property("X-KDE-RekallPart-DisplayName") ;

			if (display.isEmpty()) display = language ;

			map->append (KBAttrLanguageMap(display, language)) ;
		}
	}

	return	map	;

}

/*  ------------------------------------------------------------------	*/

/*  KBAttrLanguageDlg							*/
/*  KBAttrLanguageDlg							*/
/*		: Constructor for language selection dialog		*/
/*  parent	: QWidget *		: Parent widget			*/
/*  attr	: KBAttr *		: Associated attribute		*/
/*  item	: KBAttrItem *		: Associated item		*/
/*  attrDict	: QDict<KBAttrItem> &	: Attribute dictionary		*/	
/*  (returns)	: KBAttrLanguageDlg:					*/

KBAttrLanguageDlg::KBAttrLanguageDlg
	(	QWidget			*parent,
		KBAttr			*attr,
		KBAttrItem		*item,
		QDict<KBAttrItem>	&attrDict
	)
	:
	KBAttrDlg	(parent, attr, item, attrDict)
{
	RKVBox	*layout	= new RKVBox	(parent) ;
	setTopWidget (layout) ;

	m_language	= new RKComboBox(layout) ;
	layout->addFiller () ;

	QValueList<KBAttrLanguageMap> *map = getAttrLanguageMap() ;
	for (uint idx = 0 ; idx < map->count() ; idx += 1)
		m_language->insertItem ((*map)[idx].m_display) ;
}

/*  KBAttrLanguageDlg							*/
/*  init	: Initialise for editing				*/
/*  value	: const QString & : Value				*/
/*  (returns)	: bool		  : Always false, no modal dialog	*/

bool	KBAttrLanguageDlg::init
	(	const QString	&value
	)
{
	QValueList<KBAttrLanguageMap> *map = getAttrLanguageMap() ;
	for (uint idx = 0 ; idx < map->count() ; idx += 1)
		if ((*map)[idx].m_language == value)
		{	m_language->setCurrentItem (idx) ;
			break	;
		}

	return	false	;
}

/*  KBAttrLanguageDlg							*/
/*  value	: Get value						*/
/*  (returns)	: void		:					*/

QString	KBAttrLanguageDlg::value ()
{
	return	(*getAttrLanguageMap())[m_language->currentItem()].m_language ;
}

QString	KBAttrLanguageDlg::mapLanguageToDisplay
	(	const QString 	&language
	)
{
	QValueList<KBAttrLanguageMap> *map = getAttrLanguageMap() ;
	for (uint idx = 0 ; idx < map->count() ; idx += 1)
		if ((*map)[idx].m_language == language)
			return(*map)[idx].m_display ;

	return	language ;
}

