/***************************************************************************
    file	         : kb_attrmargindlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qlabel.h>
#include	<qspinbox.h>

#if		__KB_AUTOMOC
#include	"kb_attrmargindlg.moc"
#else
#include	"kb_attrmargindlg.h"
#endif

#include	"kb_attrmargin.h"

#include	"rk_hbox.h"
#include	"rk_gridbox.h"



/*  KBAttrMarginItem							*/
/*  KBAttrMarginItem							*/
/*		: Constructor for report margins for property dialog	*/
/*  attrMargin	: KBAttrMargin *   : Parent attribute			*/
/*  (returns)	: KBAttrMarginItem :					*/

KBAttrMarginItem::KBAttrMarginItem
	(	KBAttrMargin	*attrMargin
	)
	:
	KBAttrItem	(attrMargin),
	m_attrMargin	(attrMargin)
{
	m_lMargin	= m_attrMargin->m_lMargin ;
	m_rMargin	= m_attrMargin->m_rMargin ;
	m_tMargin	= m_attrMargin->m_tMargin ;
	m_bMargin	= m_attrMargin->m_bMargin ;
}

/*  KBAttrMarginItem							*/
/* ~ KBAttrMarginItem							*/
/*		: Destructor for report margins for property dialog	*/
/*  (returns)	:		:					*/

KBAttrMarginItem::~KBAttrMarginItem ()
{
}

/*  KBAttrMarginItem							*/
/*  getAttrDlg	: Get dialog for editing margin settings		*/
/*  parent	: QWidget *	      : Parent widget			*/
/*  attrDict	: QDict<KBAttrItem> & : Dictionary or other attributes	*/
/*  (returns)	: KBAttrDlg *	      :					*/

KBAttrDlg
	*KBAttrMarginItem::getAttrDlg
	(	QWidget			*parent,
		QDict<KBAttrItem>	&attrDict
	)
{
	return	new KBAttrMarginDlg (parent, m_attrMargin, this, attrDict) ;
}

/*  KBAttrMarginItem							*/
/*  clear	: Clear values						*/
/*  (returns)	: void		:					*/

void	KBAttrMarginItem::clear ()
{
	m_lMargin = 0 ;
	m_rMargin = 0 ;
	m_tMargin = 0 ;
	m_bMargin = 0 ;
}

/*  KBAttrMarginItem							*/
/*  save	: Save values back to attribute				*/
/*  (returns)	: void		:					*/

void	KBAttrMarginItem::save ()
{
	m_attrMargin->m_lMargin = m_lMargin ;
	m_attrMargin->m_rMargin = m_rMargin ;
	m_attrMargin->m_tMargin = m_tMargin ;
	m_attrMargin->m_bMargin = m_bMargin ;
}

/*  KBAttrMarginItem							*/
/*  displayValuue: Get text for display in preorty dialog		*/
/*  (returns)	 : QString	:					*/

QString	KBAttrMarginItem::displayValue ()
{
	return	QString ("%1,%3,%3,%4")
			.arg(m_lMargin)
			.arg(m_rMargin)
			.arg(m_tMargin)
			.arg(m_bMargin)
			;
}

/*  ------------------------------------------------------------------	*/

/*  KBAttrMarginDlg							*/
/*  KBAttrMarginDlg							*/
/*		: Constructor for margin dialog				*/
/*  parent	: QWidget *		: Parent widget			*/
/*  attr	: KBAttrMargin *	: Associated attribute		*/
/*  item	: KBAttrMarginItem *	: Associated item		*/
/*  attrDict	: QDict<KBAttrItem> &	: Attribute dictionary		*/
/*  (returns)	: KBAttrMarginDlg		:				*/

KBAttrMarginDlg::KBAttrMarginDlg
	(	QWidget			*parent,
		KBAttrMargin		*attr,
		KBAttrMarginItem	*item,
		QDict<KBAttrItem>	&attrDict
	)
	:
	KBAttrDlg	(parent, attr, item, attrDict),
	m_attrItem	(item)
{
	RKHBox		*layTop	 = new RKHBox    (parent) ;
	RKGridBox	*layGrid = new RKGridBox (2, layTop) ;
	layTop->addFiller () ;

	QLabel *l1	= new QLabel	(TR("Le&ft margin"  ), layGrid) ;
	m_lMargin	= new QSpinBox	(0, 1000, 1, layGrid) ;
	m_lMargin->setFixedWidth (100) ;
	l1->setBuddy (m_lMargin) ;

	QLabel *l2	= new QLabel	(TR("Ri&ght margin" ), layGrid) ;
	m_rMargin	= new QSpinBox	(0, 1000, 1, layGrid) ;
	m_rMargin->setFixedWidth (100) ;
	l2->setBuddy (m_rMargin) ;

	QLabel *l3	= new QLabel	(TR("To&p margin"   ), layGrid) ;
	m_tMargin	= new QSpinBox	(0, 1000, 1, layGrid) ;
	m_tMargin->setFixedWidth (100) ;
	l3->setBuddy (m_tMargin) ;

	QLabel *l4	= new QLabel	(TR("&Bottom margin"), layGrid) ;
	m_bMargin	= new QSpinBox	(0, 1000, 1, layGrid) ;
	m_bMargin->setFixedWidth (100) ;
	l4->setBuddy (m_bMargin) ;

	layGrid->addFillerRow () ;

	setTopWidget    (layTop) ;
}

/*  KBAttrMarginDlg							*/
/*  value	: Get value						*/
/*  (returns)	: QString	: Value					*/

QString	KBAttrMarginDlg::value ()
{
	return	QString("") ;
}

/*  KBAttrMarginDlg							*/
/*  init	: Initialise for editing				*/
/*  value	: const QString & : Value				*/
/*  (returns)	: bool		  : Always false, no modal dialog	*/

bool	KBAttrMarginDlg::init
	(	const QString	&value
	)
{
	return	false	;
}

/*  KBAttrMarginDlg							*/
/*  init	: Initialise for editing				*/
/*  (returns)	: bool		: True if modal dialog executed		*/

bool	KBAttrMarginDlg::init()
{
	m_lMargin->setValue (m_attrItem->m_lMargin) ;
	m_rMargin->setValue (m_attrItem->m_rMargin) ;
	m_tMargin->setValue (m_attrItem->m_tMargin) ;
	m_bMargin->setValue (m_attrItem->m_bMargin) ;
	return	false	;
}

/*  KBAttrMarginDlg							*/
/*  save	: Save values back to associated attribute item		*/
/*  (returns)	: bool		: True if modal dialog executed		*/

void	KBAttrMarginDlg::save ()
{
	m_attrItem->m_lMargin	= m_lMargin->value() ;
	m_attrItem->m_rMargin	= m_rMargin->value() ;
	m_attrItem->m_tMargin	= m_tMargin->value() ;
	m_attrItem->m_bMargin	= m_bMargin->value() ;
}
