/***************************************************************************
    file	         : kb_attroptlistdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<errno.h>
#include	<qimage.h>
#include	<qgroupbox.h>
#include	<qfileinfo.h>
#include	<qlabel.h>


#include	"rk_vbox.h"
#include	"rk_checkbox.h"

#include	"kb_classes.h"
#include	"kb_attr.h"
#include	"kb_attroptlist.h"

#if		__KB_AUTOMOC
#include 	"kb_attroptlistdlg.moc"
#else
#include 	"kb_attroptlistdlg.h"
#endif

#define		__DEBUG_ATTROPTLISTDLG	(0)
#if		__DEBUG_ATTROPTLISTDLG
#include	"kb_dprintf.h"
#endif

struct	OptionItem
{	const char	*m_name		;
	const char	*m_tip		;
}	;

static	OptionItem	optionItems[]	=
{
	{	"bold",		"Bold"			},
	{	"italic",	"Italic"		},
	{	"under",	"Underline"		},
	{	"font",		"Set font"		},
	{	"font-up",	"Increase font size"	},
	{	"font-down",	"Decrease font size"	},
	{	"color",	"Set font color"	},
	{	"text-left",	"Left align text"	},
	{	"text-right",	"Right align text"	},
	{	"text-center",	"Center text"		},
	{	0,		0,			}
}	;


/*  KBAttrOptlistDlg							*/
/*  KBAttrOptlistDlg							*/
/*		: Constructor for image selection/load dialog		*/
/*  parent	: QWidget *		: Parent widget			*/
/*  attr	: KBAttrOptlist *	: Associated attribute		*/
/*  item	: KBAttrItem *		: Associated item		*/
/*  attrDict	: QDict<KBAttrItem> &	: All attributes		*/
/*  (returns)	: KBImageDlg		:				*/

KBAttrOptlistDlg::KBAttrOptlistDlg
	(	QWidget			*parent,
		KBAttrOptlist		*attr,
		KBAttrItem		*item,
		QDict<KBAttrItem>	&attrDict
	)
	:
	KBAttrDlg (parent, attr, item, attrDict)
{
	RKVBox *layMain   = new RKVBox (parent) ;
	setTopWidget(layMain) ;

	for (OptionItem *oi = &optionItems[0] ; oi->m_name != 0 ; oi += 1)
	{
		RKCheckBox *cb = new RKCheckBox (layMain, oi->m_name) ;
		cb->setText (TR(oi->m_tip)) ;
		m_options.append (cb) ;		
	}

	layMain->addFiller () ;
}

/*  KBAttrOptlistDlg							*/
/*  init	: Initialise for editing				*/
/*  value	: const QString & : Value				*/
/*  (returns)	: bool  	  : Always false, no modal dialog	*/

bool	KBAttrOptlistDlg::init
	(	const QString	&value
	)
{
	QStringList options = QStringList::split(',', value) ;
	for (uint idx = 0 ; idx < m_options.count() ; idx += 1)
		m_options.at(idx)->setChecked (options.contains(m_options.at(idx)->name()) > 0) ;
	return	false	;
}

/*  KBAttrOptlistDlg							*/
/*  value	: Get value						*/
/*  (returns)	: void		:					*/

QString	KBAttrOptlistDlg::value ()
{
	QStringList options ;
	for (uint idx = 0 ; idx < m_options.count() ; idx += 1)
		if (m_options.at(idx)->isChecked())
			options.append (m_options.at(idx)->name()) ;
	return	options.join(",") ;
}

