/***************************************************************************
    file	         : kb_attrskindlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_lineedit.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"

#include	"kb_classes.h"
#include	"kb_attr.h"
#include	"kb_node.h"
#include	"kb_docroot.h"
#include	"kb_dbinfo.h"
#include	"kb_dbdociter.h"
#include	"kb_skin.h"
#include	"kb_skindlg.h"
#include	"kb_notifier.h"

#if		__KB_AUTOMOC
#include 	"kb_attrskindlg.moc"
#else
#include 	"kb_attrskindlg.h"
#endif


/*  KBAttrSkinDlg							*/
/*  KBAttrSkinDlg							*/
/*		: Constructor for skin dialog				*/
/*  parent	: QWidget *		: Parent widget			*/
/*  attr	: KBAttr *		: Associated attribute		*/
/*  item	: KBAttrItem *		: Associated item		*/
/*  attrDict	: QDict<KBAttrItem> &	: Attribute dictionary		*/	
/*  (returns)	: KBAttrSkinDlg:					*/

KBAttrSkinDlg::KBAttrSkinDlg
	(	QWidget			*parent,
		KBAttr			*attr,
		KBAttrItem		*item,
		QDict<KBAttrItem>	&attrDict
	)
	:
	KBAttrDlg	(parent, attr, item, attrDict)
{
	RKVBox	*layout	= new RKVBox	(parent) ;
	setTopWidget (layout) ;

	RKGridBox *grid = new RKGridBox (2, layout) ;

	KBNode		 *node	 = m_item->attr()->getOwner() ;
	const KBLocation &locn	= node->getDocRoot()->getDocLocation() ;
	if (!locn.getServerInfo()->skinSuffix().isEmpty())
	{
		new QLabel (TR("Suffix"), grid) ;
		RKLineEdit *suffix = new RKLineEdit (grid) ;
		suffix->setText		  (locn.getServerInfo()->skinSuffix()) ;
		suffix->setReadOnly	  (true) ;
		suffix->setBackgroundMode (Qt::PaletteMid) ;
	}

	new QLabel (TR("Skin"), grid) ;
	m_skin	= new RKComboBox(grid) ;
	m_skin->setEditable (true) ;

	RKHBox	*button = new RKHBox	(layout) ;
	button->addFiller () ;

	m_new	= new RKPushButton (TR("New" ), button) ;
	connect	(m_new,  SIGNAL(clicked ()), SLOT(slotNew ())) ;

	m_edit	= new RKPushButton (TR("Edit"), button) ;
	connect	(m_edit, SIGNAL(clicked ()), SLOT(slotEdit())) ;

	layout->addFiller () ;
	loadSkins	  () ;
}

/*  KBAttrSkinDlg							*/
/*  loadSkins	: Load skin names					*/
/*  (returns)	: void		:					*/

void	KBAttrSkinDlg::loadSkins ()
{
	KBNode		 *node	 = m_item->attr()->getOwner() ;
	const KBLocation &locn	 = node->getDocRoot()->getDocLocation() ;
	KBDBInfo	 *dbInfo = node->getDocRoot()->getDBInfo     () ;

	KBDBDocIter docIter (false) ;
	KBError	    error   ;
	QString	    name    ;
	QString	    stamp   ;

	if (!docIter.init
		(	dbInfo,
			locn.server(),
			"skin",
			"skn",
			error
		))
	{
		error.DISPLAY() ;
		return	;
	}

	QString	curSkin	= m_skin->currentText() ;

	m_skin->clear	   ()   ;
	m_skin->insertItem ("") ;
	while (docIter.getNextDoc (name, stamp))
		m_skin->insertItem (name) ;

	init	(curSkin) ;
}

/*  KBAttrSkinDlg							*/
/*  slotNew	: Edit new skin						*/
/*  (returns)	: void		:					*/

void	KBAttrSkinDlg::slotNew ()
{
	KBNode	  	 *node	= m_item->attr()->getOwner() ;
	const KBLocation &locn	= node->getDocRoot()->getDocLocation() ;

	KBLocation   skinLoc
		     (		locn.dbInfo(),
				"skin",
				locn.server(),
				QString::null,
				"skn"
		     )	  ;
	KBSkinDlg    (0, skinLoc, false, true).exec() ;
	loadSkins    ()	  ;
}

/*  KBAttrSkinDlg							*/
/*  slotEdit	: Edit currently selected skin				*/
/*  (returns)	: void		:					*/

void	KBAttrSkinDlg::slotEdit ()
{
	if (!m_skin->currentText().isEmpty())
	{
		KBNode	  	 *node	= m_item->attr()->getOwner()	    ;
		const KBLocation &locn	= node->getDocRoot()->getDocLocation() ;

		QString		 skin	= m_skin->currentText()		    ;
		if (!locn.getServerInfo()->skinSuffix().isEmpty())
			skin = skin + "_" + locn.getServerInfo()->skinSuffix() ;

		KBLocation   skinLoc
			     (		locn.dbInfo(),
					"skin",
					locn.server(),
					skin,
					"skn"
			     )	  ;

		if (!skinLoc.exists())
		{
			KBError::EError
			(	QString(TR("Skin '%1' does not exist")).arg(skin),
				QString::null,
				__ERRLOCN
			)	;
			return	;
		}

		KBSkinDlg    (0, skinLoc, true, true).exec() ;
		loadSkins    ()	  ;
	}
}

/*  KBAttrSkinDlg							*/
/*  init	: Initialise for editing				*/
/*  value	: const QString & : Value				*/
/*  (returns)	: bool		  : Always false, no modal dialog	*/

bool	KBAttrSkinDlg::init
	(	const QString	&value
	)
{
	m_skin->setCurrentText (value) ;
	return	false	;
}

/*  KBAttrSkinDlg							*/
/*  value	: Get value						*/
/*  (returns)	: void		:					*/

QString	KBAttrSkinDlg::value ()
{
	return	m_skin->currentText() ;
}

