/***************************************************************************
    file	         : kb_attrskinelemdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"
#include	"rk_lineedit.h"

#include	"kb_classes.h"
#include	"kb_attr.h"
#include	"kb_node.h"
#include	"kb_docroot.h"
#include	"kb_font.h"
#include	"kb_skin.h"
#include	"kb_skindlg.h"

#include	<qlabel.h>
#include	<qapplication.h>

#if		__KB_AUTOMOC
#include 	"kb_attrskinelemdlg.moc"
#else
#include 	"kb_attrskinelemdlg.h"
#endif


/*  KBAttrSkinElemDlg							*/
/*  KBAttrSkinElemDlg							*/
/*		: Constructor for skin element dialog			*/
/*  parent	: QWidget *		: Parent widget			*/
/*  attr	: KBAttr *		: Associated attribute		*/
/*  item	: KBAttrItem *		: Associated item		*/
/*  attrDict	: QDict<KBAttrItem> &	: Attribute dictionary		*/	
/*  (returns)	: KBAttrSkinElemDlg	:				*/

KBAttrSkinElemDlg::KBAttrSkinElemDlg
	(	QWidget			*parent,
		KBAttr			*attr,
		KBAttrItem		*item,
		QDict<KBAttrItem>	&attrDict
	)
	:
	KBAttrDlg	(parent, attr, item, attrDict)
{
	KBNode	  	*node	= m_item->attr()->getOwner() ;
	const KBLocation &locn	= node->getDocRoot()->getDocLocation() ;

	RKGridBox *layout	= new RKGridBox	(3, parent) ;
	setTopWidget (layout) ;

	new QLabel (TR("Skin"), layout) ;

	RKLineEdit *uses	= new RKLineEdit(layout) ;
	QString	   skin		= node->getRoot()->getAttrVal ("skin") ;
	if (!skin.isEmpty())
	{
		if (!locn.getServerInfo()->skinSuffix().isEmpty())
			skin = skin + "_" + locn.getServerInfo()->skinSuffix() ;
		uses->setText(skin) ;
	}
	else	uses->setText(TR("Document does not specify a skin")) ;
	uses->setReadOnly	(true) ;
	uses->setBackgroundMode (Qt::PaletteMid) ;

	if (!skin.isEmpty())
	{
		RKPushButton *ed = new RKPushButton (TR("Edit skin"), layout) ;
		connect (ed, SIGNAL(clicked()), SLOT(editSkin())) ;
	}
	else	new QWidget (layout) ;

	new QLabel	(TR("Element"),  layout) ;
	m_skinElem	= new RKComboBox(layout) ;
	new QWidget	(layout) ;

	connect
	(	m_skinElem,
		SIGNAL	(activated(int)),
		SLOT	(setSwatch())
	)	;

	new QLabel	(TR("Swatch"),	 layout) ;
	m_swatch	= new QLabel	(layout) ;
	m_swatch->setMinimumHeight (m_swatch->sizeHint().height() * 3) ;

	layout->addFillerRow () ;
	loadSkinElements     () ;
}

/*  KBAttrSkinElemDlg							*/
/*  loadSkinElements							*/
/*		: Load elements for selected skin			*/
/*  (returns)	: void		:					*/

void	KBAttrSkinElemDlg::loadSkinElements ()
{

	KBNode	*node	= m_item->attr()->getOwner() ;
	QDictIterator<KBSkinElement> eIter  (node->getDocRoot()->skinElements()) ;
	const KBSkinElement	     *elem  ;
	QStringList		     names  ;

	while ((elem = eIter.current()) != 0)
	{
		names.append (eIter.currentKey()) ;
		eIter += 1 ;
	}

	names.sort() ;
	m_skinElem->clear      	    ()   ;
	m_skinElem->insertItem 	    ("") ;
	m_skinElem->insertStringList(names, 1) ;
}

void	KBAttrSkinElemDlg::setSwatch ()
{
	KBNode	  	*node	= m_item ->attr()->getOwner () ;
	KBDocRoot	*root	= node->getRoot()->isDocRoot() ;
	QPalette	pal	= qApp->palette() ;
	QFont		font	= qApp->font   () ;
	QString		element	= m_skinElem->currentText() ;

	if (!element.isEmpty())
	{
		QString	fgstr	= root->skinFGColor(element) ;
		QString	bgstr	= root->skinBGColor(element) ;
		QString	fontstr	= root->skinFont   (element) ;

		if (!fgstr.isEmpty())
		{	QColor fg = QColor(QRgb(fgstr.toInt(0,0))) ;
			pal.setColor (QColorGroup::Text,       fg) ;
			pal.setColor (QColorGroup::ButtonText, fg) ;
			pal.setColor (QColorGroup::Foreground, fg) ;
		}
		if (!bgstr.isEmpty())
		{	QColor bg = QColor(QRgb(bgstr.toInt(0,0))) ;
			pal.setColor (QColorGroup::Base,       bg) ;
			pal.setColor (QColorGroup::Button,     bg) ;
			pal.setColor (QColorGroup::Background, bg) ;
		}
		if (!fontstr.isEmpty())
			font	= QFont(KBFont::specToFont(fontstr)) ;
	}

	m_swatch->setPalette (pal ) ;
	m_swatch->setFont    (font) ;
	m_swatch->setText    (TR("Sample")) ;
}

/*  KBAttrSkinElemDlg							*/
/*  editSkin	: Edit the document skin				*/
/*  (returns)	: void		:					*/

void	KBAttrSkinElemDlg::editSkin ()
{
	KBNode	  	 *node	= m_item->attr()->getOwner()	    ;
	const KBLocation &locn	= node->getDocRoot()->getDocLocation() ;

	QString		 skin	= node->getRoot()->getAttrVal("skin") ;
	if (!locn.getServerInfo()->skinSuffix().isEmpty())
		skin = skin + "_" + locn.getServerInfo()->skinSuffix() ;

	KBLocation   skinLoc
		     (		locn.dbInfo(),
				"skin",
				locn.server(),
				skin,
				"skn"
		     )	  ;

	if (!skinLoc.exists())
	{
		KBError::EError
		(	QString(TR("Skin '%1' does not exist")).arg(skin),
			QString::null,
			__ERRLOCN
		)	;
		return	;
	}

	KBSkinDlg    (0, skinLoc, true, true).exec() ;
	loadSkinElements () ;
}


/*  KBAttrSkinElemDlg							*/
/*  init	: Initialise for editing				*/
/*  value	: const QString & : Value				*/
/*  (returns)	: bool		  : Always false, no modal dialog	*/

bool	KBAttrSkinElemDlg::init
	(	const QString	&value
	)
{
	for (int idx = 0 ; idx < m_skinElem->count() ; idx += 1)
		if (m_skinElem->text(idx) == value)
		{	m_skinElem->setCurrentItem (idx) ;
			break	;
		}

	setSwatch ()	;
	return	false	;
}

/*  KBAttrSkinElemDlg							*/
/*  value	: Get value						*/
/*  (returns)	: void		:					*/

QString	KBAttrSkinElemDlg::value ()
{
	return	m_skinElem->currentText() ;
}

