/***************************************************************************
    file	         : kb_block.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#if		__KB_AUTOMOC
#include	"kb_block.moc"
#else
#include	"kb_block.h"
#endif

#include	"kb_location.h"
#include	"kb_formblock.h"
#include	"kb_rowmark.h"
#include	"kb_reportblock.h"
#include	"kb_docroot.h"
#include	"kb_framer.h"

#include	"kb_qrybase.h"
#include	"kb_qrytable.h"
#include	"kb_qryquery.h"
#include	"kb_qrynull.h"
#include	"kb_qrysql.h"

#include	"kb_query.h"
#include	"kb_table.h"
#include	"kb_qryexpr.h"

#include	"kb_hidden.h"
#include	"kb_display.h"
#include	"kb_options.h"
#include	"kb_nodereg.h"
#include	"kb_parse.h"

#include	"kb_writer.h"

#include	"kb_blockevents.h"


#define		__DEBUG_BLOCK	(0)
#if		__DEBUG_BLOCK
#include	"kb_printf.h"
#endif

/*  KBBlock								*/
/*  KBBlock	: Constructor for extant block element			*/
/*  parent	: KBNode *	: Parent element			*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  element	: cchar *	: Element name				*/
/*  (returns)	: KBBlock	:					*/

KBBlock::KBBlock
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		cchar			*element
	)
	:
	KBItem	   	(parent, element,	"master", aList),
	m_cexpr	   	(this,   "child",	aList,  KAF_REQD),
	m_bgcolor	(this,   "bgcolor",	aList),
	m_autosync	(this,   "autosync", 	aList,  KAF_FORM),
	m_title		(this,   "title",	aList,  KAF_FORM),
	m_frame 	(this,   "frame",	aList,  KAF_FORM),
	m_showbar	(this,   "showbar",	aList,  KAF_FORM),
	m_rowcount	(this,   "rowcount",	aList,	KAF_FORM),
	m_dx		(this,   "dx",		aList,	KAF_FORM),
	m_dy		(this,   "dy",		aList,	KAF_FORM)
{
	m_expr.setFlags (KAF_REQD) ;

	init	  () ;
	m_events  = new KBBlockEvents (this, aList) ;

	/* If the element us "KBSubBlock" then the block type is	*/
	/* definitely BTSubBlock. If not then we have to await the	*/
	/* query to figure it out.					*/
	m_blkType = getElement() == "KBFormSubBlock"   ? BTSubBlock :
		    getElement() == "KBReportSubBlock" ? BTSubBlock :
							 BTUnknown  ;

	m_topLevel = (getBlock() == 0) || (getBlock()->getBlkType() == BTNull) ;
}

/*  KBBlock								*/
/*  KBBlock	: Constructor for new block element			*/
/*  parent	: KBObject *	: Parent element			*/
/*  rect	: const QRect &	: Size and position			*/
/*  blkType	: BlkType	: Initial block type			*/
/*  ok		: bool &	: Success				*/
/*  element	: cchar *	: Element name				*/
/*  (returns)	: KBBlock	:					*/

KBBlock::KBBlock
	(	KBObject	*parent,
		const QRect	&rect,
		BlkType		blkType,
		bool		&ok,
		cchar		*element
	)
	:
	KBItem		(parent, element, rect, "master", "", 0),
	m_cexpr		(this,   "child",	"",	KAF_REQD),
	m_bgcolor	(this,   "bgcolor",	""),
	m_autosync	(this,   "autosync",   	true,	KAF_FORM),
	m_title		(this,   "title",	"",	KAF_FORM),
	m_frame		(this,   "frame",	"",	KAF_FORM),
	m_showbar	(this,   "showbar",	"No", 	KAF_FORM),
	m_rowcount	(this,   "rowcount",	0,	KAF_FORM),
	m_dx		(this,   "dx",	  	KBOptions::getDefaultDX(), KAF_FORM),
	m_dy		(this,   "dy",	  	KBOptions::getDefaultDY(), KAF_FORM)
{
	m_expr.setFlags (KAF_REQD) ;

	init	 () ;
	m_events = new KBBlockEvents (this) ;

#if	! __KB_RUNTIME
	/* If this is not a subblock then go through the full setup	*/
	/* procedure. If it is then just grab the query from our parent	*/
	/* which must be a block (or subblock).				*/
	if (blkType != BTSubBlock)
	{
		if (!setBlkType (blkType))
		{	ok	= false	;
			return	;
		}
	}
	else
#endif
	{
		m_blkType = BTSubBlock ;
		m_query	  = getBlock()->getQuery() ;
		return	;
	}

	m_topLevel = (getBlock() == 0) || (getBlock()->getBlkType() == BTNull) ;
	ok	   = true	    ;
}


/*  KBBlock	: Constructor for extant block element			*/
/*  parent	: KBNode *	: Parent element			*/
/*  block	: KBBlock *	: Extant block				*/
/*  (returns)	: KBBlock	:					*/

KBBlock::KBBlock
	(	KBNode		*parent,
		KBBlock		*block
	)
	:
	KBItem		(parent, "master",	block),
	m_cexpr		(this,    "child",	block, KAF_REQD),
	m_bgcolor	(this,    "bgcolor",	block),
	m_autosync	(this,    "autosync",	block, KAF_FORM),
	m_title		(this,    "title",	block, KAF_FORM),
	m_frame		(this,    "frame",	block, KAF_FORM),
	m_showbar	(this,    "showbar",	block, KAF_FORM),
	m_rowcount	(this,    "rowcount",	block, KAF_FORM),
	m_dx		(this,    "dx",	  	block, KAF_FORM),
	m_dy		(this,    "dy",	  	block, KAF_FORM)
{
	m_expr.setFlags (KAF_REQD) ;

	init	   () ;
	m_events   = new KBBlockEvents  (this, block) ;
	m_blkType  = block->getBlkType () ;

	m_topLevel = (getBlock() == 0) || (getBlock()->getBlkType() == BTNull) ;
}

/*  KBBlock								*/
/*  ~KBBlock	: Destructor for form-block element			*/
/*  (returns)	: void		:					*/

KBBlock::~KBBlock ()
{
	DPRINTF
	((	"KBBlock::~KBBlock: %p\n",
		(void *)this
	))	;

	tearDown() ;
	DELOBJ	(m_events ) ;
	DELOBJ	(m_blkDisp) ;
}

/*  KBBlock								*/
/*  init	: Common constructor initialisation			*/
/*  (returns)	: void		:					*/

void	KBBlock::init ()
{
	m_curQRow	= 0	;
	m_curDRow	= 0	;
	m_parentRows	= 0	;
	m_qryLvl	= 0	;
	m_header	= 0	;
	m_footer	= 0	;
	m_blkDisp	= 0	;
	m_blkType	= BTUnknown ;
	m_query		= 0	;
	m_numRows	= 0	;

	m_attribs.remove (&m_defVal	) ;
	m_attribs.remove (&m_onDblClick	) ;
	m_attribs.remove (&m_disabled   ) ;
	m_attribs.remove (&m_hidden     ) ;
	m_attribs.remove (&m_valFlag	) ;

	bool	hasL2	= !getRoot()->getAttrVal("language2").isEmpty() ;
	if (!hasL2) m_attribs.remove (&m_tabOrd) ;
}

/*  KBBlock								*/
/*  newSubBlocks: Build nested sub-blocks				*/
/*  (returns)	: bool		: Success				*/

bool	KBBlock::newSubBlocks ()
{
	/* We need to know the number of levels in the query, so locate	*/
	/* and parse it.						*/
	KBLocation location (getDocRoot()->getDBInfo  (),
			     "query",
			     getDocRoot()->getDocLocation().server(),
			     m_query->getAttrVal ("query")
			    ) ;

	KBError	 	error	;
	KBQuery	 	*qry	;
	QByteArray	doc	;

	if (!location.contents (doc, error))
		return	 false	;

	if ((qry = KBOpenQueryText (location, doc, error)) == 0)
	{
		setError (error) ;
		return	 false	 ;
	}

	QString			svName	 ;
	QPtrList<KBTable>		qryList	 ;
	QPtrList<KBTable>		tabList	 ;
	QPtrList<KBQryExpr>	exprList ;

	qry->getQueryInfo (svName, qryList, exprList) ;
	if (!KBTable::blockUp (qryList, m_query->getAttrVal("toptable"), tabList, error))
	{
		setError (error) ;
		return	 false	 ;
	}


	/* The first subb-lock will use just about half of the area of	*/
	/* this block. Any further subblocks will use successively	*/
	/* smaller areas.						*/
	uint	gridX	= KBOptions::getGridXStep () ;
	uint	gridY	= KBOptions::getGridYStep () ;
	QRect	r	= geometry () ;

	uint	xDelta	= isReportBlock() == 0 ? gridX : 0 ;
	uint	yRedux	= isReportBlock() == 0 ? 0 :
			  r.height() / 2 <= 120 ? 0 : 40 ;

	QRect	area
		(	xDelta,
			r.height() / 2,
			r.width () - 2 * xDelta,
			r.height() / 2 - gridY - yRedux
		)	;


	/* The subblocks are nested inside one-another. We need to note	*/
	/* the top-most subblock, so that we can subsequently pass	*/
	/* down the query level and the query.				*/
	KBObject *insBelow = this ;
	KBBlock  *level1   = 0	  ;
	bool 	 ok	   ;

	DPRINTF
	((	"KBBlock::newSubBlocks: build for %d\n",
		tabList.count()
	))	;

	for (uint tab = 1 ; tab < tabList.count() ; tab += 1)
	{
		KBBlock	*subBlk	= 0 ;

		if (getForm  () != 0)
			subBlk	= new KBFormBlock
				  (	insBelow,
					area,
					BTSubBlock,
					ok,
					"KBFormSubBlock"
				  )	;

		if (getReport() != 0)
			subBlk	= new KBReportBlock
				  (	insBelow,
					area,
					BTSubBlock,
					ok,
					"KBReportSubBlock"
				  )	;

		if (subBlk == 0)
			KBError::EFatal
			(	TR("Block error"),
				TR("Cannot identify object type"),
				__ERRLOCN
			)	;

		if (tab == 1) level1 = subBlk ;

		uint	 yRedux	= isReportBlock() ==   0 ? 0 :
				  area.height  () <= 120 ? 0 : 40 ;

		insBelow = subBlk ;
		area	 = QRect  (xDelta,
				   2 * gridY,
				   area.width () - 2 * xDelta,
				   area.height() - 3 * gridY - yRedux) ;
	}

	/* Pass down the query and level information. The odd case is	*/
	/* where the KBase-level query has but a single level, in which	*/
	/* case there is nothing to do.					*/
	if (level1 != 0) level1->setQryLevel (1, m_query) ;		

	delete	qry	;
	return	true	;
}

#if	! __KB_RUNTIME
/*  KBBlock								*/
/*  setBlkType	: Set the block type					*/
/*  newType	: BlkType	: Block type				*/
/*  (returns)	: bool		: Success				*/

bool	KBBlock::setBlkType
	(	BlkType	newType
	)
{
	KBQryBase *nQry	= 0 ;

	/* It should not be possible to change a subblock into some	*/
	/* other block type.						*/
	if (m_blkType == BTSubBlock)
		KBError::EFatal
		(	TR("Attempt to change block from subblock"),
			QString::null,
			__ERRLOCN
		)	;

	switch (newType)
	{
		case BTNull 	:
		case BTTable	:
		case BTQuery	:
		case BTSQL	:
			break	;

		case BTSubBlock	:
			/* This should never happen, the user should	*/
			/* never be able to change the block type to a	*/
			/* sub-block.					*/
			KBError::EFatal
			(	TR("Attempt to change block to subblock"),
				QString::null,
				__ERRLOCN
			)	;

		default	:
			KBError::EFault
			(	TR("Unrecognised block type"),
				QString::null,
				__ERRLOCN
			)	;
			return	false	;
	}

	switch (m_blkType = newType)
	{
		case BTTable	:
			/* The block accesses a table. Create the	*/
			/* table query object and running its		*/
			/* properties dialog.				*/
			nQry	= new KBQryTable    (this) ;

			if (!nQry->propertyDlg ())
			{	delete	nQry	;
				return	false	;
			}

			break	;

		case BTQuery	:
			/* Like a table ...				*/
			nQry	= new KBQryQuery (this) ;

			if (!nQry->propertyDlg ())
			{	delete	nQry	;
				return	false	;
			}

			break	;

		case BTSQL	:
			/* Also like a table ...			*/
			nQry	= new KBQrySQL (this) ;

			if (!nQry->propertyDlg ())
			{	delete	nQry	;
				return	false	;
			}

			break	;

		case BTNull	:
			/* This is a null block. Create a null query	*/
			/* object; this is a dummy query which always	*/
			/* succeeds, and returns zero rows of zero	*/
			/* fields. Hence, we dont have to check every	*/
			/* place that the block query is used.		*/
			nQry	= new KBQryNull  (this) ;
			break	;

		default	:
			/* Oops! Should have trapped this above! We'd	*/
			/* probably crash soon so lets go cleanly.	*/
			KBError::EFatal
			(	TR("Unrecognised block type"),
				QString::null,
				__ERRLOCN
			)	;
	}

	m_children.remove (nQry) ;
	while (m_children.count() > 0) delete m_children.first() ;

	m_children.append (nQry) ;
	m_query	= nQry	 ;

	if (m_blkType == BTQuery)
		if (!newSubBlocks()) return false ;


	return	true	;
}
#endif

/*  KBNode								*/
/*  endParse	: End parsing for this node				*/
/*  (returns)	: KBNode *	: Parent element			*/

KBNode	*KBBlock::endParse ()
{
	/* This is a hack, in case the number of rows/columns in a	*/
	/* dynamic grid is smaller than the range required by the	*/
	/* objects inside the grid.					*/
	if (isDynamic())
		CITER
		(	Object,
			obj,

			const KBAttrGeom &geom = obj->attrGeom() ;
			m_geom.extendGrid (geom.x() + geom.w(), geom.y() + geom.h()) ;
		)

	return	KBItem::endParse () ;
}

/*  KBBlock								*/
/*  remChild	: Remove a child node					*/
/*  child	: KBNode *	: Child in question			*/
/*  (returns)	: void		:					*/

void	KBBlock::remChild
	(	KBNode		*child
	)
{
	/* Intercept this call since, if the is an item, then we must	*/
	/* notify our query, and we also need to check for various	*/
	/* specific pointers.						*/
	if (child->isItem () != 0) m_query->remItem (m_qryLvl, child->isItem()) ;

	if (child == m_blkInfo.rowmark) m_blkInfo.rowmark = 0 ;
	if (child == m_header) m_header = 0 ;
	if (child == m_footer) m_footer = 0 ;

	KBObject::remChild (child) ;
}

/*  KBBlock								*/
/*  setGeometry	: Position the block in its parent			*/
/*  rect	: const QRect &	: New position				*/
/*  (returns)	: void		:					*/

void	KBBlock::setGeometry
	(	const QRect	&rect
	)
{
	KBObject::setGeometry (rect) ;
	if (m_blkDisp) m_blkDisp->setDisplayGeometry (rect) ;
}

/*  KBBlock								*/
/*  redoControls: Maybe update number of controls after geometry change	*/
/*  (returns)	: void		:					*/

void	KBBlock::redoControls ()
{
	if (m_blkDisp)
	{
		/* The number of rows possible in this block, and in	*/
		/* any nested framer, will have been calculated when	*/
		/* the (form)block or framer was resized. We now walk	*/
		/* the tree to find the maximum number and adjust.	*/
		uint	numrows	= rowsInBlock () ;

		DPRINTF
		((	"KBBlock::redoControls: [%p] m_numRows=%d->%d data=%d\n",
			(void *)this,
			numrows,
			m_numRows,
			showingData()
		))	;

		if (showingData() && (numrows == m_numRows)) return   ;

		m_numRows = numrows ;

		CITER
		(	Item,
			i,
			i->setupControls () ;
		)
		CITER
		(	Framer,
			f,
			f->setupControls () ;
		)
	}
}

/*  KBBlock								*/
/*  displayResized							*/
/*		: Block display has been resized			*/
/*  size	: const QSize & : New size				*/
/*  (returns)	: void		:					*/

void	KBBlock::displayResized
	(	const QSize		&
	)
{
	redoControls	() ;
}

/*  KBBlock								*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  qrow	: uint		: Starting row in query			*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBBlock::clearValue
	(	uint	,
		bool
	)
{
}

/*  KBBlock								*/
/*  changed	: See if user has changed an item			*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Item changed				*/

bool	KBBlock::changed
	(	uint
	)
{
	return	false	;
}

/*  KBBlock								*/
/*  isEmpty	: See if item it empty					*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Item empty				*/

bool	KBBlock::isEmpty
	(	uint
	)
{
	return	true	;
}

/*  KBBlock								*/
/*  isValid	: Test if value is valid to be saved			*/
/*  qrow	: uint		: Query row				*/
/*  allowNull	: bool		: Ignore not-null checks		*/
/*  (returns)	: bool		: Value can be saved			*/

bool	KBBlock::isValid
	(	uint		,
		bool
	)
{
	return	true	;
}

/*  KBBlock								*/
/*  requery	: (Re)query associated query for all rows		*/
/*  (returns)	: bool		  : Success				*/

bool	KBBlock::requery ()
{
	KBValue *mvp	= getBlockVal () ;
	bool	evRc	= true ;

	/* If this is a sub-block then we don't ever actually perform a	*/
	/* query, this will have been done in the block at the top of	*/
	/* the sub-block chain. A requery is simply interpreted as	*/
	/* returning to the first record.				*/
	if (m_blkType == BTSubBlock)
	{
		m_query->setCurrentRow (m_qryLvl, 0) ;
		return	true ;
	}

	m_curQRow	= 0	;
	m_curDRow	= 0	;

	m_query->clearItems (m_qryLvl, 0) ;

	/* Execute the event hook. A false result (as opposed to an	*/
	/* error) is interpreted as killing the query without any error	*/
	/* as if the query never executed.				*/
	if (!eventHook (m_events->preQuery, 0, 0, evRc))
		return	false	;

	if (!m_query->doSelect
		(	m_qryLvl,
			mvp,
			m_cexpr.getValue (),
			m_userFilter,
			m_userSorting,
			false,
			0,
			!evRc
	   )	)
	{
		setError (m_query->lastError()) ;
		return	 false ;
	}

	if (!eventHook (m_events->postQuery, 0, 0, evRc)) return false ;

	m_query->setCurrentRow (m_qryLvl, 0) ;
	return	true	;
}

/*  KBBlock								*/
/*  getQuery	: Get associated query					*/
/*  (returns)	: KBQryBase *	: Query					*/

KBQryBase *KBBlock::getQuery ()
{
	return	m_query ;
}

/*  KBBlock								*/
/*  getNumRows	: Get number of rows in query				*/
/*  (returns)	: uint		: Number of rows			*/

uint	KBBlock::getNumRows ()
{
	return	m_query == 0 ? 0 : m_query->getNumRows (m_qryLvl) ;
}

/*  KBBlock								*/
/*  getRowValue	: Get value from query					*/
/*  name	: const QString & : Query column name			*/
/*  qrow	: uint		  : Query row				*/
/*  (returns)	: KBValue	  : Value				*/

KBValue	KBBlock::getRowValue
	(	const QString	&name,
		uint		qrow
	)
{
	if (qrow <= getNumRows())
		CITER
		(	Item,
			i,
			if (i->getName() == name)
				return	m_query->getField (m_qryLvl, qrow, i->getQueryIdx()) ;
		)

	return	KBValue() ;
}

/*  KBBlock								*/
/*  getRowValue	: Get value from query					*/
/*  item	: KBItem *	: Child item				*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBBlock::getRowValue
	(	KBItem		*item,
		uint		qrow
	)
{
	DPRINTF
	((	"KBBlock::getRowValue: %s: lvl=%d qrow=%d idx=%d: q.nrows=%d\n",
		getName().latin1(),
		m_qryLvl,
		qrow,
		item->getQueryIdx(),
		m_query->getNumRows(m_qryLvl)
	))	;
	return	m_query->getField (m_qryLvl, qrow, item->getQueryIdx()) ;
}

/*  KBBlock								*/
/*  setRowValue	: Set value						*/
/*  name	: const QString & : Query column name			*/
/*  qrow	: uint		  : Query row				*/
/*  value	: const KBValye & : Value to set			*/
/*  (returns)	: void		  :					*/

void	KBBlock::setRowValue
	(	const QString	&name,
		uint		qrow,
		const KBValue	&value
	)
{
	if (qrow <= getNumRows())
		CITER
		(	Item,
			i,
			if (i->getName() == name)
			{	m_query->setField (m_qryLvl, qrow, i->getQueryIdx(), value) ;
				return	;
			}
		)
}

/*  KBBlock								*/
/*  findQuery	: Find child query node					*/
/*  (returns)	: bool		: Success				*/

bool	KBBlock::findQuery ()
{
	m_query	= 0 ;

	CITER
	(	QryBase,
		q,
		{	m_query	= q	;
			return	true	;
		}
	) 	;

	return	false	;
}

/*  KBBlock								*/
/*  blockSetup	: Prime block for operations				*/
/*  (returns)	: bool		: Success				*/

bool	KBBlock::blockSetup ()
{
	m_blkInfo.scroll  = m_blkDisp ;
	m_blkInfo.rowmark = 0 ;

	/* If we are not a sub-block (in which case the query will have	*/
	/* been propogated down from the enclosing block) then locate	*/
	/* our query. It should either be a table, a query or a dummy	*/
	/* null query. The total absense of one is bad news, as also	*/
	/* if we cannot tell what sort of query it is.			*/
	if (m_blkType != BTSubBlock)
	{
		if (!findQuery ())
		{
			setError
			(	KBError::Fault,
				TR("KBBlock lacks a query"),
				TR("Form or Report Error"),
				__ERRLOCN
			) 	;
			return	false	;
		}

		if	(m_query->isQryNull  () != 0) m_blkType = BTNull   ;
		else if (m_query->isQryTable () != 0) m_blkType = BTTable  ;
		else if (m_query->isQryQuery () != 0) m_blkType = BTQuery  ;
		else if (m_query->isQrySQL   () != 0) m_blkType = BTSQL    ;
		else if (m_query->isQryDesign() != 0) m_blkType = BTDesign ;
		else
		{
			setError
			(	KBError::Fault,
				TR("KBBlock has unrecognised query"),
				TR("Form or Report Error"),
				__ERRLOCN
			)	;
			return	false	;
		}
	}


	/* Sanity check: If we are a sub-block then make sure that we	*/
	/* have a correct query type.					*/
	if (m_blkType == BTSubBlock)
	{
		if ((m_query->isQryQuery() == 0) && (m_query->isQryDesign() == 0))
		{
			setError
			(	KBError::Fault,
				TR("KBSubBlock has invalid query"),
				TR("Form or Report Error"),
				__ERRLOCN
			)	;
			return	false	;
		}

		if (m_query->isQryQuery() != 0)
		{
			QString	mexpr	;
			QString	cexpr ;

			if (!m_query->isQryQuery()->getLinkage(m_qryLvl, mexpr, cexpr))
			{
				setError
				(	KBError::Error,
					TR("Failed to get subblock child expression"),
					TR("Form Error"),
					__ERRLOCN
				)	;
				return	false	;
			}

			m_expr .setValue (mexpr) ;
			m_cexpr.setValue (cexpr) ;
		}
	}

	/* If this is the top of a KBase-level query, then scan for	*/
	/* nested sub-blocks, and propogate the query and query level	*/
	/* down. Note that, even if we have a query-query, there might	*/
	/* not be any sub-blocks (ie., if the KBase-level query only	*/
	/* contains on table).						*/
	if ((m_blkType == BTQuery) || (m_blkType == BTDesign))
	{
		CITER
		(	Block,
			b,
			if (b->getBlkType() == BTSubBlock)
				b->setQryLevel (1, m_query) ;
		)
		CITER
		(
			Framer,
			f,
			f->setQryLevel(0, m_query) ;
		)
	}

	/* Finally, recurse down to all nested blocks (including sub-	*/
	/* blocks) and framers. This process will traverse the entire	*/
	/* tree of blocks and header, footers, etc.			*/
	CITER
	(	Block,
		b,
		if (!b->blockSetup())
		{	setError (b->lastError ()) ;
			return false ;
		}
	)
	CITER
	(	Framer,
		f,
		if (!f->framerSetup (m_query, m_qryLvl, &m_blkInfo))
		{	setError (f->lastError ()) ;
			return false ;
		}
	)

	return	true ;
}

/*  KBBlock								*/
/*  addAllItems	: Add all items to the query				*/
/*  (returns)	: bool		: Any item fetches from database	*/

bool	KBBlock::addAllItems ()
{
	bool	fetch	= m_blkType == BTNull ;

	/* Pass pointers to all child items (ie., fields and blocks)	*/
	/* to the query so that it knows what to retrieve from the	*/
	/* database; the initial call with a null pointer clears the	*/
	/* query level's list. Also note the row mark if any.		*/
	m_query->addItem (m_qryLvl, 0) ;

	CITER
	(	Item,
		i,
		{	if (m_query->addItem (m_qryLvl, i))
				fetch	= true	;
			if (i->isRowMark() != 0)
				m_blkInfo.rowmark = i->isRowMark() ;
		}
	)	;

	/* We recurse down into nested framers. Note that items in	*/
	/* framers are at the same query level as the block in which	*/
	/* they are embedded.						*/
	CITER
	(
		Framer,
		f,
		if ( f->addAllItems ()) fetch = true  ;
	)

	/* Also recurse down into nested blocks ...			*/
	CITER
	(
		Block,
		b,
		if (!b->addAllItems ()) fetch = false ;
	)

	return	fetch	;
}

/*  KBBlock								*/
/*  setQryLevel	: Pass query level information into sub-blocks		*/
/*  qryLvl	: uint		: Query level				*/
/*  query	: KBQryBase *	: Query					*/
/*  (returns)	: void		:					*/

void	KBBlock::setQryLevel
	(	uint		qryLvl,
		KBQryBase	*query
	)
{
	m_qryLvl = qryLvl ;
	m_query	 = query  ;

	CITER
	(	Block,
		b,
		if (b->getBlkType() == BTSubBlock)
			b->setQryLevel (m_qryLvl + 1, m_query) ;
	)

	CITER
	(	Framer,
		f,
		f->setQryLevel (m_qryLvl, m_query) ;
	)
}

/*  KBBlock								*/
/*  setCurrent	: Note whether this block contains the current item	*/
/*  current	: bool		: True if so				*/
/*  (returns)	: void		:					*/

void	KBBlock::setCurrent
	(	bool	current
	)
{
	if (m_blkInfo.rowmark != 0) m_blkInfo.rowmark->setInBlock (current) ;
}

/*  KBBlock								*/
/*  write	: Write block 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  extra	: int &		: Return extra space			*/
/*  prior	: bool		: Write prior value flag		*/
/*  (returns)	: bool		: Success				*/

bool	KBBlock::write
	(	KBWriter	*writer,
		QPoint		offset,
		bool		first,
		int		&extra,
		bool		prior
	)
{
	QString	bgcolor	;
	bgcolor.sprintf	("0x%06x", m_blkDisp->getDisplayWidget()->backgroundColor().rgb() & 0x00ffffff) ;

	new KBWriterBG (writer, geometry(), bgcolor) ;
	if (showingDesign()) new KBWriterBox (writer, geometry()) ;

	QPoint	s = writer->setOffset (false, geometry().topLeft()) ;
	KBNode::write (writer, offset, first, extra, prior) ;
	writer->setOffset (true, s)  ;
	return	true	;
}

/*  KBBlock								*/
/*  getObjects	: Get list of all objects in block			*/
/*  olist	: QPtrList<KBObject> & : List				*/
/*  (returns)	: void		    :					*/

void	KBBlock::getObjects
	(	QPtrList<KBObject>	&olist
	)
{
	CITER
	(	Object,
		o,
		olist.append (o)
	)
}

/*  KBBlock								*/
/*  getItems	: Get list of all items in block			*/
/*  ilist	: QPtrList<KBItem> &   : List				*/
/*  (returns)	: void		    :					*/

void	KBBlock::getItems
	(	QPtrList<KBItem>	&ilist
	)
{
	CITER
	(	Item,
		i,
		ilist.append (i)
	)
}

/*  KBBlock								*/
/*  setupControls							*/
/*		: Set up controls					*/
/*  (returns)	: void		:					*/

void	KBBlock::setupControls ()
{
	/* NOTE: This is the equivalent to the KBItem::setupControls()	*/
	/* method, not to be confused with ::redoControls() above.	*/
	if (getBlock() != 0)
		m_parentRows = getBlock()->getDisplayRows() ;
}

/*  KBBlock								*/
/*  getQueryComment							*/
/*		: Get query comment from associated query		*/
/*  (returns)	: QString	: Comment				*/

QString	KBBlock::getQueryComment ()
{
	return	m_query == 0 ? QString::null : m_query->getComment (m_qryLvl) ;
}

/*  KBBlock								*/
/*  setTabOrder	: Return tab order value				*/
/*  order	: int		: Tab order				*/
/*  (returns)	: void		:					*/

void	KBBlock::setTabOrder
	(	int		order
	)
{
	KBItem::setTabOrder (order) ;
}

/*  KBBlock								*/
/*  getTabOrder	: Return tab order value				*/
/*  (returns)	: int		: Tab order				*/

int	KBBlock::getTabOrder ()
{
	/* If there is no second language then blocks never appear in	*/
	/* the tab order. If there is the they appear for design but	*/
	/* are not actually tabable.					*/
	if (getRoot()->getAttrVal("language2").isEmpty())
		return	-1 ;

	return	KBItem::getTabOrder () ;
}

/*  KBBlock								*/
/*  getBlockVal	: Get block value pointer				*/
/*  (returns)	: KBValue *	: Value pointer or null at top level	*/

KBValue	*KBBlock::getBlockVal ()
{

	KBBlock	*parent	= getBlock () ;
	if ((parent == 0) || (parent->getBlkType() == BTNull))
		return	0 ;

	DPRINTF
	((	"KBBlock::getBlockVal: %s: p.crow=%d\n",
		getName().latin1(),
		parent->getCurQRow()
	))	;

	static	KBValue	mValue	;
	mValue	= parent->getRowValue (this, parent->getCurQRow()) ;
	return	&mValue	;
}

/*  KBBlock								*/
/*  minimumSize	: Get minimum size					*/
/*  (returns)	: QSize		: Minimum size				*/

QSize	KBBlock::minimumSize ()
{
	/* If we are dynamic and have a display then get the size hint	*/
	/* via that (hence via QT's layout management). Otherwise, work	*/
	/* directly from the configuration.				*/
	if (isDynamic() && m_blkDisp)
		return	m_blkDisp->sizeHint() ;

	return	KBObject::minimumSize () ;
}

/*  KBBlock								*/
/*  enumKBProperty							*/
/*		: Get list of properties				*/
/*  list	: QStringList &	: Property list				*/
/*  (returns)	: void		:					*/

void	KBBlock::enumKBProperty
	(	QStringList	&list
	)
{
	KBItem::enumKBProperty (list) ;
}

struct	ActionMap
{	cchar		*name	;
	int		code	;
}	;
static	ActionMap actionMap[]	=
{
	{	"actFirst",		KB::First		},
	{	"actPrevious",		KB::Previous		},
	{	"actNext",		KB::Next		},
	{	"actLast",		KB::Last		},
	{	"actAdd",		KB::Add			},
	{	"actSave",		KB::Save		},
	{	"actDelete",		KB::Delete		},
	{	"actQuery",		KB::Query		},
	{	"actExecute",		KB::Execute		},
	{	"actCancel",		KB::Cancel		},
	{	"actInsert",		KB::Insert		},
//	{	"actPrevBack",		KB::PrevBack		},
//	{	"actNextForw",		KB::NextForw		},
	{	"actReset",		KB::Reset		},
	{	"actGotoQRow",		KB::GotoQRow		},
	{	"actSyncAll",		KB::SyncAll		},
	{	"actStore",		KB::Store		},
	{	"actPrevPage",		KB::PrevPage		},
	{	"actNextPage",		KB::NextPage		},
	{	"actReload",		KB::Reload		},
	{	"actScroll",		KB::Scroll		},
	{	"actSaveAsNew",		KB::SaveAsNew		},
	{	"actSaveUnchecked",	KB::SaveUnchecked	},
	{	0,			0			}
}	;


/*  KBBlock								*/
/*  hasKBProperty: Check if object has this property			*/
/*  name	 : cchar *	: Property name				*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBBlock::hasKBProperty
	(	cchar		*name
	)
{
	for (ActionMap *amp = &actionMap[0] ; amp->name != 0 ; amp += 1)
		if (qstrcmp (amp->name, name) == 0)
			return	true ;

	return	KBItem::hasKBProperty (name) ;
}

/*  KBBlock								*/
/*  getKBProperty: Get a property rekall value				*/
/*  name	 : cchar *	: Proprty name				*/
/*  value	 : KBValue &	: Return property value			*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBBlock::getKBProperty
	(	cchar		*name,
		KBValue		&value
	)
{
	DPRINTF
	((	"KBBlock::getKBProperty: for [%s]\n",
		name
	))	;

	/* This method is implemented to make the action contants	*/
	/* available as properties. Doing this here means that it can	*/
	/* be independant of the script interfaces.			*/
	for (ActionMap *amp = &actionMap[0] ; amp->name != 0 ; amp += 1)
		if (qstrcmp (amp->name, name) == 0)
		{	value	= amp->code ;
			return	true ;
		}

	return	KBItem::getKBProperty (name, value) ;
}

/*  KBBlock								*/
/*  overLimit	: Check if query returned data over limit		*/
/*  (returns)	: uint		: Actual rows if over limit		*/

uint	KBBlock::overLimit ()
{
	return	m_query == 0 ? 0 : m_query->overLimit() ;
}

/*  KBBlock								*/
/*  getCurQRow	: Get current query row number				*/
/*  (returns)	: uint		: Query row number			*/

uint	KBBlock::getCurQRow ()
{
	return	m_curQRow ;
}

/*  KBBlock								*/
/*  getCurQRow	: Get current display row number				*/
/*  (returns)	: uint		: Display row number			*/

uint	KBBlock::getCurDRow ()
{
	return	m_curDRow ;
}

/*  KBBlock								*/
/*  getBlkType	: Get block type					*/
/*  (returns)	: BlkType	: Block type				*/

KBBlock::BlkType
	KBBlock::getBlkType ()
{
	return	m_blkType ;
}

/*  KBBlock								*/
/*  getQryLevel	: Get query level of this block				*/
/*  (returns)	: uint		: Query level				*/

uint	KBBlock::getQryLevel ()
{
	return	m_qryLvl  ;
}

/*  KBBlock								*/
/*  getDisplayRows							*/
/*		: Get number of rows displayed				*/
/*  (returns)	: uint		: Number of rows			*/

uint	KBBlock::getDisplayRows ()
{
	return	m_numRows ;
}

/*  KBBlock								*/
/*  getDisplayDX: Get X delta between rows				*/
/*  (returns)	: int		: X delta				*/

int	KBBlock::getDisplayDX	()
{
	return	m_dx.getIntValue() ;
}

/*  KBBlock								*/
/*  getDisplayDY: Get Y delta between rows				*/
/*  (returns)	: int		: X delta				*/

int	KBBlock::getDisplayDY	()
{
	return	m_dy.getIntValue() ;
}

/*  KBBlock								*/
/*  getShowbarFlags							*/
/*		: Get scrollbar/mininav flags				*/
/*  (returns)	: uint		: Flags					*/

uint	KBBlock::getShowbarFlags ()
{
	return	m_showbar.getFlags () ;
}

/*  KBBlock								*/
/*  setUserFilter							*/
/*		: Set query user filter					*/
/*  filter	: const QString & : Filter expression			*/
/*  (returns)	: void		  :					*/

void	KBBlock::setUserFilter
	(	const QString	&filter
	)
{
	m_userFilter	= filter	;
}

/*  KBBlock								*/
/*  setUserSorting							*/
/*		: Set query user sorting				*/
/*  sorting	: const QString & : Sorting expression			*/
/*  (returns)	: void		  :					*/

void	KBBlock::setUserSorting
	(	const QString	&sorting
	)
{
	m_userSorting	= sorting	;
}

bool	KBBlock::showAllRows ()
{
	return	(m_rowcount.getIntValue() & 0x8000) != 0 ;
}

bool	KBBlock::isUpdateVal
	(	bool
	)
{
	return	false	;
}