/***************************************************************************
    file	         : kb_block.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_BLOCK_H
#define		__KB_BLOCK_H

#include	"libkbase_exports.h"

#include	"kb_item.h"
#include	"kb_attrframe.h"
#include	"kb_attrnav.h"



class	KBBlockEvents	;

class	LIBKBASE_API	KBBlockInfo
{
public	:

	KBDisplay	*scroll		;
	KBRowMark	*rowmark	;

	inline	KBBlockInfo ()
	{
		scroll	= 0	;
		rowmark	= 0	;
	}
}	;

/*  KBBlock								*/
/*  -----------								*/
/*  This node represents the KBaseBlock XML element. It is essentially	*/
/*  a wrapper round a query and the data this is displayed from it.	*/

class	LIBKBASE_API KBBlock : public KBItem
{
	Q_OBJECT

public	:

	enum	BlkType
	{
		BTUnknown,
		BTTable,
		BTQuery,
		BTSubBlock,
		BTNull,
		BTDesign,
		BTSQL
	}	;


	enum	ShowData
	{	SDRedisplay	= 0x0001,
		SDScrollToRow	= 0x0002,
		SDBackToTop	= 0x0004
	}	;

protected :

	KBAttrStr		m_cexpr	    	;
	KBAttrStr		m_bgcolor	;

	KBAttrBool		m_autosync 	;
	KBAttrStr		m_title	    	;
	KBAttrFrame		m_frame	    	;
	KBAttrNav		m_showbar	;
	KBAttrInt		m_rowcount    	;
	KBAttrInt		m_dx	    	;
	KBAttrInt		m_dy	    	;

	BlkType			m_blkType	;

	KBBlockEvents		*m_events	;

	KBQryBase		*m_query    	;
	KBDisplay		*m_blkDisp    	;
	KBBlockInfo		m_blkInfo     	;

	uint			m_curQRow	; /* Current query row	*/
	uint			m_curDRow	; /* First display row	*/
	uint			m_qryLvl	; /* Block query level	*/

	/* The nRows member contains the number of rows displayed 	*/
	/* within this block; pRows contains the number of rows		*/
	/* displayed by the parent block. mValues contains the set of	*/
	/* values from the parent.					*/
	uint			m_numRows    	;
	uint			m_parentRows   	;

	bool			m_topLevel    	;

	KBFramer		*m_header	;
	KBFramer		*m_footer	;
	bool			m_anyChildBlock	;

	void			init		() 	;
	bool			newSubBlocks	()	;

	virtual void		setPalette	()	;
	virtual void		setFont		()	;
	bool			eventHook	(KBEvent &, uint, KBValue *, bool &, bool = true) ;

	bool			findQuery	()	;

	QString			m_userFilter	;
	QString			m_userSorting	;

public	:

	KBBlock	(KBNode   *, const QDict<QString> &, cchar *) ;
	KBBlock	(KBObject *, const QRect &, BlkType, bool  &, cchar *) ;
	KBBlock (KBNode   *, KBBlock *) ;
virtual~KBBlock	()		 ;

#if	! __KB_RUNTIME
	virtual	bool		propertyDlg	(cchar  *)	;
#endif
	virtual	void		showAs		(KB::ShowAs)	;
	virtual	void		remChild	(KBNode *)	;
	virtual	KBNode		*endParse  	()	 	;

	virtual	void		setGeometry	(const QRect &)	;
	virtual	void		displayResized	(const QSize &)	;

	virtual	KBBlock 	*isBlock	() { return this      ; }
	virtual	KBDisplay	*getContainer	() { return m_blkDisp ; }

	virtual	bool		isUpdateVal	(bool)	;
	virtual	KBValue		*getBlockVal	()	;
	virtual	KBValue		getValue	(uint) = 0 ;
	virtual	bool		setValue	(uint, const KBValue &) = 0 ;
	virtual	uint		rowsInBlock	()     = 0 ;

	virtual void		clearValue	(uint, bool) ;
	virtual	bool		changed		(uint) ;
	virtual	bool		isEmpty		(uint) ;
	virtual	bool		isValid		(uint, bool) ;

	virtual	uint		calcNumRows 	(int,  int,  uint    ) ;
	virtual	uint		calcNumRows 	(uint, uint, int, int) ;
	virtual void		setupControls	() ;

	bool			showAllRows	() ;
	void			redoControls	() ;
	void			setupDisplay	() ;
	void			buildTopDisplay	(KBDisplay *) ;
	void			buildDisplay	(KBDisplay *) ;
	bool			setBlkType	(BlkType) ;
	void			setCurrent	(bool)	  ;

	KBQryBase		*getQuery       () ;

	void			getObjects	(QPtrList<KBObject> &) ;
	void			getItems	(QPtrList<KBItem  > &) ;

	virtual	bool		blockSetup	() ;
	bool			addAllItems	() ;
	virtual bool		requery		() ;
	void			setQryLevel 	(uint, KBQryBase *) ;

	uint			getCurQRow	() ;
	BlkType			getBlkType	() ;
	uint			getCurDRow	() ;
	uint			getQryLevel	() ;
	uint			getDisplayRows	() ;
	int			getDisplayDX	() ;
	int			getDisplayDY	() ;
	uint			getShowbarFlags	() ;

	virtual	void		displayGone	() ;

	uint			getNumRows	() ;
	KBValue			getRowValue	(const QString &, uint) ;
	KBValue			getRowValue	(KBItem	       *, uint) ;
	void			setRowValue	(const QString &, uint, const KBValue &) ;

#if	! __KB_RUNTIME
	virtual	void		whatsThisExtra	(QValueList<KBWhatsThisPair> &) ;
	virtual	KBPopupMenu	*designPopup	(KBPopupMenu   *, QRect);
	virtual	QString		contextMenuType	()	;
#endif

	virtual	void		setTabOrder	(int)	;
	virtual	int		getTabOrder	()	;

	virtual	KBNode		*replicate	(KBNode *) = 0  ;
	virtual	bool		write		(KBWriter *, QPoint, bool, int &, bool = false) ;

	QString			getQueryComment	() ;
	virtual	QSize		minimumSize	() ;

	void			setUserFilter	(const QString	&) ;
	void			setUserSorting	(const QString	&) ;

	virtual	void		enumKBProperty	(QStringList	  &) ;
	virtual	bool		hasKBProperty	(cchar *	   ) ;
	virtual	bool		getKBProperty	(cchar *, KBValue &) ;

	uint			overLimit	() ;

public	slots :

	virtual void		docPropDlg	()    = 0 ;
	virtual	void		showQuery	() ;

	virtual void		pasteObjects	() ;
	virtual void		pasteHere	() ;
	virtual void		pasteComponent	() ;
	virtual void		linkComponent	() ;
}	;

#endif		// __KB_BLOCK_H
