/***************************************************************************
    file	         : kb_blockact.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"

#include	"kb_qrybase.h"
#include	"kb_rowmark.h"

#include	"kb_display.h"
#include	"kb_block.h"
#include	"kb_formblock.h"
#include	"kb_framer.h"
#include	"kb_form.h"
#include	"kb_event.h"
#include	"kb_blockevents.h"


#define		__DEBUG_FORMBLOCK	(0)
#if		__DEBUG_FORMBLOCK
#include	"kb_dprintf.h"
#endif

/*  KBFormBlock								*/
/*  doSyncAll	: Handle all-rows synchronisation			*/
/*  mvp		: KBValue *	: Value pointer to parent block		*/
/*  (returns)	: bool		: Success				*/

bool	KBFormBlock::doSyncAll
	(	KBValue		*mvp
	)
{
	if (!m_query->syncAll
		(	m_qryLvl,
			mvp,
			m_cexpr.getValue(),
			this
	   )	)
	{
		setError (m_query->lastError ()) ;
		return	 false ;
	}

	getLayout()->setChanged (false) ;

	KBValue	args[3] ;
	bool	evRc 	;

	args[0]	= 0	;
	args[1]	= (int)KB::SyncAll ;
	if (!eventHook (m_events->postSync, 3, args, evRc))
		return false ;

	m_dChanged = false ;
	return	true ;
}

/*  KBFormBlock								*/
/*  doSyncRow	: Handle current row synchronisation			*/
/*  mvp		: KBValue *	: Value pointer to parent block		*/
/*  qrow	: uint		: Row to synchronise			*/
/*  (returns)	: bool		: Success				*/

bool	KBFormBlock::doSyncRow
	(	KBValue		*mvp,
		uint		qrow
	)
{
	KB::Action	oper	;
	KBValue		args[3] ;
	bool		evRc 	;

	if (!m_query->syncRow
		(	m_qryLvl,
			qrow,
			mvp,
			m_cexpr.getValue(),
			this,
			oper,
			args[2]
	    )	)
	{
		setError (m_query->lastError ()) ;
		return	 false ;
	}

	getLayout()->setChanged (false) ;


	if (oper != KB::Null)
	{
		args[0]	= 0		;
		args[1]	= (int)oper	;
		if (!eventHook (m_events->postSync, 3, args, evRc))
			return false ;

		m_dChanged = false	;
	}

	return	true	;
}

/*  KBFormBlock								*/
/*  doOperation	: Handle various block actions				*/
/*  action	: KB::Action	   : Action required			*/
/*  toQRow	: uint		   : Target row for GotoQRow action	*/
/*  tabList	: KBTabOrderList * : Tab order list to be used		*/
/*  (returns)	: bool		   : Success				*/

bool	KBFormBlock::doOperation
	(	KB::Action	action,
		uint		toQRow,
		KBTabOrderList	*tabList
	)
{
	if (action == KB::Reload)
		return	doOperation (KB::Query,    0, tabList) &&
			doOperation (KB::Execute,  0, tabList) ;

	DPRINTF
	((	"KBFormBlock::doOperation: %s: oper=%d m_curQRow=%d\n",
		getName().latin1(),
		action,
		m_curQRow
	))	;

	/* If the block is a null (menu-only) block then there is	*/
	/* nothing to do within this block, but we do scan down through	*/
	/* any nested form blocks or framers.				*/
	if (m_blkType == BTNull)
	{
		switch (action)
		{
			case KB::Save		:
			case KB::SyncAll	:
			case KB::Store		:
			case KB::SaveUnchecked	:
				CITER
				(	FormBlock,
					b,
					if (!b->doOperation (action, 0))
					{	setError (b->lastError()) ;
						return   false ;
					}
				)
				CITER
				(	Framer,
					f,
					if (!f->doAction (action))
					{	setError (f->lastError()) ;
						return   false ;
					}
				)
				break	;

			default	:
				break	;
		}

		return	true	;
	}


	/* Note whether focus is in this block. If not then some	*/
	/* operations will not change focus.				*/
	bool	focusInBlock	= getForm()->focusInBlock (this) ;


	/* If there is a current item and it has changed then mark the	*/
	/* form as changed.						*/
	markChanged () ;

	/* As a prelude, carry out some checks which are common to	*/
	/* several of the actions. The first checks are for actions	*/
	/* which cannot be carried out during a query, or only during a	*/
	/* query.							*/
	switch (action)
	{
		case KB::First		:
		case KB::Previous	:
		case KB::Next		:
		case KB::Last		:
		case KB::Add		:
		case KB::Insert		:
		case KB::PrevBack	:
		case KB::NextForw	:
		case KB::PrevPage	:
		case KB::NextPage	:
		case KB::Query		:
		case KB::Save		:
		case KB::SyncAll	:
		case KB::Store		:
		case KB::Delete		:
		case KB::GotoQRow	:
		case KB::SaveAsNew	:
		case KB::SaveUnchecked	:
			if (m_inQuery)
			{	setError
				(	KBError::Warning,
					TR("Executing a query"),
					QString::null,
					__ERRLOCN
				)	;
				return	 false ;
			}
			break	;

		case KB::Execute	:
		case KB::Cancel		:
			if (!m_inQuery)
			{	setError
				(	KBError::Warning,
					TR("Not executing a query"),
					QString::null,
					__ERRLOCN
				)	;
				return	false	;
			}
			break	;

		case KB::Reset	:
			break	;

		default		:
			break	;
	}


	/* Let the event hook if any have a look in here. If this	*/
	/* returns false then the action is cancelled (although it is	*/
	/* not an error).						*/
	/* NOTE: Map PrevBack/NextForw to Previous/Next since the	*/
	/*	 distinction makes no sense at the scripting level.	*/
	KB::Action act2	;
	switch (action)
	{	case KB::PrevBack : act2 = KB::Previous ; break ;
		case KB::NextForw : act2 = KB::Next	; break ;
		default		  : act2 = action	; break ;
	}
	KBValue	arg	((int)act2, &_kbFixed) ;
	bool	evRc 	;

	if (!eventHook (m_events->onAction, 1, &arg, evRc)) return false ;
	if (!evRc) return true ;


	/* Next check for permission constraints. These may result from	*/
	/* underlying serverdatabase grant restrictions, or from unique	*/
	/* key restrictions.						*/
	switch (action)
	{
		case KB::Next		:
		case KB::NextForw	:
		case KB::NextPage	:
			/* These are no problem unless we are at the	*/
			/* last row, in which case we need to check	*/
			/* whether insert is allowed ...		*/
			if (m_curQRow + 1 < m_query->getNumRows (m_qryLvl))
				break	;

		case KB::Add		:
		case KB::Insert		:
		case KB::SaveAsNew	:
			/* ... also Next when at the last record.	*/
			if ((m_query->getPermission (m_qryLvl) & QP_INSERT) == 0)
			{	setError
				(	KBError::Warning,
					TR("Cannot insert rows: No unique key available after insert"),
					QString("%1: %2").arg(getElement()).arg(getName()),
					__ERRLOCN
				)	;
				return	false	;
			}
			break	;

		case KB::Delete	 	:
			if ((m_query->getPermission (m_qryLvl) & QP_DELETE) == 0)
			{	setError
				(	KBError::Warning,
					TR("Cannot delete rows: No unique key column"),
					QString("%1: %2").arg(getElement()).arg(getName()),
					__ERRLOCN
				)	;
				return	false	;
			}

			/* Run the pre-delete hook at this point, as it	*/
			/* may cancel the deletion.			*/
			{
				KBValue	aCurQRow ((int)m_curQRow, &_kbFixed) ;

				if (!eventHook (m_events->preDelete, 1, &aCurQRow, evRc))
					return	false	;
				if (!evRc)
					return	true	;
			}

			break	;

		case KB::Save		:
		case KB::SyncAll	:
		case KB::Store		:
		case KB::SaveUnchecked	:
			/* FIXME: Record might be new in which case we	*/
			/* ought to check for QP_INSERT.		*/
			if ((m_query->getPermission (m_qryLvl) & QP_UPDATE) == 0)
			{	setError
				(	KBError::Warning,
					TR("Cannot update rows: No unique key column"),
					QString("%1: %2").arg(getElement()).arg(getName()),
					__ERRLOCN
				)	;
				return	false	;
			}
			break	;


		default	:
			break	;
	}

	/* This flag gets set if the row is saved to the database ...	*/
	bool	rowDelta = false ;


	/* SaveAsNew saves the current record as a new record. We	*/
	/* insert (open up) a new new record at the current position.	*/
	/* Then the call to "checkChange()" below will save it. Since	*/
	/* the new record is marked as interted, we get a new record.	*/
	if (action == KB::SaveAsNew)
	{
		DPRINTF
		((	"KBFormBlock::doOperation: %s: saveAsNew\n",
			getName().latin1()
		))	;

		if (!m_query->insertRow (m_qryLvl, m_curQRow))
		{
			setError (m_query->lastError ()) ;
			return	 false ;
		}

		DPRINTF
		((	"KBFormBlock::doOperation: %s: saveAsNew ... inserted\n",
			getName().latin1()
		))	;

		/* The newly inserted row becomes the new current row.	*/
		getLayout()->setChanged() ;
		rowDelta = true ;
	}

	/* Next pick up all the actions which may result in an implicit	*/
	/* save to the database, and see if there are any changes.	*/
	switch (action)
	{
		case KB::First		:
		case KB::Previous	:
		case KB::Next		:
		case KB::Last		:
		case KB::Add		:
		case KB::Insert		:
		case KB::PrevBack	:
		case KB::NextForw	:
		case KB::PrevPage	:
		case KB::NextPage	:
		case KB::Query		:
		case KB::GotoQRow	:
			if (!checkChange (true,  rowDelta, true )) return false ;
			break	;

		case KB::Save		:
		case KB::SyncAll	:
		case KB::Store		:
		case KB::SaveAsNew	:
			if (!checkChange (false, rowDelta, true )) return false ;
			break	;

		case KB::SaveUnchecked	:
			if (!checkChange (false, rowDelta, false)) return false ;
			break	;

		default	:
			break	;
	}

	/* In the case of a save request, see if anything has changed	*/
	/* here .... FIX ME!						*/
	switch (action)
	{
		case KB::Save		:
		case KB::SyncAll	:
		case KB::Store		:
		case KB::SaveUnchecked	:
			CITER
			(	FormBlock,
				b,
				if (!b->doOperation (action, 0))
				{	setError (b->lastError()) ;
					return   false ;
				}
			)
			CITER
			(	Framer,
				f,
				if (!f->doAction (action))
				{	setError (f->lastError()) ;
					return   false ;
				}
			)
			break	;

		case KB::PrevPage	:
			/* Next page. This is handled by scrolling by	*/
			/* one less than the number of rows, or one,	*/
			/* whichever is greater.			*/
			{	uint	newDRow	;
				uint	dr = m_numRows <= 1 ? 1 : m_numRows - 1 ;

				if (dr < m_curDRow)
					newDRow	= m_curDRow - dr ;
				else	newDRow	= 0 ;

				scrollToRow (newDRow) ;
			}

			return	true	;

		case KB::NextPage	:
			/* Similarly next page. Note that "scrollToRow"	*/
			/* will limit the movement to the number of	*/
			/* query rows.					*/
			{	uint	dr = m_numRows <= 1 ? 1 : m_numRows - 1 ;
				scrollToRow (m_curDRow + dr) ;
			}
			return	true	;

		default	:
			break	;
	}


	/* Now the real work. In the process, set values for the item	*/
	/* and row which will become current, and note if the current	*/
	/* row may have changed.					*/
	KBObject	*newObj	 = 0	     ;
	KBItem		*newItem = m_curItem ;
	uint		newQRow	 = m_curQRow ;
	KBValue		*mvp	 = getBlockVal () ;
	KB::RState	rstate	 = m_query->getRowState (m_qryLvl, m_curQRow) ; 
	uint		nRowsQry = m_query->getNumRows  (m_qryLvl) ;
	uint		nRowsDel ;

	switch (action)
	{
		case KB::First		:
			/* Go to first extant record. Note that we do	*/
			/* not include the notional empty record at the	*/
			/* end.						*/
			if (nRowsQry > 0)
			{	newQRow	= 0 ;
				break	;
			}

			setError
			(	KBError::Warning,
				TR("No data was returned by server"),
				QString::null,
				__ERRLOCN
			)	;
			return	 false ;

		case KB::Previous	:
		case KB::PrevBack	:
			/* Go back one row ...				*/
			if (m_curQRow > 0)
			{	newQRow	= m_curQRow - 1 ;
				break	;
			}

			setError
			(	KBError::Warning,
				TR("Already at the first row"),
				QString::null,
				__ERRLOCN
			)	;
			return	 false ;

		case KB::Next		:
		case KB::NextForw	:
			/* Go to the next row. This is implemented such	*/
			/* that we can move from the last real record	*/
			/* to the notional empty record at the end.	*/
			if (m_curQRow < nRowsQry)
			{	newQRow	= m_curQRow + 1 ;
				break	;
			}

			setError
			(	KBError::Warning,
				TR("Already at the last row"),
				QString::null,
				__ERRLOCN
			)	;
			return	 false ;

		case KB::Last		:
			/* This is the reverse of first, again we do	*/
			/* not include the notional empty record unless	*/
			/* there are none at all.			*/
			if (m_query->getNumRows (m_qryLvl) > 0)
			{	newQRow	= m_query->getNumRows (m_qryLvl) - 1 ;
				break	;
			}

			setError
			(	KBError::Warning,
				TR("No data was returned by server"),
				QString::null,
				__ERRLOCN
			)	;
			return	 false ;

		case KB::GotoQRow	:
			/* We require that the row actually exists, or	*/
			/* is the next new row.				*/
			if (toQRow <= m_query->getNumRows (m_qryLvl))
			{
				newQRow	= toQRow ;
				break	;
			}

			setError
			(	KBError::Warning,
				TR("Attempt to go to non-existent row"),
				QString	(TR("Target %1 but only %1 rows"))
					.arg(toQRow)
					.arg(m_query->getNumRows (m_qryLvl)),
				__ERRLOCN
			)	;
			return	false	;

		case KB::Add	:
			/* Actually, this doesn't add, it just moves	*/
			/* to the notional empty record.		*/
			newQRow	= m_query->getNumRows (m_qryLvl) ;
			break	;
	
		case KB::Save		:
		case KB::SyncAll	:
		case KB::SaveAsNew	:
		case KB::SaveUnchecked	:
			/* If auto-sync is not on then save does a sync	*/
			/* on all rows, in which case do a complete	*/
			/* requery and redisplay.			*/
			if (!m_autosync.getBoolValue())
			{
				if (!doSyncAll (mvp))
					return false ;

				if (!requery   ())
					return false ;

				if (!showData  ())
					return false ;

				enterBlock (true, 0) ;
				return	true  ;
			}

			/* If autosync was set then the row will	*/
			/* have been synchronised in checkChange.	*/
			getLayout()->setChanged(false) ;
			break	 ;

		case KB::Insert	:
			/* Insert is not very meaningful for a normal	*/
			/* database query, since rows will be ordered	*/
			/* by the query, though it may be useful for	*/
			/* the user to add a new record while being	*/
			/* able see an extant record. The main use is	*/
			/* for table design, since we do have control	*/
			/* over the column order.			*/
			if (!m_query->insertRow (m_qryLvl, m_curQRow))
			{
				setError (m_query->lastError ()) ;
				return	 false ;
			}

			getLayout()->setChanged() ;

			/* The newly inserted row becomes the new	*/
			/* current row.					*/
			rowDelta = true ;
			break	 ;

		case KB::Delete	:
			/* Delete the current row, or all marked rows	*/
			/* if there are any; check for this first and	*/
			/* if not then delete the current row ....	*/
			if (!m_query->deleteAllMarked (m_qryLvl, nRowsDel))
			{
				setError (m_query->lastError ()) ;
				return	false	;
			}

			if (nRowsDel == 0)
			{
				/* No rows marked, so go for the	*/
				/* current row. This is silently	*/
				/* ignored if we are at the notional	*/
				/* empty row, except that the fields	*/
				/* are cleared.				*/
				if (m_curQRow >= m_query->getNumRows (m_qryLvl))
				{	clearFields (m_curQRow) ;
					return	 true  ;
				}

				if (!m_query->deleteRow (m_qryLvl, m_curQRow))
				{
					setError (m_query->lastError ()) ;
					return	 false ;
				}

				/* If autosync mode is on then sync	*/
				/* with the database, except if the row	*/
				/* was marked as previously inserted,	*/
				/* in which case it was never in the	*/
				/* database.				*/
				if (m_autosync.getBoolValue() && (rstate != KB::RSInserted))
					if (!doSyncRow(mvp, m_curQRow))
						return	false	;
			}
			else if (m_autosync.getBoolValue ())
			{
				/* Marked rows now marked for deletion,	*/
				/* and auto-sync is enabled. We do the	*/
				/* actual delete synchronisations one	*/
				/* at a time so the any post-sync hooks	*/
				/* are run.				*/
				uint	qrow	= 0 ;

				while (qrow < m_query->getNumRows (m_qryLvl))
					if (m_query->getRowMarked (m_qryLvl, qrow))
					{
						if (!doSyncRow (mvp, qrow))
							return	false	;
					}
					else	qrow	+= 1 ;
			}

			getLayout()->setChanged() ;

			/* Leave the query row where it is unless we	*/
			/* have just deleted the last row, in which	*/
			/* case go to the last row.			*/
			if (newQRow > 0)
			{
				uint nQryRows = m_query->getNumRows(m_qryLvl) ;

				if (newQRow >= nQryRows)
				{	if (nQryRows > 0)
						newQRow	= nQryRows - 1 ;
					else	newQRow	= 0 ;
				}
			}
			rowDelta = true ;
			break	;

		case KB::Query	:
			/* Clear the fields in the current row for the	*/
			/* used to enter query values, and flag that we	*/
			/* are in a query.				*/
			clearFields (m_curQRow, true) ;
			m_inQuery = true ;
			return	true ;

		case KB::Execute:
			/* Get the query to execute a SELECT, noting	*/
			/* that it should use the current field values	*/
			/* as query terms.				*/
			if (!eventHook (m_events->preQuery, 0, 0, evRc))
				return false ;

			if (!m_query->doSelect
				(	m_qryLvl,
					mvp,
					m_cexpr.getValue(),
					m_userFilter,
					m_userSorting,
					true,
					m_curQRow
			   )	)
			{
				setError (m_query->lastError ()) ;
				return	 false ;
			}

			if (!eventHook (m_events->postQuery, 0, 0, evRc))
				return	false ;

			m_inQuery  = false ;
			newQRow    = 0	   ;
			rowDelta   = true  ;
			break	;

		case KB::Cancel	:
			/* Clear the query flag and redisplay the	*/
			/* original values.				*/
			m_query->loadItems (m_qryLvl, m_curQRow) ;
			m_inQuery  = false ;
			break	;

		case KB::Reset	:
			/* Terminate any avtive transaction first.	*/
			if (!endUpdate (false)) lastError().DISPLAY () ;

			m_dChanged = false ;
			/* Reset controls. If we are in a query, or if	*/
			/* this is an inserted row, then they are all	*/
			/* cleared.					*/

			if (m_inQuery)
			{	clearFields (m_curQRow) ;
				return	true ;
			}
			if (m_curQRow >= m_query->getNumRows(m_qryLvl))
			{	clearFields (m_curQRow) ;
				return	true ;
			}
			if (m_query->getRowState (m_qryLvl, m_curQRow) == KB::RSInserted)
			{	clearFields (m_curQRow) ;
				return	true ;
			}

			/* Otherwise, reload them ....			*/
			m_query->resetData (m_qryLvl, m_curQRow) ;
			m_query->loadItems (m_qryLvl, m_curQRow) ;

			getLayout()->setChanged (false) ;
			return	true ;

		case KB::Store	:
			/* Store does not move elsewhere, so nothing to	*/
			/* do here.					*/
			break	;

		default	:
			setError
			(	KBError::Fault,
				QString(TR("Unknown KBFormBlock action %1")).arg((int)action),
				QString::null,
				__ERRLOCN
			)	;
			return	 false ;
	}

	/* If the caller has not specified a tab order list then use	*/
	/* the one from our own navigation object.			*/
	if (tabList == 0) tabList = &m_tabList ;

	/* Next switch is just concerned with determining which control	*/
	/* item will get focus.						*/
	switch (action)
	{
		case KB::First		:
		case KB::Last		:
		case KB::Add		:
		case KB::Insert		:
		case KB::Delete		:
		case KB::Query		:
		case KB::Execute	:
		case KB::Cancel		:
		case KB::NextForw	:
		case KB::GotoQRow	:
			newObj	= goFirst (false) ;
			newItem = newObj != 0 ? newObj->isItem() : 0 ;
			break	;

		case KB::PrevBack	:
			newObj	= goLast  (false) ;
			newItem = newObj != 0 ? newObj->isItem() : 0 ;
			break	;

		case KB::Store		:
		case KB::SaveAsNew	:
			/* Store does not move elsewhere, so nothing to	*/
			/* do here.					*/
			break	;

		default		:
			if (newItem == 0)
			{	newObj	= goFirst (false) ;
				newItem = newObj != 0 ? newObj->isItem() : 0 ;
			}
			break	;
	}

	/* Note whether we are switching to a new or item before	*/
	/* setting the now-current item and query row.			*/
	bool	chgItem	= (newItem != m_curItem) && (newItem != 0) ;
	bool	chgRow	= (newQRow != m_curQRow) ;

	/* Note whether we are switching to a new row before setting	*/
	/* the now-current item and query row.				*/
	bool	newRow	= newQRow != m_curQRow ;

	DPRINTF
	((	"KBFormBlock::doOperation: %s: newobj=[%p](%s) newitem=[%p]\n",
		getName().latin1(),
		(void *)newObj,
		newObj == 0 ? "" : (cchar *)newObj->getElement(),
		(void *)newItem
	))	;
	DPRINTF
	((	"KBFormBlock::doOperation: %s: newRow=%d rowDelta=%d\n",
		getName().latin1(),
		newRow,
		rowDelta
	))	;

	/* If on a new row or the row hash changed then notify the	*/
	/* query (neccessary for KBase-level queries) and redisplay.	*/
	/* We also run the UnCurrent event for the row we are leaving	*/
	/* (unless it has been deleted), and the Current event for the	*/
	/* noew current row.						*/
	if (newRow || rowDelta)
	{
		KBValue	argOld	((int)m_curQRow) ;
		KBValue	argNew	((int)newQRow) ;

		if (action != KB::Delete)
			if (!eventHook (m_events->onUnCurrent, 1, &argOld, evRc))
				return	false  ;

		m_curQRow = newQRow ;
		m_query->setCurrentRow (m_qryLvl, m_curQRow) ;

		if (!showData())
			return	false ;

		if (!eventHook (m_events->onCurrent, 1, &argNew, evRc))
			return	false ;
	}

	/* If focus is not in the block then don't do any focus		*/
	/* operations for queries, this means that script code can do	*/
	/* stuff like Query/Execute from another block without changing	*/
	/* focus.							*/
	if (!focusInBlock)
		switch (action)
		{
			case KB::Query	 :
			case KB::Execute :
			case KB::Cancel	 :
				return	true	;

			default	:
				break	;
		}

	/* Changing item. Update the current item pointer and give	*/
	/* focus to the current item at the current row.		*/
	if (chgItem)
	{
		m_curItem	= newItem ;
		m_curItem->setFocus   (m_curQRow) ;
		getForm()->focusAtRow (m_curQRow) ;
		return	true ;
	}

	/* Changing row and an item has focus; give focus to that item	*/
	/* at the current row.						*/
	if (chgRow && (m_curItem != 0))
	{
		m_curItem->setFocus   (m_curQRow) ;
		getForm()->focusAtRow (m_curQRow) ;
		return	true ;
	}

	/* Either the item and the row are both unchanged or there is	*/
	/* no current item, so nothing to do.				*/
	return true ;
}

/*  KBFormBlock								*/
/*  doAction	: Handle various query actions				*/
/*  action	: KBAction	   : Action required			*/
/*  tabList	: KBTabOrderList * : Tab order list to be used		*/
/*  (returns)	: bool		   : Success				*/

bool	KBFormBlock::doAction
	(	KB::Action	action,
		KBTabOrderList	*tabList
	)
{
	bool	rc = doOperation (action, 0, tabList) ;
	getForm()->setFocusAtRow (this) ;
	return	rc ;
}

/*  KBFormBlock								*/
/*  gotoQRow	: Move to specified query row				*/
/*  qRow	: uint		: Query row number			*/
/*  (returns)	: bool		: Success				*/

bool	KBFormBlock::gotoQRow
	(	uint		qRow
	)
{
	if (m_curQRow != qRow)
	{
		bool	rc = doOperation (KB::GotoQRow, qRow) ;
		getForm()->setFocusAtRow (this) ;
		return	rc ;
	}

	return	true	;
}

/*  KBFormBlock								*/
/*  gotoQRow	: Move to specified query row by key			*/
/*  key		: const KBValue & : Key value				*/
/*  (returns)	: int		  : Row number or negative on error	*/

int	KBFormBlock::gotoQRow
	(	const KBValue	&key
	)
{

	uint	nRows	= m_query->getNumRows (m_qryLvl) ;
	for (uint r = 0 ; r < nRows ; r += 1)
	{
		DPRINTF
		((	"KBFormBlock::gotoQRow: %d: [%s][%s]\n",
			r,
			key.getRawText().latin1(),
			m_query->getField(m_qryLvl, r, 0).getRawText().latin1()
		))	;

		if (m_query->getField (m_qryLvl, r, 0) == key)
		{
			DPRINTF
			((	"KBFormBlock::gotoQRow: [%s] at %d\n",
				key.getRawText().latin1(),
				r
			))	;
			return	gotoQRow (r) ? (int)r : -1 ;
		}
	}

	setError
	(	KBError::Warning,
		TR("Requested record not found"),
		QString(TR("Primary key: %1")).arg(key.getRawText()),
		__ERRLOCN
	)	;

	return	-1 ;
}


/*  KBFormBlock								*/
/*  insertRow	: Insert empty row at specified position		*/
/*  qrow	: uint		: Insertion point			*/
/*  (returns)	: bool		: Success				*/

bool	KBFormBlock::insertRow
	(	uint	qrow
	)
{
	if (!checkChange (true)) return false ;
	m_curQRow	= qrow	;
	return	doAction (KB::Insert) ;
}

/*  KBFormBlock								*/
/*  deleteRow	: Delete row at specified position			*/
/*  qrow	: uint		: Deletion point			*/
/*  (returns)	: bool		: Success				*/

bool	KBFormBlock::deleteRow
	(	uint	qrow
	)
{
	if (!checkChange (false)) return false ;
	m_curQRow	= qrow	;
	return	doAction (KB::Delete) ;
}


/*  KBFormBlock								*/
/*  moveFocusTo	: Move focus to specified item				*/
/*  item	: KBItem *	: Item in question			*/
/*  (returns)	: void		:					*/

void	KBFormBlock::moveFocusTo
	(	KBItem		*item
	)
{
	DPRINTF
	((	"KBFormBlock::moveFocusTo: %p -> %p\n",
		(void *)m_curItem,
		(void *)item
	))	;


	if (item != m_curItem)
	{
		/* If moving controls, verify that the current item	*/
		/* has valid contents, unless we are in a query, in	*/
		/* which case anything goes.				*/
		if (!m_inQuery && (m_curItem != 0))
		{
			if (!m_curItem->doLeave (m_curQRow))
				return	;

			if (!m_curItem->isValid (m_curQRow, true))
			{	m_curItem->lastError().DISPLAY() ;
				return	;
			}
		}

		/* OK, so move ...					*/
		m_curItem = item ;
		getForm()->focusInEvent (m_curItem, m_curQRow) ;
		m_curItem->setFocus     (m_curQRow) ;
	}
}

/*  KBFormBlock								*/
/*  scrollToRow	: Scroll to specified row				*/
/*  newDRow	: uint		: Row number				*/
/*  (returns)	: void		:					*/

void	KBFormBlock::scrollToRow
	(	uint	newDRow
	)
{
	uint	extra	= (m_query->getPermission(m_qryLvl) & QP_INSERT) != 0 ? 1 : 0 ;

	/* Start by bounding the upper display row to the valid range.	*/
	/* If it is unchanged then we can return.			*/
	if (newDRow + m_numRows > m_query->getNumRows (m_qryLvl) + extra)
	{
		if (m_query->getNumRows (m_qryLvl) + extra < m_numRows)
			newDRow	= 0 ;
		else	newDRow = m_query->getNumRows (m_qryLvl) + extra - m_numRows ;
	}

	if (newDRow == m_curDRow)
		return	;

	DPRINTF
	((	"KBFormBlock::scrollToRow: curDRow=%d newDRow=%d\n",
		m_curDRow,
		newDRow
	))	;

	/* Generate a pseudo-action for the scroll so that the onAction	*/
	/* event handler can stop the scroll from occurring.		*/
	KBValue	arg	((int)KB::Scroll, &_kbFixed) ;
	bool	evRc 	;

	if (!eventHook (m_events->onAction, 1, &arg, evRc)) return ;
	if (!evRc) return ;

	/* If the current query row will be outside the displayed range	*/
	/* and (a) there are any child blocks or (b) sloppy focus is	*/
	/* off (the default) then we will always move the query row to	*/
	/* stay visible.						*/
	if ((m_curQRow < newDRow) || (m_curQRow >= newDRow + m_numRows))
		if (m_anyChildBlock || !m_sloppy.getBoolValue())
		{
			DPRINTF
			((	"KBFormBlock::scrollToRow: moving curQRow\n"
			))	;

			if (!checkChange (true))
			{
				lastError().DISPLAY()	;
				goto	revert		;
			}

			uint	newQRow	= m_curQRow < newDRow ?
						newDRow :
						newDRow + m_numRows - 1 ;

			focusMovesRow (newQRow) ;

			if (m_curItem != 0)
			{	focusMovesItem       (m_curItem, QFocusEvent::Tab) ;
				m_curItem->setFocus  (m_curQRow) ;
			}

			getForm()->setFocusAtRow   (this) ;
			return	;
		}


	/* We need to save any changed values into the query set so	*/
	/* that they will be correctly reloaded into the scrolled	*/
	/* control, unless this is an empty new row.			*/
	if ((m_curQRow >= m_curDRow) && (m_curQRow < m_curDRow + m_numRows))
		if (!m_query->newRowEmpty (m_qryLvl, m_curQRow))
			if (!m_query->saveRow (m_qryLvl, m_curQRow, true))
			{
				m_query->lastError().DISPLAY() ;
				goto	revert	;
			}

	m_curDRow	= newDRow ;
	showData  (KBBlock::SDRedisplay) ;

	/* If the current row is now out of sight then pass focus to	*/
	/* the block display widget and clear any rowmark current row.	*/
	if ((m_curQRow < m_curDRow) || (m_curQRow >= m_curDRow + m_numRows))
	{
		m_blkDisp->getDisplayWidget()->setFocus() ;
		if (m_blkInfo.rowmark != 0)
			m_blkInfo.rowmark->setCurrent (m_curQRow) ;
	}
	else	/* Current row on display, pass focus to the current	*/
		/* item if there is one.				*/
		if (m_curItem != 0) m_curItem->setFocus (m_curQRow) ;

	return	;

	revert	:
		m_blkInfo.scroll->setRowRange
		(	m_query->getNumRows(m_qryLvl),
			extra,
			m_curQRow,
			m_curDRow,
			m_numRows
		)	;
}

void	KBFormBlock::scrollBy
	(	int		delta
	)
{
	uint	newDRow	= m_curDRow	;

	if (delta < 0)
		newDRow	= (uint)(-delta) > newDRow ? 0 : newDRow - (uint)(-delta) ;
	else	newDRow	= newDRow + (uint)delta ;

	scrollToRow (newDRow)	;
}
