/***************************************************************************
    file	         : kb_blockdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qwidgetstack.h>
#include	<qspinbox.h>
#include	<qcheckbox.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"

#if		__KB_AUTOMOC
#include	"kb_blockdlg.moc"
#else
#include	"kb_blockdlg.h"
#endif

#include	"kb_block.h"
#include	"kb_hidden.h"
#include	"kb_qrybase.h"


#include	"rk_combobox.h"
#include	"rk_pushbutton.h"


static	IntChoice	choicePThrow[] =
{
	{	0,		"None"			},
	{	1,		"Group"			},
	{	2,		"Record"		},
	{	-1,		0			}
}	;

static	IntChoice	choiceLocking[] =
{
	{	0,		"None"			},
	{	1,		"Lock row on update"	},
	{	-1,		0			}
}	;

static	cchar		*attrExclude[] =
{
	"rdonly",
	"default",
	"onenter",
	"onleave",
	"onset",
	"stretch",
	0
}	;

static	HelpMap	helpMap	[] =
{
	{	"dx",		"spacing"	},
	{	"dy",		"spacing"	},
	{	"master",	"linkage"	},
	{	"child",	"linkage"	},
	{	0,		0		}
}	;


/*  ------------------------------------------------------------------	*/

/*  KBAttrRowCount							*/
/*  --------------							*/
/*  Derived attribute item used to row counts, where there is a flag	*/
/*  bit used to mean "show all rows"					*/

class	KBAttrRowCount : public KBAttrItem
{
public	:

	virtual	QString	displayValue	() 		;

	KBAttrRowCount			(KBAttr *)	;
	virtual ~KBAttrRowCount		()		;
}	;



/*  KBAttrRowCount							*/
/*  KBAttrRowCount							*/
/*		: Constructor for row count item			*/
/*  attr	: KBAttr *	  : Actual attribute			*/
/*  (returns)	: KBAttrIntChoice :					*/

KBAttrRowCount::KBAttrRowCount
	(	KBAttr		*attr
	)
	:
	KBAttrItem	(attr)
{
}

/*  KBAttrRowCount							*/
/*  ~KBAttrRowCount							*/
/*		: Destructor for row count item				*/
/*  (returns)	:		:					*/

KBAttrRowCount::~KBAttrRowCount ()
{
}

/*  KBAttrRowCount							*/
/*  displayValue: Get value for display in attribute list		*/
/*  (returns)	: QString	: Display text				*/

QString	KBAttrRowCount::displayValue ()
{
	int	v = m_value.toInt() ;
	QString	t = QString::number(v & 0x7fff) ;

	if ((v & 0x8000) != 0)
		t += TR(", show all rows") ;

	return	t ;
}

/*  ------------------------------------------------------------------  */

/*  KBPropBlockDlg							*/
/*  KBPropBlockDlg: Constructor for block properties dialog		*/
/*  block	  : KBBlock * 	    : Associated block			*/
/*  caption	  : cchar *    	    : Dialog box caption		*/
/*  attribs	  : QPtrList<KBAttr>&  : List of attributes		*/
/*  iniattr	  : cchar *	    : Initial attribute			*/
/*  (returns)	  : KBBlockPropDlg  :					*/

KBBlockPropDlg::KBBlockPropDlg
	(	KBBlock			*block,
		cchar			*caption,
		QPtrList<KBAttr>	&attribs,
		cchar			*iniattr
	)
	:
	KBItemPropDlg	(block, caption, attribs, iniattr),
	m_block		(block)
{
	m_hiddenDlg	= new KBHiddenDlg  (m_propStack, block) ;
	m_hiddenDlg->hide() ;

	switch (m_block->getBlkType())
	{
		case KBBlock::BTTable	:
		case KBBlock::BTQuery	:
		case KBBlock::BTSQL	:
			m_bQuery = getUserButton(TR("Query")) ;
			connect (m_bQuery, SIGNAL(clicked()), this, SLOT(clickQuery ())) ;
			break	;

		default	:
			m_bQuery = 0	;
			break	;
	}
}

/*  KBPropBlockDlg							*/
/*  ~KBPropBlockDlg: Destructor for block properties dialog		*/
/*  (returns)	   : void	:					*/

KBBlockPropDlg::~KBBlockPropDlg ()
{
}

/*  KBBlockPropDlg							*/
/*  findHelpMapping							*/
/*		: Find possible help mapping for name			*/
/*  name	: const QString & : Name				*/
/*  (returns)	: cchar *	  : Mapping or null if none		*/

cchar	*KBBlockPropDlg::findHelpMapping
	(	const QString	&name
	)
{
	for (HelpMap *ptr = &helpMap[0] ; ptr->name != 0 ; ptr += 1)
		if (ptr->name == name)
			return	ptr->help ;

	return	KBItemPropDlg::findHelpMapping (name) ;
}

/*  KBBlockPropDlg								*/
/*  resizeEvent	: Handle dialog resizing				*/
/*  e		: QResizeEvent * : Resize event				*/
/*  (returns)	: void		 :					*/

void	KBBlockPropDlg::resizeEvent
	(	QResizeEvent	*e
	)
{
	KBItemPropDlg::resizeEvent (e) ;
}

/*  KBBlockPropDlg							*/
/*  getAttrItem	: Get attribute item for attribute			*/
/*  attr	: KBAttr *	: Attribute				*/
/*  (returns)	: KBAttrItem *	: Associated attribute item		*/

KBAttrItem *KBBlockPropDlg::getAttrItem
	(	KBAttr	*attr
	)
{
	if (attr->getName() == "pthrow"  )
		return	new KBAttrIntChoice (attr, choicePThrow ) ;

	if (attr->getName() == "locking" )
		return	new KBAttrIntChoice (attr, choiceLocking) ;

	if (attr->getName() == "rowcount")
		return	new KBAttrRowCount  (attr) ;


	return	KBItemPropDlg::getAttrItem (attr) ;
}

/*  KBBlockPropDlg							*/
/*  dropProperty: Drop property after user clicks ignore button		*/
/*  (returns)	: void		:					*/

void	KBBlockPropDlg::dropProperty ()
{
	KBItemPropDlg::dropProperty () ;
}

/*  KBBlockPropDlg							*/
/*  showProperty: Show property						*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property on show			*/

bool	KBBlockPropDlg::showProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Hidden field. We just show the hidden field widget and	*/
	/* enable/disable to remove button appropriately.		*/
	if (aName == "__hidden")
	{
		setUserWidget (m_hiddenDlg) ;
		return	true  ;
	}

	/* Child							*/
	/* This should display a list of all columns available from	*/
	/* the associated query, although the user can type in an	*/
	/* arbitrary expression.					*/
	if (aName == "child")
	{
		KBQryBase    *query  ;

		if ((query = m_block->getQuery()) == 0)
			return	warning (TR("Block lacks a query")) ;

		return	pickQueryField
			(	query,
				m_block->getQryLevel(),
				item->value(),
				m_block->getParent() == 0
			)	;
	}

	/* PThrow							*/
	/* This is the pagethrow property (none, per record, per group)	*/
	/* which is a choice.						*/
	if (aName == "pthrow" )
	{
		showChoices (item, choicePThrow,  item->value()) ;
		return	true ;
	}

	if (aName == "locking")
	{
		showChoices (item, choiceLocking, item->value()) ;
		return	true ;
	}

	if (aName == "rowcount")
	{
		int	v = item->value().toInt() ;

		if (!m_block->isDynamic())
		{
			m_spinBox ->setRange    (0, INT_MAX) ;
			m_spinBox ->setValue    (v & 0x7fff) ;
			m_spinBox ->show	() ;
		}

		m_checkBox->setChecked  (v & 0x8000) ;
		m_checkBox->setText     (TR("Display all rows")) ;
		m_checkBox->show	() ;

		m_spinBox ->setFocus    () ;
		return	true	;
	}

	if (aName == "exportrs")
	{
		m_comboBox->clear	() ;
		m_comboBox->insertItem	("No"  ) ;
		m_comboBox->insertItem	("Yes" ) ;
		m_comboBox->insertItem	("Here") ;

		if	(item->value() == "Yes" ) m_comboBox->setCurrentItem(1) ;
		else if (item->value() == "Here") m_comboBox->setCurrentItem(2) ;

		m_comboBox->show	() ;
		return	true	;
	}

	return	KBItemPropDlg::showProperty (item) ;
}

/*  KBBlockPropDlg							*/
/*  saveProperty: Check and save attribute				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Attribute OK				*/

bool	KBBlockPropDlg::saveProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Hidden fields						*/
	/* The actual information is in the list view, but stitch	*/
	/* together the appropriate string.				*/
	if (aName == "__hidden")
	{
		setProperty (aName, m_hiddenDlg->getText()) ;
		return	true ;
	}

	/* Child							*/
	/* In this case, copy pback the current selection from the	*/
	/* combo box.							*/
	if (aName == "child")
	{
		setProperty (aName, m_comboBox->currentText()) ;
		return	true ;
	}

	/* PThrow							*/
	/* This is the pagethrow property (none, per record, per group)	*/
	/* which is a choice.						*/
	if (aName == "pthrow" )
	{
		saveChoices (item, choicePThrow ) ;
		return	true	;
	}

	if (aName == "locking")
	{
		saveChoices (item, choiceLocking) ;
		return	true	;
	}

	if (aName == "rowcount")
	{
		int	v = 0 ;
		if (!m_block->isDynamic())
			v = m_spinBox->cleanText().toInt() & 0x7fff ;
		if (m_checkBox->isChecked()) v |= 0x8000 ;
		setProperty (item, QString::number(v)) ;
		return	true ;
	}

	if (aName == "exportrs")
	{
		setProperty (item, m_comboBox->currentText()) ;
		return	true ;
	}

	return	KBItemPropDlg::saveProperty (item) ;
}

/*  KBBlockPropDlg							*/
/*  hideProperty: Hide property						*/
/*  attr	: KBAttr *	: Attribute in question			*/
/*  (returns)	: void		:					*/

bool	KBBlockPropDlg::hideProperty
	(	KBAttr	*attr
	)
{
	const QString	&aName	= attr->getName() ;
	bool  masterChild	= (aName == "master") || (aName == "child") ;

	if (aName == "__hidden") return false ;

	/* Check for the properties which are actually in a form or	*/
	/* report, or are otherwise not relevant here, and ignore	*/
	/* them, and also the read-only property.			*/
	if (qstrcmp (attr->getOwnerName(), "KBForm"     ) == 0)
		return	true	;
	if (qstrcmp (attr->getOwnerName(), "KBReport"   ) == 0)
		return	true	;

	for (cchar **exp = &attrExclude[0] ; *exp != 0 ; exp += 1)
		if (*exp == aName)
			return	true ;

	/* If the block is a menu then there is no master-child		*/
	/* relationship.						*/
	if (m_block->getQuery() && m_block->getQuery()->isQryNull())
		if (masterChild)
			return	true ;

	/* Ditto the above if the block has a non-zero query level, in	*/
	/* which case it is a nested subblock.				*/
	if (m_block->getQryLevel() != 0)
		if (masterChild)
			return	true ;

	/* Unless we have a parent block, there is no point in showing	*/
	/* the master attribute, nor the X and Y values which should	*/
	/* always be zero at this level). This also hides the frame	*/
	/* and title properties. Also skip the name since it has no	*/
	/* use at the top level.					*/
	KBBlock	*parentBlk = m_block->getBlock () ;

	if (parentBlk == 0)
		if ( (aName == "master"  ) ||
		     (aName == "x"       ) ||
		     (aName == "y"       ) ||
		     (aName == "xmode"   ) ||
		     (aName == "ymode"   ) ||
		     (aName == "title"   ) ||
		     (aName == "frame"   ) ) return true ;

	/* If the parent block has a null query then again there is no	*/
	/* master-child relationship. HACK: Unless the parent is a	*/
	/* component ....						*/
	if ((parentBlk != 0) && parentBlk->getQuery()->isQryNull())
		if (masterChild)
			if (parentBlk->isComponent() == 0)
				return	true	;

//	/* Hide for dynamic layouts which always have one row. May be	*/
//	/* relaxed later.						*/
//	if (aName == "rowcount")
//		if (m_block->isDynamic())
//			return	true	;

	return	false	;
}

/*  KBBlockPropDlg:							*/
/*  propertyOK	: Check if property is OK				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property is OK			*/

bool	KBBlockPropDlg::propertyOK
	(	KBAttrItem	*item
	)
{
	const QString	&aName	= item->attr()->getName() ;

	/* The "child" property can be empty if the block has no	*/
	/* parent. This is the usual situation in such cases; "child"	*/
	/* would then only be set if doing across-form linking.		*/
	if (aName == "child")
		if (m_block->getParent() == 0)
			return	true ;

	/* If the block is embedded in a component then both the	*/
	/* "master" and "child" properties can be empty, since we do	*/
	/* not know where the component will be used.			*/
	KBBlock	*parentBlk = m_block->getBlock () ;
	if ((parentBlk != 0) && (parentBlk->isComponent() != 0))
		if ((aName == "master") || (aName == "child"))
			return	true	;

	return	KBItemPropDlg::propertyOK (item) ;
}

/*  KBBlockPropDlg:							*/
/*  clockQuery	  : Handle query button click				*/
/*  (returns)	  : void	:					*/

void	KBBlockPropDlg::clickQuery ()
{
	m_block->getQuery()->propertyDlg () ;
}

/*  KBBlockPropDlg:							*/
/*  clickOK	  : Handle OK button click				*/
/*  (returns)	  : void	:					*/

void	KBBlockPropDlg::clickOK ()
{
	m_hiddenDlg  ->clickOK () ;
	KBItemPropDlg::clickOK () ;
}

/*  KBBlockPropDlg:							*/
/*  clickCancel	  : Handle cancel button click				*/
/*  (returns)	  : void	:					*/

void	KBBlockPropDlg::clickCancel ()
{
	m_hiddenDlg  ->clickCancel () ;
	KBItemPropDlg::clickCancel () ;
}

/*  KBBlockPropDlg:							*/
/*  preExec	  : Called immediately before KBDialog::exec		*/
/*  (returns)	  : void	:					*/

void	KBBlockPropDlg::preExec ()
{
	/* Trap this and load the notional hidden attribute. We have to	*/
	/* do this here since it is not until "KBPropDlg::exec" has	*/
	/* been called that "setProperty" will work.			*/
	setProperty ("__hidden", m_hiddenDlg->getText()) ;
}

