/***************************************************************************
    file	         : kb_blockevents.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_attr.h"
#include	"kb_event.h"
#include	"kb_block.h"
#include	"kb_blockevents.h"
#include	"kb_nodereg.h"
#include	"kb_script.h"


#define		__DEBUG_BLOCKEVENTS	(0)
#if		__DEBUG_BLOCKEVENTS
#include	"kb_printf.h"
#endif

/*  KBBlockEvents							*/
/*  KBBlockEvents: Constructor for block events object			*/
/*  aList	 : ...		: Attributes list			*/
/*  parent	 : KBNode *	: Parent node				*/
/*  (returns)	 : KBBlockEvents: 					*/

KBBlockEvents::KBBlockEvents
	(	KBNode			*parent,
		const QDict<QString>	&aList
	)
	:
	onAction    (parent, "onaction",    aList, KAF_FORM),
	onUnCurrent (parent, "onuncurrent", aList, KAF_FORM),
	onCurrent   (parent, "oncurrent",   aList,	    KAF_EVCS),
	onDisplay   (parent, "ondisplay",   aList, KAF_FORM|KAF_EVCS),
	preQuery    (parent, "prequery",    aList),
	preInsert   (parent, "preinsert",   aList, KAF_FORM),
	preUpdate   (parent, "preupdate",   aList, KAF_FORM),
	preDelete   (parent, "predelete",   aList, KAF_FORM),
	postQuery   (parent, "postquery",   aList),
	postSync    (parent, "postsync",    aList, KAF_FORM),
	onChange    (parent, "onchange",    aList, KAF_FORM|KAF_EVCS)
{
}

/*  KBBlockEvents							*/
/*  KBBlockEvents: Constructor for new block events object		*/
/*  parent	 : KBNode *	: Parent node				*/
/*  (returns)	 : KBBlockEvents: 					*/

KBBlockEvents::KBBlockEvents
	(	KBNode		*parent
	)
	:
	onAction    (parent, "onaction",    "", KAF_FORM),
	onUnCurrent (parent, "onuncurrent", "", KAF_FORM),
	onCurrent   (parent, "oncurrent",   "",	         KAF_EVCS),
	onDisplay   (parent, "ondisplay",   "", KAF_FORM|KAF_EVCS),
	preQuery    (parent, "prequery",    ""),
	preInsert   (parent, "preinsert",   "", KAF_FORM),
	preUpdate   (parent, "preupdate",   "", KAF_FORM),
	preDelete   (parent, "predelete",   "", KAF_FORM),
	postQuery   (parent, "postquery",   ""),
	postSync    (parent, "postsync",    "", KAF_FORM),
	onChange    (parent, "onchange",    "", KAF_FORM|KAF_EVCS)
{
}

/*  KBBlockEvents							*/
/*  KBBlockEvents: Constructor for new block events object		*/
/*  parent	 : KBNode *	: Parent node				*/
/*  block	 : KBBlock *	: Block being replicated		*/
/*  (returns)	 : KBBlockEvents: 					*/

KBBlockEvents::KBBlockEvents
	(	KBNode		*parent,
		KBBlock		*block
	)
	:
	onAction    (parent, "onaction",    block, KAF_FORM),
	onUnCurrent (parent, "onuncurrent", block, KAF_FORM),
	onCurrent   (parent, "oncurrent",   block,	    KAF_EVCS),
	onDisplay   (parent, "ondisplay",   block, KAF_FORM|KAF_EVCS),
	preQuery    (parent, "prequery",    block),
	preInsert   (parent, "preinsert",   block, KAF_FORM),
	preUpdate   (parent, "preupdate",   block, KAF_FORM),
	preDelete   (parent, "predelete",   block, KAF_FORM),
	postQuery   (parent, "postquery",   block),
	postSync    (parent, "postsync",    block, KAF_FORM),
	onChange    (parent, "onchange",    block, KAF_FORM|KAF_EVCS)
{
}

/*  ------------------------------------------------------------------	*/

/*  KBBlockEvents							*/
/*  eventHook	 : Event execution hook					*/
/*  event	 : KBEvent &	: Event to execute			*/
/*  argc	 : uint		: Argument count			*/
/*  argv	 : KBValue *	: Argument vector			*/
/*  evRc	 : bool &	: Event return code			*/
/*  defres	 : bool	 	: Default return code			*/
/*  (returns)	 : bool		: No event execution error		*/

bool	KBBlock::eventHook
	(	KBEvent		&event,
		uint		argc,
		KBValue		*argv,
		bool		&evRc,
		bool		defval
	)
{
	KBValue	resval	;

	DPRINTF
	((	"KBBlock::eventHook: [%s][%s][%s] executing event ....\n",
		(cchar *)getElement   (),
		(cchar *)getName      (),
		(cchar *)event.getName()
	))	;

	KBScriptError *rc = event.execute (resval, argc, argv, defval) ;
	if (rc != 0) 
	{
		KBScriptError::processError (rc, KBScriptError::Block) ;
		return	false	;
	}

	/* No execution errors, so result is true with "evRc" set to	*/
	/* the value returned by the script.				*/
	evRc	= resval.isTrue() ;
	return	true	;
}
