/***************************************************************************
    file	         : kb_blockgui.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<time.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"
#include	"kb_hidden.h"
#include	"kb_rowmark.h"
#include	"kb_dispwidget.h"
#include	"kb_options.h"
#include	"kb_event.h"
#include	"kb_attrdict.h"
#include	"kb_nodereg.h"
#include	"kb_framer.h"
#include	"kb_parse.h"

#include	"kb_block.h"
#include	"kb_blockdlg.h"
#include	"kb_formblock.h"
#include	"kb_reportblock.h"
#include	"kb_form.h"
#include	"kb_report.h"
#include	"kb_blockevents.h"
#include	"kb_qrydata.h"

#include	"kb_component.h"

#include	"kb_designpopup.h"

#if		! __KB_RUNTIME
#include	"kb_popupmenu.h"
#include	"kb_sizer.h"
#include	"kb_formcopier.h"
#include	"kb_compaccessdlg.h"
#endif


/*  KBBlock								*/
/*  setPalette	: Set palette						*/
/*  (returns)	: void		:					*/

void	KBBlock::setPalette ()
{
	m_blkDisp->setPalette (getPalette (false)) ;
	KBObject::setPalette () ;
}

/*  KBBlock								*/
/*  setFont	: Set font						*/
/*  (returns)	: void		:					*/

void	KBBlock::setFont ()
{
	m_blkDisp->setFont (getFont (false)) ;
	KBObject::setFont () ;
}


/*  KBBlock								*/
/*  calcNumRows	: Calculate number of rows which can be show		*/
/*  bw		: uint		: Block display width			*/
/*  bh		: uint		: Block display height			*/
/*  dx		: int		: Control X-spacing			*/
/*  dy		: int		: Control Y-spacing			*/
/*  (returns)	: uint		: Number of rows that can be displayed	*/

uint	KBBlock::calcNumRows
	(	uint	,
		uint	,
		int	,
		int
	)
{
	/* NOTE: This refers to the number of rows which a block can	*/
	/* "display" from its parent. There is no limit on this; the	*/
	/* number of rows that the parent shows will be controlled by	*/
	/* other items in the parent.					*/
	return	999	;
}

/*  KBBlock								*/
/*  dx		: int		: Control X-spacing			*/
/*  dy		: int		: Control Y-spacing			*/
/*  limit	: uint		: Upper limit on number of rows		*/
/*  (returns)	: uint		: Number of rows that can be displayed	*/

uint	KBBlock::calcNumRows
	(	int	,
		int	,
		uint	limit
	)
{
	return	limit	;
}

void	KBBlock::setupDisplay ()
{
	m_blkInfo.scroll = m_blkDisp ;

	setPalette () ;
	setFont	   () ;

	QString	fs	= m_frame.getValue() ;
	int	coff	= fs.find (',')	;

	if (coff >= 0)
		m_blkDisp->setFrame
		(	fs.left(coff    ).toInt(),
			fs.mid (coff + 1).toInt()
		)	;
	else	m_blkDisp->setFrame (0, 0) ;
}

/*  KBBlock								*/
/*  buildDisplay: Build display						*/
/*  display	: KBDisplay *	: Top display				*/
/*  (returns)	: void		:					*/

void	KBBlock::buildTopDisplay
	(	KBDisplay *display
	)
{
	m_blkDisp = display  ;
	KBItem::buildDisplay (m_blkDisp) ;

	CITER
	(	Object,
		o,
		o->buildDisplay (m_blkDisp)
	) ;

	setupDisplay () ;
}

/*  KBBlock								*/
/*  buildDisplay: Build display						*/
/*  display	: KBDisplay *	: Parent display			*/
/*  (returns)	: void		:					*/

void	KBBlock::buildDisplay
	(	KBDisplay	*display
	)
{
	m_blkDisp = new KBDispWidget (display, this, getShowbarFlags()) ;

	KBItem::buildDisplay (display) ;

	CITER
	(	Object,
		o,
		o->buildDisplay (m_blkDisp)
	) ;

	setupDisplay () ;
}

/*  KBBlock								*/
/*  displayGone	: Display has been destroyed				*/
/*  (returns)	: void		:					*/

void	KBBlock::displayGone ()
{
	m_blkDisp = 0 ;
}

/*  KBBlock								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBBlock::showAs
	(	KB::ShowAs	mode
	)
{
	m_blkDisp->showAs (mode) ;

	/* A special case when creating a sizer is if there is no	*/
	/* parent, which is the case for the top-most (form or report)	*/
	/* level block.							*/
#if	! __KB_RUNTIME
	if ((mode == KB::ShowAsDesign) && (getSizer() == 0))
		setSizer
		(	new KBSizer
			(	this, 
			 	getDisplay() == 0 ? (KBDisplay *)m_blkDisp : getDisplay(),
				m_blkDisp->getDisplayWidget()
		)	)	;

	if ((mode == KB::ShowAsData  ) && (getSizer() != 0))
		setSizer (0) ;
#endif

	m_anyChildBlock	= false	;
	m_header	= 0	;
	m_footer	= 0	;
	m_userSorting	= QString::null	;
	m_userFilter	= QString::null	;

	CITER
	(	Framer,
		f,

		if (f->getElement() == "KBBlockHeader") m_header = f ;
		if (f->getElement() == "KBBlockFooter") m_footer = f ;
	)

	CITER
	(	Object,
		o,
		if ((o->isFramer() != 0) && o->isFramer()->anyChildBlock())
		{	m_anyChildBlock = true ;
			break	;
		}
		if ((o->isBlock () != 0) && (o->isBlock()->getBlkType() != KBBlock::BTNull))
		{	m_anyChildBlock = true ;
			break	;
		}
	)

	/* Clear all items. This is mainly intended when switching into	*/
	/* design mode, to clear field contents.			*/
	CITER
	(	Item,
		i, 
		i->clearBelow (1) ;
	)

	/* Pass the request on down to the base class, which will in	*/
	/* turn recurse down to any children.				*/
	KBObject::showAs (mode) ;

	/* If we are switching out of design mode then clear the slaved	*/
	/* master values (so that the block will requery). Also a fair	*/
	/* time to ensure the tab order is sorted.			*/
	if (showingData())
	{
		m_curQRow = 0 ;
		m_curDRow = 0 ;
	}

	/* Update the display widget, so that we correctly add or	*/
	/* remove the grid and such like.				*/
	m_blkDisp->getDisplayWidget()->update() ;
}

#if	! __KB_RUNTIME

/*  KBBlock								*/
/*  whatsThisExtra							*/
/*		: Extra stuff for pseudo-what's this popup		*/
/*  info	: QValueList<KBWhatsThisPair> &	: List of items		*/
/*  (returns)	: void				:			*/

void	KBBlock::whatsThisExtra
	(	QValueList<KBWhatsThisPair>	&info
	)
{
	/* Skip KBItem, we want to label the expression differently.	*/
	KBObject::whatsThisExtra (info) ;

	if (m_query != 0)
	{
		m_query->whatsThisExtra (info) ;
		info.append (KBWhatsThisPair(TR("Parent column"), m_expr .getValue())) ;
		info.append (KBWhatsThisPair(TR("Child column"),  m_cexpr.getValue())) ;
	}
}

/*  KBBlock								*/
/*  designPopup	: Handle design menu request				*/
/*  parent	: KBPopupMenu * : Possible parent menu			*/
/*  cell	: QRect		: Positioning information		*/
/*  (returns)	: KBPopupMenu *	: Popup menu				*/

KBPopupMenu
	*KBBlock::designPopup
	(	KBPopupMenu	*parent,
		QRect		cell
	)
{
	bool	noBlk	= getBlock() == 0 ;
	bool	noQry	= false	;

	switch (m_blkType)
	{
		case BTSubBlock	:
			noBlk	= true	;
			noQry	= true	;
			break	;

		case BTNull	:
			noQry	= true	;
			break	;

		case BTTable	:
		case BTQuery	:
		case BTSQL	:
		case BTDesign	:
			break	;

		default	:
			break	;
	}

	KBPopupMenu *popupMain	= new KBPopupMenu (parent, &m_bState) ;
	KBPopupMenu *popupEdit	= 0 	;
	KBPopupMenu *popupNew	= 0 	;

	/* Construct the container edit popup menu; create the new	*/
	/* popup only if there is no parent (ie., part of the block is	*/
	/* actually exposed) and either this is not a dynamic block or	*/
	/* we are in an empty cell.					*/
	popupEdit = makeContainerEditPopup
			(	popupMain,
				this,
				TR("block"),
				noBlk
			)	;

	if (parent == 0)
		if (!isDynamic() || (objectInCell (cell) == 0))
			popupNew = makeNewPopup (popupMain, cell) ;

	makeContainerMainPopup
	(	popupMain,
		this,
		TR("Block"),
		popupNew,
		popupEdit
	)	;

	setCtrlRect (cell) ;
	return	popupMain  ;
}

/*  KBBlock								*/
/*  contextMenuType							*/
/*		: Return type text for context menu			*/
/*  (returns)	: QString	: Text					*/

QString	KBBlock::contextMenuType ()
{
	switch (getBlkType())
	{
		case KBBlock::BTTable	: return TR("Table block") ;
		case KBBlock::BTQuery	: return TR("Query block") ;
		case KBBlock::BTSQL	: return TR("SQL block"  ) ;
		case KBBlock::BTNull	: return TR("Menu block" ) ;
		default			: break	 ;
	}

	return	KBItem::contextMenuType () ;
}

/*  KBBlock								*/
/*  propertyDlg	: Show block property dialog				*/
/*  iniattr	: cchar *	: Initial attribute			*/ 
/*  (returns)	: bool		: Success				*/

bool	KBBlock::propertyDlg
	(	cchar	*iniattr
	)
{
	KBBlockPropDlg	bDlg (this, "Block", m_attribs, iniattr) ;
	if (!bDlg.exec())
		return false ;

	/* Special case: This method will be called during form/report	*/
	/* creation, at which time there is no block display.		*/
	if (m_blkDisp)
	{
		uint	ssb1	= getShowbarFlags() ;

		m_blkDisp->setShowbar    (ssb1) ;
		m_blkDisp->setTitle      (m_title.getValue()) ;
		m_blkDisp->updateDynamic () ;

		setPalette    () ;
		setFont       () ;

		QString	fs	= m_frame.getValue() ;
		int	coff	= fs.find (',')	;

		if (coff >= 0)
			m_blkDisp->setFrame
			(	fs.left(coff    ).toInt(),
				fs.mid (coff + 1).toInt()
			)	;
		else	m_blkDisp->setFrame (0, 0) ;

		if (getSizer() != 0)
			getLayout()->addSizer (getSizer(), false) ;
	}

	getLayout()->setChanged() ;
	return	true  ;
}

#endif

/*  KBBlock								*/
/*  pasteObjects: Paste in cut or copied objects			*/
/*  (returns)	: void		:					*/

void	KBBlock::pasteObjects ()
{
#if	! __KB_RUNTIME
	insertObjects (m_blkDisp) ;
#endif
}

/*  KBBlock								*/
/*  pasteHere	: Paste in cut or copied objects at specified position	*/
/*  (returns)	: void		:					*/

void	KBBlock::pasteHere ()
{
#if	! __KB_RUNTIME
	insertHere
	(	KBFormCopier::self()->getCopied (objType()),
		m_blkDisp,
		newCtrlRect()
	)	;
#endif
}

/*  KBBlock								*/
/*  pasteComponent							*/
/*		: Paste component object				*/
/*  (returns)	: void		:					*/

void	KBBlock::pasteComponent ()
{
#if	! __KB_RUNTIME
	insertComponent (m_blkDisp, newCtrlRect(), true) ;
#endif
}

/*  KBBlock								*/
/*  linkComponent							*/
/*		: Link component object					*/
/*  (returns)	: void		:					*/

void	KBBlock::linkComponent ()
{
#if	! __KB_RUNTIME
	insertComponent (m_blkDisp, newCtrlRect(), false) ;
#endif
}

void	KBBlock::showQuery ()
{
#if	! __KB_RUNTIME
	addAllItems ()	;

	QString	sql	= m_query->getSQLText (m_qryLvl, true) ;
	QString	reason	;

	if (isFormBlock() != 0) reason = m_query->getSQLReason (m_qryLvl) ;

	KBQryDisplay(sql, reason).exec() ;
#endif
}

