/***************************************************************************
    file	         : kb_button.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_button.moc"
#else
#include 	"kb_button.h"
#endif

#include	"kb_nodereg.h"
#include	"kb_navigator.h"
#include	"kb_docroot.h"
#include	"kb_attrimage.h"
#include	"kb_options.h"

#include        "tk_icons.h"

#if		! __KB_RUNTIME
#include	"kb_recorder.h"
#include	"kb_popupmenu.h"
#endif

#define		__DEBUG_BUTTON	(0)
#if		__DEBUG_BUTTON
#include	"kb_dprintf.h"
#endif

#include	<qpainter.h>


#if	! __KB_RUNTIME
static	cchar	*imageLegends[] =
{
	"Normal",
	"Rollover",
	"Disable",
	0
}	;
#endif

/*  KBButton								*/
/*  KBButton	: Consructor for button node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBButton	:					*/

KBButton::KBButton
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBObject 	(parent, "KBButton",	aList),
	m_fgcolor	(this,   "fgcolor",	aList),
	m_bgcolor	(this,   "bgcolor",	aList),
	m_font		(this,	 "font",	aList),
	m_text	 	(this,   "text",     	aList, KAF_EDITOR),
	m_image		(this,	 "image",	aList, KAF_CLEAR ),
	m_tabOrd	(this,	 "taborder", 	aList),
	m_toggle	(this,	 "toggle", 	aList),
	m_onClick	(this,   "onclick",  	aList, KAF_EVCS  ),
	m_onToggle	(this,   "ontoggle",  	aList),
	m_toolTip	(this,	 "tooltip",	aList, KAF_EDITOR|KAF_CLEAR)
{
	m_button	= 0 	;
	m_code		= 0 	;
	m_disable	= false	;


#if	! __KB_RUNTIME
	m_image.setNumImages (3);
	m_image.setLegends   (imageLegends) ;

	if (ok != 0)
	{
		if (!KBNode::basePropertyDlg ("Button"))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif
}

/*  KBButton								*/
/*  KBButton	: Consructor for button node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  button	: KBButton *	: Extant button				*/
/*  (returns)	: KBButton	:					*/

KBButton::KBButton
	(	KBNode	 	*parent,	
		KBButton 	*button
	)
	:
	KBObject	(parent, button),
	m_fgcolor	(this,   "fgcolor",	button),
	m_bgcolor	(this,   "bgcolor",	button),
	m_font		(this,	 "font",	button),
	m_text	 	(this,   "text",	button, KAF_EDITOR),
	m_image		(this,	 "image",	button, KAF_CLEAR ),
	m_tabOrd	(this,	 "taborder", 	button),
	m_toggle	(this,	 "toggle", 	button),
	m_onClick	(this,   "onclick",	button, KAF_EVCS  ),
	m_onToggle	(this,   "ontoggle",	button),
	m_toolTip	(this,	 "tooltip",	button, KAF_EDITOR|KAF_CLEAR)
{
#if	! __KB_RUNTIME
	m_image.setNumImages (3);
	m_image.setLegends   (imageLegends) ;
#endif

	m_button	= 0	;
	m_code		= 0 	;
	m_disable	= false	;
}

/*  KBButton								*/
/*  ~KBButton	: Desructor for button node				*/
/*  (returns)	:		:					*/

KBButton::~KBButton ()
{
}

/*  KBButton								*/
/*  buildDisplay: Build controls for this node				*/
/*  display	: KBDisplay *	: Owning display			*/
/*  (returns)	: void		:					*/

void	KBButton::buildDisplay
	(	KBDisplay	*display
	)
{
	KBObject::buildDisplay (display) ;

	if (m_button == 0)
	{
		m_button = new KBCtrlButton (display, this) ;
		setControl (m_button) ;
	}

	setupProperties	() ;
}

/*  KBButton								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBButton::showAs
	(	KB::ShowAs	mode
	)
{
	KBObject::showAs (mode) ;
}

/*  KBButton								*/
/*  loadImage	: Load a named image					*/
/*  image	: const QString & : Image name				*/
/*  (returns)	: QPixmap	  : Image or null on error		*/

QPixmap	KBButton::loadImage
	(	const QString	&image
	)
{
	QStringList	iparts	= QStringList::split ('.', image) ;

	DPRINTF
	((	"KBButton::loadImage: [%s][%s]\n",
		(cchar *)iparts[0],
		(cchar *)iparts[1]
	))	;

	KBError		error	;
	QPixmap		pixmap	= KBAttrImage::pixmapFromLocation
				  (	getDocRoot(),
					iparts[0],
					iparts[1],
					error
				  )	;


	if (pixmap.isNull()) setError (error)   ;
	return	pixmap	;
}

/*  KBButton								*/
/*  setPixmaps	: Set button pixmaps					*/
/*  name1	: const QString & : First pixmap name			*/
/*  name2	: const QString & : Secons pixmap name			*/
/*  (returns)	: bool		  : Success				*/

bool	KBButton::setPixmaps
	(	const QString	&name1,
		const QString	&name2
	)
{
	QPixmap	pm1 = name1.isEmpty() ? QPixmap() : loadImage (name1) ;
	QPixmap	pm2 = name2.isEmpty() ? QPixmap() : loadImage (name2) ;

	if (!pm1.isNull() && !KBOptions::getNoButtonImages())
	{
		m_button->setPixmaps (pm1, pm2) ;
		return	true	;
	}

	m_button->setPixmaps (QPixmap(), QPixmap()) ; 
	return	false	;
}

/*  KBButton								*/
/*  loadPixmaps	: Load image into button				*/
/*  (returns)	: bool		: Success				*/

bool	KBButton::loadPixmaps ()
{
	QStringList images = QStringList::split (';', m_image.getValue()) ;

	if (images.count() == 0)
		return	setPixmaps (QString::null, QString::null) ;
	if (images.count() == 1)
		return	setPixmaps (images[0],	  QString::null) ;

	return	setPixmaps (images[0], images[1]) ;
}

/*  KBButton								*/
/*  clicked	: Handle button click					*/
/*  (returns)	: void		:					*/

void	KBButton::clicked ()
{
#if	! __KB_RUNTIME
	KBRecorder *recorder = KBRecorder::self() ;
	if ((recorder != 0) && recorder->isRecording(getDocRoot()))
		recorder->mouseClick (this, 0, QString::null) ;
#endif

	bool	evRc ;
	eventHook (m_onClick, 0, 0, evRc) ;
}

/*  KBButton								*/
/*  toggled	: Handle button toggle					*/
/*  (returns)	: void		:					*/

void	KBButton::toggled
	(	bool		on
	)
{
	bool	evRc	;
	KBValue	arg	= on	;

	eventHook (m_onToggle, 1, &arg, evRc) ;
}

/*  KBButton								*/
/*  keyStroke	: Handle keystroke					*/
/*  e		: QKeyEvent *	: Event to be handled			*/
/*  (returns)	: bool		: True if event consumed		*/

bool	KBButton::keyStroke
	(	QKeyEvent	*k
	)
{
	DPRINTF
	((	"KBButton::keyStroke: %d\n",
		k->key()
	))	;
	KBNavigator *navi = getNavigator() ;
	bool	    shift = (k->state () & Qt::ShiftButton) != 0 ;

	if (m_button != 0)
		switch (k->key())
		{
			case Qt::Key_Enter	:
			case Qt::Key_Return	:
				m_button->animateClick () ;
				return	true	;

			case Qt::Key_Tab	:
				if (navi != 0)
				{
					if (shift)
						navi->goPrevious (this, true) ;
					else	navi->goNext 	 (this, true) ;
				}
				return	true	;

			case Qt::Key_Backtab	:
				if (navi != 0)
					navi->goPrevious (this, true) ;
				return	true	;
			
			default	:
				break	;
		}

	return	false	;
}

/*  KBButton								*/
/*  setTabOrder	: Return tab order value				*/
/*  order	: int		: Tab order				*/
/*  (returns)	: void		:					*/

void	KBButton::setTabOrder
	(	int	order
	)
{
	m_tabOrd.setValue (order) ;
}

/*  KBButton								*/
/*  getTabOrder	: Return tab order value				*/
/*  (returns)	: int		: Tab order				*/

int	KBButton::getTabOrder ()
{
	return	m_tabOrd.getIntValue () ;
}

/*  KBButton								*/
/*  replicate	: Replicate this button					*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New button node			*/

KBNode	*KBButton::replicate
	(	KBNode	*_parent
	)
{
	return	new KBButton (_parent, this) ;
}

/*  KBButton								*/
/*  enumKBProperty							*/
/*		: Get list of properties				*/
/*  list	: QStringList &	: Property list				*/
/*  (returns)	: void		:					*/

void	KBButton::enumKBProperty
	(	QStringList	&list
	)
{
	list.append ("text") ;
	KBObject::enumKBProperty (list) ;
}

/*  KBButton								*/
/*  hasKBProperty: Check if object has this property			*/
/*  name	 : cchar *	: Property name				*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBButton::hasKBProperty
	(	cchar		*name
	)
{
	if (qstrcmp (name, "text") == 0)
		return	true ;

	return	KBObject::hasKBProperty (name) ;
}

/*  KBButton								*/
/*  getKBProperty: Get a property rekall value				*/
/*  name	 : cchar *	: Proprty name				*/
/*  value	 : KBValue &	: Return property value			*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBButton::getKBProperty
	(	cchar		*name,
		KBValue		&value
	)
{
	if (qstrcmp (name, "text") == 0)
	{
		value	= m_button->getValue() ;
		return	true ;
	}

	return	KBObject::getKBProperty (name, value) ;
}

/*  KBButton								*/
/*  setKBProperty: Set a rekall property value				*/
/*  name	 : cchar *	: Proprty name				*/
/*  value	 : KBValue	: Property value			*/
/*  (returns)	 : bool		: Success				*/

bool	KBButton::setKBProperty
	(	cchar		*name,
		const KBValue	&value
	)
{
	if (qstrcmp (name, "text") == 0)
	{
		m_button->setValue (value) ;
		return	true	;
	}

	return	KBObject::setKBProperty (name, value) ;
}

/*  KBButton								*/
/*  setText	: Set button text					*/
/*  text	: const QString & : New button text			*/
/*  (returns)	: void		  :					*/

void	KBButton::setText
	(	const QString	&text
	)
{
	m_button->setValue (KBValue(text)) ;
}

/*  KBButton								*/
/*  setToolTip	: Set tooltip text					*/
/*  text	: const QString & : New tooltip text			*/
/*  (returns)	: void		  :					*/

void	KBButton::setToolTip
	(	const QString	&tip
	)
{
	m_button->setToolTip (tip) ;
}

/*  KBButton								*/
/*  getText	: Get button text					*/
/*  (returns)	: QString	: Text					*/

QString	KBButton::getText ()
{
	return	m_button->getValue().getRawText() ;
}

/*  KBButton								*/
/*  getToggle	: See if this should be a toggle button			*/
/*  (returns)	: bool		: True if toggle button			*/

bool	KBButton::getToggle ()
{
	return	m_toggle.getBoolValue () ;
}

/*  KBButton								*/
/*  setOn	: Set the button on or off				*/
/*  on		: bool		: Required state			*/
/*  (returns)	: void		:					*/

void	KBButton::setOn
	(	bool		on
	)
{
	m_button->setOn (on) ;
}

/*  KBButton								*/
/*  isOn	: Get button on/off state				*/
/*  (returns)	: bool		: State					*/

bool	KBButton::isOn ()
{
	return	m_button->isOn () ;
}

/*  KBButton								*/
/*  recordVerifyState							*/
/*		: Record state for verification				*/
/*  (returns)	: void		:					*/

void	KBButton::recordVerifyText ()
{
#if	! __KB_RUNTIME
	if (m_button != 0)
	{
		KBRecorder *recorder = KBRecorder::self() ;
		if ((recorder != 0) && recorder->isRecording(getDocRoot()))
			recorder->verifyText
			(	this,
				0,
				m_button->getValue().getRawText()
			)	;
	}
#endif
}

#if	! __KB_RUNTIME

/*  KBButton								*/
/*  makeRecordPopup							*/
/*		: Construct recording popup menu			*/
/*  popup	: KBPopupMenu *	: Popup menu to add to			*/
/*  drow	: uint		: Display row number			*/
/*  parents	: bool		: Add submenus for parents		*/
/*  (returns)	: void		:					*/

void	KBButton::makeRecordPopup
	(	KBPopupMenu		*popup,
		uint			drow,
		bool			parents
	)
{
	popup->insertItem
	(	TR("Verify text"),
		this,
		SLOT(recordVerifyText())
	)	;

	KBObject::makeRecordPopup (popup, drow, parents) ;
}

/*  KBButton								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar *	: Initial property			*/
/*  (returns)	: bool		: Success				*/

bool	KBButton::propertyDlg
	(	cchar	*iniAttr
	)
{
	if (KBNode::basePropertyDlg ("Button", iniAttr))
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}
#endif


NEWNODE(Button, __TR("New &Button"), 0, KNF_FORM|KNF_BLOCK|KNF_STATIC)
