/***************************************************************************
    file	         : kb_check.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<stdio.h>
#include	<stdlib.h>
#include	<ctype.h>

#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>
#include	<qcursor.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_sizer.h"
#include	"kb_display.h"
#include	"kb_ctrl.h"
#include	"kb_formblock.h"
#include	"kb_qtcheck.h"
#include	"kb_nodereg.h"
#include	"kb_layout.h"

#if		__KB_AUTOMOC
#include	"kb_check.moc"
#else
#include	"kb_check.h"
#endif

#if		! __KB_RUNTIME
#include	"kb_itemdlg.h"
#endif

/*  KBCheck								*/
/*  KBCheck	: Constructor for simple form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBNode	:					*/

KBCheck::KBCheck
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBItem		(parent, "KBCheck",	"expr", aList),
	m_fgcolor	(this,   "fgcolor",	aList),
	m_bgcolor	(this,   "bgcolor",	aList),
	m_nullIsZero	(this,	 "nulliszero",	aList, KAF_GRPDATA),
	m_onChange	(this,	 "onchange",	aList, KAF_EVCS)
{
#if	! __KB_RUNTIME
	if (ok != 0)
	{
		KBPropDlg cDlg (this, "Field", m_attribs, 0) ;
		if (!cDlg.exec ())
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true     ;
	}
#endif
}

/*  KBCheck								*/
/*  KBCheck	: Constructor for simple form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  check	: KBCheck *	: Extant check				*/
/*  (returns)	: KBNode	:					*/

KBCheck::KBCheck
	(	KBNode		*parent,
		KBCheck		*check
	)
	:
	KBItem		(parent, "expr", 	check),
	m_fgcolor	(this,   "fgcolor",	check),
	m_bgcolor	(this,   "bgcolor",	check),
	m_nullIsZero	(this,	 "nulliszero",	check, KAF_GRPDATA),
	m_onChange	(this,	 "onchange",	check, KAF_EVCS)
{
}

/*  KBCheck								*/
/*  ~KBCheck	: Destructor for simple form field			*/
/*  (returns)	:		:					*/

KBCheck::~KBCheck ()
{
}

/*  KBCheck								*/
/*  makeCtrl	: Make a check control					*/
/*  drow	: uint		: Display row number			*/
/*  (returns)	: KBControl *	: Associated control			*/

KBControl *KBCheck::makeCtrl
	 (	uint	drow
	 )
{
	return	new KBCtrlCheck (getDisplay(), this, drow) ;
}

/*  KBCheck								*/
/*  replicate	: Replicate this check					*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New check node			*/

KBNode	*KBCheck::replicate
	(	KBNode	*parent
	)
{
	return	new KBCheck (parent, this) ;
}

/*  KBCheck								*/
/*  getValue	: Get value						*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCheck::getValue
	(	uint		qrow
	)
{
	KBValue	value	= KBItem::getValue (qrow) ;

	if (value.isNull ())
		if (m_nullIsZero.getBoolValue())
			return	KBValue  ("0", m_type) ;

	return	value	;
}

/*  KBCheck								*/
/*  isChecked	: See of control is checked				*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Checked				*/

bool	KBCheck::isChecked
	(	uint		qrow
	)
{
	KBControl *ctrl	 = ctrlAtQRow (qrow) ;
	return	ctrl == 0 ? false : ((KBCtrlCheck *)ctrl)->isChecked() ;
}


/*  KBCheck								*/
/*  setChecked	: Set checked state					*/
/*  qrow	: uint		: Query row				*/
/*  checked	: bool		: Required state			*/
/*  (returns)	: void		:					*/

void	KBCheck::setChecked
	(	uint		qrow,
		bool		checked
	)
{
	KBControl *ctrl	 = ctrlAtQRow (qrow) ;
	if (ctrl != 0) ((KBCtrlCheck *)ctrl)->setChecked(checked) ;
}

#if	! __KB_RUNTIME
/*  KBCheck								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar	*	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBCheck::propertyDlg
	(	cchar		*iniAttr
	)
{
	if (KBItemPropDlg (this, "CheckBox", m_attribs, iniAttr).exec())
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}
#endif

NEWNODE(Check, __TR("New Chec&k"), 0, KNF_FORM|KNF_BLOCK|KNF_BLOCK|KNF_DATA)
