/***************************************************************************
    file	         : kb_choice.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_formblock.h"

#if		__KB_AUTOMOC
#include	"kb_choice.moc"
#else
#include	"kb_choice.h"
#endif

#include	"kb_qtchoice.h"
#include	"kb_nodereg.h"
#include	"kb_layout.h"
#include	"kb_finddlg.h"
#include	"kb_popupmenu.h"
#include	"kb_recorder.h"


#if		! __KB_RUNTIME
#include	"kb_propdlg.h"
#endif

#include 	<qstringlist.h>


#define		__DEBUG_CHOICE	(0)
#if		__DEBUG_CHOICE
#include	"kb_dprintf.h"
#endif


/*  KBChoice								*/
/*  KBChoice	: Constructor for simple choice field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBNode	:					*/

KBChoice::KBChoice
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBItem		(parent, "KBChoice",	"master", aList),
	m_values	(this,   "values",	aList),
	m_nullval	(this,	 "nullval",	aList),
	m_nullOK	(this,	 "nullok",	aList),
	m_noblank	(this,	 "noblank",	aList, KAF_FORM|KAF_GRPDATA),
	m_editable	(this,	 "editable",	aList, KAF_FORM   ),
	m_fgcolor	(this,   "fgcolor",	aList),
	m_bgcolor	(this,   "bgcolor",	aList),
	m_font		(this,	 "font",	aList),
	m_morph		(this,   "morph",	aList, KAF_FORM),
	m_onChange	(this,	 "onchange",	aList, KAF_EVCS)
{
#if	! __KB_RUNTIME
	if (ok != 0)
	{
		if (!::choicePropDlg (this, "Choice", m_attribs))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true		;
	}
#endif
}

/*  KBChoice								*/
/*  KBChoice	: Constructor for simple choice field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  choice	: KBChoice *	: Extant choice				*/
/*  (returns)	: KBNode	:					*/

KBChoice::KBChoice
	(	KBNode		*parent,
		KBChoice	*choice
	)
	:
	KBItem		(parent,  "master", 	choice),
	m_values	(this,    "values",	choice),
	m_nullval	(this,	  "nullval",	choice),
	m_nullOK	(this,	  "nullok",	choice),
	m_noblank	(this,	  "noblank",	choice, KAF_FORM|KAF_GRPDATA),
	m_editable	(this,	  "editable",	choice, KAF_FORM),
	m_fgcolor	(this,    "fgcolor",	choice),
	m_bgcolor	(this,    "bgcolor",	choice),
	m_font		(this,	  "font",	choice),
	m_morph		(this,    "morph",	choice, KAF_FORM),
	m_onChange	(this,	  "onchange",	choice, KAF_EVCS)
{
}

/*  KBChoice								*/
/*  ~KBChoice	: Destructor for simple form field			*/
/*  (returns)	:		:					*/

KBChoice::~KBChoice ()
{
}

/*  KBChoice								*/
/*  setValues	: Set a new set of values				*/
/*  vals	: QStringList &	: Set of values				*/
/*  (returns)	: void		:					*/

void	KBChoice::setValues
	(	const QStringList	&vals
	)
{
	m_valset	= vals ;

	if (!m_noblank.getBoolValue())
		m_valset.insert (m_valset.begin(), m_nullval.getValue()) ;

	for (uint idx2 = 0 ; idx2 < m_ctrls.count() ; idx2 += 1)
		((KBCtrlChoice *)m_ctrls.at(idx2))->setValues (m_valset) ;
}

/*  KBChoice								*/
/*  loadValues	: Load value set into combo boxes			*/
/*  (returns)	: void		:					*/

void	KBChoice::loadValues ()
{
	m_valset  = QStringList::split ('|', m_values.getValue(), true) ;

	if (!m_noblank.getBoolValue())
		m_valset.insert (m_valset.begin(), m_nullval.getValue()) ;
}

/*  KBChoice								*/
/*  keyStroke	: Control has received keystroke			*/
/*  k		: QKeyEvent *	: Key event in question			*/
/*  (returns)	: bool		: Key event consumed			*/

bool	KBChoice::keyStroke
	(	QKeyEvent	*k
	)
{
	/* Override for key up and key down so that we can change	*/
	/* selection, otherwise these keys will navigate between	*/
	/* records.							*/
	if (k->type() == QEvent::KeyPress)
		if ((k->key() == Qt::Key_Up) || (k->key() == Qt::Key_Down))
			return	false	;

	return	KBItem::keyStroke (k) ;
}

/*  KBChoice								*/
/*  showAs	: Switch in and out of design mode			*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBChoice::showAs
	(	KB::ShowAs	mode
	)
{
	if (mode == KB::ShowAsData) loadValues  () ;
	KBItem::showAs (mode) ;
}

/*  KBChoice								*/
/*  makeCtrl	: Make a choice control					*/
/*  drow	: uint		: Display row number			*/
/*  (returns)	: KBControl *	: Associated control			*/

KBControl *KBChoice::makeCtrl
	  (	uint		drow
	  )
{
	return	new KBCtrlChoice (getDisplay(), this, drow) ;
}

/*  KBChoice								*/
/*  isMorphing	: Test if field is morphed				*/
/*  (returns)	: bool		: Morphed				*/

bool	KBChoice::isMorphing ()
{
	return	m_morph.getBoolValue() ;
}

/*  KBChoice								*/
/*  checkValid	: Check whether value can be saved in database		*/
/*  value	: const KBValue & : Text value				*/
/*  alowNull	: bool		  : Ignore not-null check		*/
/*  (returns)	: bool		  : True if OK				*/

bool	KBChoice::checkValid
	(	const KBValue	&value,
		bool		allowNull
	)
{
	KBError		err	;

	DPRINTF
	((	"KBChoice::checkValid: [%s]: an=%d no=%d ve=%d\n",
		value.getRawText().latin1(),
		allowNull,
		m_nullOK.getBoolValue(),
		value.isEmpty()
	))	;


	if (!allowNull)
		if (!m_nullOK.getBoolValue() && value.isEmpty())
		{
			setError
			(	KBError::Error,
				QString(TR("Value must be selected from list for %1")).arg(errorText()),
				QString::null,
				__ERRLOCN
			)	;
			return	false ;
		}

	if (!m_type->isValid (value.getRawText(), err, getName()))
	{
		setError (err) ;
		return	 false ;
	}

	return	true	;
}

/*  KBChoice	:							*/
/*  setCurrentItem							*/
/*		: Set currently selected item by index			*/
/*  qrow	: uint		: Query row				*/
/*  item	: uint		: Item index				*/
/*  (returns)	: void		:					*/

void	KBChoice::setCurrentItem
	(	uint		qrow,
		uint		item
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	if (ctrl != 0) ((KBCtrlChoice *)ctrl)->setCurrentItem (item) ;
}

/*  KBChoice	:							*/
/*  currentItem	: Get currently selected item index			*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: uint		: Item index				*/

uint	KBChoice::currentItem
	(	uint		qrow
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	return	ctrl == 0 ? 0 : ((KBCtrlChoice *)ctrl)->currentItem () ;
}

/*  KBChoice								*/
/*  replicate	: Replicate this choice					*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New choice node			*/

KBNode	*KBChoice::replicate
	(	KBNode	*_parent
	)
{
	return	new KBChoice (_parent, this) ;
}

/*  KBChoice								*/
/*  doSearch	: Do record search					*/
/*  (returns)	: void		:					*/

void	KBChoice::doSearch ()
{
	KBFindChoiceDlg findDlg (getFormBlock(), this, m_valset, m_valset) ;
	findDlg.exec () ;
}

/*  KBChoice								*/
/*  canBeInvalid: Check if control can be invalid			*/
/*  (returns)	: bool		: True if so				*/

bool	KBChoice::canBeInvalid ()
{
	return	! m_nullOK.getBoolValue () ;
}

/*  KBChoice								*/
/*  getValues	: Return current value set				*/
/*  (returns)	: const QStringList &	: Values			*/

const QStringList &KBChoice::getValues	()
{
	return	m_valset ;
}

/*  KBChoice								*/
/*  getNumValues: Return count of values				*/
/*  (returns)	: uint		: Count of values			*/

uint	KBChoice::getNumValues ()
{
	return	m_valset.count() ;
}

/*  KBChoice								*/
/*  canEdit	: Check if control is editable				*/
/*  (returns)	: bool		: True if editable			*/

bool	KBChoice::canEdit ()
{
	return	m_editable.getBoolValue	() ;
}

/*  KBItem								*/
/*  recordVerifyChoices							*/
/*  		: Record values verification for test			*/
/*  (returns)	: void		:					*/

void	KBChoice::recordVerifyChoices ()
{
#if	! __KB_RUNTIME
	KBRecorder *recorder = KBRecorder::self() ;
	if ((recorder == 0) || !recorder->isRecording(getDocRoot()))
		return	;

	KBCtrlChoice	*ctrl	= (KBCtrlChoice *)m_ctrls.at(m_recordDRow) ;
	RKComboBox	*combo	= (RKComboBox   *)ctrl->getWidget() ;

	QStringList	values	;
	for (int idx = 0 ; idx < combo->count() ; idx += 1)
		values.append (combo->text(idx)) ;

	recorder->verifyChoices
	(	this,
		m_recordDRow,
		values.join("|")
	)	;
#endif
}

#if	! __KB_RUNTIME

/*  KBChoice								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar *	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBChoice::propertyDlg
	(	cchar	*iniAttr
	)
{
	if (::choicePropDlg (this, "Choice", m_attribs, iniAttr))
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}

/*  KBChoice								*/
/*  makeRecordPopup							*/
/*		: Construct recording popup menu			*/
/*  popup	: KBPopupMenu *	: Popup menu to add to			*/
/*  drow	: uint		: Display row number			*/
/*  parents	: bool		: Add submenus for parents		*/
/*  (returns)	: void		:					*/

void	KBChoice::makeRecordPopup
	(	KBPopupMenu		*popup,
		uint			drow,
		bool			parents
	)
{
	popup->insertItem
	(	TR("Verify choices"),
		this,
		SLOT(recordVerifyChoices())
	)	;
	KBItem::makeRecordPopup (popup, drow, parents) ;
}

#endif

NEWNODE(Choice, __TR("New &Choice"), 0, KNF_FORM|KNF_BLOCK|KNF_DATA)
