/***************************************************************************
    file	         : kb_combowidget.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	<qwidgetstack.h>
#include	<qgroupbox.h>

#include	"kb_classes.h"
#include	"rk_vbox.h"
#include	"rk_combobox.h"

#if		__KB_AUTOMOC
#include	"kb_combowidget.moc"
#else
#include	"kb_combowidget.h"
#endif


/*  KBComboWidget							*/
/*  KBComboWidget: Constructor for list-based tab widget replacement	*/
/*  parent	: QWidget *	: Parent widget				*/
/*  (returns)	: KBComboWidget	:					*/

KBComboWidget::KBComboWidget
	(	QWidget		*parent
	)
	:
	QWidget		(parent)
{
	RKVBox		*layout	= new RKVBox	(parent) ;
	QGroupBox	*group	= new QGroupBox (1, Qt::Vertical, layout) ;

	m_stack	= new QWidgetStack   (group )	;
	m_combo	= new RKComboBox     (layout)	;

	connect
	(	m_combo,
		SIGNAL(activated     (int)),
		SLOT  (slotChangePage(int))
	)	;
}

/*  KBComboWidget							*/
/*  addPage	: Add a page to the widget				*/
/*  page	: QWidget *	  : Page to add				*/
/*  label	: const QString & : Text for pseudo-tab			*/
/*  icon	: const QPixmap & : Icon for pseudo-tab			*/
/*  (returns)	: void		  :					*/

void	KBComboWidget::addPage
	(	QWidget		*page,
		const QString	&label,
		const QPixmap	&
	)
{
	m_stack->addWidget	(page, m_pages.count()) ;
	m_combo->insertItem	(label) ;
	m_stack->setMinimumSize	(m_stack->minimumSize().expandedTo(page->minimumSize())) ;
	m_pages.append 		(page) ;
}

/*  KBComboWidget							*/
/*  addTab	: Add a page to the widget				*/
/*  page	: QWidget *	  : Page to add				*/
/*  label	: const QString & : Text for pseudo-tab			*/
/*  icon	: const QPixmap & : Icon for pseudo-tab			*/
/*  (returns)	: void		  :					*/

void	KBComboWidget::addTab
	(	QWidget		*page,
		const QString	&label,
		const QPixmap	&icon
	)
{
	/* Convenience function, aids compatability			*/
	addPage	(page, label, icon) ;
}

/*  KBComboWidget							*/
/*  slotChangePage							*/
/*		: User changes page					*/
/*  page	: int		: Page number				*/
/*  (returns)	: void		:					*/

void	KBComboWidget::slotChangePage
	(	int		page
	)
{
	if ((page >= 0) && (page < (int)m_pages.count()))
	{
		m_stack->raiseWidget (m_pages.at(page)) ;
		emit currentChanged  (m_pages.at(page)) ;
	}
}

void	KBComboWidget::setCurrentPage
	(	const QString	&name
	)
{
	for (uint idx = 0 ; idx < m_pages.count() ; idx += 1)
		if (m_pages.at(idx)->name() == name)
		{	m_combo->setCurrentItem (idx) ;
			m_stack->raiseWidget    (m_pages.at(idx)) ;
			break	;
		}
}

QWidget	*KBComboWidget::visibleWidget ()
{
	return	m_stack->visibleWidget() ;
}

