/***************************************************************************
    file	         : kb_complink.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<stdio.h>
#include	<qobjectlist.h>
#include	<qcursor.h>
#include	<qapplication.h>


#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_nodereg.h"
#include	"kb_docroot.h"
#include	"kb_component.h"
#include	"kb_parse.h"
#include	"kb_display.h"
#include	"kb_popupmenu.h"
#include	"kb_config.h"
#include	"kb_override.h"
#include	"kb_designpopup.h"
#include	"kb_slot.h"
#include	"kb_ascii.h"

#if		__KB_AUTOMOC
#include 	"kb_complink.moc"
#else		
#include 	"kb_complink.h"
#endif

#include	"tk_icons.h"


#define		__DEBUG_COMPLINK	(0)
#if		__DEBUG_COMPLINK
#include	"kb_dprintf.h"
#endif


#define	KAF_CONFIG	(KAF_GRPOTHER|KAF_CUSTOM|KAF_HIDDEN)


/*  KBCompLink								*/
/*  KBCompLink	: Constructor for linked component node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBCompLink	:					*/

KBCompLink::KBCompLink
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBFramer	(parent, aList,  	"KBCompLink"),
	m_server	(this,	 "server",	aList,	KAF_REQD|KAF_GRPDATA),
	m_component	(this,	 "component",	aList,	KAF_REQD|KAF_GRPDATA)
{
	KBError	error	;

	m_attrOver = new KBAttrStr (this, "override", "", KAF_CONFIG) ;

	if (!initialise(error))
		if (ok != 0)
		{
			error.DISPLAY()	;
			delete	this	;
			*ok	= false	;
			return	;
		}

	if (ok != 0) *ok = true ;
}

/*  KBCompLink								*/
/*  KBCompLink	: Constructor for component link node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  compLink	: KBCompLink *	: Extant component link			*/
/*  (returns)	: KBCompLink	:					*/

KBCompLink::KBCompLink
	(	KBNode		*parent,
		KBCompLink 	*compLink
	)
	:
	KBFramer	(parent, compLink),
	m_server	(this,	 "server",	compLink,	KAF_REQD|KAF_GRPDATA),
	m_component	(this,	 "component",	compLink,	KAF_REQD|KAF_GRPDATA)
{
	m_attrOver = new KBAttrStr (this, "override", "", KAF_CONFIG) ;
}

/*  KBCompLink								*/
/*  KBCompLink	: Destructor for component link node			*/
/*  (returns)	:		:					*/

KBCompLink::~KBCompLink ()
{
	DELOBJ	(m_attrOver) ;
}

/*  KBCompLink								*/
/*  initialise	: Initialise component					*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBCompLink::initialise
	(	KBError		&pError
	)
{
	QByteArray data	  ;
	QString	   server = m_server.getValue() ;

	if (server == "Self")
		server	= getDocRoot()->getDocLocation().server() ;

	KBLocation locn
		   (	getDocRoot()->getDBInfo(),
			"component",
			server,
			m_component.getValue(),
			"cmp"
		   )	;

	if (!locn.contents (data, pError))
		return	false	;


	KBLocation  doc		= getDocRoot()->getDocLocation () ;
	KBComponent *component	= KBOpenComponentText (doc, data, pError) ;
	if (component == 0)
		return	false	;


	/* The link gets the geometry management properties of the	*/
	/* component (but keeps its own settings within its parent).	*/
	const KBAttrGeom &geom = component->attrGeom() ;
	m_geom.set
	(	geom.manage (),
		geom.numRows(true),
		geom.numCols(true),
		geom.margin (),
		geom.spacing()
	)	;

	/* We have a valid component, so replicate the objects in the	*/
	/* component into this node, with the exception of any top-	*/
	/* level null query node, adjusted by the minimum position.	*/
	/* Also grab the configuration object for later use.		*/
	const QPtrList<KBNode> &cList	= component->getChildren() ;
	KBObject	    *oObj	;
	int		    minx	;
	int		    miny	;

	minPosition (cList, minx, miny) ;

	LITER
	(	KBNode,
		cList,
		child,

		if (child->isQryNull())
			continue ;

		if ((oObj = child->isObject()) == 0)
			continue ;

//		QRect r  = oObj->geometry() ;
//		r.moveBy (-minx, -miny) ;

		KBObject *nObj = oObj->replicate(this)->isObject() ;

		QRect r  = nObj->geometry() ;
		r.moveBy (-minx, -miny) ;
		nObj->setGeometry (r) ;
	)

	delete	component ;
	return	true ;
}

/*  KBCompLink								*/
/*  checkOverrides							*/
/*		: Check the overrides are valid				*/
/*  (returns)	: void		:					*/

void	KBCompLink::checkOverrides ()
{
	/* The configurations for the component may have been changed.	*/
	/* Add any new ones that are not known about, and remove any	*/
	/* that we defunct. This also handles the case where the user	*/
	/* add a linked component to a form or report.			*/
	QPtrList<KBConfig> configSet ;

	CITER
	(	Object,
		o,
		o->findAllConfigs (configSet, QString::null) ;
	)

	/* The nested loops scan the overrides, and look for the	*/
	/* corresponding config. If found, then remove the config from	*/
	/* its list; if not found then delete the override. At the end	*/
	/* all remaining overrides still have configs, and the config	*/
	/* list has new configs.					*/
	/* NOTE: This is horribly inneficient but we should only have	*/
	/*	 small numbers.						*/
	CITER
	(	Override,
		r,

		DPRINTF
		((	"KBCompLink::checkOverrides: extant [%s][%s]\n",
			(cchar *)r->path  (),
			(cchar *)r->attrib()
		))	;

		bool	found	= 0 ;
		LITER
		(	KBConfig,
			configSet,
			c,

			if ((c->path() == r->path()) && (c->attrib() == r->attrib()))
			{
				configSet.remove (c) ;
				found	= true	;
				break	;
			}
		)

		if (!found)
		{
			DPRINTF
			((	"KBCompLink::initialise: dropping [%s][%s]\n",
				(cchar *)r->path  (),
				(cchar *)r->attrib()
			))	;
			delete	r ;
		}
	)

	LITER
	(	KBConfig,
		configSet,
		c,

		DPRINTF
		((	"KBCompLink::initialise: adding [%s][%s]\n",
			(cchar *)c->path  (),
			(cchar *)c->attrib()
		))	;

		/* NOTE:						*/
		/* The new override is created in the disabled state so	*/
		/* that we do not change the behaviour of this		*/
		/* component as a result of adding a new configuration	*/
		/* entry to the target component.			*/
		new KBOverride
			(	this,
				c->ident (),
				c->path  (),
				c->attrib(),
				c->value (),
				false
			)	;
	)
}

/*  KBCompLink								*/
/*  endParse	: Post-parsing handler					*/
/*  (returns)	: KBNode *	: The node				*/

KBNode	*KBCompLink::endParse ()
{
	/* Check if the component configuration has changed. We do this	*/
	/* there since we at the time the constructor is called, any	*/
	/* child overrides will not yet exist.				*/
	checkOverrides () ;
	return	KBFramer::endParse() ;
}

/*  KBCompLink								*/
/*  replicate	: Replicate this component link				*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New component link node		*/

KBNode	*KBCompLink::replicate
	(	KBNode	*_parent
	)
{
	return	replicateBelow (new KBCompLink (_parent, this)) ;
}

/*  KBCompLink								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCompLink::showAs
	(	KB::ShowAs	mode
	)
{
	KBFramer::showAs (mode) ;

	/* We need to disable access to the widgets inside the		*/
	/* component but still process mouse events on the component	*/
	/* display widget itself. To do this we disable the top-level	*/
	/* widgets in the component, and but install an application	*/
	/* level event filter so we still get a look at the events	*/
	const QObjectList *children ;
	if ((children = m_frmDisp->getTopWidget()->children()) != 0)
	{
		QObjectListIt iter (*children) ;
		QObject	      *obj ;

		while ((obj = iter.current()) != 0)
		{
			++iter ;
			if (obj->isWidgetType() && !obj->isA("KBSizerBlob"))
				((QWidget *)obj)->setEnabled (mode == KB::ShowAsData) ;

		}
	}

	if (mode == KB::ShowAsDesign)
		qApp->installEventFilter (this) ;
	else	qApp->removeEventFilter  (this) ;

	if (mode == KB::ShowAsData)
		setOverrides	() ;
}

/*  KBCompLink								*/
/*  eventFilter	: Local event filter					*/
/*  o		: QObject *	: Associated object			*/
/*  e		: QEvent *	: Event					*/
/*  (returns)	: bool		: True if event consumed		*/

bool	KBCompLink::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	/* First skip anything that is not a widget. Can't think of	*/
	/* any cases but lets be safe.					*/
	if (!o->isWidgetType())
		return	false	;

	if (!m_frmDisp)
		return	false	;

	/* If the object is the components top-level widget then allow	*/
	/* normal processing, note that we will be sending events to	*/
	/* this widget below.						*/
	QWidget	*w	= m_frmDisp->getTopWidget() ;
	if (o == w)
		return	false	;

	/* Now see if the object has the component top-level widget as	*/
	/* an ancestor. If not then we will allow normal processing	*/
	/* to continue, except that we also allow normal processing on	*/
	/* top-level sizer blobs.					*/
	bool	found	= false	;
	for (QWidget *t = (QWidget *)o ; t != 0 ; t = t->parentWidget())
	{
		if (t == w)
		{	found	= true	;
			break	;
		}

		if ((t->parentWidget() == w) && t->isA("KBSizerBlob"))
			return	false	;
	}

	if (!found)
		return	false	;

	/* Next check on the events. Basically, are will handle		*/
	/* mouse clicks, kill mouse movement and allow other stuff	*/
	/* through as normal.						*/
	switch (e->type())
	{
		case QEvent::ContextMenu	:
			/* Context events get handled below ....	*/
			break	;

		case QEvent::MouseButtonPress	:
		case QEvent::MouseButtonRelease	:
		case QEvent::MouseButtonDblClick:
			/* .... and similarly mouse button presses.	*/
			break	;

		case QEvent::MouseMove		:
			/* Mouse movement is swallowed up since we will	*/
			/* never need to mark out inside the linked	*/
			/* component.					*/
			return	true	;

		default	:
			return	false	;
	}

	/* Context and mouse events are rerouted to the display widget	*/
	/* They are unchanged except that the widget position is fixed	*/
	/* up for the receiver.						*/
	if (e->type() == QEvent::ContextMenu)
	{
		QContextMenuEvent *c = (QContextMenuEvent *)e ;

		QContextMenuEvent c2
		(	c->reason   (),
			w->mapFromGlobal(c->globalPos()),
			c->globalPos(),
			c->state    ()
		)	;

		QApplication::sendEvent (w, &c2) ;
		return	true	;
	}

	QMouseEvent *m	= (QMouseEvent *)e ;

	QMouseEvent m2
	(	m->type	    (),
		w->mapFromGlobal(m->globalPos()),
		m->globalPos(),
		m->button   (),
		m->state    ()
	)	;

	QApplication::sendEvent (w, &m2) ;
	return	true	;
}

/*  KBCompLink								*/
/*  setOverrides: Set override values					*/
/*  (returns)	: void		:					*/

void	KBCompLink::setOverrides ()
{
	QStringList		missed	;
	QPtrList<KBNode>	subbed	;

	/* This is done in two passes (a) find override targets and (b)	*/
	/* substitute. It is done this way in case one override changes	*/
	/* an object name such that a subsequence override cannot find	*/
	/* that object. The ::findTarget call returns success, but the	*/
	/* KBOverride::substitute() method is implemented so that it is	*/
	/* safe to call it even if ::findTarget fails, whence it	*/
	/* returns an error.						*/
	CITER
	(	Override,
		o,

		if (o->enabled()) o->findTarget() ;
	)

	CITER
	(	Override,
		o,

		if (!o->enabled()) continue ;

		KBNode	*node = o->substitute() ;

		if (node == 0)
			missed.append(QString("%1: %2").arg(o->path()).arg(o->attrib())) ;
		else	subbed.append(node) ;

		DPRINTF
		((	"KBCompLink::setOverrides: [%p][%s][%s][%s]\n",
			(void  *)node,
			(cchar *)o->path  (),
			(cchar *)o->attrib(),
			(cchar *)o->value ()
		))	;
	)

	if (missed.count() > 0)
	{
		KBError::EError
		(	TR("Failed to find some attributes when linking component"),
			missed.join("\n"),
			__ERRLOCN
		)	;
		return	;
	}

	LITER
	(	KBNode,
		subbed,
		n,
		n->setupProperties()
	)
}

/*  KBCompLink								*/
/*  printNode	: Output node XML					*/
/*  text	: QString &	: Result string				*/
/*  indent	: int		: Indent level of this node		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBCompLink::printNode
	(	QString		&text,
		int		indent,
		bool		flat
	)
{
	if (flat)
	{
		QString	nodeText ;

		setOverrides ()  ;

		text	+= QString("%1<%2").arg("", indent).arg("KBContainer") ;
		for (uint idx = 0 ; idx < m_attribs.count () ; idx += 1)
		{	if (m_attribs.at(idx) == &m_server   ) continue ;
			if (m_attribs.at(idx) == &m_component) continue ;
			m_attribs.at(idx)->printAttr(text, nodeText, indent + 2, flat) ;
		}
		text	+= ">\n" ;

		for (uint idx1 = 0 ; idx1 < m_children.count () ; idx1 += 1)
			m_children.at(idx1)->printNode (text, indent + 2, flat) ;
		for (uint idx2 = 0 ; idx2 < m_slotList.count () ; idx2 += 1)
			m_slotList.at(idx2)->printNode (text, indent + 2, flat) ;

		text	+= nodeText ;
		text	+= QString("%1</%2>\n").arg("", indent).arg("KBContainer") ;
		return	;
	}

	QString	nodeText ;

	text	+= QString("%1<%2").arg("", indent).arg(getElement()) ;
	for (uint idx1 = 0 ; idx1 < m_attribs.count () ; idx1 += 1)
		m_attribs.at(idx1)->printAttr(text, nodeText, indent + 2, flat) ;
	text	+= ">\n" ;

	CITER
	(	Config,
		c,

		c->printNode (text, indent + 2, flat) ;
	)
	CITER
	(	Override,
		o,

		o->printNode (text, indent + 2, flat) ;
	)

	for (uint idx2 = 0 ; idx2 < m_slotList.count () ; idx2 += 1)
		m_slotList.at(idx2)->printNode (text, indent + 2, flat) ;

	text	+= nodeText ;
	text	+= QString("%1</%2>\n").arg("", indent).arg(getElement()) ;
}

/*  KBCompLink								*/
/*  sizeHint	: Get ideal size for object				*/
/*  (returns)	: QSize		: Size					*/

QSize	KBCompLink::sizeHint ()
{
	DPRINTF
	((	"KBCompLink::sizeHint: %s: %s\n",
		(cchar *)getName(),
		_TEXT(KBFramer::sizeHint())
	))	;

	return	KBFramer::sizeHint() ;
}

/*  KBCompLink								*/
/*  getPalette	: Get palette for this object				*/
/*  useParent	: bool		: Use parent's palette			*/
/*  (returns)	: QPalette *	: The palette				*/

QPalette *KBCompLink::getPalette
	 (	bool	
	 )
{
	return	KBFramer::getPalette (true) ;
}

#if	! __KB_RUNTIME

/*  KBCompLink								*/
/*  designPopup	: Handle design menu request				*/
/*  parent	: KBPopupMenu * : Possible parent menu			*/
/*  cell	: QRect		: Position settings			*/
/*  (returns)	: KBPopupMenu * : Popup menu				*/

KBPopupMenu
	*KBCompLink::designPopup
	(	KBPopupMenu	*parent,
		QRect		cell
	)
{
	KBPopupMenu *popupMain	= new KBPopupMenu (parent, &m_bState) ;
	KBPopupMenu *popupEdit	= new KBPopupMenu (popupMain) ;


	popupEdit->insertItem
	(	getSmallIcon("editcut"),
		TR("C&ut component"  ),
		this,
		SLOT(cutObj   	())
	)	;
	popupEdit->insertItem
	(	getSmallIcon("editcopy"),
		TR("&Copy component" ),
		this,
		SLOT(copyObj  	())
	)	;
	popupEdit->insertItem (TR("Delete component"),  this, SLOT(deleteObj	())) ;

	popupMain->setTitle   (this) ;
	popupMain->insertItem (TR("&Edit"), popupEdit) ;

	/* Add a menu option for the properties of this object itself.	*/
	popupMain->insertItem
	(	getSmallIcon("properties"),
		TR("Component properties"),
		this,
		SLOT(propertyDlg ())
	)	;

	if ((parent == 0) && (parentObject() != 0))
		makeAncestorPopup (popupMain, this) ;

	popupMain->insertSeparator() ;
	popupMain->insertItem
	(	getSmallIcon("info"),
		TR("&Information"),
		this,
		SLOT(whatsThis())
	)	;

	setCtrlRect (cell) ;
	return	popupMain  ;
}


/*  KBCompLink								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar *	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBCompLink::propertyDlg
	(	cchar	*iniAttr
	)
{
	if (::compLinkPropDlg (this, "Linked Component", m_attribs, iniAttr))
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}
#endif

NEWNODE(CompLink, (cchar *)0, 0, KNF_FORM|KNF_REPORT)
