/***************************************************************************
    file	         : kb_component.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#if		__KB_AUTOMOC
#include	"kb_component.moc"
#else
#include	"kb_component.h"
#endif

#include	"kb_param.h"
#include	"kb_dispscroller.h"
#include	"kb_sizer.h"
#include	"kb_formcopier.h"
#include	"kb_compinitdlg.h"
#include	"kb_nodereg.h"
#include	"kb_popupmenu.h"
#include	"kb_qrynull.h"
#include	"kb_slot.h"

#include	"kb_designpopup.h"

#define		__DEBUG_COMPONENT	(0)
#if		__DEBUG_COMPONENT
#include	"kb_dprintf.h"
#endif

/*  KBComponent								*/
/*  KBComponent	: Constructor for form root class for extant component	*/
/*  location	: KBLocation &	 : Form location			*/
/*  aList	: const QDict<QString> &				*/
/*				 : List of attributes			*/
/*  (returns)	: KBComponent	 :					*/

KBComponent::KBComponent
	(	const KBLocation	&location,
		const QDict<QString>	&aList
	)
	:
	KBBlock		(0,     aList,	   	"KBComponent"),
	KBNavigator	(this,   0,		m_children),
	KBLayout	(this),
	m_type		(this,  "type",    	aList,	KAF_REQD),
	m_language	(this,  "language",	aList   ),
	m_language2	(this,  "language2",	aList   ),
	m_skin		(this,  "skin",		aList	),
	m_docRoot	(this,  m_children,   	location)
{
	m_root	     = this 	;
	m_display    = 0    	;

	m_dx.setValue  ( 0)   	;
	m_dy.setValue  (20)   	;

	m_blkType    = BTNull	;
	new KBQryNull (this)	;

	switch (objType())
	{
		case KB::ObjForm :
			KBNode::m_flags	|= KNF_FORM   ;
			break	;

		case KB::ObjReport :
			KBNode::m_flags	|= KNF_REPORT ;
			break	;

		default	:
			break	;
	}

	m_geom.set	(KBAttrGeom::FMFixed, KBAttrGeom::FMFixed) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;
	DELOBJ	(m_attrConfig)	;
}

#if	! __KB_RUNTIME

/*  KBComponent								*/
/*  KBComponent	: Constructor for form root class for new component	*/
/*  location	: KBLocation &	 : Form location			*/
/*  aList	: const QDict<QString> &				*/
/*				 : List of attributes			*/
/*  ok		: bool &	 : Success				*/
/*  (returns)	: KBComponent	 :					*/

KBComponent::KBComponent
	(	const KBLocation	&location,
		const QDict<QString>	&aList,
		bool			&ok
	)
	:
	KBBlock		(0,     aList,		"KBComponent"),
	KBNavigator	(this,  0,		m_children),
	KBLayout	(this),
	m_type		(this,  "type",		aList,	KAF_REQD),
	m_language	(this,  "language",	aList   ),
	m_language2	(this,  "language2",	aList   ),
	m_skin		(this,  "skin",		aList	),
	m_docRoot	(this,  m_children,	location)
{
	m_root	     = this ;
	m_display    = 0    ;

	KBCompInitDlg	ciDlg (ok) ;
	if (!ciDlg.exec())
	{	ok	= false	;
		return	;
	}

	m_type    .setValue (ciDlg.type()) ;
	m_geom    .set	    (ciDlg.management(), 2, 2) ;
	m_language.setValue (ciDlg.language  ()) ;

	if (!propertyDlg  (0))
	{	ok	= false ;
		return	;
	}

	m_blkType    = BTNull ;
	new KBQryNull (this)  ;

	switch (objType())
	{
		case KB::ObjForm :
			KBNode::m_flags	|= KNF_FORM   ;
			break	;

		case KB::ObjReport :
			KBNode::m_flags	|= KNF_REPORT ;
			break	;

		default	:
			break	;
	}

	ok	= true	;
	KBLayout::setChanged() ;

	m_dx.setValue ( 0)   ;
	m_dy.setValue (20)   ;

	m_geom.set	(KBAttrGeom::FMFixed, KBAttrGeom::FMFixed) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;
	DELOBJ	(m_attrConfig) ;
}

#endif

/*  KBComponent								*/
/*  KBComponent	: Component replication constructor			*/
/*  parent	: KBNode *	: Parent element			*/
/*  extant	: KBComponent *	: Extant component			*/
/*  (returns)	: KBComponent	:					*/

KBComponent::KBComponent
	(	KBNode		*parent,
		KBComponent	*extant
	)
	:
	KBBlock		(parent, 		extant),
	KBNavigator	(this,   0,		m_children),
	KBLayout	(this),
	m_type		(this,  "type",		extant, KAF_REQD),  
	m_language	(this,  "language",	extant),
	m_language2	(this,  "language2",	extant),
	m_skin		(this,  "skin",		extant),
	m_docRoot	(this,	m_children,	extant->getDocRoot()->getDocLocation())
{
	m_root	     = this ;
	m_display    = 0    ;

	m_dx.setValue ( 0)  ;
	m_dy.setValue (20)  ;

	m_geom.set	(KBAttrGeom::FMFixed, KBAttrGeom::FMFixed) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;
	DELOBJ	(m_attrConfig) ;
}

/*  KBComponent								*/
/*  ~KBComponent	: Destructor for form root class			*/
/*  (returns)	:		:					*/

KBComponent::~KBComponent ()
{
	showMonitor ((RKListView *)0) ;
}

/*  KBComponent								*/
/*  printNode	: Output node XML					*/
/*  text	: QString &	: Resulting text			*/
/*  indent	: int		: Indent level of this node		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBComponent::printNode
	(	QString		&text,
		int		indent,
		bool		flat
	)
{
	/* Implement this specially, since we need to output the XML	*/
	/* header. Also output script stuff and paramaters near the	*/
	/* front.							*/
	extern	QString	kbXMLEncoding  () ;
	QString		nodeText ;

	text	+= QString
		   (	"<?xml version=\"1.0\" encoding=\"%1\"?>\n"
		   )
		   .arg	(kbXMLEncoding()) ;

	text	+= QString("%1<%2").arg("", indent).arg(getElement()) ;
	for (uint idx1 = 0 ; idx1 < m_attribs.count () ; idx1 += 1)
		m_attribs.at(idx1)->printAttr(text, nodeText, indent + 2, flat) ;
	text	+= ">\n" ;

	/* Print the child nodes, except that we skip the dummy null	*/
	/* query node.							*/
	LITER
	(	KBNode,
		m_children,
		n,
		if (n->isQryBase() == 0)
			n->printNode (text, indent + 2, flat)  ;
	)

	for (uint idx2 = 0 ; idx2 < m_slotList.count () ; idx2 += 1)
		m_slotList.at(idx2)->printNode (text, indent + 2, flat) ;

	text	+= nodeText ;
	text	+= QString("%1</%2>\n").arg("", indent).arg(getElement()) ;
}

#if	! __KB_RUNTIME

/*  KBComponent								*/
/*  showDesign	: Show form in design mode				*/
/*  parent	: QWidget *	  : Parent widget			*/
/*  size	: QSize &	  : Return required size		*/
/*  (returns)	: KB::ShowRC	  : Startup return code			*/

KB::ShowRC
	KBComponent::showDesign
	(	QWidget		*parent,
		QSize		&size
	)
{
	/* Start with the block setup. This links blocks to queries and	*/
	/* such. Any error here is a real problem.			*/
	if (!blockSetup ())
		return	KB::ShowRCError  ;

	/* If this is the first time the form is displayed, create the	*/
	/* top-level display and build the form GUI.			*/
	if (m_display == 0)
	{
		m_display = KBDisplay::newTopDisplay (parent, this, 0, 0, false) ;
		buildTopDisplay (m_display) ;
	}

	KBLayout::clear      (true)  ;
	KBLayout::initSizer  ()	;
	KBBlock ::showAs     (KB::ShowAsDesign) ;

	size	= geometry().size() ;
	size	+= QSize (100, 100) ;

	m_display->resizeContents (QSize(1600, 1600)) ;
	m_display->setDisplayGeometry (geometry()) ;

	setSizer (new KBSizer
			(	this, 
				m_display,
				m_display->getDisplayWidget()
		 )	) ;

	return	KB::ShowRCDesign ;
}

/*  KBComponent								*/
/*  showData	: Show component in data mode				*/
/*  parent	: QWidget *	  : Parent widget			*/
/*  size	: QSize &	  : Return required size		*/
/*  (returns)	: KB::ShowRC	  : Startup return code			*/

KB::ShowRC
	KBComponent::showData
	(	QWidget		*parent,
		QSize		&size
	)
{
	/* Start with the block setup. This links blocks to queries and	*/
	/* such. Any error here is a real problem.			*/
	if (!blockSetup ())
		return	KB::ShowRCError  ;

	/* If this is the first time the form is displayed, create the	*/
	/* top-level display and build the form GUI.			*/
	if (m_display == 0)
	{
		m_display = KBDisplay::newTopDisplay (parent, this, 0, 0, false) ;
		buildTopDisplay (m_display) ;
	}

	KBLayout::clear      (true)  ;
	KBLayout::initSizer  ()	;
	KBBlock ::showAs     (KB::ShowAsPreview) ;

	size	= geometry().size() ;
	m_display->resizeContents (size) ;

	return	KB::ShowRCData ;
}
#endif

/*  KBComponent								*/
/*  setGeometry	: Set position of component in parent			*/
/*  rect	: const QRect &	: New position				*/
/*  (returns)	: void		:					*/

void	KBComponent::setGeometry
	(	const QRect	&rect
	)
{
	KBBlock::setGeometry (rect) ;
}

#if	! __KB_RUNTIME

/*  KBComponent								*/
/*  makeNewPopup: Make a new-thingy popup menu for a framer		*/
/*  parent	: KBPopupMenu *	: Parent popup menu			*/
/*  rect	: QRect		: New object area			*/
/*  (returns)	: void		:					*/

KBPopupMenu
	*KBComponent::makeNewPopup
	(	KBPopupMenu	*parent,
		QRect		rect
	)
{
	setCtrlRect (rect)	;

	switch (objType())
	{
		case KB::ObjForm	:
			return	makeFormNewPopup   (parent, rect, this, &m_bState) ;

		case KB::ObjReport	:
			return	makeReportNewPopup (parent, rect, this, &m_bState) ;

		default	:
			break	;
	}

	return	0		;
}

/*  KBComponent								*/
/*  designPopup	: Handle design menu request				*/
/*  parent	: KBPopupMenu * : Possible parent menu			*/
/*  cell	: QRect		: Position information			*/
/*  (returns)	: KBPopupMenu * : Popup menu				*/

KBPopupMenu
	*KBComponent::designPopup
	(	KBPopupMenu	*parent,
		QRect		cell
	)
{
	KBPopupMenu *popupMain	= new KBPopupMenu (parent, &m_bState) ;
	KBPopupMenu *popupEdit	= new KBPopupMenu (popupMain) ;
	KBPopupMenu *popupNew	= makeNewPopup    (popupMain, cell) ;

	KBNode	*dummy	;
	bool	noIns	= !KBFormCopier::self()->anyCopied(dummy) ;
	bool	noItem	= true	   ;
	CITER (Item, i, noItem = false ; break )

	popupEdit->insertEntry (noIns,  TR("&Paste objects"),  this, SLOT(pasteObjects	())) ;
	popupEdit->insertEntry (noIns,  TR("Paste &here"),     this, SLOT(pasteHere	())) ;

	popupMain->setTitle    (this) ;
	popupMain->insertItem  (TR("&Edit"), popupEdit) ;
	popupMain->insertItem  (TR("&New"),  popupNew ) ;

	if (isDynamic())
	{
		extern void makeDynamicPopup(KBPopupMenu *, KBObject *) ;
		KBPopupMenu *popupDynamic = new KBPopupMenu (popupMain) ;
		makeDynamicPopup (popupDynamic, this) ;
		popupMain->insertItem	(TR("Dynamic layout"), popupDynamic) ;
	}

	popupMain->insertEntry
	(	noItem,
		TR  ("Set tab order"),
		this,
		SLOT(newTabOrder())
	)	;

//	popupMain->insertItem
//	(	TR("Block &properties"),
//		this,
//		SLOT(propertyDlg())
//	)	;

	popupMain->insertItem
	(	TR("&Component properties"),
		this,
		SLOT(docPropDlg ())
	)	;

	if ((parent == 0) && (parentObject() != 0))
		makeAncestorPopup (popupMain, this) ;


	return	popupMain ;
}

/*  KBComponent								*/
/*  compProdDlg	: Run root property dialog				*/
/*  iniAttr	: cchar *	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBComponent::compPropDlg
	(	cchar	*iniAttr
	)
{
	QPtrList<KBParam > params  ;

	KBAttrStr	pattr	(this, "paramlist", "") ;

	CITER
	(	Param,
		p,
		params .append (p)
	)

	if (!::componentPropDlg (this, "Component", m_attribs, params, iniAttr))
		return false ;

	KBLayout::setChanged ()	;
	return	true	;
}

#endif

/*  KBComponent								*/
/*  replicate	: Replicate this node					*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: Replicated component			*/

KBNode	*KBComponent::replicate
	(	KBNode	*_parent
	)
{
	return	replicateBelow (new KBComponent (_parent, this)) ;
}

/*  KBComponent								*/
/*  newNullBlock: Add a new menu block					*/
/*  (returns)	: void		:					*/

void	KBComponent::newNullBlock ()
{
#if	! __KB_RUNTIME
	newFormBlock	(m_blkDisp, KBBlock::BTNull) ;
#endif
}

/*  KBComponent								*/
/*  newTableBlock: Add a new table block				*/
/*  (returns)	 : void		:					*/

void	KBComponent::newTableBlock ()
{
#if	! __KB_RUNTIME
	newFormBlock	(m_blkDisp, KBBlock::BTTable) ;
#endif
}

/*  KBComponent								*/
/*  newQueryBlock: Add a new query block				*/
/*  (returns)	 : void		:					*/

void	KBComponent::newQueryBlock ()
{
#if	! __KB_RUNTIME
	newFormBlock	(m_blkDisp, KBBlock::BTQuery) ;
#endif
}

/*  KBComponent								*/
/*  newSQLBlock	: Add a new SQL block					*/
/*  (returns)	: void		:					*/

void	KBComponent::newSQLBlock ()
{
#if	! __KB_RUNTIME
	newFormBlock	(m_blkDisp, KBBlock::BTSQL) ;
#endif
}

/*  KBComponent								*/
/*  newContainer : Add a new container					*/
/*  (returns)	 : void		:					*/

void	KBComponent::newContainer ()
{
#if	! __KB_RUNTIME
	KBObject::newContainer (m_display) ;
#endif
}

/*  KBComponent								*/
/*  newNode	: Add a new node					*/
/*  id		: int		: Actually NodeSpec pointer for node	*/
/*  (returns)	: KBNode *	: The new node				*/

KBNode	*KBComponent::newNode
	(	int		id
	)
{
#if	! __KB_RUNTIME
	return	KBNavigator::newNode
		(	idToNodeSpec(id),
			KBObject::autoCtrlRect (false),
			this,
			m_blkDisp,
			m_bState
		)	;
#else
	return	0	 ;
#endif
}

/*  KBComponent								*/
/*  pasteObjects: Paste in cut or copied objects			*/
/*  (returns)	: void		:					*/

void	KBComponent::pasteObjects ()
{
#if	! __KB_RUNTIME
	insertObjects (m_blkDisp) ;
#endif
}

/*  KBComponent								*/
/*  pasteHere	: Paste in cut or copied objects at specified position	*/
/*  (returns)	: void		:					*/

void	KBComponent::pasteHere ()
{
#if	! __KB_RUNTIME
	insertHere
	(	KBFormCopier::self()->getCopied (objType()),
		m_display,
		newCtrlRect()
	)	;
#endif
}

/*  KBComponent								*/
/*  pasteComponent							*/
/*		: Paste component object				*/
/*  (returns)	: void		:					*/

void	KBComponent::pasteComponent ()
{
#if	! __KB_RUNTIME
	insertComponent (m_display, newCtrlRect(), true) ;
#endif
}

/*  KBComponent								*/
/*  linkComponent							*/
/*		: Link component object					*/
/*  (returns)	: void		:					*/

void	KBComponent::linkComponent ()
{
#if	! __KB_RUNTIME
	insertComponent (m_display, newCtrlRect(), false) ;
#endif
}

/*  KBComponent								*/
/*  objType	: Get object type for this node				*/
/*  (returns)	: KB::ObjType	: Object type				*/

KB::ObjType KBComponent::objType ()
{
	return	(KB::ObjType)m_type.getIntValue() ;
}

KBValue	KBComponent::getValue
	(	uint
	)
{
	return	KBValue() ;
}

bool	KBComponent::setValue
	(	uint		,
		const KBValue	&
	)
{
	return	true	;
}

KBValue	*KBComponent::getBlockVal ()
{
	static	KBValue	dummy	;
	return	&dummy	;
}

uint	KBComponent::rowsInBlock ()
{
	return	1 ;
}

void	KBComponent::docPropDlg	()
{
#if	! __KB_RUNTIME
	compPropDlg (0) ;
#endif
}

/*  KBComponent								*/
/*  newTabOrder	: Invoke tab order dialog				*/
/*  (returns)	: void		:					*/

void	KBComponent::newTabOrder ()
{
#if	! __KB_RUNTIME
	if (KBNavigator::newTabOrder()) getLayout()->setChanged() ;
#endif
}
