/***************************************************************************
    file	         : kb_component.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_COMPONENT_H
#define	_KB_COMPONENT_H

#include	"libkbase_exports.h"

#include	"kb_framer.h"
#include	"kb_attrlanguage.h"
#include	"kb_attrskin.h"
#include	"kb_layout.h"
#include	"kb_docroot.h"



/*  KBComponent								*/
/*  ----------								*/
/*  This node is the root of a particular XML component definition. It	*/
/*  handles things which are global to the component, as well as	*/
/*  representing the top-most block.					*/

class	LIBKBASE_API	KBComponent : public KBBlock, public KBNavigator, public KBLayout
{
	Q_OBJECT

	KBAttrUInt	m_type		;	/* Type (form, ...)	*/
	KBAttrLanguage	m_language	;	/* Script language	*/
	KBAttrLanguage	m_language2	;	/* 2nd script language	*/
	KBAttrSkin	m_skin		;	/* Skin name		*/

	KBDocRoot	m_docRoot	;	/* Common document	*/
	KBDisplay	*m_display	;	/* Top-level display	*/


public	:

	KBComponent	(const KBLocation &, const QDict<QString> &) ;
	KBComponent	(const KBLocation &, const QDict<QString> &, bool &) ;
	KBComponent	(KBNode     *, KBComponent *) ;
virtual~KBComponent	() ;


#if	! __KB_RUNTIME
	virtual	KBPopupMenu	*designPopup	(KBPopupMenu *,  QRect) ;
	virtual	KBPopupMenu	*makeNewPopup	(KBPopupMenu *,  QRect) ;
	virtual	bool		compPropDlg	(cchar *) ;
#endif

	virtual	KBComponent	*isComponent	() { return this       ; }
	virtual	KBLayout	*isLayout	() { return this       ; }
	virtual	KBDocRoot	*isDocRoot 	() { return &m_docRoot ; }

	virtual	KB::ObjType	objType		() ;

	virtual	KBNode		*replicate	(KBNode *) ;

	virtual	KBValue		getValue	(uint)	;
	virtual	bool		setValue	(uint, const KBValue &) ;
	virtual	KBValue		*getBlockVal	()	;
	virtual	uint		rowsInBlock	()	;
	virtual void		docPropDlg	()	;


	virtual	void		printNode	(QString &, int, bool) 	;

	virtual	void		setGeometry	(const QRect  &) ;

#if	! __KB_RUNTIME
	KB::ShowRC		showDesign	(QWidget *, QSize &) ;
	KB::ShowRC		showData	(QWidget *, QSize &) ;
	KB::ShowRC		showPrint	(KBWriter *) ;
#endif

	inline	KBDisplay	*getDisplay	()
	{
		return	m_display ;
	}
	inline	KB::ObjType	getType	()
	{
		return	(KB::ObjType)m_type.getIntValue() ;
	}


public	slots	:

	virtual KBNode		*newNode	(int)	;

	virtual void		pasteObjects	()	;
	virtual void		pasteHere	()	;
	virtual void		pasteComponent	()	;
	virtual void		linkComponent	()	;
	virtual	void		newTabOrder	()	;

public	:

	friend	class		KBBlock	;

	virtual	void		newNullBlock	()	;
	virtual	void		newTableBlock	()	;
	virtual	void		newSQLBlock	()	;
	virtual	void		newQueryBlock	()	;
	virtual	void		newContainer	()	;
}	;


extern	LIBKBASE_API	bool	componentPropDlg
			(	KBComponent	*,
				cchar		*,
				QPtrList<KBAttr  > &,
				QPtrList<KBParam > &,
				cchar		* = 0
			)	;
#endif	// _KB_COMPONENT_H
