/***************************************************************************
    file	         : kb_ctrl.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_ctrl.moc"
#else
#include	"kb_ctrl.h"
#endif

#include	"kb_block.h"
#include	"kb_display.h"
#include	"kb_writer.h"
#include	"kb_ascii.h"
#include	"kb_nodemonitor.h"
#include	"kb_sizer.h"


#include	<qscrollview.h>
#include	<qobjectlist.h>
#include	<qtimer.h>


#define		__DEBUG_CONTROL	(0)
#if		__DEBUG_CONTROL
#include	"kb_dprintf.h"
#endif


/*  KBControl								*/
/*  KBControl	: Constructor for KBase QT widget base class		*/
/*  widget	: QWidget *	: Actual Qt widget			*/
/*  display	: KBDisplay *	: Owning display			*/
/*  object	: KBObject *	: Owning object				*/
/*  (returns)	: KBCtrlHelper	:					*/

KBControl::KBControl
	(	KBDisplay	*display,
		KBObject	*object
	)
	:
	RKNotifyFilter	(0),		// display->getDisplayWidget()),
	m_display	(display),
	m_object	(object),
	m_item		(0),
	m_drow		(0)
{
	init		() ;
}

/*  KBControl								*/
/*  KBControl	: Constructor for KBase QT widget base class		*/
/*  widget	: QWidget *	: Actual Qt widget			*/
/*  display	: KBDisplay *	: Owning display			*/
/*  item	: KBItem *	: Owning item				*/
/*  drow	: uint		: Display row				*/
/*  (returns)	: KBCtrlHelper	:					*/

KBControl::KBControl
	(	KBDisplay	*display,
		KBItem		*item,
		uint		drow
	)
	:
	RKNotifyFilter	(0),		// display->getDisplayWidget()),
	m_display	(display),
	m_object	(item),
	m_item		(item),
	m_drow		(drow)
{
	init		()	 ;
}

/*  KBControl								*/
/*  ~KBControl	: Destructor for KBase QT widget base class		*/
/*  (returns)	: void		:					*/

KBControl::~KBControl ()
{
	DPRINTF
	((	"KBControl::~KBControl: %s: m_d=%d m_w=%p\n",
		m_object->getName().latin1(),
		m_drow,
		(void *)m_widget
	))	;

	if (m_widget == 0)
	{
		/* The control may have been morphed, and is being	*/
		/* destroyed as the display is shrunk, so we need to	*/
		/* clear the area that it occupied.			*/
		QWidget	  *bg = m_display->getDisplayWidget() ;
		if (bg != 0)
		{
			QPainter p (bg) ;
			QRect	 rRect  = m_rect ;
			m_display->cvtCtrlToView(rRect) ;
			p.eraseRect             (rRect) ;
		}
	}

	if (m_object != 0)
		m_object->ctrlGone (this) ;
	if (m_layoutItem != 0)
		m_layoutItem->dropValidator () ;

	DELOBJ	(m_widget     ) ;
	DELOBJ	(m_monitor    ) ;
}

/*  KBControl								*/
/*  init	: Common initialisation					*/
/*  (returns)	: void		:					*/

void	KBControl::init ()
{
	m_widget	= 0	;
	m_showing	= KB::ShowAsUnknown ;
	m_monitor	= 0	;
	m_lastState	= 0	;
	m_enabled	= true	;
	m_visible	= true	;
	m_hidden	= false	;
	m_inSetValue	= false	;
	m_layoutItem	= 0	;
	m_allWidgets	= 0	;
	m_dynamic	= m_object->parentIsDynamic() ;
	m_morphTimer	= 0	;
}

/*  KBControl								*/
/*  setupWidget	: Set up the Qt widget being used			*/
/*  widget	: QWidget *	     : Widget in question		*/
/*  rtti	: KBLayoutItem::RTTI : Layout type information		*/
/*  (returns)	: void		     :					*/

void	KBControl::setupWidget
	(	QWidget			*widget,
		KBLayoutItem::RTTI	rtti
	)
{
	const KBAttrGeom &geom = m_object->attrGeom() ;

	/* Note the widget, create a layout item to handle its		*/
	/* geometry, and insert the widget into the display (via the	*/
	/* layout item). Note that the layout item RTTI value must be	*/
	/* set *before* insertion.					*/
	m_widget	= widget	;
	m_layoutItem	= new KBCtrlLayoutItem
				(	m_widget,
					m_display,
					geom,
					m_object->parentIsDynamic() ?
						KBAttrGeom::MgmtDynamic :
						KBAttrGeom::MgmtStatic,
					m_drow,
					m_object->getBlock()
				)	;

	m_layoutItem->setCtrlGeometry (m_rect) ;
	m_layoutItem->setRtti	      (rtti)   ;
	m_display   ->insertWidget    (m_layoutItem) ;


	m_widget->setPalette (*m_object->getPalette(false)) ;
	m_widget->setFont    (*m_object->getFont   (false)) ;

	DPRINTF
	((	"KBControl::setupWidget: %s: m_dynamic=%d\n",
		m_object->getName().latin1()
	))	;

	if (m_dynamic)
	{
		uint	minw = geom.minimumWidth () ;
		uint	maxw = geom.maximumWidth () ;
		uint	minh = geom.minimumHeight() ;
		uint	maxh = geom.maximumHeight() ;

		DPRINTF
		((	"KBControl::setupWidget: %s: minw=%d maxw=%d minh=%d maxh=%d\n",
			m_object->getName().latin1(),
			minw,
			maxw,
			minh,
			maxh
		))	;

		if (maxw == 0) maxw = 4096 ;
		if (maxh == 0) maxh = 4096 ;
		m_widget->setMinimumWidth (minw) ;
		m_widget->setMaximumWidth (maxw) ;
		m_widget->setMinimumHeight(minh) ;
		m_widget->setMaximumHeight(maxh) ;
	}

	/* All controls get tab and click focus. This is important,	*/
	/* not just for user convenience, but to make update work!	*/
	m_widget ->setFocusPolicy (QWidget::StrongFocus) ;


	/* Nasty bit. The widget may have children which we need to	*/
	/* filter on to handle focus changes and the like. So, to make	*/
	/* life easy (and reliable) use brute force and find all	*/
	/* such widgets.						*/
	m_allWidgets = m_widget->queryList ("QWidget", 0, false, true) ;
	if (m_allWidgets == 0) m_allWidgets = new QObjectList ;
	m_allWidgets->append (m_widget) ;

	for
	(	QObjectListIt iter (*m_allWidgets) ;
		iter.current() != 0	;
		++iter
	)
	{
		iter.current()->installEventFilter (this) ;
		RKApplication::self()->installMousePressFilter (iter.current(), this) ;
	}

	m_widget->setEnabled (m_enabled) ;
	if (m_visible)
		m_widget->show () ;
	else	m_widget->hide () ;
}

/*  KBControl								*/
/*  clearWidget	: Clear down widget					*/
/*  (returns)	: void		:					*/

void	KBControl::clearWidget ()
{
	if (m_widget)
	{
		/* Remove filters installed above. We can safely use	*/
		/* the same list as we know that we are destroyed	*/
		/* before the widgets. Also nasty.			*/
		for
		(	QObjectListIt iter (*m_allWidgets) ;
			iter.current() != 0	;
			++iter
		)
		{
			iter.current()->removeEventFilter (this) ;
			RKApplication::self()->removeMousePressFilter (iter.current()) ;
		}

		delete	m_allWidgets	;
		delete	m_widget	;
		delete	m_layoutItem	;

		m_allWidgets = 0	;
		m_widget     = 0	;
		m_layoutItem = 0	;
	}
	
}

/*  KBControl								*/
/*  startMorphTimer							*/
/*		: Start morph timer to enmorph the control		*/
/*  (returns)	: void		:					*/

void	KBControl::startMorphTimer ()
{
	/* Create and wire up the morph timer on demand ...		*/
	if (m_morphTimer == 0)
	{
		m_morphTimer = new QTimer (this) ;

		connect
		(	m_morphTimer,
			SIGNAL	(timeout	()),
			SLOT	(clearWidget	())
		)	;
	}

	m_morphTimer->start (0, true) ;
}

/*  KBControl								*/
/*  stopMorphTimer							*/
/*		: Cancel enmorph timeer					*/
/*  (returns)	: void		:					*/

void	KBControl::stopMorphTimer ()
{
	if (m_morphTimer != 0)
		m_morphTimer->stop  () ;
}

/*  KBControl								*/
/*  getDisplay	: Get underlying display				*/
/*  (returns)	: KBDisplay *	: Display				*/

KBDisplay *KBControl::getDisplay ()
{
	return	m_display ;
}

/*  KBControl								*/
/*  getWidget	: Get top-level widget					*/
/*  (returns)	: QWidget *	: Widget				*/

QWidget	*KBControl::getWidget ()
{
	return	m_widget  ;
}

/*  KBControl								*/
/*  getIniValue	: Get initial value					*/
/*  (returns)	: KBValue	: Initial value				*/

KBValue	KBControl::getIniValue ()
{
	return	m_item == 0 ?
			KBValue() :
			m_item->getIniValue (m_item->getBlock()->getCurDRow() + m_drow) ;
}

/*  KBControl								*/
/*  eventFilter	: Widget event filter					*/
/*  obj		: QObject *	: Object originating event		*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: False to continue processing		*/

bool	KBControl::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	DPRINTF
	((	"KBControl::eventFilter: %d: %s: m=%d\n",
		e->type(),
		o->className(),
		m_showing
	))	;

#if	! __KB_RUNTIME
	/* Do nothing if in design mode; the sizer will install an	*/
	/* event filter to do the work.					*/
	if (m_showing == KB::ShowAsDesign)
		return	false	;
#endif

	/* Focus in event. Pass this to the object, so that is needed	*/
	/* the owning block can keep track of the current row. The	*/
	/* event is also processed normally.				*/
	if (e->type() == QEvent::FocusIn)
	{
		DPRINTF
		((	"KBControl::eventFilter: %s: r=%d\n",
			(cchar *)m_object->getName(),
			QFocusEvent::reason()
		))	;

		m_object->focusInEvent (m_drow, QFocusEvent::reason()) ;
		return	false ;
	}

	/* A right-button press may be used to bring up a context menu	*/
	/* for the associated object.					*/
	if (e->type() == QEvent::MouseButtonPress)
	{
		if (((QMouseEvent *)e)->stateAfter() & QMouseEvent::RightButton)
		{	m_object->contextMenu ((QMouseEvent *)e, m_drow) ;
			return	true ;
		}
		return	false	;
	}

	/* General double-click trapping so that all controls can pick	*/
	/* up duble-click events.					*/
	if (e->type() == QEvent::MouseButtonDblClick)
		if (m_item != 0)
			if (m_item->doDblClick (m_item->getBlock()->getCurDRow() + m_drow))
				return	true ;


	/* Last case is a key press. These are trapped to handle such	*/
	/* stuff primarily as form navigation.				*/
	if (e->type() == QEvent::KeyPress)
		return	m_object->keyStroke ((QKeyEvent *)e) ;


	/* Anything else is handled by the underlying widget ...	*/
	return	false	;
}

/*  KBControl								*/
/*  notifyFilter: Notification filter					*/
/*  o		: QObject *	: Object to receive event		*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBControl::notifyFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (m_showing == KB::ShowAsData)
		if (e->type() == QEvent::MouseButtonPress)
		{
			DPRINTF
			((	"KBCtrl::notifyFilter: called: hasFocus=%d\n",
				m_widget->hasFocus()
			))	;

			if (m_widget->hasFocus())
				return	false ;

			if (m_object->moveFocusOK (m_drow))
				return	false ;

			DPRINTF
			((	"KBCtrl::notifyFilter: moveFocusOK failed\n"
			))	;
			return	true ;
		}

	return	false	;
}

/*  KBControl								*/
/*  updateMorph	: Update morphed control				*/
/*  (returns)	: void		:					*/

void	KBControl::updateMorph ()
{
	QRect	aRect	= m_rect ;

	DPRINTF
	((	"KBControl::updateMorph: aRect=%s m_object=%p m_display=%p\n",
		_TEXT(aRect),
		(void *)m_object,
		(void *)m_display
	))	;

	if (getDisplay() != 0)
	{
		aRect	= m_object->attrGeom().mapGeometry (getDisplay()->getTopRect(), aRect) ;
		getDisplay()->updateMorph (m_item, aRect) ;
	}
}

/*  KBControl								*/
/*  setCtrlGeometry							*/
/*		: Set control geometry					*/
/*  rect	: const QRect &	: Position and size			*/
/*  (returns)	: void		:					*/

void	KBControl::setCtrlGeometry
	(	const QRect	&rect,
		int		align
	)
{
	DPRINTF
	((	"KBControl::setCtrlGeometry: [%s:%d] %s\n",
		(cchar *)m_object->getName(),
		m_drow,
		_TEXT(rect)
	))	;


	KBBlock	*block = m_object->getBlock() ;
	m_rect = rect ;
	m_rect.moveBy
	(	block->getDisplayDX() * m_drow,
		block->getDisplayDY() * m_drow
	)	;

	if (m_layoutItem != 0)
	{
		m_layoutItem->setCtrlGeometry (m_rect) ;
		if (align != -1)
			m_layoutItem->setAlignment (align) ;

		m_display->insertWidget (m_layoutItem) ;
	}

	if (m_widget == 0)
	{
		updateMorph () ;
		return	;
	}

	if (m_dynamic)
	{
		const KBAttrGeom &geom = m_object->attrGeom() ;

		uint	minw = geom.minimumWidth () ;
		uint	maxw = geom.maximumWidth () ;
		uint	minh = geom.minimumHeight() ;
		uint	maxh = geom.maximumHeight() ;

		DPRINTF
		((	"KBControl::setCtrlGeometry: %s: minw=%d maxw=%d minh=%d maxh=%d\n",
			m_object->getName().latin1(),
			minw,
			maxw,
			minh,
			maxh
		))	;

		if (maxw == 0) maxw = 4096 ;
		if (maxh == 0) maxh = 4096 ;
		m_widget->setMinimumWidth (minw) ;
		m_widget->setMaximumWidth (maxw) ;
		m_widget->setMinimumHeight(minh) ;
		m_widget->setMaximumHeight(maxh) ;
	}

}

/*  KBControl								*/
/*  ctrlGeometry: Get control geometry					*/
/*  (returns)	: QRect		: Geometry				*/

QRect	KBControl::ctrlGeometry ()
{
	return	m_rect	;
}

/*  KBControl								*/
/*  setPalette	: Set palette on widget					*/
/*  pal		: const QPalette *: Palette in question			*/
/*  (returns)	: void		  :					*/

void	KBControl::setPalette
	(	const QPalette *pal
	)
{
	m_morphPalette	= *pal	;

	if (m_widget)
	{
		DPRINTF
		((	"KBControl::setPalette: %s: bg=%s fg=%s\n",
			m_widget->className(),
			pal->color(QPalette::Active, QColorGroup::Foreground).name().latin1(),
			pal->color(QPalette::Active, QColorGroup::Background).name().latin1()
		))	;
		m_widget->setPalette (*pal) ;
		return	;
	}

	DPRINTF
	((	"KBControl::setPalette: morph: t=%s b=%s\n",
		pal->active().text().name().latin1(),
		pal->active().base().name().latin1()
	))	;

	updateMorph	()	;
}

/*  KBControl								*/
/*  setFont	: Set font on widget					*/
/*  font	: const QFont *	: Font in question			*/
/*  (returns)	: void		:					*/

void	KBControl::setFont
	(	const QFont *font
	)
{
	if (m_widget)
		m_widget->setFont (*font) ;
	else	updateMorph () ;
}

/*  KBControl								*/
/*  setupProperties							*/
/*		: Set control properties after design change		*/
/*  (returns)	: void		:					*/

void	KBControl::setupProperties ()
{
	/* Default does nothing, interested controls reimplement.	*/
}

/*  KBControl								*/
/*  showAs	: Set/unset design mode					*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBControl::showAs
	(	KB::ShowAs	mode
	)
{
	m_showing	= mode	;
	m_enabled	= true	;
	m_visible	= true	;
	m_hidden	= false	;
	m_inSetValue	= false	;

	if (mode == KB::ShowAsData)
		m_morphPalette	= *m_object->getPalette (false) ;
	else	m_morphPalette	= QApplication::palette() ;	

	/* Widget may not exist if the control is morphing, otherwise	*/
	/* make sure it is enabled and visible in design mode, or set	*/
	/* appropriately in data mode.					*/
	if (mode == KB::ShowAsData)
	{
		m_enabled	= ! m_object->showDisabled() ;
		m_visible	= ! m_object->showHidden  () ;

		DPRINTF
		((	"KBControl::showAs: %s: %p: %d: e=%d v=%d\n",
			m_object->getName().latin1(),
			(void *)m_widget,
			m_drow,
			m_enabled,
			m_visible
		))	;
	}

	if (m_widget)
	{
		m_widget->setEnabled (m_enabled) ;
		if (m_visible)
			m_widget->show () ;
		else	m_widget->hide () ;
	}
}

/*  KBControl								*/
/*  setFocus	: Set focus to widget					*/
/*  (returns)	: void		:					*/

void	KBControl::setFocus ()
{
	DPRINTF
	((	"KBControl::setFocus: %s: has=%d r=%d\n",
		(cchar *)m_object->getName(),
		m_widget->hasFocus (),
		QFocusEvent::reason()
	))	;

	if (!m_widget->hasFocus()) m_widget->setFocus() ;
}

/*  KBControl								*/
/*  setMonitor	: Set monitor display value				*/
/*  value	: const KBValue&: Value					*/
/*  (returns)	: void		:					*/

void	KBControl::setMonitor
	(	const KBValue &v
	)
{
	if (m_monitor != 0)
	{
		QString	t = v.getRawText() ;
		if (t.length() > 80)
		{	t.truncate  (77)    ;
			t.append    ("...") ;
		}

		m_monitor->setText (2, t) ;
	}

}

/*  KBControl								*/
/*  startUpdate	: Check that record update can start			*/
/*  (returns)	: bool		: Update started			*/

bool	KBControl::startUpdate ()
{
	/* Check that we are in data view (since we might get start	*/
	/* update events in design mode as controls are changed) and	*/
	/* there is an item (paranoia check for when deleting).		*/
	if ((m_showing == KB::ShowAsData) && (m_item != 0))
	{
		/* If an update cannot be started then restore the	*/
		/* current value. This must be the original value.	*/
		if (!m_item->startUpdate (m_item->getBlock()->getCurDRow() + m_drow))
		{
			setValue (m_curVal) ;
			return	 false	    ;
		}

		return	true	;
	}

	return	false	;
}

/*  KBControl								*/
/*  setValue	: Set value						*/
/*  value	: const KBValue&: Value					*/
/*  (returns)	: void		:					*/

void	KBControl::setValue
	(	const KBValue &v
	)
{
	DPRINTF
	((	"KBControl::setValue: [%s][%s:%d]<-[%s]\n",
		(cchar *)m_object->getElement(),
		(cchar *)m_object->getName   (),
		m_drow,
		(cchar *)v.getRawText()
	))	;

	m_curVal = v ;
	if (m_layoutItem != 0)
		m_layoutItem->setValid (isValid (false)) ;

	setMonitor (m_curVal) ;
}

/*  KBControl								*/
/*  getValue	: Get control value					*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBControl::getValue ()
{
	/* Default returns the set value, with the type possibly	*/
	/* changed to the actual type (lest it was set with a value	*/
	/* with some other type).					*/
	KBType	*type	= m_item->getFieldType() ;
	return	type == 0 ? m_curVal : KBValue (m_curVal, type) ;
}

/*  KBControl								*/
/*  clearValue	: Clear value						*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBControl::clearValue
	(	bool
	)
{
	setMonitor (m_curVal = KBValue()) ;
}

/*  KBControl								*/
/*  changed	: Get value changed status				*/
/*  (returns)	: bool		: Value changed				*/

bool	KBControl::changed ()
{
	KBValue	curValue = getValue   () ;
	KBValue	iniValue = getIniValue() ;

	DPRINTF
	((	"KBControl::changed: %s: cur=[%s] ini=[%s]\n",
		m_object->getName  ().latin1(),
		curValue.getRawText().latin1(),
		iniValue.getRawText().latin1()
	))	;

	/* Special case: This handles the situation where the inital	*/
	/* value is null, otherwise we get empty != null.		*/
	if (curValue.isEmpty() && iniValue.isEmpty())
		return	false	;

	return	curValue != iniValue ;
}

/*  KBControl								*/
/*  isEmpty	: See if control is empty				*/
/*  (returns)	: bool		: Empty					*/

bool	KBControl::isEmpty ()
{
	return	getValue().isEmpty() ;
}

/*  KBControl								*/
/*  isValid	: See if control is valid				*/
/*  allowNull	: bool		: Ignore not-null checks		*/
/*  (returns)	: bool		: Valid					*/

bool	KBControl::isValid
	(	bool		allowNull
	)
{
	if (m_item != 0)
		if (!m_item->checkValid (getValue(), allowNull))
		{
			setError (m_item->lastError()) ;
			return	false	;
		}

	return	true ;
}

/*  KBControl								*/
/*  setVisibility: Set visibility/enable state				*/
/*  (returns)	 : void		:					*/

void	KBControl::setVisibility ()
{
	DPRINTF
	((	"KBControl::setVisibility: %s: %p: %d: v=%d e=%d h=%d\n",
		m_object->getName().latin1(),
		(void *)m_widget,
		m_drow,
		m_visible,
		m_enabled,
		m_hidden
	))	;

	/* If the control is explicitely not visible then hide it. Note	*/
	/* that this can cause layout changes if the control is		*/
	/* embedded in a dynamic layout. Probably no need to set the	*/
	/* enabled state but keeps things orthogonal.			*/
	if (!m_visible)
	{
		m_widget->setEnabled	    (m_enabled) ;
		m_widget->hide	     	    ()		;
		m_layoutItem->showValidator (false)	;
		return	;
	}

	/* "hidden" is used by KBItem::hideBelow() when hiding controls	*/
	/* for inaccessible rows. In order to stop dynamic layout	*/
	/* changes, if the control is in a dynamic layout, we disable	*/
	/* the control rather than actually hide it.			*/
	if (m_hidden)
	{
		if (m_object->parentIsDynamic() || m_object->getBlock()->showAllRows())
		{
			m_widget->setEnabled	    (false) ;
			m_layoutItem->showValidator (false) ;
			return	;
		}

		m_widget->setEnabled	    (m_enabled) ;
		m_widget->hide	     	    ()		;
		m_layoutItem->showValidator (false)	;
		return	;
	}

	/* None of the above, the control must be genuinely visible,	*/
	/* and the "setEnabled" call does matter.			*/
	DPRINTF
	((	"KBControl::setVisibility: %s: %p: %d: show with e=%d\n",
		m_object->getName().latin1(),
		(void *)m_widget,
		m_drow,
		m_enabled
	))	;

	m_widget->setEnabled	    (m_enabled) ;
	m_widget->show	     	    ()		;
	m_layoutItem->showValidator (true)	;
}

/*  KBControl								*/
/*  setEnabled	: Set control enabled/disabled				*/
/*  enabled	: bool		: Enabled/disabled			*/
/*  (returns)	: void		:					*/

void	KBControl::setEnabled
	(	bool	enabled
	)
{
	if (m_enabled != enabled)
	{
		m_enabled = enabled ;

		if (m_widget)
			setVisibility () ;
		else	getDisplay()->updateMorph (m_item, m_drow) ;
	}
}

/*  KBControl								*/
/*  setVisible	: Set visible/hidden					*/
/*  visible	: bool		: Set visible				*/
/*  (returns)	: void		:					*/

void	KBControl::setVisible
	(	bool	visible
	)
{
	DPRINTF
	((	"KBControl::setVisible: %s: %d: visible=%d\n",
		m_object->getName().latin1(),
		m_drow,
		visible
	))	;

	if (m_visible != visible)
	{
		m_visible = visible ;

		if (m_widget)
			setVisibility () ;
		else	getDisplay()->updateMorph (m_item, m_drow) ;
	}

}

/*  KBControl								*/
/*  setHidden	: Hide/unhide control for KBItem::hideBelow()		*/
/*  hidden	: bool		: Hidden				*/
/*  (returns)	: void		:					*/

void	KBControl::setHidden
	(	bool	hidden
	)
{
	DPRINTF
	((	"KBControl::setHidden: %s: %d: hidden=%d\n",
		m_object->getName().latin1(),
		m_drow,
		hidden
	))	;

	if (m_hidden != hidden)
	{
		m_hidden = hidden ;

		if (m_widget)
			setVisibility () ;
		else	getDisplay()->updateMorph (m_item, m_drow) ;
	}
}

/*  KBControl								*/
/*  isEnabled	: Get control enabled/disabled				*/
/*  (returns)	: bool		: Enabled/disabled			*/

bool	KBControl::isEnabled ()
{
	return	m_enabled ;
}

/*  KBControl								*/
/*  isVisible	: Get control visibility				*/
/*  (returns)	: bool		: Visibility				*/

bool	KBControl::isVisible ()
{
	return	m_visible ;
}

/*  KBControl								*/
/*  write	: Dummy write routine					*/
/*  writer	: KBWriter *	: Writer				*/
/*  rect	: QRect		: Area elative to writer		*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: init &	: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBControl::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	extra	= 0	;

	/* We should not get called, unless the actual control has not	*/
	/* implemented this method. The default report action is to	*/
	/* write some appropriate text, otherwise grap the widget.	*/
	if (writer->asReport())
	{
		new KBWriterText
		(	writer,
			rect,
			m_item->getPalette(true),
			m_item->getFont	  (true),
			value.getRawText  (),
			Qt::AlignLeft|Qt::AlignVCenter,
			fSubs
		)	;
	
		return	true	;
	}

	new KBWriterPixmap
	(	writer,
		rect,
		QPixmap::grabWidget (m_widget),
		-1
	)	;

	return	true	;
}

/*  KBControl								*/
/*  showMonitor	: Show or clear monitor for this node			*/
/*  parent	: QListViewItem * : Parent monitor			*/
/*  (returns)	: void		  :					*/

void	KBControl::showMonitor
	(	QListViewItem	*parent
	)
{
	if (parent == 0)
	{	m_monitor = 0 ;
		return	;
	}

	if (m_item != 0)
	{
		QString	t = getValue().getRawText() ;
		if (t.length() > 80)
		{	t.truncate (77)	   ;
			t.append   ("...") ;
		}

		m_monitor = new KBNodeMonitor (0, parent) ;

		m_monitor->setText (0, "Control") ;
		m_monitor->setText (1, QString("Row %1").arg(m_drow)) ;
		m_monitor->setText (2, t) ;
	}
}

/*  KBControl								*/
/*  property	: Get QT widget property				*/
/*  name	: cchar *	: Property name				*/
/*  (returns)	: QVariant	: Property value			*/

QVariant
	KBControl::property
	(	cchar		*name
	)
{
	if (m_widget)
		return	m_widget->property (name) ;

	return	QVariant() ;
}

/*  KBControl								*/
/*  setProperty	: Set QT widget property				*/
/*  name	: cchar *	   : Property name			*/
/*  value	: const QVariant & : Value				*/
/*  (returns)	: bool		   : Success				*/

bool	KBControl::setProperty
	(	cchar		*name,
		const QVariant	&value
	)
{
	if (m_widget)
		return	m_widget->setProperty (name, value) ;

	return	false	;
}

bool	KBControl::getFrameSettings
	(	int		&style,
		int		&width,
		int		defFS,
		int		defLW
	)
{
	style	= defFS	;
	width	= defLW	;

	if (m_object  == 0) return false ;

	QString	spec	= m_object->getAttrVal("frame") ;
	if (spec.isEmpty()) return false ;

	int	coff	= spec.find (',');
	if (coff       < 0) return false ;

	style	= spec.left(coff    ).toInt() ;
	width	= spec.mid (coff + 1).toInt() ;

	if ((style == 0) && (width == 0))
	{
		style	= defFS	;
		width	= defLW	;
	}

	return	true	;
}

/*  KBControl								*/
/*  writerSetFrame							*/
/*		: Set frame style on writer item			*/
/*  writer	: KBWriterItem * : Writer item				*/
/*  defFS	: int		 : Default frame style			*/
/*  defLW	: int		 : Default line width			*/
/*  (returns)	: void		 :					*/

void	KBControl::writerSetFrame
	(	KBWriterItem	*writer,
		int		defFS,
		int		defLW
	)
{
	if (writer != 0)
	{
		int	style	;
		int	width	;
		if (getFrameSettings (style, width, defFS, defLW))
			writer->setFrame (style, width) ;
	}

//	if ((m_object != 0) && (writer != 0))
//	{
//		QString	spec	= m_object->getAttrVal("frame") ;
//		if (spec.isEmpty()) return ;
//
//		int	coff	= spec.find (',') ;
//		if (coff       < 0) return ;
//
//		int	fs = spec.left(coff    ).toInt() ;
//		int	lw = spec.mid (coff + 1).toInt() ;
//
//		if ((fs == 0) && (lw == 0))
//		{
//			fs	= defFS	;
//			lw	= defLW	;
//		}
//
//		writer->setFrame (fs, lw) ;
//	}
}

/*  KBControl								*/
/*  ctrlSetFrame: Set frame style on frame object			*/
/*  frame	: QFrame *	: QFrame frame object in question	*/
/*  defFS	: int		: Default frame style			*/
/*  defLW	: int		: Default line width			*/
/*  (returns)	: uint		: Frame width				*/

uint	KBControl::ctrlSetFrame
	(	QFrame		*frame,
		int		defFS,
		int		defLW
	)
{
	if (frame != 0)
	{
		int	style	;
		int	width	;
		getFrameSettings (style, width, defFS, defLW) ;
		frame->setFrameStyle (style) ;
		frame->setLineWidth  (width) ;
		return	width	;
	}

//	if ((m_object != 0) && (frame != 0))
//	{
//		QString	spec	= m_object->getAttrVal("frame") ;
//		int	fs	= 0 ;
//		int	lw	= 0 ;
//
//		int	coff	= spec.find (',') ;
//		if (coff >= 0)
//		{	fs = spec.left(coff    ).toInt() ;
//			lw = spec.mid (coff + 1).toInt() ;
//		}
//
//		if ((fs == 0) && (lw == 0))
//		{
//			fs	= defFS	;
//			lw	= defLW	;
//		}
//
//		frame->setFrameStyle (fs) ;
//		frame->setLineWidth  (lw) ;
//
//		return	lw ;
//	}

	return	0 ;
}

/*  KBControl								*/
/*  unMorphed	: Un-morph a previously morphed control			*/
/*  (returns)	: void		:					*/

void	KBControl::unMorph ()
{
	DPRINTF
	((	"KBControl::unMorphed: [%s][%d] [%d]\n",
		(cchar *)m_object->getName(),
		m_drow,
		(void *)m_widget
	))	;

	/* Dummy base method, reimplemented as needed.			*/
}

/*  KBControl								*/
/*  paintMorph	: Paint value when morphed				*/
/*  p		: QPainter *	  : Painter				*/
/*  text	: const QString & : Value				*/
/*  align	: int		  : Alignment flags			*/
/*  (returns)	: void		  :					*/

void	KBControl::paintMorph
	(	QPainter	*p,
		const QString	&text,
		int		align
	)
{
	QRect	aRect	= m_rect ;
	QRect	dRect
		(	m_rect.x     () + 3,
			m_rect.y     () + 1,
			m_rect.width () - 4,
			m_rect.height() - 2
		)	;


	aRect	= m_object->attrGeom().mapGeometry (m_display->getTopRect(), aRect) ;
	dRect	= m_object->attrGeom().mapGeometry (m_display->getTopRect(), dRect) ;

	DPRINTF
	((	"KBControl::paintMorph: [%s/%d] %s: t=%s b=%s\n",
		(cchar *)m_item->getName(),
		m_drow,
		_TEXT(m_rect),
		m_morphPalette.active().text().name().latin1(),
		m_morphPalette.active().base().name().latin1()
	))	;

	if (m_visible && !m_hidden)
	{
		p->setPen	(m_morphPalette.active().text()) ;
		p->setBrush	(m_morphPalette.active().base()) ;
		p->setFont	(*m_object->getFont     (false)) ;
		p->drawRect	(aRect) ;
		p->drawText	(dRect, align, text) ;
	}
	else	p->eraseRect	(aRect) ;
}

/*  KBControl								*/
/*  repaintMorph: Paint value when morphed				*/
/*  p		: QPainter *	  : Painter				*/
/*  (returns)	: void		  :					*/

void	KBControl::repaintMorph
	(	QPainter	*p
	)
{
	int	align	= Qt::AlignVCenter|Qt::AlignLeft ;
	QString	text	= morphText (align)		 ;

	paintMorph (p, text, align) ;
}

/*  KBControl								*/
/*  morphText	: Get text for morphed control				*/
/*  align	: int &		: Return alignment			*/
/*  (returns)	: QString	: Text					*/

QString	KBControl::morphText
	(	int		&
	)
{
	return	m_curVal.getRawText() ;
}

/*  KBControl								*/
/*  userChange	: User changes value					*/
/*  (returns)	: bool		: Changed value OK			*/

bool	KBControl::userChange ()
{
	if (m_item == 0)
		return	false	;

	DPRINTF
	((
		"KBControl::userChange: [%s] %d/%d\n",
		getValue().getRawText().latin1(),
		!m_inSetValue,
		m_showing == KB::ShowAsData
	))	;


	if (!m_inSetValue && (m_showing == KB::ShowAsData))
	{
		if (!startUpdate ())
			return	false	;

		m_item	    ->userChange (m_item->getBlock()->getCurDRow() + m_drow, getValue()) ;
		m_layoutItem->setValid   (isValid (false)) ;
		return	true	;
	}

	return	false	;
}

/*  KBControl								*/
/*  deferUserChange							*/
/*		: Defer user change event				*/
/*  (returns)	: void		*					*/

void	KBControl::deferUserChange ()
{
	/* This is used by controls where we need to get outside of	*/
	/* the user-change event (typically a signal) before the	*/
	/* "onChange" event is processed. Needed since the "onChange"	*/
	/* code might change the state of the control, including a	*/
	/* switch to design view.					*/
	QTimer::singleShot (0, this, SLOT(userChange())) ;
}

/*  KBControl								*/
/*  getDRow	: Get display row of this control			*/
/*  (returns)	: uint		: Display row				*/

uint	KBControl::getDRow ()
	{
		return	m_drow	;
	}

void	KBControl::setError
	(	const KBError	&pError
	)
{
	m_lError = pError;
}

const KBError
	&KBControl::lastError ()
{
	return	m_lError ;
}

#if	! __KB_RUNTIME

/*  KBControl								*/
/*  setSizerBlobs: Position sizer blobs on widget			*/
/*  btl		 : KBSizerBlob * : Top-left blob			*/
/*  btr		 : KBSizerBlob * : Top-lright blob			*/
/*  bbl		 : KBSizerBlob * : Bottom-left blob			*/
/*  bbr		 : KBSizerBlob * : Bottom-right blob			*/
/*  (returns)	 : void		 :					*/

void	KBControl::setSizerBlobs
	(	KBSizerBlob 	*btl,
		KBSizerBlob	*btr,
		KBSizerBlob	*bbl,
		KBSizerBlob	*bbr
	)
{
	int	w	= m_widget->width () ;
	int	h	= m_widget->height() ;

	btl->move (0, 		    0		   ) ;
	btr->move (w - SZ_BLOBSIZE, 0		   ) ;
	bbl->move (0, 		    h - SZ_BLOBSIZE) ;
	bbr->move (w - SZ_BLOBSIZE, h - SZ_BLOBSIZE) ;
}

#endif


/*  KBControl								*/
/*  setReadOnly	: Explicitely set readonly state			*/
/*  readOnly	: bool		: Required state			*/
/*  (returns)	: void		:					*/

void	KBControl::setReadOnly
	(	bool		readOnly,
		const QColor	&fgColor,
		const QColor	&bgColor
	)
{
	/* This method is overridden to handle the readOnly setting	*/
	/* according to the argument, but this base method should still	*/
	/* be called to handle the optional colours.			*/
	setFGColor (fgColor) ;
	setBGColor (bgColor) ;
}

bool	KBControl::isReadOnly ()
{
	/* Dummy, must be implemented in specific control classes.	*/
	return	false	;
}

/*  KBControl								*/
/*  setFGColor	: Explicitely set foreground colour			*/
/*  color	: const QColor & : Required colour			*/
/*  (returns)	: void		 :					*/

void	KBControl::setFGColor
	(	const QColor	&color
	)
{
	if ((m_widget != 0) && color.isValid())
	{
		QPalette p = m_widget->palette() ;
		p.setColor (QPalette::Active, QColorGroup::Foreground, color) ;
		p.setColor (QPalette::Active, QColorGroup::ButtonText, color) ;
		p.setColor (QPalette::Active, QColorGroup::Text,       color) ;
		m_widget->setPalette (p) ;
	}
}

/*  KBControl								*/
/*  setBGColor	: Explicitely set background colour			*/
/*  color	: const QColor & : Required colour			*/
/*  (returns)	: void		 :					*/

void	KBControl::setBGColor
	(	const QColor	&color
	)
{
	if ((m_widget != 0) && color.isValid())
	{
		QPalette p = m_widget->palette() ;
		p.setColor (QPalette::Active, QColorGroup::Background, color) ;
		p.setColor (QPalette::Active, QColorGroup::Button,     color) ;
		p.setColor (QPalette::Active, QColorGroup::Base,       color) ;
		m_widget->setPalette (p) ;
	}
}
