/***************************************************************************
    file	         : kb_ctrl.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_CTRL_H
#define		__KB_CTRL_H

#include	"libkbase_exports.h"

#include	"rk_application.h"
#include	"kb_item.h"
#include	"kb_layoutitem.h"


class		QTimer		;
class		KBSizerBlob	;
class		QObjectList	;
class		KBWriterItem	;

/*  KBControl								*/
/*  --------								*/
/*  This class is used as a standard interface to things that appear on	*/
/*  the display, for instance buttons and text fields.			*/

class	LIBKBASE_API	KBControl : public RKNotifyFilter
{
	Q_OBJECT

	QWidget			*m_widget	;
	KBDisplay		*m_display	;
	KBObject		*m_object	;
	KBItem			*m_item		;
	KBNodeMonitor		*m_monitor	;

	KBError			m_lError	;
	bool			m_lastState	;

	QRect			m_rect		;
	bool			m_dynamic	;

	/* The "enabled" and "visible" states are stored so that we	*/
	/* can correctly handle morphed widgets. The "hidden" flag is	*/
	/* used when a control is hidden via the ::hideBelow() method.	*/
	bool			m_enabled	;
	bool			m_visible	;
	bool			m_hidden	;

	QObjectList		*m_allWidgets	;
	QTimer			*m_morphTimer	;
	QPalette		m_morphPalette	;

	void			init		()	;
	void			updateMorph	()	;

protected :

	KBValue			m_curVal	;
	KB::ShowAs		m_showing	;
	uint			m_drow		;
	KBCtrlLayoutItem	*m_layoutItem	;
	bool			m_inSetValue	;

	virtual	bool		eventFilter	(QObject *, QEvent *)	;
	virtual	bool		notifyFilter	(QObject *, QEvent *)	;

	void			setupWidget
				(	QWidget *,
					KBLayoutItem::RTTI = KBLayoutItem::RTTICtrl
				)	;

	bool			getFrameSettings(int &, int &,     int, int)	;
	uint			ctrlSetFrame	(QFrame		*, int, int)	;
	void			writerSetFrame	(KBWriterItem 	*, int, int)	;

	virtual	void		setMonitor	(const KBValue &)	;
	bool			startUpdate	() ;
	void			setVisibility	() ;

	void			startMorphTimer	() ;
	void			stopMorphTimer	() ;

protected slots	:

	virtual bool		userChange	() ;
	virtual	void		clearWidget	() ;
	void			deferUserChange	() ;

public	:

		KBControl	(KBDisplay *, KBObject *) ;
		KBControl	(KBDisplay *, KBItem   *, uint) ;
	virtual~KBControl	() ;

	virtual	void		setCtrlGeometry	(const QRect &, int)	;
	virtual	QRect		ctrlGeometry	()			;

	KBDisplay 		*getDisplay	()			;
	virtual	QWidget		*getWidget	()			;

	virtual void		showMonitor	(QListViewItem  *)	;
	virtual	void		setPalette	(const QPalette *)	;
	virtual	void		setFont		(const QFont    *)	;
	virtual	void		showAs		(KB::ShowAs)		;
	virtual	void		setFocus	()			;
	virtual	void		setupProperties	()			;

	virtual void		setEnabled	(bool)			;
	virtual void		setVisible	(bool)			;
	virtual void		setHidden	(bool)			;
	virtual bool		isEnabled	() 			;
	virtual bool		isVisible	()			;

	virtual	void		setReadOnly	(bool, const QColor &, const QColor &)	;
	virtual	bool		isReadOnly	()			;
	virtual	void		setFGColor	(const QColor  &)	;
	virtual	void		setBGColor	(const QColor  &)	;
	
	KBValue			getIniValue	() 			;
	virtual	void		setValue	(const KBValue &)	;
	virtual	KBValue		getValue	()			;
	virtual	void		clearValue	(bool)			;
	virtual	bool		changed		()			;
	virtual	bool		isEmpty		()			;
	virtual	bool		isValid		(bool)			;

	virtual	void		repaintMorph	(QPainter *)		;
	virtual	void		paintMorph	(QPainter *, const QString &, int) ;

	void			unMorph		()			;
	virtual	QString		morphText	(int &)			;
	void			eraseArea	()			;

	virtual	QVariant	property	(cchar *)		;
	virtual	bool		setProperty	(cchar *, const QVariant &) ;

	uint			getDRow		()	;
	void			setError	(const KBError	&)	;
	const KBError		&lastError	()	;

	virtual	bool		write		(KBWriter *, QRect, const KBValue &, bool, int &) ;

#if	! __KB_RUNTIME
	void			setSizerBlobs
				(	KBSizerBlob 	*,
					KBSizerBlob	*,
					KBSizerBlob	*,
					KBSizerBlob	*
				)	;
#endif

	friend	class	KBInSetValueLock	;
	friend	class	KBTestAccessor		;
}	;


/*  ------------------------------------------------------------------	*/

/*  KBInSetValueLock							*/
/*  ----------------							*/
/*  This is a little helper class used to make it easy to bracket	*/
/*  operations with then "m+inSetValue" flag set in the KBControl	*/
/*  class.								*/

class	KBInSetValueLock
{
	KBControl	&m_control	;

public	:

	inline	KBInSetValueLock
		(	KBControl	&control
		)
		:
		m_control (control)
	{
		m_control.m_inSetValue	= true	;
	}

	inline	~KBInSetValueLock ()
	{
		m_control.m_inSetValue	= false	;
	}
}	;


#endif		// __KB_CTRL_H
