/***************************************************************************
    file	         : kb_ctrlwizard.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_formblock.h"
#include	"kb_framer.h"
#include	"kb_docroot.h"

#include	"kb_navigator.h"
#include	"kb_grid.h"

#include	"kb_attrdict.h"
#include	"kb_wizard.h"
#include	"kb_parse.h"
#include	"kb_locator.h"
#include	"kb_component.h"
#include	"kb_config.h"
#include	"kb_locator.h"

#include	"kb_table.h"
#include	"kb_qrytable.h"
#include	"kb_qryquery.h"
#include	"kb_qrynull.h"


#define		__DEBUG_CTRLWIZARD	(0)
#if		__DEBUG_CTRLWIZARD
#include	"kb_dprintf.h"
#endif

#if	! __KB_RUNTIME


LIBKBASE_API
	QString	runCtrlWizard
	(	KBNode		*parent,
		KBQryBase	*query,
		cchar		*wizName,
		KBAttrDict	&aDict,
		bool		&cancel
	)
{
	KBLocation location = parent->getDocRoot()->getDocLocation() ;

	/* First create and execute the wizard. If there is no wizard	*/
	/* then the result is null, but cancel is unset, so the caller	*/
	/* can fall back on the context menus. If there is a wizard	*/
	/* and the user cancels, set the cancel flag so that the caller	*/
	/* can give up.							*/
	KBWizard *wizard
			= KBWizardReg::makeWizard
			  (	wizName,
				location,
				location.server()
			  )	;

	if (wizard == 0)
	{
		DPRINTF
		((	"runCtrlWizard: no wizard for [%s]\n",
			wizName
		))	;
		cancel	= false	;
		return	QString::null ;
	}

	wizard->setCookie ("exprquery", KBValue(query)) ;

	if (!wizard->execute ())
	{
		cancel	= true	;
		delete	wizard	;
		return	QString::null ;
	}

	/* The wizard should implement the "results()" script to return	*/
	/* a list. The first element is the component name for the	*/
	/* control; the remainder of the list is (attribute,value)	*/
	/* pairs to be substituted. This arrangement means that all	*/
	/* component knowledge is in the wizard, and not in this code.	*/
	QValueList<QVariant> results = wizard->results() ;

	for (uint idx = 1 ; idx < results.count() ; idx += 2)
	{
		DPRINTF
		((	"runCtrlWizard: %3d: [%s]->[%s]\n",
			idx,
			(cchar *)results[idx + 0].toString(),
			(cchar *)results[idx + 1].toString()
		))	;

		aDict.addValue
		(	results[idx + 0].toString(),
			results[idx + 1].toString()
		)	;
	}

	delete	wizard	;
	return	results[0].toString() ;
}

LIBKBASE_API
	KBNode	*makeCtrlFromComponent
	(	KBNode		*parent,
		const QString	&compName,
		KBAttrDict	&aDict,
		bool		&cancel
	)
{
	QString	   language = parent->getRoot   ()->getAttrVal    ("language") ;

	/* Locate, load and parse the component. This is much the same	*/
	/* as when component is directly pasted in.			*/
	DPRINTF
	((	"makeCtrlFromComponent: l=[%s] n=[%s]\n",
		language.latin1(),
		compName.latin1()
	))	;

	QString	source	= locateFile
			  (	"appdata", 
				QString	("stock/component/%1/%2.cmp")
					.arg(language)
					.arg(compName)
			  )	;

	if (source.isEmpty())
	{
		KBError::EError
		(	QString(TR("Cannot locate '%1' component")).arg(compName),
			QString(TR("Using language '%1'")).arg(language),
			__ERRLOCN
		)	;
		cancel	= false	;
		return	0	;
	}

	QFile		file	(source) ;
	if (!file.open (IO_ReadOnly))
	{
		KBError::EError
		(	QString(TR("Cannot open '%1' component")).arg(compName),
			QString(TR("Using language '%1'")).arg(language),
			__ERRLOCN
		)	;
		cancel	= false	;
		return	0	;
	}

	KBError		error	;
	QByteArray	xml	= file.readAll () ;
	if (xml.count() == 0)
	{
		KBError::EError
		(	QString(TR("'%1' component is empty")).arg(compName),
			QString(TR("Using language '%1'")).arg(language),
			__ERRLOCN
		)	;
		cancel	= false	;
		return	0	;
	}

	KBLocation	locn	;
	KBComponent	*comp	= KBOpenComponentText (locn, xml, error) ;
	if (comp == 0)
	{
		error.DISPLAY()	;
		cancel	= false	;
		return	0	;
	}

	/* Locate and process all configuration objects. Each is	*/
	/* checked against the attribute dictionary, and has its value	*/
	/* set if found. As usual, non-user-configs are then dropped.	*/
	QPtrList<KBConfig>	configSet  	;
	comp->findAllConfigs (configSet, QString::null) ;

	LITER
	(	KBConfig,
		configSet,
		config,

		QString	*sv = aDict.find(config->attrib()) ;
		if (sv != 0) config->setValue(*sv) ;

		DPRINTF
		((	"makeCtrlFromComponent: conf=[%s] sv=[%s]\n",
			(cchar *)config->attrib(),
			sv == 0 ? "NONE" : (cchar *)*sv
		))	;

		config->substitute (false) ;
		if (!config->user()) delete config ;
	)


	/* Now replicate the component into the parent. We skip the	*/
	/* dummy null query, and assume that there is only a single	*/
	/* top-level object in the component, which is the node that is	*/
	/* returned to the caller.					*/
	KBNode	*ctrl	= 0 ;

	LITER
	(	KBNode,
		comp->getChildren(),
		node,

		if (node->isQryNull())
			continue ;

		if (ctrl == 0)
			ctrl = node->replicate(parent) ;
		else	node->replicate(parent) ;
	)

	delete	comp	;
	cancel	= false	;
	return	ctrl	;
}

/*  makeCtrlFromWizard							*/
/*		: Build a control using a control wizard		*/
/*  parent	: KBNode *	: Parent into which to insert control	*/
/*  query	: KBQryBase *	: Associated query if any		*/
/*  wizName	: cchar *	: Wizard name				*/
/*  aDict	: KBAttrDict &	: Attribute dictionary			*/
/*  cancel	: bool &	: Set if user hit cancel		*/
/*  (returns)	: KBNode *	: New node or null if none		*/

LIBKBASE_API
	KBNode	*makeCtrlFromWizard
	(	KBNode		*parent,
		KBQryBase	*query,
		cchar		*wizName,
		KBAttrDict	&aDict,
		bool		&cancel
	)
{
	QString	compName = runCtrlWizard
				(	parent,
					query,
					wizName,
					aDict,
					cancel
				)	;

	if (compName.isEmpty())
		return 0 ;

	return	makeCtrlFromComponent
		(	parent,
			compName,
			aDict,
			cancel
		)	;

#if	0
	KBLocation location = parent->getDocRoot()->getDocLocation() ;
	QString	   language = parent->getRoot   ()->getAttrVal    ("language") ;

	extern	bool	execWizardByName
			(	cchar		*,
				KBLocation	&,
				const QString	&
			)	;

	/* First create and execute the wizard. If there is no wizard	*/
	/* then the result is null, but cancel is unset, so the caller	*/
	/* can fall back on the context menus. If there is a wizard	*/
	/* and the user cancels, set the cancel flag so that the caller	*/
	/* can give up.							*/
	KBWizard *wizard
			= KBWizardReg::makeWizard
			  (	wizName,
				location,
				location.server()
			  )	;

	if (wizard == 0)
	{
		DPRINTF
		((	"makeCtrlFromWizard: no wizard for [%s]\n",
			wizName
		))	;
		cancel	= false	;
		return	0	;
	}

	wizard->setCookie ("exprquery", KBValue(query)) ;

	if (!wizard->execute ())
	{
		cancel	= true	;
		delete	wizard	;
		return	0	;
	}

	/* The wizard should implement the "results()" script to return	*/
	/* a list. The first element is the component name for the	*/
	/* control; the remainder of the list is (attribute,value)	*/
	/* pairs to be substituted. This arrangement means that all	*/
	/* component knowledge is in the wizard, and not in this code.	*/
	QValueList<QVariant> results = wizard->results() ;
	QString		     name    = results[0].toString() ;

	for (uint idx = 1 ; idx < results.count() ; idx += 2)
	{
		DPRINTF
		((	"makeCtrlFromWizard: %3d: [%s]->[%s]\n",
			idx,
			(cchar *)results[idx + 0].toString(),
			(cchar *)results[idx + 1].toString()
		))	;

		aDict.addValue
		(	results[idx + 0].toString(),
			results[idx + 1].toString()
		)	;
	}

	delete	wizard	;

	/* Locate, load and parse the component. This is much the same	*/
	/* as when component is directly pasted in.			*/
	DPRINTF
	((	"makeCtrlFromWizard: l=[%s] n=[%s]\n",
		language.latin1(),
		name    .latin1()
	))	;

	QString	source	= locateFile
			  (	"appdata", 
				QString	("stock/component/%1/%2.cmp")
					.arg(language)
					.arg(name)
			  )	;

	if (source.isEmpty())
	{
		KBError::EError
		(	QString(TR("Cannot locate '%1' component")).arg(name),
			QString(TR("Using language '%1'")).arg(language),
			__ERRLOCN
		)	;
		cancel	= false	;
		return	0	;
	}

	QFile		file	(source) ;
	if (!file.open (IO_ReadOnly))
	{
		KBError::EError
		(	QString(TR("Cannot open '%1' component")).arg(name),
			QString(TR("Using language '%1'")).arg(language),
			__ERRLOCN
		)	;
		cancel	= false	;
		return	0	;
	}

	KBError		error	;
	QByteArray	xml	= file.readAll () ;
	if (xml.count() == 0)
	{
		KBError::EError
		(	QString(TR("'%1' component is empty")).arg(name),
			QString(TR("Using language '%1'")).arg(language),
			__ERRLOCN
		)	;
		cancel	= false	;
		return	0	;
	}

	KBLocation	locn	;
	KBComponent	*comp	= KBOpenComponentText (locn, xml, error) ;
	if (comp == 0)
	{
		error.DISPLAY()	;
		cancel	= false	;
		return	0	;
	}

	/* Locate and process all configuration objects. Each is	*/
	/* checked against the attribute dictionary, and has its value	*/
	/* set if found. As usual, non-user-configs are then dropped.	*/
	QPtrList<KBConfig>	configSet  	;
	comp->findAllConfigs (configSet, QString::null) ;

	LITER
	(	KBConfig,
		configSet,
		config,

		QString	*sv = aDict.find(config->attrib()) ;
		if (sv != 0) config->setValue(*sv) ;

		DPRINTF
		((	"makeCtrlFromWizard: conf=[%s] sv=[%s]\n",
			(cchar *)config->attrib(),
			sv == 0 ? "NONE" : (cchar *)*sv
		))	;

		config->substitute (false) ;
		if (!config->user()) delete config ;
	)


	/* Now replicate the component into the parent. We skip the	*/
	/* dummy null query, and assume that there is only a single	*/
	/* top-level object in the component, which is the node that is	*/
	/* returned to the caller.					*/
	KBNode	*ctrl	= 0 ;

	LITER
	(	KBNode,
		comp->getChildren(),
		node,

		if (node->isQryNull())
			continue ;

		if (ctrl == 0)
			ctrl = node->replicate(parent) ;
		else	node->replicate(parent) ;
	)

	delete	comp	;
	cancel	= false	;
	return	ctrl	;
#endif
}

/*  makeCtrlFromWizard							*/
/*		: Build a control using a control wizard		*/
/*  parent	: KBNode *	: Parent into which to insert control	*/
/*  query	: KBQryBase *	: Associated query if any		*/
/*  spec	: NodeSpec *	: Node information			*/
/*  aDict	: KBAttrDict &	: Attribute dictionary			*/
/*  cancel	: bool &	: Set if user hit cancel		*/
/*  (returns)	: KBNode *	: New node or null if none		*/

LIBKBASE_API
	KBNode	*makeCtrlFromWizard
	(	KBNode		*parent,
		KBQryBase	*query,
		NodeSpec	*nodeSpec,
		KBAttrDict	&aDict,
		bool		&cancel
	)
{
	if (nodeSpec->m_wizardFunc != 0)
		return	(*nodeSpec->m_wizardFunc) (parent, query, aDict, cancel) ;

	return	makeCtrlFromWizard
		(	parent,
			query,
			nodeSpec->m_nodeName,
			aDict,
			cancel
		)	;
}

/*  ------------------------------------------------------------------	*/

/*  makeSubFormFromWizard						*/
/*		: Build a block using a block wizard			*/
/*  parent	: KBObject *	: Parent into which to insert block	*/
/*  query	: KBQryBase *	: Query for enclosing block		*/
/*  type	: KBBlock::BlkType					*/
/*				: Required block type			*/
/*  blkDict	: KBAttrDict &	: Block attribute dictionary		*/
/*  cancel	: bool &	: Set if user hit cancel		*/
/*  (returns)	: KBBlock *	: New block or null if none		*/

LIBKBASE_API
	KBBlock	*makeSubFormFromWizard
	(	KBObject		*parent,
		KBQryBase		*query,
		KBBlock::BlkType	type,
		KBAttrDict		&blkDict,
		bool			&cancel
	)
{
	/* We only support table and query blocks. Anything else	*/
	/* returns a null rsult but with "cancel" set false, so that	*/
	/* the caller can continue via the full propery dialogs.	*/
	switch (type)
	{
		case KBBlock::BTTable	:
		case KBBlock::BTQuery	:
			break	;

		default	:
			cancel	= false	;
			return	0	;
	}

	/* First create and execute the wizard. If there is no wizard	*/
	/* then the result is null, but cancel is unset, so the caller	*/
	/* can fall back on the context menus. If there is a wizard	*/
	/* and the user cancels, set the cancel flag so that the caller	*/
	/* can give up.							*/
	QString	source	= locateFile ("appdata", "wizards/wizSubForm.wiz") ;
	if (source.isEmpty())
	{
		DPRINTF
		((	"makeBlockFromWizard: Cannot locate wizSubForm.wiz\n"
		))	;
		cancel	= false	;
		return	0	;
	}

	KBLocation location = parent->getDocRoot()->getDocLocation() ;
	KBWizard   wizard   (location.dbInfo (), location.server ()) ;

	/* Set a "query" cookie that points to the parent block's query	*/
	/* and a "sourcetype" cookie that specifies whether we want	*/
	/* tables or queries. These are done _before_ calling the	*/
	/* wizard ::init method since some are needed as the wizard	*/
	/* controls are assembled.					*/
	wizard.setCookie ("exprquery", KBValue(query)) ;
	wizard.setCookie ("ischild",   KBValue(query->isQryNull() == 0)) ;

	switch (type)
	{
		case KBBlock::BTTable	:
			DPRINTF
			((	"makeBlockFromWizard: set sourcetype=T\n"
			))	;
			wizard.setCookie ("sourcetype", KBValue("T")) ;
			break		;

		case KBBlock::BTQuery	:
			DPRINTF
			((	"makeBlockFromWizard: set sourcetype=Q\n"
			))	;
			wizard.setCookie ("sourcetype", KBValue("Q")) ;
			break		;

		default	:
			break	;
	}

	if (!wizard.init (source))
	{
		DPRINTF
		((	"makeBlockFromWizard: Cannot initialise wizSubForm.wiz\n"
		))	;
		cancel	= false	;
		return	0	;
	}


	if (!wizard.execute ())
	{
		cancel	= true	;
		return	0	;
	}

	/* Wizard executed. Create the subform block and the associated	*/
	/* query. The "new KBQryNull" case should never be executed	*/
	/* but prevents a crash if we've got something badly wrong.	*/
	blkDict.addValue ("master",   wizard.ctrlValue ("link", "master")) ;
	blkDict.addValue ("child",    wizard.ctrlValue ("link", "child" )) ;
	blkDict.addValue ("autosync", "Yes") ;

	if ((parent != 0) && parent->isDynamic())
	{
		blkDict.addValue ("rowcount", "1") ;
		blkDict.addValue ("manage",   "2") ;
		blkDict.addValue ("m_rows",   "2") ;
		blkDict.addValue ("m_cols",   "2") ;
	}

	KBFormBlock *fblk   = new KBFormBlock (parent, blkDict, "KBFormBlock") ;

	QString	    sName   = wizard.ctrlValue ("source", "object") ;
	KBAttrDict  qryDict ;

	switch (type)
	{
		case KBBlock::BTTable	:
			qryDict.addValue ("server",	"Self")	;
			qryDict.addValue ("table",	sName ) ;
			qryDict.addValue ("primary",	""    ) ;
			qryDict.addValue ("ptype",	(int)KBTable::Auto) ;
			new KBQryTable	 (fblk, qryDict) ;
			break		;

		case KBBlock::BTQuery	:
			qryDict.addValue ("query",	sName ) ;
			new KBQryQuery	 (fblk, qryDict) ;
			break		;

		default	:
			new KBQryNull	 (fblk, qryDict) ;
			break	;
	}

	fblk->blockSetup () ;
	cancel	= false	;
	return	fblk	;
}

#endif
