/***************************************************************************
    file	         : kb_designpopup.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qmetaobject.h>
#include	<qlabel.h>

#include	"kb_object.h"
#include	"kb_popupmenu.h"
#include	"kb_formcopier.h"
#include	"tk_icons.h"


#if	! __KB_RUNTIME


void	makeDynamicPopup
	(	KBPopupMenu	*popupDynamic,
		KBObject	*receiver
	)
{
	popupDynamic->insertItem
	(	getSmallIcon("insertrow"),
		TR("Insert row"),
		receiver,
		SLOT(insertDynamicRow   ())
	)	;
	popupDynamic->insertItem
	(	getSmallIcon("deleterow"),
		TR("Delete row"),
		receiver,
		SLOT(deleteDynamicRow   ())
	)	;
	popupDynamic->insertItem
	(	getSmallIcon("insertcol"),
		TR("Insert column"),
		receiver,
		SLOT(insertDynamicColumn())
	)	;
	popupDynamic->insertItem
	(	getSmallIcon("deletecol"),
		TR("Delete column"),
		receiver,
		SLOT(deleteDynamicColumn())
	)	;
	popupDynamic->insertItem
	(	TR("Grid setup"),
		receiver,
		SLOT(gridSetup          ())
	)	;
}


/*  makeAncestorPopup							*/
/*		: Make popup menu to access ancestors			*/
/*  popupMain	: KBPopupMenu *	: Menu into which to build		*/
/*  from	: KBObject *	: Object from which to start		*/
/*  (returns)	: void		:					*/

void	makeAncestorPopup
	(	KBPopupMenu	*popupMain,
		KBObject	*from
	)
{
	KBObject    *up		= from->parentObject()    ;

	popupMain->insertSeparator () ;

	while (up != 0)
	{
		popupMain->insertItem
		(	QString("%1: %2").arg(up->contextMenuType()).arg(up->getName()),
			up->designPopup(popupMain, QRect())
		)	;

		up	= up->parentObject() ;
	}
}

/*  makeContainerEditPopup						*/
/*		: Populate the edit popup for a container-like object	*/
/*  parent	: KBPopupMenu	* : Parent menu				*/
/*  receiver	: KBObject *	  : Receiver for menu events		*/
/*  type	: const QString & : Object type for menu entries	*/
/*  skipCutCopy	: bool		  : Skip cut/copy/delete operations	*/
/*  (returns)	: KBPopupMenu	* : Edit menu				*/

KBPopupMenu
	*makeContainerEditPopup
	(	KBPopupMenu	*parent,
		KBObject	*receiver,
		const QString	&type,
		bool		skipCutCopy
	)
{
	KBNode		*dummy		;
	KBPopupMenu	*popupEdit	= new KBPopupMenu (parent) ;
	bool		skipClipboard	= !KBFormCopier::self()->anyCopied(dummy) ;

	popupEdit->insertEntry
	(	skipCutCopy,
		getSmallIcon("editcut"),
		QString(TR("C&ut %1"  )).arg(type),
		receiver,
		 SLOT(cutObj   	  ())
	)	;
	popupEdit->insertEntry
	(	skipCutCopy,
		getSmallIcon("editcopy"),
		QString(TR("&Copy %1" )).arg(type),
		receiver,
		SLOT(copyObj	  ())
	)	;

	/* "Paste objects" is disabled if the received is dynamic, as	*/
	/* it only makes sense to paste (single) objects into an	*/
	/* explicit cell.						*/
	popupEdit->insertEntry
	(	receiver->isDynamic() || skipClipboard,
		TR("&Paste objects"),
		receiver,
		SLOT(pasteObjects  ())
	)	;

	popupEdit->insertEntry
	(	skipClipboard,
		TR("Paste &here"),
		receiver,
		SLOT(pasteHere	  ())
	)	;

	popupEdit->insertEntry
	(	skipCutCopy,
		getSmallIcon("editdelete"),
		QString(TR("Delete %1")).arg(type),
		receiver,
		SLOT(deleteObj	  ())
	)	;

	return	popupEdit ;
}


/*  makeContainerMainPopup						*/
/*		: Populate the main popup for a container-like object	*/
/*  popupMain	: KBPopupMenu *	  : Popup to populate			*/
/*  receiver	: KBObject *	  : Receiver for menu events		*/
/*  type	: const QString & : Object type for menu entries	*/
/*  popupNew	: KBPopupMenu *	  : New popup				*/
/*  popupEdit	: KBPopupMenu *	  : Edit popup				*/
/*  (returns)	: void		  :					*/

void	makeContainerMainPopup
	(	KBPopupMenu	*popupMain,
		KBObject	*receiver,
		const QString	&type,
		KBPopupMenu	*popupNew,
		KBPopupMenu	*popupEdit
	)
{
	/* See if the receiver has any child items. Unless there are,	*/
	/* the tab order option will be disabled.			*/
	bool	noItem	= true	   ;
	TITER
	(	Item,
		receiver->getChildren(),
		i,
		noItem = false ; break
	)

	popupMain->setTitle (receiver) ;

	/* Add the edit and new popup submenus, except that we may be	*/
	/* called without any; also skipped if this menu is parented as	*/
	/* it is then for an ancestor.					*/
	if (popupEdit != 0)
		popupMain->insertItem	(TR("&Edit"), popupEdit) ;
	if (popupNew  != 0)
		popupMain->insertItem	(TR("&New"),  popupNew ) ;


	if (receiver->isDynamic())
	{
		KBPopupMenu *popupDynamic = new KBPopupMenu (popupMain) ;
		makeDynamicPopup (popupDynamic, receiver) ;
		popupMain->insertItem	(TR("Dynamic layout"), popupDynamic) ;
	}

	/* The remaining entries are added dependant on whether the	*/
	/* receiver has a slot to handle the option. Not slot, no entry	*/
	/* This makes dynamic configuration easier.			*/
	QStrList slotNames = receiver->metaObject()->slotNames(true) ;

	if (slotNames.find("newTabOrder()") >= 0)
		popupMain->insertEntry
		(	noItem,
			TR  ("Set tab order"),
			receiver,
			SLOT(newTabOrder())
		)	;

	if (slotNames.find("saveAsComponent()") >= 0)
		popupMain->insertItem
		(	getSmallIcon("filesave"),
			TR("Save as component"),
			receiver,
			SLOT(saveAsComponent())
		)	;

	if (slotNames.find("propertyDlg()") >= 0)
		popupMain->insertItem
		(	getSmallIcon("properties"),
			QString(TR("%1 properties")).arg(type),
			receiver,
			SLOT(propertyDlg())
		)	;

	if (slotNames.find("docPropDlg()") >= 0)
		popupMain->insertItem
		(	TR("&Document properties"),
			receiver,
			SLOT(docPropDlg())
		)	;

	if ((popupMain->parent() == 0) && (receiver->parentObject() != 0))
		makeAncestorPopup (popupMain, receiver) ;

	popupMain->insertSeparator() ;
	popupMain->insertItem
	(	getSmallIcon("info"),
		TR("&Information"),
		receiver,
		SLOT(whatsThis())
	)	;

	if (slotNames.find("showQuery()") >= 0)
		popupMain->insertItem 
		(	getSmallIcon("querylog"),
			TR("&Show query"),
			receiver,
			SLOT(showQuery())
		)	;
}

/*  makeNestingPopup							*/
/*		: Make popup menu for nesting containers		*/
/*  popupNest	: KBPopupMenu *	: Menu to populate			*/
/*  receiver	: KBObject *	: Receiver for menu events		*/
/*  (returns)	: void		:					*/

void	makeNestingPopup
	(	KBPopupMenu	*popupNest,
		KBObject	*receiver
	)
{
	/* As above, add entries only if the target slot exists in the	*/
	/* received.							*/
	QStrList slotNames = receiver->metaObject()->slotNames(true) ;

	if (slotNames.find("newNullBlock()") >= 0)
		popupNest->insertItem
		(	TR("&Menu Block" ),
			receiver,
			SLOT(newNullBlock())
		)	;

	if (slotNames.find("newTableBlock()") >= 0)
		popupNest->insertItem
		(	TR("&Table Block"),
			receiver,
			SLOT(newTableBlock())
		)	;

	if (slotNames.find("newQueryBlock()") >= 0)
		popupNest->insertItem
		(	TR("&Query Block"),
			receiver,
			SLOT(newQueryBlock())
		)	;

	if (slotNames.find("newSQLBlock()") >= 0)
		popupNest->insertItem
		(	TR("&SQL Block"),
			receiver,
			SLOT(newSQLBlock())
		)	;

	if (slotNames.find("newContainer()") >= 0)
		popupNest->insertItem
		(	TR("&Container"),
			receiver,
			SLOT(newContainer())
		)	;
}


KBPopupMenu
	*makeFormNewPopup
	(	KBPopupMenu	*parent,
		QRect		rect,
		KBObject	*object,
		Qt::ButtonState	*bState
	)
{
	extern void 	makeFormMenu (QPopupMenu *, QObject *, uint, Qt::ButtonState *) ;

	KBPopupMenu	*newPopup = new KBPopupMenu (parent,   bState) ;
	KBPopupMenu	*blkPopup = new KBPopupMenu (newPopup, bState) ;

	makeNestingPopup (blkPopup, object) ;

	newPopup->insertItem      (TR("New B&lock"), blkPopup) ;
	newPopup->insertSeparator () ;

	makeFormMenu (newPopup, object, KNF_BLOCK|KNF_STATIC|KNF_DATA, bState) ;

	if (parent == 0)
	{
		KBNode	*dummy		;
		bool	skipClipboard	= !KBFormCopier::self()->anyCopied(dummy) ;

		newPopup->insertSeparator () ;
		newPopup->insertEntry
		(	object->isDynamic() || skipClipboard,
			TR("&Paste objects"),
			object,
			SLOT(pasteObjects  ())
		)	;

		newPopup->insertEntry
		(	skipClipboard,
			TR("Paste &here"),
			object,
			SLOT(pasteHere	  ())
		)	;

	}

	newPopup->insertSeparator () ;
	newPopup->insertEntry     (false,  TR("Paste component"), object, SLOT(pasteComponent())) ;
	newPopup->insertEntry     (false,  TR("Link component"),  object, SLOT(linkComponent ())) ;

	return	newPopup   ;
}

KBPopupMenu
	*makeReportNewPopup
	(	KBPopupMenu	*parent,
		QRect		rect,
		KBObject	*object,
		Qt::ButtonState	*bState
	)
{
	extern void 	makeReportMenu (QPopupMenu *, QObject *, uint, Qt::ButtonState *) ;

	KBPopupMenu	*newPopup = new KBPopupMenu (parent,   bState) ;
	KBPopupMenu	*blkPopup = new KBPopupMenu (newPopup, bState) ;

	blkPopup->insertEntry	  (false,  TR("Table Block"), object, SLOT(newTableBlock())) ;
	blkPopup->insertEntry	  (false,  TR("SQL Block"  ), object, SLOT(newSQLBlock  ())) ;
	blkPopup->insertEntry	  (false,  TR("Query Block"), object, SLOT(newQueryBlock())) ;
	newPopup->insertItem      (TR("New B&lock"),   blkPopup) ;
	newPopup->insertSeparator () ;

	if (object->isFramer() == 0)
	{
		bool	header	  = false ;
		bool	footer	  = false ;

		LITER
		(	KBNode,
			object->getChildren(),
			c,

			if (c->isHeader()) header = true ;
			if (c->isFooter()) footer = true ;
		)

		newPopup->insertEntry	  (header, TR("Add Header" ), object, SLOT(addHeader    ())) ;
		newPopup->insertEntry	  (footer, TR("Add Footer" ), object, SLOT(addFooter    ())) ;
		newPopup->insertSeparator () ;
	}

	makeReportMenu (newPopup, object, KNF_BLOCK|KNF_STATIC|KNF_DATA, bState) ;

	return	newPopup	;
}
#endif
