/***************************************************************************
    file	         : kb_dialog.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<stdarg.h>
#include	<qobjectlist.h>
#include	<qapplication.h>

#include	"tk_config.h"
#include	"tk_icons.h"
#include	"rk_hbox.h"
#include	"rk_pushbutton.h"

#if		__KB_AUTOMOC
#include 	"kb_dialog.moc"
#else
#include 	"kb_dialog.h"
#endif

#include	"kb_classes.h"
#include	"kb_options.h"
#include	"kb_ascii.h"
#include	"kb_manual.h"


#define		__DEBUG_DIALOG	(0)
#if		__DEBUG_DIALOG
#include	"kb_dprintf.h"
#endif


static	char	configGroup[]	= "Dialog Sizes" ;


/*  KBDialog								*/
/*  KBDialog	: Constructor for kbase dialog base class		*/
/*  caption	: QString	  : Dialog box caption			*/
/*  modal	: bool		  : Modal flag				*/
/*  name	: cchar *	  : Dialog name				*/
/*  size	: const QString & : Default size			*/
/*  (returns)	: KBDialog	  :					*/

KBDialog::KBDialog
	(	QString		caption,
		bool		modal,
		const char	*name,
		const QSize	&size
	)
	:
	RKDialog (qApp->activeWindow(), name, modal),
	m_name	 (name)
{
	init	 (caption, name, size) ;
}

/*  KBDialog								*/
/*  KBDialog	: Constructor for kbase dialog base class		*/
/*  caption	: QString	  : Dialog box caption			*/
/*  modal	: bool		  : Modal flag				*/
/*  name	: cchar *	  : Dialog name				*/
/*  size	: const QString & : Default size			*/
/*  (returns)	: KBDialog	  :					*/

KBDialog::KBDialog
	(	QString		caption,
		const char	*name,
		const QSize	&size
	)
	:
	RKDialog (qApp->activeWindow(), name, true),
	m_name	 (name)
{
	init	 (caption, name, size) ;
}

/*  KBDialog								*/
/*  KBDialog	: Destructor for kbase dialog base class		*/
/*  (returns)	:		:					*/

KBDialog::~KBDialog ()
{
	if (!m_name.isEmpty())
	{
		TKConfig *config = TKConfig::getConfig() ;
		config->setGroup   (configGroup) ;
		config->writeEntry (m_name, m_lastSize) ;
	}
}

/*  KBDialog								*/
/*  init	: Common initialisation code				*/
/*  caption	: QString	  : Dialog box caption			*/
/*  name	: cchar *	  : Dialog name				*/
/*  size	: const QString & : Default size			*/
/*  (returns)	: void		  :					*/

void	KBDialog::init
	(	const QString	&caption,
		const char	*name,
		const QSize	&size
	)
{
	setIcon	   (getSmallIcon ("rekall")) ;
	setCaption (caption) ;

	m_showMaxed  = false ;

	if (name != 0)
	{
		TKConfig *config = TKConfig::getConfig() ;
		config->setGroup (configGroup) ;
		m_lastSize = config->readSizeEntry (name, &size) ;
	}
}

/*  KBDialog								*/
/*  show	: Trap show to set layout				*/
/*  (returns)	: void		:					*/

void	KBDialog::show ()
{
	static	bool	inShow	= false	;

	QObjectList   *blist = queryList ("RKPushButton", 0, false, true) ;
	if (blist != 0)
	{
		RKPushButton	*ok	= 0	;
		RKPushButton	*cancel	= 0	;
		QSize		max	= QSize(0,0) ;

		QObjectListIt	iter (*blist) ;
		QObject		*obj ;

		while ((obj = iter.current()) != 0)
		{
			RKPushButton *b = (RKPushButton *)obj ;

			DPRINTF
			((	"KBDialog::show: class=[%s] name=[%s] text=[%s]\n",
				b->className(),
				b->name(),
				b->text().latin1()
			))	;

			if (qstrcmp(obj->name(), "ok"    ) == 0)
			{
				if (b->text().isEmpty()) b->setText(TR("&OK"    )) ;
				connect (obj, SIGNAL(clicked()),   SLOT(accept())) ;

				ok	= b ;
				max	= max.expandedTo (b->sizeHint()) ;
			}
			if (qstrcmp(obj->name(), "cancel") == 0)
			{
				if (b->text().isEmpty()) b->setText(TR("&Cancel")) ;
				connect (obj, SIGNAL(clicked()),   SLOT(reject())) ;

				cancel	= b ;
				max	= max.expandedTo (b->sizeHint()) ;
			}

			iter += 1 ;
		}

		DPRINTF
		((	"KBDialog::show: ok=%p cancel=%p max=%s\n",
			ok,
			cancel,
			_TEXT(max)
		))	;

		if ((ok != 0) && (cancel != 0))
		{
			ok    ->setMinimumSize (max) ;
			cancel->setMinimumSize (max) ;
		}
	}

	resizeLayout (this, marginHint(), spacingHint()) ;

	if	(m_showMaxed && !inShow)
	{
		inShow	= true	;
		QWidget::showMaximized() ;
		inShow	= false	;
	}
	else if	(m_lastSize.isValid())
	{
		resize		(m_lastSize) ;
		RKDialog::show	() ;
	}
	else	RKDialog::show	() ;
}

/*  KBDialog								*/
/*  resizeEvent	: Trap resize events					*/
/*  e		: QResizeEvent * : The event				*/
/*  (returns)	: void		 :					*/

void	KBDialog::resizeEvent
	(	QResizeEvent	*e
	)
{
	m_lastSize =  e->size()  ;
	RKDialog::resizeEvent(e) ;
}

int	KBDialog::marginHint ()
{
	return	KBOptions::getDlgMargin() ;
}

int	KBDialog::spacingHint ()
{
	return	KBOptions::getDlgSpacing() ;
}

/*  KBDialog								*/
/*  setupLayout	: Setup layout						*/
/*  widget	: QWidget *	: Parent widget for layout		*/
/*  (returns)	: void		:					*/

void	KBDialog::setupLayout
	(	QWidget		*widget
	)
{
	if (widget != 0)
		resizeLayout (widget, marginHint(), spacingHint()) ;
}


/*  [ STATIC METHODS ] -----------------------------------------------  */

/*  KBDialog								*/
/*  addOKCancel	: Add OK and cancel buttons				*/
/*  box		: RKBox *	  : Layout box to add buttons to	*/
/*  ok		: RKPushButton ** : Return OK button			*/
/*  cancel	: RKPushButton ** : Return Cancel button		*/
/*  help	: const char *	  : Help key				*/
/*  (returns)	: void		  :					*/

void	KBDialog::addOKCancel
	(	RKBox		*box,
		RKPushButton	**ok,
		RKPushButton	**cancel,
		const char	*help
	)
{
	RKHBox	*buttons = new RKHBox (box) ;
	buttons->addFiller () ;

	QPushButton *h = 0 ;
	if (help != 0)
		h = new KBManualPushButton (buttons, help) ;

	RKPushButton *o = new RKPushButton (buttons, "ok"    ) ;
	RKPushButton *c = new RKPushButton (buttons, "cancel") ;

	if (ok     != 0) *ok     = o ;
	if (cancel != 0) *cancel = c ;

	KBDialog::sameSize (o, c, h, 0) ;
}


void	KBDialog::sameSize
	(	QWidget		*widget,
		...
	)
{
	QPtrList<QWidget>	wList	;
	QWidget		*w	;
	va_list		ap	;

	va_start (ap, widget)	;
	wList.append (widget)	;

	while ((w = va_arg(ap, QWidget *)) != 0)
		wList.append (w);

	QSize	s = QSize(0,0)	;
	for (unsigned int idx1 = 0 ; idx1 < wList.count() ; idx1 += 1)
		s = s.expandedTo(wList.at(idx1)->sizeHint()) ;

	for (unsigned int idx2 = 0 ; idx2 < wList.count() ; idx2 += 1)
		wList.at(idx2)->setMinimumSize (s) ;
}
