/***************************************************************************
    file	         : kb_display.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qpainter.h>
#include	<qcursor.h>

#include	"kb_display.h"
#include	"kb_formblock.h"
#include	"kb_options.h"
#include	"kb_layout.h"
#include	"kb_dispwidget.h"
#include	"kb_dispscroller.h"
#include	"kb_report.h"
#include	"kb_ascii.h"
#include	"kb_writer.h"
#include	"kb_popupmenu.h"

#if		! __KB_RUNTIME
#include	"kb_toolbox.h"
#endif

#define		__DEBUG_DISPLAY	(0)
#if		__DEBUG_DISPLAY
#include	"kb_dprintf.h"
#endif

/*  snapPosition: Snap position to grid					*/
/*  pos		: int		: Position to be snapped		*/
/*  near	: bool		: True to snap to nearest		*/
/*  x		: bool		: True for X, else Y			*/
/*  (returns)	: int		: Position snapped to grid		*/

int	snapPosition
	(	int	pos,
		bool	bNear,
		bool	x
	)
{
	uint	step	;
	uint	delta	;

	/* Get the appropriate grid step. If zero (only if the user	*/
	/* edits the configuration file) or one then the position is	*/
	/* left unchanged (ie., treat zero as one).			*/
	step	= x ? KBOptions::getGridXStep() : KBOptions::getGridYStep() ;
	if (step <= 1) return pos ;

	/* To avoid round-towards-zero problems (ie, to get round-down	*/
	/* behaviour) we use an offset. THis must be a multiple of the	*/
	/* grid step and larger than any reasonable coordinate.		*/
	delta	= step	;
	while (delta < 100000) delta *= 100 ;

	/* If snapping to nearest rather than left/up, adjust the	*/
	/* position by half the grid step.				*/
	if (bNear) pos += step / 2 ;

	/* Rock and roll. Remember, sex-and-drugs-and-rock-and-roll are	*/
	/* only substitutes for caving.					*/
	return	((pos + delta) / step) * step - delta ;
}

/*  snapRect	: Snap rectangle to nearest grid points			*/
/*  rect	: QRect &	: Area to snap				*/
/*  (returns)	: void		:					*/

void	snapRect
	(	QRect		&rect
	)
{
	int	x	= rect.x()	;
	int	y	= rect.y()	;
	int	w	= rect.width ()	;
	int	h	= rect.height()	;

	w	= snapPosition (x + w, true, true ) ;
	h	= snapPosition (y + h, true, false) ;
	x	= snapPosition (x,     true, true ) ;
	y	= snapPosition (y,     true, false) ;
	w	= w - x ;
	h	= h - y ;

	rect	= QRect (x, y, w, h) ;
}


/*  ------------------------------------------------------------------  */



/*  KBDisplay								*/
/*  KBDisplay	: Constructor for root display window			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  object	: KBObject	: Owning object				*/
/*  (returns)	: KBDisplay	:					*/

KBDisplay::KBDisplay
	(	QWidget		*parent,
		KBObject	*object
	)
	:
	m_parent	(0),
	m_object	(object)
{
	m_marking  = false ; 
	m_lMargin  = 0	   ;
	m_rMargin  = 0	   ;
	m_tMargin  = 0	   ;
	m_bMargin  = 0	   ;
	m_canvas   = 0	   ;
	m_depth	   = 0	   ;
}



/*  KBDisplay								*/
/*  KBDisplay	: Constructor for display window			*/
/*  parent	: KBDisplay *	: Parent display			*/
/*  object	: KBObject	: Owning object				*/
/*  showBar	: uint		: Show row scroller bar/mini-nav	*/
/*  (returns)	: KNDisplay	:					*/

KBDisplay::KBDisplay
	(	KBDisplay	*parent,
		KBObject	*object
	)
	:
	m_parent	(parent),
	m_object	(object)
{
	m_marking  = false ;
	m_lMargin  = 0	   ;
	m_rMargin  = 0	   ;
	m_tMargin  = 0	   ;
	m_bMargin  = 0	   ;
	m_canvas   = 0	   ;
	m_depth	   = parent->depth() + 1 ;

	DPRINTF
	((	"KBDisplay::KBDisplay[nest]: %p\n",
		(void *)this
	))	;

}

/*  KBDisplay								*/
/*  KBDisplay	: Destructor for display window				*/
/*  (returns)	:		:					*/

KBDisplay::~KBDisplay ()
{
	DPRINTF
	((	"KBDisplay::~KBDisplay: %p\n",
		(void *)this
	))	;

	tearDown () ;
	m_object->displayGone () ;
}

void	KBDisplay::tearDown ()
{
	DPRINTF
	((	"KBDisplay::tearDown: %p: parent=%p\n",
		(void *)this,
		(void *)m_parent
	))	;

//	if (m_parent != 0)
//		m_object->tearDown () ;
}



#if	! __KB_RUNTIME

/*  KBDisplay								*/
/*  doDrawDisplay							*/
/*		: Handle painting					*/
/*  p		: QPainter *	: Painter				*/
/*  clip	: QRect		: Area to paint				*/
/*  (returns)	: void		:					*/

void	KBDisplay::doDrawDisplay
	(	QPainter 	*p,
		QRect		rect
	)
{
	KBReport *report = m_object->isReport() ;
	double	  ppm	 = pixelsPerMM () ;

	DPRINTF
	((	"KBDisplay::doDrawDisplay: %s\n",
		_TEXT(rect)
	))	;

	p->setPen(Qt::black) ;

	if (report != 0)
	{
		report->margins (m_lMargin, m_rMargin, m_tMargin, m_bMargin) ;

		m_lMargin = (int)(m_lMargin * ppm) ;
		m_rMargin = (int)(m_rMargin * ppm) ;
		m_tMargin = (int)(m_tMargin * ppm) ;
		m_bMargin = (int)(m_bMargin * ppm) ;
	}

	if (m_marking)
	{
		int	x = m_markX + m_lMargin ;
		int	y = m_markY + m_tMargin ;

		m_rubber  = QRect() ;
		p->save	  () ;
		moveRubberRect (p, QRect(x, y, m_markW, m_markH)) ;
		p->restore() ;
	}

	int	gx	= (int)KBOptions::getGridXStep() ;
	int	gy	= (int)KBOptions::getGridXStep() ;

	QSize	size	= getTopSize  () ;
	int	dw	= size.width  () + m_lMargin ;
	int	dh	= size.height () + m_tMargin ;

	DPRINTF
	((	"KBDisplay::doDrawDisplay: %s->(%d,%d)\n",
		_TEXT(size),
		dw,
		dh
	))	;

	if (gx <= 0) gx = 10 ;
	if (gy <= 0) gy = 10 ;

	for (int px = m_lMargin ; px <= dw -1 ; px += gx)
		for (int py = m_tMargin ; py <= dh - 1 ; py += gy)
			p->drawPoint (px, py) ;


	QRect	newArea	= QRect(m_lMargin, m_tMargin, dw - m_lMargin, dh - m_tMargin) ;
	p->drawRect (newArea) ;

	clearExposedArea (p, newArea) ;

	if (report != 0)
	{
		int	width	;
		int	height	;

		getPixelPageSize("A4", width, height) ;

		QBrush	brush	(QColor(0xe0, 0xe0, 0xe0)) ;

		p->fillRect (0, 0, width, m_tMargin, brush) ;
		p->fillRect (0, height - m_bMargin, width, m_bMargin, brush) ;
		p->fillRect (0, m_tMargin, m_lMargin, height - m_tMargin - m_bMargin, brush) ;
		p->fillRect (width - m_rMargin, m_tMargin, m_rMargin, height - m_tMargin - m_bMargin, brush) ;

		p->drawLine (m_lMargin, 0, m_lMargin, height) ;
		p->drawLine (width - m_rMargin,  0, width - m_rMargin, height) ;
		p->drawLine (width,   0, width,   height) ;

		p->drawLine (0, m_tMargin, width,   m_tMargin) ;
		p->drawLine (0, height - m_bMargin, width, height - m_bMargin) ;
		p->drawLine (0, height,  width,   height) ;
	}
}

/*  KBDisplay								*/
/*  doContextEvent							*/
/*		: Handler for mouse context menu events			*/
/*  cell	: QRect		: Cell for dynamic geometry		*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBDisplay::doContextEvent
	(	QRect		cell
	)
{
	if (m_object->showingDesign ())
	{
		KBPopupMenu *popup = m_object->designPopup(0, cell) ;
		popup->exec (QCursor::pos()) ;
		delete	popup	;
		return	true	;
	}

	m_object->contextMenu (0, 0) ;
	return	true	;
}

#endif

/*  KBDisplay								*/
/*  getDisplayWidget							*/
/*		: Get widget used for display				*/
/*  (returns)	: QWidget *	: Display widget			*/

QWidget	*KBDisplay::getDisplayWidget ()
{
	return	m_canvas	;
}

/*  KBDisplay								*/
/*  displayResized							*/
/*		: Pass size change up to owning object			*/
/*  (returns)	: void		:					*/

void	KBDisplay::displayResized
	(	const QSize	&size
	)
{
	/* This is called from the geometry management code *after* any	*/
	/* nested displays have been resized.				*/
	m_object->displayResized (size) ;
}


/*  KBDisplay								*/
/*  setPalette	: Set pallette						*/
/*  pal		: const QPalette *: Palette in question			*/
/*  (returns)	:		  : void				*/

void	KBDisplay::setPalette
	(	const QPalette *pal
	)
{
	m_canvas->setPalette (*pal) ;
	getDisplayWidget()->setEraseColor (pal->active().background()) ;
}

/*  KBDisplay								*/
/*  setFont	: Set font						*/
/*  font	: const QFont *	: Font in question			*/
/*  (returns)	:		: void					*/

void	KBDisplay::setFont
	(	const QFont *font
	)
{
	m_canvas->setFont (*font) ;
}

/*  KBDisplay								*/
/*  getPalette	: Get pallette						*/
/*  (returns)	: const QPalette: The palette				*/

const	QPalette KBDisplay::palette ()
{
	DPRINTF
	((	"KBDisplay::palette: %p: called\n",
		(void *)this
	))	;

	return	m_canvas->palette () ;
}

/*  KBDisplay								*/
/*  getFont	: Get font						*/
/*  (returns)	: const QFont	: The font				*/

const	QFont	KBDisplay::font ()
{
	return	m_canvas->font () ;
}

/*  KBDisplay								*/
/*  scrollToRow	: Pass row scroll up to owning form block if any	*/
/*  row		: uint		: Row number				*/
/*  (returns)	: void		:					*/

void	KBDisplay::scrollToRow
	(	uint	row
	)
{
	KBFormBlock *block = m_object->isFormBlock () ;

	if (block == 0) block = m_object->getFormBlock () ;
	if (block != 0) block->scrollToRow (row) ;
}

/*  KBDisplay								*/
/*  scrollBy	: Scroll by number of rows				*/
/*  delta	: int		: Number of rows			*/
/*  (returns)	: void		:					*/

void	KBDisplay::scrollBy
	(	int	delta
	)
{
	KBFormBlock *block = m_object->isFormBlock () ;

	if (block == 0) block = m_object->getFormBlock () ;
	if (block != 0) block->scrollBy (delta) ;
}

/*  KBDisplay								*/
/*  doOperation	: Pass action though to block				*/
/*  action	: KB::Action	: Action in question			*/
/*  qrow	: uint		: Query for for gotorow			*/
/*  (returns)	: void		:					*/

void	KBDisplay::doOperation
	(	KB::Action	action,
		uint		qrow
	)
{
	KBFormBlock *block = m_object->isFormBlock () ;

	if (block == 0) block = m_object->getFormBlock () ;
	if (block != 0) block->doOperation (action, qrow) ;
}

/*  KBDisplay								*/
/*  keyStroke	: Handle keystroke					*/
/*  key		: QKeyEvent *	: Key event				*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBDisplay::keyStroke
	(	QKeyEvent	*key
	)
{
	KBFormBlock *block = m_object->isFormBlock () ;
	if ((block == 0) || !m_object->showingData ()) return false ;
	return	block->KBNavigator::keyStroke (0, key) ;
}

/*  KBDisplay								*/
/*  sizeHint	: Get size hint for dynamic displays			*/
/*  (returns)	: QSize		: Base size				*/

QSize	KBDisplay::sizeHint ()
{
	return	m_canvas->sizeHint() ;
}

/*  KBDisplay								*/
/*  morphDestoyed: Catch destuction of morphed control			*/
/*  (returns)	 : void		:					*/

void	KBDisplay::morphDestroyed
	(	KBItem		*morph
	)
{
	m_morphList.removeRef (morph) ;
}

/*  KBDisplay								*/
/*  addMorph	: Add a morphing control				*/
/*  morph	: KBItem *	: Control in question			*/
/*  (returns)	: void		:					*/

void	KBDisplay::addMorph
	(	KBItem		*morph
	)
{
	m_morphList.append (morph) ;
}

/*  KBDisplay								*/
/*  remMorph	: Remove a morphing control				*/
/*  morph	: KBItem *	: Control in question			*/
/*  (returns)	: void		:					*/

void	KBDisplay::remMorph
	(	KBItem		*morph
	)
{
	m_morphList.removeRef (morph) ;
}


/*  KBDisplay								*/
/*  setRowColSetup							*/
/*		: Set dynamic layout rows and columns			*/
/*  rowSetup	: KBGridSetupList &	: Row setup			*/
/*  colSetup	: KBGridSetupList &	: Column setup			*/
/*  (returns)	: void			:				*/

void	KBDisplay::setRowColSetup
	(	const KBGridSetupList	&rowSetup,
		const KBGridSetupList	&colSetup
	)
{
	m_geometry.setRowColSetup (rowSetup, colSetup) ;
}

/*  KBDisplay								*/
/*  insertWidget: Insert widget into display				*/
/*  item	: KBLayoutItem * : Layout item for widget in question	*/
/*  (returns)	: void		 :					*/

void	KBDisplay::insertWidget
	(	KBLayoutItem		*item
	)
{
	/* Pass this call to the geometry object which handles the	*/
	/* request appropriately to the geometry management.		*/
	m_geometry.insertWidget (item) ;
}

/*  KBDisplay								*/
/*  updateDynamic: Update dynamic layout settings			*/
/*  (returns)	 : void		:					*/

void	KBDisplay::updateDynamic ()
{
	m_geometry.updateDynamic (m_object->attrGeom()) ;
}

void	KBDisplay::repaintMorphs
	(	QPainter	&p,
		const QRect	&rect
	)
{
	LITER
	(	KBItem,
		m_morphList,
		morph,
		morph->repaintMorph (&p, rect) ;
	)

}

/*  KBDisplay								*/
/*  redoLayout	: Recompute layout					*/
/*  invalidate	: bool		: Invalidate first			*/
/*  (returns)	: void		:					*/

void	KBDisplay::redoLayout
	(	bool		invalidate
	)
{
	if (m_canvas->isVisible())
	{	m_canvas->update      () ;
		m_geometry.redoLayout (invalidate) ;
	}
}

/*  KBDisplay								*/
/*  getParent	: Get parent display					*/
/*  (returns)	: KBDisplay *	: Parent display			*/

KBDisplay *KBDisplay::getParent ()
{
	return	m_parent   ;
}

/*  KBDisplay								*/
/*  getObject	: Get associated object					*/
/*  (returns)	: KBObject *	: Object				*/

KBObject  *KBDisplay::getObject	()
{
	return	m_object   ;
}

/*  KBDisplay								*/
/*  depth	: Get depth in display tree				*/
/*  (returns)	: uint		: Depth					*/

uint	KBDisplay::depth ()
{
	return	m_depth	;
}


/*  KBDisplay								*/
/*  doMousePressEvent							*/
/*		: Handler for mouse press events			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  cell	: QPoint	: Cell for dynamic geometry		*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBDisplay::doMousePressEvent
	(	QMouseEvent	*e,
		QPoint		cell
	)
{
	m_rubber = QRect() ;

	if (e->button() != QMouseEvent::LeftButton)
		return	false	;


	LITER
	(	KBItem,
		m_morphList,
		morph,
		if (morph->mouseClickHit (e->pos()))
			return true	;
	)

#if	! __KB_RUNTIME
	if (!m_object->showingDesign ()) return false ;
	if (m_marking) return true ;


	m_left	= 0	 ;
	m_top	= 0	 ;

	/* If the owner is a report then get the left and top margins	*/
	/* and adjust the position accordingly; if not then the		*/
	/* effective margin is zero.					*/
	KBReport *report ;
	if ((report = m_object->isReport()) != 0)
	{
		report->margins (m_left, m_top) ;

		m_left	= (int)(m_left * pixelsPerMM()) ;
		m_top	= (int)(m_top  * pixelsPerMM()) ;
	}

	int x	= e->x() - m_left ;
	int y	= e->y() - m_top  ;

	/* At the top level, check that we are inside the block area.	*/
	/* In either case, set the marking area bounds.			*/
	m_bounds	= getTopSize ()	;
	if ((x >= m_bounds.width()) || (y >= m_bounds.height()))
		return	false	;

	if ((x < 0) || (y < 0))
		return	false	;

	m_marking	= true	;
	m_startMX	= m_markX = x	;
	m_startMY	= m_markY = y	;
	m_startCell	= cell		;
	m_markW		= m_markH = 0	;

	m_canvas->grabMouse ()  ;
	return	true	;
#endif

	return	false	;
}

/*  KBDisplay								*/
/*  dpMouseMoveEvent							*/
/*		: Handle mouse movement during marking			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  cell	: QPoint	: Cell for dynamic geometry		*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBDisplay::doMouseMoveEvent
	(	QMouseEvent 	*e,
		QPoint
	)
{
#if	! __KB_RUNTIME
	if ((e->state() & Qt::LeftButton) == 0) return true ;
	if (!m_marking) return true ;

	int	x = e->x() ;
	int	y = e->y() ;

	/* Zaurus fix: We seem to get a (-2001,-2001) as the button is	*/
	/* released ...							*/
	if ((x < 0) || (y < 0)) return true ;

	/* Adjust by the margins if any, and then bound to the current	*/
	/* bounding rectangle.						*/
	x	-= m_left ;
	y	-= m_top  ;

	if (x >= m_bounds.width ()) x = m_bounds.width () - 1 ;
	if (y >= m_bounds.height()) y = m_bounds.height() - 1 ;

	if (x < 0) x = 0 ;
	if (y < 0) y = 0 ;


	if (x >= m_startMX)
		m_markX = m_startMX, m_markW = x - m_startMX ;
	else	m_markX = x,	     m_markW = m_startMX - x ;
	if (y >= m_startMY)
		m_markY = m_startMY, m_markH = y - m_startMY ;
	else	m_markY = y,	     m_markH = m_startMY - y ;

	QPainter p(m_canvas)  ;
	QRect	 r(m_markX + m_lMargin, m_markY + m_tMargin, m_markW, m_markH) ;
	cvtCtrlToView (r)     ;
	moveRubberRect(&p, r) ;
#endif
	return	true	;
}

/*  KBDisplay								*/
/*  doMouseReleaseEvent							*/
/*		: Handle mouse release to end marking			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  cell	: QPoint	: Cell for dynamic displaus		*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBDisplay::doMouseReleaseEvent
	(	QMouseEvent 	*e,
		QPoint		cell
	)
{
#if	! __KB_RUNTIME

	m_canvas->releaseMouse () ;
	if (!m_marking) return false ;
	m_marking = false    ;


	/* Small movements are ignored and suffice to select the object	*/
	/* in which the click took place.				*/
	if ((m_markW < 5) && (m_markH < 5))
	{
		endRubberRect () ;
		m_object->getLayout()->addSizer (m_object->getSizer(), false) ;
		return	true ;
	}


	/* If marking, then there are two possibilities, marking out to	*/
	/* create a new object, and marking out to select existing	*/
	/* objects.							*/
	int	cellx	= m_startCell.x()  ;
	int	celly	= m_startCell.y()  ;
	int	cellw	= cell.x() - cellx ;
	int	cellh	= cell.y() - celly ;

	if (cellw < 0) { cellx += cellw ; cellw = -cellw ; }
	if (cellh < 0) { celly += cellh ; cellh = -cellh ; }

	cellw	+= 1	;
	cellh	+= 1	;


	/* If the object is a report then adjust for the margins. These	*/
	/* will have been noted when the mouse was pressed.		*/
	if (m_object->isReport())
	{
		cellx	-= m_left ;
		celly	-= m_top  ; 
	}

	QRect	rect	(cellx, celly, cellw, cellh) ;

	/* If the shift key is down we always interpret the range as	*/
	/* identifying objects that it covers.				*/
	if ((e->state() & Qt::ShiftButton) != 0)
	{
		endRubberRect () ;
		m_object->selectOverlap (rect) ;
		return	true ;
	}

	DPRINTF
	((	"KBDisplay::mouseReleaseEvent: rect=%s\n",
		_TEXT(rect)
	))	;

	/* If the toolbox is in use then check for overlap. On overlap	*/
	/* select as above, otherwise it is a request for a new		*/
	/* object.							*/
	if (KBOptions::getUseToolbox() && KBToolBox::self()->showing())
	{
		if (m_object->checkOverlap (rect))
		{
			endRubberRect () ;
			m_object->selectOverlap (rect) ;
			return	true ;
		}

		m_object->addNewObject (rect) ;
		endRubberRect () ;
		return	true ;
	}

	/* No toolbox, then if there is no overlap, add a new object	*/
	/* for the specified area.					*/
	if (!m_object->checkOverlap (rect))
		m_object->addNewObject (rect) ;

	endRubberRect () ;
#endif
	return	true ;
}

#if	! __KB_RUNTIME

void	KBDisplay::moveRubberRect
	(	QPainter	*p,
		const QRect	&rect
	)
{
	p->setPen     (Qt::white ) ;
	p->setRasterOp(Qt::XorROP) ;

	if (m_rubber.isValid())
		p->drawRect (m_rubber) ;

	m_rubber = rect ;
	p->drawRect (m_rubber) ;
}

void	KBDisplay::endRubberRect ()
{
	if (m_rubber.isValid())
	{
		QPainter p (m_canvas) ;

		p.setPen     (Qt::white ) ;
		p.setRasterOp(Qt::XorROP) ;
		p.drawRect   (m_rubber  ) ;

		m_rubber = QRect() ;
	}
}

#endif


KBDisplay *KBDisplay::newTopDisplay
	  (	QWidget		*parent,
		KBObject	*object,
		uint		showBar,
		uint		stretch,
		bool		showRuler
	  )
{
	if (object->isDynamic())
		return	new KBDispWidget (parent, object, showBar) ;

	return	new KBDispScroller   (parent, object, showBar, stretch, showRuler) ;
}

