/***************************************************************************
    file		 : kb_display.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_DISPLAY_H
#define	_KB_DISPLAY_H

#include	<qsize.h>
#include	<qguardedptr.h>


#include 	"libkbase_exports.h"

#include	"kb_classes.h"
#include	"kb_attrgeom.h"
#include	"kb_geometry.h"

class		KBDispWidget	;
class		KBDispScroller	;
class		KBGridSetup	;
class		KBLayoutItem	;

class		QWidget		;
class		QScrollView	;


/*  KBDisplay								*/
/*  ---------								*/
/*  This class is effectively the widget onto which forms and the like	*/
/*  place their controls. However, since the top-most block must be	*/
/*  shows in a scroller (becuase the form can be very large, especially	*/
/*  the width of a table data display), KBDisplay provides an interface	*/
/*  which hides the various differences.				*/

class	LIBKBASE_API	KBDisplay
{
	uint			m_depth		;

	bool			m_marking	;
	int			m_startMX	;
	int			m_startMY	;
	QPoint			m_startCell	;
	int			m_markX	 	;
	int			m_markY	 	;
	int			m_markW	 	;
	int			m_markH	 	;

	QSize			m_bounds	;
	QRect			m_rubber	;

	int			m_left	 	;
	int			m_top	 	;

	QPtrList<KBItem>	m_morphList	;

#if	! __KB_RUNTIME
	void		moveRubberRect	(QPainter *, const QRect &) ;
	void		endRubberRect	()		;
#endif

protected :

	int			m_lMargin	;
	int			m_rMargin	;
	int			m_tMargin	;
	int			m_bMargin	;

	KBGeometry		m_geometry	;
	QGuardedPtr<QWidget>	m_canvas	;
	KBDisplay		*m_parent	;
	KBObject		*m_object	;

	void			morphDestroyed		(KBItem  *)	;

public	:

	KBDisplay (QWidget   *, KBObject *)	;
	KBDisplay (KBDisplay *, KBObject *)	;
	virtual	~KBDisplay () ;

	void		tearDown		() ;

	virtual void	showAs			(KB::ShowAs) = 0 ;
	virtual void	setShowbar		(uint)	     = 0 ;
	virtual void	setFrame		(int, int)   = 0 ;
	virtual void	setRowRange		(uint, uint, uint, uint, uint ) = 0 ;	
	virtual QSize	effectiveSize		() = 0	;
	virtual void	setStretchable		(uint)	     = 0 ;
	virtual void	setTitle		(const QString &)	= 0 ;
	virtual void	setTagLabel		(const QString &)	= 0 ;
	virtual void	setBackgroundPixmap	(const QPixmap &, int) 	= 0 ;
	virtual QSize  	getTopSize 		() = 0	;
	virtual QRect  	getTopRect 		() = 0	;
	virtual void	resizeContents		(const QSize   &)	= 0 ;
	virtual void	setDisplayGeometry	(const QRect   &)	= 0 ;
	virtual void	updateMorph		(KBItem       *, const QRect &)	= 0 ;
	virtual void	updateMorph		(KBItem       *, uint) 		= 0 ;
	virtual void	cvtViewToCtrl		(QRect &) 		= 0 ;
	virtual void	cvtCtrlToView		(QRect &) 		= 0 ;
	virtual void	setWidgetGeometry	(QWidget *, const QRect &) = 0 ;
	virtual QWidget	*getTopWidget		() = 0	;
	virtual	void	clearExposedArea	(QPainter *, const QRect &)	= 0 ;
	virtual void	addMorph		(KBItem      *) ;
	virtual void	remMorph		(KBItem      *) ;
	virtual void	makeVisible		(const QRect &, QFocusEvent::Reason) = 0 ;
	virtual QSize	showBarUsed		() = 0 ;
	virtual	void	show			() = 0 ;
	virtual	int	width			() = 0 ;
	virtual	int	height			() = 0 ;
	virtual	void	moveTags		(KBDispWidget *dispWidget, uint)= 0 ;

	void		setRowColSetup
			(	const KBGridSetupList	&,
				const KBGridSetupList	&
			)	;

	void		insertWidget   		(KBLayoutItem 	*) ;
	void		updateDynamic		() ;
	void		repaintMorphs		(QPainter &, const QRect &) ;
	void		setPalette 		(const QPalette *) ;
	void		setFont    		(const QFont    *) ;

	QSize		getBaseSize		() ;
	QSize		sizeHint		() ;

	QWidget		*getDisplayWidget	() ;


	const QPalette  palette			() ;
	const QFont     font			() ;

	bool		doMousePressEvent	(QMouseEvent *, QPoint)	;
	bool		doMouseMoveEvent	(QMouseEvent *, QPoint)	;
	bool		doMouseReleaseEvent	(QMouseEvent *, QPoint)	;

#if	! __KB_RUNTIME
	bool		doContextEvent		(QRect) ;
	void		doDrawDisplay		(QPainter    *, QRect ) ;
#endif

	void		scrollToRow		(uint)			;
	void		scrollBy		( int)			;
	void		doOperation		(KB::Action, uint)	;
	bool		keyStroke		(QKeyEvent *)		;

	void		displayResized		(const QSize &)		;
	void		redoLayout		(bool =  false)		;

	KBDisplay 	*getParent 		()			;
	KBObject  	*getObject		()			;
	uint		depth			()			;



	static	KBDisplay *newTopDisplay
			  (	QWidget 	*parent,
				KBObject	*object,
				uint		showBar,
				uint		stretch,
				bool		showRuler
			  )	;
}	;

#endif	// _KB_DISPLAY_H
