/***************************************************************************
    file	         : kb_dispscroller.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qpainter.h>
#include	<qobjectlist.h>

#if		__KB_AUTOMOC
#include	"kb_dispscroller.moc"
#else
#include 	"kb_dispscroller.h"
#endif

#include	"kb_item.h"
#include	"kb_display.h"
#include	"kb_dispwidget.h"
#include	"kb_options.h"
#include	"kb_ascii.h"
#include	"kb_report.h"
#include	"kb_attrnav.h"
#include	"kb_attrstretch.h"
#include	"kb_sizer.h"
#include	"kb_writer.h"
#include	"kb_layoutitem.h"


#define		__DEBUG_DISPSCROLLER	(0)
#if		__DEBUG_DISPSCROLLER
#include	"kb_dprintf.h"
#endif


/*  KBDispScrollArea							*/
/*  KBDispScrollArea							*/
/*		: Display scrollview constructor			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  display	: KBDisplay *	: Owning display object			*/
/*  topSize	: QSize &	: Access in parent			*/
/*  rulerX	: KBRuler *	: Associated X ruler			*/
/*  rulerY	: KBRuler *	: Associated Y ruler			*/
/*  showBar	: uint		: Show row scroll bar			*/
/*  stretch	: bool		: Form in stretch mode			*/
/*  (returns)	: KBDispScrollArea:					*/

KBDispScrollArea::KBDispScrollArea
	(	QWidget			*parent,
		KBDisplay		*display,
		QSize			&topSize,
		KBGeometry		&geometry,
		KBRuler			*rulerX,
		KBRuler			*rulerY,
		uint			showBar,
		uint			stretch
	)
	:
	QScrollView	(parent, 0, WNoAutoErase|WStaticContents),
	m_display	(display),
	m_topSize	(topSize),
	m_geometry	(geometry),
	m_rulerX	(rulerX),
	m_rulerY	(rulerY),
	m_showBar	(0),
	m_stretch	(false)
{
	m_inSetRange	= false	;
	m_rLabel	= 0	;
	m_vBar		= 0	;
	m_rLabel_d	= 0	;
	m_vBar_d	= 0	;
	m_showing	= KB::ShowAsUnknown ;

	enableClipper	(true)	  ;
	setShowbar	(showBar) ;
	setStretchable	(stretch) ;
	setResizePolicy	(Manual ) ;
}

/*  KBDispScrollArea							*/
/*  ~KBDispScrollArea							*/
/*		: Display scrollview destructor				*/
/*  (returns)	:		:					*/

KBDispScrollArea::~KBDispScrollArea ()
{
	m_display->tearDown () ;
}

void	KBDispScrollArea::addChild
	(	QWidget		*child,
		int		x,
		int		y
	)
{
//	KBReport *report = m_display->getObject()->isReport() ;
//
//	if (report != 0)
//	{
//		int	left	;
//		int	top	;
//
//		report->margins (left, top) ;
//
//		x	+= (int)(left * pixelsPerMM()) ;
//		y	+= (int)(top  * pixelsPerMM()) ;
//	}

	QScrollView::addChild (child, x, y) ;
}

/*  KBDispScrollArea							*/
/*  setShowbar	: Enable/display display of row scroll bar		*/
/*  showBar	: uint		: Show row scroll bar			*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::setShowbar
	(	uint	showBar
	)
{
	if (showBar == m_showBar) return ;

	m_showBar = showBar ;

	if (m_showBar == 0)
	{
		DELOBJ	(m_vBar    ) ;
		DELOBJ	(m_vBar_d  ) ;
		DELOBJ	(m_rLabel  ) ;
		DELOBJ	(m_rLabel_d) ;
		return	;
	}

	if (m_vBar == 0)
	{
		/* Setting visible. We need two sets, since in design	*/
		/* mode they must be anchored to the viewport, while	*/
		/* in data mode to the scroller itself.			*/
		m_vBar_d   = new QScrollBar  (QScrollBar::Vertical, viewport()) ;
		m_rLabel_d = new KBRecordNav (viewport(), m_vBar_d->sizeHint().width()) ;
		addChild (m_vBar_d  ) ;
		addChild (m_rLabel_d) ;

		m_vBar	   = new QScrollBar  (QScrollBar::Vertical, this) ;
		m_rLabel   = new KBRecordNav (this, m_vBar->sizeHint().width()) ;

		/* Wire up the controls. The one in design mode is for	*/
		/* visual effect only.					*/
		connect
		(	m_vBar,
			SIGNAL	(valueChanged (int)), 
			SLOT	(vbarMoved    ())
		)	;
		connect
		(	m_rLabel,
			SIGNAL	(operation    (KB::Action, uint)),
			SLOT  	(slotOperation(KB::Action, uint))
		)	;
	}

	/* We must be in design mode, so maybe show the design widgets	*/
	/* and hide the data mode ones.					*/
	if ((m_showBar & NAV_SCROLL) == 0)
		m_vBar_d  ->hide () ;
	else	m_vBar_d  ->show () ;

	if ((m_showBar & NAV_MINI  ) == 0)
		m_rLabel_d->hide () ;
	else	m_rLabel_d->show () ;

	m_vBar    ->hide () ;
	m_rLabel  ->hide () ;

	sizeAdjusted   () ;
}

/*  KBDispScrollArea							*/
/*  setSetstretchable							*/
/*		: Set display stretching mode				*/
/*  stretch	: uint		: Mode					*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::setStretchable
	(	uint		stretch
	)
{
	m_stretch = stretch ;
}

/*  KBDispScrollArea							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::showAs
	(	KB::ShowAs	mode
	)
{
	m_showing = mode ;

#if	! __KB_RUNTIME
	if (mode == KB::ShowAsDesign)
		setVScrollBarMode (Auto) ;
	else
#endif
		setVScrollBarMode ((m_showBar & NAV_SCROLL) != 0 ? AlwaysOff : Auto) ;

	if (m_showBar != 0)
	{
#if	! __KB_RUNTIME
		/* If showing the scroll bar and record label then show	*/
		/* and hide the appropriate pair.			*/
		if (mode == KB::ShowAsDesign)
		{
			if ((m_showBar & NAV_SCROLL) == 0)
				m_vBar_d  ->hide () ;
			else	m_vBar_d  ->show () ;

			if ((m_showBar & NAV_MINI  ) == 0)
				m_rLabel_d->hide () ;
			else	m_rLabel_d->show () ;

			m_vBar    ->hide () ;
			m_rLabel  ->hide () ;
		}
		else
#endif
		{
			if ((m_showBar & NAV_SCROLL) == 0)
				m_vBar    ->hide () ;
			else	m_vBar    ->show () ;

			if ((m_showBar & NAV_MINI  ) == 0)
				m_rLabel  ->hide () ;
			else	m_rLabel  ->show () ;

			m_vBar_d  ->hide () ;
			m_rLabel_d->hide () ;

			m_vBar	  ->raise() ;
		}
	}

	setEnabled (true)   ;
	viewport()->erase() ;

	repaintContents (contentsX(), contentsY(), visibleWidth(), visibleHeight()) ;
}

/*  KBDispScrollArea							*/
/*  updateMorph	: Update a morphed control				*/
/*  morph	: KBItem *	: Morphed item				*/
/*  mRect	: const QRect &	: Area occupied by control		*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::updateMorph
	(	KBItem		*,
		const QRect	&mRect
	)
{
	viewport()->update (mRect) ;
}

/*  KBDispScrollArea							*/
/*  updateMorph	: Update a morphed control				*/
/*  morph	: KBItem *	: Morphed item				*/
/*  drop	: uint		: Control display row			*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::updateMorph
	(	KBItem		*morph,
		uint		drow
	)
{
	QPainter p  (viewport  ()) ;
	QPoint	 o = contentsToViewport(QPoint(0,0)) ;
	p.translate (o.x(), o.y()) ;

	morph->repaintMorph  (&p, drow) ;
}

void	KBDispScrollArea::contentsWheelEvent
	(	QWheelEvent	*e
	)
{
	QScrollView::contentsWheelEvent (e) ;
}

/*  KBDispScrollArea							*/
/*  contentsMousePressEvent						*/
/*		: Handler for mouse press events			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::contentsMousePressEvent
	(	QMouseEvent	*e
	)
{
	if (!m_display->doMousePressEvent (e, m_geometry.getCell (e->pos())))
		QScrollView::contentsMousePressEvent (e) ;
}

/*  KBDispScrollArea							*/
/*  contentsMouseMoveEvent						*/
/*		: Handle mouse movement during marking			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::contentsMouseMoveEvent
	(	QMouseEvent 	*e
	)
{
	m_display->doMouseMoveEvent (e, m_geometry.getCell (e->pos())) ;
}

/*  KBDispScrollArea							*/
/*  contentsMouseReleaseEvent						*/
/*		: Handle mouse release to end marking			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::contentsMouseReleaseEvent
	(	QMouseEvent 	*e
	)
{
	m_display->doMouseReleaseEvent (e, m_geometry.getCell (e->pos())) ;
}

/*  KBDispScrollArea							*/
/*  contentsContextMenuEvent						*/
/*		: Handle context menu event				*/
/*  e		: QContextMenuEvent * : Event				*/
/*  (returns)	: void		      :					*/

void	KBDispScrollArea::contentsContextMenuEvent
	(	QContextMenuEvent	*e
	)
{
#if	! __KB_RUNTIME
	m_display->doContextEvent
	(	QRect (m_geometry.getCell (e->pos()), QSize(1,1))
	)	;
#endif
}

/*  KBDispScrollArea							*/
/*  contentsMouseDoubleClickEvent					*/
/*		: Handler for mouse double click events			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::contentsMouseDoubleClickEvent
	(	QMouseEvent	*e
	)
{
	/* Debugging hook. Double-click with the control and shift keys	*/
	/* down prints the widget tree.					*/
	if ((e->state() & (ShiftButton|ControlButton)) == (ShiftButton|ControlButton))
	{
		extern	void	printWidgetTree (QWidget *, uint, int, int = 0) ;
		printWidgetTree (this, 0, -1, 0xff) ;
		return	;
	}

#if	! __KB_RUNTIME
	/* Treat a double-click as a context event for the position at	*/
	/* which the click occurs.					*/
	m_display->doContextEvent
	(	QRect(m_geometry.getCell(e->pos()), QSize(1,1))
	)	;
#endif
}

/*  KBDispScrollArea							*/
/*  viewportPaintEvent							*/
/*		: Handle viewport repaint				*/
/*  pe		: QPaintEvent *	: Paint event				*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::viewportPaintEvent
	(	QPaintEvent  	*pe
	)
{
	/* Originally used "drawContents" but this appears always to	*/
	/* pass the entire visible area, which means that in table data	*/
	/* view, each time you move widget (and hence morphing moves)	*/
	/* all (visible) morphs got redrawn!				*/
	QRect	pRect	= pe->rect() ;
	QRect	cRect
		(	viewportToContents(pRect.topLeft    ()),
			viewportToContents(pRect.bottomRight())
		)	;

	QPainter p	(viewport()) ;
	QPoint	 tr	= contentsToViewport(QPoint(0,0)) ;
	p.translate (tr.x(), tr.y()) ;

#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		p.eraseRect (cRect ) ;
		m_display->doDrawDisplay (&p, cRect) ;
	}
#endif

	if (m_rulerX != 0) m_rulerX->setOffset (contentsX()) ;
	if (m_rulerY != 0) m_rulerY->setOffset (contentsY()) ;

	m_display->repaintMorphs (p, cRect) ;
}

/*  KBDispScrollArea							*/
/*  setVBarGeometry							*/
/*		: Set geometry of the vertical scroll bar		*/
/*  vbar	: QScrollBar &	: Vertical scroll bar			*/
/*  x, y	: int, int	: Default x and y			*/
/*  w, h	: int, int	: Default w and h			*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::setVBarGeometry
	(	QScrollBar	&vbar,
		int		x,
		int		y,
		int		w,
		int		h
	)
{
	if (h > height() - w) h -= w ;
	vbar.setGeometry (x, y, w, h) ;
}

/*  KBDispScrollArea							*/
/*  setHBarGeometry							*/
/*		: Set geometry of the horizontal scroll bar		*/
/*  hbar	: QScrollBar &	: Horizontal scroll bar			*/
/*  x, y	: int, int	: Default x and y			*/
/*  w, h	: int, int	: Default w and h			*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::setHBarGeometry
	(	QScrollBar	&hbar,
		int		x,
		int		y,
		int		w,
		int		h
	)
{
	if (w > width() - h)
		w -= h ;

	if (m_showBar && (m_showing == KB::ShowAsData))
	{
		x += m_rLabel->size().width() ;
		w -= m_rLabel->size().width() ;
	}

	hbar .setGeometry (x, y, w, h) ;
}


/*  KBDispScrollArea							*/
/*  resizeEvent	: Handle a resize event					*/
/*  e		: QResizeEvent * : The event				*/
/*  (returns)	: void		 :					*/

void	KBDispScrollArea::resizeEvent
	(	QResizeEvent	*e
	)
{
	QScrollView::resizeEvent (e) ;

	DPRINTF
	((	"KBDispScrollArea::resizeEvent: %s\n mode=%d str=%da\n",
		_TEXT(e->size()),
		m_showing,
		m_stretch
	))	;

	switch (m_showing)
	{
		case KB::ShowAsData :
			/* If the display is stretchable and we are	*/
			/* _not_ in design mode, then resize the	*/
			/* contents to be exactly the same size as the	*/
			/* visible area, and notify the display.	*/
			if (m_stretch == STR_STRETCH)
			{
				int	vw	= visibleWidth () ;
				int	vh	= visibleHeight() ;
				QSize	hint	= m_geometry.sizeHint() ;

				if (hint.width() > vw) vw = hint.width() ;

				resizeContents	  (vw, vh)	;
				m_topSize = QSize (vw, vh)	;
				m_geometry.resize (m_topSize)	;
			}
			if (m_stretch == STR_GRID)
			{
				int	cw	= contentsWidth () ;
				int	vh	= visibleHeight () ;

				resizeContents	  (cw, vh)	;
				m_topSize = QSize (cw, vh)	;
				m_geometry.resize (m_topSize)	;
			}
			break	;

		case KB::ShowAsDesign :
			break	;

		default	:
			break	;
	}

	DPRINTF
	((	"KBDispScrollArea::resizeEvent: v=(%d,%d) c=(%d,%d,%d,%d) s=(%d,%d)\n",
		visibleWidth  (),
		visibleHeight (),
		contentsX     (),
		contentsY     (),
		contentsWidth (),
		contentsHeight(),
		hScrollBarMode(),
		vScrollBarMode()
	))	;

	sizeAdjusted () ;
}

/*  KBDispScrollArea							*/
/*  sizeAdjusted: Size has been adjusted				*/
/*  (returns)	: void		 :					*/

void	KBDispScrollArea::sizeAdjusted ()
{
	/* If we are showing our own scrollbar then we have to fix its	*/
	/* geometry outselves; we must also move the record count label	*/
	/* Where it goes also depends on whether we are in design mode	*/
	/* or not.							*/
	if (m_showBar)
	{
		int	w	;
		int	h	;
		int	wsfix	= frameWidth() ;

#if	! __KB_RUNTIME
		if (m_showing == KB::ShowAsDesign)
		{
			/* In design mode, our scrollbar and label are	*/
			/* aligned to the notional area occupied by the	*/
			/* topmost form widget.				*/
			w	= m_topSize.width  () ;
			h	= m_topSize.height () ;
		}
		else
#endif
		{	/* Not in design mode so our bar and label go	*/
			/* along the edges of the scrollview.		*/
			w	= width () ;
			h	= height() ;
		}


		QScrollBar *hBar = horizontalScrollBar   () ;
		int	   sbw   = m_vBar->sizeHint().width() ;
		int	   sbh   = hBar == 0 ? sbw : hBar->sizeHint().height() ;


		/* The design mode scrollbar and label are adjusted so	*/
		/* that they stay fixed relative to the viewport ...	*/
		moveChild (m_vBar_d,   w - sbw - wsfix * 2, 0) ;
		moveChild (m_rLabel_d, 0, h - sbh - wsfix * 2) ;
		m_vBar_d  ->resize (sbw,  h - sbh - wsfix * 2) ;

		/* ... while the data mode scroller and label are kept	*/
		/* at the edjes of the scroll view.			*/
		m_vBar  ->setGeometry
		(	w - sbw - wsfix, // - contentsX(),
			wsfix,		 // - contentsY(),
			sbw,
			h - sbh - wsfix *2
		)	;
		m_rLabel->move   (wsfix, h - m_rLabel->size().height() - wsfix) ;

#if	! __KB_RUNTIME
		if (m_showing == KB::ShowAsDesign)
		{
			m_vBar_d  ->raise  () ;
			m_rLabel_d->raise  () ;
		}
		else
#endif
		{
			m_vBar    ->raise  () ;
			m_rLabel  ->raise  () ;
		}
	}
}

/*  KBDispScrollArea							*/
/*  setRowRange	: Set range of rows being displayed			*/
/*  extra	: uint		: Extra insertion rows			*/
/*  curQRow	: uint		: Current query row			*/
/*  curDRow	: uint		: Index of topmost displayed row	*/
/*  dispRows	: uint		: Number of rows actually visible	*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::setRowRange
	(	uint	totalRows,
		uint	extra,
		uint	curQRow,
		uint	curDRow,
		uint	dispRows
	)
{
	if (m_showBar)
	{
		/* Note the scrollbar-moved feedback is disabled while	*/
		/* setting the range. This is because range setting is	*/
		/* invoked from the block object, which will already	*/
		/* have scrolled its contents.				*/
		m_inSetRange = true  ;
		m_vBar->setRange   (0, totalRows + extra - dispRows) ;
		m_vBar->setSteps   (1, dispRows <= 1 ? 1 : dispRows  - 1) ;
		m_vBar->setValue   (curDRow) ;
		m_inSetRange = false ;

		m_rLabel->setRecord(curQRow, totalRows) ;
	}
}

/*  KBDispScrollArea							*/
/*  vbarMoved	: Notification of scrollbar change			*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::vbarMoved ()
{
	/* If we are not setting the value then we need to notify the	*/
	/* owning block. But, it is possible that the block may reject	*/
	/* the scroll and move us back, and there does not seem to be	*/
	/* a "allow this scroll" hook in QScrollBar. Now, if we allow	*/
	/* a reset _before_ the scroll bar move-and-redraw completes,	*/
	/* we get some nasty recursive error loops. So, set a timer to	*/
	/* give time for things to settle down ....			*/
	if (!m_inSetRange)
		m_scrollTimer.singleShot (200, this, SLOT(scrollTimeout())) ;
}

/*  KBDispScrollArea							*/
/*  scrollTimeout: Timeout on scroll bar movement			*/
/*  (returns)	 : void		:					*/

void	KBDispScrollArea::scrollTimeout ()
{
	int	value = m_vBar->value() ;
	m_display->scrollToRow (value < 0 ? 0 : value) ;
}


/*  KBDispScrollArea							*/
/*  slotOperation: Action requested					*/
/*  action	 : KB::Action	: Action in question			*/
/*  qrow	 : uint		: Possible query row			*/
/*  (returns)	 : void		:					*/

void	KBDispScrollArea::slotOperation
	(	KB::Action	action,
		uint		qrow
	)
{
	DPRINTF
	((	"KBDispScrollArea::slotOperation: size=(%d,%d)\n",
		contentsWidth (),
		contentsHeight()
	))	;

	m_display->doOperation (action, qrow) ;
}

/*  KBDispScrollArea							*/
/*  effectiveSize : Get effective display size				*/
/*  topSize	  : QSize	: Top object size			*/
/*  (returns)	  : QSize	: Effective size			*/

QSize	KBDispScrollArea::effectiveSize
	(	QSize		topSize
	)
{
#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		int	w = topSize.width () ;
		int	h = topSize.height() ;

		if (m_showBar)
		{	w -= verticalScrollBar  ()->sizeHint().width () ;
			h -= horizontalScrollBar()->sizeHint().height() ;
		}

		return	QSize (w, h) ;
	}
#endif

	if (m_stretch == STR_STRETCH)
	{
		int	w = visibleWidth () ;
		int	h = visibleHeight() ;

		if (m_showBar)
			w -= verticalScrollBar  ()->sizeHint().width () ;

		if (m_showBar || horizontalScrollBar()->isVisible())
			h -= horizontalScrollBar()->sizeHint().height() ;

		return	QSize (w, h) ;
	}

	return	topSize	;
}

/*  KBDispScrollArea							*/
/*  o		: QObject *	: Actually pointer at the widget	*/
/*  e		: QEvent *	: Event to process			*/
/*  (returns)	: bool		: False to continue processing		*/

bool	KBDispScrollArea::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	/* NOTE: It seems that the QScrollView installs an event filter	*/
	/* on the viewport itself, so we do not need install it. The	*/
	/* other point is to call QScrollView::eventFilter if we are	*/
	/* not consuming the event.					*/

	/* Pass key presses to the display object and thence the block	*/
	/* navigator. This is used to bring the current control back	*/
	/* into view if it has been scrolled out of site.		*/
	if ((o == viewport()) && (e->type() == QEvent::KeyPress))
		return	m_display->keyStroke ((QKeyEvent *)e) ;


	/* Trap wheel events. These are passed to the display object	*/
	/* to scroll, and are then accepted and consumed.		*/
	if (e->type() == QEvent::Wheel)
	{
		QWheelEvent *w = (QWheelEvent *)e ;
		m_display->scrollBy (- w->delta() / 120) ;
		w->accept() 	;
		return	true	;
	}

	/* On show we need to set up the layout, since the geometry	*/
	/* management will not have worked down through the previously	*/
	/* hidden widgets.						*/
	if (e->type() == QEvent::Show)
	{
		m_display->redoLayout(true) ;
	}

	return	QScrollView::eventFilter (o, e) ;
}

/*  KBDispScrollArea							*/
/*  moveTags	: Move design tags					*/
/*  widget	: KBDispWidget * : Widget requiring move		*/
/*  y		: uint		 : Y position of tag			*/
/*  (returns)	: void		 :					*/

void	KBDispScrollArea::moveTags
	(	KBDispWidget	*widget,
		uint		y
	)
{
	QLabel	*tag	= widget->getTagLabel (viewport()) ;
	QPoint	pos	= viewportToContents  (QPoint(widget->width(), y)) ;

	if (tag != 0)
	{
		KBReport *report = m_display->getObject()->isReport() ;

		int	x = widget->width() ;
		int	y = pos.y() ;

		if (report != 0)
		{
			int	left	;
			int	top	;

			report->margins (left, top) ;

			y -= (int)(top * pixelsPerMM()) ;
		}

		moveChild (tag, x, y) ;
	}


	QObjectList *children = widget->queryList("KBDispWidget", 0, true, false) ;
	if (children == 0) return ;

	QObjectListIt iter (*children) ;
	QObject	      *obj ;

	while ((obj = iter.current()) != 0)
	{
		KBDispWidget *dw  = (KBDispWidget *)obj	  ;
		moveTags (dw, y + dw->y()) ;
		*++iter ;
	}

	delete	children ;
}

/*  KBDispScrollArea							*/
/*  makeVisible	: Make a control area visible in the viewport		*/
/*  rRect	: const QRect &		: Control area			*/
/*  reason	: QFocusEvent::Reason	: Associated focus reason	*/
/*  (returns)	: void			:				*/

void	KBDispScrollArea::makeVisible
	(	const QRect		&rRect,
		QFocusEvent::Reason	reason
	)
{
	int	contX	= contentsX	() ;
	int	contY	= contentsY	() ;
	int	visW	= visibleWidth	() ;
	int	visH	= visibleHeight	() ;
	bool	move	= false	;

	if (verticalScrollBar  () != 0) visW -= verticalScrollBar  ()->width () ;
	if (horizontalScrollBar() != 0) visH -= horizontalScrollBar()->height() ;

	DPRINTF
	((	"KBDispScrollArea::makeVisible: rl=%d rr=%d cx=%d vw=%d\n",
		rRect.left (),
		rRect.right(),
		contX,
		visW
	))	;

	if ((rRect.right () >= contX + visW) || (rRect.left() < contX))
	{
		if (reason == QFocusEvent::Backtab)
			contX	= rRect.right() - visW	;
		else	contX	= rRect.left ()		;

		move	= true	;
	}

	if ((rRect.bottom() >= contY + visH) || (rRect.top () < contY))
	{
		if (reason == QFocusEvent::Backtab)
			contY	= rRect.bottom() - visH ;
		else	contY	= rRect.top   ()	;

		move	= true	;
	}

	if (move)
	{
		DPRINTF
		((	"KBDispScrollArea::makeVisible: to (%d,%d\n",
			contX,
			contY
		))	;

		setContentsPos (contX, contY) ;
	}
}

/*  KBDispScrollArea							*/
/*  focusNextPrevChild							*/
/*		: Next/prev focus change handler			*/
/*  next	: bool		: Moving to next control		*/
/*  (returns)	: void		:					*/

bool	KBDispScrollArea::focusNextPrevChild
	(	bool
	)
{
	/* This method is overridden since we handle all this ourselves	*/
	/* Otherwise, we get problems as controls are morphed back and	*/
	/* forth.							*/
	return	true ;
}

/*  KBDispScrollArea							*/
/*  setContentsPos							*/
/*		: Set contents position in viewport			*/
/* x, y		: int, int	: Top-left position			*/
/*  (returns)	: void		:					*/

void	KBDispScrollArea::setContentsPos
	(	int		x,
		int		y
	)
{
	DPRINTF
	((	"KBDispScrollArea::setContentsPos: (%d,%d)\n",
		x, y
	))	;

	QScrollView::setContentsPos (x, y) ;

}

QSize	KBDispScrollArea::showBarUsed ()
{
	int	w	= 0	;
	int	h	= 0	;

	if ((m_showBar & NAV_SCROLL) != 0) w = m_vBar_d  ->width () ;
	if ((m_showBar & NAV_MINI  ) != 0) h = m_rLabel_d->height() ;

	return	QSize (w, h)	;
}

/*  ------------------------------------------------------------------  */

/*  KBDispScroller							*/
/*  KBDispScroller							*/
/*		: Display scrollview constructor			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  object	: KBObject *	: Owning object				*/
/*  showBar	: uint		: Show row scroll bar			*/
/*  stretch	: uint		: Form in stretch mode			*/
/*  showRuler	: bool		: Show design ruler			*/
/*  (returns)	: KBDispScrollArea:					*/

KBDispScroller::KBDispScroller
	(	QWidget		*parent,
		KBObject	*object,
		uint		showBar,
		uint		stretch,
		bool		showRuler
	)
	:
	QWidget		(parent),
	KBDisplay	(parent, object),
	m_rulerX	(showRuler ? new KBRuler (this, true ) : 0),
	m_rulerY	(showRuler ? new KBRuler (this, false) : 0),
	m_scroller	(new KBDispScrollArea
				(	this,
					this,
					m_topSize,
					m_geometry,
					m_rulerX,
					m_rulerY,
					showBar,
					stretch
			)	),
	m_grid		(new QGridLayout (this))
{
	m_geometry.init	(0, this)	;

	if (showRuler)
	{
		double	step	= 10	;
		int	frac	= 5	;
		int	lincr	= 10	;

		if (KBOptions::getDesignInches())
		{
			step	= 25.4	;
			frac	= 10	;
			lincr	= 1	;
		}


		m_rulerX->setFrame	(m_scroller->frameWidth()) ;
		m_rulerX->setStep	(step, pixelsPerMM(), frac, lincr) ;

		m_rulerY->setFrame	(m_scroller->frameWidth()) ;
		m_rulerY->setStep	(step, pixelsPerMM(), frac, lincr) ;

		m_grid  ->addWidget 	(m_rulerX,   0, 1) ;
		m_grid  ->addWidget 	(m_rulerY,   1, 0) ;
		m_grid  ->addWidget 	(m_scroller, 1, 1) ;
	}
	else	m_grid  ->addWidget	(m_scroller, 0, 0) ;

	m_canvas = viewport() ;

	DPRINTF
	((	"KBDispScroller::KBDispScroller: %p: canvas=%p\n",
		(void *)this,
		(void *)m_canvas
	))	;

}

KBDispScroller::~KBDispScroller ()
{
	m_canvas = 0 ;
}

/*  KBDispScroller							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBDispScroller::showAs
	(	KB::ShowAs	mode
	)
{
	m_scroller->showAs (mode) ;

	if (mode == KB::ShowAsDesign)
	{
		if (m_rulerX != 0) m_rulerX->show () ;
		if (m_rulerY != 0) m_rulerY->show () ;
	}
	else
	{
		if (m_rulerX != 0) m_rulerX->hide () ;
		if (m_rulerY != 0) m_rulerY->hide () ;
	}
}

/*  KBDispScroller							*/
/*  setSetstretchable							*/
/*		: Set stretch mode					*/
/*  stretch	: uint		: Mode					*/
/*  (returns)	: void		:					*/

void	KBDispScroller::setStretchable
	(	uint		stretch
	)
{
	m_scroller->setStretchable (stretch) ;
}

/*  KBDispScrollArea							*/
/*  setShowbar	: Enable/display display of row scroll bar		*/
/*  _showBar	: uint		: True to enable			*/
/*  (returns)	: void		:					*/

void	KBDispScroller::setShowbar
	(	uint	showBar
	)
{
	m_scroller->setShowbar (showBar) ;
}

/*  KBDispScroller							*/
/*  setRowRange	: Set range of rows being displayed			*/
/*  totalRows	: uint		: Total number of rows			*/
/*  extra	: uint		: Extra insertion rows			*/
/*  curQRow	: uint		: Current query row			*/
/*  curDRow	: uint		: Index of topmost displayed row	*/
/*  dispRows	: uint		: Number of rows actually visible	*/
/*  (returns)	: void		:					*/

void	KBDispScroller::setRowRange
	(	uint		totalRows,
		uint		extra,
		uint		curQRow,
		uint		curDRow,
		uint		dispRows
	)
{
	m_scroller->setRowRange (totalRows, extra, curQRow, curDRow, dispRows) ;
}

/*  KBDispWidget							*/
/*  setDisplayGeometry							*/
/*		: Set the displays geometry				*/
/*  rect	: const QRect &	: Geometry				*/
/*  (returns)	: void			:				*/

void	KBDispScroller::setDisplayGeometry
	(	const QRect	&rect
	)
{
	DPRINTF
	((	"KBDispScroller::setDisplayGeometry: %s\n",
		_TEXT(rect)
	))	;


	QSize	updSize	= m_topSize.expandedTo (rect.size()) ;
	m_topSize  = rect.size() ;

	sizeAdjusted	  () ;
	updateContents	  (0, 0, updSize.width(), updSize.height()) ;
	m_geometry.resize (m_topSize) ;

#if	! __KB_RUNTIME
	/* Since there is no real top-level widget we need to update	*/
	/* any sizer blobs manually rather than rely on resize event	*/
	/* processing.							*/
	if (m_object->getSizer() != 0)
		m_object->getSizer()->setBlobs() ;
#endif
}

/*  KBDispScroller							*/
/*  effectiveSize : Get effective display size				*/
/*  (returns)	  : QSize	: Effective size			*/

QSize	KBDispScroller::effectiveSize ()
{
	return	m_scroller->effectiveSize (m_topSize) ;
}

/*  KBDispScroller							*/
/*  sizeAdjusted: Size has been adjusted				*/
/*  (returns)	: void		 :					*/

void	KBDispScroller::sizeAdjusted ()
{
	m_scroller->sizeAdjusted () ;
}

void	KBDispScroller::resizeContents
	(	const QSize		&size
	)
{
	DPRINTF
	((	"KBDispScroller::resizeContents: size=%s\n",
		_TEXT(size)
	))	;

	if (size.height() == 0)
		m_scroller->resizeContents (size.width(), m_scroller->contentsHeight()) ;
	else	m_scroller->resizeContents (size.width(), size.height()) ;
}

void	KBDispScroller::show ()
{
	QWidget::show () ;
}

int	KBDispScroller::width  ()
{
	return	QWidget::width  () ;
}

int	KBDispScroller::height ()
{
	return	QWidget::height () ;
}

void	KBDispScroller::setFrame
	(	int		,
		int
	)
{
}

void	KBDispScroller::setTitle
	(	const QString	&
	)
{
}

void	KBDispScroller::setTagLabel
	(	const QString	&
	)
{
}

void	KBDispScroller::setBackgroundPixmap
	(	const QPixmap	&,
		int		
	)
{
}

QSize	KBDispScroller::getTopSize ()
{
	return	m_topSize	;
}

QRect	KBDispScroller::getTopRect ()
{
	return	QRect(QPoint(0, 0), m_topSize) ;
}

void	KBDispScroller::updateMorph
	(	KBItem		*morph,
		const QRect	&mRect
	)
{
	QRect	vRect	= mRect	;
	cvtCtrlToView (vRect)	;
	m_scroller->updateMorph (morph, vRect) ;
}

void	KBDispScroller::updateMorph
	(	KBItem		*morph,
		uint		drow
	)
{
	m_scroller->updateMorph (morph, drow) ;
}

void	KBDispScroller::cvtViewToCtrl
	(	QRect		&cRect
	)
{
	cRect	= QRect
		  (	m_scroller->viewportToContents (cRect.topLeft    ()),
			m_scroller->viewportToContents (cRect.bottomRight())
		  )	;
}

void	KBDispScroller::cvtCtrlToView
	(	QRect		&cRect
	)
{
	cRect	= QRect
		  (	m_scroller->contentsToViewport (cRect.topLeft    ()),
			m_scroller->contentsToViewport (cRect.bottomRight())
		  )	;
}

void	KBDispScroller::setWidgetGeometry
	(	QWidget		*widget,
		const QRect	&cRect
	)
{
	QRect	mRect	= cRect	;


	KBReport *report = m_object->isReport() ;
	if (report != 0)
	{
		int	left	;
		int	top	;

		report->margins
		(	left,
			top
		)	;
		mRect.moveBy
		(	(int)(left * pixelsPerMM()),
			(int)(top  * pixelsPerMM())
		)	;
	}

	DPRINTF
	((	"KBDispScroller::setItemGeometry: %s -> %s\n",
		_TEXT(cRect),
		_TEXT(mRect)
	))	;


	m_scroller->moveChild (widget,  mRect.x(), mRect.y()) ;
	widget   ->resize     (mRect.width(), mRect.height()) ;
}


void	KBDispScroller::addMorph
	(	KBItem		*morph
	)
{
	KBDisplay::addMorph (morph) ;
	connect	   (morph, SIGNAL(destroyed()), this, SLOT(morphDestroyed())) ;
}

void	KBDispScroller::remMorph
	(	KBItem		*morph
	)
{
	KBDisplay::remMorph (morph) ;
	disconnect (morph, SIGNAL(destroyed()), this, SLOT(morphDestroyed())) ;
}

QWidget	*KBDispScroller::getTopWidget ()
{
	return	this	;
}

void	KBDispScroller::clearExposedArea
	(	QPainter	*p,
		const QRect	&newArea
	)
{
	QRect	maxArea	= m_oldArea.unite (newArea) ;
	QBrush bg	(p->backgroundColor()) ;

	if (maxArea.width () > newArea.width ())
		p->fillRect (newArea.width() + m_lMargin,  m_tMargin, maxArea.width()  - newArea.width(),  maxArea.height(), bg) ;
	if (maxArea.height() > newArea.height())
		p->fillRect (m_lMargin, newArea.height() + m_tMargin, maxArea.width(),  maxArea.height() - newArea.height(), bg) ;

	m_oldArea = newArea ;
}

void	KBDispScroller::makeVisible
	(	const QRect		&rRect,
		QFocusEvent::Reason	reason
	)
{
	if (isVisible())
		m_scroller->makeVisible (rRect, reason) ;
}

QSize	KBDispScroller::showBarUsed ()
{
	return	m_scroller->showBarUsed () ;
}

void	KBDispScroller::morphDestroyed ()
{
	KBDisplay::remMorph ((KBItem *)sender()) ;
}

void	KBDispScroller::moveTags
	(	KBDispWidget	*dispWidget,
		uint		y
	)
{
	m_scroller->moveTags (dispWidget, y) ;
}
