/***************************************************************************
    file	         : kb_dispscroller.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_DISPSCROLLER_H
#define		__KB_DISPSCROLLER_H

#include 	"libkbase_exports.h"

#include	<qscrollview.h>
#include	<qlabel.h>
#include	<qlayout.h>
#include	<qptrlist.h>
#include	<qtimer.h>

#include	"kb_classes.h"
#include	"kb_ruler.h"
#include	"kb_recordnav.h"
#include	"kb_display.h"

class		KBDispWidget	;
class		KBAttrGeom	;

/*  KBDispScroller							*/
/*  --------------							*/
/*  This class is a wrapper around a scroll view, and is used as the	*/
/*  canvas at the top level of a form, or report in design view.	*/

class	LIBKBASE_API	KBDispScrollArea : public QScrollView
{
	Q_OBJECT

	KBDisplay		*m_display	;
	QSize			&m_topSize	;
	KBGeometry		&m_geometry	;	
	KBRuler			*m_rulerX	;
	KBRuler			*m_rulerY	;

	uint			m_showBar	;
	uint			m_stretch	;

						  /* Data mode		*/
	QScrollBar		*m_vBar		; /* Record scroll bar	*/
	KBRecordNav		*m_rLabel	; /* Record count label	*/
						  /* Design mode	*/
	QScrollBar		*m_vBar_d	; /* Record scroll bar	*/
	KBRecordNav		*m_rLabel_d	; /* Record count label	*/

	bool			m_inSetRange	;
	KB::ShowAs		m_showing	;
	QTimer			m_scrollTimer	;
	QPoint			m_mouseDownAt	;

protected :

	virtual void	contentsWheelEvent	  (QWheelEvent  *) ;
	virtual void	contentsMousePressEvent	  (QMouseEvent  *) ;
	virtual void	contentsMouseMoveEvent	  (QMouseEvent  *) ;
	virtual	void	contentsMouseReleaseEvent (QMouseEvent  *) ;
	virtual	void	contentsContextMenuEvent  (QContextMenuEvent *) ;

	virtual	void	viewportPaintEvent	  (QPaintEvent  *) ;

	virtual	void	resizeEvent		  (QResizeEvent *) ;
	virtual	void	setHBarGeometry		  (QScrollBar   &, int, int, int, int) ;
	virtual	void	setVBarGeometry		  (QScrollBar   &, int, int, int, int) ;

	virtual	void	contentsMouseDoubleClickEvent
						  (QMouseEvent  *) ;

	virtual	bool	focusNextPrevChild	  (bool);

protected slots :

	void		vbarMoved		()	;
	virtual	bool	eventFilter		(QObject *, QEvent *)	;
	void		scrollTimeout		()	;
	void		slotOperation		(KB::Action, uint) ;

public	:

	KBDispScrollArea
	(	QWidget 		*,
		KBDisplay		*,
		QSize			&,
		KBGeometry		&,
		KBRuler			*,
		KBRuler 		*,
		uint			,
		uint
	)	;

	virtual		~KBDispScrollArea () ;

	void		showAs		(KB::ShowAs)	;
	void		setStretchable	(uint)	;
	void		setShowbar	(uint)	;

	void		setRowRange	(uint, uint, uint, uint, uint) ;
	QSize		effectiveSize	(QSize) ;
	void		sizeAdjusted	() ;
	QSize		showBarUsed	() ;

	void		moveTags	(KBDispWidget *, uint) ;

	void		makeVisible	(const QRect &, QFocusEvent::Reason) ;
	void		updateMorph	(KBItem *, const QRect &) ;
	void		updateMorph	(KBItem *, uint) ;

	virtual	void	addChild	(QWidget *, int = 0, int = 0) ;
	virtual	void	setContentsPos	(int, int) ;
}	;


/*  KBDispScrollArea							*/
/*  ----------------							*/
/*  At the top-level of a table in design view, we show a pair of	*/
/*  rulers, as well as the scroll view that is the canvas that the	*/
/*  design appears on. This class encapsulates the rulers and the	*/
/*  canvas. Note that the rulers to not appear in form design.		*/

class	LIBKBASE_API	KBDispScroller : protected QWidget, public KBDisplay
{
	Q_OBJECT

	KBRuler			*m_rulerX	;
	KBRuler			*m_rulerY	;
	KBDispScrollArea	*m_scroller	;
	QGridLayout		*m_grid		;
	QSize			m_topSize	;
	QRect			m_oldArea	;

protected slots :

	void		morphDestroyed		()	;

public	:

	KBDispScroller	(QWidget *, KBObject *, uint, uint, bool = false)  ;
	virtual		~KBDispScroller () ;

	virtual void	showAs			(KB::ShowAs)	;
	virtual void	setShowbar		(uint)		;
	virtual void	setFrame		(int, int)	;
	virtual void	setRowRange		(uint, uint, uint, uint, uint) ;
	virtual QSize	effectiveSize		()		;
	virtual void	setStretchable		(uint)		;
	virtual void	setTitle		(const QString    &) ;
	virtual void	setTagLabel		(const QString    &) ;
	virtual void	setBackgroundPixmap	(const QPixmap &, int) ;
	virtual QSize  	getTopSize 		() ;
	virtual QRect  	getTopRect 		() ;
	virtual void	resizeContents		(const QSize &)	  ;
	virtual void	setDisplayGeometry	(const QRect &)	  ;
	virtual void	updateMorph		(KBItem  *, const QRect &) ;
	virtual void	updateMorph		(KBItem  *, uint) ;
	virtual void	cvtViewToCtrl		(QRect &) ;
	virtual void	cvtCtrlToView		(QRect &) ;
	virtual void	setWidgetGeometry	(QWidget *, const QRect &) ;
	virtual	QWidget	*getTopWidget		() ;
	virtual	void	clearExposedArea	(QPainter *, const QRect &)	  ;
	virtual	void	addMorph		(KBItem *)	  ;
	virtual	void	remMorph		(KBItem *)	  ;
	virtual void	makeVisible		(const QRect &, QFocusEvent::Reason) ;
	virtual QSize	showBarUsed		() 		  ;
	virtual	void	show			() ;
	virtual	int	width			() ;
	virtual	int	height			() ;
	virtual	void	moveTags		(KBDispWidget *dispWidget, uint)  ;

	void		sizeAdjusted	() ;

	inline	QWidget	*viewport	()
	{
		return	m_scroller->viewport () ;

	}

	inline	void	updateContents
		(	int	x,
			int	y,
			int	w,
			int	h
		)
	{
		m_scroller->updateContents (x, y, w, h) ;
	}
	inline	int	frameWidth ()
	{
		return	m_scroller->frameWidth () ;
	}
	inline	QScrollView *getScroller ()
	{
		return	m_scroller ;
	}
}	;

#endif		// __KB_DISPSCROLLER_H
