/***************************************************************************
    file	         : kb_dispwidget.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qpainter.h>
#include	<qstyle.h>
#include	<qapplication.h>

#if		__KB_AUTOMOC
#include	"kb_dispwidget.moc"
#else
#include	"kb_dispwidget.h"
#endif

#include	"kb_item.h"
#include	"kb_display.h"
#include	"kb_layoutitem.h"
#include	"kb_ascii.h"


#define		__DEBUG_DISPWIDGET	(0)
#if		__DEBUG_DISPWIDGET
#include	"kb_dprintf.h"
#endif

/*  KBDispWidget: Display widget constructor				*/
/*  parent	: KBDisplay *	: Parent display			*/
/*  object	: KBObject *	: Owning object				*/
/*  showBar	: uint		: Show row scroll bar			*/
/*  (returns)	: KBDispScroller:					*/

KBDispWidget::KBDispWidget
	(	KBDisplay	*parent,
		KBObject	*object,
		uint		showBar
	)
	:
	QFrame 		(parent->getDisplayWidget()),
	KBDisplay	(parent, object),
	m_showBar	(0)
{
	m_inSetRange	= false	;
	m_vBar		= 0	;
	m_rLabel	= 0	;
	m_tagLabel	= 0	;
	m_showing	= KB::ShowAsUnknown ;
	m_bgScale	= 0	;

	/* This cannot be a top-level display, since top-level displays	*/
	/* are either scrollers or (for dynamic top-levels) the		*/
	/* constructor below is called.					*/
	m_canvas	= this ;

	m_geometry.init	(m_canvas, this) ;
	setShowbar (showBar) ;

	m_layoutItem	= new KBLayoutItem
				(	this,
					parent,
					m_object->attrGeom(),
					KBAttrGeom::MgmtStatic
				)	;

	m_parent->insertWidget (m_layoutItem) ;

	DPRINTF
	((	"KBDispWidget::KBDispWidget[nested]: %p: canvas=%p\n",
		(void *)this,
		(void *)m_canvas
	))	;
}

/*  KBDispWidget: Display widget constructor				*/
/*  parent	: QWidget *	: Parent widget				*/
/*  object	: KBObject *	: Owning object				*/
/*  showBar	: uint		: Show row scroll bar			*/
/*  (returns)	: KBDispScroller:					*/

KBDispWidget::KBDispWidget
	(	QWidget		*parent,
		KBObject	*object,
		uint		showBar
	)
	:
	QFrame 		(parent),
	KBDisplay	(parent, object),
	m_showBar	(0)
{
	m_inSetRange	= false	;
	m_vBar		= 0	;
	m_rLabel	= 0	;
	m_tagLabel	= 0	;
	m_showing	= KB::ShowAsUnknown ;
	m_bgScale	= 0	;
	m_layoutItem	= 0	;

	/* This case must be dynamic; used when creating a top-level	*/
	/* display for dynamic layouts. Non-dynamic top-level displays	*/
	m_canvas   = new QWidget    (this) ;
	m_geometry.init	 (m_canvas,  this) ;
	m_canvas->installEventFilter(this) ;

	setShowbar (showBar) ;

	DPRINTF
	((	"KBDispWidget::KBDispWidget[topdyn]: %p: canvas=%p\n",
		(void *)this,
		(void *)m_canvas
	))	;
}

/*  KBDispWidget							*/
/*  ~KBDispWidget: Display widget destructor				*/
/*  (returns)	 :		:					*/

KBDispWidget::~KBDispWidget ()
{
	DPRINTF
	((	"KBDispWidget::~KBDispWidget: %p\n",
		(void *)this
	))	;
	DELOBJ (m_tagLabel) ;
	m_canvas = 0	    ;
}


/*  KBDispScrollArea							*/
/*  setShowbar	: Enable/display display of row scroll bar		*/
/*  showBar	: uint		: Show row scroll bar			*/
/*  (returns)	: void		:					*/

void	KBDispWidget::setShowbar
	(	uint	showBar
	)
{
	if (showBar == m_showBar) return ;

	m_showBar = showBar ;

	if (m_showBar == 0)
	{
		DELOBJ	(m_vBar  ) ;
		DELOBJ	(m_rLabel) ;
		return	;
	}

	if (m_vBar == 0)
	{
		m_vBar	 = new QScrollBar (QScrollBar::Vertical, this) ;
		m_rLabel = new KBRecordNav (this, m_vBar->sizeHint().width()) ;

		connect
		(	m_vBar,
			SIGNAL	(valueChanged (int)),
			SLOT	(vbarMoved    ())
		)	;
		connect
		(	m_rLabel,
			SIGNAL	(operation    (KB::Action, uint)),
			SLOT  	(slotOperation(KB::Action, uint))
		)	;
	}

	if ((m_showBar & NAV_SCROLL) == 0)
		m_vBar  ->hide () ;
	else	m_vBar  ->show () ;

	if ((m_showBar & NAV_MINI  ) == 0)
		m_rLabel->hide () ;
	else	m_rLabel->show () ;

	int sbw   = m_vBar->sizeHint().width() ;

	m_vBar->setGeometry
	(	width () - sbw,
		0,
		sbw,
		height()
	)	;

	m_rLabel->move (0, height() - m_rLabel->size().height()) ;
}

/*  KBDispWidget							*/
/*  updateMorph	: Update a morphed control				*/
/*  morph	: KBItem *	: Morphed item				*/
/*  mRect	: const QRect &	: Area occupied by control		*/
/*  (returns)	: void		:					*/

void	KBDispWidget::updateMorph
	(	KBItem		*,
		const QRect	&mRect
	)
{
	update (mRect) ;
}

/*  KBDispWidget							*/
/*  updateMorph	: Update a morphed control				*/
/*  morph	: KBItem *	: Morphed item				*/
/*  drop	: uint		: Control display row			*/
/*  (returns)	: void		:					*/

void	KBDispWidget::updateMorph
	(	KBItem		*morph,
		uint		drow
	)
{
	QPainter p (this) ;
	morph->repaintMorph  (&p, drow) ;
}


/*  KBDispWidget							*/
/*  mousePressEvent							*/
/*		: Handler for mouse press events			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBDispWidget::mousePressEvent
	(	QMouseEvent	*e
	)
{
	if (!KBDisplay::doMousePressEvent (e, m_geometry.getCell(e->pos())))
		QWidget::mousePressEvent (e) ;
}

/*  KBDispWidget							*/
/*  mouseMoveEvent							*/
/*		: Handle mouse movement during marking			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBDispWidget::mouseMoveEvent
	(	QMouseEvent 	*e
	)
{
	KBDisplay::doMouseMoveEvent (e, m_geometry.getCell(e->pos())) ;
}

/*  KBDispWidget							*/
/*  mouseReleaseEvent							*/
/*		: Handle mouse release to end marking			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBDispWidget::mouseReleaseEvent
	(	QMouseEvent 	*e
	)
{
	KBDisplay::doMouseReleaseEvent (e, m_geometry.getCell (e->pos())) ;
}

/*  KBDispWidet								*/
/*  contentsContextMenuEvent						*/
/*		: Handle context menu event				*/
/*  e		: QContextMenuEvent * : Event				*/
/*  (returns)	: void		      :					*/

void	KBDispWidget::contextMenuEvent
	(	QContextMenuEvent	*e
	)
{
#if	! __KB_RUNTIME
	KBDisplay::doContextEvent (QRect (m_geometry.getCell (e->pos()), QSize(1,1))) ;
#endif
}

/*  KBDispWidet								*/
/*  mouseDoubleClickEvent						*/
/*		: Handler for mouse double click events			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBDispWidget::mouseDoubleClickEvent
	(	QMouseEvent	*e
	)
{
	/* Debugging hook. Double-click with the control and shift keys	*/
	/* down prints the widget tree.					*/
	if ((e->state() & (ShiftButton|ControlButton)) == (ShiftButton|ControlButton))
	{
		extern	void	printWidgetTree (QWidget *, uint, int, int = 0) ;
		printWidgetTree (this, 0, -1, 0xff) ;
		return	;
	}

#if	! __KB_RUNTIME
	/* Treat a double-click as a context event for the position at	*/
	/* which the click occurs.					*/
	KBDisplay::doContextEvent (QRect(m_geometry.getCell (e->pos()), QSize(1,1))) ;
#endif
}

/*  KBDispWidget							*/
/*  paintEvent	: Handle painting for widget				*/
/*  e		: QPaintEvent *	: Paint event				*/
/*  (returns)	: void		:					*/

void	KBDispWidget::paintEvent
	(	QPaintEvent *e
	)
{
	QPainter paint     (this)   ;
	QRect    r	= rect ()   ;

	/* If there is a separate canvas then all we need to is to	*/
	/* erase any newly exposed area, since the canvas will contain	*/
	/* all the controls and stuff.					*/
	if ((QWidget *)m_canvas != (QWidget *)this)
	{
		paint.eraseRect (r) ;
		return   ;
	}


	if (!m_title.isEmpty ())
		r.setTop  (paint.fontMetrics().height() / 2) ;

	setFrameRect  (r)  ;
	repaintMorphs (paint, e->rect()) ;

	if (!m_title.isEmpty ())
	{
	        QFontMetrics fm = paint.fontMetrics () ;

		int	h	= fm.height() ;
		int	w	= fm.width (m_title) + 2 * fm.width (QChar(' ')) ;
		int	x	;
		int	align	;

		align	= QApplication::reverseLayout() ? AlignRight : AlignLeft  ;

		switch (align)
		{
	        	case AlignHCenter :
				x = frameRect().width() / 2 - w / 2 ;
				break	;

			case AlignRight	  :
				x = frameRect().width() - w - 8 ;
				break	;

			default	:
				x = 8	;
				break	;
		}

		QRect	r (x, 0, w, h)	;

		style().drawItem
		(	&paint,
			r,
			AlignCenter|ShowPrefix,
			colorGroup(),
                       	isEnabled (),
			0,
			m_title
		)	;
		paint.setClipRegion (e->region().subtract(r)) ;
	}

	drawFrame    (&paint) ;
	drawContents (&paint) ;

#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		doDrawDisplay (&paint, e->rect()) ;
		m_geometry.outlineCells   (paint) ;
	}
#endif
}

/*  KBDispWidget							*/
/*  eventFilter	: Filter events						*/
/*  o		: QObject *	: Object receiving event		*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBDispWidget::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (o == m_canvas) switch (e->type())
	{
		case QEvent::Paint :
			/* We are interested in paint events on the	*/
			/* canvas in which case, in design mode, draw	*/
			/* the grid and cell outlines, and repaint any	*/
			/* morphs.					*/
			if (m_showing == KB::ShowAsDesign)
			{
				QPainter paint (m_canvas) ;
				QRect	 rect  = ((QPaintEvent *)e)->rect() ;

#if	! __KB_RUNTIME
				doDrawDisplay    (&paint, rect) ;
				m_geometry.outlineCells (paint) ;
#endif
				repaintMorphs( paint, rect) ;

			}
			return	true	;

		default	:
			break	;
	}

	return	false	;
}

/*  KBDispWidget							*/
/*  resizeEvent	: Handle resize for widget				*/
/*  e		: QResizeEvent *: Resize event				*/
/*  (returns)	: void		:					*/

void	KBDispWidget::resizeEvent
	(	QResizeEvent *e
	)
{
	DPRINTF
	((	"KBDispWidget::resizeEvent: m_canvas=%p m_bgPixmap=%d m_bgScale=%d\n",
		(void *)m_canvas,
		!m_bgPixmap.isNull(),
		m_bgScale
	))	;
	if ((QWidget *)m_canvas != (QWidget *)this)
	{
		if (m_showing == KB::ShowAsData)
		{
			m_topSize = e->size() ;
			m_canvas->resize (m_topSize) ;
		}

		return	;
	}

	m_topSize = e->size() ;

	if (m_showBar)
	{
		int sbw   = m_vBar->sizeHint().width() ;

		m_vBar->setGeometry
		(	width () - sbw,
			0,
			sbw,
			height()
		)	;

		m_rLabel->move (0, height() - m_rLabel->size().height()) ;
	}

	if (!m_bgPixmap.isNull() && (m_bgScale != 0))
	{
		extern	const	QPixmap	&scalePixmap
				(	const QPixmap	&,
					const QRect	&,
					uint
				)	;
		setPaletteBackgroundPixmap (scalePixmap (m_bgPixmap, geometry(), m_bgScale)) ;
		emit	backgroundChanged  () ;
	}

	update	() ;
}

/*  KBDispWidget							*/
/*  setDisplayGeometry							*/
/*		: Set the displays geometry				*/
/*  rect	: const QRect &	: Geometry				*/
/*  (returns)	: void		:					*/

void	KBDispWidget::setDisplayGeometry
	(	const QRect	&rect
	)
{
	DPRINTF
	((	"KBDispWidget::setDisplayGeometry: %s c=%p\n",
		_TEXT(rect),
		(void *)m_canvas
	))	;
	KBDisplay *pp = m_parent;
	uint	  ty  = y()	;

	if (pp == 0)
	{
		m_topSize = rect.size() ;

		/* If we have a separate canvas then resize that, else	*/
		/* resize directly.					*/
		if ((QWidget *)m_canvas != (QWidget *)this)
			m_canvas->resize (m_topSize) ;
		else	resize		 (m_topSize) ;

		return	;
	}

	pp->insertWidget  (m_layoutItem) ;

	while (pp->getParent() != 0)
	{
		ty += pp->getTopWidget()->y() ;
		pp  = pp->getParent   ()      ;
	}

	pp->moveTags (this, ty) ;
}


/*  KBDispWidget							*/
/*  setFrame	: Set frame style and width				*/
/*  style	: int		: Style					*/
/*  width	: int		: Width					*/
/*  (returns)	: void		:					*/

void	KBDispWidget::setFrame
	(	int	style,
		int	width
	)
{
	setFrameStyle	(style) ;
	setLineWidth	(width) ;
}

/*  KBDisplWidget							*/
/*  setTitle	: Set "group box" title					*/
/*  title	: const QString & : Title text				*/
/*  (returns)	: void		  :					*/

void	KBDispWidget::setTitle
	(	const QString	&title
	)
{
	m_title	= title	  ;
	update	     ()	  ;
}

/*  KBDispWidget							*/
/*  setRowRange	: Set range of rows being displayed			*/
/*  totalRows	: uint		: Total number of rows			*/
/*  extra	: uint		: Extra insertion rows			*/
/*  curQRow	: uint		: Current query row			*/
/*  curDRow	: uint		: Index of topmost displayed row	*/
/*  dispRows	: uint		: Number of rows actually visible	*/
/*  (returns)	: void		:					*/

void	KBDispWidget::setRowRange
	(	uint	totalRows,
		uint	extra,
		uint	curQRow,
		uint	curDRow,
		uint	dispRows
	)
{
	if (m_showBar)
	{
		/* Note the scrollbar-moved feedback is disabled while	*/
		/* setting the range. This is because range setting is	*/
		/* invoked from the block object, which will already	*/
		/* have scrolled its contents.				*/
		m_inSetRange = true  ;
		m_vBar->setRange  (0, totalRows + extra - dispRows) ;
		m_vBar->setSteps  (1, dispRows <= 1 ? 1 : dispRows  - 1) ;
		m_vBar->setValue  (curDRow) ;
		m_inSetRange = false ;

		m_rLabel->setRecord(curQRow, totalRows) ;
	}
}

/*  KBDispWidget							*/
/*  vbarMoved	: Notification of scrollbar change			*/
/*  (returns)	: void		:					*/

void	KBDispWidget::vbarMoved ()
{
	/* See comment in kb_dispscroller.cpp about this ...		*/
	if (!m_inSetRange)
		m_scrollTimer.singleShot (200, this, SLOT(scrollTimeout())) ;
}

/*  KBDispScrollWidget							*/
/*  scrollTimeout: Timeout on scroll bar movement			*/
/*  (returns)	 : void		:					*/

void	KBDispWidget::scrollTimeout ()
{
	int	value = m_vBar->value() ;
	scrollToRow (value < 0 ? 0 : value) ;
}

/*  KBDispWidget							*/
/*  slotOperation: Action requested					*/
/*  action	 : KB::Action	: Action in question			*/
/*  qrow	 : uint		: Possible query row			*/
/*  (returns)	 : void		:					*/

void	KBDispWidget::slotOperation
	(	KB::Action	action,
		uint		qrow
	)
{
	doOperation (action, qrow) ;
}

/*  KBDispWidget							*/
/*  effectiveSize: Get effective display size				*/
/*  (returns)	 : QSize	: Effective size			*/

QSize	KBDispWidget::effectiveSize ()
{
	int	w	= width	() ;
	int	h	= height() ;

	if ((m_showBar & NAV_SCROLL) != 0) w -= m_vBar  ->width () ;
	if ((m_showBar & NAV_MINI  ) != 0) h -= m_rLabel->height() ;

	return	QSize(w,h) ;
}

/*  KBDispWidget							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBDispWidget::showAs
	(	KB::ShowAs	mode
	)
{
	m_showing = mode  ;
	setEnabled (true) ;
}

/*  KBDispWidget							*/
/*  event	: Event handler						*/
/*  e		: QEvent *	: Event					*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBDispWidget::event
	(	QEvent		*e
	)
{
	/* Pass keypresses to the display object and thence the block	*/
	/* navigator. This is used to bring the current control back	*/
	/* into view if it has been scrolled out of site.		*/


	/* NOTE: If we do this at the QWidget::keyPressEvent level then	*/
	/* we don't get tabs.						*/
	if (e->type() == QEvent::KeyPress)
		return	keyStroke ((QKeyEvent *)e) ;


	/* Trap wheel events. These are passed to the display object	*/
	/* to scroll, and are then accepted and consumed.		*/
	if (e->type() == QEvent::Wheel)
	{
		QWheelEvent *w = (QWheelEvent *)e ;
		scrollBy (- w->delta() / 120) ;
		w->accept() 	;
		return	true	;
	}

	return	QWidget::event (e) ;
}

/*  KBDispWidget							*/
/*  setTagLabel	: Set tag label for report design			*/
/*  text	: const QString & : text				*/
/*  (returns)	: void		  :					*/

void	KBDispWidget::setTagLabel
	(	const QString	&text
	)
{
	m_tagText = text	;
}

/*  KBDispWidget							*/
/*  setBackgroundPixmap							*/
/*		: Set bgackground pixmap				*/
/*  pixmap	: const QPixmap & : Pixmap				*/
/*  scale	: uint		  : Scaling				*/
/*  (returns)	: void		  :					*/

void	KBDispWidget::setBackgroundPixmap
	(	const QPixmap	&pixmap,
		int		scale
	)
{
	DPRINTF
	((	"KBDispWidget::setBackgroundPixmap: pm=%d scale=%d\n",
		!pixmap.isNull(),
		scale
	))	;
	m_bgPixmap	= pixmap ;
	m_bgScale	= scale	 ;

	if (m_bgPixmap.isNull())
	{
		setBackgroundMode (Qt::PaletteBackground) ;
		emit	backgroundChanged  () ;
		return	;
	}

	if (m_bgScale != 0)
	{
		extern	const	QPixmap	&scalePixmap
				(	const QPixmap	&,
					const QRect	&,
					uint
				)	;
		setPaletteBackgroundPixmap (scalePixmap (m_bgPixmap, geometry(), m_bgScale)) ;
		emit	backgroundChanged  () ;
		return	;
	}

	setPaletteBackgroundPixmap (pixmap) ;
	emit	backgroundChanged  () ;
}

/*  KBDispWidget							*/
/*  getTagLabel	: Get report design tagging label			*/
/*  parent	: QWindget *	: Parent for label			*/
/*  (returns)	: QLabel   *	: The label				*/

QLabel	*KBDispWidget::getTagLabel
	(	QWidget		*parent
	)
{
	if ((m_tagLabel == 0) && !m_tagText.isNull())
	{
		m_tagLabel = new QLabel (parent) ;

		m_tagLabel->setFrameStyle (QFrame::Box|QFrame::Plain) ;
		m_tagLabel->setLineWidth  (1) ;

		m_tagLabel->setText	  ("WWWWWWWWWWWWWWWWWW") ;
		m_tagLabel->setFixedSize  (m_tagLabel->sizeHint()) ;
		m_tagLabel->setText	  (m_tagText) ;

		m_tagLabel->setBackgroundMode (QWidget::PaletteLight) ;
	}

	return	m_tagLabel ;
}

/*  KBDispWidget							*/
/*  sizeHint	: Get size hint for this display			*/
/*  (returns)	: QSize		: Size hint				*/

QSize	KBDispWidget::sizeHint () const
{
	return	m_geometry.sizeHint () ;
}

/*  KBDispWidget							*/
/*  resizeContents							*/
/*		: Resize contents area					*/
/*  size	: const QString &	: Required size			*/
/*  (returns)	: void			:				*/

void	KBDispWidget::resizeContents
	(	const QSize		&
	)
{
	/* Dummy, only used in KBDispScroller				*/
}

/*  KBDispWidget							*/
/*  showBarUsed	: Get space used by scroll and mini-nav bars		*/
/*  (returns)	: QSize		: Size used				*/

QSize	KBDispWidget::showBarUsed ()
{
	int	w	= 0	;
	int	h	= 0	;

	if ((m_showBar & NAV_SCROLL) != 0) w = m_vBar  ->width () ;
	if ((m_showBar & NAV_MINI  ) != 0) h = m_rLabel->height() ;

	return	QSize (w, h)	;
}

/*  KBDispWidget							*/
/*  show	: Show wigget						*/
/*  (returns)	: void			:				*/

void	KBDispWidget::show ()
{
	QFrame::show () ;
}

/*  KBDispWidget							*/
/*  width	: Get display width					*/
/*  (returns)	: int		: Width					*/

int	KBDispWidget::width  ()
{
	return	QFrame::width  () ;
}

/*  KBDispWidget							*/
/*  height	: Get display height					*/
/*  (returns)	: int		: Height				*/

int	KBDispWidget::height ()
{
	return	QFrame::height () ;
}

/*  KBDispScrollArea							*/
/*  setSetstretchable							*/
/*		: Set display stretching mode				*/
/*  stretch	: uint		: Mode					*/
/*  (returns)	: void		:					*/

void	KBDispWidget::setStretchable
	(	uint		
	)
{
}

/*  KBDispWidget							*/
/*  (returns)	: void			:				*/

void	KBDispWidget::cvtViewToCtrl
	(	QRect		&
	)
{
}

/*  KBDispWidget							*/
/*  (returns)	: void			:				*/

void	KBDispWidget::cvtCtrlToView
	(	QRect		&
	)
{
}

void	KBDispWidget::setWidgetGeometry
	(	QWidget		*widget,
		const QRect	&cRect
	)
{
	widget->setGeometry (cRect) ;
}

/*  KBDispWidget							*/
/*  (returns)	: void			:				*/

void	KBDispWidget::addMorph
	(	KBItem		*morph
	)
{
	KBDisplay::addMorph (morph) ;
	connect	   (morph, SIGNAL(destroyed()), this, SLOT(morphDestroyed())) ;
}

/*  KBDispWidget							*/
/*  (returns)	: void			:				*/

void	KBDispWidget::remMorph
	(	KBItem		*morph
	)
{
	KBDisplay::remMorph (morph) ;
	disconnect (morph, SIGNAL(destroyed()), this, SLOT(morphDestroyed())) ;
}

/*  KBDispWidget							*/
/*  (returns)	: void			:				*/

void	KBDispWidget::makeVisible
	(	const QRect		&,
		QFocusEvent::Reason	
	)
{
}

/*  KBDispWidget							*/
/*  (returns)	: void			:				*/

void	KBDispWidget::clearExposedArea
	(	QPainter	*,
		const QRect	&
	)
{
}

/*  KBDispWidget							*/

QWidget	*KBDispWidget::getTopWidget ()
{
	return	this	;
}

/*  KBDispWidget							*/
/*  (returns)	: void			:				*/

void	KBDispWidget::morphDestroyed ()
{
	KBDisplay::remMorph ((KBItem *)sender()) ;
}

/*  KBDispWidget							*/

QSize	KBDispWidget::getTopSize ()
{
	return	m_topSize	;
}

/*  KBDispWidget							*/
/*  getTopRect	: Get geometry, converted to control coordinates	*/
/*  (returns)	: QRect		: Geometry				*/

QRect	KBDispWidget::getTopRect ()
{
	QRect rect = geometry () ;

	if (m_parent != 0)
                m_parent->cvtViewToCtrl (rect) ;
 
	DPRINTF
	((	"KBDispWidget::getTopRect: %s: %s: fw=%d m=%d\n",
		m_object->getName().latin1(),
		_TEXT(rect),
		frameWidth(),
		margin()
	))	;

	return	rect	;
}

void	KBDispWidget::moveTags
	(	KBDispWidget	*,
		uint
	)
{
}
