/***************************************************************************
    file	         : kb_dispwidget.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_DISPWIDGET
#define	__KB_DISPWIDGET

#include 	"libkbase_exports.h"

#include	<qtimer.h>
#include	<qframe.h>
#include	<qscrollbar.h>
#include	<qptrlist.h>
#include	<qlabel.h>
#include	<qvaluelist.h>
#include	<qpixmap.h>

#include	"kb_classes.h"
#include	"kb_recordnav.h"
#include	"kb_attrnav.h"
#include	"kb_attrgeom.h"
#include	"kb_display.h"

class		KBAttrGeom	;
class		KBGridSetup	;


/*  KBDispWidget							*/
/*  ------------							*/
/*  This is a wrapper for QWidget so that we can pick up mouse and	*/
/*  paint events and pass them up to the owning KBDisplay for possible	*/
/*  processing.								*/

class 	LIBKBASE_API	KBDispWidget : protected QFrame, public KBDisplay
{
	Q_OBJECT

	KBLayoutItem	*m_layoutItem	;
	uint		m_showBar	;
	bool		m_inSetRange	;

	QSize		m_topSize	;

	QScrollBar	*m_vBar		;
	KBRecordNav	*m_rLabel	;	/* Record count label	*/

	KB::ShowAs	m_showing	;
	QTimer		m_scrollTimer	;

	QPoint		m_mouseDownAt	;

	QString		m_title		;
	QPixmap		m_bgPixmap	;
	int		m_bgScale	;
	QString		m_tagText	;
	QLabel		*m_tagLabel	;

protected slots	:

	void		morphDestroyed		() ;

signals	  :

	void		backgroundChanged	() ;

protected :

	virtual void	mousePressEvent		(QMouseEvent  *) ;
	virtual void	mouseMoveEvent		(QMouseEvent  *) ;
	virtual	void	mouseReleaseEvent	(QMouseEvent  *) ;
	virtual	void	contextMenuEvent	(QContextMenuEvent *) ;
	virtual	void	paintEvent 		(QPaintEvent  *) ;
	virtual	void	resizeEvent		(QResizeEvent *) ;
	virtual	bool	event			(QEvent       *) ;
	virtual	bool	eventFilter		(QObject      *, QEvent *) ;
	virtual	void	mouseDoubleClickEvent	(QMouseEvent  *) ;

protected slots :

	void		vbarMoved		()	;
	void		scrollTimeout		()	;
	void		slotOperation		(KB::Action, uint) ;

public	:

	KBDispWidget	(KBDisplay *, KBObject *, uint)	;
	KBDispWidget	(QWidget   *, KBObject *, uint)	;
	virtual	~KBDispWidget	() ;

	virtual void	showAs			(KB::ShowAs)	;
	virtual void	setShowbar		(uint)		;
	virtual void	setFrame		(int, int)	;
	virtual void	setRowRange		(uint, uint, uint, uint, uint) ;
	virtual QSize	effectiveSize		() 		;
	virtual void	setStretchable		(uint)		;
	virtual void	setTitle		(const QString    &) ;
	virtual void	setTagLabel		(const QString    &) ;
	virtual void	setBackgroundPixmap	(const QPixmap &, int) ;
	virtual QSize  	getTopSize 		() ;
	virtual QRect  	getTopRect 		() ;
	virtual void	resizeContents		(const QSize &)	  ;
	virtual void	setDisplayGeometry	(const QRect &)	  ;
	virtual void	updateMorph		(KBItem  *, const QRect &) ;
	virtual void	updateMorph		(KBItem  *, uint) ;
	virtual void	cvtViewToCtrl		(QRect &) ;
	virtual void	cvtCtrlToView		(QRect &) ;
	virtual void	setWidgetGeometry	(QWidget *, const QRect &) ;
	virtual	QWidget	*getTopWidget		() ;
	virtual	void	clearExposedArea	(QPainter *, const QRect &)	  ;
	virtual	void	addMorph		(KBItem *)	  ;
	virtual	void	remMorph		(KBItem *)	  ;
	virtual void	makeVisible		(const QRect &, QFocusEvent::Reason) ;
	virtual QSize	showBarUsed		() 		  ;
	virtual	void	show			() ;
	virtual	int	width			() ;
	virtual	int	height			() ;
	virtual	void	moveTags		(KBDispWidget *dispWidget, uint)  ;

	QLabel		*getTagLabel		(QWidget *)	  ;

	virtual	QSize	sizeHint		() const ;

	friend	class	KBDispScrollArea	;
}	;

#endif	// __KB_DISPWIDGET
