/***************************************************************************
    file	         : kb_docchooser.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_docchooser.moc"
#else
#include	"kb_docchooser.h"
#endif

#include	"kb_dbdociter.h"
#include	"kb_serverinfo.h"

#include	"rk_vbox.h"
#include	"rk_gridbox.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"


/*  KBDocChooser							*/
/*  KBDocChooser: Constructor for document chooser			*/
/*  dbInfo	: KBDBInfo *	  : Database information		*/
/*  cbServer	: RKComboBox *	  : Server selector			*/
/*  cbDocument	: RKComboBox *	  : Document selector			*/
/*  docType	: const QString & : Document type			*/
/*  docExtn	: const QString & : Document extension			*/
/*  stock	: bool		  : Include stock compents		*/
/*  (returns)	: KBDocChooser	  :					*/

KBDocChooser::KBDocChooser
	(	KBDBInfo	*dbInfo,
		RKComboBox	*cbServer,
		RKComboBox	*cbDocument,
		const QString	&docType,
		const QString	&docExtn,
		bool		stock
	)
	:
	m_dbInfo	(dbInfo),
	m_cbServer	(cbServer),
	m_cbDocument 	(cbDocument),
	m_docType	(docType),
	m_docExtn	(docExtn)
{
	QPtrListIterator<KBServerInfo> *svIter = dbInfo->getServerIter () ;
	KBServerInfo		    *svInfo ;

	if (stock)
		m_cbServer->insertItem (KBLocation::m_pStock) ;

	m_cbServer->insertItem (KBLocation::m_pFile) ;

	while ((svInfo = svIter->current()) != 0)
	{
		m_cbServer->insertItem (svInfo->serverName()) ;
		(*svIter) += 1 ;
	}

	delete	svIter	;

	connect
	(	m_cbServer,
		SIGNAL(activated       (const QString &)),
		SLOT  (serverSelected  (const QString &))
	)	;
	connect
	(	m_cbDocument,
		SIGNAL(activated       (const QString &)),
		SLOT  (documentSelected(const QString &))
	)	;

	serverSelected (m_cbServer->currentText()) ;
}

/*  KBDocChooser								*/
/*  serverSelected							*/
/*		: User selects a database server			*/
/*  svName	: const QString & : Server name				*/
/*  (returns)	: void		  :					*/

void	KBDocChooser::serverSelected
	(	const QString	&svName
	)
{
	m_cbDocument->clear () ;
	emit serverChanged  () ;

	QString	    name    ;
	QString	    stamp   ;
	KBError	    error   ;

	KBDBDocIter docIter ;
	if (!docIter.init (m_dbInfo, svName, m_docType, m_docExtn, error))
	{	error.DISPLAY() ;
		return	;
	}

	while (docIter.getNextDoc (name, stamp))
		m_cbDocument->insertItem (name) ;

	emit	documentChanged () ;
}

/*  KBDocChooser								*/
/*  documentSelected							*/
/*		: User selects an document				*/
/*  objName	: const QString & : Document name			*/
/*  (returns)	: void		  :					*/

void	KBDocChooser::documentSelected
	(	const QString	&
	)
{
	emit	documentChanged	() ;
}

/*  KBDocChooser								*/
/*  setServer	: Set specifier server					*/
/*  svrName	: const QString & : Server name				*/
/*  (returns)	: bool		  : Server name valid			*/

bool	KBDocChooser::setServer
	(	const QString	&svrName
	)
{
	bool	rc	= false ;

	m_cbServer->setCurrentItem (0) ;

	for (int idx = 0 ; idx < m_cbServer->count() ; idx += 1)
		if (m_cbServer->text(idx) == svrName)
		{	m_cbServer->setCurrentItem (idx) ;
			serverSelected	(svrName) ;
			rc	= true	;
			break	;
		}

	emit	serverChanged () ;
	return	rc ;
}

/*  KBDocChooser							*/
/*  setDocument	: Set specifier document				*/
/*  docName	: const QString & : Document name			*/
/*  (returns)	: bool		  : Document name valid			*/

bool	KBDocChooser::setDocument
	(	const QString	&docName
	)
{
	bool	rc	= false ;

	m_cbDocument->setCurrentItem (0) ;

	for (int idx = 0 ; idx < m_cbDocument->count() ; idx += 1)
		if (m_cbDocument->text(idx) == docName)
		{	m_cbDocument->setCurrentItem (idx) ;
			documentSelected (docName) ;
			rc	= true	 ;
			break	;
		}

	emit	documentChanged () ;
	return	rc ;
}


/*  ------------------------------------------------------------------  */

/*  KBDocChooserDlg							*/
/*  KBDocChooserDlg							*/
/*		: Simple server/document selection dialog		*/
/*  dbInfo	: KBDBInfo *	  : Database information		*/
/*  docType	: const QString & : Document type			*/
/*  docExtn	: const QString & : Document extension			*/
/*  stock	: bool		  : Include stock compents		*/
/*  (returns)	: KBDocChooserDlg :					*/

KBDocChooserDlg::KBDocChooserDlg
	(	KBDBInfo	*dbInfo,
		const QString	&docType,
		const QString	&docExtn,
		bool		stock
	)
	:
	KBDialog	("Pick document", true),
	m_dbInfo	(dbInfo)
{
	RKVBox		*layMain  = new RKVBox    (this) ;
	layMain->setTracking() ;

	RKGridBox	*layGrid  = new RKGridBox (2, layMain) ;

	new QLabel (TR("Server"  ), layGrid) ;
	m_cbServer	= new RKComboBox (layGrid) ;

	new QLabel (TR("Document"), layGrid) ;
	m_cbDocument	= new RKComboBox (layGrid) ;

	layMain->addFiller () ;
	addOKCancel  (layMain, &m_bOK) ;

	m_chooser = new KBDocChooser (m_dbInfo, m_cbServer, m_cbDocument, docType, docExtn, stock) ;
	connect  (m_chooser, SIGNAL(documentChanged()), SLOT(changed())) ;

	changed	 () ;
}

/*  KBDocChooserDlg							*/
/*  changed	: Selection changed					*/
/*  (returns)	: void		:					*/

void	KBDocChooserDlg::changed ()
{
	m_bOK->setEnabled
	(	!m_cbServer  ->currentText().isEmpty() &&
		!m_cbDocument->currentText().isEmpty()
	)	;
}

bool	KBDocChooserDlg::setServer
	(	const QString	&server
	)
{
	return	m_chooser->setServer   (server  ) ;
}

bool	KBDocChooserDlg::setDocument
	(	const QString	&document
	)
{
	return	m_chooser->setDocument (document) ;
}

QString	KBDocChooserDlg::getServer   ()
{
	return	m_cbServer  ->currentText() ;
}

QString	KBDocChooserDlg::getDocument ()
{
	return	m_cbDocument->currentText() ;
}
