/***************************************************************************
    file	         : kb_docchooser.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_DOCCHOOSER_H
#define	__KB_DOCCHOOSER_H

#include "libkbase_exports.h"

#include	<qlabel.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"
#include	"kb_dialog.h"

#include	"kb_dblink.h"
#include	"kb_dbinfo.h"


class		RKComboBox	;
class		RKPushButton	;


/*  KBDocChooser							*/
/*  -----------								*/
/*  This is a simple helper class which controls a pair of combo boxes	*/
/*  which allow server/file and document selection for a specified	*/
/*  document type and extension.					*/
/*									*/
/*  NOTE: This class does not handle GUI layout, the constructor is	*/
/*	  passed the server and table combo boxes.			*/

class	LIBKBASE_API	KBDocChooser : public QObject
{
	Q_OBJECT

	KBDBInfo	*m_dbInfo		;
	RKComboBox	*m_cbServer		;
	RKComboBox	*m_cbDocument		;
	QString		m_docType		;
	QString		m_docExtn		;

signals	:

	void		serverChanged	()	;
	void		documentChanged	()	;

protected slots :

	void		serverSelected	(const QString &) ;
	void		documentSelected(const QString &) ;

public	:

	KBDocChooser
	(	KBDBInfo	*,
		RKComboBox	*,
		RKComboBox	*,
		const QString	&,
		const QString	&,
		bool		= false
	)	;

	bool		setServer	(const QString &) ;
	bool		setDocument	(const QString &) ;
}	;


/*  KBDocChooserDlg							*/
/*  ---------------							*/
/*  This class provies a simple server/document selection dialog, based	*/
/*  on the chooser class above.						*/

class	LIBKBASE_API	KBDocChooserDlg : public KBDialog
{
	Q_OBJECT

	KBDBInfo	*m_dbInfo	;
	RKComboBox	*m_cbServer	;
	RKComboBox	*m_cbDocument	;
	RKPushButton	*m_bOK		;
	KBDocChooser	*m_chooser	;

protected slots :

	virtual	void	changed		()	;

public	:

	KBDocChooserDlg (KBDBInfo *, const QString &, const QString &, bool = false) ;

	QString		getServer	() ;
	QString		getDocument	() ;
	bool		setServer	(const QString	&) ;
	bool		setDocument	(const QString	&) ;
}	;

#endif	// __KB_DOCCHOOSER_H
