/***************************************************************************
    file	         : kb_docroot.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#if		__KB_AUTOMOC
#include	"kb_docroot.moc"
#else
#include	"kb_docroot.h"
#endif

#include	"kb_callback.h"
#include	"kb_script.h"
#include	"kb_import.h"
#include	"kb_form.h"
#include	"kb_display.h"
#include	"kb_param.h"
#include	"kb_appptr.h"
#include	"kb_paramsetdlg.h"
#include	"kb_progress.h"
#include	"kb_skin.h"
#include	"kb_ascii.h"
#include	"kb_mimefactory.h"
#include	"kb_notifier.h"

#define		__DEBUG_DOCROOT		(0)
#if		__DEBUG_DOCROOT
#include	"kb_dprintf.h"
#endif

#include	<quuid.h>
#include	<qregexp.h>



/*  KBDocRoot								*/
/*  KBDoctRoot	: Common document root class constructor		*/
/*  parent	: KBNode *	    : Parent (actually, enclosing) node	*/
/*  children	: QPtrList<KBNode> &: Children of said node		*/
/*  location	: KBLocation &	    : Document location			*/
/*  (returns)	: KBDocRoot	    :					*/

KBDocRoot::KBDocRoot
	(	KBNode			*parent,
		QPtrList<KBNode>	&children,
		const KBLocation	&location
	)
	:
	m_parent	(parent),
	m_children	(children),
	m_docLocation 	(location),
	m_dataLocation 	(location)
{
	m_progress	= 0	;
	m_locking	= 0	;
	m_loading	= false	;
	m_mimeSource	= 0	;
	m_docServerInfo	= m_docLocation .getServerInfo() ;
	m_skin		= 0	;
	m_test		= 0	;

	if (!m_dataLocation.dataServer().isNull())
		m_dataLocation.setServer (m_dataLocation.dataServer()) ;

	reset () ;

	connect
	(	KBNotifier::self(),
		SIGNAL	(   sSkinChanged(const KBLocation &)),
		SLOT	(slotSkinChanged(const KBLocation &))
	)	;

	KBAttr	*uuid	= parent->getAttr("uuid") ;
	if ((uuid != 0) && uuid->getValue().isEmpty())
		uuid->setValue ("M" + QUuid::createUuid().toString().replace(QRegExp("[-{}]"), "")) ;
}

/*  KBDocRoot								*/
/*  ~KBDoctRoot	: Common document root class destructor			*/

KBDocRoot::~KBDocRoot ()
{
	DELOBJ	(m_paramDict )	;
	DELOBJ	(m_mimeSource)	;
}

KBMimeSourceFactory
	*KBDocRoot::mimeSource ()
{
	if (m_mimeSource == 0) m_mimeSource = new KBMimeSourceFactory(this) ;
	return	m_mimeSource ;
}

/*  KBDocRoot								*/
/*  setParamDict: Set parameter dictionary				*/
/*  _paramDict	: QDict<QString>&: Parameter dictionary			*/
/*  pError	: KBError &	 : Error return				*/
/*  (returns)	: KB::ShowRC	 : OK/Error/Cancel			*/

KB::ShowRC
	KBDocRoot::setParamDict
	(	const QDict<QString>	&_paramDict,
		KBError			&pError
	)
{
	/* Delete any existing parameter dictionary and set a new copy	*/
	/* of the new one. Then scan the document to locate all		*/
	/* defined paramaters; if there are none then we can return	*/
	/* without anything more to do.					*/
	DELOBJ	(m_paramDict) ;
	m_paramDict = new KBAttrDict (_paramDict) ;

	QDict<KBParamSet> paramSet    ;
	paramSet.setAutoDelete  (true) ;
	m_parent->findAllParams (paramSet) ;

	if (paramSet.count() == 0)
		return KB::ShowRCOK ;

	/* Scan the parameter dictionary and update any matching	*/
	/* parameters with the supplied values. Hence passed parameter	*/
	/* values override parameters defined in the document.		*/
	QDictIterator<QString> pIter (*m_paramDict) ;
	while (pIter.current() != 0)
	{
		KBParamSet *pSet = paramSet.find(pIter.currentKey()) ;
		if (pSet != 0)
		{
			pSet->m_value	= *pIter.current() ;
			pSet->m_set	= true	;
		}
		pIter += 1 ;
	}

	/* Run the user paramater set dialog, unless there we no	*/
	/* parameters left to set. If the user cancels this then	*/
	/* return in error.						*/
	bool	      ok    ;
	KBParamSetDlg pDlg  (TR("Set Parameters"), paramSet, this, pError, ok) ;
	if (!ok) return	 KB::ShowRCError ;

	if (pDlg.anyParams() && !pDlg.exec ())
	{
		pError	= KBError
			  (	KBError::Warning,
				TR("User cancelled parameter dialog"),
				QString::null,
				__ERRLOCN
			  )	;
		return	KB::ShowRCCancel ;
	}

	QDictIterator<KBParamSet> iter (paramSet) ;
	KBParamSet		  *pSet		  ;

	while ((pSet = iter.current()) != 0)
	{
		m_paramDict->replace (iter.currentKey(), new QString(pSet->m_value)) ;
		iter += 1 ;
	}

	return	KB::ShowRCOK ;
}

/*  KBDocRoot								*/
/*  parameterDict							*/
/*		: Get entire parameter dictionary			*/
/*  (returns)	: const KBAttrDict & :					*/

const KBAttrDict &KBDocRoot::parameterDict ()
{
	static	KBAttrDict *empty ;
	if (empty == 0) empty = new KBAttrDict ;
	return	m_paramDict == 0 ? *empty : *m_paramDict ;
}

/*  KBDocRoot								*/
/*  reset	: Reset document root information			*/
/*  (returns)	: void		 :					*/

void	KBDocRoot::reset ()
{
	m_test			= 0	;
	m_scrIface[0]  		= 0     ;
	m_scrIface[1] 		= 0     ;
	m_paramDict 		= 0	;
	m_gotExecError		= false	;
	m_namedNodeMap.clear ()	;

	DELOBJ	(m_skin)	;
}

/*  KBDocRoot								*/
/*  doExecError	: Execution error reported				*/
/*  (returns)	: void		:					*/

void	KBDocRoot::doExecError ()
{
	if (!m_gotExecError)
	{
		m_gotExecError	= true	;
		emit	execError ()	;
	}
}

/*  KBDocRoot								*/
/*  doRequestClose							*/
/*		: Request to close the document				*/
/*  rc		: int		: Return code				*/
/*  (returns)	: void		:					*/

void	KBDocRoot::doRequestClose
	(	int	rc
	)
{
	emit	requestClose (rc) ;
}

/*  KBDocRoot								*/
/*  doSetMessage: Display message in status bar				*/
/*  message	: const QString &    : Message				*/
/*  (returns)	: void		     :					*/

void	KBDocRoot::doSetMessage
	(	const QString    	&message
	)
{
	if (m_message)
		m_message->setText (message) ;
}

/*  KBDocRoot								*/
/*  doSetLocking: Display locking state in status bar			*/
/*  locking	: KBQryBase::Locking : Locking state			*/
/*  (returns)	: void		     :					*/

void	KBDocRoot::doSetLocking
	(	KBQryBase::Locking	locking
	)
{
	if (m_locking)
		m_locking->setText
		(	locking == KBQryBase::NoLocking ?
				TR("Unlocked") :
				TR("Record locked")
		)	;
}

/*  KBDocRoot								*/
/*  getParamValue: Get value of named parameter				*/
/*  param	 : cchar *	: Parameter				*/
/*  (returns)	 : QString &	: Parameter value			*/

QString	KBDocRoot::getParamValue
	(	cchar	*param
	)
{
	/* The parameter may be either "name" or "name:defval", so we	*/
	/* split on the colon first ...					*/
	QString		res	= QString::null ;
	QStringList	bits	= QStringList::split (':', param) ;

	/* ... then, if there is a paramater dictionary, see if the	*/
	/* parameter has been defined.					*/
	if (m_paramDict != 0)
	{	QString	*val = m_paramDict->find (bits[0]) ;
		if (val != 0) res = *val ;
	}

	/* If we don't have a value, and the parameter was of the form	*/
	/* "name:defval", then use that default value.			*/
	if (res.isEmpty() && (bits.count() > 1)) res = bits[1] ;

	return	res	;
}

/*  KBDocRoot								*/
/*  setParamValue: Set value of named parameter				*/
/*  name	 : cchar *	   : Parameter name			*/
/*  value	 : const QString & : Parameter value			*/
/*  (returns)	 : void		   :					*/

void	KBDocRoot::setParamValue
	(	cchar		*name,
		const QString	&value
	)
{
	if (m_paramDict != 0)
		m_paramDict->replace (name, new QString(value)) ;
}

/*  KBDocRoot	: 							*/
/*  loadScriptModules							*/
/*		: Load scripting modules				*/
/*  scriptIF	: KBScriptIF *	  : Scripting interface			*/
/*  scripts	: QStringList &	  : List of scripts			*/
/*  (returns)	: KBScriptError * : Error or null if OK			*/

KBScriptError
	*KBDocRoot::loadScriptModules
	(	KBScriptIF		*scriptIF,
		const QString		&language,
		const QStringList	&scripts
	)
{
	if (getenv("REKALL_NOLOADSCRIPTMODULES") != 0)
		return	0	;

	KBError	error	;

	for (uint idx = 0 ; idx < scripts.count() ; idx += 1)
	{
		QString	   name	 = scripts[idx] ;
		KBLocation scrloc
			   (	m_docLocation.dbInfo(),
				"script",
				m_docLocation.server(),
				name,
				language
			   )	  ;

		DPRINTF
		((	"KBDocRoot::loadScriptModules loading [%s.%s]\n",
			name.latin1(),
			lang.latin1()
		))	;

		if (!scriptIF->load (scrloc, error))
		{
			DPRINTF
			((	"KBDocRoot::loadScriptModules: load failed [%s][%s][%s]\n",
				scrloc.ident     ().latin1(),
				pError.getMessage().latin1(),
				pError.getDetails().latin1()
			))	;

			return	new KBScriptError
					(	error,
						m_parent,
						scrloc,
						QString::null,
						0,
						false
					)	;
		}
	}

	return	0 ;
}

/*  KBDocRoot								*/
/*  getScriptIF	: Get script interface					*/
/*  l2		: bool		  : Use second language			*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: KBScriptIF *	  : Script interface or null on error	*/

KBScriptIF
	*KBDocRoot::getScriptIF
	(	bool		l2,
		KBError		&pError
	)
{
	/* This method is used to get a scripting interface. It (a)	*/
	/* assumes that there is a defined language and (b) does not do	*/
	/* and script module loading or import setup.			*/
	KBScriptIF	*iface	= 0 ;
	QString		language    ;

	if (l2)
		language = m_parent->getAttrVal ("language2") ;
	else	language = m_parent->getAttrVal ("language" ) ;

	DPRINTF
	((	"KBDocRoot::getScriptIF: l2=%d l=%s\n",
		l2,
		(cchar *)language
	))	;

	if (language.isEmpty())
	{
		pError	= KBError
			  (	KBError::Error,
				l2 ? 
					TR("No second scripting language specified") :
					TR("No scripting language specified"),
				QString::null,
				__ERRLOCN
			  )	;
		return	0	;
	}

	if ((iface = LinkKBScript (language, pError)) == 0)
		return	0	;

	return	iface	;
}

/*  KBDocRoot								*/
/*  loadScripting: Load scripting and modules				*/
/*  pError	 : KBScriptError *& : Error return			*/
/*  (returns)	 : KBScriptIF *	    : Script interface or null on error	*/

KBScriptIF
	*KBDocRoot::loadScripting
	(	KBScriptError	*&pError
	)
{
	KBError	error	;
	pError	= 0	;

	if (m_scrIface[0] != 0)
		return	m_scrIface[0] ;

	if ((m_scrIface[0] = getScriptIF (false, error)) == 0)
	{
		pError	= new KBScriptError (error) ;
		return	0 ;
	}

	m_scripts.clear () ;
	m_imports.clear () ;

	TITER
	(	Import,
		KBDocRoot::m_children,
		i,
		m_imports.append (i->getModuleName())
	)
	TITER
	(	Script,
		KBDocRoot::m_children,
		s,
		if (!s->isL2())
			m_scripts.append (s->getModuleName()) ;
	)

	pError	= loadScriptModules
			(	m_scrIface[0],
				m_parent->getAttrVal ("language"),
				m_scripts
			)	;
	if (pError != 0)
		return	0 ;

	KBAttr	*a	= m_parent->getAttr("local") ;
	KBEvent	*e	= a == 0 ? 0 : a->isEvent () ;

	DPRINTF
	((	"KBDocRoot::loadScripting: local %p/%p\n",
		(void *)a,
		(void *)e
	))	;

	if ((e != 0) && !e->getValue().stripWhiteSpace().isEmpty())
	{
		QString	   psname = m_parent->getAttrVal("uuid") + "$RekallLocal" ;
		KBLocation locLoc
			(	0,
				"script",
				KBLocation::m_pInline,
				psname,
				e->getValue()
			)	;

		DPRINTF
		((	"KBDocRoot::loadScripting: loading .....\n"
		))	;

		if (!m_scrIface[0]->load (locLoc, error))
		{
			DPRINTF
			((	"KBDocRoot::loadScripting: local failed [%s][%s][%s]\n",
				scrloc.ident     ().latin1(),
				pError.getMessage().latin1(),
				pError.getDetails().latin1()
			))	;

			pError	= new KBScriptError (error, e) ;
			return	0 ;
		}

		m_imports.append (psname) ;
	}
	
	return	m_scrIface[0] ;
}

/*  KBDocRoot								*/
/*  loadScripting2: Load second language scripting and modules		*/
/*  pError	  : KBScriptError *& : Error return			*/
/*  (returns)	  : KBScriptIF *     : Script interface, null on error	*/

KBScriptIF
	*KBDocRoot::loadScripting2
	(	KBScriptError	*&pError
	)
{
	KBError	error	;
	pError	= 0	;

	if (m_scrIface[1] != 0)
		return	m_scrIface[1] ;

	if ((m_scrIface[1] = getScriptIF (true, error)) == 0)
	{
		pError	= new KBScriptError (error) ;
		return	0 ;
	}

	m_scripts2.clear () ;

	TITER
	(	Script,
		KBDocRoot::m_children,
		s,
		if (s->isL2())
			m_scripts2.append (s->getModuleName()) ;
	)

	pError	= loadScriptModules
			(	m_scrIface[1],
				m_parent->getAttrVal ("language2"),
				m_scripts2
			)	;
	if (pError != 0)
		return	0 ;

	return	m_scrIface[1] ;
}

/*  KBDocRoot								*/
/*  doLayoutChanged							*/
/*		: Notify layout has been changed			*/
/*  (returns)	: void		:					*/

void	KBDocRoot::doLayoutChanged ()
{
//	KBForm	*form	= m_parent->isForm() ;
//	if (form != 0)
//	{
//		QSize	size	= form->sizeHint() ;
//		form->resize (form->sizeHint()) ;
//
//		DPRINTF
//		((	"KBDocRoot::doLayoutChanged: %s\n",
//			_TEXT(size)
//		))	;
//	}
//
	emit	layoutChanged () ;
}

void	KBDocRoot::setStatusBar
	(	QLabel		*message,
		QLabel		*locking,
		KBProgressBox	*progress
	)
{
	m_message  = message	;
	m_locking  = locking	;
	m_progress = progress	;

	if (m_locking)
		m_locking->setText (TR("Unlocked")) ;

}

void	KBDocRoot::loadingStart ()
{
	if (m_progress) m_progress->start() ;
	m_loading = true  ;
}

void	KBDocRoot::loadingDone ()
{
	m_loading = false ;
	if (m_progress) m_progress->stop() ;
}


/*  KBDocRoot								*/
/*  loadingProgress							*/
/*		: Set progress						*/
/*  total	: uint		: Target value				*/
/*  done	: uint		: Done so far				*/
/*  message	: QString	: Message				*/
/*  (returns)	: bool		: True if cancelled			*/

bool	KBDocRoot::loadingProgress
	(	uint		total,
		uint		done,
		QString		message
	)
{
	DPRINTF
	((	"KBDocRoot::setProgress: %d %d [%s]\n",
		total,
		done,
		(cchar *)message
	))	;

	if (m_progress)
	{
		m_progress->setTotal (total)	;
		m_progress->setDone  (done )	;
		return	m_progress->cancelled() ;
	}

	return	false	;
}

/*  KBDocRoot								*/
/*  appFont	: Get application font for the document			*/
/*  (returns)	: const QString & : Font specification			*/

const QString
	&KBDocRoot::appFont ()
{
	/* Check for serverInfo, since this will not be set if the form	*/
	/* is running as part of the test suite.			*/
	if (m_docServerInfo == 0)
	{
		static	QString	dummy	;
		return	dummy		;
	}

	return	m_docServerInfo->appFont() ;
}

/*  KBDocRoot								*/
/*  skinName	: Get skin name for this document			*/
/*  (returns)	: const QString & : Skin name				*/

QString	KBDocRoot::skinName ()
{
	DPRINTF
	((	"KBDocRoot::skinName: m_docServerInfo=%p\n",
		(void *)m_docServerInfo
	))	;

	QString	name	= m_parent->getAttrVal ("skin") ;
	if (name.isEmpty   ()) return name ;

	if (m_docServerInfo != 0)
	{
		QString	suffix = m_docServerInfo->skinSuffix () ;
		if (!suffix.isEmpty ()) name = name + "_" + suffix ;
	}

	return	name	;
}

/*  KBDocRoot								*/
/*  showTests	: Get show-tests setting				*/
/*  (returns)	: KBServerInfo::TestsEnable : Setting			*/

KBServerInfo::TestsEnable KBDocRoot::showTests ()
{
	return	m_docServerInfo == 0 ?
			KBServerInfo::TestsOff : m_docServerInfo->showTests () ;
}

/*  KBDocRoot								*/
/*  gotExecError: Get reference to error flag				*/
/*  (returns)	: const bool &	: Reference to error flag		*/

const bool &KBDocRoot::gotExecError ()
{
	return	m_gotExecError	;
}

/*  KBDocRoot								*/
/*  getScripts	: Get list of scripts					*/
/*  (returns)	: const QStringList & : List of scripts			*/

const QStringList &KBDocRoot::getScripts ()
{
	return	m_scripts ;
}

/*  KBDocRoot								*/
/*  getImports	: Get list of imports					*/
/*  (returns)	: const QStringList & : List of imports			*/

const QStringList &KBDocRoot::getImports ()
{
	return	m_imports ;
}

/*  KBDocRoot								*/
/*  isLoading	: Return data-loading flag				*/
/*  (returns)	: bool		: Loading in progress			*/

bool	KBDocRoot::isLoading ()
{
	return	m_loading ;
}

/*  KBDocRoot								*/
/*  getDBInfo	: Get database information object			*/
/*  (returns)	: KBDBInfo *	: Database information			*/

KBDBInfo *KBDocRoot::getDBInfo ()
{
	return	m_docLocation.dbInfo() ;
}

/*  KBDocRoot								*/
/*  getDocLocation: Get document location object			*/
/*  (returns)	  : const KBLocation & : Document location		*/

const KBLocation &KBDocRoot::getDocLocation ()
{
	return	m_docLocation ;
}

const KBLocation &KBDocRoot::getDataLocation ()
{
	return	m_dataLocation ;
}

/*  KBDocRoot								*/
/*  loadSkin	: Load skin if not already loaded			*/
/*  (returns)	: void		:					*/

void	KBDocRoot::loadSkin ()
{
	if (m_skin == 0)
	{
		/* The skin is loaded on demand. Get the skin name for	*/
		/* the form or report; if empty then create an empty	*/
		/* skin, so that we do not keep doing a lookup.		*/
		QString	name	= skinName () ;

		DPRINTF
		((	"KBDocRoot::loadSkin: name=[%s]\n",
			name.latin1()
		))	;


		if (name.isEmpty())
		{
			m_skin	= new KBSkin() ;
			return	;
		}

		/* Load the skin definition. If an error occurs, report	*/
		/* it, and create an empty skin as above.		*/
		KBError	     error	;
		KBLocation   skinloc
			     (		m_docLocation.dbInfo(),
					"skin",
					m_docLocation.server(),
					m_docLocation.samePlace(name),
					"skn"
			     )	  ;
		QDomDocument doc  = skinloc.contentsAsDom (TR("Skin"), error) ;
		if (doc.isNull())
		{
			DPRINTF
			((	"KBDocRoot::loadSkin: error [%s]\n",
				skinloc.ident().latin1()
			))	;

			error.DISPLAY()	;
			m_skin	= new KBSkin() ;
			return	;
		}

		/* Got the definition, create a skin from the		*/
		/* definition.						*/
		m_skin	= new KBSkin (doc.documentElement()) ;
		return	;
	}
}

/*  KBDocRoot								*/
/*  skinFGColor	: Get skin foreground colour				*/
/*  name	: const QString & : Element name			*/
/*  (returns)	: const QString & : Colour				*/

const QString
	&KBDocRoot::skinFGColor
	(	const QString	&name
	)
{
	if (m_skin == 0) loadSkin () ;

	return	m_skin->fgColor (name) ;
}

/*  KBDocRoot								*/
/*  skinBGColor	: Get skin background colour				*/
/*  name	: const QString & : Element name			*/
/*  (returns)	: const QString & : Colour				*/

const QString
	&KBDocRoot::skinBGColor
	(	const QString	&name
	)
{
	if (m_skin == 0) loadSkin () ;

	return	m_skin->bgColor (name) ;
}

/*  KBDocRoot								*/
/*  skinFont	: Get skin font						*/
/*  name	: const QString & : Element name			*/
/*  (returns)	: const QString & : Font				*/

const QString
	&KBDocRoot::skinFont
	(	const QString	&name
	)
{
	if (m_skin == 0) loadSkin () ;
	return	m_skin->font  (name) ;
}

/*  KBDocRoot								*/
/*  skinElements: Get skin element dictionary				*/
/*  (returns)	: const QDict<KBSkinElement> & : Dictionary		*/

const QDict<KBSkinElement>
	&KBDocRoot::skinElements ()
{
	if (m_skin == 0) loadSkin () ;
	return	m_skin->elements  () ;
}

/*  KBDocRoot								*/
/*  skinChanged	: Form skin has been changed				*/
/*  (returns)	: void		     :					*/

void	KBDocRoot::skinChanged ()
{
	DPRINTF
	((	"KBDocRoot::skinChanged: called .....\n"
	))	;

	DELOBJ	    (m_skin) ;
	skinChanged (m_parent->isObject()) ;
}

/*  KBDocRoot								*/
/*  skinChanged	: Update after skin change				*/
/*  object	: KBObject *	: Object to update			*/
/*  (returns)	: void		:					*/

void	KBDocRoot::skinChanged
	(	KBObject	*object
	)
{
	if (object != 0)
	{
		object->setupProperties () ;

		TITER
		(	Object,
			object->getChildren(),
			obj,
			skinChanged(obj)
		)
	}
}

/*  KBDocRoot								*/
/*  slotSkinChanged							*/
/*		: Called when a skin has been changed			*/
/*  skinLoc	: const KBLocation & : Skin location			*/
/*  (returns)	: void		     :					*/

void	KBDocRoot::slotSkinChanged
	(	const KBLocation	&skinLoc
	)
{
	DPRINTF
	((	"KBDocRoot::slotSkinChanged: [%s][%s]\n",
		skinLoc.server().latin1(),
		skinLoc.name  ().latin1()
	))	;

	if (m_docLocation.server() == skinLoc.server())
		if (m_parent->getAttrVal("skin") == skinName())
			skinChanged () ;
}

/*  KBDocRoot								*/
/*  buildNodeMap: Construct global node name map			*/
/*  node	: KBNode *	: Node from which to work		*/
/*  (returns)	: void		:					*/

void	KBDocRoot::buildNodeMap
	(	KBNode		*node
	)
{
	/* Put this node in the map. We use replace so that duplicate	*/
	/* names don't build up inaccessible entries, not that it	*/
	/* really matters.						*/
	m_namedNodeMap.replace (node->getName(), node) ;

	/* Recurse down, except that we stop at a component link. We	*/
	/* do not go lower since the same component might be linked	*/
	/* multiple times. We will construct global maps inside the	*/
	/* links.							*/
	if (node->isCompLink() == 0)
		LITER
		(	KBNode,
			node->getChildren(),
			n,
			buildNodeMap (n)
		)	;
}

/*  KBDocRoot								*/
/*  getNamedNode : Get globally named node				*/
/*  name	 : const QString & : Node name				*/
/*  (returns)	 : KBNode *	   : Node or null if not found		*/

KBNode	*KBDocRoot::getNamedNode
	(	const QString	&name
	)
{
	/* The named node map is cleared when the parent document	*/
	/* resets and is then populated on demand. There must be at	*/
	/* least one node (the form or report) so the count check is OK	*/
	if (m_namedNodeMap.count() == 0)
		buildNodeMap (m_parent) ;

	return	m_namedNodeMap.find(name) ;
}

KBNode	*KBDocRoot::parent ()
{
	return	m_parent ;
}

#if	! __KB_RUNTIME

KBTest	*KBDocRoot::enterTest
	(	KBTest		*test
	)
{
	KBTest	*orig	= m_test ;
	m_test		= test	 ;
	return	orig	;
}

KBTest	*KBDocRoot::runningTest ()
{
	return	m_test	;
}

#endif
