/***************************************************************************
    file	         : kb_docroot.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		_KB_DOCROOT_H
#define		_KB_DOCROOT_H

#include	"libkbase_exports.h"

#include 	<qptrlist.h>
#include 	<qstringlist.h>
#include 	<qguardedptr.h>

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_attrdict.h"
#include	"kb_qrybase.h"


class	QLabel			;

class	KBMimeSourceFactory	;
class	KBProgressBox		;
class	KBSkin			;
class	KBSkinElement		;
class	KBTest			;
class	KBScriptError		;

/*  ------------------------------------------------------------------  */

/*  KBDocRoot								*/
/*  ---------								*/
/*  This class is used at the top-most node in parsed documents, such	*/
/*  as forms and reports. It holds information, such as the document	*/
/*  location, that is common to all such documents.			*/

class	LIBKBASE_API	KBDocRoot : public QObject
{
	Q_OBJECT

	KBNode				*m_parent		;
	QPtrList<KBNode>		&m_children		;
	bool				m_donePDlg		;
	bool				m_gotExecError		;

 	QStringList			m_scripts		;
 	QStringList			m_scripts2		;
	QStringList			m_imports		;

	KBSkin				*m_skin			;
	KBTest				*m_test			;

	bool				m_loading		;
	QGuardedPtr<QLabel>		m_message		;
	QGuardedPtr<QLabel>		m_locking		;
	QGuardedPtr<KBProgressBox>	m_progress		;

	KBMimeSourceFactory		*m_mimeSource		;

	KBLocation			m_docLocation		;
	KBLocation			m_dataLocation		;
	KBServerInfo			*m_docServerInfo	;

	KBScriptIF			*m_scrIface[2]		;
	KBAttrDict			*m_paramDict		;

	QDict<KBNode>			m_namedNodeMap		;


	KBScriptError		*loadScriptModules
				(	KBScriptIF		*scriptIF,
					const QString		&language,
					const QStringList	&scripts
				)	;

	void			buildNodeMap		(KBNode   *)	;
	void			skinChanged		(KBObject *)	;
	void			loadSkin		()		;

signals	:

	void			execError 		()		;
	void			requestClose		(int)		;
	void			layoutChanged		()		;

protected slots	:

	void			slotSkinChanged	(const KBLocation &)	;

public	:

	KBDocRoot
	(	KBNode			*,
		QPtrList<KBNode> 	&,
		const KBLocation 	&
	)	;
       ~KBDocRoot	() ;

	KBDBInfo	 	*getDBInfo		() ;

	const KBLocation 	&getDocLocation		() ;
	const KBLocation 	&getDataLocation	() ;

	void			reset			() ;

	KBScriptIF		*getScriptIF		(bool, KBError  &) ;
	KBScriptIF	 	*loadScripting		(KBScriptError *&) ;
	KBScriptIF	 	*loadScripting2		(KBScriptError *&) ;

	KB::ShowRC		setParamDict		(const QDict<QString> &, KBError &) ;
	const KBAttrDict	&parameterDict		() ;
	QString			getParamValue		(cchar *) ;
	void			setParamValue		(cchar *, const QString &) ;

	const QString		&appFont		() ;
	QString			skinName		() ;
	KBServerInfo::TestsEnable
				showTests		() ;

	void			setStatusBar		(QLabel *, QLabel *, KBProgressBox *) ;
	KBMimeSourceFactory	*mimeSource		() ;

	const QStringList	&getScripts		() ;
	const QStringList	&getImports		() ;

	void			loadingStart		() ;
	void			loadingDone		() ;
	bool			loadingProgress		(uint, uint, QString = QString::null) ;

	bool			isLoading		() ;

	void			doExecError		() ;
	void			doRequestClose		(int) ;
	void			doSetMessage		(const QString    &) ;
	void			doSetLocking		(KBQryBase::Locking) ;
	void			doLayoutChanged		() ;

	void			skinChanged		() ;

	const QString		&skinFGColor		(const QString &) ;
	const QString		&skinBGColor		(const QString &) ;
	const QString		&skinFont		(const QString &) ;
	const QDict<KBSkinElement>
				&skinElements 		() ;

	KBNode			*parent			() ;
	KBNode			*getNamedNode		(const QString &) ;

#if	! __KB_RUNTIME
	KBTest			*enterTest		(KBTest *)	;
	KBTest			*runningTest		()		;
#endif
	/* The following method indicates whether an execution error	*/
	/* has occurred. This is returned as a (const) reference to the	*/
	/* error flag for reasons of efficiency on the scripting	*/
	/* interfaces, viz:						*/
	/*								*/
	/*	const bool &e = xxx->gotExecError() ;			*/
	/*	if (e) return ....	;				*/
	/* 	make_api_call ()	;				*/
	/*	if (e) return ....	;				*/
	/*								*/
	/* Hence, error occurance can be checked before and after the	*/
	/* API call at minimum cost.					*/

	const bool		&gotExecError		() ;
}	;


#endif		// _KB_DOCROOT_H
