/***************************************************************************
    file	         : kb_downloader.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	"kb_classes.h"
#include	"kb_deleter.h"

#if		__KB_AUTOMOC
#include	"kb_downloader.moc"
#else		
#include	"kb_downloader.h"
#endif


#define		__DEBUG_DOWNLOADER	(0)
#if		__DEBUG_DOWNLOADER
#include	"kb_dprintf.h"
#endif


KBHttpWrapper::KBHttpWrapper
	(	KBDownloader	*downloader
	)
	:
	m_downloader	(downloader)
{
	connect
	(	this,
		SIGNAL	(requestStarted	 (int)),
		SLOT	(slotHTTPStarted (int))
	)	;
	connect
	(	this,
		SIGNAL	(requestFinished (int, bool)),
		SLOT	(slotHTTPFinished(int, bool))
	)	;
	connect
	(	this,
		SIGNAL	(dataReadProgress(int, int)),
		SLOT	(slotHTTPProgress(int, int))
	)	;
	connect
	(	this,
		SIGNAL	(readyRead	 (const QHttpResponseHeader &)),
		SLOT	(slotHTTPReady	 (const QHttpResponseHeader &))
	)	;

}

KBHttpWrapper::~KBHttpWrapper ()
{
	closing	() ;
}

void	KBHttpWrapper::closing ()
{
	/* Disconnect signals before deleting, stupid class can emit	*/
	/* signals while inside the destructor.				*/
	disconnect(this) ;
}

void	KBHttpWrapper::slotHTTPStarted
	(	int		id
	)
{
	m_downloader->slotHTTPStarted (id) ;
}

void	KBHttpWrapper::slotHTTPFinished
	(	int		id,
		bool		rc
	)
{
	m_downloader->slotHTTPFinished (id, rc) ;
}

void	KBHttpWrapper::slotHTTPProgress
	(	int		done,
		int		total
	)
{
	m_downloader->slotHTTPProgress (done, total) ;
}

void	KBHttpWrapper::slotHTTPReady
	(	const QHttpResponseHeader	&resp
	)
{
	m_downloader->slotHTTPReady (resp) ;
}

/*  ------------------------------------------------------------------	*/

/*  KBDownloader								*/
/*  KBDownloader: Constructor for URL downloader			*/
/*  (returns)	: KBDownloader 	  :					*/

KBDownloader::KBDownloader ()
{
	m_http		= 0	;
	m_httpSetHostID	= -1	;
	m_httpGetID	= -1	;
	m_active	= false	;
}

/*  KBDownloader							*/
/*  ~KBDownloader: Destructor for URL downloader			*/
/*  (returns)	 :		:					*/

KBDownloader::~KBDownloader ()
{
}

/*  KBDownloader							*/
/*  setHTTPError: Display HTTP error					*/
/*  (returns)	: void		:					*/

void	KBDownloader::setHTTPError ()
{
	QString	msg	= TR("Unrecognised error occurred") ;

	if (m_http != 0)
	{
		switch (m_http->error())
		{
			case QHttp::NoError :
				msg	= TR("No error occured") ;
				break	;

			case QHttp::HostNotFound :
				msg	= TR("Host not found") ;
				break	;

			case QHttp::ConnectionRefused :
				msg	= TR("Connection to host refused") ;
				break	;

			case QHttp::UnexpectedClose :
				msg	= TR("Host unexpectedly closed connection") ;
				break	;

			case QHttp::InvalidResponseHeader :
				msg	= TR("Invalid response from host") ;
				break	;

			case QHttp::WrongContentLength :
				msg	= TR("Host sent wrong content length") ;
				break	;

			case QHttp::Aborted :
				msg	= TR("Transfer aborted") ;
				break	;

			case QHttp::UnknownError :
				msg	= TR("Unknown error occurred") ;
				break	;

			default	:
				break	;
		}

		m_http->closing	() ;
		KBDeleter::addObject (m_http) ;
		m_http	= 0	;
	}

	setError (msg)   ;
	m_active = false ;
}

/*  KBDownloader							*/
/*  slotHTTPStarted							*/
/*		: QHttp object signals start of an operation		*/
/*  id		: int		: Operation identifier			*/
/*  (returns)	: void		:					*/

void	KBDownloader::slotHTTPStarted
	(	int		id
	)
{
	if (id == m_httpSetHostID)
	{
		DPRINTF
		((	"KBDownloader::slotHTTPStarted: Connecting to host\n"
		))	;
		setProgress (TR("Connecting to remote host")) ;
		return	;
	}
	if (id == m_httpGetID)
	{
		DPRINTF
		((	"KBDownloader::slotHTTPStarted: retrieveing data\n"
		))	;
		setProgress (QString(TR("Retrieving %1")).arg(m_what)) ;
		return	;
	}
}

/*  KBDownloader							*/
/*  slotHTTPFinished							*/
/*		: QHttp object signals completion of an operation	*/
/*  id		: int		: Operation identifier			*/
/*  error	: bool		: Error occurred			*/
/*  (returns)	: void		:					*/

void	KBDownloader::slotHTTPFinished
	(	int		id,
		bool		error
	)
{
	/* First case: ::setHost() completes. This is actually the	*/
	/* connection to the server, so we display that (or an error).	*/
	/* If the connection was OK then the QHttp object will move on	*/
	/* automatically to download the database specification.	*/
	if (id == m_httpSetHostID)
	{
		DPRINTF
		((	"KBDownloader::slotHTTPFinished: Connected to host: err=%d\n",
			error
		))	;
		if (error)
		{	setHTTPError () ;
			return	;
		}
		m_httpSetHostID	= -1	;
		setProgress (TR("Connected to remote host")) ;
		return	;
	}

	/* Second case: ::get() completes. This is the end of the data	*/
	/* download, possibly with an error.				*/
	if (id == m_httpGetID)
	{
		DPRINTF
		((	"KBDownloader::slotHTTPFinished: retrieved data: err=%d\n",
			error
		))	;
		if (error)
		{	setHTTPError () ;
			return	;
		}
		m_httpGetID	= -1	;
		m_active	= false	;
		setProgress (QString(TR("Retrieved %1")).arg(m_what)) ; ;
		downloadComplete ()  ;
		return	;
	}
}

/*  KBDownloader								*/
/*  slotHTTPProgress							*/
/*		: QHttp object reports progress				*/
/*  done	: int		: Download completed so far		*/
/*  total	: int		: Total download			*/
/*  (returns)	: void		:					*/

void	KBDownloader::slotHTTPProgress
	(	int		done,
		int		total
	)
{
	DPRINTF
	((	"KBDownloader::slotHTTPProgress: got %d of %d\n",
		done,
		total
	))	;
	setProgress (done, total) ;
}

/*  KBDownloader								*/
/*  slotHTTPReady							*/
/*		: Data available from HTTP object			*/
/*  (returns)	: void		:					*/

void	KBDownloader::slotHTTPReady
	(	const QHttpResponseHeader	&resp
	)
{
	DPRINTF
	((	"KBDownloader::slotHTTPReady: resp.rc=%d\n",
		resp.statusCode()
	))	;

	if (resp.statusCode() != 200)
	{
		setError
		(	QString(TR("%1 (code %2)"))
				.arg(resp.reasonPhrase())
				.arg(resp.statusCode  ())
		)	;

		m_http->closing	() ;
		KBDeleter::addObject  (m_http) ;
		m_http	= 0	;
		return	;
	}

	downloadData (m_http->readAll()) ;
}

/*  KBDownloader								*/
/*  exec	: Execute download					*/
/*  url		: const QString & : Specification URL			*/
/*  what	: const QString & : What is being downloaded		*/
/*  (returns)	: QString	: Error or null on OK			*/

QString	KBDownloader::exec
	(	const QString	&url,
		const QString	&what
	)
{
	m_url	= QUrl(url)	;
	m_what	= what		;

	/* See if the specification originates on an HTTP server, in	*/
	/* which case set up to download the specification.		*/
	if (m_url.protocol().lower() == "http")
	{
		m_http		= new KBHttpWrapper (this) ;
		m_active	= true	;

		DPRINTF
		((	"KBDownloader::exec: host=[%s] port=[%d] get=[%s]\n",
			(cchar *)m_url.host(),
			m_url.port(),
			(cchar *)m_url.path()
		))	;

		int	port	= m_url.port()	;
		if (port < 0) port = 80		;
		m_httpSetHostID	= m_http->setHost (m_url.host(), port) ;
		m_httpGetID	= m_http->get     (m_url.path()) ;

		return	QString::null	;
	}

	return	TR("Unknown download protocol") ;
}

void	KBDownloader::halt ()
{
	if (m_http != 0)
	{
		m_http->closing	() ;
		KBDeleter::addObject (m_http) ;
		m_http	= 0	;
	}

	m_active = false	;
}
