/***************************************************************************
    file	         : kb_downloader.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_DOWNLOADER_H
#define	__KB_DOWNLOADER_H


#include	"libkbase_exports.h"

#include	<qstring.h>
#include	<qurl.h>
#include	<qhttp.h>


class	KBError			;
class	QObject			;
class	QHttpResponseHeader	;
class	KBDownloader		;


/*  KBHttpWrapper							*/
/*  -------------							*/
/*  This is a wrapper around QHttp which translates QHttp signals into	*/
/*  calls back into the parent KBDownloader. This is needed since we	*/
/*  do not want KBDownloader to be QObject-derived, as we want to be	*/
/*  able to use it in a multiply-inherited class which already has a	*/
/*  QObject inheritence.						*/

class	KBHttpWrapper : public QHttp
{
	Q_OBJECT

	KBDownloader	*m_downloader	;

protected slots	:

	void		slotHTTPStarted	(int)		  ;
	void		slotHTTPFinished(int, bool)	  ;
	void		slotHTTPProgress(int, int )	  ;
	void		slotHTTPReady	(const QHttpResponseHeader &) ;

public	:

	KBHttpWrapper	(KBDownloader *) ;
	~KBHttpWrapper	() ;

	void		closing		()	;
}	;

/*  KBDownloader							*/
/*  ------------							*/

class	LIBKBASE_API	KBDownloader
{
	QUrl		m_url			;
	QString		m_what			;

	KBHttpWrapper	*m_http			;
	int		m_httpSetHostID		;
	int		m_httpGetID		;
	bool		m_active		;

	void		slotHTTPStarted	(int)		  ;
	void		slotHTTPFinished(int, bool)	  ;
	void		slotHTTPProgress(int, int )	  ;
	void		slotHTTPReady	(const QHttpResponseHeader &) ;

protected	:

	void		setHTTPError	()		  ;

	/* The following bunch of methods are pur virtual and	*/
	/* should be implemented appropriately in the derived	*/
	/* class						*/
	virtual	void	setError	(const KBError &) 	= 0	;
	virtual	void	setError	(const QString &, const QString & = QString::null) = 0 ;

	virtual	void	setProgress	(const QString &) 	= 0	;
	virtual	void	setProgress	(int, int )	  	= 0	;

	virtual	void	downloadData	(const QByteArray &)	= 0	;
	virtual	void	downloadComplete()		  	= 0	;

public	:

	KBDownloader			() ;
	virtual	~KBDownloader		() ;

	QString		exec		(const QString &, const QString &) ;
	void		halt		() ;

	inline bool	active	()
	{
		return	m_active	;
	}

	friend	class	KBHttpWrapper	;
}	;

#endif	// __KB_DOWNLOADER_H
