/***************************************************************************
    file	         : kb_dumper.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<time.h>
#include	<errno.h>

#include	<qstring.h>
#include	<qfile.h>
#include	<qtextstream.h>
#include	<qtimer.h>
#include	<qlabel.h>

#include	"tk_messagebox.h"
#include	"rk_pushbutton.h"
#include	"rk_lineedit.h"
#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_listview.h"
#include	"tk_icons.h"
#include	"tk_filedialog.h"

#include	"kb_database.h"
#include	"kb_dbinfo.h"
#include	"kb_dblink.h"
#include	"kb_dbdociter.h"

#include	"kb_dialog.h"
#include	"kb_file.h"

#include	"kb_copybase.h"
#include	"kb_copytable.h"
#include	"kb_copyxml.h"
#include	"kb_copyexec.h"
#include	"kb_paramsetdlg.h"
#include	"kb_manual.h"

#if		__KB_AUTOMOC
#include	"kb_dumper.moc"
#else
#include	"kb_dumper.h"
#endif

#define		__DEBUG_DUMPER	(0)
#if		__DEBUG_DUMPER
#include	"kb_dprintf.h"
#endif



class	LIBKBASE_API	KBCopyExecDumper : public KBCopyExec
{
	KBDumper	*m_dumper	;

public	:

	KBCopyExecDumper
	(	KBCopyBase	*,
		KBCopyBase	*,
		KBDumper	*
	)	;

	virtual	bool	showProgress	(int)	;
}	;

KBCopyExecDumper::KBCopyExecDumper
	(	KBCopyBase	*srce,
		KBCopyBase	*dest,
		KBDumper	*dumper
	)
	:
	KBCopyExec	(srce, dest),
	m_dumper	(dumper)
{
}

bool	KBCopyExecDumper::showProgress
	(	int		nRows
	)
{
	return	m_dumper->showProgress (nRows) ;
}

/*  ------------------------------------------------------------------  */

/*  KBDumper								*/
/*  KBDumper	: Constructor for database dumper			*/
/*  dbInfo	: KBDBInfo *	  : Database information		*/
/*  server	: const QString & : Server name				*/
/*  dir		: const QString & : Output directory			*/
/*  (returns)	: KBDumper	  :					*/

KBDumper::KBDumper
	(	KBDBInfo	*dbInfo,
		const QString	&server,
		const QString	&dir
	)
	:
	KBDialog	(TR("Dump database"), true, "kbdumperdlg"),
	m_dbInfo	(dbInfo),
	m_server	(server),
	m_dir		(dir)
{
	RKVBox	  *layMain = new RKVBox (this) ;
	layMain->setTracking () ;

	RKGridBox *layGrid = new RKGridBox (2, layMain) ;

	m_dumpAll	= new QCheckBox (TR("Dump all objects"), layGrid) ;
	new QWidget(layGrid) ;

	m_dumpDefn	= new QCheckBox (TR("Dump table definitions"), layGrid) ;
	new QWidget(layGrid) ;

	m_dumpData	= new QCheckBox (TR("Dump table data"), layGrid) ;
	new QWidget(layGrid) ;

	m_dumpSingle	= new QCheckBox    (TR("Dump to single file"), layGrid) ;

	m_singleFile	= new RKLineEdit   (layGrid) ;

	m_tableList	= new RKListView(layMain) ;

	RKHBox	*layProg= new RKHBox	(layMain) ;

	QLabel	*l1	= new QLabel	(TR("Object"  ), layProg) ;
	m_dumping	= new QLabel	(layProg) ;

	QLabel	*l2	= new QLabel	(TR("Record"  ), layProg) ;
	m_record	= new QLabel	(layProg) ;

	QLabel	*l3	= new QLabel	(TR("Object"  ), layProg) ;
	m_of		= new QLabel	(layProg) ;

	l1->setAlignment(Qt::AlignRight) ;
	l2->setAlignment(Qt::AlignRight) ;
	l3->setAlignment(Qt::AlignRight) ;

	addOKCancel	(layMain, &m_bOK, &m_bCancel, KBHELPKEY("Chap14DumperLoader"));
	m_bOK->setDefault (true) ;

	connect
	(	m_dumpSingle,
		SIGNAL	(toggled(bool)),
		SLOT	(slotSingleChange())
	)	;

	m_dumping->setMinimumWidth (200) ;

	m_dumping->setFrameStyle (QFrame::Box|QFrame::Plain) ;
	m_dumping->setLineWidth  (1) ;
	m_record ->setFrameStyle (QFrame::Box|QFrame::Plain) ;
	m_record ->setLineWidth  (1) ;
	m_of	 ->setFrameStyle (QFrame::Box|QFrame::Plain) ;
	m_of	 ->setLineWidth  (1) ;


	m_cancelled	= false	;
	m_finished	= false ;
	m_index		= 0	;
	m_currItem	= 0	;

	m_tableList->addColumn(TR("Name")) ;
	m_tableList->addColumn(TR("Type")) ;
	slotSingleChange ()	;
}

/*  KBDumper								*/
/*  accept	: User clicks OK button					*/
/*  (returns)	: void		:					*/

void	KBDumper::accept ()
{
	if (!m_finished)
	{
		if (!m_dumpDefn->isChecked() && !m_dumpData->isChecked())
		{
			TKMessageBox::sorry
			(	0,
				TR("Please select definition and/or data dumping"),
				TR("Dump Database")
			)	;
			return	;
		}

		if (!m_dumpAll->isChecked())
		{
			bool	any = false ;
			QCheckListItem *i = (QCheckListItem *)m_tableList->firstChild() ;
			while (i != 0)
				if (i->isOn())
				{	any	= true ;
					break	;
				}
				else	i = (QCheckListItem *)i->nextSibling() ;

			if (!any)
			{
				TKMessageBox::sorry
				(	0,
					TR("No tables selected for dumping"),
					TR("Dump Database")
				)	;
				return	;
			}
		}

		m_dumpAll   ->setEnabled (false) ;
		m_dumpDefn  ->setEnabled (false) ;
		m_dumpData  ->setEnabled (false) ;
		m_dumpSingle->setEnabled (false) ;
		m_tableList ->setEnabled (false) ;
		m_bOK	    ->setEnabled (false) ;

		/* If dumping to a single file create a DOM document	*/
		/* and document element. This will be populated as we	*/
		/* dump the various bits.				*/
		if (m_dumpSingle->isChecked())
		{
			m_document = QDomDocument ("dbspec") ;
			m_document.appendChild
			(	m_document.createProcessingInstruction
				(	"xml",
					"version=\"1.0\" encoding=\"UTF-8\""
			)	)	;

			m_element  = m_document.createElement ("dbspec") ;
			m_document.appendChild (m_element) ;
		}

		/* Note the start time. Used so we can update the	*/
		/* progress more frequently if things are running	*/
		/* slowly.						*/
		m_ldtime = time(0) ;

		slotTimer () ;
		return	;
	}

	done	(1) ;
}


void	KBDumper::slotSingleChange ()
{
	m_singleFile  ->setEnabled (m_dumpSingle->isChecked()) ;
}

/*  KBDumper								*/
/*  reject	: User clicks cancel button				*/
/*  (returns)	: void		:					*/

void	KBDumper::reject ()
{
	if (!m_cancelled)
	{
		m_cancelled = true ;
		done	(0) ;
	}
}

bool	KBDumper::addFileObjects
	(	cchar		*type,
		cchar		*extn
	)
{
	KBDBDocIter	docIter	(true) ;
	KBError		error	;

	if (!docIter.init (m_dbInfo, m_server, type, extn, error))
	{	error.DISPLAY()	;
		return	false	;
	}

	QString	name	;
	QString	stamp	;
	while (docIter.getNextDoc(name, stamp))
	{
		new KBDumperItem (m_tableList, name, type) ;
	}

	return	true	;
}

/*  KBDumper								*/
/*  exec	: Execute dumper					*/
/*  (returns)	: int		: Exit code				*/

int	KBDumper::exec	()
{
	/* See if there are any likely looking files already in the	*/
	/* target directory, in which case check with the user.		*/
	QDir	dir ;

	dir.setPath	  (m_dir)	  ;
	dir.setFilter	  (QDir::Files)   ;
	dir.setNameFilter ("*.tabledef;*.tabledata;*.viewdef;*.seqdef;*.rkl.*") ;
	dir.setSorting	  (QDir::Name)    ;

	if (dir.entryList().count() > 0)
		if (TKMessageBox::questionYesNo
			(	0,
				TR("Directory already contains database dump files: continue anyway?"),
				TR("Dump Database")
			)
			!= TKMessageBox::Yes
		   )
		   return  0 ; 


	/* Link to the database and get the list of tables here and now	*/
	/* If OK then fire off a timer to start the dump cycle and	*/
	/* enter the execution loop.					*/
	if (!m_dbLink.connect (m_dbInfo, m_server))
	{	m_dbLink.lastError().DISPLAY() ;
		return 0 ;
	}

	if (!m_dbLink.listTables (m_tables, KB::IsAny))
	{	m_dbLink.lastError().DISPLAY() ;
		return 0 ;
	}

	m_tableList->setSorting (0) ;
	for (uint idx = 0 ; idx < m_tables.count() ; idx += 1)
		new KBDumperItem (m_tableList, &m_tables[idx]) ;

	if (m_server == KBLocation::m_pFile)
	{
		if (!addFileObjects ("form",      "frm")) return 0 ;
		if (!addFileObjects ("report",    "rep")) return 0 ;
		if (!addFileObjects ("query",     "qry")) return 0 ;
		if (!addFileObjects ("copier",    "cpy")) return 0 ;
		if (!addFileObjects ("component", "cmp")) return 0 ;
		if (!addFileObjects ("script",    "py" )) return 0 ;
		if (!addFileObjects ("script",    "kjs")) return 0 ;
		if (!addFileObjects ("print",     "prn")) return 0 ;
		if (!addFileObjects ("graphic",   "*"  )) return 0 ;
	}

	m_currItem = (KBDumperItem *)m_tableList->firstChild() ;
	m_index	   = 0 ;

	return	KBDialog::exec () ;
}

/*  KBDumper								*/
/*  dumpTableDef: Dump table definition					*/
/*  tabSpec	: KBTableSpec &	  : Table definition			*/
/*  dir		: const QString & : Destination directory		*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: bool		  : Success				*/

bool	KBDumper::dumpTableDef
	(	KBTableSpec	&tabSpec,
		KBError		&pError
	)
{
	/* If dumping to a single file then we create a table element	*/
	/* and attach the table definition to that. The ::toXML call	*/
	/* will set the name as an attribute.				*/
	if (m_dumpSingle->isChecked())
	{
		QDomElement tabElem	= m_document.createElement ("table") ;
		m_element.appendChild (tabElem) ;
		tabSpec.toXML (tabElem, 0) ;
		return	true  ;
	}

	QDomDocument	doc ("tablelist") ;
	doc.appendChild
	(	doc.createProcessingInstruction
		(	"xml",
			"version=\"1.0\" encoding=\"UTF-8\""
	)	)	;

	QDomElement	docElem	= doc.createElement ("tablelist") ;
	QDomElement	tabElem	= doc.createElement ("table"    ) ;

	doc    .appendChild (docElem) ;
	docElem.appendChild (tabElem) ;
	tabSpec.toXML	 (tabElem, 0) ;

	QString	eName	= m_dir + "/" + tabSpec.m_name + ".tabledef" ;
	QFile	eFile	(eName) ;
	if (!eFile.open (IO_WriteOnly|IO_Truncate))
	{
		pError	= KBError
			  (	KBError::Error,
				QString	(TR("Cannot open \"%1\"")).arg(eName),
				strerror(errno),
				__ERRLOCN
			)	;
		return	false	;
	}

	QTextStream(&eFile) << doc.toString() ;
	return	true	;
}

/*  KBDumper								*/
/*  dumpViewDef	: Dump view definition					*/
/*  viewSpec	: KBTableSpec &	  : View definition			*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: bool		  : Success				*/

bool	KBDumper::dumpViewDef
	(	KBTableSpec	&viewSpec,
		KBError		&pError
	)
{
	/* If dumping to a single file then we create a view element	*/
	/* and attach the view definition to that. The ::toXML call	*/
	/* will set the name as an attribute.				*/
	if (m_dumpSingle->isChecked())
	{
		QDomElement viewElem	= m_document.createElement ("view") ;
		m_element.appendChild (viewElem) ;
		viewSpec.toXML (viewElem, 0) ;
		return	true  ;
	}

	QDomDocument	doc ("viewlist") ;
	doc.appendChild
	(	doc.createProcessingInstruction
		(	"xml",
			"version=\"1.0\" encoding=\"UTF-8\""
	)	)	;

	QDomElement	docElem	 = doc.createElement ("viewlist") ;
	QDomElement	viewElem = doc.createElement ("view"    ) ;

	doc    .appendChild (docElem ) ;
	docElem.appendChild (viewElem) ;
	viewSpec.toXML	 (viewElem, 0) ;

	QString	eName	= m_dir + "/" + viewSpec.m_name + ".viewdef" ;
	QFile	eFile	(eName) ;
	if (!eFile.open (IO_WriteOnly|IO_Truncate))
	{
		pError	= KBError
			  (	KBError::Error,
				QString	(TR("Cannot open \"%1\"")).arg(eName),
				strerror(errno),
				__ERRLOCN
			)	;
		return	false	;
	}

	QTextStream(&eFile) << doc.toString() ;
	return	true	;
}

/*  KBDumper								*/
/*  dumpSequenceDef							*/
/*		: Dump sequence definition				*/
/*  seqSpec	: KBSequenceSpec & : Sequence definition		*/
/*  pError	: KBError &	   : Error return			*/
/*  (returns)	: bool		   : Success				*/

bool	KBDumper::dumpSequenceDef
	(	KBSequenceSpec	&seqSpec,
		KBError		&pError
	)
{
	/* If dumping to a single file then we create a sequence	*/
	/* element and attach the sequence definition to that. The	*/
	/* ::toXML call will set the name as an attribute.		*/
	if (m_dumpSingle->isChecked())
	{
		QDomElement seqElem	= m_document.createElement ("sequence") ;
		m_element.appendChild (seqElem) ;
		seqSpec.toXML (seqElem) ;
		return	true  ;
	}

	QDomDocument	doc ("sequencelist") ;
	doc.appendChild
	(	doc.createProcessingInstruction
		(	"xml",
			"version=\"1.0\" encoding=\"UTF-8\""
	)	)	;

	QDomElement	docElem	= doc.createElement ("sequencelist") ;
	QDomElement	seqElem	= doc.createElement ("sequence"    ) ;

	doc    .appendChild (docElem) ;
	docElem.appendChild (seqElem) ;
	seqSpec.toXML	    (seqElem) ;

	QString	eName	= m_dir + "/" + seqSpec.m_name + ".seqdef" ;
	QFile	eFile	(eName) ;
	if (!eFile.open (IO_WriteOnly|IO_Truncate))
	{
		pError	= KBError
			  (	KBError::Error,
				QString	(TR("Cannot open \"%1\"")).arg(eName),
				strerror(errno),
				__ERRLOCN
			)	;
		return	false	;
	}

	QTextStream(&eFile) << doc.toString() ;
	return	true	;
}

/*  KBDumper								*/
/*  dumpTableData: Dump table data					*/
/*  tabSpec	: KBTableSpec &	  : Table definition			*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: bool		  : Success				*/

bool	KBDumper::dumpTableData
	(	KBTableSpec	&tabSpec,
		KBError		&pError
	)
{
	/* This is done using a copier. Create a dummy location and	*/
	/* thence a table source and an XML destination. These can be	*/
	/* appropriately initialised with the table structure.		*/
	KBLocation	location   (m_dbInfo, "copier", m_server, "unnamed") ;
	KBCopyTable	*copyTable = new KBCopyTable (true,  location) ;
	KBCopyXML	*copyXML   = new KBCopyXML   (false, location) ;

	copyTable->setServer  (m_server) ;
	copyTable->setTable   (tabSpec.m_name) ;
	copyTable->setOption  (KBCopyTable::OptReplace, "") ;

	copyXML	 ->setMainTag (tabSpec.m_name) ;
	copyXML	 ->setRowTag  ("row") ;
	copyXML	 ->setErrOpt  (KBCopyXML::ErrPad) ;

	/* If dumping to a single file then we create a data element,	*/
	/* to which the copier will add the data. Otherwise, set the	*/
	/* required file name.						*/
	if (m_dumpSingle->isChecked())
	{
		QDomElement data = m_document.createElement ("data") ;
		data.setAttribute     ("name", tabSpec.m_name) ;
		m_element.appendChild (data) ;
		copyXML->setElement   (data) ;
	}
	else	copyXML->setFile      (m_dir + "/" + tabSpec.m_name + ".tabledata") ;


	for (uint idx = 0 ; idx < tabSpec.m_fldList.count() ; idx += 1)
	{
		KBFieldSpec *fSpec = tabSpec.m_fldList.at(idx) ;

		copyTable->addField (fSpec->m_name) ;
		copyXML  ->addField (fSpec->m_name, false) ;
	}

	/* Now create the actual copier and execute it. Most of the	*/
	/* arguments are dummy in this situation.			*/
	KBCopyExecDumper  copy   (copyTable, copyXML, this) ;
	QString	   	  report ;
	int	   	  nRows  ;
	QDict<QString>	  d1	 ;
	QDict<KBParamSet> d2	 ;
	if (!copy.execute
		(	report,
			pError,
			nRows,
			d1, d2,
			false
		))
	{
		return	false	;
	}

	return	true	;
}

/*  KBDumper								*/
/*  nowDumping	: Highlight item now being dumped			*/
/*  item	: KBDumperItem * : Dumper item in question		*/
/*  (returns)	: void		 :					*/

void	KBDumper::nowDumping
	(	KBDumperItem	*item
	)
{
	if (item->details() != 0)
		m_dumping->setText (item->details()->m_name) ;
	else	m_dumping->setText (item->name   ()) ;

	m_record->setText ("") ;
	m_of    ->setText
		(	QString(TR("%1 of %2"))
				.arg(m_index + 1)
				.arg(m_tableList->childCount())
		)	;

	m_tableList->ensureItemVisible (item) ;
	m_tableList->setCurrentItem    (item) ;
	qApp->processEvents() ;
}

/*  KBDumper								*/
/*  dumpDetails	: Dump database object details				*/
/*  item	: KBDumperItem * : Dumper item for database object	*/
/*  (returns)	: bool		 : Success				*/

bool	KBDumper::dumpDetails
	(	KBDumperItem	*item
	)
{
	KBTableDetails	*details = item->details () ;

	if (details->m_type == KB::IsTable)
	{
		KBTableSpec	tabSpec (details->m_name) ;
		KBError		error	;

		nowDumping (item) ;

		/* In the event of an error, display the error and	*/
		/* simulate the user clicking the cancel button, to	*/
		/* terminate the dialog.				*/
		if (!m_dbLink.listFields (tabSpec))
		{	m_dbLink.lastError().DISPLAY() ;
			return	false ;
		}

		qApp->processEvents() ;

		if (m_dumpDefn->isChecked())
			if (!dumpTableDef  (tabSpec, error))
			{	error.DISPLAY() ;
				return	false	;
			}

		qApp->processEvents() ;

		if (m_dumpData->isChecked())
			if (!dumpTableData (tabSpec, error))
			{	error.DISPLAY() ;
				return	false	;
			}

		qApp->processEvents() ;
		return	true	;
	}

	if (details->m_type == KB::IsView)
	{
		KBTableSpec	tabSpec (details->m_name) ;
		KBError		error	;

		if (!m_dumpDefn->isChecked()) return true  ;

		nowDumping (item) ;

		/* In the event of an error, display the error and	*/
		/* simulate the user clicking the cancel button, to	*/
		/* terminate the dialog.				*/
		if (!m_dbLink.listFields (tabSpec))
		{	m_dbLink.lastError().DISPLAY() ;
			return	false	;
		}

		qApp->processEvents() ;

		if (!dumpViewDef  (tabSpec, error))
		{	error.DISPLAY() ;
			return	false	;
		}

		qApp->processEvents() ;
		return	true	;
	}

	if (details->m_type == KB::IsSequence)
	{
		KBSequenceSpec	seqSpec (details->m_name) ;
		KBError		error	;

		if (!m_dumpDefn->isChecked()) return true  ;

		nowDumping (item) ;

		/* In the event of an error, display the error and	*/
		/* simulate the user clicking the cancel button, to	*/
		/* terminate the dialog.				*/
		if (!m_dbLink.descSequence (seqSpec))
		{	m_dbLink.lastError().DISPLAY() ;
			return	false	;
		}

		qApp->processEvents() ;

		if (!dumpSequenceDef  (seqSpec, error))
		{	error.DISPLAY() ;
			return	false	;
		}

		qApp->processEvents() ;
		return	true	;
	}

	return	true	;
}

/*  KBDumper								*/
/*  dumpObject	: Dump a file system object				*/
/*  item	: KBDumperItem * : Dumper item for database object	*/
/*  (returns)	: bool		  : Success				*/

bool	KBDumper::dumpObject
	(	KBDumperItem	*item
	)
{
	const QString	&name	 = item->name() ;
	QString		srceName = m_dbInfo->getDBPath() + "/" + name ;
	QString		destName = m_dir		 + "/" + name ;

	DPRINTF
	((	"KBDumper::dumpObject: [%d]: [%s]->[%s]\n",
		m_dumpSingle->isChecked(),
		(cchar *)srceName,
		(cchar *)destName
	))	;

	nowDumping (item) ;

	KBFile	srceFile (srceName) ;
	if (!srceFile.open(IO_ReadOnly))
	{
		srceFile.lastError().DISPLAY() ;
		return	false	;
	}

	QByteArray data	 = srceFile.readAll() ;

	if (m_dumpSingle->isChecked())
	{
		QDomElement objElem	= m_document.createElement ("object") ;
		objElem.setAttribute	("name", name) ;
		objElem.setAttribute	("type", item->type()) ;
		m_element.appendChild	(objElem) ;

		QDomText    objText	= m_document.createTextNode(data) ;
		objElem.appendChild	(objText) ;
		return	true	;
	}

	KBFile	destFile (destName) ;
	if (!destFile.open (IO_WriteOnly|IO_Truncate))
	{
		destFile.lastError().DISPLAY() ;
		return	false	;
	}

	destFile.writeBlock (data) ;
	return	true	;
}

/*  KBDumper								*/
/*  slotTimer	: Start dump of next object				*/
/*  (returns)	: void		:					*/

void	KBDumper::slotTimer ()
{
	/* Look for the next object to dump. The loop will exit if we	*/
	/* run out of objects before finding a candidate.		*/
	while (m_currItem != 0)
	{
		bool		pause	= false	;

		if (m_dumpAll->isChecked() || m_currItem->isOn())
		{
			bool	rc	;

			if (m_currItem->details() != 0)
				rc	= dumpDetails (m_currItem) ;
			else	rc	= dumpObject  (m_currItem) ;

			if (!rc)
			{
				reject	() ;
				return	;
			}

			pause	= true	;
		}

		m_currItem = (KBDumperItem *)m_currItem->nextSibling() ;
		m_index   += 1 ;

		if (pause)
		{
			QTimer::singleShot (200, this, SLOT(slotTimer())) ;
			qApp->processEvents() ;
			return	;
		}
	}

	if (m_dumpSingle->isChecked())
	{
		if (m_singleFile->text().isEmpty())
		{
			TKMessageBox::sorry
			(	0,
				TR("Please specify a single dump file name")
			)	;
			return	;
		}

		QString	eName	= m_dir + "/" + m_singleFile->text() + ".dbspec" ;
		KBFile	eFile	(eName) ;
		if (eFile.open (IO_WriteOnly|IO_Truncate))
			QTextStream(&eFile) << m_document.toString() ;
		else	eFile.lastError().DISPLAY() ;
	}

	/* All done. Change the cancel button to "OK" to indicate this	*/
	/* to the user, and set the finished flag. This leaves the	*/
	/* dialog just awaiting user confirmation before terminating.	*/
	m_bCancel->setEnabled (false) ;
	m_bOK    ->setEnabled (true ) ;
	m_finished = true ;
}

bool	KBDumper::showProgress
	(	int		nRows
	)
{
	if ((nRows % 10 == 0) || (time(0) >= m_ldtime + 2))
	{
		m_record->setText (QString::number(nRows)) ;
		m_ldtime = time   (0) ;
		qApp->processEvents() ;
	}

	return	m_cancelled ;
}

/*  dumpDatabase: Dump database to a set of files			*/
/*  dbInfo	: KBDBInfo *	  : Database information		*/
/*  server	: const QString & : Server name				*/
/*  dir		: const QString & : Destination directory		*/
/*  (returns)	: void		  :					*/

LIBKBASE_API	void	dumpDatabase
	(	KBDBInfo	*dbInfo,
		const QString	&server,
		const QString	&dir
	)
{
	KBDumper (dbInfo, server, dir).exec() ;
}
