/***************************************************************************
    file	         : kb_dumper.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qstring.h>

#include	"kb_dbinfo.h"
#include	"kb_dblink.h"
#include	"kb_dialog.h"
#include	"kb_dom.h"

#include	"rk_listview.h"

class	QCheckBox	;
class	RKListView	;
class	QLabel		;
class	RKLineEdit	;
class	RKPushButton	;


class	LIBKBASE_API KBDumperItem : public QCheckListItem
{
	KBTableDetails	*m_details	;
	QString		m_name		;
	QString		m_type		;

public	:

	inline	KBDumperItem
		(	RKListView	*parent,
			KBTableDetails	*details
		)
		:
		QCheckListItem
		(	parent,
			details->m_name,
			QCheckListItem::CheckBox
		),
		m_details	(details)
	{
		setText	(1, details->typeText()) ;
	}

	inline	KBDumperItem
		(	RKListView	*parent,
			const QString	&name,
			const QString	&type
		)
		:
		QCheckListItem
		(	parent,
			name,
			QCheckListItem::CheckBox
		),
		m_details	(0),
		m_name		(name),
		m_type		(type)
	{
		setText	(1, type) ;
	}

	inline	KBTableDetails	*details ()
	{
		return	m_details	;
	}
	inline	const QString	&name    ()
	{
		return	m_name		;
	}
	inline	const QString	&type    ()
	{
		return	m_type		;
	}
}	;


/*  KBDumper								*/
/*  --------								*/
/*  Database dumper class, used when bulk dumping table definitions	*/
/*  and data.								*/

class	LIBKBASE_API KBDumper : public KBDialog
{
	Q_OBJECT

	QCheckBox		*m_dumpAll	;
	QCheckBox		*m_dumpDefn	;
	QCheckBox		*m_dumpData	;
	QCheckBox		*m_dumpSingle	;
	RKLineEdit		*m_singleFile	;
	RKListView		*m_tableList	;

	QLabel			*m_dumping	;
	QLabel			*m_record	;
	QLabel			*m_of		;
	RKPushButton		*m_bOK		;
	RKPushButton		*m_bCancel	;

	KBDBInfo		*m_dbInfo	;
	QString			m_server	;
	QString			m_dir		;

	bool			m_cancelled	;
	bool			m_finished	;

	KBDBLink		m_dbLink	;
	KBTableDetailsList	m_tables 	;
	uint			m_index		;
	KBDumperItem		*m_currItem	;
	int			m_ldtime	;

	QDomDocument		m_document	;
	QDomElement		m_element	;

	void			nowDumping	(KBDumperItem	*) ;
	bool			addFileObjects	(cchar *, cchar *) ;

	bool			dumpTableDef	(KBTableSpec	&, KBError &) ;
	bool			dumpTableData	(KBTableSpec	&, KBError &) ;
	bool			dumpViewDef	(KBTableSpec	&, KBError &) ;
	bool			dumpSequenceDef	(KBSequenceSpec	&, KBError &) ;

	bool			dumpDetails	(KBDumperItem	*) ;
	bool			dumpObject	(KBDumperItem	*) ;

protected slots :

	void		slotSingleChange()	;
	void		slotTimer	()	;

	void		accept	 	()	;
	void		reject	 	()	;

public	:

	KBDumper (KBDBInfo *, const QString &, const QString &) ;

	int		exec		()	;
	bool		showProgress	(int)	;
}	;
