/***************************************************************************
    file	         : kb_errordlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdlib.h>

#include	<qapplication.h>
#include	<qlabel.h>
#include	<qtextview.h>
#include	<qregexp.h>
#include	<qcombobox.h>


#include	"tk_icons.h"
#include	"rk_hbox.h"
#include	"rk_vbox.h"
#include	"rk_pushbutton.h"

#include	"kb_classes.h"
#include	"kb_script.h"

#if		__KB_AUTOMOC
#include	"kb_errordlg.moc"
#else
#include	"kb_errordlg.h"
#endif



/*  KBErrorDlg								*/
/*  KBErrorDlg	: Constructor for error report dialog			*/
/*  caption	: const QString &: Optional caption			*/
/*  error	: const KBError &: Error to display			*/
/*  dfile	: cchar *	 : Display source file			*/
/*  dlno	: uint		 : Display line number			*/
/*  (returns)	: KBErrorDlg	 :					*/

KBErrorDlg::KBErrorDlg
	(	const QString	&caption,
		const KBError	&error,
		cchar		*dFile,
		uint		dLno
	)
	:
	KBDialog	(QString::null, true),
	m_error		(error),
	m_dFile		(dFile),
	m_dLno		(dLno)
{
	const   KBErrorInfo &ei	= m_error.m_errors[0]	;

	setIcon (getSmallIcon ("rekall")) ;




	m_layout = new RKVBox (this) ;
	m_layout->setTracking () ;

	RKHBox	*layInfo	= new RKHBox (m_layout) ;
	RKHBox	*layButt	= new RKHBox (m_layout) ;

	/* Display icon based on the worst error type amongst	*/
	/* logged errors, and check whether any error has	*/
	/* details set.						*/
	int	maxerr	= 0 		;
	cchar	*icon	= "warning"	;
	bool	anydet	= false		;

	for (uint idx = 0 ; idx < m_error.m_errors.count() ; idx += 1)
	{
		if ((int)m_error.m_errors[idx].m_etype > maxerr)
			maxerr = (int)m_error.m_errors[idx].m_etype ;

		if (!m_error.m_errors[idx].m_details.isEmpty())
			anydet	= true	;
	}

	switch (ei.m_etype)
	{
		case KBError::Info	: icon	= "note"      ; break ;
		case KBError::Warning	: icon	= "caution"   ; break ;
		case KBError::Error	: icon	= "important" ; break ;
		case KBError::Fault	: icon	= "warning"   ; break ;
		default			: break	;
	}

	QLabel	*graphic	= new QLabel (layInfo ) ;
	graphic->setPixmap	(getDesktopIcon (icon)) ;

	if (m_error.m_errors.count() > 1)
	{
		m_cbWhich	= new QComboBox (layInfo) ;
		for (uint idx = 0 ; idx < m_error.m_errors.count() ; idx += 1)
			m_cbWhich->insertItem (m_error.m_errors[idx].m_message) ;

		connect
		(	m_cbWhich,
			SIGNAL	(activated	(int)),
			SLOT	(slotShowError	(int))
		)	;
	}
	else
	{
		QLabel	*what		= new QLabel (layInfo ) ;
//		what   ->setTextFormat	(Qt::RichText) ;
		what   ->setText	(ei.m_message) ;
		what   ->setMinimumWidth(300) ;
		m_cbWhich = 0	;
	}

	layInfo->addFiller	() ;

	RKPushButton *pb ;
	pb	= new RKPushButton (layButt) ;
	pb->setText (TR("OK")) ;
	connect	(pb, SIGNAL(clicked()), SLOT(accept())) ;

	if (anydet || ((dFile != 0) && (KBError::errDebug > 0)))
	{
		m_bDetails= new RKPushButton (TR("Show Details >>"), layButt) ;
		m_bDetails->setToggleButton (true) ;
		connect
		(	m_bDetails,
			SIGNAL	(toggled	(bool)),
			SLOT	(slotShowDetails(bool))
		)	;
	}

	layButt->addFiller () ;

	if (caption.isEmpty())
		setCaption
		(
			ei.m_etype == KBError::Info    ? "Information"		:
			ei.m_etype == KBError::Warning ? "Warning"	    	:
			ei.m_etype == KBError::Error   ? "Error"		:
			ei.m_etype == KBError::Fault   ? "Internal error"	:
							 "Unknown error"
		)	;
	else	setCaption (caption) ;

	m_tDetails = 0	;

	setFixedSize	(sizeHint()) ;
}

/*  KBErrorDlg								*/
/*  ~KBErrorDlg	: Destructor for error report dialog			*/
/*  (returns)	:		:					*/

KBErrorDlg::~KBErrorDlg ()
{
}

/*  KBErrorDlg								*/
/*  slotShowError: Show particular error				*/
/*  index	 : int		: Error index				*/
/*  (returns)	 : void		:					*/

void	KBErrorDlg::slotShowError
	(	int		index
	)
{
	if (m_tDetails != 0)
	{
		const   KBErrorInfo &ei	= m_error.m_errors[index] ;

		QString	text	= ei.m_details	;
		QString	srceMsg	;
		QString	destMsg	;

		if (text[0] != '<')
		{
			text.replace(QRegExp("<" ), "&lt;" ) ;
			text.replace(QRegExp(">" ), "&gt;" ) ;

			if (text.isEmpty())
				text	= "<i>" + TR("(No details)") + "</i>" ;

			text.replace(QRegExp("\n"), "<br/>") ;
		}

		cchar	*sFile		= ei.m_file	;
		uint	sLno		= ei.m_lno	;
		if (  sFile != 0) srceMsg = QString("%1:%2").arg(  sFile).arg(  sLno) ;
		if (m_dFile != 0) destMsg = QString("%1:%2").arg(m_dFile).arg(m_dLno) ;

		if ((KBError::errDebug != 0) && !destMsg.isEmpty())
			text	+=
				"<table>"
				"<tr><td>Source </td><td>" + srceMsg + "</td></tr>"
				"<tr><td>Display</td><td>" + destMsg + "</td></tr>"
				"</table>"
				;

		m_tDetails->setText	  ("<qt>" + text + "</qt>") ;
		m_tDetails->show	  () ;
		m_bDetails->setText	  (TR("<< Hide details")) ;

		QApplication::sendPostedEvents () ;
		resize		(sizeHint()) ;
		setMinimumSize	(sizeHint()) ;
		setMaximumSize	(QSize(1000,400)) ;
	}
}

/*  KBErrorDlg								*/
/*  slotShowdetails							*/
/*		: Toggle details view					*/
/*		: bool		: On					*/
/*  (returns)	: void		:					*/

void	KBErrorDlg::slotShowDetails
	(	bool		
	)
{
	if (m_tDetails == 0)
	{
		m_tDetails = new QTextView(m_layout) ;
		m_tDetails->setTextFormat (Qt::RichText) ;

		if (m_cbWhich == 0)
			slotShowError	(0) ;
		else	slotShowError	(m_cbWhich->currentItem()) ;

		return	;
	}

	DELOBJ		    (m_tDetails) ;
	m_bDetails->setText (TR("Show details >>")) ;

	/* Resize back to the fized size. Process events first so that	*/
	/* child removal is processed.					*/
	QApplication::sendPostedEvents () ;
	setFixedSize	(sizeHint()) ;
}

/*  KBErrorDlg								*/
/*  exec	: Execute dialog					*/
/*  (returns)	: int		: Dialog completion code		*/

int	KBErrorDlg::exec ()
{
	if (m_error.m_errors[0].m_etype == KBError::None)
		if (getenv ("REKALL_ALLERRORS") == 0)
			return	1 ;

	return	KBDialog::exec ()  ;
}

/*  ------------------------------------------------------------------	*/

static	KBErrorBlock	*currentBlock	;

/*  KBErrorBlock							*/
/*  KBErrorBlock: Constructor for error blocker wrapper 		*/
/*  block	: EBlock	: Blocking mode				*/
/*  (returns)	: KBErrorBlock	:					*/

KBErrorBlock::KBErrorBlock
	(	EBlock		block
	)
	:
	KBErrorBlockBase (block)
{
	/* In the wrapper we maintain a pointer to a script error that	*/
	/* is yet to be processed, and out own error blocker chain.	*/
	m_scriptError	= 0		;
	m_prior		= currentBlock	;
	currentBlock	= this		;
}

/*  KBErrorBlock							*/
/*  ~KBErrorBlock: Destructor for error blocker wrapper 		*/
/*  (returns)	 :		:					*/

KBErrorBlock::~KBErrorBlock ()
{
	/* The error dialog should appear *before* any script error is	*/
	/* handled, so call the "display" method here. When it is	*/
	/* later called in the "KBErrorBlockBase" destructor it will be	*/
	/* a null operation.						*/
	display	() ;

	if (m_scriptError != 0)
	{
		m_scriptError->processError () ;
		delete m_scriptError	;
	}

	currentBlock	= m_prior	;
}

/*  KBErrorBlock							*/
/*  processError: See wether a script error should be processed now	*/
/*  scriptError	: KBScriptError * : Error in question			*/
/*  (returns)	: bool		  : True to process now			*/

bool	KBErrorBlock::processError
	(	KBScriptError	*scriptError
	)
{
	/* If there is an extant error blocker then queue the script	*/
	/* error on it to be processed when exiting from the blocker.	*/
	if (currentBlock != 0)
	{
		if (currentBlock->m_scriptError != 0)
		{
			fprintf
			(	stderr,
				"KBErrorBlock::processError: *** MULTIPLE ***\n"
			)	;
			return	false	;
		}

		currentBlock->m_scriptError = scriptError ;
		return	false	;
	}

	return	true	;
}
