/***************************************************************************
    file	         : kb_event.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qregexp.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_database.h"
#include	"kb_location.h"
#include	"kb_attr.h"
#include	"kb_script.h"
#include	"kb_object.h"
#include	"kb_event.h"
#include	"kb_callback.h"
#include	"kb_location.h"
#include	"kb_callback.h"
#include	"kb_appptr.h"
#include	"kb_docroot.h"
#include	"kb_eventdlg.h"
#include	"kb_emitter.h"
#include	"kb_macro.h"

#define		EFLAGS		(KAF_GRPEVENT|KAF_EDITOR|KAF_CLEAR|KAF_CUSTOM)

#define		__DEBUG_EVENT	(0)
#if		__DEBUG_EVENT
#include	"kb_dprintf.h"
#endif


/*  l2Warning	: Check on spurious second language			*/
/*  event	: KBEvent *	: Event to check			*/
/*  (returns)	: void		:					*/

static	void	l2Warning
	(	KBEvent		*event
	)
{
	/* HACK ALERT:							*/
	/* Warn if there is second language code and this is not a	*/
	/* second language (client-side) event. We use this check since	*/
	/* the KAF_EVCS was introduced later than the second language	*/
	/* support and I want to be warned before losing such code (on	*/
	/* document save) simply because I've missed an KAF_EVCS flag.	*/
	if (!event->getValue2().isEmpty())
		if ((event->getFlags() & KAF_EVCS) == 0)
			KBError::EError
			(	TR("Second language code in non-client-side event"),
				QString("%1: %2.%3")
					.arg(event->getOwner()->getElement())
					.arg(event->getOwnerName())
					.arg(event->getName()),
				__ERRLOCN
			)	;
}

static	QString	baseName
	(	KBNode		*node
	)
{
	if (node == 0) return QString::null ;
	return	QString("on%1").arg(&node->className()[2]) ;
}

/*  ------------------------------------------------------------------	*/

/*  KBEvent								*/
/*  KBEvent	: Constructor for event class				*/
/*  node	: KBNode *	: Owning node				*/
/*  name	: cchar *	: Attribute name			*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  nflags	: uint		: Required node flags			*/
/*  (returns)	: KBEvent	:					*/

KBEvent::KBEvent
	(	KBNode			*node,
		cchar			*name,
		const QDict<QString>	&aList,
		uint			nFlags
	)
	:
	KBAttrStr (node, name, aList, nFlags | EFLAGS),
	m_base	  (baseName(node))
{
	init	  () ;

	QString	  *l2 = aList.find (QString("%1_l2").arg(name)) ;
	m_value2  =  l2 == 0 ? QString::null : *l2 ;

	QString	*bpt = aList.find (QString("%1_bpt").arg(name)) ;
	if (bpt != 0)
	{	QStringList l = QStringList::split (',', *bpt) ;
		for (uint idx = 0 ; idx < l.count() ; idx += 1)
			m_breakpoints.append (l[idx].toInt()) ;
	}

	DPRINTF
	((	"KBEvent::KBEvent: [%s][%s]\n",
		name,
		m_base.latin1()
	))	;

	l2Warning (this) ;
}

/*  KBEvent								*/
/*  KBEvent	: Constructor for event class				*/
/*  node	: KBNode *	: Owning node				*/
/*  name	: cchar *	: Attribute name			*/
/*  text	: cchar *	: Initial text value			*/
/*  nflags	: uint		: Required node flags			*/
/*  (returns)	: KBEvent	:					*/

KBEvent::KBEvent
	(	KBNode	*node,
		cchar	*name,
		cchar	*text,
		uint	nFlags
	)
	:
	KBAttrStr (node, name, text, nFlags | EFLAGS),
	m_base	  (baseName(node))

{
	init	  () ;
	l2Warning (this) ;
}

/*  KBEvent								*/
/*  KBEvent	: Constructor for event class				*/
/*  node	: KBNode *	: Owning node				*/
/*  name	: cchar *	: Attribute name			*/
/*  srce	: KBNode *	: Extant source				*/
/*  nflags	: uint		: Required node flags			*/
/*  (returns)	: KBEvent	:					*/

KBEvent::KBEvent
	(	KBNode	*node,
		cchar	*name,
		KBNode	*srce,
		uint	nFlags
	)
	:
	KBAttrStr (node, name, srce, nFlags | EFLAGS),
	m_base	  (baseName(node))
{
	init	  () ;

	/* See if we have an extant event and if such an event has a	*/
	/* macro, in which case replicate the macro.			*/
	KBAttr	*attr	= srce->getAttr(name) ;
	if (attr  == 0) return ;

	KBEvent	*event	= attr->isEvent() ;
	if (event == 0) return ;

	m_value2	= event->m_value2 ;
	m_comment	= event->m_comment;
	m_macro		= 0 ;

	KBMacroExec *macro = event->getMacro() ;
	if (macro != 0)
	{	m_macro = new KBMacroExec (macro) ;
		m_macro->setName    (event->getName()) ;
		m_macro->setComment (event->comment()) ;
	}

	l2Warning (this) ;
}

/*  KBEvent								*/
/*  ~KBEvent	: Destructor for event class				*/
/*  (returns)	:		:					*/

KBEvent::~KBEvent ()
{
	clearOverride () ;

	if (m_code    != 0) delete m_code    ;
	if (m_code2   != 0) delete m_code2   ;
	if (m_emitter != 0) delete m_emitter ;
	if (m_macro   != 0) delete m_macro   ;
}

/*  KBEvent								*/
/*  init	: Common initialisation					*/
/*  (returns)	: void		  :					*/

void	KBEvent::init ()
{
	m_disable	= false	;
	m_code		= 0	;
	m_code2		= 0	;
	m_emitter	= 0	;
#if	__NOTDEF
	m_cppFunc	= 0	;
#endif
	m_macro		= 0	;
	m_override	= 0	;
	m_inherit	= 0	;
}

/*  KBEvent								*/
/*  clearOverride:							*/
/*  (returns)	 : void		  :					*/

void	KBEvent::clearOverride ()
{
	/* Clear the override/inherit chain. Note that such a chain	*/
	/* ends up back at this event so careful with the loop		*/
	/* termiantion.							*/
	/* NOTE: The m_override pointer is only set in the original	*/
	/*	 event so we don't need to worry about enexpected	*/
	/*	 recursion.						*/
	KBEvent	*over	;
	KBEvent	*next	;

	if ((over = m_override) != 0)
		while ((over != 0) && (over != this))
		{	next	= over->m_inherit ;
			delete	over	;
			over	= next	;
		}

	m_override = 0 ;
	m_inherit  = 0 ;
}

/*  KBEvent								*/
/*  showAs	: Set display mode					*/
/*  showAs	: KB::ShowAs	: New mode				*/
/*  (returns)	: bool		: Value has been changed		*/

bool	KBEvent::showAs
	(	KB::ShowAs	showAs
	)
{
	/* Always clear the override/inherit chain. This only matters	*/
	/* if the user goes data->design->data but just to it on all	*/
	/* changes.							*/
	if (showAs != m_showing) clearOverride () ;


	/* Clear code from previous execution, needed in case the	*/
	/* compiled code contains per-execution state (eg., the "root"	*/
	/* symbol in python).						*/
	DELOBJ	(m_code ) ;
	DELOBJ	(m_code2) ;


	return	KBAttrStr::showAs (showAs) ;
}

/*  KBEvent								*/
/*  setOverride	: Set override event function				*/
/*  over	: const QString & : Override function			*/
/*  (returns)	: void		  :					*/

void	KBEvent::setOverride
	(	const QString	&over
	)
{
	/* Create a new event attribute object, set the override and	*/
	/* link into this event. The new evet is chained such that	*/
	/* if goes to the head of a list which will chain back to this	*/
	/* event (which is effectively the base method).		*/
	QDict<QString> dummy ;

	KBEvent	*oe = new KBEvent
			  (	m_owner,
				m_name,
				dummy,
				m_nFlags|EFLAGS
			  )	;

	oe->setValue  (over)	;
	oe->m_inherit = m_override == 0 ? this : m_override ;
	m_override    = oe	;
}

/*  KBEvent								*/
/*  setValue	: Set event text value					*/
/*  value	: const QString & : Value				*/
/*  (returns)	: void		  :					*/

void	KBEvent::setValue
	(	const QString	&value
	)
{
	/* This method is overridden so that we can clear any code	*/
	/* compiled for an earlier value, and clear the disabled flag	*/
	DELOBJ	(m_code)  ;
	m_disable = false ;

	KBAttrStr::setValue (value) ;
}


/*  KBEvent								*/
/*  setValue	: Set value from attribute item				*/
/*  item	: KBAttrItem *	: Item					*/
/*  (returns)	: void		:					*/

void	KBEvent::setValue
	(	KBAttrItem	*item
	)
{
#if	! __KB_RUNTIME
	setValue (item->value()) ;
#endif
}

/*  KBEvent								*/
/*  setValue2	: Set second language value				*/
/*  value2	: const QString & : Second language value		*/
/*  (returns)	: void		  :					*/

void	KBEvent::setValue2
	(	const QString	&value2
	)
{
	DELOBJ	(m_code2) ;
	m_value2 = value2 ;
	l2Warning (this)  ;
}

/*  KBEvent								*/
/*  getValue2	: Get second language value				*/
/*  (returns)	: QString	: Second language value			*/

QString	KBEvent::getValue2 ()
{
	return	m_value2 ;
}


#if	__NOTDEF

/*  KBEvent								*/
/*  setCPPFunc	: Set C++ function					*/
/*  cppFunc	: CPPFunc	: C++ function				*/
/*  (returns)	: void		:					*/

void	KBEvent::setCPPFunc
	(	CPPFunc		cppFunc
	)
{
	m_disable = false 	;
	m_cppFunc = cppFunc	;
}

#endif

/*  KBEvent								*/
/*  isEmpty	: Check if event is empty				*/
/*  (returns)	: bool		: Empty					*/

bool	KBEvent::isEmpty ()
{
	if (m_macro	   != 0) return false ;
#if	__NOTDEF
	if (m_cppFunc	   != 0) return false ;
#endif
	if (!m_value2.isEmpty()) return false ;

	return	getValue().isEmpty () ;
}

/*  KBEvent								*/
/*  execCode	: Execute script code					*/
/*  scrIface	: KBScriptIF *	  : Scripting interface			*/
/*  code	: KBScriptCode *  : Precompiled script code		*/
/*  event	: const QString & : Event code				*/
/*  resval	: KBValue &	  : Return value 			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: KBValue *	  : Argument vector			*/
/*  (returns)	: KBScriptError * : Error or null if ok			*/

KBScriptError
	*KBEvent::execCode
	(	KBScriptIF	*scrIface,
		KBScriptCode	*&code,
		const QString	&event,
		KBValue		&resval,
		uint		argc,
		KBValue		*argv
	)
{
	/* Handle explicit events, where we compile the code and keep a	*/
	/* copy. First see if the code has already been compiled and	*/
	/* if not then so so now.					*/
	if (code == 0)
	{
		KBError	error	;
		QString	ident	= QString("%1.%2")
					.arg(m_owner->getPath())
					.arg(getName())
					;
		DPRINTF
		((	"KBEvent::execCode: compiling code .......\n"
		))	;

		if ((code = scrIface->compileFunc
			    (	m_owner,
				event,
				ident,
				"eventFunc",
				m_owner->getDocRoot()->getImports(),
				m_inherit,
				error
			    )	) == 0)
		{
			/* The event is disabled, and we return false	*/
			/* so that the caller will bring up the		*/
			/* required property dialog.			*/
			m_disable = true ;
			return	new KBScriptError (error, this) ;
		}

		code->setBreakpoints (m_breakpoints) ;
	}

	/* Code was or has been compiled successfully, in which case	*/
	/* execute it. Check for an execution time error ....		*/
	KBScript::ExeRC rc = code->execute (argc, argv, resval) ;

	switch (rc)
	{
		case KBScript::ExeAbort	:
		case KBScript::ExeTest	:
			return	new KBScriptError () ;

		case KBScript::ExeError	:
		case KBScript::ExeFail	:
			{
			QString	   errMsg   ;
			uint	   errLno   ;
			QString	   errText  ;
			KBLocation errLocn  = scrIface->lastError (errMsg, errLno, errText) ;

			m_disable = true    ;

			DPRINTF
			((	"KBEvent::execCode: locn=[%s] ident=[%s]\n",
				errLocn.name ().latin1(),
				errLocn.ident().latin1()
			))	;

			Origin	   origin   = errorOrigin
						(	m_owner->getRoot(),
							errLocn
						)	;

			if (origin == KBEvent::OriginScript)
				return	new KBScriptError
					(	KBError
						(	KBError::Error,
							errMsg,
							errText,
							__ERRLOCN
						),
						m_owner->isObject(),
						errLocn,
						errText,
						errLno,
						rc == KBScript::ExeFail
			  		)	;

			if (origin == KBEvent::OriginLocal )
				return	new KBScriptError
					(	KBError
						(	KBError::Error,
							errMsg,
							errText,
							__ERRLOCN
						),
						m_owner->getRoot()->getAttr("local")->isEvent()
					)	;

			return	new KBScriptError
				(	KBError
					(	KBError::Error,
						errMsg,
						errText,
						__ERRLOCN
					),
					this
				)	;
			}

		default	:
			break	;
	}

	/* Return true; success here means that there were no execution	*/
	/* errors, not that the event script itself returned true.	*/
	return	0 ;
}

/*  KBEvent								*/
/*  doExecuteL1	: Execute event - first language			*/
/*  resval	: KBValue &	  : Return value 			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: KBValue *	  : Argument vector			*/
/*  defval	: bool		  : Default execution return code	*/
/*  (returns)	: KBScriptError * : Error or null if OK			*/

KBScriptError
	*KBEvent::doExecuteL1
	(	KBValue		&resval,
		uint		argc,
		KBValue		*argv,
		bool		defval
	)
{
	DPRINTF
	((	"KBEvent::doExecuteL1: [%s] e=[%p] m=[%p] [%s]\n",
		(cchar *)m_name,
		(void  *)m_emitter,
		(void  *)m_macro,
		(cchar *)getValue()
	))	;


	/* SIGNAL CODE							*/
	/* -----------							*/
	/* If this event has been wired up to any slots then signal	*/
	/* these first.							*/
	if (m_emitter != 0)
	{
		KBScriptError *rc = m_emitter->doSignal (argc, argv, 1) ;
		if (rc != 0) return rc ;
	}


	/* MACRO							*/
	/* -----							*/
	/* Event may be a macro, in which case all the work is done	*/
	/* inside the macro itself.					*/
	if (m_macro != 0)
	{
		KBScriptError *rc = m_macro->execute (m_owner->getRoot()) ;
		if (rc != 0) return rc ;

		if (defval)
			resval.setTrue	() ;
		else	resval.setFalse	() ;

		return	0 ;
	}


#if	__NOTDEF
	/* EVENT CODE							*/
	/* ----------							*/
	/* C++-level function. Used internally, should really never	*/
	/* fail!							*/
	if (m_cppFunc != 0)
	{
		KBError		error	;
		KBScript::ExeRC rc	= (*m_cppFunc) (this, resval, argc, argv, error) ;

		switch (rc)
		{
			case KBScript::ExeAbort	:
			case KBScript::ExeTest	:
				return	new KBScriptError () ;

			case KBScript::ExeError	:
			case KBScript::ExeFail	:
				return	new KBScriptError (error) ;

			default	:
				break	;
		}

		return	0 ;
	}
#endif

	/* Next case, there is no text associated with the event, in	*/
	/* which case do nothing. This is considered successful.	*/
	QString	 event	= getValue () ;
	if (event.isEmpty())
	{
		if (defval)
			resval.setTrue	() ;
		else	resval.setFalse	() ;

		return	0 ;
	}

	/* Set the result value false until proven otherwise. This is	*/
	/* the catch-all case.						*/
	resval.setFalse ()   ;

	/* A script interface is needed to be able to execute the event	*/
	/* whether the code is implicit or explicit ....		*/
	KBScriptError	*error	  ;
	KBDocRoot	*root	  = m_owner->getDocRoot   () ;
	KBScriptIF	*scrIface = root  ->loadScripting (error) ;

	if (scrIface == 0)
		return	error	;

	/* SHORTCUT							*/
	/* --------							*/
	/* If the event is marked with #[A-Za-z]* then this is an	*/
	/* implicit event, and we execute a routine named for the	*/
	/* event base (eg., "onClick" for a button)  and the reset of	*/
	/* the event text.						*/
	if ((event.at(0) == '#') && event.at(1).isLetter())
	{
		QString		name	= m_base + event.mid(1).stripWhiteSpace() ;
		KBScript::ExeRC	rc 	  ;

		rc	= scrIface->execute
			  (	m_owner->getDocRoot()->getScripts(),
				name,
				m_owner,
				argc,
				argv,
				resval
			  )	;

		switch (rc)
		{
			case KBScript::ExeAbort	:
			case KBScript::ExeTest	:
				return	new KBScriptError () ;

			case KBScript::ExeError	:
			case KBScript::ExeFail	:
				{
				/* Ho-hum, got an error which		*/
				/* executing the code.			*/
				QString	   errMsg   ;
				uint	   errLno   ;
				QString	   errText  ;
				KBLocation errLocn  = scrIface->lastError (errMsg, errLno, errText) ;

				m_disable = true    ;

				if (errLocn.name() != KBLocation::m_pInline)
					return	new KBScriptError
						(	KBError
							(	KBError::Error,
								errMsg,
								errText,
								__ERRLOCN
							),
							m_owner->isObject(),
							errLocn,
							errText,
							errLno,
							rc == KBScript::ExeFail
				  		)	;

				return	new KBScriptError
					(	KBError
						(	KBError::Error,
							errMsg,
							errText,
							__ERRLOCN
						),
						this
					)	;
				}

			default	:
				break	;
		}

		return	0 ;
	}


	/* EVENT CODE							*/
	/* ----------							*/
	/* This is the real thing!!!					*/
	return	execCode (scrIface, m_code, event, resval, argc, argv) ;
}

/*  KBEvent								*/
/*  doExecuteL2	: Execute event - second language			*/
/*  resval	: KBValue &	  : Return value 			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: KBValue *	  : Argument vector			*/
/*  defval	: bool		  : Default execution return code	*/
/*  (returns)	: KBScriptError * : Error or null if OK			*/

KBScriptError
	*KBEvent::doExecuteL2
	(	KBValue		&resval,
		uint		argc,
		KBValue		*argv,
		bool		defval
	)
{
	DPRINTF
	((	"KBEvent::doExecuteL2: [%s] e=[%p] m=[%p] [%s]\n",
		(cchar *)m_name,
		(void  *)m_emitter,
		(void  *)m_macro,
		(cchar *)m_value2
	))	;

	/* If there is no second language then set the result to the	*/
	/* default and return at once. We cannot execute any second	*/
	/* langauge events or slots in this case.			*/
	if (m_owner->getRoot()->getAttrVal("language2").isEmpty())
	{
		if (defval)
			resval.setTrue	() ;
		else	resval.setFalse	() ;

		return	0 ;
	}

	/* Similarly if this is not a client side (second language)	*/
	/* event ...							*/
	if ((m_nFlags & KAF_EVCS) == 0)
	{
		if (defval)
			resval.setTrue	() ;
		else	resval.setFalse	() ;

		return	0 ;
	}

	/* SIGNAL CODE							*/
	/* -----------							*/
	/* If this event has been wired up to any slots then signal	*/
	/* these first.							*/
	if (m_emitter != 0)
	{
		KBScriptError *rc = m_emitter->doSignal (argc, argv, 2) ;
		if (rc != 0) return rc ;
	}

	/* If there is no second language event code then wind up here	*/
	/* with the default result and no error.			*/
	if (m_value2.isEmpty())
	{
		if (defval)
			resval.setTrue	() ;
		else	resval.setFalse	() ;

		return	0 ;
	}

	/* A script interface is needed to be able to execute the event	*/
	/* code ...							*/
	KBScriptError	*error	  ;
	KBDocRoot	*root	  = m_owner->getDocRoot   () ;
	KBScriptIF	*scrIface = root  ->loadScripting2(error) ;

	if (scrIface == 0)
		return	error	;

	/* EVENT CODE							*/
	/* ----------							*/
	/* This is the real thing!!!					*/
	return	execCode (scrIface, m_code2, m_value2, resval, argc, argv) ;
}

/*  KBEvent								*/
/*  doExecute	: Execute event						*/
/*  resval	: KBValue &	  : Return value 			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: KBValue *	  : Argument vector			*/
/*  defval	: bool		  : Default execution return code	*/
/*  (returns)	: KBScriptError * : Error or null if OK			*/

KBScriptError
	*KBEvent::doExecute
	(	KBValue		&resval,
		uint		argc,
		KBValue		*argv,
		bool		defval
	)
{
	DPRINTF
	((	"KBEvent::doExecute: [%s] e=[%p] m=[%p]\n",
		(cchar *)m_name,
		(void  *)m_emitter,
		(void  *)m_macro
	))	;

	/* Execute any second-language code. If this throws an error or	*/
	/* returns a non-true result then stop execution, passing back	*/
	/* the script error if any.					*/
	KBScriptError *rc  = doExecuteL2 (resval, argc, argv, true) ;
	if ((rc != 0) || !resval.isTrue())
		return	rc ;

	return	doExecuteL1 (resval, argc, argv, defval) ;
}

/*  KBEvent								*/
/*  execute	: Execute event						*/
/*  resval	: KBValue &	  : Return value 			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: KBValue *	  : Argument vector			*/
/*  defval	: bool		  : Default execution return code	*/
/*  (returns)	: KBScriptError * : Error or null if OK			*/

KBScriptError
	*KBEvent::execute
	(	KBValue		&resval,
		uint		argc,
		KBValue		*argv,
		bool		defval
	)
{
	DPRINTF
	((	"KBEvent::execute: [%p] [%s] o=[%p] i=[%p] e=[%p]\n",
		(void  *)this,
		(cchar *)m_name,
		(void  *)m_override,
		(void  *)m_inherit,
		(void  *)m_emitter
	))	;

	KBError		error	;
	KBScriptError	*rc	;
	KBCallback	*cb 	= KBAppPtr::getCallback() ;

	void		*logid	= cb == 0 ?
					0 :
					cb->logEvent
					(	"Event",
						m_ownerName,
						m_owner->getAttrVal("name"),
						m_name,
						argc,
						argv
					)	;

	/* If the event is flagged as disabled then there has been an	*/
	/* earlier error ...						*/
	if (m_disable)
		return	new KBScriptError
			(	KBError
				(	KBError::Error,
					TR("Event has been disabled due to earlier error"),
					QString::null,
					__ERRLOCN
				),
				this
			)	;

	/* If the override pointer is set then there is a chain of	*/
	/* overriding (and inheriting) events starting here and leading	*/
	/* back to this event, so start following the chain.		*/
	if (m_override != 0)
		rc	= m_override->doExecute (resval, argc, argv, defval) ;
	else	rc	= doExecute		(resval, argc, argv, defval) ;

	if (cb != 0)
	{
		if (rc == 0)
			cb->logEventResult (resval, true, logid) ;
		else	cb->logEventResult (KBValue(rc->error().getMessage()), true, logid) ;
	}

	DPRINTF
	((	"KBEvent::execute: returns [%d]\n",
		rc == 0
	))	;

	return	rc	;
}

/*  KBEvent								*/
/*  getDescription							*/
/*		: Get description for property dialogs			*/
/*  (returns)	: QString	: Legend				*/


QString	KBEvent::getDescription () const
{
	QString	descr = KBAttr::getDescription () ;
	if (descr.find ('%') >= 0) descr = descr.arg(m_base) ;
	return	descr ;
}

/*  KBEvent								*/
/*  getEmitter	: Get emitter object					*/
/*  (returns)	: KBEmitter *	: Emitter				*/

KBEmitter
	*KBEvent::getEmitter ()
{
	/* The emitter object is created as needed, so that we do not	*/
	/* waste space by needing KBEvent to embed or be derived from	*/
	/* QObject.							*/
	if (m_emitter == 0)
		if (m_owner->isObject() != 0)
		{
			m_emitter = new KBEmitter (m_owner->isObject(), this) ;

			DPRINTF
			((	"KBEvent::getEmitter: event=[%p] emit=[%p]\n",
				(void *)this,
				(void *)m_emitter
			))	;
		}

	return	m_emitter	;
}

/*  KBEvent								*/
/*  clearEmitter: Clear emitter object if any				*/
/*  (returns)	: void		:					*/

void	KBEvent::clearEmitter ()
{
	/* Having a dynamically allocated emitter also gives an easy	*/
	/* way to clear down all signals ...				*/
	DELOBJ	(m_emitter) ;
}


/*  KBEvent								*/
/*  isEvent	: Checkes whether attribute is an event			*/
/*  (returns)	: KBEvent *	: Always the object			*/

KBEvent	*KBEvent::isEvent ()
{
	return	this ;
}

/*  KBEvent								*/
/*  setCode	: Set code associated with event			*/
/*  code	: const QString & : Code				*/
/*  append	: bool		  : Append code else replace		*/
/*  (returns)	: void		  :					*/

void	KBEvent::setCode
	(	const QString	&code,
		bool		append
	)
{
	if (append)
	{
		setValue (getValue() + code) ;
		return	;
	}

	setValue (code) ;
}

/*  KBEvent								*/
/*  setCode2	: Set second code associated with event			*/
/*  code	: const QString & : Code				*/
/*  append	: bool		  : Append code else replace		*/
/*  (returns)	: void		  :					*/

void	KBEvent::setCode2
	(	const QString	&code,
		bool		append
	)
{
	if (append)
	{
		setValue2 (getValue2() + code) ;
		return	;
	}

	setValue2 (code) ;
}

/*  KBEvent								*/
/*  tidy	: Tidy up after loading					*/
/*  (returns)	: void		:					*/

void	KBEvent::tidy ()
{
	/* This is just so we can output the code with a leading 	*/
	/* newline in the XML document.					*/
	QString	v  = getValue ().stripWhiteSpace() ;
	if (!v .isEmpty()) v  += "\n" ;
	setValue  (v ) ;

	QString	v2 = getValue2().stripWhiteSpace() ;
	if (!v2.isEmpty()) v2 += "\n" ;
	setValue2 (v2) ;
}

/*  KBEvent								*/
/*  setMacro	: Set a macro						*/
/*  macro	: KBMacro *	: Macro in question			*/
/*  (returns)	: void		:					*/

void	KBEvent::setMacro
	(	KBMacroExec	*macro
	)
{
	if (m_macro != 0)
		delete m_macro ;

	m_macro	= macro	;

	if (m_macro != 0)
	{	m_macro->setName    (getName()) ;
		m_macro->setComment (comment()) ;
	}
}

/*  KBEvent								*/
/*  getMacro	: Get macro if any					*/
/*  (returns)	: KBMacro *	: Macro or null of none			*/

KBMacroExec *KBEvent::getMacro ()
{
	return	m_macro	;
}

/*  KBEvent								*/
/*  printAttr	: Print attribute					*/
/*  attrText	: QString &	: Result text for attribute output	*/
/*  nodeText	: QString &	: Result text for node output		*/
/*  indent	: int		: Indent depth for node output		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBEvent::printAttr
	(	QString		&attrText,
		QString		&nodeText,
		int		indent,
		bool		flat
	)

{
	if ((getFlags() & (KAF_HIDDEN|KAF_USER)) != 0)
		return	;

	if (m_macro != 0)
	{
		nodeText += QString("%1<macro name=\"%2\">\n")
				   .arg("", indent)
				   .arg(m_name) ;

		m_macro->save (nodeText, indent + 2) ;

		nodeText += QString("%1</macro>\n")
				   .arg("", indent) ;
		return	;
	}

	static int asText = -1 ;
	if (asText < 0)
	{
		cchar *e = getenv("REKALL_EVENTS_ASTEXT") ;

		if (e == 0)
			asText	= 0 ;
		else	asText	= atoi(getenv("REKALL_EVENTS_ASTEXT")) ;

		DPRINTF
		((	"KBEvent::printAttr: asText=%d [%s]\n",
			asText,
			e == 0 ? "NULL" : e
		))	;
	}


	if (asText)
	{
		if (!getValue().isEmpty())
		{
			nodeText += QString("%1<event name=\"%2\">\n" ).arg("", indent).arg(m_name) ;
			nodeText += KBAttr::escapeText (getValue (), false) ;
			nodeText += QString("%1</event>\n" ).arg("", indent) ;
		}
	}
	else	KBAttrStr::printAttr (attrText, nodeText, indent, flat) ;


	/* Save the second language code if present. The KAF_EVCS check	*/
	/* is there since the flag was not introduced at the same time	*/
	/* as second language support, so it was possible to write	*/
	/* second language code in non-second-language (ie., not client	*/
	/* side) events).						*/
	if (!m_value2.isEmpty() && ((m_nFlags & KAF_EVCS) != 0))
	{
		if (asText)
			nodeText += QString("%1<event2 name=\"%2\">\n%3%4</event2>\n")
					   .arg("", indent)
					   .arg(m_name)
					   .arg(escapeText(m_value2, false))
				   	   .arg("", indent) ;
		else	addAttrText (attrText, QString("%1_l2").arg(m_name), m_value2) ;
	}

	if (m_breakpoints.count() != 0)
	{
		QString	list	;
		for (uint idx = 0 ; idx < m_breakpoints.count(); idx += 1)
		{	if (idx > 0) list += ',' ;
			list += QString::number(m_breakpoints[idx]) ;
		}

		attrText += QString (" %1_bpt=\"%2\"").arg(m_name).arg(list) ;
	}

}

/*  KBEvent								*/
/*  setComment	: Set commend						*/
/*  comment	: const QString & : Comment string			*/
/*  (returns)	: void		  :					*/

void	KBEvent::setComment
	(	const QString	&comment
	)
{
	m_comment = comment ;
}

/*  KBEvent								*/
/*  comment	: Get comment string					*/
/*  (returns)	: QString	: Comment				*/

QString	KBEvent::comment ()
{
	return	m_comment   ;
}

/*  KBEvent								*/
/*  errorOrigin	: Determin origin of error				*/
/*  root	: KBNode *	     : Document root node		*/
/*  location	: const KBLocation & : Error location			*/
/*  (returns)	: KBEvent::Origin    : Error origin			*/

KBEvent::Origin
	KBEvent::errorOrigin
	(	KBNode			*root,
		const KBLocation	&location
	)
{
	const QString	&name	= location.name() ;

	DPRINTF
	((	"KBEvent::errorOrigin: name=[%s] uuid=[%s]\n",
		name.latin1(),
		root->getAttrVal("uuid").latin1()
	))	;

	/* This is the special case. The "local" code is compiled as if	*/
	/* it were a script module, but needs to be handled as an event	*/
	/* for display after an error. If it is *our* "local" code (and	*/
	/* it might not be) then handle as such, otherwise fall back	*/
	/* on the event (or slot) which was invoked.			*/
	if (name.find("$RekallLocal") >= 0)
	{
		QString	uuid	= root->getAttrVal("uuid") ;
		if (name.left(uuid.length()) == uuid)
			return	KBEvent::OriginLocal ;

		return	KBEvent::OriginEvent ;
	}

	/* The script engines report errors in event and slot code as	*/
	/* having the "inline" name.					*/
	if (name == KBLocation::m_pInline)
		return	KBEvent::OriginEvent ;

	/* Anything else must be a real script module ...		*/
	return	KBEvent::OriginScript ;
}

void	KBEvent::setBreakpoints
	(	QValueList<int>	&breakpoints
	)
{
	m_breakpoints = breakpoints ;

#if	__DEBUG_EVENT
	for (uint idx = 0 ; idx < m_breakpoints.count() ; idx += 1)
		DPRINTF
		((	"KBEvent::setBreakpoints: breakpoint %d\n",
			m_breakpoints[idx]
		))	;
#endif
}

const QValueList<int>
	&KBEvent::breakpoints ()
{
#if	__DEBUG_EVENT
	for (uint idx = 0 ; idx < m_breakpoints.count() ; idx += 1)
		DPRINTF
		((	"KBEvent::breakpoints: breakpoint %d\n",
			m_breakpoints[idx]
		))	;
#endif
	return	m_breakpoints ;
}

#if	! __KB_RUNTIME

QString	KBEvent::getCodeError  ()
{
	return	m_code  == 0 ? QString::null : m_code ->errorMessage() ;
}

QString	KBEvent::getCodeError2 ()
{
	return	m_code2 == 0 ? QString::null : m_code2->errorMessage() ;
}

/*  KBEvent	:							*/
/*  trimEvent	: Trim event text					*/
/*  event	: QString	: User text				*/
/*  (returns)	: QString	: Trimmed text				*/

QString KBEvent::trimEvent
	(	QString		event
	)
{
	int	offset	;

	/* If this is a shortcut then strip leading whitespace and	*/
	/* anything from the first non-leading whitespace onwards.	*/
	/* This leaves just the shortcut itself.			*/
	if ((offset = QRegExp("^\\s*#[A-Za-z]").match (event)) >= 0)
	{
		event	= event.stripWhiteSpace() ;

		if ((offset = QRegExp("\\s").match (event)) >= 0)
			event	= event.left (offset) ;

		return	event	;
	}

	/* Otherwise, it is script code. In this case, strip leading	*/
	/* and trailing	whitespace and add a newline *unless* the text	*/
	/* is empty.							*/
	QString	trimmed	= event.stripWhiteSpace() ;
	return	trimmed.isEmpty() ? QString::null : trimmed + "\n" ;
}

/*  KBEvent								*/
/*  getAttrItem	: Get item for property dialogs				*/
/*  (returns)	: KBAttrItem *	: Actually KBEventItem			*/

KBAttrItem
	*KBEvent::getAttrItem ()
{
	return	new KBAttrEventItem (this) ;
}

/*  KBEvent								*/
/*  getAttrDlg	: Get dialog component for frame			*/
/*  parent	: QWidget *		: Parent widget			*/
/*  item	: KBAttrItem *		: Associated item		*/
/*  attrDict	: QDict<KBAttrItem> &	: Attribute dictionary		*/
/*  (returns)	: KBAttrDlg *		: Actually KBAttrFrameDlg	*/

KBAttrDlg
	*KBEvent::getAttrDlg
	(	QWidget			*parent,
		KBAttrItem		*,
		QDict<KBAttrItem>	&attrDict
	)
{
	return	new KBEventDlg (parent, this, 0, attrDict) ;
}

#endif
