/***************************************************************************
    file	         : kb_event.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_EVENT_H
#define	_KB_EVENT_H

#include	<qvaluelist.h>

#include	"libkbase_exports.h"
#include	"kb_attr.h"

class	KBMacroExec	;
class	KBScriptCode	;
class	KBScriptError	;
class	KBEmitter	;

/*  Create a typedef for C++-level event functions. The arguments are:	*/
/*	* Pointer to invoking event					*/
/*	* Reference to value return value				*/
/*	* Number of arguments						*/
/*	* Vector of arguments						*/
/*	* Error return							*/

//  typedef KBScript::ExeRC (*CPPFunc)
//		(	KBEvent		*,
//			KBValue 	&,
//			uint		,
//			KBValue		*,
//			KBError		&
//		)	;

class	KBAttrDlg	;
class	KBAttrItem	;
class	KBLocation	;

/*  KBEvent								*/
/*  -------								*/
/*  This class encapsulates the script associated with an event, such	*/
/*  as a button click or a new record becoming current.			*/

class	LIBKBASE_API	KBEvent : public KBAttrStr
{
	QString	 	m_base	    	;
	bool	 	m_disable  	;

	KBScriptCode	*m_code	    	;
	KBEmitter	*m_emitter 	;

//	CPPFunc		m_cppFunc	;
	KBMacroExec	*m_macro	;

	KBEvent		*m_override	;
	KBEvent		*m_inherit	;

	QString		m_value2	;
	QString		m_comment	;

	KBScriptCode	*m_code2	;

	QValueList<int>	m_breakpoints	;

	void		init		() ;
	void		clearOverride	() ;

	KBScriptError	*execCode
			(	KBScriptIF	*,
				KBScriptCode	*&,
				const QString	&,
				KBValue		&,
				uint		,
				KBValue		*
			)	;

	KBScriptError		*doExecuteL1	(KBValue &, uint,     KBValue *,     bool) ;
	KBScriptError		*doExecuteL2	(KBValue &, uint,     KBValue *,     bool) ;

public	:

	KBEvent	(KBNode *, cchar *, const QDict<QString> &, uint = 0) ;
	KBEvent	(KBNode *, cchar *, cchar  *, uint = 0) ;
	KBEvent	(KBNode *, cchar *, KBNode *, uint = 0) ;
	virtual	~KBEvent	() ;

	void			setOverride	(const QString &) ;
	virtual	bool		showAs		(KB::ShowAs)	  ;

	virtual	void		setValue	(KBAttrItem    *) ;
	virtual	void		setValue	(const QString &) ;

	virtual	void		setCode		(const QString &, bool) ;
	virtual	void		setCode2	(const QString &, bool) ;
	virtual	void		tidy		()		  ;

	virtual	void		setMacro	(KBMacroExec   *) ;
	virtual	KBMacroExec	*getMacro	()	;

	void			setComment	(const QString &) ;
	QString			comment		() ;

	bool			isEmpty		()	;

	QString			getValue2	()	;
	void			setValue2	(const QString &) ;
//	virtual	void		setCPPFunc	(CPPFunc	) ;

	virtual	void		printAttr 	(QString &, QString &, int, bool) ;
	virtual	KBEvent		*isEvent	() ;

	KBEmitter		*getEmitter	() ;
	void			clearEmitter	() ;

	KBScriptError		*doExecute	(KBValue &, uint,     KBValue *,     bool) ;
	virtual KBScriptError	*execute	(KBValue &, uint = 0, KBValue * = 0, bool = false) ;

	virtual QString		getDescription	() const ;

	void			setBreakpoints	(QValueList<int> &) ;
	const QValueList<int>	&breakpoints	() ;

	enum	Origin
	{
		OriginLocal,
		OriginScript,
		OriginEvent
	}	;

	static	Origin		errorOrigin	(KBNode *, const KBLocation &) ;

	KBATTRDLGDEF

#if	! __KB_RUNTIME
	QString			getCodeError	() ;
	QString			getCodeError2	() ;

	virtual	KBAttrItem	*getAttrItem	()		;
	static	QString		trimEvent	(QString)	;
#endif
}	;

#endif	// _KB_EVENT_H
