/***************************************************************************
    file	         : kb_eventdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qlayout.h>
#include	<qregexp.h>
#include	<qpopupmenu.h>
#include	<qevent.h>
#include	<qsplitter.h>
#include	<qtextbrowser.h>

#include	"rk_vbox.h"
#include	"rk_lineedit.h"


#include	"kb_classes.h"
#include	"kb_object.h"
#include	"kb_options.h"
#include	"kb_docroot.h"
#include	"kb_prompt.h"
#include	"kb_font.h"
#include	"kb_propdict.h"
#include	"kb_attr.h"
#include	"kb_attrlanguagedlg.h"
#include	"kb_script.h"
#include	"kb_event.h"

#include	"kb_eventdlg.moc"

#include	"kb_textedit.h"
#include	"kb_keymapper.h"
#include	"tk_messagebox.h"


#define		__DEBUG_EVENTDLG	(0)
#if		__DEBUG_EVENTDLG
#include	"kb_dprintf.h"
#endif

QString	fixLanguageName
	(	const QString	&language
	)
{
	/* Hack, strip underscore and following. Allows use of a single	*/
	/* set of descriptions for, say, "kjs" and "kjs_cs".		*/
	int	us = language.find('_') ;
	return	us < 0 ? language : language.left(us) ;
}

/*  checkCompile: Check script compiles OK				*/
/*  parent	: KBNode *	: Parent node for script code		*/
/*  text	: QString	: Script				*/
/*  func	: QString	: Script entry function			*/
/*  l2		: bool		: Use second scripting language		*/
/*  (returns)	: void		:					*/

bool	checkCompile
	(	KBNode		*parent,
		QString		text,
		QString		func,
		bool		l2
	)
{
	KBError		error	 ;
	KBDocRoot	*root	 = parent->getRoot()->isDocRoot() ;

	DPRINTF
	((	"checkCompile: l2=%d f=[%s] t=[%50.50s...]\n",
		l2,
		func.latin1(),
		text.latin1()
	))	;

	/* Locate the scripting interface for the document. If there is	*/
	/* none, then report an error.	If we have one then verify that	*/
	/* the script compiles OK.					*/
	KBScriptIF	*iface	= root->getScriptIF (l2, error) ;
	if (iface == 0)
	{	error.DISPLAY() ;
		return	false	;
	}

	KBScriptCode	*code	;

	DPRINTF
	((	"checkCompile: %s: calling compileFunc\n",
		iface->className()
	))	;

	if ((code = iface->compileFunc
		    (	parent,
			text,
			parent->getPath   (),
			func,
			root  ->getImports(),
			0,
			error
		    )	) == 0)
	{
		DPRINTF
		((	"checkCompile: failed\n"
		))	;

		error.DISPLAY()	;
		return	false	;
	}

	/* Hey! Sucess!							*/
	DELOBJ	(code)	;
	return	true	;
}

static	void	printBreakpoints
	(	cchar			*where,
		const QValueList<int>	&m_breakpoints
	)
{
#if	__DEBUG_EVENTDLG
	for (uint idx = 0 ; idx < m_breakpoints.count() ; idx += 1)
		DPRINTF
		((	"%s: breakpoint %d\n",
			where,
			m_breakpoints[idx]
		))	;
#endif
}


/*  ------------------------------------------------------------------  */

/*  KBAttrEventItem							*/
/*  KBAttrEventItem							*/
/*		: Constructor for event attribute item			*/
/*  event	: KBEvent *	  : Associated event			*/
/*  (returns)	: KBAttrEventItem :					*/

KBAttrEventItem::KBAttrEventItem
	(	KBEvent		*event
	)
	:
	KBAttrItem	(event),
	m_event		(event)
{
	DPRINTF
	((	"KBAttrEventItem::KBAttrEventItem: e=%s\n",
		(cchar *)m_event->getName()
	))	;

	KBMacroExec *m	= event->getMacro    () ;
	m_macro		= m == 0 ? 0 : new KBMacroExec (m) ;
	m_breakpoints	= event->breakpoints () ;
	m_value2	= event->getValue2   () ;

	printBreakpoints ("KBAttrEventItem::KBAttrEventItem", m_breakpoints) ;
}

/*  KBAttrEventItem							*/
/*  ~KBAttrEventItem							*/
/*		: Destructor for event attribute item			*/
/*  (returns)	: void		:					*/

KBAttrEventItem::~KBAttrEventItem ()
{
	if (m_macro != 0) delete m_macro ;
}

/*  KBAttrEventItem							*/
/*  displayValue: Get display value for event				*/
/*  (returns)	: QString	: Display text	*			*/

QString	KBAttrEventItem::displayValue ()
{
	return	m_macro != 0	  ? QString("[Macro]")			:
		m_value.isEmpty() ? m_event->displayValue(m_value2)	:
				    KBAttrItem::displayValue()		;
}

/*  KBAttrEventItem							*/
/*  getAttrDlg	: Get attribute dialog for this item			*/
/*  parent	: QWidget *		: Parent widget			*/
/*  attrDict	: QDict<KBAttrItem> &	: Disctionary of all attributes	*/
/*  (returns)	: KBAttrDlg *		:				*/

KBAttrDlg
	*KBAttrEventItem::getAttrDlg
	(	QWidget			*parent,
		QDict<KBAttrItem>	&attrDict
	)
{
	return	m_dialog = new KBEventDlg (parent, m_event, this, attrDict) ;
}

QString	KBAttrEventItem::getDescription ()
{
	return	m_dialog == 0 ?
			KBAttrItem::getDescription() :
			m_dialog  ->getDescription() ;

}

/*  KBAttrEventItem							*/
/*  save	: Save value back to event				*/
/*  (returns)	: void		:					*/

void	KBAttrEventItem::save ()
{
	if (m_macro != 0)
	{	m_event->setMacro (m_macro) ;
		m_macro	= 0   ;
		return	;
	}

	m_event->setMacro (0) ;

	QString	text	 = KBEvent::trimEvent (m_value ) ;
	QString	text2	 = KBEvent::trimEvent (m_value2) ;
	QString	funcName = m_event->getName() == "local" ? QString::null : QString("eventFunc") ;

	/* Check whether a non-empty script compiles. Note that we	*/
	/* don't stop the save, for instance in case the user has	*/
	/* forgotten to set the script language. For the first language	*/
	/* we check in case of shortcuts, for the second these are not	*/
	/* allowed.							*/
	if (!text .isEmpty())
		if ((text.at(0) != '#') || !text.at(1).isLetter())
			checkCompile (m_event->getOwner()->isObject(), text, funcName, false) ;
	if (!text2.isEmpty())
		checkCompile (m_event->getOwner()->isObject(), text2, funcName, true) ;


	m_event->setValue	(text ) ;
	m_event->setBreakpoints (m_breakpoints) ;
	m_event->setValue2	(text2) ;
}

/*  KBAttrEventItem							*/
/*  clear	: Clear value						*/
/*  (returns)	: void		:					*/

void	KBAttrEventItem::clear ()
{
	DELOBJ	(m_macro)   ;
	KBAttrItem::clear() ;
}

/*  KBAttrEventItem							*/
/*  setBreakpoints							*/
/*		: Set breakpoint list					*/
/*  breakpoints	: QValueList<int> & : list to set			*/
/*  (returns)	: void		    :					*/

void	KBAttrEventItem::setBreakpoints
	(	QValueList<int>	&breakpoints
	)
{
	m_breakpoints = breakpoints ;
	printBreakpoints ("KBAttrEventItem::setBreakpoints", m_breakpoints) ;
}

/*  KBAttrEventItem							*/
/*  breakpoints	: Get breakpoint list					*/
/*  (returns)	: QValueList<int> & : List				*/

const QValueList<int>
	&KBAttrEventItem::breakpoints ()
{
	printBreakpoints ("KBAttrEventItem::breakpoints", m_breakpoints) ;
	return	m_breakpoints ;
}



/*  ------------------------------------------------------------------  */

/*  KBEventBaseDlg							*/
/*  KBEventBaseDlg							*/
/*		: Constructor for event dialog				*/
/*  parent	: QWidget *		: Parent widget			*/
/*  language	: const QString &	: Main language			*/
/*  language2	: const QString &	: Second language		*/
/*  skeleton	: const QString &	: Main language skeleton	*/
/*  skeleton2	: const QString &	: Second language skeleton	*/
/*  macroSet	: const QString &	: Macro instruction set		*/
/*  evcs	: bool			: Event exists client side	*/
/*  (returns)	: KBEventBaseDlg	:				*/

KBEventBaseDlg::KBEventBaseDlg
	(	QWidget			*parent,
		const KBLocation	&location,
		const QString		&language,
		const QString		&language2,
		const QString		&skeleton,
		const QString		&skeleton2,
		const QString		&macroSet,
		bool			evcs
	)
	:
	RKVBox		(parent),
	m_language	(language),
	m_language2	(language2),
	m_skeleton	(skeleton),
	m_skeleton2	(skeleton2)
{
	DPRINTF
	((	"KBEventBaseDlg::KBEventBaseDlg: l=[%s] l2=[%s]\n",
		(cchar *)m_language,
		(cchar *)m_language2
	))	;

	/* Only display a language selector if a second language is	*/
	/* specified, and then only if this event is executed client-	*/
	/* side. We assume that the second language *is* client side,	*/
	/* this should be enforced in the language selection dialog.	*/
	m_useLanguage	= 0 ;
	if (!m_language2.isEmpty())
	{
		QString	l1 = KBAttrLanguageDlg::mapLanguageToDisplay(m_language ) ;
		QString	l2 = KBAttrLanguageDlg::mapLanguageToDisplay(m_language2) ;

		if (evcs)
		{
			m_useLanguage = new RKComboBox (this) ;
			m_useLanguage->insertItem (l1) ;
			m_useLanguage->insertItem (l2) ;

			connect
			(	m_useLanguage,
				SIGNAL	(activated	(int)),
				SLOT	(switchLanguage	())
			)	;
		}
		else
		{
			RKLineEdit *le = new RKLineEdit (this) ;
			le->setText    	(QString(TR("%1: event is server-side only")).arg(l1)) ;
			le->setReadOnly	(true) ;
			le->setBackgroundMode (Qt::PaletteMid) ;
		}
	}

	/* The editor widget goes into a stack as there may be a second	*/
	/* language. This is not actually needed if there is no second	*/
	/* language, but its not worth optimizing out.			*/
	m_stack		= new QWidgetStack  (this   )	;

//	m_splitter	= new QSplitter	    (Qt::Vertical, m_stack) ;
	m_editor	= new KBTextEdit    (m_stack)	;
	m_editor->setWordWrap    (RKTextEdit::NoWrap)	;
	m_editor->setFont	 (KBFont::specToFont(KBOptions::getScriptFont(), true)) ;
	m_editor->showLineNumbers(true) ;

	m_editor->showSkeleton	 (!m_skeleton.isEmpty()) ;

	connect
	(	m_editor,
		SIGNAL	(skeletonClicked()),
		SLOT	(skeletonClicked())
	)	;
	connect
	(	m_editor,
		SIGNAL	(textChanged	()),
		SLOT	(slotTextChanged())
	)	;

	if (m_useLanguage != 0)
	{
		m_editor2	= new KBTextEdit     (m_stack)	;
		m_editor2->setWordWrap    (RKTextEdit::NoWrap)	;
		m_editor2->setFont	  (KBFont::specToFont(KBOptions::getScriptFont(), true)) ;
		m_editor2->showLineNumbers(true) ;

		m_editor2->showSkeleton	  (!m_skeleton2.isEmpty()) ;

		connect
		(	m_editor2,
			SIGNAL	(skeletonClicked()),
			SLOT	(skeletonClicked())
		)	;
		connect
		(	m_editor2,
			SIGNAL	(textChanged	()),
			SLOT	(slotTextChanged())
		)	;
	}
	else	m_editor2 = 0 ;

	m_editor->setHighlight (fixLanguageName(m_language)) ;
	if (m_editor2 != 0)
		m_editor2->setHighlight (fixLanguageName(m_language2)) ;

	connect
	(	m_editor,
		SIGNAL	(clickMarkers	  (QEvent *, int)),
		SLOT	(slotClickMarkers (QEvent *, int))
	)	;

	m_macroEdit	= new KBMacroEditor
			  (	m_stack,
				location.dbInfo(),
				location.server(),
				macroSet
			  )	;
}

/*  KBEventBaseDlg							*/
/*  ~KBEventBaseDlg							*/
/*		: Destructor for event dialog				*/
/*  (returns)	:		:					*/

KBEventBaseDlg::~KBEventBaseDlg ()
{
}

/*  KBEventBaseDlg							*/
/*  init	: Initialise editor for script editing			*/
/*  value	: const QString & : Text for first language editor	*/
/*  value2	: const QString & : Text for second language editor	*/
/*  (returns)	: void		  :					*/

void	KBEventBaseDlg::init
	(	const QString		&value,
		const QString		&value2,
		const QString		&funcName,
		const QString		&error,
		const QString		&error2
	)
{
	m_funcName = funcName	  ;
	m_editor->setText (value) ;
	m_editor->setFocus()	  ;

//	if (!error.isEmpty ())
//	{
//		QTextBrowser *b = new QTextBrowser (m_splitter) ;
//		b->setText (error) ;
//	}

	/* If we have a second language then load its editor. If the	*/
	/* first language script is empty and the second has some text	*/
	/* then show the second editor.					*/
	if (m_editor2 != 0)
	{
		m_editor2->setText (value2) ;
		if (value.isEmpty() && !value2.isEmpty())
		{
			m_useLanguage->setCurrentItem (1) ;
			m_stack	     ->raiseWidget    (m_editor2) ;
			m_editor2    ->setFocus	      ()  ;
		}
		else	m_stack->raiseWidget (m_editor) ;

//		if (!error2.isEmpty ())
//		{
//			QTextBrowser *b = new QTextBrowser (m_splitter2) ;
//			b->setText (error2) ;
//		}
	}
	else	m_stack->raiseWidget (m_editor) ;
}

void	KBEventBaseDlg::init
	(	KBMacroExec 	*macro
	)
{
	KBError	error ;
	m_stack    ->raiseWidget (m_macroEdit ) ;
	m_macroEdit->startup	 (macro, error) ;

	if (m_useLanguage != 0)
		m_useLanguage->hide() ;
}

/*  KBEventBaseDlg							*/
/*  skeletonClicked							*/
/*		: User has clicked the skeleton button			*/
/*  (returns)	: void		:					*/

void	KBEventBaseDlg::skeletonClicked ()
{
	QString    skeleton ;
	KBTextEdit *editor  ;

	/* The sender is one of the editors, pick up the correspinding	*/
	/* skeleton. If there is none then drop out.			*/
	if (sender() == m_editor2)
		skeleton = m_skeleton2, editor = m_editor2 ;
	else	skeleton = m_skeleton,  editor = m_editor  ;

	DPRINTF
	((	"KBEventBaseDlg::skeletonClicked: e2=%d s=[%s]\n",
		sender() == m_editor2,
		skeleton.latin1()
	))	;

	if (skeleton.isEmpty()) return ;

	/* The text may contain the string __MARK__ in which case it	*/
	/* is edited out and used to position the caret.		*/
	bool	mark	= skeleton.find("__MARK__") >= 0 ;

	editor->setText (skeleton) ;
	if (mark)
		if (editor->find ("__MARK__", false, false))
			editor->removeSelectedText () ;

	editor->setFocus () ;
}

/*  KBEventBaseDlg							*/
/*  slotTextChanged							*/
/*		: Pass through text changed signals			*/
/*  (returns)	: void		:					*/

void	KBEventBaseDlg::slotTextChanged ()
{
	DPRINTF
	((	"KBEventBaseDlg::slotTextChanged: emitting textChanged()\n"
	))	;
	emit	textChanged ()	;
}

void	KBEventBaseDlg::slotClickMarkers
	(	QEvent		*e,
		int		lno
	)
{
	m_lastLno = lno ;

	if (e->type() == QEvent::MouseButtonDblClick)
	{
		toggleBreakpoint () ;
		return	;
	}

	if (e->type() == QEvent::ContextMenu)
	{
		QPopupMenu pop ;

		pop.insertItem (TR("Cancel")) ;
		pop.insertItem (TR("Toggle breakpoint"), this, SLOT(toggleBreakpoint())) ;
		pop.insertItem (TR("Clear breakpoints"), this, SLOT(clearBreakpoints())) ;

		pop.exec (((QMouseEvent *)e)->globalPos()) ;
		return		;
	}
}

/*  KBEventBaseDlg								*/
/*  switchLanguage							*/
/*		: User switches language setting			*/
/*  (returns)	: void		:					*/

void	KBEventBaseDlg::switchLanguage ()
{
	if ((m_editor2 != 0) && (m_useLanguage->currentItem() == 1))
		m_stack->raiseWidget (m_editor2) ;
	else	m_stack->raiseWidget (m_editor ) ;

	emit	languageChanged () ;
}

void	KBEventBaseDlg::toggleBreakpoint ()
{
	if (m_breakpoints.findIndex (m_lastLno) < 0)
	{
		m_editor->setMark    (m_lastLno, 0x01) ;
		m_breakpoints.append (m_lastLno) ;
		return	;
	}

	m_editor->setMark    (m_lastLno, 0x00) ;
	m_breakpoints.remove (m_lastLno) ;

}

void	KBEventBaseDlg::clearBreakpoints ()
{
	for (uint slot = 0 ; slot < m_breakpoints.count() ; slot += 1)
		m_editor->setMark (m_breakpoints[slot], 0x00) ;

	m_breakpoints.clear () ;
}

/*  KBEventBaseDlg							*/
/*  value	: Get value						*/
/*  (returns)	: QString	:					*/

QString	KBEventBaseDlg::value ()
{
	return	KBEvent::trimEvent (m_editor->text()) ;
}

/*  KBEventBaseDlg							*/
/*  value2	: Get second value					*/
/*  (returns)	: QString	:					*/

QString	KBEventBaseDlg::value2 ()
{
	return	m_editor2 == 0 ?
			QString::null :
			KBEvent::trimEvent (m_editor2->text()) ;
}

KBMacroExec
	*KBEventBaseDlg::macro
	(	KBError		&error,
		KBNode		*owner
	)
{
	return	m_macroEdit->macro (error, owner) ;
}

/*  KBEventBaseDlg							*/
/*  verify	: Verify current value					*/
/*  node	: KBNode *	: Owning node				*/
/*  (returns)	: void		:					*/

void	KBEventBaseDlg::verify 
	(	KBNode		*node
	)
{
	bool	ok1	 = true	;
	bool	ok2	 = true	;

	/* Default assumed script text. Get the text and see if it	*/
	/* looks like a shortcut, in which case give the user a hint,	*/
	/* else check that it compiles.					*/
	QString	text	= value () ;
	if (!text.isEmpty())
	{
		if ((text.at(0) == '#') && text.at(1).isLetter())
		{
			KBError::EWarning
			(	TR("Event shortcut, cannot compile"),
				TR("Event code should be in a script module"),
				__ERRLOCN
			)	;
			ok1	= false	;
		}
		else
			ok1	= checkCompile (node, text, m_funcName, false) ;
	}

	if (m_editor2 != 0)
	{
		QString	text2	= KBEvent::trimEvent (m_editor2->text()) ;
		if (!text2.isEmpty())
			ok2 = checkCompile (node, text2, m_funcName, true) ;
	}

	if (ok1 && ok2)
		TKMessageBox::information (0, TR("Event compiles OK")) ;
}

/*  KBEventBaseDlg							*/
/*  language	: Get current language					*/
/*  (returns)	: QString	: Text					*/

QString	KBEventBaseDlg::language ()
{
	if ((m_useLanguage != 0) && (m_useLanguage->currentItem() == 1))
		return	m_language2 ;
	return	m_language ;
}

/*  KBEventBaseDlg							*/
/*  setEventNode: Set node for object location				*/
/*  node	: KBNode *	: Node in question			*/
/*  (returns)	: void		:					*/

void	KBEventBaseDlg::setEventNode
	(	KBNode		*node
	)
{
	m_editor->mapper()->setEventNode (node, m_language) ;
	if (m_editor2 != 0)
		m_editor2->mapper()->setEventNode (node, m_language2) ;
}

/*  ------------------------------------------------------------------	*/

/*  KBEventDlg								*/
/*  KBEventDlg	: Constructor for event dialog				*/
/*  parent	: QWidget *		: Parent widget			*/
/*  event	: KBEvent *		: Actual event property		*/
/*  item	: KBAttrEventItem *	: Associated item		*/
/*  attrDict	: QDict<KBAttrItem> &	: Attribute dictionary		*/
/*  (returns)	: KBEventDlg		:				*/

KBEventDlg::KBEventDlg
	(	QWidget			*parent,
		KBEvent			*event,
		KBAttrEventItem		*item,
		QDict<KBAttrItem>	&attrDict
	)
	:
	KBAttrDlg	(parent, event, item, attrDict),
	m_event		(event),
	m_eventItem	(item)
{
	KBNode	*node	= m_attr->getOwner() ;
	QString	l	= node->getRoot   ()->getAttrVal("language" ) ;
	QString	l2	= node->getRoot   ()->getAttrVal("language2") ;

	m_eventDlg	= new KBEventBaseDlg
				(	parent,
					node->getDocRoot()->getDocLocation(),
					l,
					l2,
					loadSkeleton(l ),
					loadSkeleton(l2),
					"standard",
					(event->getFlags() & KAF_EVCS) != 0
				)	;
	m_eventDlg->setEventNode (event->getOwner()) ;

	setTopWidget	(m_eventDlg) ;

	connect
	(	m_eventDlg,
		SIGNAL	(languageChanged()),
		SLOT	(languageChanged())
	)	;
}

/*  KBEventDlg								*/
/*  ~KBEventDlg	: Destructor for event dialog				*/
/*  (returns)	:		:					*/

KBEventDlg::~KBEventDlg ()
{
}

/*  KBEventDlg								*/
/*  loadSkeleton: Load skeleton for specified language			*/
/*  language	: const QString & : Language				*/
/*  (returns)	: QString	  : Skeleton				*/

QString	KBEventDlg::loadSkeleton
	(	const QString	&language
	)
{
	if (language.isEmpty())
		return	QString::null	;

	/* Get the dictionary entry for the attribute and then extract	*/
	/* the entry named for the event name and the language.		*/
	QString		l	= language == "kjs_cs" ? "kjs" : language ;
	KBPropDictEntry *entry	= m_attr->dictEntry (m_attr->getName() + "." + l) ;
	if ((entry == 0) || (entry->m_extra == 0))
		return	QString::null	;

	/* Skeleton code if present is in the extra values under the	*/
	/* key "skeleton"						*/
	QString	*extra	= entry->m_extra->find("skeleton") ;
	if (extra == 0)
		return	QString::null	;

	/* The text is processed so that __TYPE__ is replaced by an	*/
	/* appropriate type name based on the node in which the event	*/
	/* resides.							*/
	QString	text	= *extra;
	QString	element = m_eventItem->attr()->getOwner()->getElement() ;
	if (element.left(2) == "KB")
		text.replace (QRegExp("__TYPE__"), element.mid(2).lower()) ;

	return		text	;
}

/*  KBEventDlg								*/
/*  init	: Initialise for editing				*/
/*  value	: const QString & : Script text				*/
/*  (returns)	: bool		  : True if modal dialog executed	*/

bool	KBEventDlg::init
	(	const QString	&value
	)
{
	return	false	;
}

/*  KBEventDlg								*/
/*  init	: Initialise for editing				*/
/*  (returns)	: bool		: True if modal dialog executed		*/

bool	KBEventDlg::init ()
{
	KBOptions::ScriptType type ;

	/* First need to decide whether this is a script or a macro.	*/
	/* First case is if there is no event item, in which case we	*/
	/* must be using scripting. The check whether there is an	*/
	/* extant value, in which case use that irrespective of the	*/
	/* options setting.						*/
	if	(m_eventItem == 0)
	{
		type	= KBOptions::ScriptTypeAlwaysScript ;
	}
	else if	(m_eventItem->getMacro()    != 0)
	{
		type	= KBOptions::ScriptTypeAlwaysMacro  ;
	}
	else if	(!m_eventItem->value().isEmpty())
	{
		type	= KBOptions::ScriptTypeAlwaysScript ;
	}
	else
	{
		/* If the event is empty then use the option setting,	*/
		/* prompting the user if set to user choice.		*/
		type	= KBOptions::getScriptType() ;

		if (type == KBOptions::ScriptTypeUserChoice)
		{
			QStringList	opts	   ;
			opts.append (TR("Script")) ;
			opts.append (TR("Macro" )) ;
			static	QString	res	   ;

			KBChoiceDlg	dDlg
					(	TR("Event"),
						TR("Select script or macro"),
						opts,
						res
					)	;

			if (!dDlg.exec()) return true ;

			if (res == TR("Script"))
				type	= KBOptions::ScriptTypeAlwaysScript ;
			else	type	= KBOptions::ScriptTypeAlwaysMacro  ;
		}
	}
						

	/* First case is a macro. This now runs inside the property	*/
	/* dialog.							*/
	if (type == KBOptions::ScriptTypeAlwaysMacro)
	{
		m_showing = KBOptions::ScriptTypeAlwaysMacro ;
		m_eventDlg->init (m_eventItem->getMacro()) ;
		return	false	;
	}

	/* Using a script. The script is edited within the property	*/
	/* dialog. If the current text is empty then see if there is a	*/
	/* skeleton and if so, insert that and move the cursor to the	*/
	/* initial point.						*/
	m_showing = KBOptions::ScriptTypeAlwaysScript ;

	m_eventDlg->init
	(	m_eventItem->value	  (),
		m_eventItem->value2	  (),
		m_event->getName() == "local" ? QString::null : QString("eventFunc") ,
		m_event	   ->getCodeError (),
		m_event	   ->getCodeError2()
	)	;


//	m_breakpoints	= m_eventItem->breakpoints () ;
//	printBreakpoints ("KBEventDlg::init", m_breakpoints) ;
//	for (uint slot = 0 ; slot < m_breakpoints.count() ; slot += 1)
//		m_editor->setMark (m_breakpoints[slot], 0x01) ;

	emit	setDescription (getDescription()) ;
	return	false	;
}

/*  KBEventDlg								*/
/*  save	: Save values back to correspinding attribute item	*/
/*  (returns)	: void		:					*/

void	KBEventDlg::save ()
{
	if (m_eventItem != 0)
	{
		if (m_showing == KBOptions::ScriptTypeAlwaysMacro)
		{
			KBError		error	;
			KBNode		*owner	= m_item->attr()->getOwner() ;
			KBMacroExec	*macro	= m_eventDlg->macro (error, owner) ;

			if (macro != 0)
				m_eventItem->setMacro (macro) ;
			else	error.DISPLAY() ;

			return	;
		}

		m_eventItem->setValue	    (m_eventDlg->value ()) ;
		m_eventItem->setValue2	    (m_eventDlg->value2()) ;
		m_eventItem->setBreakpoints (m_breakpoints) ;
	}
}

/*  KBEventDlg								*/
/*  switchLanguage							*/
/*		: User switches language setting			*/
/*  (returns)	: void		:					*/

void	KBEventDlg::languageChanged ()
{
	emit setDescription (getDescription ()) ;
}

void	KBEventDlg::toggleBreakpoint ()
{
//	if (m_breakpoints.findIndex (m_lastLno) < 0)
//	{
//		m_editor->setMark    (m_lastLno, 0x01) ;
//		m_breakpoints.append (m_lastLno) ;
//		return	;
//	}
//
//	m_editor->setMark    (m_lastLno, 0x00) ;
//	m_breakpoints.remove (m_lastLno) ;

}

void	KBEventDlg::clearBreakpoints ()
{
//	for (uint slot = 0 ; slot < m_breakpoints.count() ; slot += 1)
//		m_editor->setMark (m_breakpoints[slot], 0x00) ;

//	m_breakpoints.clear () ;
}

/*  KBEventDlg								*/
/*  value	: Get value						*/
/*  (returns)	: QString	:					*/

QString	KBEventDlg::value ()
{
	return	m_eventDlg->value() ;
}

/*  KBEventDlg								*/
/*  verify	: Verify current value					*/
/*  (returns)	: void		:					*/

void	KBEventDlg::verify  ()
{
	m_eventDlg->verify (m_attr->getOwner()->isObject()) ;
}

/*  KBEventDlg								*/
/*  getDescription							*/
/*		: Get event description text				*/
/*  (returns)	: QString	: Text					*/

QString	KBEventDlg::getDescription ()
{
	QString		language = fixLanguageName   (m_eventDlg->language()) ;
	KBPropDictEntry *entry	 = m_attr->dictEntry (m_attr->getName() + "." + language) ;

	DPRINTF
	((	"KBEventDlg::getDescription: [%s:%s] e=[%p] [%s.%s] d=[%s]\n",
		(cchar *)m_language,
		(cchar *)m_language2,
		(void  *)entry,
		(cchar *)m_attr->getName(),
		(cchar *)language,
		entry == 0 ? "" : (cchar *)entry->m_descrip
	))	;

	if (entry == 0) return QString::null ;

	return	"<qt>" + entry->m_descrip + "</qt>" ;
}

#if	0

/*  ------------------------------------------------------------------  */

/*  KBEventMacroDlg							*/
/*  KBEventMacroDlg							*/
/*		: Constructor for inline macro editing dialog		*/
/*  dbInfo	: KBDBInfo *	  : Database information		*/
/*  svName	: const QString & : Server name				*/	
/*  (returns)	: KBEventMacroDlg	:				*/

KBEventMacroDlg::KBEventMacroDlg
	(	KBDBInfo	*dbInfo,
		const QString	&svName
	)
	:
	KBDialog	("Macro", true, "KBEventMacroDlg")
{
	RKVBox	*layMain = new RKVBox (this) ;
	layMain->setTracking () ;

	m_editor	= new KBMacroEditor (layMain, dbInfo, svName) ;
	addOKCancel	(layMain) ;
}

/*  KBEventMacroDlg							*/
/*  macro	: Get macro definition					*/
/*  owner	: KBNode *	: Owning node				*/
/*  error	: KBError &	: Error return				*/
/*  (returns)	: KBMacroExec *	: Macro or null on error		*/

KBMacroExec
	*KBEventMacroDlg::macro
	(	KBNode		*owner,
		KBError		&error
	)
{
	return	m_editor->macro (error, owner) ;
}

#endif
