/***************************************************************************
    file	         : kb_eventdlg.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		_KB_EVENTDLG_H
#define		_KB_EVENTDLG_H

#if		! __KB_RUNTIME

#include	<qspinbox.h>
#include	<qcombobox.h>
#include	<qpushbutton.h>
#include	<qframe.h>
#include	<qvaluelist.h>

#include	"libkbase_exports.h"

#include	"kb_classes.h"
#include	"kb_attrdlg.h"
#include	"kb_attritem.h"
#include	"kb_propdlg.h"
#include	"kb_macroeditor.h"


class	KBTextEdit	;
class	TKCTKEMapper	;
class	QEvent		;
class	QWidgetStack	;
class	QSplitter	;

class	KBEventItem	;
class	KBEventDlg	;


/*  KBAttrEventItem							*/
/*  ---------------							*/
/*  Attribute item for events, knows about scripted and macro-based	*/
/*  events.								*/

class	LIBKBASE_API	KBAttrEventItem : public KBAttrItem
{
	KBEvent			*m_event	;
	KBMacroExec		*m_macro	;
	QValueList<int>		m_breakpoints	;

	QString			m_value2	;
	KBEventDlg		*m_dialog	;

public	:

	KBAttrEventItem		(KBEvent *)	;
	~KBAttrEventItem	()		;

	virtual	QString		displayValue	() ;
	virtual	void		clear		() ;
	virtual	void		save		() ;

	virtual	KBAttrDlg	*getAttrDlg	(QWidget *, QDict<KBAttrItem> &) ;
	virtual	QString		getDescription	() ;

	inline	const QString	value2	  ()
	{
		return	m_value2	;
	}
	inline	void	setValue2
		(	const QString	&value2
		)
	{
		m_value2 = value2 ;
	}

	inline	KBMacroExec	*getMacro ()
	{
		return	m_macro	;
	}
	inline	void	setMacro
		(	KBMacroExec	*macro
		)
	{
		if (m_macro != 0) delete m_macro ;
		m_macro	= macro ;
	}

	void			setBreakpoints	(QValueList<int> &) ;
	const QValueList<int>	&breakpoints	() ;
}	;

/*  ------------------------------------------------------------------	*/

/*  KBEventBaseDlg							*/
/*  ----------								*/
/*  Dialog component for events. This is the basic dialog component; it	*/
/*  is split out since the dialogs for normal events and test events	*/
/*  are different.							*/

class	LIBKBASE_API	KBEventBaseDlg : public RKVBox
{
	Q_OBJECT

	RKComboBox		*m_useLanguage	;
	QWidgetStack		*m_stack	;

	QSplitter		*m_splitter	;
	QSplitter		*m_splitter2	;
	KBTextEdit		*m_editor	;
	KBTextEdit		*m_editor2	;
	KBMacroEditor		*m_macroEdit	;

	QString			m_language	;
	QString			m_language2	;
	QString			m_skeleton	;
	QString			m_skeleton2	;
	QString			m_funcName	;
	int			m_lastLno	;
	QValueList<int>		m_breakpoints	;

signals	:

	void		languageChanged		()	;
	void		textChanged		()	;

protected slots :

	void		slotClickMarkers	(QEvent *, int) ;
	void		toggleBreakpoint	()	;
	void		clearBreakpoints	()	;
	void		switchLanguage		()	;
	void		skeletonClicked		()	;
	void		slotTextChanged		()	;

public	:

	KBEventBaseDlg
			(	QWidget			*parent,
				const KBLocation	&location,
				const QString		&language,
				const QString		&language2,
				const QString		&skeleton,
				const QString		&skeleton2,
				const QString		&macroSet,
				bool			evcs
			)	;
	~KBEventBaseDlg	() ;

	void		setEventNode		(KBNode  *) ;
	QString		language		() ;
	QString		value			() ;
	QString		value2			() ;
	KBMacroExec	*macro			(KBError &, KBNode *) ;
	void		verify			(KBNode  *) ;

	void		init	
			(	const QString	&,
				const QString	&,
				const QString	&,
				const QString	&	= QString::null,
				const QString	&	= QString::null
			)	;
	void		init
			(	KBMacroExec	*
			)	;
}	;

/*  KBEventDlg								*/
/*  ----------								*/
/*  Dialog component for events. This is the plugin dialog for the	*/
/*  property editors.							*/

class	LIBKBASE_API	KBEventDlg : public KBAttrDlg
{
	Q_OBJECT

	KBEvent			*m_event	;
	KBAttrEventItem		*m_eventItem	;

	KBEventBaseDlg		*m_eventDlg	;

	QString			m_language	;
	QString			m_language2	;
	QString			m_skeleton	;
	QString			m_skeleton2	;
	int			m_lastLno	;
	QValueList<int>		m_breakpoints	;
	KBOptions::ScriptType	m_showing	;

	QString	loadSkeleton		(const QString &) ;

protected slots :

	void		toggleBreakpoint	()	;
	void		clearBreakpoints	()	;
	void		languageChanged		()	;

public	:

	KBEventDlg
	(	QWidget			*,
		KBEvent			*,
		KBAttrEventItem		*,
		QDict<KBAttrItem>	&
	)	;

	~KBEventDlg() ;

	virtual	void		verify		() ;
	virtual	bool		init		() ;
	virtual	bool		init		(const QString &) ;
	virtual	void		save		() ;
	virtual	QString		value		() ;

	QString			getDescription	() ;
}	;

/*  KBEventMacroDlg							*/
/*  ---------------							*/
/*  Wrapper around the macro editor, used when editing macros embedded	*/
/*  in events.								*/

class	LIBKBASE_API	KBEventMacroDlg : public KBDialog
{
	Q_OBJECT

	KBMacroEditor	*m_editor	;

public	:

	KBEventMacroDlg (KBDBInfo *, const QString &)	;

	inline	KB::ShowRC startup
		(	KBMacroExec	*macro,
			KBError		&pError
		)
	{
		return	m_editor->startup (macro, pError) ;
	}

	KBMacroExec	*macro	(KBNode *, KBError &)	 ;
}	;

#endif		// ! __KB_RUNTIME
#endif		// _KB_EVENTDLG_H
