/***************************************************************************
    file	         : kb_field.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<ctype.h>

#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>
#include	<qcursor.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_sizer.h"
#include	"kb_display.h"
#include	"kb_ctrl.h"
#include	"kb_formblock.h"

#if		__KB_AUTOMOC
#include	"kb_field.moc"
#else
#include	"kb_field.h"
#endif

#include	"kb_qtfield.h"
#include	"kb_nodereg.h"
#include	"kb_writer.h"
#include	"kb_layout.h"
#include	"kb_finddlg.h"
#include	"kb_form.h"

#if		! __KB_RUNTIME
#include	"kb_recorder.h"
#endif


#define		__DEBUG_FIELD	(0)
#if		__DEBUG_FIELD
#include	"kb_dprintf.h"
#endif


/*  KBField								*/
/*  KBField	: Constructor for simple form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBNode	:					*/

KBField::KBField
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBItem		(parent,"KBField",	"expr",  aList),
	m_fgcolor	(this,  "fgcolor",	aList),
	m_bgcolor	(this,  "bgcolor",	aList),
	m_font		(this,	"font",		aList),
	m_frame		(this,	"frame",	aList),
	m_passwd	(this,	"passwd",	aList, KAF_FORM  ),
	m_nullOK	(this,  "nullok",	aList, KAF_FORM  ),
	m_emptyNull	(this,	"emptynull",	aList, KAF_FORM	 ),
	m_eValid	(this,  "evalid",	aList, KAF_FORM  ),
	m_ignCase	(this,  "igncase", 	aList, KAF_FORM  ),
	m_mask		(this,	"mask",		aList, KAF_FORM  ),
	m_format	(this,	"format",	aList),
	m_deformat	(this,	"deformat",	aList, KAF_FORM  ),
	m_align		(this,	"align",	aList),
	m_supress	(this,	"supress",	aList, KAF_REPORT),
	m_helper	(this,	"helper",	aList, KAF_FORM  ),
	m_morph		(this,  "morph",	aList, KAF_FORM  ),
	m_mapCase	(this,	"mapcase",	aList, KAF_FORM  ),
	m_focusCaret	(this,	"focuscaret",	aList, KAF_FORM  ),
	m_onChange	(this,  "onchange",	aList, KAF_FORM|KAF_EVCS),
	m_onReturn	(this,  "onreturn",	aList, KAF_FORM|KAF_EVCS),
	m_onHelper	(this,	"onhelper",	aList, KAF_FORM  )
{
#if	! __KB_RUNTIME
	if (ok != 0)
	{
		if (!::fieldPropDlg (this, "Field", m_attribs))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif

	if (isReportDoc())
		m_report = getParent()->getReport () ;
	else	m_report = 0 ;
}

/*  KBField								*/
/*  KBField	: Constructor for simple form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  field	: KBField *	: Extant field				*/
/*  (returns)	: KBField	:					*/

KBField::KBField
	(	KBNode		*parent,
		KBField		*field
	)
	:
	KBItem		(parent,"expr",	field),
	m_fgcolor	(this,  "fgcolor",	field),
	m_bgcolor	(this,  "bgcolor",	field),
	m_font		(this,	"font",		field),
	m_frame		(this,	"frame",	field),
	m_passwd	(this,	"passwd",	field, KAF_FORM  ),
	m_nullOK	(this,  "nullok",	field, KAF_FORM  ),
	m_emptyNull	(this,	"emptynull",	field, KAF_FORM  ),
	m_eValid	(this,  "evalid",	field, KAF_FORM  ),
	m_ignCase	(this,  "igncase", 	field, KAF_FORM  ),
	m_mask		(this,	"mask",		field, KAF_FORM  ),
	m_format	(this,  "format", 	field),
	m_deformat	(this,	"deformat",	field, KAF_FORM  ),
	m_align		(this,	"align",	field),
	m_supress	(this,	"supress",	field, KAF_REPORT),
	m_helper	(this,	"helper",	field, KAF_FORM  ),
	m_morph		(this,   "morph",	field, KAF_FORM  ),
	m_mapCase	(this,	"mapcase",	field, KAF_FORM  ),
	m_focusCaret	(this,	"focuscaret",	field, KAF_FORM  ),
	m_onChange	(this,  "onchange",	field, KAF_FORM|KAF_EVCS),
	m_onReturn	(this,  "onreturn",	field, KAF_FORM|KAF_EVCS),
	m_onHelper	(this,	"onhelper",	field, KAF_FORM  )
{
	if (isReportDoc())
		m_report = getParent()->getReport () ;
	else	m_report = 0 ;
}

/*  KBField								*/
/*  ~KBField	: Destructor for simple form field			*/
/*  (returns)	:		:					*/

KBField::~KBField ()
{
}

/*  KBField								*/
/*  makeCtrl	: Make a field control					*/
/*  morph	: bool		: Get morped control			*/
/*  (returns)	: KBControl *	: New control				*/

KBControl
	*KBField::makeCtrl
	(	uint		drow
	)
{
	DPRINTF
	((	"KBField::makeCtrl: [%s] drow=%d\n",
		(cchar *)getName(),
		drow
	))	;
	return	new KBCtrlField (getDisplay(), this, drow) ;
}

/*  KBField								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBField::showAs
	(	KB::ShowAs	mode
	)
{
	m_rValid = QRegExp () ;
	KBItem::showAs (mode) ;

	DPRINTF
	((	"KBField::showAs: %s: m_rValid.isEmpty=%d\n",
		getName().latin1(),
		m_rValid.isEmpty()
	))	;
}

/*  KBField								*/
/*  doCheckValid: Check whether value can be saved in database		*/
/*  value	: const QString & : Putative value			*/
/*  allowNull	: bool		  : Ignore not-null check		*/
/*  (returns)	: bool		  : True if OK				*/

bool	KBField::doCheckValid
	(	const QString	&value,
		bool		allowNull
	)
{
	KBError	err	;

	DPRINTF
	((	"KBField::doCheckValid: %s: [%s] an=%d m_rValid.isEmpty=%d p=[%s]\n",
		getName().latin1(),
		value.latin1(),
		allowNull,
		m_rValid.isEmpty(),
		m_rValid.pattern().latin1()
	))	;

	/* First check is for empty fields when the field must be not-	*/
	/* null, except that we skip this test if nulls are being	*/
	/* allowed (while tabbing around a form).			*/
	if (!allowNull)
		if (value.isEmpty () && !m_nullOK.getBoolValue())
		{	
			setError
			(	KBError::Error,
		  		QString (TR("Field %1 may not be empty")).arg(errorText()),
				QString::null,
			  	__ERRLOCN
			)	;
			return	false	;
		}

	/* If the value is not empty and there is a validator then see	*/
	/* if the new value is OK.					*/
	if (!value.isEmpty() && !m_eValid.getValue().isEmpty())
	{
		if (m_rValid.isEmpty())
		{
			m_rValid.setPattern	  ( m_eValid .getValue    ()) ;
			m_rValid.setCaseSensitive (!m_ignCase.getBoolValue()) ;

			if (!m_rValid.isValid())
			{
				setError
				(	KBError::Error,
					QString (TR("Field %1: bad validator")).arg(getName()),
					QString::null,
					__ERRLOCN
				)	;
				return	false	;
			}
		}

		if (m_rValid.match (value) < 0)
		{
			DPRINTF
			((	"KBField::doCheckValid: INVALID\n"
			))	;

			setError
			(	KBError::Error,
			  	QString (TR("Field %1 has invalid contents")).arg(errorText()),
				QString::null,
			  	__ERRLOCN
			)	;
			return	false	;
		}

	}

	/* If the value is not empty or we are doing not-null checks	*/
	/* then see if the value is OK to be saved to the database.	*/
	if (!value.isEmpty() || !allowNull)
		if (!m_type->isValid (value, err, getName()))
		{
			setError (err) ;
			return	 false ;
		}

	return	true	;
}

/*  KBField								*/
/*  checkValid	: Check whether value can be saved in database		*/
/*  value	: const QString & : Putative value			*/
/*  allowNull	: bool		  : Ignore not-null check		*/
/*  (returns)	: bool		  : True if OK				*/

bool	KBField::checkValid
	(	const KBValue	&value,
		bool		allowNull
	)
{
	/* Special case. If the value is empty and empty values are to	*/
	/* be treated as null, then validate it as such.		*/
	if (value.isEmpty() && m_emptyNull.getBoolValue())
		return	doCheckValid (QString::null, allowNull) ;

	return	doCheckValid (value.getRawText(), allowNull) ;
}

void	KBField::returnPressed
	(	uint		qrow,
		const QString	&text
	)
{
	KBValue	args[2]	;
	bool	evRc	;

	args[0] = (int)qrow	;
	args[1] = KBValue(text)	;
	eventHook (m_onReturn, 2, args, evRc) ;
}

void	KBField::onHelper
	(	uint		qrow,
		const QString	&text
	)
{
	KBValue	args[2]	;
	bool	evRc	;

	args[0] = (int)qrow	;
	args[1] = KBValue(text)	;
	eventHook (m_onHelper, 2, args, evRc) ;
}

/*  KBField								*/
/*  getValue	: Get value						*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBField::getValue
	(	uint		qrow
	)
{
	KBValue	value	= KBItem::getValue (qrow) ;

	if (value.isEmpty ())
		if (m_emptyNull.getBoolValue())
			return	KBValue  (m_type) ;

	return	value	;
}

/*  KBField								*/
/*  getReportValue							*/
/*		: Get value for report writing				*/
/*  first	: bool		: First output flag			*/
/*  prior	: bool		: Return prior value flag		*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBField::getReportValue
	(	bool		first,
		bool
	)
{
	QString	text	;

	if (first || (m_curVal != m_lastVal) || !m_supress.getBoolValue())
	{	m_lastVal = m_curVal ;
		return	    m_curVal ;
	}

	return	KBValue() ;
}

/*  KBField								*/
/*  isMorphing	: Test if field is morphed				*/
/*  (returns)	: bool		: Morphed				*/

bool	KBField::isMorphing ()
{
	return	m_morph.getBoolValue() ;
}

/*  KBField								*/
/*  replicate	: Replicate this field					*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New field node			*/

KBNode	*KBField::replicate
	(	KBNode		*parent
	)
{
	return	new KBField (parent, this) ;
}

/*  KBField								*/
/*  doLeave	: Handle focus exit					*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: OK to leave				*/

bool	KBField::doLeave
	(	uint	qrow
	)
{
	/* Handle the onChange event here since we don't do it on a	*/
	/* keystroke by keystroke basis.				*/
	KBControl *ctrl	 = ctrlAtQRow (qrow) ;
	if ((ctrl != 0) && ctrl->changed ())
	{
#if		! __KB_RUNTIME
		KBValue	cval = getValue(qrow) ;
		switch (getMapCase())
		{	case KBField::MapUpper :
				cval	= KBValue(cval.getRawText().upper(), cval.getType()) ;
				break	;
			case KBField::MapLower :
				cval	= KBValue(cval.getRawText().lower(), cval.getType()) ;
				break	;
			default	:
				break	;
		}
		recordUpdateValue (qrow, cval) ;
#endif
		KBValue	args[2]	;
		bool	evRc	;

		args[0] = (int)qrow	 		;
		args[1] = getValue (qrow)		;

		eventHook (m_onChange, 2, args, evRc)	;

		KBFormBlock *fBlk = getFormBlock()	;
		if (fBlk != 0) fBlk->dataChanged (qrow) ;
	}

	return	KBItem::doLeave (qrow) ;
}

/*  KBField								*/
/*  doSearch	: Do record search					*/
/*  (returns)	: void		:					*/

void	KBField::doSearch ()
{
	KBFindTextDlg findDlg (getFormBlock(), this) ;
	findDlg.exec () ;

}

/*  KBField								*/
/*  canBeInvalid: Check if control can be invalid			*/
/*  (returns)	: bool		: True if so				*/

bool	KBField::canBeInvalid ()
{
	return	! m_nullOK.getBoolValue() || !m_eValid.getValue().isEmpty() ;
}

/*  KBField								*/
/*  setSelection: Set text selection					*/
/*  qrow	: uint		: Query row number			*/
/*  start	: int		: Selection start point			*/
/*  length	: int		: Selection length			*/
/*  (returns)	: void		:					*/

void	KBField::setSelection
	(	uint		qrow,
		int		start,
		int		length
	)
{
	KBControl *ctrl = ctrlAtQRow  (qrow)   ;
	if (ctrl != 0) ((KBCtrlField *)ctrl)->setSelection (start, length) ;
}

QString	KBField::getFormat ()
{
	return	m_format.getValue    () ;
}

int	KBField::getAlign  ()
{
	return	m_align .getIntValue () ;
}

QString	KBField::getHelper ()
{
	return	m_helper.getValue () ;
}

bool	KBField::getDeFormat ()
{
	return	m_deformat.getBoolValue () ;
}

bool	KBField::isPasswd  ()
{
	return	m_passwd.getBoolValue () ;
}

KBField::MapCase
	KBField::getMapCase ()
{
	return	(KBField::MapCase)m_mapCase.getIntValue() ;
}

KBField::FocusCaret
	KBField::getFocusCaret ()
{
	return	(KBField::FocusCaret)m_focusCaret.getIntValue() ;
}

#if	! __KB_RUNTIME
/*  KBField								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar *	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBField::propertyDlg
	(	cchar	*iniAttr
	)
{
	if (::fieldPropDlg (this, "Field", m_attribs, iniAttr))
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}

/*  KBField								*/
/*  whatsThisExtra							*/
/*		: Extra stuff for pseudo-what's this popup		*/
/*  info	: QValueList<KBWhatsThisPair> &	: List of items		*/
/*  (returns)	: void				:			*/

void	KBField::whatsThisExtra
	(	QValueList<KBWhatsThisPair>	&info
	)
{
	KBItem::whatsThisExtra (info) ;

	if (!getFormat().isEmpty())
		info.append (KBWhatsThisPair(TR("Format"), getFormat())) ;
}

#endif

NEWNODE(Field, __TR("New &Field"), 0, KNF_FORM|KNF_REPORT|KNF_BLOCK|KNF_DATA)
