/***************************************************************************
    file	         : kb_field.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_FIELD_H
#define		__KB_FIELD_H

#include	"libkbase_exports.h"

#include 	<qregexp.h>

#include	"kb_classes.h"
#include	"kb_item.h"
#include	"kb_qtfield.h"
#include	"kb_attralign.h"
#include	"kb_attrhelper.h"
#include	"kb_attrframe.h"


/*  KBField								*/
/*  -----------								*/
/*  This class is used to represent a KBaseField element in the XML	*/
/*  form description. Note that it may map onto more than one actual	*/
/*  field.								*/

class	LIBKBASE_API	KBField : public KBItem
{
	Q_OBJECT

	KBAttrStr	m_fgcolor   ;	/* Foreground (text) colour	*/
	KBAttrStr	m_bgcolor   ;	/* Background colour		*/
	KBAttrStr	m_font	    ;	/* Report font			*/
	KBAttrFrame	m_frame	    ;	/* Frame style			*/
	KBAttrBool	m_passwd    ;	/* Treat as a password field	*/
	KBAttrBool	m_nullOK    ;	/* Is empty field OK		*/
	KBAttrBool	m_emptyNull ;	/* Treat empty fields as null	*/
	KBAttrStr	m_eValid    ;	/* Validity regular expression 	*/
	KBAttrBool	m_ignCase   ;	/* Ignore case			*/
	KBAttrStr	m_mask	    ;	/* Input mask			*/
	KBAttrStr	m_format    ;	/* Format string		*/
	KBAttrBool	m_deformat  ;	/* Deformat on save		*/
	KBAttrAlign	m_align	    ;	/* Alignment			*/
	KBAttrBool	m_supress   ;	/* Supress repeats		*/
	KBAttrHelper	m_helper    ;	/* Input helper			*/
	KBAttrBool	m_morph	    ;	/* Morph control on focus out	*/
	KBAttrUInt	m_mapCase   ;	/* Case mapping on save		*/
	KBAttrUInt	m_focusCaret;	/* Caret option on focus	*/

	KBEvent		m_onChange  ;
	KBEvent		m_onReturn  ;
	KBEvent		m_onHelper  ;

	QRegExp		m_rValid    ;	/* Compiled validity pattern	*/
	
	KBReport	*m_report   ;	/* Embedding report		*/
	KBValue		m_lastVal   ;	/* Last row value		*/

	bool			doCheckValid	(const QString &, bool) ;

protected :

	virtual	KBControl	*makeCtrl	(uint)	  ;

#if	! __KB_RUNTIME
	virtual	void		whatsThisExtra	(QValueList<KBWhatsThisPair> &) ;
#endif

public	:

	enum	MapCase
	{	MapNone		= 0,
		MapUpper	= 1,
		MapLower	= 2
	}	;
	enum	FocusCaret
	{	Default		= 0,
		AtStart		= 1,
		AtEnd		= 2,
		SelectAll	= 3
	}	;
#if	! __KB_RUNTIME
	virtual	bool		propertyDlg	(cchar *) ;
#endif
	virtual	KBNode		*replicate	(KBNode *) ;
	virtual	KBValue		getReportValue	(bool, bool = false) ;
	virtual	KBValue		getValue	(uint) ;

	virtual	void		showAs		(KB::ShowAs) ;

	virtual bool		checkValid	(const KBValue &, bool) ;
	void			returnPressed	(uint, const QString &) ;
	void			onHelper	(uint, const QString &) ;
	void			setSelection	(uint, int, int)	;

	virtual	bool		isMorphing	()	;

	QString			getFormat	()	;
	int			getAlign	()	;
	QString			getHelper	()	;
	bool			getDeFormat	()	;
	bool			isPasswd	()	;
	KBField::MapCase	getMapCase	()	;
	KBField::FocusCaret	getFocusCaret	()	;

	virtual	bool		doLeave		(uint)	;
	virtual	void		doSearch	()	;
	virtual	bool		canBeInvalid	()	;

	KBField					(KBNode  *, const QDict<QString> &, bool * = 0) ;
	KBField 				(KBNode  *, KBField *) ;
	virtual	~KBField			() ;
}	;


#if	! __KB_RUNTIME
extern	LIBKBASE_API	bool	fieldPropDlg
			(	KBField			*,
				cchar			*,
				QPtrList<KBAttr>	&,
				cchar			* = 0
			)	;
#endif
#endif		// _KB_FIELD_H
