/***************************************************************************
    file	         : kb_fieldchooser.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_fieldchooser.moc"
#else
#include	"kb_fieldchooser.h"
#endif

#include	"kb_serverinfo.h"
#include	"kb_dialog.h"
#include	"kb_qryquery.h"


#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_pushbutton.h"
#include	"rk_listbox.h"


/*  KBFieldChooser							*/
/*  KBFieldChooser							*/
/*		: Constructor for field chooser				*/
/*  location	: KBLocation &	 : Database location			*/
/*  cbServer	: RKComboBox &	 : Server selector			*/
/*  cbObject	: RKComboBox &	 : Table/query selector			*/
/*  lbFields	: RKListBox &	 : List of all fields			*/
/*  lbSelect	: RKListBox &	 : List of selected fields		*/
/*  bAdd	: QPustButton &	 : Add field button			*/
/*  bAddAll	: QPustButton &	 : Add all button			*/
/*  bRemove	: QPustButton &	 : Remove button			*/
/*  bUp		: QPustButton &	 : Move up button			*/
/*  bDown	: QPustButton &	 : Move down button			*/
/*  table	: bool		 : Selecting from query			*/
/*  multi	: bool		 : Field can be selected more than once	*/
/*  (returns)	: KBFieldChooser :					*/

KBFieldChooser::KBFieldChooser
	(	KBLocation	&location,
		RKComboBox	*cbServer,
		RKComboBox	*cbObject,
		RKListBox	*lbFields,
		RKListBox	*lbSelect,
		RKPushButton	*bAdd,
		RKPushButton	*bAddAll,
		RKPushButton	*bRemove,
		RKPushButton	*bUp,
		RKPushButton	*bDown,
		bool		table,
		bool		multi
	)
	:
	m_location	(location),
	m_cbServer	(cbServer),
	m_cbObject	(cbObject),
	m_tabChooser
	(	table ?
			new KBTableChooser(location, cbServer, cbObject) :
			0
	),
	m_qryChooser
	(	table ?
			0 :
			new KBQueryChooser(location, cbServer, cbObject)
	),
	m_lbFields	(lbFields),
	m_lbSelect	(lbSelect)
{
	m_lbPair  = new KBListBoxPairText
			(	m_lbFields,
				m_lbSelect,
				bAdd,
				bAddAll,
				bRemove,
				bUp,
				bDown,
				multi
			)	;

	if (m_tabChooser != 0)
	{
		connect (m_tabChooser, SIGNAL(serverChanged()),	SLOT(setFields	   ())) ;
		connect (m_tabChooser, SIGNAL(tableChanged ()),	SLOT(setFields	   ())) ;
	}
	if (m_qryChooser != 0)
	{
		connect (m_qryChooser, SIGNAL(serverChanged()),	SLOT(setFields	   ())) ;
		connect (m_qryChooser, SIGNAL(queryChanged ()),	SLOT(setFields	   ())) ;
	}

	connect
	(	m_lbPair,
		SIGNAL	(destChanged(bool)),
		SLOT	(destChanged(bool))
	)	;
}

KBFieldChooser::~KBFieldChooser ()
{
	DELOBJ	(m_tabChooser) ;
	DELOBJ	(m_qryChooser) ;
}

void	KBFieldChooser::destChanged
	(	bool		contents
	)
{
	emit	selectChanged (contents) ;
}

/*  KBFieldChooser							*/
/*  setFields	: Set fields for selected server and table/query	*/
/*  (returns)	: void		  :					*/

void	KBFieldChooser::setFields ()
{
	m_lbSelect->clear () ;
	m_lbFields->clear () ;

	QString	svrName	= m_cbServer->currentText() ;
	QString	objName	= m_cbObject->currentText() ;

	if (!svrName.isEmpty() && !objName.isEmpty())
	{
		if (m_tabChooser != 0)
		{
			KBTableSpec 	tabSpec (objName) ;
			KBDBLink    	dbLink  ;

			if (!dbLink.connect (m_location, svrName))
			{	dbLink.lastError().DISPLAY() ;
				return	;
			}

			if (!dbLink.listFields (tabSpec))
			{	dbLink.lastError().DISPLAY() ;
				return	;
			}

			LITER
			(	KBFieldSpec,
				tabSpec.m_fldList,
				fldSpec,

				m_lbFields->insertItem (fldSpec->m_name) ;
			)
		}

		if (m_qryChooser != 0)
		{
			KBLocation	locn
					(	m_location.dbInfo(),
						"query",
						svrName,
						objName
					)	;

			KBDummyRoot	dummy (locn) ;

			KBQryQuery *qry	 = new KBQryQuery (&dummy) ;

			if (!qry->loadQueryDef (locn))
			{
				qry->lastError().DISPLAY() ;
				return	;
			}

			QPtrList<KBFieldSpec> fldList ;
			int		   pKey	   ;
			fldList.setAutoDelete (true) ;

			if (!qry->getFieldList (0, fldList, pKey))
			{
				qry->lastError().DISPLAY() ;
				return	;
			}

			LITER
			(	KBFieldSpec,
				fldList,
				fldSpec,

				m_lbFields->insertItem (fldSpec->m_name) ;
			)
		}
	}

	m_lbPair->setButtonState () ;
	emit   fieldsChanged () ;
	emit   selectChanged (true) ;
}


/*  ------------------------------------------------------------------  */

/*  KBFieldChooserDlg							*/
/*  KBFieldChooserDlg							*/
/*		: Constructor for field chooser dialog			*/
/*  location	: KBLocation &	    : Database location			*/
/*  table	: bool		    : Selecting from tables		*/
/*  multi	: bool		    : Allow multiple selection		*/
/*  (returns)	: KBFieldChooserDlg :					*/

KBFieldChooserDlg::KBFieldChooserDlg
	(	KBLocation	&location,
		bool		table,
		bool		multi
	)
	:
	KBDialog 	("Select fields", true),
	m_location	(location)
{
	RKVBox		*layMain	= new RKVBox    (this) ;
	layMain->setTracking () ;

	RKGridBox	*layObject	= new RKGridBox (2, layMain) ;

	new QLabel (TR("Select server"), layObject) ;
	m_cbServer	= new RKComboBox  (layObject) ;
	new QLabel (table ? TR("Select table") : TR("Select query"), layObject) ;
	m_cbObject	= new RKComboBox  (layObject) ;

	RKHBox		*laySelect	= new RKHBox	(layMain) ;
	m_lbFields	= new RKListBox	(laySelect) ;

	RKVBox		*layButton	= new RKVBox	(laySelect) ;

	m_bAdd		= new RKPushButton (TR("Add"    ), layButton) ;
	m_bAddAll	= new RKPushButton (TR("Add All"), layButton) ;
	m_bRemove	= new RKPushButton (TR("Remove" ), layButton) ;
	m_bUp		= new RKPushButton (TR("Up"     ), layButton) ;
	m_bDown		= new RKPushButton (TR("Down"   ), layButton) ;
	layButton  ->addFiller() ;

	m_lbSelect	= new RKListBox	(laySelect) ;

	addOKCancel	(layMain, &m_bOK) ;

	m_fldChooser	= new KBFieldChooser
			  (	m_location,
				m_cbServer, m_cbObject, m_lbFields, m_lbSelect,
				m_bAdd,	    m_bAddAll,  m_bRemove,  m_bUp,      m_bDown,
				table,
				multi
			  )	;

	m_lbFields->setMinimumWidth (120) ;
	m_lbSelect->setMinimumWidth (120) ;
	m_bOK	  ->setEnabled (false   ) ;

	connect
	(	m_fldChooser,
		SIGNAL(selectChanged(bool)),
		SLOT  (selectChanged())
	)	;
}

/*  KBFieldChooserDlg							*/
/*  selectChanged: User has changed selected fields list		*/
/*  (returns)	: void		:					*/

void	KBFieldChooserDlg::selectChanged ()
{
	m_bOK->setEnabled (m_lbSelect->count() > 0) ;
}

/*  KBFieldChooserDlg							*/
/*  getFields	: Get list of fields					*/
/*  (returns)	: QStringList	: Field list				*/

QStringList	KBFieldChooserDlg::getFields ()
{
	QStringList	fields	;
	for (uint idx = 0 ; idx < m_lbSelect->count() ; idx += 1)
		fields.append (m_lbSelect->text (idx)) ;
	return		fields	;
}

QString	KBFieldChooserDlg::getServer ()
{
	return	m_cbServer->currentText() ;
}

QString	KBFieldChooserDlg::getObject ()
{
	return	m_cbObject->currentText() ;
}
