/***************************************************************************
    file	         : kb_fieldchooser.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_FIELDCHOOSER_H
#define	__KB_FIELDCHOOSER_H

#include	<qlabel.h>

#include	"rk_combobox.h"

#include	"libkbase_exports.h"

#include	"kb_tablechooser.h"
#include	"kb_querychooser.h"
#include	"kb_listboxpair.h"


class	RKPushButton	;
class	RKListBox	;

/*  KBFieldChooser							*/
/*  --------------							*/
/*  This is a simple helper class which handles various controls used	*/
/*  to allow server/table/field1,field2,... selection.			*/
/*  NOTE: This class does not handle GUI layout, the constructor is	*/
/*	  passed the controls.						*/

class	LIBKBASE_API	KBFieldChooser : public QObject
{
	Q_OBJECT

	KBLocation		&m_location	;
	RKComboBox		*m_cbServer	;
	RKComboBox		*m_cbObject	;
	KBTableChooser		*m_tabChooser	;
	KBQueryChooser		*m_qryChooser	;

	RKListBox		*m_lbFields	;
	RKListBox		*m_lbSelect	;
	KBListBoxPairText	*m_lbPair	;

signals	:

	void		fieldsChanged	() ;
	void		selectChanged	(bool) ;

public slots	:

	void		setFields	() ;
	void		destChanged	(bool) ;

public	:

	KBFieldChooser
	(	KBLocation 	&,
		RKComboBox	*, RKComboBox   *,
		RKListBox	*, RKListBox	*,
		RKPushButton	*, RKPushButton	*,
		RKPushButton	*, RKPushButton	*,
		RKPushButton	*,
		bool		= true,
		bool		= false
	)	;

	~KBFieldChooser	() ;

	inline	bool	setServer
		(	const QString	&server
		)
	{
		if (m_tabChooser != 0) return m_tabChooser->setServer (server) ;
		if (m_qryChooser != 0) return m_qryChooser->setServer (server) ;
		return	false ;
	}
	inline	bool	setObject
		(	const QString	&object
		)
	{
		if (m_tabChooser != 0) return m_tabChooser->setTable  (object) ;
		if (m_qryChooser != 0) return m_qryChooser->setQuery  (object) ;
		return	false ;
	}
}	;


/*  KBTableChooserDlg							*/
/*  -----------------							*/
/*  This class provies a simple server/table/field... selection dialog,	*/
/*  based on the field chooser class above.				*/

class LIBKBASE_API	KBFieldChooserDlg : public KBDialog
{
	Q_OBJECT

	KBLocation	m_location	;

	RKComboBox	*m_cbServer	;
	RKComboBox	*m_cbObject	;

	RKListBox	*m_lbFields	;
	RKListBox	*m_lbSelect	;
	RKPushButton	*m_bAdd		;
	RKPushButton	*m_bAddAll	;
	RKPushButton	*m_bRemove	;
	RKPushButton	*m_bUp		;
	RKPushButton	*m_bDown	;

	RKPushButton	*m_bOK		;

	KBFieldChooser	*m_fldChooser	;

protected slots :

	void		selectChanged	() ;

public	:

	KBFieldChooserDlg   (KBLocation &, bool = true, bool = false) ;

	QString		getServer	() ;
	QString		getObject	() ;

	QStringList	getFields	() ;
}	;


#endif	// __KB_FIELDCHOOSER_H
