/***************************************************************************
    file	         : kb_fielddlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<stdio.h>

#include	<qregexp.h>
#include	<qmultilineedit.h>
#include	<qwidgetstack.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"

#include	"kb_attr.h"
#include	"kb_dbinfo.h"
#include	"kb_qrybase.h"
#include	"kb_block.h"
#include	"kb_field.h"

#include	"kb_formatdlg.h"

#if		__KB_AUTOMOC
#include 	"kb_fielddlg.moc"
#else
#include 	"kb_fielddlg.h"
#endif

#include	"tk_messagebox.h"
#include	"kb_formatdlg.h"


IntChoice	choiceMapCase	[] =
{
{	KBField::MapNone,	"No case mapping"			},
{	KBField::MapUpper,	"Map to upper case"			},
{	KBField::MapLower,	"Map to lower case"			},
{	-1,			0					}
}	;

IntChoice	choiceFocusCaret[] =
{
{	KBField::Default,	"System default"			},
{	KBField::AtStart,	"Position caret at start"		},
{	KBField::AtEnd,		"Position caret at end"			},
{	KBField::SelectAll,	"Select all text"			},
{	-1,			0					}
}	;



/*  KBFieldPropDlg							*/
/*  KBFieldPropDlg: Constructor for block properties dialog		*/
/*  field	  : KBField *	 : Parent field				*/
/*  caption	  : const char * : Dialog box caption			*/
/*  attribs	  : QPtrList<KBAttr> &					*/
/*				 : List of attributes			*/
/*  apply	  : bool	 : Show apply button			*/
/*  iniAttr	  : cchar *	 : Initial attribute			*/
/*  (returns)	  : KBFieldPropDlg:					*/

KBFieldPropDlg::KBFieldPropDlg
	(	KBField			*field,
		cchar			*caption,
		QPtrList<KBAttr>	&attribs,
		cchar			*iniAttr
	)
	:
	KBItemPropDlg	(field, caption, attribs, iniAttr),
	m_field		(field)
{
	m_fDialog = new KBFormatDlg (m_propStack) ;
	m_fDialog->hide () ;
}

/*  KBFieldPropDlg							*/
/*  ~KBFieldPropDlg: Destructor for block properties dialog		*/
/*  (returns)	   : void	:					*/

KBFieldPropDlg::~KBFieldPropDlg ()
{
}

/*  KBFieldPropDlg							*/
/*  getAttrItem	: Get attribute item for attribute			*/
/*  attr	: KBAttr *	: Attribute				*/
/*  (returns)	: KBAttrItem *	: Associated attribute item		*/

KBAttrItem
	*KBFieldPropDlg::getAttrItem
	(	KBAttr	*attr
	)
{
	if (attr->getName() == "mapcase"   )
		return	new KBAttrIntChoice (attr, choiceMapCase   ) ;
	if (attr->getName() == "focuscaret")
		return	new KBAttrIntChoice (attr, choiceFocusCaret) ;

	return	KBItemPropDlg::getAttrItem (attr) ;
}

/*  KBFieldPropDlg							*/
/*  showProperty: Show property						*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property on show			*/

bool	KBFieldPropDlg::showProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName	= item->attr()->getName () ;

	/* For format, show the format type and specification list	*/
	/* boxes. We have a look at the field type to give a hint as	*/
	/* to the appropriate types of format.				*/
	if (aName == "format")
	{
		const QString	&display = getProperty ("expr") ;
		uint		qlvl	 ;
		KBQryBase	*query	 ;

		if ((query = getBlockQuery(qlvl)) == 0) return false ;

		if (!m_fDialog->showFormats
			(	item->value(),
				display,
				query,
				qlvl
			))
			return false ;

		setUserWidget (m_fDialog) ;

		return	true	;
	}

	if (aName == "mapcase")
	{
		showChoices (item, choiceMapCase,    item->value()) ;
		return	true ;
	}

	if (aName == "focuscaret")
	{
		showChoices (item, choiceFocusCaret, item->value()) ;
		return	true ;
	}

	return	KBItemPropDlg::showProperty (item) ;
}

/*  KBFieldPropDlg							*/
/*  saveProperty: Check and save attribute				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Attribute OK				*/

bool	KBFieldPropDlg::saveProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Validity expression						*/
	/* Check that this compiles as a valid (extended) regular	*/
	/* expression.							*/
	if (aName == "evalid")
	{
		QString	 v = m_lineEdit->text() ;

		if (v.isEmpty())
		{
			setProperty (aName,  v) ;
			return	true		;
		}

		if (!QRegExp(v).isValid())
		{
			TKMessageBox::sorry
			(	0,
				"Validator error",
				"The validator is not a valid regular expression"
			)	;
			return	false	;
		}

		setProperty (aName,  v) ;
		return	true	;
	}

	/* In the case of the format specification, we need to stitch	*/
	/* together the type and the specification itself.		*/
	if (aName == "format")
	{
		setProperty (aName, m_fDialog->getValue()) ;
		return	true	;
	}

	if (aName == "mapcase")
	{
		saveChoices (item, choiceMapCase   ) ;
		return	true	;
	}
	if (aName == "focuscaret")
	{
		saveChoices (item, choiceFocusCaret) ;
		return	true	;
	}

	return	KBItemPropDlg::saveProperty (item) ;
}

/*  fieldPropDlg  : Run the field properties dialog			*/
/*  field	  : KBField *	  : Parent field			*/
/*  caption	  : const char *  : Dialog box caption			*/
/*  attribs	  : QPtrList<KBAttr>&					*/
/*				  : List of attributes			*/
/*  iniAttr	  : cchar *	  : Initial attribute			*/
/*  (returns)	  : bool	  : Success				*/

bool LIBKBASE_API fieldPropDlg
	(	KBField		*field,
		cchar		*caption,
		QPtrList<KBAttr>	&attribs,
		cchar		*iniAttr
	)
{
	KBFieldPropDlg fDlg (field, caption, attribs, iniAttr) ;
	return	fDlg.exec () ;
}
