/***************************************************************************
    file	         : kb_finddlg.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_finddlg.moc"
#else
#include	"kb_finddlg.h"
#endif


#include	"kb_classes.h"
#include	"kb_formblock.h"

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_combobox.h"
#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"


static	bool	use_regexp	;
static	bool	use_sensitive	;
static	bool	use_backwards	;
static	bool	use_wholeText	;
static	QString	use_text	;
static	uint	use_choice	;

/*  KBFindDlg								*/
/*  KBFindDlg	: Constructor for record find dialog			*/
/*  block	: KBFormBlock *	: Invoking block			*/
/*  item	: KBItem *	: Invoking item				*/
/*  options	: uint		: Options flags				*/
/*  (returns)	: KBFindDlg	:					*/

KBFindDlg::KBFindDlg
	(	KBFormBlock	*block,
		KBItem		*item,
		uint		options
	)
	:
	KBDialog	(TR("Search for"), true),
	m_block		(block),
	m_item		(item),
	m_options	(options)
{
	RKVBox	*layMain= new RKVBox (this) ;
	layMain->setTracking () ;

	m_findBox	= new QGroupBox (1, Qt::Horizontal, TR("Find"   ), layMain) ;
	m_optionsBox	= new QGroupBox (1, Qt::Horizontal, TR("Options"), layMain) ;
	m_statusBox	= new QGroupBox (1, Qt::Horizontal, TR("Status" ), layMain) ;

	m_finder	= new QWidgetStack (m_findBox) ;
	m_regexp	= new QCheckBox    (TR("Regular expression"), m_findBox) ;
	m_sensitive	= new QCheckBox    (TR("Case sensitive"    ), m_findBox) ;
	m_backwards	= new QCheckBox    (TR("Backwards"         ), m_findBox) ;
	m_wholeText	= new QCheckBox    (TR("Whole text"        ), m_findBox) ;

	m_status	= new QLabel	   (layMain) ;

	addOKCancel	(layMain, &m_bFind) ;
	m_bFind->setText ("Find") ;

	m_bFind		->setDefault (true) ;

	m_regexp	->setChecked (use_regexp	)  ;
	m_sensitive	->setChecked (use_sensitive	)  ;
	m_backwards	->setChecked (use_backwards	)  ;
	m_wholeText	->setChecked (use_wholeText	)  ;

	m_regexp	->setEnabled ((m_options & ShowRegexp   ) != 0) ;
	m_sensitive	->setEnabled ((m_options & ShowSensitive) != 0) ;
	m_wholeText	->setEnabled ((m_options & ShowWholeText) != 0) ;

	m_status->setText
	(	QString	(TR("At record %1 of %2"))
			.arg(m_block->getCurQRow())
			.arg(m_block->getNumRows())
	)	;
}

/*  KBFindDlg								*/
/*  accept	: User clicks the find button				*/
/*  (returns	: void		:					*/

void	KBFindDlg::accept ()
{
	uint	curQRow	= m_block->getCurQRow   () ;
	uint	gotoRow	= 0x7fffffff	;

	use_regexp	= m_regexp   ->isChecked () ;
	use_sensitive	= m_sensitive->isChecked () ;
	use_backwards	= m_backwards->isChecked () ;
	use_wholeText	= m_wholeText->isChecked () ;

	if (!prepare ()) return ;

	if (use_backwards)
	{
		for (uint row = 1 ; row <= curQRow ; row += 1)
			if (matched (curQRow - row))
			{	gotoRow	= curQRow - row ;
				break	;
			}
	}
	else
	{
		for (uint row = curQRow + 1 ; row < m_block->getNumRows () ; row += 1)
			if (matched (row))
			{	gotoRow	= row ;
				break	;
			}
	}

	if (gotoRow == 0x7fffffff)
	{
		m_status->setText (TR("No match found")) ;
		return	;
	}

	m_block->doOperation (KB::GotoQRow, gotoRow, 0) ;

	m_status->setText
	(	QString	(TR("At record %1 of %2"))
			.arg(m_block->getCurQRow() + 1)
			.arg(m_block->getNumRows())
	)	;
}


/*  ------------------------------------------------------------------  */

/*  KBFindTextDlg							*/
/*  KBFindTextDlg							*/
/*		: Constructor for record find text dialog		*/
/*  block	: KBFormBlock *	: Invoking block			*/
/*  item	: KBItem *	: Invoking item				*/
/*  (returns)	: KBFindDlg	:					*/

KBFindTextDlg::KBFindTextDlg
	(	KBFormBlock	*block,
		KBItem		*item
	)
	:
	KBFindDlg
	(	block,
		item,
		KBFindDlg::ShowRegexp|KBFindDlg::ShowSensitive|KBFindDlg::ShowWholeText
	)
{
	m_findText = new RKLineEdit (m_finder) ;
	m_findText->setText (use_text) ;

	if (m_regexp->isChecked())
		m_bFind->setEnabled (QRegExp(use_text).isValid()) ;
	else	m_bFind->setEnabled (!use_text.isEmpty()) ;

	m_finder->setFixedHeight  (m_findText->sizeHint().height()) ;

	connect
	(	m_findText,
		SIGNAL(    textChanged(const QString &)),
		SLOT  (slotTextChanged(const QString &))
	)	;
	connect
	(	m_regexp,
		SIGNAL(    	 toggled(bool)),
		SLOT  (slotRegexpToggled(bool))
	)	;

	m_findText->setFocus () ;
}

/*  KBFindTextDlg							*/
/*  slotTextChanged							*/
/*		: User changes text					*/
/*  text	: const QString & : New text				*/
/*  (returns)	: void		  :					*/

void	KBFindTextDlg::slotTextChanged
	(	const QString	&text
	)
{
	if (m_regexp->isChecked())
		m_bFind->setEnabled (QRegExp(text).isValid()) ;
	else	m_bFind->setEnabled (!text.isEmpty()) ;
}


/*  KBFindTextDlg							*/
/*  slotRegexpToggled							*/
/*		: User changes regexp selection				*/
/*  on		: bool		: Use regexp				*/
/*  (returns)	: void		:					*/

void	KBFindTextDlg::slotRegexpToggled
	(	bool		on
	)
{
	if (on)
		m_bFind->setEnabled (QRegExp(m_findText->text()).isValid()) ;
	else	m_bFind->setEnabled (!m_findText->text().isEmpty()) ;
}


/*  KBFindTextDlg							*/
/*  prepare	: Prepare search					*/
/*  (returns)	: bool		: Success				*/

bool	KBFindTextDlg::prepare ()
{
	use_text = m_findText->text  () ;
	if (use_text.isEmpty())
		return	false ;

	if (!use_sensitive)
		use_text = use_text.lower () ;

	if ( use_regexp)
	{
		if (((m_options & ShowWholeText) != 0) && use_wholeText)
			m_srchRegexp = QRegExp ("^" + use_text + "$") ;
		else	m_srchRegexp = QRegExp (use_text) ;
	}

	return	true	;
}

/*  KBFindTextDlg							*/
/*  matched	: See if row is matched					*/
/*  qrow	: uint		  : Row to check			*/
/*  (returns	: bool		  : Matched				*/

bool	KBFindTextDlg::matched
	(	uint		qrow
	)
{
	QString	qrowText = m_item->getSearchText (qrow) ;

	if (!use_sensitive)
		qrowText = qrowText.lower() ;

//	fprintf
//	(	stderr,
//		"KBFindTextDlg::matched: [%s]==[%s]\n",
//		(cchar *)qrowText,
//		(cchar *)use_text
//	)	;

	if (use_regexp)
		return	qrowText.find (m_srchRegexp) >= 0 ;

	if (use_wholeText)
		return	qrowText == use_text ;

	return	qrowText.find (use_text) >= 0 ;
}

/*  ------------------------------------------------------------------  */

/*  KBFindChoiceDlg							*/
/*  KBFindChoiceDlg							*/
/*		: Constructor for record find choice dialog		*/
/*  block	: KBFormBlock *	: Invoking block			*/
/*  item	: KBItem *	: Invoking item				*/
/*  choices	: QStringList &	: List of choices			*/
/*  values	: QStringList &	: List of values			*/
/*  (returns)	: KBFindDlg	:					*/

KBFindChoiceDlg::KBFindChoiceDlg
	(	KBFormBlock		*block,
		KBItem			*item,
		const QStringList	&choices,
		const QStringList	&values
	)
	:
	KBFindDlg
	(	block,
		item,
		0
	),
	m_choices	(choices),
	m_values	(values)
{
	m_findChoice = new RKComboBox  (m_finder ) ;
	m_findChoice->insertStringList (m_choices) ;
	m_finder    ->setFixedHeight   (m_findChoice->sizeHint().height()) ;
	m_findChoice->setFocus	       () ;

	m_findChoice->setCurrentItem   (use_choice) ;
}


/*  KBFindChoiceDlg							*/
/*  prepare	: Prepare search					*/
/*  (returns)	: bool		: Success				*/

bool	KBFindChoiceDlg::prepare ()
{
	use_choice   = m_findChoice->currentItem () ;
	m_choiceText = m_values[use_choice] ;
	return	true	;
}

/*  KBFindChoiceDlg							*/
/*  matched	: See if row is matched					*/
/*  qrow	: uint		  : Row to check			*/
/*  (returns	: bool		  : Matched				*/

bool	KBFindChoiceDlg::matched
	(	uint		qrow
	)
{
	QString	qrowText = m_item->getSearchText (qrow) ;

//	fprintf
//	(	stderr,
//		"KBFindChoiceDlg::matched: [%s]==[%s]\n",
//		(cchar *)qrowText,
//		(cchar *)m_choiceText
//	)	;

	return	qrowText == m_choiceText ;
}

