/***************************************************************************
    file	         : kb_form.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#if		__KB_AUTOMOC
#include	"kb_form.moc"
#else
#include	"kb_form.h"
#endif

#include	"kb_framer.h"
#include	"kb_blockevents.h"
#include	"kb_script.h"
#include	"kb_import.h"
#include	"kb_param.h"
#include	"kb_dispscroller.h"
#include	"kb_popupmenu.h"
#include	"kb_sizer.h"
#include	"kb_ctrl.h"
#include	"kb_designpopup.h"
#include	"kb_forminitdlg.h"
#include	"kb_slot.h"
#include	"kb_test.h"
#include	"kb_font.h"
#include	"kb_dcopobject.h"


#if		! __KB_RUNTIME
#include	"kb_toolbox.h"
#endif

#define		__DEBUG_FORM	(0)
#if		__DEBUG_FORM
#include	"kb_dprintf.h"
#endif


static	cchar	*makeDCOPName
	(	const QString	&name
	)
{
	return	qstrdup (QString("rekall-form/" + name).latin1()) ;
}



/*  ------------------------------------------------------------------	*/

/*  KBForm								*/
/*  KBForm	: Constructor for form root class for extant form	*/
/*  location	: KBLocation &	 : Form location			*/
/*  aList	: const QDict<QString> &				*/
/*				 : List of attributes			*/
/*  (returns)	: KBForm	 :					*/

KBForm::KBForm
	(	const KBLocation	&location,
		const QDict<QString>	&aList
	)
	:
	KBFormBlock	(0,     aList,		"KBForm"),
	KBLayout	(this),
	m_language	(this,  "language",	aList),
	m_language2	(this,  "language2",	aList),
	m_caption	(this,  "caption",   	aList),
	m_stretch	(this,  "stretch",	aList),
	m_skin		(this,  "skin",		aList),
	m_modal		(this,  "modal",	aList),
	m_hideBars	(this,  "hidebars",	aList, KAF_GRPFORMAT),
	m_hideStatus	(this,  "hidestatus",	aList, KAF_GRPFORMAT),
	m_onAuth	(this,  "onauth",	aList),
	m_onLoad	(this,  "onload",	aList),
	m_onOpened	(this,  "onopened",	aList),
	m_onClient	(this,  "onclient",	aList, KAF_EVCS     ),
	m_onUnload	(this,  "onunload",	aList),
	m_onClose	(this,  "onclose",	aList),
	m_local		(this,  "local",	aList),
	m_uuid		(this,	"uuid",		aList),
	m_docRoot	(this,  m_children,	location)
{
	m_root		= this	;
	m_display	= 0	;
	m_formCurItem	= 0	;
	m_formCurQRow	= 0	;
	m_accel		= 0	;
	m_dcopIface	= new KBDCOPObject (this, makeDCOPName (m_name.getValue())) ;

	m_geom.set	(KBAttrGeom::FMFixed, KBAttrGeom::FMFixed) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;

}

#if	! __KB_RUNTIME

/*  KBForm								*/
/*  KBForm	: Constructor for form root class for new form		*/
/*  location	: KBLocation &	 : Form location			*/
/*  aList	: const QDict<QString> &				*/
/*				 : List of attributes			*/
/*  _ok		: bool &	 : Success				*/
/*  (returns)	: KBForm	 :					*/

KBForm::KBForm
	(	const KBLocation	&location,
		const QDict<QString>	&aList,
		bool			&_ok
	)
	:
	KBFormBlock	(0,	aList,		"KBForm"),
	KBLayout	(this),
	m_language	(this,  "language",  	aList),
	m_language2	(this,  "language2",  	aList),
	m_caption	(this,  "caption",   	aList),
	m_stretch	(this,  "stretch",	aList),
	m_skin		(this,  "skin",		aList),
	m_modal		(this,  "modal",	aList),
	m_hideBars	(this,  "hidebars",	aList, KAF_GRPFORMAT),
	m_hideStatus	(this,	"hidestatus",	aList, KAF_GRPFORMAT),
	m_onAuth	(this,  "onauth",	aList),
	m_onLoad	(this,  "onload",	aList),
	m_onOpened	(this,  "onopened",	aList),
	m_onClient	(this,  "onclient",	aList, KAF_EVCS     ),
	m_onUnload	(this,  "onunload",	aList),
	m_onClose	(this,  "onclose",	aList),
	m_local		(this,  "local",	aList),
	m_uuid		(this,	"uuid",		aList),
	m_docRoot	(this,  m_children,	location  )
{
	m_root		= this	;
	m_display	= 0	;
	m_formCurItem	= 0	;
	m_formCurQRow	= 0	;
	m_accel		= 0	;
	m_dcopIface	= new KBDCOPObject (this, makeDCOPName (m_name.getValue())) ;

	m_geom.set	(KBAttrGeom::FMFixed, KBAttrGeom::FMFixed) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;


	if (!formPropDlg (0))
	{	_ok	= false ;
		return	;
	}

	if (!KBBlock::propertyDlg (0))
	{	_ok	= false ;
		return	;
	}

	_ok	= true	;
	KBLayout::setChanged ()	;
}
#endif

/*  KBForm								*/
/*  ~KBForm	: Destructor for form root class			*/
/*  (returns)	:		:					*/

KBForm::~KBForm ()
{
	showMonitor ((RKListView *)0) ;
	delete	m_dcopIface ;
}

/*  KBForm								*/
/*  moveFocusOK	: Query focus movement					*/
/*  item	: KBItem *	: Target item				*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Focus allowed				*/

bool	KBForm::moveFocusOK
	(	KBItem	*item,
		uint	qrow
	)
{
#if	__KB_DEBUG_FOCUS
	fprintf
	(	kbDPrintfGetStream(),
		"KBForm::moveFocusOK: %s: [%p][%p][%d]->[%p][%d]\n",
		(cchar *)getName(),
		(void  *)m_formCurItem,
		(void  *)item->getFormBlock()->getCurItem(),
		m_formCurQRow,
		(void  *)item,
		qrow
	)	;
#endif
	if (m_docRoot.isLoading())
		return	false ;

	/* If the item gaining focus is the current item and the same	*/
	/* row (for instance after a dialog box has been displayed)	*/
	/* then there are no problems.					*/
	if ((item == m_formCurItem) && (qrow == m_formCurQRow))
		return	true  ;

	/* If there is no current item then again no problems.		*/
	if (m_formCurItem == 0)
		return	true  ;

	/* Control is moving out of an item, so we need to check that	*/
	/* this is OK. Call the block which owns the item to verify	*/
	/* this.							*/
	KBFormBlock *cBlk = m_formCurItem->getFormBlock() ;
	KBFormBlock *nBlk = item         ->getFormBlock() ;

#if	__KB_DEBUG_FOCUS
	fprintf
	(	kbDPrintfGetStream(),
		"KBForm::moveFocusOK: %s: cBlk=%p crow=%d nBlk=%p nRow=%d\n",
		(cchar *)getName(),
		(void  *)cBlk,
		m_formCurQRow,
		(void  *)nBlk,
		qrow
	)	;
#endif

	if (!cBlk->focusOutOK ((qrow != m_formCurQRow) || (nBlk != cBlk)))
		return	false  ;

//**
//**	m_formCurItem = 0 ;
//**	m_formCurQRow = 0 ;
//**
//**	if (nBlk != cBlk) nBlk->enterBlock(false, 0) ;
//**
	return	true	;
}

/*  KBForm								*/
/*  focusInEvent: Note new item focus					*/
/*  item	: KBItem *	: Item which has gained focus		*/
/*  qrow	: uint		: Query row number			*/
/*  (return)	: void		:					*/

void	KBForm::focusInEvent
	(	KBItem	*item,
		uint	qrow
	)
{
	KBFormBlock *cBlk = m_formCurItem == 0 ? 0 : m_formCurItem->getFormBlock() ;
	KBFormBlock *nBlk = item->getFormBlock() ;
	bool	    delta = (m_formCurItem != item) || (m_formCurQRow != qrow) ;

	DPRINTF
	((	"KBForm::focusInEvent: %s: [%s][%d]\n",
		(cchar *)getName(),
		(cchar *)item->getName(),
		qrow
	))	;


//**
//**	if ((m_formCurItem != 0) && (m_formCurItem->getBlock() != item->getBlock()))
//**		m_formCurItem->getBlock()->setCurrent(false) ;
//**
	if (nBlk != cBlk)
	{	nBlk->enterBlock(false, 0) ;
		if (cBlk != 0) cBlk->setCurrent(false) ;
	}

	DPRINTF
	((	"KBForm::focusInEvent: %s: now at [%s][%s][%d]\n",
		(cchar *)getName(),
		(cchar *)item->getElement(),
		(cchar *)item->getName(),
		qrow
	))	;

	m_formCurItem	= item	;
	m_formCurQRow	= qrow	;

	item->getBlock()->setCurrent(true) ;

	if (delta) item->doEnter (m_formCurQRow) ;
}


/*  KBForm								*/
/*  printNode	: Output node XML					*/
/*  text	: QString &	: Resulting text			*/
/*  indent	: int		: Indent level of this node		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBForm::printNode
	(	QString		&text,
		int		indent,
		bool		flat
	)
{
	/* Implement this specially, since we need to output the XML	*/
	/* header. Also output script stuff and paramaters near the	*/
	/* front.							*/
	extern	QString	kbXMLEncoding  () ;
	QString		nodeText ;

	text	+= QString
		   (	"<?xml version=\"1.0\" encoding=\"%1\"?>\n"
		   )
		   .arg	(kbXMLEncoding()) ;

	text	+= QString("%1<%2").arg("", indent).arg(getElement()) ;
	for (uint idx1 = 0 ; idx1 < m_attribs.count () ; idx1 += 1)
	{
		if (flat && (m_attribs.at(idx1)->getName() == "rowcount"))
		{
			uint	rcnt	= m_rowcount.getIntValue() ;
			uint	rows	= rowsInBlock()	;
			uint	showAll	= rcnt & 0x8000	;
			if (rows == 0) rows = 1 ;
			text += QString(" rowcount=\"%1\"" ).arg(rows | showAll) ;
			continue ;
		}
		m_attribs.at(idx1)->printAttr(text, nodeText, indent + 2, flat) ;
	}

	if (flat)
		text += QString(" appfont=\"%1\"" ).arg(KBFont::fontToSpec(qApp->font())) ;

	text	+= ">\n" ;

	CITER	(Script, s, s->printNode (text, indent + 2, flat))
	CITER	(Param,  p, p->printNode (text, indent + 2, flat))

	LITER
	(	KBNode,
		m_children,
		child,
		{
			if (child->isScript ()) continue ;
			if (child->isParam  ()) continue ;
			child->printNode (text, indent + 2, flat)  ;
		}
	)

	for (uint idx2 = 0 ; idx2 < m_slotList.count () ; idx2 += 1)
		m_slotList.at(idx2)->printNode (text, indent + 2, flat) ;
	for (uint idx3 = 0 ; idx3 < m_testList.count () ; idx3 += 1)
		m_testList.at(idx3)->printAttr (text, nodeText, indent + 2, flat) ;

	text	+= nodeText ;
	text	+= QString("%1</%2>\n").arg("", indent).arg(getElement()) ;
}


/*  KBForm								*/
/*  requery	: Rerun form queries					*/
/*  (returns)	: bool		  : Success				*/

bool	KBForm::requery ()
{
	uint	cdRow	= m_curDRow 	;
	uint	cqRow	= m_curQRow 	;

	DPRINTF
	((	"KBForm::requery: cdRow=%d cqRow=%d\n",
		cdRow,
		cqRow
	))	;

	if (!KBFormBlock::requery ())
		return	false ;

	DPRINTF
	((	"KBForm::requery: done requery\n"
	))	;

	if (!KBFormBlock::showData(KBBlock::SDBackToTop|KBBlock::SDRedisplay))
		return	false ;

	DPRINTF
	((	"KBForm::requery: done showData\n"
	))	;

	scrollToRow (cdRow) ;
	enterBlock  (true, cqRow ) ;

	KBValue	arg ((int)m_curQRow) ;
	KBValue	resval	;
	bool	evRc	;

	if (!eventHook (m_events->onCurrent, 1, &arg, evRc))
		return false ;

	/* Success .....						*/
	return	true	;
}

/*  KBForm								*/
/*  finish	: End form operation					*/
/*  (returns)	: void		:					*/

void	KBForm::finish ()
{
	KBValue		resval	;
	KBScriptError	*rc	= m_onUnload.execute (resval, 0, 0) ;

	if (rc != 0)
		KBScriptError::processError (rc, KBScriptError::Form) ;
}

/*  KBForm								*/
/*  resize	: Handle resize in design mode				*/
/*  size	: QSize		: Raw size				*/
/*  (returns)	: void		:					*/

void	KBForm::resize
	(	QSize	size
	)
{
#if	! __KB_RUNTIME
	if (showingDesign())
	{
		KBFormBlock::setGeometry(QRect(QPoint(0, 0), size)) ;
		KBLayout   ::setChanged () ;
	}
#endif
}

/*  KBForm								*/
/*  formAction	: Handle various query actions				*/
/*  action	: KBAction	: Action required			*/
/*  (returns)	: bool		: Success				*/

bool	KBForm::formAction
	(	KB::Action	action
	)
{
	/* This routine is called by the parent viewer in response to	*/
	/* or menu or toolbar request. With the exception of the Save	*/
	/* action. the action is passed to the block in which the	*/
	/* current item resides; if there is no such item then apply	*/
	/* the action to the top-level block.				*/
	KBFormBlock	*block	;

	DPRINTF
	((	"KBForm::formAction: action=%d form=%p fci=%p blk=%p\n",
		(int)action,
		(void *)this,
		(void *)m_formCurItem,
		m_formCurItem == 0 ? 0 : (void *)m_formCurItem->getFormBlock()
	))	;

	if ((action != KB::Save) && (m_formCurItem != 0))
		if ((block = m_formCurItem->getFormBlock()) != 0)
		{
			bool	rc = block->doAction
					(	action,
						m_formCurItem->getNavigator()->getTabList()
					)	;

			/* If there was an error and the action is	*/
			/* being applied to other than this block then	*/
			/* we need to propogate the error back here.	*/
			if (!rc && (block != this))
				setError (block->lastError()) ;

			return	rc ;
		}

	return	doAction (action) ;
}

/*  KBForm								*/
/*  showData	: Show form in data mode				*/
/*  parent	: QWidget *	  : Parent widget			*/
/*  pDict	: QDict<QString>& : Paramater dictionary		*/
/*  key		: const KBValue	& : Parent key				*/
/*  size	: QSize &	  : Return required size		*/
/*  (returns)	: KB::ShowRC	  : Startup return code			*/

KB::ShowRC
	KBForm::showData
	(	QWidget			*parent,
		const QDict<QString>	&pDict,
		const KBValue		&key,
		QSize			&size
	)
{
	/* In the event of any error while starting up, we switch to	*/
	/* design mode, and indicate this to the called via the return	*/
	/* code.							*/
	KBScriptError	*eRc	;
	KB::ShowRC	sRc	;
	KBError		error	;
	KBValue		resval	;

	m_formCurItem	= 0	;
	m_formCurQRow	= 0	;

	m_parentKey	= key	;
	DPRINTF
	((	"KBForm::showData: key=[%s]\n",
		(cchar *)m_parentKey.getRawText()
	))	;

	/* Reset the document root object (which, for instance, clears	*/
	/* an extant scripting interface).				*/
	m_docRoot.reset	() ;

	/* We can now set the parameter dictionary (which may invoke	*/
	/* scripts for default values). This may return indicating an	*/
	/* error or that the user cancelled.				*/
	sRc = m_docRoot.setParamDict (pDict, error) ;
	if (sRc != KB::ShowRCOK)
	{
		if (sRc == KB::ShowRCCancel)
			return	sRc ;

		setError (error)    ;
		goto	 showError  ;
	}


	/* Next run the block setup. This may pick up parameter values	*/
	/* that are now set.						*/
	if (!blockSetup ())
		goto	showError   ;


	/* If this is the first time the form is displayed, create the	*/
	/* top-level display and build the form GUI.			*/
	if (m_display == 0)
	{
		m_display = KBDisplay::newTopDisplay
			    (		parent,
					this,
					m_showbar.getFlags(),
					m_stretch.getFlags(),
					false
			    )	;

		buildTopDisplay (m_display) ;
	}

	if (m_accel != 0)
	{
		m_accel  ->clear () ;
		m_accelMap.clear () ;
	}
	else
	{
		m_accel	= new QAccel (getDisplay()->getTopWidget()) ;

		connect
		(	m_accel,
			SIGNAL(activated  (int)),
			this,
			SLOT  (accelerator(int))
		)	;
	}

	/* Initialize the layout object and set data mode. We can then	*/
	/* pick up the size required by the form			*/
#if	! __KB_RUNTIME
	KBLayout   ::clear      (false) ;
	KBLayout   ::initSizer  ()	;
#endif
	KBFormBlock::showAs 	(KB::ShowAsData) ;

	size	 = sizeHint () ;

	DPRINTF
	((	"KBForm::showData: sizeNeeded (%d,%d) (%d,%d) -> (%d,%d)\n",
		m_geom.w(),
		m_geom.h(),
		geometry().size().width (),
		geometry().size().height(),
		size.width (),
		size.height()
	))	;

	m_display->resizeContents     (geometry().size()) ;
	m_display->setDisplayGeometry (geometry()) ;

	if (!connectLinks (error))
	{
		setError (error)   ;
		goto	 showError ;
	}


	/* Add all items in the form into the queries ...		*/
	if (!addAllItems())
	{
		setError
		(	KBError::Error,
			TR
			(	"Form contains blocks which retrieve no values"
			),
			TR
			(	"At least one field in each block should "
				"have a non-empty display expression"
			),
			__ERRLOCN
		)	;
		goto	showError ;
	}

	/* Scripting is up and ready if required, and blocks know	*/
	/* what is going on, so start the form off by executing the	*/
	/* onLoad event, if any.					*/
	eRc	= m_onLoad.execute (resval, 0, 0) ;
	if (eRc != 0)
	{
		KBScriptError::processError (eRc, KBScriptError::Form) ;
		goto	showError ;
	}

	/* Prepare all controls prior to displaying any data. This	*/
	/* will be used by links, for instance, to load their value	*/
	/* sets.							*/
	prepare () ;
	if (!requery ())
		goto	showError ;

	eRc	= m_onOpened.execute (resval, 0, 0) ;
	if (eRc != 0)
	{
		KBScriptError::processError (eRc, KBScriptError::Form) ;
		goto	showError ;
	}

	eRc	= m_onClient.execute (resval, 0, 0) ;
	if (eRc != 0)
	{
		KBScriptError::processError (eRc, KBScriptError::Form) ;
		goto	showError ;
	}

	emit	sigShowingAs (KB::ShowAsData) ;
	return	KB::ShowRCData ;

	showError :

#if	! __KB_RUNTIME

		if (showDesign (parent, size) == KB::ShowRCDesign)
			return	KB::ShowRCDesign  ;
#endif
		return	KB::ShowRCError	  ;
}

#if	! __KB_RUNTIME

/*  KBForm								*/
/*  showPreview	: Show form preview					*/
/*  parent	: QWidget *	  : Parent widget			*/
/*  size	: QSize &	  : Return required size		*/
/*  (returns)	: KB::ShowRC	  : Startup return code			*/

KB::ShowRC
	KBForm::showPreview
	(	QWidget			*parent,
		QSize			&size
	)
{

	m_docRoot.reset	() ;

	if (!blockSetup ())
		return	KB::ShowRCError	;


	if (m_display == 0)
	{
		m_display = KBDisplay::newTopDisplay
			    (		parent,
					this,
					m_showbar.getFlags(),
					m_stretch.getFlags(),
					false
			    )	;

		buildTopDisplay (m_display) ;
	}


	KBLayout   ::clear      (false) ;
	KBLayout   ::initSizer  ()	;
	KBFormBlock::showAs 	(KB::ShowAsData) ;

	size   = geometry().size() ;

	m_display->resizeContents (size) ;

	m_formCurItem	= 0	;
	m_formCurQRow	= 0 	;
	return	KB::ShowRCData 	;
}


/*  KBForm								*/
/*  showDesign	: Show form in design mode				*/
/*  parent	: QWidget *	  : Parent widget			*/
/*  size	: QSize &	  : Return required size		*/
/*  (returns)	: KB::ShowRC	  : Startup return code			*/

KB::ShowRC
	KBForm::showDesign
	(	QWidget		*parent,
		QSize		&size
	)
{
	emit	sigShowingAs (KB::ShowAsDesign) ;

	m_formCurItem	= 0	;
	m_formCurQRow	= 0	;

	/* Start with the block setup. This links blocks to queries and	*/
	/* such. Any error here is a real problem.			*/
	if (!blockSetup ())
		return	KB::ShowRCError  ;

	/* If this is the first time the form is displayed, create the	*/
	/* top-level display and build the form GUI.			*/
	if (m_display == 0)
	{
		m_display = KBDisplay::newTopDisplay
			    (		parent,
					this,
					m_showbar.getFlags(),
					m_stretch.getFlags(),
					false
			    )	;

		buildTopDisplay (m_display) ;
	}

	KBLayout   ::clear      (true)  ;
	KBLayout   ::initSizer  ()	;
	KBFormBlock::showAs 	(KB::ShowAsDesign) ;

	size	 = sizeHint () ;
	size	+= QSize (100, 100) ;

	m_display->resizeContents     (QSize(1600, 1600)) ;
	m_display->setDisplayGeometry (geometry()) ;

	KBLayout::setChanged (false);
	return	KB::ShowRCDesign ;
}

/*  KBForm								*/
/*  contextMenuType							*/
/*		: Return type text for context menu			*/
/*  (returns)	: QString	: Text					*/

QString	KBForm::contextMenuType ()
{
	return	TR("Form") ;
}

#endif

/*  KBForm								*/
/*  setFocusAtRow: Set focus row information for current block		*/
/*  block	 : KBFormBlock *: Block in question			*/
/*  (returns)	 : void		:					*/

void	KBForm::setFocusAtRow
	(	KBFormBlock		*block
	)
{
#if	__KB_DEBUG_FOCUS
	fprintf
	(	kbDPrintfGetStream(),
		"KBForm::setFocusAtRow: %s: fi=%d cr=%d nr=%d inq=%d\n",
		(cchar *)getName (),
		block->firstItem () != 0,
		block->getCurQRow(),
		block->getNumRows(),
		block->isInQuery ()
	)	;
#endif
	emit	focusAtRow
		(	block->firstItem () != 0,
			block->getCurQRow(),
			block->getNumRows(),
			block->isInQuery ()
		)	;

}

/*  KBForm								*/
/*  objType	: Get object type for this node				*/
/*  (returns)	: KB::ObjType	: Object type				*/

KB::ObjType KBForm::objType ()
{
	return	KB::ObjForm	;
}

/*  KBForm								*/
/*  queryClose	: Query close						*/
/*  (returns)	: bool		: True to allow close			*/

bool	KBForm::queryClose ()
{
	/* Actually what this does is to execute the close event, if	*/
	/* any, and return the value from that.				*/
	if (!m_onClose.getValue().isEmpty())
	{
		bool	evRc 	;

		/* If the event itself throws an error then return true	*/
		/* otherwise it would be impossible to close a form	*/
		/* with a duff onClose event.				*/
		if (!eventHook (m_onClose, 0, 0, evRc)) return true ;

		return	evRc	;
	}

	return	true	;
}

/*  KBForm								*/
/*  getBlockVal	: Get block value pointer				*/
/*  (returns)	: KBValue *	: Value pointer or null at top level	*/

KBValue	*KBForm::getBlockVal ()
{
	if (!m_cexpr.getValue().isEmpty() && !m_parentKey.isEmpty())
	{
		DPRINTF
		((	"KBForm::getBlockVal: returns [%s]\n",
			(cchar *)m_parentKey.getRawText()
		))	;
		return	&m_parentKey ;
	}

	return	0	;
}

/*  KBFormBlock								*/
/*  focusInBlock: See if focus is in specified block			*/
/*  block	: KBBlock *	: Block in question			*/
/*  (returns)	: bool		: True if in block			*/

bool	KBForm::focusInBlock
	(	KBBlock		*block
	)
{
	return	(m_formCurItem != 0) && (m_formCurItem->getFormBlock() == block) ;
}

void	KBForm::addAccelerator
	(	const QKeySequence	&keySeq,
		KBItem			*item
	)
{
	if (m_accel != 0)
	{
		static	int	accelMapID ;

		accelMapID += 1	;
		m_accel->insertItem (keySeq, accelMapID) ;

		m_accelMap.insert (accelMapID, item) ;
	}
}

void	KBForm::accelerator
	(	int		id
	)
{
	KBItem	*item	= m_accelMap.find (id) ;
	if (item == 0)
	{
		DPRINTF
		((	"KBForm::accelerator: no mapping for id=%d\n",
			id
		))	;
		return	;
	}

	KBControl *ctrl = item->ctrlAtQRow (m_formCurQRow) ;
	if (ctrl == 0)
	{
		DPRINTF
		((	"KBForm::accelerator: row out of range %d->%d+%d\n",
			m_formCurQRow,
			item->getBlock()->getCurDRow	(),
			item->getBlock()->getDisplayRows()
		))	;
		return	;
	}

	ctrl->setFocus () ;
}


/*  KBObject	:							*/
/*  sizeHint	: Get ideal size for object				*/
/*  (returns)	: QSize		: Size					*/

QSize	KBForm::sizeHint ()
{
	/* If the form is dynamic then get the minimum size, but expand	*/
	/* it to the geometry, hence porividing a minimum size.		*/
	if (isDynamic())
		return	minimumSize().expandedTo(m_geom.geometry(true).size()) ;

	return	m_geom.geometry(true).size() ;
}

/*  KBForm								*/
/*  getGlobalNode: Get globally named node				*/
/*  name	 : const QString & : Node name				*/
/*  (returns)	 : KBNode *	   : Node or null if not found		*/

KBNode	*KBForm::getGlobalNode
	(	const QString	&name
	)
{
	return	m_docRoot.getNamedNode (name) ;
}

#if	! __KB_RUNTIME

/*  KBForm								*/
/*  formPropDlg	: Run form-level property dialog			*/
/*  iniAttr	: cchar *	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBForm::formPropDlg
	(	cchar	*iniAttr
	)
{
	QPtrList<KBModule>	scripts	 ;
	QPtrList<KBModule>	scripts2 ;
	QPtrList<KBModule>	imports	 ;
	QPtrList<KBParam > 	params   ;

	QString		oldSkin	 = m_skin.getValue() ;
	bool		creating = false ;

	if (getBlkType() == KBBlock::BTUnknown)
	{
		bool		ok ;
		KBFormInitDlg	fiDlg (ok) ;
		if (!ok || !fiDlg.exec()) return false ;

		m_blkType = fiDlg.toplevel() ;
		m_geom.set	    (fiDlg.management(), 2, 2) ;
		m_language.setValue (fiDlg.language  ()) ;

		creating  = true ;
	}

//	KBAttrStr	mattr	(this, "modlist",    "", KAF_SYNTHETIC) ;
//	KBAttrStr	m2attr	(this, "modlist2",   "", KAF_SYNTHETIC) ;
//	KBAttrStr	iattr	(this, "implist",    "", KAF_SYNTHETIC) ;
//	KBAttrStr	pattr	(this, "paramlist",  "", KAF_SYNTHETIC) ;

	CITER
	(	Script,
		s,
		if (s->isL2())
			scripts2.append (s) ;
		else	scripts .append (s) ;
	)
	CITER
	(	Import,
		i,
		imports.append (i)
	)
	CITER
	(	Param,
		p,
		params .append (p)
	)

	if (!::formPropDlg
			(	this,
				"Form",
				m_attribs,
				iniAttr
			))
		return	false ;

	if (creating)
	{
		bool	rc = false ;

		switch (getBlkType())
		{
			case KBBlock::BTNull	:
			case KBBlock::BTTable	:
			case KBBlock::BTSQL	:
			case KBBlock::BTQuery	:
				rc	= setBlkType (getBlkType()) ;
				break	;

			default	:
				break	;
		}

		if (!rc) return false ;
	}

	DPRINTF
	((	"KBForm::formPropDlg: m_blkDisp=%p creating=%d m_skin=%s oldSkin=%s\n",
		(void *)m_blkDisp,
		creating,
		m_skin.getValue().latin1(),
		oldSkin.latin1()
	))	;

	if (m_blkDisp)
		m_blkDisp->setStretchable (m_stretch.getFlags()) ;

	if (!creating && (m_skin.getValue() != oldSkin))
		m_docRoot.skinChanged () ;

	KBLayout::setChanged ()	;
	return	true	;
}
#endif
