/***************************************************************************
    file	         : kb_form.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_FORM_H
#define		__KB_FORM_H

#include	<qaccel.h>
#include	<qintdict.h>
#include	<qguardedptr.h>

#include	"libkbase_exports.h"

#include	"kb_formblock.h"
#include	"kb_layout.h"
#include	"kb_docroot.h"
#include	"kb_attrlanguage.h"
#include	"kb_attrstretch.h"
#include	"kb_attrskin.h"


class	KBDCOPObject	;

/*  KBForm								*/
/*  ----------								*/
/*  This node is the root of a particular XML form definition. It	*/
/*  handles things which are global to the form, such as field focus	*/
/*  and field movement and resizing, as well as representing the top-	*/
/*  most block.								*/

class	LIBKBASE_API	KBForm : public KBFormBlock, public KBLayout
{
	Q_OBJECT

	KBAttrLanguage		m_language	;
	KBAttrLanguage		m_language2	;

	KBAttrStr		m_caption	;
	KBAttrStretch		m_stretch	;
	KBAttrSkin		m_skin		;
	KBAttrBool		m_modal		;
	KBAttrBool		m_hideBars	;
	KBAttrBool		m_hideStatus	;

	KBEvent			m_onAuth	;	/* FormAuth	*/
	KBEvent			m_onLoad	;	/* PreDatabase	*/
	KBEvent			m_onOpened	;	/* PostDatabase	*/
	KBEvent			m_onClient	;	/* ClientSide	*/
	KBEvent			m_onUnload	;
	KBEvent			m_onClose	;
	KBEvent			m_local		;

	KBAttrStr		m_uuid		;

	KBDocRoot		m_docRoot	;
	KBDisplay		*m_display	;

	KBItem			*m_formCurItem	;
	uint			m_formCurQRow	;

	KBValue			m_parentKey	;

	QAccel			*m_accel	;
	QIntDict<KBItem>	m_accelMap	;
	QGuardedPtr<KBNode>	m_opener	;
	KBDCOPObject		*m_dcopIface	;

protected slots :

	void			accelerator	(int)	;

signals :

	void			focusAtRow	(bool, uint, uint, bool) ;
	void			sigShowingAs	(KB::ShowAs)		 ;

public	:

	KBForm	(const KBLocation &, const QDict<QString> &) ;
	KBForm	(const KBLocation &, const QDict<QString> &, bool &) ;
virtual~KBForm	() ;


#if	! __KB_RUNTIME
	bool			formPropDlg	(cchar *) ;
	virtual	QString		contextMenuType	()	  ;
#endif

	virtual	KBForm		*isForm     () 	{ return this        ; }
	virtual	KBLayout	*isLayout   () 	{ return this        ; }
	virtual	KBDocRoot	*isDocRoot  () 	{ return &m_docRoot  ; }

	virtual	KB::ObjType	objType		() ;
	virtual	QSize		sizeHint	() ;

	virtual	void		printNode	(QString &, int, bool) ;

	bool			formAction      (KB::Action)	    ;
	bool			moveFocusOK	(KBItem    *, uint) ;
	void			focusInEvent	(KBItem    *, uint) ;
	void			resize		(QSize)		    ;

	virtual bool		requery		() ;
	void			finish		() ;
	virtual	KBValue		*getBlockVal	() ;

	void			setFocusAtRow	(KBFormBlock *) ;

#if	! __KB_RUNTIME
	KB::ShowRC		showDesign	(QWidget  *, QSize &) ;
	KB::ShowRC		showPreview	(QWidget  *, QSize &) ;
#endif
	KB::ShowRC		showData	(QWidget  *, const QDict<QString> &, const KBValue &, QSize &) ;
	KB::ShowRC		showPrint	(KBWriter *) ;

	KBNode			*getGlobalNode	(const QString &) ;
	bool			queryClose	() ;
	bool			focusInBlock	(KBBlock  *) ;

	inline	KBDisplay	*getDisplay	()
	{
		return	m_display ;
	}

	inline	bool	hideMenuToolBars()
	{
		return	m_hideBars.getBoolValue () ;
	}

	inline	bool	hasStatusBar	()
	{
		return	! m_hideStatus.getBoolValue () ;
	}

	inline	bool	enforceMinSize	()
	{
		return	m_stretch.getFlags() != STR_ANY ;
	}

	inline	void	focusAtRow
		(	uint	qrow
		)
	{
		m_formCurQRow = qrow ;
	}

	inline	void	setOpener
		(	KBNode	*opener
		)
	{
		m_opener = opener	;
	}

	inline	KBNode	*opener	()
	{
		return	m_opener	;
	}

	void	addAccelerator	(const QKeySequence &, KBItem *) ;

	friend	class		KBTestAccessor	;
	friend	class		KBFormBlock	;
}	;

#if	! __KB_RUNTIME
extern	LIBKBASE_API	bool	formPropDlg
			(	KBForm			*,
				cchar			*,
				QPtrList<KBAttr  >	&,
				cchar			* = 0
			)	;
#endif
#endif	// __KB_FORM_H
