/***************************************************************************
    file	         : kb_formblock.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#if		__KB_AUTOMOC
#include	"kb_formblock.moc"
#else
#include	"kb_formblock.h"
#endif

#include	"kb_form.h"
#include	"kb_framer.h"
#include	"kb_layout.h"
#include	"kb_qrybase.h"
#include	"kb_hidden.h"
#include	"kb_rowmark.h"
#include	"kb_display.h"
#include	"kb_nodereg.h"
#include	"kb_popupmenu.h"
#include	"kb_blockevents.h"
#include	"kb_attrdict.h"
#include	"kb_grid.h"
#include	"kb_designpopup.h"
#include	"kb_recordnav.h"
#include	"kb_component.h"
#include	"kb_ascii.h"

#if		! __KB_RUNTIME
#include	"kb_wizard.h"
#endif


#define		__DEBUG_FORMBLOCK	(0)
#if		__DEBUG_FORMBLOCK
#include	"kb_dprintf.h"
#endif


/*  KBFormBlock								*/
/*  KBFormBlock	: Constructor for extant form block element		*/
/*  parent	: KBNode *	: Parent element			*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  element	: cchar *	: Element name				*/
/*		: bool *	:					*/
/*  (returns)	: KBFormBlock	:					*/

KBFormBlock::KBFormBlock
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		cchar			*element,
		bool			*
	)
	:
	KBBlock	   	(parent, aList,		element),
	KBNavigator	(this,   this,		m_children),
	m_sloppy	(this,   "sloppy",	aList),
	m_blkrdonly	(this,   "blkrdonly",	aList),
	m_tabsWrap	(this,   "tabswrap", 	aList),
	m_locking	(this,	 "locking",	aList, KAF_GRPDATA),
	m_exportRS	(this,	 "exportrs",	aList, KAF_GRPWEB ),
	m_noRestore	(this,	 "norestore",	aList, KAF_GRPWEB )
{
	m_curItem  = 0	     ;
	m_inQuery  = false   ;
	m_dChanged = false   ;
	m_prepared = false   ;
	m_maxRows  = 1	     ;

	if ((m_dx.getIntValue() == 0) && (m_dy.getIntValue() == 0))
			m_dy.setValue (25) ;
}

/*  KBFormBlock								*/
/*  KBFormBlock	: Constructor for new form block element		*/
/*  parent	: KBObject *	: Parent element			*/
/*  rect	: const QRect &	: Size and position			*/
/*  blkType	: BlkType	: Initial block type			*/
/*  ok		: bool &	: Success				*/
/*  element	: cchar *	: Element name				*/
/*  (returns)	: KBFormBlock	:					*/

KBFormBlock::KBFormBlock
	(	KBObject	*parent,
		const QRect	&rect,
		BlkType		_blkType,
		bool		&ok,
		cchar		*element
	)
	:
	KBBlock		(parent, rect, 	  	_blkType,  ok, element),
	KBNavigator	(this,   this,	  	m_children),
	m_sloppy	(this,   "sloppy",	false),
	m_blkrdonly 	(this,   "blkrdonly",	false),
	m_tabsWrap	(this,   "tabswrap",	false),
	m_locking	(this,	 "locking",	(uint)0, KAF_GRPDATA),
	m_exportRS	(this,	 "exportrs",	"",   	 KAF_GRPWEB ),
	m_noRestore	(this,	 "norestore",	false,   KAF_GRPWEB )
{
	m_curItem  = 0	     ;
	m_inQuery  = false   ;
	m_dChanged = false   ;
	m_prepared = false   ;
	m_maxRows  = 1	     ;

	if (parentIsDynamic())
		m_geom.set (KBAttrGeom::MgmtDynamic, 2, 2) ;

#if	! __KB_RUNTIME
	if (ok)
	{
		if (parentIsDynamic())
			m_geom.set (KBAttrGeom::MgmtDynamic, 2, 2) ;

		if (!propertyDlg (0))
		{	ok	= false	;
			return	;
		}
	}
#endif
}

/*  KBFormBlock								*/
/*  KBFormBlock	: Constructor for extant form block element		*/
/*  _parent	: KBNode *	: Parent element			*/
/*  _block	: KBFormBlock *	: Extant block				*/
/*  (returns)	: KBFormBlock	:					*/

KBFormBlock::KBFormBlock
	(	KBNode		*parent,
		KBFormBlock	*block
	)
	:
	KBBlock	   	(parent,  block),
	KBNavigator	(this,    this,		m_children),
	m_sloppy	(this,    "sloppy",	block),
	m_blkrdonly 	(this,    "blkrdonly",	block),
	m_tabsWrap	(this,    "tabswrap",	block),
	m_locking	(this,	  "locking",	block, KAF_GRPDATA),
	m_exportRS	(this,	  "exportrs",	block, KAF_GRPWEB ),
	m_noRestore	(this,	  "norestore",	block, KAF_GRPWEB )
{
	m_curItem  = 0	     ;
	m_inQuery  = false   ;
	m_dChanged = false   ;
	m_prepared = false   ;
	m_maxRows  = 1	     ;
}

/*  KBFormBlock								*/
/*  ~KBFormBlock: Destructor for form block element			*/
/*  (returns)	: void		:					*/

KBFormBlock::~KBFormBlock ()
{
}

/*  KBFormBlock								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBFormBlock::showAs
	(	KB::ShowAs	mode
	)
{
	m_dChanged = false	;
	m_prepared = false	;
	KBBlock::showAs (mode)	;

	if (mode == KB::ShowAsData) 
	{
		setupTabOrder   () ;
		setupGridLayout	() ;
		m_curItem = 0	   ;
		m_inQuery = false  ;
	}

	if (m_blkDisp)
		m_blkDisp->setTitle (m_title.getValue()) ;
}

/*  KBFormBlock								*/
/*  prepare	: Prepare for form execution				*/
/*  (returns)	: void		:					*/

void	KBFormBlock::prepare ()
{
	/* Set the prepared flage, see the "displayData" data for	*/
	/* comments on this						*/
	m_prepared = true	;
	KBBlock::prepare ()	;
}

/*  KBFormBlock								*/
/*  newTabOrder	: Invoke tab order dialog				*/
/*  (returns)	: void		:					*/

void	KBFormBlock::newTabOrder ()
{
#if	! __KB_RUNTIME
	if (KBNavigator::newTabOrder()) getLayout()->setChanged() ;
#endif
}

/*  KBFormBlock								*/
/*  checkChange	: Check for field changes				*/
/*  doLeaveEvent: bool		: Do leave event on current item	*/
/*  rowDelta	: bool &	: Row was changed and saved		*/
/*  verify	: bool		: Verify controls are valid		*/
/*  (returns)	: bool		: Success (rather, no error)		*/

bool	KBFormBlock::checkChange
	(	bool		doLeaveEvent,
		bool		&rowDelta,
		bool		verify
	)
{
	/* NOTE:							*/
	/* On exit from this method, any outstanding transaction will	*/
	/* have been terminated if required (eg., for row update	*/
	/* locking). This is handled locally or within the query	*/
	/* depending on circumstances.					*/

	bool	changed	= false ;

	rowDelta = false  ;

	/* If there is a current item and requested to do so, then	*/
	/* execute the leave event on that item.			*/
	/* FIX: Need to have a tri-state return from checkChange, which	*/
	/*	can now be OK, Error, or Cancel				*/
	if (doLeaveEvent && (m_curItem != 0))
		if (!m_curItem->doLeave (m_curQRow))
		{
			setError
			(	KBError::Warning,
				TR("Leave event cancelled operation"),
				QString::null,
				__ERRLOCN
			)	;

			/* Note: any transaction is *not* terminated	*/
			/* since this may just be the application	*/
			/* saying a value is not allowed.		*/
			return	false	;
		}

	/* If this is the new row at the end of the query and all	*/
	/* fields are empty, then focus can move out. We just need to	*/
	/* end any transaction, since there is no actual server action.	*/
	if (m_query->newRowEmpty (m_qryLvl, m_curQRow))
	{
		return	endUpdate (true) ;
	}

	/* Scan all items in the current row to see if any of them	*/
	/* have changed ....						*/
	CITER
	(	Item,
		i,

		DPRINTF
		((	"KBFormBlock::checkChange: [%s]\n",
			i->getName().latin1()
		))	;

		if (i->isUpdateVal (true) && i->changed(m_curQRow))
		{
			changed	= true	;
			DPRINTF
			((	"KBFormBlock::checkChange: [%s] ... changed\n",
				i->getName().latin1()
			))	;
			break	;
		}
	)

	if (!changed)
		CITER
		(	Framer,
			f,
			if (f->changed(m_curQRow))
			{
				changed	= true	;
				break	;
			}
		)

	DPRINTF
	((	"KBFormBlock::checkChange: %s: changed=%d\n",
		getName().latin1(),
		changed
	))	;

	/* If nothing has changed then do nothing, except, clear the	*/
	/* dChange flag, as the user might have changed a field like	*/
	/*	mike -> mik -> mike					*/
	/* and also commit any outstanding transaction.			*/
	if (!changed)
	{
		m_dChanged = false ;
		return	endUpdate  (true) ;
	}

	/* Changes have been made, so run either the pre-update hook	*/
	/* or, this is an inserted row, the pre-update hook. After this	*/
	/* we can save into the associated query structure whatever is	*/
	/* in the fields.						*/
	KBValue	aCurQRow ((int)m_curQRow, &_kbFixed) ;
	KBEvent	*hook	 ;
	bool	evRc	 ;

	if ( (m_query->getRowState(m_qryLvl, m_curQRow) == KB::RSInserted) ||
	     (m_query->getNumRows (m_qryLvl)	        <= m_curQRow) )
		hook	= &m_events->preInsert ;
	else	hook	= &m_events->preUpdate ;

	if (!eventHook (*hook, 1, &aCurQRow, evRc))
	{	endUpdate (false) ;
		return	  false	  ;
	}

	if (!evRc)
	{
		/* False simply means the insert/update does not	*/
		/* proceed, so the transaction state is left unchanged.	*/
		return	true	;
	}

	DPRINTF
	((	"KBFormBlock::checkChange: %s: calling saveRow\n",
		getName().latin1()
	))	;

	if (!m_query->saveRow (m_qryLvl, m_curQRow, verify))
	{
		setError  (m_query->lastError ()) ;
		endUpdate (false) ;
		return	  false	  ;
	}

	/* If autosync is enabled then call the query to synchronise	*/
	/* the row with the database itself.				*/
	if (m_autosync.getBoolValue())
	{
		KBValue		*mvp	= getBlockVal () ;
		KB::Action	oper 	;
		KBValue	  	args[3] ;

		if ((mvp != 0) && mvp->isNull())
		{
			DPRINTF
			((	"KBFormBlock::checkChange: %s: mvp=NULL\n",
				getName().latin1()
			))	;

			setError
			(	KBError::Warning,
				TR("No parent record: cannot save data"),
				QString(TR("In block: %1")).arg(getName()),
				__ERRLOCN
			)	;
			endUpdate (false) ;
			return	  false	  ;
		}

		DPRINTF
		((	"KBFormBlock::checkChange: %s: calling syncRow\n",
			getName().latin1()
		))	;

		/* Synchronise the row. Note that this call will result	*/
		/* in the termination, if needed of any outstanding	*/
		/* transaction, so there is no need beyond this point	*/
		/* to call "endUpdate()".				*/
		if (!m_query->syncRow
			(	m_qryLvl,
				m_curQRow,
				mvp,
				m_cexpr.getValue(),
				this,
				oper,
				args[2]
		   )	)
		{
			setError (m_query->lastError ()) ;
			return	 false ;
		}

		DPRINTF
		((	"KBFormBlock::checkChange: %s: finishing\n",
			getName().latin1()
		))	;

		if (oper != KB::Null)
		{
			args[0]	= (int)m_curQRow;
			args[1]	= (int)oper	;
			if (!eventHook (m_events->postSync, 3, args, evRc))
				return false ;

			m_dChanged = false	;
		}

		getLayout ()->setChanged   (false) ;
		getDocRoot()->doSetLocking (m_query->lockingState(m_qryLvl)) ;
		rowDelta = true ;
	}

	return	true ;
}

/*  KBFormBlock								*/
/*  checkChange	: Check for field changes				*/
/*  doLeaveEvent: bool		: Do leave event on current item	*/
/*  (returns)	: bool		: Success (rather, no error)		*/

bool	KBFormBlock::checkChange
	(	bool	doLeaveEvent
	)
{
	bool	dummy	;
	return	checkChange (doLeaveEvent, dummy, true) ;
}

/*  KBFormBlock								*/
/*  clearFields	: Clear values in fields in row				*/
/*  qrow	: uint		: Query row				*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBFormBlock::clearFields
	(	uint	qrow,
		bool	query
	)
{
	CITER
	(	Item,
		i,
		i->clearValue  (qrow, query)
	)	;
	CITER
	(	Framer,
		f,
		f->clearFields (qrow, query)
	)	;
}

/*  KBFormBlock								*/
/*  displayData	: Display data for range of rows			*/
/*  force	: bool		: Forcibly redisplay all		*/
/*  fromQRow	: uint		: From query row			*/
/*  toQRow	: uint		: To query row (plus one)		*/
/*  (returns)	: void		:					*/

void	KBFormBlock::displayData
	(	bool	force,
		uint	fromQRow,
		uint	toQRow
	)
{
	/* Do nothing if the "m_prepared" flag is not yet set. This is	*/
	/* because we can get a resize event (which will result in this	*/
	/* method being calleed) before form setup is complete.		*/
	if (!m_prepared)
	{
		DPRINTF
		((	"KBFormBlock::displayData: not prepared yet!!\n"
		))	;
		return	;
	}

	uint	saveQRow = m_curQRow ;

	DPRINTF
	((	"KBFormBlock::displayData(%d,%d,%d) (at %d/%d)\n",
		force,
		fromQRow,
		toQRow,
		m_curQRow,
		m_curDRow
	))	;

	/* Redisplay all rows marked as dirty, or all rows if the force	*/
	/* flag is set. Note that the test is such that even if force	*/
	/* is set, any dirty flags will be cleared.			*/
	for (m_curQRow = fromQRow ; m_curQRow < toQRow ; m_curQRow += 1)
	{
		if (m_blkInfo.rowmark != 0)
		{
			m_blkInfo.rowmark->setRowNumber	(m_curQRow)	;
			m_blkInfo.rowmark->setState	(m_curQRow, m_query->getRowState (m_qryLvl, m_curQRow)) ;
		}

		if	(m_curQRow >= m_query->getNumRows (m_qryLvl))
		{
			clearFields (m_curQRow) ;
		}
		else if (m_query->rowIsDirty (m_qryLvl, m_curQRow, true) || force)
		{
			m_query->loadItems (m_qryLvl, m_curQRow) ;
		}

		bool	marked	= m_query->getRowMarked (m_qryLvl, m_curQRow) ;

		CITER
		(	Item,
			i,
			i->setMarked (m_curQRow, marked)
		)

		bool	evRc  ;
		KBValue	rowNo = (int)m_curQRow ;
		eventHook (m_events->onDisplay, 1, &rowNo, evRc) ;
	}

	m_curQRow = saveQRow ;
	if (m_blkInfo.rowmark != 0)
		m_blkInfo.rowmark->setCurrent (m_curQRow) ;

	/* ... and also ensure that the correct row rang is being	*/
	/* displayed.							*/
	uint	extra	= (m_query->getPermission(m_qryLvl) & QP_INSERT) != 0 ? 1 : 0 ;

	m_blkInfo.scroll->setRowRange
	(	m_query->getNumRows(m_qryLvl),
		extra,
		m_curQRow,
		m_curDRow,
		m_numRows
	)	;
}

/*  KBFormBlock								*/
/*  showData	: Display values from query in fields			*/
/*  options	: uint		: Options				*/
/*  (returns)	: bool		: Success				*/

bool	KBFormBlock::showData
	(	uint		options
	)
{
	DPRINTF
	((	"KBFormBlock::showData: %p: p=%p: r=%d s=%d t=%d\n",
		(void *)this,
		(void *)getParent(),
		(options & KBBlock::SDRedisplay  ) != 0,
		(options & KBBlock::SDScrollToRow) != 0,
		(options & KBBlock::SDBackToTop  ) != 0
	))	;
	if ((options & KBBlock::SDBackToTop  ) != 0)
	{
		m_curDRow = 0 ;
		m_curQRow = 0 ;
	}

	/* See if we need to scroll the display backwards or forwards,	*/
	/* in which case we need to force a complete redisplay.		*/
	if ((options & KBBlock::SDScrollToRow) != 0)
	{
		if (m_curDRow > m_curQRow)
		{	m_curDRow = m_curQRow ;
			options	 |= KBBlock::SDRedisplay ;
		}
		if (m_curDRow + m_numRows <= m_curQRow)
		{	m_curDRow = m_curQRow - m_numRows + 1 ;
			options	 |= KBBlock::SDRedisplay ;
		}
	}

	displayData
	(	(options & KBBlock::SDRedisplay) != 0,
		m_curDRow,
		m_curDRow + m_numRows
	)	;

	/* Scan through for nested blocks, requerying and redisplaying	*/
	/* for each. Note that this method should only be called when	*/
	/* we have moved row or changed a row, in which case the	*/
	/* requery/redisplay is required.				*/
	CITER
	(	FormBlock,
		b,
		if (!b->requery ())
		{
			DPRINTF
			((	"KBFormBlock::showData: failed nested requery: %s\n",
				b->lastError().getMessage().latin1()
			))	;
			setError (b->lastError()) ;
			return   false ;
		}
		if (!b->showData (KBBlock::SDBackToTop|KBBlock::SDRedisplay))
		{
			DPRINTF
			((	"KBFormBlock::showData: failed nested show  : %s\n",
				b->lastError().getMessage().latin1()
			))	;
			setError (b->lastError()) ;
			return   false ;
		}
	)
	CITER
	(	Framer,
		f,
		if (!f->showData ())
		{
			DPRINTF
			((	"KBFormBlock::showData: failed nested framer: %s\n",
				f->lastError().getMessage().latin1()
			))	;
			setError (f->lastError()) ;
			return   false ;
		}
	)

	uint	extra	= (m_query->getPermission(m_qryLvl) & QP_INSERT) != 0 ? 1 : 0 ;
	CITER
	(	Item,
		i, 
		i->hideBelow (m_query->getNumRows(m_qryLvl) + extra)
	)
	CITER
	(	Framer,
		f, 
		f->hideBelow (m_query->getNumRows(m_qryLvl) + extra)
	)


	return	true ;
}

/*  KBFormBlock								*/
/*  moveFocusOK	: Query focus movement					*/
/*  item	: KBItem *	: Item which has gained focus		*/
/*  drow	: uint		: Field row number			*/
/*  (return)	: bool		: Focus allowed				*/

bool	KBFormBlock::moveFocusOK
	(	KBItem	*item,
		uint	drow
	)
{
	DPRINTF
	((	"KBFormBlock::moveFocusOK: %s: %s: drow=%d\n",
		(cchar *)getName(),
		(cchar *)item->getName(),
		drow
	))	;

	/* May not be a form if we are in a component ...		*/
	if (getForm() == 0)
		return	true	;

	/* This routine propogates information up to the root node,	*/
	/* which can then decide who needs to know what about the	*/
	/* focus change, if any.					*/
	if (getForm()->moveFocusOK (item, m_curDRow + drow))
	{
		/* Special case: If focus is moving into a rowmark then	*/
		/* explicitely ensure that there are no morphed		*/
		/* controls.						*/
//**		if (item->isRowMark() != 0) getLayout()->setUnMorphedItem (0, 0) ;
		return	true	;
	}

	return	false	;
}

/*  KBFormBlock								*/
/*  focusOutOK	: Check whether focus can move out of control		*/
/*  newRow	: bool		: Moving to a different row		*/
/*  (return)	: bool		: No update errors			*/

bool	KBFormBlock::focusOutOK
	(	bool	newRow
	)
{
	DPRINTF
	((	"KBFormBlock::focusOutOK: %s: newRow=%d\n",
		(cchar *)getName(),
		newRow
	))	;

	/* May not be a form if we are in a component ...		*/
	if (getForm() == 0) return true ;

	/* If there is no current item then no problem. Actually, this	*/
	/* can should not arrise, as we cannot be the current block	*/
	/* unless we have the current item, but lets be paranoid.	*/
	if (m_curItem == 0) return true ;

	/* There is no checking when in a query, so that (for example)	*/
	/* non-null fields can be left blank, and numbers entered as	*/
	/* "> 666".							*/
	if (m_inQuery	  ) return true ;


	/* Check whether the current item has changed and if so mark	*/
	/* the form as changed.						*/
	markChanged () ;

	/* If this is the new row at the end of the query and all	*/
	/* fields are empty, then focus can move out dependant on the	*/
	/* "onLeave" event.						*/
	if (m_query->newRowEmpty (m_qryLvl, m_curQRow))
		return	m_curItem->doLeave (m_curQRow) ;


	/* Row has some data in it. First check the "onLeave" event and	*/
	/* then check if the current value is valid; note that we do	*/
	/* not force null checks at this point so that the user can	*/
	/* fill values into controls in any order.			*/
	if (!m_curItem->doLeave (m_curQRow))
		return	false ;

	if (!m_curItem->isValid (m_curQRow, true))
	{
		DPRINTF
		((	"KBFormBlock::focusOutOK: %s: not valid\n",
			m_curItem->getName().latin1()
		))	;
		setError (m_curItem->lastError ()) ;
		lastError().DISPLAY() ;
		return	 false ;
	}


	/* If we are changing rows, then see if there have been any	*/
	/* changes in the current row which need to be saved to the	*/
	/* database.							*/
	DPRINTF
	((	"KBFormBlock::focusOutOK: %s: call checkChange if newRow=%d\n",
		(cchar *)getName(),
		newRow
	))	;

	if (newRow)
	{
		bool rowDelta = false ;
		if (!checkChange (false, rowDelta, true))
		{
			lastError().DISPLAY() ;
			return	 false ;
		}

		if (rowDelta)
			displayData
			(	false,
				m_curDRow,
				m_curDRow + m_numRows
			)	;
	}

	if (m_blkInfo.rowmark != 0)
		m_blkInfo.rowmark->setState (m_curQRow, m_query->getRowState (m_qryLvl, m_curQRow)) ;

	return	true	;
}

/*  KBFormBlock								*/
/*  startUpdate	: Check if update can be started			*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: True if OK to update			*/

bool	KBFormBlock::startUpdate
	(	uint		qrow
	)
{
	DPRINTF
	((	"KBFormBlock::startUpdate: qrow=%d dChanged=%d locking=%d\n",
		qrow,
		m_dChanged,
		m_locking.getIntValue()
	))	;

	/* Should not get called unless in data view (code in KBControl	*/
	/* should filter this, but check just in case ....		*/
	if (!showingData())
		return	true	;	

	/* If the data has already changed then we are already in an	*/
	/* update and can proceed. If the data has not changed go see	*/
	/* what locking option is in effect.				*/
	if (m_dChanged)
		return	true	;

	bool	rc = true	;

	switch ((KBQryBase::Locking)m_locking.getIntValue())
	{
		case KBQryBase::NoLocking	:
			/* No locking so no problems, we can allow the	*/
			/* update.					*/
			break	;

		case KBQryBase::LockRowUpdate	:
			/* Lock on change in effect, see if we can	*/
			/* start an update. If OK then the query will	*/
			/* set the record to the current values, so	*/
			/* the data is redisplayed.			*/
			if (!m_query->startUpdate (m_qryLvl, qrow, KBQryBase::LockRowUpdate))
			{
				setError (m_query->lastError()) ;
				showData (false) ;
				rc	= false	 ;
			}
			break	;

		default	:
			break	;
	}

	getDocRoot()->doSetLocking (m_query->lockingState(m_qryLvl)) ;
	return	rc	;
}

/*  KBFormBlock								*/
/*  startUpdate	: Check if update can be started			*/
/*  commit	: bool		: Commit changes			*/
/*  (returns)	: bool		: True if OK to update			*/

bool	KBFormBlock::endUpdate
	(	bool		commit
	)
{
	DPRINTF
	((	"KBFormBlock::endUpdate: commit=%d dChanged=%d locking=%d\n",
		commit,
		m_dChanged,
		m_locking.getIntValue()
	))	;

	bool	rc = true	;

	switch ((KBQryBase::Locking)m_locking.getIntValue())
	{
		case KBQryBase::NoLocking	:
			/* No locking so nothing needs to be done ...	*/
			break	;

		case KBQryBase::LockRowUpdate	:
			/* Lock on change in effect, so we should be	*/
			/* ending the update at the query.		*/
			if (!m_query->endUpdate (m_qryLvl, commit))
			{
				setError(m_query->lastError()) ;
				showData(false)	;
				rc	= false	;
			}
			break	;

		default	:
			break	;
	}

	getDocRoot()->doSetLocking (m_query->lockingState(m_qryLvl)) ;
	return	true	;
}

/*  KBFormBlock								*/
/*  focusMovesRow: Focus moves into a new query row			*/
/*  newQRow	 : uint		: New row				*/
/*  (returns)	 : void		:					*/

void	KBFormBlock::focusMovesRow
	(	uint		newQRow
	)
{
	DPRINTF
	((	"KBFormBlock::focusMovesRow: %s: newQRow=%d\n",
		(cchar *)getName(),
		newQRow
	))	;

	bool	evRc	;
	KBValue	argOld	((int)m_curQRow) ;
	KBValue	argNew	((int)newQRow  ) ;

	eventHook (m_events->onUnCurrent, 1, &argOld, evRc) ;

	m_curQRow = newQRow ;
	m_query->setCurrentRow (m_qryLvl, m_curQRow) ;
	if (!showData()) lastError().DISPLAY() ;

	eventHook (m_events->onCurrent,   1, &argNew, evRc) ;
}

/*  KBFormBlock								*/
/*  focusMovesItem: Focus moves into a new data control			*/
/*  item	  : KBItem *	: Item					*/
/*  reason	  : Reason	: Reason focus moved			*/
/*  (returns)	  : void	:					*/

void	KBFormBlock::focusMovesItem
	(	KBItem			*item,
		QFocusEvent::Reason	reason
	)
{
	DPRINTF
	((	"KBFormBlock::focusMovesItem: %s: %s: r=%d\n",
		(cchar *)getName(),
		(cchar *)item->getName(),
		reason
	))	;

	QRect	rCtrl  ;

	/* May not be a form if we are in a component ...		*/
	if (getForm() == 0)
		return	;

	/* Ensure that any rowmark is showing the correct current item.	*/
	if (m_blkInfo.rowmark != 0)
		m_blkInfo.rowmark->setCurrent (m_curQRow) ;

	/* If focus is arriving in an item other than a row mark then	*/
	/* all marks are cleared. Hence, only row marks are used for	*/
	/* selecting multiple rows.					*/
	if (item->isRowMark() == 0) setRowMarked (0, KB::MarkOpClear) ;

	m_curItem	= item ;
	getForm()->focusInEvent  (m_curItem, m_curQRow) ;
	getForm()->setFocusAtRow (this) ;

	switch (reason)
	{
		case QFocusEvent::ActiveWindow	:
		case QFocusEvent::Other		:
			break	;

		default	:
			/* Try to make sure that the current item and	*/
			/* row therein is visible. This can have an	*/
			/* affect if the display is a scroll view.	*/
			if (m_curItem->ctrlGeometry (m_curQRow, rCtrl))
				m_blkDisp->makeVisible (rCtrl, reason) ;
			break	;
	}
}

/*  KBFormBlock								*/
/*  focusInEvent: Note arrival of focus in item				*/
/*  item	: KBItem *	: Item in question			*/
/*  drow	: uint		: Display row in item			*/
/*  reason	: Reason	: Reason focus arrives			*/
/*  (returns)	: void		:					*/

void	KBFormBlock::focusInEvent
	(	KBItem			*item,
		uint			drow,
		QFocusEvent::Reason	reason
	)
{
	DPRINTF
	((	"KBFormBlock::focusInEvent: %s: %s: drow=%d r=%d\n",
		(cchar *)getName(),
		(cchar *)item->getName(),
		drow,
		reason
	))	;

	if (m_curQRow != drow + m_curDRow)
		focusMovesRow (drow + m_curDRow) ;

	focusMovesItem (item, reason) ;
}

/*  KBFormBlock								*/
/*  getValue	: Get value						*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBFormBlock::getValue
	(	uint	qrow
	)
{
	return	m_query->getField (m_qryLvl, qrow, m_qryIdx.qryIdx()) ;
}

/*  KBFormBlock								*/
/*  setValue	: Set value display row					*/
/*  qrow	: uint		 : Query row				*/
/*  value	: const KBValue &: Value to set				*/
/*  (returns)	: bool		 : Success				*/

bool	KBFormBlock::setValue
	(	uint		,
		const KBValue	&
	)
{
	return	true	;
}

/*  KBFormBlock								*/
/*  newNode	: Add a new node					*/
/*  id		: int		: Actually NodeSpec pointer for node	*/
/*  (returns)	: KBNode *	: The new node				*/

KBNode	*KBFormBlock::newNode
	(	int		id
	)
{
#if	! __KB_RUNTIME
	return	KBNavigator::newNode
		(	idToNodeSpec (id),
			autoCtrlRect (false),
			this,
			m_blkDisp,
			m_bState
		)	;

#else
	return	0 ;
#endif
}

/*  KBFormBlock								*/
/*  replicate	: Replicate the block					*/
/*  _parent	: KBNode *	: Parent node				*/
/*  (returns)	: KBNode *	: Replicant				*/

KBNode	*KBFormBlock::replicate
	(	KBNode		*parent
	)
{
	KBFormBlock *rep = new KBFormBlock (parent, this) ;
	KBNode	    *res = replicateBelow  (rep) ;

	rep->findQuery	() ;
	return	res	;
}

/*  KBFormBlock								*/
/*  docPropDlg	: Show form property dialog				*/
/*  (returns)	: void		:					*/

void	KBFormBlock::docPropDlg ()
{
#if	! __KB_RUNTIME
	if (getForm     () != 0)
	{	getForm     ()->formPropDlg (0) ;
		return	;
	}
	if (getComponent() != 0)
	{	getComponent()->compPropDlg (0) ;
		return	;
	}
#endif
}

/*  KBFormBlock								*/
/*  newNullBlock: Add a new menu block					*/
/*  (returns)	: void		:					*/

void	KBFormBlock::newNullBlock ()
{
#if	! __KB_RUNTIME
	newFormBlock	(m_blkDisp, KBBlock::BTNull) ;
#endif
}

/*  KBFormBlock								*/
/*  newTableBlock: Add a new table block				*/
/*  (returns)	 : void		:					*/

void	KBFormBlock::newTableBlock ()
{
#if	! __KB_RUNTIME
	newFormBlock	(m_blkDisp, KBBlock::BTTable) ;
#endif
}

/*  KBFormBlock								*/
/*  newQueryBlock: Add a new query block				*/
/*  (returns)	 : void		:					*/

void	KBFormBlock::newQueryBlock ()
{
#if	! __KB_RUNTIME
	newFormBlock	(m_blkDisp, KBBlock::BTQuery) ;
#endif
}

/*  KBFormBlock								*/
/*  newSQLBlock	: Add a new SQL block					*/
/*  (returns)	: void		:					*/

void	KBFormBlock::newSQLBlock ()
{
#if	! __KB_RUNTIME
	newFormBlock	(m_blkDisp, KBBlock::BTSQL) ;
#endif
}

/*  KBFormBlock								*/
/*  newContainer : Add a new container					*/
/*  (returns)	 : void		:					*/

void	KBFormBlock::newContainer ()
{
#if	! __KB_RUNTIME
	KBObject::newContainer (m_blkDisp) ;
#endif
}

#if	! __KB_RUNTIME

/*  KBFormBlock								*/
/*  makeNewPopup: Make a new-thingy popup menu for a form		*/
/*  parent	: KBPopupMenu *	: Parent popup menu			*/
/*  rect	: QRect		: New object area			*/
/*  (returns)	: void		:					*/

KBPopupMenu
	*KBFormBlock::makeNewPopup
	(	KBPopupMenu	*parent,
		QRect		rect
	)
{
	setCtrlRect (rect) ;
	return	makeFormNewPopup (parent, rect, this, &m_bState) ;
}
#endif

/*  KBFormBlock								*/
/*  setGeometry	: Block moves in its parent				*/
/*  rect	: const QRect &	: New position				*/
/*  (returns)	: void		:					*/

void	KBFormBlock::setGeometry
	(	const QRect	&rect
	)
{
	DPRINTF
	((	"KBFormBlock::setGeometry: %s\n",
		_TEXT(rect)
	))	;
	KBBlock::setGeometry (rect) ;
}

void	KBFormBlock::recalcNumRows
	(	const QSize		&size
	)
{
	/* Start by calculating how may rows could be displayed based	*/
	/* on the data controls immediately in this block.		*/
	uint	fw	= size.width  () ;
	uint	fh	= size.height () ;
	uint	dx	= getDisplayDX() ;
	uint	dy	= getDisplayDY() ;

	m_maxRows	= 999	;

	if ((getShowbarFlags() & NAV_MINI) != 0)
	{
		uint nh = KBRecordNav::getHeight() ;
		fh = nh > fh ? 0 : fh - nh ;
	}

	CITER
	(	Item,
		i,

		uint nr = i->calcNumRows (fw, fh, dx, dy) ;
		if (nr < m_maxRows) m_maxRows = nr ;
	)

	if (m_maxRows <= 0) m_maxRows = 1 ;
}

/*  KBFormBlock								*/
/*  displayResized							*/
/*		: Block display has been resized			*/
/*  size	: const QSize & : New size				*/
/*  (returns)	: void		:					*/

void	KBFormBlock::displayResized
	(	const QSize		&size
	)
{
	recalcNumRows	(size) ;

	DPRINTF
	((	"KBFormBlock::displayResized: %s: m_maxRows=%d\n",
		_TEXT(size),
		m_maxRows
	))	;

	/* Note how many rows are currently displayed, then pass the	*/
	/* resize to the block. On return, if the number of rows has	*/
	/* decreased, then we do not need to do anything more (nor	*/
	/* unless showing data).					*/
	uint	numrows	= m_numRows	;
	KBBlock::displayResized (size)	;
	adjustGridLayout()		;

	DPRINTF
	((	"KBFormBlock::displayResized: [%p] m_numRows=%d->%d data=%d\n",
		(void *)this,
		numrows,
		m_numRows,
		showingData()
	))	;

	if (!showingData() || (m_numRows <= numrows)) return ;

	/* FIXME! Need to handle the case where the current item is	*/
	/* updated and disappears from view ....			*/

	uint	extra	= 0	;
	uint	qrows	= 1	;

	if (m_query != 0)
	{
		extra	= (m_query->getPermission(m_qryLvl) & QP_INSERT) != 0 ? 1 : 0 ;
		qrows	= m_query->getNumRows(m_qryLvl) ;
	}

	CITER
	(	Item,
		i,
		i->hideBelow   (qrows + extra)   ;
	)
	CITER
	(	Framer,
		f,
		f->hideBelow   (qrows + extra)   ;
	)

	displayData (true, m_curDRow + numrows, m_curDRow + m_numRows) ;
}

/*  KBFormBlock								*/
/*  remChild	: Remove a child node					*/
/*  child	: KBNode *	: Child in question			*/
/*  (returns)	: void		:					*/

void	KBFormBlock::remChild
	(	KBNode	*child
	)
{
	KBBlock::remChild (child) ;
}

/*  KBFormBlock								*/
/*  blockSetup	: Form block setup					*/
/*  (returns)	: bool		: Success				*/

bool	KBFormBlock::blockSetup ()
{
	/* We implement this in order to check for a grid header, in	*/
	/* which case the layout and header will need fixing up.	*/
	setupGridLayout() ;

	return	KBBlock::blockSetup() ;
}

/*  KBFormBlock								*/
/*  enterBlock	: Focus enters block					*/
/*  move	: bool		: True to move focus to first item	*/
/*  toRow	: uint		: Goto row on move			*/
/*  (returns)	: void		:					*/

void	KBFormBlock::enterBlock
	(	bool		move,
		uint		toRow
	)
{
	DPRINTF
	((	"KBFormBlock::enterBlock: %s: move=%d toRow=%d\n",
		(cchar *)getName(),
		move,
		toRow
	))	;

	KBItem	*first	;

	if (move)
		if ((first = firstItem ()) != 0)
		{
			m_curItem = first	;
			m_curQRow = toRow	;
			getForm()->focusInEvent (m_curItem, m_curQRow) ;
			m_curItem->setFocus     (m_curQRow) ;
		}

	getForm()->setFocusAtRow (this) ;
}

/*  KBFormBlock								*/
/*  markChanged	: Check for marking change when changing focus		*/
/*  (returns)	: void		:					*/

void	KBFormBlock::markChanged ()
{
	/* If there is a current item and it has changed then mark the	*/
	/* form as changed (unless we are in a query).			*/
	if (!m_inQuery || (m_curItem == 0) || !m_curItem->changed(m_curQRow))
		return	;

	/* Also ignore changes if the associated query is the null	*/
	/* (menu-only) query.						*/
	if (m_query->isQryNull() != 0)
		return	;

	getLayout()->setChanged() ;
}

/*  KBFormBlock								*/
/*  sortByColumn: Sort data by column					*/
/*  column	: KBItem *	: Item for column			*/
/*  asc		: bool		: Sort ascending			*/
/*  (returns)	: void		:					*/

void	KBFormBlock::sortByColumn
	(	KBItem		*column,
		bool		asc
	)
{
	m_query->sortByColumn  (m_qryLvl, column->getQueryIdx(), asc, column) ;

	m_curQRow = 0	;
	m_curDRow = 0	;
	m_query->setCurrentRow (m_qryLvl, 0) ;
	if (!showData (true)) lastError().DISPLAY() ;
}

/*  KBFormBlock								*/
/*  rowsInBlock	: Calculate number of rows that can be displayed	*/
/*  (returns)	: void		:					*/

uint	KBFormBlock::rowsInBlock ()
{
	if (isDynamic())
	{
		/* Special case, dynamic blocks only display one row.	*/
		/* This may be relaxed later.				*/
		return	1 ;
	}

	uint	numrows	= m_rowcount.getIntValue() & 0x7fff ;
	if (numrows != 0)
	{
		/* Explicit number of rows set, so return that. Assume	*/
		/* the user knows what they are doing.			*/
		return	numrows	;
	}

	/* NOTE: We assume that the maximum number of rows that can be	*/
	/* be displayed based on the data controls directly in this	*/
	/* block has already been set in "m_maxRows". We then just	*/
	/* need to cut back for any framers that are more restricted.	*/
	numrows	= m_maxRows	;

	CITER
	(	Framer,
		f,
	
		uint nr = f->rowsInFrame ()	;
		if (nr < numrows) numrows = nr	;
	)

	if (numrows >= 999) numrows = 1 ;
	if (numrows ==   0) numrows = 1 ;

	return	numrows	;
}

/*  KBFormBlock								*/
/*  setRowMarked: Row marking operation					*/
/*  qrow	: uint		: Query row number			*/
/*  op		: KB::MarkOp	: Mark operation			*/
/*  (returns)	: void		:					*/

void	KBFormBlock::setRowMarked
	(	uint		qrow,
		KB::MarkOp	op
	)
{
	if (m_blkInfo.rowmark != 0)
	{
		m_query->setRowMarked (m_qryLvl, qrow, op) ;

		for (uint drow = 0 ; drow < m_numRows ; drow += 1)
		{
			uint	qrow	= m_curDRow + drow ;
			bool	marked	= m_query->getRowMarked (m_qryLvl, qrow) ;

			CITER
			(	Item,
				i,
				i->setMarked    (qrow, marked)
			)
		}

		CITER
		(	Framer,
			f,
			f->setRowMarked (m_curDRow, m_curDRow + m_numRows) ;
		)
	}
}

/*  KBFormBlock								*/
/*  getRowMarked: Get row marked state					*/
/*  qrow	: uint		: Query row number			*/
/*  (returns)	: bool		: True if marked			*/

bool	KBFormBlock::getRowMarked
	(	uint		qrow
	)
{
	return	m_query->getRowMarked (m_qryLvl, qrow) ;
}

/*  KBFormBlock								*/
/*  requery	: (Re)query associated query for all rows		*/
/*  (returns)	: bool		  : Success				*/

bool	KBFormBlock::requery ()
{
	KBGrid	*grid	= 0 ;

	if (!KBBlock::requery ()) return false ;

	CITER
	(	Grid,
		g,
		grid = g ; break ;
	)

	if (grid != 0) grid->columnSort () ;
	
	return	true ;
}

/*  KBFormBlock								*/
/*  dataChanged	: User has changed data in this block			*/
/*  qrow	: uint		: Query row number			*/
/*  (returns)	: void		:					*/

void	KBFormBlock::dataChanged
	(	uint	qrow
	)
{
	DPRINTF
	((	"KBFormBlock::dataChanged: qrow=%d m_dChanged=%d\n",
		qrow,
		m_dChanged
	))	;

	if (!m_dChanged)
	{
		bool	evRc  ;
		KBValue	rowNo = (int)qrow ;
		eventHook (m_events->onChange, 1, &rowNo, evRc) ;

		m_dChanged = true ;
	}
}

void	KBFormBlock::getResults
	(	const QString	&path,
		QDict<QString>	&dict
	)
{
	QString	prefix	= QString("%1%2").arg(path).arg(path.isEmpty() ? "" : ".") ;

	DPRINTF
	((	"KBFormBlock::getResults: [%s]\n",
		(cchar *)prefix
	))	;

	CITER
	(
		Item,
		item,

		if (item->isBlock() == 0)
			dict.insert
			(	QString	("%1%2")
					.arg(prefix)
					.arg(item->getName()),
				new QString(item->getValue(m_curQRow).getRawText())
			)	;
	)

	CITER
	(
		Framer,
		framer,
		framer->getResults
		(	QString	("%1%2").arg(prefix).arg(framer->getName()),
			dict
		)	;
	)

	CITER
	(
		FormBlock,
		block,
		block->getResults
		(	QString	("%1%2").arg(prefix).arg(block->getName()),
			dict
		)	;
	)
}

/*  KBFormBlock								*/
/*  invalidControls							*/
/* 		: Scan for any controls which are not valid		*/
/*  row		: uint		     : Row number			*/
/*  errors	: QPtrList<KBItem> & : Return list of invalid items	*/
/*  recurse	: bool		     : Recurse into blocks		*/
/*  (returns)	: bool		     : True if any data in row		*/

bool	KBFormBlock::invalidControls
	(	uint			row,
		QPtrList<KBItem>	&errors,
		bool			recurse
	)
{
	bool	rc = false ;
	
	CITER
	(
		Item,
		item,

		KBFormBlock *fblk = item->isFormBlock() ;
		if (fblk != 0)
		{
			if (recurse)
				if (item->isFormBlock()->invalidControls (fblk->getCurQRow(), errors, true))
					rc = true ;
			continue ;
		}

		if (!item->isEnabled (row)) continue ;
		if (!item->isVisible (row)) continue ;

		if (item->isUpdateVal (true) && !item->isValid (row, false))
			errors.append (item) ;
	)

	CITER
	(
		Framer,
		framer,
		if (framer->invalidControls (row, errors, recurse))
			rc = true ;
	)

	return	rc || ! m_query->newRowEmpty (m_qryLvl, m_curQRow) ;
}

/*  KBFormBlock								*/
/*  changedControls							*/
/* 		: Scan for any controls which have changed		*/
/*  row		: uint		     : Row number			*/
/*  changed	: QPtrList<KBItem> & : Return list of changed items	*/
/*  recurse	: bool		     : Recurse into blocks		*/
/*  (returns)	: bool		     : True unless row is empty		*/

bool	KBFormBlock::changedControls
	(	uint			row,
		QPtrList<KBItem>	&changed,
		bool			recurse
	)
{
	CITER
	(
		Item,
		item,

		KBFormBlock *fblk = item->isFormBlock() ;
		if (fblk != 0)
		{
			if (recurse)
				item->isFormBlock()->changedControls (fblk->getCurQRow(), changed, true) ;
			continue ;
		}

		if (item->isUpdateVal(true) && item->changed (row))
		{
			DPRINTF
			((	"KBFormBlock::changedControls: adding [%s]\n",
				item->getName().latin1()
			))	;

			changed.append (item) ;
		}
	)

	CITER
	(
		Framer,
		framer,
		framer->changedControls (row, changed, recurse) ;
	)

	return	! m_query->newRowEmpty (m_qryLvl, m_curQRow) ;
}

void	KBFormBlock::setCtrlReadOnly
	(	uint		qrow,
		bool		readOnly,
		const QColor	&fgColor,
		const QColor	&bgColor
	)
{
	CITER
	(	Item,
		item,
		if (item->isBlock() == 0)
			item->setCtrlReadOnly (qrow, readOnly, fgColor, bgColor) ;
	)

	CITER
	(	Framer,
		framer,
		framer->setCtrlReadOnly (qrow, readOnly, fgColor, bgColor) ;
	)
}

#if	! __KB_RUNTIME

/*  KBFormBlock								*/
/*  makeRecordPopup							*/
/*		: Construct recording popup menu			*/
/*  popup	: KBPopupMenu *	: Popup menu to add to			*/
/*  drow	: uint		: Display row number			*/
/*  parents	: bool		: Add submenus for parents		*/
/*  (returns)	: void		:					*/

void	KBFormBlock::makeRecordPopup
	(	KBPopupMenu		*popup,
		uint			drow,
		bool			parents
	)
{
	KBPopupMenu *hpop = 0 ;

	CITER
	(	Hidden,
		h,

		if (hpop == 0) hpop = new KBPopupMenu (popup) ;

		hpop->insertItem
		(	h->getName(),
			h,
			SLOT(recordVerifyValue())
		)	;
	)

	if (hpop != 0)
		popup->insertItem
		(	TR("Verify hidden field"),
			hpop
		)	;

	m_recordDRow	= getCurDRow() ;
}
#endif

/*  KBFormBlock								*/
/*  printNode	: Output node XML					*/
/*  text	: QString &	: Result string				*/
/*  indent	: int		: Indent level of this node		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBFormBlock::printNode
	(	QString		&text,
		int		indent,
		bool		flat
	)
{

	if (flat)
	{
		uint	rcnt	= m_rowcount.getIntValue() ;
		uint	rows	= rowsInBlock()	;
		uint	showAll	= rcnt & 0x8000	;

		if (rows == 0) rows = 1 ;
		m_rowcount.setValue (rows | showAll) ;

		KBBlock::printNode  (text, indent, flat) ;
		m_rowcount.setValue (rcnt) ;
		return	;
	}

	KBBlock::printNode  (text, indent, flat) ;
}


NEWNODENAMED	(FormBlock,   (cchar *)0, 0, KNF_FORM,   FormBlock   )
NEWNODENAMED	(FormBlock,   (cchar *)0, 0, KNF_FORM,   FormSubBlock)
