/***************************************************************************
    file	         : kb_formdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qwidgetstack.h>

#if		__KB_AUTOMOC
#include	"kb_formdlg.moc"
#else
#include	"kb_formdlg.h"
#endif

#include	"kb_form.h"
#include	"kb_param.h"
#include	"kb_paramdlg.h"
#include	"kb_script.h"
#include	"kb_import.h"
#include	"kb_testsuite.h"
#include	"kb_testsuitedlg.h"

#include	"tk_messagebox.h"
#include	"rk_combobox.h"


/*  KBFormPropDlg							*/
/*  KBFormPropDlg: Constructor for root properties dialog		*/
/*  root	 : KBForm *		: Root node			*/
/*  caption	 : cchar *		: Caption text			*/
/*  attribs	 : QPtrList<KBAttr  > &	: Attributes list		*/
/*  iniAttr	 : cchar *		: Initial attribute		*/
/*  (returns)	 : KBFormPropDlg	:				*/

KBFormPropDlg::KBFormPropDlg
	(	KBForm			*root,
		cchar			*caption,
		QPtrList<KBAttr  >	&attribs,
		cchar			*iniAttr
	)
	:
	KBPropDlg	(root,	caption,	attribs, iniAttr),
	m_mattr		(root,	"__modlist",    "",	 KAF_SYNTHETIC),
	m_m2attr	(root,	"__modlist2",   "",	 KAF_SYNTHETIC),
	m_iattr		(root,	"__implist",    "",	 KAF_SYNTHETIC),
	m_pattr		(root,	"__paramlist",  "",	 KAF_SYNTHETIC),
	m_tattr		(root,	"__testsuites", "",	 KAF_SYNTHETIC),
	m_root	  	(root)
{
	TITER
	(	Script,
		m_root->getChildren(),
		s,
		if (s->isL2())
			m_scripts2.append (s) ;
		else	m_scripts .append (s) ;
	)
	TITER
	(	Import,
		m_root->getChildren(),
		i,
		m_imports.append (i)
	)
	TITER
	(	Param,
		m_root->getChildren(),
		p,
		m_params .append (p)
	)
	TITER
	(	TestSuite,
		m_root->getChildren(),
		t,
		m_testSuites.append (t)
	)

	m_scriptDlg	= new KBScriptDlg    (m_propStack, m_root, m_scripts,  false) ;
	m_script2Dlg	= new KBScriptDlg    (m_propStack, m_root, m_scripts2, true ) ;
	m_importDlg	= new KBImportDlg    (m_propStack, m_root, m_imports	    ) ;
	m_paramDlg   	= new KBParamDlg     (m_propStack, m_root, m_params	    ) ;
	m_testSuiteList	= new KBTestSuiteList(m_propStack, m_root, m_testSuites	    ) ;

	m_scriptDlg	->hide() ;
	m_script2Dlg	->hide() ;
	m_importDlg	->hide() ;
	m_paramDlg	->hide() ;
	m_testSuiteList	->hide() ;
}

/*  KBFormPropDlg							*/
/*  ~KBFormPropDlg: Constructor for root properties dialog		*/
/*  (returns)	  :		:					*/

KBFormPropDlg::~KBFormPropDlg ()
{
}

/*  KBFormPropDlg							*/
/*  hideProperty : Check whether to hide attribute property		*/
/*  attr	 : KBAttr *	: Attribute in question			*/
/*  (returns)	 : bool		: True to hide				*/

bool	KBFormPropDlg::hideProperty
	(	KBAttr	*attr
	)
{
	/* Since KBForm is derived from KBBlock, there are a whole lot	*/
	/* of properties which we must ignore; so check just for those	*/
	/* of our own.							*/
	if (qstrcmp (attr->getOwnerName(), "KBForm") == 0)
		return	false	;

	return	true	;
}


/*  KBFormPropDlg							*/
/*  showProperty: Show property						*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property on show			*/

bool	KBFormPropDlg::showProperty
	(	KBAttrItem	*item
	)
{
	QString	aName	= item->attr()->getName () ;

	/* The script module list is handled entirely locally, with	*/
	/* none of the default KBPropDlg controls.			*/
	if (aName == "__modlist"  )
	{
		setUserWidget (m_scriptDlg ) ;
		return	true	 ;
	}
	if (aName == "__modlist2" )
	{
		setUserWidget (m_script2Dlg) ;
		return	true	 ;
	}
	if (aName == "__implist"  )
	{
		setUserWidget (m_importDlg ) ;
		return	true	 ;
	}
	if (aName == "__paramlist")
	{
		setUserWidget (m_paramDlg  ) ;
		return	true	 ;
	}
	if (aName == "__testsuites")
	{
		setUserWidget (m_testSuiteList) ;
		return	true	 ;
	}

	return	KBPropDlg::showProperty (item) ;
}

/*  KBFormPropDlg							*/
/*  saveProperty: Check and save attribute				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Attribute OK				*/

bool	KBFormPropDlg::saveProperty
	(	KBAttrItem	*item
	)
{
	QString	aName	= item->attr()->getName() ;

	/* As for showing the script module list, saving the results is	*/
	/* handled locally.						*/
	if (aName == "__modlist"  )
	{
		setProperty  ("__modlist",   m_scriptDlg ->getText()) ;
		return	true ;
	}
	if (aName == "__modlist2" )
	{
		setProperty  ("__modlist2",  m_script2Dlg->getText()) ;
		return	true ;
	}
	if (aName == "__implist"  )
	{
		setProperty  ("__implist",   m_importDlg ->getText()) ;
		return	true ;
	}
	if (aName == "__paramlist")
	{
		setProperty  ("__paramlist", m_paramDlg  ->getText()) ;
		return	true ;
	}
	if (aName == "__testsuites")
	{
		setProperty  ("__testsuites", m_testSuiteList->getText()) ;
		return	true ;
	}


	return	KBPropDlg::saveProperty (item) ;
}

/*  KBFormPropDlg:							*/
/*  clickOK	 : Handle OK button click				*/
/*  (returns)	 : void		:					*/

void	KBFormPropDlg::clickOK ()
{
	m_scriptDlg	->fixUp () ;
	m_script2Dlg	->fixUp () ;
	m_importDlg	->fixUp () ;
	m_paramDlg	->fixUp () ;
	m_testSuiteList	->fixUp (m_testSuites) ;

	KBPropDlg::clickOK () ;
}

/*  KBFormPropDlg:							*/
/*  preExec	  : Called immediately before KBDialog::exec		*/
/*  (returns)	  : void	:					*/

void	KBFormPropDlg::preExec ()
{
	/* Trap this and load the notional hidden attribute. We have to	*/
	/* do this here since it is not until "KBPropDlg::exec" has	*/
	/* been called that "setProperty" will work.			*/
	setProperty  ("__modlist",    m_scriptDlg    ->getText()) ;
	setProperty  ("__modlist2",   m_script2Dlg   ->getText()) ;
	setProperty  ("__implist",    m_importDlg    ->getText()) ;
	setProperty  ("__paramlist",  m_paramDlg     ->getText()) ;
	setProperty  ("__testsuites", m_testSuiteList->getText()) ;
}



/*  ------------------------------------------------------------------	*/

/*  formPropDlg	: Run the root properties dialog box			*/
/*  form	: KBForm *	  	: Form node			*/
/*  caption	: cchar *	 	: Caption text			*/
/*  attribs	: QPtrList<KBAttr  > &	: Attributes list		*/
/*  iniAttr	: cchar *	  	: Initial attribute		*/
/*  (returns)	: bool		  	: Success			*/

bool	LIBKBASE_API formPropDlg
	(	KBForm			*form,
		cchar			*caption,
		QPtrList<KBAttr  >	&attribs,
		cchar			*iniAttr
	)
{
	KBFormPropDlg	fDlg
			(	form,
				caption,
				attribs,
				iniAttr
			)	;
	return	fDlg.exec  () ;
}
