/***************************************************************************
    file	         : kb_framer.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#ifndef		__KB_FRAMER_H
#define		__KB_FRAMER_H

#include	"libkbase_exports.h"

#include	"kb_block.h"
#include	"kb_navigator.h"
#include	"kb_attrimage.h"


/*  KBFramer								*/
/*  --------								*/
/*  Class used to implement block headers and footers, plus tab widgets	*/
/*  and simple form containers. Basically, its a subdivision of a block	*/


class	LIBKBASE_API	KBFramer : public KBObject, public KBNavigator
{
	Q_OBJECT

	KBQryBase		*m_query	;
	uint			m_qryLvl	;
	uint			m_numRows	;
	uint			m_maxRows	;
	KBBlockInfo		*m_blkInfo	;

	virtual void		setPalette  	()	;
	virtual void		setFont	    	()	;

protected :

#if	! __KB_RUNTIME
	virtual	KBPopupMenu	*makeNewPopup	(KBPopupMenu *, QRect) ;
#endif

	KBAttrStr		m_bgcolor    	;
	KBAttrStr		m_title		;
	KBAttrFrame		m_frame		;
	KBAttrBool		m_showbar	;
	KBAttrUInt		m_tabOrd  	;

	KBDisplay		*m_frmDisp	;

public	:

	virtual	KBFramer	*isFramer	() { return this      ; }
	virtual	KBNavigator	*isNavigator	() { return this      ; }
	virtual	KBDisplay	*getContainer	() { return m_frmDisp ; }

	virtual	void		remChild	(KBNode *) ;
	virtual	KBNode		*endParse  	()	   ;

	virtual	KBNode		*replicate	(KBNode *) ;
	virtual	bool		write		(KBWriter *, QPoint, bool, int &, bool = false) ;

	virtual	QRect		getPosition	() ;
	virtual	QSize		minimumSize	() ;

	virtual	void		setGeometry	(const QRect &)	;
	virtual	void		displayResized	(const QSize &)	;

	virtual	void		buildDisplay	(KBDisplay *)	;
	virtual	void		showAs		(KB::ShowAs)	;
	bool			writeData	(bool = false)	;

	virtual	void		setCtrlBGColor	(const QString &) ;

	bool			doAction	(KB::Action) ;
	void			setQryLevel 	(uint, KBQryBase *) ;

	bool			anyChildBlock	() ;

	bool			changed		(uint)	;
	void			hideBelow	(uint)	;
	void			clearFields	(uint, bool) ;
	virtual void		setupControls	()	;

	virtual	void		setEnabled	(bool)	;
	virtual	void		setVisible	(bool)	;
	virtual bool		isEnabled	()	;
	virtual bool		isVisible	()	;

	void			setCtrlReadOnly	(uint, bool, const QColor &, const QColor &) ;

	uint			getNumRows	() ;
	uint			getCurQRow	() ;
	KBValue			getRowValue	(const QString &, uint) ;
	void			setRowValue	(const QString &, uint, const KBValue &) ;

	void			setRowMarked	(uint, uint) ;

	virtual	void		setTabOrder	(int)	;
	virtual	int		getTabOrder	()	;

	virtual	void		recalcNumRows	(const QSize  &) ;

	uint			rowsInFrame	() ;
	int			height		() ;
	virtual	bool		framerSetup	(KBQryBase *, uint, KBBlockInfo *) ;
	bool			addAllItems	() ;
	bool			showData	() ;

	void			getResults	(const QString &, QDict<QString> &) ;
	bool			invalidControls	(uint, QPtrList<KBItem> &, bool) ;
	void			changedControls	(uint, QPtrList<KBItem> &, bool) ;

#if	! __KB_RUNTIME
	virtual	KBPopupMenu	*designPopup	(KBPopupMenu *, QRect) ;
	virtual	bool		propertyDlg	(cchar *) ;
	virtual	void		setupProperties	()	;
#endif

public	slots :

	virtual void		pasteObjects	()	;
	virtual void		pasteHere	()	;
	virtual void		pasteComponent	()	;
	virtual void		linkComponent	()	;

	virtual KBNode		*newNode	(int)	;
	virtual void		newNullBlock	() 	;
	virtual void		newTableBlock	() 	;
	virtual void		newQueryBlock	() 	;
	virtual void		newSQLBlock	() 	;
	virtual void		newContainer	() 	;
	virtual void		newTabOrder	() 	;

	virtual	void		displayGone	()	;

public	:

	KBFramer 		(KBNode *, const QDict<QString> &, cchar *, bool * = 0) ;
	KBFramer 		(KBNode *, KBFramer *) ;
	virtual	 ~KBFramer	() ;
}	;


/*  KBHeader								*/
/*  --------								*/
/*  Block header, used in reports					*/

class	LIBKBASE_API	KBHeader : public KBFramer
{
	Q_OBJECT

public	:

	KBHeader (KBNode *, const QDict<QString> &, cchar *, bool * = 0) ;
	KBHeader (KBNode *, KBHeader *) ;
	virtual	 ~KBHeader () ;

	virtual	void		showAs		(KB::ShowAs)	;
	virtual	void		setGeometry	(const QRect &)	;
	virtual	KBNode		*replicate	(KBNode *)	;
	virtual	void		buildDisplay	(KBDisplay *)	;

	virtual	KBHeader	*isHeader	() { return this ; }
}	;

/*  KBFooter								*/
/*  --------								*/
/*  Block footer, used in reports					*/

class	LIBKBASE_API	KBFooter : public KBFramer
{
	Q_OBJECT

public	:

	KBFooter (KBNode *, const QDict<QString> &, cchar *, bool * = 0) ;
	KBFooter (KBNode *, KBFooter *) ;
	virtual	 ~KBFooter () ;

	virtual	void		showAs		(KB::ShowAs)	;
	virtual	void		setGeometry	(const QRect &)	;
	virtual	KBNode		*replicate	(KBNode *)	;
	virtual	void		buildDisplay	(KBDisplay *)	;

	virtual	KBFooter	*isFooter	() { return this ; }
}	;


/*  KBContainer								*/
/*  -----------								*/
/*  A container is used to provide resizing (floating or stretching)	*/
/*  of an area which can itself contain nested controls.		*/

class	LIBKBASE_API	KBContainer : public KBFramer
{
	Q_OBJECT

	KBAttrImage	m_image		;
	KBAttrUInt	m_autosize	;

public	:

	KBContainer (KBNode *, const QDict<QString> &, cchar *, bool * = 0) ;
	KBContainer (KBNode *, KBContainer *) ;
	virtual	 ~KBContainer () ;

#if	! __KB_RUNTIME
	virtual	void		setupProperties	()		;
#endif

	virtual	void		showAs		(KB::ShowAs)	;
	virtual	void		setGeometry	(const QRect &)	;
	virtual	KBNode		*replicate	(KBNode *)	;

	virtual	KBContainer	*isContainer	() { return this ; }
}	;

#endif		// __KB_FRAMER_H
