/***************************************************************************
    file	         : kb_geometry.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_GEOMETRY_H
#define		__KB_GEOMETRY_H

#include 	<qvaluelist.h>
#include 	<qptrdict.h>
#include	<qpoint.h>
#include	<qobject.h>
#include	<qlayout.h>

class	KBDisplay	;
class	KBLayoutItem	;

class	KBGridLayout	;
class	KBStaticLayout	;
class	QPainter	;


typedef	QPtrDict<KBLayoutItem> KBLayoutMap	;


/*  KBGridLayout							*/
/*  ------------							*/
/*  Wrapper around QGridLayout, implements a buch of support stuff	*/
/*  that we need to drive grid layouts.					*/

class	KBGridLayout : public QGridLayout
{
	Q_OBJECT

	QWidget			*m_parent	;
	KBDisplay		*m_display	;
	KBLayoutMap		m_gridMap	;
	bool			m_reactivate	;

protected slots :

	void		childDestroyed	(QObject *) ;

public	:

	KBGridLayout
	(	QWidget 		*,
		const KBAttrGeom	&,
		KBDisplay		*
	)	;
	~KBGridLayout	() ;

	void		setRowColSetup	(const KBGridSetupList	&, const KBGridSetupList &) ;
	void		init		(const KBAttrGeom	&) ;
	void		outlineCells	(QPainter &)	;

	void		insertWidget	(KBLayoutItem *);
	QPoint		getCell		(QPoint)	;

	const KBLayoutMap
			&layoutMap	()		;

	virtual	void	setGeometry	(const QRect &) ;
}	;


/*  KBStaticLayout							*/
/*  --------------							*/
/*  Wrapper around QLayout, implements a buch of support stuff that we	*/
/*  that we need to drive static layouts.				*/

class	KBStaticLayout : public QLayout
{
	Q_OBJECT

	QWidget			*m_parent	;
	KBDisplay		*m_display	;
	KBLayoutMap		m_staticMap	;
	KBLayoutItem		*m_gridItem	;

	bool			m_invalid	;
	QSize			m_size		;

protected slots :

	void		childDestroyed	(QObject *) ;

public	:

	KBStaticLayout
	(	QWidget 		*,
		const KBAttrGeom	&,
		KBDisplay		*
	)	;
	~KBStaticLayout	()	;

	void		insertWidget	(KBLayoutItem *);
	QPoint		getCell		(QPoint)	;

	virtual	void	invalidate	()		;
	virtual	void	setGeometry	(const QRect &) ;
	QSize		sizeHint	() const	;

	virtual	QLayoutIterator
			iterator ()	;
	virtual	void	addItem		(QLayoutItem *)	;
}	;



/*  KBGeometry								*/
/*  ----------								*/
/*  Geometry management class. Runs in either "static" mode which is	*/
/*  provides original fixed/stretch/float operation, or "dynamic" in	*/
/*  which case a "QGridLayout" manager is used.				*/

class	KBGeometry
{
	QWidget			*m_widget	;
	KBDisplay		*m_display	;
	KBAttrGeom::MgmtMode	m_mgmtMode	;
	KBGridLayout		*m_grid		;
	KBStaticLayout		*m_static	;

public	:

	KBGeometry		()		;
	~KBGeometry		()		;

	void			init		(QWidget *, KBDisplay *) ;
	void			insertWidget	(KBLayoutItem *) ;

	void			setRowColSetup
				(	const KBGridSetupList	&,
					const KBGridSetupList	&
				)	;

	QSize			sizeHint	() const	;

	void			resize		(const QSize &) ;
	void			outlineCells	(QPainter    &) ;
	QPoint			getCell		(QPoint) ;

	bool			isDynamic 	()	 ;

	void			updateDynamic	(const KBAttrGeom &) ;
	void			redoLayout	(bool)	 ;
}	;

#endif		// __KB_GEOMETRY_H
