/***************************************************************************
    file	         : kb_gui.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#if		__KB_KDE
#include 	<kstddirs.h>
#include 	<kinstance.h>
#endif

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_appptr.h"
#include	"kb_callback.h"
#include	"kb_gui.h"
#include	"kb_pluginaction.h"


#define		__DEBUG_GUI	(0)
#if		__DEBUG_GUI
#include	"kb_dprintf.h"
#endif


static 	GUIElement kbaseGUI[] =
{
//	Element name		GType			GGroup			Enable	Text			Icon		Accelerator	Slot			  	Code			Tool Tip
{	"KB_closeDoc",		KBaseGUI::GTAction,	KBaseGUI::GRNone,	true,	"&Close Document",	"fileclose",	ACCEL(Close),	SLOT(slotClose	      ()),	0,			"Close document"				},
{	"KB_printDoc",		KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"Print",		"fileprint",	ACCEL(NoAccel),	SLOT(showAs 	   (int)), 	KB::ShowAsPrint,	"Print document",				},
{	"KB_preview",		KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Preview",		"kghostview",	ACCEL(NoAccel),	SLOT(showAs 	   (int)),	KB::ShowAsPreview,	"Preview print"					},
{	"KB_firstRecord",	KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&First record",	"firstrecord",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::First,		"Move to the first record"			},
{	"KB_prevRecord",	KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Previous record",	"prevrecord",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::Previous,		"Move to the previous record"			},
{	"KB_nextRecord",	KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Next record",		"nextrecord",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::Next,		"Move to the next record"			},
{	"KB_lastRecord",	KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Last record",		"lastrecord",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::Last,		"Move to the last record"			},
{	"KB_saveRecord",	KBaseGUI::GTCoded,	KBaseGUI::GRDelta,	false,	"&Save record",		"saverecord",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::Save,		"Save current record"				},
{	"KB_addRecord",		KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Add record",		"addrecord",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::Add,		"Add a new record"				},
{	"KB_delRecord",		KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Delete record",	"deleterecord",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::Delete,		"Delete current record"				},
{	"KB_startQuery",	KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"Start &query",		"filter",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::Query,		"Start a database query"			},
{	"KB_execQuery",		KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Execute query",	"find",		ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::Execute,		"Execute database query"			},
{	"KB_cancelQuery",	KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Cancel query",	"endquery",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::Cancel,		"Cancel database query"				},
{	"KB_firstPage",		KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&First page",		"firstpage",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::First,		"Move to the first page"			},
{	"KB_prevPage",		KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Previous page",	"prevpage",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::Previous,		"Move to the previous page"			},
{	"KB_nextPage",		KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Next page",		"nextpage",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::Next,		"Move to the next page"				},
{	"KB_lastPage",		KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Last page",		"lastpage",	ACCEL(NoAccel),	SLOT(dbaseAction   (int)),	KB::Last,		"Move to the last page"				},
{	"KB_redo",		KBaseGUI::GTAction,	KBaseGUI::GRNone,	true,	"&Reload",		"redo",		ACCEL(NoAccel),	SLOT(reload	      ()),	0,			"Reload data"					},
{	"KB_orderByExpr",	KBaseGUI::GTAction,	KBaseGUI::GRNone,	true,	"Order columns",	"orderexpr",	ACCEL(NoAccel),	SLOT(orderByExpr      ()),	0,			"Order columns alphabetically"			},

#if	! __KB_RUNTIME
{	"KB_dataView",		KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Data view",		"dataview",	ACCEL(NoAccel),	SLOT(showAs 	   (int)),	KB::ShowAsData,		"Show document in data view"			},
{	"KB_saveDoc",		KBaseGUI::GTAction,	KBaseGUI::GRDelta,	false,	"&Save Document",	"filesave",	ACCEL(Save),	SLOT(saveDocument     ()),	0,			"Save definition"				},
{	"KB_saveDocAs",		KBaseGUI::GTAction,	KBaseGUI::GRNone,	true,	"Save Document &As...",	0,		ACCEL(NoAccel),	SLOT(saveDocumentAs   ()),	0,			"Save definition in a different location"	},
{	"KB_designView",	KBaseGUI::GTCoded,	KBaseGUI::GRNone,	true,	"&Design view",		"designview",	ACCEL(NoAccel),	SLOT(showAs 	   (int)),	KB::ShowAsDesign,	"Show document in design view"			},
{	"KB_alignTop",		KBaseGUI::GTCoded,	KBaseGUI::GRAlign,	false,	"Align &top",		"aligntop",	ACCEL(NoAccel),	SLOT(doCtrlAlign   (int)),	KB::AlignTop,		"Align control to top"				},
{	"KB_alignBottom",	KBaseGUI::GTCoded,	KBaseGUI::GRAlign,	false,	"Align &botton",	"alignbottom",	ACCEL(NoAccel),	SLOT(doCtrlAlign   (int)),	KB::AlignBottom,	"Align controls to bottom"			},
{	"KB_alignLeft",		KBaseGUI::GTCoded,	KBaseGUI::GRAlign,	false,	"Align &left",		"alignleft",	ACCEL(NoAccel),	SLOT(doCtrlAlign   (int)),	KB::AlignLeft,		"Align controls to the left"			},
{	"KB_alignRight",	KBaseGUI::GTCoded,	KBaseGUI::GRAlign,	false,	"Align &right",		"alignright",	ACCEL(NoAccel),	SLOT(doCtrlAlign   (int)),	KB::AlignRight, 	"Align controls to the right"			},
{	"KB_sameWidth",		KBaseGUI::GTCoded,	KBaseGUI::GRAlign,	false,	"Same &width",		"samewidth",	ACCEL(NoAccel),	SLOT(doCtrlAlign   (int)),	KB::SameWidth,		"Make controls the same width"			},
{	"KB_sameHeight",	KBaseGUI::GTCoded,	KBaseGUI::GRAlign,	false,	"Same &height",		"sameheight",	ACCEL(NoAccel),	SLOT(doCtrlAlign   (int)),	KB::SameHeight,		"Make controls the same height"			},
{	"KB_sameSize",		KBaseGUI::GTCoded,	KBaseGUI::GRAlign,	false,	"Same &size",		"samesize",	ACCEL(NoAccel),	SLOT(doCtrlAlign   (int)),	KB::SameSize,		"Make controls the same size"			},
{	"KB_cutObjects",	KBaseGUI::GTAction,	KBaseGUI::GRCopy,	false,	"C&ut objects",		"editcut",	ACCEL(Cut),	SLOT(doCut	      ()),	0,			"Cut selected objects"				},
{	"KB_copyObjects",	KBaseGUI::GTAction,	KBaseGUI::GRCopy,	false,	"&Copy objects",	"editcopy",	ACCEL(Copy),	SLOT(doCopy	      ()),	0,			"Copy selected objects"				},
{	"KB_saveComponent",	KBaseGUI::GTAction,	KBaseGUI::GRCopy,	false,	"Save as component",	"savecomponent",ACCEL(NoAccel),	SLOT(doSaveComponent  ()),	0,			"Save selected objects as components"		},
{	"KB_snapToGrid",	KBaseGUI::GTAction,	KBaseGUI::GRCopy,	false,	"Snap to grid",		"snaptogrid",	ACCEL(NoAccel),	SLOT(snapToGrid	      ()),	0,			"Snap selected controls to grid"		},
{	"KB_pasteObjects",	KBaseGUI::GTAction,	KBaseGUI::GRNone,	false,	"&Paste objects",	"editpaste",	ACCEL(Paste),	SLOT(doPaste	      ()),	0,			"Paste copied or cut objects"			},
{	"KB_singleProp",	KBaseGUI::GTAction,	KBaseGUI::GRCopy,	false,	"Properties",		"properties",	ACCEL(NoAccel),	SLOT(doProperties     ()),	0,			"Update properties"				},
{	"KB_multiProp",		KBaseGUI::GTAction,	KBaseGUI::GRAlign,	false,	"Common properties",	"commonprop",	ACCEL(NoAccel),	SLOT(doMultiProp      ()),	0,			"Edit common properties"			}, 
{	"KB_showObjTree",	KBaseGUI::GTToggle,	KBaseGUI::GRNone,	true,	"Show object tree",	"tree",		ACCEL(NoAccel),	SLOT(showObjTree      ()),	0,			"Show object XML tree"				},
{	"KB_compile",		KBaseGUI::GTAction,	KBaseGUI::GRNone,	true,	"Compile",		"compfile",	ACCEL(NoAccel),	SLOT(doCompile	      ()),	0,			"Compile script source"				},
#endif

{	0,			KBaseGUI::GTNone,	KBaseGUI::GRNone,	false,	0,			0,		ACCEL(NoAccel),	0,			  	0,		 	0						}
}	;

static 	GUIElement nullGUI =
{
//	Element name		GType			GGroup			Enable	Text			Icon		Accelerator	Slot			  	Code			Tool Tip
	"noName",		KBaseGUI::GTAction,	KBaseGUI::GRNone,	true,	"",			"",		ACCEL(NoAccel),	"",				0,			""
}	;

static	QPtrList<KBaseGUI>		allGUIs;



inline	LIBKBASE_API int	mapAccel
	(	GUIAccel	accel
	)
{
#if	__KB_KDE
	return	accel < 0 ? 0 :  KStdAccel::key(accel) ;
#endif
#if	__KB_TKC
	return	accel < 0 ? 0 : TKStdAccel::key(accel) ;
#endif
}


/*  ------------------------------------------------------------------  */


/*  KBaseGUI								*/
/*  KBaseGUI	: Constructor for KBase GUI client			*/
/*  parent	: QObject *	  : Parent object			*/
/*  receiver	: const QObject * : Receiver of signals			*/
/*  guiName	: const QString & : UI definition file			*/
/*  (returns)	: KBaseGUI	  :					*/

KBaseGUI::KBaseGUI
	(	QObject		*parent,
		const QObject	*receiver,
		const QString	&guiName
	)
	:
	QObject		(parent),
	m_receiver	(receiver),
	m_guiName	("gui/" + guiName)
{
	m_hasStatusBar	= true	;
	allGUIs.append	(this)	;
}

/*  KBaseGUI								*/
/*  ~KBaseGUI	: Destructor for KBase GUI client			*/
/*  (returns)	:		:					*/

KBaseGUI::~KBaseGUI ()
{
	allGUIs.remove (this) ;
}


/*  KBaseGUI								*/
/*  makeAction	: Construct action for GUI element			*/
/*  element	: const QDomElement &	: Action specification		*/
/*  guiSpec	: const GUIElement *	: GUI element specification	*/
/*  (returns)	: TKAction *		: New action			*/

TKAction*KBaseGUI::makeAction
	(	const QDomElement	&element,
		const GUIElement	*guiSpec
	)
{
	TKAction *act	;

	/* Extract the various bits of information from the DOM		*/
	/* specification element, in each case defaulting to the value	*/
	/* in the GUI specification					*/
	GType	 gtype	= (GType   )element.attribute("gtype",	QString::number(guiSpec->m_gtype )).toInt() ;
	GUIAccel accel	= (GUIAccel)element.attribute("accel",	QString::number(guiSpec->m_accel )).toInt() ;
	int	 ggroup	= 	    element.attribute("ggroup",	QString::number(guiSpec->m_ggroup)).toInt() ;
	bool	 enable	= 	    element.attribute("enable",	QString::number(guiSpec->m_enable)).toInt() ;
	QString	 text	= 	    element.attribute("text",	guiSpec->m_text) ;
	QString	 icon	= 	    element.attribute("icon",	guiSpec->m_icon) ;
	uint	 code	= 	    element.attribute("code",	QString::number(guiSpec->m_code  )).toInt() ;
	QString	 tip	= 	    element.attribute("tip",	guiSpec->m_tip ) ;
	QString	 es	= 	    element.attribute("slot") ;

	/* Similarly for the slot, except we need to process it into	*/
	/* standard Qt form.						*/
	QString	 slot	;
	if (es.isEmpty())
		slot	= guiSpec->m_slot  ;
	else	slot	= "1" + QObject::normalizeSignalSlot(es) ;


	/* Create the appropriate action type as specified in the	*/
	/* information structure.					*/
	switch (gtype)
	{
		case GTAction	:
			/* Simple case, the action is mapped to a	*/
			/* received slot that takes no arguments.	*/
			act	= new TKAction
				  (	text,
				  	icon,
					mapAccel(accel),
					m_receiver,
					slot,
					this
				  )	;
			break	;

		case GTToggle	:
			/* Toggle action ...				*/
			act	= new TKToggleAction 
				  (	text,
				  	icon,
					mapAccel(accel),
					m_receiver,
					slot,
					this
				  )	;
			break	;

		case GTCoded 	:
			/* Like GTAction, except that the receiver gets	*/
			/* an action code. Actually, this is exactly	*/
			/* the same here as a simple GTAction.		*/
			act	= new TKAction
				  (	text,
				  	icon,
					mapAccel(accel),
					m_receiver,
					slot,
					this
				  )	;
			break	;


		case GTPlugin	:
			/* GTPlugin actions support menu plugins. These	*/
			/* are handled via the KBPluginAction class.	*/
			act	= new KBPluginAction
				  (	text,
				  	icon,
					mapAccel(accel),
					m_receiver,
					element,
					this
				  )	;
			break	;

		default	:
			KBError::EFault
			(	QString("Unknown gtype %1").arg((int)guiSpec->m_gtype),
				QString::null,
				__ERRLOCN
			)	;
			return	0	;
	}

	/* Set the action code and group. These may be ignored		*/
	/* depending on the action type, but this keeps the API simple.	*/
	/* Also set the tool tip if there us one.			*/
	act->setCode  (code  ) ;
	act->setGroup (ggroup) ;
	if (!tip.isEmpty()) act->setToolTip (tip ) ;


	/* Set the initial enabled state, and add the action into the	*/
	/* actions dictionary, whence it will be found again if it	*/
	/* appears at multiple places in the GUI.			*/
	act->setEnabled  (enable) ;
	m_actions.insert (element.attribute("name"), act) ;
	return	act ;
}


/*  KBaseGUI								*/
/*  getAction	: Locate action for XML element				*/
/*  element	: QDomElement &	: Element in question			*/
/*  mySpec	: GUIElement  *	: Optional local specification		*/
/*  (returns)	: TKAction *	: Action or null if not found		*/

TKAction*KBaseGUI::getAction
	(	const QDomElement	&element,
		const GUIElement	*mySpec
	)
{
	TKAction	*act	;
	const QString	&name	= element.attribute("name") ;
	const QString	&mode	= element.attribute("mode") ;
	const QString	&gui	= element.attribute("gui" ) ;


	/* If the element specifies a mode then it should only appear	*/
	/* in that (MDI/SDI) mode.					*/
	if (!mode.isNull())
	{
		bool	useMDI	= KBAppPtr::getCallback()->useMDIMode() ;

		if ( useMDI && (mode == "sdi")) return 0 ;
		if (!useMDI && (mode == "mdi")) return 0 ;
	}

	/* Similarly, if a GUI is specified then only show the element	*/
	/* if running that (KDE/TKC) gui.				*/
	if (!gui.isNull())
	{
#if	__KB_KDE
		if (gui != "kde"  ) return 0 ;
#endif

#if	__KB_TKC
#ifdef	Q_OS_WIN32
		if (gui != "win32") return 0 ;
#else
		if (gui != "tkc"  ) return 0 ;
#endif
#endif
	}

	/* This is the clever bit, if I say so myself. GUI elements are	*/
	/* described once-and-for-all, so that the same description is	*/
	/* used one, however many different GUIs it appears in. Once	*/
	/* created, the action is stored in a dictionary so that it is	*/
	/* correctly reused for, say, a menu and a tool-bar. Finally,	*/
	/* by creating a separate instance of a GUI element for each	*/
	/* MDI child window's GUI, action state is preserved.		*/

	if ((act = m_actions.find (name)) != 0)
		return act ;

	if (mySpec != 0)
		for (const GUIElement *guiSpec = mySpec ; guiSpec->m_text != 0 ; guiSpec += 1)
			if (guiSpec->m_name == name)
				return	makeAction (element, guiSpec) ;

	for (const GUIElement *guiSpec = kbaseGUI ; guiSpec->m_text != 0 ; guiSpec += 1)
		if (guiSpec->m_name == name)
			return	makeAction (element, guiSpec) ;


	/* Not found in the definition tables so see if this is a	*/
	/* global action, such as snap-enable.				*/
	if ((act = KBAppPtr::getCallback()->globalAction (name)) != 0)
		return	act	;


	DPRINTF
	((	"KBaseGUI::getAction: [%s]: not found in code\n",
		(cchar *)name
	))	;

	return	makeAction (element, &nullGUI) ;
}

/*  KBaseGUI								*/
/*  getAction	: Get action by (XML GUI) name				*/
/*  name	: const QString & : Name				*/
/*  (returns)	: TKAction *	  : Action or null			*/

TKAction*KBaseGUI::getAction
	(	const QString		&name
	)
{
	TKAction *action	;
	if ((action = m_actions.find (name)) != 0) return action ;
	return	0 ;
}

/*  KBaseGUI								*/
/*  addAction	: Add an action						*/
/*  name	: const QString & : Action name				*/
/*  action	: TKAction *	  : The action				*/
/*  (returns)	: void		  :					*/

void	KBaseGUI::addAction
	(	const QString	&name,
		TKAction	*action
	)
{
	m_actions.insert (name, action) ;
}

/*  KBaseGUI								*/
/*  setChecked	: Set action checked state				*/
/*  name	: const QString & : Name				*/
/*  checked	: bool		  : Checked state			*/
/*  (returns)	: void		  :					*/

void	KBaseGUI::setChecked
	(	const QString	&name,
		bool		checked
	)
{
	TKAction	*action ;

	if ((action = getAction (name)) == 0)
		return	;

	if (!action->inherits("TKToggleAction"))
		return	;

	((TKToggleAction *)(action))->setChecked (checked) ;
}

/*  KBaseGUI								*/
/*  setEnabled	: Set action enabled state by name			*/
/*  name	: const QString & : Name				*/
/*  enabled	: bool		  : Enabled state			*/
/*  (returns)	: void		  :					*/

void	KBaseGUI::setEnabled
	(	const QString	&name,
		bool		enable
	)
{
	TKAction	*action	;
	if ((action = getAction (name)) != 0) action->setEnabled (enable) ;
}

/*  KBaseGUI								*/
/*  setEnabled	: Set action enabled state by group			*/
/*  ggroup	: KB::GGroup	  : Action group			*/
/*  enabled	: bool		  : Enabled state			*/
/*  (returns)	: void		  :					*/

void	KBaseGUI::setEnabled
	(	KBaseGUI::GGroup	ggroup,
		bool			enable
	)
{
	QDictIterator<TKAction>	aIter (m_actions) ;
	TKAction		*aPtr	;

	while ((aPtr = aIter.current()) != 0)
	{
		if (aPtr->getGroup () == ggroup)
			aPtr->setEnabled (enable) ;

		aIter	+= 1 ;
	}
}

/*  KBaseGUI								*/
/*  setAllChecked: Set all actions checked state			*/
/*  name	 : const QString & : Name				*/
/*  checked	 : bool		   : Checked state			*/
/*  (returns)	 : void		   :					*/

void	KBaseGUI::setAllChecked
	(	const QString	&name,
		bool		checked
	)
{
	LITER
	(	KBaseGUI,
		allGUIs,
		gui,
		gui->setChecked (name, checked)
	)
}

/*  KBaseGUI								*/
/*  setAllEnabled: Set all actions enabled state by name		*/
/*  name	 : const QString & : Name				*/
/*  enabled	 : bool		   : Enabled state			*/
/*  (returns)	 : void		   :					*/

void	KBaseGUI::setAllEnabled
	(	const QString	&name,
		bool		enable
	)
{
	LITER
	(	KBaseGUI,
		allGUIs,
		gui,
		gui->setEnabled (name, enable)
	)
}

/*  KBaseGUI								*/
/*  setAllEnabled: Set all actions enabled state by group		*/
/*  ggroup	 : KB::GGroup	  : Action group			*/
/*  enabled	 : bool		  : Enabled state			*/
/*  (returns)	 : void		  :					*/

void	KBaseGUI::setAllEnabled
	(	KBaseGUI::GGroup	ggroup,
		bool			enable
	)
{
	LITER
	(	KBaseGUI,
		allGUIs,
		gui,
		gui->setEnabled (ggroup, enable)
	)
}

