/***************************************************************************
    file	         : kb_helper.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qpushbutton.h>
#include	<qstringlist.h>

#include	"rk_hbox.h"
#include	"rk_vbox.h"
#include	"rk_pushbutton.h"
#include	"rk_modalfilter.h"

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_ascii.h"
#include	"kb_object.h"
#include	"kb_slot.h"
#include	"kb_script.h"

#if		__KB_AUTOMOC
#include 	"kb_helper.moc"
#else
#include 	"kb_helper.h"
#endif

#include	"kb_formatdlg.h"


static	KBHelperReg	*helperList	;
static	QStringList 	*helperSet	;

/*  getHelperSet: Get set of available helpers				*/
/*  (returns)	: QStringList *	: List of helpers			*/

QStringList *getHelperSet()
{
	if (helperSet == 0)
		helperSet = new QStringList ;
	return	helperSet ;
}

/*  ------------------------------------------------------------------  */

/*  KBHelperReg								*/
/*  KBHelperReg	: Helper registration class				*/
/*  helperName	: cchar *	: Helper name				*/
/*  helperFN	: MKHELPER	: Helper creation function		*/
/*  (returns)	: KBHelperReg	:					*/

KBHelperReg::KBHelperReg
	(	cchar		*helperName,
		MKHELPER	helperFN
	)
	:
	m_helperName	(helperName),
	m_helperFN	(helperFN)
{
	m_next		= helperList	;
	helperList	= this		;

	if (m_helperName[0] != '_') getHelperSet()->append(m_helperName) ;
}

bool	KBHelperReg::helperExists
	(	const QString	&helper
	)
{
	for (uint idx = 0 ; idx < getHelperSet()->count() ; idx += 1)
		if (helper == (*getHelperSet())[idx])
			return	true	;

	return	helper[0] == '_'	;
}

/*  ------------------------------------------------------------------  */

/*  KBFormatHelper							*/
/*  KBFormatHelper: Constructor for helper for field formats		*/
/*  parent	  : QWidget *	   	: Parent widget			*/
/*  dbInfo	  : const KBLocation &	: Owning document location	*/
/*  (returns)	  : KBFormatHelper 	:				*/

KBFormatHelper::KBFormatHelper
	(	QWidget			*parent,
		const KBLocation	&
	)
	:
	KBFormatDlg  (parent),
	KBHelperBase (this)
{
}

/*  KBFormatHelper							*/
/*  setValue	: Set initial format value				*/
/*  value	: const QString & : The value				*/
/*  (returns)	: void		  :					*/

void	KBFormatHelper::setValue
	(	const QString	&value
	)
{
	showFormats (value, KB::ITUnknown) ;
}

/*  KBFormatHelper							*/
/*  getValue	: Get the current format value				*/
/*  format	: const QString & : Optional format			*/
/*  (returns)	: QString	  : Format				*/

QString	KBFormatHelper::getValue
	(	const QString	&
	)
{
	return	KBFormatDlg::getValue () ;
}



static	KBHelperBase	*makeFormatHelper
	(	QWidget			*parent,
		const KBLocation	&location
	)
{
	return	new	KBFormatHelper (parent, location) ;
}

static	KBHelperReg	_mkFormatHelper	("format",	&makeFormatHelper) ;


/*  ------------------------------------------------------------------  */

/*  KBDateHelper							*/
/*  KBDateHelper: Constructor for helper for dates			*/
/*  parent	: QWidget *    		: Parent widget			*/
/*  dbInfo	: const KBLocation &	: Owning document location	*/
/*  months	: bool			: Show year/months only		*/
/*  (returns)	: KBDateHelper 		:				*/

KBDateHelper::KBDateHelper
	(	QWidget			*parent,
		const KBLocation	&,
		bool			months
	)
	:
	RKDatePicker 	(parent),
	KBHelperBase	(this),
	m_months	(months)
{
}

/*  KBDateHelper							*/
/*  setValue	: Set initial date value				*/
/*  value	: const QString & : The value				*/
/*  (returns)	: void		  :					*/

void	KBDateHelper::setValue
	(	const QString	&value
	)
{
	fprintf
	(	stderr,
		"KBDateHelper::setValue: [%s]->[%s]\n",
		value.latin1(),
		_TEXT(KBDateTime(value).getDate())
	)	;

	setDate (KBDateTime(value).getDate()) ;
}

/*  KBDateHelper							*/
/*  getValue	: Get the current format value				*/
/*  format	: const QString & : Optional format			*/
/*  (returns)	: QString	  : Date				*/

QString	KBDateHelper::getValue
	(	const QString	&format
	)
{
	fprintf
	(	stderr,
		"KBDateHelper::getValue: [%s] format=[%s]\n",
		date().toString().latin1(),
		format.latin1()
	)	;

	QDate	d = date () ;

	if (m_months)
		d = QDate (d.year(), d.month(), 1) ;

	if (format.isNull ())
		return	d.toString() ;

	return	KBDateTime(d).format (format) ;
}



static	KBHelperBase	*makeDateHelper
	(	QWidget			*parent,
		const KBLocation	&location
	)
{
	return	new	KBDateHelper   (parent, location, false) ;
}

static	KBHelperBase	*makeMonthHelper
	(	QWidget			*parent,
		const KBLocation	&location
	)
{
	return	new	KBDateHelper   (parent, location, true) ;
}

static	KBHelperReg	_mkDateHelper	("date",	&makeDateHelper ) ;
static	KBHelperReg	_mkMonthHelper	("month",	&makeMonthHelper) ;


/*  ------------------------------------------------------------------  */

/*  KBHelperDlg								*/
/*  KBHelperDlg	: Constructor for helper dialog				*/
/*  helper	: const QString & 	: Required helper		*/
/*  dbInfo	: const KBLocation &	: Owning document location	*/
/*  (returns)	: KBHelperDlg	  	:				*/

KBHelperDlg::KBHelperDlg
	(	const QString		&helper,
		const KBLocation	&location
	)
	:
	KBDialog  ("Helper", true)
{
	RKVBox	*layMain = new RKVBox (this) ;
	layMain->setTracking () ;

	m_dialog =  0	;

	for (KBHelperReg *reg = helperList ; reg != 0 ; reg = reg->m_next)
		if (helper == reg->m_helperName)
		{	m_dialog = (reg->m_helperFN) (layMain, location) ;
			break	 ;
		}

	addOKCancel (layMain) ;

	if (m_dialog == 0)
	{
		KBError::EError
		(	QString(TR("Helper %1 not known")).arg(helper),
			QString::null,
			__ERRLOCN
		)	;
		return	;
	}
}

/*  KBHelperDlg								*/
/*  ~KBHelperDlg: Destructor for helper dialog				*/
/*  (returns)	:		:					*/

KBHelperDlg::~KBHelperDlg ()
{
	DELOBJ	(m_dialog) ;
}


/*  KBHelperDlg								*/
/*  run		: Create and run helper					*/
/*  helper	: const QString &    : Helper name			*/
/*  initval	: const QString &    : Initial (ie., current) value	*/
/*  location	: const KBLocation & : Document location information	*/
/*  format	: const QString &    : Optional format			*/
/*  (returns)	: QString	     : New value or QString::null	*/

QString	KBHelperDlg::run
	(	const QString		&helper,
		const QString		&initval,
		const KBLocation	&location,
		const QString		&format
	)
{
	KBHelperDlg hDialog (helper, location) ;
	hDialog.setValue    (initval) ;
 
	if (hDialog.exec ()) return hDialog.getValue(format) ;

	return	 QString::null ;
}

/*  ------------------------------------------------------------------	*/

/*  KBHelperPopup							*/
/*  KBHelperPopup: Constructor for helper popup				*/
/*  helper	 : const QString &    : Required helper			*/
/*  location	 : const KBLocation & : Document location information	*/
/*  target	 : KBObject *	      : Object to notify on success	*/
/*  slotName	 : const QString &    : Target slot to invoke		*/
/*  format	 : const QString &    : Optional format			*/
/*  (returns)	 : KBHelperPopup      :					*/

KBHelperPopup::KBHelperPopup
	(	const QString		&helper,
		const KBLocation	&location,
		KBObject		*target,
		const QString		&slotName,
		const QString		&format
	)
	:
	QWidget
	(	0,
		"kbhelperpopup",
		WStyle_Customize|WStyle_Title|WStyle_NormalBorder|WStyle_Tool|WStyle_StaysOnTop|Qt::WDestructiveClose
	),
	m_helper (helper),
	m_target (target),
	m_format (format)
{
	RKVBox	*layMain = new RKVBox (this) ;
	layMain->setTracking () ;

	m_dialog = 0	;
	m_slot	 = 0	;

	for (KBHelperReg *reg = helperList ; reg != 0 ; reg = reg->m_next)
		if (helper == reg->m_helperName)
		{	m_dialog = (reg->m_helperFN) (layMain, location) ;
			break	 ;
		}

	const QPtrList<KBSlot> &slist = target->getSlots() ;
	LITER
	(	KBSlot,
		slist,
		slot,
		if (slot->name() == slotName)
		{	m_slot	 = slot ;
			break	 ;
		}
	)

	RKHBox	*buttons = new RKHBox (layMain) ;
	buttons->addFiller () ;
	RKPushButton *o  = new RKPushButton (TR("OK"),     buttons, "ok"    ) ;
	RKPushButton *c  = new RKPushButton (TR("Cancel"), buttons, "cancel") ;

	connect (o, SIGNAL(clicked()), SLOT(accept())) ;
	connect (c, SIGNAL(clicked()), SLOT(reject())) ;

	KBDialog::sameSize (o, c, 0) ;

	if (m_dialog == 0)
		KBError::EError
		(	QString(TR("Helper %1 not known")).arg(helper),
			QString::null,
			__ERRLOCN
		)	;

	if (m_slot   == 0)
		KBError::EError
		(	QString(TR("Slot %1 not found")).arg(slotName),
			QString::null,
			__ERRLOCN
		)	;

	RKModalFilter::self()->push (this) ;

	connect
	(	m_target,
		SIGNAL	(destroyed()),
		SLOT	(reject	  ())
	)	;
}

/*  KBHelperPopup							*/
/*  ~KBHelperPopup: Destructor for helper dialog			*/
/*  (returns)	:		:					*/

KBHelperPopup::~KBHelperPopup ()
{
	RKModalFilter::self()->pop () ;
	DELOBJ	(m_dialog) ;
}

/*  KBHelperPopup							*/
/*  ok		: Check helper is good to go				*/
/*  (returns)	: bool		: True if OK				*/

bool	KBHelperPopup::ok ()
{
	return	(m_dialog != 0) && (m_slot != 0) ;
}

/*  KBHelperPopup							*/
/*  accept	: User OK's dialog					*/
/*  (returns)	: void		:					*/

void	KBHelperPopup::accept ()
{
	if ((m_dialog != 0) && (m_slot != 0))
	{
		KBScriptError	*rc	= 0 ;
		KBValue		resval	;
		KBValue		arg	(m_dialog->getValue(m_format)) ;

		m_slot->eventSignal (m_target, m_helper, 1, &arg, resval, rc, 0) ;

		if (rc != 0) 
			KBScriptError::processError (rc, KBScriptError::Normal) ;
	}

	deleteLater () ;
}

/*  KBHelperPopup							*/
/*  reject	: User cancels dialog					*/
/*  (returns)	: void		:					*/

void	KBHelperPopup::reject ()
{
	deleteLater () ;
}

/*  KBHelperPopup							*/
/*  run		: Create and run helper					*/
/*  helper	: const QString &    : Helper name			*/
/*  initval	: const QString &    : Initial (ie., current) value	*/
/*  location	: const KBLocation & : Document location information	*/
/*  target	: KBObject *	     : Object to notify on success	*/
/*  slotName	: const QString &    : Target slot to invoke		*/
/*  format	: const QString &    : Optional format			*/
/*  (returns)	: void		     :					*/

void	KBHelperPopup::run
	(	const QString		&helper,
		const QString		&initval,
		const KBLocation	&location,
		KBObject		*target,
		const QString		&slotName,
		const QString		&format
	)
{
	KBHelperPopup *hDialog = new KBHelperPopup
					(	helper,
						location,
						target,
						slotName,
						format
					)	;

	if (!hDialog->ok())
	{
		delete	hDialog	;
		return	;
	}

	hDialog->setValue (initval) ;
	hDialog->show	  ()	    ;
}
