/***************************************************************************
    file	         : kb_helper.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_HELPER_H
#define	__KB_HELPER_H


#include	"libkbase_exports.h"

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"
#include	"kb_dialog.h"
#include	"kb_formatdlg.h"


#include	"rk_datepicker.h"

class	KBSlot	;


/*  KBHelperBase							*/
/*  ------------							*/
/*  Virtual base class from which helpers must be derived. Provides a	*/
/*  common setValue/getValue interface.					*/

class	LIBKBASE_API	KBHelperBase
{
	QWidget		*m_widget	;

public	:

	inline	KBHelperBase
		(	QWidget	*widget
		)
		:
		m_widget (widget)
	{
	}

	virtual	~KBHelperBase () {} ;

	virtual	void	setValue (const QString &) = 0 ;
	virtual QString	getValue (const QString & = QString::null) = 0 ;
	inline	QWidget	*widget	 () { return m_widget ; }
}	;

/*  KBHelperFormat							*/
/*  --------------							*/
/*  Helper class for format selection					*/

class	LIBKBASE_API	KBFormatHelper : public KBFormatDlg, public KBHelperBase
{
public	:

	KBFormatHelper (QWidget *, const KBLocation &) ;
	virtual	~KBFormatHelper () {} ;

	virtual	void	setValue (const QString &) ;
	virtual QString	getValue (const QString & = QString::null) ;
}	;

/*  KBHelperDate							*/
/*  ------------							*/
/*  Helper class for date selection					*/

class	LIBKBASE_API	KBDateHelper : public RKDatePicker, public KBHelperBase
{
	bool		m_months	;

public	:

	KBDateHelper (QWidget *, const KBLocation &, bool = false) ;
	virtual	~KBDateHelper () {} ;

	virtual	void	setValue (const QString &) ;
	virtual QString	getValue (const QString & = QString::null) ;
}	;

/*  KBHelperDlg								*/
/*  -----------								*/
/*  Dialog used for input helpers. Invoked with a helper specification	*/
/*  to show the corresponging helper.					*/

class	LIBKBASE_API	KBHelperDlg : public KBDialog
{
	Q_OBJECT

	KBHelperBase		*m_dialog	;

public	:

	KBHelperDlg (const QString &, const KBLocation &) ;
       ~KBHelperDlg () ;

	inline	QString	getValue
		(	const QString	&fmt	= QString::null
		)
	{
		return	m_dialog == 0 ? QString::null : m_dialog->getValue (fmt) ;
	}

	void	setValue
		(	const QString 	&value
		)
	{
		if (m_dialog != 0) m_dialog->setValue (value) ;
	}

	static	QString	run
		(	const QString		&,
			const QString		&,
			const KBLocation 	&,
			const QString		& = QString::null
		)	;
}	;

/*  KBHelperPopup							*/
/*  -------------							*/

class	LIBKBASE_API	KBHelperPopup : public QWidget
{
	Q_OBJECT

	QString			m_helper	;
	KBObject		*m_target	;
	KBHelperBase		*m_dialog	;
	KBSlot			*m_slot		;
	QString			m_format	;

protected slots :

	void			accept		() ;
	void			reject		() ;

public	:

	KBHelperPopup
	(	const QString 		&,
		const KBLocation	&,
		KBObject		*,
		const QString		&,
		const QString		&
	)	;

       ~KBHelperPopup () ;

	void	setValue
		(	const QString 	&value
		)
	{
		if (m_dialog != 0) m_dialog->setValue (value) ;
	}

	bool		ok	()	;

	static	void	run
		(	const QString		&,
			const QString		&,
			const KBLocation	&,
			KBObject		*,
			const QString		&,
			const QString		& = QString::null
		)	;
}	;


/*  KBHelperReg								*/
/*  -----------								*/
/*  Instance of this class can be created to register a helper.		*/

typedef	KBHelperBase	*(*MKHELPER) (QWidget *, const KBLocation &) ;

class	LIBKBASE_API	KBHelperReg
{
	cchar		*m_helperName	;
	MKHELPER	m_helperFN	;
	KBHelperReg	*m_next		;

public	:

	KBHelperReg	(cchar *, MKHELPER) ;

	friend	class	KBHelperDlg	;
	friend	class	KBHelperPopup	;

	static	bool	helperExists	(const QString &) ;
}	;

#endif	// __KB_HELPER_H
