/***************************************************************************
    file	         : kb_hidden.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_sizer.h"
#include	"kb_ctrl.h"
#include	"kb_itemdlg.h"
#include	"kb_block.h"
#include	"kb_nodemonitor.h"
#include	"kb_nodereg.h"
#include	"kb_script.h"

#if		__KB_AUTOMOC
#include	"kb_hidden.moc"
#else
#include	"kb_hidden.h"
#endif

#if		! __KB_RUNTIME
#include	"kb_recorder.h"
#endif

#define		__DEBUG_HIDDEN	(0)
#if		__DEBUG_HIDDEN
#include	"kb_dprintf.h"
#endif


/*  KBHiddenValue							*/
/*  -------------							*/
/*  This class is to a hidden control what the KBControl class is to	*/
/*  "real" controls. It stores the current value, plus a pointer to a	*/
/*  monitor node if monitoring is enabled.				*/

class	KBHiddenValue
{
public	:

	KBValue		m_value		;
	KBNodeMonitor	*m_monitor	;
	KBHiddenValue ()		;
}	;


/*  KBHiddenValue							*/
/*  KBHiddenValue: Constructor for hidden control value			*/
/*  (returns)	 : KBHiddenValue :					*/

KBHiddenValue::KBHiddenValue ()
{
	m_monitor = 0 ;
}



/*  ------------------------------------------------------------------	*/

/*  KBHidden								*/
/*  KBHidden	: Constructor for hidden form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*		: bool *	:					*/
/*  (returns)	: KBNode	:					*/

KBHidden::KBHidden
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*
	)
	:
	KBItem	(parent, "KBHidden",	"expr",  aList)
{
	init	()  ;
}

/*  KBHidden								*/
/*  KBHidden	: Constructor for hidden form field			*/
/*  parent	: KBObject *	: Parent node				*/
/*  name	: const QString&: Control name				*/
/*  expr	: const QString&: Expression				*/
/*  (returns)	: KBHidden	:					*/

KBHidden::KBHidden
	(	KBObject	*parent,
		bool		&ok
	)
	:
	KBItem	(parent, "KBHidden",	QRect(), "expr", "", 0)
{
	init	() ;

#if	! __KB_RUNTIME
	KBItemPropDlg hDlg (this, "Hidden", m_attribs) ;
	if (!hDlg.exec())
	{	ok	= false	;
		return	;
	}
#endif
	ok	= true	;
}

/*  KBHidden								*/
/*  KBHidden	: Constructor for hidden form field			*/
/*  _parent	: KBNode *	: Parent node				*/
/*  _hidden	: KBHidden *	: Extant hidden field			*/
/*  (returns)	: KBNode	:					*/

KBHidden::KBHidden
	(	KBNode		*_parent,
		KBHidden	*_hidden
	)
	:
	KBItem	(_parent, "expr", _hidden)
{
	init	() ;
}

/*  KBHidden								*/
/*  ~KBHidden	: Destructor for simple form field			*/
/*  (returns)	:		:					*/

KBHidden::~KBHidden ()
{
	for (uint idx = 0 ; idx < m_values.count() ; idx += 1)
		delete	m_values.at(idx) ;
}

/*  KBHidden								*/
/*  init	: Initialise						*/
/*  (returns)	: void		:					*/

void	KBHidden::init ()
{
	m_tabOrd.setValue (0)	;

	m_geom.set	(0, 0, 0, 0) ;
	m_geom.setMask	(KBAttrGeom::HideAll) ;

	/* We detach all the attributes that are not relevent, so that	*/
	/* they will not show up in the properties dialog.		*/
	m_attribs.remove (&m_tabOrd    ) ;
	m_attribs.remove (&m_defVal    ) ;
	m_attribs.remove (&m_rdonly    ) ;
	m_attribs.remove (&m_onEnter   ) ;
	m_attribs.remove (&m_onLeave   ) ;
	m_attribs.remove (&m_onDblClick) ;
}

/*  KBHidden								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBHidden::showAs
	(	KB::ShowAs	mode
	)
{
	KBItem::showAs (mode) ;
}

/*  KBHidden								*/
/*  calcNumRows	: Calculate number of rows which can be show		*/
/*  bw		: uint		: Block display width			*/
/*  bh		: uint		: Block display height			*/
/*  dx		: int		: Control X-spacing			*/
/*  dy		: int		: Control Y-spacing			*/
/*  (returns)	: uint		: Number of rows that can be displayed	*/

uint	KBHidden::calcNumRows
	(	uint	,
		uint	,
		int	,
		int
	)
{
	/* There is no limit since hidden controls do not actually	*/
	/* appear on the display.					*/
	return	999	;
}

/*  KBHidden								*/
/*  dx		: int		: Control X-spacing			*/
/*  dy		: int		: Control Y-spacing			*/
/*  limit	: uint		: Upper limit on number of rows		*/
/*  (returns)	: uint		: Number of rows that can be displayed	*/

uint	KBHidden::calcNumRows
	(	int	,
		int	,
		uint	limit
	)
{
	return	limit	;
}

/*  KBHidden								*/
/*  setupControls							*/
/*		: Extend (or decrease) the number of controls		*/
/*  (returns)	: void		:					*/

void	KBHidden::setupControls ()
{
	uint	newRows	= getBlock()->getDisplayRows()	;
	uint	oldRows	= m_values.count()		;

	DPRINTF
	((	"KBHidden::setupControls: %s: %d->%d\n",
		getPath().latin1(),
		oldRows,
		newRows
	))	;
	if (newRows <= oldRows)
	{
		for (uint idx1 = newRows ; idx1 < oldRows ; idx1 += 1)
		{	delete	m_values.at(idx1)->m_monitor ;
			delete	m_values.at(idx1) ;
		}

		m_values.resize (newRows) ;
		return	;
	}

	m_values.resize (newRows) ;
	for (uint idx2 = oldRows ; idx2 < newRows ; idx2 += 1)
	{
		KBHiddenValue *hv = new KBHiddenValue () ;
		m_values.at(idx2) = hv ;

		if (m_ctrlMon == 0) continue ;

		KBNodeMonitor *nm = new KBNodeMonitor (0, m_ctrlMon) ;
		nm->setText (0, "Control") ;
		nm->setText (1, QString("Row %1").arg(idx2)) ;

		hv->m_monitor = nm ;
	}
}

/*  KBHidden								*/
/*  setMonitor	: Set monitor values for this node			*/
/*  myitem	: KBNodeMonitor * : Monitor item for this node		*/
/*  (returns)	: void		  :					*/

void	KBHidden::setMonitor
	(	KBNodeMonitor	*myitem
	)
{
	KBItem::setMonitor (myitem) ;

	if (m_ctrlMon == 0)
	{
		for (uint idx = 0 ; idx < m_values.count() ; idx += 1)
			m_values.at(idx)->m_monitor = 0 ;
		return	;
	}

	for (uint idx = 0 ; idx < m_values.count() ; idx += 1)
	{
		KBNodeMonitor *nm = new KBNodeMonitor (0, m_ctrlMon) ;
		nm->setText (0, "Control") ;
		nm->setText (1, QString("Row %1").arg(idx)) ;
		m_values.at(idx)->m_monitor = nm ;
	}
}

/*  KBHidden								*/
/*  valuesAtQRow: Get value pair at specified row			*/
/*  qrow	: uint		  : Specific row			*/
/*  (returns)	: KBHiddenValue * : Value or none			*/

KBHiddenValue
	*KBHidden::valueAtQRow
	(	uint	qrow
	)
{
	uint	curDRow  = getBlock()->getCurDRow () ;

	if ((qrow >= curDRow) && (qrow < curDRow + m_values.count()))
		return	m_values.at(qrow - curDRow) ;

	DPRINTF
	((	"KBHidden::valueAtQRow: %s: qrow=%d: showing [%p] %d for %d\n",
		getPath().latin1(),
		qrow,
		curDRow,
		m_values.count()
	))	;

	setError
	(	KBError::Error,
		TR("Row in query not currently displayed"),
		QString	(TR("Hidden %1, query row %2: showing %3 for %4"))
			.arg(getName())
			.arg(qrow)
			.arg(curDRow)
			.arg( m_values.count()),
		__ERRLOCN
	)	;

	return	0 ;
}

/*  KBHidden								*/
/*  getValue	: Get value						*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBHidden::getValue
	(	uint	qrow
	)
{
	KBHiddenValue *value = valueAtQRow (qrow) ;

	DPRINTF
	((	"KBHidden::getValue: %s: %d: %p\n",
		getPath().latin1(),
		qrow,
		value
	))	;

	if (value == 0)
		return	KBValue() ;

	if (value->m_value.isEmpty())
		if (!m_defVal.getValue().isEmpty())
			return	KBValue (m_defVal.getValue(), m_type) ;

	return	value->m_value	;
}

/*  KBHidden								*/
/*  setValue	: Set value display row					*/
/*  qrow	: uint		 : Query row				*/
/*  value	: const KBValue &: Value to set				*/
/*  (returns)	: bool		 : Success				*/

bool	KBHidden::setValue
	(	uint		qrow,
		const KBValue	&value
	)
{
	KBScriptError	*eRC	;
	bool		subs	;
	KBValue		v	= m_expr.evaluate (value, eRC, subs) ;

	if (eRC != 0)
	{
		KBScriptError::processError (eRC, KBScriptError::Hidden) ;
		return	false	;
	}

	KBHiddenValue *vp	= valueAtQRow (qrow) ;

	DPRINTF
	((	"KBHidden::setValue: %s: qrow=%d vp=%p v=[%s]\n",
		getPath().latin1(),
		qrow,
		(void  *)vp,
		(cchar *)v.getRawText()
	))	;
	if (vp == 0) return true ;

	vp->m_value = v ;

	if (vp->m_monitor != 0)
	{
		QString	t = v.getRawText() ;
		if (t.length() > 80)
		{	t.truncate  (77)    ;
			t.append    ("...") ;
		}

		vp->m_monitor->setText (2, t) ;
	}

	return	true	;
}

/*  KBHidden								*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  qrow	: uint		: Starting row in query			*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBHidden::clearValue
	(	uint	qrow,
		bool
	)
{
	KBHiddenValue	*value = valueAtQRow (qrow) ;
	if (value != 0) value->m_value = KBValue (&_kbString) ;
}

/*  KBHidden								*/
/*  changed	: See if item has changed				*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Item changed				*/

bool	KBHidden::changed
	(	uint	qrow
	)
{
	KBHiddenValue	*value = valueAtQRow (qrow) ;

	DPRINTF
	((	"KBHidden::changed: %s: %d: [%s][%s]\n",
		getName().latin1(),
		qrow,
		value == 0 ? "/NONE/" : value->m_value.getRawText().latin1(),
		getIniValue(qrow).getRawText().latin1()
	))	;

	return	value == 0 ? false : value->m_value != getIniValue (qrow) ;
}

/*  KBHidden								*/
/*  isValid	: Test if value is valid to be saved			*/
/*  qrow	: uint		: Query row				*/
/*  alowNull	: bool		: Ignore not-null check			*/
/*  (returns)	: bool		: Value can be saved			*/

bool	KBHidden::isValid
	(	uint	qrow,
		bool
	)
{
	return	valueAtQRow (qrow) != 0 ;
}

/*  KBHidden								*/
/*  isEmpty	: See if value is empty					*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Empty					*/

bool	KBHidden::isEmpty
	(	uint	qrow
	)
{
	KBHiddenValue	*value	= valueAtQRow (qrow) ;
	return	value == 0 ? true : value->m_value.isEmpty() ;
}

/*  KBHidden								*/
/*  replicate	: Replicate this hidden field				*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New hidden node			*/

KBNode	*KBHidden::replicate
	(	KBNode	*parent
	)
{
	return	new KBHidden (parent, this) ;
}

/*  KBHidden								*/
/*  recordVerifyValue							*/
/*  		: Record value verification for test			*/
/*  (returns)	: void		:					*/

void	KBHidden::recordVerifyValue ()
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBHidden::recordVerifyValue:\n"
	))	;

	KBRecorder *recorder = KBRecorder::self() ;
	if ((recorder != 0) && recorder->isRecording(getDocRoot()))
		recorder->verifyValue
		(	this,
			getBlock()->getCurQRow() - getBlock()->getCurDRow(),
			getValue(getBlock()->getCurQRow())
		)	;
#endif
}

#if	! __KB_RUNTIME
bool	KBHidden::propertyDlg
	(	cchar		*iniAttr
	)
{
	KBItemPropDlg hDlg (this, "Hidden", m_attribs) ;
	return	hDlg.exec() ;
}
#endif


NEWNODE(Hidden, (cchar *)0, 0, KNF_FORM)
