/***************************************************************************
    file	         : kb_highlighter.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_HIGHLIGHTER_H
#define	__KB_HIGHLIGHTER_H

#include	<qsyntaxhighlighter.h>
#include	<qdict.h>
#include	<qregexp.h>
#include	<qvaluelist.h>


class	QDomElement		;
class	KBSyntaxHighlighter	;
class	QStringList		;
class	KBTextEdit		;


/*  KBHLHighlight							*/
/*  -------------							*/
/*  Class used to store a highlight configuration, specifically a color	*/
/*  and a style.							*/

class	KBHLHighlight
{
public	:

	enum	Style
	{
		Normal	= 0,
		Bold	= 1,
		Italic	= 2
	}	;

	KBHLHighlight	(const QDomElement &) ;

private	:

	QString		m_name		;
	QColor		m_color		;
	Style		m_style		;

public	:

	inline	const QString	&name	()
	{
		return	m_name	;
	}
	inline	const QColor	&color	()
	{
		return	m_color	;
	}
	inline	Style		style	()
	{
		return	m_style	;
	}
}	;


/*  KBHLSection								*/
/*  -----------								*/
/*  This class stores information about a highlight section. This is a	*/
/*  regular expression which matches the highlight area (and an		*/
/*  optional expression for the end of the section where the section	*/
/*  can straddle paragraphs).						*/

class	KBHLSection
{
	int			m_esolp		;
	QRegExp			m_reFrom	;
	QRegExp			m_reUntil	;
	QPtrList<KBHLHighlight>	m_highlight	;

public	:

	KBHLSection	(const QDomElement &, const QDict<KBHLHighlight> &) ;

	QColor			color	(uint)	;
	QFont			font	(uint, KBSyntaxHighlighter *) ;

	inline	bool		ok	()
	{
		return	m_highlight.count() > 0	;
	}

	inline	int		esolp	()
	{
		return	m_esolp		;
	}

	inline	QRegExp		&from	()
	{
		return	m_reFrom	;
	}
	inline	QRegExp		&until	()
	{
		return	m_reUntil	;
	}
}	;


/*  KBHLHighlighter							*/
/*  ---------------							*/
/*  This class contains all information for a specific highlight	*/
/*  configuration. The "highlight" method then does the highlighting on	*/
/*  specified text.							*/

class	KBHLHighlighter
{
	QDict<KBHLHighlight>		m_highlights	;
	QPtrList<KBHLSection>		m_sections	;

	static	QDict<KBHLHighlighter>	*m_highlighters	;

public	:

	KBHLHighlighter	(const QString &) ;

	int		highlight
			(	const QString		&,
				int,
				KBSyntaxHighlighter	*
			)	;

	static	KBHLHighlighter	*find		(const QString &) ;
}	;


/*  KBSyntaxHighlighter							*/
/*  -------------------							*/
/*  This is our implementation of the QSyntaxHighlighter abstract base	*/
/*  class.								*/

class	KBSyntaxHighlighter : public QSyntaxHighlighter 
{
	KBHLHighlighter	*m_highlighter	;

	QFont		m_normal	;
	QFont		m_bold		;
	QFont		m_italic	;

	static	void	loadHighlights	() ;

public	:

	KBSyntaxHighlighter (KBTextEdit *, const QString &, const QFont &) ;

	virtual	int	highlightParagraph
			(	const QString	&,
				int
			)	;

	void	setFont	(const QFont &font)	;

	inline	const QFont	&bold	()
	{
		return	m_bold		;
	}
	inline	const QFont	&italic	()
	{
		return	m_italic	;
	}
	inline	const QFont	&normal	()
	{
		return	m_normal	;
	}

	static	QStringList	highlightNames	() ;
}	;

#endif	// __KB_HIGHLIGHTER_H
