/***************************************************************************
    file	         : kb_intelli.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qapplication.h>
#include	<qcursor.h>
#include	<qlabel.h>
#include	<qfontmetrics.h>
#include	<qtextbrowser.h>
#include	<qsimplerichtext.h>
#include	<qtooltip.h>

#include	<qdict.h>

#include	"rk_listbox.h"
#include	"kb_classes.h"
#include	"kb_textedit.h"

#include	"kb_dom.h"
#include	"kb_node.h"
#include	"kb_block.h"
#include	"kb_ascii.h"

#include	"kb_intelli.moc"

#define		__DEBUG_INTELLI	(0)
#if		__DEBUG_INTELLI
#include	"kb_dprintf.h"
#endif

class	QPainter	;


KBIntelliScanResult::KBIntelliScanResult ()
	:
	m_node		(0),
	m_offset	(-1),
	m_complete	(true)
{
}


/*  ------------------------------------------------------------------	*/

KBIntelliScanMappingItem::KBIntelliScanMappingItem ()
{
}

KBIntelliScanMappingItem::KBIntelliScanMappingItem
	(	const QString	&pattern,
		const QString	&klass
	)
	:
	m_regexp	(pattern),
	m_class		(klass)
{
	DPRINTF
	((	"KBIntelliScanMappingItem: [%s]->[%s]\n",
		pattern.latin1(),
		klass  .latin1()
	))	;
}

QString	KBIntelliScanMappingItem::match
	(	const QString	&text
	)
	const
{
	return	m_regexp.search (text) == 0 ? m_class : QString::null ;
}

/*  ------------------------------------------------------------------	*/

KBIntelliScan::KBIntelliScan
	(	const QString		&language
	)
	:
	KBMethDict	(language, "methods"),
	m_language	(language)
{
	for
	(	QMap<QString,QDomElement>::Iterator iter = m_classes.begin() ;
		iter != m_classes.end() ;
		++iter
	)
		if (!iter.data().attribute ("intelli").isEmpty())
			m_mappings.append
			(	KBIntelliScanMappingItem
				(	iter.data().attribute ("intelli"),
					iter.data().attribute ("name"   )
			)	)	;
}


bool	KBIntelliScan::wordChar
	(	const QChar	&ch
	)
{
	if (ch.isLetterOrNumber()) return true ;
	if (ch == '_'		 ) return true ;
	return	false ;
}

QStringList
	KBIntelliScan::tokenize
	(	const QString	&line,
		int		limit
	)
{
	QStringList	tokens		;
	QString		token		;
	QChar		quoted	= '.'	;
	int		offset	= 0	;

	while (offset < limit)
	{
		QChar ch = line.at(offset) ;

		if (ch.isSpace() && (quoted == '.'))
		{
			offset	+= 1	;
			continue	;
		}

		/* Backslash is treated as an escape both inside and	*/
		/* outside quoted text.					*/
		if (ch == '\\')
		{	token    += ch	;
			token    += line.at(offset+1) ;
			offset   += 2	;
			continue	;
		}

		/* Unless escaped, a quote character toggles the quote	*/
		/* state.						*/
		if ((ch == '\'') || (ch == '"'))
		{
			if	(ch ==  quoted)
			{
				tokens.append	("'" + token) ;
				quoted	 = '.'	;
				offset	+= 1	;
				token	 = QString::null ;
				continue	;
			}

			if (quoted == '.')
				quoted  = ch	;
			else	token  += ch	;
			offset += 1	  ;
			continue  ;
		}

		/* If quoted then the character passes though unchanged	*/
		/* as also if it cannot start a new word.		*/
		if (quoted != '.')
		{	token  += ch	;
			offset += 1	;
			continue  	;
		}

		if (!wordChar(ch))
		{
			tokens.append	(QString(ch)) ;
			offset += 1	;
			continue	;
		}

		while ((offset < limit) && wordChar(ch))
		{
			token  += ch	;
			offset += 1	;
			ch	= line.at(offset) ;
		}

		tokens.append (token)	;
		token	= QString::null	;
	}

	return	tokens	;
}

KBIntelliScanResult
	KBIntelliScan::scanTokens
		(	const QString			&selfName,
			KBNode				*selfNode,
			const QString			&prefix,
			const QStringList		&tokens
		)
{
	KBNode	    		*atNode = 0 ;
	uint	    		tokp    = 0 ;
	KBIntelliScanResult	result	;

	result.m_prefix	= prefix ;

	if (tokens.count() == 0) return result ;

	/* The "atNode" variable points to whatever node we have	*/
	/* reached as we process the tokens. Whenever we hit anything	*/
	/* that we cannot process, reset this and continue. This allows	*/
	/* things like:							*/
	/*								*/
	/*	sum = self.getValue(row) + self.get^			*/
	/*								*/
	/* ... where ^ is the initial offset, to get processed in a	*/
	/* hopefully plausible way.					*/
	while (tokp < tokens.count())
	{
		QString token = tokens[tokp] ;

		if (atNode == 0)
		{
			if (token == selfName)
			{
				atNode	= selfNode ;
				tokp   += 1	   ;
				continue	   ;
			}

			if (m_language == "py")
				if (token == "root")
				{
					atNode	= selfNode == 0 ? 0 : selfNode->getRoot() ;
					tokp   += 1	;
					continue	;
				}

			tokp	+= 1 ;
			continue     ;
		}

		/* If the next token is a period we should have either	*/
		/* a method application or a property ....		*/
		if (token == ".")
		{
			/* ... and we need at least one more token	*/
			/* beyond the period.				*/
			tokp	+= 1 ;
			if (tokp >= tokens.count()) return result ;

			token = tokens[tokp] ;

			/* Assume that this is a property in which case	*/
			/* we gobble just the token, then check to see	*/
			/* if we actually have a method application. In	*/
			/* the latter case we will gobble up to the	*/
			/* matching closing bracket.			*/
			bool	isfn	= false	   ;
			uint	nextp	= tokp + 1 ;

			if ((tokens.count() - tokp >= 3) && (tokens[tokp + 1] == "("))
			{
				uint	ketp	= tokp + 2 ;
				uint	nest	= 1	   ;

				while (ketp < tokens.count())
				{
					if (tokens[ketp] == "(") nest += 1 ;
					if (tokens[ketp] == ")") nest -= 1 ;
					if (nest == 0) break ;
					ketp += 1 ;
				}

				/* If we did not find a matching	*/
				/* closing bracket then we are inside	*/
				/* some function arguments, so try	*/
				/* recursively on just the arguments.	*/
				if (nest > 0)
				{
					QStringList subtok ;

					for (uint idx = tokp + 2 ; idx < tokens.count() ; idx += 1)
						subtok.append (tokens[idx]) ;

					return	scanTokens
						(	selfName,
							selfNode,
							prefix,
							subtok
						)	;
				}

				nextp	= ketp + 1 ;
				isfn	= true	   ;
			}

			/* The following three cases can be handled in	*/
			/* same way for both the property and the	*/
			/* method application cases.			*/
			if ((token == "__root__"  ) || (token == "getRoot"  ))
			{
				atNode	= atNode->getRoot () ;
				tokp	= nextp	;
				continue    ;
			}
			if ((token == "__parent__") || (token == "getParent"))
			{
				atNode	= atNode->getParent() ;
				tokp	= nextp	;
				continue    ;
			}
			if ((token == "__block__" ) || (token == "getBlock" ))
			{
				atNode	= atNode->getBlock () ;
				tokp	= nextp	;
				continue    ;
			}

			/* If none of the above, see if we have a	*/
			/* "getNamedCtrl" method or, equivalently, a	*/
			/* single (non-method) word which may be the	*/
			/* name of a child object. Both these cases	*/
			/* can then be handled the same way.		*/
			bool	gnc	= false	;

			if (isfn)
			{
				if
				(	(token == "getNamedCtrl"  ) &&
					(nextp == tokp + 4        ) &&
					(tokens[tokp+2][0] == '\'')
				)
				{
					gnc	= true	;
					token	= tokens[tokp+2].mid(1) ;
				}
			}
			else	gnc	= nextp == tokp + 1 ;

			if (gnc)
			{
				KBNode *child = atNode->getNamedNode (token) ;
				if (child != 0)
				{
					atNode	= child ;
					tokp	= nextp	;
					continue  ;
				}
			}

			/* Nothing found that we can interpret so reset	*/
			/* and continue.				*/
			atNode	= 0	;
			tokp	= nextp	;
			continue	;
		}

		/* Nothing found that we can interpret so reset and	*/
		/* continue.						*/
		atNode	= 0	;
		tokp   += 1	;
	}

	/* If we have a known node then return results based in it.	*/
	/* This is the best case.					*/
	if (atNode != 0)
	{
		result.m_node	= atNode	;
		result.m_class	= atNode->getElement().mid(2) ;
		return	result	;
	}

	/* No node put if the token list ends with __root__, __block__	*/
	/* or their method equivalents, then we can deduce something	*/
	/* about the object.						*/
	int		tokCnt	 = tokens.count  () ;
	const QString	&lastTok = tokens[tokCnt-1] ;
	bool		isForm	 = selfNode == 0 ? false : selfNode->getRoot()->isForm  () != 0 ;
	bool		isReport = selfNode == 0 ? false : selfNode->getRoot()->isReport() != 0 ;
	bool		goRoot	 = false ;
	bool		goBlock	 = false ;

	if	(lastTok == "__root__" )
	{
		goRoot	= true	;
	}
	else if (lastTok == "__block__")
	{
		goBlock	= true	;
	}
	else if
		(	(tokCnt           >=   3) &&
			(tokens[tokCnt-1] == ")") &&
			(tokens[tokCnt-2] == "(")
		)
	{
		if (tokens[tokCnt-3] == "getRoot" ) goRoot  = true ;
		if (tokens[tokCnt-3] == "getBlock") goBlock = true ;
	}

	if (goRoot )
	{
		if (selfNode != 0)
		{
			result.m_node	= selfNode->getRoot() ;
			result.m_class	= selfNode->getRoot()->getElement().mid(2) ;
		}
		else	result.m_class	= "Block" ;

		return	result	;
	}

	if (goBlock)
	{
		if 	(isForm  ) result.m_class = "FormBlock"	 ;
		else if (isReport) result.m_class = "ReportBlock";
		else		   result.m_class = "Block"	 ;
		return	result	;
	}

	/* If no node then see if the last token matches any of the	*/
	/* mappings. This handles cases like:				*/
	/*								*/
	/*	time.strftime						*/
	/*	btnClose.setText					*/
	/*								*/
	for (uint idx = 0 ; idx < m_mappings.count() ; idx += 1)
	{
		QString	klass	= m_mappings[idx].match(lastTok) ;
		if (!klass.isEmpty())
		{
			result.m_class	= klass	;
			return	result	;
		}
	}

	return	result	;
}

KBIntelliScanResult
	KBIntelliScan::scanLine
		(	const QString			&selfName,
			KBNode				*selfNode,
			const QString			line
		)
{
	int	dotp	= line.length() ;
	QString	prefix	= QString::null	;

	/* Skip back over white space to the end of the last token on	*/
	/* the line prior to the start position. Fail if nothing is	*/
	/* found. If something is found and there was no whitespace	*/
	/* then add a match-anything to the end of the prefix.		*/
	int	ptr1	= dotp	;
	while ((ptr1 > 0) && line[ptr1 - 1].isSpace())
		ptr1	-= 1	;
	if (ptr1 <= 0)
		return	KBIntelliScanResult ()	;

	/* Now scan back over characters that can appear are part of a	*/
	/* word, accumulating these as the prefix. Then check that the	*/
	/* first character of the prefix is a letter.			*/
	int	ptr2	= ptr1	;
	while ((ptr2 >= 0) && wordChar(line[ptr2-1]))
		ptr2	-= 1	;
	if (ptr2 <= 0)
		return	KBIntelliScanResult () ;

	if (ptr2 == ptr1)
		prefix	= "" ;
	else	prefix	= line.mid (ptr2, ptr1 - ptr2) ;

	DPRINTF
	((	"KBIntelliScan::scanLine: dotp=%d ptr1=%d ptr2=%d pfx=[%s]\n",
		dotp,
		ptr1,
		ptr2,
		prefix.latin1()
	))	;

	if ((ptr2 < ptr1) && !prefix[0].isLetter())
		return	KBIntelliScanResult () ;

	/* Next check that the first non-whitespace character prior to	*/
	/* the prefix is a period. If this is OK then we have something	*/
	/* that looks like a possible method application.		*/
	int	ptr3	= ptr2	;
	while ((ptr3 > 0) && line[ptr3 - 1].isSpace())
		ptr3	-= 1	;
	if (ptr3 <= 0)
		return	KBIntelliScanResult () ;

	if (line[ptr3 - 1] != '.')
		return	KBIntelliScanResult () ;
	ptr3	-= 1	;

	/* Tokenize the line up to the point just before the period	*/
	/* that we just found. The string of tokens will then be	*/
	/* processed to try to figure out the node to which this	*/
	/* evaluates. Note that this should work OK for both Python and	*/
	/* for JavaScript.						*/
	QStringList tokens  = tokenize (line, ptr3) ;

//	for (uint idx = 0 ; idx < tokens.count() ; idx += 1)
//		fprintf
//		(	stderr,
//			"scanLine: token: [%s]\n",
//			tokens[idx].latin1()
//		)	;

	KBIntelliScanResult result = scanTokens (selfName, selfNode, prefix, tokens) ;
	result.m_offset	  = ptr2	;
	result.m_complete = ptr1 < dotp ;
	return	result	;
}

/*  KBIntelliScan							*/
/*  scanForMethods							*/
/*		: Scan for methods 					*/
/*  selfName	: const QString & : Name of "self" variable		*/
/*  selfNode	: KBNode *	  : "Self" node if known		*/
/*  text	: const QString & : Text to scan			*/
/*  (returns)	: bool		  : Scan succeeded			*/

bool	KBIntelliScan::scanForMethods
	(	const QString		&selfName,
		KBNode			*selfNode,
		const QString		&text
	)
{
	DPRINTF
	((	"KBIntelliScan::scanForMethods: %s: %s\n",
		selfName.latin1(),
		text    .latin1()
	))	;

	m_methods.clear () ;
	m_result  = scanLine (selfName, selfNode, text) ;

	if (m_result.m_prefix.isNull())
		return	false	;

	XPRINTF
	((	"KBIntelliScan::scanForMethods: m_class=[%s] m_prefix=[%s] m_offset=%d m_complete=%d\n",
		m_result.m_class .latin1(),
		m_result.m_prefix.latin1(),
		m_result.m_offset,
		m_result.m_complete
	))	;

	QString	mode	;
	if (selfNode != 0)
	{
		if (selfNode->getRoot()->isForm  () != 0) mode = "F" ;
		if (selfNode->getRoot()->isReport() != 0) mode = "R" ;
	}

	m_methods = getMethods
		    (		m_result.m_class,
				QString::null,		// m_result.m_prefix,
				m_result.m_complete,
				mode
		    )	;

	return	m_methods.count() > 0 ;
}

/*  KBIntelliScan							*/
/*  getSelfName	: Set name of "self" variable				*/
/*  text	: const QString & : Script text				*/
/*  (returns)	: QString	  : Name or null if not known		*/

QString	KBIntelliScan::getSelfName
	(	const QString		&text
	)
{
	DPRINTF
	((	"KBIntelliScan::getSelfName: m_language=[%s]\n",
		m_language.latin1()
	))	;

	if (m_language == "kjs")
		return	"this"	;

	if (m_language == "py" )
	{
		QRegExp	re ("(event|slot)Func\\s*\\(\\s*([_a-zA-Z][_a-zA-Z0-9]*)") ;
		int	ar = re.match (text) ;
		return	ar >= 0 ? re.cap(2) : QString::null ;
	}

	return	QString::null	;
}

/*  KBIntelliScan							*/
/*  methods	: Get list of methods from most recent scan		*/
/*  (returns)	: const QPtrList<KBMethDictEntry>			*/
/*				: Method list				*/

const QPtrList<KBMethDictEntry> &KBIntelliScan::methods ()
{
	return	m_methods ;
}

/*  KBIntelliScan							*/
/*  klass	: Get class from most recent scan			*/
/*  (returns)	: QString	: Class, null if none			*/

const QString &KBIntelliScan::klass ()
{
	return	m_result.m_class ;
}

/*  KBIntelliScan							*/
/*  node	: Get node from most recent scan			*/
/*  (returns)	: KBNode *	: Node, null if none			*/

KBNode	*KBIntelliScan::node ()
{
	return	m_result.m_node ;
}

/*  KBIntelliScan							*/
/*  prefix	: Get prefix from most recent scan			*/
/*  (returns)	: QString	: Prefix, null if none			*/

const QString &KBIntelliScan::prefix ()
{
	return	m_result.m_prefix ;
}

/*  KBIntelliScan							*/
/*  offset	: Get offset of prefix from most recent scan		*/
/*  (returns)	: int		: Offset				*/

int	KBIntelliScan::offset ()
{
	return	m_result.m_offset ;
}

/*  KBIntelliScan							*/

KBIntelliScan *KBIntelliScan::getScanner
	(	const QString	&language
	)
{
	static	QDict<KBIntelliScan> *scanners ;

	if (scanners == 0)
		scanners = new QDict<KBIntelliScan> () ;

	
	KBIntelliScan *scanner = scanners->find (language) ;
	if (scanner == 0)
	{
		scanner = new KBIntelliScan (language) ;
		scanners->insert   (language, scanner) ;
	}

	return	scanner	;
}

/*  ------------------------------------------------------------------	*/

class	KBIntelliItem : public QListBoxItem
{
	QSimpleRichText		m_rt		;
	QRect			m_null		;

public	:

	KBMethDictEntry		*m_method	;

	virtual	int		width	(const QListBox *) const ;
	virtual	int		height	(const QListBox *) const ;
	virtual	void		paint	(QPainter *)	   ;

	KBIntelliItem			(RKListBox*, KBMethDictEntry *) ;

}	;

KBIntelliItem::KBIntelliItem
	(	RKListBox	*parent,
		KBMethDictEntry	*method
	)
	:
	QListBoxItem	(parent),
	m_rt		("<nobr>" + method->prototype() + "</nobr>", parent->font()),
	m_method	(method)
{
	m_rt.adjustSize	() ;
}

int	KBIntelliItem::width
	(	const QListBox		*
	)
	const
{
	return	m_rt.widthUsed () ;
}

int	KBIntelliItem::height
	(	const QListBox		*
	)
	const
{
	return	m_rt.height () ;
}

void	KBIntelliItem::paint
	(	QPainter	*p
	)
{
	m_rt.draw (p, 0, 0, m_null, listBox()->colorGroup(), &listBox()->backgroundBrush()) ;
}

/*  ------------------------------------------------------------------	*/

static	KBIntelli	*currentIntelli	;

/*  KBIntelli								*/
/*  KBIntelli	: Constructor for syntax helper widget			*/
/*  editor	: KBTextEdit *	  : Editor widget			*/
/*  legend	: const QString & : Optional legend			*/
/*  prefix	: const QString & : Method prefix			*/
/*  (returns)	: QWidget *	  : Helper widget			*/

KBIntelli::KBIntelli
	(	KBTextEdit			*editor,
		const QString			&legend,
		const QPtrList<KBMethDictEntry>	&methods,
		const QString			&prefix
	)
	:
	RKVBox
	(	editor,
		"kbintelli",
		WType_TopLevel|WStyle_Customize|WStyle_NoBorder|WStyle_StaysOnTop
	),
	m_editor	(editor),
	m_methods	(methods),
	m_prefix	(prefix)
{
	int		maxWid	= 0 ;
	QFontMetrics	fm	(font()) ;

	DPRINTF
	((	"KBIntelli::KBIntelli: showing %d\n",
		m_methods.count()
	))	;

	setFrameStyle	(QFrame::Box|QFrame::Plain) ;
	setLineWidth	(2) ;
	setMargin	(2) ;

	if (!legend.isEmpty())
	{
	        QPalette pal = QApplication::palette  () ;
	        pal.setColor (QColorGroup::Foreground,  white	 ) ;
	        pal.setColor (QColorGroup::Background,  darkGreen) ;
		QLabel  *l   = new QLabel (legend, this) ;
		l->setPalette (pal) ;
		l->setMargin  (2  ) ;
	}

	m_listBox 	= new RKListBox    (this) ;
	m_description	= new QTextBrowser (this) ;
	m_displayed	= 0 	;
	m_locked	= 0	;

	m_description->setTextFormat 	(QTextBrowser::RichText) ;
	m_description->setPalette    	(QToolTip::palette())    ;
	m_listBox    ->setMinimumSize	(300, 200) ;


#ifdef	Q_OS_LINUX
	QFont font = QToolTip::font	() ;
	font.setPointSize (font.pointSize() - 2) ;
	m_listBox    ->setFont		(font) ;
	m_description->setFont		(font) ;
#endif
	connect
	(	m_listBox,
		SIGNAL	(doubleClicked(	QListBoxItem *)),
		SLOT	(slotChosen   	(QListBoxItem *))
	)	;
	connect
	(	m_listBox,
		SIGNAL	(returnPressed	(QListBoxItem *)),
		SLOT	(slotChosen   	(QListBoxItem *))
	)	;
	connect
	(	m_listBox,
		SIGNAL	(highlighted  	(QListBoxItem *)),
		SLOT	(slotHighlighted(QListBoxItem *))
	)	;

	LITER
	(	KBMethDictEntry,
		methods,
		entry,

		KBIntelliItem *item = new KBIntelliItem (m_listBox, entry) ;
		if (item->width(m_listBox) > maxWid)
			maxWid = item->width(m_listBox) ;
	)

	qApp	 	->installEventFilter (this) ;
	m_listBox	->installEventFilter (this) ;
	m_description	->installEventFilter (this) ;

	connect
	(	&m_timer,
		SIGNAL	(timeout    ()),
		SLOT	(slotTimeout())
	)	;

	setMinimumWidth (maxWid + 16)	;
	setCaption	("Rekall")	;
	polish	   	()		;


	QPoint	absPos	= m_editor->textCursorPoint() ;
	QSize	size	= sizeHint	() ;
	int	x	= absPos.x	() ;
	int	y	= absPos.y	() + 10 ;
	int	w	= size  .width 	() ;
	int	h	= size  .height	() ;

        int     lheight = m_editor->lineHeight () ;

	QWidget	*dt	= QApplication::desktop() ;
	int	sx	= dt->x     () ;
	int	sy	= dt->y     () ;
	int	sw	= dt->width () ;
	int	sh	= dt->height() ;

        if (x + w > sw) x  = sw - w ;
        if (y + h > sh) y  = y - lheight - 2 * 6 - h ;

        if (x     < sx) x = sx ;
        if (y     < sy) y = sy ;

	DPRINTF
	((	"KBIntelli::KBIntelli: sizeHint=%s at (%d,%d)\n",
		_TEXT(sizeHint()),
		x,
		y
	))	;

	m_ctrlDown	= false	;

	loadMethods	()	;
	move		(x, y)	;
	show		()	;

	currentIntelli	= this	;
}

KBIntelli::~KBIntelli ()
{
	currentIntelli	= 0	;
}

void	KBIntelli::loadMethods ()
{
	int	plen	= m_prefix.length() ;

	m_listBox    ->blockSignals   (true) ;
	m_listBox    ->clear 	      ()     ;
	m_description->clear () ;

	XPRINTF
	((	"KBIntelli::loadMethods: prefix=[%s]\n",
		m_prefix.latin1()
	))	;

	LITER
	(	KBMethDictEntry,
		m_methods,
		entry,

		if (entry->name().left(plen) == m_prefix)
			new KBIntelliItem (m_listBox, entry) ;
	)

	m_listBox    ->setCurrentItem (-1)    ;
	m_listBox    ->blockSignals   (false) ;

	m_locked = 1	;
}

void	KBIntelli::setPrefix
	(	const QString	&prefix
	)
{
	if (m_prefix != prefix)
	{
		m_prefix = prefix ;
		loadMethods ()	  ;
	}
}

/*  KBIntelli								*/
/*  showItem	: Show description for specified item			*/
/*  lbi		: QListBoxItem * : Item in question			*/
/*  (returns)	: void		 :					*/

void	KBIntelli::showItem
	(	QListBoxItem	*lbi
	)
{
	KBIntelliItem *item = (KBIntelliItem *)lbi ;
	if (item == 0) 
	{
		if (m_displayed != 0)
		{	m_description->clear () ;
			m_displayed =  0    ;
		}
		return	;
	}
	if (item != m_displayed)
	{
		m_description->setText (item->m_method->description()) ;
		m_displayed = item ;
	}
}

/*  KBIntelli								*/
/*  eventFilter	: Event filter for things we need to Know About(tm)	*/
/*  o		: QObject *	: Target object				*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBIntelli::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (!o->isWidgetType()) 
		return	RKVBox::eventFilter (o, e) ;

	if ((o == m_listBox) || (o == m_description))
	{
		switch (e->type())
		{
			case QEvent::KeyPress	:
			case QEvent::KeyRelease	:
				switch (((QKeyEvent *)e)->key())
				{
					case Qt::Key_Up		:
					case Qt::Key_Down	:
					case Qt::Key_Prior	:
					case Qt::Key_Next	:
					case Qt::Key_Enter	:
					case Qt::Key_Return	:
						return	RKVBox::eventFilter (o, e) ;

					default	:
						break	;
				}

//				DPRINTF
//				((	"KBIntelli::event: rerouting key event\n"
//				))	;
				QApplication::sendEvent (m_editor->textEdit(), e) ;
				return	true	;

			case QEvent::MouseMove	:
			{
				if (m_locked <= 1)
					showItem (m_listBox->itemAt(((QMouseEvent *)e)->pos())) ;
				return	RKVBox::eventFilter (o, e) ;
			}

			case QEvent::MouseButtonDblClick :
				m_ctrlDown = (((QMouseEvent *)e)->state() && Qt::ControlButton) != 0 ;
				return	RKVBox::eventFilter (o, e) ;

			default	:
				break	;
		}

		return	RKVBox::eventFilter (o, e) ;
	}


	/* The kill flag will be set if we decide that we need to	*/
	/* undertake voluntary euthenasia ...				*/
	bool	kill	= false	;
	QWidget	*w	= 0	;

	switch (e->type())
	{
		case QEvent::FocusIn	:
			/* Focus-out kills us unless the focus is	*/
			/* moving into us or into the parent editor.	*/
			kill	= true ;
			for (w = (QWidget *)o ; w != 0 ; w = w->parentWidget())
				if ((w == this) || (w == m_editor))
				{	kill	= false	;
					break	;
				}
			break	;

		case QEvent::WindowDeactivate	:
			/* If deactivating either this widget or the	*/
			/* editor then we should die, unless the other	*/
			/* one is about to be activated ... so start a	*/
			/* timer.					*/
			if ((o == this->topLevelWidget()) || (o == m_editor->topLevelWidget()))
			{
				DPRINTF
				((	"KBIntelli::eventFilter: start timer\n"
				))	;
				m_timer.start (100, false) ;
			}
			break	;

		case QEvent::WindowActivate	:
			/* If activating us or the editor then cancel	*/
			/* any deactivation timer.			*/
			if ((o == this->topLevelWidget()) || (o == m_editor->topLevelWidget()))
			{
				DPRINTF
				((	"KBIntelli::eventFilter: stop  timer\n"
				))	;
				m_timer.stop  () ;
			}
			break	;

		default	:
			break	;
	}

	if (kill)
		deleteLater ()	;

	return	RKVBox::eventFilter (o, e) ;
}

/*  KBIntelli								*/
/*  slotTimeout	: Deactivation timer					*/
/*  (returns)	: void		:					*/

void	KBIntelli::slotTimeout ()
{
	deleteLater ()	;
}

/*  KBIntelli								*/
/*  slotChosen	: User chooses an entry					*/
/*  item	: QListBoxItem * : Entry chosen				*/
/*  (returns)	: void		 :					*/

void	KBIntelli::slotChosen
	(	QListBoxItem	*item
	)
{
	if (item != 0)
		emit sigChosen (((KBIntelliItem *)item)->m_method, m_ctrlDown) ;
}

/*  KBIntelli								*/
/*  slotHighlighted							*/
/*		: User highlights item					*/
/*  lbi		: QListBoxItem * : Item in question			*/
/*  (returns)	: void		 :					*/

void	KBIntelli::slotHighlighted
	(	QListBoxItem	*lbi
	)
{
	/* Explicitely clicking an item locks the description for that	*/
	/* item; but we seem to get an initial call so count and lock	*/
	/* (see the ::eventFilter() method) when the count goes over	*/
	/* unity.							*/
	m_locked += 1  ;
	showItem (lbi) ;
}


KBIntelli
	*KBIntelli::create
	(	KBTextEdit			*editor,
		const QString			&legend,
		const QPtrList<KBMethDictEntry>	&methods,
		const QString			&prefix
	)
{
	KBIntelli::destroy () ;
	return	new KBIntelli (editor, legend, methods, prefix) ;
}

void	KBIntelli::destroy ()
{
	if (currentIntelli != 0)
	{
		delete	currentIntelli	;
		currentIntelli = 0	;
	}
}
