/***************************************************************************
    file	         : kb_intelli.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#ifndef		__KB_INTELLI_H
#define		__KB_INTELLI_H

#include	<qtimer.h>
#include	<qregexp.h>
#include	<qvaluelist.h>

#include	"kb_methdict.h"
#include	"rk_vbox.h"

#include	"kb_methdict.h"

#include	"libkbase_exports.h"

class		QTextBrowser	;
class		KBTextEdit	;
class		RKListBox	;
class		QListBoxItem	;
class		KBIntelliItem	;
class		KBNode		;


/*  KBIntelliScanResult							*/
/*  -------------------							*/
/*  Class used to aggregate results from a scan. Various fields may or	*/
/*  may not be set depending on what the scanner could deduce.		*/

class	KBIntelliScanResult
{
public	:

	KBNode		*m_node		;	/* Target node		*/
	QString		m_class		;	/* Target class		*/
	QString		m_prefix	;	/* Method prefix	*/
	int		m_offset	;	/* Offset into line	*/
	bool		m_complete	;	/* Method name complete	*/

	KBIntelliScanResult	()	;
}	;


/*  KBIntelliScanMappingItem						*/
/*  ------------------------						*/
/*  Classed used to store a pattern->class mapping, used when the	*/
/*  scanner cannot deduce a node from the code itself, and falls back	*/
/*  on prefix-based identification.					*/

class	KBIntelliScanMappingItem
{
	QRegExp		m_regexp	;
	QString		m_class		;

public	:

	QString		match		(const QString &) const ;

	KBIntelliScanMappingItem	()		;
	KBIntelliScanMappingItem	(const QString &, const QString &) ;
}	;


/*  KBIntelliScan							*/
/*  -------------							*/
/*  This is the main scanner class. . Separate instances will be	*/
/*  created for each language, wrapping the method dictionary class.	*/

class	LIBKBASE_API	KBIntelliScan : public KBMethDict
{
	QString					m_language	;
	QValueList<KBIntelliScanMappingItem>	m_mappings	;

	KBIntelliScanResult			m_result	;
	QPtrList  <KBMethDictEntry>		m_methods	;

	bool			wordChar	(const QChar   &)	;
	QStringList		tokenize	(const QString &, int)	;

	KBIntelliScanResult	scanTokens
				(	const QString			&,
					KBNode				*,
					const QString			&,
					const QStringList		&
				)	;
	KBIntelliScanResult	scanLine
				(	const QString			&,
					KBNode				*,
					const QString			
				)	;
	KBIntelliScan		(const QString &)	;

public	:

	const QPtrList<KBMethDictEntry>	&methods	() ;
	KBNode				*node		() ;
	const QString			&klass		() ;
	const QString			&prefix		() ;
	int				offset		() ;

	bool			scanForMethods
				(	const QString			&,
					KBNode				*,
					const QString			&
				)	;
	QString			getSelfName	(const QString &) ;

	static	KBIntelliScan	*getScanner	(const QString &) ;

	friend	class	KBTestIntelliScan	;
}	;


/*  KBIntelli								*/
/*  ---------								*/
/*  This class is used to display IntelliSense(tm)-like popups.		*/

class	LIBKBASE_API	KBIntelli : public RKVBox
{
	Q_OBJECT

	KBTextEdit			*m_editor	;
	QPtrList<KBMethDictEntry>	m_methods	;
	QString				m_prefix	;
	RKListBox			*m_listBox	;
	QTextBrowser			*m_description	;
	QTimer				m_timer		;
	KBIntelliItem			*m_displayed	;
	int				m_locked	;
	bool				m_ctrlDown	;

	void		loadMethods	()			;
	void		showItem	(QListBoxItem	 *)	;

signals	:

	void		sigChosen	(KBMethDictEntry *, bool) ;

protected slots :

	void		slotTimeout	()			;
	void		slotChosen	(QListBoxItem    *)	;
	void		slotHighlighted	(QListBoxItem    *)	;


private	:

	KBIntelli
	(	KBTextEdit			*,
		const QString			&,
		const QPtrList<KBMethDictEntry>	&,
		const QString			&
	)	;

public	:

	~KBIntelli				()	;

	virtual	bool		eventFilter	(QObject *, QEvent *) ;
	void			setPrefix	(const QString &    ) ;

	static	KBIntelli	*create	
				(	KBTextEdit			*,
					const QString			&,
					const QPtrList<KBMethDictEntry>	&,
					const QString			&
				)	;
	static	void		destroy	() ;
}	;

#endif	// __KB_INTELLI_H
