/***************************************************************************
    file	         : kb_item.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qapplication.h>
#include	<qregexp.h>

#if		__KB_AUTOMOC
#include	"kb_item.moc"
#else
#include	"kb_item.h"
#endif

#include	"kb_ctrl.h"
#include	"kb_formblock.h"
#include	"kb_qrybase.h"
#include	"kb_nodemonitor.h"
#include	"kb_writer.h"
#include	"kb_layout.h"
#include	"kb_display.h"
#include	"kb_sizer.h"
#include	"kb_script.h"
#include	"kb_ascii.h"
#include	"kb_attrimage.h"
#include	"kb_font.h"
#include	"kb_popupmenu.h"
#include	"kb_prompt.h"

#if		! __KB_RUNTIME
#include	"kb_recorder.h"
#endif

#define		__DEBUG_ITEM	(0)
#if		__DEBUG_ITEM
#include	"kb_dprintf.h"
#endif

/*  KBItem								*/
/*  KBItem	: Constructor for form item class			*/
/*  node	: KBNode *	: Parent node				*/
/*  name	: cchar *	: Name of actual node			*/
/*  ename	: cchar *	: Name of expression attribute		*/
/*  aList	: const QDict<QString> &				*/
/*				: Attribute list			*/
/*  (returns)	: KBItem	:					*/

KBItem::KBItem
	(	KBNode			*node,
		cchar 			*name,
		cchar			*ename,
		const QDict<QString>	&aList
	)
	:
	KBObject	(node, name,       	aList),
	m_expr		(this, ename,      	aList),
	m_rdonly	(this, "rdonly",   	aList, KAF_FORM),
	m_noupdate	(this, "noupdate",   	aList, KAF_FORM),
	m_tabOrd	(this, "taborder", 	aList, KAF_FORM),
	m_transfer  	(this, "transfer",	aList,          KAF_GRPWEB ),
	m_valFlag	(this, "valflag", 	aList, KAF_FORM|KAF_GRPDATA),
	m_defVal	(this, "default",  	aList),
	m_errtext	(this, "errtext",	aList),
	m_markbgcolor	(this, "markbgcolor",  	aList),
	m_markfgcolor	(this, "markfgcolor",  	aList),
	m_onEnter	(this, "onenter",	aList, KAF_FORM|KAF_EVCS),
	m_onLeave	(this, "onleave",	aList, KAF_FORM|KAF_EVCS),
	m_onSet		(this, "onset",		aList),
	m_onDblClick	(this, "ondblclick",	aList)
{
	m_ctrlMon	= 0	;
	m_table		= 0	;
	m_flags		= 0	;
	m_allEnabled	= true	;
	m_allVisible	= true	;
	m_isUpdateVal	= -1	;
	m_type		= 0	;
	m_markedPalette	= 0	;
}

/*  KBItem								*/
/*  KBItem	: Constructor for form item class			*/
/*  parent	: KBObject *	: Parent object				*/
/*  name	: cchar *	: Name of actual node			*/
/*  rect	: const QRect &	: Size and position			*/
/*  ename	: cchar *	: Name of expression attribute		*/
/*  value	: cchar *	: Value for exprssion attribute		*/
/*  tabOrd	: uint		: Tab order				*/
/*  (returns)	: KBItem	:					*/

KBItem::KBItem
	(	KBObject	*parent,
		cchar 		*name,
		const QRect &	rect,
		cchar 		*ename,
		cchar		*value,
		uint		tabOrd
	)
	:
	KBObject	(parent, name, rect),
	m_expr		(this,   ename,       	value),
	m_rdonly	(this,   "rdonly",    	"",		KAF_FORM),
	m_noupdate 	(this,   "noupdate",  	"",		KAF_FORM),
	m_tabOrd	(this,	 "taborder",  	tabOrd + 1,	KAF_FORM),
	m_transfer  	(this,   "transfer",	(uint)0, 	         KAF_GRPWEB ),
	m_valFlag	(this,   "valflag", 	"",		KAF_FORM|KAF_GRPDATA),
	m_defVal	(this,   "default",   	""),
	m_errtext	(this,   "errtext",	""),
	m_markbgcolor	(this,   "markbgcolor",	""),
	m_markfgcolor	(this,   "markfgcolor",	""),
	m_onEnter	(this,   "onenter",	"",		KAF_FORM|KAF_EVCS),
	m_onLeave	(this,   "onleave",	"",		KAF_FORM|KAF_EVCS),
	m_onSet		(this,   "onset", 	""),
	m_onDblClick	(this,   "ondblclick",	"")
{
	m_ctrlMon	= 0	;
	m_table		= 0	;
	m_flags		= 0	;
	m_allEnabled	= true	;
	m_allVisible	= true	;
	m_isUpdateVal	= -1	;
	m_type		= 0	;
	m_markedPalette	= 0	;
}

/*  KBItem								*/
/*  KBItem	: Constructor for form item class			*/
/*  parent	: KBNode *	: Parent node				*/
/*  ename	: cchar *	: Name of expression attribute		*/
/*  item	: KBItem *	: Extant item				*/
/*  (returns)	: KBItem	:					*/

KBItem::KBItem
	(	KBNode		*parent,
		cchar		*ename,
		KBItem		*item
	)
	:
	KBObject	(parent, item),
	m_expr		(this,	ename,     	item),
	m_rdonly	(this, "rdonly",   	item, KAF_FORM),
	m_noupdate	(this, "noupdate",   	item, KAF_FORM),
	m_tabOrd	(this, "taborder", 	item, KAF_FORM),
	m_transfer  	(this, "transfer",	item,          KAF_GRPWEB ),
	m_valFlag	(this, "valflag", 	item, KAF_FORM|KAF_GRPDATA),
	m_defVal	(this, "default",  	item),
	m_errtext	(this, "errtext",	item),
	m_markbgcolor	(this, "markbgcolor",	item),
	m_markfgcolor	(this, "markfgcolor",	item),
	m_onEnter	(this, "onenter", 	item, KAF_FORM|KAF_EVCS),
	m_onLeave	(this, "onleave", 	item, KAF_FORM|KAF_EVCS),
	m_onSet		(this, "onset", 	item),
	m_onDblClick	(this, "ondblclick",	item)
{
	m_ctrlMon	= 0	;
	m_table		= 0	;
	m_flags		= 0	;
	m_allEnabled	= true	;
	m_allVisible	= true	;
	m_isUpdateVal	= -1	;
	m_type		= 0	;
	m_markedPalette	= 0	;
}

/*  KBItem								*/
/*  ~KBItem	: Destructor for form item class			*/
/*  (returns)	:		:					*/

KBItem::~KBItem ()
{
	DPRINTF
	((	"KBItem::~KBItem: %s.%s: c[0]=%p\n",
		getElement().latin1(),
		getName   ().latin1(),
		m_ctrls.count() == 0 ? (void *)0 : (void *)m_ctrls.at(0)
	))	;

	for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
		delete	m_ctrls.at(idx) ;

	setControl	(0) ;
	if (m_type != 0) m_type->deref() ;

	DELOBJ	(m_markedPalette) ;
}

/*  KBItem								*/
/*  moveFocusOK	: Check whether focus can be set			*/
/*  drow	: uint		: Display row				*/
/*  (returns)	: bool		: True if focus can be set		*/

bool	KBItem::moveFocusOK
	(	uint	drow
	)
{
	DPRINTF
	((	"KBItem::moveFocusOK: %s: row=%d\n",
		(cchar *)getName(),
		drow
	))	;

	/* Unless in design mode, or there is no parent block (and	*/
	/* there should be!), check with the block whether focus is	*/
	/* allowed into this item (really, out of a current item).	*/
	return	showingDesign()	     ? true :
		getFormBlock () == 0 ? true :
				     getFormBlock()->moveFocusOK (this, drow) ;
}

/*  KBItem								*/
/*  focusInEvent: Handle focus in					*/
/*  drow	: uint		: Display row				*/
/*  reason	: Reason	: Reason focus arrives			*/
/*  (returns)	: void		:					*/

void	KBItem::focusInEvent
	(	uint			drow,
		QFocusEvent::Reason	reason
	)
{
	DPRINTF
	((	"KBItem::focusInEvent: %s:row=%d r=%d\n",
		(cchar *)getName(),
		drow,
		reason
	))	;

#if	! __KB_RUNTIME
	if (reason == QFocusEvent::Mouse)
	{
		KBRecorder *recorder = KBRecorder::self() ;
		if ((recorder != 0) && recorder->isRecording(getDocRoot()))
			recorder->mouseNavigation
			(	this,
				getBlock()->getCurQRow() - getBlock()->getCurDRow()
			)	;
	}
#endif

	if (getFormBlock() != 0)
		getFormBlock()->focusInEvent (this, drow, reason) ;
}

/*  KBItem								*/
/*  doEnter	: Handle focus entry					*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: void		:					*/

void	KBItem::doEnter
	(	uint	qrow
	)
{
	DPRINTF
	((	"KBItem::doEnter: %s: qrow=%d\n",
		(cchar *)getName(),
		qrow
	))	;

	KBValue	arg	= (int)qrow ;
	bool	evRc	;
	eventHook (m_onEnter, 1, &arg, evRc) ;
}

/*  KBItem								*/
/*  doLeave	: Handle focus exit					*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: OK to leave				*/

bool	KBItem::doLeave
	(	uint	qrow
	)
{
	DPRINTF
	((	"KBItem::doLeave: %s: %d\n",
		(cchar *)getName(),
		qrow
	))	;

	KBValue	arg	= (int)qrow ;
	bool	evRc	;
	bool	rc	= eventHook (m_onLeave, 1, &arg, evRc) ? evRc : false ;

	DPRINTF
	((	"KBItem::doLeave [%s][%s] -> %d\n",
		(cchar *)getElement(),
		(cchar *)getName(),
		rc
	))	;


	/* If execution OK then return the exection return code. On	*/
	/* error return false to prevent moving focus (and thence maybe	*/
	/* getting further errors).					*/
	return	rc ? evRc : false ;
}

/*  KBItem								*/
/*  doDblClick	: Handle double click					*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBItem::doDblClick
	(	uint	qrow
	)
{
	KBValue	arg	= (int)qrow ;
	bool	evRc	;
	bool	rc	= eventHook (m_onDblClick, 1, &arg, evRc, false) ? evRc : false ;

	DPRINTF
	((	"KBItem::doDblClick [%s][%s] -> rc=%d\n",
		(cchar *)getElement(),
		(cchar *)getName(),
		rc
	))	;

	return	rc ;
}

/*  KBItem								*/
/*  setupControls							*/
/*		: Setup controls					*/
/*  (returns)	: void		:					*/

void	KBItem::setupControls ()
{
	/* We may get called before there is a display, in consequence	*/
	/* of a layout change higher in the tree ...			*/
	if (getDisplay() == 0)
		return	;

	KBBlock	*block	= getBlock()		  ;
	uint	newRows	= block->getDisplayRows() ;
	uint	oldRows	= m_ctrls.count()	  ;

	DPRINTF
	((	"KBItem::setupControls: %s: %s: mode=%d: %d->%d\n",
		getElement().latin1(),
		getName   ().latin1(),
		isShowing (),
		oldRows,
		newRows
	))	;

	/* If the number of controls is the same or reduced, then dump	*/
	/* dump those that are no longer needed. See the comment in	*/
	/* the destructor re deleting extant controls.			*/
	if (newRows <= oldRows)
	{
		for (uint idx = newRows ; idx < oldRows ; idx += 1)
			delete	m_ctrls.at(idx) ;

		m_ctrls.resize (newRows) ;
		if (newRows == 0) setControl (0) ;
		return	;
	}

	/* Increasing the number of controls. Extend the vector, and	*/
	/* allocate as many more controls as are needed. These are	*/
	/* stored in the controls array, set up and positioned.		*/
	m_ctrls.resize (newRows) ;
	for (uint idx2 = oldRows ; idx2 < newRows ; idx2 += 1)
		m_ctrls.at(idx2) = 0 ;

	for (uint idx3 = oldRows ; idx3 < newRows ; idx3 += 1)
	{
		KBControl *ctrl   = makeCtrl (idx3) ;

		ctrl->setCtrlGeometry
		(	QRect(m_geom.x(), m_geom.y(), m_geom.w(), m_geom.h()),
			m_geom.alignment()
		)	;
		ctrl->showAs		(isShowing ()) ;
		ctrl->setupProperties	()	;
		ctrl->setVisible	(showingDesign() ? true : m_allVisible && ! showHidden  ()) ;
		ctrl->setEnabled	(showingDesign() ? true : m_allEnabled && ! showDisabled()) ;

		if (m_ctrlMon != 0) ctrl->showMonitor (m_ctrlMon) ;

		m_ctrls.at(idx3) = ctrl	;
	}

	/* If in design mode then update the geometry and properties on	*/
	/* any extant controls.						*/
	if (showingDesign())
		for (uint idx4 = 0 ; idx4 < oldRows ; idx4 += 1)
		{
			KBControl *ctrl	= m_ctrls.at(idx4) ;
	
			ctrl->setCtrlGeometry
			(	QRect(m_geom.x(), m_geom.y(), m_geom.w(), m_geom.h()),
				m_geom.alignment()
			)	;
			ctrl->setupProperties	() ;
		}

	/* Pass the zeroth control down to the object base class. If	*/
	/* control gets here there _must_ be at least one.		*/
	setControl (m_ctrls.at(0)) ;
}

/*  KBItem								*/
/*  ctrlGone	: Note death of control					*/
/*  ctrl	: KBControl *	: The control				*/
/*  (returns)	: void		:					*/

void	KBItem::ctrlGone
	(	KBControl	*ctrl
	)
{
	for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
		if (m_ctrls.at(idx) == ctrl)
		{	m_ctrls.at(idx) = 0 ;
			break ;
		}

	KBObject::ctrlGone (ctrl) ;
}

void	KBItem::setCtrlGeometry
	(	const QRect	&rect
	)
{
	DPRINTF
	((	"KBItem::setCtrlGeometry: %s\n",
		_TEXT(rect)
	))	;


	/* If we have a parent block then the change may mean that the	*/
	/* number of data controls needs to be changed.			*/
	KBBlock	*block	= getBlock() ;
	if (block != 0)
		for (uint idx1 = 0 ; idx1 < m_ctrls.count() ; idx1 += 1)
			if (m_ctrls.at(idx1) != 0)
				m_ctrls.at(idx1)->setCtrlGeometry
				(	rect,
					m_geom.alignment()
				)	;
}

/*  KBItem								*/
/*  setGeometry	: Set new geometry					*/
/*  rect	: const QRect &	: Position				*/
/*  (returns)	: void		:					*/

void	KBItem::setGeometry
	(	const QRect	&rect
	)
{
	DPRINTF
	((	"KBItem::setGeometry: %s: %s\n",
		getName().latin1(),
		_TEXT(rect)
	))	;

	/* The geometry will be invalid if pasting into a dynamic	*/
	/* container. Just redo the controls.				*/
	if (rect.isValid ())
	{
		KBObject::setGeometry (rect) ;
	}

	if (getDisplay() != 0)
	{
		getParent()->isObject()->recalcNumRows(getDisplay()->getTopSize()) ;
		getBlock ()->redoControls() ;
	}
}

/*  KBItem								*/
/*  setPalette	: Set palette						*/
/*  (returns)	: void		:					*/

void	KBItem::setPalette ()
{
	KBObject::setPalette () ;

	QPalette  *pal 	= getPalette (false) ;
	for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
		m_ctrls.at(idx)->setPalette (pal) ;
}

/*  KBItem								*/
/*  setFont	: Set font						*/
/*  (returns)	: void		:					*/

void	KBItem::setFont ()
{
	KBObject::setFont () ;

	QFont	*font 	= getFont (false) ;
	for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
		m_ctrls.at(idx)->setFont (font) ;
}

void	KBItem::setCtrlReadOnly
	(	uint		qrow,
		bool		readOnly,
		const QColor	&fgColor,
		const QColor	&bgColor
	)
{
	KBControl *ctrl = ctrlAtQRow  (qrow) ;
	if (ctrl != 0)
		ctrl->setReadOnly (readOnly, fgColor, bgColor) ;
}

bool	KBItem::isCtrlReadOnly
	(	uint		qrow
	)
{
	KBControl *ctrl = ctrlAtQRow  (qrow) ;
	return	ctrl == 0 ? false : ctrl->isReadOnly() ;
}

/*  KBItem								*/
/*  getVTrans	: Get value translation mode				*/
/*  (returns)	: KBValue::VTrans : Translation mode			*/

KBValue::VTrans	KBItem::getVTrans ()
{
	/* Default is text, since this is true for almost all items,	*/
	/* the obvious exception being pixmaps.				*/
	return	KBValue::VText	;
}

/*  KBItem								*/
/*  makeCtrl	: Make specific control					*/
/*  drow	: uint		: Display row number			*/
/*  (returns)	: KBControl *	: New control				*/

KBControl
	*KBItem::makeCtrl
	(	uint
	)
{
	KBError::EFault
	(	QString	(TR("KBItem::makeCtrl called for \"%1::%2\""))
			.arg (getElement())
			.arg (getName   ()),
		QString::null,
		__ERRLOCN
	)	;
	return	0 ;
}

/*  KBItem								*/
/*  getExpr	: Get item expression and set query column index	*/
/*  (returns)	: QString	: Expression				*/

QString	KBItem::getExpr ()
{
	return	m_expr.getValue ()   ;
}


/*  KBItem								*/
/*  setFieldType: Set database field type for the item			*/
/*  _type	: KBType *	: Type					*/
/*  (returns)	: void		:					*/

void	KBItem::setFieldType
	(	KBType	*_type
	)
{
	if (m_type != 0) m_type->deref() ;
	m_type	= _type	;
	if (m_type != 0) m_type->ref  () ;
}

/*  KBItem								*/
/*  isUpdateVal	: Test if item is used when updating database		*/
/*  toVerify	: bool		: True if checking for verification	*/
/*  (returns)	: bool		: TRUE if so				*/

bool	KBItem::isUpdateVal
	(	bool		toVerify
	)
{
	if (m_isUpdateVal < 0)
	{
		/* NOTE : Qt2.2.1: Setting the case sensitive argument	*/
		/* false does not seem to work, hence [a-zA-Z]		*/

		/* FIXME: This should be handled in the drivers ...	*/
		static	QRegExp e1	("^\\s*[_a-zA-Z0-9]*[_a-zA-Z][_a-zA-Z0-9]*\\s*$", false) ;
		static	QRegExp e2	("^\\s*[_a-zA-Z0-9]*[_a-zA-Z][_a-zA-Z0-9]*\\s*\\.\\s*[_a-zA-Z0-9]*[_a-zA-Z][_a-zA-Z0-9]*\\s*$", false) ;
		const 	QString	&e	= m_expr.getValue () ;

		m_isUpdateVal = (e1.match (e) >= 0) || (e2.match (e) >= 0) ;

		DPRINTF
		((	"KBItem::isUpdateVal: [%s] update [%d]\n",
			(cchar *)e,
			m_isUpdateVal
		))	;

	}

	switch (m_noupdate.getFlags())
	{
		case ANO_NO	: return m_isUpdateVal	;
		case ANO_YES	: return false		;
		case ANO_VERIFY	: return toVerify	;
		default		: break			;
	}

	return	m_isUpdateVal	;
}

/*  KBItem								*/
/*  getValue	: Get value						*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBItem::getValue
	(	uint	qrow
	)
{
	/* Get the control for the specified row. If there is none then	*/
	/* return an untyped null, the result being undefined when the	*/
	/* row is out of range.						*/
	KBControl *ctrl = ctrlAtQRow  (qrow) ;

	DPRINTF
	((	"KBItem::getValue: %-12s: qr=%d cqr=%d cdr=%d ctrl=[%p]\n",
		m_name.getValue().latin1(),
		qrow,
		getBlock()->getCurQRow(),
		getBlock()->getCurDRow(),
		(void  *)ctrl
	))	;

	/* Get the current value from the control. If is is empty and	*/
	/* there is a default value then process that to get the value.	*/
	/* Special case is if we are in a report and this is the	*/
	/* current query row, in which case use the m_curVal value.	*/
	KBValue	value	;

	if	((getRoot()->isReport() != 0) && (qrow == getBlock()->getCurQRow()))
	{
		value	= m_curVal ;
	}
	else if (ctrl != 0)
	{
		value   = ctrl->getValue ()  ;
	}
	else	return KBValue () ;


	if (value.isEmpty())
		if (!m_defVal.getValue().isEmpty())
		{
			KBScriptError	*eRC	;
			bool		_fSubs	;

			value	= m_defVal.evaluate
					  (	KBValue(m_defVal.getValue()),
						eRC,
						_fSubs
					  )	;

			if (eRC != 0)
			{
				KBScriptError::processError (eRC, KBScriptError::Normal) ;
				return	KBValue() ;
			}

			return	KBValue (value.getRawText(), m_type) ;
		}

	return	value	;
}

/*  KBItem								*/
/*  getRowValue	: Get value for row from query				*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBItem::getRowValue
	(	uint	qrow
	)
{
	return	getBlock()->getQuery()->getField
				(	m_qryIdx.qryLvl(),
					qrow,
					m_qryIdx.qryIdx()
				)	;
}

/*  KBItem								*/
/*  getOrderType: Get ordering type					*/
/*  (returns)	: KB::IType	: Type					*/

KB::IType KBItem::getOrderType()
{
	/* Return the ordering type for the item given a raw value.	*/
	/* The default is the value itself, but the method may be	*/
	/* overridden, for instance by a link control.			*/
	return	m_type == 0 ? KB::ITUnknown : m_type->getIType() ;
}

/*  KBItem								*/
/*  getOrderValue							*/
/*		: Get ordering value					*/
/*  value	: const KBValue & : Value				*/
/*  (returns)	: QString	  : Ordering value			*/

QString	KBItem::getOrderValue
	(	const KBValue	&value
	)
{
	/* Return the ordering value for the item given a raw value.	*/
	/* The default is the value itself, but the method may be	*/
	/* overridden, for instance by a link control.			*/
	return	value.getRawText() ;
}

/*  KBItem								*/
/*  setValue	: Set value display row					*/
/*  qrow	: uint		 : Query row				*/
/*  value	: const KBValue &: Value to set				*/
/*  (returns)	: bool		 : Success				*/

bool	KBItem::setValue
	(	uint		qrow,
		const KBValue	&value
	)
{
	KBScriptError	*eRC	;
	KBControl	*ctrl	= ctrlAtQRow  (qrow)   ;

	m_curVal = m_expr.evaluate (value, eRC, m_fSubs) ;
	if (eRC != 0)
	{
		KBScriptError::processError (eRC, KBScriptError::Normal) ;
		return	false	;
	}

	DPRINTF
	((	"KBItem::setValue: %-12s: qr=%d cqr=%d cdr=%d ctrl=[%p] v=[%s] cv=[%s]\n",
		m_name.getValue().latin1(),
		qrow,
		getBlock()->getCurQRow(),
		getBlock()->getCurDRow(),
		(void  *)ctrl,
		value   .getRawText().latin1(),
		m_curVal.getRawText().latin1()
	))	;

	if (ctrl != 0)
		ctrl->setValue (m_curVal) ;

	/* Normally the type will be set from the query as soon as the	*/
	/* select query has been executed, but in case we set a value	*/
	/* before the query (eg., in the block "PreQuery" event) then	*/
	/* use the value type if the current type is unknown.		*/
	if (m_type->getIType() == KB::ITUnknown)
	{
		m_type->deref() ;
		m_type	= m_curVal.getType() ;
		m_type->ref  () ;
	}

	KBValue	args[2]	    ;
	bool	evRc	    ;

	args[0] = (int)qrow ;
	args[1] = m_curVal  ;

	return	eventHook (m_onSet, 2, args, evRc) ;
}

/*  KBItem								*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  qrow	: uint		: Starting row in query			*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBItem::clearValue
	(	uint	qrow,
		bool	query
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	if (ctrl != 0) ctrl->clearValue (query) ;
}

/*  KBItem								*/
/*  changed	: See if user has changed an item			*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Item changed				*/

bool	KBItem::changed
	(	uint	qrow
	)
{
	if (isUpdateVal(true))
	{
		KBControl *ctrl	 = ctrlAtQRow (qrow) ;
		bool	  delta	 = ctrl == 0 ? false : ctrl->changed () ;

//		if (delta)
//			DPRINTF
//			((	"KBItem::changed: %s: %d: changed=%d: [%s]->[%s]\n",
//				getName().latin1(),
//				qrow,
//				delta,
//				ctrl->getIniValue().getRawText().latin1(),
//				ctrl->getValue   ().getRawText().latin1()
//			))	;

		return	  delta  ;
	}

	return	false	;
}

/*  KBItem								*/
/*  isEmpty	: See if control is empty				*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: Empty					*/

bool	KBItem::isEmpty
	(	uint	qrow
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	return	ctrl == 0 ? true : ctrl->isEmpty () ;
}

/*  KBItem								*/
/*  checkValid	: Check whether value can be saved in database		*/
/*  value	: const QString & : Putative value			*/
/*  allowNull	: bool		  : Ignore not-null check		*/
/*  (returns)	: bool		  : True if OK				*/

bool	KBItem::checkValid
	(	const KBValue	&,
		bool		
	)
{
	/* Default is always true, override as needed.			*/
	return	true	;
}

/*  KBItem								*/
/*  isValid	: Test if value is valid to be saved			*/
/*  qrow	: uint		: Query row				*/
/*  allowNull	: bool		: Ignore not-null checks		*/
/*  (returns)	: bool		: Value can be saved			*/

bool	KBItem::isValid
	(	uint	qrow,
		bool	allowNull
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	if (ctrl == 0) return true	    ;

#if	0
	/* At this point check for an empty value along with a default	*/
	/* value or expression, in which case evaluate the default	*/
	/* expression: note that we pass the default itself as the	*/
	/* 'default' for the evaluation, so that writing a simple	*/
	/* default value has the obvious effect. On return check for	*/
	/* errors.							*/
	KBValue	  value	= ctrl->getValue () ;
	if (value.isEmpty() && !m_defVal.getValue().isEmpty())
	{
		KBScriptError	*eRC	;
		bool		_fSubs	;
		KBValue		value2	= m_defVal.evaluate
					  (	KBValue(m_defVal.getValue()),
						eRC,
						_fSubs
					  )	;

		if (eRC != 0)
		{
			KBScriptError::processError (eRC, KBScriptError::Normal) ;
			return	false	;
		}

		/* Got a default, so pass this down to the control,	*/
		/* whence it can be retrieved and checked ...		*/
		if (!setValue (qrow, KBValue(value2.getRawText(), m_type)))
			return	false	;
	}
#endif

	if (!ctrl->isValid (allowNull))
	{	setError (ctrl->lastError ()) ;
		return	 false ;
	}

	return	true ;
}

/*  KBItem								*/
/*  setupProperties							*/
/*		: Set current properties				*/
/*  (returns)	: void		:					*/

void	KBItem::setupProperties ()
{
	DELOBJ	(m_markedPalette) ;
	setupControls () ;
	KBObject::setupProperties () ;
}

/*  KBItem								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBItem::showAs
	(	KB::ShowAs	mode
	)
{
	m_allEnabled	= true	;
	m_allVisible	= true	;
	DELOBJ(m_markedPalette) ;

	if (mode != KB::ShowAsData) m_isUpdateVal = -1 ;

	/* Note: If the expression is empty then set the type to string	*/
	/* so that anything is acceptable. This is needed since without	*/
	/* an expression, the item will not be set from the database	*/
	/* and hence we would not get a type.				*/
	if (m_type != 0) m_type->deref() ;

	m_type = m_expr.getValue().isEmpty() ? &_kbString : &_kbUnknown ;
	m_type->ref  ()	   ;

	DPRINTF
	((	"KBItem::showAs: [%s] [0 ... %d]\n",
		(cchar *)getName(),
		m_ctrls.count()
	))	;

	if (isHidden() == 0)
	{
		for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
			m_ctrls.at(idx)->showAs	(mode) ;

		if (isMorphing() && (mode == KB::ShowAsData))
			getDisplay()->addMorph (this) ;
		else	getDisplay()->remMorph (this) ;
	}

	KBObject::showAs (mode) ;
}

/*  KBItem								*/
/*  hideBelow	: Hide controls from specified row downwards		*/
/*  qrow	: uint		: Specific row				*/
/*  (returns)	: void		:					*/

void	KBItem::hideBelow
	(	uint	qrow
	)
{
	bool	hide	= false ;

#if	! __KB_RUNTIME
	if (!showingData ())
	{
		for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
		{	m_ctrls.at(idx)->setVisible (true ) ;
			m_ctrls.at(idx)->setHidden  (false) ;
		}
		return	;
	}
#endif
	for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
	{
		if (!hide && (qrow - getBlock()->getCurDRow() == idx))
			hide	= true ;

		m_ctrls.at(idx)->setHidden (hide) ;
	}
}

/*  KBItem								*/
/*  clearBelow	: Clear controls from specified row downwards		*/
/*  qrow	: uint		: Specific row				*/
/*  (returns)	: void		:					*/

void	KBItem::clearBelow
	(	uint	qrow
	)
{
	bool	clear	= false ;

	for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
	{
		if (!clear && (qrow - getBlock()->getCurDRow() == idx))
			clear	= true ;

		m_ctrls.at(idx)->clearValue (false) ;
	}
}

/*  KBItem								*/
/*  ctrlAtQRow	: Get control at specified query row if any		*/
/*  qrow	: uint		: Specific row				*/
/*  (returns)	: KBControl *	: Control or none			*/

KBControl *KBItem::ctrlAtQRow
	(	uint	qrow
	)
{
	uint	curDRow  = getBlock()->getCurDRow () ;

	if ((qrow >= curDRow) && (qrow < curDRow + m_ctrls.count()))
		return	m_ctrls.at(qrow - curDRow) ;

	setError
	(	KBError::Error,
		TR("Row in query not currently displayed"),
		QString	(TR("%1 %2, query row %3: showing %4 for %5"))
			.arg(getElement())
			.arg(getName   ())
			.arg(qrow    )
			.arg(curDRow )
			.arg(m_ctrls.count()),
		__ERRLOCN
	)	;
	return	0	;
}

/*  KBItem								*/
/*  setMarked	: Set or clear marked state				*/
/*  qrow	: uint		: Specific row				*/
/*  enable	: bool		: Enable				*/
/*  (returns)	: void		:					*/

void	KBItem::setMarked
	(	uint	qrow,
		bool	marked
	)
{
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	if (ctrl == 0) return ;

	DPRINTF
	((	"KBItem::setMarked: qrow=%d marked=%d\n",
		qrow,
		marked
	))	;

	QPalette *pal = marked ? getMarkedPalette() : getPalette(true) ;
	ctrl->setPalette (pal) ;
}

/*  KBItem								*/
/*  setVisible	: Set control visibility				*/
/*  qrow	: uint		: Specific row				*/
/*  visible	: bool		: Visibility				*/
/*  (returns)	: void		:					*/

void	KBItem::setVisible
	(	uint	qrow,
		bool	visible
	)
{
	KBControl *ctrl	= ctrlAtQRow (qrow) ;
	if (ctrl != 0) ctrl->setVisible (visible) ;
}

/*  KBItem								*/
/*  setEnabled	: Enable or disable a control				*/
/*  qrow	: uint		: Specific row				*/
/*  enable	: bool		: Enable				*/
/*  (returns)	: void		:					*/

void	KBItem::setEnabled
	(	uint	qrow,
		bool	enabled
	)
{
	KBControl *ctrl	= ctrlAtQRow (qrow) ;
	if (ctrl != 0) ctrl->setEnabled (enabled) ;
}

/*  KBItem								*/
/*  qrow	: uint		: Specific row				*/
/*  isEnabled	: Get control enabled/disabled				*/
/*  (returns)	: bool		: Enabled/disabled			*/

bool	KBItem::isEnabled
	(	uint	qrow
	)

{
	KBControl *ctrl	= ctrlAtQRow (qrow) ;
	return	ctrl == 0 ? false : ctrl->isEnabled () ;
}

/*  KBItem								*/
/*  isVisible	: Get control visibility				*/
/*  qrow	: uint		: Specific row				*/
/*  (returns)	: bool		: Visibility				*/

bool	KBItem::isVisible 
	(	uint	qrow
	)
{
	KBControl *ctrl	= ctrlAtQRow (qrow) ;
	return	ctrl == 0 ? false : ctrl->isVisible () ;
}

/*  KBItem								*/
/*  setAllVisible: Set control visibility of all controls		*/
/*  visible	 : bool		: Visibility				*/
/*  (returns)	 : void		:					*/

void	KBItem::setAllVisible
	(	bool	visible
	)
{
	m_allVisible = visible ;
	for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
		if (m_ctrls.at(idx) != 0)
			m_ctrls.at(idx)->setVisible (m_allVisible) ;
}

/*  KBItem								*/
/*  setAllEnabled: Enable or disable all controls			*/
/*  enabled	 : bool		: Enabled				*/
/*  (returns)	 : void		:					*/

void	KBItem::setAllEnabled
	(	bool	enabled
	)
{
	m_allEnabled = enabled ;
	for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
		if (m_ctrls.at(idx) != 0)
			m_ctrls.at(idx)->setEnabled (m_allEnabled) ;
}

/*  KBItem								*/
/*  setFocus 	: Give focus to specified row				*/
/*  qrow	: uint		: Query row number			*/
/*  (returns)	: void		:					*/

void	KBItem::setFocus 
	(	uint	qrow
	)
{
	DPRINTF
	((	"KBItem::setFocus : %s: qrow=%d\n",
		(cchar *)getName(),
		qrow
	))	;

	KBControl *ctrl = ctrlAtQRow (qrow) ;
	if (ctrl != 0) ctrl->setFocus () ;
}

/*  KBItem								*/
/*  setTabOrder	: Return tab order value				*/
/*  order	: int		: Tab order				*/
/*  (returns)	: void		:					*/

void	KBItem::setTabOrder
	(	int	order
	)
{
	m_tabOrd.setValue (order) ;
}

/*  KBItem								*/
/*  getTabOrder	: Return tab order value				*/
/*  (returns)	: int		: Tab order				*/

int	KBItem::getTabOrder ()
{
	return	m_tabOrd.getIntValue () ;
}

/*  KBItem								*/
/*  ctrlGeometry: Get control geometry					*/
/*  qrow	: uint		: Query row number			*/
/*  rCtrl	: QRect &	: Result				*/
/*  (returns)	: bool		: Success				*/

bool	KBItem::ctrlGeometry
	(	uint	qrow,
		QRect	&rCtrl
	)
{
	KBControl *ctrl = ctrlAtQRow(qrow) ;

	if (ctrl != 0)
	{	rCtrl	= ctrl->ctrlGeometry() ;
		return	true ;
	}

	rCtrl	= QRect() ;
	return	true  ;
}


/*  KBItem								*/
/*  keyStroke	: Control has received keystroke			*/
/*  k		: QKeyEvent *	: Key event in question			*/
/*  (returns)	: bool		: Key event consumed			*/

bool	KBItem::keyStroke
	(	QKeyEvent *k
	)
{
	KBNavigator *navi = getNavigator() ;
	return	navi == 0 ? false : navi->keyStroke (this, k) ;
}

/*  KBItem								*/
/*  isReadOnly	: See if item is read only				*/
/*  (returns)	: bool		: True for read only			*/

bool	KBItem::isReadOnly ()
{
	/* Check the embedding form block. Fields are never read-only	*/
	/* if we are in a query; fields are always read-only if the	*/
	/* block is marked read-only.					*/
	KBFormBlock *fb	  = getBlock()->isFormBlock()  ;
	if (fb != 0)
	{
		DPRINTF
		((	"KBItem::isReadOnly: fb.inq=%d fb.blkro=%d\n",
			fb->isInQuery    (),
			fb->isBlkReadOnly()
		))	;

		if (fb->isInQuery    ()) return false ;
		if (fb->isBlkReadOnly()) return true  ;
	}

	DPRINTF
	((	"KBItem::isReadOnly: m_rd=%d fl=%d\n",
		m_rdonly.getBoolValue (),
		(m_flags & KBFieldSpec::ReadOnly) != 0
	))	;

	if (isA("KBRichText"))
		if (m_rdonly.getValue().isEmpty())
			return	true	;

	return	m_rdonly.getFlags() != ARO_NO ;
}

/*  KBItem								*/
/*  getReportValue							*/
/*		: Get value for report writing				*/
/*  first	: bool		: First output flag			*/
/*  prior	: bool		: Return prior value flag		*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBItem::getReportValue
	(	bool		,
		bool
	)
{
	return	m_curVal ;
}

/*  KBItem								*/
/*  write	: Write control values					*/
/*  writer	: KBWriter *	: Writer				*/
/*  offset	: QPoint	: Writer offset				*/
/*  first	: bool		: First on page flag			*/
/*  extra	: init &	: Return extra space			*/
/*  prior	: bool		: Write prior value flag		*/
/*  (returns)	: bool		: Success				*/

bool	KBItem::write
	(	KBWriter	*writer,
		QPoint		offset,
		bool		first,
		int		&extra,
		bool		prior
	)
{
	if (!writer->asReport())
	{
		/* This method will be called in design mode and when	*/
		/* printing forms. The work is all done at the control	*/
		/* level, but we iterate through each row of controls.	*/
		QRect	posn	= geometry(offset) ;
		int	dx	= getBlock()->getAttrVal("dx").toInt() ;
		int	dy	= getBlock()->getAttrVal("dy").toInt() ;

		for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
		{
			if (showingDesign() || m_ctrls.at(idx)->isVisible ())
				m_ctrls.at(idx)->write (writer, posn, KBValue(), false, extra) ;
			posn.moveBy (dx, dy) ;
		}

		return	true	;
	}

	DPRINTF
	((	"KBItem::write: %s: %s: %p\n",
		(cchar *)getElement(),
		(cchar *)getName   (),
		(void  *)m_ctrls.at(0)
	))	;

	if (m_ctrls.isEmpty())
		setupControls () ;

	return	m_ctrls.at(0)->write
		(	writer,
			geometry (offset),
			getReportValue (first, prior),
			m_fSubs,
			extra
		)	;
}

/*  KBItem								*/
/*  setMonitor	: Set monitor values for this node			*/
/*  myitem	: KBNodeMonitor * : Monitor item for this node		*/
/*  (returns)	: void		  :					*/

void	KBItem::setMonitor
	(	KBNodeMonitor	*myitem
	)
{
	KBNode::setMonitor (myitem) ;

	if (myitem != 0)
	{
		m_ctrlMon = new KBNodeMonitor (0, myitem) ;
		m_ctrlMon->setText (0, "Controls") ;
		m_ctrlMon->setSelectable (false) ;
	}
	else	m_ctrlMon = 0 ;

	for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
		if (m_ctrls.at(idx) != 0)
			m_ctrls.at(idx)->showMonitor (m_ctrlMon) ;
}

/*  KBItem								*/
/*  property	: Get QT widget property				*/
/*  qrow	: uint		 : Query row				*/
/*  name	: cchar *	: Property name				*/
/*  (returns)	: QVariant	: Property value			*/

QVariant
	KBItem::property
	(	uint		qrow,
		cchar		*name
	)
{
	KBControl *ctrl = ctrlAtQRow  (qrow)   ;
	return	ctrl == 0 ? QVariant() : ctrl->property (name) ;
}

/*  KBItem								*/
/*  setProperty	: Set QT widget property				*/
/*  qrow	: uint		 : Query row				*/
/*  name	: cchar *	   : Property name			*/
/*  value	: const QVariant & : Value				*/
/*  (returns)	: bool		   : Success				*/

bool	KBItem::setProperty
	(	uint		qrow,
		cchar		*name,
		const QVariant	&value
	)
{
	KBControl *ctrl = ctrlAtQRow  (qrow)   ;
	return	ctrl == 0 ? false : ctrl->setProperty (name, value) ;
}

/*  KBItem								*/
/*  repaintMorph: Repaint morphed control				*/
/*  p		: QPainter *	: Painter				*/
/*  drow	: uint		: Display row				*/
/*  (returns)	: void							*/

void	KBItem::repaintMorph
	(	QPainter	*p,
		uint		drow
	)
{
	if (drow < m_ctrls.count())
	{	KBControl *ctrl = m_ctrls.at   (drow) ;
		if (ctrl != 0) ctrl->repaintMorph (p) ;
	}
}

/*  KBItem								*/
/*  repaintMorph: Repaint morphed controls				*/
/*  p		: QPainter *	: Painter				*/
/*  area	: const QRect &	: Area to be repainted			*/
/*  (returns)	: void							*/

void	KBItem::repaintMorph
	(	QPainter	*p,
		const QRect	&area
	)
{
	DPRINTF
	((	"KBItem::repaintMorph: %s: %s\n",
		 getName().latin1(),
		 _TEXT(area)
	))	;

	/* Optimisation: if the delta-x setting is zero (usual case	*/
	/* with controls in columns) then ignore if the control is	*/
	/* entirely to the left or right of the update area.		*/
	if (getBlock()->getDisplayDX() == 0)
	{
		QRect		cRect	= m_ctrls.at(0)->ctrlGeometry() ;
		if (cRect.right() < area.left ()) return ;
		if (cRect.left () > area.right()) return ;
		
	}

	for (uint drow = 0 ; drow < m_ctrls.count() ; drow += 1)
	{
		KBControl	*ctrl	= m_ctrls.at(drow)	;
		QRect		cRect	= ctrl->ctrlGeometry()	;

		if (cRect.intersects(area))
			ctrl->repaintMorph (p) ;
	}
}

/*  KBItem								*/
/*  mouseClickHit: Check for hit into morphed control			*/
/*  pos		 : const QPoint & : Mouse position			*/
/*  (returns)	 : bool		  : True if a hit			*/

bool	KBItem::mouseClickHit
	(	const QPoint	&pos
	)
{
	KBBlock	*block	= getBlock () ;
	uint	curDRow	= block->getCurDRow () ;
	uint	numRows	= block->getNumRows () ;

	/* Optimisation: if the delta-x setting is zero (usual case	*/
	/* with controls in columns) then ignore if the control is	*/
	/* entirely to the left or right of the update area.		*/
	if (block->getDisplayDX() == 0)
	{
		QRect  cRect = m_ctrls.at(0)->ctrlGeometry() ;
		if (cRect.right() < pos.x()) return false ;
		if (cRect.left () > pos.x()) return false ;
		
	}

	for (uint drow = 0 ; drow < m_ctrls.count() ; drow += 1)
	{
		KBControl *ctrl = m_ctrls.at(drow) ;

		if (drow + curDRow > numRows + 1)
			break	 ;

		if (!ctrl->isVisible ()) continue ;
		if (!ctrl->isEnabled ()) continue ;

		if (ctrl->ctrlGeometry().contains(pos))
		{
			if (moveFocusOK (drow))
			{	setFocus  (curDRow + drow) ;
				if (showingData())
					focusInEvent (drow, QFocusEvent::Mouse) ;
			}

			return	  true	;
		}
	}

	return	false	;
}

/*  KBItem								*/
/*  isMorphing	: Test if item is morphed				*/
/*  (returns)	: bool		: Morphed				*/

bool	KBItem::isMorphing ()
{
	/* This is always false. Specific data controls like fields and	*/
	/* links may override this.					*/
	return	false	;
}

/*  KBItem								*/
/*  getIniValue	: Get initial value					*/
/*  qrow	: uint		: Row number				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBItem::getIniValue
	(	uint	qrow
	)
{
	KBQryBase	*query	= getBlock()->getQuery() ;

	return	query->getField (getQueryLvl(), qrow, getQueryIdx(), true) ;
}


void	KBItem::recordVerifyState ()
{
#if	! __KB_RUNTIME
	KBControl *ctrl = m_ctrls.at(m_recordDRow) ;
	if (ctrl != 0)
	{
		KBRecorder *recorder = KBRecorder::self() ;
		if ((recorder != 0) && recorder->isRecording(getDocRoot()))
			recorder->verifyState
			(	this,
				m_recordDRow,
				ctrl->isEnabled(),
				ctrl->isVisible()
			)	;
	}
#endif
}

#if	! __KB_RUNTIME
void	KBItem::recordUpdateValue
	(	uint		qrow,
		const KBValue	&value
	)
{
	KBRecorder *recorder = KBRecorder::self() ;
	if ((recorder != 0) && recorder->isRecording(getDocRoot()))
		recorder->updateValue
		(	this,
			qrow - getBlock()->getCurDRow(),
			value
		)	;
}
#endif


/*  KBItem								*/
/*  userChange	: User has changed value				*/
/*  qrow	: uint		  : Query row number			*/
/*  value	: const KBValue & : New value				*/
/*  force	: doChange	  : Force "onchange"			*/
/*  (returns)	: void		  :					*/

void	KBItem::userChange
	(	uint		qrow,
		const KBValue	&value,
		bool		doChange
	)
{
	KBFormBlock *fBlk = getFormBlock() ;
	KBAttr	    *onc  ;

	DPRINTF
	((	"KBItem::userChange: !iuv=%d fb=%p iiq=%d\n",
		!isUpdateVal(),
		(void *)fBlk,
		fBlk == 0 ? false : fBlk->isInQuery()
	))	;


	/* Handle the "onchange" event (if it exists) unless this is a	*/
	/* field, memo or rich text (since these do "onchange" when	*/
	/* focus leaves the control).					*/
	bool	change	= !inherits("KBField") && !inherits("KBMemo") && !inherits("KBRichText") ;

	if (change || doChange)
	{
#if		! __KB_RUNTIME
		recordUpdateValue (qrow, value) ;
#endif
		if ((onc = getAttr ("onchange")) != 0)
		{
			KBValue	args[2]		;
			bool	evRc		;

			args[0] = (int)qrow	;
			args[1] = value		;

			eventHook (*(onc->isEvent()), 2, args, evRc) ;
		}

		if (fBlk != 0) fBlk->dataChanged (qrow) ;
	}


	/* Check if this item is updatable (should not get called	*/
	/* unless it is, but ...), and that it is in a form (ditto ...)	*/
	/* and ignore otherwise; also, more interestingly, ignore if we	*/
	/* are in a query.						*/
	if (!isUpdateVal(true) || (fBlk == 0) || fBlk->isInQuery())
		return	;

	/* Also ignore changes if the associated query is the null	*/
	/* (menu-only) query.						*/
	if (fBlk->getQuery()->isQryNull() != 0)
		return	;

	/* Notify the layout object so that the toolbar gets updated,	*/
	/* and the form block so that the block-level change event can	*/
	/* be triggered.						*/
	DPRINTF
	((	"KBItem::userChange: notifying\n"
	))	;

	getLayout()->setChanged  (true, errorText()) ;
}

/*  KBItem								*/
/*  isInQuery	: See if user has started a query			*/
/*  (returns)	: bool		: In query				*/

bool	KBItem::isInQuery ()
{
	KBFormBlock *fBlk = getBlock()->isFormBlock() ;
	return	fBlk == 0 ? false : fBlk->isInQuery() ;
}

/*  KBItem								*/
/*  startUpdate	: Start locked update					*/
/*  qrow	: uint		: Row to be updated			*/
/*  (returns)	: bool		: Locked OK				*/

bool	KBItem::startUpdate
	(	uint		qrow
	)
{
	/* Paranoia check that we are in data view, in case of events	*/
	/* triggered by design changes in design view. Code in the	*/
	/* KBControl class should trap these, but better safe than	*/
	/* sorry. Also check in case we are in a query.			*/
	if (showingData() && !isInQuery())
	{
		KBFormBlock *fBlk = getFormBlock() ;
		if ((fBlk != 0) && !fBlk->startUpdate (qrow))
		{
			fBlk->lastError().DISPLAY() ;
			return	false ;
		}
	}

	return	true	;
}

/*  KBItem								*/
/*  doSearch	: Do record search					*/
/*  (returns)	: void		:					*/

void	KBItem::doSearch ()
{
	/* Default is to do nothing. Controls that can actually do a	*/
	/* record search override this.					*/
}

/*  KBItem								*/
/*  getSearchText: Get sarch text for record search			*/
/*  qrow	 : uint		: Query row				*/
/*  (returns)	 : QString	: Text					*/

QString	KBItem::getSearchText
	(	uint	qrow
	)
{
	return	getBlock()->getRowValue(this, qrow).getRawText() ;
}

/*  KBItem								*/
/*  errorText	: Get error message text				*/
/*  (returns)	: QString	: Text					*/

QString	KBItem::errorText ()
{
	if (!m_errtext.getValue().isEmpty())
		return m_errtext.getValue() ;

	QStringList bits ;
	if (!getName().isEmpty()) bits.append (getName()) ;
	if (!getExpr().isEmpty()) bits.append (getExpr()) ;
	if (bits.count() == 1) return bits[0] ;
	if (bits.count() == 2) return bits.join (": ") ;

	return	TR("unidentified control") ;
}

/*  KBItem								*/
/*  enumKBProperty							*/
/*		: Get list of properties				*/
/*  list	: QStringList &	: Property list				*/
/*  (returns)	: void		:					*/

void	KBItem::enumKBProperty
	(	QStringList	&list
	)
{
	/* Note: Don't include "visible" and "enabled" as these are	*/
	/*	 present at the KBObject level as well.			*/
	list.append ("value"   ) ;
	list.append ("readOnly") ;
	KBObject::enumKBProperty (list) ;
}

/*  KBItem								*/
/*  hasKBProperty: Check if object has this property			*/
/*  name	 : cchar *	: Property name				*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBItem::hasKBProperty
	(	cchar		*name
	)
{
	DPRINTF
	((	"KBItem::hasKBProperty: [%s]\n",
		name
	))	;

	if (qstrcmp (name, "value"   ) == 0)
		return	true ;
	if (qstrcmp (name, "visible" ) == 0)
		return	true ;
	if (qstrcmp (name, "enabled" ) == 0)
		return	true ;
	if (qstrcmp (name, "readOnly") == 0)
		return	true ;

	return	KBObject::hasKBProperty (name) ;
}

/*  KBItem								*/
/*  getKBProperty: Get a rekall property value				*/
/*  value	 : KBValue &	: Return property value			*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBItem::getKBProperty
	(	cchar		*name,
		KBValue		&value
	)
{
	DPRINTF
	((	"KBItem::getKBProperty: for [%s] row=%d\n",
		name,
		getBlock()->getCurQRow()
	))	;

	if (qstrcmp (name, "value"  ) == 0)
	{
		if (getBlock() != 0)
			value	= getValue 	(getBlock()->getCurQRow()) ;
		else	value	= KBValue  	() ;

		return	true ;
	}

	if (qstrcmp (name, "visible" ) == 0)
	{
		if (getBlock() != 0)
			value	= isVisible	(getBlock()->getCurQRow()) ;
		else	value	= KBValue  	() ;

		return	true ;
	}

	if (qstrcmp (name, "enabled" ) == 0)
	{
		if (getBlock() != 0)
			value	= isEnabled	(getBlock()->getCurQRow()) ;
		else	value	= KBValue  	() ;

		return	true ;
	}

	if (qstrcmp (name, "readOnly") == 0)
	{
		if (getBlock() != 0)
			value	= isCtrlReadOnly(getBlock()->getCurQRow()) ;
		else	value	= KBValue 	() ;

		return	true ;
	}

	return	KBObject::getKBProperty (name, value) ;
}

/*  KBItem								*/
/*  setKBProperty: Set a rekall property value				*/
/*  name	 : cchar *	: Proprty name				*/
/*  value	 : KBValue	: Property value			*/
/*  (returns)	 : bool		: Success				*/

bool	KBItem::setKBProperty
	(	cchar		*name,
		const KBValue	&value
	)
{
	KBBlock	  *block  = getBlock () ;
	uint	  qrow	  = block == 0 ? 0 : block->getCurQRow() ;
	KBControl *ctrl	  = block == 0 ? 0 : ctrlAtQRow   (qrow) ;

	DPRINTF
	((	"KBItem::setKBProperty: %s: %p/%p/%d: %s\n",
		name,
		(void *)block,
		(void *)ctrl,
		qrow,
		value.getRawText().latin1()
	))	;

	if (qstrcmp (name, "value"    ) == 0)
	{
		if (block != 0) setValue   (qrow, value) ;
		return	true ;
	}

	if (qstrcmp (name, "visible"  ) == 0)
	{
		if (block != 0) setVisible (qrow, value.isTrue()) ;
		return	true ;
	}

	if (qstrcmp (name, "enabled" ) == 0)
	{
		if (block != 0) setEnabled (qrow, value.isTrue()) ;
		return	true ;
	}

	if (qstrcmp (name, "readOnly") == 0)
	{
		if (ctrl  != 0) ctrl->setReadOnly (value.isTrue(), QColor(), QColor()) ;
		return	true ;
	}

	if (qstrcmp (name, "fgcolor" ) == 0)
	{
		if (ctrl  != 0) ctrl->setFGColor  (QColor(value.getRawText().toInt(0, 0))) ;
		return	true ;
	}

	if (qstrcmp (name, "bgcolor" ) == 0)
	{
		if (ctrl  != 0) ctrl->setBGColor  (QColor(value.getRawText().toInt(0, 0))) ;
		return	true ;
	}

	return	KBObject::setKBProperty (name, value) ;
}

/*  KBItem								*/
/*  canBeInvalid: Check if control can be invalid			*/
/*  (returns)	: bool		: True if so				*/

bool	KBItem::canBeInvalid ()
{
	return	false	;
}

/*  KBItem								*/
/*  validatorMode							*/
/*		: Get validator setting					*/
/*  graphicGood	: QPixmap &	  : Return good graphic			*/
/*  graphicBad	: QPixmap &	  : Return bad graphic			*/
/*  (returns)	: KBItem::ValFlag :					*/

KBItem::ValFlag
	KBItem::validatorMode
	(	QPixmap		&graphicGood,
		QPixmap		&graphicBad
	)
{
	/* Get the validator mode from the validator property. If this	*/
	/* specifies good and/or bad pixmaps then load the associated	*/
	/* images. These are returned via the arguments.		*/
	QString	good	;
	QString	bad	;
	ValFlag	vf	= (ValFlag)m_valFlag.validatorMode(good, bad) ;
	KBError	error	;

	if (!good.isEmpty())
	{
		QStringList	iparts	= QStringList::split ('.', good) ;

		DPRINTF
		((	"KBItem::validatorMode: %s: [%s][%s]\n",
			 getName().latin1(),
			(cchar *)iparts[0],
			(cchar *)iparts[1]
		))	;

		graphicGood	= KBAttrImage::pixmapFromLocation
					  (	getDocRoot(),
						iparts[0],
						iparts[1],
						error
					  )	;

		if (graphicGood.isNull()) error.DISPLAY() ;
	}
	else	graphicGood	= QPixmap() ;

	if (!bad .isEmpty())
	{
		QStringList	iparts	= QStringList::split ('.', bad ) ;

		DPRINTF
		((	"KBItem::validatorMode: %s: [%s][%s]\n",
			 getName().latin1(),
			(cchar *)iparts[0],
			(cchar *)iparts[1]
		))	;

		graphicBad	= KBAttrImage::pixmapFromLocation
					  (	getDocRoot(),
						iparts[0],
						iparts[1],
						error
					  )	;

		if (graphicBad.isNull()) error.DISPLAY() ;
	}
	else	graphicBad	= QPixmap() ;

	/* Process the flag itself. In the case of the Auto settings,	*/
	/* convert depending on whether the control can actuall be	*/
	/* invalid.							*/
	switch (vf)
	{
		case VFNone	:
		case VFShow	:
		case VFBlank	:
			return	vf	;

		case VFAutoShow	:
			return	canBeInvalid() ? VFShow : VFNone  ;

		case VFAutoBlank:
			return	canBeInvalid() ? VFShow : VFBlank ;

		default	:
			break	;
	}

	return	VFNone	;
}

#if	! __KB_RUNTIME

/*  KBItem								*/
/*  whatsThisExtra							*/
/*		: Extra stuff for pseudo-what's this popup		*/
/*  info	: QValueList<KBWhatsThisPair> &	: List of items		*/
/*  (returns)	: void				:			*/

void	KBItem::whatsThisExtra
	(	QValueList<KBWhatsThisPair>	&info
	)
{
	QString	expr	= getExpr() ;
	if (expr.isEmpty()) expr = TR("<i>None</i>") ;
	info.append (KBWhatsThisPair(TR("Expression"), expr)) ;
}

#endif

/*  KBItem								*/
/*  getMarkedPalette							*/
/*		: Get pallete for marked data controls			*/
/*  (returns)	: QPalette *	: Required palette			*/

QPalette
	*KBItem::getMarkedPalette()
{
	/* Thanks to							*/
	/*	Tim Southerwood <ts@dionic.net>				*/
	/* for this code (and related code in property dialog, etc.	*/
	if (m_markedPalette == 0)
	{
		QColor markbg = QColor (0, 0, 0)	;
		QColor markfg = QColor (255, 255, 255)	;

		if (!m_markbgcolor.getValue().isEmpty())
			markbg = QColor (QRgb(m_markbgcolor.getValue().toInt(0,0))) ;

		if (!m_markfgcolor.getValue().isEmpty())
			markfg = QColor (QRgb(m_markfgcolor.getValue().toInt(0,0))) ;

		DPRINTF
		((	"KBItem::getMarkedPalette: bg=%s fg=%s\n",
			markbg.name().latin1(),
			markfg.name().latin1()
		))	;

		/* If there is an error the (silently to the user)	*/
		/* fall back on the default palette.			*/
		if (!markbg.isValid() || !markfg.isValid())
		{
			fprintf
			(	kbDPrintfGetStream(),	
				"KBItem::KBItemInitMarkedPalette(): "
				"Error, one or more palette colours is invalid\n"
			)	;
			return	getPalette (true) ;
		}

		m_markedPalette = new QPalette (QApplication::palette()) ;
		m_markedPalette->setColor  (QColorGroup::Text,       markfg) ;
		m_markedPalette->setColor  (QColorGroup::Foreground, markfg) ;
		m_markedPalette->setColor  (QColorGroup::Base,       markbg) ;
		m_markedPalette->setColor  (QColorGroup::Background, markbg) ;
	}

	return	m_markedPalette	;
}


/*  KBItem								*/
/*  setCtrlFGColor							*/
/*		: Set control foreground colour				*/
/*  qrow	: uint		  : Specific row			*/
/*  color	: const QString & : Color				*/
/*  (returns)	: void		  :					*/

void	KBItem::setCtrlFGColor
	(	uint		qrow,
		const QString	&color
	)
{
	KBControl *ctrl	= ctrlAtQRow (qrow) ;
	if (ctrl != 0)
	{
		QPalette pal = ctrl->getWidget()->palette () ;
		pal.setColor (QColorGroup::Text,       color) ;
		pal.setColor (QColorGroup::ButtonText, color) ;
		pal.setColor (QColorGroup::Foreground, color) ;
		ctrl->getWidget()->setPalette (pal) ;
	}
}

/*  KBItem								*/
/*  setCtrlBGColor							*/
/*		: Set control background colour				*/
/*  qrow	: uint		  : Specific row			*/
/*  color	: const QString & : Color				*/
/*  (returns)	: void		  :					*/

void	KBItem::setCtrlBGColor
	(	uint		qrow,
		const QString	&color
	)
{
	KBControl *ctrl	= ctrlAtQRow (qrow) ;
	if (ctrl != 0)
	{
		QPalette pal = ctrl->getWidget()->palette () ;
		pal.setColor (QColorGroup::Base,       color) ;
		pal.setColor (QColorGroup::Button,     color) ;
		pal.setColor (QColorGroup::Background, color) ;
		ctrl->getWidget()->setPalette (pal) ;
	}
}

/*  KBItem								*/
/*  setCtrlFont	: Set control font					*/
/*  qrow	: uint		  : Specific row			*/
/*  font	: const QString & : Font				*/
/*  (returns)	: void		  :					*/

void	KBItem::setCtrlFont
	(	uint		qrow,
		const QString	&spec
	)
{
	KBControl *ctrl	= ctrlAtQRow (qrow) ;
	if (ctrl != 0)
	{
		QFont	font	;

		if	(spec.isEmpty())
			font	= QApplication::font() ;
		else	font	= KBFont::specToFont (spec) ;

		ctrl->getWidget()->setFont (font) ;
	}
}

/*  KBItem								*/
/*  recordVerifyValue							*/
/*  		: Record value verification for test			*/
/*  (returns)	: void		:					*/

void	KBItem::recordVerifyValue ()
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBItem::recordVerifyValue: drow=%d\n",
		m_recordDRow
	))	;

	KBRecorder *recorder = KBRecorder::self() ;
	if ((recorder == 0) || !recorder->isRecording(getDocRoot()))
		return	;

	recorder->verifyValue
	(	this,
		m_recordDRow,
		getValue(getBlock()->getCurDRow() + m_recordDRow)
	)	;
#endif
}

/*  KBItem								*/
/*  recordVerifyRegexp							*/
/*  		: Record value verification (using regexp) for test	*/
/*  (returns)	: void		:					*/

void	KBItem::recordVerifyRegexp ()
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBItem::recordVerifyRegexp: drow=%d\n",
		m_recordDRow
	))	;

	KBRecorder *recorder = KBRecorder::self() ;
	if ((recorder == 0) || !recorder->isRecording(getDocRoot()))
		return	;

	QString		  regexp ;
	KBPromptRegexpDlg reDlg
		(	TR("Verify Regular Expression"),
			TR("Enter regular expression" ),
			getValue(getBlock()->getCurDRow() + m_recordDRow).getRawText(),
			regexp
		)	;
	if (reDlg.exec())
		recorder->verifyRegexp
		(	this,
			m_recordDRow,
			regexp
		)	;
#endif
}

#if	! __KB_RUNTIME

/*  KBItem								*/
/*  makeRecordPopup							*/
/*		: Construct recording popup menu			*/
/*  popup	: KBPopupMenu *	: Popup menu to add to			*/
/*  drow	: uint		: Display row number			*/
/*  parents	: bool		: Add submenus for parents		*/
/*  (returns)	: void		:					*/

void	KBItem::makeRecordPopup
	(	KBPopupMenu		*popup,
		uint			drow,
		bool			parents
	)
{
	popup->insertItem
	(	TR("Verify value"),
		this,
		SLOT(recordVerifyValue ())
	)	;
	popup->insertItem
	(	TR("Verify value with regexp"),
		this,
		SLOT(recordVerifyRegexp())
	)	;
	m_recordDRow	= drow	;

	KBObject::makeRecordPopup (popup, drow, parents) ;
}
#endif
