/***************************************************************************
    file	         : kb_item.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#ifndef		__KB_FORMITEM_H
#define		__KB_FORMITEM_H

#include	"libkbase_exports.h"

#include	"kb_object.h"
#include	"kb_qryidx.h"
#include	"kb_attrexpr.h"
#include	"kb_attrvalidator.h"
#include	"kb_attrnoupdate.h"
#include	"kb_attrreadonly.h"

#include	<qmemarray.h>

class		KBPopupMenu	;
class		QMouseEvent	;

/*  KBItem								*/
/*  ----------								*/
/*  This class is used for form elements which are related to values	*/
/*  returned by queries, and hence have an expression (whicn might just	*/
/*  be a column name) and a query column index. It is this class that	*/
/*  the KBSQLQuery class accesses.					*/


class	LIBKBASE_API	KBItem : public KBObject
{
public	:

	enum	ValFlag
	{
		VFNone		= 0,
		VFShow		= 1,
		VFBlank		= 2,
		VFAutoShow	= 3,
		VFAutoBlank	= 4
	}	;

private	:

	Q_OBJECT

	KBTable		*m_table	;	/* Associated table	*/
	uint		m_flags		;	/* KBFieldSpec flags	*/
	QString		m_defval  	;	/* RDBMS-level default	*/
	QPalette	*m_markedPalette;	/* Used for marked rows */

	int		m_isUpdateVal	;

protected slots :

	void		recordVerifyValue 	()	;
	void		recordVerifyRegexp 	()	;
	virtual	void	recordVerifyState	()	;

protected :

	KBAttrExpr	m_expr		;	/* Query expression	*/
	KBAttrReadOnly	m_rdonly	;	/* Value is read-only	*/
	KBAttrNoupdate	m_noupdate	;	/* Do not update item	*/
	KBAttrUInt	m_tabOrd  	;	/* Tab ordering		*/
	KBAttrUInt	m_transfer  	;	/* Web trasnferc		*/
	KBAttrValidator	m_valFlag	;	/* Validator flag	*/
	KBAttrExpr	m_defVal	;	/* Default value	*/
	KBAttrStr	m_errtext	;	/* Error message text	*/
	KBAttrStr	m_markbgcolor	;	/* Marked row background colour	*/
	KBAttrStr	m_markfgcolor	;	/* Marked row foreground colour	*/

	KBEvent		m_onEnter	;	/* Focus enters field	*/
	KBEvent		m_onLeave	;	/* Focus leaves field	*/
	KBEvent		m_onSet		;	/* Field value is set	*/
	KBEvent		m_onDblClick	;	/* Double-click handler	*/

	KBQryIdx	m_qryIdx	;	/* Query column index	*/
	bool		m_needed	;	/* Query needs column	*/

	KBType		*m_type		;	/* Column type		*/

	QMemArray<KBControl *>
			m_ctrls		;	/* Child controls	*/

	KBValue		m_curVal	;	/* Current row value	*/
	bool		m_fSubs		;	/* Value substitution	*/

	bool		m_allEnabled	;	/* Global enable	*/
	bool		m_allVisible	;	/* Global visible	*/

	KBNodeMonitor	*m_ctrlMon	;	/* Monitor display	*/

	uint		m_recordDRow	;

	virtual void		setPalette 		() ;
	virtual void		setFont			() ;
	KBControl		*ctrlAtQRow		(uint) ;
	virtual	void		setMonitor		(KBNodeMonitor *) ;

public	:

	virtual	KBItem  	*isItem () { return this  ; }

	virtual	void		setupProperties		()			;

	virtual void		setupControls		()			;
	virtual	KBControl	*makeCtrl		(uint)			;
	virtual	void		ctrlGone		(KBControl *)		;

	virtual	void		setGeometry		(const QRect  &)	;
	virtual void		setCtrlGeometry		(const QRect  &) 	;
	virtual	bool		ctrlGeometry		(uint, QRect  &)	;

	void			reposition		()			;
	virtual	void		setFocus     		(uint)			;

	bool			startUpdate		(uint)		  	;
	virtual	bool		isUpdateVal		(bool = false)		;
	virtual	void		setFieldType		(KBType *)		;

	virtual	KBValue::VTrans	getVTrans		() ;

	/* NOTE:							*/
	/* In the following methods which refer to data values, the	*/
	/* first unsigned-int argument is the associated query row	*/
	/* number, ie., the row number in the query which returned the	*/
	/* original value.						*/
	virtual	bool		setValue		(uint, const KBValue &) ;
	virtual	KBValue		getValue		(uint) ;
	virtual	KBValue		getRowValue		(uint) ;
	virtual	KBValue		getIniValue		(uint) ;
	virtual	bool		isEmpty			(uint) ;
	virtual void		clearValue		(uint, bool) ;
	virtual	bool		isValid			(uint, bool) ;
	virtual	bool		changed			(uint) ;
	virtual void		userChange		(uint, const KBValue &, bool = false) ;
	virtual bool		checkValid		(const KBValue &, bool) ;

 	QPalette 		*getMarkedPalette	()   ;

	virtual	QString		getOrderValue		(const KBValue &) ;
	virtual	KB::IType	getOrderType		()     ;

	virtual	void		hideBelow		(uint) ;
	virtual	void		clearBelow		(uint) ;

	virtual	void		setTabOrder		(int ) ;
	virtual	int		getTabOrder		()     ;

	virtual	void		setMarked		(uint, bool) ;
	virtual	void		setEnabled		(uint, bool) ;
	virtual	void		setVisible		(uint, bool) ;
	virtual bool		isEnabled		(uint) ;
	virtual bool		isVisible		(uint) ;

	virtual	void		setAllEnabled		(bool) ;
	virtual	void		setAllVisible		(bool) ;

	virtual	void		showAs			(KB::ShowAs) ;

	virtual	void		setCtrlFGColor		(uint, const QString &) ;
	virtual	void		setCtrlBGColor		(uint, const QString &) ;
	virtual	void		setCtrlFont		(uint, const QString &) ;


	QString			getExpr			() ;
	bool			isInQuery		() ;
	bool			isReadOnly		() ;
	QString			errorText		() ;

	virtual	void		setCtrlReadOnly		(uint, bool, const QColor &, const QColor &) ;
	virtual	bool		isCtrlReadOnly		(uint)		;

	inline	uint		getQueryIdx		()	 	{ return m_qryIdx.qryIdx() ; }
	inline	uint		getQueryLvl		()	 	{ return m_qryIdx.qryLvl() ; }

	inline	void		setQueryIdx		(KBQryIdx qi)	{ m_qryIdx   =  qi ; } 

	inline	KBType		*getFieldType		()	 	{ return m_type	 ; }
	inline	void		setNeeded		(bool v) 	{ m_needed = v	 ; }

	inline	void		setTable		(KBTable *t)	{ m_table  =   t ; }
	inline	KBTable		*getTable 		()		{ return m_table ; }

	inline	void		setFlags		(uint f )	{ m_flags  =   f ; }
	inline	void		addFlags		(uint f )	{ m_flags |=   f ; }
	inline	void		remFlags		(uint f )	{ m_flags &=  ~f ; }
	inline	uint		flags			()		{ return m_flags ; }

	inline	void		setDefVal		(QString v)	{ m_defval =   v ; }
	inline	const QString	&getDefVal		()		{ return m_defval; }

	virtual	bool		canBeInvalid		() ;
	KBItem::ValFlag		validatorMode		(QPixmap &, QPixmap &) ;

	virtual	bool		moveFocusOK		(uint) ;
	virtual	void		focusInEvent		(uint, QFocusEvent::Reason) ;
	virtual	bool		isMorphing		()     ;
	virtual	bool		keyStroke		(QKeyEvent *) ;

	virtual	void		doEnter			(uint) ;
	virtual	bool		doLeave			(uint) ;
	virtual	bool		doDblClick		(uint) ;

	virtual	KBValue		getReportValue		(bool, bool = false) ;
	virtual	bool		write			(KBWriter *, QPoint, bool, int &, bool = false) ;

	virtual	QVariant	property		(uint, cchar *)		;
	virtual	bool		setProperty		(uint, cchar *, const QVariant &) ;

#if	! __KB_RUNTIME
	void			recordUpdateValue	(uint, const KBValue &) ;
	virtual	void		whatsThisExtra		(QValueList<KBWhatsThisPair> &) ;
	virtual void		makeRecordPopup		(KBPopupMenu *, uint, bool) ;
#endif

	virtual	void		repaintMorph		(QPainter *, uint) ;
	virtual	void		repaintMorph		(QPainter *, const QRect &) ;
	virtual	bool		mouseClickHit		(const QPoint &) ;


	virtual	void		doSearch		()	;
	virtual	QString		getSearchText		(uint)	;

	inline	QString		getErrText ()
	{
		return	m_errtext.getValue () ;
	}

	/* These two methods are used for access-by-attribute from	*/
	/* scripts.							*/
	virtual	void		enumKBProperty		(QStringList	  	&) ;
	virtual	bool		hasKBProperty		(cchar *)		   ;
	virtual	bool		getKBProperty		(cchar *,	KBValue &) ;
	virtual	bool		setKBProperty		(cchar *, const KBValue &) ;

	friend	class		KBMacroTest	;
	friend	class		KBTestAccessor	;
	friend	class		KBForm		;

	KBItem			(KBNode   *, cchar *, cchar *, const QDict<QString> &) ;
	KBItem			(KBObject *, cchar *, const QRect &, cchar *, cchar *, uint) ;
	KBItem			(KBNode   *, cchar *, KBItem *) ;
	virtual ~KBItem 	() ;

}	;


#endif		// __KB_FORMITEM_H
