;/***************************************************************************
    file	         : kb_label.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#if		__KB_AUTOMOC
#include	"kb_label.moc"
#else
#include	"kb_label.h"
#endif

#include	"kb_nodereg.h"

#if		! __KB_RUNTIME
#include	"kb_recorder.h"
#include	"kb_popupmenu.h"
#endif

/*  KBLabel								*/
/*  KBLabel	: Consructor for label node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBLabel	:					*/

KBLabel::KBLabel
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBObject	(parent, "KBLabel", 	aList),
	m_text		(this,   "text",	aList, KAF_EDITOR),
	m_fgcolor	(this,   "fgcolor",     aList),
	m_bgcolor	(this,   "bgcolor",     aList),
	m_frame		(this,	 "frame",	aList),
	m_font		(this,	 "font",	aList),
	m_align		(this,	 "align",	aList),
	m_buddy		(this,	 "buddy",	aList, KAF_FORM),
	m_onClick	(this,	 "onclick",	aList, KAF_EVCS)
{
	m_label	= 0 ;

#if	! __KB_RUNTIME
	if (ok != 0)
	{
		if (!labelPropDlg (this, "Label", m_attribs))
		{	delete this	;
			*ok	= false ;
			return	    ;
		}

		*ok	= true	;
	}
#endif
	if (getParent() != 0) m_report = getParent()->getReport () ;
}

/*  KBLabel								*/
/*  KBLabel	: Consructor for label node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  label	: KBLabel *	: Extant label				*/
/*  (returns)	: KBLabel	:					*/

KBLabel::KBLabel
	(	KBNode		*parent,
		KBLabel		*label
	)
	:
	KBObject	(parent,		label),
	m_text		(this,   "text",	label, KAF_EDITOR),
	m_fgcolor	(this,   "fgcolor",     label),
	m_bgcolor	(this,   "bgcolor",     label),
	m_frame		(this,	 "frame",	label),
	m_font		(this,	 "font",	label),
	m_align		(this,	 "align",	label),
	m_buddy		(this,	 "buddy",	label, KAF_FORM),
	m_onClick	(this,	 "onclick",	label, KAF_EVCS)
{
	m_label	= 0 ;

	if (getParent() != 0) m_report = getParent()->getReport () ;
}

/*  KBLabel								*/
/*  ~KBLabel	: Desructor for label node				*/
/*  (returns)	:		:					*/

KBLabel::~KBLabel ()
{
}

/*  KBLabel								*/
/*  buildDisplay: Build control for this node				*/
/*  display	: KBDisplay *	: Owning display			*/
/*  (returns)	: void		:					*/

void	KBLabel::buildDisplay
	(	KBDisplay	*display
	)
{
	KBObject::buildDisplay (display) ;

	if (m_label == 0)
	{
		m_label	= new KBCtrlLabel (display, this) ;
		setControl (m_label)  ;
	}

	setupProperties	() ;
}

/*  KBLabel								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBLabel::showAs
	(	KB::ShowAs	mode
	)
{
	KBObject::showAs (mode) ;
	setText	(m_text.getValue()) ;
}


/*  KBLabel								*/
/*  setPalette	: Set palette						*/
/*  (returns)	: void		:					*/

void	KBLabel::setPalette ()
{
	DELOBJ	(m_curPal) ;
	if (m_label != 0)
		m_label->KBControl::setPalette (getPalette (true)) ;
}

/*  KBLabel								*/
/*  replicate	: Replicate this label					*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New label node			*/

KBNode	*KBLabel::replicate
	(	KBNode		*parent
	)
{
	return	new KBLabel (parent, this) ;
}


/*  KBLabel								*/
/*  setText	: Set label text					*/
/*  text	: const QString & : New label text			*/
/*  (returns)	: void		  :					*/

void	KBLabel::setText
	(	const QString	&text
	)
{
	m_label->setValue (KBValue(text)) ;
}

/*  KBLabel								*/
/*  getText	: Get current label text				*/
/*  (returns)	: QString	: Current text				*/

QString	KBLabel::getText ()
{
	return	m_label->getValue().getRawText() ;
}

/*  KBLabel								*/
/*  getAlign	: Get alignment setting					*/
/*  (returns)	: int		: Current setting			*/

int	KBLabel::getAlign  ()
{
	return	m_align.getIntValue() ;
}

/*  KBLabel								*/
/*  enumKBProperty							*/
/*		: Get list of properties				*/
/*  list	: QStringList &	: Property list				*/
/*  (returns)	: void		:					*/

void	KBLabel::enumKBProperty
	(	QStringList	&list
	)
{
	list.append ("text") ;
	KBObject::enumKBProperty (list) ;
}

/*  KBLabel								*/
/*  hasKBProperty: Check if object has this property			*/
/*  name	 : cchar *	: Property name				*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBLabel::hasKBProperty
	(	cchar		*name
	)
{
	if (qstrcmp (name, "text") == 0)
		return	true ;

	return	KBObject::hasKBProperty (name) ;
}

/*  KBLabel								*/
/*  getKBProperty: Get a property rekall value				*/
/*  name	 : cchar *	: Proprty name				*/
/*  value	 : KBValue &	: Return property value			*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBLabel::getKBProperty
	(	cchar		*name,
		KBValue		&value
	)
{
	if (qstrcmp (name, "text") == 0)
	{
		value	= getText() ;
		return	true ;
	}

	return	KBObject::getKBProperty (name, value) ;
}

/*  KBLabel								*/
/*  setKBProperty: Set a rekall property value				*/
/*  name	 : cchar *	: Proprty name				*/
/*  value	 : KBValue	: Property value			*/
/*  (returns)	 : bool		: Success				*/

bool	KBLabel::setKBProperty
	(	cchar		*name,
		const KBValue	&value
	)
{
	if (qstrcmp (name, "text") == 0)
	{
		setText	(value.getRawText()) ;
		return	true	;
	}

	return	KBObject::setKBProperty (name, value) ;
}

/*  KBLabel								*/
/*  clicked	: User clicked in label					*/
/*  (returns)	: void		:					*/

void	KBLabel::clicked ()
{
	bool	evRc ;
	eventHook (m_onClick, 0, 0, evRc) ;
}

/*  KBLabel								*/
/*  recordVerifyState							*/
/*		: Record state for verification				*/
/*  (returns)	: void		:					*/

void	KBLabel::recordVerifyText ()
{
#if	! __KB_RUNTIME
	if (m_label != 0)
	{
		KBRecorder *recorder = KBRecorder::self() ;
		if ((recorder != 0) && recorder->isRecording(getDocRoot()))
			recorder->verifyText
			(	this,
				0,
				m_label->getValue().getRawText()
			)	;
	}
#endif
}

#if	! __KB_RUNTIME

/*  KBLabel								*/
/*  makeRecordPopup							*/
/*		: Construct recording popup menu			*/
/*  popup	: KBPopupMenu *	: Popup menu to add to			*/
/*  drow	: uint		: Display row number			*/
/*  parents	: bool		: Add submenus for parents		*/
/*  (returns)	: void		:					*/

void	KBLabel::makeRecordPopup
	(	KBPopupMenu		*popup,
		uint			drow,
		bool			parents
	)
{
	popup->insertItem
	(	TR("Verify text"),
		this,
		SLOT(recordVerifyText())
	)	;

	KBObject::makeRecordPopup (popup, drow, parents) ;
}

/*  KBLabel								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar *	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBLabel::propertyDlg
	(	cchar	*iniAttr
	)
{
	if (labelPropDlg (this, "Label", m_attribs, iniAttr))
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}
#endif

NEWNODE(Label, __TR("New &Label"), 0, KNF_FORM|KNF_REPORT|KNF_BLOCK|KNF_STATIC)
