/***************************************************************************
    file	         : kb_layout.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_LAYOUT_H
#define	_KB_LAYOUT_H

#include	"libkbase_exports.h"

#include	<qptrlist.h>
#include	<qstringlist.h>

#include	"kb_location.h"
#include	"kb_sizer.h"

/*  KBLayout								*/
/*  --------								*/
/*  This class is used at the top-most node in parsed documents, such	*/
/*  as forms and reports. It coordinates stuff like control positioning	*/
/*  and cut-and-paste.							*/

class	LIBKBASE_API	KBLayout
{
	KBNode		*m_parent	;	/* Document root	*/

#if	! __KB_RUNTIME
	QPtrList<KBSizer>
			m_sizerList	;	/* Active sizers	*/
#endif

	bool		m_dData		;	/* Data changed		*/
	bool		m_dDesign	;	/* Design changed	*/
	bool		m_design	;	/* Design mode		*/
	KBaseGUI	*m_currGUI  	;	/* Currently active GUI	*/

//	KBItem		*m_unMorphedItem;
//	uint		m_unMorphedDRow	;
	QStringList	m_changeList	;

	void	setGUIEnables	()	;

public	:

	inline	KBLayout
		(	KBNode	*parent
		)
		:
		m_parent	(parent)
	{
		m_dData			= false ;
		m_dDesign		= false	;
		m_design		= false	;
		m_currGUI		= 0	;
//		m_unMorphedItem		= 0	;
//		m_unMorphedDRow		= 0	;
	}

#if	! __KB_RUNTIME
	/* Methods directly related to currently selected sizers. We	*/
	/* keep track of those selected, with the head of the list	*/
	/* being the active sizer.					*/
	void		initSizer       ()			 ;
	void		releaseSizer    ()			 ;
	void		dropSizer       (KBSizer *)		 ;
	KBLimit		addSizer	(KBSizer *, bool = true) ;

	/* The layout class is passed control movements when tracking,	*/
	/* so that these can be passed on to all selected sizers.	*/
	void		trackMove       (int, int,  bool)	 ;

	QRect		autoCtrlRect     (KBObject *, Qt::ButtonState, QRect) ;

	void		doCtrlAlign     (KB::CtrlAlign)		 ;
	void		snapToGrid	() ;

	void		doCut		() ;
	void		doCopy		() ;
	void		doSaveComponent	(KBLocation  &) ;
	void		doPaste		() ;
	void		doMultiProp	() ;
	void		doSingleProp	() ;
	void		clear		(bool)		;
#endif
	void		setChanged	(bool = true, const QString & = QString::null)	;
	cchar		*getChanged	(bool, QStringList &)	;

	inline	void	setGUI		(KBaseGUI *g)
	{
		m_currGUI = g ;
	}

//	/* Similarly keep track of a currently unmorphed control, if	*/
//	/* any, so that we will have at most one visible at a time if	*/
//	/* any controls are morphing.					*/
//	void		setUnMorphedItem(KBItem *, uint)	 ;
}	;

#endif	// _KB_LAYOUT_H
