/***************************************************************************
    file	         : kb_layoutitem.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_LAYOUTITEM_H
#define		__KB_LAYOUTITEM_H

#include	"libkbase_exports.h"
#include	"kb_attrgeom.h"
#include	"kb_item.h"

#include	<qlayout.h>

class	KBDisplay	;
class	KBAttrGeom	;
class	KBBlock		;
class	KBCtrlValidator	;

/*  KBLayoutItem							*/
/*  ------------							*/
/*  Derived class for widget layout. Extends the base class so that	*/
/*  the raw design geometry can be retrieved, and the geometry can be	*/
/*  mapped for stretch and float setting.				*/

class	LIBKBASE_API	KBLayoutItem : public QWidgetItem
{
protected :

	KBDisplay		*m_display	;
	const KBAttrGeom	&m_geom		;
	KBAttrGeom::MgmtMode	m_mgmtMode	;
	int			m_rtti		;

	QRect			adjustGeometry	(const QRect &)	;

public	:

 	QRect			designGeometry	()	;
	int			rtti		()	;
	void			setRtti		(int)	;

	virtual QRect		mapGeometry	(const QRect &)	;
	virtual	void		setGeometry	(const QRect &)	;

	enum	RTTI
	{	RTTIWidget,
		RTTICtrl,
		RTTIGrid
	}	;

	KBLayoutItem
	(	QWidget			*,
		KBDisplay		*,
		const KBAttrGeom	&,
		KBAttrGeom::MgmtMode
	)	;
	virtual	~KBLayoutItem	()	;
}	;


/*  ------------------------------------------------------------------	*/

/*  KBCtrlLayoutItem							*/
/*  ----------------							*/
/*  Layout class for controls. Handles validation flags and helper	*/
/*  buttons.								*/

class	LIBKBASE_API	KBCtrlLayoutItem : public KBLayoutItem
{
	uint			m_drow		;
	KBBlock			*m_block	;

	QWidget			*m_helper	;
	KBCtrlValidator		*m_validator	;

	QRect			m_ctrlRect	;
	QRect			m_rect		;
	KBItem::ValFlag		m_mode		;
	QString			m_graphicGood	;
	QString			m_graphicBad	;
	bool			m_isValid	;

	bool			m_showHelper	;

public	:

	virtual QRect		mapGeometry	(const QRect &)	;

	void			setHelper	(QWidget *	) ;
	void			setValidatorMode(KBItem  *	) ;
	void			dropValidator	()		;

	void			showHelper	(bool)		;
	void			showValidator	(bool)		;
	void			setValid	(bool)		;
	bool			isValidating	()		;
	void			setCtrlGeometry	(const QRect &)	;

	virtual	void		setGeometry	(const QRect &)	;

	KBCtrlLayoutItem
	(	QWidget			*,
		KBDisplay 		*,
		const KBAttrGeom	&,
		KBAttrGeom::MgmtMode	,
		uint			,
		KBBlock			*
	)	;
	virtual ~KBCtrlLayoutItem	() ;
}	;

#endif		// __KB_LAYOUTITEM_H
