/***************************************************************************
    file	         : kb_link.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qpopupmenu.h>
#include	<qcursor.h>

#include	"kb_formblock.h"
#include	"kb_select.h"

#if		__KB_AUTOMOC
#include	"kb_link.moc"
#else
#include	"kb_link.h"
#endif

#include	"kb_qtlink.h"
#include	"kb_qtreplink.h"
#include	"kb_nodereg.h"
#include	"kb_popupmenu.h"
#include	"kb_qrytable.h"
#include	"kb_qryquery.h"
#include	"kb_qrysql.h"

#if		! __KB_RUNTIME
#include	"kb_recorder.h"
#endif

/*  KBLink								*/
/*  KBLink	: Constructor for link control				*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBNode	:					*/

KBLink::KBLink
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBLinkTree	(parent, aList, 	"KBLink"),
	m_showCols	(this,	 "showcols",	aList,	  KAF_FORM),
	m_morph		(this,   "morph",	aList,    KAF_FORM)
{
#if	! __KB_RUNTIME
	if (ok != 0)
	{
		QString	*lt = aList.find ("linktype") ;

		if (lt != 0)
			if	(*lt == "query") m_query = new KBQryQuery (this) ;
			else if (*lt == "sql"  ) m_query = new KBQrySQL   (this) ;

		if (m_query == 0)
			m_query	= new KBQryTable (this) ;

		if (!m_query->propertyDlg ())
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		if (!::linkPropDlg (this, "Link", m_attribs))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	 ;
	}
#endif
}

/*  KBLink								*/
/*  KBLink	: Constructor for link control				*/
/*  parent	: KBNode *	: Parent node				*/
/*  link	: KBLink *	: Extant link				*/
/*  (returns)	: KBNode	:					*/

KBLink::KBLink
	(	KBNode		*parent,
		KBLink		*link
	)
	:
	KBLinkTree	(parent,  link),
	m_showCols	(this,	  "showcols",	link,	KAF_FORM),
	m_morph		(this,    "morph",	link,	KAF_FORM)
{
}

/*  KBLink								*/
/*  ~KBLink	: Destructor for simple form field			*/
/*  (returns)	:		:					*/

KBLink::~KBLink ()
{
}

void	KBLink::loadControl
	(	uint			drow,
		const QStringList	&keyset,
		const KBLTValueSet	&valset
	)
{
	if (getRoot()->isForm() != 0)
		if (m_ctrls.at(drow) != 0)
			((KBCtrlLink *)m_ctrls.at(drow))->loadControl (keyset, valset) ;
}

/*  KBLink								*/
/*  isMorphing	: Test if control is morphed				*/
/*  (returns)	: bool		: Morphed				*/

bool	KBLink::isMorphing ()
{
	return	m_morph.getBoolValue() ;
}

/*  KBLink								*/
/*  makeCtrl	: Make a link control					*/
/*  drow	: uint		: Display row number			*/
/*  (returns)	: KBControl *	: Associated control			*/

KBControl
	*KBLink::makeCtrl
	(	uint		drow
	)
{
	if (getRoot()->isReport() != 0)
		return	new KBCtrlRepLink (getDisplay(), this, drow) ;

	return	new KBCtrlLink (getDisplay(), this, drow) ;
}

/*  KBLink								*/
/*  replicate	: Replicate this link					*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New link node				*/

KBNode	*KBLink::replicate
	(	KBNode	*parent
	)
{
	KBLink	*nLink	= new KBLink (parent, this) ;

	CITER
	(	QryBase,
		q,

		nLink->m_query = q->replicate(nLink)->isQryBase() ;
		break	;
	)	;


	return	nLink	;
}

/*  KBLink							*/
/*  setUserFilter						*/
/*		: Set user filter for link query		*/
/*  qrow	: uint		  : Applicable query row	*/
/*  filter	: const QString & : Filter expression		*/
/*  (returns)	: void		  :				*/

void	KBLink::setUserFilter
	(	uint		qrow,
		const QString	&filter
	)
{
	if (getRoot()->isForm() != 0)
	{
		KBControl *ctrl = ctrlAtQRow  (qrow)   ;
		if (ctrl != 0) ((KBCtrlLink *)ctrl)->setUserFilter (filter ) ;
	}
}

/*  KBLink							*/
/*  setUserSorting						*/
/*		: Set user sorting for link query		*/
/*  qrow	: uint		  : Applicable query row	*/
/*  sorting	: const QString & : Sorting expression		*/
/*  (returns)	: void		  :				*/

void	KBLink::setUserSorting
	(	uint		qrow,
		const QString	&sorting
	)
{
	if (getRoot()->isForm() != 0)
	{
		KBControl *ctrl = ctrlAtQRow  (qrow)   ;
		if (ctrl != 0) ((KBCtrlLink *)ctrl)->setUserSorting(sorting) ;
	}
}

/*  KBLink							*/
/*  reload	: Reload values list				*/
/*  qrow	: uint		  : Applicable query row	*/
/*  (returns)	: void		  :				*/

void	KBLink::reload
	(	uint		qrow
	)
{
	if (getRoot()->isForm() != 0)
	{
		KBControl *ctrl = ctrlAtQRow  (qrow)   ;
		if (ctrl != 0) ((KBCtrlLink *)ctrl)->reload() ;
	}
}

/*  KBLink							*/
/*  getExtra	: Get extra value				*/
/*  qrow	: uint		: Query row number		*/
/*  which	: bool		: Display or extra		*/
/*  index	: uint		: Extra value index		*/
/*  (returns)	: KBValue	: Value				*/

KBValue	KBLink::getExtra
	(	uint		qrow,
		bool		which,
		uint		index
	)
{
	if (getRoot()->isForm() != 0)
	{
		KBControl *ctrl = ctrlAtQRow  (qrow) ;
		return	ctrl == 0 ? KBValue() : ((KBCtrlLink *)ctrl)->getExtra (which, index) ;
	}

	return	KBValue	() ;
}

/*  KBLink							*/
/*  getNumValues: Get number of values showing at row		*/
/*  qrow	: uint		: Query row number		*/
/*  (returns)	: uint		: Number of values		*/

uint	KBLink::getNumValues
	(	uint		qrow
	)
{
	if (getRoot()->isForm() != 0)
	{
		KBControl *ctrl = ctrlAtQRow  (qrow) ;
		uint	  nv	= 0 ;

		if (ctrl != 0)
			if (((KBCtrlLink *)ctrl)->getNumValues (nv))
				return	nv ;

		return	m_valset.count() ;
	}

	return	m_valset.count() ;
}

/*  KBLink							*/
/*  getDisplayList						*/
/*		: Get list of displayed text values at row	*/
/*  qrow	: uint		: Query row number		*/
/*  (returns)	: QStringList	: List of text values		*/

QStringList
	KBLink::getDisplayList
	(	uint		qrow
	)
{
	QStringList	results	;

	if (getRoot()->isForm() != 0)
	{
		KBControl *ctrl = ctrlAtQRow  (qrow) ;

		if (ctrl != 0)
			if (((KBCtrlLink *)ctrl)->getDisplayList (results))
				return	results ;
	}

	for (uint idx = 0 ; idx < m_valset.count() ; idx += 1)
		results.append (m_valset[idx][0]) ;

	return	results	;
}

bool	KBLink::setCurrentDisplay
	(	uint		qrow,
		const QString	&text
	)
{
	KBControl *ctrl = ctrlAtQRow  (qrow) ;
	return	ctrl == 0 ? false : ((KBCtrlLink *)ctrl)->setCurrentDisplay (text) ;
}

/*  KBLink								*/
/*  setCurrentItem							*/
/*		: Set currently selected item by index			*/
/*  qrow	: uint		: Query row				*/
/*  item	: uint		: Item index				*/
/*  (returns)	: void		:					*/

void	KBLink::setCurrentItem
	(	uint		qrow,
		uint		item
	)
{
	if (getRoot()->isForm() != 0)
	{
		KBControl *ctrl = ctrlAtQRow (qrow) ;
		if (ctrl != 0) ((KBCtrlLink *)ctrl)->setCurrentItem (item) ;
	}
}

/*  KBLink								*/
/*  currentItem	: Get currently selected item index			*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: uint		: Item index				*/

uint	KBLink::currentItem
	(	uint		qrow
	)
{
	if (getRoot()->isForm() != 0)
	{
		KBControl *ctrl = ctrlAtQRow (qrow) ;
		return	ctrl == 0 ? 0 : ((KBCtrlLink *)ctrl)->currentItem () ;
	}

	return	0 ;
}

/*  KBLink								*/
/*  recordVerifyChoices							*/
/*  		: Record choices verification for test			*/
/*  (returns)	: void		:					*/

void	KBLink::recordVerifyChoices ()
{
#if	! __KB_RUNTIME
	KBRecorder *recorder = KBRecorder::self() ;
	if ((recorder == 0) || !recorder->isRecording(getDocRoot()))
		return	;

	KBCtrlLink	*ctrl	= (KBCtrlLink   *)m_ctrls.at(m_recordDRow) ;
	RKComboBox	*combo	= (RKComboBox   *)ctrl->getWidget() ;

	QStringList	values	;
	for (int idx = 0 ; idx < combo->count() ; idx += 1)
		values.append (combo->text(idx)) ;

	recorder->verifyChoices
	(	this,
		m_recordDRow,
		values.join("|")
	)	;
#endif
}

#if	! __KB_RUNTIME

/*  KBLink								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar	*	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBLink::propertyDlg
	(	cchar		*iniAttr
	)
{
	if (::linkPropDlg (this, "Link", m_attribs, iniAttr))
	{
		setupProperties () ;
		addDummyItems   () ;
		return	true ;
	}

	return	false	;
}


/*  KBLink								*/
/*  makeRecordPopup							*/
/*		: Construct recording popup menu			*/
/*  popup	: KBPopupMenu *	: Popup menu to add to			*/
/*  drow	: uint		: Display row number			*/
/*  parents	: bool		: Add submenus for parents		*/
/*  (returns)	: void		:					*/

void	KBLink::makeRecordPopup
	(	KBPopupMenu		*popup,
		uint			drow,
		bool			parents
	)
{
	popup->insertItem
	(	TR("Verify choices"),
		this,
		SLOT(recordVerifyChoices())
	)	;
	KBItem::makeRecordPopup (popup, drow, parents) ;
}
#endif

static	KBNode	*newLinkTable
	(	KBNode			*parent,
		const QDict<QString>	&aDict,
		bool			*ok
	)
{
	QDict<QString>	d2 (aDict  ) ;
	QString		lt ("table") ;

	d2.insert ("linktype", &lt)   ;
	return	new KBLink (parent, d2, ok) ;
}

static	KBNode	*newLinkQuery
	(	KBNode			*parent,
		const QDict<QString>	&aDict,
		bool			*ok
	)
{
	QDict<QString>	d2 (aDict  ) ;
	QString		lt ("query") ;

	d2.insert ("linktype", &lt)   ;
	return	new KBLink (parent, d2, ok) ;
}

static	KBNode	*newLinkSQL
	(	KBNode			*parent,
		const QDict<QString>	&aDict,
		bool			*ok
	)
{
	QDict<QString>	d2 (aDict ) ;
	QString		lt ("sql" ) ;

	d2.insert ("linktype", &lt)   ;
	return	new KBLink (parent, d2, ok) ;
}



/*  ------------------------------------------------------------------  */

/*  The following code is used to generate the "New Link" popup menu,	*/
/*  which allows the insertion of links based on Queries (both Rekall	*/
/*  and SQL) as well as tables.						*/


static	KBNode	*makeTableLinkFromWizard
	(	KBNode		*parent,
		KBQryBase	*query,
		KBAttrDict	&aDict,
		bool		&cancel
	)
{
#if	! __KB_RUNTIME
	extern	KBNode	*makeCtrlFromWizard
		(	KBNode		*parent,
			KBQryBase	*query,
			cchar		*wizName,
			KBAttrDict	&aDict,
			bool		&cancel
		)	;

	return	makeCtrlFromWizard
		(	parent,
			query,
			"KBLinkTable",
			aDict,
			cancel
		)	;
#else
	return	0	;
#endif
}

static	KBNode	*makeQueryLinkFromWizard
	(	KBNode		*parent,
		KBQryBase	*query,
		KBAttrDict	&aDict,
		bool		&cancel
	)
{
#if	! __KB_RUNTIME
	extern	KBNode	*makeCtrlFromWizard
		(	KBNode		*parent,
			KBQryBase	*query,
			cchar		*wizName,
			KBAttrDict	&aDict,
			bool		&cancel
		)	;

	return	makeCtrlFromWizard
		(	parent,
			query,
			"KBLinkQuery",
			aDict,
			cancel
		)	;
#else
	return	0	;
#endif
}


static	NodeSpec nodeSpecTable =
{	
	0,
	QString::null,
	0,
	newLinkTable,
	makeTableLinkFromWizard,
	KNF_FORM|KNF_REPORT|KNF_BLOCK|KNF_DATA,
	-1
}	;

static	NodeSpec nodeSpecQuery =
{	
	0,
	QString::null,
	0,
	newLinkQuery,
	makeQueryLinkFromWizard,
	KNF_FORM|KNF_REPORT|KNF_BLOCK|KNF_DATA,
	-1
}	;

static	NodeSpec nodeSpecSQL   =
{	
	0,
	QString::null,
	0,
	newLinkSQL,
	0,
	KNF_FORM|KNF_REPORT|KNF_BLOCK|KNF_DATA,
	-1
}	;


/*  makeLinkPopup: Make new link popup menu				*/
/*  popup	 : QWidget *		: Parent widget			*/
/*  receiver	 : QObject *		: Reciever for menu signals	*/
/*  bState	 : Qt::ButtonState &	: Return button state		*/
/*  dflt	 : NodeSpec **		: Return default case		*/
/*  (returns)	 : KBPopupMenu *	: New popup menu		*/

KBPopupMenu
	*KBLink::makeLinkPopup
	(	QWidget		*parent,
		QObject		*receiver,
		Qt::ButtonState	*bState,
		NodeSpec	**dflt
	)
{
	KBPopupMenu *lp = new KBPopupMenu (parent, bState) ;

	fprintf
	(	stderr,
		"KBLink::makeLinkPopup: called: %p/%p/%p\n",
		&nodeSpecTable,
		&nodeSpecQuery,
		&nodeSpecSQL
	)	;

	lp->insertItem
	(	TR("Table link"),
		receiver,
		SLOT(newNode(int)),
		QKeySequence(),
		nodeSpecToId(&nodeSpecTable)
	)	;
	lp->insertItem
	(	TR("Query link"),
		receiver,
		SLOT(newNode(int)),
		QKeySequence(),
		nodeSpecToId(&nodeSpecQuery)
	)	;
	lp->insertItem
	(	TR("SQL link"  ),
		receiver,
		SLOT(newNode(int)),
		QKeySequence(),
		nodeSpecToId(&nodeSpecSQL)
	) 	;

	*dflt	= &nodeSpecTable ;

	fprintf
	(	stderr,
		"KBLink::makeLinkPopup: done\n"
	)	;
	return	lp	;
}

NEWNODE(Link, __TR("New &Link"), makeLinkPopup, KNF_FORM|KNF_REPORT|KNF_BLOCK|KNF_DATA)
