/***************************************************************************
    file	         : kb_linktree.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_formblock.h"
#include	"kb_select.h"
#include	"kb_ctrl.h"
#include	"kb_popupmenu.h"
#include	"kb_designpopup.h"

#if		__KB_AUTOMOC
#include	"kb_linktree.moc"
#else
#include	"kb_linktree.h"
#endif

#include	"kb_qrytable.h"
#include	"kb_qryquery.h"
#include	"kb_qrysql.h"
#include	"kb_queryset.h"
#include	"kb_layout.h"
#include	"kb_finddlg.h"

#include	"tk_icons.h"

#define		__DEBUG_LINKTREE	(0)
#if		__DEBUG_LINKTREE
#include	"kb_dprintf.h"
#endif


KBLinkTreeDummy::KBLinkTreeDummy
	(	KBObject 	*parent,
		const QString	&eName,
		const QString	show
	)
	:
	KBItem	(parent, "KBLinkTreeDummy", QRect(), eName, show, 0)
{
}

KBLinkTreeDummy::~KBLinkTreeDummy ()
{
}

void	KBLinkTreeDummy::buildGUI
	(	uint	,
		int	,
		int
	)
{
	/* No GUI to build.						*/
}

void	KBLinkTreeDummy::showAs
	(	KB::ShowAs
	)
{
	/* Switching in and out of design mode should not do anything.	*/
}

void	KBLinkTreeDummy::printNode
	(	QString		&,
		int		,
		bool
	)
{
	/* Dont want to print anything for this node, it is always	*/
	/* built on the fly.						*/
}

bool	KBLinkTreeDummy::isUpdateVal
	(	bool
	)
{
	/* Don't get a value from here, is should come from the read	*/
	/* link node.							*/
	return	false	;
}


/*  ------------------------------------------------------------------	*/

/*  KBLinkTree								*/
/*  KBLinkTree	: Constructor for link control				*/
/*  parent	: KBNode *	  : Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				  : List of attributes			*/
/*  element	: const QString & : Element name			*/
/*  (returns)	: KBNode	  :					*/

KBLinkTree::KBLinkTree
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		const QString		&element
	)
	:
	KBItem		(parent, element,
				 "master", 	aList),
	m_child		(this,   "child",	aList, KAF_REQD),
	m_show		(this,   "show",	aList, KAF_REQD),
	m_fgcolor	(this,   "fgcolor",	aList),
	m_bgcolor	(this,   "bgcolor",	aList),
	m_font		(this,	 "font",	aList),
	m_nullval	(this,	 "nullval",	aList),
	m_nullOK	(this,	 "nullok",	aList, KAF_FORM),
	m_noblank	(this,	 "noblank",	aList, KAF_FORM|KAF_GRPDATA),
	m_dynamic	(this,	 "dynamic",	aList),
	m_onChange	(this,	 "onchange",	aList, KAF_EVCS),
	m_extra		(this,	 "extra",	aList, KAF_FORM),
	m_preload	(this,	 "preload",	aList, KAF_FORM)
{
	m_query	 	= 0	;
	m_iKey	 	= 0	;
	m_loaded 	= false ;
	m_exprCount	= 0	;
	m_extraCount	= 0	;

	m_iShow .setAutoDelete (true) ;
}

/*  KBLinkTree								*/
/*  KBLinkTree	: Constructor for link control				*/
/*  parent	: KBNode *	: Parent node				*/
/*  link	: KBLinkTree *	: Extant link				*/
/*  (returns)	: KBNode	:					*/

KBLinkTree::KBLinkTree
	(	KBNode		*parent,
		KBLinkTree	*link
	)
	:
	KBItem		(parent,  "master",	link),
	m_child		(this,    "child",	link, KAF_REQD),
	m_show		(this,    "show",	link, KAF_REQD),
	m_fgcolor	(this,    "fgcolor",	link),
	m_bgcolor	(this,    "bgcolor",	link),
	m_font		(this,	  "font",	link),
	m_nullval	(this,	  "nullval",	link),
	m_nullOK	(this,	  "nullok",	link),
	m_noblank	(this,	  "noblank",	link, KAF_FORM|KAF_GRPDATA),
	m_dynamic	(this,	  "dynamic",	link),
	m_onChange	(this,	  "onchange",	link, KAF_EVCS),
	m_extra		(this,	  "extra",	link, KAF_FORM),
	m_preload	(this,	  "preload",	link, KAF_FORM)
{
	m_iKey	 	= 0 	;
	m_loaded 	= false	;
	m_query	 	= 0	;
	m_exprCount	= 0	;
	m_extraCount	= 0	;

	m_iShow .setAutoDelete (true) ;
}

/*  KBLinkTree								*/
/*  ~KBLinkTree	: Destructor for simple form field			*/
/*  (returns)	:		:					*/

KBLinkTree::~KBLinkTree ()
{
	DELOBJ	(m_iKey) ;
	m_iShow .clear() ;
}

/*  KBLinkTree								*/
/*  keyStroke	: Control has received keystroke			*/
/*  k		: QKeyEvent *	: Key event in question			*/
/*  (returns)	: bool		: Key event consumed			*/

bool	KBLinkTree::keyStroke
	(	QKeyEvent	*k
	)
{
	/* Override for key up and key down so that we can change	*/
	/* selection, otherwise these keys will navigate between	*/
	/* records.							*/
	if (k->type() == QEvent::KeyPress)
		if ((k->key() == Qt::Key_Up) || (k->key() == Qt::Key_Down))
			return	false	;

	return	KBItem::keyStroke (k) ;
}

/*  KBLinkTree								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBLinkTree::showAs
	(	KB::ShowAs	mode
	)
{
#if	! __KB_RUNTIME
	if (mode == KB::ShowAsDesign)
	{
		m_keyset.clear ()	;
		m_valset.clear ()	;
		remDummyItems  () 	;
		m_loaded = false  	;
	}
#endif
	m_query		= 0		;

	CITER
	(	QryBase,
		q,
		m_query = q
	)	;

	if (m_query == 0)
		KBError::EFault
		(	"Link/Tree control lacks a query",
			QString::null,
			__ERRLOCN
		)	;

	KBItem::showAs (mode) ;
}

/*  KBLinkTree								*/
/*  remDummyItems: Remove dummy items for query				*/
/*  (returns)	 : void		:					*/

void	KBLinkTree::remDummyItems ()
{
	if (m_iKey  != 0)
	{
		if (m_query != 0) m_query->remItem (0, m_iKey ) ;
		DELOBJ	(m_iKey)	;
	}

	if (m_query != 0)
		LITER
		(	KBLinkTreeDummy,
			m_iShow,
			item,
			m_query->remItem (0, item)
		)

	m_iShow .clear () ;
}

/*  KBLinkTree								*/
/*  addExprItems: Add expression item(s) for query			*/
/*  exprs	: QString	: Expressions				*/
/*  (returns)	: uint		: Number of expressions added		*/

uint	KBLinkTree::addExprItems
	(	QString		exprs
	)
{
	/* Strip leading and trailing white space; if the expression(s)	*/
	/* string is then empty, return. This handles the case where	*/
	/* there are no "extra" expressions.				*/
	exprs	= exprs.stripWhiteSpace () ;
	if (exprs.isEmpty()) return 0	   ;

	/* Split the expressions into separate expressions and add a	*/
	/* dummy item for each. We use the select parser to do this;	*/
	/* if it fails to parse then assume that we have a single	*/
	/* expression and hope!						*/
	KBSelect select ;
	if (select.parseExprList (exprs))
	{
		const QValueList<KBSelectExpr> exprList = select.getExprList () ;
		QValueList<KBSelectExpr>::ConstIterator iter ;
		uint idx = 0 ;

		for (iter = exprList.begin() ; iter != exprList.end() ; iter++, idx++)
		{
			KBLinkTreeDummy
				*item = new KBLinkTreeDummy
					(	this,
						QString("__show_%1").arg(idx),
						(*iter).exprText(0)
					)	;

			m_iShow.append   (item)	   ;
			m_query->addItem (0, item) ;

			DPRINTF
			((	"KBLinkTree::addDummyItems: add %d [%s]\n",
				idx,
				(cchar *)(*iter).exprText(0)
			))	;
		}

		return	exprList.count() ;
	}

	KBLinkTreeDummy
		*item = new KBLinkTreeDummy
			(	this,
				"__show_0",
				m_show.getValue().stripWhiteSpace()
			)	;

	m_iShow.append   (item)	   ;
	m_query->addItem (0, item) ;
	return	1 ;
}

/*  KBLinkTree								*/
/*  addDummyItems: Add dummy items for query				*/
/*  (returns)	 : uint		: Number of items added excluding key	*/

uint	KBLinkTree::addDummyItems ()
{
	/* The link works by adding two items, one to contain the key	*/
	/* from the query (ie., the child value), and the other for	*/
	/* the values from the query (ie., the displayed expressions).	*/

	/* If any dummy items already exist then remove them. This is	*/
	/* needed since this routine is called after the link		*/
	/* properties have been changed.				*/
	remDummyItems	  	() ;

	m_query->addItem	(0, 0) ;
	m_query->addItem	(0, m_iKey = new KBLinkTreeDummy (this, "_key",  m_child.getValue())) ;

	m_exprCount  = addExprItems (m_show .getValue()) ;
	m_extraCount = addExprItems (m_extra.getValue()) ;

	return	m_exprCount + m_extraCount ;
}

/*  KBLinkTree								*/
/*  checkValid	: Check whether value can be saved in database		*/
/*  value	: const KBValue & : Value				*/
/*  alowNull	: bool		  : Ignore not-null check		*/
/*  (returns)	: bool		  : True if OK				*/

bool	KBLinkTree::checkValid
	(	const KBValue	&value,
		bool		allowNull
	)
{
	DPRINTF
	((	"KBLinkTree::checkValid: [%s]: an=%d no=%d ve=%d\n",
		value.getRawText().latin1(),
		allowNull,
		m_nullOK.getBoolValue(),
		value.isEmpty()
	))	;

	if (!allowNull)
		if (!m_nullOK.getBoolValue () && value.isEmpty())
		{
			setError
			(	KBError::Error,
				QString(TR("Value must be selected from list for %1")).arg(errorText()),
				QString::null,
				__ERRLOCN
			)	;
			return	false ;
		}

	return	true	;
}

/*  KBLinkTree								*/
/*  filter	: const QString & : User filter expression		*/
/*  sorting	: const QString & : User sorting expression		*/
/*  keyset	: QStringList &	  : Return list of keys			*/
/*  valset	: KBLTValueSet &  : Return list of value lists		*/
/*  (returns)	: void		  :					*/

void	KBLinkTree::loadValues
	(	const QString		&filter,
		const QString		&sorting,
		QStringList		&keyset,
		KBLTValueSet		&valset
	)
{
	DPRINTF
	((	"KBLinkTree::loadValues: f=[%s] s=[%s] nb=%d\n",
		(cchar *)filter,
		(cchar *)sorting,
		m_noblank.getBoolValue()
	))	;

	keyset.clear   () ;
	valset.clear   () ;

	if (!m_noblank.getBoolValue())
	{
		keyset.append  ("")  ;
		valset.append  (QStringList(m_nullval.getValue())) ;
	}

	/* HACK:							*/
	/* In case we get called before everything is set up properly..	*/
	if (m_iKey == 0)
		return	;

	if (! m_query->doSelect
		(	0,
			(KBValue *)0,
			QString::null,
			filter,
			sorting
	   ))
	{
		m_query->lastError().DISPLAY () ;
		return	;
	}

	for (uint row = 0 ; row < m_query->getNumRows(0) ; row += 1)
	{
		QString     key = m_query->getField(0, row, m_iKey->getQueryIdx()).getRawText() ;
		QStringList qsl ;

		LITER
		(	KBLinkTreeDummy,
			m_iShow,
			item,

			/* FIX: Including a null value in a string	*/
			/* list which is then loaded into a combobox	*/
			/* seems to terminate the combo list at that	*/
			/* entry.					*/
			QString	show = m_query->getField(0, row, item->getQueryIdx()).getRawText() ;
			if (show.isNull()) show = "" ;
			qsl.append (show) ;
		)

		keyset.append (key) ;
		valset.append (qsl) ;
	}
}

/*  KBLinkTree								*/
/*  loadValues	: Load keys and values					*/
/*  (returns)	: void		:					*/

void	KBLinkTree::loadValues ()
{
	loadValues (QString::null, QString::null, m_keyset, m_valset) ;
}

/*  KBLinkTree								*/
/*  prepare	: Prepare link prior to display				*/
/*  (returns)	: void		:					*/

void	KBLinkTree::prepare ()
{
	if (!m_loaded)
	{
		addDummyItems	() ;

		if (m_preload.getIntValue() == KBLinkTree::PreloadDefer)
		{
			if (!m_noblank.getBoolValue())
			{	m_keyset.append  ("")  ;
				m_valset.append  (QStringList(m_nullval.getValue())) ;
			}
		}
		else	loadValues	() ;

		m_loaded = true	   ;
	}

	KBItem::prepare() ;

	for (uint idx = 0 ; idx < m_ctrls.count() ; idx += 1)
		loadControl (idx, m_keyset, m_valset) ;
}

/*  KBLinkTree								*/
/*  setupControls							*/
/*		: Set up controls					*/
/*  (returns)	: void		:					*/

void	KBLinkTree::setupControls ()
{
	uint	nCtrls	= m_ctrls.count() ;
	KBItem::setupControls ()   ;

	if (!showingData()) return ;

	for (uint idx = nCtrls ; idx < m_ctrls.count() ; idx += 1)
		loadControl (idx, m_keyset, m_valset) ;
}


/*  KBLinkTree								*/
/*  itemToValue	: Map item number to value				*/
/*  item	: uint		: Item number				*/
/*  keyset	: QStringList *	: Optional keyset			*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBLinkTree::itemToValue
	(	uint			item,
		const QStringList	*keyset
	)
{
	if (keyset == 0)
		keyset = &m_keyset ;

	if ((item == 0) && !m_noblank.getBoolValue())
		return	KBValue (m_type) ;

	return	KBValue	((*keyset)[item], m_type) ;
}

/*  KBLinkTree								*/
/*  valueToItem	: Map value to item number				*/
/*  value	: const KBValue &: Value to map				*/
/*  keyset	: QStringList *	 : Optional keyset			*/
/*  (returns)	: uint		 : Item number				*/

uint	KBLinkTree::valueToItem
	(	const KBValue		&value,
		const QStringList	*keyset
	)
{
	if (keyset == 0) keyset = &m_keyset ;

	int	idx = keyset->findIndex (value.getRawText()) ;

	DPRINTF
	((	"KBLinkTree::valueToItem: [%s] in (%s)[%s] -> %d\n",
		value.getRawText().latin1(),
		keyset == &m_keyset ? "global" : "local",
		keyset->join(" ,").latin1(),
		idx
	))	;

	return	idx < 0 ? 0 : idx ;
}

/*  KBLinkTree								*/
/*  valueToText	: Map value to display text				*/
/*  value	: const KBValue &: Value to map				*/
/*  (returns)	: QString	 : Display text				*/

QString	KBLinkTree::valueToText
	(	const KBValue	&value
	)
{
	int	idx = m_keyset.findIndex (value.getRawText()) ;
	return	idx < 0 ? QString::null :
			  m_valset[idx].join(" ") ;
}

/*  KBLinkTree								*/
/*  getReportValue							*/
/*		: Get value for report writing				*/
/*  first	: bool		: First output flag			*/
/*  prior	: bool		: Return prior value flag		*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBLinkTree::getReportValue
	(	bool		,
		bool
	)
{
	int	slot	;

	if ((slot = m_keyset.findIndex (m_curVal.getRawText())) >= 0)
		return	KBValue (m_valset[slot].join(" ")) ;

	return	KBValue	() ;
}

/*  KBLinkTree								*/
/*  getOrderType: Get ordering type					*/
/*  (returns)	: KB::IType	  : Ordering type			*/

KB::IType KBLinkTree::getOrderType()
{
	return	KB::ITString	;
}

/*  KBLinkTree								*/
/*  getOrderValue							*/
/*		: Get ordering value					*/
/*  value	: const KBValue & : Value				*/
/*  (returns)	: QString	  : Ordering value			*/

QString	KBLinkTree::getOrderValue
	(	const KBValue	&value
	)
{
	return	valueToText (value) ;
}

/*  KBLinkTree								*/
/*  getQuery	: Get associated query					*/
/*  (returns)	: KBQryBase *	: Associated query			*/

KBQryBase *KBLinkTree::getQuery ()
{
	return	m_query	;
}

/*  KBLinkTree								*/
/*  doRefresh	: Possibly refresh selections				*/
/*  drow	: uint		: Display row to refresh		*/
/*  (returns)	: void		:					*/

void	KBLinkTree::doRefresh
	(	uint		drow
	)
{
	loadValues	() ;
	loadControl	(drow, m_keyset, m_valset) ;
}

/*  KBLinkTree								*/
/*  doSearch	: Do record search					*/
/*  (returns)	: void		:					*/

void	KBLinkTree::doSearch ()
{
	QStringList list ;

	for (uint idx = 0 ; idx < m_valset.count() ; idx += 1)
		list.append (m_valset[idx].join(" ")) ;

	KBFindChoiceDlg findDlg (getFormBlock(), this, list, m_keyset) ;
	findDlg.exec () ;
}

/*  KBLinkTree								*/
/*  itemToValue	: Map item number to value				*/
/*  item	: uint		: Item number				*/
/*  which	: bool		: Display/extra				*/
/*  extra	: uint		: Extra index				*/
/*  valset	: const KBLTValueSet *					*/
/*				: Optional value set			*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBLinkTree::itemToExtra
	(	uint			item,
		bool			which,
		uint			extra,
		const KBLTValueSet	*valset
	)
{
	if (valset == 0)
		valset	= &m_valset	;

	DPRINTF
	((	"KBLinkTree::itemToExtra: item=%d which=%d extra=%d: nb=%d\n",
		item,
		which,
		extra,
		m_noblank.getBoolValue()
	))	;

	if ((item == 0) && !m_noblank.getBoolValue())
		return KBValue() ;

	if (which)
		if (extra < m_exprCount)
		{
			DPRINTF
			((	"KBLinkTree::itemToExtra: which=1: [%s]\n",
				(*valset)[item][extra].latin1()
			))	;

			return	(*valset)[item][extra] ;
		}
		else	return	KBValue() ;

	if (extra < m_extraCount)
	{
		DPRINTF
		((	"KBLinkTree::itemToExtra: which=0: [%s]\n",
			(*valset)[item][m_exprCount + extra].latin1()
		))	;

		return	(*valset)[item][m_exprCount + extra] ;
	}

	return KBValue() ;
}

KBValue	KBLinkTree::getRowExtra
	(	uint			qrow,
		bool			which,
		uint			index
	)
{
	KBValue	value	= KBItem::getRowValue (qrow) ;
	int	item	= m_keyset.findIndex  (value.getRawText()) ;

	DPRINTF
	((	"KBLinkTree::getRowExtra: qr=%d w=%d idx=%d: [%s]->[%d]->[%s]\n",
		qrow,
		which,
		index,
		value.getRawText().latin1(),
		item,
		item < 0 ?
			"NULL" :
			itemToExtra (item, which, index, 0).getRawText().latin1()
	))	;

	if (item < 0)
		return	KBValue () ;

	return	itemToExtra (item, which, index, 0) ;
}

/*  KBLinkTree								*/
/*  canBeInvalid: Check if control can be invalid			*/
/*  (returns)	: bool		: True if so				*/

bool	KBLinkTree::canBeInvalid ()
{
	return	! m_nullOK.getBoolValue() ;
}

/*  KBLinkTree								*/
/*  showQuery	: Show associated query					*/
/*  (returns)	: void		:					*/

void	KBLinkTree::showQuery ()
{
#if	! __KB_RUNTIME
	if (m_query != 0)
	{
		addDummyItems  () ;

		QString	sql	= m_query->getSQLText   (0, true) ;
		KBQryDisplay(sql, QString::null).exec() ;

		remDummyItems  () ;
	}
#endif
}

bool	KBLinkTree::dynamic	()
{
	return	m_dynamic.getBoolValue	() ;
}

bool	KBLinkTree::noblank	()
{
	return	m_noblank.getBoolValue	() ;
}

uint	KBLinkTree::exprCount	()
{
	return	m_exprCount	;
}

uint	KBLinkTree::extraCount	()
{
	return	m_extraCount	;
}


#if	! __KB_RUNTIME

/*  KBLinkTree								*/
/*  whatsThisExtra							*/
/*		: Extra stuff for pseudo-what's this popup		*/
/*  info	: QValueList<KBWhatsThisPair> &	: List of items		*/
/*  (returns)	: void				:			*/

void	KBLinkTree::whatsThisExtra
	(	QValueList<KBWhatsThisPair>	&info
	)
{
	KBItem::whatsThisExtra (info) ;

	if (m_query != 0) m_query->whatsThisExtra (info) ;

	info.append (KBWhatsThisPair(TR("Link"),   m_child.getValue())) ;
	info.append (KBWhatsThisPair(TR("Display"),m_show .getValue())) ;
}


/*  KBLinkTree								*/
/*  designPopup	: Display design menu					*/
/*  parent	: KBPopupMenu *	: Possible parent menu			*/
/*  cell	: QRect		: Cell for dynamic layouts		*/
/*  (returns)	: KBPopupMenu *	: Popup menu				*/

KBPopupMenu
	*KBLinkTree::designPopup
	(	KBPopupMenu	*parent,
		QRect		cell
	)
{
	KBPopupMenu *popup = KBObject::designPopup (parent, cell) ;
	popup->insertItem
	(	getSmallIcon("querylog"),
		TR("&Show query"),
		this,
		SLOT(showQuery())
	)	;
	return	popup	;
}
#endif
