/***************************************************************************
    file	         : kb_linktree.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_LINKTREE_H
#define		__KB_LINKTREE_H

#include	"libkbase_exports.h"

#include	<qdict.h>
#include	<qstringlist.h>
#include	<qvaluelist.h>
#include	<qptrlist.h>

#include	"kb_classes.h"
#include	"kb_item.h"

class		KBLink		;
class		KBTree		;
typedef		QValueList<const QStringList>	KBLTValueSet	;


/*  KBLinkTreeDummy							*/
/*  ---------------							*/
/*  The KBLinkTree class works by creating some dummy items. These are	*/
/*  of class KBLinkTreeDummy, which overrides the various methods ....	*/

class	KBLinkTreeDummy : public KBItem
{
public	:

	KBLinkTreeDummy	(KBObject *, const QString &, const QString $) ;
	virtual	~KBLinkTreeDummy () ;

	virtual	void	buildGUI	(uint, int, int	)	;
	virtual	void	showAs		(KB::ShowAs	)	;
	virtual	void	printNode	(QString &, int, bool)	;
	virtual	bool	isUpdateVal	(bool = false)		;
}	;



/*  KBLinkTree								*/
/*  ----------								*/
/*  This class is a base class for Link and Tree controls, which have	*/
/*  most of their functionality in common.				*/

class	LIBKBASE_API	KBLinkTree : public KBItem
{
	Q_OBJECT

	KBAttrStr		m_child		;
	KBAttrStr		m_show		;
	KBAttrStr		m_fgcolor	;
	KBAttrStr		m_bgcolor	;
	KBAttrStr		m_font		;
	KBAttrStr		m_nullval	;
	KBAttrBool		m_nullOK	;
	KBAttrBool		m_noblank	;
	KBAttrBool		m_dynamic	;
	KBEvent			m_onChange	;
	KBAttrStr		m_extra		;
	KBAttrInt		m_preload	;


	KBLinkTreeDummy		*m_iKey		;
	QPtrList<KBLinkTreeDummy>
				m_iShow		;
	bool			m_loaded	;

	uint			m_exprCount	;
	uint			m_extraCount	;

protected	:

	QStringList		m_keyset	;
	KBLTValueSet		m_valset	;

	uint			addExprItems	(QString) ;
	virtual	uint		addDummyItems 	() ;
	virtual	void		remDummyItems	() ;
	void			loadValues	() ;
	virtual	void		loadControl	(uint, const QStringList &, const KBLTValueSet &) = 0 ;

	KBQryBase 	*m_query	;


protected slots	:

	void		showQuery	() ;

public	:


	virtual	void		prepare		()			;
	virtual void		setupControls	()			;
	virtual	KBValue		getReportValue	(bool, bool = false)	;

#if	! __KB_RUNTIME
	virtual	void		whatsThisExtra	(QValueList<KBWhatsThisPair> &) ;
	virtual	KBPopupMenu	*designPopup	(KBPopupMenu *, QRect)	;
#endif
	virtual	QString		getOrderValue	(const KBValue &)	;
	virtual	KB::IType	getOrderType	() ;

	virtual	void		showAs		(KB::ShowAs)	;
	virtual	bool		keyStroke	(QKeyEvent   *) ;

	KBQryBase		*getQuery	() ;

	virtual bool		checkValid	(const KBValue &, bool)	;
	KBValue			itemToValue	(uint, 		  const QStringList *) ;
	uint			valueToItem	(const KBValue &, const QStringList *) ;
	QString			valueToText	(const KBValue &) ;
	void			doRefresh	(uint) ;

	void			loadValues
				(	const QString			&,
					const QString			&,
					QStringList			&,
					KBLTValueSet			&
				)	;

	virtual	void		doSearch	() ;
	virtual	bool		canBeInvalid	() ;

	virtual void		setUserFilter	(uint, const QString	&) = 0 ;
	virtual void		setUserSorting	(uint, const QString	&) = 0 ;
	virtual	void		reload		(uint)			   = 0 ;
	virtual	KBValue		getExtra	(uint, bool, uint)	   = 0 ;
	KBValue			getRowExtra	(uint, bool, uint)	;
	virtual uint		getNumValues	(uint)			   = 0 ;
	KBValue			itemToExtra	(uint, bool, uint, const KBLTValueSet *) ;
	virtual QStringList	getDisplayList	(uint) 			   = 0 ;

	bool			dynamic		() ;
	bool			noblank		() ;
	uint			exprCount	() ;
	uint			extraCount	() ;

	KBLinkTree			(KBNode  *, const QDict<QString> &, const QString &) ;
	KBLinkTree			(KBNode  *, KBLinkTree *) ;
	virtual	~KBLinkTree		()		 ;

	enum	Preload
	{	PreloadNone	= 0,
		PreloadDefer	= 1,
		PreloadPrior	= 2
	}	;
}	;

#endif		// __KB_LINKTREE_H
