/***************************************************************************
    file	         : kb_linktreedlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"

#include	"kb_attr.h"
#include	"kb_dbinfo.h"
#include	"kb_propdlg.h"
#include	"kb_qrybase.h"
#include	"kb_block.h"
#include	"kb_linktree.h"

#if		__KB_AUTOMOC
#include	"kb_linktreedlg.moc"
#else
#include	"kb_linktreedlg.h"
#endif

#include	"rk_pushbutton.h"
#include	"rk_combobox.h"
#include	"rk_lineedit.h"


static	IntChoice	choicePreload[] =
{
	{	KBLinkTree::PreloadNone,	"Load when form is loaded"	},
	{	KBLinkTree::PreloadDefer,	"Defer to user reload"		},
	{	KBLinkTree::PreloadPrior,	"Preload data (Web only)"	},
	{	-1,				0				}
}	;

/*  KBLinkTreePropDlg							*/
/*  KBLinkTreePropDlg: Constructor for block properties dialog		*/
/*  link	 : KBLinkTree *	     : Parent link/tree			*/ 
/*  caption	 : const char *      : Dialog box caption		*/
/*  attribs	 : QPtrList<KBAttr>& : List of attributes		*/
/*  iniAttr	 : cchar *	     : Initial attribute		*/
/*  (returns)	 : KBLinkTreePropDlg :					*/

KBLinkTreePropDlg::KBLinkTreePropDlg
	(	KBLinkTree		*linkTree,
		cchar			*caption,
		QPtrList<KBAttr>	&attribs,
		cchar			*iniAttr
	)
	:
	KBItemPropDlg	(linkTree, caption, attribs, iniAttr),
	m_linkTree	(linkTree)
{
	m_bQuery = getUserButton (TR("Query")) ;
	connect (m_bQuery, SIGNAL(clicked()), SLOT(clickQuery())) ;
}

/*  KBLinkTreePropDlg							*/
/*  ~KBLinkTreePropDlg: Destructor for block properties dialog		*/
/*  (returns)	   : void	:					*/

KBLinkTreePropDlg::~KBLinkTreePropDlg ()
{
}

/*  KBLinkTreePropDlg							*/
/*  getAttrItem	: Get attribute item for attribute			*/
/*  attr	: KBAttr *	: Attribute				*/
/*  (returns)	: KBAttrItem *	: Associated attribute item		*/

KBAttrItem
	*KBLinkTreePropDlg::getAttrItem
	(	KBAttr	*attr
	)
{
	if (attr->getName() == "preload")
		return	new KBAttrIntChoice (attr, choicePreload) ;

	return	KBItemPropDlg::getAttrItem (attr) ;
}

/*  KBLinkTreePropDlg							*/
/*  dropProperty: Drop property after user clicks ignore button		*/
/*  (returns)	: void		:					*/

void	KBLinkTreePropDlg::dropProperty ()
{
	KBItemPropDlg::dropProperty () ;
}

/*  KBLinkTreePropDlg							*/
/*  showProperty: Show property						*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property on show			*/

bool	KBLinkTreePropDlg::showProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Child							*/
	/* This should display a list of all columns available from	*/
	/* the associated query, although the user can type in an	*/
	/* arbitrary expression.					*/
	if (aName == "child")
	{
		KBQryBase    *query  ;

		if ((query = m_linkTree->getQuery()) == 0)
			return	warning ("Link/Tree lacks a query") ;

		return	pickQueryField (query, 0, item->value()) ;
	}

	/* Show/Extra							*/
	/* This is the links display query. Show all the fields		*/
	/* associated with that query.					*/
	if ((aName == "show") || (aName == "extra"))
	{
		KBQryBase *query ;

		if ((query = m_linkTree->getQuery()) == 0)
			return	warning ("Link/Tree lacks a query") ;

		return	showQueryField (query, 0, item->value()) ;
	}

	if (aName == "preload")
	{
		showChoices (item, choicePreload, item->value()) ;
		return	true ;
	}

	return	KBItemPropDlg::showProperty (item) ;
}

/*  KBLinkTreePropDlg							*/
/*  saveProperty: Check and save attribute				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Attribute OK				*/

bool	KBLinkTreePropDlg::saveProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Child							*/
	/* In this case, copy back the current selection from the	*/
	/* combo box.							*/
	if (aName == "child")
	{
		setProperty (aName, m_comboBox->currentText()) ;
		return	true ;
	}

	/* Show								*/
	/* Copy the result back ....					*/
	if (aName == "show")
	{
		if (strcmp (m_lineEdit->text(), item->value()) != 0)
			setProperty (aName, m_lineEdit->text()) ;

		return	true ;
	}

	if (aName == "preload")
	{
		saveChoices (item, choicePreload) ;
		return	true	;
	}

	return	KBItemPropDlg::saveProperty (item) ;
}

void	KBLinkTreePropDlg::clickQuery ()
{
	m_linkTree->getQuery()->propertyDlg () ;
}


