/***************************************************************************
    file	         : kb_loader.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<errno.h>
#include	<time.h>

#include	<qstring.h>
#include	<qfile.h>
#include	<qtextstream.h>
#include	<qlayout.h>
#include	<qtimer.h>
#include	<qlabel.h>
#include	<qcheckbox.h>
#include	<qwidgetstack.h>
#include	<qprogressbar.h>
#include	<qtextbrowser.h>
#include	<qgroupbox.h>


#include	"tk_filedialog.h"
#include	"tk_messagebox.h"

#include	"rk_lineedit.h"
#include	"rk_hbox.h"
#include	"rk_vbox.h"
#include	"rk_gridbox.h"
#include	"rk_pushbutton.h"
#include	"rk_listview.h"
#include	"rk_lineedit.h"

#include	"kb_dbinfo.h"
#include	"kb_dblink.h"

#include	"kb_dialog.h"

#include	"kb_copybase.h"
#include	"kb_copytable.h"
#include	"kb_copyxml.h"
#include	"kb_copyexec.h"
#include	"kb_paramsetdlg.h"
#include	"kb_deleter.h"
#include	"kb_sidepanel.h"
#include	"kb_manual.h"
#include 	"kb_popupmenu.h"
#include 	"kb_prompt.h"
#include 	"kb_dom.h"
#include	"kb_errordlg.h"

#if		__KB_AUTOMOC
#include	"kb_loader.moc"
#else		
#include	"kb_loader.h"
#endif


#define		__DEBUG_LOADER	(0)
#if		__DEBUG_LOADER
#include	"kb_dprintf.h"
#endif



class	KBLoaderItem : public QCheckListItem
{
	QString		m_name		;
	uint		m_flags		;
	bool		m_exists	;

public	:

	KBLoaderItem	(RKListView *, const QString &, uint) ;

	inline	const QString	&name  ()
	{
		return	m_name	;
	}
	inline	uint		flags  ()
	{
		return	m_flags	;
	}
	inline	uint		exists ()
	{
		return	m_exists;
	}


	void	setExists	(bool)		;
	bool	checkExists	(KBDBLink &)	;
}	;


/*  KBLoaderItem							*/
/*  KBLoaderItem: Constructor for loader list item			*/
/*  parent	: RKListView *	  : Parent list view			*/
/*  name	: const QString & : Object name				*/
/*  flags	: uint		  : Definition/data flags		*/
/*  (returns)	: KBLoaderItem	  :					*/

KBLoaderItem::KBLoaderItem
	(	RKListView	*parent,
		const QString	&name,
		uint		flags
	)
	:
	QCheckListItem	(parent, name, QCheckListItem::CheckBox),
	m_name		(name),
	m_flags		(flags),
	m_exists	(false)
{
	setText	(1, "")	;

	setText
	(	2,
		(flags & FL_TABLEDEF ) != 0 ? TR("Table"   ) :
		(flags & FL_VIEWDEF  ) != 0 ? TR("View"	   ) :
		(flags & FL_SEQDEF   ) != 0 ? TR("Sequence") :
		(flags & FL_TABLEDATA) != 0 ? TR("Table"   ) :
					      TR("Unknown" )
	)	;

	setText (4, (flags & FL_ANYDEF ) != 0 ? TR("Yes") : TR("No")) ;
	setText (5, (flags & FL_ANYDATA) != 0 ? TR("Yes") : TR("No")) ;

	setText
	(	6,
		((flags & FL_SEQDEF   ) != 0 ? "aa_" :
		 (flags & FL_TABLEDEF ) != 0 ? "bb_" :
		 (flags & FL_TABLEDATA) != 0 ? "bb_" : "cc_") + name
	)	;

	setExists (false) ;
}

void	KBLoaderItem::setExists
	(	bool	exists
	)
{
	m_exists = exists ;
	setText (3, exists ? TR("Yes") : TR("No")) ;		
}

bool	KBLoaderItem::checkExists
	(	KBDBLink	&dbLink
	)
{
	bool	exists	= false	;
	QString	name	= m_name;
	if (!text(1).isEmpty()) name = text(1) ;

	setExists (false)	;

	if	((m_flags & (FL_TABLEDEF|FL_TABLEDATA)) != 0)
	{
		if (!dbLink.tableExists    (name, exists))
		{	dbLink.lastError().DISPLAY() ;
			return	false	;
		}
	}
	else if	((m_flags & FL_VIEWDEF) != 0)
	{
		if (!dbLink.viewExists     (name, exists))
		{	dbLink.lastError().DISPLAY() ;
			return	false	;
		}
	}
	else if	((m_flags & FL_SEQDEF ) != 0)
	{
		if (!dbLink.sequenceExists (name, exists))
		{	dbLink.lastError().DISPLAY() ;
			return	false	;
		}
	}

	setExists (exists)	;
	return	true		;
}


/*  ------------------------------------------------------------------  */

static	void	getFileList
	(	QDict<uint>	&dict,
		const QString	&dir,
		cchar		*extn,
		uint		flag
	)
{
	QDir		scan	;

	scan.setPath	   (dir )	 ;
	scan.setNameFilter (extn)	 ;
	scan.setFilter	   (QDir::Files) ;
	scan.setSorting	   (QDir::Name ) ;

	QStringList sl = scan.entryList () ;
	for (uint idx = 0 ; idx < sl.count() ; idx += 1)
	{
		const QString &file = sl[idx] ;
		QString	      name  = file.left(file.length() - qstrlen(extn) + 1) ;
		uint	      *val  = dict.find(name) ;

		if (val == 0)
		{	val	= new uint ;
			*val	= 0 ;
			dict.insert (name, val) ;
		}

		*val	|= flag	;
	}
}

/*  ------------------------------------------------------------------  */


class	LIBKBASE_API	KBCopyExecLoader : public KBCopyExec
{
	KBLoaderDlg	*m_loader	;

public	:

	KBCopyExecLoader
	(	KBCopyBase	*,
		KBCopyBase	*,
		KBLoaderDlg	*
	)	;

	virtual	bool	showProgress	(int)	;
}	;

KBCopyExecLoader::KBCopyExecLoader
	(	KBCopyBase	*srce,
		KBCopyBase	*dest,
		KBLoaderDlg	*loader
	)
	:
	KBCopyExec	(srce, dest),
	m_loader	(loader)
{
}

bool	KBCopyExecLoader::showProgress
	(	int		nRows
	)
{
	return	m_loader->showProgress (nRows) ;
}

/*  ------------------------------------------------------------------  */

/*  KBLoader								*/
/*  KBLoader	: Constructor for database loader			*/
/*  dbInfo	: KBDBInfo *	  : Database information		*/
/*  server	: const QString & : Server name				*/
/*  (returns)	: KBLoader	  :					*/

KBLoader::KBLoader
	(	KBDBInfo	*dbInfo,
		const QString	&server
	)
	:
	m_dbInfo	(dbInfo),
	m_server	(server)
{
}

bool	KBLoader::loadXMLSpec
	(	const QString	&name,
		cchar		*extn,
		QDomDocument	&doc,
		KBError		&pError
	)
{
	KBFile	file	(name + extn) ;

	if (!file.open (IO_ReadOnly))
	{
		pError	= file.lastError () ;
		return	false	;
	}

	if (!doc.setContent (&file))
	{
		pError	= KBError
			  (	KBError::Error,
				QString	(TR("Cannot parse \"%1\"")).arg(file.name()),
				QString::null,
				__ERRLOCN
			  )	;
		return	false	;
	}

	return	true	;
}

/*  KBLoader								*/
/*  loadTableDef: Load table definition					*/
/*  elem	: const QDomElement &					*/
/*				: Table definition element		*/
/*  drop	: bool		: Drop extant table			*/
/*  best	: bool		: Use best match for columns		*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBLoader::loadTableDef
	(	const QDomElement	&elem,
		bool			drop,
		bool			best,
		KBError			&pError
	)
{
	KBTableSpec	tabSpec  (elem) ;

	/* Check the (forward) map for mapped column names, and also	*/
	/* for a mapped table name, updating the table specification	*/
	/* for any found.						*/
	for (uint idx = 0 ; idx < tabSpec.m_fldList.count() ; idx += 1)
	{
		QString	mapped	= m_forwardMap[tabSpec.m_name + "/" + tabSpec.m_fldList.at(idx)->m_name] ;
		if (!mapped.isEmpty()) tabSpec.m_fldList.at(idx)->m_name = mapped ;
	}

	QString	mapped	= m_forwardMap[tabSpec.m_name]  ;
	if (!mapped.isEmpty()) tabSpec.m_name = mapped ;

	if (drop)
		if (!m_dbLink.dropTable (tabSpec.m_name, false))
		{	pError	= m_dbLink.lastError() ;
			return	false	;
		}

	if (!m_dbLink.createTable (tabSpec, false, best))
	{	pError = m_dbLink.lastError() ;
		return	 false	;
	}

	return	true	;
}

/*  KBLoader								*/
/*  loadViewDef	: Load view definition					*/
/*  elem	: const QDomElement &					*/
/*				: View definition element		*/
/*  drop	: bool		: Drop extant view			*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBLoader::loadViewDef
	(	const QDomElement	&elem,
		bool			drop,
		KBError			&pError
	)
{
	KBTableSpec	viewSpec  (elem) ;

	if (drop)
		if (!m_dbLink.dropView (viewSpec.m_name))
		{	pError	= m_dbLink.lastError() ;
			return	false	;
		}


	if (!m_dbLink.createView (viewSpec))
	{	pError = m_dbLink.lastError() ;
		return	 false	;
	}

	return	true	;
}

/*  KBLoader								*/
/*  loadSequenceDef							*/
/*		: Load sequence definition				*/
/*  elem	: const QDomElement &					*/
/*				: Sequence definition element		*/
/*  drop	: bool		: Drop extant sequence			*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBLoader::loadSequenceDef
	(	const QDomElement	&elem,
		bool			drop,
		KBError			&pError
	)
{
	KBSequenceSpec	seqSpec  (elem) ;

	if (drop)
		if (!m_dbLink.dropSequence (seqSpec.m_name))
		{	pError	= m_dbLink.lastError() ;
			return	false	;
		}

	if (!m_dbLink.createSequence (seqSpec))
	{	pError = m_dbLink.lastError() ;
		return	 false	;
	}

	return	true	;
}


/*  ------------------------------------------------------------------	*/

/*  KBLoaderDlg								*/
/*  KBLoaderDlg	: Constructor for database loader dialog		*/
/*  dbInfo	: KBDBInfo *	  : Database information		*/
/*  server	: const QString & : Server name				*/
/*  dir		: const QString & : Output directory			*/
/*  (returns)	: KBLoader	  :					*/

KBLoaderDlg::KBLoaderDlg
	(	KBDBInfo	*dbInfo,
		const QString	&server,
		const QString	&dir
	)
	:
	KBDialog	(TR("Load database"), true, "kbloaderdlg"),
	KBLoader	(dbInfo, server),
	m_dir		(dir)
{
	RKVBox	*layMain = new RKVBox (this) ;
	layMain->setTracking () ;

	RKHBox	*layCtrl = new RKHBox (layMain) ;

	QGroupBox *laySet= new QGroupBox (1, Qt::Horizontal, TR("Options" ), layCtrl) ;
	QGroupBox *layMap= new QGroupBox (1, Qt::Horizontal, TR("Map Case"), layCtrl) ;

	m_loadAll	 = new QCheckBox (TR("Load all objects"         ), laySet) ;

	RKHBox	  *layW1 = new RKHBox    (laySet) ;
	QWidget	  *layS1 = new QWidget   (layW1 ) ;
	RKVBox	  *layV1 = new RKVBox	 (layW1 ) ;
	layS1->setFixedWidth (32) ;
	m_loadTables	 = new QCheckBox (TR("Load tables"		), layV1 ) ;
	m_loadViews	 = new QCheckBox (TR("Load views"		), layV1 ) ;
	m_loadSequences	 = new QCheckBox (TR("Load sequences"		), layV1 ) ;
	

	m_loadDefn	 = new QCheckBox (TR("Load object definitions"  ), laySet) ;
	RKHBox	  *layW2 = new RKHBox    (laySet) ;
	QWidget	  *layS2 = new QWidget   (layW2 ) ;
	RKVBox	  *layV2 = new RKVBox	 (layW2 ) ;
	layS2->setFixedWidth (32) ;
	m_dropObjects	 = new QCheckBox (TR("Replace existing objects" ), layV2 ) ;
	m_bestMatch	 = new QCheckBox (TR("Use best column match"    ), layV2 ) ;

	connect
	(	m_loadDefn,
		SIGNAL	(toggled	(bool)),
		layW2,
		SLOT	(setEnabled	(bool))
	)	;
	layW2->setEnabled (false) ;


	m_loadData	 = new QCheckBox (TR("Load object data"         ), laySet) ;
	RKHBox	  *layW3 = new RKHBox    (laySet) ;
	QWidget	  *layS3 = new QWidget   (layW3 ) ;
	layS3->setFixedWidth (32) ;
	m_dropData	 = new QCheckBox (TR("Replace existing data" 	), layW3 ) ;

	connect
	(	m_loadData,
		SIGNAL	(toggled	(bool)),
		layW3,
		SLOT	(setEnabled	(bool))
	)	;
	layW3->setEnabled (false) ;

	m_mapUpper	 = new RKPushButton (TR("Upper case"   ), layMap) ;
	m_mapLower	 = new RKPushButton (TR("Lower case"   ), layMap) ;
	m_mapUnmap	 = new RKPushButton (TR("Unmap all"    ), layMap) ;
	m_mapLoad	 = new RKPushButton (TR("Load mappings"), layMap) ;
	m_mapSave	 = new RKPushButton (TR("Save mappings"), layMap) ;


	connect
	(	m_mapUpper,
		SIGNAL	(clicked    ()),
		SLOT	(mapAllUpper())
	)	;
	connect
	(	m_mapLower,
		SIGNAL	(clicked    ()),
		SLOT	(mapAllLower())
	)	;
	connect
	(	m_mapUnmap,
		SIGNAL	(clicked    ()),
		SLOT	(unmapAll   ())
	)	;
	connect
	(	m_mapLoad,
		SIGNAL	(clicked    ()),
		SLOT	(loadMapping())
	)	;
	connect
	(	m_mapSave,
		SIGNAL	(clicked    ()),
		SLOT	(saveMapping())
	)	;

	m_tableList	 = new RKListView(layMain) ;
	RKHBox	*layProg = new RKHBox    (layMain) ;

	QLabel	*l1	 = new QLabel (TR("Object"), layProg) ;
	l1->setAlignment(Qt::AlignRight) ;
	m_loading	 = new QLabel (layProg) ;

	QLabel	*l2	 = new QLabel (TR("Record"), layProg) ;
	l2->setAlignment(Qt::AlignRight) ;
	m_record	 = new QLabel (layProg) ;

	QLabel	*l3	 = new QLabel (TR("Object"), layProg) ;
	l3->setAlignment(Qt::AlignRight) ;
	m_of		 = new QLabel (layProg) ;

	addOKCancel	(layMain, &m_bOK, &m_bCancel, KBHELPKEY("Chap14DumperLoader"));
	m_bOK    ->setText (TR("Go"    )) ;
	m_bCancel->setText (TR("Finish")) ;

	m_loading->setMinimumWidth  (250) ;

	m_cancelled	= false		;
	m_currentItem	= 0		;
	m_currentType	= KB::IsAny	;
	m_index		= 0		;

	m_tableList->addColumn(TR("Name"	)) ;
	m_tableList->addColumn(TR("Mapped name"	)) ;
	m_tableList->addColumn(TR("Type"	)) ;
	m_tableList->addColumn(TR("Exists"	)) ;
	m_tableList->addColumn(TR("Definition"	)) ;
	m_tableList->addColumn(TR("Data"	)) ;

	m_loading->setFrameStyle (QFrame::Box|QFrame::Plain) ;
	m_loading->setLineWidth  (1) ;
	m_record ->setFrameStyle (QFrame::Box|QFrame::Plain) ;
	m_record ->setLineWidth  (1) ;
	m_of	 ->setFrameStyle (QFrame::Box|QFrame::Plain) ;
	m_of	 ->setLineWidth  (1) ;

	connect
	(	m_loadAll,
		SIGNAL	(toggled		(bool)),
		SLOT	(loadAllChanged		())
	)	;
	connect
	(	m_loadTables,
		SIGNAL	(toggled		(bool)),
		SLOT	(loadTablesChanged	())
	)	;
	connect
	(	m_loadViews,
		SIGNAL	(toggled		(bool)),
		SLOT	(loadViewsChanged	())
	)	;
	connect
	(	m_loadSequences,
		SIGNAL	(toggled		(bool)),
		SLOT	(loadSequencesChanged	())
	)	;

	connect
	(	m_tableList,
		SIGNAL	(contextMenuRequested	(QListViewItem *, const QPoint &, int)),
		SLOT	(contextMenu		(QListViewItem *, const QPoint &, int))
	)	;

	m_tableList->setRootIsDecorated (true) ;
	m_popItem = 0	;

	m_widgetList.append	(m_loadAll	) ;
	m_widgetList.append	(m_loadTables	) ;
	m_widgetList.append	(m_loadViews	) ;
	m_widgetList.append	(m_loadSequences) ;
	m_widgetList.append	(m_loadDefn	) ;
	m_widgetList.append	(m_loadData	) ;
	m_widgetList.append	(m_dropObjects	) ;
	m_widgetList.append	(m_dropData	) ;
	m_widgetList.append	(m_tableList	) ;
	m_widgetList.append	(m_bestMatch	) ;
	m_widgetList.append	(m_mapUpper	) ;
	m_widgetList.append	(m_mapLower	) ;
	m_widgetList.append	(m_mapUnmap	) ;
	m_widgetList.append	(m_mapLoad	) ;
	m_widgetList.append	(m_mapSave	) ;
}


/*  KBLoaderDlg								*/
/*  loadAllChanged							*/
/*		: Load-all checkbox changed				*/
/*  (void)	:		:					*/

void	KBLoaderDlg::loadAllChanged ()
{
	for
	(	KBLoaderItem *ti = (KBLoaderItem *)m_tableList->firstChild() ;
		ti != 0 ;
		ti  = (KBLoaderItem *)ti->nextSibling()
	)
		ti->setOn (m_loadAll->isOn()) ;

	m_loadTables   ->setChecked (m_loadAll->isOn()) ;
	m_loadViews    ->setChecked (m_loadAll->isOn()) ;
	m_loadSequences->setChecked (m_loadAll->isOn()) ;
}

/*  KBLoaderDlg								*/
/*  loadTablesChanged							*/
/*		: Load-tables checkbox changed				*/
/*  (void)	:		:					*/

void	KBLoaderDlg::loadTablesChanged ()
{
	for
	(	KBLoaderItem *ti = (KBLoaderItem *)m_tableList->firstChild() ;
		ti != 0 ;
		ti  = (KBLoaderItem *)ti->nextSibling()
	)
		if ((ti->flags() & (FL_TABLEDEF|FL_TABLEDATA)) != 0)
			ti->setOn (m_loadTables->isOn()) ;
}

/*  KBLoaderDlg								*/
/*  loadViewssChanged							*/
/*		: Load-views checkbox changed				*/
/*  (void)	:		:					*/

void	KBLoaderDlg::loadViewsChanged ()
{
	for
	(	KBLoaderItem *ti = (KBLoaderItem *)m_tableList->firstChild() ;
		ti != 0 ;
		ti  = (KBLoaderItem *)ti->nextSibling()
	)
		if ((ti->flags() & FL_VIEWDEF) != 0)
			ti->setOn (m_loadViews->isOn()) ;
}

/*  KBLoaderDlg								*/
/*  loadSequencesChanged						*/
/*		: Load-sequences checkbox changed			*/
/*  (void)	:		:					*/

void	KBLoaderDlg::loadSequencesChanged ()
{
	for
	(	KBLoaderItem *ti = (KBLoaderItem *)m_tableList->firstChild() ;
		ti != 0 ;
		ti  = (KBLoaderItem *)ti->nextSibling()
	)
		if ((ti->flags() & FL_SEQDEF) != 0)
			ti->setOn (m_loadSequences->isOn()) ;
}

/*  KBLoaderDlg								*/
/*  contextMenu	: Show context menu					*/
/*  (void)	:		:					*/

void	KBLoaderDlg::contextMenu
	(	QListViewItem	*item,
		const QPoint	&pos,
		int
	)
{
	if (item == 0)
		return	;

	KBPopupMenu pop (0) ;
	m_popItem = item    ;


	/* Generate a popup menu that is appropriate to the type of	*/
	/* entry for which the context menu is requested.		*/
	if (item->depth() == 0)
	{
		pop.setTitle   (TR("Object name mapping" )) ;

		pop.insertItem
		(	TR("&Map object name"),
			this,
			SLOT(mapName  ())
		)	;
		pop.insertItem
		(	TR("&Unmap object name"),
			this,
			SLOT(unmapName())
		)	;

		pop.exec (pos) ;
		return	;
	}

	if (item->depth() == 1)
	{
		pop.setTitle   (TR("Column name mapping" )) ;

		pop.insertItem
		(	TR("&Map column name"),
			this,
			SLOT(mapName  ())
		)	;
		pop.insertItem
		(	TR("&Unmap column name"),
			this,
			SLOT(unmapName())
		)	;
		pop.exec (pos) ;
		return	;
	}
}

/*  Mapping functions followed, passed as argument to ::mapAllCase	*/
/*  mapToLower	: Map name to lower case				*/
/*  mapToUpper	: Map name to upper case				*/
/*  unmap	: Remove mapping					*/

static	QString	mapToLower
	(	const QString	&name
	)
{
	return	name.lower()	;
}

static	QString	mapToUpper
	(	const QString	&name
	)
{
	return	name.upper()	;
}

static	QString	unmap
	(	const QString	&
	)
{
	return	QString::null	;
}

/*  KBLoaderDlg								*/
/*  mapAllCase	: Map case on all entries				*/
/*  mapfn	: 		: Mapping function			*/
/*  (returns)	: void		:					*/

void	KBLoaderDlg::mapAllCase
	(	QString (*mapfn) (const QString &)
	)
{
	for
	(	KBLoaderItem *ti = (KBLoaderItem *)m_tableList->firstChild() ;
		ti != 0 ;
		ti  = (KBLoaderItem *)ti->nextSibling()
	)
		if ((ti->flags() & FL_TABLEDEF) != 0)
		{
			for
			(	QListViewItem *ci = ti->firstChild() ;
				ci != 0 ;
				ci  = ci->nextSibling()
			)
				ci->setText(1, (*mapfn)(ci->text(0))) ;

			ti->setText	(1, (*mapfn)(ti->text(0))) ;
			ti->checkExists (m_dbLink) ;
		}
}

/*  KBLoaderDlg								*/
/*  mapAllLower	: Map all entries to lower case				*/
/*  (returns)	: void		:					*/

void	KBLoaderDlg::mapAllLower ()
{
	mapAllCase (mapToLower) ;
}

/*  KBLoaderDlg								*/
/*  mapAllUpper	: Map all entries to upper case				*/
/*  (returns)	: void		:					*/

void	KBLoaderDlg::mapAllUpper ()
{
	mapAllCase (mapToUpper) ;
}

/*  KBLoaderDlg								*/
/*  unmapAll	: Unmap all entries					*/
/*  (returns)	: void		:					*/

void	KBLoaderDlg::unmapAll ()
{
	mapAllCase (unmap     ) ;
}

/*  KBLoaderDlg								*/
/*  setupMaps	: Set up name mapping					*/
/*  (returns)	: void		:					*/

void	KBLoaderDlg::setupMaps ()
{
	/* Construct two column name maps. The forward map is used to	*/
	/* map names in the definition files to different names used	*/
	/* when creating objects. The backward map is needed when data	*/
	/* is inserted; see the "::loadTableData" method for details.	*/
	m_forwardMap .clear() ;
	m_backwardMap.clear() ;

	for
	(	KBLoaderItem *ti = (KBLoaderItem *)m_tableList->firstChild() ;
		ti != 0 ;
		ti  = (KBLoaderItem *)ti->nextSibling()
	)
		if ((ti->flags() & FL_TABLEDEF) != 0)
		{
			for
			(	QListViewItem *ci = ti->firstChild() ;
				ci != 0 ;
				ci  = ci->nextSibling()
			)
				if (!ci->text(1).isEmpty())
				{
					m_forwardMap [ti->text(0) + "/" + ci->text(0)] = ci->text(1) ;
					m_backwardMap[ti->text(0) + "/" + ci->text(1)] = ci->text(0) ;
				}

			if (!ti->text(1).isEmpty())
				m_forwardMap[ti->text(0)] = ti->text(1) ;
		}

#if	__DEBUG_LOADER
	QMap<QString,QString>::Iterator it ;
	for (it = m_forwardMap.begin() ; it != m_forwardMap.end() ; ++it)
		DPRINTF
		((	"KBLoaderDlg::accept: [%s]->[%s]\n",
			it.key ().latin1(),
			it.data().latin1()
		))	;
#endif
}

/*  KBLoaderDlg								*/
/*  saveMapping	: Save current mappings to file				*/
/*  (returns)	: void		:					*/

void	KBLoaderDlg::saveMapping ()
{
	QString	fname	= KBFileDialog::getSaveFileName
				(	QString::null,
					QString::null,
					0,
					TR("Save mappings to file ...")
				)	;
	if (fname.isEmpty()) return	;

	setupMaps ()	;

	QDomDocument	doc	("TableMappings") ;
	QDomElement	root	= doc.createElement ("TableMappings") ;
	doc.appendChild(root)	;

	QMap<QString,QString>::Iterator fit ;
	for (fit = m_forwardMap .begin() ; fit != m_forwardMap .end() ; ++fit)
	{
		QDomElement	elem = doc.createElement ("forward" ) ;
		elem.setAttribute ("key", fit.key ()) ;
		elem.setAttribute ("map", fit.data()) ;
		root.appendChild  (elem) ;
	}
	QMap<QString,QString>::Iterator bit ;
	for (bit = m_backwardMap.begin() ; bit != m_backwardMap.end() ; ++bit)
	{
		QDomElement	elem = doc.createElement ("backward") ;
		elem.setAttribute ("key", bit.key ()) ;
		elem.setAttribute ("map", bit.data()) ;
		root.appendChild  (elem) ;
	}

	KBFile	file (fname) ;
	if (!file.open (IO_WriteOnly))
	{	file.lastError().DISPLAY() ;
		return	;
	}

	QTextStream (&file) << doc.toString() ;
	file.close ()	;
}

/*  KBLoaderDlg								*/
/*  loadMapping	: Load mappings from file				*/
/*  (returns)	: void		:					*/

void	KBLoaderDlg::loadMapping ()
{
	QString	fname	= KBFileDialog::getOpenFileName
				(	QString::null,
					QString::null,
					0,
					TR("Load mappings from file ...")
				)	;
	if (fname.isEmpty()) return	;

	KBFile	file (fname) ;
	if (!file.open (IO_ReadOnly))
	{
		file.lastError().DISPLAY() ;
		return	;
	}

	QDomDocument	doc	;
	if (!doc.setContent (&file))
	{
		KBError::EError
		(	TR("Cannot parse mappings file"),
			QString::null,
			__ERRLOCN
		)	;
		return	;
	}

	QDomElement	root	= doc.documentElement() ;
	if (root.tagName() != "TableMappings")
	{
		KBError::EError
		(	TR("Unexpected root element in mappings file"),
			QString(TR("Expected 'TableMappings', got '%s'")).arg(root.tagName()),
			__ERRLOCN
		)	;
		return	;
	}

	m_forwardMap .clear() ;
	m_backwardMap.clear() ;

	DOMITER_BEGIN(root,"forward", elem)
	{
		QString	    key	   = elem.attribute("key") ;
		QString	    map	   = elem.attribute("map") ;
		QStringList bits   = QStringList::split('/', key) ;

		m_forwardMap [key] = map ;

		for
		(	KBLoaderItem *ti = (KBLoaderItem *)m_tableList->firstChild() ;
			ti != 0 ;
			ti  = (KBLoaderItem *)ti->nextSibling()
		)
			if (ti->text(0) == bits[0])
			{
				if (bits.count() == 1)
				{
					ti->setText	(1, map  ) ;
					ti->checkExists (m_dbLink) ;
					break	;
				}

				for
				(	QListViewItem *ci = ti->firstChild() ;
					ci != 0 ;
					ci  = ci->nextSibling()
				)
					if (ci->text(0) == bits[1])
					{
						ci->setText (1, map) ;
						break	;
					}

				break	;
			}
	}
	DOMITER_END

	DOMITER_BEGIN(root,"backward",elem)
	{
		m_backwardMap[elem.attribute("key")] = elem.attribute("map") ;
	}
	DOMITER_END
}

/*  KBLoaderDlg	: 							*/
/*  mapName	: Set up column or table name mapping			*/
/*  (returns)	: void		:					*/

void	KBLoaderDlg::mapName ()
{
	if (m_popItem != 0)
	{
		QString	    name = m_popItem->text(1) ;
		if (name.isEmpty()) name = m_popItem->text(0) ;

		KBPromptDlg pDlg (TR("Map name"), TR("Mapped name"), name) ;
		if (pDlg.exec()) m_popItem->setText (1, name) ;

		if (m_popItem->depth() == 0)
			((KBLoaderItem *)m_popItem)->checkExists(m_dbLink) ;

		m_popItem = 0 ;
	}
}

/*  KBLoaderDlg	: 							*/
/*  unmapName	: Remove a column or table name mapping			*/
/*  (returns)	: void		:					*/

void	KBLoaderDlg::unmapName ()
{
	if (m_popItem != 0)
	{
		m_popItem->setText (1, "") ;

		if (m_popItem->depth() == 0)
			((KBLoaderItem *)m_popItem)->checkExists(m_dbLink) ;

		m_popItem = 0 ;
	}
}


/*  KBLoaderDlg								*/
/*  accept	: User clicks OK button					*/
/*  (returns)	: void		:					*/

void	KBLoaderDlg::accept ()
{
	if (!m_loadDefn->isChecked() && !m_loadData->isChecked())
	{
		TKMessageBox::sorry
		(	0,
			TR("Please select definition and/or data loading"),
			TR("Load Database")
		)	;
		return	;
	}

	bool	any = false ;
	QCheckListItem *i = (QCheckListItem *)m_tableList->firstChild() ;
	while (i != 0)
		if (i->isOn())
		{	any	= true ;
			break	;
		}
		else	i = (QCheckListItem *)i->nextSibling() ;

	if (!any)
	{
		TKMessageBox::sorry
		(	0,
			TR("No tables selected for loading"),
			TR("Load Database")
		)	;
		return	;
	}

	setupMaps ()	;

	LITER
	(	QWidget,
		m_widgetList,
		w,
		w->setEnabled (false)
	)	;

	m_bOK	     ->setEnabled (false) ;

	/* Note the start time. Used so we can update the progress	*/
	/* more frequently if things are running slowly.		*/
	m_ldtime = time(0) ;

	m_currentItem = (KBLoaderItem *)m_tableList->firstChild() ;
	m_currentType = KB::IsSequence	 ;
	m_index	      = 0 ;

	slotTimer () ;
}

/*  KBLoaderDlg								*/
/*  clickCancel	: User clicks cancel button				*/
/*  (returns)	: void		:					*/

void	KBLoaderDlg::reject ()
{
	if (!m_cancelled)
	{
		m_cancelled = true ;
		done	(0) ;
	}
}

/*  KBLoaderDlg								*/
/*  exec	: Execute dumper					*/
/*  (returns)	: int		: Exit code				*/

int	KBLoaderDlg::exec	()
{
	if (!m_dbLink.connect (m_dbInfo, m_server))
	{
		m_dbLink.lastError().DISPLAY() ;
		return 0 ;
	}

	{
	KBErrorBlock eb (KBErrorBlock::Accrue) ;


	QDict<uint>	names	   ;
	names.setAutoDelete (true) ;

	getFileList	(names, m_dir, "*.tabledef",  FL_TABLEDEF ) ;
	getFileList	(names, m_dir, "*.viewdef",   FL_VIEWDEF  ) ;
	getFileList	(names, m_dir, "*.seqdef",    FL_SEQDEF	  ) ;
	getFileList	(names, m_dir, "*.tabledata", FL_TABLEDATA) ;

	m_tableList->setSorting (5) ;

	QDictIterator<uint> iter (names) ;
	uint		    *val	 ;

	while ((val = iter.current()) != 0)
	{
		QString	     name  = iter.currentKey() ;
		KBLoaderItem *item = new KBLoaderItem (m_tableList, name, *val) ;

		item->checkExists (m_dbLink)	;

		if ((*val  & FL_TABLEDEF) != 0)
		{
			QDomDocument	doc	;
			KBError		error	;
			if (!loadXMLSpec (m_dir + "/" + name, ".tabledef", doc, error))
				continue	;

			KBTableSpec	tabSpec  (doc.documentElement().firstChild().toElement()) ;
			QListViewItem	*last	= 0 ;

			for (uint idx = 0 ; idx < tabSpec.m_fldList.count() ; idx += 1)
				last	= new QListViewItem
					  (	item,
						last,
						tabSpec.m_fldList.at(idx)->m_name,
						""
					  )	;
		}

		iter += 1 ;
	}
	}

	return	KBDialog::exec () ;
}

/*  KBLoaderDlg								*/
/*  loadTableDef: Load table definition					*/
/*  name	: const QString & : Table/definition file name		*/
/*  drop	: bool		  : Drop extant table			*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: bool		  : Success				*/

bool	KBLoaderDlg::loadTableDef
	(	const QString	&name,
		bool		drop,
		KBError		&pError
	)
{
	QDomDocument	doc	;
	if (!loadXMLSpec (m_dir + "/" + name, ".tabledef", doc, pError))
		return	false	;

	return	KBLoader::loadTableDef
		(	doc.documentElement().firstChild().toElement(),
			drop,
			m_bestMatch->isChecked(),
			pError
		)	;
}

/*  KBLoaderDlg								*/
/*  loadTableData: Load table data					*/
/*  name	 : const QString & : Table data file name		*/
/*  drop	 : bool		   : Drop extant data			*/
/*  pError	 : KBError &	   : Error return			*/
/*  (returns)	 : bool		   : Success				*/

bool	KBLoaderDlg::loadTableData
	(	const QString	&name,
		bool		drop,
		KBError		&pError
	)
{
	QString	name2	= name	;
	QString	mapped	= m_forwardMap[name]  ;
	if (!mapped.isEmpty()) name2 = mapped ;

	KBTableSpec	tabSpec	(name2) ;
	if (!m_dbLink.listFields(tabSpec))
	{	pError	= m_dbLink.lastError() ;
		return	false	;
	}

	/* The work is done using a copier. Create a dummy location and	*/
	/* thence an XML source and an table destination. These can be	*/
	/* appropriately initialised with the table structure.		*/
	KBLocation	location   (m_dbInfo, "copier", m_server, "unnamed") ;
	KBCopyXML	*copyXML   = new KBCopyXML   (true,  location) ;
	KBCopyTable	*copyTable = new KBCopyTable (false, location) ;

	copyXML	 ->setMainTag (name ) ;
	copyXML	 ->setRowTag  ("row") ;
	copyXML	 ->setErrOpt  (KBCopyXML::ErrPad) ;
	copyXML	 ->setFile    (m_dir + "/" + name + ".tabledata") ;

	copyTable->setServer  (m_server) ;
	copyTable->setTable   (name2   ) ;
	copyTable->setOption  (drop ? KBCopyTable::OptReplace : KBCopyTable::OptReplace, "") ;

	for (uint idx = 0 ; idx < tabSpec.m_fldList.count() ; idx += 1)
	{
		KBFieldSpec *fSpec = tabSpec.m_fldList.at(idx) ;

		/* The table column is added directly as the field in	*/
		/* the table specification; but check the *backward*	*/
		/* column map to ensure that the XML element tag is the	*/
		/* original (dumped) column name.			*/
		QString	    mapped = m_backwardMap[name + "/" + fSpec->m_name] ;
		if (mapped.isEmpty())
			copyXML  ->addField (fSpec->m_name, false) ;
		else	copyXML  ->addField (mapped,	    false) ;

		copyTable->addField (fSpec->m_name) ;
	}

	/* Now create the actual copier and execute it. Most of the	*/
	/* arguments are dummy in this situation.			*/
	KBCopyExecLoader  copy   (copyXML, copyTable, this) ;
	QString	   	  report ;
	int	   	  nRows  ;
	QDict<QString>	  d1	 ;
	QDict<KBParamSet> d2	 ;
	if (!copy.execute
		(	report,
			pError,
			nRows,
			d1, d2,
			false
		))
	{
		return	false	;
	}

	return	true	;
}

/*  KBLoaderDlg								*/
/*  loadViewDef	: Load view definition					*/
/*  name	: const QString & : View/definition file name		*/
/*  drop	: bool		  : Drop extant table			*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: bool		  : Success				*/

bool	KBLoaderDlg::loadViewDef
	(	const QString	&name,
		bool		drop,
		KBError		&pError
	)
{
	QDomDocument	doc	;
	if (!loadXMLSpec (m_dir + "/" + name, ".viewdef", doc, pError))
		return	false	;

	return	KBLoader::loadViewDef
		(	doc.documentElement().firstChild().toElement(),
			drop,
			pError
		)	;
}

/*  KBLoaderDlg								*/
/*  loadSequenceDef							*/
/*		: Load sequence definition				*/
/*  name	: const QString & : Sequence/definition file name	*/
/*  drop	: bool		  : Drop extant table			*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: bool		  : Success				*/

bool	KBLoaderDlg::loadSequenceDef
	(	const QString	&name,
		bool		drop,
		KBError		&pError
	)
{
	QDomDocument	doc	;
	if (!loadXMLSpec (m_dir + "/" + name, ".seqdef", doc, pError))
		return	false	;

	return	KBLoader::loadSequenceDef
		(	doc.documentElement().firstChild().toElement(),
			drop,
			pError
		)	;
}

bool	KBLoaderDlg::loadDetails
	(	KBLoaderItem	*item,
		bool		&pause
	)
{
	const QString	&name	= item->name  () ;
	uint		flags	= item->flags () ;
	bool		dropobj	= item->exists() && m_dropObjects->isChecked() ;
	bool		dropdat	= item->exists() && m_dropData   ->isChecked() ;
	KBError		error	;

	m_loading->setText (name) ;
	m_record ->setText (""  ) ;

	m_of->setText
	(	QString(TR("%1 of %2"))
			.arg(m_index + 1)
			.arg(m_tableList->childCount())
	)	;

	m_tableList->ensureItemVisible (m_currentItem) ;
	m_tableList->setCurrentItem    (m_currentItem) ;

	qApp->processEvents() ;


	if ((flags & (FL_TABLEDEF|FL_TABLEDATA)) != 0)
	{
		if (m_currentType != KB::IsTable)
			return	true	;

		/* In the event of an error, display the error and	*/
		/* simulate the user clicking the cancel button, to	*/
		/* terminate the dialog.				*/
		if (m_loadDefn->isChecked() && ((flags & FL_TABLEDEF ) != 0))
		{
			bool	rc	= loadTableDef (name, dropobj, error) ;
			bool	exists	;

			/* Update the display to reflect whether	*/
			/* the table now exists (or not ...)		*/
			if (m_dbLink.tableExists (name, exists))
				item->setExists (exists) ;

			if (!rc)
			{	error.DISPLAY() ;
				return	false	;
			}
		}

		if (m_loadData->isChecked() && ((flags & FL_TABLEDATA) != 0))
			if (!loadTableData (name, dropdat, error))
			{	error.DISPLAY() ;
				return	false	;
			}

		item->setOn	(false) ;
		pause = true	;
		return	true	;
	}

	if ((flags & FL_VIEWDEF) != 0)
	{
		if (m_currentType != KB::IsView)
			return	true	;

		if (m_loadDefn->isChecked())
		{
			bool	rc	= loadViewDef  (name, dropobj, error) ;
			bool	exists	;

			/* Update the display to reflect whether	*/
			/* the view now exists (or not ...)		*/
			if (m_dbLink.viewExists (name, exists))
				item->setExists (exists) ;

			if (!rc)
			{	error.DISPLAY() ;
				return	false	;
			}
		}

		item->setOn	(false) ;
		pause = true	;
		return	true	;
	}

	if ((flags & FL_SEQDEF) != 0)
	{
		if (m_currentType != KB::IsSequence)
			return	true	;

		if (m_loadDefn->isChecked())
		{
			bool	rc	= loadSequenceDef  (name, dropobj, error) ;
			bool	exists	;

			/* Update the display to reflect whether	*/
			/* the sequence now exists (or not ...)		*/
			if (m_dbLink.sequenceExists (name, exists))
				item->setExists (exists) ;

			if (!rc)
			{	error.DISPLAY() ;
				return	false	;
			}
		}

		item->setOn	(false) ;
		pause = true	;
		return	true	;
	}

	return	true	;
}

/*  KBLoaderDlg								*/
/*  slotTimer	: Start dump of next object				*/
/*  (returns)	: void		:					*/

void	KBLoaderDlg::slotTimer ()
{
	/* Look for the next object to dump. The loop will exit if we	*/
	/* run out of objects before finding a candidate.		*/
	while (m_currentItem != 0)
	{
		bool	pause	= false	;

		if (m_currentItem->isOn())
			if (!loadDetails (m_currentItem, pause))
			{
				m_currentItem = 0 ;
				break	;
			}

		m_currentItem = (KBLoaderItem *)m_currentItem->nextSibling() ;
		m_index       += 1 ;

		/* When at the end of the list we go back to the start	*/
		/* depending on type, so that sequences are done first,	*/
		/* then tables and lastly views.			*/
		if (m_currentItem == 0)
			switch (m_currentType)
			{
				case KB::IsSequence :
					m_currentType	= KB::IsTable	;
					m_currentItem	= (KBLoaderItem *)m_tableList->firstChild() ;
					break	;

				case KB::IsTable    :
					m_currentType	= KB::IsView	;
					m_currentItem	= (KBLoaderItem *)m_tableList->firstChild() ;
					break	;

				case KB::IsView	    :
					break	;

				default		    :
					break	;
			}

		if (pause)
		{
			QTimer::singleShot (200, this, SLOT(slotTimer())) ;
			qApp->processEvents() ;
			return	;
		}
	}

	LITER
	(	QWidget,
		m_widgetList,
		w,
		w->setEnabled (true)
	)	;

	m_bOK	     ->setEnabled (true) ;
}

bool	KBLoaderDlg::showProgress
	(	int		nRows
	)
{
	if ((nRows % 10 == 0) || (time(0) >= m_ldtime + 2))
	{
		m_record->setText (QString::number(nRows)) ;
		m_ldtime = time   (0) ;
		qApp->processEvents() ;
	}

	return	m_cancelled ;
}

/*  loadDatabase: Load database from a set of files			*/
/*  dbInfo	: KBDBInfo *	  : Database information		*/
/*  server	: const QString & : Server name				*/
/*  dir		: const QString & : Destination directory		*/
/*  (returns)	: void		  :					*/

LIBKBASE_API
	void	loadDatabase
	(	KBDBInfo	*dbInfo,
		const QString	&server,
		const QString	&dir
	)
{
	KBLoaderDlg (dbInfo, server, dir).exec() ;
}

/*  ------------------------------------------------------------------	*/

/*  KBLoaderStockDB							*/
/*  KBLoaderStockDB							*/
/*		: Constructor for stock database loader			*/
/*  dbInfo	: KBDBInfo *	  : Database information		*/
/*  server	: const QString & : Server name				*/
/*  url		: const QString & : Specification URL			*/
/*  (returns)	: KBLoaderStockDB :					*/

KBLoaderStockDB::KBLoaderStockDB
	(	KBDBInfo	*dbInfo,
		const QString	&server,
		const QString	&url
	)
	:
	KBDialog	(TR("Load stock database"), true),
	KBLoader	(dbInfo, server),
	m_url		(url)
{
	RKVBox	   *layMain  = new RKVBox    (this) ;
	layMain->setTracking () ;

	RKHBox	   *layTop   = new RKHBox    (layMain) ;
	new KBSidePanel (layTop, TR("Stock database loader")) ;
	
	RKGridBox  *layGrid  = new RKGridBox (2, layTop) ;

	new QLabel (TR("Location"), layGrid) ;
	RKLineEdit *location = new RKLineEdit(layGrid) ;

	location->setText	(url ) ;
	location->setReadOnly	(true) ;

	new QLabel (TR("Progress"),  layGrid) 		;
	m_progress 	= new QWidgetStack (layGrid)	;
	m_progressText	= new RKLineEdit   (m_progress) ;
	m_progressBar	= new QProgressBar (m_progress)	;

	m_progress->addWidget	(m_progressText) ;
	m_progress->addWidget	(m_progressBar ) ;
	m_progress->raiseWidget	(m_progressText) ;

	m_progressText->setReadOnly (true) ;

	new QWidget (layGrid) ;
	m_details	= new QTextBrowser (layGrid) ;
	m_details->setMinimumWidth (400) ;

	addOKCancel	(layMain, &m_bOK, &m_bCancel) ;

	m_http		= 0	;
	m_httpSetHostID	= -1	;
	m_httpGetID	= -1	;
	m_document	= 0	;

	m_bOK    ->setEnabled (false) ;
	m_bCancel->setEnabled (true ) ;
}

/*  KBLoaderStockDB							*/
/*  ~KBLoaderStockDB							*/
/*		: Destructor for stock database loader			*/
/*  (returns)	:		:					*/

KBLoaderStockDB::~KBLoaderStockDB ()
{
	DELOBJ	(m_document) ;
}

/*  KBLoaderStockDB							*/
/*  setProgress	: Set installation progress				*/
/*  msg		: const QString & : Progress message			*/
/*  (returns)	: void		  :					*/

void	KBLoaderStockDB::setProgress
	(	const QString	&msg
	)
{
	/* This method sets the message and raises the text progress	*/
	/* widget into view.						*/
	m_progressText->setText    (msg) ;
	m_progress    ->raiseWidget(m_progressText) ;
}

/*  KBLoaderStockDB							*/
/*  setProgress	: Set installation progress				*/
/*  progress	: int		: Download progress			*/
/*  total	: int		: Total to download			*/
/*  (returns)	: void		:					*/

void	KBLoaderStockDB::setProgress
	(	int		progress,
		int		total
	)
{
	/* This method sets the progress bar and raises the text	*/
	/* progress widget into view.					*/
	m_progressBar->setProgress (progress, total) ;
	m_progress   ->raiseWidget (m_progressText ) ;
}

void	KBLoaderStockDB::setError
	(	const QString	&error,
		const QString	&details
	)
{
	setProgress	      (error)	;
	m_details->setText    (details) ;
	m_bCancel->setEnabled (true)	;
}

void	KBLoaderStockDB::setError
	(	const KBError	&pError
	)
{
	setError (pError.getMessage(), pError.getDetails()) ;
}

/*  setHTTPError							*/
/*  Display HTTP error							*/
/*  (returns)	: void		:					*/

void	KBLoaderStockDB::setHTTPError ()
{
	QString	msg	= TR("Unrecognised error occurred") ;

	if (m_http != 0)
	{
		switch (m_http->error())
		{
			case QHttp::NoError :
				msg	= TR("No error occured") ;
				break	;

			case QHttp::HostNotFound :
				msg	= TR("Host not found") ;
				break	;

			case QHttp::ConnectionRefused :
				msg	= TR("Connection to host refused") ;
				break	;

			case QHttp::UnexpectedClose :
				msg	= TR("Host unexpectedly closed connection") ;
				break	;

			case QHttp::InvalidResponseHeader :
				msg	= TR("Invalid response from host") ;
				break	;

			case QHttp::WrongContentLength :
				msg	= TR("Host sent wrong content length") ;
				break	;

			case QHttp::Aborted :
				msg	= TR("Transfer aborted") ;
				break	;

			case QHttp::UnknownError :
				msg	= TR("Unknown error occurred") ;
				break	;

			default	:
				break	;
		}

		/* Disconnect signals before deleting, stupid class can	*/
		/* emit signals while inside the desctructor.		*/
		m_http->disconnect   (this  ) ;
		KBDeleter::addObject (m_http) ;
		m_http	= 0	;
	}

	setError (msg) ;
}

/*  KBLoaderStockDB							*/
/*  slotHTTPStarted							*/
/*		: QHttp object signals start of an operation		*/
/*  id		: int		: Operation identifier			*/
/*  (returns)	: void		:					*/

void	KBLoaderStockDB::slotHTTPStarted
	(	int		id
	)
{
	if (id == m_httpSetHostID)
	{
		DPRINTF
		((	"KBLoaderStockDB::slotHTTPStarted: Connecting to host\n"
		))	;
		setProgress (TR("Connecting to remote host")) ;
		return	;
	}
	if (id == m_httpGetID)
	{
		DPRINTF
		((	"KBLoaderStockDB::slotHTTPStarted: retrieveing data\n"
		))	;
		setProgress (TR("Retrieving database specification")) ;
		return	;
	}
}

/*  KBLoaderStockDB							*/
/*  slotHTTPFinished							*/
/*		: QHttp object signals completion of an operation	*/
/*  id		: int		: Operation identifier			*/
/*  error	: bool		: Error occurred			*/
/*  (returns)	: void		:					*/

void	KBLoaderStockDB::slotHTTPFinished
	(	int		id,
		bool		error
	)
{
	/* First case: ::setHost() completes. This is actually the	*/
	/* connection to the server, so we display that (or an error).	*/
	/* If the connection was OK then the QHttp object will move on	*/
	/* automatically to download the database specification.	*/
	if (id == m_httpSetHostID)
	{
		DPRINTF
		((	"KBLoaderStockDB::slotHTTPFinished: Connected to host: err=%d\n",
			error
		))	;
		if (error)
		{	setHTTPError () ;
			return	;
		}
		m_httpSetHostID	= -1 ;
		setProgress (TR("Connected to remote host")) ;
		return	;
	}

	/* Second case: ::get() completes. This is the end of the data	*/
	/* download, possibly with an error.				*/
	if (id == m_httpGetID)
	{
		DPRINTF
		((	"KBLoaderStockDB::slotHTTPFinished: retrieved data: err=%d\n",
			error
		))	;
		if (error)
		{	setHTTPError () ;
			return	;
		}
		m_httpGetID	= -1 ;
		setProgress (TR("Retrieved database specification")) ;

		loadSpecification () ;
		return	;
	}
}

/*  KBLoaderStockDB							*/
/*  slotHTTPProgress							*/
/*		: QHttp object reports progress				*/
/*  done	: int		: Download completed so far		*/
/*  total	: int		: Total download			*/
/*  (returns)	: void		:					*/

void	KBLoaderStockDB::slotHTTPProgress
	(	int		done,
		int		total
	)
{
	DPRINTF
	((	"KBLoaderStockDB::slotHTTPProgress: got %d of %d\n",
		done,
		total
	))	;
	setProgress (done, total) ;
}

void	KBLoaderStockDB::slotHTTPReady
	(	const QHttpResponseHeader	&
	)
{
	m_specification.append (m_http->readAll()) ;
}

/*  KBLoaderStockDB							*/
/*  loadTableData: Load table data					*/
/*  (returns)	 : int		: Number of rows or negative on error	*/

int	KBLoaderStockDB::loadTableData ()
{
	QString		name	= m_nextElement.attribute("name") ;
	KBTableSpec	tabSpec	(name) ;
	if (!m_dbLink.listFields(tabSpec))
	{
		setError (m_dbLink.lastError()) ;
		return	 -1 ;
	}

	/* The work is done using a copier. Create a dummy location and	*/
	/* thence an XML source and an table destination. These can be	*/
	/* appropriately initialised with the table structure.		*/
	KBLocation	location   (m_dbInfo, "copier", m_server, "unnamed") ;
	KBCopyXML	*copyXML   = new KBCopyXML   (true,  location) ;
	KBCopyTable	*copyTable = new KBCopyTable (false, location) ;

	copyXML	 ->setMainTag (name ) ;
	copyXML	 ->setRowTag  ("row") ;
	copyXML	 ->setErrOpt  (KBCopyXML::ErrPad) ;
	copyXML	 ->setElement (m_nextElement) ;

	copyTable->setServer  (m_server) ;
	copyTable->setTable   (name )	 ;
	copyTable->setOption  (KBCopyTable::OptReplace, "") ;


	for (uint idx = 0 ; idx < tabSpec.m_fldList.count() ; idx += 1)
	{
		KBFieldSpec *fSpec = tabSpec.m_fldList.at(idx) ;

		copyXML  ->addField (fSpec->m_name, false) ;
		copyTable->addField (fSpec->m_name) ;
	}

	/* Now create the actual copier and execute it. Most of the	*/
	/* arguments are dummy in this situation.			*/
	KBCopyExec	  copy   (copyXML, copyTable) ;
	QString	   	  report ;
	int	   	  nRows  ;
	QDict<QString>	  d1	 ;
	QDict<KBParamSet> d2	 ;
	KBError		  error	 ;
	if (!copy.execute
		(	report,
			error,
			nRows,
			d1, d2,
			false
		))
	{
		setError (error) ;
		return	 -1	 ;
	}

	return	nRows	;
}

/*  KBLoaderStockDB							*/
/*  loadFileObject : Load file object					*/
/*  (returns)	   : bool	: Success				*/

bool	KBLoaderStockDB::loadFileObject ()
{
	QString	name	= m_nextElement.attribute("name")	;
	QString	path	= m_dbInfo->getDBPath() + "/" + name	;

	DPRINTF
	((	"KBLoaderStockDB::loadTableData: %s: [%s]->[%s]\n",
		(cchar *)m_nextElement.attribute("type"),
		(cchar *)name,
		(cchar *)path
	))	;

	KBFile	file (path) ;
	if (!file.open(IO_WriteOnly|IO_Truncate))
	{
		file.lastError().DISPLAY() ;
		return	false	;
	}

	file.writeBlock (m_nextElement.text().utf8()) ;

	return	true	;
}

/*  KBLoaderStockDB							*/
/*  exec	: Execute loader					*/
/*  (returns)	: void		:					*/

int	KBLoaderStockDB::exec ()
{
	if (!m_dbLink.connect (m_dbInfo, m_server))
	{
		m_dbLink.lastError().DISPLAY() ;
		return 0 ;
	}

	/* See if the specification originates on an HTTP server, in	*/
	/* which case set up to download the specification.		*/
	if (m_url.protocol().lower() == "http")
	{
		m_http		= new QHttp (this) ;

		connect
		(	m_http,
			SIGNAL	(requestStarted	 (int)),
			SLOT	(slotHTTPStarted (int))
		)	;
		connect
		(	m_http,
			SIGNAL	(requestFinished (int, bool)),
			SLOT	(slotHTTPFinished(int, bool))
		)	;
		connect
		(	m_http,
			SIGNAL	(dataReadProgress(int, int)),
			SLOT	(slotHTTPProgress(int, int))
		)	;
		connect
		(	m_http,
			SIGNAL	(readyRead	 (const QHttpResponseHeader &)),
			SLOT	(slotHTTPReady	 (const QHttpResponseHeader &))
		)	;

		DPRINTF
		((	"KBLoaderStockDB::exec: host=[%s] port=[%d] get=[%s]\n",
			(cchar *)m_url.host(),
			m_url.port(),
			(cchar *)m_url.path()
		))	;

		int	port	= m_url.port()	;
		if (port < 0) port = 80		;
		m_httpSetHostID	= m_http->setHost (m_url.host(), port) ;
		m_httpGetID	= m_http->get     (m_url.path()) ;

		/* Download should be set up, so drop into the base	*/
		/* dialog so that the remainder of the opertion		*/
		/* executes inside the modal loop. The completion of	*/
		/* the download will trigger the actual database load.	*/
		return	KBDialog::exec () 	;
	}

	/* Specification must be a local file, so open and read the	*/
	/* file (checking for errors). Then start a single-shot timer	*/
	/* before dropping into the base dialog. This enters the dialog	*/
	/* modal loop, whence the timer will trigger database load.	*/
	KBFile	file (m_url.path()) ;
	if (!file.open(IO_ReadOnly))
	{
		file.lastError().DISPLAY() ;
		return	0 ;
	}

	m_specification.append (file.readAll()) ;
	file.close () ;

	QTimer::singleShot (0, this, SLOT(loadSpecification())) ;
	return	KBDialog::exec () ;
}


void	KBLoaderStockDB::addCommentary
	(	const QString	&comment
	)
{
	m_commentary.append	 (comment)	;
	m_details->setTextFormat (Qt::RichText) ;
	m_details->setText	 ("<qt><ul><li><nobr>" + m_commentary.join("</nobr></li><li><nobr>") + "</nobr></li></ul></qt>") ;
	qApp->processEvents	 () ;
}

/*  KBLoaderStockDB							*/
/*  loadSpecification							*/
/*		: Load database specification				*/
/*  (returns)	: void		:					*/

void	KBLoaderStockDB::loadSpecification ()
{
	/* This method is called repeatedly as the specification is	*/
	/* loaded, so it is in effect a simple state machine ....	*/

	/* If there is no DOM document then this is the first time	*/
	/* that we have been called ....				*/
	if (m_document == 0)
	{
		DPRINTF
		((	"KBLoaderStockDB::loadSpecification:\n%s\n",
			(cchar *)m_specification
		))	;


		/* ... so create and parse the document, then find the	*/
		/* first top-level element node in the specification.	*/
		/* If there is no such element then the specification	*/
		/* is empty.						*/
		m_document = new QDomDocument () ;
		if (!m_document->setContent(m_specification))
		{
			setError (TR("Database specification is not a valid XML document")) ;
			return	 ;
		}

		QDomNode node = m_document->documentElement().firstChild() ;
		while (!node.isNull() && !node.isElement())
			node = node.nextSibling() ;

		if (!node.isElement())
		{
			setError (TR("Database specification is is empty")) ;
			return	 ;
		}

		m_nextElement = node.toElement() ;
	}

	QString	tag	= m_nextElement.tagName() ;

	if (tag == "table")
	{
		bool	exists	;
		KBError	error	;

		if (!m_dbLink.tableExists (m_nextElement.attribute("name"), exists))
		{	setError (m_dbLink.lastError()) ;
			return   ;
		}
		if (!loadTableDef (m_nextElement, exists, false, error))
		{	setError (error) ;
			return	 ;
		}

		QString	text = QString(TR("Created table '%1'"))
					.arg(m_nextElement.attribute("name"))
					;
		setProgress	(text)	;
		addCommentary	(text)	;
		goto nextElement	;
	}

	if (tag == "data")
	{
		int nRows = loadTableData() ;
		if (nRows < 0) return	;

		QString	text = QString(TR("Loaded table '%1', %2 records"))
					.arg(m_nextElement.attribute("name"))
					.arg(nRows)
					;
		setProgress	(text)	;
		addCommentary	(text)	;
		goto nextElement	;
	}

	if (tag == "object")
	{
		if (!loadFileObject()) return ;

		QString	text = QString(TR("Loaded %1 '%2'"))
					.arg(m_nextElement.attribute("type"))
					.arg(m_nextElement.attribute("name"))
					;
		setProgress	(text)	;
		addCommentary	(text)	;
		goto nextElement	;
	}

	setError (TR("Unrecognised node in database specification"), tag) ;
	return	 ;

	nextElement :

		QDomNode node = m_nextElement.nextSibling() ;
		while (!node.isNull() && !node.isElement())
			node = node.nextSibling() ;

		if (!node.isElement())
		{	setProgress (TR("Database loaded")) ;
			m_bOK	 ->setEnabled (true ) ;
			m_bCancel->setEnabled (false) ;
			return	;
		}

		m_nextElement = node.toElement() ;
		QTimer::singleShot (200, this, SLOT(loadSpecification())) ;
}

void	KBLoaderStockDB::accept ()
{
	KBDialog::accept () ;
}

void	KBLoaderStockDB::reject ()
{
	KBDialog::reject () ;
}

