/***************************************************************************
    file	         : kb_loader.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_LOADER_H
#define	__KB_LOADER_H


#include	<qstring.h>
#include	<qstringlist.h>
#include	<qurl.h>
#include	<qhttp.h>
#include	<qmap.h>
#include	<qptrlist.h>

#include	"kb_dbinfo.h"
#include	"kb_dblink.h"
#include	"kb_dialog.h"
#include	"kb_file.h"
#include	"kb_databuffer.h"
#include	"kb_dom.h"


class	QCheckBox	;
class	QLabel		;
class	QHttp		;
class	QProgressBar	;
class	QWidgetStack	;
class	QTextBrowser	;
class	QListViewItem	;

class	RKPushButton	;
class	RKListView	;
class	RKLineEdit	;
class	KBLoaderItem	;

/*  KBLoader								*/
/*  --------								*/
/*  Database loader class, used when bulk loading table definitions	*/
/*  and data. This base class contains common schema and data handling	*/
/*  code.								*/

class	LIBKBASE_API	KBLoader
{
protected :

	KBDBInfo		*m_dbInfo	;
	QString			m_server	;
	KBDBLink		m_dbLink	;

	QMap<QString,QString>	m_forwardMap	;
	QMap<QString,QString>	m_backwardMap	;

	bool			loadXMLSpec
				(	const QString	&,
					cchar		*,
					QDomDocument	&,
					KBError		&
				)	;

	bool			loadTableDef	(const QDomElement &, bool, bool, KBError &) ;
	bool			loadViewDef	(const QDomElement &, bool, KBError &) ;
	bool			loadSequenceDef	(const QDomElement &, bool, KBError &) ;


public	:

	KBLoader	(KBDBInfo *, const QString &) ;
}	;

/*  KBLoaderDlg								*/
/*  -----------								*/
/*  Database loader class, used when bulk loading table definitions	*/
/*  and data. 								*/

class	LIBKBASE_API	KBLoaderDlg : public KBDialog, public KBLoader
{
	Q_OBJECT

	QCheckBox		*m_loadAll	;
	QCheckBox		*m_loadTables	;
	QCheckBox		*m_loadViews	;
	QCheckBox		*m_loadSequences;
	QCheckBox		*m_loadDefn	;
	QCheckBox		*m_loadData	;
	QCheckBox		*m_dropObjects	;
	QCheckBox		*m_dropData	;
	QCheckBox		*m_bestMatch	;
	RKListView		*m_tableList	;

	RKPushButton		*m_mapUpper	;
	RKPushButton		*m_mapLower	;
	RKPushButton		*m_mapUnmap	;
	RKPushButton		*m_mapLoad	;
	RKPushButton		*m_mapSave	;

	QPtrList<QWidget>	m_widgetList	;
	QLabel			*m_loading	;
	QLabel			*m_record	;
	QLabel			*m_of		;
	RKPushButton		*m_bOK		;
	RKPushButton		*m_bCancel	;


	bool			m_cancelled	;

	KB::TableType		m_currentType	;
	KBLoaderItem		*m_currentItem	;

	QListViewItem		*m_popItem	;
	uint			m_index		;
	QString			m_dir		;
	int			m_ldtime	;

	bool			loadTableDef		(const QString &, bool, KBError &) ;
	bool			loadViewDef		(const QString &, bool, KBError &) ;
	bool			loadSequenceDef		(const QString &, bool, KBError &) ;
	bool			loadTableData		(const QString &, bool, KBError &) ;

	void			setupMaps		()	;
	bool			loadDetails		(KBLoaderItem  *, bool &) ;
	void			mapAllCase		(QString (*) (const QString &)) ;

protected slots :

	void			contextMenu		(QListViewItem *, const QPoint &, int)	;
	void			mapName			()	;
	void			unmapName		()	;
	void			mapAllLower		()	;
	void			mapAllUpper		()	;
	void			unmapAll		()	;

	void			saveMapping		()	;
	void			loadMapping		()	;

	void			loadAllChanged		()	;
	void			loadTablesChanged	()	;
	void			loadViewsChanged	()	;
	void			loadSequencesChanged	()	;

	void			slotTimer		()	;
	void			accept 			()	;
	void			reject			()	;

public	:

	KBLoaderDlg	(KBDBInfo *, const QString &, const QString &) ;

	int		exec				()	;
	bool		showProgress			(int)	;
}	;

/*  KBLoaderStockDB							*/
/*  ---------------							*/
/*  Class used when loading stock databses				*/

class	LIBKBASE_API	KBLoaderStockDB : public KBDialog, public KBLoader
{
	Q_OBJECT

	QUrl		m_url			;
	QWidgetStack	*m_progress		;
	RKLineEdit	*m_progressText		;
	QProgressBar	*m_progressBar		;
	QTextBrowser	*m_details		;
	RKPushButton	*m_bOK			;
	RKPushButton	*m_bCancel		;

	QHttp		*m_http			;
	int		m_httpSetHostID		;
	int		m_httpGetID		;

	QString		m_specification		;
	QDomDocument	*m_document		;
	QDomElement	m_nextElement		;

	QStringList	m_commentary		;

	void		setHTTPError	()		  ;
	void		setError	(const KBError &) ;
	void		setError	(const QString &, const QString & = QString::null) ;
	void		setProgress	(const QString &) ;
	void		setProgress	(int, int )	  ;
	void		addCommentary	(const QString &) ;

	int		loadTableData	()		  ;
	bool		loadFileObject	()		  ;

protected slots	:

	void		slotHTTPStarted	(int)		  ;
	void		slotHTTPFinished(int, bool)	  ;
	void		slotHTTPProgress(int, int )	  ;
	void		slotHTTPReady	(const QHttpResponseHeader &) ;

	void		loadSpecification	()	  ;

	void		accept 		()	;
	void		reject		()	;

public	:

	KBLoaderStockDB	(KBDBInfo *, const QString &, const QString &) ;
	~KBLoaderStockDB() ;

	int		exec		()	;
}	;

#define	FL_TABLEDEF	0x01
#define	FL_VIEWDEF	0x02
#define	FL_SEQDEF	0x04
#define	FL_ANYDEF	0x0f
#define	FL_TABLEDATA	0x10
#define	FL_ANYDATA	0xf0

#endif	// __KB_LOADER_H
