/***************************************************************************
    file	         : kb_manual.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qprocess.h>
#include	<qsocket.h>
#include	<qsocketdevice.h>
#include	<qsocketnotifier.h>
#include	<qhostaddress.h>
#include	<qmessagebox.h>
#include	<qdir.h>

#ifdef		Q_OS_WIN32
#include	<io.h>
#endif

#ifdef		Q_OS_LINUX 
#include	<unistd.h>
#endif

#ifdef		Q_OS_MACX
#include	<unistd.h>
#endif

#ifdef		__KB_AUTOMOC
#include	"kb_manual.moc"
#else
#include	"kb_manual.h"
#endif

#include	"kb_classes.h"
#include	"kb_locator.h"


#define		__DEBUG_MANUAL	(0)
#if		__DEBUG_MANUAL
#include	"kb_dprintf.h"
#endif


/*  KBManual								*/
/*  KBManual	: Constructor for help class				*/
/*  (returns)	: KBManual	:					*/

KBManual::KBManual ()
{
	m_helpProcess	= 0 ;
	m_listener	= 0 ;
	m_notify	= 0 ;
	m_comms		= 0 ;
}

KBManual::~KBManual ()
{
}

/*  KBManual								*/
/*  startHelpProcess							*/
/*		: Start help process running				*/
/*  key		: const char *	: Help key to pass to process		*/
/*  (returns)	: void		:					*/

void	KBManual::startHelpProcess
	(	const char	*key
	)
{
	if (m_listener == 0)
	{
		QSocketDevice *sd   = new QSocketDevice () ;
		QHostAddress  addr  ;
		uint	      bound = 0	;

		sd->setAddressReusable (true) ;
		addr.setAddress ("127.0.0.1") ;

		for (uint port = 12000 ; port < 12256 ; port += 1)
			if (sd->bind(addr, port))
			{	bound	= port	;
				break	;
			}

		if (bound == 0)
		{
			DPRINTF
			((	"KBManual::startHelpProcess: failed to bind address\n"
			))	;
			delete	sd	;
			sd	= 0	;
		}

		if (sd != 0)
			if (!sd->listen (10))
			{
				DPRINTF
				((	"KBManual::startHelpProcess: failed to listen\n"
				))	;
				delete	sd	;
				sd	= 0	;
			}

		if (sd != 0)
		{
			m_notify   = new QSocketNotifier (sd->socket(), QSocketNotifier::Read, this) ;
			m_listener = sd		;
			m_port	   = bound	;

			connect
			(	m_notify,
				SIGNAL(activated   (int)),
				SLOT  (slotCommsCalled())
			)	;
		}
	}

	m_helpProcess = new QProcess( this );

#ifdef	Q_OS_LINUX
	m_helpProcess->addArgument (__KB_APPNAME "Manual" ) ;
	m_helpProcess->addArgument ("--helpdir") ;
	m_helpProcess->addArgument (locateDir("appdata", "manual/rekall.xml") + "manual/") ;
#endif
#ifdef	Q_OS_WIN32
	m_helpProcess->addArgument (__KB_APPNAME "Manual" ) ;
	m_helpProcess->addArgument ("--helpdir") ;
	m_helpProcess->addArgument (locateDir("appdata", "manual/rekall.xml") + "manual/") ;
#endif
#ifdef	Q_OS_MACX
	extern 	const QString	&getRootDir () ;
	m_helpProcess->addArgument (getRootDir() + "/../Helpers/RekallManual.app/Contents/MacOS/" __KB_APPNAME "Manual" ) ;
	m_helpProcess->addArgument ("--helpdir") ;
	m_helpProcess->addArgument (locateDir("appdata", "manual/rekall.xml") + "manual/") ;
#endif

	m_helpProcess->addArgument ("--helpkey") ;
	m_helpProcess->addArgument (key) ;


	if (m_notify != 0)
	{
		m_helpProcess->addArgument ("--port") ;
		m_helpProcess->addArgument (QString::number (m_port)) ;
	}

	m_helpProcess->setCommunication (0) ;

	connect
	(	m_helpProcess,
		SIGNAL	(processExited	  ()),
		SLOT	(slotProcessExited())
	)	;

	QStringList args = m_helpProcess->arguments() ;
	for (uint a = 0 ; a < args.count() ; a += 1)
		DPRINTF
		((	"KBManual::startHelpProcess: %3d: [%s]\n",
			a,
			(const char *)args[a]
		))	;

	if (!m_helpProcess->start())
	{
		delete	m_helpProcess ;
		m_helpProcess = 0     ;

		QMessageBox::warning
		(	0,
			"Rekall",
			"Help process failed to start"
		)	;
	}
}

/*  KBManual								*/
/*  showContents: Show the help contents				*/
/*  (returns)	: void		:					*/

void	KBManual::showContents ()
{
	DPRINTF
	((	"KBManual::showContents: process=%p\n",
		(void *)m_helpProcess
	))	;

	if (m_helpProcess == 0)
	{
		startHelpProcess ("$Contents") ;
		return	;
	}

	if (m_comms != 0)
		m_comms->writeBlock("$Contents\n", 10) ;
}

/*  KBManual								*/
/*  showIndex	: Show the help index					*/
/*  (returns)	: void		:					*/

void	KBManual::showIndex ()
{
	if (m_helpProcess == 0)
	{
		startHelpProcess ("$Index") ;
		return	;
	}

	if (m_comms != 0)
		m_comms->writeBlock("$Index\n", 7) ;
}

void	KBManual::slotCommsCalled ()
{
	DPRINTF
	((	"KBManual::slotCommsCalled: m_listener=%p\n",
		(void *)m_listener
	))	;

	if (m_listener == 0)
		return	;

	int	socket	= m_listener->accept() ;
	if (socket == -1)
	{
		DPRINTF
		((	"KBManual::slotCommsCalled: failed to accept\n"
		))	;
		return	;
	}

	if (m_comms != 0)
	{
		::close (socket) ;
		return	;
	}

	m_comms	= new QSocket ()   ;
	m_comms->setSocket (socket);
}

/*  KBManual								*/
/*  slotProcessExited							*/
/*		: Help process exits					*/
/*  (returns)	: void		:					*/

void	KBManual::slotProcessExited ()
{
	if (m_helpProcess != 0)
	{	delete	m_helpProcess ;
		m_helpProcess = 0     ;
	}

	if (m_comms != 0)
	{	delete	m_comms	      ;
		m_comms	= 0	      ;
	}
}

/*  KBManual								*/
/*  slotHelp	: Show specified help					*/
/*  key		: const char *	: Help key				*/
/*  (returns)	: void		:					*/

void	KBManual::slotHelp
	(	const char	*key
	)
{
	DPRINTF
	((	"KBManual::slotHelp: key [%s]\n",
		key
	))	;

	if (m_helpProcess == 0)
	{
		startHelpProcess (key) ;
		return	;
	}

	if (m_comms != 0)
	{
		m_comms->writeBlock (key,  qstrlen(key)) ;
		m_comms->writeBlock ("\n", 1) ;
	}
}


/*  KBManual								*/
/*  slotHelp	: Show help						*/
/*  (returns)	: void		:					*/

void	KBManual::slotHelp ()
{
	/* The help key is the name of the sender object ...		*/
	slotHelp (sender()->name()) ;
}


/*  KBManual								*/
/*  self	: Get help object					*/
/*  (returns)	: KBManual *	: Help object				*/

KBManual
	*KBManual::self()
{
	static	KBManual *kbManual ;
	if (kbManual == 0) kbManual = new KBManual ;
	return	kbManual ;
}


/*  ------------------------------------------------------------------	*/

/*  KBManualPushButton							*/
/*  KBManualPushButton							*/
/*		: Help push button constructor				*/
/*  parent	: QWidget *	     : Parent widget			*/
/*  key		: const char *	     : Help key				*/
/*  manual	: const char *	     : Manual name			*/
/*  (returns)	: KBManualPushButton : Button				*/

KBManualPushButton::KBManualPushButton
	(	QWidget		*parent,
		const char	*key,
		const char	*manual
	)
	:
	QPushButton (parent)
{
	/* The "manual" argument is an optional manual name, so that we	*/
	/* can support multiple manuals. If defaults to "rekall".	*/
	m_name	= qstrdup(QString("%1/%2").arg(manual == 0 ? "rekall" : manual).arg(key).latin1()) ;

	/* Wire the button directly to the help system, whence the	*/
	/* button name is used as the help key.				*/
	connect (this, SIGNAL(clicked()), KBManual::self(), SLOT(slotHelp())) ;
	setText	(TR("&Help")) ;
	setName	(m_name) ;
}

KBManualPushButton::~KBManualPushButton ()
{
	delete	[] m_name ;
}

