/***************************************************************************
    file	         : kb_memo.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<ctype.h>


#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>
#include	<qcursor.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_sizer.h"
#include	"kb_display.h"
#include	"kb_ctrl.h"
#include	"kb_formblock.h"

#if		__KB_AUTOMOC
#include	"kb_memo.moc"
#else
#include	"kb_memo.h"
#endif

#include	"kb_qtmemo.h"
#include	"kb_nodereg.h"
#include	"kb_writer.h"
#include	"kb_layout.h"
#include	"kb_font.h"

#include	"kb_locator.h"

#include	"kb_dialog.h"

#if		! __KB_RUNTIME
#include	"kb_recorder.h"
#endif


/*  KBMemo								*/
/*  KBMemo	: Constructor for simple form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBNode	:					*/

KBMemo::KBMemo
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBItem	 	(parent, "KBMemo",	"expr", aList),
	m_fgcolor	(this,   "fgcolor",	aList),
	m_bgcolor	(this,   "bgcolor",	aList),
	m_frame	 	(this,	 "frame",	aList),
	m_font	 	(this,	 "font",	aList),
	m_nullOK	(this,	 "nullok",	aList, 	KAF_FORM),
	m_hilite	(this,	 "hilite",	aList,	KAF_FORM),
	m_wrap	 	(this,	 "wrap",	aList,	KAF_FORM),
	m_emptyNull	(this,	 "emptynull",	aList,	KAF_FORM),
	m_mapCase	(this,	 "mapcase",	aList,  KAF_FORM),
	m_focusCaret	(this,	 "focuscaret",	aList,  KAF_FORM),
	m_onChange 	(this,	 "onchange",	aList,	KAF_FORM|KAF_EVCS)
{
#if	! __KB_RUNTIME
	if (ok != 0)
	{
		if (!::memoPropDlg (this, "Memo", m_attribs))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif
	if (isReportDoc())
		m_report = getParent()->getReport () ;
	else	m_report = 0 ;
}

/*  KBMemo								*/
/*  KBMemo	: Constructor for simple form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  memo	: KBMemo *	: Extant memo				*/
/*  (returns)	: KBMemo	:					*/

KBMemo::KBMemo
	(	KBNode		*parent,
		KBMemo		*memo
	)
	:
	KBItem	 	(parent, "expr",	memo),
	m_fgcolor	(this,   "fgcolor",	memo),
	m_bgcolor	(this,   "bgcolor",	memo),
	m_frame	 	(this,	 "frame",	memo),
	m_font	 	(this,	 "font",	memo),
	m_nullOK	(this,	 "nullok",	memo),
	m_hilite	(this,	 "hilite",	memo),
	m_wrap	 	(this,	 "wrap",	memo),
	m_emptyNull	(this,	 "emptynull",	memo),
	m_mapCase	(this,	 "mapcase",	memo),
	m_focusCaret	(this,	 "focuscaret",	memo),
	m_onChange 	(this,	 "onchange",	memo)
{
	if (isReportDoc())
		m_report = getParent()->getReport () ;
	else	m_report = 0 ;
}

/*  KBMemo								*/
/*  ~KBMemo	: Destructor for simple form field			*/
/*  (returns)	:		:					*/

KBMemo::~KBMemo ()
{
}

/*  KBMemo								*/
/*  makeCtrl	: Make a field control					*/
/*  drow	: uint		: Display row number			*/
/*  (returns)	: KBControl *	: New control				*/

KBControl
	*KBMemo::makeCtrl
	(	uint		drow
	)
{
	return	new KBCtrlMemo (getDisplay(), this, drow) ;
}

/*  KBMemo								*/
/*  doLeave	: Handle focus exit					*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: bool		: OK to leave				*/

bool	KBMemo::doLeave
	(	uint	qrow
	)
{
	/* Handle the onChange event here since we don't do it on a	*/
	/* keystroke by keystroke basis.				*/
	KBControl *ctrl	 = ctrlAtQRow (qrow) ;
	if ((ctrl != 0) && ctrl->changed ())
	{
#if		! __KB_RUNTIME
		KBValue	cval = getValue(qrow) ;
		switch (getMapCase())
		{	case KBField::MapUpper :
				cval	= KBValue(cval.getRawText().upper(), cval.getType()) ;
				break	;
			case KBField::MapLower :
				cval	= KBValue(cval.getRawText().lower(), cval.getType()) ;
				break	;
			default	:
				break	;
		}
		recordUpdateValue (qrow, cval) ;
#endif
		KBValue	args[2]	;
		bool	evRc	;

		args[0] = (int)qrow	 		;
		args[1] = getValue (qrow)		;

		eventHook (m_onChange, 2, args, evRc)	;

		KBFormBlock *fBlk = getFormBlock()	;
		if (fBlk != 0) fBlk->dataChanged (qrow) ;
	}

	return	KBItem::doLeave (qrow) ;
}

/*  KBMemo								*/
/*  doCheckValid: Check whether value can be saved in database		*/
/*  value	: const QString & : Putative value			*/
/*  allowNull	: bool		  : Ignore not-null check		*/
/*  (returns)	: bool		  : True if OK				*/

bool	KBMemo::doCheckValid
	(	const QString	&value,
		bool		allowNull
	)
{
	KBError	err	;

	if (!allowNull)
		if (value.isEmpty () && !m_nullOK.getBoolValue())
		{	
			setError
			(	KBError::Error,
			  	QString (TR("Memo %1 may not be empty")).arg(getName()),
				QString::null,
			  	__ERRLOCN
			)	;
			return	false	;
		}

	/* If the value is not empty or we are doing not-null checks	*/
	/* then see if the value is OK to be saved to the database.	*/
	if (!value.isEmpty() || !allowNull)
		if (!m_type->isValid (value, err, getName()))
		{
			setError (err) ;
			return	 false ;
		}

	return	true	;
}

/*  KBMemo								*/
/*  checkValid	: Check whether value can be saved in database		*/
/*  value	: const KBValue & : Putative value			*/
/*  allowNull	: bool		  : Ignore not-null check		*/
/*  (returns)	: bool		  : True if OK				*/

bool	KBMemo::checkValid
	(	const KBValue	&value,
		bool		allowNull
	)
{
	/* Special case. Ig the value is empty and empty values are to	*/
	/* be treated as null, then validate it as such.		*/
	if (value.isEmpty() && m_emptyNull.getBoolValue())
		return	doCheckValid (QString::null, allowNull) ;

	return	doCheckValid (value.getRawText(), allowNull) ;
}

/*  KBMemo								*/
/*  getValue	: Get value						*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBMemo::getValue
	(	uint	qrow
	)
{
	/* Get the control for the specified row. If there isn't one	*/
	/* (outside the displayed range) then return a null.		*/
	KBControl *ctrl = ctrlAtQRow (qrow) ;
	if (ctrl == 0) return KBValue()	    ;

	/* Get the value. If this is empty and empty values are to be	*/
	/* treated as nulls then also return a null ...			*/
	KBValue	value	= ctrl->getValue()  ;
	if (value.dataLength() == 0)
		if (m_emptyNull.getBoolValue())
			return	KBValue()   ;

	/* ... otherwise use the value as-is.				*/
	return	value	;
}

/*  KBMemo								*/
/*  replicate	: Replicate this field					*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New field node			*/

KBNode	*KBMemo::replicate
	(	KBNode		*parent
	)
{
	return	new KBMemo (parent, this) ;
}

/*  KBMemo								*/
/*  canBeInvalid: Check if control can be invalid			*/
/*  (returns)	: bool		: True if so				*/

bool	KBMemo::canBeInvalid ()
{
	return	! m_nullOK.getBoolValue() ;
}

bool	KBMemo::isWrapping ()
{
	return	m_wrap.getBoolValue() ;
}

KBField::MapCase
	KBMemo::getMapCase ()
{
	return	(KBField::MapCase)m_mapCase.getIntValue() ;
}

KBField::FocusCaret
	KBMemo::getFocusCaret ()
{
	return	(KBField::FocusCaret)m_focusCaret.getIntValue() ;
}


#if	! __KB_RUNTIME
/*  KBMemo								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar *	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBMemo::propertyDlg
	(	cchar	*iniAttr
	)
{
	if (memoPropDlg (this, "Memo", m_attribs, iniAttr))
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}
#endif

NEWNODE(Memo, __TR("New &Memo"), 0, KNF_FORM|KNF_REPORT|KNF_BLOCK|KNF_DATA)



// void	timedMsg
// 	(	const QString	&message
// 	)
// {
// #ifndef	Q_OS_WIN32
// 	struct	timeval	tv ;
// 	gettimeofday (&tv, 0) ;
// 	fprintf	     (stderr, "%09d.%06d: %s\n", tv.tv_sec, tv.tv_usec, (cchar *)message) ;
// #endif
// }
