/***************************************************************************
    file	         : kb_memodlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#if		__KB_AUTOMOC
#include	"kb_memodlg.moc"
#else
#include 	"kb_memodlg.h"
#endif

#include	"kb_highlighter.h"
#include	"rk_combobox.h"


extern	IntChoice	choiceMapCase	[] ;
extern	IntChoice	choiceFocusCaret[] ;


/*  KBMemoPropDlg							*/
/*  KBMemoPropDlg: Constructor for memo properties dialog		*/
/*  memo	 : KBMemo *	 : Parent memo				*/
/*  caption	 : const char *  : Dialog box caption			*/
/*  attribs	 : QPtrList<KBAttr>&					*/
/*				 : List of attributes			*/
/*  apply	 : bool	  	 : Show apply button			*/
/*  iniAttr	 : cchar *	 : Initial attribute			*/
/*  (returns)	 : KBMemoPropDlg :					*/

KBMemoPropDlg::KBMemoPropDlg
	(	KBMemo		*memo,
		cchar		*caption,
		QPtrList<KBAttr>	&attribs,
		cchar		*iniAttr
	)
	:
	KBItemPropDlg	(memo, caption, attribs, iniAttr),
	m_memo		(memo)
{
}

/*  KBMemoPropDlg							*/
/*  ~KBMemoPropDlg: Destructor for memo properties dialog		*/
/*  (returns)	  : void	:					*/

KBMemoPropDlg::~KBMemoPropDlg ()
{
}

/*  KBMemoPropDlg							*/
/*  getAttrItem	: Get attribute item for attribute			*/
/*  attr	: KBAttr *	: Attribute				*/
/*  (returns)	: KBAttrItem *	: Associated attribute item		*/

KBAttrItem
	*KBMemoPropDlg::getAttrItem
	(	KBAttr	*attr
	)
{
	if (attr->getName() == "mapcase"   )
		return	new KBAttrIntChoice (attr, choiceMapCase   ) ;
	if (attr->getName() == "focuscaret")
		return	new KBAttrIntChoice (attr, choiceFocusCaret) ;

	return	KBItemPropDlg::getAttrItem (attr) ;
}

/*  KBMemoPropDlg							*/
/*  showProperty: Show property						*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property on show			*/

bool	KBMemoPropDlg::showProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName	= item->attr()->getName () ;

	/* Hilite							*/
	/* Display the combobox with a list of possible highlights	*/
	if (aName == "hilite")
	{
		m_comboBox->clear	     ()	;
		m_comboBox->insertItem	     ("") ;
		m_comboBox->insertStringList (KBSyntaxHighlighter::highlightNames()) ;

		for (int idx = 0 ; idx < m_comboBox->count() ; idx += 1)
			if (m_comboBox->text(idx) == item->value())
			{	m_comboBox->setCurrentItem(idx) ;
				break	;
			}

		m_comboBox->show    () ;
		m_comboBox->setFocus() ;
		return	true	  ;
	}

	if (aName == "mapcase")
	{
		showChoices (item, choiceMapCase,    item->value()) ;
		return	true ;
	}

	if (aName == "focuscaret")
	{
		showChoices (item, choiceFocusCaret, item->value()) ;
		return	true ;
	}

	return	KBItemPropDlg::showProperty (item) ;
}

/*  KBMemoPropDlg							*/
/*  saveProperty: Check and save attribute				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Attribute OK				*/

bool	KBMemoPropDlg::saveProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Hilite							*/
	/* Value is in the combo box					*/
	if (aName == "hilite")
	{
		setProperty (item, m_comboBox->currentText()) ;
		return	true	;
	}

	if (aName == "mapcase")
	{
		saveChoices (item, choiceMapCase   ) ;
		return	true	;
	}
	if (aName == "focuscaret")
	{
		saveChoices (item, choiceFocusCaret) ;
		return	true	;
	}

	return	KBItemPropDlg::saveProperty (item) ;
}

/*  KBMemoPropDlg							*/
/*  hideProperty: Check whether property should be hidden		*/
/*  attr	: KBAttr *	: Attribute in question			*/
/*  (returns)	: void		:					*/

bool	KBMemoPropDlg::hideProperty
	(	KBAttr	*attr
	)
{
	return	false	;
}


/*  memoPropDlg	: Run the memo properties dialog			*/
/*  memo	: KBMemo *	: Parent memo				*/
/*  caption	: const char *  : Dialog box caption			*/
/*  attribs	: QPtrList<KBAttr>&: List of attributes			*/
/*  iniAttr	: cchar *	: Initial attribute			*/
/*  (returns)	: bool	  	: Success				*/

bool	memoPropDlg
	(	KBMemo		*memo,
		cchar		*caption,
		QPtrList<KBAttr>	&attribs,
		cchar		*iniAttr
	)
{
	KBMemoPropDlg mDlg (memo, caption, attribs, iniAttr) ;
	return	mDlg.exec () ;
}
