/***************************************************************************
    file	         : kb_methdict.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qdir.h>
#include	<qregexp.h>


#include	"kb_classes.h"
#include	"kb_methdict.h"
#include	"kb_locator.h"
#include	"kb_dom.h"

#define		__DEBUG_METHDICT	(0)
#if		__DEBUG_METHDICT
#include	"kb_dprintf.h"
#endif

#if		__KB_KDE

#include	<klocale.h>
#include	<kglobal.h>

#endif		// __KB_KDE

class KBMethDictEntryList : public QPtrList<KBMethDictEntry>
{
protected :

	int	compareItems (QCollection::Item, QCollection::Item) ;

}	;

KBMethDictArg::KBMethDictArg
	(	const QDomElement	&elem
	)
{
	m_name		= elem.attribute("name"   ) ;
	m_type		= elem.attribute("type"   ) ;
	m_comment	= elem.attribute("comment") ;
	m_opt		= elem.attribute("opt"	  ) == "Yes" ;

	DPRINTF
	((	"KBMethDictArg::KBMethDictArg: for [%s]\n",
		m_name.latin1()
	))	;
}

/*  ------------------------------------------------------------------	*/

KBMethDictEntry::KBMethDictEntry
	(	const QString		&klass,
		const QDomElement	&method
	)
{
	m_class   = klass ;
	m_name	  = method.attribute("name"   ) ;
	m_comment = method.attribute("comment") ;

	DPRINTF
	((	"KBMethDictEntry::KBMethDictEntry: [%s][%s]\n",
		m_class.latin1(),
		m_name .latin1()
	))	;

	DOMITER_ELEM_BEGIN(method, child)
	{
		if (child.tagName() == "return")
		{
			m_return = child.attribute ("type")  ;
			continue ;
		}
		if (child.tagName() == "arg"  )
		{
			m_args.append (KBMethDictArg(child)) ;
			continue ;
		}
		if (child.tagName() == "description")
		{
			for
			(	QDomNode n = child.firstChild() ;
				!n.isNull() ;
				n = n.nextSibling()
			)
				m_description += n.toText().data() ;
			continue ;
		}
	}
	DOMITER_END
}

QString	KBMethDictEntry::asText
	(	bool		rich
	)
{
	QString	 d ;

	if (rich)
	{
		d.append("<b>"    ) ;
		d.append (m_return) ;
		d.append("</b>"   ) ;
		d.append (" "	  ) ;
		d.append("<i>"    ) ;
		d.append (m_class ) ;
		d.append("</i>"   ) ;
		d.append ("::"	  ) ;
	}

	d.append (m_name) ;
	d.append ("(") ;

	int optcnt = 0 ;
	for (uint idx = 0 ; idx < m_args.count() ; idx += 1)
	{
		const KBMethDictArg &arg = m_args[idx] ;
		d.append (idx > 0 ? ", " : "") ;
		if (arg.m_opt)
		{	d.append ("[")	;
			optcnt += 1	;
		}
		if (rich) d.append("<b>" ) ;
		d.append (arg.m_type)	;
		if (rich) d.append("</b>") ;
		d.append (" ")		;
		d.append (arg.m_name)	;
	}
	while (optcnt > 0)
	{	d.append ("]")	;
		optcnt -= 1	;
	}
	d.append (")")	;
	return	 d	;		

}

/*  KBMethDictEntry							*/
/*  prototype	: Get text as a prototype				*/
/*  (returns)	: QString	:					*/

QString	KBMethDictEntry::prototype  ()
{
	return	asText (true )	;
}

/*  KBMethDictEntry							*/
/*  substitute	: Get text for substitution				*/
/*  (returns)	: QString	:					*/

QString	KBMethDictEntry::substitute ()
{
	return	asText (false)	;
}

/*  KBMethDictEntry							*/
/*  description	: Get description					*/
/*  (returns)	: QString	:					*/

QString	KBMethDictEntry::description ()
{
	return	m_description	;
}

/*  KBMethDictEntry							*/
/*  name	: Get method name					*/
/*  (returns)	: QString	:					*/

const QString &KBMethDictEntry::name ()
{
	return	m_name	;
}

/*  KBMethDictEntry							*/
/*  klass	: Get class to which method applies			*/
/*  (returns)	: QString	:					*/

const QString &KBMethDictEntry::klass ()
{
	return	m_class	;
}


/*  KBMethDictEntry							*/
/*  compare	: List comparisom routine				*/
/*  other	: ...		: Other list entry			*/
/*  (returns)	: int		: Ordering value			*/

int	KBMethDictEntry::compare
	(	const KBMethDictEntry	*other
	)
{
	return	QString::localeAwareCompare (m_name, other->m_name) ;
}


/*  ------------------------------------------------------------------	*/

/*  KBMethDictEntryList							*/
/*  compareItems: Compare items in list for ordering			*/
/*  item1	: QCollection::Item	: First item			*/
/*  item2	: QCollection::Item	: Second item			*/
/*  (returns)	: int		: Ordering value			*/

int	KBMethDictEntryList::compareItems
	(	QCollection::Item	item1,
		QCollection::Item	item2
	)
{
	return	((KBMethDictEntry *)item1)->compare ((KBMethDictEntry *)item2) ;
}

/*  ------------------------------------------------------------------	*/

/*  KBMethDict								*/
/*  KBMethDict	: Dictionary constructor				*/
/*  dictPath	: const QString	& : Dictionary path			*/
/*  baseName	: const QString	  : Dase dictionary name		*/
/*  (returns)	: KBMethDict	  :					*/

KBMethDict::KBMethDict
	(	const QString	&dictPath,
		const QString	&baseName
	)
{
	QString	fullPath	;
	QDir	dictDir		;

	fullPath = locateDir ("appdata", QString("dict/%1/%2.dict").arg(dictPath).arg(baseName)) ;
	fullPath = QString   ("%1/dict/%2").arg(fullPath).arg(dictPath) ;

	DPRINTF
	((	"KBMethDict::KBMethDict: [%s][%s] -> [%s]\n",
		dictPath.latin1(),
		baseName.latin1(),
		fullPath.latin1()
	))	;

	dictDir.setPath		(fullPath) ;
	dictDir.setNameFilter	("*.dict") ;
	dictDir.setFilter	(QDir::Files|QDir::Readable) ;
	dictDir.setSorting	(QDir::Name) ;

	const QFileInfoList   *dictInfo = dictDir.entryInfoList() ;
	if (dictInfo == 0)
	{
		fprintf
		(	stderr,
			"KBMethDict::KBMethDict: no dictionary entries\n"
		)	;
		return	;
	}

	QFileInfoListIterator dictIter (*dictInfo) ;
	QFileInfo *fi ;

#if	__KB_KDE
	KLocale	*locale	= KGlobal::locale () ;
	DPRINTF
	((	"KBMethDict::KBMethDict: [%s][%s] language [%s]\n",
		dictPath.latin1(),
		baseName.latin1(),
		locale->language().latin1()
	))	;
#endif

	while ((fi = dictIter.current()) != 0)
	{
		QString	name	= fi->filePath() ; 
		loadFile (name)	;

#if	__KB_KDE
		QString	name2	= fi->dirPath () + "/" + locale->language() + "/" + fi->fileName() ;
		if (QFileInfo(name2).exists()) loadFile (name2) ;
#endif
		dictIter += 1 ;
	}
}

/*  KBMethDict								*/
/*  loadFile	: Load a dictionary file				*/
/*  name	: const QString & : File name				*/
/*  (returns)	: void		  :					*/

void	KBMethDict::loadFile
	(	const QString	&name
	)
{
	DPRINTF
	((	"KBMethDict::loadFile: [%s]\n",
		(cchar *)name
	))	;

	KBDomDocument	dXML	;
	if (!dXML.loadFile (name, 0, 0))
	{
		dXML.lastError().DISPLAY() ;
		return	;
	}

	DOMITER_BEGIN(dXML.documentElement(), "class", klass)
	{
		DPRINTF
		((	"KBMethDict::loadFile: class [%s]\n",
			klass.attribute("name").latin1()
		))	;
		m_classes.insert (klass.attribute("name"), klass) ;
	}
	DOMITER_END
}

/*  KBMethDict								*/
/*  getMethods	: Get matching methods					*/
/*  klass	: const QString & : Object class to scan		*/
/*  prefix	: const QString & : Method prefix			*/
/*  complete	: bool		  : Prefix is complete name		*/
/*  mode	: const QString & : Mode F(orm) or R(report)		*/
/*  (returns)	: QPtrList<KBMethDictEntry>				*/
/*				  : Matched method			*/

QPtrList<KBMethDictEntry>
	KBMethDict::getMethods
	(	const QString		&klass,
		const QString		&prefix,
		bool			complete,
		const QString		&mode
	)
{
	QPtrList<KBMethDictEntry>	matched	;
	QString	scan			= klass	;
	QRegExp				pattern	;

	QString	pfx	= prefix.isEmpty() ? ".*" : prefix ;

	if (complete)
		pattern.setPattern ("^" + pfx + "$") ;
	else	pattern.setPattern ("^" + pfx) ;

	while (!scan.isEmpty ())
	{
		DPRINTF
		((	"KBMethDict::getMethods: klass=[%s] scan=[%s][%s]\n",
			klass. latin1(),
			scan   .latin1(),
			prefix.latin1()
		))	;

		KBMethDictEntryList entries ;

		QDomElement elem = m_classes[scan] ;
		if (elem.isNull()) break ;

		if (elem.attribute("mode").isEmpty() || (elem.attribute("mode") == mode))
		{
			DOMITER_BEGIN(elem, "method", method)
			{
				QString		name   = method.attribute("name") ;
				bool		found	= false	;

				if (pattern.search (name) != 0)
						continue ;

				for
				(	KBMethDictEntry *e = matched.first() ;
					e != 0 ;
					e = entries.next()
				)
					if (e->name() == name)
					{
						found	= true	;
						break	;
					}

				if (found)
					continue ;

				QString		tag    = QString("%1::%2").arg(scan).arg(name) ;
				KBMethDictEntry *entry = m_entries.find (tag) ;

				if (entry == 0)
				{
					entry	= new KBMethDictEntry (scan, method) ;
					m_entries.insert (tag, entry) ;
				}

				entries.append	(entry) ;
			}
			DOMITER_END

			entries.sort() ;

			for (KBMethDictEntry *e = entries.first() ; e != 0 ; e = entries.next())
				matched.append (e) ;
		}

		scan = elem.attribute ("super") ;
	}

	return	matched	;
}
	
