/***************************************************************************
    file	         : kb_mimefactory.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include 	"kb_mimefactory.h"

#include	"kb_docroot.h"
#include	"kb_locator.h"
#include	"kb_file.h"

#include	<qimage.h>
#include	<qdragobject.h>



KBMimeSourceFactory::KBMimeSourceFactory
	(	KBDocRoot	*docRoot
	)
{
	m_docRoot	= docRoot ;
}

KBMimeSourceFactory::~KBMimeSourceFactory ()
{
}


/*  KBMimeSourceFactory							*/
/*  data	: Get data for specified source				*/
/*  abs_name	: const QString &     : Source				*/
/*  (returns)	: const QMimeSource * : Mime data			*/

const QMimeSource
	*KBMimeSourceFactory::data
	(	const QString	&abs_name
	)
	const
{
	fprintf
	(	stderr,
		"KBMimeSourceFactory::data: called for [%s]\n",
		(cchar *)abs_name
	)	;

	/* The QT documentation says that the result is only valid for	*/
	/* one call, so we can safely use a single static pointer over	*/
	/* all instances of this class. Note that we *cannot* make this	*/	
	/* class a singleton since we must store the document root.	*/
	static	QDragObject *lastObject ;
	if (lastObject != 0)
	{	delete lastObject ;
		lastObject = 0	  ;
	}

	/* If the name starts "graphic://" then interpret the rest as	*/
	/* the name of a local graphic. This is used to provide images	*/
	/* inside things like wizard blurbs.				*/
	if (abs_name.left(10).lower() == "graphic://")
	{
		QString	path	= locateFile ("appdata", QString("graphics/%1").arg(abs_name.mid(10))) ;
		if (path.isEmpty())
			return	0 ;

		KBFile	image	(path) ;
		if (!image.open (IO_ReadOnly))
		{
			image.lastError().DISPLAY() ;
			return	0 ;
		}

		lastObject = new QImageDrag(QImage(image.readAll())) ;
		return	lastObject ;
	}

	/* If the name starts with "http://" then it is an internal	*/
	/* link which will trigger an event in a rich text control.	*/
	if (abs_name.left(7).lower() == "http://")
		return	0 ;

	/* Otherwise it should be a graphic stored in the database. Use	*/
	/* the document root (checking that we have one) to identify	*/
	/* the database, then get the graphic.				*/
	if (m_docRoot != 0)
	{
		QStringList	iparts	= QStringList::split ('.', abs_name) ;
		KBLocation	location
		(	m_docRoot->getDBInfo  (),
			"graphic",
			m_docRoot->getDocLocation().server(),
			iparts[0],
			iparts[1]
		)	;

		KBError	   error	;
		QByteArray image	;
		if (!location.contents (image, error))
		{
			error.DISPLAY() ;
			return	0	;
		}

		lastObject = new QImageDrag(QImage(image)) ;
		return	lastObject ;
	}

	return	0 ;
}

