/***************************************************************************
    file	         : kb_moduleitem.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#if		__KB_AUTOMOC
#include	"kb_moduleitem.moc"
#else
#include	"kb_moduleitem.h"
#endif

#include	"kb_module.h"
#include	"kb_docroot.h"
#include	"kb_dbdociter.h"

#include	"rk_vbox.h"
#include	"rk_pushbutton.h"
#include	"rk_combobox.h"


/*  KBModuleItem							*/
/*  KBModuleItem: Constructor for extant module item			*/
/*  listBox	: RKListBox *	: Display listbox			*/
/*  module	: KBModule *	: Module				*/
/*  (returns)	: KBModuleItem	:					*/

KBModuleItem::KBModuleItem
	(	RKListBox	*listBox,
		KBModule	*module
	)
	:
	QListBoxText (listBox, module->getModuleName()),
	module	     (module)
{
}

/*  KBModuleItem							*/
/*  KBModuleItem: Constructor for new module item			*/
/*  listBox	: RKListBox *	: Display listbox			*/
/*  module	: const QString&: Module name				*/
/*  (returns)	: KBModuleItem	:					*/

KBModuleItem::KBModuleItem
	(	RKListBox	*listBox,
		const QString	&name
	)
	:
	QListBoxText (listBox, name),
	module	     (0)
{
}

/*  KBModuleItem							*/
/*  KBModuleItem: Destructor for module item				*/
/*  (returns)	:		:					*/

KBModuleItem::~KBModuleItem ()
{
}

/*  KBModuleItem							*/
/*  fixUp	: Fix up script item on dialog acceptance		*/
/*  root	: KBNode *	  : Owning root block			*/
/*  (returns)	: void		  :					*/

void	KBModuleItem::fixUp
	(	KBNode	*node
	)
{
	if (module == 0) module = makeModule (node, text()) ;
}


/*  ------------------------------------------------------------------  */

/*  KBModuleDlg								*/
/*  KBModuleDlg	: Module dialog constructor				*/
/*  parent	: QWidget *	: Parent widget				*/
/*  node	: KBNode *	: Parent node				*/
/*  editable	: bool		: Make selector editable		*/
/*  (returns)	: KBModuleDlg	:					*/

KBModuleDlg::KBModuleDlg
	(	QWidget		*parent,
		KBNode		*node,
		bool		editable,
		const QString	&language
	)
	:
	RKHBox		(parent),
	m_node		(node),
	m_language	(language)
{
	RKVBox  *layLeft = new RKVBox   (this) ;

	m_cbModule	 = new RKComboBox   (layLeft) ;
	m_bAdd		 = new RKPushButton (TR("Add >>"   ), layLeft) ;
	m_bRemove	 = new RKPushButton (TR("<< Remove"), layLeft) ;
	layLeft->addFiller () ;

	/* Module entry is via an editable combo box. This is loaded	*/
	/* with the database script modules; it may be editable so that	*/
	/* library modules cab be loaded.				*/
	if (editable)
		m_cbModule->setEditable (true) ;

	const KBLocation &locn	 = node->getDocRoot()->getDocLocation() ;
	KBDBInfo	 *dbInfo = node->getDocRoot()->getDBInfo     () ;

	KBDBDocIter docIter (false) ;
	KBError	    error   ;

	if (docIter.init
		(	dbInfo,
			locn.server(),
			"script",
			m_language,
			error
		))
	{
		QString	    name    ;
		QString	    stamp   ;

		m_cbModule->insertItem ("") ;
		while (docIter.getNextDoc (name, stamp))
			m_cbModule->insertItem (name) ;
	}
	else
		error.DISPLAY() ;

	m_vModules	 = new RKListBox    (this) ;

	m_bRemove->setEnabled(false)	;
	
	connect	(m_bAdd,     SIGNAL(clicked()), SLOT(clickAdd   ())) ;
	connect	(m_bRemove,  SIGNAL(clicked()), SLOT(clickRemove())) ;

	connect
	(	m_vModules,
		SIGNAL(highlighted(int)),
		SLOT  (highlighted(int))
	)	;
}

/*  KBModuleDlg								*/
/*  ~KBModuleDlg: Module dialog destructor				*/
/*  (returns)	:		:					*/

KBModuleDlg::~KBModuleDlg ()
{
}

/*  KBModuleDlg								*/
/*  setModules	: Set list of extant modules				*/
/*  modules	: QPtrList<KBModule> & : List of extant modules		*/
/*  (returns)	: void		    :					*/

void	KBModuleDlg::setModules
	(	QPtrList<KBModule>	&modules
	)
{
	LITER
	(	KBModule,
		modules,
		m,
		makeModuleItem (m_vModules, m)
	)
}

/*  KBModuleDlg:							*/
/*  clickAdd	 : Handle click of add button				*/
/*  (returns)	 : void		:					*/

void	KBModuleDlg::clickAdd ()
{
	if (!m_cbModule->currentText().isEmpty())
	{	KBModuleItem *item = makeModuleItem (m_vModules, m_cbModule->currentText()) ;
		m_vModules->setCurrentItem (item) ;
		m_cbModule->setEditText    (QString::null) ;
	}
}

/*  KBModuleDlg:							*/
/*  clickRemove	 : Handle click of remove button			*/
/*  (returns)	 : void		:					*/

void	KBModuleDlg::clickRemove ()
{
	KBModuleItem	*module	;
	KBModule	*m	;

	if ((module = (KBModuleItem *)m_vModules->item(m_vModules->currentItem())) != 0)
	{
		if ((m = module->getModule()) != 0)
			m_dropped.append (m) ;

		m_vModules->removeItem (m_vModules->currentItem()) ;
		m_bRemove ->setEnabled (m_vModules->currentItem() >= 0) ;
	}
}

/*  KBModuleDlg:							*/
/*  highlighted	 : Handle highlight of modules list entry		*/
/*  (returns)	 : void		:					*/

void	KBModuleDlg::highlighted
	(	int	index
	)
{
	m_bRemove ->setEnabled  (true) ;
	m_vModules->setSelected (index, true) ;
}

/*  KBModuleDlg	:							*/
/*  fixUp	: Fix up when used OK's changes				*/
/*  (returns)	: void		:					*/

void	KBModuleDlg::fixUp ()
{
	LITER (KBModule, m_dropped, m, delete m) ;

	for (uint idx = 0 ; idx < m_vModules->count() ; idx += 1)
		((KBModuleItem *)m_vModules->item(idx))->fixUp (m_node) ;
}


QString	KBModuleDlg::getText ()
{
	QString	res	;

	for (uint idx = 0 ; idx < m_vModules->count() ; idx += 1)
	{	if (!res.isEmpty()) res += ", " ;
		res += m_vModules->text (idx) ;
	}

	return	res	;
}

