/***************************************************************************
    file	         : kb_navigator.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_formblock.h"
#include	"kb_framer.h"
#include	"kb_docroot.h"

#include	"kb_navigator.h"
#include	"kb_grid.h"

#include	"kb_attrdict.h"
#include	"kb_parse.h"
#include	"kb_locator.h"
#include	"kb_component.h"
#include	"kb_display.h"
#include	"kb_config.h"
#include	"kb_locator.h"
#include	"kb_ascii.h"

#if		! __KB_RUNTIME
#include	"kb_wizard.h"
#include	"kb_toolbox.h"
#include	"kb_recorder.h"
#include	"kb_nodereg.h"
#include	"kb_dbinfo.h"
#endif


#define		__DEBUG_NAVIGATOR	(0)
#if		__DEBUG_NAVIGATOR
#include	"kb_dprintf.h"
#endif


/*  tabCanEnter	: See if control can be entered on tab			*/
/*  obj		: KBObject *	: Object in question			*/
/*  qrow	: uint		: Query row number			*/
/*  (returns)	: bool		: True if entry OK			*/

static	bool	tabCanEnter
	(	KBObject	*obj,
		uint		qrow
	)
{
	/* If the object is an item then check that the item is	visible	*/
	/* and enabled for the sepecified row ...			*/
	KBItem	*item	;

	DPRINTF
	((	"tabCanEnter[%p][%s] item=[%p]\n",
		(void  *)obj,
		(cchar *)obj->getName(),
		(void  *)obj->isItem ()
	))	;
	if ((item = obj->isItem()) != 0)
		return	item->isVisible (qrow) && item->isEnabled (qrow) ;

	/* ... and otherwise just check the object as a whole.		*/
	return	obj->isVisible() && obj->isEnabled() ;
}


/*  KBTabOrderList							*/
/*  compareItems  : Ordering function using tab order values		*/
/*  item1	  : KBItem *	: First item				*/
/*  item2	  : KBItem *	: Second item				*/
/*  (returns)	  : int		: Ordering value			*/

int	KBTabOrderList::compareItems
	(	QCollection::Item	item1,
		QCollection::Item	item2
	)
{
	return	(int)((KBObject *)item1)->getTabOrder() -
		(int)((KBObject *)item2)->getTabOrder() ;
}

/*  KBTabOrderList							*/
/*  getFirst	  : Get first object ignoring hidden and disabled items	*/
/*  qrow	  : uint	: Query row number			*/
/*  (returns)	  : KBObject *	: First object or null if none		*/

KBObject
	*KBTabOrderList::getFirst
	(	uint	qrow
	)
{
	for (KBObject *obj = first() ; obj != 0 ; obj = next())
		if (tabCanEnter (obj, qrow))
			return	obj ;

	return	0 ;
}

/*  KBTabOrderList							*/
/*  getPrev	  : Get previous object ignoring hidden and disabled	*/
/*  qrow	  : uint	: Query row number			*/
/*  (returns)	  : KBObject *	: Object or null if none		*/

KBObject
	*KBTabOrderList::getPrev
	(	uint	qrow
	)
{
	for (KBObject *obj = prev() ; obj != 0 ; obj = prev())
		if (tabCanEnter (obj, qrow))
			return	obj ;

	return	0 ;
}

/*  KBTabOrderList							*/
/*  getNext	  : Get next object ignoring hidden and disabled	*/
/*  qrow	  : uint	: Query row number			*/
/*  (returns)	  : KBObject *	: Object or null of none		*/

KBObject
	*KBTabOrderList::getNext
	(	uint	qrow
	)
{
	for (KBObject *obj = next() ; obj != 0 ; obj = next())
		if (tabCanEnter (obj, qrow))
			return	obj ;

	return	0 ;
}

/*  KBTabOrderList							*/
/*  getLast	  : Get last object ignoring hidden and disabled items	*/
/*  qrow	  : uint	: Query row number			*/
/*  (returns)	  : KBObject *	: Object or null if none		*/

KBObject
	*KBTabOrderList::getLast
	(	uint	qrow
	)
{
	for (KBObject *obj = last() ; obj != 0 ; obj = prev())
		if (tabCanEnter (obj, qrow))
			return	obj ;

	return	0 ;
}



/*  ------------------------------------------------------------------	*/

/*  KBNavigator								*/
/*  KBNavigator	: Constructor for navigator object			*/
/*  owner	: KBObject *	  : Owing object			*/
/*  block	: KBBlock *	  : Containing block			*/
/*  childList	: QPtrList<KBNode> & : Block child list			*/
/*  (returns)	: KBNavigator	  :					*/

KBNavigator::KBNavigator
	(	KBObject		*owner,
		KBBlock			*block,
		QPtrList<KBNode>	&childList
	)
	:
	m_owner		(owner),
	m_block		(block),
	m_childList	(childList)
{
}

KBFormBlock
	*KBNavigator::formBlock ()
{
	return	m_block->isFormBlock() ;
}

/*  KBNavigator								*/
/*  findGrid	: Find grid control if any				*/
/*  (returns)	: KBGrid *	: Grid or null				*/

KBGrid	*KBNavigator::findGrid ()
{
	TITER
	(	Grid,
		m_childList,
		grid,
		return	grid
	)

	return	0 ;
}

/*  KBNavigator								*/
/*  setupGridLayout							*/
/*		: Set up items in grid layout if present		*/
/*  (returns)	: void		:					*/

void	KBNavigator::setupGridLayout ()
{
	KBGrid	*grid	;
	if ((grid = findGrid ()) != 0)
	{
		grid->clearItems (true) ;

		LITER
		(	KBObject,
			m_tabList,
			obj,
			if (obj->isItem() != 0)
				grid->appendItem  (obj->isItem(), true) ;
		)
	}

}

void	KBNavigator::adjustGridLayout ()
{
	KBGrid	*grid	;
	if ((grid = findGrid ()) != 0) grid->adjustItems () ;
}

/*  KBNavigator								*/
/*  orderGridByExpr: Order grid columns by expression (or not ...)	*/
/*  byExpr	   : bool	: Order by expression			*/
/*  (returns)	   : void	:					*/

void	KBNavigator::orderGridByExpr
	(	bool	byExpr
	)
{
	KBGrid	*grid	= findGrid () ;
	if (grid != 0) grid->orderByExpr (byExpr) ;
}

/*  KBNavigator								*/
/*  setupTabOrder							*/
/*		: Fix tab ordering of control items			*/
/*  (returns)	: void		:					*/

void	KBNavigator::setupTabOrder ()
{
	m_tabList.clear () ;

	TITER
	(	Object,
		m_childList,
		o,
		if (o->isBlock() == 0)
			if (o->getTabOrder() > 0)
				m_tabList.inSort (o) ;
	)
}

/*  KBNavigator								*/
/*  firstItem	: Get the first item					*/
/*  (returns)	: KBItem *	: First item				*/

KBItem	*KBNavigator::firstItem ()
{
	KBItem	*first	= 0 ;

	/* If there are any items on the tab order list we use the	*/
	/* first, otherwise pick the first amongst the children other	*/
	/* than blocks.							*/
	if (m_tabList.count() == 0)
	{
		TITER
		(	Item,
			m_childList,
			i,
			if (i->isBlock() == 0)
			{
				first	= i ;
				break	;
			}
		)
	}
	else	for (uint idx = 0 ; idx < m_tabList.count() ; idx += 1)
			if (m_tabList.at(idx)->isItem() != 0)
			{
				first	= m_tabList.at(idx)->isItem() ;
				break	;
			}

	/* If we still do not have an item then look through any	*/
	/* containers ...						*/
	if (first == 0)
		TITER
		(	Framer,
			m_childList,
			f,
			if ((first = f->firstItem()) != 0) break ;
		)

	return	first	;
}

#if	! __KB_RUNTIME

/*  KBNavigator								*/
/*  newTabOrder	: Invoke tab order dialog				*/
/*  (returns)	: bool		: Success				*/

bool	KBNavigator::newTabOrder ()
{
	QPtrList<KBObject>	objList ;

	TITER
	(	Object,
		m_childList,
		o,
		if (o->getTabOrder() >= 0) objList.append (o) ;
	)

	return	::tabOrderDlg (m_block, objList) ;
}

void	KBNavigator::installNewNode
	(	KBNode		*node,
		KBDisplay	*blkDisp
	)
{
	/* If the node is an object then get it displayed. This is	*/
	/* a bit of a hack since the node might be a container (for	*/
	/* instance, a tabber) hence the container check and subsequent	*/
	/* show.							*/
	KBObject *obj ;
	if ((obj = node->isObject()) != 0)
	{
		obj->buildDisplay  (blkDisp) ;

		if (obj->getContainer   ()) obj->getContainer()->show() ;
		if (obj->getTabOrder() > 0) m_tabList.append (obj) ;

		obj->getBlock()->redoControls () ;
	}

	node->showAs (KB::ShowAsDesign)	   ;
	m_owner->getLayout()->setChanged() ;

	/* Last thing, if the node is an object then add it to the	*/
	/* design layout manager, which will get the sizer blobs up.	*/
	if (obj != 0)
		m_owner->getLayout()->addSizer (obj->getSizer(), true) ;
}

/*  KBNavigator								*/
/*  newNode	: Create a new node					*/
/*  spec	: NodeSpec *	  : Specification form new node		*/
/*  cRect	: QRect		  : Geometry for new node		*/
/*  wrapper	: KBObject *	  : 					*/
/*  blkDist	: KBDisplay *	  : Associated display object		*/
/*  bState	: Qt::ButtonState : Popup menu button state		*/
/*  (returns)	: KBNode *	  : New node or null			*/

KBNode	*KBNavigator::newNode
	(	NodeSpec	*nodeSpec,
		QRect		cRect,
		KBObject	*wrapper,
		KBDisplay	*blkDisp,
		Qt::ButtonState	bState
	)
{
	extern	KBNode	*makeCtrlFromWizard
		(	KBNode		*parent,
			KBQryBase	*query,
			NodeSpec	*nodeSpec,
			KBAttrDict	&aDict,
			bool		&cancel
		)	;

	KBAttrDict	aDict	(0) 	;
	bool		ok		;
	KBNode		*node	= 0	;
	bool		cancel	= false	;

	DPRINTF
	((	"KBNavigator::newNode: %s\n",
		_TEXT(cRect)
	))	;

	aDict.addValue (cRect) ;
	aDict.addValue ("taborder", m_tabList.count () + 1) ;

	/* If using wizards then try that first. If this does not	*/
	/* yield a new node then either (a) there was no wizard or (b)	*/
	/* was an error or (c) the user cancelled. In any case, if	*/
	/* the cancel flag is not set then access the node factory	*/
	/* function directly and see if that yields a node,		*/
	if (KBToolBox::useWizard (bState))
		node	= makeCtrlFromWizard
				(	m_owner,
					m_block == 0 ? 0 : m_block->getQuery(),
					nodeSpec,
					aDict,
					cancel
				)	;

	if (node == 0)
		if (!cancel)
		{
			node	= nodeSpec->m_nodeFunc (m_owner, aDict, &ok) ;
			if (!ok) return 0 ;
		}
		else	return	0 ;

	installNewNode (node, blkDisp) ;
	return	node	;
}

KBTableInfo
	*KBNavigator::getTableInfo
	(	const QString		&table
	)
{
	fprintf
	(	stderr,
		"KBNavigator::getTableInfo: block=%p query=%p\n",
		(void *)m_block,
		m_block == 0 ? (void *)0 : (void *)m_block->getQuery()
	)	;

	/* Paranoia check for a block, then ensure that we have a	*/
	/* query, and that it is a data query.				*/
	if (m_block == 0)
		return	0 ;

	KBQryBase *qryBase = m_block->getQuery () ;
	if ((qryBase == 0) || (qryBase->isQryData() == 0)) return 0 ;

	/* Get the server specified in the query. If there is no server	*/
	/* (which would be KBQryQuery) or the server is "Self" then get	*/
	/* the server used for the form.				*/
	QString	  server   = qryBase->getAttrVal ("server") ;
	if (server.isEmpty() || (server == "Self"))
		server	= m_block->getDocRoot()->getDocLocation().server() ;

	fprintf
	(	stderr,
		"KBNavigator::getTableInfo: server=[%s] table=[%s]\n",
		server.latin1(),
		table .latin1()
	)	;

	return	m_block->getDocRoot()->getDBInfo()->findTableInfo (server, table) ;
}

/*  KBNavigator								*/
/*  newNode	: Create a new node					*/
/*  fieldSpec	: KBFieldSpec &	  : Specification form new node		*/
/*  table	: const QString & : Table name				*/
/*  cRect	: QRect		  : Geometry for new node		*/
/*  wrapper	: KBObject *	  : 					*/
/*  blkDist	: KBDisplay *	  : Associated display object		*/
/*  bState	: Qt::ButtonState : Popup menu button state		*/
/*  (returns)	: KBNode *	  : New node or null			*/

KBNode	*KBNavigator::newNode
	(	const KBFieldSpec	&fieldSpec,
		const QString		&table,
		QRect			cRect,
		KBObject		*wrapper,
		KBDisplay		*blkDisp,
		Qt::ButtonState		bState
	)
{
	fprintf
	(	stderr,
		"KBNavigator::newNode: %s/%s\n",
		fieldSpec.m_name.latin1(),
		table.latin1()
	)	;

	/* Get the table and field names. If the field name contains a	*/
	/* period then it comes from a query, in which case it is in	*/
	/* the form "table.column" so split and and use those values.	*/
	QString	    useTable = table ;
	QString	    useField = fieldSpec.m_name ;
	QStringList useBits  = QStringList::split ('.', fieldSpec.m_name) ;
	if (useBits.count() > 1)
	{	useTable = useBits[0] ;
		useField = useBits[1] ;
	}

	fprintf
	(	stderr,
		"KBNavigator::newNode: %s: [%s][%s]\n",
		_TEXT(cRect),
		useTable.latin1(),
		useField.latin1()
	)	;

	/* See if there is any column information for the column and	*/
	/* table; if so then we can pick up stuff like link information	*/
	/* and formatting.						*/
	KBTableInfo   *tabInfo	= getTableInfo (useTable) ;
	KBTableColumn *colInfo  = tabInfo == 0 ? 0 : tabInfo->getColumn (useField) ;
	QString	      link	;
	QString	      format	;

	if (colInfo != 0)
	{
		link	= colInfo->designValue (TI_LINK  ) ;
		format	= colInfo->designValue (TI_FORMAT) ;
	}

	fprintf
	(	stderr,
		"KBNavigator::newNode:: tabInfo=%p colInfo=%p: l=[%s] f=[%s]\n",
		(void *)tabInfo,
		(void *)colInfo,
		link  .latin1(),
		format.latin1()
	)	;


	/* Assemble a settings dictionary for the new contain and pick	*/
	/* which component to use ....					*/
	KBAttrDict	aDict	(0) 	;
	aDict.addValue (cRect) ;
	aDict.addValue ("taborder", m_tabList.count () + 1) ;
	aDict.addValue ("name",    fieldSpec.m_name) ;


	QString	compName ;
	bool	cancel	 ;

	if (!link.isEmpty())
	{
		QStringList bits = QStringList::split(':', link) ;
		compName = "Controls/LinkTable" ;
		aDict.addValue ("master", 	fieldSpec.m_name) ;
		aDict.addValue ("table", 	bits[0]		) ;
		aDict.addValue ("child", 	bits[1]		) ;
		aDict.addValue ("show", 	bits[2]		) ;
	}
	else
	{
		compName = "Controls/Field" ;
		aDict.addValue ("expr",		fieldSpec.m_name) ;
		aDict.addValue ("format",	format		) ;
	}

	/* Paranoia check, then create and install the control ....	*/
	if (compName.isEmpty()) return 0 ;

	extern	KBNode	*makeCtrlFromComponent
			(	KBNode		*parent,
				const QString	&compName,
				KBAttrDict	&aDict,
				bool		&cancel
			)	;
	KBNode	*node = makeCtrlFromComponent
			(	m_owner,
				compName,
				aDict,
				cancel
			)	;

	installNewNode (node, blkDisp) ;
	return	node	;
}

#endif


/*  KBNavigator								*/
/*  goNext	: Locate next item on tab ordering			*/
/*  curObj	: KBObject *	: Current object			*/
/*  set		: bool		: True to set focus			*/
/*  (returns)	: KBObject *	: Next object or null if none		*/

KBObject
	*KBNavigator::goNext
	(	KBObject	*curObj,
		bool		set
	)
{
	KBItem	 *item	;
	KBObject *res	;
	KBObject *obj	;

	DPRINTF
	((	"KBNavigator::goNext[%p]: [%s][%d] at %d\n",
		(void  *)this,
		(cchar *)curObj->getName(),
		set,
		m_block->getCurQRow()
	))	;

	if (m_tabList.findRef (curObj) >= 0)
	{
		/* First stage is to scan the objects in increasing	*/
		/* tab order (and which are visible and enabled).	*/
		/* Descend into any nested containers, and finish as	*/
		/* soon as we find the next item.			*/
		while ((obj = m_tabList.getNext(m_block->getCurQRow())) != 0)
		{
			DPRINTF
			((	"KBNavigator::goNext[%p]: ... [%s][%s]\n",
				(void  *)this,
				(cchar *)obj->className(),
				(cchar *)obj->getName  ()
			))	;
			if (obj->isFramer() != 0)
			{
				if ((res = obj->isFramer()->goFirst(set)) != 0)
					return	res	;

				continue  ;
			}

			if ((item = obj->isItem()) != 0)
			{
				if (set) formBlock()->moveFocusTo (item) ;
				return	item	;
			}

			obj->setFocus ();
			return	obj	;
		}
	}

	/* No next item at this or lower levels in the hierarchy. If	*/
	/* this is *not* the block level then go back up a level and	*/
	/* search for the next item relative to this object.		*/
	if (m_block != m_owner)
	{
		KBNode		*parent	= m_owner->getParent() ;
		KBFramer	*framer	;
		KBFormBlock	*fblock	;

		DPRINTF
		((	"KBNavigator::goNext[%p]: up  [%s][%s]\n",
			(void  *)this,
			(cchar *)parent->isObject()->className(),
			(cchar *)parent->isObject()->getName  ()
		))	;
		if ((framer = parent->isFramer   ()) != 0)
			if ((res = framer->goNext (m_owner, set)) != 0)
				return	res	;

		if ((fblock = parent->isFormBlock()) != 0)
			if ((res = fblock->goNext (m_owner, set)) != 0)
				return	res	;
	}

	/* No next item and this is the block level, which means	*/
	/* wrapping to the first item. If we are not setting focus or	*/
	/* the block is set to wrap on tabs, then just locate said	*/
	/* item; otherwise use the block-level NextForward action to	*/
	/* move row at the same time.					*/
	DPRINTF
	((	"KBNavigator::goNext[%p]: next (wrap %d) at %d\n",
		(void  *)this,
		formBlock()->tabsWrap  (),
		m_block	   ->getCurQRow()
	))	;
	if (!set || formBlock()->tabsWrap())
		return goFirst (formBlock()->tabsWrap()) ;

	if (!formBlock()->doAction (KB::NextForw, &m_tabList))
		m_block->lastError().DISPLAY() ;

	return	formBlock()->getCurItem () ;
}

/*  KBNavigator								*/
/*  goPrevious	: Locate previois item on tab ordering			*/
/*  curObj	: KBObject *	: Current object			*/
/*  set		: bool		: True to set focus			*/
/*  (returns)	: KBObject *	: Next object or null if none		*/

KBObject
	*KBNavigator::goPrevious
	(	KBObject	*curObj,
		bool		set
	)
{
	KBItem	 *item	;
	KBObject *res	;
	KBObject *obj	;

	if (m_tabList.findRef (curObj) >= 0)
	{
		while ((obj = m_tabList.getPrev(m_block->getCurQRow())) != 0)
		{
			if (obj->isFramer())
			{
				if ((res = obj->isFramer()->goLast(set)) != 0)
					return	res	;

				continue  ;
			}

			if ((item = obj->isItem()) != 0)
			{
				if (set) formBlock()->moveFocusTo (item) ;
				return	item	;
			}

			obj->setFocus ();
			return	obj	;
		}
	}

	if (m_block != m_owner)
	{
		KBNode		*parent	= m_owner->getParent() ;
		KBFramer	*framer	;
		KBFormBlock	*fblock	;

		if ((framer = parent->isFramer   ()) != 0)
			if ((res = framer->goPrevious (m_owner, set)) != 0)
				return	res	;

		if ((fblock = parent->isFormBlock()) != 0)
			if ((res = fblock->goPrevious (m_owner, set)) != 0)
				return	res	;
	}

	if (!set || formBlock()->tabsWrap())
		return	goLast (formBlock()->tabsWrap()) ;

	if (!formBlock()->doAction (KB::PrevBack, &m_tabList))
		formBlock()->lastError().DISPLAY() ;

	return	formBlock()->getCurItem () ;
}

/*  KBNavigator								*/
/*  goFirst	: Locate first item on tab ordering			*/
/*  set		: bool		: True to set focus			*/
/*  (returns)	: KBObject *	: Next object or null if none		*/

KBObject
	*KBNavigator::goFirst
	(	bool		set
	)
{
	KBObject *res	;
	KBItem	 *item	;

	for (KBObject *obj  = m_tabList.getFirst(formBlock()->getCurQRow()) ;
		       obj != 0 ;
		       obj  = m_tabList.getNext (formBlock()->getCurQRow()) )
	{
		DPRINTF
		((	"KBNavigator::goFirst: check [%p][%s]\n",
			(void  *)obj,
			(cchar *)obj->getName()
		))	;
		if (obj->isFramer())
		{
			if ((res = obj->isFramer()->goFirst(set)) != 0)
				return	res	;

			continue  ;
		}

		if ((item = obj->isItem()) != 0)
		{
			if (set) formBlock()->moveFocusTo (item) ;
			return	item	;
		}

		/* Unlike ::goNext and ::goPrevious, ::goFirst does not	*/
		/* include objects other than framers (eg., buttons).	*/
		/* This simplifies the logic in KBBlock::doOperation,	*/
		/* and in any case is probably the right thing to do.	*/
	}

	DPRINTF
	((	"KBNavigator::goFirst: nothing in %d!!\n",
		m_tabList.count()
	))	;
	return	0 ;
}

/*  KBNavigator								*/
/*  goLast	: Locate last item on tab ordering			*/
/*  set		: bool		: True to set focus			*/
/*  (returns)	: KBObject *	: Next object or null if none		*/

KBObject
	*KBNavigator::goLast
	(	bool		set
	)
{
	KBObject *res	;
	KBItem	 *item	;

	for (KBObject *obj  = m_tabList.getLast(m_block->getCurQRow()) ;
		       obj != 0 ;
		       obj  = m_tabList.getPrev(m_block->getCurQRow()) )
	{
		if (obj->isFramer())
		{
			if ((res = obj->isFramer()->goLast(set)) != 0)
				return	res	;

			continue  ;
		}

		if ((item = obj->isItem()) != 0)
		{
			if (set) formBlock()->moveFocusTo (item) ;
			return	item	;
		}

		/* See comment in ::goFirst above re ignoring object	*/
		/* other than framers.					*/
	}

	return	0 ;
}

/*  KBNavigator								*/
/*  keyStroke	: Control has received keystroke			*/
/*  item	: KBItem *	: Item in question			*/
/*  k		: QKeyEvent *	: Key event				*/
/*  (returns)	: bool		: Key consumed				*/

bool	KBNavigator::keyStroke
	(	KBItem		*item,
		QKeyEvent	*k
	)
{
	int		key	= k->key ()	;
	bool		shift	= (k->state () & Qt::ShiftButton  ) != 0 ;
	bool		ctrl	= (k->state () & Qt::ControlButton) != 0 ;
	bool		oov	= false		;
	KBFormBlock	*fblk	= formBlock ()	;

	/* Item is zero if the control with (notional) focus is out	*/
	/* of view, so get the current control from the block.		*/
	if (item == 0)
	{
		if ((item = fblk->getCurItem ()) == 0)
			return	false	;

		oov	= true	;
	}


	/* Locate the specified item in the tab order list. Abandon	*/
	/* the operation if not found, otherwise this means we can use	*/
	/* next and previous on the list.				*/
	// if (m_tabList.findRef (item) < 0) return false ;


	/* First check for keys that we are interested in, and do some	*/
	/* initial checks.						*/
	switch (key)
	{
		case Qt::Key_Shift	:
		case Qt::Key_Control	:
		case Qt::Key_Meta	:
		case Qt::Key_Alt	:
		case Qt::Key_CapsLock	:
		case Qt::Key_NumLock	:
		case Qt::Key_ScrollLock	:
		case Qt::Key_Pause	:
		case Qt::Key_Print	:
		case Qt::Key_SysReq	:
		case Qt::Key_Super_L	:
		case Qt::Key_Super_R	:
		case Qt::Key_Menu	:
		case Qt::Key_Hyper_L	:
		case Qt::Key_Hyper_R	:
		case Qt::Key_Help	:
		case Qt::Key_Direction_L:
		case Qt::Key_Direction_R:
		case Qt::Key_unknown	:
			/* Ignore all the modifier keys when on their	*/
			/* own, and a number of others.			*/
			return	false	;

		case Qt::Key_Tab	:
		case Qt::Key_Backtab	:
		case Qt::Key_Up		:
		case Qt::Key_Down	:
		case Qt::Key_Escape	:
		case Qt::Key_Prior	:
		case Qt::Key_Next	:
			/* These are all navigation keys ...		*/
			break	;

		case Qt::Key_Enter	:
		case Qt::Key_Return	:
			/* ... while enter/return will either navigate	*/
			/* or save depending on the control key.	*/
			break		;

		case Qt::Key_Left	:
		case Qt::Key_Right	:
		case Qt::Key_Plus	:
			/* This do navigation if the control key is	*/
			/* down ...					*/
			if (!ctrl) return false ;
			break	;

		case Qt::Key_F		:
			/* Ctrl-F brings up the find dialog for those	*/
			/* controls that support it.			*/
			if (ctrl)
			{
				item->doSearch	() ;
				return	true	;
			}

			if (oov)
				fblk->scrollToRow (fblk->getCurQRow()) ;
			return	false	;

		case Qt::Key_A		:
			/* Ctrl-A is used to mark all rows in the	*/
			/* block.					*/
			if (ctrl)
			{
				fblk->setRowMarked (0, KB::MarkOpSetAll) ;
				return	true	;
			}

			if (oov)
				fblk->scrollToRow (fblk->getCurQRow()) ;
			return	false	;

		default	:
			/* All other keystrokes are passed back for	*/
			/* default handling; scroll into view if needed	*/
			if (oov)
				fblk->scrollToRow (fblk->getCurQRow()) ;
			return	false	;
	}

	DPRINTF
	((	"KBNavigator::keyStroke: key=%d\n",
		key
	))	;

#if		! __KB_RUNTIME
	KBRecorder *recorder = KBRecorder::self() ;
	if ((recorder != 0) && recorder->isRecording(item->getDocRoot()))
	{
		KBBlock	*block	= item->getBlock() ;

		recorder->keyNavigation
		(	item,
			block->getCurQRow() - block->getCurDRow(),
			k
		)	;
	}
#endif

	switch (key)
	{
		case Qt::Key_Tab	:
			/* Shift-tab is mapped into back-tab ...	*/
			if (shift) key = Qt::Key_Backtab ;
			break	;

		case Qt::Key_Enter	:
		case Qt::Key_Return	:
			/* These are save if the control key is down ..	*/
			if (ctrl)
			{
				if (!fblk->doAction (KB::Save, &m_tabList))
					fblk->lastError().DISPLAY() ;
				return	true  ;
			}

			/* ... or the same as Tab otherwise		*/
			break		;

		case Qt::Key_Prior	:
			/* Handle previous page where since we do not	*/
			/* need to scroll if the current item is out of	*/
			/* view ....					*/
			if (!fblk->doAction (KB::PrevPage, &m_tabList))
				fblk->lastError().DISPLAY() ;
			return	true  ;

		case Qt::Key_Next	:
			/* .... and similarly next page.		*/
			if (!fblk->doAction (KB::NextPage, &m_tabList))
				fblk->lastError().DISPLAY() ;
			return	true  ;

		case Qt::Key_Plus	:
			if (!fblk->doOperation (KB::GotoQRow, fblk->getNumRows(), &m_tabList))
				fblk->lastError().DISPLAY() ;
			return	true  ;

		default	:
			break	;
	}


	if (oov)
		fblk->scrollToRow (fblk->getCurQRow()) ;

	/* If there is a current item and it has changed then mark the	*/
	/* form as changed.						*/
	fblk->markChanged () ;


	/* Now comes the key-specific stuff. Where control is passed to	*/
	/* "doAction" we do not need any more checking here.		*/
	switch (key)
	{
		case Qt::Key_Tab    :
		case Qt::Key_Enter  :
		case Qt::Key_Return :
			/* Get the next item in the tab order ....	*/
			QFocusEvent::setReason (QFocusEvent::Tab    ) ;
			goNext	(item, true) 	;
			return	true	;

		case Qt::Key_Backtab :
			/* .... or the previous one			*/
			QFocusEvent::setReason (QFocusEvent::Backtab) ;
			goPrevious (item, true) ;
			return	true	;

		case Qt::Key_Left :
			/* Move to the first control in the row. We	*/
			/* know the control key is down.		*/
			fblk->goFirst (true) ;
			break	;

		case Qt::Key_Right  :
			/* ... and similarly to the last. Again, the	*/
			/* control key is down.				*/
			fblk->goLast  (true) ;
			break	;

		case Qt::Key_Up :
			/* Either move to the previous row, if any, or	*/
			/* move to the first row, depending on control	*/
			/* key state.					*/
			if (!fblk->doAction
				(	ctrl ? KB::First : KB::Previous,
					&m_tabList
			   )	)
				fblk->lastError().DISPLAY() ;
			return	true ;

		case Qt::Key_Down :
			/* Similarly down, which is either next row or	*/
			/* last row.					*/
			if (!fblk->doAction
				(	ctrl ? KB::Last : KB::Next,
					&m_tabList
			   )	)
				fblk->lastError().DISPLAY() ;
			return	true  ;

		case Qt::Key_Escape :
			/* This invokes the row-reset function to set	*/
			/* the controls back to the last saved state.	*/
			if (!fblk->doAction (KB::Reset, &m_tabList))
				fblk->lastError().DISPLAY() ;
			return	true  ;

		default	:
			return	false ;
	}

	/* If control gets here then we are switching controls within	*/
	/* the same row.						*/
	fblk->moveFocusTo (item) ;
	return	true ;
}


/*  KBNavigator								*/
/*  isNavigationKey							*/
/*		: See if keystroke is a navigation event		*/
/*  k		: QKeyEvent *	: Keystroke event			*/
/*  (returns)	: bool		: True for navigation			*/

bool	KBNavigator::isNavigationKey
	(	QKeyEvent 	*k
	)
{
	/* This static method can be called to test if a key event is	*/
	/* a navigation event; this is needed for classes like		*/
	/* TKComboBox in order to correctly filter key events when in	*/
	/* read-only mode.						*/
	int	key	= k->key () ;
	bool	ctrl	= (k->state () & Qt::ControlButton) != 0 ;

	switch (key)
	{
		case Qt::Key_Prior	:
		case Qt::Key_Next	:
		case Qt::Key_Tab	:
		case Qt::Key_Backtab	:
		case Qt::Key_Up		:
		case Qt::Key_Down	:
		case Qt::Key_Escape	:
			/* These are all navigation keys ...		*/
			return	true	;

		case Qt::Key_Enter	:
		case Qt::Key_Return	:
		case Qt::Key_Left	:
		case Qt::Key_Right	:
			/* These are navigation keys if the control key	*/
			/* is down ...					*/
			return	ctrl	;

		case Qt::Key_F		:
			/* Ctrl-F brings up the find dialog for those	*/
			/* controls that support it.			*/
			return	ctrl	;

		case Qt::Key_A		:
			/* Ctrl-A is used to mark all rows in the	*/
			/* block.					*/
			return	ctrl	;

		default	:
			/* All other keystrokes are passed back for	*/
			/* default handling; scroll into view if needed	*/
			break	;
	}

	return	false	;
}

