/***************************************************************************
    file	         : kb_navigator.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_NAVIGATOR_H
#define	__KB_NAVIGATOR_H

#include	"libkbase_exports.h"

class	KBTableInfo	;
class	KBFieldSpec	;


/*  KBTabOrderList							*/
/*  --------------							*/
/*  Wrapper class for QPtrList<KBObject> to provide an ordering function.	*/

class	LIBKBASE_API	KBTabOrderList	: public QPtrList<KBObject>
{
protected :

	int	compareItems (QCollection::Item, QCollection::Item) ;

public	:

	KBObject	*getFirst (uint) ;
	KBObject	*getPrev  (uint) ;
	KBObject	*getNext  (uint) ;
	KBObject	*getLast  (uint) ;
}	;



/*  KBNavigator								*/
/*  -----------								*/
/*  This class is used to handle control navigation in forms. It is	*/
/*  separated out into a class since it is needed both for blocks and	*/
/*  for containers.							*/

class	LIBKBASE_API	KBNavigator
{
	KBObject		*m_owner	;
	KBBlock			*m_block	;
	QPtrList<KBNode>	&m_childList	;

	KBGrid			*findGrid 	()	   ;
	KBFormBlock		*formBlock	()	   ;
	void			installNewNode	(KBNode *, KBDisplay *) ;

#if	! __KB_RUNTIME
	KBTableInfo		*getTableInfo	(const QString &)	;
#endif

protected :

	KBTabOrderList		m_tabList	;

public	:

	KBNavigator	(KBObject *, KBBlock *, QPtrList<KBNode> &) ;

	void		setupGridLayout	()	;
	void		setupTabOrder	()	;

	void		adjustGridLayout()	;
	void		orderGridByExpr	(bool)	;

#if	! __KB_RUNTIME

	bool		newTabOrder	()	;
	KBNode		*newNode
			(	NodeSpec 		*,
				QRect			,
				KBObject		*,
				KBDisplay		*,
				Qt::ButtonState
			)	;

	KBNode		*newNode
			(	const KBFieldSpec 	&,
				const QString		&,
				QRect			,
				KBObject		*,
				KBDisplay		*,
				Qt::ButtonState
			)	;

#endif
	KBItem		*firstItem	()	;

	KBObject	*goNext		(KBObject *,  bool) ;
	KBObject	*goPrevious	(KBObject *,  bool) ;
	KBObject	*goFirst	(bool)	;
	KBObject	*goLast		(bool)	;

	bool		keyStroke	(KBItem *, QKeyEvent *) ;

	inline	KBTabOrderList	*getTabList ()
	{
		return	&m_tabList	;
	}

	static	bool	isNavigationKey	(QKeyEvent *) ;
}	;


#if	! __KB_RUNTIME

extern 	LIBKBASE_API	bool	tabOrderDlg 
			(	KBBlock 	*,
				QPtrList<KBObject>	&
			)	;

#endif
#endif	// __KB_NAVIGATOR_H
