/***************************************************************************
    file	         : kb_node.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_node.moc"
#else
#include	"kb_node.h"
#endif

#include	"kb_param.h"
#include	"kb_config.h"
#include	"kb_block.h"
#include	"kb_nodemonitor.h"
#include	"kb_slot.h"
#include	"kb_test.h"
#include	"kb_docroot.h"
#include	"kb_noobjdlg.h"

#if		! __KB_RUNTIME
#include	"kb_propdlg.h"
#endif

#include	"rk_listview.h"

#define		__DEBUG_NODE	(0)
#if		__DEBUG_NODE
#include	"kb_dprintf.h"
#endif

extern	LIBKBASE_API	uint GetNodeFlags(const QString &);


/*  KBNode								*/
/*  KBNode	: Constructor for node base class			*/
/*  parent	: KBNode *	: Parent node if any			*/
/*  element	: cchar *	: Element tag				*/
/*  (returns)	: KBNode	:					*/

KBNode::KBNode
	(	KBNode	*parent,
		cchar	*element
	)
	:
	m_parent	(parent),
	m_element	(element),
	m_root		(0),
	m_attrNotes	(0),
	m_showing	(KB::ShowAsUnknown),
	m_attrCnt	(0),
	m_flags		(GetNodeFlags (m_element)),
	m_monitor	(0),
	m_name		(this,		"name",		"")
{
	if (m_parent != 0)
	{	m_root = m_parent->getRoot () ;
		m_parent->addChild (this) ;
	}
	else	m_root	= this	;

	m_attrNotes	= new KBAttrStr (this, "notes", "", KAF_NOTES) ;

	m_slotList.setAutoDelete (true) ;
	m_testList.setAutoDelete (true) ;
}

/*  KBNode								*/
/*  KBNode	: Constructor for node base class			*/
/*  parent	: KBNode *	: Parent node if any			*/
/*  element	: cchar *	: Element tag				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  (returns)	: KBNode	:					*/

KBNode::KBNode
	(	KBNode			*parent,
		cchar			*element,
		const QDict<QString>	&aList
	)
	:
	m_parent	(parent),
	m_element	(element),
	m_root		(0),
	m_attrNotes	(0),
	m_showing	(KB::ShowAsUnknown),
	m_attrCnt	(0),
	m_flags		(GetNodeFlags (m_element)),
	m_monitor	(0),
	m_name		(this,		"name",		aList)
{
	if (m_parent != 0)
	{	m_root = m_parent->getRoot () ;
		m_parent->addChild (this) ;
	}
	else	m_root	= this	;

	m_attrNotes	= new KBAttrStr (this, "notes", aList, KAF_NOTES) ;

	m_slotList.setAutoDelete (true) ;
	m_testList.setAutoDelete (true) ;
}

/*  KBNode								*/
/*  KBNode	: Constructor for node base class			*/
/*  parent	: KBNode *	: Parent node if any			*/
/*  node	: KBNode *	: Extant node				*/
/*  (returns)	: KBNode	:					*/

KBNode::KBNode
	(	KBNode	*parent,
		KBNode	*node
	)
	:
	m_parent	(parent),
	m_element	(node->m_element),
	m_root		(0),
	m_attrNotes	(0),
	m_showing	(KB::ShowAsUnknown),
	m_attrCnt	(0),
	m_flags		(GetNodeFlags(m_element)),
	m_monitor	(0),
	m_name		(this,		"name",		node)
{
	if (m_parent != 0)
	{	m_root = m_parent->getRoot () ;
		m_parent->addChild (this) ;
	}
	else	m_root	= this	;

	m_attrNotes	= new KBAttrStr (this, "notes", node, KAF_NOTES) ;

	m_slotList.setAutoDelete (true) ;
	m_testList.setAutoDelete (true) ;
	LITER
	(	KBSlot,
		node->m_slotList,
		slot,
		new KBSlot (this, slot)
	)
	LITER
	(	KBTest,
		node->m_testList,
		test,
		new KBTest (this, test)
	)

	/* Normally we use ::replicateBelow() when replicating nodes	*/
	/* that can have children, but handle children that are		*/
	/* configuration objects as a special case, since they are	*/
	/* quite pervasive *and* do not depend on their parent (ie.,	*/
	/* this node) being completely constructed.			*/
	LITER
	(	KBNode,
		node->getChildren(),
		child,

		if (child->isConfig() != 0) child->replicate (this)
	)
}

/*  KBNode								*/
/*  ~KBNode	: Destructor for node base class			*/
/*  (returns)	:		:					*/

KBNode::~KBNode ()
{
	LITER
	(	KBAttr,
		m_attribs,
		attrib,
		if ((attrib->getFlags() & KAF_USER) != 0) delete attrib ;
	)

	tearDown() ;
	DELOBJ	(m_monitor  ) ;
	DELOBJ	(m_attrNotes) ;

	if (m_parent != 0) m_parent->remChild (this) ;
}

void	KBNode::tearDown ()
{
	while (m_children.count() > 0) delete m_children.first() ;
}

/*  KBNode								*/
/*  startParse	: Start parsing for this node				*/
/*  (returns)	: void		:					*/

void	KBNode::startParse ()
{
}

/*  KBNode								*/
/*  endParse	: End parsing for this node				*/
/*  (returns)	: KBNode *	: Parent element			*/

KBNode *KBNode::endParse ()
{
	return	m_parent ;
}

/*  KBNode								*/
/*  addAttr	: Add an attribute					*/
/*  attr	: KBAttr *	  : The attribute			*/
/*  attrIdx	: uint &	  : Return attribute index		*/
/*  (returns)	: KBNodeMonitor * : Current node monitor if any		*/

KBNodeMonitor
	*KBNode::addAttr
	(	KBAttr		*attr,
		uint		&attrIdx
	)
{
	m_attribs.append (attr) ;

	m_attrCnt += 1	   	;
	attrIdx	   = m_attrCnt	;

	return	m_monitor	;
}

/*  KBNode								*/
/*  remAttr	: Remove an attribute					*/
/*  attr	: KBAttr *	: The attribute				*/
/*  (returns)	: void		:					*/

void	KBNode::remAttr
	(	KBAttr	*attr
	)
{
	m_attribs.remove (attr) ;
}

/*  KBNode								*/
/*  getAttrVal	: Find a named attribute value				*/
/*  name	: const QString & : Attribute name			*/
/*  (returns)	: QString	  : Value or null			*/

QString	KBNode::getAttrVal
	(	const QString	&name
	)
{
	KBAttr	*attr	;
	if ((attr = getAttr (name)) != 0) return attr->getValue() ;
	return	QString	() ;
}

/*  KBNode								*/
/*  setAttrVal	: Set a named attribute value				*/
/*  name	: const QString & : Attribute name			*/
/*  value	: const QString & : Attribute value			*/
/*  mkuser	: bool		  : Create user attribute if needed	*/
/*  update	: bool		  : Update objects			*/
/*  (returns)	: bool		  : Success				*/

bool	KBNode::setAttrVal
	(	const QString	&name,
		const QString	&value,
		bool		mkuser,
		bool		
	)
{
	/* Look for the attribute. If not found then create it as a	*/
	/* user attribute. Note sure anyone ever uses this but keep it	*/
	/* for backward compatability.					*/
	KBAttr	*attr	;
	if ((attr = getAttr (name)) == 0)
		if (mkuser)
		{
			new	KBAttrStr (this, name, value, KAF_USER) ;
			return	true   ;
		}
		else	return	false  ;


	/* Updating actual attributes is, in retrospect, a bad idea,	*/
	/* but we're stuck with it now. Feed it into the property	*/
	/* mechanism; this has the useful site effect that the property	*/
	/* mechanism gains whichever attributes are implemented.	*/

	/* The value is passed as a string, unless the attribute is 	*/
	/* numeric or boolean, in which case convert ....		*/
	KBValue	v (value) ;
	switch (attr->getType())
	{
		case KBAttr::Int  :
		case KBAttr::UInt :
			v	= KBValue(value.toInt()) ;			
			break	;

		case KBAttr::Bool :
			if	(value == "Yes") v = KBValue(1, &_kbBool) ;
			else if (value == "No" ) v = KBValue(0, &_kbBool) ;
			else	v = KBValue(value.toInt() != 0, &_kbBool) ;
			break	;

		default	:
			break	;

	}

	return	setKBProperty (name, v) ;
}

/*  KBNode								*/
/*  getAttr	: Find a named attribute				*/
/*  name	: const QString & : Attribute name			*/
/*  (returns)	: KBAttr *	  : Attribute or null			*/

KBAttr *KBNode::getAttr
	(	const QString	&name
	)
{
	for (KBAttr *attr = m_attribs.first() ; attr != 0 ; attr = m_attribs.next())
		if (attr->getName() == name)
			return	attr	;

	return	0 ;
}

/*  KBNode								*/
/*  addChild	: Add a child node					*/
/*  child	: KBNode *	: Child in question			*/
/*  (returns)	: void		:					*/

void	KBNode::addChild
	(	KBNode	*child
	)
{
	m_children.append (child) ;

	/* If we are displaying a monitor then locate the children	*/
	/* sub-entry and get a monitor displayed for the child. Note:	*/
	/* at this stage, the new node will not have any attributes,	*/
	/* these will be attached to the monitor as the are created.	*/
	if (m_monitor != 0)
		for (QListViewItem *entry  = m_monitor->firstChild () ;
				    entry != 0 ;
				    entry  = entry->nextSibling())
			if (entry->text (0) == "Children")
			{
				child->showMonitor (entry) ;
				break	;
			}
}

/*  KBNode								*/
/*  remChild	: Remove a child node					*/
/*  child	: KBNode *	: Child in question			*/
/*  (returns)	: void		:					*/

void 	KBNode::remChild
	(	KBNode	*child
	)
{
	if (!m_children.remove (child))
		fprintf	(stderr, "remChild: lost child\n") ;
}

/*  KBNode								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBNode::showAs
	(	KB::ShowAs	mode
	)
{
	bool	ac	= false	;
	m_showing	= mode	;

	DPRINTF
	((	"KBNode::showAs: %s: mode=%d\n",
		getName().latin1(),
		mode
	))	;

	LITER
	(	KBAttr,
		m_attribs,
		attrib,

		if ((attrib->getFlags() & KAF_USER) != 0)
		{	m_attribs.removeRef (attrib) ;
			delete	attrib	;
			continue	;
		}

		if (attrib->showAs (mode))
			ac	= true	;
	)

	/* The default action is just to recurse down through all the	*/
	/* nodes.							*/
	LITER
	(	KBNode,
		m_children,
		child,
		child->showAs (mode)
	)

	if (ac) setupProperties () ;
}

/*  KBNode								*/
/*  setupProperties							*/
/*		: Set up current properties				*/
/*  (returns)	: void		:					*/

void	KBNode::setupProperties ()
{
}

/*  KBNode								*/
/*  prepare	: Prepare prior to execution				*/
/*  (returns)	: void		:					*/

void	KBNode::prepare ()
{
	LITER
	(	KBNode,
		m_children,
		child,
		child->prepare ()
	)
}

/*  KBNode								*/
/*  printNode	: Output node XML					*/
/*  text	: QString &	: Result string				*/
/*  indent	: int		: Indent level of this node		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBNode::printNode
	(	QString		&text,
		int		indent,
		bool		flat
	)
{
	DPRINTF
	((	"KBNode::printNode: %s: c=%d s=%d t=%d\n",
		getName().latin1(),
		m_children.count(),
		m_slotList.count(),
		m_testList.count()
	))	;
	bool	anyChild = (m_children.count() > 0) ||
			   (m_slotList.count() > 0) ||
			   (m_testList.count() > 0) ;
	QString	nodeText ;

	text	+= QString("%1<%2").arg("", indent).arg(m_element) ;
	for (uint idx0 = 0 ; idx0 < m_attribs.count () ; idx0 += 1)
		m_attribs.at(idx0)->printAttr(text, nodeText, indent + 2, flat) ;

	if (!anyChild && nodeText.isEmpty())
	{
		text	+= "/>\n" ;
		return	;
	}

	text	+= ">\n" ;

	for (uint idx1 = 0 ; idx1 < m_children.count () ; idx1 += 1)
		m_children.at(idx1)->printNode (text, indent + 2, flat) ;

	for (uint idx2 = 0 ; idx2 < m_slotList.count () ; idx2 += 1)
		m_slotList.at(idx2)->printNode (text, indent + 2, flat) ;

	for (uint idx3 = 0 ; idx3 < m_testList.count () ; idx3 += 1)
		m_testList.at(idx3)->printAttr (text, nodeText, indent + 2, flat) ;


	text	+= nodeText ;
	text	+= QString("%1</%2>\n").arg("", indent).arg(m_element) ;
}

#if	! __KB_RUNTIME

bool	KBNode::propertyDlg
	(	cchar		*iniattr
	)
{
	return	basePropertyDlg (getElement().mid(2), iniattr) ;
}

/*  KBNode								*/
/*  propertyDlg	: Show property dialog for node				*/
/*  caption	: const QString & : Caption for dialog			*/
/*  iniattr	: cchar *	  : Attribute to show			*/
/*  (returns)	: bool		  : Used clicked OK			*/

bool	KBNode::basePropertyDlg
	(	const QString	&caption,
		cchar		*iniattr
	)
{
	/* This is the default, so that we can always get a property	*/
	/* dialog. Most node types will have their own derived property	*/
	/* dialogs, which handle specific attributes.			*/
	KBPropDlg propDlg (this, caption, m_attribs, iniattr) ;
	return	  propDlg.exec () ;
}

/*  KBNode								*/
/*  doMultiProp	: Handle multiple properties editing			*/ 
/*  nodeList	: QPtrList<KBNode> &					*/
/*				: Nodes to be handled			*/
/*  (returns)	: bool		: Success				*/

bool	KBNode::doMultiProp
	(	QPtrList<KBNode> &nodeList
	)
{
	QPtrList<KBAttr>	attrList	;
	KBNode	multi	(this, "MultiProp")	;
	bool	gotone	= false			;

	multi.m_attribs.find	(multi.m_attrNotes) ;
	multi.m_attribs.take	()	 ;
	multi.m_attribs.find	(&multi.m_name	  ) ;
	multi.m_attribs.take	()	 ;

	attrList.setAutoDelete	(true)	 ;

	/* Iterate over each of the attributes which this node has. For	*/
	/* each, see if _all_ of the nodes in the list also have that	*/
	/* attribute; if so then replicate the attribute into the new	*/
	/* node.							*/
	LITER
	(	KBAttr,
		m_attribs,
		attr,

		bool	missing	= false ;

		LITER
		(	KBNode,
			nodeList,
			node,

			if (node->getAttr (attr->getName()) == 0)
			{	missing	= true ;
				break	;
			}
			if ((attr->getFlags() & KAF_COMMON) == 0)
			{	missing	= true ;
				break	;
			}
		)

		if (!missing)
		{	attrList.append (attr->replicate (&multi)) ;
			gotone = true ;
		}
	)

	if (!gotone)
		return	false ;

	if (!multi.propertyDlg (TR("Common properties")))
		return	false ;

	LITER
	(
		KBNode,
		nodeList,
		node,

		node->setMultiProp (&multi)
	)

	return	true ;
}

/*  KBNode								*/
/*  setMultiProp: Set multiple properties from node			*/
/*  multi	: KBNode *	: Node from which to set		*/
/*  (returns)	: void		:					*/

void	KBNode::setMultiProp
	(	KBNode	*multi
	)
{
	/* For each of our attributes, see if the setting node defines	*/
	/* it, in which case copy the value.				*/
	LITER
	(
		KBAttr,
		m_attribs,
		attr,

		if (multi->getAttr(attr->getName()) != 0)
			attr->setValue (multi->getAttrVal (attr->getName())) ;
	)

	if (isObject()) isObject()->setupProperties () ;
}


#endif

/*  KBNode								*/
/*  applyDialog	: Apply attributes, called from property dialogs	*/
/*  (returns)	: void		:					*/

void	KBNode::applyDialog ()
{
	/* This is a dummy. Interested parties will implement it.	*/
}

/*  KBNode								*/
/*  replicate	: Replicate this node					*/
/*  _parent	: KBNode *	: Parent node				*/
/*  (returns)	: KBNode *	: Always null				*/

KBNode	*KBNode::replicate
	(	KBNode	*
	)
{
	/* This is an error case. The replicate method must be		*/
	/* implemented in the final derived class.			*/
	KBError::EFault
	(	QString(TR("KBNode::replicate() called for %1")).arg(m_element),
		QString::null,
		__ERRLOCN
	)	;
	return	0 ;
}

/*  KBNode								*/
/*  replicateBelow							*/
/*		: Replicate children of this node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  (returns)	: KBNode *	: Always null				*/

KBNode	*KBNode::replicateBelow
	(	KBNode	*parent
	)
{
	LITER
	(	KBNode,
		m_children,
		child,

		/* Don't replicate configuration objects, this will	*/
		/* have been done in the KBNode replication constructor	*/
		if (child->isConfig() == 0)
			child->replicate (parent) ;
	)

	return	parent	;
}

/*  KBObject								*/
/*  write	: Write object to report writer				*/
/*  writer	: KBWriter *	: Writer object				*/
/*  offset	: QPoint	: Geometry offset			*/
/*  first	: bool		: First on page				*/
/*  extra	: init &	: Return extra space			*/
/*  prior	: bool		: Write prior value flag		*/
/*  (returns)	: void		:					*/

bool	KBNode::write
	(	KBWriter	*writer,
		QPoint		offset,
		bool		first,
		int		&extra,
		bool		prior
	)
{
	LITER
	(	KBNode,
		m_children,
		child,
		if (!child->write (writer, offset, first, extra, prior))
			return false	;
	)

	return	true	;
}

/*  KBNode								*/
/*  setMonitor	: Set monitor values for this node			*/
/*  myitem	: KBNodeMonitor * : Monitor item for this node		*/
/*  (returns)	: void		  :					*/

void	KBNode::setMonitor
	(	KBNodeMonitor	*myitem
	)
{
	KBNodeMonitor	*attritem = 0 ;
	KBNodeMonitor	*nodeitem = 0 ;

	if (myitem != 0)
	{
		KBAttr	*name	= getAttr ("name") ;
		myitem->setText (0, m_element) ;
		if (name != 0) myitem->setText (1, name->getValue()) ;

		attritem = new KBNodeMonitor  (0, myitem) ;
		attritem->setText	(0, "Attributes") ;
		attritem->setSelectable (false) ;

		nodeitem = new KBNodeMonitor  (0, myitem) ;
		nodeitem->setText	(0, "Children"  ) ;
		nodeitem->setSelectable (false) ;
	}

	LITER
	(	KBAttr,
		m_attribs,
		attr,
		attr->showMonitor (attritem)
	)
	LITER
	(	KBNode,
		m_children,
		node,
		node->showMonitor (nodeitem)
	)
}

/*  KBNode								*/
/*  showMonitor	: Show or clear monitor for this node			*/
/*  parent	: RKListView *	: Parent monitor			*/
/*  (returns)	: void		:					*/

void	KBNode::showMonitor
	(	RKListView	*parent
	)
{
	if (parent == 0)
		setMonitor (m_monitor = 0) ;
	else	setMonitor (m_monitor = new KBNodeMonitor (this, parent)) ;
}

/*  KBNode								*/
/*  showMonitor	: Show or clear monitor for this node			*/
/*  parent	: QListViewItem * : Parent monitor			*/
/*  (returns)	: void		  :					*/

void	KBNode::showMonitor
	(	QListViewItem	*parent
	)
{
	if (parent == 0)
		setMonitor (m_monitor = 0) ;
	else	setMonitor (m_monitor = new KBNodeMonitor (this, parent)) ;
}

/*  KBNode								*/
/*  setMonitorSelect							*/
/*		: Set whether this nodes moditor is selected		*/
/*  select	: bool		: Selected or not			*/
/*  (returns)	: void		  :					*/

void	KBNode::setMonitorSelect
	(	bool	select
	)
{
	if (m_monitor != 0)
		m_monitor->listView()->setSelected (m_monitor, select) ;
}

/*  KBNode								*/
/*  findAllParams: Locate all paramaters in document			*/
/*  paramSet	 : QDict<KBParamSet> &  : Dictionary of found entries	*/
/*  (returns)	 : void			:				*/

void	KBNode::findAllParams
	(	QDict<KBParamSet>	&paramSet
	)
{
	/* Default is just to iterate down. The paramater node class	*/
	/* will override this method and add itself to the dictionary.	*/
	LITER
	(	KBNode,
		m_children,
		child,
		child->findAllParams (paramSet)
	)
}


/*  KBNode								*/
/*  objType	: Get object type for this node				*/
/*  (returns)	: KB::ObjType	: Object type				*/

KB::ObjType KBNode::objType ()
{
	/* Default is to return the type of the root object. This is	*/
	/* overridden by top-level types (KBForm, ...)			*/
	return	getRoot()->objType() ;
}

bool	KBNode::setExtraAttribs
	(	const	QDict<QString>	&aDict
	)
{
	QString	*name	;
	KBAttr	*attr	;

	if ((name = aDict.find ("name")) == 0) return false ;
	if ((attr = getAttr    (*name )) == 0) return false ;

	return	attr->setExtraAttribs (aDict) ;
}

/*  KBNode								*/
/*  enumKBProperty							*/
/*		: Get list of properties				*/
/*  list	: QStringList &	: Property list				*/
/*  (returns)	: void		:					*/

void	KBNode::enumKBProperty
	(	QStringList	&list
	)
{
	list.append ("name") ;
	list.append ("element") ;
}

/*  KBNode								*/
/*  hasKBProperty: Check if object has this property			*/
/*  name	 : cchar *	: Property name				*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBNode::hasKBProperty
	(	cchar		*name
	)
{
	if (qstrcmp (name, "name"   ) == 0)
		return	true ;
	if (qstrcmp (name, "element") == 0)
		return	true ;

	return	false	;
}

/*  KBNode								*/
/*  getKBProperty: Get a rekall property value				*/
/*  name	 : cchar *	: Property name				*/
/*  value	 : KBValue &	: Return property value			*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBNode::getKBProperty
	(	cchar		*name,
		KBValue		&value
	)
{
	if (qstrcmp (name, "name"   ) == 0)
	{
		value	= getName   () ;
		return	true ;
	}
	if (qstrcmp (name, "element") == 0)
	{
		value	= getElement() ;
		return	true ;
	}

//	fprintf
//	(	stderr,
//		"KBNode::getKBProperty: called for [%s]\n",
//		name
//	)	;

	return	false	;
}

/*  KBNode								*/
/*  setKBProperty: Set a rekall property value				*/
/*  name	 : cchar *	: Proprty name				*/
/*  value	 : KBValue	: Property value			*/
/*  (returns)	 : bool		: Success				*/

bool	KBNode::setKBProperty
	(	cchar		*name,
		const KBValue	&value
	)
{
//	fprintf
//	(	stderr,
//		"KBNode::setKBProperty: called for [%s]\n",
//		name
//	)	;
	return	false	;
}


/*  KBNode								*/
/*  findAllConfigs							*/
/*		: Locate all configuration nodes			*/
/*  paramSet	: QPtrList<KBConfig> &	: List of found entries		*/
/*  path	: QString		: Path to this object		*/
/*  (returns)	: void			:				*/

void	KBNode::findAllConfigs
	(	QPtrList<KBConfig>	&configSet,
		QString		path
	)
{
	path	= path.isEmpty() ? getName() : path + '/' + getName() ;

	LITER
	(	KBNode,
		getChildren(),
		child,

		if (child->isCompLink() != 0)
			continue ;

		KBConfig *config = child->isConfig() ;

		if (config != 0)
		{
			configSet.append(config) ;
			config->setPath	(path)   ;
			continue	;
		}

		child->findAllConfigs (configSet, path) ;
	)
}

/*  KBNode								*/
/*  getPath	: Get path name for this node				*/
/*  withTop	: bool		: Include top-level name		*/
/*  (returns)	: QString	: Path					*/

QString	KBNode::getPath
	(	bool		withTop
	)
{
	if (m_parent != 0)
		return	m_parent->getPath(withTop) + "/" + getName() ;

	return	withTop ? getName() : QString::null ;
}

/*  KBNode								*/
/*  connectLinks: Connect event/slot links				*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBNode::connectLinks
	(	KBError		&pError
	)
{
	DPRINTF
	((	"KBNode::connectLinks: [%s][%s]\n",
		(cchar *)getElement(),
		(cchar *)getName   ()
	))	;
	
	LITER
	(	KBSlot,
		m_slotList,
		slot,
		if (!slot->connectLinks (pError)) return false ;
	)
	LITER
	(	KBNode,
		m_children,
		node,
		if (!node->connectLinks (pError)) return false ;
	)

	return	true	;
}			

/*  KBNode								*/
/*  getEmitter	: Get signal emitter object for named event attribute	*/
/*  name	: const QString & : Event name				*/
/*  (returns)	: KBEmitter *	  : Emitter or null if none		*/

KBEmitter
	*KBNode::getEmitter
	(	const QString	&name
	)
{
	DPRINTF
	((	"KBNode::getEmitter: called: [%s][%s][%s]\n",
		(cchar *)getElement(),
		(cchar *)getName   (),
		(cchar *)name
	))	;

	KBAttr	*attr	= getAttr (name) ;
	if (attr  == 0) return 0 ;

	KBEvent	*event	= attr->isEvent() ;
	if (event == 0) return 0 ;

	DPRINTF
	((	"KBNode::getEmitter: got event\n"
	))	;

	return	event->getEmitter () ;
}

/*  KBNode								*/
/*  addSlot	: Add a slot to this node				*/
/*  slot	: KBSlot *	: The slot in question			*/
/*  (returns)	: void		:					*/

void	KBNode::addSlot
	(	KBSlot		*slot
	)
{
	m_slotList.append (slot) ;

	DPRINTF
	((	"KBNode::addSlot : [%s] adds slot [%s]\n",
		(cchar *)getAttrVal("name"),
		(cchar *)slot->name()
	))	;

}

/*  KBNode								*/
/*  addTest	: Add a test to this node				*/
/*  test	: KBTest *	: The test in question			*/
/*  (returns)	: void		:					*/

void	KBNode::addTest
	(	KBTest		*test
	)
{
	m_testList.append (test) ;

	DPRINTF
	((	"KBNode::addTest : [%s] adds test [%s]\n",
		(cchar *)getAttrVal("name"),
		(cchar *)test->getName()
	))	;

}

/*  KBNode								*/
/*  getSlots	: Get list of slots					*/
/*  (returns)	: const QPtrList<KBSlot> & :				*/

const QPtrList<KBSlot> &KBNode::getSlots ()
{
	return	m_slotList ;
}

/*  KBNode								*/
/*  getTests	: Get list of tests					*/
/*  (returns)	: const QPtrList<KBTest> & :				*/

const QPtrList<KBTest> &KBNode::getTests ()
{
	return	m_testList ;
}

/*  KBNode								*/
/*  clearSlots	: Clear current slot list				*/
/*  (returns)	: void		:					*/

void	KBNode::clearSlots ()
{
	m_slotList.clear () ;
}

/*  KBNode								*/
/*  clearTests	: Clear current test list				*/
/*  (returns)	: void		:					*/

void	KBNode::clearTests ()
{
	m_testList.clear () ;
}



const bool &KBNode::gotExecError ()
{
	return	getDocRoot()->gotExecError() ;
}

/*  KBNode								*/
/*  getNamedNode: Get named node relative to this one			*/
/*  name	: QString	: Name of node				*/
/*  showErr	: bool		: Show error dialog on error		*/
/*  abort	: bool *	: Abort on return flag			*/
/*  (returns)	: KBObject *	: Object or null if not found		*/

KBNode	*KBNode::getNamedNode
	(	QString		string,
		bool		showErr,
		bool		*abort
	)
{
	QString	name	= string	;
	KBNode	*node	= this		;

	DPRINTF
	((	"KBNode::getNamedNode: [%s]\n",
		name.latin1()
	))	;

	/* A leading separator moves us to the root of the tree, as in	*/
	/* UNIX file system names. The walk the node tree ....		*/
	if (name[0] == '/')
	{
		node	= getRoot  () 	;
		name	= name.mid (1)	;
	}

	QStringList bits = QStringList::split ('/', name) ;
	for (uint idx0 = 0 ; idx0 < bits.count() ; idx0 += 1)
	{
		const QString &part = bits[idx0] ;

		DPRINTF
		((	"KBNode::getNamedNode: find [%s]\n",
			part.latin1()
		))	;

		if (part == "" )
			continue ;

		if (part == ".")
			continue ;

		if (part == "..")
		{
			if ((node = node->getParent()) == 0)
				break	  ;
			else	continue  ;
		}
		if (part == "getRoot()" )
		{
			if ((node = node->getRoot ()) == 0)
				break	  ;
			else	continue  ;
		}
		if (part == "getBlock()")
		{
			if ((node = node->getBlock()) == 0)
				break	  ;
			else	continue  ;
		}

		KBNode	*next	= 0 ;
		for (uint idx1 = 0 ; idx1 < node->m_children.count() ; idx1 += 1)
			if (node->m_children.at(idx1)->getName() == part)
			{	next	= node->m_children.at(idx1) ;
				break	;
			}

		if ((node = next) == 0) return 0 ;
	}

	/* If we have not found an node and the show-error option is	*/
	/* set then show the no-object error dialog. If the user OK's	*/
	/* this then continue with the selected object, otherwise	*/
	/* really return not-found.					*/
#if	! __KB_RUNTIME
	if ((node == 0) && showErr)
	{
		DPRINTF
		((	"KBNode::getNamedNode: failed: abort=%p\n",
			(void *)abort
		))	;

		KBNoNodeDlg noNodeDlg (this, string, abort) ;

		if (noNodeDlg.exec ())
			node	= noNodeDlg.selectedNode () ;
	}
#endif

	return	node	;
}

/*  KBNode								*/
/*  getNamedNode							*/
/*		: Get named node relative to this one			*/
/*  name	: cchar *	: Name of node				*/
/*  pError	: KBError &	: Error message return			*/
/*  showErr	: bool		: Show error dialog on error		*/
/*  abort	: bool *	: Abort on return flag			*/
/*  (returns)	: KBNode *	: Node or null if not found		*/

KBNode	*KBNode::getNamedNode
	(	QString		string,
		KBError		&pError,
		bool		showErr,
		bool		*abort
	)
{
	KBNode	*node	= getNamedNode (string, showErr, abort) ;

	if (node == 0)
		pError	= KBError
			  (	KBError::Error,
				"Cannot find named object",
				string,
				__ERRLOCN
			  )	;

	return	node	;
}


/*  KBNode								*/
/*  getBlock	: Get enclosing block for node				*/
/*  (returns)	: KBBlock *	: Block or null if none			*/

KBBlock	*KBNode::getBlock ()
{
	for (KBNode *p = getParent () ; p != 0 ; p = p->getParent())
		if (p->isBlock() != 0)
			return	p->isBlock() ;
	return	0 ;
}

/*  KBNode								*/
/*  contextMenuType							*/
/*		: Return type text for context menu			*/
/*  (returns)	: QString	: Text					*/

QString	KBNode::contextMenuType ()
{
	/* This method is called when generating the ancestor entries.	*/
	/* Defaults to return the element name (with the leading "KB"	*/
	/* stripped off). Overriden where needed.			*/
	return	getElement().mid(2) ;
}
