/***************************************************************************
    file	         : kb_node.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_NODE_H
#define		__KB_NODE_H

#include	"libkbase_exports.h"

#include	<qobject.h>
#include	<qptrlist.h>
#include	<qdict.h>
#include	<qstring.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_attr.h"


class	RKListView	;
class	QListViewItem	;
class	QXmlAttributes	;
class	KBTest		;
class	KBSlot		;

/*  KBNode								*/
/*  ----------								*/
/*  This is the base class for the set of classes used to represent	*/
/*  nodes in the XML parse tree.					*/

class	LIBKBASE_API	KBNode : public QObject
{
	Q_OBJECT

	KBNode		*m_parent	;	/* Parent node		*/
	QString		m_element	;	/* Node type		*/
	KBError		m_lError	;	/* Last error		*/
	KBNode		*m_root		;	/* Root of this tree	*/
	KBAttrStr	*m_attrNotes	;	/* Documentation	*/
	KB::ShowAs	m_showing	;	/* Showing mode		*/
	uint		m_attrCnt	;

protected :

	QPtrList<KBAttr>m_attribs	;	/* List of attributes	*/
	QPtrList<KBNode>m_children	;	/* List of child nodes	*/
	uint		m_flags		;	/* Miscellaneous flags	*/
	KBNodeMonitor	*m_monitor	;	/* Tree monitor entry	*/
	QPtrList<KBSlot>m_slotList	;	/* List of slots	*/
	QPtrList<KBTest>m_testList	;	/* List of tests	*/


	KBAttrStr	m_name		;	/* Node name		*/

	virtual	void	setMonitor  	(KBNodeMonitor *) ;
	virtual void	tearDown ()	;

public	:

	KBNode	(KBNode  *, cchar  *, const QDict<QString> &) ;
	KBNode	(KBNode  *, cchar  *) ;
	KBNode	(KBNode  *, KBNode *) ;
virtual~KBNode	()		     ;


	bool			setExtraAttribs	(const QDict<QString> &);

	virtual	void		addChild   	(KBNode *)	 	;
	virtual	void		remChild   	(KBNode *)	  	;

	/* Attribute methods: Attributes attach themselves to their	*/
	/* parent on creation, and detach on deletion. We also provide	*/
	/* arbitrary access for script code.				*/
	KBNodeMonitor	 	*addAttr    	(KBAttr *, uint &)	;
	void			remAttr	   	(KBAttr *)	 	;
	KBAttr 			*getAttr   	(const QString  &) 	;

	bool			setAttrVal 	(const QString  &, const QString &, bool = false, bool = false) ;
	QString			getAttrVal 	(const QString  &) 	;


	virtual	void		setupProperties	()			;
	virtual	bool		write		(KBWriter *, QPoint, bool, int &, bool = false) ;

	inline	const QPtrList<KBNode>	&getChildren ()
	{
		return	m_children ;
	}
	inline	const QPtrList<KBAttr>	&getAttribs  ()
	{
		return	m_attribs  ;
	}
	inline	QString		getName 	()
	{
		return	m_name.getValue() ;
	}

	QString			getPath		(bool = true)	;

	virtual	void		findAllParams	(QDict<KBParamSet> &) ;

	void			showMonitor	(RKListView    *) ;
	void			showMonitor	(QListViewItem *) ;
	void			setMonitorSelect(bool)		  ;
	void			findAllConfigs	(QPtrList<KBConfig> &, QString) ;

#if	! __KB_RUNTIME
	virtual	bool		propertyDlg	(cchar *)	  ;
	bool			basePropertyDlg	(const QString &, cchar * = 0) ;
	bool			doMultiProp	(QPtrList<KBNode>   &) ;
	void			setMultiProp	(KBNode *) ;
#endif

	/* Base methods. Some of these will be overridden by the	*/
	/* derived classes in order to handle class-specific actions.	*/
	virtual	void		startParse 	()	 	  ;
	virtual	KBNode		*endParse  	()	 	  ;
	virtual	KBNode		*replicate 	(KBNode *)	  ;
	virtual	KBNode		*replicateBelow (KBNode *)	  ;
	virtual	void		showAs 		(KB::ShowAs)	  ;
	virtual	void		printNode  	(QString &, int, bool)	;
	virtual	void		applyDialog	()		  ;
	virtual	void		prepare		()		  ;

	KBNode			*getNamedNode	(QString,	      bool = false, bool * = 0) ;
	KBNode			*getNamedNode	(QString, KBError &,  bool = false, bool * = 0) ;

	/* Common access to protected stuff ...				*/
	KBBlock			*getBlock	() ;
	inline	KBNode		*getRoot	() { return m_root    ; }
	inline	KBNode		*getParent	() { return m_parent  ; }
	inline	const QString	&getElement	() { return m_element ; }

	inline	KB::ShowAs	isShowing	()
	{
		return	m_showing ;
	}
	inline	bool		showingDesign	()
	{
#if	__KB_RUNTIME
		return	false	;
#else
		return	m_showing == KB::ShowAsDesign ;
#endif
	}
	inline	bool		showingData	()
	{
		return	m_showing == KB::ShowAsData   ;
	}

	inline	uint		getFlags	() { return m_flags   ; }

	inline	KBLayout	*getLayout	() { return m_root->isLayout 	() ; }
	inline	KBDocRoot	*getDocRoot	() { return m_root->isDocRoot	() ; }
	inline	KBForm		*getForm	() { return m_root->isForm   	() ; }
	inline	KBReport	*getReport	() { return m_root->isReport 	() ; }
	inline	KBComponent	*getComponent	() { return m_root->isComponent () ; }

	/* The folloing methods are are used to test if a node is	*/
	/* is actually an instance of a specific type. Individual	*/
	/* derived classes will define instance for their themselves,	*/
	/* eg., "KBScript" will define "isScript" which will return	*/
	/* "this".							*/
	virtual	KBParam		*isParam	() { return 0 ; }
	virtual	KBModule	*isModule	() { return 0 ; }
	virtual	KBScript	*isScript	() { return 0 ; }
	virtual	KBConfig	*isConfig	() { return 0 ; }
	virtual	KBOverride	*isOverride	() { return 0 ; }
	virtual	KBImport	*isImport	() { return 0 ; }
	virtual	KBQryBase	*isQryBase	() { return 0 ; }
	virtual	KBQryData	*isQryData	() { return 0 ; }
	virtual	KBQryTable	*isQryTable	() { return 0 ; }
	virtual	KBQryQuery	*isQryQuery	() { return 0 ; }
	virtual	KBQryNull	*isQryNull	() { return 0 ; }
	virtual	KBQrySQL	*isQrySQL	() { return 0 ; }
	virtual	KBQryDesign     *isQryDesign	() { return 0 ; }
	virtual	KBItem		*isItem		() { return 0 ; }
	virtual	KBObject	*isObject	() { return 0 ; }
	virtual	KBBlock		*isBlock	() { return 0 ; }
	virtual	KBFormBlock	*isFormBlock	() { return 0 ; }
	virtual	KBReportBlock	*isReportBlock	() { return 0 ; }
	virtual	KBFramer	*isFramer	() { return 0 ; }
	virtual	KBHeader	*isHeader	() { return 0 ; }
	virtual	KBFooter	*isFooter	() { return 0 ; }
	virtual	KBContainer	*isContainer	() { return 0 ; }
	virtual	KBComponent	*isComponent	() { return 0 ; }
	virtual	KBCompLink	*isCompLink	() { return 0 ; }
	virtual	KBTabberPage	*isTabberPage	() { return 0 ; }
	virtual	KBTabber	*isTabber	() { return 0 ; }
	virtual	KBStackPage	*isStackPage	() { return 0 ; }
	virtual	KBStack		*isStack	() { return 0 ; }
	virtual	KBNavigator	*isNavigator	() { return 0 ; }
	virtual	KBForm		*isForm		() { return 0 ; }
	virtual	KBReport	*isReport	() { return 0 ; }
	virtual	KBRowMark	*isRowMark	() { return 0 ; }
	virtual	KBGrid		*isGrid		() { return 0 ; }
	virtual	KBHidden	*isHidden	() { return 0 ; }
	virtual	KBQuery		*isQuery	() { return 0 ; }
	virtual	KBTable		*isTable	() { return 0 ; }
	virtual	KBQryExpr	*isQryExpr	() { return 0 ; }
	virtual	KBDocRoot	*isDocRoot	() { return 0 ; }
	virtual	KBLayout	*isLayout	() { return 0 ; }
	virtual	KBTestSuite	*isTestSuite	() { return 0 ; }

	virtual	KB::ObjType	objType		() ;
	virtual	QString		contextMenuType	() ;

	const bool		&gotExecError	() ;

	void			addSlot		(KBSlot  *) ;
	void			addTest		(KBTest  *) ;
	void			clearSlots	() ;
	void			clearTests	() ;
	const QPtrList<KBSlot>	&getSlots	() ;
	const QPtrList<KBTest>	&getTests	() ;
	bool			connectLinks	(KBError &) ;
	KBEmitter		*getEmitter	(const QString  &) ;


	/* Environment checking, ie., what soirt of document we are	*/
	/* in.								*/
	inline	bool	isFormDoc   ()
	{
		return	getRoot()->isForm  () != 0 ;
	}
	inline	bool	isReportDoc ()
	{
		return	getRoot()->isReport() != 0 ;
	}
	inline	bool	isQueryDoc ()
	{
		return	getRoot()->isQuery () != 0 ;
	}

	inline	void	setError
			(	const KBError	&e
			)
		{
			m_lError = e ;
		}
	inline	void	setError
			(	KBError::EType	etype,
				const QString	&message,
				const QString	&details,
				cchar		*file,
				uint		lno
			)
		{
			m_lError = KBError (etype, message, details, file, lno) ;
		}
	inline	void	setError
			(	KBError::EType	etype,
				const QString	&message,
				cchar		*file,
				uint		lno
			)
		{
			m_lError = KBError (etype, message, "", file, lno) ;
		}

	inline	const	KBError	&lastError () { return m_lError ; }

	/* These two methods are used for access-by-attribute from	*/
	/* scripts.							*/
	virtual	void	enumKBProperty	(QStringList	  	&) ;
	virtual	bool	hasKBProperty	(cchar *)		   ;
	virtual	bool	getKBProperty	(cchar *,	KBValue &) ;
	virtual	bool	setKBProperty	(cchar *, const KBValue &) ;

	friend	class	KBBlock	 	;
	friend	class	KBForm	 	;
	friend	class	KBReport 	;
	friend	class	KBComponent	;
	friend	class	KBDisplay	;

	friend	class	KBTestAccessor	;
}	;


/*  CITER(Type,Ptr,Action)						*/
/*  This macro is provided for convenience. It generates code which	*/
/*  iterates over all children, doing "Action" for each child which is	*/
/*  of type "Type", where "Ptr" is the pointer at the child in the	*/
/*  action.								*/


#define	CITER(Type,Ptr,Action) \
	{	QPtrListIterator<KBNode> __iter (KBNode::m_children) ;	\
		KBNode *__##Ptr ;					\
		KB##Type *Ptr ;						\
		while ((__##Ptr = __iter.current()) != 0)		\
		{	__iter += 1 ;					\
			if ((Ptr = __##Ptr->is##Type()) != 0)		\
			{	Action ;				\
			}						\
		}							\
	}

#endif		// __KB_NODE_H
