/***************************************************************************
    file	         : kb_nodereg.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_NODEREG_H
#define	_KB_NODEREG_H


#include	"libkbase_exports.h"



/*  XML node type registration						*/
/*  --------------------------						*/
/*  Most node types that can be parsed from XML source files (forms,	*/
/*  queries, reports, ...) are registered via static instances of the	*/
/*  KBNodeReg class. This inserts an entry into a vector that is picked	*/
/*  up in the kb_parse file. Basically, for a node <KBMyNode ...> which	*/
/*  maps to a class KBMyNode, insert a macro call like			*/
/*									*/
/*	NEWNODE	    (MyNode,menu_text,flags)				*/
/*	NEWNODENAMED(MyNode,menu_text,flags,name)			*/
/*	NEWNODE	    (MyNode,popupfunc,flags)				*/
/*	NEWNODENAMED(MyNode,popupfunc,flags,name)			*/
/*									*/
/*  into the source file for KBMyNode.					*/


class	QWidget		;
class	QPopupMenu	;
class	QObject		;
struct	NodeSpec	;

typedef	KBPopupMenu	*(*MKPOPUP) (QWidget	*, QObject *, Qt::ButtonState *, NodeSpec **) ;
typedef	KBNode		*(*MKNODE ) (KBNode 	*, const QDict<QString> &,    bool *) ;
typedef	KBNode		*(*MKBYWIZ) (KBNode 	*, KBQryBase *, KBAttrDict &, bool &) ;

struct	LIBKBASE_API	NodeSpec
{
	const char	*m_nodeName	;	/* Parse tree element	*/
	const char	*m_msgText	;	/* Menu/tooltip text	*/
	MKPOPUP		m_popupFunc	;	/* Create popup menu	*/
	MKNODE		m_nodeFunc	;	/* Create new node	*/
	MKBYWIZ		m_wizardFunc	;	/* Create using wizard	*/
	uint		m_flags		;	/* Various flags	*/
	int		m_id		;	/* Menu identifier	*/
}	;

extern	LIBKBASE_API	int	 nodeSpecToId	(NodeSpec *)	;
extern	LIBKBASE_API	NodeSpec *idToNodeSpec	(int	   )	;

extern	LIBKBASE_API
	void	registerNode
		(	cchar		*,
			cchar 		*,
			MKPOPUP		,
			MKNODE		,
			uint
		)	;

#define	REGNEWNODE(type,ident,popup,flags)				\
									\
	extern	KBNode	*new##type					\
			(	KBNode			*,		\
				const QDict<QString>	&,		\
				bool   			*		\
			)	;					\
	registerNode	("KB"#type, ident, popup, new##type, flags)	;


#define	REGNEWNODENAMED(type,ident,popup,flags,name)			\
									\
	extern	KBNode	*new##name						\
			(	KBNode			*,		\
				const QDict<QString>	&,		\
				bool			*		\
			)	;					\
	registerNode	("KB"#name, ident, popup, new##name, flags)	;

#define	NEWNODE(type,ident,popup,flags)					\
									\
	KBNode	*new##type						\
		(	KBNode			*parent,		\
			const QDict<QString>	&aList,			\
			bool			*ok			\
		)							\
	{								\
		return	new KB##type (parent, aList, ok) ;		\
	}								\

#define	NEWNODENAMED(type,ident,popup,flags,name)			\
									\
	KBNode	*new##name							\
		(	KBNode			*parent,		\
			const QDict<QString>	&aList,			\
			bool			*ok			\
		)							\
	{								\
		return	new KB##type (parent, aList, "KB"#name, ok) ;	\
	}								\


#endif	// _KB_NODEREG_H
