/***************************************************************************
    file	         : kb_noobjdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"
#include	"rk_listview.h"

#if		__KB_AUTOMOC
#include 	"kb_noobjdlg.moc"
#else
#include 	"kb_noobjdlg.h"
#endif

#include	"kb_object.h"
#include	"kb_config.h"
#include	"kb_sidepanel.h"


/*  KBNodeTreeNode							*/
/*  KBNodeTreeNode							*/
/*		: Constructor for node entry				*/
/*  parent	: RKListView *	   : Parent list view			*/
/*  node	: KBNode *	   : Associated node			*/
/*  (returns)	: KBNodeTreeNode :					*/

KBNodeTreeNode::KBNodeTreeNode
	(	RKListView	*parent,
		KBNode		*node
	)
	:
	QListViewItem	(parent, node->getElement(), node->getName()),
	m_node		(node)
{
	setExpandable	(true)	;
	m_expanded =	false	;
	m_start	   =	false	;
}


/*  KBNodeTreeNode							*/
/*  KBNodeTreeNode							*/
/*		: Constructor for node entry 				*/
/*  parent	: QListViewItem *  : Parent list view item		*/
/*  node	: KBNode *	   : Associated node			*/
/*  (returns)	: KBNodeTreeNode :					*/

KBNodeTreeNode::KBNodeTreeNode
	(	QListViewItem	*parent,
		KBNode		*node
	)
	:
	QListViewItem	(parent, node->getElement(), node->getName()),
	m_node		(node)
{
	setExpandable	(true)	;
	m_expanded =	false	;
	m_start	   =	false	;
}

/*  KBNodeTreeNode							*/
/*  setOpen	: Open or close the item				*/
/*  open	: bool		: True to open				*/
/*  (returns)	: void		:					*/

void	KBNodeTreeNode::setOpen
	(	bool		open
	)
{
	/* If we are opening and there are no known chilren then now	*/
	/* is the time to do it. If there really no children then make	*/
	/* the item non-expandable.					*/
	if (open && (childCount() == 0))
	{
		const QPtrList<KBNode> &childList = m_node->getChildren() ;

		LITER
		(	KBNode,
			childList,
			child,
			new KBNodeTreeNode (this, child)
		)

		if (childCount() == 0) setExpandable (false) ;
	}

	QListViewItem::setOpen (open) ;
}

void	KBNodeTreeNode::paintCell
	(	QPainter	  *p,
		const QColorGroup &cg1,
		int		  column,
		int		  width,
		int		  align
	)
{
	if (m_start)
	{
		QColorGroup cg2 (cg1) ;
		cg2.setColor (QColorGroup::Text, Qt::red) ;
		QListViewItem::paintCell (p, cg2, column, width, align) ;
	}
	else	QListViewItem::paintCell (p, cg1, column, width, align) ;
}


/*  KBNodeTreeNode							*/
/*  expandToNode							*/
/*		: Expand tree above specified node			*/
/*  listView	: RKListView *	    : Parent list view			*/
/*  node	: KBNode *	    : Node in question			*/
/*  root	: KBNode *	    : Root node for expansion		*/
/*  (returns)	: KBNodeTreeNode *  : List view item for the object	*/

KBNodeTreeNode
	*KBNodeTreeNode::expandToNode
	(	RKListView	*listView,
		KBNode		*node,
		KBNode		*root
	)
{
	/* See if there is a parent node above this one. If there	*/
	/* is, the expand to that node, then add an entry for each	*/
	/* child of said parent; as we do so, make a note of the	*/
	/* item for this object.					*/
	KBNode	 *pNode	  = node->getParent () ;
	if ((node != root) && (pNode != 0))
	{
		KBNodeTreeNode	*pItem	= expandToNode (listView, pNode, root) ;
		QListViewItem	*oItem	= 0 ;

		if (!pItem->expanded())
		{

			const QPtrList<KBNode> &childList = pNode->getChildren() ;

			LITER
			(	KBNode,
				childList,
				child,
				new KBNodeTreeNode (pItem, child)
			)

			pItem->setExpanded() ;
		}

		for (QListViewItem *lvi  = pItem->firstChild () ;
				    lvi != 0 ;
				    lvi  = lvi  ->nextSibling())
			if (((KBNodeTreeNode *)lvi)->node() == node)
			{
				oItem	= lvi	;
				break	;
			}

		/* Paranoia! We should never fail to find this object	*/
		/* amongst its parents children, but .....		*/
		if (oItem == 0)
			oItem	= new KBNodeTreeNode (pItem, node) ;

		pItem->setOpen (true) ;
		return	(KBNodeTreeNode *)oItem ;
	}

	if (listView->firstChild() != 0)
		return	(KBNodeTreeNode *)listView->firstChild() ;

	return	new KBNodeTreeNode (listView, node) ;
}



/*  ------------------------------------------------------------------  */

/*  KBNoNodeDlg								*/
/*  KBNoNodeDlg	: Constructor for node-not-found dialog			*/
/*  node	: KBNode *	: Node from which search started	*/
/*  string	: QString	: Search string				*/
/*  abort	: bool *	: Return for abort option		*/
/*  caption	: QString	: Optional caption			*/
/*  (returns)	: KBNoNodeDlg	:					*/

KBNoNodeDlg::KBNoNodeDlg
	(	KBNode		*node,
		QString		string,
		bool		*abort,
		const QString	&caption,
		cchar		*name
	)
	:
	KBDialog
	(	caption.isNull() ?
			TR("Named control not found ...") :
			caption,
		true,
		name
	),
	m_starting	(node),
	m_abort		(abort)
{
	RKVBox	*laymain= new RKVBox (this)	  ;
	laymain->setTracking () ;

	RKHBox	*laytop	= new RKHBox (laymain)	  ;
	new KBSidePanel (laytop, caption)	  ;

	RKVBox	*laydlg	= new RKVBox     (laytop) ;
	new QLabel (string, laydlg) ;
	m_nodeTree	= new RKListView (laydlg) ;

	m_layButt	= new RKHBox    (laymain) ;
	m_layButt->addFiller	() ;

	m_bFail     = new RKPushButton (TR("&Fail"    ), m_layButt) ;
	connect	(m_bFail,     SIGNAL(clicked()), SLOT(clickFail      ())) ;

	if (m_abort != 0)
	{
		m_bAbort  = new RKPushButton (TR("&Abort"), m_layButt) ;
		connect	(m_bAbort, SIGNAL(clicked()), SLOT(clickAbort())) ;
		*m_abort  = false ;
	}

	m_bContinue = new RKPushButton (TR("&Continue"), m_layButt) ;
	connect	(m_bContinue, SIGNAL(clicked()), SLOT(slotNodeSelected())) ;

	m_nodeTree  ->addColumn  (TR("Object"), 200) ;
	m_nodeTree  ->addColumn  (TR("Name"  ),  80) ;
	m_nodeTree  ->setMinimumWidth    ( 280)	;
	m_nodeTree  ->setRootIsDecorated (true)	;
	m_nodeTree  ->setResizeMode      (QListView::LastColumn) ;

	connect
	(	m_nodeTree,
		SIGNAL(clicked	    (QListViewItem *)), SLOT(slotNodeClicked ())
	)	;
	connect
	(	m_nodeTree,
		SIGNAL(doubleClicked(QListViewItem *)), SLOT(slotNodeSelected())
	)	;
	connect
	(	m_nodeTree,
		SIGNAL(returnPressed(QListViewItem *)), SLOT(slotNodeSelected())
	)	;


	QListViewItem	*oi = KBNodeTreeNode::expandToNode(m_nodeTree, node) ;

	m_nodeTree  ->setSelected	(oi, true) ;
	m_nodeTree  ->ensureItemVisible	(oi) ;
	m_bContinue->setEnabled		(false) ;
}

/*  KBNoNodeDlg								*/
/*  slotNodeClicked							*/
/*		: User clicks node					*/
/*  (returns)	: void		:					*/

void	KBNoNodeDlg::slotNodeClicked ()
{
	m_bContinue->setEnabled (true) ;
}

/*  KBNoNodeDlg								*/
/*  slotNodeSelected							*/
/*		: User selects node					*/
/*  (returns)	: void		:					*/

void	KBNoNodeDlg::slotNodeSelected ()
{
	KBNodeTreeNode *node = (KBNodeTreeNode *)m_nodeTree->currentItem() ;
	if (node != 0)
	{	m_selected = node->node() ;
		done (1) ;
	}
}


/*  KBNoNodeDlg								*/
/*  clickAbort	: User aborts operation					*/
/*  (returns)	: void		:					*/

void	KBNoNodeDlg::clickAbort ()
{
	if (m_abort != 0) *m_abort = true ;
	done	(0) ;
}

/*  KBNoNodeDlg								*/
/*  clickFail	: User fails operation					*/
/*  (returns)	: void		:					*/

void	KBNoNodeDlg::clickFail ()
{
	done	(0) ;
}

/*  ------------------------------------------------------------------	*/

KBRouteToNodeDlg::KBRouteToNodeDlg
	(	KBNode			*node,
		QString			string,
		const QString		&caption
	)
	:
	KBNoNodeDlg (node, string, 0, caption, "objectfinders")
{
	m_bAsPath = new RKPushButton (TR("Return as path"), m_layButt) ;
	m_bAsPath->setEnabled  (false) ;	
	m_bContinue->setText   (TR("Return as name")) ;

	connect	(m_bAsPath, SIGNAL(clicked()), SLOT(slotNodeAsPath())) ;
}

/*  KBRouteToNodeDlg							*/
/*  slotNodeClicked							*/
/*		: User clicks node					*/
/*  (returns)	: void		:					*/

void	KBRouteToNodeDlg::slotNodeClicked ()
{
	m_bAsPath->setEnabled    (true) ;	
	KBNoNodeDlg::slotNodeClicked () ;
}

void	KBRouteToNodeDlg::slotNodeAsPath ()
{
	KBNodeTreeNode *node = (KBNodeTreeNode *)m_nodeTree->currentItem() ;
	if (node != 0)
	{	m_selected = node->node() ;
		done (2) ;
	}
}

QString	KBRouteToNodeDlg::routeToNode
	(	bool		asPath
	)
{
	QPtrList<KBNode>	fromStarting	;
	fromStarting.append (m_starting)	;

	for
	(	KBNode *parent = m_starting->getParent() ;
		parent != 0 ;
		parent  = parent->getParent()
	)
		fromStarting.append (parent) ;

	QString	route	  = m_selected->getName  () ;
	KBNode	*ancestor = m_selected->getParent() ;
	int	found	   ;

	while ((ancestor != 0) && ((found = fromStarting.find(ancestor)) < 0))
	{
		route	 = QString("%1%2%3")
				.arg(ancestor->getName())
				.arg(asPath ? "." : "/")
				.arg(route)
			   ;
		ancestor = ancestor->getParent() ;
	}

	if (ancestor == 0)
	{
		fprintf
		(	stderr,
			"KBNoNodeDlg::routeToObject: no common ancestor??\n"
		)	;
		return	QString::null	;
	}

	for (int idx = 0 ; idx < found ; idx += 1)
		if (asPath)
			route	= "__parent__." + route	;
		else	route	= "../"		+ route	;

	return	route	;
}

/*  ------------------------------------------------------------------  */

/*  KBSlotFindDlg							*/
/*  KBSlotFindDlg: Constructor for slot/event finder			*/
/*  parent	 : KBNode *	   : Slot's parent			*/
/*  target	 : KBNode *	   : Current target node		*/
/*  event	 : const QString & : Current event			*/
/*  csonly	 : bool		   : Only allow client-side events	*/
/*  (returns)	 : KBSlotFindDlg   :					*/

KBSlotFindDlg::KBSlotFindDlg
	(	KBNode		*parent,
		KBNode		*target,
		const QString	&event,
		bool		csonly
	)
	:
	KBDialog	(TR("Locate node and event"), true, "objectfinders")
{
	m_selected	= 0	 ;
	m_csonly	= csonly ;

	RKVBox	*laymain= new RKVBox (this)	 ;
	laymain->setTracking () ;

	RKHBox	*laytop	= new RKHBox (laymain)	 ;
	new KBSidePanel (laytop, caption())	 ;

	RKVBox	*laydlg	= new RKVBox    (laytop) ;
	m_nodeTree	= new RKListView(laydlg) ;
	m_events	= new RKComboBox(laydlg) ;

	addOKCancel (laymain) ;

	m_nodeTree  ->addColumn  (TR("Object"), 200) ;
	m_nodeTree  ->addColumn  (TR("Name"  ),  80) ;
	m_nodeTree  ->setMinimumWidth    ( 280)	;
	m_nodeTree  ->setRootIsDecorated (true)	;
	m_nodeTree  ->setResizeMode      (QListView::LastColumn) ;

	connect
	(	m_nodeTree,
		SIGNAL(clicked	      (QListViewItem *)),
		SLOT  (slotNodeClicked(QListViewItem *))
	)	;


	KBNodeTreeNode	*pi = KBNodeTreeNode::expandToNode(m_nodeTree, parent) ;
	KBNodeTreeNode	*ti = KBNodeTreeNode::expandToNode(m_nodeTree, target) ;

	pi->setStart () ;

	m_nodeTree->setSelected	     (ti, true) ;
	m_nodeTree->ensureItemVisible (ti) ;
	slotNodeClicked (ti) ;

	for (int idx = 0 ; idx < m_events->count() ; idx += 1)
		if (event == m_eventList.at(idx)->getLegend())
		{
			m_events->setCurrentItem (idx) ;
			break	;
		}
}

/*  KBSlotFindDlg							*/
/*  slotNodeClicked							*/
/*		: User clicks node					*/
/*  lvi		: QListViewItem * : Item clicked			*/
/*  (returns)	: void		  :					*/

void	KBSlotFindDlg::slotNodeClicked
	(	QListViewItem	*lvi
	)
{
	if (lvi != 0)
	{
		m_selected = ((KBNodeTreeNode *)lvi)->node() ;

		m_events   ->clear () ;
		m_eventList .clear () ;

		LITER
		(	KBAttr,
			m_selected->getAttribs(),
			attr,

			if (attr->isEvent() == 0)
				continue ;

			if (m_csonly)
				if ((attr->getFlags() & KAF_EVCS) == 0)
					continue ;

			m_events->insertItem (attr->getLegend()) ;
			m_eventList.append   (attr) ;
		)
	}
}

KBNode	*KBSlotFindDlg::selectedNode
	(	QString		&event
	)
{
	event	= m_eventList.at(m_events->currentItem())->getName() ;
	return	m_selected ;
}

/*  ------------------------------------------------------------------  */

/*  KBConfigFindDlg							*/
/*  KBConfigFindDlg							*/
/*		: Constructor for object/config finder			*/
/*  parent	: KBNode *	   : Override's parent			*/
/*  target	: KBNode *	   : Current target node		*/
/*  config	: const QString &  : Current config			*/
/*  (returns)	: KBConfigFindDlg  :					*/

KBConfigFindDlg::KBConfigFindDlg
	(	KBNode		*parent,
		KBNode		*target,
		const QString	&
	)
	:
	KBDialog	(TR("Locate object and config"), true, "objectfinders")
{
	m_selected	= 0 ;

	RKVBox	*laymain= new RKVBox (this)	  ;
	laymain->setTracking () ;

	RKHBox	*laytop	= new RKHBox (laymain)	  ;
	new KBSidePanel (laytop, caption())	  ;

	RKVBox	*laydlg	= new RKVBox     (laytop) ;
	m_nodeTree	= new RKListView (laydlg) ;
	m_configs	= new RKComboBox (laydlg) ;

	addOKCancel	(laymain, &m_bOK) ;

	m_nodeTree  ->addColumn  (TR("Object"), 200) ;
	m_nodeTree  ->addColumn  (TR("Name"  ),  80) ;
	m_nodeTree  ->setMinimumWidth    ( 280)	;
	m_nodeTree  ->setRootIsDecorated (true)	;
	m_nodeTree  ->setResizeMode      (QListView::LastColumn) ;

	connect
	(	m_nodeTree,
		SIGNAL(clicked	      (QListViewItem *)),
		SLOT  (slotNodeClicked(QListViewItem *))
	)	;


	KBNodeTreeNode	*pi = new KBNodeTreeNode (m_nodeTree, parent) ;
	KBNodeTreeNode	*ti = KBNodeTreeNode::expandToNode (m_nodeTree, target, parent) ;

	pi->setStart () ;

	m_nodeTree->setSelected       (ti, true) ;
	m_nodeTree->ensureItemVisible (ti) ;
	slotNodeClicked (m_nodeTree->firstChild()) ;
}

/*  KBConfigFindDlg							*/
/*  slotNodeClicked							*/
/*		: User clicks node					*/
/*  lvi		: QListViewItem * : Item clicked			*/
/*  (returns)	: void		  :					*/

void	KBConfigFindDlg::slotNodeClicked
	(	QListViewItem	*lvi
	)
{
	if (lvi != 0)
	{
		m_selected = ((KBNodeTreeNode *)lvi)->node() ;

		m_configs->clear () ;

		TITER
		(	Config,
			m_selected->getChildren(),
			c,

			m_configs->insertItem (c->attrib()) ;
		)

		m_bOK->setEnabled (m_configs->count() > 0) ;
	}
}


KBNode	*KBConfigFindDlg::selectedConfig
	(	QString		&path,
		QString		&attrib
	)
{
	KBNodeTreeNode *lvi = (KBNodeTreeNode *)m_nodeTree->currentItem() ;

	path	= QString::null	;
	while ((lvi != 0) && (lvi->node() != m_start))
	{
		if (!path.isEmpty()) path = "/" + path ;
		path	= lvi->text  (1) + path ;
		lvi	= (KBNodeTreeNode *)lvi->parent()  ;
	}

	attrib	= m_configs->currentText() ;
	return	lvi == 0 ? 0 : m_selected  ;
}

