/***************************************************************************
    file	         : kb_noobjdlg.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_NOOBJDLG_H
#define	__KB_NOOBJDLG_H

#if	! __KB_RUNTIME

#include	"libkbase_exports.h"

#include	<qptrlist.h>
#include	<qlabel.h>


#include	"kb_classes.h"
#include	"kb_attr.h"
#include	"kb_dialog.h"

class		RKComboBox	;
class		RKListView	;
class		RKPushButton	;
class		RKHBox		;


class	LIBKBASE_API	KBNodeTreeNode : public QListViewItem
{
	KBNode			*m_node		;
	bool			m_expanded	;
	bool			m_start		;

public	:

	virtual	void		paintCell	(QPainter *, const QColorGroup &, int, int, int) ;
	virtual	void		setOpen 	(bool)	;

	inline	KBNode		*node	 	() { return m_node     ; }
	inline	bool		expanded 	() { return m_expanded ; }

	inline	void		setExpanded	() { m_expanded = true ; }
	inline	void		setStart 	() { m_start	= true ; }

	static	KBNodeTreeNode	*expandToNode	(RKListView *, KBNode *, KBNode * = 0) ;

	KBNodeTreeNode		(RKListView 	*, KBNode *)	;
	KBNodeTreeNode		(QListViewItem	*, KBNode *)	;
}	;


/*  KBNoNodeDlg								*/
/*  -----------								*/
/*  This dialog is used when KBObject::getNamedNode fails to locate	*/
/*  an object and the show-error option is set. It displays a tree	*/
/*  view showing objects and their names, and has the option to allow	*/
/*  the user to pick an object and continue with it. Useful for		*/
/*  debugging.								*/

class	LIBKBASE_API	KBNoNodeDlg : public KBDialog
{
	Q_OBJECT

protected	:

	RKListView		*m_nodeTree	;
	KBNode			*m_starting	;
	KBNode			*m_selected	;
	bool			*m_abort	;

	RKHBox			*m_layButt	;
	RKPushButton		*m_bFail	;
	RKPushButton		*m_bAbort	;
	RKPushButton		*m_bContinue	;

protected slots :

	virtual	void		slotNodeClicked	()	;
	void			slotNodeSelected()	;
	void			clickAbort	()	;
	void			clickFail	()	;

public	:

	inline	KBNode	*selectedNode	()
	{
		return	 m_selected	;
	}

	KBNoNodeDlg
	(	KBNode		*,
		QString		,
		bool		*,
		const QString	& = QString::null,
		cchar		* = "noobjdlg"
	)	;

}	;

/*  KBRouteToNodeDlg							*/
/*  -------------=--							*/
/*  Variant of KBNoNodeDlg used to allow quick location (and automatic	*/
/*  entry of the path) of node with event script editing.		*/

class	LIBKBASE_API	KBRouteToNodeDlg : public KBNoNodeDlg
{
	Q_OBJECT

	RKPushButton		*m_bAsPath	;

protected slots	:

	virtual	void		slotNodeClicked	() ;
	void			slotNodeAsPath	() ;

public	:

	QString			routeToNode	(bool)	;

	KBRouteToNodeDlg	(KBNode *, QString, const QString & = QString::null) ;
}	;

/*  KBSlotFindDlg							*/
/*  -------------							*/
/*  This dialog is used to aid location of objects and their events	*/
/*  in the slot dialogs.						*/

class	LIBKBASE_API	KBSlotFindDlg : public KBDialog
{
	Q_OBJECT

	RKListView		*m_nodeTree	;
	RKComboBox		*m_events	;
	RKPushButton		*m_bOK		;
	RKPushButton		*m_bCancel	;

	QPtrList<KBAttr>	m_eventList	;
	KBNode			*m_selected	;
	bool			m_csonly	;


protected slots :

	void			slotNodeClicked	(QListViewItem *) ;

public	:

	KBSlotFindDlg	(KBNode *, KBNode *, const QString &, bool = false) ;

	KBNode			*selectedNode	(QString &) ;
}	;

/*  KBConfigFindDlg							*/
/*  ---------------							*/
/*  This dialog is used to aid location of objects and their child	*/
/*  configuration objects in the component override dialog.		*/

class	LIBKBASE_API	KBConfigFindDlg : public KBDialog
{
	Q_OBJECT

	RKListView		*m_nodeTree	;
	RKComboBox		*m_configs	;
	KBNode			*m_start	;
	RKPushButton		*m_bOK		;
	KBNode			*m_selected	;

protected slots :

	void			slotNodeClicked	(QListViewItem *) ;

public	:

	KBConfigFindDlg	(KBNode *, KBNode *, const QString &) ;

	KBNode			*selectedConfig	(QString &, QString &) ;
}	;

#endif
#endif	// __KB_NOOBJDLG_H
