/***************************************************************************
    file	         : kb_object.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qapplication.h>
#include	<qpalette.h>
#include	<qwhatsthis.h>
#include	<qtooltip.h>

#if		__KB_AUTOMOC
#include	"kb_object.moc"
#else
#include 	"kb_object.h"
#endif

#include	"kb_slot.h"
#include	"kb_ctrl.h"
#include	"kb_display.h"
#include	"kb_layout.h"
#include	"kb_formblock.h"
#include	"kb_reportblock.h"
#include	"kb_form.h"
#include	"kb_options.h"
#include	"kb_font.h"
#include	"kb_docroot.h"
#include	"kb_location.h"
#include	"kb_script.h"
#include	"kb_config.h"
#include	"kb_override.h"
#include	"kb_complink.h"
#include	"kb_quicktext.h"
#include	"kb_directprop.h"
#include	"kb_rowcoldialog.h"
#include	"kb_deleter.h"
#include	"kb_designpopup.h"
#include	"kb_ascii.h"
#include	"kb_test.h"
#include	"kb_recorder.h"


#define		__DEBUG_OBJECT	(0)
#if		__DEBUG_OBJECT
#include	"kb_dprintf.h"
#endif

#if		! __KB_RUNTIME
#include	"kb_popupmenu.h"
#include	"kb_formcopier.h"
#include	"kb_sizer.h"
#include	"kb_noobjdlg.h"
#include	"kb_component.h"
#include	"kb_compaccessdlg.h"
#include	"kb_nodereg.h"
#include	"kb_toolbox.h"
#include	"kb_recorder.h"
#endif

#include	"tk_fontdialog.h"
#include	"tk_colordialog.h"
#include	"tk_messagebox.h"
#include	"tk_icons.h"


static	int	_indent	;

/*  KBObject								*/
/*  KBObject	: Constructor for object class				*/
/*  parent	: KBNode *	: Parent node				*/
/*  element	: cchar *	: Element tag				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  (returns)	: KBObject	:					*/

KBObject::KBObject
	(	KBNode			*parent,
		cchar			*element,
		const QDict<QString>	&aList
	)
	:
	KBNode		(parent,   element, 		aList),
	m_geom		(this,	   	    		aList),
	m_disabled	(this,	   "disabled",		aList, KAF_FORM ),
	m_hidden	(this,	   "hidden",		aList, KAF_FORM ),
	m_skinElem	(this,	   "skinelement",	aList, KAF_CLEAR|KAF_COMMON)
{
	m_control	= 0	;
	m_block		= parent == 0 ? 0 : parent->isBlock () ;
	m_display	= 0	;
	m_curPal	= 0	;
	m_curFont	= 0	;

#if	! __KB_RUNTIME
	m_sizer		= 0	;
	m_quickText	= 0	;
#endif

	m_scriptObj	= 0 ;
	m_attrConfig	= new KBAttrStr (this, "configs", "", KAF_CONFIG) ;
	m_attrSlots	= new KBAttrStr	(this, "slots",   "", KAF_SLOTS ) ;
	m_attrTests	= new KBAttrStr	(this, "tests",   "", KAF_SLOTS ) ;
}

/*  KBObject								*/
/*  KBObject	: Constructor for new object				*/
/*  parent	: KBObject *	: Parent object				*/
/*  element	: cchar *	: Element tag				*/
/*  rect	: const QRect &	: Size and position			*/
/*  (returns)	: KBObject	:					*/

KBObject::KBObject
	(	KBObject	*parent,
		cchar		*element,
		const QRect	&rect
	)
	:
	KBNode	(parent,   element),
	m_geom
	(	this,
		rect.x     (),
		rect.y     (),
		rect.width (),
		rect.height(),
		KBAttrGeom::FMFixed, 
		KBAttrGeom::FMFixed
	),
	m_disabled	(this,	   "disabled",		false,	KAF_FORM ),
	m_hidden	(this,	   "hidden",		false,	KAF_FORM ),
	m_skinElem	(this,	   "skinelement",	"",	KAF_CLEAR|KAF_COMMON)
{
	m_control	= 0	;
	m_block		= parent == 0 ? 0 : parent->isBlock() ;
	m_display	= 0	;
	m_curPal	= 0	;
	m_curFont	= 0	;

#if	! __KB_RUNTIME
	m_sizer		= 0 	;
	m_quickText	= 0	;
#endif

	m_scriptObj	= 0 ;
	m_attrConfig	= new KBAttrStr (this, "configs", "", KAF_CONFIG) ;
	m_attrSlots	= new KBAttrStr	(this, "slots",   "", KAF_SLOTS ) ;
	m_attrTests	= new KBAttrStr	(this, "tests",   "", KAF_SLOTS ) ;
}

/*  KBObject								*/
/*  KBObject	: Constructor for new form object			*/
/*  parent	: KBNode *	: Parent				*/
/*  object	: KBObject *	: Extant object				*/
/*  (returns)	: KBObject	:					*/

KBObject::KBObject
	(	KBNode		*parent,
		KBObject	*object
	)
	:
	KBNode		(parent, 			object),
	m_geom		(this,				object),
	m_disabled	(this,	   "disabled",		object,	KAF_FORM ),
	m_hidden	(this,	   "hidden",		object,	KAF_FORM ),
	m_skinElem	(this,	   "skinelement",	object)
{
	/* If we are replicating without a parent (used for copy/paste)	*/
	/* and there is a parent, then save that parents geometry into	*/
	/* the geometry. This is needed so that copying and pasting	*/
	/* stretching and floating objects works as expected.		*/
	if
	(	(parent 			 == 0) &&
		(object->getParent()		 != 0) &&
		(object->getParent()->isObject() != 0)
	)
		m_geom.saveParentSize (object->getParent()->isObject()->geometry().size()) ;

	m_control	= 0	;
	m_display	= 0	;
	m_block		= parent == 0 ? 0 : parent->isBlock() ;
	m_curPal	= 0	;
	m_curFont	= 0	;

#if	! __KB_RUNTIME
	m_sizer		= 0	;
	m_quickText	= 0	;
#endif

	m_scriptObj	= 0 ;
	m_attrConfig	= new KBAttrStr (this, "configs", "", KAF_CONFIG) ;
	m_attrSlots	= new KBAttrStr	(this, "slots",   "", KAF_SLOTS ) ;
	m_attrTests	= new KBAttrStr	(this, "tests",   "", KAF_SLOTS ) ;
}

/*  KBObject								*/
/*  ~KBObject	: Destructor for form object class			*/
/*  (returns)	 :		:					*/

KBObject::~KBObject ()
{
	DPRINTF
	((	"KBObject::~KBObject: %s.%s: ctrl=%p\n",
		getElement().latin1(),
		getName   ().latin1(),
		(void *)m_control
	))	;

	if (m_scriptObj != 0)
	{
		for (uint code = 0 ; code < KBScriptIF::languageCount() ; code += 1)
			DELOBJ	(m_scriptObj[code]) ;
		delete	[] m_scriptObj ;
	}

	DELOBJ	(m_attrConfig) ;
	DELOBJ	(m_attrSlots ) ;
	DELOBJ	(m_control   ) ;
	DELOBJ	(m_curPal    ) ;
	DELOBJ	(m_curFont   ) ;

#if	! __KB_RUNTIME
	DELOBJ	(m_quickText ) ;
	DELOBJ	(m_sizer     ) ;
#endif
}

/*  KBObject								*/
/*  showAs	: Switch in or out of design mode			*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBObject::showAs
	(	KB::ShowAs	mode
	)
{
#if	__KB_RUNTIME
	if (m_control)
	{
		m_control->showAs (mode) ;
	}
#else	
	DELOBJ	(m_quickText ) ;

	/* If there is an associated control then create or destroy any	*/
	/* associated sizer, and show the control in the requested mode	*/
	if (m_control)
	{
		if ((mode == KB::ShowAsDesign) && (m_sizer == 0))
		{
			createSizer () ;
		}
		if ((mode == KB::ShowAsData  ) && (m_sizer != 0))
		{	getLayout()->dropSizer (m_sizer) ;
			DELOBJ	(m_sizer) ;
		}
		m_control->showAs (mode) ;
	}

	/* When toggling between design and data modes, destroy any	*/
	/* extant scripting object (so that when entering data mode we	*/
	/* start with a clean slate), and clear eny event emitters.	*/
	bool	toggle	= false	;
	if ((mode == KB::ShowAsDesign) && showingData  ()) toggle = true ;
	if ((mode == KB::ShowAsData  ) && showingDesign()) toggle = true ;

	DPRINTF
	((	"KBObject::showAs: %s: toggle=%d so=%p lc=%d\n",
		getName().latin1(),
		toggle,
		(void *)m_scriptObj,
		KBScriptIF::languageCount()
	))	;

	if (toggle)
	{
		if (m_scriptObj != 0)
		{
			DPRINTF
			((	"KBObject::showAs: %s: %d: %p %p %p\n",
				getName().latin1(),
				KBScriptIF::languageCount(),
				KBScriptIF::languageCount() > 0 ? (void *)m_scriptObj[0] : (void *)0,
				KBScriptIF::languageCount() > 1 ? (void *)m_scriptObj[1] : (void *)0,
				KBScriptIF::languageCount() > 2 ? (void *)m_scriptObj[2] : (void *)0
			))	;

			for (uint code = 0 ; code < KBScriptIF::languageCount() ; code += 1)
				DELOBJ	(m_scriptObj[code]) ;
			delete [] m_scriptObj ;
			m_scriptObj = 0	;
		}

		LITER
		(	KBAttr,
			m_attribs,
			attrib,
			if (attrib->isEvent() != 0)
				attrib->isEvent()->clearEmitter() ;
		)
	}
#endif

	KBNode::showAs (mode) ;
}

/*  KBObject								*/
/*  setChanged	: Set changed state after design operation		*/
/*  (returns)	: void		:					*/

void	KBObject::setChanged ()
{
	getLayout()->setChanged () ;
}

/*  KBObject								*/
/*  setupProperties							*/
/*		: Set current properties				*/
/*  (returns)	: void		:					*/

void	KBObject::setupProperties ()
{
	/* This method handles the basic common properties. Subclasses	*/
	/* will override it to set up control-specific properties.	*/

	/* (a)	Clear cached palettes and fonts				*/
	/* (b)	Update all properties of the current control.		*/
	setPalette   	() ;
	setFont      	() ;

	if (m_control != 0)
	{
		m_control->setCtrlGeometry
		(	QRect(m_geom.x(), m_geom.y(), m_geom.w(), m_geom.h()),
			m_geom.alignment()
		)	;
		m_control->setupProperties () ;
	}

#if	! __KB_RUNTIME
	if (showingDesign())
	{
		setChanged   () ;
		getLayout    ()->addSizer (getSizer(), false) ;
	}
#endif

	KBBlock	  *block   = getBlock() ;
	KBDisplay *display = block == 0 ? getDisplay() : block->getContainer() ;
	if (display != 0) display->redoLayout() ;
}


/*  KBObject								*/
/*  minimumGeometry							*/
/*		: Calculate minimum geometry for the object		*/
/*  (returns)	: QSize		: Minimum size				*/

QSize	KBObject::minimumGeometry ()
{
	/* This routine calculates the minimum (x, y) position of the	*/
	/* bottom-right corner of the object in its parent, ie., the	*/
	/* smallest size that the parent can be shrunk down to.		*/

	/* Start by getting the mimumum size of this object if either	*/
	/* direction stretches (otherwise it is not needed).		*/
	QSize	minSize	;
	uint	x	;
	uint	y	;

	DPRINTF
	((	"%*sKBObject::minimumGeom: .....\n",
		_indent,
		""
	))	;
	_indent	+= 4	;

	if
	(	(m_geom.xmode() == KBAttrGeom::FMStretch) ||
		(m_geom.ymode() == KBAttrGeom::FMStretch)
	)
		minSize	= minimumSize () ;

	/* The default "fixed" cases need to check whether the width	*/
	/* (or height) is zero, in which case use the corresponding	*/
	/* size hint. This is analagous to the code in ::geometry()	*/
	int	w	;
	int	h	;

	/* Now adjust for the geometry mode of this object within its	*/
	/* parent.							*/
	switch (m_geom.xmode())
	{
		case KBAttrGeom::FMFloat	:
			x = m_geom.x() ;
			break	;

		case KBAttrGeom::FMStretch	:
			x = m_geom.x() + minSize.width () ;
			break	;

		default	:
			w = m_geom.w()		;
			if ((w == 0) && (m_control != 0))
				w = m_control->getWidget()->sizeHint().width () ;
			x = m_geom.x() + w	;
			break	;
	}

	switch (m_geom.ymode())
	{
		case KBAttrGeom::FMFloat	:
			y = m_geom.y() ;
			break	;

		case KBAttrGeom::FMStretch	:
			y = m_geom.y() + minSize.height() ;
			break	;

		default	:
			h = m_geom.h()		;
			if ((h == 0) && (m_control != 0))
				h = m_control->getWidget()->sizeHint().height() ;
			y = m_geom.y() + h	;
			break	;
	}

	DPRINTF
	((	"%*sKBObject::minimumGeom: (%d,%d)\n",
		_indent,
		"",
		x, y
	))	;
	_indent	-= 4	;

	return	QSize(x, y) ;
}

/*  KBObject								*/
/*  minimumSize	: Get minimum size allowable				*/
/*  (returns)	: QSize		: Size					*/

QSize	KBObject::minimumSize()
{
	QSize	size	(0, 0) ;

	DPRINTF
	((	"KBObject::minimumSize: %s: called: pid=%d ctrl=%p mins=%s\n",
		getName().latin1(),
		parentIsDynamic (),
		(void *)m_control,
		_TEXT(QSize(m_geom.minimumWidth(), m_geom.minimumHeight()))
	))	;

	CITER
	(	Object,
		obj,
		size	= size.expandedTo (obj->minimumGeometry())
	)		

	DPRINTF
	((	"KBObject::minimumSize: %s: after scan: %s\n",
		getName().latin1(),
		_TEXT(size)
	))	;

	/* If the parent is dynamic then (a) expand to the size hint of	*/
	/* the associated control if present and (b) expand to the	*/
	/* minimum geometry for this object.				*/
	if (parentIsDynamic())
	{
		if (m_control != 0)
			size	= size.expandedTo (m_control->getWidget()->sizeHint ()) ;
		size	= size.expandedTo (QSize(m_geom.minimumWidth(), m_geom.minimumHeight())) ;
	}

	DPRINTF
	((	"KBObject::minimumSize: %s: %s\n",
		(cchar *)getName(),
		_TEXT(size)
	))	;

	return	size	;
}

/*  KBObject								*/
/*  setPalette	: Set palette						*/
/*  (returns)	: void		:					*/

void	KBObject::setPalette ()
{
	/* This is more in the nature of a reset. The cached palette	*/
	/* will be recreated as soon as any attempt is made to use it.	*/
	DELOBJ	(m_curPal) ;

	CITER
	(	Object,
		o,
		o->setPalette()
	)
}

/*  KBObject								*/
/*  setFont	: Set font						*/
/*  (returns)	: void		:					*/

void	KBObject::setFont ()
{
	/* This is more in the nature of a reset. The cached font will	*/
	/* be recreated as soon as any attempt is made to use it ...	*/
	/* which is immediately as it it passed to the geometry attr.	*/
	DELOBJ	(m_curFont) ;

	m_geom.setFont (*getFont(false)) ;

	CITER
	(	Object,
		o,
		o->setFont   ()
	)
}

/*  KBObject								*/
/*  designPopup	: Display design menu					*/
/*  parent	: KBPopupMenu * : Possible parent menu			*/
/*  cell	: QRect		: Cell for dynamic layouts		*/
/*  (returns)	: void		:					*/

KBPopupMenu
	*KBObject::designPopup
	(	KBPopupMenu	*parent,
		QRect
	)
{
#if	! __KB_RUNTIME
	KBPopupMenu *popupMain = new KBPopupMenu (parent) ;

	popupMain->setTitle   (this) ;

	popupMain->insertItem
	(	getSmallIcon("editcut"),
		TR("C&ut"),
		this,
		SLOT(cutObj         ())
	)	;
	popupMain->insertItem
	(	getSmallIcon("editcopy"),
		TR("&Copy"),
		this,
		SLOT(copyObj        ())
	)	;
	popupMain->insertItem
	(	getSmallIcon("editdelete"),
		TR("&Delete"),
	   	this,
		SLOT(deleteObj      ())
	)	;
	popupMain->insertItem
	(	getSmallIcon("filesave"),
		TR("Save as component"),
		this,
		SLOT(saveAsComponent())
	)	;
	popupMain->insertItem
	(	getSmallIcon("properties"),
		TR("&Properties"),
		this,
		SLOT(propertyDlg    ())
	)	;

	/* The structure below lists attribute shortcuts. Scan through	*/
	/* looking for attributes that appear in the object, and	*/
	/* create a submenu for each one found. The code is passed to	*/
	/* the menu item so that it will be passed to the handler slot.	*/
	struct	DirectMap
	{
		const char	*attr	;
		const char	*menu	;
		int		code	;
	}	;

	static	DirectMap	dmap[]	=
	{
		{	"font",		"&Font",	DP_FONT		},
		{	"fgcolor",	"&Foreground",	DP_FOREGROUND	},
		{	"bgcolor",	"&Background",	DP_BACKGROUND	},
		{	"text",		"&Text",	DP_TEXT		},
		{	0,		0,		0		}
	}	;

	bool	direct	= false	;
	for (DirectMap *d = &dmap[0] ; d->attr != 0 ; d += 1)
		if (getAttr(d->attr) != 0)
		{	if (!direct)
			{	popupMain->insertSeparator() ;
				direct	= true	;
			}
			popupMain->insertItem (TR(d->menu), this, SLOT(setPropDirect(int)), 0, d->code) ;
		}

	if ((parent == 0) && (parentObject() != 0))
		makeAncestorPopup (popupMain, this) ;

	popupMain->insertSeparator() ;
	popupMain->insertItem
	(	getSmallIcon("info"),
		TR("&Information"),
		this,
		SLOT(whatsThis())
	)	;

	return	popupMain ;
#endif
#if	__KB_RUNTIME
	return	0 ;
#endif
}

/*  KBObject								*/
/*  setPropDirect: Handler for direct property setting			*/
/*  code	 : int		: Property code				*/
/*  (returns)	 : void		:					*/

void	KBObject::setPropDirect
	(	int		code
	)
{
#if	! __KB_RUNTIME
	KBAttr	*attr	= 0	;
	QString	value	;

	/* First step is to get the attribute. We should find it but	*/
	/* check for the sake if paranoia.				*/
	switch (code)
	{
		case DP_FONT		: attr	= getAttr("font"   ) ; break ;
		case DP_FOREGROUND	: attr	= getAttr("fgcolor") ; break ;
		case DP_BACKGROUND	: attr	= getAttr("bgcolor") ; break ;
		case DP_TEXT		: attr	= getAttr("text"   ) ; break ;
		default			: break	;
	}

	if (attr == 0)
	{
		DPRINTF
		((	"KBObject::setPropDirect: unknown for code %d\n",
			code
		))	;
		return	;
	}

	/* Got an attribute to pick up the current value and run the	*/
	/* appropriate dialog. If the user OK's then set the value back	*/
	/* and call the property update method.				*/
	value	= attr->getValue () ;

	switch (code)
	{
		case DP_FONT		:
			{
			TKFontDialog  fDialog (0, TR("Font"), false, true) ;
			fDialog.setFont (KBFont::specToFont (value)) ;
			if (!fDialog.exec()) return ;
			value	= KBFont::fontToSpec (fDialog.font()) ;
			}
			break	;

		case DP_FOREGROUND	:
		case DP_BACKGROUND	:
			{
			TKColorDialog cDialog (0, TR("Colour"), true) ;
			cDialog.setColor (QColor (value.toInt())) ;
			if (!cDialog.exec ()) return ;
			value.sprintf ("0x%06x", cDialog.color().rgb() & 0x00ffffff) ;
			}
			break	;

		case DP_TEXT	:
			{
			/* This is special. Create a quick text edit	*/
			/* and then leave it to run. The completion	*/
			/* event will be picked up in the event handler	*/
			/* below.					*/
			DELOBJ	(m_quickText) ;
			m_quickText = new KBQuickText
					  (	m_control->getWidget(),
						QRect
						(	0,
							0,
							m_control->getWidget()->width (),
							m_control->getWidget()->height()
						),
						value,
						this,
						(QEvent::Type)(QEvent::User + 1000)
					  )	;
			m_quickText->show     () ;
			m_quickText->setFocus () ;
			return	   ;
			}

		default	:
			return	;
	}

	attr->setValue	(value) ;
	setupProperties	()	;
#endif
}

/*  KBObject								*/
/*  event	: Local event handler					*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBObject::event
	(	QEvent		*e
	)
{
#if	! __KB_RUNTIME
	/* Check for the quick text completion event. If it is *and*	*/
	/* there is a quick text widget (paranoia check) *and* there	*/
	/* is a text property (more paranoia) then update it.		*/
	if (e->type() == (QEvent::Type)(QEvent::User + 1000))
	{
		if (m_quickText != 0)
		{
			KBAttr	*attr	= getAttr("text") ;
			if (attr != 0)
			{	attr->setValue	(m_quickText->text()) ;
				setupProperties	() ;
			}

			DELOBJ	(m_quickText) ;
		}

		return	true	;
	}
#endif
	return	KBNode::event (e) ;
}

/*  KBObject								*/
/*  recordVerifyState							*/
/*		: Record state for verification				*/
/*  (returns)	: void		:					*/

void	KBObject::recordVerifyState ()
{
#if	! __KB_RUNTIME
	if (m_control != 0)
	{
		KBRecorder *recorder = KBRecorder::self() ;
		if ((recorder != 0) && recorder->isRecording(getDocRoot()))
			recorder->verifyState
			(	this,
				0,
				m_control->isEnabled(),
				m_control->isVisible()
			)	;
	}
#endif
}


/*  KBObject								*/
/*  contextMenu	: Display context menu					*/
/*  e		: QMouseEvent *	: Triggering mouse event		*/
/*  drow	: uint		: Display row				*/
/*  (returns)	: bool		: Menu displayed			*/

bool	KBObject::contextMenu
	(	QMouseEvent	*,
		uint		drow
	)
{
#if	! __KB_RUNTIME
	KBPopupMenu *popup = makeTestsPopup (0, drow) ;
	if (popup != 0)
	{
		popup->exec (QCursor::pos()) ;
		delete	popup	;
		return	true	;
	}
#endif
	return	false	;
}

/*  KBObject								*/
/*  slotExecTest: Execute a test					*/
/*  id		: int		: Test index				*/
/*  (returns)	: void		:					*/

void	KBObject::slotExecTest
	(	int		idx
	)
{
	if (idx < (int)m_testList.count())
	{	bool evRc ;
		eventHook (*m_testList.at(idx), 0, 0, evRc) ;
	}
}


/*  KBObject								*/
/*  moveFocusOK	: Check if OK to set focus				*/
/*  drow	: uint		: Display row				*/
/*  (returns)	: bool		: OK					*/

bool	KBObject::moveFocusOK
	(	uint	
	)
{
#if	__KB_DEBUG_FOCUS
	fprintf
	(	kbDPrintfGetStream(),
		"KBObject::moveFocusOK: %s:\n",
		(cchar *)getName()
	)	;
#endif
	return	showingDesign()	     ? true :
		getFormBlock () == 0 ? true :
				       getFormBlock()->focusOutOK (false) ;
}

void	KBObject::setFocus ()
{
#if	__KB_DEBUG_FOCUS
	fprintf
	(	kbDPrintfGetStream(),
		"__KB_DEBUG_FOCUS: %s: m_control=%p\n",
		(cchar *)getName(),
		(void  *)m_control
	)	;
#endif
	if (m_control != 0) m_control->setFocus () ;
}

/*  KBObject								*/
/*  focusInEvent: Handle focus in event					*/
/*		: uint		: Display row				*/
/*		: Reason	: Reason focus arrives			*/
/*  (returns)	: void		:					*/

void	KBObject::focusInEvent
	(	uint		,
		QFocusEvent::Reason
	)
{
	/* Nothing to do here. Derived classes will implement this as	*/
	/* neccessary.							*/
#if	__KB_DEBUG_FOCUS
	fprintf
	(	kbDPrintfGetStream(),
		"KBObject::focusInEvent: %s:\n",
		(cchar *)getName()
	)	;
#endif
}

/*  KBObject								*/
/*  geometry	: Get geometry of this object				*/
/*  (returns)	: QRect		: Extent				*/

QRect	KBObject::geometry ()
{
	QRect	g = m_geom.geometry () ;

	/* Zero widths or heights are mapped to the corresponding size	*/
	/* hint. Mostly useful for height, so that controls can adjust	*/
	/* to fonts when not in a dynamic parent.			*/
	if ((g.width () == 0) || (g.height() == 0))
		if (m_control != 0)
		{
			QSize hint = m_control->getWidget()->sizeHint() ;
			if (g.width () == 0) g.setWidth (hint.width ()) ;
			if (g.height() == 0) g.setHeight(hint.height()) ;
		}

	return	g ;
}

/*  KBObject								*/
/*  geometry	: Get geometry of this object with offset		*/
/*  offset	: const QPoint & : Offset to apply			*/
/*  (returns)	: QRect		 : Extent				*/

QRect	KBObject::geometry
	(	const QPoint	&offset
	)
{
	return	m_geom.geometry (offset) ;
}

QRect	KBObject::realGeometry ()
{
	DPRINTF
	((	"KBObject::realGeometry: [%s.%s] cont=%p ctrl=%p\n",
		(cchar *)getElement  (),
		(cchar *)getName     (),
		(void  *)getContainer(),
		(void  *)m_control
	))	;

	if (getContainer() != 0)
	{
		if (getParent() == 0)
			return	QRect(QPoint(0,0), getContainer()->getTopSize()) ;

		return	getContainer()->getTopRect() ;
	}

	if (m_control != 0)
	{
		QRect	r = m_control->getWidget()->geometry() ;
		getDisplay()->cvtViewToCtrl (r) ;
		return	r ;
	}

	return	QRect	(0,0,0,0) ;
}

/*  KBObject	:							*/
/*  sizeHint	: Get ideal size for object				*/
/*  (returns)	: QSize		: Size					*/

QSize	KBObject::sizeHint ()
{
	/* If the parent is dynamic then return the minimum size that	*/
	/* will completely display our content, otherwise use the	*/
	/* design geometry.						*/
	return	parentIsDynamic() ?
			minimumSize() :
			m_geom.geometry(true).size() ;
}

/*  KBObject								*/
/*  ctrlGone	: Note death of control					*/
/*  ctrl	: KBControl *	: The control				*/
/*  (returns)	: void		:					*/

void	KBObject::ctrlGone
	(	KBControl	*ctrl
	)
{
	if (ctrl == m_control)
		m_control = 0 ;
}

/*  KBObject								*/
/*  displayResized							*/
/*		: Block display has been resized			*/
/*  size	: const QSize & : New size				*/
/*  (returns)	: void		:					*/

void	KBObject::displayResized
	(	const QSize		&
	)
{
	/* Null operation. Derived classes override this method.	*/
}

void	KBObject::recalcNumRows
	(	const QSize		&
	)
{
}

/*  KBObject								*/
/*  propertyDlg	: Default property dialog				*/
/*  (returns)	: void		:					*/

bool	KBObject::propertyDlg ()
{
	return	propertyDlg (0) ;
}

/*  KBObject								*/
/*  propertyDlg	: Default property dialog				*/
/*  iniAttr	: cchar *	: Initial attribute to display		*/
/*  (returns)	: void		:					*/

bool	KBObject::propertyDlg
	(	cchar	*iniAttr
	)
{
#if	! __KB_RUNTIME
	if (KBNode::basePropertyDlg (TR("Properties"), iniAttr))
	{
		getLayout()->addSizer (getSizer(), false) ;
		return	true	;
	}
#endif
	return	false	;
}

/*  KBObject								*/
/*  deleteObj	: Delete the object					*/
/*  (returns)	: void		:					*/

void	KBObject::deleteObj ()
{
#if	! __KB_RUNTIME
	if (getParent() == 0)
	{
		TKMessageBox::sorry
		(	0,
			QString(TR("Cannot delete the top-most object")),
			TR("Error deleting object")
		)	;
		return	;
	}

	getLayout()->setChanged  () ;
	KBDeleter::addObject (this) ;
#endif
}

/*  KBObject								*/
/*  saveAsComponent							*/
/*		: Save object as component				*/
/*  (returns)	: void		:					*/

void	KBObject::saveAsComponent ()
{
#if	! __KB_RUNTIME

	const KBLocation  &locn1   = getRoot()->getDocRoot()->getDocLocation() ;
	QString		  svName   = locn1.server() ;
	QString		  objName  ;
	QString		  comment  ;

	/* Run the component save-as dialog, which will return the	*/
	/* desired object name, server, and optionally a comment.	*/
	bool		   toFile ;
	KBComponentSaveDlg csDlg
			   (	objName,
				svName,
				comment,
				locn1.dbInfo(),
				&toFile
			   )	;

	if (!csDlg.exec()) return ;


	/* We make a copy of the object, then adjust its position so	*/
	/* that the top-left one goes to (20,20). Note that this is	*/
	/* done to function correctly for stretching/floating objects.	*/
	KBObject *copy	= (KBObject *)replicate(0) ;
	QRect	 r	= copy->geometry() ;
	copy->setGeometry (QRect(QPoint(20, 20), r.size())) ;

	/* OK, got a name and stuff, so stitch together the text with	*/
	/* the top level component settings ....			*/
	extern	QString	kbXMLEncoding   () ;

	QString	 text	= QString
			  (	"<?xml version=\"1.0\" encoding=\"%1\"?>\n"
			  	"<KBComponent w=\"%2\" h=\"%3\" type=\"%4\" notes=\"%5\">\n" 
			  )
			  .arg	(kbXMLEncoding())
			  .arg	(r.width () + 40)
			  .arg	(r.height() + 40)
			  .arg	((uint)objType())
			  .arg	(comment) ;


	copy->printNode (text, 2, false)  ;
	text += "</KBComponent>\n" ;

	/*  .... and save the component where requested. The special	*/
	/* case is if the used specified save-to-file.			*/
	if (toFile)
	{
		saveComponentToFile (objName, text) ;
		return	;
	}

	KBLocation	locn2
			(	locn1.dbInfo(),
				"component",
				svName,
				objName
			)	;
	KBError		error	;

	if (!locn2.save (QString::null, QString::null, text, error))
		error.DISPLAY() ;
#endif
}

/*  KBObject								*/
/*  copyObj	: Copy the object					*/
/*  (returns)	: void		:					*/

void	KBObject::copyObj ()
{
#if	! __KB_RUNTIME
	KBFormCopier::self()->clearCopy () ;
	KBFormCopier::self()->addToCopy (replicate(0), objType()) ;
#endif
}

/*  KBObject								*/
/*  cutObj	: Cut the object					*/
/*  (returns)	: void		:					*/

void	KBObject::cutObj ()
{
#if	! __KB_RUNTIME
	if (getParent() == 0)
	{
		TKMessageBox::sorry
		(	0,
			QString(TR("Cannot cut the top-most object")),
			TR("Error cutting object")
		)	;
		return	;
	}

	copyObj   () ;
	deleteObj () ;

	getLayout ()->setChanged () ;
#endif
}


/*  KBObject								*/
/*  pasteObjects: Paste in cut or copied objects			*/
/*  (returns)	: void		:					*/

void	KBObject::pasteObjects ()
{
	/* Placeholder only. Overridden in KBBlock, KBFramer and	*/
	/* KBComponent.							*/
}

/*  KBObject								*/
/*  calcNumRows	: Calculate number of rows of controls			*/
/*  bw		: uint		: Block display width			*/
/*  bh		: uint		: Block display height			*/
/*  dx		: int		: Control X-spacing			*/
/*  dy		: int		: Control X-spacing			*/
/*  (returns)	: uint		: Number of rows that can be displayed	*/

uint	KBObject::calcNumRows
	(	uint	bw,
		uint	bh,
		int	dx,
		int	dy
	)
{
	if (getRoot()->isReport() != 0) return 1 ;

	QRect	g	= geometry() ;

	/* Calculate dynamically how many rows can be displayed (but	*/
	/* always display at least one whatever).			*/
	int	xmax	= dx <= 0 ? 0 : (bw - g.width () - m_geom.x()) / dx + 1 ;
	int	ymax	= dy <= 0 ? 0 : (bh - g.height() - m_geom.y()) / dy + 1 ;

	int	numrows	= xmax == 0    ? ymax :
			  ymax == 0    ? xmax :
			  xmax >  ymax ? ymax :
					 xmax ;

	DPRINTF
	((	"KBObject::calcNumRows: dy=%2d bh=%3d y=%3d -> %d\n",
		dy,
		bh,
		m_geom.y(),
		numrows
	))	;

	return	numrows	<= 0 ? 1 : numrows    ;
}

/*  KBObject								*/
/*  calcNumRows	: Calculate number of rows of controls			*/
/*  dx		: int		: Control X-spacing			*/
/*  dy		: int		: Control X-spacing			*/
/*  mRows	: uint		: Maximum number of rows		*/
/*  (returns)	: uint		: Number of rows that can be displayed	*/

uint	KBObject::calcNumRows
	(	int	dx,
		int	dy,
		uint	mRows
	)
{
	uint	nRows	= calcNumRows (m_display->width(), m_display->height(), dx, dy) ;
	return	nRows > mRows ? mRows : nRows ;
}

/*  KBObject								*/
/*  buildDisplay: Build display for this item				*/
/*  parent	: KBDisplay *	: Parent display			*/
/*  (returns)	: void		:					*/

void	KBObject::buildDisplay
	(	KBDisplay	*parent
	)
{
	m_display = parent  ;
}

/*  KBItem								*/
/*  ctrlGeometry: Get control geometry					*/
/*  qrow	: uint		: Query row number			*/
/*  rCtrl	: QRect &	: Result				*/
/*  (returns)	: bool		: Success				*/

bool	KBObject::ctrlGeometry
	(	QRect		&rCtrl
	)
{
	if (m_control != 0)
	{	rCtrl	= m_control->ctrlGeometry() ;
		return	true ;
	}

	rCtrl	= QRect() ;
	return	true ;
}

void	KBObject::setCtrlGeometry
	(	const QRect	&rect
	)
{
	if (m_control != 0)
		m_control->setCtrlGeometry (rect, m_geom.alignment()) ;
}

/*  KBObject								*/
/*  setGeometry	: Set object geometry					*/
/*  rect	: const QRect &	: New geometry				*/
/*  (returns)	: void		:					*/

void	KBObject::setGeometry
	(	const QRect	&rect
	)
{
	/* The geometry will be invalid if pasting into a dynamic	*/
	/* contained, in which case ignore it. Just redo the layout.	*/
	if (rect.isValid())
	{
		m_geom.setGeometry (rect) ;
		setCtrlGeometry	   (QRect(m_geom.x(), m_geom.y(), m_geom.w(), m_geom.h())) ;
	}

	if (m_display != 0)
		m_display->redoLayout () ;
}

/*  KBObject								*/
/*  getPalette	: Get palette for this object				*/
/*  useParent	: bool		: Use parent's palette			*/
/*  (returns)	: QPalette *	: The palette				*/

QPalette *KBObject::getPalette
	 (	bool	useParent
	 )
{
	static	QPalette *appPalette	= 0 ;
	static	QPalette *whitePalette	= 0 ;

	if (appPalette == 0)
	{
		appPalette	= new QPalette (QApplication::palette()) ;
		whitePalette	= new QPalette (QApplication::palette()) ;

		whitePalette->setColor (QColorGroup::Background, Qt::white) ;
	}

	if (m_curPal == 0)
	{
		KBDocRoot *docRoot = getRoot()->getDocRoot() ;
		QString	  skinElem = m_skinElem.getValue  () ;
		QString	  fgcolor  = getAttrVal  ("fgcolor") ;
		QString	  bgcolor  = getAttrVal  ("bgcolor") ;

		if (fgcolor.isEmpty() && !skinElem.isEmpty())
			fgcolor = docRoot->skinFGColor (skinElem) ;
		if (bgcolor.isEmpty() && !skinElem.isEmpty())
			bgcolor = docRoot->skinBGColor (skinElem) ;

		QPalette pal	 ;

		if	((getDisplay () != 0) && useParent)
			pal	= getDisplay()->palette() ;
		else if (getRoot()->isReport())
			pal	= *whitePalette ;
		else	pal	= *appPalette   ;

		if (!fgcolor.isEmpty())
		{	const QColor fg = QColor (QRgb(fgcolor.toInt(0,0))) ;
			pal.setColor (QColorGroup::Text,       fg) ;
			pal.setColor (QColorGroup::ButtonText, fg) ;
			pal.setColor (QColorGroup::Foreground, fg) ;
		}
		if (!bgcolor.isEmpty())
		{	const QColor bg = QColor (QRgb(bgcolor.toInt(0,0))) ;
			pal.setColor (QColorGroup::Base,       bg) ;
			pal.setColor (QColorGroup::Button,     bg) ;
			pal.setColor (QColorGroup::Background, bg) ;
		}

		m_curPal = new QPalette (pal) ;
	}

	return	m_curPal ;
}

/*  KBObject								*/
/*  getFont	: Get font for this object				*/
/*  useParent	: bool		: Use parent's font			*/
/*  (returns)	: QFont *	: The font				*/

QFont	*KBObject::getFont
	(	bool	useParent
	)
{
	if (m_curFont == 0)
	{
		/* Try in order:					*/
		/* (a) Locally defined font				*/
		/* (b) Skin defined font				*/
		/* (c) Parent font					*/
		/* (d) Rekall-specified application font		*/
		/* (e) Application font					*/
		QString	spec = getAttrVal ("font") ;
		if (!spec.isEmpty())
			return	m_curFont = new QFont(KBFont::specToFont (spec)) ;

		QString	skinElem = m_skinElem.getValue  () ;
		if (!skinElem.isEmpty())
		{
			KBDocRoot *docRoot = getRoot()->getDocRoot()	 ;
			QString	  sspec	   = docRoot->skinFont(skinElem) ;
			if (!sspec.isEmpty())
				return	m_curFont = new QFont(KBFont::specToFont (sspec)) ;
		}

		if ((getDisplay () != 0) && useParent)
			return	m_curFont = new QFont(getDisplay()->font()) ;

		const QString &font = getDocRoot()->appFont() ;
		if (!font.isEmpty())
			return	m_curFont = new QFont(KBFont::specToFont (font)) ;

		return	m_curFont = new QFont(QApplication::font()) ;
	}

	return	m_curFont ;
}

/*  KBObject								*/
/*  setControl	: Set the control to be used for sizing			*/
/*  control	: KBControl *	: Control in question			*/
/*  (returns)	: void		:					*/

void	KBObject::setControl
	(	KBControl *control
	)
{
	if (control != m_control)
	{
		m_control = control ;

#if	! __KB_RUNTIME
		DELOBJ (m_sizer) ;
		if (showingDesign() && (m_control != 0)) createSizer ()	;
#endif
	}
}

/*  KBObject								*/
/*  getDisplayWidget							*/
/*		: Get widget used for display				*/
/*  (returns)	: QWidget *	: Display widget			*/

QWidget	*KBObject::getDisplayWidget ()
{
	return	m_display == 0 ? 0 : m_display->getDisplayWidget () ;
}

/*  KBObject								*/
/*  getTopWidget: Get top-most widget					*/
/*  (returns)	: QWidget *	: Top-most widget			*/

QWidget	*KBObject::getTopWidget ()
{
	return	m_display == 0 ? 0 : m_display->getTopWidget () ;
}

/*  KBObject								*/
/*  keyStroke	: Handle keystroke					*/
/*  e		: QKeyEvent *	: Event to be handled			*/
/*  (returns)	: bool		: True if event consumed		*/

bool	KBObject::keyStroke
	(	QKeyEvent	*
	)
{
	return	false	;
}

/*  KBObject								*/
/*  getFormBlock: Get form block containing this object			*/
/*  (returns)	: KBFormBlock *	: Form block or null if none		*/

KBFormBlock *KBObject::getFormBlock ()
{
	for (KBNode *p = getParent () ; p != 0 ; p = p->getParent())
		if (p->isFormBlock() != 0)
			return	p->isFormBlock() ;
	return	0 ;
}

/*  KBObject								*/
/*  getReportBlock: Get report block containing this object		*/
/*  (returns)	  : KBReportBlock *	: Report block or null if none	*/

KBReportBlock *KBObject::getReportBlock ()
{
	for (KBNode *p = getParent () ; p != 0 ; p = p->getParent())
		if (p->isReportBlock() != 0)
			return	p->isReportBlock() ;
	return	0 ;
}

/*  KBObject								*/
/*  getNavigator: Get navigator containing this object			*/
/*  (returns)	: KBNavigator *	: Navigator or null if none		*/

KBNavigator *KBObject::getNavigator ()
{
	for (KBNode *p = getParent () ; p != 0 ; p = p->getParent())
		if (p->isNavigator() != 0)
			return	p->isNavigator() ;
	return	0 ;
}

/*  KBObject								*/
/*  write	: Write object to report writer				*/
/*  writer	: KBWriter *	: Writer object				*/
/*  offset	: QPoint	: Geometry offset			*/
/*  first	: bool		: First on page				*/
/*  extra	: init &	: Return extra space			*/
/*  prior	: bool		: Write prior value flag		*/
/*  (returns)	: void		:					*/

bool	KBObject::write
	(	KBWriter	*writer,
		QPoint		offset,
		bool		,
		int		&extra,
		bool		prior
	)
{
	/* The work is all done at the control level.			*/
	return	m_control->write
		(	writer,
			geometry(offset),
			KBValue(),
			false,
			extra
		)	;
}

/*  KBObject								*/
/*  setEnabled	: Set control enabled/disabled				*/
/*  enabled	: bool		: Enabled/disabled			*/
/*  (returns)	: void		:					*/

void	KBObject::setEnabled
	(	bool	enabled
	)
{
	if (m_control != 0) m_control->setEnabled (enabled) ;
}

/*  KBObject								*/
/*  setVisible	: Set control visibility				*/
/*  visible	: bool		: Visibility				*/
/*  (returns)	: void		:					*/

void	KBObject::setVisible
	(	bool	visible
	)
{
	if (m_control != 0) m_control->setVisible (visible) ;
}

/*  KBObject								*/
/*  isEnabled	: Get control enabled/disabled				*/
/*  (returns)	: bool		: Enabled/disabled			*/

bool	KBObject::isEnabled ()
{
	return	m_control == 0 ? false : m_control->isEnabled () ;
}

/*  KBObject								*/
/*  isVisible	: Get control visibility				*/
/*  (returns)	: bool		: Visibility				*/

bool	KBObject::isVisible ()
{
	return	m_control == 0 ? false : m_control->isVisible () ;
}

/*  KBObject								*/
/*  getMoveLimit: Get control movement limits				*/
/*  (returns)	: KBLimit	: Limit information			*/

KBLimit	KBObject::getMoveLimit ()
{
	if (getParent() == 0) return KBLimit () ;

	QSize	pSize	= getDisplay()->getTopSize() ;
	QRect	mRect	= geometry  () ;
 
	return	KBLimit
		(	-mRect.left(), pSize.width () - mRect.right (),
			-mRect.top (), pSize.height() - mRect.bottom()
		)	;
}

/*  KBObject								*/
/*  eventHook	 : Event execution hook					*/
/*  event	 : KBEvent &	: Event to execute			*/
/*  argc	 : uint		: Argument count			*/
/*  argv	 : KBValue *	: Argument vector			*/
/*  evRc	 : bool &	: Event return code			*/
/*  defres	 : bool	 	: Default return code			*/
/*  (returns)	 : bool		: No execution error			*/

bool	KBObject::eventHook
	(	KBEvent		&event,
		uint		argc,
		KBValue		*argv,
		bool		&evRc,
		bool		defval
	)
{
	KBValue		resval	;
	KBScriptError	*rc	;

#if	__KB_DEBUG_EVENT
	fprintf
	(	kbDPrintfGetStream(),
		"KBObject::eventHook: [%s][%s][%s] executing event ....\n",
		(cchar *)getElement   (),
		(cchar *)getName      (),
		(cchar *)event.getName()
	)	;
#endif
	rc = event.execute (resval, argc, argv, defval) ;
	if (rc != 0)
	{
		KBScriptError::processError (rc, KBScriptError::Normal) ;
		return	false	;
	}

	/* No execution errors, so result is true with "evRc" set to	*/
	/* the value returned by the script.				*/
	evRc	= resval.isTrue() ;
	return	true	;
}

/*  KBObject								*/
/*  property	: Get QT widget property				*/
/*  name	: cchar *	: Property name				*/
/*  (returns)	: QVariant	: Property value			*/

QVariant
	KBObject::property
	(	cchar		*name
	)
{
	return	m_control == 0 ? QVariant() : m_control->property (name) ;
}

/*  KBObject								*/
/*  setProperty	: Set QT widget property				*/
/*  name	: cchar *	   : Property name			*/
/*  value	: const QVariant & : Value				*/
/*  (returns)	: bool		   : Success				*/

bool	KBObject::setProperty
	(	cchar		*name,
		const QVariant	&value
	)
{
	return	m_control == 0 ? false : m_control->setProperty (name, value) ;
}

void	KBObject::setTabOrder
	(	int
	)
{
}

int	KBObject::getTabOrder ()
{
	return	-1 ;
}


/*  KBObject								*/
/*  setScriptObject							*/
/*		: Set persistent script object				*/
/*  code	: uint			: Language code			*/
/*  scriptObj	: KBScriptObject *	: Persistent script object	*/
/*  (returns)	: void			:				*/

void	KBObject::setScriptObject
	(	uint		code,
		KBScriptObject	*scriptObj
	)
{
	DPRINTF
	((	"KBObject::setScriptObject: %s: code=%d obj=%p\n",
		getName().latin1(),
		code,
		(void *)scriptObj
	))	;

	/* Script objects are stored against the language code so that	*/
	/* we handle the case where we have multiple languages that	*/
	/* access the same object at the same time. Needed, say, when	*/
	/* running a mixed Python/KJS form.				*/
	if (m_scriptObj == 0)
	{
		m_scriptObj = new KBScriptObject * [KBScriptIF::languageCount()] ;
		memset	(m_scriptObj, 0, KBScriptIF::languageCount() * sizeof(KBScriptObject *)) ;
	}

	m_scriptObj[code] = scriptObj ;
}

/*  KBObject								*/
/*  scriptObject: Get persisistent script object			*/
/*  code	: uint			: Language code			*/
/*  (returns)	: KBScriptObject *	: Persistent script object	*/

KBScriptObject
	*KBObject::scriptObject
	(	uint		code
	)
{
	/* We assume that the language code is valid ....		*/
	return	m_scriptObj == 0 ? 0 : m_scriptObj[code] ;
}



/*  KBObject								*/
/*  parentObject: Return parent if parent is an object			*/
/*  (returns)	: KBObject *	: Parent if object			*/

KBObject*KBObject::parentObject()
{
	if (getParent() == 0) return false ;
	return	getParent()->isObject()	   ;
}
	
/*  KBObject								*/
/*  parentIsDynamic							*/
/*		: See if object is embedded in a dynamic layout parent	*/
/*  (returns)	: bool		: True if so				*/

bool	KBObject::parentIsDynamic ()
{
	if (parentObject() == 0) return false ;
	return	parentObject()->isDynamic()   ;
}

/*  KBObject								*/
/*  minPosition	: Get minimal x- and y- positions from a list of nodes	*/
/*  nodes	: const QPtrList<KBNode> &	: List of notes			*/
/*  minx	: int &			: Return minimum x-position	*/
/*  miny	: int &			: Return minimum y-position	*/
/*  (returns)	: void			:				*/

void	KBObject::minPosition
	(	const QPtrList<KBNode>	&nodes,
		int			&minx,
		int			&miny
	)
{
	KBObject *oObj	      ;
	minx	 = 0x7fffffff ;
	miny	 = 0x7fffffff ;

	LITER
	(	KBNode,
		nodes,
		oNode,

		if ((oObj = oNode->isObject()) == 0)
			continue ;

		if (oObj->isHidden() != 0)
			continue ;

		QRect	r = oObj->geometry() ;
		int 	x = r.x() ;
		int	y = r.y() ;

		if (x < minx) minx = x ;
		if (y < miny) miny = y ;
	)
}

/*  KBObject								*/
/*  newNode	: Add a new node					*/
/*  id		: int		: Actually NodeSpec pointer for node	*/
/*  (returns)	: KBNode *	: The new node				*/

KBNode	*KBObject::newNode
	(	int		
	)
{
	/* Stub implementation. Overridden in classes that need to	*/
	/* implement it.						*/
	return	0	;
}

/*  KBObject								*/
/*  newContainer: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::newContainer ()
{
}

/*  KBObject								*/
/*  newTableBlock							*/
/*		: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::newTableBlock ()
{
}

/*  KBObject								*/
/*  newQueryBlock							*/
/*		: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::newQueryBlock ()
{
}

/*  KBObject								*/
/*  newSQLBlock	: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::newSQLBlock ()
{
}

/*  KBObject								*/
/*  newNullBlock: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::newNullBlock ()
{
}

/*  KBObject								*/
/*  pasteComponent							*/
/*		: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::pasteComponent ()
{
}

/*  KBObject								*/
/*  linkComponent							*/
/*		: Virtual place holder					*/
/*  (returns)	:		:					*/

void	KBObject::linkComponent ()
{
}

/*  ------------------------------------------------------------------	*/

/*  KBObject								*/
/*  setCtrlFGColor							*/
/*		: Set control foreground colour				*/
/*  color	: const QString & : Color				*/
/*  (returns)	: void		  :					*/

void	KBObject::setCtrlFGColor
	(	const QString	&color
	)
{
	/* This call is used to allow scripts to conveniently change	*/
	/* the colour of non-data controls like buttons.		*/
	if (m_control != 0)
	{
		QPalette pal = m_control->getWidget()->palette () ;
		pal.setColor (QColorGroup::Text,       color) ;
		pal.setColor (QColorGroup::ButtonText, color) ;
		pal.setColor (QColorGroup::Foreground, color) ;
		m_control->getWidget()->setPalette (pal) ;
	}
}

/*  KBObject								*/
/*  setCtrlBGColor							*/
/*		: Set control background colour				*/
/*  color	: const QString & : Color				*/
/*  (returns)	: void		  :					*/

void	KBObject::setCtrlBGColor
	(	const QString	&color
	)
{
	if (m_control != 0)
	{
		QPalette pal = m_control->getWidget()->palette () ;
		pal.setColor (QColorGroup::Base,       color) ;
		pal.setColor (QColorGroup::Button,     color) ;
		pal.setColor (QColorGroup::Background, color) ;
		m_control->getWidget()->setPalette (pal) ;
	}
}

/*  KBObject								*/
/*  setCtrlFont	: Set control font					*/
/*  font	: const QString & : Font				*/
/*  (returns)	: void		  :					*/

void	KBObject::setCtrlFont
	(	const QString	&spec
	)
{
	/* .... similarly to the above for fonts			*/
	if (m_control != 0)
	{
		QFont	font	;

		if	(spec.isEmpty())
			font	= QApplication::font() ;
		else	font	= KBFont::specToFont (spec) ;

		m_control->getWidget()->setFont (font) ;
	}
}

/*  ------------------------------------------------------------------	*/

/*  KBObject								*/
/*  enumKBProperty							*/
/*		: Get list of properties				*/
/*  list	: QStringList &	: Property list				*/
/*  (returns)	: void		:					*/

void	KBObject::enumKBProperty
	(	QStringList	&list
	)
{
	list.append ("visible"	 ) ;
	list.append ("enabled"	 ) ;
	list.append ("__parent__") ;
	list.append ("__block__" ) ;
	list.append ("__root__"  ) ;

	KBNode::enumKBProperty (list) ;

	LITER
	(	KBNode,
		getChildren(),
		node,
		list.append (node->getName()) ;
	)
}

/*  KBObject								*/
/*  hasKBProperty: Check if object has this property			*/
/*  name	 : cchar *	: Property name				*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBObject::hasKBProperty
	(	cchar		*name
	)
{
	if (qstrcmp (name, "visible"   ) == 0)
		return	true ;
	if (qstrcmp (name, "enabled"   ) == 0)
		return	true ;
	if (qstrcmp (name, "__parent__") == 0)
		return	true ;
	if (qstrcmp (name, "__block__" ) == 0)
		return	true ;
	if (qstrcmp (name, "__root__"  ) == 0)

	if (KBNode::hasKBProperty (name))
		return	true	;

	LITER
	(	KBNode,
		getChildren(),
		node,
		if (node->getName() == name) return true ;
	)

	return	false	;
}

/*  KBObject								*/
/*  getKBProperty: Get a rekall property value				*/
/*  value	 : KBValue &	: Return property value			*/
/*  (returns)	 : bool		: Property exists			*/

bool	KBObject::getKBProperty
	(	cchar		*name,
		KBValue		&value
	)
{
	DPRINTF
	((	"KBObject::getKBProperty: for [%s]\n",
		name
	))	;

	if (qstrcmp (name, "visible") == 0)
	{
		value	= isVisible() ;
		return	true ;
	}

	if (qstrcmp (name, "enabled") == 0)
	{
		value	= isEnabled() ;
		return	true ;
	}

	/* The names "__parent__", "__block__" and "__root__" map to	*/
	/* the obvious ancestral objects. We uses __ as a prefixsuffix	*/
	/* to try to avoid clashes with names that people might		*/
	/* plausibly have used. It also distinguishes them from "real"	*/
	/* properties and likely child names (see below).		*/
	if (qstrcmp (name, "__parent__") == 0)
	{
		value	= KBValue(getParent()) ;
		return	true ;
	}

	if (qstrcmp (name, "__block__" ) == 0)
	{
		value	= KBValue(getBlock ()) ;
		return	true ;
	}

	if (qstrcmp (name, "__root__"  ) == 0)
	{
		value	= KBValue(getRoot  ()) ;
		return	true ;
	}

	/* Recurse down to the base class. Note that we do not fail	*/
	/* immediatley if this fails ...				*/
	if (KBNode::getKBProperty (name, value))
		return	true	;


	/* No match so far so see if the name identifies a child	*/
	/* object. This means that the user (as of Rekall 2.3.3) can	*/
	/* access other objects in the tree without needing to call	*/
	/* the "getNamedCtrl()" method.					*/
	LITER
	(	KBNode,
		getChildren(),
		node,

		if (node->getName() == name)
		{
			value	= KBValue (node) ;
			return	true	;
		}
	)

	/* Bollox ...							*/
	return	false	;
}

/*  KBObject								*/
/*  setKBProperty: Set a rekall property value				*/
/*  name	 : cchar *	: Proprty name				*/
/*  value	 : KBValue	: Property value			*/
/*  (returns)	 : bool		: Success				*/

bool	KBObject::setKBProperty
	(	cchar		*name,
		const KBValue	&value
	)
{
	if (qstrcmp (name, "visible") == 0)
	{
		setVisible (value.isTrue()) ;
		return	true ;

	}

	if (qstrcmp (name, "enabled") == 0)
	{
		setEnabled (value.isTrue()) ;
		return	true ;

	}

	return	KBNode::setKBProperty (name, value) ;
}

/*  KBObject								*/
/*  insertDynamicRow							*/
/*		: Insert row into dynamic layout			*/
/*  (returns)	: void		:					*/

void	KBObject::insertDynamicRow ()
{
#if	! __KB_RUNTIME
	if (isDynamic())
	{
		int	from	= newCtrlRect().y() ;

		m_geom.insertRow (from) ;
		getContainer()->updateDynamic() ;

		CITER
		(	Object,
			o,

			QRect	rect	= o->geometry() ;
			if	(rect.y() 		  >= from) rect.moveBy    (0, 1) ;
			else if (rect.y() + rect.height() >  from) rect.setHeight (rect.height() + 1) ;
			o->setGeometry (rect)	;
		)

		getLayout()->setChanged() ;
	}
#endif
}

/*  KBObject								*/
/*  deleteDynamicRow							*/
/*		: Delete row from dynamic layout			*/
/*  (returns)	: void		:					*/

void	KBObject::deleteDynamicRow ()
{
#if	! __KB_RUNTIME
	if (isDynamic())
	{
		uint	numCols	= m_geom.numCols(true)	;
		int	from	= newCtrlRect().y()	;
		QRect	overRow	= QRect (0, from, numCols, 1) ;
		bool	overLap = false ;

		CITER
		(	Object,
			obj,
			if (obj->overlaps (overRow) != 0)
				if (obj->geometry().height() < 2)
				{
					overLap	= true ;
					break	;
				}
		)

		if (overLap)
		{
			TKMessageBox::sorry
			(	0,
				QString(TR("Row contains objects: please remove first")),
				TR("Cannot delete row")
			)	;
			return	;
		}

		CITER
		(	Object,
			o,

			QRect	rect	= o->geometry() ;
			if	(rect.y() > from)
			{
				rect.moveBy    (0, -1)	;
			}
			else if (rect.y() + rect.height() <= from)
			{
			}
			else
			{
				rect.setHeight (rect.height() - 1) ;
			}

			o->setGeometry (rect )	;
		)

		m_geom.removeRow (from) ;
		getContainer()->updateDynamic() ;
		getLayout   ()->setChanged () ;
	}
#endif
}

/*  KBObject								*/
/*  insertDynamicColumn							*/
/*		: Insert column into dynamic layout			*/
/*  (returns)	: void		:					*/

void	KBObject::insertDynamicColumn ()
{
#if	! __KB_RUNTIME
	if (isDynamic())
	{
		int	from	= newCtrlRect().x() ;

		m_geom.insertCol (from) ;
		getContainer()->updateDynamic() ;

		CITER
		(	Object,
			o,

			QRect	rect	= o->geometry() ;
			if	(rect.x() 		 >= from) rect.moveBy   (1, 0)	;
			else if (rect.x() + rect.width() >  from) rect.setWidth (rect.width() + 1) ;
			o->setGeometry (rect)	;
		)

		getLayout()->setChanged() ;
	}
#endif
}

/*  KBObject								*/
/*  deleteDynamicColumn							*/
/*		: Delete column from dynamic layout			*/
/*  (returns)	: void		:					*/

void	KBObject::deleteDynamicColumn ()
{
#if	! __KB_RUNTIME
	if (isDynamic())
	{
		uint	numRows	= m_geom.numRows(true)	;
		int	from	= newCtrlRect().x()	;
		QRect	overCol	= QRect (from, 0, 1, numRows) ;
		bool	overLap = false ;

		CITER
		(	Object,
			obj,
			if (obj->overlaps (overCol) != 0)
				if (obj->geometry().width() < 2)
				{
					overLap	= true ;
					break	;
				}
		)

		if (overLap)
		{
			TKMessageBox::sorry
			(	0,
				QString(TR("Column contains objects: please remove first")),
				TR("Cannot delete column")
			)	;
			return	;
		}

		CITER
		(	Object,
			o,

			QRect	rect	= o->geometry() ;

			if	(rect.x() > from)
			{
				rect.moveBy    (-1, 0)	;
			}
			else if (rect.x() + rect.width() <= from)
			{
			}
			else
			{
				rect.setWidth  (rect.width() - 1) ;
			}

			o->setGeometry (rect)	;
		)

		m_geom.removeCol (from) ;
		getContainer()->updateDynamic() ;
		getLayout   ()->setChanged () ;
	}
#endif
}

/*  KBObject								*/
/*  gridSetup	: Run the grid setup dialog				*/
/*  (returns)	: void		:					*/

void	KBObject::gridSetup ()
{
#if	! __KB_RUNTIME
	/* If the user cancels the dialog then call the container's	*/	
	/* "setRowColSetup" method to restore the original settings.	*/
	if (!KBRowColDialog (m_geom, this, newCtrlRect().y(), newCtrlRect().x()).exec())
	{
		getContainer()->setRowColSetup
		(	m_geom.rowSetup(),
			m_geom.colSetup()
		)	;
		return	;
	}

	setChanged() ;
#endif
}

#if	! __KB_RUNTIME

/*  KBBlock								*/
/*  whatsThisExtra							*/
/*		: Extra stuff for pseudo-what's this popup		*/
/*  info	: QValueList<KBWhatsThisPair> &	: List of items		*/
/*  (returns)	: void				:			*/

void	KBObject::whatsThisExtra
	(	QValueList<KBWhatsThisPair>	&
	)
{
}

#endif

/*  KBObject								*/
/*  whatsThis	: Display pseudo-what;s this popup			*/
/*  (returns)	: void		:					*/

void	KBObject::whatsThis ()
{
#if	! __KB_RUNTIME
	QPoint	showAt	= QCursor::pos() ;
	if (m_control != 0)
		showAt	= m_control->getWidget()->mapToGlobal(QPoint(0,0)) ;

	QSize	hint	= sizeHint    () ;
	QRect	geom	= realGeometry() ;

	QValueList<KBWhatsThisPair> info ;
	info.append	(KBWhatsThisPair(TR("Element" ), getElement().mid(2))) ;
	info.append	(KBWhatsThisPair(TR("Name"    ), getName(),     true)) ;
	whatsThisExtra	(info)	;
	info.append	(KBWhatsThisPair(TR("Size Hint"), _TEXT(hint))) ;
	info.append	(KBWhatsThisPair(TR("Geometry" ), _TEXT(geom))) ;

	if (parentIsDynamic ())
	{
		QSize	minSize	= minimumSize   () ;
//		uint	minW	= minSize.width () ;
//		uint	minH	= minSize.height() ;

		info.append
		(	KBWhatsThisPair
			(	TR("Cell Range"),
				QString("(%1,%2) (%3,%4)").
					arg(m_geom.x()).
					arg(m_geom.y()).
					arg(m_geom.w()).
					arg(m_geom.h())
		)	)	;

		info.append (KBWhatsThisPair(TR("Min. Size"), _TEXT(minSize))) ;
	}

	QString	notes	= getAttrVal("notes") ;
	if (!notes.isEmpty())
		info.append (KBWhatsThisPair(TR("Notes"), notes)) ;

	QString	text	;
	text	+= "<font size='-1'><table>" ;

	for
	(	QValueList<KBWhatsThisPair>::iterator it = info.begin() ;
		it != info.end() ;
		++it
	)
		if (!(*it).m_info.isEmpty())
			text += QString	("<tr><td><nobr><b>%1</b></nobr></td><td width='32'/><td><font color='%2'><nobr>%3</nobr></font></td><td width='32'/></tr>")
					.arg((*it).m_legend)
					.arg((*it).m_bold  ? "red" : "black")
					.arg((*it).m_info  )
				;
	text	+= "</table></font>" ;

	QWhatsThis::setFont (QToolTip::font()) ;
	QWhatsThis::display (text, showAt) ;
#endif
}

/*  KBObject								*/
/*  displayGone	: Display has been destroyed				*/
/*  (returns)	: void		:					*/

void	KBObject::displayGone ()
{
	/* Dummy here, implemented in KBBlock and KBFramer		*/
}

bool	KBObject::showHidden ()
{
	return	m_hidden.getBoolValue () ;
}

bool	KBObject::showDisabled ()
{
	return	m_disabled.getBoolValue () ;
}
