/***************************************************************************
    file	         : kb_object.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_OBJECT_H
#define	_KB_OBJECT_H

#include	"libkbase_exports.h"

#include	<qpoint.h>
#include	<qsize.h>
#include	<qrect.h>
#include	<qvariant.h>

#include	"kb_node.h"
#include	"kb_nodereg.h"
#include	"kb_event.h"
#include	"kb_limit.h"
#include	"kb_attrgeom.h"
#include	"kb_attrskinelem.h"


#if	! __KB_RUNTIME

class	KBQuickText	;
class	KBSizer		;
class	KBPopupMenu	;

class	KBWhatsThisPair
{
public	:

	QString		m_legend ;
	QString		m_info	 ;
	bool		m_bold	 ;

	KBWhatsThisPair ()	 ;
	KBWhatsThisPair
	(	const QString	&,
		const QString	&,
		bool toNone	= false,
		bool bold	= false
	)	;
}	;

#endif

/*  KBObject								*/
/*  -------								*/
/*  The KBObject class is used for elements in the XML parse tree	*/
/*  which correspond to displayed objects. It contains stuff common to	*/
/*  attributes and object properties, plus methods to support sizing	*/
/*  and moving, and right-button operations.				*/

class	LIBKBASE_API	KBObject : public KBNode
{
	Q_OBJECT

	KBBlock		*m_block	;	/* Owning block if any	*/
	KBDisplay	*m_display	;	/* Associated display	*/
	KBControl	*m_control	;	/* Control for sizer	*/

	KBScriptObject	**m_scriptObj	;

protected :

#if	! __KB_RUNTIME
	KBSizer		*m_sizer	;	/* Design mode sizer	*/
	QRect		m_newRect	;	/* For new controls	*/
#endif


	KBAttrGeom	m_geom		;	/* Geometry settings	*/
	KBAttrBool	m_disabled	;	/* Start disabled	*/
	KBAttrBool	m_hidden	;	/* Start hidden		*/
	KBAttrSkinElem	m_skinElem	;	/* Skin element if any	*/
	KBAttrStr	*m_attrSlots	;	/* Dummy for slots	*/
	KBAttrStr	*m_attrConfig	;	/* Dummy for config	*/
	KBAttrStr	*m_attrTests	;	/* Dummy for tests	*/

	QPalette	*m_curPal	;	/* Current palette	*/
	QFont		*m_curFont	;	/* Current font		*/

#if	! __KB_RUNTIME
	Qt::ButtonState	m_bState	;	/* Popup key state	*/
	KBQuickText	*m_quickText	;	/* Quick text editor	*/
#endif


#if	! __KB_RUNTIME
	virtual	KBPopupMenu	*makeNewPopup		(KBPopupMenu *, QRect) ;
	virtual void		makeRecordPopup		(KBPopupMenu *, uint, bool) ;
	KBPopupMenu		*makeTestsPopup		(KBPopupMenu *, uint ) ;

	virtual	void		whatsThisExtra		(QValueList<KBWhatsThisPair> &) ;

	void			setCtrlRect		(QRect)	;
	QRect			newCtrlRect		()	;
	QRect			autoCtrlRect		(bool )	;

	void			createSizer		()	;
	void			setSizer		(KBSizer   *) ;

	KBNode			*insertObjectDynamic	(KBNode *,	     KBDisplay *, QRect ) ;
	KBNode			*insertObjectStatic	(KBNode *,	     KBDisplay *, QPoint) ;
	QPtrList<KBNode>	insertObjectsStatic	(QPtrList<KBNode> &, KBDisplay *, QPoint) ;

	QPtrList<KBNode>	insertObjects		(KBDisplay *) ;
	QPtrList<KBNode>	insertHere		(QPtrList<KBNode> &, KBDisplay *, QRect ) ;
	void			insertComponent		(KBDisplay *, QRect, bool) ;

	void			newFormBlock		(KBDisplay *, int) ;
	void			newReportBlock		(KBDisplay *, int) ;
	void			newContainer		(KBDisplay *) ;
#endif

	void			setChanged		() ;

	virtual void		setPalette		() ;
	virtual void		setFont			() ;

protected slots :

	virtual void		recordVerifyState	()	;

public slots :

	virtual KBNode		*newNode		(int)	;
	virtual	void		newTableBlock		()	;
	virtual	void		newSQLBlock		()	;
	virtual	void		newQueryBlock		()	;
	virtual void		newNullBlock		()	;
	virtual	void		newContainer		()	;
	virtual void		pasteComponent		()	;
	virtual void		linkComponent		()	;

	virtual	bool		propertyDlg		() 	;
	virtual	void		cutObj			()	;
	virtual	void		copyObj			()	;
	virtual	void		deleteObj		()	;
	virtual	void		saveAsComponent		()	;

	virtual	void		pasteObjects		() 	;

	virtual	void		insertDynamicRow	()	;
	virtual	void		deleteDynamicRow	()	;
	virtual	void		insertDynamicColumn	()	;
	virtual	void		deleteDynamicColumn	()	;
	virtual	void		gridSetup		()	;

	void			slotExecTest		(int)	;

	virtual	void		setPropDirect		(int)	;
	virtual	void		whatsThis		()	;


public	:

	enum	FloatMode
	{
		FMFixed,
		FMFloat,
		FMStretch
	}	;

	virtual	bool		event		(QEvent *)		;

	virtual	KBObject	*isObject	() { return this  ; }

	virtual	bool		propertyDlg	(cchar *)		;
	virtual	void		buildDisplay	(KBDisplay *)		;
	virtual	void		showAs		(KB::ShowAs )		;
	virtual	void		ctrlGone	(KBControl *)		;

	virtual	void		setupProperties	()			;

	inline	KBDisplay	*getDisplay	() { return m_display ; }
	virtual	KBDisplay	*getContainer	() { return 0	      ; }

#if	! __KB_RUNTIME
	bool			addNewObject	(QRect)	;
	void			selectOverlap	(QRect)	;
	bool			checkOverlap	(QRect)	;
	KBSizer			*overlaps  	(QRect)	;
	KBObject		*objectInCell	(QRect) ;
#endif

	virtual	QSize		minimumSize	() ;
	QSize			minimumGeometry	() ;

	KBLimit			getMoveLimit	() ;
	virtual	uint		calcNumRows	(uint, uint, int, int) ;
	virtual	uint		calcNumRows	(int, int, uint) ;

	virtual	bool		keyStroke	(QKeyEvent    *) ;
	virtual	bool		moveFocusOK	(uint) ;
	virtual	void		focusInEvent	(uint, QFocusEvent::Reason ) ;
	virtual	KBPopupMenu	*designPopup	(KBPopupMenu *, QRect) ;
	virtual	bool		contextMenu	(QMouseEvent *, uint ) ;

	virtual	void		displayGone	() ;

	void			setFocus	() ;

	/* The "geometry" methods return the logical geometry of the	*/
	/* object in its parent display. This is in pixels for "static"	*/
	/* layouts, and in cells for "dynamic" layouts.			*/
	virtual	QRect		geometry	() ;
	virtual	QRect		geometry	(const QPoint &) ;

	/* The "realGeometry" method returns the on-screen geometry of	*/
	/* the objects display widget, which may be a control or a	*/
	/* display.							*/
	QRect			realGeometry	() ;

	/* The "setGeometry" method is called to change the objects	*/
	/* geometry attributes. As a side effect, the on-screen		*/
	/* geometry may change.						*/
	virtual void		setGeometry	(const QRect  &) ;
	virtual void		setCtrlGeometry	(const QRect  &) ;
	virtual	bool		ctrlGeometry	(QRect  &) ;

	/* "displayResized" is called when a container is resized, and	*/
	/* is passed the new size. The method does nothing at the	*/
	/* object level.						*/
	virtual	void		displayResized	(const QSize  &) ;
	virtual	void		recalcNumRows	(const QSize  &) ;

	bool			showDisabled	() ;
	bool			showHidden	() ;
	virtual	QSize		sizeHint	() ;

	virtual	QPalette	*getPalette	(bool) ;
	virtual	QFont		*getFont	(bool) ;
	void			setControl	(KBControl * ) ;
	virtual bool		write		(KBWriter *, QPoint, bool, int &, bool = false) ;

	virtual	void		setEnabled	(bool)	;
	virtual	void		setVisible	(bool)	;
	virtual bool		isEnabled	()	;
	virtual bool		isVisible	()	;

	virtual	void		setTabOrder	(int )	;
	virtual	int		getTabOrder	()     	;

#if	! __KB_RUNTIME
	inline	KBSizer		*getSizer	()	{ return m_sizer   ; }
#endif

	KBFormBlock		*getFormBlock	()	;
	KBReportBlock		*getReportBlock	()	;
	KBNavigator		*getNavigator	()	;

	QWidget		  	*getDisplayWidget ()	;
	QWidget			*getTopWidget	  ()	;

	void			setScriptObject	(uint, KBScriptObject *) ;
	KBScriptObject		*scriptObject	(uint )	;

	bool			parentIsDynamic	() ;
	KBObject		*parentObject	() ;

	virtual	void		setCtrlFGColor	(const QString &) ;
	virtual	void		setCtrlBGColor	(const QString &) ;
	virtual	void		setCtrlFont	(const QString &) ;

	inline	const KBAttrGeom &attrGeom	()
	{
		return	m_geom	;
	}
	inline	bool	isDynamic ()
	{
		return	m_geom.manage() == KBAttrGeom::MgmtDynamic ;
	}

	static	void		minPosition	(const QPtrList<KBNode> &, int &, int &) ;

	/* These methods are used for access-by-attribute from scripts.	*/
	virtual	void		enumKBProperty	(QStringList	  	&) ;
	virtual	bool		hasKBProperty	(cchar *)		   ;
	virtual	bool		getKBProperty	(cchar *,	KBValue &) ;
	virtual	bool		setKBProperty	(cchar *, const KBValue &) ;

	virtual	QVariant	property	(cchar *)		;
	virtual	bool		setProperty	(cchar *, const QVariant &) ;

	bool			eventHook	(KBEvent &, uint, KBValue *, bool &, bool = true) ;

#if	! __KB_RUNTIME
	friend	class		KBSizer		;
#endif

	friend	class		KBMacroTest	;
	friend	class		KBTestAccessor	;

	KBObject		(KBNode   *, cchar *, const QDict<QString> &) ;
	KBObject		(KBObject *, cchar *, const QRect          &) ;
	KBObject		(KBNode   *, KBObject *) ;
	virtual	~KBObject	()	;
}	;


#endif	// _KB_OBJECT_H
