;/***************************************************************************
    file	         : kb_options.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
			   (C)	2005
				by Abel Deuring
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdlib.h>

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_appptr.h"
#include	"kb_callback.h"
#include	"kb_options.h"

#include	"tk_config.h"
#include	"tk_action.h"



#define	DEF_GRIDX		10
#define	DEF_GRIDY		10
#define	DEF_DEFDX		0
#define	DEF_DEFDY		20
#define	DEF_MARGIN		25
#define	DEF_DPI			0
#define	DEF_DESIGNINCHES	false

#define	DEF_MAXQUERIES		64
#define	DEF_MAXEVENTS		256
#define	DEF_MAXARGS		16
#define	DEF_MAXARGLEN		80

#define	DEF_OPENLAST		false
#define	DEF_SPACE		20
#define	DEF_FORMW		500
#define	DEF_FORMH		400
#define	DEF_TABLESSMODAL	false
#define	DEF_FORMSMODAL		false

#define	DEF_REPORTSMODAL	false
#define	DEF_QUERIESMODAL	false
#define	DEF_SINGLEDBONLY	false

#define	DEF_USETOOLBOX		true
#define	DEF_USEWIZARDS		true
#define	DEF_SUSPENDTOOLBOX	true

#define	DEF_MINCELLWIDTH	20
#define	DEF_MINCELLHEIGHT	20



/*  There is but a single static instance of the KBOptions object, used	*/
/*  to collect all options into one place.				*/

static	KBOptions	kbOptions ;


/*  ------------------------------------------------------------------  */

/*  KBOptions								*/
/*  loadOptions	: Load options from configuration			*/
/*  (returns)	: void		:					*/

void	KBOptions::loadOptions ()
{
	TKConfig *config = TKConfig::getConfig () ;
	config->setGroup(KB_OPTIONS_GROUP) ;

	kbOptions.m_gridX	= config->readUnsignedNumEntry	("gridX",		DEF_GRIDX 	) ;
	kbOptions.m_gridY	= config->readUnsignedNumEntry	("gridY",		DEF_GRIDY 	) ;
	kbOptions.m_formW	= config->readUnsignedNumEntry	("formW",		DEF_FORMW 	) ;
	kbOptions.m_formH	= config->readUnsignedNumEntry	("formH",		DEF_FORMH 	) ;
	kbOptions.m_defDX	= config->readNumEntry		("defDX",		DEF_DEFDX 	) ;
	kbOptions.m_defDY	= config->readNumEntry		("defDY",		DEF_DEFDY 	) ;
	kbOptions.m_space	= config->readUnsignedNumEntry	("space",		DEF_SPACE 	) ;
	kbOptions.m_marginL	= config->readUnsignedNumEntry	("marginL",		DEF_MARGIN	) ;
	kbOptions.m_marginR	= config->readUnsignedNumEntry	("marginR",		DEF_MARGIN	) ;
	kbOptions.m_marginT	= config->readUnsignedNumEntry	("marginT",		DEF_MARGIN	) ;
	kbOptions.m_marginB	= config->readUnsignedNumEntry	("marginB",		DEF_MARGIN	) ;
	kbOptions.m_dpi		= config->readUnsignedNumEntry	("dpi",			DEF_DPI		) ;
	kbOptions.m_useMdi	= config->readBoolEntry		("usemdi",		false     	) ;
	kbOptions.m_singleDBOnly= config->readBoolEntry		("singledbonly",	DEF_SINGLEDBONLY) ;
	kbOptions.m_openLast	= config->readBoolEntry		("openLast",		DEF_OPENLAST	) ;
	kbOptions.m_designInches= config->readBoolEntry		("designInches",	DEF_DESIGNINCHES) ;
	kbOptions.m_tablesModal	= config->readBoolEntry		("tablesModal", 	DEF_TABLESSMODAL) ;
	kbOptions.m_formsModal	= config->readBoolEntry		("formsModal",  	DEF_FORMSMODAL  ) ;
	kbOptions.m_reportsModal= config->readBoolEntry		("reportsModal",	DEF_REPORTSMODAL) ;
	kbOptions.m_queriesModal= config->readBoolEntry		("queriesModal",	DEF_QUERIESMODAL) ;
	kbOptions.m_logMaxQueries
				= config->readUnsignedNumEntry	("logMaxQueries",	DEF_MAXQUERIES	) ;
	kbOptions.m_logMaxEvents= config->readUnsignedNumEntry	("logMaxEvents",	DEF_MAXEVENTS	) ;
	kbOptions.m_logMaxArgs	= config->readUnsignedNumEntry	("logMaxArgs",		DEF_MAXARGS	) ;
	kbOptions.m_logMaxArgLen= config->readUnsignedNumEntry	("logMaxArgLen",	DEF_MAXARGLEN	) ;
	kbOptions.m_useToolbox	= config->readNumEntry 		("useToolbox",		DEF_USETOOLBOX	) ;
	kbOptions.m_useWizards	= config->readNumEntry 		("useWizards",		DEF_USEWIZARDS	) ;
	kbOptions.m_suspendToolbox
				= config->readNumEntry 		("suspendToolbox",	DEF_SUSPENDTOOLBOX);
	kbOptions.m_scriptFont	= config->readEntry		("scriptFont"	) ;
	kbOptions.m_keymap	= config->readEntry		("keymap"	) ;
	kbOptions.m_autoIntelli	= config->readBoolEntry		("autointelli"	) ;

	kbOptions.m_scriptType	= (ScriptType)config->readNumEntry
				  (	"scriptType",
					ScriptTypeAlwaysScript
				  )	;

	kbOptions.m_macroDebug	= (MacroDebug  )config->readNumEntry ("macroDebug") ;
	kbOptions.m_verInsert	= (VerifyOption)config->readNumEntry ("verInsert" ) ;
	kbOptions.m_verUpdate	= (VerifyOption)config->readNumEntry ("verUpdate" ) ;
	kbOptions.m_verDelete	= (VerifyOption)config->readNumEntry ("verDelete" ) ;
	kbOptions.m_verMultiple	= config->readNumEntry	("verMulti"  ) ;

	kbOptions.m_minCellWidth= config->readNumEntry 	("minCellWidth",  DEF_MINCELLWIDTH	) ;
	kbOptions.m_minCellHeight
				= config->readNumEntry 	("minCellHeight", DEF_MINCELLHEIGHT	) ;
	kbOptions.m_noButtonImages
				= config->readNumEntry	("noButtonImages",0			) ;
	kbOptions.m_cacheSize	= config->readNumEntry	("cacheSize",	  0			) ;
	kbOptions.m_cacheOption	= config->readNumEntry	("cacheOption",	  0			) ;
	kbOptions.m_cacheInTests
				= config->readNumEntry	("cacheInTests",  false			) ;
	kbOptions.m_style	= config->readEntry	("style") ;

	kbOptions.m_webDisabled	= false	;

 	config->setGroup ("Debug Options/py") ;
 	kbOptions.m_pythonEncoding
				= config->readEntry	("pyencoding") ;


	KBLocation::setCacheSize
	(	kbOptions.m_cacheSize,
		(KBLocation::CacheOption)kbOptions.m_cacheOption
	)	;
}

void	KBOptions::setOption
	(	const QString	&name,
		const QString	&value
	)
{
	if (name == "pyencoding")
	{	kbOptions.m_pythonEncoding = value ;
		return	;
	}

	fprintf
	(	stderr,
		"KBOptions::setOption: %s: not handled\n",
		name.latin1()
	)	;
}

/*  KBOptions								*/
/*  snappingOn	: Report whether snap-to-grid is enabled		*/
/*  (returns)	: bool		: True if enabled			*/

bool	KBOptions::snappingOn ()
{
	static	TKToggleAction *snap = 0 ;

	if (snap == 0)
		snap = (TKToggleAction *)
				KBAppPtr::getCallback()->globalAction("KB_snapEnable") ;

	return	snap == 0 ? false : snap->isChecked () ;
}

/*  KBOptions								*/
/*  getDlgMargin: Get dialog layout margin				*/
/*  (returns)	: int		: Margin				*/

int	KBOptions::getDlgMargin ()
{
	return	4	;
}

/*  KBOptions								*/
/*  getDlgSpacing: Get dialog layout spacing				*/
/*  (returns)	 : int		: Spacing				*/

int	KBOptions::getDlgSpacing ()
{
	return	4	;
}

bool	KBOptions::getNoButtonImages ()
{
	if (kbOptions.m_noButtonImages != 0)
		return	kbOptions.m_noButtonImages > 1 ;

#ifdef	Q_OS_MACX
	return	true	;
#else
	return	false	;
#endif
}



KBOptions &KBOptions::self()
{
	return	kbOptions ;
}

/*  The remainder of the code retrieves values. This is all pretty	*/
/*  samey, so define a macro and use that to generate the code.		*/

#define	GETVALUE(type,func,value)		\
	type	KBOptions::get##func ()		\
	{					\
		return	kbOptions.value;	\
	}

GETVALUE(uint,				GridXStep,	m_gridX  	)
GETVALUE(uint,				GridYStep,	m_gridY  	)
GETVALUE(uint,				FormWidth,	m_formW  	)
GETVALUE(uint,				FormHeight,	m_formH  	)
GETVALUE(int,				DefaultDX,	m_defDX  	)
GETVALUE(int,				DefaultDY,	m_defDY  	)
GETVALUE(uint,				CtrlSpace,	m_space  	)

GETVALUE(bool,				UseMDI,		m_useMdi 		)
GETVALUE(bool,				SingleDBOnly,	m_singleDBOnly	)

GETVALUE(uint,				LeftMargin,	m_marginL	)
GETVALUE(uint,				RightMargin,	m_marginR	)
GETVALUE(uint,				TopMargin,	m_marginT	)
GETVALUE(uint,				BottomMargin,	m_marginB	)
GETVALUE(uint,				DPI,		m_dpi		)

GETVALUE(bool,				OpenLast,	m_openLast	)
GETVALUE(bool,				DesignInches,	m_designInches	)
GETVALUE(bool,				TablesModal,	m_tablesModal	)
GETVALUE(bool,				FormsModal,	m_formsModal	)
GETVALUE(bool,				ReportsModal,	m_reportsModal	)
GETVALUE(bool,				QueriesModal,	m_queriesModal	)

GETVALUE(uint,				LogMaxQueries,	m_logMaxQueries	)
GETVALUE(uint,				LogMaxEvents,	m_logMaxEvents	)
GETVALUE(uint,				LogMaxArgs,	m_logMaxArgs	)
GETVALUE(uint,				LogMaxArgLen,	m_logMaxArgLen	)

GETVALUE(KBOptions::ScriptType,		ScriptType,	m_scriptType	)
GETVALUE(KBOptions::MacroDebug,		MacroDebug,	m_macroDebug	)

GETVALUE(KBOptions::VerifyOption,	VerInsert,	m_verInsert	)
GETVALUE(KBOptions::VerifyOption,	VerUpdate,	m_verUpdate	)
GETVALUE(KBOptions::VerifyOption,	VerDelete,	m_verDelete	)
GETVALUE(bool,				VerMultiple,	m_verMultiple	)

GETVALUE(QString,			ScriptFont,	m_scriptFont	)
GETVALUE(bool,				UseToolbox,	m_useToolbox	)
GETVALUE(bool,				UseWizards,	m_useWizards	)
GETVALUE(bool,				SuspendToolbox,	m_suspendToolbox)
GETVALUE(bool,				AutoIntelli,	m_autoIntelli	)

GETVALUE(uint,				MinCellWidth,	m_minCellWidth	)
GETVALUE(uint,				MinCellHeight,	m_minCellHeight	)

GETVALUE(QString,			Keymap,		m_keymap	)
GETVALUE(bool,				WebDisabled,	m_webDisabled	)
GETVALUE(uint,				CacheSize,	m_cacheSize	)
GETVALUE(bool,				CacheInTests,	m_cacheInTests	)
GETVALUE(uint,				CacheOption,	m_cacheOption	)
GETVALUE(QString,			Style,		m_style		)


/*  KBOptions								*/
/*  getPythonEncoding							*/
/*		: Get python encoding to be used			*/
/*  (returns)	: QString	: Encoding				*/

QString	KBOptions::getPythonEncoding ()
{
	return	kbOptions.m_pythonEncoding ;
}
