/***************************************************************************
    file	         : kb_options.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
			   (C)	2005
				by Abel Deuring
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_OPTIONS_H
#define	_KB_OPTIONS_H



#include	"libkbase_exports.h"
#include	"kb_dom.h"

/*  KBOptions								*/
/*  ---------								*/
/*									*/
/*  Global options class. This is effectively a namespace, and is done	*/
/*  like this so that we can split up the options themselves from the	*/
/*  configuration dialogs, which are mostly not used in the runtime	*/
/*  only version.							*/
/*									*/
class	LIBKBASE_API	KBOptions
{
public	:

	enum	ScriptType
	{
		ScriptTypeAlwaysScript	= 0,
		ScriptTypeUserChoice	= 1,
		ScriptTypeAlwaysMacro	= 2
	}	;

	enum	MacroDebug
	{
		MacroDebugOnDemand	= 0,
		MacroDebugAlwaysOff	= 1,
		MacroDebugAlwaysOn	= 2
	}	;

	enum	VerifyOption
	{
		VerifyNever		= 0,
		VerifyAlways		= 1,
		VerifyMulti		= 2
	}	;

private	:

	uint		m_gridX	 	;
	uint 		m_gridY	 	;

	uint		m_formW	 	;
	uint		m_formH	 	;
	int		m_defDX	 	;
	int		m_defDY	 	;
	uint		m_space 	 ;

	uint		m_marginL  	;
	uint		m_marginR  	;
	uint		m_marginB  	;
	uint		m_marginT  	;
	bool		m_designInches	;
	uint		m_dpi		;

	bool		m_openLast 	;
	bool		m_useMdi   	;
	bool		m_singleDBOnly	;

	bool		m_tablesModal	;
	bool		m_formsModal	;
	bool		m_reportsModal	;
	bool		m_queriesModal	;

	uint		m_logMaxQueries	;
	uint		m_logMaxEvents	;
	uint		m_logMaxArgs	;
	uint		m_logMaxArgLen	;

	uint		m_minCellWidth	;
	uint		m_minCellHeight	;

	ScriptType	m_scriptType	;
	MacroDebug	m_macroDebug	;

	VerifyOption	m_verInsert	;
	VerifyOption	m_verUpdate	;
	VerifyOption	m_verDelete	;
	bool		m_verMultiple	;

	bool		m_useToolbox	;
	bool		m_useWizards	;
	bool		m_suspendToolbox;

	QString		m_scriptFont	;
	QString		m_keymap	;
	QString		m_pythonEncoding;
	bool		m_autoIntelli	;

	int		m_noButtonImages;
	bool		m_webDisabled	;
	uint		m_cacheSize	;
	int		m_cacheOption	;
	bool		m_cacheInTests	;

	QString		m_style		;

public	:

	static	void		loadOptions		() ;

	static	uint		getGridXStep 		() ;
	static	uint		getGridYStep		() ;
	static	uint		getFormWidth		() ;
	static	uint		getFormHeight 		() ;
	static	int		getDefaultDX 		() ;
	static	int		getDefaultDY 		() ;
	static	uint		getCtrlSpace		() ;

	static	bool		getUseMDI		() ;
	static	bool		getSingleDBOnly		() ;

	static	bool		snappingOn		() ;
	static	uint		getLeftMargin		() ;
	static	uint		getRightMargin		() ;
	static	uint		getTopMargin		() ;
	static	uint		getBottomMargin		() ;
	static	uint		getDPI			() ;

	static	int		getDlgMargin		() ;
	static	int		getDlgSpacing		() ;

	static	bool		getOpenLast		() ;
	static	bool		getDesignInches		() ;

	static	bool		getTablesModal		() ;
	static	bool		getFormsModal		() ;
	static	bool		getReportsModal		() ;
	static	bool		getQueriesModal		() ;

	static	uint		getLogMaxQueries	() ;
	static	uint		getLogMaxEvents		() ;
	static	uint		getLogMaxArgs		() ;
	static	uint		getLogMaxArgLen		() ;

	static	QString		getScriptFont		() ;
	static	bool		getAutoIntelli		() ;

	static	ScriptType	getScriptType		() ;
	static	MacroDebug	getMacroDebug		() ;

	static	VerifyOption	getVerInsert		() ;
	static	VerifyOption	getVerUpdate		() ;
	static	VerifyOption	getVerDelete		() ;
	static	bool		getVerMultiple		() ;

	static	bool		getUseToolbox		() ;
	static	bool		getUseWizards		() ;

	static	uint		getMinCellWidth		() ;
	static	uint		getMinCellHeight	() ;

	static	bool		getSuspendToolbox	() ;

	static	QString		getKeymap		() ;
	static	bool		getWebDisabled		() ;
	static	uint		getCacheSize		() ;
	static	uint		getCacheOption		() ;
	static	bool		getCacheInTests		() ;
	static	QString		getPythonEncoding	() ;
	static	QString		getStyle		() ;

	static	bool		getNoButtonImages	() ;

	static	KBOptions	&self			() ;
	static	void		loadFromXML		(QDomElement &) ;
	static	void		saveToXML		(QDomElement &) ;

	static	void		setOption		(const QString &, const QString &) ;

	friend	class		KBInterfaceOpts	;
	friend	class		KBModalOpts	;
	friend	class		KBDesignOpts	;
	friend	class		KBLayoutOpts	;
	friend	class		KBReportOpts	;
	friend	class		KBPythonOpts	;
	friend	class		KBScriptOpts	;
	friend	class		KBLoggingOpts	;
	friend	class		KBVerifyOpts	;
	friend	class		KBCacheOpts	;

	friend	class		KBOptionsDlg	;
}	;

#define	KB_OPTIONS_GROUP	"Setup Options"
#endif	// _KB_OPTIONS_H
