/***************************************************************************
    file	         : kb_optionsdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
			   (C)	2005
				by Abel Deuring
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	<qfile.h>
#include	<qlabel.h>
#include	<qspinbox.h>
#include	<qradiobutton.h>
#include	<qvbuttongroup.h>
#include	<qtextview.h>
#include	<qstylefactory.h>


#include	"kb_classes.h"
#include	"kb_options.h"
#include	"kb_optionsdlg.h"
#include	"kb_font.h"
#include	"kb_locator.h"
#include	"kb_wizard.h"
#include	"kb_sidepanel.h"
#include	"kb_notifier.h"

#include	"rk_lineedit.h"
#include	"rk_combobox.h"
#include	"rk_checkbox.h"
#include	"rk_pushbutton.h"
#include	"rk_tabwidget.h"
#include	"tk_messagebox.h"
#include	"tk_fontdialog.h"
#include	"tk_filedialog.h"

#include	"kb_combowidget.h"

#if		__KB_AUTOMOC
#include	"kb_optionsdlg.moc"
#else
#include	"kb_optionsdlg.h"
#endif

/*  ------------------------------------------------------------------  */

static  cchar   *codecNames[] =
{
	"UTF8",
	"BIG5",
	"CP 874",
	"CP 1250",
	"CP 1251",
	"CP 1252",
	"CP 1253",
	"CP 1254",
	"CP 1255",
	"CP 1256",
	"CP 1257",
	"CP 1258",
	"EUCJP",
	"EUCKR",
	"GBK",
	"JIS7",
	"KOI8-R",
	"KOI8-U",
	"SHIFT-JIS",
	"ISO8859-1",
	"ISO8859-2",
	"ISO8859-3",
	"ISO8859-4",
	"ISO8859-5",
	"ISO8859-6",
	"ISO8859-7",
	"ISO8859-8",
	"ISO8859-8-I",
	"ISO8859-9",
	"ISO8859-10",
	"ISO8859-13",
	"ISO8859-14",
	"ISO8859-15",
	"TIS-620",
	"TSCII",
	0
}       ;


cchar	LIBKBASE_API	**getCodecNames ()
{
	return	codecNames	;
}

/*  ------------------------------------------------------------------  */

/*  KBVerifyOpts							*/
/*  KBVerifyOpts: Constructor for verification page			*/
/*  parent	: KBComboWidget * : Parent				*/
/*  options	: KBOptions    &  : Options object			*/
/*  (returns)	: void		  :					*/

KBVerifyOpts::KBVerifyOpts
	(	KBComboWidget	*parent,
		KBOptions	&options
	)
	:
	RKGridBox	(2, parent, "verify"),
	m_options	(options)
{
	parent->addTab	(this, TR("Verification")) ;

	new QLabel (TR("Verify inserts"		 ), this) ;
	m_cbInsert = new RKComboBox (this) ;
	new QLabel (TR("Verify updates"		 ), this) ;
	m_cbUpdate = new RKComboBox (this) ;
	new QLabel (TR("Verify deletes"		 ), this) ;
	m_cbDelete = new RKComboBox (this) ;
	new QLabel (TR("Verify multi-record sync"), this) ;
	m_cbMulti  = new RKComboBox (this) ;

	addFillerRow () ;

	m_cbInsert->insertItem ("Never"   ) ;
	m_cbInsert->insertItem ("Always"  ) ;
	m_cbInsert->insertItem ("Multiple") ;

	m_cbUpdate->insertItem ("Never"   ) ;
	m_cbUpdate->insertItem ("Always"  ) ;
	m_cbUpdate->insertItem ("Multiple") ;

	m_cbDelete->insertItem ("Never"   ) ;
	m_cbDelete->insertItem ("Always"  ) ;
	m_cbDelete->insertItem ("Multiple") ;

	m_cbMulti ->insertItem ("No")	 ; ;
	m_cbMulti ->insertItem ("Yes")	 ; ;

	m_cbInsert->setCurrentItem (m_options.m_verInsert  ) ;
	m_cbUpdate->setCurrentItem (m_options.m_verUpdate  ) ;
	m_cbDelete->setCurrentItem (m_options.m_verDelete  ) ;
	m_cbMulti ->setCurrentItem (m_options.m_verMultiple) ;
}

/*  KBVerifyOpts							*/
/*  save	: Save verification settings				*/
/*  config	: TKConfig *	: Configuration				*/
/*  (returns)	: void		:					*/

void	KBVerifyOpts::save
	(	TKConfig	*config
	)
{
#if	! __KB_RUNTIME
	m_options.m_verInsert	= (KBOptions::VerifyOption)m_cbInsert->currentItem() ;
	m_options.m_verUpdate	= (KBOptions::VerifyOption)m_cbUpdate->currentItem() ;
	m_options.m_verDelete	= (KBOptions::VerifyOption)m_cbDelete->currentItem() ;
	m_options.m_verMultiple	= m_cbMulti->currentItem() ;

	config->writeEntry ("verInsert",  m_options.m_verInsert  ) ;
	config->writeEntry ("verUpdate",  m_options.m_verUpdate  ) ;
	config->writeEntry ("verDelete",  m_options.m_verDelete  ) ;
	config->writeEntry ("verMulti",   m_options.m_verMultiple) ;
#endif
}



/*  ------------------------------------------------------------------  */

/*  KBLayoutOpts							*/
/*  KBLayoutOpts: Constructor for layout page				*/
/*  parent	: KBComboWidget * : Parent				*/
/*  options	: KBOptions &	  : Options object			*/
/*  (returns)	: void		  :					*/

KBLayoutOpts::KBLayoutOpts
	(	KBComboWidget	*parent,
		KBOptions	&options
	)
	:
	RKGridBox	(5, parent, "layout"),
	m_options	(options)
{
	parent->addTab	(this, TR("Layout Options")) ;

	new QLabel (TR("Design grid"	), this)	;
	(new QLabel (TR("X"), this))->setAlignment(AlignRight|AlignVCenter) ;
	m_sGridX   = new QSpinBox (   1,   50, 1, this)	;
	(new QLabel (TR("Y"), this))->setAlignment(AlignRight|AlignVCenter) ;
	m_sGridY   = new QSpinBox (   1,   50, 1, this)	;

	new QLabel (TR("Form default"	), this)	;
	(new QLabel (TR("Width" ), this))->setAlignment(AlignRight|AlignVCenter) ;
	m_sFormW   = new QSpinBox (  10, 5000, 1, this)	;
	(new QLabel (TR("Height"), this))->setAlignment(AlignRight|AlignVCenter) ;
	m_sFormH   = new QSpinBox (  10, 5000, 1, this)	;

	new QLabel (TR("Block default"	), this)	;
	(new QLabel (TR("DX"), this))->setAlignment(AlignRight|AlignVCenter) ;
	m_sDefDX   = new QSpinBox (-100,  100, 1, this)	;
	(new QLabel (TR("DY"), this))->setAlignment(AlignRight|AlignVCenter) ;
	m_sDefDY   = new QSpinBox (-100,  100, 1, this)	;

	new QLabel  (TR("Minimum cell sizes"), this)	;
	(new QLabel (TR("Width" ), this))->setAlignment(AlignRight|AlignVCenter) ;
	m_sCellWidth   = new QSpinBox (  10, 5000, 1, this)	;
	(new QLabel (TR("Height"), this))->setAlignment(AlignRight|AlignVCenter) ;
	m_sCellHeight  = new QSpinBox (  10, 5000, 1, this)	;

	new QLabel (TR("Control spacing"), this)	;

	new QWidget(this) ;
	m_sSpace   = new QSpinBox (   0,  100, 1, this)	;
	new QWidget(this) ;
	new QWidget(this) ;

	addFillerRow ()	  ;

	m_sGridX     ->setValue (m_options.m_gridX) ;
	m_sGridY     ->setValue (m_options.m_gridY) ;
	m_sFormW     ->setValue (m_options.m_formW) ;
	m_sFormH     ->setValue (m_options.m_formH) ;
	m_sDefDX     ->setValue (m_options.m_defDX) ;
	m_sDefDY     ->setValue (m_options.m_defDY) ;
	m_sSpace     ->setValue	(m_options.m_space) ;

	m_sCellWidth ->setValue (m_options.m_minCellWidth ) ;
	m_sCellHeight->setValue (m_options.m_minCellHeight) ;
}

/*  KBLayoutOpts							*/
/*  save	: Save layout settings					*/
/*  config	: TKConfig *	: Configuration				*/
/*  (returns)	: void		:					*/

void	KBLayoutOpts::save
	(	TKConfig	*config
	)
{
#if	! __KB_RUNTIME
	m_options.m_gridX	= m_sGridX->cleanText().toInt() ;
	m_options.m_gridY	= m_sGridY->cleanText().toInt() ;
	m_options.m_formW	= m_sFormW->cleanText().toInt() ;
	m_options.m_formH	= m_sFormH->cleanText().toInt() ;
	m_options.m_defDX	= m_sDefDX->cleanText().toInt() ;
	m_options.m_defDY	= m_sDefDY->cleanText().toInt() ;
	m_options.m_space	= m_sSpace->cleanText().toInt() ;

	m_options.m_minCellWidth  = m_sCellWidth ->cleanText().toInt() ;
	m_options.m_minCellHeight = m_sCellHeight->cleanText().toInt() ;

	config->writeEntry ("gridX",  m_options.m_gridX) ;
	config->writeEntry ("gridY",  m_options.m_gridY) ;
	config->writeEntry ("formW",  m_options.m_formW) ;
	config->writeEntry ("formH",  m_options.m_formH) ;
	config->writeEntry ("defDX",  m_options.m_defDX) ;
	config->writeEntry ("defDY",  m_options.m_defDY) ;
	config->writeEntry ("space",  m_options.m_space) ;

	config->writeEntry ("minCellWidth",  m_options.m_minCellWidth ) ;
	config->writeEntry ("minCellHeight", m_options.m_minCellHeight) ;
#endif
}


/*  ------------------------------------------------------------------  */

KBDesignOpts::KBDesignOpts
	(	KBComboWidget 	*parent,
		KBOptions	&options
	)
	:
	RKVBox		(parent, "design"),
	m_options	(options)
{
	parent->addTab	(this, TR("Design Options")) ;

	m_useToolbox	= new RKCheckBox(TR("Use design toolbox"), this) ;
	m_suspendToolbox= new RKCheckBox(TR("Suspend toolbox in property editor"), this) ;
	m_useWizards	= new RKCheckBox(TR("Use design wizards"), this) ;

	RKHBox	*hb	= new RKHBox	(this)	;
	new QLabel(TR("Button Images"),  hb)	;
	m_noButtonImages= new RKComboBox(hb)	;
	m_noButtonImages->insertItem(TR("Auto"	  )) ;
	m_noButtonImages->insertItem(TR("Enabled" )) ;
	m_noButtonImages->insertItem(TR("Disabled")) ;

	addFiller ()	;

	m_useToolbox	->setChecked    (m_options.m_useToolbox    ) ;
	m_suspendToolbox->setChecked	(m_options.m_suspendToolbox) ;
	m_useWizards	->setChecked    (m_options.m_useWizards    ) ;
	m_noButtonImages->setCurrentItem(m_options.m_noButtonImages) ;
}

void	KBDesignOpts::save
	(	TKConfig	*config
	)
{
	m_options.m_useToolbox	= m_useToolbox	  ->isChecked	() ;
	m_options.m_suspendToolbox= m_suspendToolbox->isChecked	() ;
	m_options.m_useWizards	= m_useWizards	  ->isChecked	() ;
	m_options.m_noButtonImages= m_noButtonImages->currentItem	() ;

	config->writeEntry ("useToolbox",  	m_options.m_useToolbox	) ;
	config->writeEntry ("suspendToolbox",	m_options.m_suspendToolbox) ;
	config->writeEntry ("useWizards",  	m_options.m_useWizards	) ;
	config->writeEntry ("noButtonImages",	m_options.m_noButtonImages) ;
}


/*  ------------------------------------------------------------------  */

/*  KBInterfaceOpts							*/
/*  KBInterfaceOpts							*/
/*		: Constructor for interface page			*/
/*  parent	: KBComboWidget * : Parent				*/
/*  options	: KBOptions &	  : Options object			*/
/*  (returns)	: void		  :					*/

KBInterfaceOpts::KBInterfaceOpts
	(	KBComboWidget	*parent,
		KBOptions	&options
	)
	:
	RKVBox		(parent, "interface"),
	m_options	(options)
{
	parent->addTab	(this, TR("User Interface")) ;

	m_useGroup	= new QVButtonGroup (TR("SDI/MDI Mode"),  this) ;
	m_useMDI	= new QRadioButton  (TR("Use MDI"), m_useGroup) ;
	m_useSDI	= new QRadioButton  (TR("Use SDI"), m_useGroup) ;
	m_useMDI->setChecked (  m_options.m_useMdi) ;
	m_useSDI->setChecked (! m_options.m_useMdi) ;

	m_openLast	= new RKCheckBox    (TR("Open last database at startup"), this) ;
	m_singleDBOnly	= new RKCheckBox    (TR("Allow only one open database" ), this) ;
#if	__KB_TKC
	RKHBox *lay	= new RKHBox 	    (this) ;
	new QLabel (TR("Style"), lay) ;
	m_style		= new RKComboBox    (lay)  ;
#endif
	m_bSetup	= new RKPushButton  (TR("Rerun setup wizard"), this) ;

	addFiller ()	;

	m_openLast	->setChecked (m_options.m_openLast    ) ;
	m_singleDBOnly	->setChecked (m_options.m_singleDBOnly) ;

	connect	(m_bSetup, SIGNAL(clicked()), SLOT(resetSetup())) ;

#if	__KB_TKC
	m_style->insertItem	 ("") ;
	m_style->insertStringList(QStyleFactory::keys()) ;
	m_style->setCurrentItem  (0)  ;

	for (int idx = 1 ; idx < m_style->count() ; idx += 1)
		if (m_style->text(idx) == m_options.m_style)
		{	m_style->setCurrentItem (idx) ;
			break ;
		}
#endif
}

void	KBInterfaceOpts::save
	(	TKConfig	*config
	)
{
	if (m_useMDI->isChecked() != m_options.m_useMdi)
		TKMessageBox::information
		(	0,
			TR("Please note that you will need to restart\n"
			   "for SDI/MDI switching to take effect"),
			TR("SDI/MDI switching"),
			"shown_sdimdi"
		)	;

	m_options.m_useMdi	= m_useMDI      ->isChecked() ;
	m_options.m_openLast	= m_openLast    ->isChecked() ;
	m_options.m_singleDBOnly= m_singleDBOnly->isChecked() ;
#if	__KB_TKC
	if (m_options.m_style != m_style->currentText())
	{
		m_options.m_style = m_style->currentText() ;
		QApplication::setStyle (m_options.m_style) ;
	}
#endif
	config->writeEntry ("usemdi", 		m_options.m_useMdi      ) ;
	config->writeEntry ("openLast",		m_options.m_openLast    ) ;
	config->writeEntry ("singledbonly", 	m_options.m_singleDBOnly) ;
	config->writeEntry ("style",		m_options.m_style	) ;

	if (!m_bSetup->isEnabled())
		config->writeEntry ("version", 	0) ;
}

void	KBInterfaceOpts::resetSetup ()
{
	m_bSetup->setEnabled (false) ;

	TKMessageBox::information
	(	0,
		TR("Setup wizard will be rerun next time you start Rekall"),
		TR("Rerun Setup Wizard")
	)	;
}

/*  ------------------------------------------------------------------  */

/*  KBModalOpts								*/
/*  KBModalOpts	: Constructor for modal execution page			*/
/*  parent	: KBComboWidget * : Parent				*/
/*  options	: KBOptions &	  : Options object			*/
/*  (returns)	: void		  :					*/

KBModalOpts::KBModalOpts
	(	KBComboWidget	*parent,
		KBOptions	&options
	)
	:
	RKVBox		(parent, "modal"),
	m_options	(options)
{
	parent->addTab	(this, TR("Modal Forms and Reports")) ;

	m_tablesModal	= new RKCheckBox(TR("Tables displayed modally" ), this) ;
	m_formsModal	= new RKCheckBox(TR("Create forms as modal"    ), this) ;
	m_reportsModal	= new RKCheckBox(TR("Create reports as modal"  ), this) ;
	m_queriesModal	= new RKCheckBox(TR("Queries displayed modally"), this) ;

	addFiller ()	;

	m_tablesModal	->setChecked (m_options.m_tablesModal ) ;
	m_formsModal	->setChecked (m_options.m_formsModal  ) ;
	m_reportsModal	->setChecked (m_options.m_reportsModal) ;
	m_queriesModal	->setChecked (m_options.m_queriesModal) ;
}

void	KBModalOpts::save
	(	TKConfig	*config
	)
{
	m_options.m_tablesModal  = m_tablesModal ->isChecked() ;
	m_options.m_formsModal   = m_formsModal  ->isChecked() ;
	m_options.m_reportsModal = m_reportsModal->isChecked() ;
	m_options.m_queriesModal = m_queriesModal->isChecked() ;

	config->writeEntry ("tablesModal",	m_options.m_tablesModal ) ;
	config->writeEntry ("formsModal",	m_options.m_formsModal  ) ;
	config->writeEntry ("reportsModal",	m_options.m_reportsModal) ;
	config->writeEntry ("queriesModal",	m_options.m_queriesModal) ;
}

/*  ------------------------------------------------------------------  */

/*  KBReportOpts							*/
/*  KBReportOpts: Constructor for report options page			*/
/*  parent	: KBComboWidget * : Parent				*/
/*  options	: KBOptions &	  : Options object			*/
/*  (returns)	: void		  :					*/

KBReportOpts::KBReportOpts
	(	KBComboWidget	*parent,
		KBOptions	&options
	)
	:
	RKGridBox	(2, parent, "report"),
	m_options	(options)
{
	parent->addTab	(this, TR("Report Settings" )) ;

	new QLabel (TR("Left margin (mm)"  ), this) ;
	m_slMargin	= new QSpinBox (0, 0x7fffffff, 1, this),

	new QLabel (TR("Top margin (mm)"   ), this) ;
	m_srMargin	= new QSpinBox (0, 0x7fffffff, 1, this),

	new QLabel (TR("Right margin (mm)" ), this) ;
	m_stMargin	= new QSpinBox  (0, 0x7fffffff, 1, this),

	new QLabel (TR("Bottom margin (mm)"), this) ;
	m_sbMargin	= new QSpinBox  (0, 0x7fffffff, 1, this),

	new QLabel (TR("Print DPI (zero default)" ), this) ;
	m_sDPI		= new QSpinBox  (0, 0x7fffffff, 1, this),

	new QLabel (TR("Design rulers show inches"), this) ;
	m_cbDesign	= new RKCheckBox(this) ;

	addFillerRow ()	;


	m_slMargin->setValue   (m_options.m_marginL) ;
	m_srMargin->setValue   (m_options.m_marginR) ;
	m_sbMargin->setValue   (m_options.m_marginB) ;
	m_stMargin->setValue   (m_options.m_marginT) ;
	m_sDPI	  ->setValue   (m_options.m_dpi	 ) ;
	m_cbDesign->setChecked (m_options.m_designInches) ;
}

/*  KBReportOpts							*/
/*  save	: Save report settings					*/
/*  config	: TKConfig *	: Configuration				*/
/*  (returns)	: void		:					*/

void	KBReportOpts::save
	(	TKConfig	*config
	)
{
	m_options.m_marginL	= m_slMargin->cleanText().toInt() ;
	m_options.m_marginR	= m_srMargin->cleanText().toInt() ;
	m_options.m_marginB	= m_sbMargin->cleanText().toInt() ;
	m_options.m_marginT	= m_stMargin->cleanText().toInt() ;
	m_options.m_dpi		= m_sDPI    ->cleanText().toInt() ;
	m_options.m_designInches	= m_cbDesign->isChecked() ;

#if	! __KB_RUNTIME
	config->writeEntry ("marginL",		m_options.m_marginL     ) ;
	config->writeEntry ("marginR",		m_options.m_marginR     ) ;
	config->writeEntry ("marginB",		m_options.m_marginB     ) ;
	config->writeEntry ("marginT",		m_options.m_marginT     ) ;
	config->writeEntry ("dpi",		m_options.m_dpi	      ) ;
	config->writeEntry ("designInches",	m_options.m_designInches) ;
#endif
}

/*  ------------------------------------------------------------------  */

KBPythonOpts::KBPythonOpts
	(	KBComboWidget	*parent,
		KBOptions	&options
	)
	:
	RKGridBox	(2, parent, "python"),
	m_options	(options)
{
	parent->addTab	(this, TR("Python Settings")) ;

	TKConfig *config = TKConfig::getConfig() ;
        config->setGroup ("Debug Options/py")	 ;

	new QLabel	(TR("Enable user settings"  ), this)	;
	m_eUserpy	= new RKCheckBox (this) ;

#if	! __KB_USESYSPY
	new QLabel	(TR("Version"), this) ;
	m_eVersion	= new RKComboBox (this) ;

	QString	 libver	 = config->readEntry ("libver" ) ;

	m_eVersion->insertItem ("1.5.2") ;
	m_eVersion->insertItem ("2.0"  ) ;
	m_eVersion->insertItem ("2.1.1") ;
	m_eVersion->insertItem ("2.2"  ) ;
	m_eVersion->insertItem ("2.3"  ) ;

	if	(libver == "23") m_eVersion->setCurrentItem (3) ;
	else if	(libver == "22") m_eVersion->setCurrentItem (3) ;
	else if	(libver == "21") m_eVersion->setCurrentItem (2) ;
	else if	(libver == "20") m_eVersion->setCurrentItem (1) ;
	else if	(libver == "15") m_eVersion->setCurrentItem (0) ;
#endif

#ifdef	Q_OS_LINUX
	new QLabel	(TR("Executable path"), this)	;
	m_eBinpath	= new RKLineEdit  (this)	;
#endif
	new QLabel	(TR("PYTHONPATH"), this)	;
	m_ePypath	= new RKLineEdit  (this)	;

	new QLabel	(TR("String encoding"), this)	;
	m_ePyEncoding	= new RKComboBox  (this)	;
	m_ePyEncoding->setEditable(true)		;

	/* Not obvious that there should be an unset value, OTOH this	*/
	/* is maybe consistent with previously not having such a	*/
	/* setting.							*/
	m_ePyEncoding->insertItem ("") ;

	int atp = -1 ;
	for (cchar **cnp = &codecNames[0]; *cnp != 0; cnp += 1)
	{
		if (*cnp == config->readEntry("pyencoding", "ISO8859-1")) 
			atp = m_ePyEncoding->count();
		m_ePyEncoding->insertItem(*cnp);
	}
	if (atp >= 0)
		m_ePyEncoding->setCurrentItem(atp) ;

	addFillerRow ()	;

	m_eUserpy ->setChecked (config->readBoolEntry ("userpy" )) ;
#ifdef	Q_OS_LINUX
	m_eBinpath->setText    (config->readEntry     ("binpath")) ;
#endif
	m_ePypath ->setText    (config->readEntry     ("pypath" )) ;

	changeUserPY() ;
	connect	(m_eUserpy, SIGNAL(toggled(bool)), SLOT(changeUserPY())) ;
}

void	KBPythonOpts::save
	(	TKConfig	*
	)
{
	TKConfig *config = TKConfig::getConfig() ;
        config->setGroup ("Debug Options/py") ;

	config->writeEntry	("userpy",	m_eUserpy    ->isChecked()) ;
#ifdef	Q_OS_LINUX
	config->writeEntry	("binpath",	m_eBinpath   ->text()) ;
#endif
	config->writeEntry	("pypath",	m_ePypath    ->text()) ;
	config->writeEntry	("pyencoding",	m_ePyEncoding->currentText());

#if	! __KB_USESYSPY
	static	char	versions[][4] =
	{	"15", "20", "21", "22", "23"
	}	;

	config->writeEntry	("libver",  QString(versions[m_eVersion->currentItem()])) ;
#endif
}


void	KBPythonOpts::changeUserPY()
{
	bool	userpy	= m_eUserpy->isChecked() ;
#if	!__KB_USESYSPY
	m_eVersion->setEnabled (userpy) ;
#endif
#ifdef	Q_OS_LINUX
	m_eBinpath->setEnabled (userpy) ;
#endif
	m_ePypath ->setEnabled (userpy) ;
}


/*  ------------------------------------------------------------------  */

/*  KBScriptOpts							*/
/*  KBScriptOpts: Constructor for script options page			*/
/*  parent	: KBComboWidget * : Parent				*/
/*  options	: KBOptions &	  : Options object			*/
/*  (returns)	: void		  :					*/

KBScriptOpts::KBScriptOpts
	(	KBComboWidget	*parent,
		KBOptions	&options
	)
	:
	RKGridBox	(3, parent, "script"),
	m_options	(options)
{
	parent->addTab	(this, TR("Scripts and Macros")) ;

	new QLabel	(TR("Script font"), this) ;
	m_eScriptFont	= new RKLineEdit   (this) ;
	RKPushButton *s	= new RKPushButton (TR("Set"), this) ;

	m_eScriptFont->setText	   (m_options.m_scriptFont) ;
	m_eScriptFont->setReadOnly (true) ;

	connect	(s, SIGNAL(clicked()), SLOT(clickSetScriptFont())) ;

	new QLabel	(TR("Scripting"  ), this) ;
	m_cbScriptType	= new RKComboBox   (this) ;
	new QWidget	(this) ;

	new QLabel	(TR("Macro debug"), this) ;
	m_cbMacroDebug	= new RKComboBox   (this) ;
	new QWidget	(this) ;

	new QLabel	(TR("Keymap file"), this) ;
	m_leKeymap	= new RKLineEdit   (this) ;
	RKPushButton *k	= new RKPushButton (TR("Set"), this) ;
	connect	(k, SIGNAL(clicked()), SLOT(clickSetKeymap())) ;

	new QWidget	(this)	;
	m_autoIntelli	= new RKCheckBox   (TR("Auto-display script help"), this) ;
	addFillerRow () ;

	m_cbScriptType->insertItem (TR("Always use scripting")) ;
	m_cbScriptType->insertItem (TR("Prompt when empty"   )) ;
	m_cbScriptType->insertItem (TR("Always use macros"   )) ;
	m_cbScriptType->setCurrentItem (m_options.m_scriptType) ;

	m_cbMacroDebug->insertItem (TR("On demand"  )) ;
	m_cbMacroDebug->insertItem (TR("Always off" )) ;
	m_cbMacroDebug->insertItem (TR("Always on"  )) ;
	m_cbMacroDebug->setCurrentItem (m_options.m_macroDebug) ;

	m_leKeymap    ->setText    (m_options.m_keymap	   ) ;
	m_autoIntelli ->setChecked (m_options.m_autoIntelli) ;
}

void	KBScriptOpts::clickSetScriptFont ()
{
	TKFontDialog  fDialog (this, TR("Font"), true, true) ;
	fDialog.setFont (KBFont::specToFont (m_eScriptFont->text(), true)) ;

	if (fDialog.exec())
		m_eScriptFont->setText (KBFont::fontToSpec (fDialog.font())) ;
}

void	KBScriptOpts::clickSetKeymap ()
{
	QString	curr	= m_leKeymap->text() ;
	if (curr.isEmpty())
		curr	= locateFile ("appdata", "keymaps/keymap.null") ;

	QString	name	= KBFileDialog::getOpenFileName
			  (	curr,
				"keymap.*|Kemap files",
				0,
				"Keymap"
			  )	;

	if (!name.isEmpty()) m_leKeymap->setText(name) ;

}


void	KBScriptOpts::save
	(	TKConfig	*config
	)
{
	m_options.m_scriptFont = m_eScriptFont->text() ;
	config->writeEntry ("scriptFont", m_options.m_scriptFont) ;

	m_options.m_scriptType   = (KBOptions::ScriptType)m_cbScriptType->currentItem() ;
	config->writeEntry ("scriptType", m_options.m_scriptType) ;

	m_options.m_macroDebug   = (KBOptions::MacroDebug)m_cbMacroDebug->currentItem() ;
	config->writeEntry ("macroDebug", m_options.m_macroDebug) ;

	m_options.m_keymap       = m_leKeymap->text() ;
	config->writeEntry ("keymap",	  m_options.m_keymap) ;

	m_options.m_autoIntelli	 = m_autoIntelli->isChecked() ;
	config->writeEntry ("autointelli",m_options.m_autoIntelli) ;
}


/*  ------------------------------------------------------------------  */

/*  KBCacheOpts								*/
/*  KBCacheOpts								*/
/*		: Constructor for cache options page			*/
/*  parent	: KBComboWidget * : Parent				*/
/*  options	: KBOptions &	  : Options object			*/
/*  (returns)	: void		  :					*/

KBCacheOpts::KBCacheOpts
	(	KBComboWidget	*parent,
		KBOptions	&options
	)
	:
	RKGridBox	(2, parent, "cache"),
	m_options	(options)
{
	parent->addTab	(this, TR("Cache")) ;

	new QLabel (TR("Cache size (MB)"  ), this) ;
	m_sCacheSize	= new QSpinBox   (0, 2000, 1, this) ;
	new QLabel (TR("Cache used (MB)"  ), this) ;
	m_lCacheUsing	= new RKLineEdit (this)	   ;
	m_lCacheUsing->setReadOnly	 (true)	   ;

	new QLabel (TR("Cache mode"), this) ;
	m_cacheOption	= new RKComboBox  (this) ;
	m_cacheOption->insertItem (TR("No cache"           )) ;
	m_cacheOption->insertItem (TR("Cache all objects"  )) ;
	m_cacheOption->insertItem (TR("Cache only graphics")) ;
	m_cacheOption->setCurrentItem (m_options.m_cacheOption) ;

	new QLabel (TR("Cache all in tests"), this) ;
	m_cacheInTests = new RKCheckBox (this) ;

	new QLabel (QString::null, this) ;
	m_clearCache	= new RKPushButton(TR("Clear cache"), this) ;

	addFillerRow () ;
	connect (m_clearCache, SIGNAL(clicked()), SLOT(slotClearCache())) ;

	m_sCacheSize ->setValue(m_options.m_cacheSize) ;

	uint	used	= KBLocation::getCacheUsed() ;
	m_lCacheUsing->setText    (QString("%1").arg(used)) ;
	m_clearCache ->setEnabled (used > 0) ;

	m_cacheInTests->setChecked (m_options.m_cacheInTests) ;
}


void	KBCacheOpts::save
	(	TKConfig	*config
	)
{
	m_options.m_cacheSize	= m_sCacheSize  ->cleanText().toInt() ;
	m_options.m_cacheOption = m_cacheOption ->currentItem () ;
	m_options.m_cacheInTests= m_cacheInTests->isChecked   () ;

	config->writeEntry ("cacheSize",    m_options.m_cacheSize   ) ;
	config->writeEntry ("cacheOption",  m_options.m_cacheOption ) ;
	config->writeEntry ("cacheInTests", m_options.m_cacheInTests) ;

	KBLocation::setCacheSize
	(	m_options.m_cacheSize,
		(KBLocation::CacheOption)m_options.m_cacheOption
	) 	;

	uint	used	= KBLocation::getCacheUsed() ;
	m_lCacheUsing->setText    (QString("%1").arg(used)) ;
	m_clearCache ->setEnabled (used > 0) ;
}

void	KBCacheOpts::slotClearCache ()
{
	KBLocation::setCacheSize
	(	0,
		(KBLocation::CacheOption)m_options.m_cacheOption
	)	;
	KBLocation::setCacheSize
	(	m_options.m_cacheSize,
		(KBLocation::CacheOption)m_options.m_cacheOption
	)	;

	uint	used	= KBLocation::getCacheUsed() ;
	m_lCacheUsing->setText    (QString("%1").arg(used)) ;
	m_clearCache ->setEnabled (used > 0) ;
}

/*  ------------------------------------------------------------------  */

/*  KBLoggingOpts							*/
/*  KBLoggingOpts							*/
/*		: Constructor for logging options page			*/
/*  parent	: KBComboWidget * : Parent				*/
/*  options	: KBOptions &	  : Options object			*/
/*  (returns)	: void		  :					*/

KBLoggingOpts::KBLoggingOpts
	(	KBComboWidget	*parent,
		KBOptions	&options
	)
	:
	RKGridBox	(2, parent, "logging"),
	m_options	(options)
{
	parent->addTab	(this, TR("Logging")) ;

	new QLabel (TR("Maximum queries logged"  ), this) ;
	m_sMaxQueries	= new QSpinBox (10, 1000, 1, this) ;
	new QLabel (TR("Maximum events logged"   ), this) ;
	m_sMaxEvents	= new QSpinBox (10, 2000, 1, this) ;
	new QLabel (TR("Maximum arguments logged"), this) ;
	m_sMaxArgs	= new QSpinBox ( 5,  500, 1, this) ;
	new QLabel (TR("Maximum recorded argument length"), this) ;
	m_sMaxArgLen	= new QSpinBox (50, 1000, 1, this) ;

	addFillerRow () ;

	m_sMaxQueries->setValue(m_options.m_logMaxQueries) ;
	m_sMaxEvents ->setValue(m_options.m_logMaxEvents ) ;
	m_sMaxArgs   ->setValue(m_options.m_logMaxArgs   ) ;
	m_sMaxArgLen ->setValue(m_options.m_logMaxArgLen ) ;
}


void	KBLoggingOpts::save
	(	TKConfig	*config
	)
{
	m_options.m_logMaxQueries	= m_sMaxQueries->cleanText().toInt() ;
	m_options.m_logMaxEvents	= m_sMaxEvents ->cleanText().toInt() ;
	m_options.m_logMaxArgs		= m_sMaxArgs   ->cleanText().toInt() ;
	m_options.m_logMaxArgLen	= m_sMaxArgLen ->cleanText().toInt() ;

	config->writeEntry ("logMaxQueries", m_options.m_logMaxQueries) ;
	config->writeEntry ("logMaxEvents",  m_options.m_logMaxEvents ) ;
	config->writeEntry ("logMaxArgs",    m_options.m_logMaxArgs   ) ;
	config->writeEntry ("logMaxArgLen",  m_options.m_logMaxArgLen ) ;
}


/*  ------------------------------------------------------------------  */

static	QString	lastPage	;

/*  KBOptionsDlg							*/
/*  KBOptionsDlg: Consttuctor for options dialog			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  (returns)	: KBOptions	:					*/

KBOptionsDlg::KBOptionsDlg
	(	QWidget		*
	)
	:
	KBDialog	(TR("Options"), true, "optionsdlg")
{
	RKVBox *layMain	= new RKVBox (this) ;
	layMain->setTracking () ;

	RKHBox *layData	= new RKHBox (layMain) ;
	new KBSidePanel  (layData, TR("Options")) ;

	addOKCancel (layMain)	;

	m_blurb		= new QTextView     (layData) ;
	m_tabber 	= new KBComboWidget (layData) ;

	m_verify	= new KBVerifyOpts	(m_tabber, KBOptions::self()) ;
	m_interface	= new KBInterfaceOpts	(m_tabber, KBOptions::self()) ;
	m_modal		= new KBModalOpts	(m_tabber, KBOptions::self()) ;
	m_design	= new KBDesignOpts	(m_tabber, KBOptions::self()) ;

#if	! __KB_RUNTIME
	m_layout	= new KBLayoutOpts	(m_tabber, KBOptions::self()) ;
	m_report	= new KBReportOpts	(m_tabber, KBOptions::self()) ;
#endif
	m_scripts	= new KBScriptOpts	(m_tabber, KBOptions::self()) ;
	m_python	= new KBPythonOpts	(m_tabber, KBOptions::self()) ;

	m_cache		= new KBCacheOpts	(m_tabber, KBOptions::self()) ;
#if	! __KB_RUNTIME
	m_logging	= new KBLoggingOpts	(m_tabber, KBOptions::self()) ;
#endif

	connect
	(	m_tabber,
		SIGNAL(currentChanged(QWidget *)),
		SLOT  (pageChanged   (QWidget *))
	)	;

	m_tabber->setCurrentPage (lastPage) ;

	QWidget *vis = m_tabber->visibleWidget() ;
	pageChanged (vis == 0 ? m_verify :  vis) ;
}


/*  KBOptionsDlg								*/
/*  accept	: Handle user clicking the OK button			*/
/*  (returns)	: void		:					*/

void	KBOptionsDlg::accept ()
{
	TKConfig *config = TKConfig::getConfig () ;
	config->setGroup   (KB_OPTIONS_GROUP) ;

	m_verify   ->save (config) ;
	m_interface->save (config) ;
	m_modal    ->save (config) ;
	m_design   ->save (config) ;
	m_cache    ->save (config) ;
	m_logging  ->save (config) ;

#if	! __KB_RUNTIME
	m_layout   ->save (config) ;
	m_report   ->save (config) ;
#endif
	m_scripts  ->save (config) ;

	/* Save the python options last, since these do not go into	*/
	/* the KB_OPTIONS_GROUP section as to the rest of the options.	*/
	m_python   ->save (config) ;

	config->sync () ;

	/* Reload new option values. Notify changes to anyone who	*/
	/* might be interested. Too tricky to be any more fine-grained	*/
	/* about this.							*/
	KBOptions::loadOptions() ;
	KBNotifier::self()->nOptionsChanged() ;

	lastPage	= m_tabber->visibleWidget()->name() ;
	done	(1) 	;
}

/*  KBOptionsDlg								*/
/*  reject	: Handle user clicking the cancel button			*/
/*  (returns)	: void		:					*/

void	KBOptionsDlg::reject ()
{
	done	(0)	;
}

void	KBOptionsDlg::pageChanged
	(	QWidget		*widget
	)
{
	fprintf
	(	stderr,
		"KBOptionsDlg::pageChanged: [%s]\n",
		widget->name()
	)	;

	QString	name	= locateFile ("appdata", QString("help/opts_%1.qt").arg(widget->name())) ;
	if (name.isEmpty())
	{
		m_blurb->setText (QString::null) ;
		return	;
	}

	QFile	file	(name)	;

	if (file.open (IO_ReadOnly))
		m_blurb->setText (QTextStream(&file).read()) ;
	else	m_blurb->setText (QString::null) ;
}


/*  ------------------------------------------------------------------  */

/*  KBOptionsDlg							*/
/*  setupWizard	: Startup configuration wizard				*/
/*  config	: TKConfig *	: Configuration object			*/
/*  (returns)	: void		:					*/

void	KBOptionsDlg::setupWizard
	(	TKConfig	*config
	)
{
	config->setGroup (KB_OPTIONS_GROUP) ;
	if (config->readNumEntry("version") >= 240) return ;

	QString	wizXML = locateFile ("appdata", "wizards/wizSetup.wiz") ;
	if (wizXML.isEmpty()) return ;

	KBWizard wizSetup (0, QString::null) ;
	if (!wizSetup.init (wizXML)) return ;

	if (wizSetup.execute() && (wizSetup.currentPageName() != "initial"))
	{
		QVariant  mode		= wizSetup.ctrlAttribute ("interface", "mode",      "index") ;
		QVariant  scripting	= wizSetup.ctrlAttribute ("scripting", "scripting", "index") ;
		QVariant  toolbox	= wizSetup.ctrlAttribute ("design",    "toolbox",   "index") ;
		QVariant  wizards	= wizSetup.ctrlAttribute ("design",    "wizards",   "index") ;
		QVariant  vInsert	= wizSetup.ctrlAttribute ("verify",    "insert",    "index") ;
		QVariant  vUpdate	= wizSetup.ctrlAttribute ("verify",    "update",    "index") ;
		QVariant  vDelete	= wizSetup.ctrlAttribute ("verify",    "delete",    "index") ;

		KBOptions &opts		= KBOptions::self() ;

		opts.m_useMdi		= mode.toInt() > 0 ;
		opts.m_scriptType	= (KBOptions::ScriptType  )scripting.toInt() ;
		opts.m_useToolbox	= toolbox.toInt() == 0 ;
		opts.m_useWizards 	= wizards.toInt() == 0 ;
		opts.m_verInsert	= (KBOptions::VerifyOption)vInsert  .toInt() ;
		opts.m_verUpdate	= (KBOptions::VerifyOption)vUpdate  .toInt() ;
		opts.m_verDelete	= (KBOptions::VerifyOption)vDelete  .toInt() ;

		config->setGroup   (KB_OPTIONS_GROUP) ;
		config->writeEntry ("usemdi", 	  opts.m_useMdi    	) ;
		config->writeEntry ("scriptType", opts.m_scriptType	) ;
		config->writeEntry ("useToolbox", opts.m_useToolbox	) ;
		config->writeEntry ("useWizards", opts.m_useWizards	) ;
		config->writeEntry ("verInsert",  opts.m_verInsert 	) ;
		config->writeEntry ("verUpdate",  opts.m_verUpdate 	) ;
		config->writeEntry ("verDelete",  opts.m_verDelete 	) ;
	}

	config->setGroup   (KB_OPTIONS_GROUP) ;
	config->writeEntry ("version", 240  ) ;
	config->sync	   () ;
}
