/***************************************************************************
    file	         : kb_optionsdlg.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
			   (C)	2005
				by Abel Deuring
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_OPTIONSDLG_H
#define	_KB_OPTIONSDLG_H


#include	"rk_gridbox.h"
#include	"rk_vbox.h"
#include	"rk_hbox.h"

#include	"libkbase_exports.h"

#include	"kb_dialog.h"
#include	"kb_options.h"
#include	"tk_config.h"


class		QSpinBox	;
class		QVButtonGroup	;
class		QRadioButton	;
class		QTextView	;
class		RKTabWidget	;
class		KBComboWidget	;

class		RKLineEdit	;
class		RKComboBox	;
class		RKCheckBox	;
class		RKPushButton	;


/*  KBVerifyOpts							*/
/*  ------------							*/
/*									*/
/*  Database verifcation options.					*/
/*									*/
class	LIBKBASE_API	KBVerifyOpts : public RKGridBox
{
	KBOptions	&m_options	;

	RKComboBox	*m_cbInsert	;
	RKComboBox	*m_cbUpdate	;
	RKComboBox	*m_cbDelete	;
	RKComboBox	*m_cbMulti	;

public	:

	KBVerifyOpts	(KBComboWidget *, KBOptions &) ;

	void	 	save (TKConfig *) ;
}	;


class	LIBKBASE_API	KBDesignOpts : public RKVBox
{
	KBOptions	&m_options		;

	RKCheckBox	*m_useToolbox		;
	RKCheckBox	*m_suspendToolbox	;
	RKCheckBox	*m_useWizards		;
	RKComboBox	*m_noButtonImages	;

public	:

	KBDesignOpts	(KBComboWidget *, KBOptions &) ;

	void	 	save (TKConfig *) ;
}	;


/*  KBLayoutOpts							*/
/*  ------------							*/
/*									*/
/*  Form layout options.						*/
/*									*/
class	LIBKBASE_API	KBLayoutOpts : public RKGridBox
{
	KBOptions	&m_options	;

	QSpinBox	*m_sGridX	;
	QSpinBox	*m_sGridY	;
	QSpinBox	*m_sFormW	;
	QSpinBox	*m_sFormH	;
	QSpinBox	*m_sDefDX	;
	QSpinBox	*m_sDefDY	;
	QSpinBox	*m_sSpace	;
	QSpinBox	*m_sCellWidth	;
	QSpinBox	*m_sCellHeight	;

public	:

	KBLayoutOpts	(KBComboWidget *, KBOptions &) ;

	void	 	save (TKConfig *) ;
}	;


/*  KBInterfaceOpts							*/
/*  ---------------							*/
/*									*/
/*  User interface options (MDI/SDI) and others				*/
/*									*/

class	LIBKBASE_API	KBInterfaceOpts : public RKVBox
{
	Q_OBJECT

	KBOptions	&m_options	;

	QVButtonGroup	*m_useGroup	;
	QRadioButton	*m_useMDI	;
	QRadioButton	*m_useSDI	;
	RKCheckBox	*m_openLast	;
	RKCheckBox	*m_singleDBOnly	;
#if	__KB_TKC
	RKComboBox	*m_style	;
#endif
	RKPushButton	*m_bSetup	;

protected slots :

	void		resetSetup ()	;

public	:

	KBInterfaceOpts	(KBComboWidget *, KBOptions &) ;

	void	 	save (TKConfig *) ;
}	;


/*  KBModalOpts								*/
/*  -----------								*/
/*									*/
/*  Modal execution options						*/
/*									*/

class	LIBKBASE_API	KBModalOpts : public RKVBox
{
	KBOptions	&m_options	;

	RKCheckBox	*m_tablesModal	;
	RKCheckBox	*m_formsModal	;
	RKCheckBox	*m_reportsModal	;
	RKCheckBox	*m_queriesModal	;

public	:

	KBModalOpts	(KBComboWidget *, KBOptions &) ;

	void	 	save (TKConfig *) ;
}	;


/*  KBReportOpts							*/
/*  ------------							*/
/*									*/
/*  Report layout options						*/
/*									*/
class	LIBKBASE_API	KBReportOpts : public RKGridBox
{
	KBOptions	&m_options	;

	QSpinBox	*m_slMargin	;
	QSpinBox	*m_srMargin	;
	QSpinBox	*m_stMargin	;
	QSpinBox	*m_sbMargin	;
	QSpinBox	*m_sDPI		;

	RKCheckBox	*m_cbDesign	;

public	:

	KBReportOpts	(KBComboWidget *, KBOptions &) ;

	void	 	save (TKConfig *) ;
}	;

/*  KBPythonOpts							*/
/*  ------------							*/
/*									*/
/*  Python interpreter options.						*/
/*  NOTE: This should really be part of the python interpreter library.	*/
/*									*/

class	LIBKBASE_API	KBPythonOpts : public RKGridBox
{
	Q_OBJECT

	KBOptions	&m_options	;

	RKCheckBox	*m_eUserpy	;

#if	!__KB_USESYSPY
	RKComboBox	*m_eVersion	;
#endif

	RKLineEdit	*m_eBinpath	;
	RKLineEdit	*m_ePypath	;
	RKComboBox	*m_ePyEncoding	;

protected slots :

	void		changeUserPY ()	;

public	:

	KBPythonOpts	(KBComboWidget *, KBOptions &) ;

	void	 	save (TKConfig *) ;
}	;


/*  KBScriptOpts							*/
/*  ------------								*/
/*									*/
/*  Global font options							*/
/*									*/
class	LIBKBASE_API	KBScriptOpts : public RKGridBox
{
	Q_OBJECT

	KBOptions	&m_options	;

	RKLineEdit	*m_eScriptFont	;
	RKComboBox	*m_cbScriptType	;
	RKComboBox	*m_cbMacroDebug	;
	RKCheckBox	*m_autoIntelli	;

	RKLineEdit	*m_leKeymap	;

private slots :

	void		clickSetScriptFont	() ;
	void		clickSetKeymap		() ;

public	:

	KBScriptOpts	(KBComboWidget *, KBOptions &) ;

	void	 	save (TKConfig *) ;
}	;

/*  KBCacheOpts								*/
/*  -----------								*/
/*									*/
/*  Global cacheoptions							*/
/*									*/
class	LIBKBASE_API	KBCacheOpts : public RKGridBox
{
	Q_OBJECT

	KBOptions	&m_options	;

	QSpinBox	*m_sCacheSize	;
	RKLineEdit	*m_lCacheUsing	;
	RKPushButton	*m_clearCache	;
	RKComboBox	*m_cacheOption	;
	RKCheckBox	*m_cacheInTests	;

protected slots :

	void		slotClearCache();

public	:

	KBCacheOpts	(KBComboWidget *, KBOptions &) ;

	void	 	save (TKConfig *) ;
}	;

/*  KBLoggingOpts							*/
/*  -------------							*/
/*									*/
/*  Global logging options							*/
/*									*/
class	LIBKBASE_API	KBLoggingOpts : public RKGridBox
{
	Q_OBJECT

	KBOptions	&m_options	;

	QSpinBox	*m_sMaxQueries	;
	QSpinBox	*m_sMaxEvents	;
	QSpinBox	*m_sMaxArgs	;
	QSpinBox	*m_sMaxArgLen	;

public	:

	KBLoggingOpts	(KBComboWidget *, KBOptions &) ;

	void	 	save (TKConfig *) ;
}	;

/*  KBOptions								*/
/*  ---------								*/
/*									*/
/*  Global options dialog						*/
/*									*/
class	LIBKBASE_API	KBOptionsDlg : public KBDialog
{
	Q_OBJECT

	QTextView	*m_blurb	;
	KBComboWidget	*m_tabber	;

	KBVerifyOpts	*m_verify	;
	KBInterfaceOpts	*m_interface	;
	KBModalOpts	*m_modal	;
	KBDesignOpts	*m_design	;
	KBLayoutOpts	*m_layout	;
	KBReportOpts	*m_report	;
	KBPythonOpts	*m_python	;
	KBScriptOpts	*m_scripts	;
	KBCacheOpts	*m_cache	;
	KBLoggingOpts	*m_logging	;

protected slots :

	virtual	void	accept		()	;
	virtual	void	reject		()	;
	void		pageChanged	(QWidget  *) ;

public	:

	KBOptionsDlg (QWidget *) ;

	static	void	setupWizard	(TKConfig *) ;
}	;

#endif	// _KB_OPTIONSDLG_H
