/***************************************************************************
    file	         : kb_override.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#include	<qptrlist.h>
#include	<qdict.h>
#include	<qstring.h>


#include	"kb_classes.h"
#include	"kb_attr.h"

#include	"kb_node.h"
#include	"kb_object.h"
#include	"kb_override.h"
#include	"kb_config.h"
#include	"kb_nodereg.h"


#define		__DEBUG__OVERRIDE	(0)
#if		__DEBUG__OVERRIDE
#include	"kb_dprintf.h"
#endif


/*  KBOverride								*/
/*  KBOverride	: Constructor for override node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: Attribute list			*/
/*		: bool *	:					*/
/*  (returns)	: KBOverride	:					*/

KBOverride::KBOverride
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*
	)
	:
	KBNode		(parent, "KBOverride"),
	m_ident		(this,	 "ident",	aList),
	m_path		(this,	 "path",	aList),
	m_attrib	(this,	 "attrib",	aList),
	m_value		(this,	 "value",	aList),
	m_enabled	(this,	 "enabled",	aList)
{
	m_target	= 0 ;
}

/*  KBOverride								*/
/*  KBOverride	: Constructor for override node				*/
/*  parent	: KBNode *	  : Parent node				*/
/*  ident	: const QString & : Identification			*/
/*  path	: const QString & : Path or target config		*/
/*  attrib	: const QString & : Attribute name			*/
/*  value	: const QString & : Default value			*/
/*  enabled	: bool		  : Override is enabled			*/
/*  (returns)	: KBOverride	  :					*/

KBOverride::KBOverride
	(	KBNode			*parent,
		const QString		&ident,
		const QString		&path,
		const QString		&attrib,
		const QString		&value,
		bool			enabled
	)
	:
	KBNode		(parent, "KBOverride"),
	m_ident		(this,	 "ident",	ident	),
	m_path		(this,	 "path",	path	),
	m_attrib	(this,	 "attrib",	attrib	),
	m_value		(this,	 "value",	value	),
	m_enabled	(this,	 "enabled",	enabled	)
{
	m_target	= 0 ;
}

/*  KBOverride								*/
/*  KBOverride	: Constructor for override node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  extant	: KBOverride *	: Extant node				*/
/*  (returns)	: KBOverride	:					*/

KBOverride::KBOverride
	(	KBNode			*parent,
		KBOverride		*extant
	)
	:
	KBNode		(parent, "KBOverride"),
	m_ident		(this,	 "ident",	extant),
	m_path		(this,	 "path",	extant),
	m_attrib	(this,	 "attrib",	extant),
	m_value		(this,	 "value",	extant),
	m_enabled	(this,	 "enabled",	extant)
{
	m_target	= 0 ;
}

/*  KBOverride								*/
/*  KBOverride	: Destructor for override node				*/
/*  (returns)	:		:					*/

KBOverride::~KBOverride ()
{
}

/*  KBOverride								*/
/*  findTarget	: Find target for substitution				*/
/*  (returns)	: bool		: Target found				*/

bool	KBOverride::findTarget ()
{
	DPRINTF
	((	"KBOverride::findTarget from [%s][%s] to [%s][%s]<-[%s]\n",
		(cchar *)getParent()->getElement(),
		(cchar *)getParent()->getName   (),
		(cchar *)m_path  .getValue(),
		(cchar *)m_attrib.getValue(),
		(cchar *)m_value .getValue()
	))	;

	QString	 path	 = m_path.getValue() ;

	if ((m_target = getParent()->getNamedNode (path)) == 0)
	{
		DPRINTF
		((	"KBOverride::substitute: target [%s] not found\n",
			path.latin1()
		))	;
		return	false	;
	}

	return	true	;
}

/*  KBOverride								*/
/*  substitute	: Substitue value into object by path			*/
/*  (returns)	: KBNode *	: Node substituted or null		*/

KBNode	*KBOverride::substitute ()
{
	DPRINTF
	((	"KBOverride::substitute from [%s][%s] to [%s][%s]<-[%s]\n",
		(cchar *)getParent()->getElement(),
		(cchar *)getParent()->getName   (),
		(cchar *)m_path  .getValue(),
		(cchar *)m_attrib.getValue(),
		(cchar *)m_value .getValue()
	))	;

	/* Check if the target is not set, in which case either		*/
	/* ::findTarget was not called or the target was not found.	*/
	if (m_target == 0)
	{
		DPRINTF
		((	"KBOverride::substitute: target was not found\n"
		))	;
		return 0 ;
	}

	QString	 attrib	 = m_attrib.getValue() ;

	LITER
	(	KBNode,
		m_target->getChildren(),
		child,

		KBConfig *c = child->isConfig() ;
		if ((c != 0) && (c->attrib() == attrib))
		{
			c->setValue   (m_value.getValue()) ;
			c->substitute (true) ;
			return	m_target ;
		}
	)

	DPRINTF
	((	"KBOverride::substitute: attribute not found\n"
	))	;
	return	0 ;
}

/*  KBOverride								*/
/*  replicate	: Replicate this configuration node			*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New choice node			*/

KBNode	*KBOverride::replicate
	(	KBNode	*_parent
	)
{
	return	new KBOverride (_parent, this) ;
}


NEWNODE(Override, (cchar *)0, 0, KNF_FORM|KNF_REPORT)


