/***************************************************************************
    file	         : kb_overridedlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<time.h>
#include	<qlistbox.h>
#include	<qwidgetstack.h>

#if		__KB_AUTOMOC
#include	"kb_overridedlg.moc"
#else
#include 	"kb_overridedlg.h"
#endif

#include	"kb_override.h"
#include	"kb_config.h"
#include	"kb_object.h"
#include	"kb_noobjdlg.h"
#include	"kb_font.h"
#include	"kb_textedit.h"

#include	"tk_colordialog.h"
#include	"tk_fontdialog.h"

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_pushbutton.h"
#include	"rk_listview.h"



/*  KBOverrideItem							*/
/*  --------------							*/
/*  This is a wrapper round a list view item, used to store the		*/
/*  properties of a override item which is being edited.		*/

class	LIBKBASE_API KBOverrideItem : public QListViewItem
{
	KBAttr		*m_attr		;
	KBAttrDlg	*m_attrDlg	;
	QString		m_value		;
	bool		m_enabled	;

	void		update	()	;

public	:

	KBOverrideItem
	(	RKListView	*,
		KBObject	*,
		const QString	&,
		const QString	&,
		const QString	&,
		bool		,	
		QWidget		*
	) ;

        virtual ~KBOverrideItem () ;

	KBOverride		*getOverride	(KBNode *, uint ) ;
	void			save		(KBTextEdit *    ) ;
	void			setValue	(const QString &) ;
	void			setEnabled	(bool)		  ;

	inline	KBAttrDlg	*getAttrDlg 	()
	{
		return	m_attrDlg ;
	}

	inline	QString		path	()	{ return text(0)   ; }
	inline	QString		attrib	()	{ return text(1)   ; }
	inline	QString		value	()	{ return m_value   ; }
	inline	bool		enabled	()	{ return m_enabled ; }
}	;


/*  KBOverrideItem							*/
/*  KBOverrideItem Constructor for configuration dialog parameter item	*/
/*  listView	: RKListView *	  : Parent list view			*/
/*  path	: const QString & : Linked component path		*/
/*  name	: const QString & : Targetconfiguration name		*/
/*  value	: const QString & : Default value			*/
/*  enabled	: bool		  : Item is enabled			*/
/*  embed	: QWidget *	  : Widget into which to embed		*/
/*  (returns)	: KBOverrideItem  :					*/

KBOverrideItem::KBOverrideItem
	(	RKListView	*listView,
		KBObject	*parent,
		const QString	&path,
		const QString	&attrib,
		const QString	&value,
		bool		enabled,
		QWidget		*embed
	)
	:
	QListViewItem
	(	listView,
		path,
		attrib,
		value,
		enabled ? TR("Yes") : TR("No")
	)
{
	KBNode	*node		;

	m_attr	   = 0		;
	m_attrDlg  = 0		;
	m_value	   = value	;
	m_enabled  = enabled	;

	if ((node = parent->getNamedNode (path)) == 0)
		return	;

	if ((m_attr = node->getAttr(attrib)) == 0)
		return	;

	QDict<KBAttrItem> dummy  ;
	if ((m_attrDlg  = m_attr->getAttrDlg (embed, 0, dummy)) != 0)
	{
		KBDialog::setupLayout	(m_attrDlg->topWidget()) ;
		m_attrDlg->init		(m_value  ) ;
	}

	update	()	;
}

/*  KBOverrideItem							*/
/*  KBOverrideItem~: Destructor for parameter dialog parameter item	*/
/*  (returns)	:		:					*/

KBOverrideItem::~KBOverrideItem ()
{
}

void	KBOverrideItem::update ()
{
	if (m_attr != 0)
		setText (2, m_attr->displayValue(m_value)) ;
	else	setText (2, m_value) ;

	setText (3, m_enabled ? TR("Yes") : TR("No")) ;
}


/*  KBOverrideItem							*/
/*  save	: Save updated value					*/
/*  textEdit	: KBTextEdit *	: Used when no special dialog		*/
/*  (returns)	: void		:					*/

void	KBOverrideItem::save
	(	KBTextEdit	*textEdit
	)
{
	if (m_attrDlg != 0)
	{	m_attrDlg->verify() ;
		m_value = m_attrDlg->value() ;
	}
	else	m_value = textEdit ->text () ;

	update	() ;
}

/*  KBOverrideItem							*/
/*  setValue	: Set to specified value				*/
/*  value	: const QString & : Specified value			*/
/*  (returns)	: void		  :					*/

void	KBOverrideItem::setValue
	(	const QString	&value
	)
{
	m_value	= value	;
	if (m_attrDlg != 0) m_attrDlg->init (m_value) ;
	update	()	;
}

void	KBOverrideItem::setEnabled
	(	bool		e
	)
{
	m_enabled = e	;
	update ()	;
}

/*  KBOverrideItem								*/
/*  getOverride	: Get override object					*/
/*  node	: KBNode *	: Parent node				*/
/*  ident	: uint		: Identifier value			*/
/*  (returns)	: KBOverride *	:					*/

KBOverride
	*KBOverrideItem::getOverride
	(	KBNode		*node,
		uint		ident
	)
{
	return	new KBOverride
		(	node,
			QString("%1.%2").arg(time(0)).arg(ident),
			text(0),
			text(1),
			m_value,
			m_enabled
		)	;
}


/*  ------------------------------------------------------------------  */

/*  KBOverrideDlg								*/
/*  KBOverrideDlg							*/
/*		: Constructor for override object dialog		*/
/*  parent	: QWidget *	: Parent widget				*/
/*  object	: KBObject *	: Owning object				*/
/*  (returns)	: KBOverrideDlg	:					*/

KBOverrideDlg::KBOverrideDlg
	(	QWidget		*parent,
		KBObject	*object
	)
	:
	RKHBox		(parent),
	m_object	(object)
{
	m_stack		= new QWidgetStack    (this) ;
	RKVBox		*layButt = new RKVBox (this) ;
	m_bEdit		= new RKPushButton (TR("Edit"),   layButt) ;
	m_bSave		= new RKPushButton (TR("Save"),   layButt) ;
	m_bCancel	= new RKPushButton (TR("Cancel"), layButt) ;
	m_bToggle	= new RKPushButton (TR(""),	  layButt) ;
	layButt->addFiller () ;

	m_lvOverride	= new RKListView   (m_stack) ;
	m_textEdit	= new KBTextEdit   (m_stack) ;

	m_lvOverride->addColumn (TR("Path"     ), 150) ;
	m_lvOverride->addColumn (TR("Property" ),  80) ;
	m_lvOverride->addColumn (TR("Value"    ), 200) ;
	m_lvOverride->addColumn (TR("Enabled"  ),  60) ;

	m_stack->raiseWidget(m_lvOverride) ;

	connect	(m_bEdit,	SIGNAL(clicked()), SLOT(clickEdit  ())) ;
	connect	(m_bSave,	SIGNAL(clicked()), SLOT(clickSave  ())) ;
	connect	(m_bCancel,	SIGNAL(clicked()), SLOT(clickCancel())) ;
	connect	(m_bToggle,	SIGNAL(clicked()), SLOT(clickToggle())) ;

	connect (m_lvOverride,	SIGNAL(selectionChanged(QListViewItem *)),
				SLOT  (selectionChanged(QListViewItem *))) ;

	connect (m_lvOverride,	SIGNAL(doubleClicked   (QListViewItem *)),
				SLOT  (clickEdit       ())) ;
	connect (m_lvOverride,	SIGNAL(returnPressed   (QListViewItem *)),
				SLOT  (clickEdit       ())) ;

	m_curItem	= 0	;
	m_editItem	= 0	;
	m_adding	= false	;
	m_attrDlg	= 0	;

	TITER
	(	Override,
		m_object->getChildren(),
		o,

		new KBOverrideItem
		(	m_lvOverride,
			m_object,
			o->path   (),
			o->attrib (),
			o->value  (),
			o->enabled(),
			m_stack
		)	;
	)

	m_bEdit  ->setEnabled (false) ;
	m_bSave  ->setEnabled (false) ;
	m_bCancel->setEnabled (false) ;
	m_bToggle->setEnabled (false) ;
}


/*  KBOverrideDlg							*/
/*  ~KBOverrideDlg							*/
/*		: Destructor for override object dialog			*/
/*  (returns)	:		:					*/

KBOverrideDlg::~KBOverrideDlg ()
{
	DELOBJ	(m_editItem)	;
}


void	KBOverrideDlg::startUp ()
{
	clickCancel () ;
	m_stack->raiseWidget(m_lvOverride) ;
}

/*  KBOverrideDlg							*/
/*  getText	: Get text to display in properties dialog		*/
/*  (returns)	: QString	: Text					*/

QString	KBOverrideDlg::getText ()
{
	QString		res	;
	KBOverrideItem	*item	= (KBOverrideItem *)m_lvOverride->firstChild() ;

	while (item != 0)
	{
		if (!res.isEmpty()) res += ", " ;

		res += item->path () + ":" + item->attrib () ;
		item = (KBOverrideItem *)item->nextSibling() ;
	}

	if (m_editItem != 0)
	{
		if (!res.isEmpty()) res += ", " ;
		res += m_editItem->path() + ":" + m_editItem->attrib() ;
	}

	return	res	;
}

/*  KBOverrideDlg							*/
/*  fixUp	: Fix up on acceptance					*/
/*  (returns)	: void		:					*/

void	KBOverrideDlg::fixUp ()
{
	fprintf
	(	stderr,
		"KBOverrideDlg::fixUp: called ....\n"
	)	;

	saveCurrent () ;

	QPtrList<KBOverride>   old	 ;
	old.setAutoDelete (true) ;

	TITER
	(	Override,
		m_object->getChildren(),
		o,
		old.append(o)
	)

	old.clear () ;

	QListViewItem	*item	= m_lvOverride->firstChild() ;
	uint		ident	= 0x000ffffff & (unsigned long)(void *)item ;

	while (item != 0)
	{
		((KBOverrideItem *)item)->getOverride (m_object, ident) ;
		item   = item->nextSibling()  ;
		ident += 1 ;
	}
}

/*  KBOverrideDlg							*/
/*  getAttrDlg	: Get specialised attribute dialog, if any		*/
/*  over	: KBOverrideItem * : Item to be edited			*/
/*  (returns)	: DlgRC		   : Result				*/

KBOverrideDlg::DlgRC
	KBOverrideDlg::getAttrDlg
	(	KBOverrideItem	*over
	)
{
	/* See if the item to be edited provides a specialised dialog	*/
	/* in which case use that.					*/
	if ((m_attrDlg = over->getAttrDlg()) != 0)
	{
		m_stack->raiseWidget (m_attrDlg->topWidget()) ;
		return	Dialog	;
	}

	/* Next check for special cases that are handled via assorted	*/
	/* modal dialogs.						*/
	QString	attrib	= over->attrib() ;

	if ((attrib == "fgcolor") || (attrib == "bgcolor"))
	{
		TKColorDialog cDialog (this, TR("Colour"), true) ;
		cDialog.setColor (QColor (over->value().toInt(0,0))) ;

		if (cDialog.exec ())
		{
			QString	rv ;
			rv.sprintf ("0x%06x", cDialog.color().rgb() & 0x00ffffff) ;
			over->setValue (rv) ;
			return	SyncOK	;
		}

		return	SyncFail ;
	}

	if (attrib == "font")
	{
		TKFontDialog  fDialog (this, TR("Font"), false, true) ;
		fDialog.setFont (KBFont::specToFont (over->value())) ;

		if (fDialog.exec())
		{
			over->setValue (KBFont::fontToSpec (fDialog.font())) ;
			return	SyncOK ;
		}

		return	SyncFail ;
	}

	/* No dialog available so return. The called will use the	*/
	/* default text editor dialog instead.				*/
	return	None	;
}

/*  KBOverrideDlg							*/
/*  saveCurrent	: Save currently edited item if any			*/
/*  (returns)	: void		:					*/

void	KBOverrideDlg::saveCurrent ()
{
	if (m_editItem != 0)
	{
		m_editItem->save	(m_textEdit) ;
		m_editItem = 0		;
		m_adding   = false	;
	}
}

/*  KBOverrideDlg							*/
/*  clickSave	: User clicks save button				*/
/*  (returns)	: void		:					*/

void	KBOverrideDlg::clickSave ()
{
	if (m_editItem != 0)
	{
		m_curItem = m_editItem	;
		m_lvOverride->setCurrentItem (m_curItem) ;
		m_curItem->setEnabled (true) ;

		saveCurrent ()	;
	}

	clickCancel ()	;
}

/*  KBOverrideDlg							*/
/*  clickCancel	: User clicks canel button				*/
/*  (returns)	: void		:					*/

void	KBOverrideDlg::clickCancel ()
{
	if (m_editItem != 0)
	{	if (m_adding) delete m_editItem ;
		m_editItem = 0 ;
	}

	m_bCancel->setEnabled (false) ;
	m_bEdit  ->setEnabled (m_curItem != 0) ;
	m_bSave  ->setEnabled (false) ;

	m_stack  ->raiseWidget(m_lvOverride) ;
}

/*  KBOverrideDlg								*/
/*  clickEdit	: User clicks edit button				*/
/*  (returns)	: void		:					*/

void	KBOverrideDlg::clickEdit ()
{
	if (m_curItem == 0)
		return ;

	switch (getAttrDlg (m_curItem))
	{
		case Dialog	:
			break	;

		case SyncOK	:
			m_curItem->setEnabled (true) ;
			return	;

		case SyncFail	:
			return	;

		default	:
			m_textEdit->setText    (m_curItem->value()) ;
			m_stack->raiseWidget   (m_textEdit) ;
			break	;
	}

	m_adding    = false	;
	m_editItem  = m_curItem	;
	m_curItem   = 0		;

	m_bCancel->setEnabled (true ) ;
	m_bEdit  ->setEnabled (false) ;
	m_bSave  ->setEnabled (true ) ;
}

void	KBOverrideDlg::clickToggle ()
{
	if (m_curItem != 0)
	{
		m_curItem->setEnabled (!m_curItem->enabled()) ;
		m_bToggle->setText    ( m_curItem->enabled()  ? TR("Disable") : TR("Enable")) ;
	}
}

/*  KBOverrideDlg							*/
/*  selectionChanged							*/
/*		: Current item selection changes			*/
/*  item	: QListViewItem * : Newly selected item			*/
/*  (returns)	: void		  :					*/

void	KBOverrideDlg::selectionChanged
	(	QListViewItem	*item
	)
{
	m_curItem = (KBOverrideItem *)item ;
	m_bEdit   ->setEnabled  (item != 0) ;
	m_bToggle ->setEnabled  (item != 0) ;
	if (item != 0)
		m_bToggle->setText (m_curItem->enabled() ? TR("Disable") : TR("Enable")) ;
}
