/***************************************************************************
    file	         : kb_paramdlg.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qcheckbox.h>
#include	<qlabel.h>


#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_listview.h"

#if		__KB_AUTOMOC
#include	"kb_paramdlg.moc"
#else
#include 	"kb_paramdlg.h"
#endif

#include	"kb_node.h"
#include	"kb_param.h"
#include	"kb_helper.h"

#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"


/*  KBParamItem								*/
/*  -----------								*/
/*  This is a wrapper round a list view item, used to store a pointer	*/
/*  at an associated KBParam object.					*/

class	LIBKBASE_API	KBParamItem : public QListViewItem
{
	KBParam		*m_param	;
	QString		m_format	;
	bool		m_prompt	;

public	:

	KBParamItem (RKListView *, KBParam       *) ;
	KBParamItem (RKListView *, const QString &, const QString &, const QString &, const QString &, bool) ;
        virtual ~KBParamItem () ;

	inline	KBParam 	*getParam	() { return m_param 	; }
	inline	QString		getFormat	() { return m_format	; }
	inline	bool		getPrompt	() { return m_prompt	; }

	void			fixUp		(KBNode *) ;
}	;

/*  ------------------------------------------------------------------  */

/*  KBParamItem								*/
/*  KBParamItem	: Constructor for parameter dialog parameter item	*/
/*  listView	: RKListView *	: Parent list view			*/
/*  param	: KBParam *	: Extant parameter			*/
/*  (returns)	: KBParamItem	:					*/

KBParamItem::KBParamItem
	(	RKListView	*listView,
		KBParam		*param
	)
	:
	QListViewItem
	(	listView,
		param->getParam (),
		param->getDefval(),
		param->getLegend()
	),
	m_param		(param)
{
	m_format	= param->getFormat() ;
	m_prompt	= param->getPrompt() ;
}

/*  KBParamItem								*/
/*  KBParamItem	: Constructor for parameter dialog parameter item	*/
/*  listView	: RKListView *	  : Parent list view			*/
/*  name	: const QString & : Parameter name			*/
/*  defval	: const QString & : Default value			*/
/*  legend	: const QString & : Prompt legend			*/
/*  format	: const QString & : Format setting			*/
/*  prompt	: bool		  : Prompt always			*/
/*  (returns)	: KBParamItem	  :					*/

KBParamItem::KBParamItem
	(	RKListView		*listView,
		const QString		&name,
		const QString		&defval,
		const QString		&legend,
		const QString		&format,
		bool			prompt
	)
	:
	QListViewItem	(listView, name, defval, legend),
	m_param		(0)
{
	m_format	= format	;
	m_prompt	= prompt	;
}

/*  KBParamItem								*/
/*  KBParamItem~: Destructor for parameter dialog parameter item	*/
/*  (returns)	:		:					*/

KBParamItem::~KBParamItem ()
{
}

/*  KBParamItem								*/
/*  fixup	: Fix up value on accept				*/
/*  node	: KBNode *	: Parent node				*/
/*  (returns)	: void		:					*/

void	KBParamItem::fixUp
	(	KBNode	*node
	)
{
	if (m_param != 0) delete m_param ;
	m_param = new KBParam (node, text(0), text(1), text(2), m_format, m_prompt) ;
}


/*  ------------------------------------------------------------------  */

/*  KBParamDlg								*/
/*  KBParamDlg	: Constructor for parameter dialog			*/
/*  parent	: QWidget *	   : Parent widget			*/
/*  node	: KBNode *	   : Owning node			*/
/*  params	: QPtrList<KBParam> & : List of parameter nodes		*/
/*  (returns)	: KBParamDlg	   :					*/

KBParamDlg::KBParamDlg
	(	QWidget		*parent,
		KBNode		*node,
		QPtrList<KBParam>	&params
	)
	:
	RKVBox		(parent),
	m_node		(node)
{
	init	() ;

	LITER
	(	KBParam,
		params,
		p,
		new KBParamItem (m_lvParams, p)
	)
}

/*  KBParamDlg								*/
/*  KBParamDlg	: Constructor for parameter dialog			*/
/*  parent	: QWidget *	   	: Parent widget			*/
/*  params	: QDict<KBParamSet> &	: List of parameter nodes	*/
/*  (returns)	: KBParamDlg	   	:				*/

KBParamDlg::KBParamDlg
	(	QWidget			*parent,
		const QDict<KBParamSet>	&params
	)
	:
	RKVBox		(parent),
	m_node		(0)
{
	init	() ;

	QDictIterator<KBParamSet> iter (params) ;
	const KBParamSet *param ;

	while ((param = iter.current()) != 0)
	{
		new KBParamItem
		(	m_lvParams,
			iter.currentKey(),
			param->m_defval,
			param->m_legend,
			param->m_format,
			param->m_prompt
		)	;

		iter += 1 ;
	}
}

/*  KBParamDlg								*/
/*  init	: Initialise						*/
/*  (returns)	: void		:					*/

void	KBParamDlg::init ()
{
	RKGridBox	*laySett = new RKGridBox(3, this) ;

	new QLabel	(TR("Name"	), laySett) ;
	m_eName		= new RKLineEdit  (laySett) ;
	m_bAdd		= new RKPushButton(TR("Add"   ), laySett) ;

	new QLabel	(TR("Default"	), laySett) ;
	m_eDefval	= new RKLineEdit  (laySett) ;
	m_bRemove	= new RKPushButton(TR("Remove"), laySett) ;

	new QLabel	(TR("Legend"	), laySett) ;
	m_eLegend	= new RKLineEdit  (laySett) ;
	m_bEdit		= new RKPushButton(TR("Edit"  ), laySett) ;

	new QLabel	(TR("Format"	), laySett) ;
	RKHBox *lf	= new RKHBox	  (laySett, "_") ;
	m_ePrompt	= new QCheckBox	  (TR("Prompt"), laySett) ;

	m_eFormat	= new RKLineEdit  (lf) ;
	m_bFormat	= new RKPushButton(TR("..."), lf) ;

	m_bRemove   ->setEnabled	(false	  ) ;
	m_bEdit     ->setEnabled	(false	  ) ;
	m_eFormat   ->setEnabled	(false	  ) ;

	m_lvParams	= new RKListView(this) ;

	m_lvParams  ->addColumn	(TR("Name"   )) ;
	m_lvParams  ->addColumn	(TR("Default")) ;
	m_lvParams  ->addColumn	(TR("Legend" )) ;
	m_lvParams  ->setColumnWidthMode (0, QListView::Maximum) ;
	m_lvParams  ->setColumnWidthMode (1, QListView::Maximum) ;
	m_lvParams  ->setColumnWidthMode (2, QListView::Maximum) ;
	m_lvParams  ->setResizeMode      (QListView::LastColumn) ;

	m_bFormat->setFixedSize (m_bFormat->sizeHint()) ;

	KBDialog::setupLayout   (this) ;

	connect	(m_bAdd,     SIGNAL(clicked()), SLOT(clickAdd   ())) ;
	connect	(m_bRemove,  SIGNAL(clicked()), SLOT(clickRemove())) ;
	connect	(m_bEdit,    SIGNAL(clicked()), SLOT(clickEdit  ())) ;
	connect	(m_bFormat,  SIGNAL(clicked()), SLOT(clickFormat())) ;

	connect
	(	m_lvParams,
		SIGNAL(selectionChanged	(QListViewItem *)),
		SLOT  (selectionChanged	(QListViewItem *))
	)	;
	connect
	(	m_lvParams,
		SIGNAL(doubleClicked	(QListViewItem *)),
		SLOT  (clickEdit	())
	)	;
	connect
	(	m_lvParams,
		SIGNAL(returnPressed	(QListViewItem *)),
		SLOT  (clickEdit	())
	)	;

	m_curItem = 0 ;
}

/*  KBParamDlg								*/
/*  getText	: Get text to display in properties dialog		*/
/*  (returns)	: QString	: Text					*/

QString	KBParamDlg::getText ()
{
	QString	res	;
	QListViewItem	*item	= m_lvParams->firstChild() ;

	while (item != 0)
	{
		if (!res.isEmpty()) res += ", " ;

		res += item->text       (0) ;
		item = item->nextSibling()  ;
	}

	return	res	;
}

/*  KBParamDlg								*/
/*  getValues	: Get result values as strings				*/
/*  params	: QDict<KBParamSet> &	: Result set			*/
/*  (returns)	: void			:				*/

void	KBParamDlg::getValues
	(	QDict<KBParamSet>	&params
	)
{
	params.clear ()	;

	KBParamItem *item = (KBParamItem *)m_lvParams->firstChild() ;
	while (item != 0)
	{
		params.insert
		(	item->text(0),
			new KBParamSet
			(	item->text	(2),
				item->text	(1),
				QString::null,
				item->getFormat	(),
				item->getPrompt	()
			)
		)	;
		item = (KBParamItem *)item->nextSibling()  ;
	}
}

/*  KBParamDlg								*/
/*  fixUp	: Fix up on acceptance					*/
/*  (returns)	: void		:					*/

void	KBParamDlg::fixUp ()
{
	LITER (KBParam, m_dropped, p, delete p) ;

	QListViewItem	*item	= m_lvParams->firstChild() ;
	while (item != 0)
	{
		((KBParamItem *)item)->fixUp (m_node) ;
		item = item->nextSibling()  ;
	}
}

/*  KBParamDlg								*/
/*  clickAdd	: User clicks add button				*/
/*  (returns)	: void		:					*/

void	KBParamDlg::clickAdd ()
{
	if (!m_eName->text().isEmpty())
	{
		KBParamItem *item = new KBParamItem
				    (	m_lvParams,
					m_eName  ->text(),
					m_eDefval->text(),
					m_eLegend->text(),
					m_eFormat->text(),
					m_ePrompt->isChecked()
				    ) 	;
		m_lvParams  ->setCurrentItem (item) ;
		m_eName     ->clear () ;
		m_eDefval   ->clear () ;
		m_eLegend   ->clear () ;
		m_eFormat   ->clear () ;
		m_ePrompt   ->setChecked (false) ;
	}
}

/*  KBParamDlg								*/
/*  clickFormat	: User clicks format button				*/
/*  (returns)	: void		:					*/

void	KBParamDlg::clickFormat ()
{
	QString	result	= KBHelperDlg::run
			  (	"format",
				m_eFormat->text(),
				KBLocation()
			  )	;

	if (!result.isNull()) m_eFormat->setText (result) ;
}

/*  KBParamDlg								*/
/*  clickRemove	: User clicks remove button				*/
/*  (returns)	: void		:					*/

void	KBParamDlg::clickRemove ()
{
	if (m_curItem != 0)
	{
		KBParam *param = m_curItem->getParam() ;
		if (param != 0) m_dropped.append (param) ;
		DELOBJ	(m_curItem) ;
	}
}

/*  KBParamDlg								*/
/*  clickEdit	: User clicks edit button				*/
/*  (returns)	: void		:					*/

void	KBParamDlg::clickEdit ()
{
	if (m_curItem != 0)
	{
		QString	format	= m_curItem->getFormat() ;
		bool	prompt	= m_curItem->getPrompt() ;

		m_eName  ->setText 	(m_curItem->text (0)) ;
		m_eDefval->setText 	(m_curItem->text (1)) ;
		m_eLegend->setText	(m_curItem->text (2)) ;
		m_eFormat->setText	(format) ;
		m_ePrompt->setChecked	(prompt) ;

		KBParam *param = m_curItem->getParam  () ;
		if (param != 0) m_dropped.append (param) ;
		DELOBJ	(m_curItem) ;
	}
}

/*  KBParamDlg								*/
/*  selectionChanged							*/
/*		: Current item selection changes			*/
/*  item	: QListViewItem * : Newly selected item			*/
/*  (returns)	: void		  :					*/

void	KBParamDlg::selectionChanged
	(	QListViewItem	*item
	)
{
	m_curItem = (KBParamItem *)item	    ;
	m_bRemove ->setEnabled  (item != 0) ;
	m_bEdit   ->setEnabled  (item != 0) ;
}


/*  ------------------------------------------------------------------  */

/*  KBParamDefDlg							*/
/*  KBParamDefDlg: Parameter setup dialog				*/
/*  params	 : QDict<KBParamSet> &	: Parameter setup dictionary	*/
/*  (returns)	 : KBParamDefDlg	:				*/

KBParamDesignDlg::KBParamDesignDlg
	(	const QDict<KBParamSet>	&params
	)
	:
	KBDialog	("Set Parameters", true)
{
	RKVBox	*layMain = new RKVBox	 (this) ;
	layMain->setTracking () ;

	m_paramDlg	 = new KBParamDlg(layMain, params) ;
	addOKCancel (layMain);
}

