/***************************************************************************
    file	         : kb_pluginaction.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_dom.h"
#include	"tk_messagebox.h"

#if		__KB_AUTOMOC
#include	"kb_pluginaction.moc"
#else
#include	"kb_pluginaction.h"
#endif


#define		__DEBUG_PLUGINACTION	(0)
#if		__DEBUG_PLUGINACTION
#include	"kb_dprintf.h"
#endif


/*  KBPluginAction							*/
/*  KBPluginAction							*/
/*		: Constructor for simple action base class		*/
/*  text	: const QString & : Menu text				*/
/*  iconName	: const QString & : Menu or toolbar icon name		*/
/*  accel	: int		  : Accelerator code			*/
/*  receiver	: const QObject * : Receiver object for action signals	*/
/*  element	: QDomElement &	  : Specification DOM element		*/
/*  parent	: QObject *	  : Parent object			*/
/*  name	: const char *	  : Object name				*/
/*  (returns)	: KBPluginAction  :					*/

KBPluginAction::KBPluginAction
	(	const QString		&text,
		const QString		&iconName,
		int			accel,
		const QObject		*receiver,
		const QDomElement	&element,
		QObject			*parent,
		const char 		*name
	)
	:
	TKAction (text, iconName, accel, 0, 0, parent, name)
{
	m_name		= element.attribute ("plugin") ;

	connect
	(	this,
		SIGNAL	(sigPluginAction (const QString &, bool &)),
		receiver,
		SLOT	(slotPluginAction(const QString &, bool &))
	)	;

	DPRINTF
	((	"KBPluginAction::KBPluginAction: %s: created\n",
		m_name.latin1()
	))	;
}



/*  KBPluginAction							*/
/*  slotActivated: Handle action activation				*/
/*  (returns)	 : void		:					*/

void	KBPluginAction::slotActivated ()
{
	DPRINTF
	((	"KBPluginAction::slotActivated: %s: invoked\n",
		m_name.latin1()
	))	;

	bool	found	= false ;
	emit	sigPluginAction (m_name, found) ;

	if (!found)
	{
		TKMessageBox::sorry
		(	0,
			TR
			(	"The plugin for this action cannot be found\n"
				"The action is being disabled"
			)
		)	;

		setEnabled (false) ;
	}
}
