/***************************************************************************
    file	         : kb_popupmenu.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qlabel.h>

#include	"kb_popupmenu.h"
#include	"kb_object.h"


/*  KBPopupMenu								*/
/*  KBPopupMenu	: Constructor for extended popup menu			*/
/*  lvState	: Qt::ButtonSate * : Return location for button state	*/
/*  (returns)	: KBPopupMenu	   :					*/

KBPopupMenu::KBPopupMenu
	(	QWidget		*parent,
		Qt::ButtonState	*lvState
	)
	:
	QPopupMenu	(parent),
	m_lvState	(lvState)
{
	m_subMenus.setAutoDelete (true) ;
}

KBPopupMenu::KBPopupMenu
	(	KBPopupMenu	*parent
	)
	:
	QPopupMenu	(parent),
	m_lvState	(parent == 0 ? 0 : parent->m_lvState)
{
	m_subMenus.setAutoDelete (true) ;
}

/*  KBPopupMenu								*/
/*  ~KBPopupMenu: Desstructor for extended popup menu			*/
/*  (returns)	: void		:					*/

KBPopupMenu::~KBPopupMenu ()
{

}

/*  KBPopupMenu								*/
/*  insertEntry	: Insert a submenu					*/
/*  disable	: bool		: Disabled				*/
/*  text	: const QString	& : Label text				*/
/*  subMenu	: KBPopupMenu *	: Submenu				*/
/*  (returns)	: void		:					*/

void	KBPopupMenu::insertEntry
	(	bool		disable,
		const QString	&text,
		KBPopupMenu	*menu
	)
{
	int id = QPopupMenu::insertItem (text, menu) ;
	if (disable) setItemEnabled (id, false) ;
	m_subMenus.append (menu) ;
}

/*  KBPopupMenu								*/
/*  insertRenty	: Insert a submenu					*/
/*  disable	: bool		  : Disabled				*/
/*  text	: const QString	& : Label text				*/
/*  receiver	: const QObject * : Signal received object		*/
/*  member	: const char *	  : Receiver slot			*/
/*  (returns)	: void		  :					*/

void	KBPopupMenu::insertEntry
	(	bool		disable,
		const QString	&text,
		const QObject	*receiver,
		const char	*member
	)
{
	int id = QPopupMenu::insertItem (text, receiver, member) ;
	if (disable) setItemEnabled (id, false) ;
}

/*  KBPopupMenu								*/
/*  insertEntry	: Insert a submenu					*/
/*  disable	: bool		  : Disabled				*/
/*  pixmap	: const QPixmap & : Icon				*/
/*  text	: const QString	& : Label text				*/
/*  subMenu	: KBPopupMenu *	  : Submenu				*/
/*  (returns)	: void		  :					*/

void	KBPopupMenu::insertEntry
	(	bool		disable,
		const QPixmap	&pixmap,
		const QString	&text,
		KBPopupMenu	*menu
	)
{
	int id = QPopupMenu::insertItem (pixmap, text, menu) ;
	if (disable) setItemEnabled (id, false) ;
	m_subMenus.append (menu) ;
}

/*  KBPopupMenu								*/
/*  insertRenty	: Insert a submenu					*/
/*  disable	: bool		  : Disabled				*/
/*  pixmap	: const QPixmap & : Icon				*/
/*  text	: QString	  : Label text				*/
/*  receiver	: const QObject * : Signal received object		*/
/*  member	: const char *	  : Receiver slot			*/
/*  (returns)	: void		  :					*/

void	KBPopupMenu::insertEntry
	(	bool		disable,
		const QPixmap	&pixmap,
		const QString	&text,
		const QObject	*receiver,
		const char	*member
	)
{
	int id = QPopupMenu::insertItem (pixmap, text, receiver, member) ;
	if (disable) setItemEnabled (id, false) ;
}

/*  KBPopupMenu								*/
/*  setTitle	: Set menu title element				*/
/*  title	: const QString & : Title text				*/
/*  (returns)	: void		  :					*/

void	KBPopupMenu::setTitle
	(	const QString	&title
	)
{
	QLabel	*l = new QLabel(title, this) ;

	static	QFont	*lfont	;
	if (lfont == 0)
	{
		lfont = new QFont (l->font()) ;
		lfont->setWeight  (QFont::Bold) ;
	}

	l->setFont	(*lfont) ;
	l->setLineWidth (2) ;
	l->setFrameStyle(QFrame::Panel|QFrame::Raised) ;

	insertItem	(l) ;

	m_title	= title	;
}

/*  KBPopupMenu								*/
/*  setTitle	: Set menu title element				*/
/*  object	: KBObject *	: Object to which menu applies		*/
/*  (returns)	: void		:					*/

void	KBPopupMenu::setTitle
	(	KBObject	*object
	)
{
//	setTitle (object->getElement().mid(2) + ": " + object->getName()) ;
	setTitle (object->contextMenuType() + ": " + object->getName()) ;
}

const QString &KBPopupMenu::title ()
{
	return	m_title	;
}

/*  KBPopupMenu								*/
/*  mouseReleaseEvent							*/
/*		: Handle mouse release					*/
/*  q		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBPopupMenu::mouseReleaseEvent
	(	QMouseEvent	*e
	)
{
	if (m_lvState != 0) *m_lvState = e->state () ;
	QPopupMenu::mouseReleaseEvent (e) ;
}


