/***************************************************************************
    file	         : kb_popupprompt.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qlabel.h>
#include	<qfontmetrics.h>

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_lineedit.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"
#include	"rk_modalfilter.h"
#include	"tk_icons.h"

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_object.h"
#include	"kb_slot.h"
#include	"kb_script.h"
#include	"kb_dialog.h"

#if		__KB_AUTOMOC
#include	"kb_popupprompt.moc"
#else
#include	"kb_popupprompt.h"
#endif


KBPopupBase::KBPopupBase
	(	KBObject		*target,
		const QString		&slotName,
		const QString		&caption
	)
	:
	QWidget
	(	0,
		"KBPopupBase",
		WStyle_Customize|WStyle_Title|WStyle_NormalBorder|WStyle_StaysOnTop|Qt::WDestructiveClose
	),
	m_target(target)
{
	m_slot	 = 0	;
	const QPtrList<KBSlot> &slist = target->getSlots() ;
	LITER
	(	KBSlot,
		slist,
		slot,
		if (slot->name() == slotName)
		{	m_slot	 = slot ;
			break	 ;
		}
	)

	if (m_slot   == 0)
		KBError::EError
		(	QString(TR("Slot %1 not found")).arg(slotName),
			QString::null,
			__ERRLOCN
		)	;

	RKModalFilter::self()->push (this) ;
	m_modal	= true	;

	connect
	(	m_target,
		SIGNAL	(destroyed()),
		SLOT	(reject	  ())
	)	;

	setIcon		(getSmallIcon ("rekall")) ;
	setCaption	(caption) ;
}

/*  KBPopupBase								*/
/*  ~KBPopupBase: Destructor for popup prompt				*/
/*  (returns)	: void		:					*/

KBPopupBase::~KBPopupBase ()
{
	if (m_modal)
		RKModalFilter::self()->pop () ;
}

/*  KBPopupBase								*/
/*  ok		: Check helper is good to go				*/
/*  (returns)	: bool		: True if OK				*/

bool	KBPopupBase::ok ()
{
	return	m_slot != 0 ;
}

void	KBPopupBase::reply
	(	cchar		*name,
		const QString	&value
	)
{
	if (m_slot != 0)
	{
		KBScriptError	*rc	= 0 ;
		KBValue		resval	;
		KBValue		arg	(value) ;

		m_slot->eventSignal (m_target, name, 1, &arg, resval, rc, 0) ;

		if (rc != 0) 
			KBScriptError::processError (rc, KBScriptError::Normal) ;
	}
}

void	KBPopupBase::addOKCancel
	(	RKBox		*layout
	)
{
	RKHBox	*buttons = new RKHBox (layout) ;
	buttons->addFiller () ;
	RKPushButton *o  = new RKPushButton (TR("OK"),     buttons, "ok"    ) ;
	RKPushButton *c  = new RKPushButton (TR("Cancel"), buttons, "cancel") ;

	connect (o, SIGNAL(clicked()), SLOT(accept())) ;
	connect (c, SIGNAL(clicked()), SLOT(reject())) ;

	KBDialog::sameSize	(o, c, 0) ;
}

/*  KBPopupBase								*/
/*  reject	: Handle cancel button click				*/
/*  (returns)	: void		:					*/

void	KBPopupBase::reject ()
{
	unmodal	     () ;
	hide	     () ;
	deleteLater  () ;
}

/*  KBPopupBase								*/
/*  unmodal	: Exit from pseudo-modal state				*/
/*  (returns)	: void		:					*/

void	KBPopupBase::unmodal ()
{
	RKModalFilter::self()->pop () ;
	m_modal	= false	;
}

/*  ------------------------------------------------------------------	*/

/*  KBPopupPrompt							*/
/*  KBPopupPrompt							*/
/*		: Constructor for popup prompt				*/
/*  caption	: const QString & : Dialog caption			*/
/*  message	: const QString & : Dialog prompt message		*/
/*  value	: const QString & : Initial value			*/
/*  target	: KBObject *	  : Object to notify on success		*/
/*  slotName	: const QString & : Target slot to invoke		*/
/*  (returns)	: KBPopupPrompt   :					*/

KBPopupPrompt::KBPopupPrompt
	(	const QString		&caption,
		const QString		&message,
		const QString		&value,
		KBObject		*target,
		const QString		&slotName
	)
	:
	KBPopupBase (target, slotName, caption)
{
	RKVBox	*layMain	= new RKVBox (this) ;
	layMain->setTracking () ;

	new QLabel  (message, layMain) ;

	m_tValue = new RKLineEdit (layMain) ;
	m_tValue->setText (value) ;

	addOKCancel (layMain) ;

	KBDialog::setupLayout	(this) ;

	m_tValue->setFocus	() ;
	m_tValue->setSelection	(0, value.length()) ;

	setMinimumSize (285, 110) ;

	int	wc	= QFontMetrics(font()).width(caption) ;
	int	wm	= QFontMetrics(font()).width(message) ;

	if ((wc > 200) || (wm > 200))
		m_tValue->setMinimumWidth (QMAX(wc,wm) + 100) ;
}

/*  KBPopupPrompt							*/
/*  ~KBPopupPrompt							*/
/*		: Constructor for popup prompt				*/
/*  (returns)	: KBPopupPrompt	:					*/

KBPopupPrompt::~KBPopupPrompt ()
{
}

/*  KBPopupPrompt							*/
/*  accept	: Handle OK button click				*/
/*  (returns)	: void		:					*/

void	KBPopupPrompt::accept ()
{
	unmodal		() ;
	hide		() ;
	reply		("prompt", m_tValue->text()) ;
	deleteLater	() ;
}

/*  ------------------------------------------------------------------	*/

/*  KBPopupChoice							*/
/*  KBPopupChoice							*/
/*		: Constructor for popup prompt				*/
/*  caption	: const QString & : Dialog caption			*/
/*  message	: const QString & : Dialog prompt message		*/
/*  values	: const QStringList &					*/
/*				  : Initial value			*/
/*  target	: KBObject *	  : Object to notify on success		*/
/*  slotName	: const QString & : Target slot to invoke		*/
/*  (returns)	: KBPopupChoice   :					*/

KBPopupChoice::KBPopupChoice
	(	const QString		&caption,
		const QString		&message,
		const QStringList	&values,
		KBObject		*target,
		const QString		&slotName
	)
	:
	KBPopupBase (target, slotName, caption)
{
	RKVBox	*layMain	= new RKVBox (this) ;
	layMain->setTracking () ;

	new QLabel  (message, layMain) ;

	m_tChoice = new RKComboBox (layMain) ;
	m_tChoice->insertStringList(values ) ;

	addOKCancel (layMain) ;

	KBDialog::setupLayout	(this) ;

	setMinimumSize (285, 110) ;

	int	wc	= QFontMetrics(font()).width(caption) ;
	int	wm	= QFontMetrics(font()).width(message) ;

	if ((wc > 200) || (wm > 200))
		m_tChoice->setMinimumWidth (QMAX(wc,wm) + 100) ;
}

/*  KBPopupChoice							*/
/*  ~KBPopupChoice							*/
/*		: Constructor for choice prompt				*/
/*  (returns)	: KBPopupChoice	:					*/

KBPopupChoice::~KBPopupChoice ()
{
}

/*  KBPopupChoice							*/
/*  accept	: Handle OK button click				*/
/*  (returns)	: void		:					*/

void	KBPopupChoice::accept ()
{
	unmodal		() ;
	hide		() ;
	reply		("prompt", m_tChoice->currentText()) ;
	deleteLater	() ;
}

