/***************************************************************************
    file	         : kb_progress.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	<qapplication.h>
#include	<qfont.h>
#include	<qfontmetrics.h>
#include	<qlabel.h>

#include	"rk_vbox.h"
#include	"rk_hbox.h"

#include	"kb_classes.h"
#include	"kb_fixedlabel.h"
#include	"rk_modalfilter.h"

#if		__KB_AUTOMOC
#include	"kb_progress.moc"
#else
#include	"kb_progress.h"
#endif


#define		__DEBUG_PROGRESS	(0)
#if		__DEBUG_PROGRESS
#include	"kb_dprintf.h"
#endif


/*  KBProgress								*/
/*  KBProgress	: Constructor for progress base class			*/
/*  (returns)	: KBProgress	:					*/

KBProgress::KBProgress ()
{
	m_totalVal  = 0		;
	m_doneVal   = 0		;
	m_cancelled = false	;
}

/*  KBProgress								*/
/*  ~KBProgress	: Destructor for progress base class			*/
/*  (returns)	:		:					*/

KBProgress::~KBProgress ()
{
}

/*  KBProgress								*/
/*  setTotal	: Set total (target) value				*/
/*  total	: uint		: Total					*/
/*  (returns)	: void		:					*/

void	KBProgress::setTotal
	(	uint		total
	)
{
	m_totalVal	= total	;
	
}

/*  KBProgress								*/
/*  setDone	: Set done (progress) value				*/
/*  done	: uint		: Progress				*/
/*  (returns)	: void		:					*/

void	KBProgress::setDone
	(	uint		done
	)
{
	m_doneVal	= done	;
	
}

/*  KBProgress								*/
/*  cancelled	: See if user has cancelled operation			*/
/*  (returns)	: bool		: True if so				*/

bool	KBProgress::cancelled ()
{
	return	m_cancelled ;
}

/*  KBProgress								*/
/*  start	: Start operation					*/
/*  (returns)	: void		:					*/

void	KBProgress::start ()
{
	m_totalVal  = 0		;
	m_doneVal   = 0		;
	m_cancelled = false	;
}

/*  KBProgress								*/
/*  stop	: Stop operation					*/
/*  (returns)	: void		:					*/

void	KBProgress::stop ()
{
	m_totalVal  = 0		;
	m_doneVal   = 0		;
	m_cancelled = false	;
}

/*  ------------------------------------------------------------------	*/

/*  KBProgressDlg							*/
/*  KBProgressDlg							*/
/*		: Constructor for progress dialog			*/
/*  caption	: const QString & : Dialog caption			*/
/*  legend	: const QString & : Legend text				*/
/*  of		: const QString & : Text for "of" label			*/
/*  showTotal	: bool		  : Show total field			*/
/*  showat	: uint		  : Count at which to show		*/
/*  (returns)	: RKProgress	  :					*/

KBProgressDlg::KBProgressDlg
	(	const QString	&caption,
		const QString	&legend,
		const QString	&of,
		bool		showTotal,
		uint		showat
	)
	:
	RKDialog	(0, "RKProgress", true),
	m_showat	(showat)
{
	RKVBox	*layMain	= new RKVBox (this)	;
	layMain->setTracking ()	;

	RKHBox	*layProg	= new RKHBox (layMain)	;
	RKHBox	*layButt	= new RKHBox (layMain)	;

	m_legend	= new QLabel 	(layProg)	;
	m_count		= new RKLineEdit(layProg)	;
	m_of		= new QLabel 	(layProg)	;
	m_total		= new RKLineEdit(layProg)	;

	layButt->addFiller() ;
	m_bCancel	= new RKPushButton (TR("Cancel"), layButt) ;

	m_legend ->setText  (legend) ;
	m_of	 ->setText  (of	   ) ;

	if (of.isEmpty()) m_of   ->hide () ;
	if (!showTotal  ) m_total->hide () ;

	int	width	= QFontMetrics(QFont()).width("000000") ;
	m_count->setFixedWidth (width) ;
	m_of   ->setFixedWidth (width) ;

	m_count->setReadOnly   (true ) ;
	m_total->setReadOnly   (true ) ;

	setCaption (caption) ;

	connect	(m_bCancel,  SIGNAL(clicked()), SLOT(clickCancel())) ;
	connect (&m_timer,   SIGNAL(timeout()), SLOT(slotTimer  ())) ;

	m_period    = 0		;
	m_lastVal   = 0x7ffffff	;
}

/*  KBProgressDlg							*/
/*  KBProgressDlg							*/
/*		: Destructor for progress dialog			*/
/*  (returns)	: 		  :					*/

KBProgressDlg::~KBProgressDlg ()
{
}

/*  KBProgressDlg								*/
/*  clickCancel	: User clicks cancel					*/
/*  (returns)	: void		:					*/

void	KBProgressDlg::clickCancel ()
{
	m_cancelled = true ;
}

/*  KBProgressDlg								*/
/*  slotTimer	: Timer event						*/
/*  (returns)	: void		:					*/

void	KBProgressDlg::slotTimer ()
{
	/* If the done value has changed then update the display with	*/
	/* the new value.						*/
	if (m_lastVal != m_doneVal)
		m_count->setText (QString("%1").arg(m_lastVal = m_doneVal)) ;
}

/*  KBProgressDlg								*/
/*  setTotal	: Set the total value					*/
/*  total	: uint		: Value					*/
/*  (returns)	: void		:					*/

void	KBProgressDlg::setTotal
	(	uint	total
	)
{
	KBProgress::setTotal (total) ;
	m_total->setText (QString("%1").arg(total)) ;
}

/*  KBProgressDlg								*/
/*  setDone	: Set the count done					*/
/*  done	: uint		: Count					*/
/*  (returns)	: void		:					*/

void	KBProgressDlg::setDone
	(	uint	done
	)
{
	KBProgress::setDone (done) ;

	if (m_period > 0)
	{
		/* If the timer has not already been started then do	*/
		/* so now. This avoids unneccessary timer events until	*/
		/* the called actually uses the dialog.			*/
		if (!m_timer.isActive()) m_timer.start (m_period) ;
	}
	else
	{
		m_count->setText (QString("%1").arg(m_lastVal = m_doneVal)) ;
	}

	if ((done > m_showat) && !isVisible()) show () ;

	/* Drive the event loop to process any outstanding events,	*/
	/* which may include our own timer events.			*/
	qApp->processEvents() ;
}

/*  ------------------------------------------------------------------	*/

/*  KBProgressBox							*/
/*  KBProgressBox							*/
/*		: Constructor for progress box				*/
/*  parent	: QWidget *	  : Parent widget			*/
/*  legend	: const QString & : Legend text				*/
/*  of		: const QString & : Text for "of" label			*/
/*  showTotal	: bool		  : Show total field			*/
/*  showat	: uint		  : Count at which to show		*/
/*  (returns)	: RKProgress	  :					*/

KBProgressBox::KBProgressBox
	(	QWidget		*parent,
		const QString	&legend,
		const QString	&of,
		bool		showTotal
	)
	:
	RKHBox	(parent)
{
	m_legend	= new KBFixedLabel	(legend, this)	;
	m_count		= new KBFixedLabel	(6,      this)	;
	m_of		= new KBFixedLabel	(of,	 this)	;
	m_total		= new KBFixedLabel	(6,	 this)	;
	m_bCancel	= new RKPushButton	(TR("Cancel"), this) ;

	m_legendText	= legend;
	m_ofText	= of	;

	m_legend ->clear      () ;
	m_of	 ->clear      () ;

	if (of.isEmpty()) m_of   ->hide () ;
	if (!showTotal  ) m_total->hide () ;

	connect	(m_bCancel,  SIGNAL(clicked()), SLOT(clickCancel())) ;
	connect (&m_timer,   SIGNAL(timeout()), SLOT(slotTimer  ())) ;

	m_period    = 0		;
	m_lastVal   = 0x7ffffff	;
	m_started   = false	;

	m_bCancel->setEnabled	  (false ) ;
	m_bCancel->setFixedHeight (QFontMetrics(QFont()).lineSpacing()) ;
	setFixedWidth (sizeHint().width()) ;

	DPRINTF
	((	"KBProgressBox::KBProgressBox: created: [%p]\n",
		(void *)this
	))	;
}

/*  KBProgressBox							*/
/*  KBProgressBox							*/
/*		: Destructor for progress dialog			*/
/*  (returns)	: 		  :					*/

KBProgressBox::~KBProgressBox ()
{
	if (m_started) stop() ;
}

/*  KBProgressBox							*/
/*  clickCancel	: User clicks cancel					*/
/*  (returns)	: void		:					*/

void	KBProgressBox::clickCancel ()
{
	m_cancelled = true ;
}

/*  KBProgressBox							*/
/*  slotTimer	: Timer event						*/
/*  (returns)	: void		:					*/

void	KBProgressBox::slotTimer ()
{
	/* If the done value has changed then update the display with	*/
	/* the new value.						*/
	if (m_lastVal != m_doneVal)
		m_count->setText (QString("%1").arg(m_lastVal = m_doneVal)) ;
}

/*  KBProgressBox							*/
/*  setTotal	: Set the total value					*/
/*  total	: uint		: Value					*/
/*  (returns)	: void		:					*/

void	KBProgressBox::setTotal
	(	uint	total
	)
{
	KBProgress::setTotal (total) ;
	m_total->setText (QString("%1").arg(total)) ;
}

/*  KBProgressBox							*/
/*  setDone	: Set the count done					*/
/*  done	: uint		: Count					*/
/*  (returns)	: void		:					*/

void	KBProgressBox::setDone
	(	uint	done
	)
{
	KBProgress::setDone (done) ;

	if (m_period > 0)
	{
		/* If the timer has not already been started then do	*/
		/* so now. This avoids unneccessary timer events until	*/
		/* the called actually uses the dialog.			*/
		if (!m_timer.isActive()) m_timer.start (m_period) ;
	}
	else
	{
		m_count->setText (QString("%1").arg(m_lastVal = m_doneVal)) ;
	}

	/* Drive the event loop to process any outstanding events,	*/
	/* which may include our own timer events.			*/
	qApp->processEvents() ;
}

/*  KBProgressBox							*/
/*  start	: Start operation					*/
/*  (returns)	: void		:					*/

void	KBProgressBox::start ()
{
	KBProgress::start() ;

	m_legend ->setText    (m_legendText) ;
	m_of	 ->setText    (m_ofText	   ) ;
	m_bCancel->setEnabled (true) ;

	RKModalFilter::self()->push (this) ;
	m_started = true ;
}

/*  KBProgressBox							*/
/*  stop	: Finish operation					*/
/*  (returns)	: void		:					*/

void	KBProgressBox::stop ()
{
	RKModalFilter::self()->pop() ;
	m_started = false  ;

	KBProgress::stop() ;

	m_timer.stop    () ;
	m_count->clear  () ;
	m_total->clear  () ;

	m_legend ->clear      () ;
	m_of	 ->clear      () ;
	m_bCancel->setEnabled (false) ;
}
