/***************************************************************************
    file	         : kb_progress.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#ifndef	__KB_PROGRESS_H
#define	__KB_PROGRESS_H


#include	<qtimer.h>

#include 	"libkbase_exports.h"
#include	"rk_hbox.h"
#include	"rk_dialog.h"
#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"

class	QLabel		;
class	KBFixedLabel	;

/*  KBProgress								*/
/*  ----------								*/
/*  This is a base class for progress displays				*/

class	LIBKBASE_API	KBProgress
{
protected :

	uint		m_doneVal	;
	uint		m_totalVal	;
	bool		m_cancelled	;

public	:

	KBProgress		()	;
	virtual	~KBProgress	()	;

	virtual	void	start		()	;
	virtual	void	stop		()	;
	virtual	bool	cancelled	()	;
	virtual	void	setTotal	(uint)	;
	virtual	void	setDone		(uint)	;

	inline	uint	getTotal  ()
	{
		return	m_totalVal	;
	}

	inline	uint	getDone	  ()
	{
		return	m_doneVal	;
	}

}	;

/*  KBProgressDlg							*/
/*  -------------							*/
/*  The intent of this is like the QProgress dialog, but for our use	*/
/*  where we may not know in advance how many operations are to be done	*/
/*  (for instance, when importing from a CSV file).			*/

class	LIBKBASE_API	KBProgressDlg : public RKDialog, public KBProgress
{
	Q_OBJECT

	QLabel		*m_legend	;
	RKLineEdit	*m_count	;
	QLabel		*m_of		;
	RKLineEdit	*m_total	;
	RKPushButton	*m_bCancel	;

	QTimer		m_timer		;

	uint		m_lastVal	;
	uint		m_showat	;
	bool		m_period	;

protected slots :

	void		clickCancel	() ;
	void		slotTimer	() ;

public	:

	KBProgressDlg (const QString &, const QString &, const QString &, bool = true, uint = 200) ;
	virtual	~KBProgressDlg	() ;

	virtual	void	setTotal	(uint)	;
	virtual	void	setDone		(uint)	;

	inline	void	setPeriod
		(	uint	period
		)
	{
		m_period = period	;
	}
}	;

/*  KBProgressBox							*/
/*  -------------							*/
/*  This is like the KBProgressDlg class above, but in this case it	*/
/*  is a single widget that will fit into a status bar.			*/

class	LIBKBASE_API	KBProgressBox : public RKHBox, public KBProgress
{
	Q_OBJECT

	KBFixedLabel	*m_legend	;
	KBFixedLabel	*m_count	;
	KBFixedLabel	*m_of		;
	KBFixedLabel	*m_total	;
	RKPushButton	*m_bCancel	;

	QString		m_legendText	;
	QString		m_ofText	;

	QTimer		m_timer		;

	uint		m_lastVal	;
	bool		m_period	;
	bool		m_started	;

protected slots :

	void		clickCancel	() ;
	void		slotTimer	() ;

public	:

	KBProgressBox (QWidget *, const QString &, const QString &, bool = true) ;
	virtual	~KBProgressBox	() ;

	virtual	void	start		()	;
	virtual	void	stop		()	;
	virtual	void	setTotal	(uint)	;
	virtual	void	setDone		(uint)	;

	inline	void	setPeriod
		(	uint	period
		)
	{
		m_period = period	;
	}
}	;


#endif	// __TK_PROGRESS_H
