/***************************************************************************
    file	         : kb_prompt.h
    copyright            : (C) 1999,2000,
			       2001,2002,2003,2004,2005 by Mike Richardson
			   (C) 2001,2002,2003,2004,2005 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_PROMPTDLG
#define	_KB_PROMPTDLG


#include	"libkbase_exports.h"
#include	"kb_dialog.h"

#include	<qmessagebox.h>

class	RKLineEdit	;
class	RKComboBox	;
class	RKVBox		;
class	RKHBox		;
class	KBDBInfo	;


/*  KBMessageBoxYNAC							*/
/*  ----------------							*/
/*  Yes/No/All/Cancel prompt dialog					*/

class	LIBKBASE_API	KBMessageBoxYNAC : public KBDialog
{
	Q_OBJECT

	RKPushButton	*addButton
			(	RKHBox 		*,
				const QString	&,
				const QString	&,
				const char	*
			)	;

protected slots	:

	void	slotYes		()	;
	void	slotNo		()	;
	void	slotYesAll	()	;
	void	slotCancel	()	;

public	:

	enum	ActRC
	{	ActYes		= QMessageBox::Yes,
		ActNo		= QMessageBox::No,
		ActYesAll	= QMessageBox::YesAll,
		ActCancel	= QMessageBox::Cancel,
		ActSingle	= 0x00ffffff
	}	;

	KBMessageBoxYNAC
	(	QWidget		*parent,
		const QString	&text,
		const QString	&caption	= QString::null,
		const QString	&buttonYes	= QString::null,
		const QString	&buttonNo	= QString::null,
		const QString	&buttonYesAll	= QString::null,
		const QString	&buttonCancel	= QString::null
	)	;

	static	ActRC	query
		(	QWidget		*parent,
			const QString	&text,
			const QString	&caption	= QString::null,
			const QString	&buttonYes	= QString::null,
			const QString	&buttonNo	= QString::null,
			const QString	&buttonYesAll	= QString::null,
			const QString	&buttonCancel	= QString::null
		)	;

	static	ActRC	query
		(	QWidget		*parent,
			const QString	&text,
			ActRC		actRC,
			const QString	&caption	= QString::null,
			const QString	&buttonYes	= QString::null,
			const QString	&buttonNo	= QString::null,
			const QString	&buttonYesAll	= QString::null,
			const QString	&buttonCancel	= QString::null
		)	;
}	;

/*  ------------------------------------------------------------------	*/


/*  KBPromptDlg								*/
/*  -----------								*/
/*  Basic prompt dialog. Shows text field with possibile initial value.	*/
/*  Result is returned via the default argumement.			*/

class	LIBKBASE_API	KBPromptDlg : public KBDialog
{
	Q_OBJECT

protected :

	RKLineEdit	*m_tValue	;
	QString		&m_value	;

protected slots :

	virtual	void	accept	()	;

public	:

	KBPromptDlg (const QString &, const QString &, QString &) ;
       ~KBPromptDlg () ;
}	;

/*  KBPromptSaveDlg							*/
/*  ---------------							*/
/*  Basic save dialog, showing combo box of possible document		*/
/*  locations. Can be passed default name and location.			*/

class	LIBKBASE_API	KBPromptSaveDlg : public KBDialog
{
	Q_OBJECT

protected :

	RKVBox		*m_layout	;

	RKLineEdit	*m_tValue	;
	RKComboBox	*m_cWhere	;
	QString		&m_value	;
	QString		&m_where	;
	bool		m_incfiles	;

protected :

	inline	RKVBox	*mainLayout ()
	{
		return	m_layout	;
	}


protected slots :

	virtual	void	accept	()	;

public	:

	KBPromptSaveDlg (const QString &, const QString &, QString &, QString &, KBDBInfo *, bool) ;
       ~KBPromptSaveDlg () ;

	virtual	int	exec	()	;
}	;


/*  KBChoiceDlg								*/
/*  -----------								*/
/*  Simple choice dialog. Given a list of choices and an optional	*/
/*  default. Returns selected value via the default argument.		*/

class	LIBKBASE_API	KBChoiceDlg : public KBDialog
{
	Q_OBJECT

	RKComboBox	*m_cValues	;
	QString		&m_value	;

protected slots :

	virtual	void	accept () ;

public	:

	KBChoiceDlg (const QString &, const QString &, const QStringList &, QString &) ;
       ~KBChoiceDlg () ;
}	;

/*  KBPromptRegexpDlg							*/
/*  -----------------							*/
/*  Prompt for a regexp. Can be passed a value against which the regexp	*/
/*  can be checked.							*/

class	LIBKBASE_API	KBPromptRegexpDlg : public KBDialog
{
	Q_OBJECT

protected :

	RKLineEdit	*m_regexp	;
	QLabel		*m_status	;
	QString		m_sample	;
	QString		&m_value	;

protected slots :

	void		test	()	;
	virtual	void	accept	()	;

public	:

	KBPromptRegexpDlg	(const QString &, const QString &, const QString &, QString &) ;
       ~KBPromptRegexpDlg	() ;
}	;


/*  ------------------------------------------------------------------	*/

extern	bool	LIBKBASE_API	doPrompt
		(	const	QString	&caption,
			const	QString	&message,
			QString		&value
		)	;

extern	bool	LIBKBASE_API	doPromptSave
		(	const QString	&caption,
			const QString	&message,
			QString		&value,
			QString		&where,
			KBDBInfo	*dbInfo,
			bool		incfiles
		)	;


#endif	// _KB_PROMPTDLG
