/***************************************************************************
    file	         : kb_propdict.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<errno.h>

#include	<qdict.h>
#include	<qfile.h>
#include	<qdir.h>
#include	<qmessagebox.h>


#include	"kb_classes.h"
#include	"kb_propdict.h"
#include	"kb_locator.h"
#include	"kb_dom.h"

#if		__KB_KDE

#include	<klocale.h>
#include	<kglobal.h>

#endif		// __KB_KDE


#define		__DEBUG_PROPDICT	(0)
#if		__DEBUG_PROPDICT
#include	"kb_dprintf.h"
#endif

/*  KBPropDict								*/
/*  KBPropDict	: Dictionary constructor				*/
/*  prefix	: const QString & : Dictionary prefix			*/
/*  (returns)	: KBPropDict	  :					*/

KBPropDict::KBPropDict
	(	const QString	&prefix
	)
{
	QString	dPath	;
	QDir	dDir	;

	dPath	= locateDir ("appdata", "dict/kb_node.dict") ;
	dPath  += "/dict";

	DPRINTF
	((	"KBPropDict::KBPropDict: [%s]\n",
		dPath.latin1()
	))	;

	dDir.setPath	   (dPath) ;
	dDir.setNameFilter (prefix + "*.dict") ;
	dDir.setFilter     (QDir::Files|QDir::Readable) ;
	dDir.setSorting    (QDir::Name) ;

	const QFileInfoList   *info = dDir.entryInfoList() ;
	if (info == 0)
	{
		QMessageBox::warning(0, "Location Error", "No dictionary directory found!!\n");
		return	;
	}

	QFileInfoListIterator dIter (*info) ;
	QFileInfo *fi ;

#if	__KB_KDE
	KLocale	*locale	= KGlobal::locale () ;
	DPRINTF
	((	"KBPropDict::KBPropDict: language [%s]\n",
		locale->language().latin1()
	))	;
#endif

	while ((fi = dIter.current()) != 0)
	{
		QString	name	= fi->filePath() ; 
		loadFile (name)	;

#if	__KB_KDE
		QString	name2	= fi->dirPath () + "/" + locale->language() + "/" + fi->fileName() ;
		if (QFileInfo(name2).exists()) loadFile (name2) ;
#endif
		dIter += 1 ;
	}

	/* Final operation is to scan the dictionary and set unset	*/
	/* legends and descriptions to the entry tag. This means that	*/
	/* something shows in the property dialogs when the dictionary	*/
	/* is missing an entry.						*/
	QDictIterator<KBPropDictEntry> eIter(m_dict) ;
	KBPropDictEntry *entry ;

	while ((entry = eIter.current()) != 0)
	{
		if (entry->m_legend .isNull()) entry->m_legend  = eIter.currentKey() ;
		if (entry->m_descrip.isNull()) entry->m_descrip = eIter.currentKey() ;
		eIter += 1 ;
	}

	m_dict.setAutoDelete (true) ;
}

/*  KBPropDict								*/
/*  loadFile	: Load a dictionary file				*/
/*  name	: const QString & : File name				*/
/*  (returns)	: void		  :					*/

void	KBPropDict::loadFile
	(	const QString	&name
	)
{
	DPRINTF
	((	"KBPropDict::loadFile: [%s]\n",
		name.latin1()
	))	;

	QFile	dFile	(name) ;

	if (!dFile.open (IO_ReadOnly))
	{
		KBError::EError
		(	"Cannot open dictionary",
			QString("%1: %2").arg(name).arg(strerror(errno)),
			__ERRLOCN
		)	;
		return	;
	}

	QDomDocument	dXML	;
	if (!dXML.setContent (&dFile))
	{
		KBError::EError
		(	"Cannot parse dictionary",
			name,
			__ERRLOCN
		)	;
		return	;
	}

	QDomNodeList	dNodes	= dXML.elementsByTagName ("attribute") ;

	for (uint idx = 0 ; idx < dNodes.length() ; idx += 1)
	{
		QDomNode	dAttr	 = dNodes.item  (idx) ;
		QDomElement	dElement = dAttr.toElement () ;

		if (dElement.isNull()) continue ;

		QString		kElement = dElement.attribute ("element" ) ;
		QString		kName	 = dElement.attribute ("name"    ) ;
		QString		kLanguage= dElement.attribute ("language") ;

		if (kElement.isEmpty() || kName.isEmpty())
			continue ;

		if (!kLanguage.isEmpty() && (kLanguage != "*"))
			kName	= kName  + "." + kLanguage ;

		QString		kLegend	 ;
		QString		kDescrip ;
		QString		kNullchk ;

		/* See if the entry already exists, if not then create	*/
		/* it in the empty state.				*/
		QString		tag	= QString("%1_%2").arg(kElement).arg(kName) ;
		KBPropDictEntry	*entry	;

		if ((entry = m_dict.find (tag)) == 0)
		{
			entry = new KBPropDictEntry;
			m_dict.insert (tag, entry) ;
			entry->m_tag	= tag	   ;
		}

		/* Scan through the children of the dictionary		*/
		/* element, looking for cases that we know about.	*/
		for (QDomNode child = dAttr.firstChild() ; !child.isNull() ; child = child.nextSibling())
		{
			QDomElement e	= child.toElement() ;
			QString	    tag	= e    .tagName  () ;

			/* First three cases are the main ones, and	*/
			/* set the corresponding values in the		*/
			/* dictionary entry.				*/
			if (tag == "legend"     )
			{	entry->m_legend  = e.text() ;
				continue ;
			}
			if (tag == "description")
			{	entry->m_descrip = e.text() ;
				continue ;
			}
			if (tag == "nullcheck"  )
			{	entry->m_nullchk = e.text() ;
				continue ;
			}

			/* The other case is an "extra" value, which	*/
			/* is stored under a tag.			*/
			if (tag == "extra"	)
			{
				if (entry->m_extra == 0)
					entry->m_extra = new QDict<QString> ;

				entry->m_extra->insert
				(	e.attribute("tag"),
					new QString(e.text().stripWhiteSpace())
				)	;
				continue;
			}
		}
	}
}

KBPropDictEntry
	*KBPropDict::getEntry
	(	const QString	&element,
		const QString	&attrib
	)
{
	DPRINTF
	((	"KBPropDict::getEntry: e=[%s] a=[%s]\n",
		element.latin1(),
		attrib .latin1()
	))	;

	return	m_dict.find (QString("%1_%2").arg(element).arg(attrib)) ;
}

