/***************************************************************************
    file	         : kb_propdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qheader.h>
#include	<qcache.h>
#include	<qregexp.h>
#include	<qfile.h>
#include	<qtextstream.h>
#include	<qtextedit.h>
#include	<qtextview.h>
#include	<qspinbox.h>
#include	<qwidgetstack.h>


#if		__KB_AUTOMOC
#include	"kb_propdlg.moc"
#else 
#include 	"kb_propdlg.h"
#endif

#include	"kb_object.h"
#include	"kb_qrybase.h"
#include	"kb_font.h"
#include	"kb_script.h"
#include	"kb_docroot.h"
#include	"kb_slot.h"
#include	"kb_slotdlg.h"
#include	"kb_test.h"
#include	"kb_testdlg.h"
#include	"kb_options.h"
#include	"kb_toolbox.h"
#include	"kb_manual.h"

#include	"tk_colordialog.h"
#include	"tk_fontdialog.h"
#include	"tk_messagebox.h"
#include	"tk_config.h"

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"
#include	"rk_lineedit.h"

#include	"kb_textedit.h"
#include	"kb_sidepanel.h"

#define		__DEBUG_PROPDLG	(0)
#if		__DEBUG_PROPDLG
#include	"kb_dprintf.h"
#endif


#define	LWIDTH		(160)



class	LIBKBASE_API	KBAttrListViewItem : public QListViewItem
{
	QString		name		;
	QString		order		;

public	:

	KBAttrListViewItem
		(	QListViewItem	*qlv,
			QListViewItem	*after,
			QString		legend,
			QString		value,
			QString		name,
			uint		_order
		)
		:
		QListViewItem	(qlv, after, legend, value),
		name		(name)
	{
		order.sprintf	("%08x", _order) ;
	}

	inline	QString	getName ()
	{ 	return	name   ;
	}

	virtual	QString	key
		(	int	,
			bool
		)
		const
	{
		return order  ;
	}
			
}	;



static	IntChoice	choiceScaling[] =
{
{	0,			"No scaling"				},
{	1,			"Exact fit"				},
{	2,			"Fit preserving aspect ratio"		},
{	3,			"Fill preserving aspect ratio"		},
{	-1,			0					}
}	;


/*  ------------------------------------------------------------------  */

/*  Generally, help tags are constructed from the attributes owner's	*/
/*  class name and the attribute name. However, in a few cases there	*/
/*  is one help item for a number of attributes. The code below is a	*/
/*  bit of a hack to handle these cases ....				*/



/*  ------------------------------------------------------------------  */

/*  KBAttrSlots								*/
/*  -----------								*/
/*  Derived attribute item used to handle slots, so the list of slot	*/
/*  names is displayed.							*/

class	LIBKBASE_API KBAttrSlots : public KBAttrItem
{
	const QPtrList<KBSlot>	&m_slotList ;

public:

	KBAttrSlots (KBAttr *, const QPtrList<KBSlot> &) ;
	virtual ~KBAttrSlots() ;

	virtual	QString	displayValue()	;
}	;

/*  KBAttrSlots								*/
/*  KBAttrSlots	: Constructor for slots attribute item			*/
/*  attr	: KBAttr *	     : Actual attribute			*/
/*  slotList	: QPtrList<KBSlot> & : Slot list			*/
/*  (returns)	: KBAttrSlots	     :					*/

KBAttrSlots::KBAttrSlots
	(	KBAttr			*attr,
		const QPtrList<KBSlot>	&slotList
	)
	:
	KBAttrItem (attr),
	m_slotList (slotList)
{
}

/*  KBAttrSlots								*/
/*  ~KBAttrSlots: Destructor for slots attribute item			*/
/*  (returns)	:		:					*/

KBAttrSlots::~KBAttrSlots ()
{
}

/*  KBAttrSlots								*/
/*  displayValue: Generate dialog display value				*/
/*  (returns)	: QString	: Text to display			*/

QString	KBAttrSlots::displayValue()
{
	QString	text	;
	cchar	*sep	= "" ;

	LITER
	(	KBSlot,
		m_slotList,
		slot,

		text	+= sep	;
		text	+= slot->name () ;
		sep	 = ", "	;
	)

	return	text	;
}

/*  ------------------------------------------------------------------  */

/*  KBAttrTests								*/
/*  -----------								*/
/*  Derived attribute item used to handle tests, so the list of slot	*/
/*  names is displayed.							*/

class	LIBKBASE_API KBAttrTests : public KBAttrItem
{
	const QPtrList<KBTest>	&m_testList ;

public:

	KBAttrTests (KBAttr *, const QPtrList<KBTest> &) ;
	virtual ~KBAttrTests() ;

	virtual	QString	displayValue()	;
}	;

/*  KBAttrTests								*/
/*  KBAttrTests	: Constructor for tests attribute item			*/
/*  attr	: KBAttr *	     : Actual attribute			*/
/*  testList	: QPtrList<KBTest> & : Test list			*/
/*  (returns)	: KBAttrTests	     :					*/

KBAttrTests::KBAttrTests
	(	KBAttr			*attr,
		const QPtrList<KBTest>	&testList
	)
	:
	KBAttrItem (attr),
	m_testList (testList)
{
}

/*  KBAttrTests								*/
/*  ~KBAttrTests: Destructor for tests attribute item			*/
/*  (returns)	:		:					*/

KBAttrTests::~KBAttrTests ()
{
}

/*  KBAttrTests								*/
/*  displayValue: Generate dialog display value				*/
/*  (returns)	: QString	: Text to display			*/

QString	KBAttrTests::displayValue()
{
	QString	text	;
	cchar	*sep	= "" ;

	LITER
	(	KBTest,
		m_testList,
		test,

		text	+= sep	;
		text	+= test->getName () ;
		sep	 = ", "	;
	)

	return	text	;
}

/*  ------------------------------------------------------------------  */

/*  KBAttrItem								*/
/*  KBAttrItem	: Constructor for property dialog attribute item	*/
/*  attr	: KBAttr *	: Attribute				*/
/*  (returns)	: KBAttrItem	:					*/

KBAttrItem::KBAttrItem
	(	KBAttr	*attr
	)
	:
	m_attr	(attr),
	m_value	(attr->getValue())
{
}

/*  KBAttrItem								*/
/*  ~KBAttrItem	: Destructor for property dialog attribute item		*/
/*  (returns)	:		:					*/

KBAttrItem::~KBAttrItem ()
{
}

/*  KBAttrItem								*/
/*  displayValue: Get value for display in attribute list		*/
/*  (returns)	: QString	: Display text				*/

QString	KBAttrItem::displayValue ()
{
	return	m_attr->displayValue (m_value) ;
}

/*  KBAttrItem								*/
/*  save	: Save value back into attribute			*/
/*  (returns)	: void		:					*/

void	KBAttrItem::save ()
{
	m_attr->setValue (m_value) ;
}

/*  KBAttrItem								*/
/*  value	: Get value						*/
/*  (returns)	:		: Value					*/

QString &KBAttrItem::value ()
{
	return	m_value	;
}

/*  KBAttrItem								*/
/*  setValue	: Set value						*/
/*  value	: const QString & : Value to set			*/
/*  (returns)	: void		  :					*/

void	KBAttrItem::setValue
	(	const QString	&value
	)
{
	m_value	= value ;
}

/*  KBAttrItem								*/
/*  clear	: Clear attribute					*/
/*  (returns)	: void		  :					*/

void	KBAttrItem::clear ()
{
	m_value	= QString::null ;
}

/*  KBAttrItem								*/
/*  appendItem	: Append display listview item to set of items		*/
/*  lvi		: QListViewItem * : Item in question			*/
/*  (returns)	: void		  :					*/

void	KBAttrItem::appendItem
	(	QListViewItem	*lvi
	)
{
	m_lvitems.append (lvi) ;
}

/*  KBAttrItem								*/
/*  display	: Display current value					*/
/*  (returns)	: void		  :					*/

void	KBAttrItem::display ()
{
	LITER
	(	QListViewItem,
		m_lvitems,
		lvi,
		lvi->setText (1, displayValue()) ;
	)
}

/*  KBAttrItem								*/
/*  getAttrDlg	: Get attribute dialog for this item			*/
/*  parent	: QWidget *		: Parent widget			*/
/*  attrDict	: QDict<KBAttrItem> &	: Disctionary of all attributes	*/
/*  (returns)	: KBAttrDlg *		:				*/

KBAttrDlg
	*KBAttrItem::getAttrDlg
	(	QWidget			*parent,
		QDict<KBAttrItem>	&attrDict
	)
{
	/* Tricky code alert!						*/
	/* Normally, the attribute dialogs can be retured via a method	*/
	/* on the attribute, which is what this methiod defaults to.	*/
	/* But, by routing the request for the dialog through a virtual	*/
	/* method on the KBAttrItem class, we get the ability to pass	*/
	/* a specific derived item to the dialog.			*/
	return	m_attr->getAttrDlg (parent, this, attrDict) ;
}

QString	KBAttrItem::getDescription ()
{
	return	m_attr->getDescription () ;
}

/*  KBAttrItem								*/
/*  showChoices	: Show set of choices					*/
/*  choice	: IntChoice *	  : Set of choices			*/
/*  value	: const QString & : Current value			*/
/*  cbox	: RKComboBox *	  : Combo box to use			*/
/*  (returns)	: void		  :					*/

void	KBAttrItem::showChoices
	(	IntChoice	*choices,
		const QString	&value,
		RKComboBox	*cbox
	)
{
	/* This method is static ....					*/
	int	at = -1 ;
	cbox->show ()   ;

	while (choices->value >= 0)
	{
		cbox->insertItem (choices->descr) ;
		if (choices->value == value.toInt())
			at = cbox->count() - 1 ;
		choices += 1 ;
	}

	if (at >= 0) cbox->setCurrentItem (at) ;
}

/*  ------------------------------------------------------------------  */

/*  KBAttrIntChoice							*/
/*  KBAttrIntChoice							*/
/*		: Constructor for integer choice attribute item		*/
/*  attr	: KBAttr *	  : Actual attribute			*/
/*  choices	: IntChoice *	  : List of choices			*/
/*  defval	: int		  : Default value				*/
/*  (returns)	: KBAttrIntChoice :					*/

KBAttrIntChoice::KBAttrIntChoice
	(	KBAttr		*attr,
		IntChoice	*choices,
		int		defval
	)
	:
	KBAttrItem	(attr),
	choices		(choices),
	defval		(defval)
{
}

/*  KBAttrIntChoice							*/
/*  ~KBAttrIntChoice							*/
/*		: Destructor for integer choice attribute item		*/
/*  (returns)	:		:					*/

KBAttrIntChoice::~KBAttrIntChoice ()
{
}

/*  KBAttrIntChoice							*/
/*  displayValue: Get value for display in attribute list		*/
/*  (returns)	: QString	: Display text				*/

QString	KBAttrIntChoice::displayValue ()
{
	int	v = m_value.isEmpty() ? defval : m_value.toInt() ;

	for (IntChoice *c = choices ; c->value >= 0 ; c += 1)
		if (c->value == v)
			return	c->descr ;

	return	TR("#Error#") ;
}

/*  ------------------------------------------------------------------  */

struct GroupMap
{	cchar	*m_legend	;
	int	m_flags		;
	bool	m_open		;
}	;

static	GroupMap groupMap[] =
{
	{	"Display",	KAF_GRPFORMAT,	false	},
	{	"Data",		KAF_GRPDATA,	false	},
	{	"Events",	KAF_GRPEVENT,	false	},
	{	"Other",	KAF_GRPOTHER,	false	},
	{	"Web",		KAF_GRPWEB,	false	},
	{	"Notes",	KAF_GRPNOTES,	false	},
	{	"All",		0xffffffff,	false	},
	{	0,		0,		false	}
}	;

static	int	leftColSize	= LWIDTH ;

/*  ------------------------------------------------------------------  */

/*  KBPropDlg								*/
/*  KBPropDlg	: Constructor for control properties dialog		*/
/*  node	: KBNode *	: Parent node				*/
/*  caption	: cchar *	: Dialog box caption			*/
/*  attribs	: QPtrList<KBAttr>&: List of attributes			*/
/*  iniattr	: cchar *	: Initial attribute			*/
/*  (returns)	: KBPropDlg	:					*/

KBPropDlg::KBPropDlg
	(	KBNode			*node,
		cchar			*caption,
		QPtrList<KBAttr>	&attribs,
		cchar			*iniattr
	)
	:
	KBDialog 	(caption, true),
	m_node	  	(node),
	m_caption	(QString(TR("Properties: %1")).arg(caption)),
	attribs	  	(attribs),
	iniattr		(iniattr)
{
	RKVBox	*topLayout	= new RKVBox	  (this) ;
	topLayout->setTracking () ;

	RKHBox	*topBox		= new RKHBox	  (topLayout 	) ;

	new KBSidePanel (topBox, m_caption) ;

	m_propAllVal  		= new RKListView  (topBox	) ;
	m_propArea		= new RKVBox	  (topBox	) ;

	m_descrip		= new QTextView	  (m_propArea	) ;
	m_propStack		= new QWidgetStack(m_propArea	) ;

	m_propArea->setStretchFactor (m_propStack, 1) ;

	m_layCommon		= new RKVBox	  (m_propStack) ;

	m_textEdit		= new KBTextEdit  (m_layCommon) ;
	m_lineEdit		= new RKLineEdit  (m_layCommon) ;
	m_comboBox		= new RKComboBox  (m_layCommon) ;
	m_checkBox		= new QCheckBox	  (m_layCommon) ;
	m_spinBox		= new QSpinBox	  (m_layCommon) ;
	m_layCommon->addFiller	      ()  ;
	m_layCommon->setStretchFactor (m_textEdit, 1000) ;

	RKGridBox *layButt	= new RKGridBox	  (6, topLayout) ;

	new QWidget	(layButt) ;
	m_bEdit			= new RKPushButton(TR("&Edit"   ), layButt) ;
	m_bAccept		= new RKPushButton(TR("&Accept" ), layButt) ;
	new QWidget	(layButt) ;
	m_bUser			= new RKPushButton(		   layButt) ;
	m_bHelp			= new RKPushButton(TR("&Help"   ), layButt) ;

	new QWidget	(layButt) ;
	m_bClear		= new RKPushButton(TR("C&lear"  ), layButt) ;
	m_bIgnore		= new RKPushButton(TR("&Ignore" ), layButt) ;
	m_bVerify		= new RKPushButton(TR("&Verify" ), layButt) ;
	m_bOK	  		= new RKPushButton(TR("&OK"     ), layButt) ;
	m_bCancel		= new RKPushButton(TR("&Cancel" ), layButt) ;

	layButt->setColStretch (0, 1) ;


	QListViewItem *group = 0 ;
	for (GroupMap *gm = &groupMap[0] ; gm->m_legend != 0 ; gm += 1)
	{	group = new QListViewItem(m_propAllVal, group, TR(gm->m_legend)) ;
		m_groups.insert (gm->m_flags, group) ;
	}

	m_slotListDlg	= 0 ;
	m_testListDlg	= 0 ;
	m_configDlg	= 0 ;

	m_userWidget	= 0 ;
	m_attrDlg	= 0 ;

	setupListView (m_propAllVal, true) ;

	m_propArea ->hide () ;
	m_descrip  ->setFixedHeight (80) ;
	m_spinBox  ->setFixedWidth  (100) ;
	m_checkBox ->setText	    ("") ;

	topLayout  ->setStretchFactor (topBox,	    1) ;
	topLayout  ->setStretchFactor (layButt,     0) ;

	m_descrip  ->setTextFormat(Qt::RichText) ;
	m_textEdit ->setWordWrap  (RKTextEdit::NoWrap) ;

	m_textEdit ->hide () ;
	m_lineEdit ->hide () ;
	m_comboBox ->hide () ;
	m_checkBox ->hide () ;
	m_spinBox  ->hide () ;
	m_bUser	   ->hide () ;

	/* The accept and ignore buttons are only enabled when a	*/
	/* property is being edited, so they start disabled.		*/
	m_bAccept  ->setEnabled(false) ;
	m_bIgnore  ->setEnabled(false) ;

	m_bEdit    ->setEnabled(false) ;
	m_bClear   ->setEnabled(false) ;
	m_bVerify  ->setEnabled(false) ;
	m_bHelp    ->setEnabled(false) ;

#if	! __KB_KDE
//**	m_bHelp	   ->hide() ;
#endif

	connect (m_bOK,	     SIGNAL(clicked()),    SLOT(clickOK     ())) ;
	connect (m_bCancel,  SIGNAL(clicked()),    SLOT(clickCancel ())) ;
	connect (m_bAccept,  SIGNAL(clicked()),    SLOT(clickAccept ())) ;
	connect (m_bEdit,    SIGNAL(clicked()),    SLOT(clickEdit   ())) ;
	connect (m_bIgnore,  SIGNAL(clicked()),    SLOT(clickIgnore ())) ;
	connect (m_bClear,   SIGNAL(clicked()),    SLOT(clickClear  ())) ;
	connect (m_bVerify,  SIGNAL(clicked()),    SLOT(clickVerify ())) ;
	connect (m_bHelp,    SIGNAL(clicked()),    SLOT(clickHelp   ())) ;

	curAttrItem 	= 0 ;
	curListItem	= 0 ;
	clrAttrItem	= 0 ;

	attrByName  .setAutoDelete (true ) ;
	attrByLegend.setAutoDelete (false) ;

	iniRSize	= true	;

	TKConfig *config = TKConfig::getConfig() ;
	config->setGroup ("Property Editor") ;
	width	= config->readNumEntry ("width",  600) ;
	height	= config->readNumEntry ("height", 260) ;

	/* Install event filters on the listview. This is used to stop	*/
	/* the user changing the selection while a property is being	*/
	/* edited.							*/
	m_propAllVal->viewport()->installEventFilter (this) ;

	KBObject *object = node->isObject() ;
	if (object != 0)
	{
		const QPtrList<KBSlot> objSlots = object->getSlots() ;
		LITER
		(	KBSlot,
			objSlots,
			slot,
			m_slotList.append(new KBSlot(0, slot))
		)
		const QPtrList<KBTest> objTests = object->getTests() ;
		LITER
		(	KBTest,
			objTests,
			test,
			m_testList.append(new KBTest(0, test))
		)
	}

	m_configDlg  = new KBConfigDlg (m_propStack, node) ;
	m_configDlg->hide() ;

	resize	    (width, height) ;
}

/*  KBPropDlg								*/
/*  ~KBPropDlg	: Destructor for control properties dialog		*/
/*  (returns)	:		:					*/

KBPropDlg::~KBPropDlg ()
{
	TKConfig *config   = TKConfig::getConfig () ;
	config->setGroup   ("Property Editor") ;

	config->writeEntry ("width",  width  ) ;
	config->writeEntry ("height", height ) ;
	config->sync	   () ;

	for (GroupMap *gm = &groupMap[0] ; gm->m_legend != 0 ; gm += 1)
	{
		QListViewItem *item = m_groups.find (gm->m_flags) ;
		if (item != 0) gm->m_open = item->isOpen() ;
	}

	leftColSize = m_propAllVal->header()->sectionSize(0) ;
}

/*  KBPropDlg								*/
/*  eventFilter	: Event filter routine					*/
/*  o		: QObject *	: Object receiving event		*/
/*  e		: QEvent *	: The event				*/
/*  (returns)	: bool		: True to consume event			*/

bool	KBPropDlg::eventFilter
	(	QObject		*o,
		QEvent 		*e
	)
{
	if (o == m_propAllVal->viewport())
	{
		if (curListItem != 0)
		{
			switch (e->type())
			{
				case QEvent::MouseButtonPress	:
				case QEvent::MouseButtonRelease	:
				case QEvent::KeyPress		:
				case QEvent::KeyRelease		:
					return	true	;

				default	:
					break	;
			}

			return	false	;
		}

		return	false	;
	}

	return	false	;
}


/*  KBPropDlg								*/
/*  getUserButton: Get extra button for derived dialogs			*/
/*  text	 : const QString & : Button text			*/
/*  (returns)	 : void		   :					*/

RKPushButton
	*KBPropDlg::getUserButton
	(	const QString	&text
	)
{
	m_bUser->setText (text) ;
	m_bUser->show	 ()	;
	return	m_bUser	;
}

/*  KBPropDlg								*/
/*  setUserWidget: Set user supplied display widget			*/
/*  userWidget  : QWidget *	: Widget in question			*/
/*  (returns)	 : void		:					*/

void	KBPropDlg::setUserWidget
	(	QWidget		*userWidget
	)
{
	if ((m_userWidget = userWidget) != 0)
	{
		m_userWidget->show () ;
		m_propStack ->raiseWidget (userWidget) ;

		QSize	cs	= size()  ;
		QSize	sh	= size().expandedTo(minimumSizeHint()) ;
		if (sh != cs) resize (sh) ;
	}
}

/*  KBPropDlg								*/
/*  resizeEvent	: Handle dialog resizing				*/
/*  e		: QResizeEvent * : Resize event				*/
/*  (returns)	: void		 :					*/

void	KBPropDlg::resizeEvent
	(	QResizeEvent	*
	)
{
	/* Catch this to (a) note the size for when we close and (b)	*/
	/* to subvert the dialog box sizing.				*/
	QSize	s = size   () ;
	width	= s.width  () ;
	height	= s.height () ;
	setMinimumSize (600, 350) ;
}


/*  KBPropDlg								*/
/*  setupListView: Set up a attributes list view			*/
/*  lView	 : RKListView *	: List view				*/
/*  visible	 : bool		: True if list view is visible		*/
/*  (returns)	 : void		:					*/

void	KBPropDlg::setupListView
	(	RKListView	*lView,
		bool		visible
	)
{
	lView->header()->setResizeEnabled (true )	 ;
	lView->header()->setMovingEnabled (false)	 ;

	lView->setHScrollBarMode (RKListView::AlwaysOff )	;
	lView->setMinimumWidth	 (LWIDTH) ;
	lView->move		 (marginHint(), marginHint())	;
	lView->addColumn	 (TR("Attribute"), leftColSize)	;
	lView->addColumn	 (TR("Value"    ), 1500)	;
	lView->setRootIsDecorated(true) ;
	lView->setSorting	 (-1)   ;

	connect
	(	lView,
		SIGNAL(currentChanged(QListViewItem *)),
		SLOT  (setCurrent    (QListViewItem *))
	)	;
	connect
	(	lView,
		SIGNAL(doubleClicked (QListViewItem *)),
		SLOT  (pickProperty  (QListViewItem *))
	)	;

	if (visible)
		lView->show ()	  ;
	else	lView->hide ()	  ;
}

/*  KBPropDlg								*/
/*  preExec	: Pre-execute hook					*/
/*  (returns)	: void		:					*/

void	KBPropDlg::preExec ()
{
}

/*  KBPropDlg								*/
/*  exec	: Execute dialog					*/
/*  (returns)	: bool		: Success				*/

bool	KBPropDlg::exec ()
{
	/* We need to trap this method in order to load the attributes.	*/
	/* This cannot be done in the constructor, since the		*/
	/* "hideProperty" mechanism will not work at that stage.	*/
	LITER
	(	KBAttr,
		attribs,
		attr,

		if (!hideProperty (attr)) addAttrib (attr) ;
	)

	setProperty ("configs", m_configDlg->getText()) ;
	preExec	    ()	;


	if (!iniattr.isNull())
	{
		QListViewItem	*got	= 0	  ;
		QString		ii	= iniattr ;
		QString		sn	;
		QString		tn	;

		/* Initial attribute name may be like "slots:name"	*/
		/* where we want to open directly to a slot dialog...	*/
		if (ii.left(6) == "slots:")
		{	sn	= ii.mid (6) ;
			ii	= "slots" ;
		}
		/* ...and similarly for tests.				*/
		if (ii.left(6) == "tests:")
		{	sn	= ii.mid (6) ;
			ii	= "tests" ;
		}

		DPRINTF
		((	"KBPropDlg::exec: [%s] [%s]:[%s]:[%s]\n",
			iniattr.latin1(),
			ii     .latin1(),
			sn     .latin1(),
			tn     .latin1()
		))	;

		for (QListViewItem *g  = m_propAllVal->firstChild () ;
				    g != 0 ;
				    g  = g->nextSibling())
			for (QListViewItem *i  = g->firstChild () ;
					    i != 0 ;
					    i  = i->nextSibling())
				if (((KBAttrListViewItem *)i)->getName() == ii)
				{
					pickProperty (got = i)    ;
					goto gotit ;
				}

		gotit :

		/* If we have got the property and there is a slot	*/
		/* or test name then go there.				*/
		if (got != 0)
		{
			if (!sn.isEmpty())
			{
				m_slotListDlg = new KBSlotListDlg
						(	m_propStack,
							m_slotList,
							m_node->isObject()
						) ;

				setUserWidget (m_slotListDlg) ;
				m_slotListDlg->openSlot  (sn) ;
			}
			if (!tn.isEmpty())
			{
				m_testListDlg = new KBTestListDlg
						(	m_propStack,
							m_testList,
							m_node->isObject()
						) ;

				setUserWidget (m_testListDlg) ;
				m_testListDlg->openTest  (tn) ;
			}
		}
	}

	for (GroupMap *gm = &groupMap[0] ; gm->m_legend != 0 ; gm += 1)
	{
		QListViewItem *item = m_groups.find (gm->m_flags) ;

		if (item->firstChild() == 0)
		{
			delete	item	;
			m_groups.remove (gm->m_flags) ;
			continue ;
		}

		if (item != 0)
			item->setOpen(gm->m_open) ;
	}

	/* Now we really can run it ....				*/
	bool	suspend	= KBOptions::getSuspendToolbox() ;

	if (suspend) KBToolBox::self()->suspendToolBox() ;
	bool	rc = KBDialog::exec () ;
	if (suspend) KBToolBox::self()->resumeToolBox () ;
	return	rc ;
}

/*  KBPropDlg								*/
/*  getAttrItem	: Get attribute item for attribute			*/
/*  attr	: KBAttr *	: Attribute				*/
/*  (returns)	: KBAttrItem *	: Associated attribute item		*/

KBAttrItem *KBPropDlg::getAttrItem
	(	KBAttr	*attr
	)
{
	if (attr->getName() == "slots" )
		return	new KBAttrSlots (attr, m_slotList) ;

	if (attr->getName() == "tests" )
		return	new KBAttrTests (attr, m_testList) ;

	if (attr->getName() == "autosize")
		return	new KBAttrIntChoice (attr, choiceScaling) ;

	return	attr->getAttrItem () ;
}

/*  KBPropDlg								*/
/*  addAttrib	: Add an attribute to the property list			*/
/*  attr	: KBAttrItem *	: Attribute item			*/
/*  lView	: RKListView &	: List view into which to insert	*/
/*  (returns)	: void		:					*/

void	KBPropDlg::addAttrib
	(	KBAttrItem	*item,
		RKListView	*lView
	)
{
	int		gFlags	= item->attr()->getFlags() & KAF_GRPMASK ;
	QListViewItem	*group	;
	QListViewItem	*last	;

	if ((group = m_groups.find (gFlags)) == 0)
	{
		group = new QListViewItem (lView, QString().sprintf("Unknown %08x", gFlags)) ;
		m_groups.insert(gFlags, group) ;
	}

	last	= group->firstChild() ;
	while ((last != 0) && (last->nextSibling() != 0))
		last = last->nextSibling() ;

	item->appendItem
		(	new KBAttrListViewItem
			(	group,
				last,
				item->attr()->getLegend(),
				item->displayValue     (),
				item->attr()->getName  (),
				item->attr()->getOrder ()
			)
		) ;

	if ((group = m_groups.find (0xffffffff)) != 0)
	{
		last	= group->firstChild() ;
		while ((last != 0) && (last->nextSibling() != 0))
			last = last->nextSibling() ;

		item->appendItem
			(	new KBAttrListViewItem
				(	group,
					last,
					item->attr()->getLegend(),
					item->displayValue     (),
					item->attr()->getName  (),
					item->attr()->getOrder ()
				)
			) ;
	}
}

/*  KBPropDlg								*/
/*  addAttrib	: Add an attribute to the property list			*/
/*  attr	: KBAttr *	: Attribute in question			*/
/*  (returns)	: void		:					*/

void	KBPropDlg::addAttrib
	(	KBAttr	*attr
	)
{
	KBAttrItem *item ;

	if ((item = attrByLegend.find (attr->getLegend())) == 0)
	{	attrByName  .insert (attr->getName  (), item = getAttrItem (attr)) ;
		attrByLegend.insert (attr->getLegend(), item ) ;
	}

	addAttrib (item, m_propAllVal) ;
}

/*  KBPropDlg								*/
/*  warning	: Display a warning message				*/
/*  msg		: cchar *	: Message				*/
/*  (returns)	: bool		: Always false				*/

bool	KBPropDlg::warning
	(	cchar	*msg
	)
{
	TKMessageBox::sorry (0, msg, TR("Property error")) ;
	return	false  ;
}


/*  KBPropDlg								*/
/*  dropProperty: Drop currently displayed property			*/
/*  (returns)	: void		:					*/

void	KBPropDlg::dropProperty ()
{
	if (m_userWidget != 0)
	{
		m_userWidget->hide () ;
		m_userWidget = 0 ;
	}

	m_propArea  ->hide() ;
	m_propAllVal->setMinimumWidth(LWIDTH) ;
	m_propAllVal->setMaximumWidth(4000) ;
	m_propStack ->raiseWidget    (m_layCommon) ;

	m_descrip   ->setText	("")	;
	m_descrip   ->hide	()	;
	m_lineEdit  ->hide	()	;
	m_lineEdit  ->clear	()	;
	m_textEdit  ->hide	()	;
	m_textEdit  ->clear	()	;
	m_comboBox  ->hide	()	;
	m_comboBox  ->clear	()	;
	m_checkBox  ->hide	() 	;
	m_spinBox   ->hide	() 	;

	disconnect
	(	m_comboBox,
		SIGNAL (activated(const QString &)),
		this,
		SLOT   (pickCombo(const QString &))
	)	;

	m_bVerify->setEnabled (false) ;
}

/*  KBPropDlg								*/
/*  hideProperty: Check whether property should be hidden		*/
/*  attr	: KBAttr *	: Attribute in question			*/
/*  (returns)	: bool		: True to hide				*/

bool	KBPropDlg::hideProperty
	(	KBAttr		*attr
	)
{
	/* This method allows selective hiding of properties. The	*/
	/* default here is to query the attribute.			*/
	return	attr->isHidden () ;
}

/*  KBPropDlg								*/
/*  showChoices	: Display choice selection				*/
/*  attr	: KBAttr *	 : Attribute in question		*/
/*  choices	: IntChoice *	 : Set of choices			*/
/*  value	: const QString &: Current value			*/
/*  cbox	: RKComboBox *	 : Combo box to use or null		*/
/*  (returns)	: void		 :					*/

void	KBPropDlg::showChoices
	(	KBAttrItem	*,
		IntChoice	*choices,
		const QString	&value,
		RKComboBox	*cbox
	)
{
	if (cbox == 0) cbox = m_comboBox ;
	KBAttrItem::showChoices (choices, value, cbox) ;
}


/*  KBPropDlg								*/
/*  saveChoices	: Save choice selection					*/
/*  item	: KBAttrIte *	 : Associated item			*/
/*  choices	: IntChoice *	 : Set of choices			*/
/*  cbox	: RKComboBox *	 : Combo box to use or null		*/
/*  (returns)	: void		 :					*/

void	KBPropDlg::saveChoices
	(	KBAttrItem	*item,
		IntChoice	*choices,
		RKComboBox	*cbox
	)
{
	int	slot	= cbox == 0 ?
				m_comboBox->currentItem () :
				cbox     ->currentItem () ;

	if (slot >= 0)
		setProperty (item, QString("%1").arg(choices[slot].value)) ;
	else	setProperty (item, "") ;
}

/*  KBPropDlg								*/
/*  showProperty: Display property for update				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property shown OK			*/

bool	KBPropDlg::showProperty
	(	KBAttrItem	*item
	)
{
	QString		text	;
	KBAttr		*attr	= item->attr() ;
	const QString	&name	= attr->getName () ;

	DPRINTF
	((	"KBPropDlg::showProperty: for [%s]\n",
		(cchar *)name
	))	;

	if ((m_attrDlg = item->getAttrDlg (m_propStack, attrByName)) != 0)
	{
		DPRINTF
		((	"KBPropDlg::showProperty: purpose dialog\n"
		))	;

		if (m_attrDlg->init())
		{
			item->display ()  ;
			return false 	  ;
		}

		setupLayout 	(m_attrDlg->topWidget()) ;
		setUserWidget	(m_attrDlg->topWidget()) ;

		m_bVerify->setEnabled((attr->getFlags() & KAF_GRPEVENT) != 0) ;

		connect
		(	m_attrDlg,
			SIGNAL	(setDescription(const QString &)),
			SLOT	(setDescription(const QString &))
		)	;
		return	true	;
	}

	/* Foreground/Background colour					*/
	/* Handle the colour attributes specially, bringing up the	*/
	/* standard colour dialog box. Note that in this case we must	*/
	/* handle the Accept/Reject functionallity here.		*/
	if
	(	(name == "fgcolor"    ) ||
		(name == "bgcolor"    ) ||
		(name == "markfgcolor") ||
		(name == "markbgcolor")
	)
	{
		TKColorDialog cDialog (this, TR("Colour"), true) ;
		cDialog.setColor (QColor (item->value().toInt(0,0))) ;
		if (cDialog.exec ())
		{	
			resultVal.sprintf ("0x%06x", cDialog.color().rgb() & 0x00ffffff) ;
			clickAccept () ;
		}
		return	false	;
	}

	/* Font								*/
	/* Similarly for a font. We have to parse a font string		*/
	/* ouselves, since we don't really want to use X font strings.	*/
	if (name == "font")
	{
		TKFontDialog  fDialog (this, TR("Font"), false, true) ;
		fDialog.setFont (KBFont::specToFont (item->value())) ;

		if (fDialog.exec())
		{
			resultVal = KBFont::fontToSpec (fDialog.font()) ;
			clickAccept () ;
		}

		return	false	;
	}

#if	0
	/*  Xmode/YMode                                                 */
	/*  X- and Y- fixed/stretch/float modes ...                     */	
	if ((name == "xmode") || (name == "ymode"))
	{
		showChoices (item, choiceFMode, item->value()) ;
		return	true ;
	}
#endif
	/* Slots							*/
	/* This brings up a list of extant slots, plus actions like add	*/
	/* and edit. All the real work is in the separate slot dialog.	*/
	if (name == "slots")
	{
		if (m_slotListDlg == 0)
			m_slotListDlg = new KBSlotListDlg
					(	m_propStack,
						m_slotList,
						m_node->isObject()
					) ;

		setUserWidget (m_slotListDlg) ;
		return	true  ;
	}

	/* tests							*/
	/* This brings up a list of extant tests, plus actions like add	*/
	/* and edit.							*/
	if (name == "tests")
	{
		if (m_testListDlg == 0)
			m_testListDlg = new KBTestListDlg
					(	m_propStack,
						m_testList,
						m_node->isObject()
					) ;

		setUserWidget (m_testListDlg) ;
		return	true  ;
	}

	if (name == "configs")
	{
		setUserWidget (m_configDlg) ;
		return	true	 ;
	}

	if (name == "autosize")
	{
		showChoices (item, choiceScaling, item->value()) ;
		return	true ;
	}

	/* Next check for boolean attributes ....			*/
	if (attr->getType() == KBAttr::Bool)
	{
		DPRINTF
		((	"KBPropDlg::showProperty: .... bool\n"
		))	;

		m_checkBox->setChecked  (item->value() == "Yes") ;
		m_checkBox->setText     (item->attr()->getLegend ()) ;
		m_checkBox->show	() ;
		m_checkBox->setFocus    () ;
		return	true ;
	}

	/* ... and integers. Note that the spinbox range limits are	*/
	/* integers, hence use INT_MAX in the KBAttr::UInt case.	*/
	if (attr->getType() == KBAttr::Int)
	{
		DPRINTF
		((	"KBPropDlg::showProperty: .... int\n"
		))	;

		m_spinBox ->setRange    (INT_MIN, INT_MAX) ;
		m_spinBox ->setValue    (item->value().toInt()) ;
		m_spinBox ->show	() ;
		m_spinBox ->setFocus    () ;
		return	true ;
	}
	if (attr->getType() == KBAttr::UInt)
	{
		DPRINTF
		((	"KBPropDlg::showProperty: .... uint\n"
		))	;

		m_spinBox ->setRange    (0, INT_MAX) ;
		m_spinBox ->setValue    (item->value().toInt()) ;
		m_spinBox ->show	() ;
		m_spinBox ->setFocus    () ;
		return	true ;
	}

	if ((attr->getFlags() & KAF_EDITOR) != 0)
	{
		DPRINTF
		((	"KBPropDlg::showProperty: .... editor [%04x]\n",
			attr->getFlags()
		))	;

		m_textEdit ->setText	(item->value()) ;
		m_textEdit ->show	() ;
		m_textEdit ->setFocus	() ;
		return	 true	;
	}

	DPRINTF
	((	"KBPropDlg::showProperty: defaulting [%s]\n",
		(cchar *)name
	))	;

	/* Default case, show the edit box which will overlay the	*/
	/* property values list box, and set the current property	*/
	/* value therein.						*/
	m_lineEdit->show	() ;
	m_lineEdit->setText	(item->value()) ;
	m_lineEdit->setFocus	() ;

	return	 true	;
}

/*  KBPropDlg								*/
/*  setProperty	: Set a property value					*/
/*  item	: KBAttrItem *	 : Attribute in question		*/
/*  value	: const QString &: Value to set				*/
/*  (returns)	: bool		 : Set OK				*/

bool	KBPropDlg::setProperty
	(	KBAttrItem	*item,
		const QString	&value
	)
{
	if (item == 0)
		return	false	;

	if (!item->attr()->valueOK (value))
		return	warning	(QString(TR("%1 has an invalid value"))
					.arg(item->attr()->getLegend ())) ;

	item->setValue (value)  ;
	item->display  ()	;
	return	true		;
}

/*  KBPropDlg								*/
/*  setProperty	: Set a property value					*/
/*  attr	: cchar *	 : Attribute name			*/
/*  value	: const QString &: Value to set				*/
/*  (returns)	: bool		 : Set OK				*/

bool	KBPropDlg::setProperty
	(	cchar		*name,
		const QString	&value
	)
{
	return	setProperty (attrByName.find(name), value) ;
}

/*  KBPropDlg								*/
/*  getProperty	: Get current property value				*/
/*  name	: cchar *	 : Attribute name			*/
/*  (returns)	: const QString &: Property value			*/

const QString
	&KBPropDlg::getProperty
	(	cchar	*name
	)
{
	KBAttrItem *attr = attrByName.find(name) ;
	return	attr == 0 ? QString::null : attr->value() ;
}

/*  KBPropDlg								*/
/*  saveProperty: Check and save attribute				*/
/*  item	: KBAttrItem *	: Assoiated item			*/
/*  (returns)	: bool		: Attribute OK				*/

bool	KBPropDlg::saveProperty
	(	KBAttrItem	*item
	)
{
	KBAttr		*attr	= item->attr() ;
	const QString	&name	= attr->getName () ;

	if (m_attrDlg != 0)
	{
		m_attrDlg->save   ()  ;
		item     ->display()  ;
		setUserWidget     (0) ;
		DELOBJ	          (m_attrDlg) ;
		return true ;
	}

	/* Colour and font results are stored on the "resultVal" member	*/
	/* whence they are stored here.					*/
	if
	(	(name == "fgcolor"    ) ||
		(name == "bgcolor"    ) ||
		(name == "markfgcolor") ||
		(name == "markbgcolor") ||
		(name == "font"	      )
	)
	{
		setProperty  (item, resultVal) ;
		return	true ;
	}


	/* Helper							*/
	/* Value is in the combo box					*/
	if (name == "helper")
	{
		setProperty (item, m_comboBox->currentText()) ;
		return	true ;
	}


	if (name == "slots")
	{
		m_slotListDlg->save (m_slotList) ;
		setProperty ("slots", QString::null); 
		return	true	;
	}

	if (name == "tests")
	{
		m_testListDlg->save (m_testList) ;
		setProperty ("tests", QString::null); 
		return	true	;
	}

	if (name == "configs")
	{
		setProperty  ("configs", m_configDlg->getText()) ;
		return	true ;
	}

	if (name == "autosize")
	{
		saveChoices (item, choiceScaling) ;
		return	true	;
	}

	/* Now some more general cases. First, if the property is a	*/
	/* boolean then we will be using the checkbox, and the property	*/
	/* is set to "Yes" or "No".					*/
	if (attr->getType() == KBAttr::Bool)
	{
		setProperty (item, m_checkBox->isChecked() ? "Yes" : "No") ;
		return	true	;
	}

	/* Next the integer and unsigned integer properties ..		*/
	if (attr->getType() == KBAttr::Int)
	{
		setProperty (item, m_spinBox->cleanText()) ;
		return	true	;
	}
	if (attr->getType() == KBAttr::UInt)
	{
		setProperty (item, m_spinBox->cleanText()) ;
		return	true	;
	}

	/* Check if the property was edited using the text editor	*/
	/* widget, in which case retrieve everything from there.	*/
	if ((attr->getFlags() & KAF_EDITOR) != 0)
	{
		setProperty (item, m_textEdit->text()) ;
		return	true ;
	}


	/* Final catchall ....						*/
	setProperty (item, m_lineEdit->text()) ;
	return	true ;
}

/*  KBPropDlg								*/
/*  findHelpMapping							*/
/*		: Find possible help mapping for name			*/
/*  name	: const QString & : Name				*/
/*  (returns)	: cchar *	  : Mapping or null if none		*/

cchar	*KBPropDlg::findHelpMapping
	(	const QString	&name
	)
{
	return	0 ;
}

/*  KBPropDlg								*/
/*  setHelpEnabled							*/
/*		: Enable or disable help button appropriately for item	*/
/*  attr	: KBAttrItem *	: Item					*/
/*  (returns)	: void		:					*/

void	KBPropDlg::setHelpEnabled
	(	KBAttrItem	*attr
	)
{
//**	m_bHelp->setEnabled (!getHelpTag(attr).isNull()) ;
	m_bHelp->setEnabled (!attr->attr()->getHelpTag().isEmpty()) ;
}

/*  KBPropDlg								*/
/*  setDescription							*/
/*		: Explicitely set description				*/
/*  text	: const QString & : Text to set				*/
/*  (returns)	: void		  :					*/

void	KBPropDlg::setDescription
	(	const QString	&text
	)
{
	m_descrip->setText(text) ;
}

/*  KBPropDlg								*/
/*  pickProperty: Pick property to be edited				*/
/*  pItem	: QListViewItem*: Property item				*/
/*  (returns)	: void		:					*/

void	KBPropDlg::pickProperty
	(	QListViewItem *pItem
	)
{
	if ((pItem == 0) || (pItem->depth() == 0))
		return ;

	/* If this is the currently edited property then ignore it,	*/
	/* else if some other property is already being edited then	*/
	/* take it that the user accepts the new value.			*/
	if (curListItem == pItem)
		return ;

	if (curListItem != 0)
		if (!clickAccept ())
			return	;

	/* Make sure the item is highlighted. This is needed because of	*/
	/* the listview event filters, which let through double-click	*/
	/* but not (always) the single clicks.				*/
	m_propAllVal->setCurrentItem (pItem) ;

	curAttrItem = attrByLegend.find (pItem->text(0)) ;
	curListItem = pItem ;
	setHelpEnabled (curAttrItem) ;

	KBAttr	*attr	= curAttrItem->attr() ;

	if (!showProperty (curAttrItem))
	{
		curAttrItem	= 0 ;
		curListItem	= 0 ;

		setCurrent  (pItem) ;
		return	    ;
	}

	m_propArea  ->show() ;
	m_propAllVal->setFixedWidth (m_propAllVal->header()->sectionSize(0)) ;

	m_bEdit   ->setEnabled (false) ;
	m_bClear  ->setEnabled (false) ;
	m_bCancel ->setEnabled (false) ;
	clrAttrItem = 0 ;

	/* The property values list box is partly hidden, and the	*/
	/* accept and ignore buttons enabled.				*/
	m_descrip  ->show       () ;
	m_descrip  ->setText    (curAttrItem->getDescription()) ;
	m_bAccept  ->setEnabled (true) ;
	m_bIgnore  ->setEnabled ((attr->getFlags() & KAF_SYNTHETIC) == 0) ;

	setCaption (QString("%1: %2").arg(m_caption).arg(curListItem->text(0))) ;
}

/*  KBPropDlg								*/
/*  clickEdit	: Handle click of the edit button			*/
/*  (returns)	: void		:					*/

void	KBPropDlg::clickEdit ()
{
	if (curListItem == 0)
	{
		QListViewItem *pItem = m_propAllVal->currentItem() ;
		if (pItem != 0) pickProperty (pItem) ;
	}
}

/*  KBPropDlg								*/
/*  clickOK	: Handle click of the OK button				*/
/*  (returns)	: void		:					*/

void	KBPropDlg::clickOK ()
{
	/* If a property is currently being edited then take it that	*/
	/* the user accepts the newly edited value.			*/
	if (!clickAccept ())
		return ;

	/* Check that the property values are OK. If not then we do not	*/
	/* allow the dialog to be OK'ed					*/
	{
		QDictIterator<KBAttrItem> aIter (attrByName) ;
		KBAttrItem		  *item ;

		while ((item = aIter.current()) != 0)
		{	if (!propertyOK (item))
				return	;
			aIter	+= 1	;
		}
	}

	/* Copy back the property values. Since all properties are, at	*/
	/* the bottom, represented as strings, we can just pass the	*/
	/* text back from the property values list box.			*/
	{
		QDictIterator<KBAttrItem> aIter (attrByName) ;
		KBAttrItem		  *attr ;

		while ((attr = aIter.current()) != 0)
		{
			attr->save () ;
			aIter	+= 1  ;
		}
	}

	/* Similarly pass back the slots and tests, if any. We just	*/
	/* clear the original slots, then associate each of the		*/
	/* current slots with the parent object.			*/
	KBObject *object = m_node->isObject() ;
	if (object != 0)
	{
		object->clearSlots () ;
		LITER
		(	KBSlot,
			m_slotList,
			slot,
			slot->setParent (object)
		)
		object->clearTests () ;
		LITER
		(	KBTest,
			m_testList,
			test,
			test->setOwner  (object)
		)
	}

	m_configDlg->fixUp() ;
	done	(true) ;
}

/*  KBPropDlg								*/
/*  clickCancel	: Handle click of the Cancel button			*/
/*  (returns)	: void		:					*/

void	KBPropDlg::clickCancel ()
{
	QDictIterator<KBAttrItem> aIter (attrByName) ;
	KBAttrItem		  *item ;

	while ((item = aIter.current()) != 0)
	{
		QString	vOld	= item->attr ()->getValue() ;
		QString	vNew	= item->value() ;

		if (vOld.isNull()) vOld = "" ;
		if (vNew.isNull()) vNew = "" ;

		DPRINTF
		((	"KBPropDlg::clickCancel: [%s] [%s][%s]\n",
			(cchar *)item->attr()->getName (),
			(cchar *)vOld,
			(cchar *)vNew
		))	;

		if (vOld != vNew)
		{
			if (TKMessageBox::questionYesNo
			   	(	0,
					TR("Some properties have been changed: cancel anyway?"),
					TR("Properties changed")
				)
				== TKMessageBox::Yes) break ;

			return	;
		}

		aIter	+= 1	;
	}

	done	(false) ;
}

/*  KBPropDlg								*/
/*  clickAccept	: Handle click of the Accept button			*/
/*  (returns)	: bool		: Value is acceptable			*/

bool	KBPropDlg::clickAccept ()
{
	/* We should only get here if there is a property being edited,	*/
	/* but check anyway. Then copy back the edited text, after	*/
	/* which we can treat it as an ignore operation.		*/
	if (curListItem != 0)
	{
		if (!saveProperty (curAttrItem))
			return false ;

		clickIgnore () ;
	}

	return	true ;
}

/*  KBPropDlg								*/
/*  clickIgnore	: Handle click of the Ignore button			*/
/*  (returns)	: void		:					*/

void	KBPropDlg::clickIgnore ()
{
	if (curListItem != 0)
	{
		QListViewItem	*i = curListItem ;

		dropProperty () ;;
		m_bAccept   ->setEnabled (false) ;
		m_bIgnore   ->setEnabled (false) ;
		m_bCancel   ->setEnabled ( true) ;

		DELOBJ	(m_attrDlg) ;

		curListItem = 0 ;
		curAttrItem = 0 ;

		setCurrent  (i) ;
		setCaption  (m_caption) ;
	}
}

/*  KBPropDlg								*/
/*  clickClear	: Handle click of the clear button			*/
/*  (returns)	: void		:					*/

void	KBPropDlg::clickClear ()
{
	if ((curListItem == 0) && (clrAttrItem != 0))
	{	clrAttrItem->clear   () ;
		clrAttrItem->display () ;
	}
}

/*  KBPropDlg								*/
/*  clickHelp	: Handle click of the help button			*/
/*  (returns)	: void		:					*/

void	KBPropDlg::clickHelp ()
{
	QListViewItem	*item	= m_propAllVal->currentItem() ;
	if ((item == 0) || (item->depth() == 0)) return ;
	KBAttrItem	*attr	= attrByLegend.find (item->text(0)) ;

#if	__KB_KDE || __KB_TKC

	QString		tag	= attr->attr()->getHelpTag   () ;
	QStringList	bits	= QStringList::split ('_', tag) ;

	DPRINTF
	((	"KBPropDlg::clickHelp: tag=[%s] [%s]_[%s]\n",
		(cchar *)tag,
		(cchar *)bits[0],
		(cchar *)bits[1]
	))	;

	if (!tag.isEmpty())
		KBManual::self()->slotHelp (QString("rekall/App4_%1#%2").arg(bits[0]).arg(bits[1])) ;
#endif

#if	__KB_KDE_OLD
	TKHelpProxy	*help	= TKHelpProxy::self() ;
	QString		legend	= item->text(0) ;
	help->showHelpPage (getHelpTag (attr)) ;
#endif
}

/*  KBPropDlg								*/
/*  clickVerify : Handle click of the verify button			*/
/*  (returns)	: void		:					*/

void	KBPropDlg::clickVerify  ()
{
	if (m_attrDlg != 0) m_attrDlg->verify() ;
}

/*  KBPropDlg								*/
/*  setCurrent	: Handle change in current item				*/
/*  pItem	: QListViewItem*: Property item				*/
/*  (returns)	: void		:					*/

void	KBPropDlg::setCurrent
	(	QListViewItem *pItem
	)
{
	if (pItem->depth() == 0)
	{
		m_bHelp ->setEnabled(false) ;
		m_bEdit ->setEnabled(false) ;
		m_bClear->setEnabled(false) ;
		return	;
	}

	if ((curListItem == 0) && (pItem != 0))
	{
		KBAttrItem *item  = attrByLegend.find (pItem->text(0)) ;
		bool	   enable = (item->attr()->getFlags() & (KAF_CLEAR|KAF_EDITOR)) != 0 ;

		clrAttrItem = enable ? item : 0 ;
		m_bClear->setEnabled  (enable) ;
		setHelpEnabled 	      (item)   ;

		m_bEdit ->setEnabled  (true)   ;

		return	;
	}

	m_bHelp ->setEnabled (false) ;
	m_bEdit ->setEnabled (false) ;
	m_bClear->setEnabled (false) ;
	clrAttrItem	= 0 ;
}


/*  KBPropDlg								*/
/*  pickCombo	: Handle user picking combo box element			*/
/*  value	: const QString &: Value selected			*/
/*  (returns)	: void		 :					*/

void	KBPropDlg::pickCombo
	(	const QString &value
	)
{
	m_lineEdit->insert    (value) ;
	m_lineEdit->setFocus  () ;
}

/*  KBPropDlg								*/
/*  propertyOK	: Check if property is OK				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property is OK			*/

bool	KBPropDlg::propertyOK
	(	KBAttrItem	*item
	)
{
	KBAttr	*attr	= item->attr() ;

	if (attr->required () && item->value().isEmpty())
		return	warning (attr->getNullcheck ()) ;

	if (!attr->valueOK (item->value()))
		return	warning	(QString(TR("%1 has an invalid value"))
					.arg(attr->getLegend ())) ;
	return	true	;
}


/*  KBPropDlg								*/
/*  mouseDoubleClickEvent						*/
/*		: Handle mouse double click				*/
/*  e		: QMouseEvent *	: Associated event			*/
/*  (returns)	: void		:					*/

void	KBPropDlg::mouseDoubleClickEvent
	(	QMouseEvent	*e
	)
{
#if	0 && ! __KB_RUNTIME
	/* Documentation and debugging hook. Double-click with the	*/
	/* control and shift keys down prints the attribute set to a	*/
	/* file which is named for the node.				*/
	if ((e->state() & (ShiftButton|ControlButton)) != (ShiftButton|ControlButton))
		return	;

	/* If the alt key is down then we will only print attributes	*/
	/* which are actually in the derived node class, rather than	*/
	/* all attributes including those in base classes.		*/
	extern	void	exportSGML (KBNode *, bool) ;
	exportSGML (node, (e->state() & AltButton) != 0) ;
#endif
	KBDialog::mouseDoubleClickEvent (e) ;
}

