/***************************************************************************
    file	         : kb_propdlg.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_PROPDLG_H
#define	_KB_PROPDLG_H

#if	! __KB_RUNTIME


#include	<qptrlist.h>
#include	<qintdict.h>

#include	"kb_classes.h"
#include	"kb_dialog.h"
#include	"kb_attritem.h"
#include	"kb_attrdlg.h"

#include	"kb_config.h"
#include	"kb_configdlg.h"


#include	"libkbase_exports.h"

class	RKComboBox	;
class	QCheckBox	;
class	QSpinBox	;
class	QTextView	;
class	RKListView	;
class	RKPushButton	;
class	RKLineEdit	;
class	KBTextEdit	;
class	RKVBox		;
class	QWidgetStack	;

struct	LIBKBASE_API	HelpMap
{
	cchar	*name	;		/* Actual attrbute name		*/
	cchar	*help	;		/* Tag used for help file	*/
}	;


class	KBSlotListDlg	;
class	KBTestListDlg	;

/*  KBAttrIntChoice							*/
/*  ---------------							*/
/*  Derived attribute item used to handle choices, so that the choice	*/
/*  text rather than the numerical representation is shown.		*/

class	LIBKBASE_API	KBAttrIntChoice : public KBAttrItem
{
	IntChoice	*choices	;
	int		defval		;

public	:


	KBAttrIntChoice	(KBAttr *, IntChoice *, int = 0) ;
	virtual ~KBAttrIntChoice ()	;

	virtual	QString	displayValue	() ;
}	;


/*  KBPropDlg								*/
/*  ---------								*/
/*  This is a general-purpose control properties dialog for displaying	*/
/*  and setting properties associated with objects such as fields and	*/
/*  push-buttons.							*/

class	LIBKBASE_API	KBPropDlg : public KBDialog
{
	Q_OBJECT

	KBNode			*m_node		;
	QString			m_caption	;

	QIntDict<QListViewItem>	m_groups	;
	KBConfigDlg		*m_configDlg 	;

	virtual	bool		eventFilter	(QObject *, QEvent *) 	;
	QString			getHelpTag	(KBAttrItem *)		;
	void			setHelpEnabled	(KBAttrItem *) 		;

protected :

	virtual	cchar		*findHelpMapping(const QString &)	;

	virtual	void		mouseDoubleClickEvent (QMouseEvent *)	;

	RKVBox			*m_propArea	;
	QWidgetStack		*m_propStack	;
	RKVBox			*m_layCommon	;

private	:

	RKListView		*m_propAllVal	;

	QTextView		*m_descrip	;
	QWidget			*m_userWidget	;

	KBAttrDlg		*m_attrDlg	;

private :

	RKPushButton		*m_bOK		;
	RKPushButton		*m_bCancel	;
	RKPushButton		*m_bAccept	;
	RKPushButton		*m_bIgnore	;
	RKPushButton		*m_bClear	;
	RKPushButton		*m_bEdit	;
	RKPushButton		*m_bUser	;

protected :

	RKPushButton		*m_bVerify	;
	RKPushButton		*m_bHelp	;

private	:

	QDict<KBAttrItem>	attrByName	;
	QDict<KBAttrItem>	attrByLegend	;
	QPtrList<KBAttr>	&attribs	;
	KBAttrItem		*curAttrItem	;
	QListViewItem		*curListItem	;
	KBAttrItem		*clrAttrItem	;
	QString			resultVal	;
	QString			iniattr		;
	QPtrList<KBSlot>	m_slotList	;
	QPtrList<KBTest>	m_testList	;
	KBSlotListDlg		*m_slotListDlg	;
	KBTestListDlg		*m_testListDlg	;

	bool			iniRSize	;
	int			width		;
	int			height		;

	void		setupListView	(RKListView *, bool) ;
	void		addAttrib	(KBAttrItem *, RKListView *) ;
	void		addAttrib	(KBAttr     *)  ;

protected :


	RKLineEdit		*m_lineEdit	;
	KBTextEdit		*m_textEdit	;
	RKComboBox		*m_comboBox	;
	QCheckBox		*m_checkBox	;
	QSpinBox		*m_spinBox	;

	virtual	void		resizeEvent	(QResizeEvent *) ;

	virtual KBAttrItem	*getAttrItem	(KBAttr *) ;

	virtual	void		dropProperty	()	   ;
	virtual	bool		showProperty	(KBAttrItem *) ;
	virtual	bool		saveProperty	(KBAttrItem *) ;
	virtual	bool		hideProperty	(KBAttr     *) ;
	virtual	bool		propertyOK	(KBAttrItem *) ;
	virtual	void		preExec		() ;

	RKPushButton		*getUserButton	(const QString &) ;
	void			setUserWidget	(QWidget *) ;
	bool			setProperty	(KBAttrItem *, const QString &) ;
	bool			setProperty	(cchar  *,     const QString &) ;
	const QString		&getProperty	(cchar  *) ;
	bool			warning		(cchar  *) ;

	void			showChoices	(KBAttrItem *, IntChoice *, const QString &, RKComboBox * = 0) ;
	void			saveChoices	(KBAttrItem *, IntChoice *, RKComboBox * = 0) ;

private slots :

	void			clickClear	()	 		;
	void			clickIgnore	()	 		;
	bool			clickAccept	()	 		;
	void			clickEdit	()	 		;

	virtual void		clickHelp	()	 		;
	virtual	void		clickVerify	()	 		;

	void			pickProperty	(QListViewItem *)	;
	void			pickCombo	(const QString &)	;
	void			setCurrent	(QListViewItem *)	;
	void			setDescription	(const QString &)	;

protected slots :

	virtual	void		clickOK		() ;
	virtual	void		clickCancel	() ;

public	:

	KBPropDlg (KBNode *, cchar *, QPtrList<KBAttr> &, cchar * = 0) ;
       ~KBPropDlg () ;

	bool			exec 		() ;
}	;

#endif

#endif	// _KB_PROPDLG_H
