/***************************************************************************
    file	         : kb_qryquery.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_object.h"
#include	"kb_location.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_database.h"
#include	"kb_queryset.h"
#include	"kb_qrylevel.h"

#if		__KB_AUTOMOC
#include	"kb_qryquery.moc"
#else
#include	"kb_qryquery.h"
#endif

#include	"kb_query.h"
#include	"kb_table.h"
#include	"kb_qryexpr.h"
#include	"kb_docroot.h"
#include	"kb_nodereg.h"
#include	"kb_parse.h"

#if		! __KB_RUNTIME
#include	"kb_propdlg.h"
#include	"kb_qryquerydlg.h"
#endif

/*  KBQryQuery								*/
/*  KBQryQuery	: Constructor for form query node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*		: bool *	:					*/
/*  (returns)	: KBQryQuery	:					*/

KBQryQuery::KBQryQuery
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*
	)
	:
	KBQryData	(parent, aList,		"KBQryQuery"),
	m_qryName	(this,	 "query", 	aList),
	m_qryWhere	(this,	 "where", 	aList),
	m_qryOrder	(this,	 "order", 	aList),
	m_qryGroup	(this,	 "group", 	aList),
	m_qryHaving	(this,	 "having", 	aList),
	m_qryDistinct	(this,	 "distinct", 	aList),
	m_topTable	(this,	 "toptable",	aList)
{
	m_qryRoot	= 0	;
}

/*  KBQryQuery								*/
/*  KBQryQuery	: Constructor for form query node			*/
/*  parent	: KBForm *	: Parent node				*/
/*  (returns)	: KBQryQuery	:					*/

KBQryQuery::KBQryQuery
	(	KBNode	*parent
	)
	:
	KBQryData	(parent, "KBQryQuery"	  ),
	m_qryName	(this,	 "query",  	""),
	m_qryWhere	(this,	 "where",  	""),
	m_qryOrder	(this,	 "order",  	""),
	m_qryGroup	(this,	 "group",  	""),
	m_qryHaving	(this,	 "having",  	""),
	m_qryDistinct	(this,	 "distinct", 	""),
	m_topTable	(this,	 "toptable",	"")
{
	m_qryRoot	= 0	;
}

/*  KBQryQuery								*/
/*  KBQryQuery	: Constructor for form query node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  query	: KBQryQuery *	: Extant form query			*/
/*  (returns)	: KBQryQuery	:					*/

KBQryQuery::KBQryQuery
	(	KBNode		*parent,
		KBQryQuery	*query
	)
	:
	KBQryData	(parent, query),
	m_qryName	(this,	 "query",  	query),
	m_qryWhere	(this,	 "where",  	query),
	m_qryOrder	(this,	 "order",  	query),
	m_qryGroup	(this,	 "group",  	query),
	m_qryHaving	(this,	 "having",  	query),
	m_qryDistinct	(this,	 "distinct", 	query),
	m_topTable	(this,	 "toptable",	query)
{
	m_qryRoot	= 0	;
}

/*  KBQryQuery								*/
/*  ~KBQryQuery	: Destructor for form query node			*/
/*  (returns)	:		:					*/

KBQryQuery::~KBQryQuery ()
{
	DELOBJ	(m_qryRoot) ;
}

/*  KBQryQuery								*/
/*  replicate	: Replicate this table query				*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New table query node			*/

KBNode	*KBQryQuery::replicate
	(	KBNode	*_parent
	)
{
	return	new KBQryQuery (_parent, this) ;
}

/*  KBQryQuery								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBQryQuery::showAs
	(	KB::ShowAs	mode
	)
{
#if	! __KB_RUNTIME
	if (mode == KB::ShowAsDesign)
	{	m_qryLevels.clear () ;
		m_tabList  .clear () ;
		DELOBJ	 (m_qryRoot) ;
	}
#endif
	KBNode::showAs (mode) ;
}

/*  KBQryQuery								*/
/*  loadQueryDef: Load the query definition				*/
/*  location	: KBLocation &	: Query location			*/
/*  (returns)	: bool		: Success				*/

bool	KBQryQuery::loadQueryDef
	(	KBLocation	&location
	)
{
	KBError		 error	 ;
	QByteArray	 doc	 ;
	QPtrList<KBTable>	 qryList ;

	DELOBJ	(m_qryRoot)	 ;

	/* Read in the query definition, and parse it. If either fails	*/
	/* then return an error, but also fabricate a dummy quert.	*/
	if (!location.contents (doc, error))
	{
		m_qryRoot  = new KBQuery () ;
		setError  (error) ;
		return	  false	  ;
	}
	if ((m_qryRoot = KBOpenQueryText (location, doc, error)) == 0)
	{
		setError  (error) ;
		m_qryRoot = new KBQuery () ;
		return	  false	  ;
	}

	/* Query loaded and parsed OK, so construct a set of query	*/
	/* levels corresponding to the top-level tables in the query.	*/
	/* Note that we process in reverse order so that they can be	*/
	/* chained top-to-bottom.					*/
	m_tabList .clear () ;
	m_exprList.clear () ;

	m_qryRoot->getQueryInfo (m_svrName, qryList, m_exprList) ;
	if (!KBTable::blockUp (qryList, m_topTable.getValue(), m_tabList, error))
	{	setError (error) ;
		return	 false	 ;
	}

	return	true	;
}

/*  KBQryQuery								*/
/*  loadQueryDef: Load the query definition				*/
/*  (returns)	: bool		: Success				*/

bool	KBQryQuery::loadQueryDef ()
{
	KBLocation location
	(	getDocRoot()->getDBInfo(),
		"query",
		getDocRoot()->getDocLocation().server(),
		m_qryName.getValue ()
	)	;

	return	loadQueryDef (location) ;
}

/*  KBQryQuery								*/
/*  loadQuery	: Load query						*/
/*  (returns)	: bool		: Success				*/

bool	KBQryQuery::loadQuery ()
{
	if (m_qryRoot == 0)
		if (!loadQueryDef ()) return false ;

	KBQryLevel *next = 0 ;

	for (int idx = m_tabList.count() - 1 ; idx >= 0 ; idx -= 1)
	{
		next	= new KBQryLevel
			  (	getParent(),
				next,
				dbLink,
				idx,
				m_tabList.at(idx)
			  )	;
		m_qryLevels.insert (0, next) ;
	}

	/* Assemble the global where and order components. These are	*/
	/* based on our properties, plus expressions from the		*/
	/* underlying query.						*/
	QString	gWhere	= m_qryWhere .getValue() ;
	QString	gOrder	= m_qryOrder .getValue() ;
	QString	gGroup	= m_qryGroup .getValue() ;
	QString	gHaving	= m_qryHaving.getValue() ;
	QString	sWhere	= gWhere.isEmpty() ? "" : " and " ;
	QString	sOrder	= gOrder.isEmpty() ? "" : ", "	  ;
	QString	sGroup	= gGroup.isEmpty() ? "" : ", "	  ;

	LITER
	(
		KBQryExpr,
		m_exprList,
		expr,

		switch (expr->getUsage())
		{
			case KBQryExpr::AsSortAsc  :
				gOrder	+= sOrder  ;
				gOrder	+= expr->getExpr() ;
				sOrder	 = ", "	   ;
				break	 ;

			case KBQryExpr::AsSortDesc :
				gOrder	+= sOrder  ;
				gOrder	+= expr->getExpr() ;
				gOrder	+= " desc" ;
				sOrder	 = ", "	   ;
				break	 ;

			case KBQryExpr::AsWhere    :
				gWhere	+= sWhere  ;
				gWhere	+= expr->getExpr() ;
				sWhere	 = " and " ;
				break	 ;

			case KBQryExpr::AsGroup	   :
				gGroup	+= sGroup  ;
				gGroup	+= expr->getExpr() ;
				sGroup	 = ", "    ;
				break	 ;

			case KBQryExpr::AsHaving   :
				if (!gHaving.isEmpty())
				{
					setError
					(	KBError::Error,
						TR("Cannot have multiple \"having\" terms"),
						QString::null,
						__ERRLOCN
					)	;
					return	false	  ;
				}

				gHaving = expr->getExpr() ;
				break	;

			default	:
				break	;
		}
	)

	if (gGroup.isEmpty() && !gHaving.isEmpty())
	{
		setError
		(	KBError::Error,
			TR("Cannot have \"having\" without \"group by\""),
			"having " + gHaving,
			__ERRLOCN
		)	;
		return	false	;
	}

	m_qryLevels.at(0)->setGlobalWhere  (gWhere ) ;
	m_qryLevels.at(0)->setGlobalOrder  (gOrder ) ;
	m_qryLevels.at(0)->setGlobalGroup  (gGroup ) ;
	m_qryLevels.at(0)->setGlobalHaving (gHaving) ;
	m_qryLevels.at(0)->setDistinct	   (m_qryDistinct.getBoolValue()) ;
	m_qryLevels.at(0)->setLimit	   (m_limit      .getIntValue ()) ;

	return	linkServer (m_svrName) ;
}


/*  KBQryQuery								*/
/*  getFieldList: Get list of fields from database			*/
/*  qlvl	: uint		: Query level				*/
/*  fldList	: QPtrList<KBFieldSpec>&				*/
/*				: Field list result			*/
/*  pKey	: int &		: Primary key index			*/
/*  (returns)	: bool		: Success				*/

bool	KBQryQuery::getFieldList
	(	uint			qlvl,
		QPtrList<KBFieldSpec>	&fldList,
		int			&pKey
	)
{
	if (m_qryRoot == 0)
		if (!loadQueryDef ()) return false ;

	LITER
	(
		KBQryExpr,
		m_exprList,
		expr,

		if ( (expr->getUsage() == KBQryExpr::AsExprOnly) &&
		     (expr->getExpr () != "*") )
			fldList.append
			(	new KBFieldSpec
				(	0xffff0000,
					expr->getSQL(),
					"",
					KB::ITUnknown,
					0,
					0,
					0
				)
			)	;
	)

	return	KBQryData::getFieldList (qlvl, fldList, pKey) ;
}

/*  KBQryQuery								*/
/*  getLinkage	: Get parent/child table linkage			*/
/*  qlvl	: uint		: Child table level			*/
/*  mexp	: QString &	: Expression in parent			*/
/*  cexpr	: QString &	: Expression (table column) in child	*/
/*  (returns)	: bool		: Success				*/

bool	KBQryQuery::getLinkage
	(	uint		qlvl,
		QString		&mexpr,
		QString		&cexpr
	)
{
	if (!getQryLevel (qlvl))
		return false ;

	if ((qlvl < 1) || (qlvl >= m_tabList.count()))
	{
		KBError::EError
		(	"Invalid query level when fetching child expression",
			QString(TR("Level %1, %2 exist")).arg(qlvl).arg(m_tabList.count()),
			__ERRLOCN
		)	;
		return	false	;
	}

	KBTable	*tParent = m_tabList.at(qlvl - 1) ;
	KBTable	*tChild	 = m_tabList.at(qlvl    ) ;

	mexpr	= QString("%1.%2").arg(tParent->getQueryName()).arg(tParent->getField ()) ;
	cexpr	= QString("%1.%2").arg(tChild ->getQueryName()).arg(tParent->getField2()) ;

	fprintf	(stderr, "Linkage: %d: %s:%s\n", qlvl, (cchar *)mexpr, (cchar *)cexpr)  ;
	return	true	;
}

/*  KBQryQuery								*/
/*  getComment	: Get brief descriptive comment				*/
/*  qryLvl	: uint		: Query level				*/
/*  (returns)	: QString	: Comment				*/

QString	KBQryQuery::getComment
	(	uint		qryLvl
	)
{
	return	QString("Query: %1:%2").arg(m_qryName.getValue()).arg(qryLvl) ;
}

#if	! __KB_RUNTIME

/*  KBQryQuery								*/
/*  whatsThisExtra							*/
/*		: Extra stuff for pseudo-what's this popup		*/
/*  info	: QValueList<KBWhatsThisPair> &	: List of items		*/
/*  (returns)	: void				:			*/

void	KBQryQuery::whatsThisExtra
	(	QValueList<KBWhatsThisPair>	&info
	)
{
	info.append (KBWhatsThisPair(TR("Query"), m_qryName.getValue())) ;
}

/*  KBQryQuery								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar	*	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBQryQuery::propertyDlg
	(	cchar		*iniAttr
	)
{

	if (!qryQueryPropDlg (this, "Query", m_attribs))
		return false ;

	m_qryLevels.clear () ;
	dropServer 	  () ;

	return true   ;
}
#endif


NEWNODE(QryQuery, (cchar *)0, 0, KNF_FORM|KNF_REPORT)
