/***************************************************************************
    file	         : kb_qryquerydlg.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qframe.h>
#include	<qlabel.h>
#include	<qwidgetstack.h>

#include	"kb_node.h"

#include	"kb_attr.h"
#include	"kb_dbinfo.h"
#include	"kb_dbdociter.h"
#include	"kb_propdlg.h"
#include	"kb_query.h"
#include	"kb_qryquery.h"
#include	"kb_table.h"
#include	"kb_qryexpr.h"
#include	"kb_nodereg.h"
#include	"kb_parse.h"
#include	"kb_docroot.h"
#include	"kb_resizewidget.h"

#if		__KB_AUTOMOC
#include 	"kb_qryquerydlg.moc"
#else
#include 	"kb_qryquerydlg.h"
#endif

#include	"rk_combobox.h"


class LIBKBASE_API KBQryAttrItem : public KBAttrItem
{
	KBQryQueryPropDlg	*qDlg	;

public	:

	inline	KBQryAttrItem
		(	KBAttr			*attr,
			KBQryQueryPropDlg	*qDlg
		)
		:
		KBAttrItem	(attr),
		qDlg		(qDlg)
	{
	}

	virtual	QString	displayValue	() ;
}	;

QString	KBQryAttrItem::displayValue ()
{
	return	qDlg->nameForIdent (m_value) ;
}


/*  KBQryQueryPropDlg							*/
/*  KBQryQueryPropDlg							*/
/*		: Constructor for query-query properties dialog		*/
/*  link	: KBLink *	    : Parent link			*/ 
/*  caption	: const char *      : Dialog box caption		*/
/*  attribs	: QPtrList<KBAttr>& : List of attributes		*/
/*  (returns)	: KBQryQueryPropDlg :					*/

KBQryQueryPropDlg::KBQryQueryPropDlg
	(	KBQryQuery		*query,
		cchar			*caption,
		QPtrList<KBAttr>	&attribs
	)
	:
	KBPropDlg	(query, caption, attribs),
	m_query		(query)
{
	m_topTableWidget = new KBResizeWidget (m_propStack) ;
	m_topTableCombo	 = new RKComboBox     (m_topTableWidget) ;

	m_idmap.setAutoDelete (true) ;
	m_topTableWidget->hide () ;

	connect
	(	m_topTableWidget,
		SIGNAL(resized       (KBResizeWidget *, QSize)),
		SLOT  (topTableResize(KBResizeWidget *, QSize))
	)	;
	connect
	(	m_topTableCombo,
		SIGNAL(activated     (int)),
		SLOT  (showBlockUp   (int))
	)	;

	m_qryRoot	= 0 ;

	if (!m_query->getQryName().isEmpty())
	{
		KBError	error	;
		if (!loadQueryList (m_query->getQryName(), m_query->getTopTable(), error))
			error.DISPLAY() ;
	}
}

/*  KBQryQueryPropDlg							*/
/*  ~KBQryQueryPropDlg							*/
/*		: Destructor for block properties dialog		*/
/*  (returns)	: void		:					*/

KBQryQueryPropDlg::~KBQryQueryPropDlg ()
{
	if (m_blockUp.count() > 0) delete m_blockUp.at(0) ;
	DELOBJ	(m_qryRoot) ;
}

/*  KBQryQueryPropDlg							*/
/*  getAttrItem	: Get attribute item for attribute			*/
/*  attr	: KBAttr *	: Attribute				*/
/*  (returns)	: KBAttrItem *	: Associated attribute item		*/

KBAttrItem *KBQryQueryPropDlg::getAttrItem
	(	KBAttr	*attr
	)
{
	if (attr->getName() == "toptable")
		return	new KBQryAttrItem (attr, this) ;

	return	KBPropDlg::getAttrItem (attr) ;
}

/*  KBQryQueryPropDlg							*/
/*  nameForIdent: Get name to display for table identifier		*/
/*  ident	: const QString & : Table identifier			*/
/*  (returns)	: QString	  : Display name			*/

QString	KBQryQueryPropDlg::nameForIdent
	(	const QString	&ident
	)
{
	LITER
	(	KBTable,
		m_qryList,
		table,

		if (table->getIdent() == ident)
		{
			if (table->getAlias().isEmpty())
				return	table->getTable() ;

			return	QString	("%1 as %2")
					.arg(table->getTable())
					.arg(table->getAlias()) ;
		}
	)

	return	ident	;
}

/*  KBQryQueryPropDlg							*/
/*  loadQueryList: Load table list for query				*/
/*  qryName	 : const QString & : Query name				*/
/*  topTable	 : const QString & : Top table identifier		*/
/*  error	 : KBError &	   : Error return			*/
/*  (returns)	 : bool		   : Success				*/

bool	KBQryQueryPropDlg::loadQueryList
	(	const QString	&qryName,
		const QString	&topTable,
		KBError		&error
	)
{
	KBLocation location
	(	m_query->getDocRoot()->getDBInfo  (),
		"query",
		m_query->getDocRoot()->getDocLocation().server(),
		qryName
	) ;

	DELOBJ	(m_qryRoot)	;
	m_qryList.clear ()	;

	/* Read in the query definition, and parse it. If either fails	*/
	/* then return an error.					*/
	QByteArray doc ;

	if (!location.contents (doc, error))
		return false	;
	if ((m_qryRoot = KBOpenQueryText (location, doc, error)) == 0)
		return false ;

	QPtrList<KBQryExpr>exprList;
	QString		svrName	;
	int		at = -1 ;

	m_topTableCombo->clear	    ()   ;
	m_topTableCombo->insertItem ("") ;

	m_idmap.clear ()		 ;
	m_idmap.append(new QString())	 ;

	m_qryRoot->getQueryInfo (svrName, m_qryList, exprList) ;
	for (uint idx = 0 ; idx < m_qryList.count() ; idx += 1)
	{
		KBTable	*table	= m_qryList.at(idx) ;
		QString	text	= table->getTable() ;

		if (!table->getAlias().isEmpty())
			text	= QString("%1 as %2")
					 .arg(text)
					 .arg(table->getAlias()) ;

		m_topTableCombo->insertItem (text) ;
		m_idmap.append (new QString(table->getIdent())) ;

		if (table->getIdent() == topTable) at = idx + 1 ;
	}

	if (at >= 0) m_topTableCombo->setCurrentItem (at) ;

	showBlockUp (at >= 0 ? at : 0) ;

	return	true	;
}

/*  KBQryQueryPropDlg							*/
/*  showProperty: Show property						*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property on show			*/

bool	KBQryQueryPropDlg::showProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Query							*/
	/* This should display a list of all available KBase-level	*/
	/* queries.							*/
	if (aName == "query")
	{
		KBDocRoot	*docRoot  = m_query->getDocRoot() ;
		KBDBInfo	*dbInfo   = docRoot->getDBInfo () ;
		int		at	  = -1  ;
		KBDBDocIter	docIter	  ;
		QString		name	  ;
		QString		stamp	  ;
		KBError		error	  ;

		m_comboBox->clear () ;

		if (!docIter.init
			(	dbInfo,
				docRoot->getDocLocation().server(),
				"query",
				QString("%1.qry").arg(dbInfo->getDBExtn()),
				error
			))
		{
			error.DISPLAY() ;
			return	false	;
		}

		while (docIter.getNextDoc (name, stamp))
		{	m_comboBox->insertItem (name) ;
			if (name == item->value())
				at = m_comboBox->count() - 1 ;
		}

		if (at >= 0) m_comboBox->setCurrentItem (at) ;
		m_comboBox->show  () ;
		return	true	  ;
	}

	/* TopTable							*/
	/* This should display a list of the tables in the query.	*/
	if (aName == "toptable")
	{
		m_topTableCombo->show () ;
		setUserWidget	      (m_topTableWidget ) ;
		return	true ;
	}

	return	KBPropDlg::showProperty (item) ;
}

/*  KBPropDlg								*/
/*  saveProperty: Check and save attribute				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Attribute OK				*/

bool	KBQryQueryPropDlg::saveProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Query							*/
	/* We need to verify that the query exists (well, someone may	*/
	/* have deleted it) and, more importantly, this it is valid.	*/
	if (aName == "query")
	{
		QString	qryName	= m_comboBox->currentText() ;
		bool	delta	= qryName != item->value () ;

		setProperty (aName,      qryName) ;
		setProperty ("toptable", ""	) ;

		KBError	error	;
		if (!loadQueryList (qryName, QString::null, error))
			error.DISPLAY() ;

		if (delta && !item->value().isEmpty())
			KBError::EWarning
			(	TR("Changing the query will probably invalidate the form or report structure"),
				QString::null,
				__ERRLOCN
			)	;

		return	true  ;
	}

	/* TopTable							*/
	/* This retrieves a value via the combobox.			*/
	if (aName == "toptable")
	{
		QString	ident	= *m_idmap.at (m_topTableCombo->currentItem()) ;
		bool	delta	= ident != item->value() ;

		setProperty (aName, ident) ;

		if (delta && !item->value().isEmpty())
			KBError::EWarning
			(	TR("Changing the top-level table will probably invalidate the form or report structure"),
				QString::null,
				__ERRLOCN
			)	;

		return	true ;
	}

	return	KBPropDlg::saveProperty (item) ;
}

/*  KBPropDlg								*/
/*  showBlockUp	: Set blocking for specified top-level tape		*/
/*  index	: int		: Index into top-level combobox		*/
/*  (returns)	: void		:					*/

void	KBQryQueryPropDlg::showBlockUp
	(	int	index
	)
{
	if (m_blockUp.count()  > 0) delete m_blockUp.at(0) ;
	m_blockUp.clear () ;
	if (m_qryList.count() == 0) return ;

	QPtrList<KBTable>	blkList ;
	KBError		error	;
	if (!KBTable::blockUp (m_qryList, *m_idmap.at(index), blkList, error))
	{	error.DISPLAY()	;
		return	;
	}

	QWidget	*parent	= m_topTableWidget ;

	LITER
	(	KBTable,
		blkList,
		table,

		QLabel	*label	= new QLabel (parent) ;

		label->setText	     (table->getTableText ()) ;
		label->setAlignment  (Qt::AlignLeft|Qt::AlignTop) ;
		label->setFrameStyle (QFrame::WinPanel|QFrame::Sunken) ;
		label->setLineWidth  (2) ;

		m_blockUp.append     (label) ;
		parent	= label	;
	)

	setBlockSizes() ;
	m_blockUp.at(0)->show() ;
}

void	KBQryQueryPropDlg::topTableResize
	(	KBResizeWidget	*,
		QSize		size
	)
{
	m_topTableCombo->move   (0, 0) ;
	m_topTableCombo->resize (size.width(), m_topTableCombo->sizeHint().height()) ;
	setBlockSizes	     () ;
}

/*  KBPropDlg								*/
/*  setBlockSizes: Set sizes in blockup diagram				*/
/*  (returns)	 : void		:					*/

void	KBQryQueryPropDlg::setBlockSizes ()
{
	if (m_blockUp.count() == 0) return ;

	int	x	= 0 ;
	int	y	= m_topTableCombo ->sizeHint().height() * 3 / 2 ;
	int	w	= m_topTableWidget->width   () ;
	int	h	= m_topTableWidget->height  () - y ;

	LITER
	(	QLabel,
		m_blockUp,
		label,

		label->setGeometry (x, y, w, h) ;

		x  = 16 ;
		w  = m_topTableWidget->width () - 32 ;

		h -= y + 16 ;
	)	
}

/*  qryQueryPropDlg							*/
/*		: Run the link properties dialog			*/
/*  query	: KBQryQuery *	: Parent link				*/ 
/*  caption	: cchar * 	: Dialog box caption			*/
/*  attribs	: QPtrList<KBAttr>&: List of attributes			*/
/*  (returns)	: bool		: Success				*/

bool LIBKBASE_API qryQueryPropDlg
	(	KBQryQuery	*query,
		cchar		*caption,
		QPtrList<KBAttr>	&attribs
	)
{
	KBQryQueryPropDlg qDlg (query, caption, attribs) ;
	return	qDlg.exec () ;
}
