/***************************************************************************
    file	         : kb_qrysqldlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qwidgetstack.h>

#include	"kb_classes.h"
#include	"kb_propdlg.h"
#include	"kb_docroot.h"

#include	"kb_select.h"
#include	"kb_qrysql.h"
#include	"kb_primarydlg.h"
#include	"kb_qrytabledlg.h"

#if		__KB_AUTOMOC
#include 	"kb_qrysqldlg.moc"
#else
#include 	"kb_qrysqldlg.h"
#endif

#include	"tk_messagebox.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"


#include	"kb_textedit.h"
#include	"kb_manual.h"


#define		__DEBUG_QRYSQLDLG	(0)
#if		__DEBUG_QRYSQLDLG
#include	"kb_dprintf.h"
#endif


/*  KBQrySQLPropDlg							*/
/*  KBQrySQLPropDlg							*/
/*		: Constructor for query-query properties dialog		*/
/*  link	: KBLink *	   : Parent link			*/ 
/*  caption	: const char *     : Dialog box caption			*/
/*  attribs	: QPtrList<KBAttr>&: List of attributes			*/
/*  (returns)	: KBQrySQLPropDlg  :					*/

KBQrySQLPropDlg::KBQrySQLPropDlg
	(	KBQrySQL		*query,
		cchar			*caption,
		QPtrList<KBAttr>	&attribs
	)
	:
	KBPropDlg	(query, caption, attribs),
	m_query		(query)
{
	m_primaryDlg	= new KBPrimaryDlg (m_propStack, m_tabSpec) ;
	m_primaryDlg->hide () ;

	m_bHelp->setEnabled (true) ;
	m_primaryItem	= 0 ;
	m_ptypeAttr	= 0 ;
	m_pexprAttr	= 0 ;
}


/*  KBQrySQLPropDlg							*/
/*  ~KBQrySQLPropDlg							*/
/*		: Destructor for block properties dialog		*/
/*  (returns)	: void		:					*/

KBQrySQLPropDlg::~KBQrySQLPropDlg ()
{
}

/*  KBQrySQLPropDlg							*/
/*  preExec	: Pre-execute hook					*/
/*  (returns)	: void		:					*/

void	KBQrySQLPropDlg::preExec ()
{
	if ( !getProperty ("server"  ).isEmpty() &&
	     !getProperty ("toptable").isEmpty() ) getTableSpec () ;

	/* Pass the unique type attribute to the primary attribute, so	*/
	/* that the latter has access to both.				*/
	m_primaryItem->setType (m_ptypeAttr, m_pexprAttr) ;
	m_primaryItem->display () ;
}

/*  KBQrySQLPropDlg							*/
/*  hideProperty: Check whether property should be hidden		*/
/*  attr	: KBAttr *	: Attribute in question			*/
/*  (returns)	: void		:					*/

bool	KBQrySQLPropDlg::hideProperty
	(	KBAttr	*attr
	)
{
	if (attr->getName() == "ptype")
	{	m_ptypeAttr = attr ;
		return	true	   ;
	}
	if (attr->getName() == "pexpr")
	{	m_pexprAttr = attr ;
		return	true	   ;
	}

	return	KBPropDlg::hideProperty (attr) ;
}

/*  KBQrySQLPropDlg							*/
/*  getAttrItem	: Get attribute item for attribute			*/
/*  attr	: KBAttr *	: Attribute				*/
/*  (returns)	: KBAttrItem *	: Associated attribute item		*/

KBAttrItem
	*KBQrySQLPropDlg::getAttrItem
	(	KBAttr		*attr
	)
{
	if (attr->getName() == "primary")
		return	m_primaryItem = new KBAttrPrimaryItem (attr) ;

	return	KBPropDlg::getAttrItem (attr) ;
}

/*  KBQrySQLPropDlg							*/
/*  showProperty: Show property						*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property on show			*/

bool	KBQrySQLPropDlg::showProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Query							*/
	/* Use the tkEditor and enable the verify button so that the	*/
	/* user can check their SQL.					*/
	if (aName == "query")
	{
		m_textEdit->setHighlight("Sql") ;
		m_textEdit->setText	(item->value()) ;
		m_textEdit->show   	() ;

		m_bVerify->setEnabled (true) ;
		return	true	;
	}

	/* This is analagous to the same setting in query-queries. It	*/
	/* will also determin which table is used for the primary	*/
	/* (or unique) key column.					*/
	if (aName == "toptable")
	{
		QString	  sql	  = getProperty ("query") ;
		KBSelect  select  ;

		m_comboBox->clear      ()   ;
		m_comboBox->insertItem ("") ;

		if (select.parseQuery (sql, 0))
			m_comboBox->insertStringList (select.tableList()) ;

		for (int idx = 0 ; idx < m_comboBox->count() ; idx += 1)
			if (m_comboBox->text(idx) == item->value())
			{	m_comboBox->setCurrentItem (idx) ;
				break	;
			}

		m_comboBox->show() ;
		return	true ;
	}

	/* Primary							*/
	/* This is the primary key column (although for our purposes,	*/
	/* any unique column will suffice). Display the primary dialog.	*/
	if (aName == "primary")
	{
		QString		    expr ;
		KBTable::UniqueType ut   = m_primaryItem->getType(expr) ;

		m_primaryDlg->set (item->value(), ut, expr) ;

		setUserWidget (m_primaryDlg) ;
		return	true ;

	}

	return	KBPropDlg::showProperty (item) ;
}

/*  KBQryTablePropDlg							*/
/*  getTableSpec: Get table specification				*/
/*  (returns)	: bool		: Success				*/

bool	KBQrySQLPropDlg::getTableSpec ()
{
	cchar		*dbName ;
	cchar		*tbName ;
	KBDBLink	dbLink	;

	if ((dbName = getProperty ("server" )) == 0)
		return	warning (TR("Please specify a server name")) ;

	if ((tbName = getProperty ("toptable")) == 0)
		return	warning (TR("Please specify a table name")) ;

	if (!dbLink.connect (m_query->getDocRoot()->getDocLocation(), dbName))
	{
		dbLink.lastError().DISPLAYCAP(TR("Cannot connect to server")) ;
		return	false	;
	}

	m_tabSpec.reset (tbName)	;
	if (!dbLink.listFields (m_tabSpec))
	{
		dbLink.lastError().DISPLAYCAP(TR("Cannot get list of fields")) ;
		return	false	;
	}

	return	true	;
}

/*  KBQrySQLPropDlg							*/
/*  findPrimary	: Find preferred primary key				*/
/*  (returns)	: void		:					*/

void	KBQrySQLPropDlg::findPrimary ()
{
	if (!getTableSpec ())
		return ;

	if (m_tabSpec.m_prefKey < 0)
	{
		KBError::EWarning
		(	TR("Unable to determine primary key column"),
			QString::null,
			__ERRLOCN
		)	;
		return	;
	}

	setProperty ("primary", m_tabSpec.m_fldList.at(m_tabSpec.m_prefKey)->m_name) ;
}

/*  KBQrySQLPropDlg							*/
/*  saveProperty: Check and save attribute				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Attribute OK				*/

bool	KBQrySQLPropDlg::saveProperty
	(	KBAttrItem	*item
	)
{
	const QString	&aName = item->attr()->getName () ;

	/* Query							*/
	/* We parse the query to see if it is valid, and if not then	*/
	/* ask the user whether to accept it ...			*/
	if (aName == "query")
	{
		QString	  sql	 = m_textEdit->text () ;
		KBSelect  select ;
		KBDBLink  dbLink ;

		if (!dbLink.connect (m_query->getDocRoot()->getDocLocation(), getProperty ("server")))
			if (TKMessageBox::questionYesNo
				(	0,
					QString(TR("Cannot parse SQL: %1\nSave anyway?"))
						.arg(dbLink.lastError().getDetails()),
					TR("SQL error")
				)
				!= TKMessageBox::Yes) return false ;

		if (!select.parseQuery (sql, &dbLink))
			if (TKMessageBox::questionYesNo
				(	0,
					QString(TR("Cannot parse SQL: %1\nSave anyway?"))
						.arg(select.lastError().getDetails()),
					TR("SQL error")
				)
				!= TKMessageBox::Yes) return false ;

		setProperty (item, sql) ;
		return	true  ;
	}

	if (aName == "toptable")
	{
		if (m_comboBox->currentText() != item->value())
		{
			setProperty (item, m_comboBox->currentText()) ;
			setProperty ("primary",  "") ;

			if (!getProperty("server").isEmpty()) findPrimary () ;
		}

		return	true  ;
	}

	if (aName == "primary")
	{
		QString	value	;
		QString	expr	;

		KBTable::UniqueType utype = m_primaryDlg->retrieve (value, expr) ;

		DPRINTF
		((	"KBQrySQLPropDlg::saveProperty: primary: v=[%s] e=[%s] y=%d\n",
			value.latin1(),
			expr .latin1(),
			(int)utype
		))	;

		m_primaryItem->setType (utype, expr) ;
		setProperty       ("primary", value) ;
		return	true ;
	}

	return	KBPropDlg::saveProperty (item) ;
}

/*  KBQrySQLPropDlg							*/
/*  clickVerify : Handle click of the verify button			*/
/*  (returns)	: void		:					*/

void	KBQrySQLPropDlg::clickVerify  ()
{
	QString	  sql	 = m_textEdit->text () ;
	KBSelect  select ;
	KBDBLink  dbLink ;

	if (!dbLink.connect (m_query->getDocRoot()->getDocLocation(), getProperty ("server")))
	{
		dbLink.lastError().DISPLAY() ;
		return	;
	}

	if (!select.parseQuery (sql, &dbLink))
	{
		select.lastError().DISPLAY() ;
		return	;
	}
}

/*  qrySQLPropDlg							*/
/*		: Run the link properties dialog			*/
/*  query	: KBQrySQL *	: Parent link				*/ 
/*  caption	: cchar * 	: Dialog box caption			*/
/*  attribs	: QPtrList<KBAttr>&: List of attributes			*/
/*  (returns)	: bool		: Success				*/

bool	qrySQLPropDlg
	(	KBQrySQL	*query,
		cchar		*caption,
		QPtrList<KBAttr>	&attribs
	)
{
	KBQrySQLPropDlg qDlg (query, caption, attribs) ;
	return	qDlg.exec () ;
}
