/***************************************************************************
    file	         : kb_qrytable.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_database.h"
#include	"kb_queryset.h"
#include	"kb_qrylevel.h"

#if		__KB_AUTOMOC
#include	"kb_qrytable.moc"
#else
#include	"kb_qrytable.h"
#endif

#include	"kb_item.h"
#include	"kb_table.h"
#include	"kb_nodereg.h"


/*  KBQryTable								*/
/*  KBQryTable	: Constructor for form query node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*		: bool *	:					*/
/*  (returns)	: KBQryTable	:					*/

KBQryTable::KBQryTable
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*
	)
	:
	KBQryData	(parent, aList,		"KBQryTable"),
	m_server	(this,	 "server",	aList, KAF_REQD),
	m_table		(this,   "table",	aList, KAF_REQD),
	m_primary	(this,   "primary",	aList),
	m_ptype		(this,   "ptype",	aList),
	m_pexpr		(this,   "pexpr",	aList),
	m_where		(this,   "where",	aList),
	m_order		(this,   "order",	aList),
	m_distinct	(this,	 "distinct",	aList)
{
	m_tabPtr	= 0 ;
}

/*  KBQryTable								*/
/*  KBQryTable	: Constructor for form query node			*/
/*  parent	: KBForm *	: Parent node				*/
/*  (returns)	: KBQryTable	:					*/

KBQryTable::KBQryTable
	(	KBNode	*parent
	)
	:
	KBQryData	(parent, "KBQryTable"  ),
	m_server	(this,	 "server",	"", KAF_REQD),
	m_table		(this,   "table",	"", KAF_REQD),
	m_primary	(this,   "primary",	""),
	m_ptype		(this,   "ptype",	KBTable::Auto),
	m_pexpr		(this,   "pexpr",	""),
	m_where		(this,   "where",	""),
	m_order		(this,   "order",	""),
	m_distinct	(this,	 "distinct",	false)
{
	m_tabPtr	= 0 ;
}

/*  KBQryTable								*/
/*  KBQryTable	: Constructor for form query node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  query	: KBQryTable *	: Extant form query			*/
/*  (returns)	: KBQryTable	:					*/

KBQryTable::KBQryTable
	(	KBNode		*parent,
		KBQryTable	*query
	)
	:
	KBQryData	(parent,  query),
	m_server	(this,    "server",     query, KAF_REQD),
	m_table		(this,    "table",      query, KAF_REQD),
	m_primary	(this,    "primary",    query),
	m_ptype		(this,    "ptype",      query),
	m_pexpr		(this,    "pexpr",      query),
	m_where		(this,    "where",      query),
	m_order		(this,    "order",      query),
	m_distinct	(this,	  "distinct",	query)
{
	m_tabPtr	= 0 ;
}

/*  KBQryTable								*/
/*  ~KBQryTable	: Destructor for form query node			*/
/*  (returns)	:		:					*/

KBQryTable::~KBQryTable ()
{
	DELOBJ	(m_tabPtr) ;
}

/*  KBQryTable								*/
/*  replicate	: Replicate this table query				*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New table query node			*/

KBNode	*KBQryTable::replicate
	(	KBNode	*_parent
	)
{
	return	new KBQryTable (_parent, this) ;
}

/*  KBQryTable								*/
/*  loadQuery	: Load query 						*/
/*  (returns)	: bool		: Success				*/

bool	KBQryTable::loadQuery ()
{
	m_qryLevels.clear  () ;
	DELOBJ	   (m_tabPtr) ;

	m_tabPtr = new KBTable
		   (	this, m_table  .getValue(), 
			      "",
			      m_primary.getValue(),
			      m_ptype  .getValue(),
			      m_pexpr  .getValue(),
			      "",
			      "",
			      m_where  .getValue(),
			      m_order  .getValue()
		   ) ;

	KBQryLevel *qlvl = new KBQryLevel (getParent(), 0, dbLink, 0, m_tabPtr) ;
	qlvl->setDistinct (m_distinct.getBoolValue()) ;
	qlvl->setLimit	  (m_limit   .getIntValue ()) ;

	m_qryLevels.append (qlvl) ;

	m_svrName = m_server.getValue() ;
	return	linkServer  (m_svrName) ;
}

/*  KBQryTable								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBQryTable::showAs
	(	KB::ShowAs	mode
	)
{
#if	! __KB_RUNTIME
	if (mode == KB::ShowAsDesign)
	{	m_qryLevels.clear () ;
		DELOBJ    (m_tabPtr) ;
	}
#endif
	KBNode::showAs (mode) ;
}

/*  KBQryTable								*/
/*  printNode	: Output node XML					*/
/*  text	: QString &	: Result string				*/
/*  indent	: int		: Indent level of this node		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBQryTable::printNode
	(	QString		&text,
		int		indent,
		bool		flat
	)
{
	QString		nodeText ;

	if (flat)
		if (m_ptype.getIntValue() != KBTable::PrimaryKey)
			KBError::EWarning
			(	TR("Table query does not specify primary key column"),
				QString(TR("Server: %1, table: %2, Path: %3"))
					.arg(m_server.getValue())
					.arg(m_table .getValue())
					.arg(getPath()),
				__ERRLOCN
			)	;

	/* We override this method since we don't want to print a child	*/
	/* KBTable nodes, which do not exist in the document.		*/
	text	+= QString("%1<%2").arg("", indent).arg(getElement()) ;

	for (uint idx0 = 0 ; idx0 < m_attribs.count () ; idx0 += 1)
		m_attribs.at(idx0)->printAttr(text, nodeText, indent + 2, flat) ;

	text	+= ">\n"    ;

	for (uint idx1 = 0 ; idx1 < m_children.count () ; idx1 += 1)
		if (m_children.at(idx1)->isTable() == 0)
			m_children.at(idx1)->printNode (text, indent + 2, flat) ;

	text	+= nodeText ;
	text	+= QString("%1</%2>\n").arg("", indent).arg(getElement()) ;
}

/*  KBQryTable								*/
/*  getComment	: Get brief descriptive comment				*/
/*  qryLvl	: uint		: Query level				*/
/*  (returns)	: QString	: Comment				*/

QString	KBQryTable::getComment
	(	uint
	)
{
	return	QString("Table: %1").arg(m_table.getValue()) ;
}

#if	! __KB_RUNTIME

/*  KBQryTable								*/
/*  whatsThisExtra							*/
/*		: Extra stuff for pseudo-what's this popup		*/
/*  info	: QValueList<KBWhatsThisPair> &	: List of items		*/
/*  (returns)	: void				:			*/

void	KBQryTable::whatsThisExtra
	(	QValueList<KBWhatsThisPair> 	&info
	)
{
	info.append (KBWhatsThisPair(TR("Table"), m_table.getValue())) ;
}

/*  KBQryTable								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar	*	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBQryTable::propertyDlg
	(	cchar		*iniAttr
	)
{
	extern	bool	queryTablePropDlg (KBQryTable *, cchar *, QPtrList<KBAttr> &) ;
	
	if (!queryTablePropDlg (this, TR("Query"), m_attribs))
		return false ;

	m_qryLevels.clear() ;
	dropServer 	 () ;

	return true   ;
}
#endif


NEWNODE(QryTable, (cchar *)0, 0, KNF_FORM|KNF_REPORT)
