/***************************************************************************
    file	         : kb_qtbutton.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_qtbutton.moc"
#else
#include	"kb_qtbutton.h"
#endif

#include	"kb_button.h"
#include	"kb_display.h"
#include	"kb_writer.h"

#include	<qtooltip.h>
#include	<qpainter.h>


#define		__DEBUG_QTBUTTON	(0)
#if		__DEBUG_QTBUTTON
#include	"kb_dprintf.h"
#endif


/*  KBCtrlButton							*/
/*  KBCtrlButton: Constructor for pushbutton wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  button	: KBButton *	: Parent button				*/
/*  design	: bool		: Design mode				*/
/*  (returns)	: KBCtrlButton	:					*/

KBCtrlButton::KBCtrlButton
	(	KBDisplay	*display,
		KBButton	*button
	)
	:
	KBControl
	(	display,
		button
	),
	m_button	(button)
{
	m_qtButton = new RKPushButton (display->getDisplayWidget()) ;
	setupWidget (m_qtButton) ;

	RKPushButton::connect
	(	m_qtButton,
		SIGNAL(clicked()),
		m_button,
 		SLOT  (clicked())
	)	;
	RKPushButton::connect
	(	m_qtButton,
		SIGNAL(toggled(bool)),
		m_button,
 		SLOT  (toggled(bool))
	)	;

	m_rolled = false ;
}

/*  KBCtrlButton							*/
/*  ~KBCtrlButton: Destructor for button widget				*/
/*  (returns)	 :		:					*/

KBCtrlButton::~KBCtrlButton ()
{
}

bool	KBCtrlButton::updateButton ()
{
	const QPixmap &pm = m_rolled ? m_pm1 : m_pm2 ;

	DPRINTF
	((	"KBCtrlButton::updateButton: r=%d pm1.null=%d pm2.null=%d t=[%s]\n",
		m_rolled,
		m_pm1.isNull(),
		m_pm2.isNull(),
		m_currentText.latin1()
	))	;

	if (pm.isNull())
	{
		m_qtButton->setText (m_currentText) ;
		return	false	;
	}

	if (m_currentText.isEmpty())
	{
		m_qtButton->setPixmap (pm) ;
		return	true	;
	}

	QPixmap	 pmu (pm  ) ;
	QPainter pmp (&pmu) ;
	pmp.setFont  (*m_button->getFont(true)) ;
	pmp.drawText
	(	QRect(0, 0, pmu.width(), pmu.height()),
		Qt::AlignCenter|Qt::SingleLine|Qt::ShowPrefix,
		m_currentText
	)	;

	m_qtButton->setPixmap (pmu) ;
	return	true	;
}

/*  KBCtrlButton							*/
/*  setPixmaps	: Set normal and rollover images			*/
/*  pm1		: const QPixmap & : Normal image			*/
/*  pm2		: const QPixmap & : Rollover image			*/
/*  (returns)	: void		  :					*/

void	KBCtrlButton::setPixmaps
	(	const QPixmap	&pm1,
		const QPixmap	&pm2
	)
{
	m_pm1	= pm1	;
	m_pm2	= pm2	;
	updateButton () ;
}

/*  KBCtrlButton							*/
/*  eventFilter	: Local event filter					*/
/*  o		: QObject *	: Object receiving event		*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: void		  :					*/

bool	KBCtrlButton::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (o == m_qtButton)
		switch (e->type())
		{
			case QEvent::Enter :
				m_rolled  = true  ;
				if (updateButton()) return true ;
				break	;

			case QEvent::Leave :
				m_rolled  = false ;
				if (updateButton()) return true ;
				break	;

			default	:
				break	;
		}

	return	KBControl::eventFilter (o, e) ;
}

/*  KBCtrlButton							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlButton::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;
	setupProperties	  ()	 ;
}

/*  KBCtrlButton							*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: void		:					*/

void	KBCtrlButton::setupProperties ()
{
	m_currentText = m_button->getAttrVal("text") ;
	m_qtButton->setToggleButton (m_button->getToggle()) ;
	m_button  ->loadPixmaps     () ;

	updateButton () ;

#if	__KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		QToolTip::remove (m_qtButton) ;
		return	;
	}
#endif

	QString	tooltip	= m_button->getAttrVal("tooltip") ;
	if (!tooltip.isEmpty()) QToolTip::add (m_qtButton, tooltip) ;
}

/*  KBCtrlButton							*/
/*  setValue	: Set button text					*/
/*  value	: const KBValue &: Value for text			*/
/*  (returns)	: void		 :					*/

void	KBCtrlButton::setValue
	(	const KBValue	&value
	)
{
	m_currentText = value.getRawText () ;
	updateButton () ;
}

/*  KBCtrlButton							*/
/*  setToopTip	: Set tooptip text					*/
/*  toolTip	: const QString & : Tooptip text			*/
/*  (returns)	: void		  :					*/

void	KBCtrlButton::setToolTip
	(	const QString	&tooltip
	)
{
	if (m_showing == KB::ShowAsData)
	{
		QToolTip::remove (m_qtButton) ;
		if (!tooltip.isEmpty()) QToolTip::add (m_qtButton, tooltip) ;
	}
}

/*  KBCtrlButton							*/
/*  animateClick: Animate a button click				*/
/*  (returns)	: void		:					*/

void	KBCtrlButton::animateClick ()
{
	m_qtButton->animateClick () ;
}

/*  KBCtrlButton							*/
/*  getValue	: Get button text					*/
/*  (returns)	: KBValue	: Button text				*/

KBValue	KBCtrlButton::getValue ()
{
	return	KBValue(m_currentText) ;
}

/*  KBCtrlButton							*/
/*  write	: Write field 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlButton::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

/*  KBCtrlButton							*/
/*  setOn	: Set the button on or off				*/
/*  on		: bool		: Required state			*/
/*  (returns)	: void		:					*/

void	KBCtrlButton::setOn
	(	bool		on
	)
{
	m_qtButton->setOn (on)	;
}

/*  KBCtrlButton							*/
/*  isOn	: Get button on/off state				*/
/*  (returns)	: bool		: State					*/

bool	KBCtrlButton::isOn ()
{
	return	m_qtButton->isOn () ;
}
