/***************************************************************************
    file	         : kb_qtcheck.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include 	"kb_qtcheck.moc"
#else
#include 	"kb_qtcheck.h"
#endif

#include	"kb_check.h"
#include	"kb_block.h"
#include	"kb_display.h"
#include	"kb_writer.h"
#include	"kb_navigator.h"

#include	<qpixmap.h>
#include	<qstyle.h>


#define		__DEBUG_QTCHECK	(0)
#if		__DEBUG_QTCHECK
#include	"kb_dprintf.h"
#endif


/*  KBCtrlCheck								*/
/*  KBCtrlCheck	: Constructor for check box wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  check	: KBCheck *	: Parent check				*/
/*  drow	: uint		: Row number in field			*/
/*  (returns)	: KBCtrlCheck	:					*/

KBCtrlCheck::KBCtrlCheck
	(	KBDisplay	*display,
		KBCheck		*check,
		uint		drow
	)
	:
	KBControl
	(	display,
		check,
		drow
	),
	m_check		(check)
{
	m_checkBox = new RKCheckBox (display->getDisplayWidget()) ;
	setupWidget (m_checkBox) ;

	connect
	(	m_checkBox,
		SIGNAL	(clicked   ()),
		SLOT	(userChange())
	)	;
}

/*  KBCtrlCheck								*/
/*  ~KBCtrlCheck: Destructor for check box wrapper			*/
/*  (returns)	:		:					*/

KBCtrlCheck::~KBCtrlCheck ()
{
}

/*  KBCtrlCheck								*/
/*  getValue	: Get value						*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCtrlCheck::getValue ()
{
	DPRINTF
	((	"KBCtrlCheck::getValue: [%s] c=[%d] [%s]\n",
		(cchar *)m_check->getName(),
		m_checkBox->isChecked(),
		(cchar *)m_check->getFieldType()->getDescrip()
	))	;
	/* If the control is unchecked and the initial value was null	*/
	/* then return a null string.					*/
	if (!m_checkBox->isChecked() && getIniValue().isNull())
		return	KBValue (m_check->getFieldType()) ;

	return	KBValue (m_checkBox->isChecked(), m_check->getFieldType()) ;
}

/*  KBCtrlCheck								*/
/*  setValue	: Set value display row					*/
/*  value	: const KBValue &: Value to set				*/
/*  (returns)	: void		 :					*/

void	KBCtrlCheck::setValue
	(	const KBValue	&value
	)
{
	DPRINTF
	((	"KBCtrlCheck::setValue: %s: %d: %s=%d\n",
		m_check->getName().latin1(),
		m_drow,
		value.getRawText().latin1(),
		value.isTrue()
	))	;
	{
		KBInSetValueLock lock (*this) ;
		m_checkBox->setChecked (value.isTrue ()) ;
	}

	KBControl::setValue (value) ;
}

/*  KBCtrlCheck								*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBCtrlCheck::clearValue
	(	bool		query
	)
{
	{
		KBInSetValueLock lock    (*this) ;
		m_checkBox  ->setChecked (false) ;
		if (m_showing == KB::ShowAsData)
			m_layoutItem->setValid   (isValid (false)) ;
	}

	KBControl::clearValue (query) ;
}

/*  KBCtrlCheck								*/
/*  isEmpty	: See if control is empty				*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlCheck::isEmpty ()
{
	return	! m_checkBox->isChecked () ;
}

/*  KBCtrlCheck								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlCheck::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;

	{
		KBInSetValueLock lock (*this) ;
		setupProperties	 () ;
	}
}

/*  KBCtrlCheck								*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: void		:					*/

void	KBCtrlCheck::setupProperties ()
{
	m_layoutItem->setValidatorMode	(m_check) ;
	m_checkBox  ->setChecked	(false  ) ;

#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		m_checkBox->setReadOnly (false)	;
		return	;
	}
#endif

	m_checkBox->setReadOnly (m_check->isReadOnly()) ;
}

/*  KBCtrlCheck								*/
/*  write	: Write field 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  extra	: int &		: Return extra space			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlCheck::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		QRect	 ur (rect.left(), rect.top(), rect.height(), rect.height()) ;
		QPixmap	 pm (ur.size()) ;
		pm.fill  () ;

		if (value.isTrue())
		{
			QPainter p(&pm)	;

			m_checkBox->style().drawPrimitive
			(	QStyle::PE_CheckMark,
				&p,
				QRect(QPoint(0,0), ur.size()),
				m_checkBox->colorGroup(),
				QStyle::Style_Default|QStyle::Style_Down,
				QStyleOption::Default
			)	; 
		}

		KBWriterItem *wi = new KBWriterPixmap (writer, rect, pm, -1) ;
		wi->setParent (m_check, m_check->getBlock()->getCurQRow()) ;

		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

bool	KBCtrlCheck::isNavigationKey
	(	QKeyEvent	*k
	)
{
	return	KBNavigator::isNavigationKey (k) ;
}

/*  KBCtrlCheck								*/
/*  setReadOnly	: Explicitely set readonly state			*/
/*  readOnly	: bool		: Required state			*/
/*  (returns)	: void		:					*/

void	KBCtrlCheck::setReadOnly
	(	bool		readOnly,
		const QColor	&fgColor,
		const QColor	&bgColor
	)
{
	if (m_checkBox != 0) m_checkBox->setReadOnly (readOnly) ;
	KBControl::setReadOnly (readOnly, fgColor, bgColor) ;
}

bool	KBCtrlCheck::isReadOnly ()
{
	return	m_checkBox == 0 ? false : m_checkBox->isReadOnly () ;
}

bool	KBCtrlCheck::isChecked ()
{
	return	m_checkBox->isChecked() ;
}

void	KBCtrlCheck::setChecked
	(	bool		checked
	)
{
	m_checkBox->setChecked (checked) ;
}

