/***************************************************************************
    file	         : kb_qtchoice.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_qtchoice.moc"
#else
#include	"kb_qtchoice.h"
#endif

#include	"kb_choice.h"
#include	"kb_display.h"
#include	"kb_block.h"
#include	"kb_navigator.h"

#include	<qlineedit.h>
#include	<qlistbox.h>


#define		__DEBUG_QTCHOICE	(0)
#if		__DEBUG_QTCHOICE
#include	"kb_dprintf.h"
#endif


/*  KBCtrlChoice							*/
/*  KBCtrlChoice: Constructor for choice combo box wrapper		*/
/*  display	: KBDisplay *	: Parent display			*/
/*  choice	: KBChoice *	: Parent choice item			*/
/*  drow	: uint		: Row number in field			*/
/*  (returns)	: KBCtrlChoice	:					*/

KBCtrlChoice::KBCtrlChoice
	(	KBDisplay	*display,
		KBChoice	*choice,
		uint		drow
	)
	:
	KBControl
	(	display,
		choice,
		drow
	),
	m_choice	(choice)
{
	m_comboBox	= 0 ;
	m_listBox	= 0 ;
}

/*  KBCtrlChoice							*/
/*  ~KBCtrlChoice: Destructor for choice class				*/
/*  (returns)	 :		:					*/

KBCtrlChoice::~KBCtrlChoice ()
{
}

/*  KBCtrlChoice							*/
/*  setupWidget	: Set up widget						*/
/*  (returns)	: void		:					*/

void	KBCtrlChoice::setupWidget ()
{
	DPRINTF
	((
		"KBCtrlChoice::setupWidget: called\n"
	))	;
	if (getDisplay() == 0)
		return	;

	m_comboBox     = new RKComboBox (getDisplay()->getDisplayWidget()) ;
	m_comboBox->setListBox		(m_listBox = new QListBox (m_comboBox)) ;
	m_comboBox->setInsertionPolicy	(RKComboBox::NoInsertion) ;

	connect
	(	m_comboBox,
		SIGNAL	(activated	(int)),
		SLOT	(deferUserChange())
	)	;

	QLineEdit *le = m_comboBox->lineEdit() ;
	if (le != 0)
		connect
		(	le,
			SIGNAL	(textChanged	(const QString &)),
			SLOT	(userChange	())
		)	;

	KBControl::setupWidget (m_comboBox) ;

//	m_comboBox->show () ;
}

/*  KBCtrlChoice							*/
/*  clearWidget	: Clear down widget					*/
/*  (returns)	: void		:					*/

void	KBCtrlChoice::clearWidget ()
{
	DPRINTF
	((	"KBCtrlChoice::clearWidget: called\n"
	))	;
	KBControl::setValue (getValue()) ;

	m_comboBox = 0	;
	m_listBox  = 0	;

	KBControl::clearWidget () ;
}

/*  KBCtrlChoice							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlChoice::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;

	{
		KBInSetValueLock lock (*this) ;
		setupProperties  () ;
	}

}

void	KBCtrlChoice::setValues
	(	const QStringList 	&values
	)
{
	KBInSetValueLock lock (*this) ;
	m_comboBox->clear            () ;
	m_comboBox->insertStringList (values) ;
}

/*  KBCtrlChoice							*/
/*  setValue	: Set value						*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlChoice::setValue
	(	const KBValue	&value
	)
{
	QString	v	  = value.getRawText() ;
	int	slot	  = m_choice->getValues().findIndex(v) ;

	if (slot < 0)
	{
		for (int p = v.length() - 1 ; p >= 0 ; p -= 1)
			if (v.at(p) != QChar(' '))
			{
				v = v.left (p + 1) ;
				break	;
			}

		slot	  = m_choice->getValues().findIndex(v) ;

		DPRINTF
		((	"KBCtrlChoice::setValue: retry [%s]->[%s]->[%d]\n",
			(cchar *)value.getRawText(),
			(cchar *)v,
			slot
		))	;
	}

	if (m_comboBox != 0)
	{
		KBInSetValueLock lock (*this) ;

		if ((slot < 0) && m_choice->canEdit ())
			m_comboBox->setEditText    (v) ;
		else	m_comboBox->setCurrentItem (slot < 0 ? 0 : slot) ;
	}

	KBControl::setValue (value) 	;
}

/*  KBCtrlChoice							*/
/*  getValue	: Get value						*/
/*  (returns)	: KBValue 	: Value					*/

KBValue	KBCtrlChoice::getValue ()
{
	if (m_comboBox != 0)
	{
		/* If the control is empty and the initial value was	*/
		/* null (as opposed to an empty string), then return a	*/
		/* null string.						*/
		if (m_comboBox->currentText().isEmpty() && getIniValue().isNull())
			return	KBValue (m_choice->getFieldType()) ;

		return	KBValue (m_comboBox->currentText(), m_choice->getFieldType()) ;
	}

	return	KBControl::getValue () ;
}

/*  KBCtrlChoice							*/
/*  clearValue	: Clear value						*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: KBValue 	: Value					*/

void	KBCtrlChoice::clearValue
	(	bool		query
	)
{
	if (m_comboBox != 0)
	{
		KBInSetValueLock lock    (*this) ;
		m_comboBox  ->setCurrentItem (0) ;
		if (m_showing == KB::ShowAsData)
			m_layoutItem->setValid (isValid (false)) ;
	}

	KBControl::clearValue (query) ;
}

/*  KBCtrlChoice							*/
/*  setFocus	: Set focus into widget					*/
/*  (returns)	: void		:					*/

void	KBCtrlChoice::setFocus ()
{
	/* If the edit widget does not exist then we are currently	*/
	/* morphed, so restore the widget.				*/
	if (m_comboBox == 0)
	{
		setupWidget	   () ;
		setupDataProperties() ;
		setValue	   (KBControl::getValue()) ;
	}

	KBControl::setFocus () ;
}

/*  KBCtrlChoice							*/
/*  morphText	: Get text for morphed control				*/
/*  align	: int &		: Return alignment			*/
/*  (returns)	: QString	: Text					*/

QString	KBCtrlChoice::morphText
	(	int		&
	)
{
	return	m_curVal.getRawText () ;
}

/*  KBCtrlChoice							*/
/*  setupDataProperties							*/
/*		: Set up widget for data mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlChoice::setupDataProperties ()
{
	if (m_comboBox != 0)
	{
		m_comboBox  ->clear		() ;
		m_comboBox  ->setReadOnly	(m_choice->isReadOnly()) ;
		m_comboBox  ->insertStringList	(m_choice->getValues ()) ;
		m_comboBox->setEditable		(m_choice->canEdit   ()) ;
		m_layoutItem->setValidatorMode	(m_choice) ;
	}
}

/*  KBCtrlChoice							*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: void		:					*/

void	KBCtrlChoice::setupProperties ()
{
#if	! __KB_RUNTIME
	/* For design view, ensure that we have a widget (if switching	*/
	/* from morphed data view), then set up.			*/
	if (m_showing == KB::ShowAsDesign)
	{
		if (m_comboBox == 0) setupWidget () ;

		m_comboBox->setReadOnly (false) ;
		m_comboBox->setEditable (false) ;
		m_comboBox->clear       ()	;
		if (m_drow == 0) m_comboBox->insertItem (m_choice->getName()) ;

		m_layoutItem->setValidatorMode (m_choice) ;
		return	;
	}
#endif

	if (m_showing == KB::ShowAsData)
	{
		/* Data view, if morphing then ensure that the widget	*/
		/* is morphed out. If not morphing, then make sure we	*/
		/* have the widget, and set up appropriate properties.	*/
		if (m_choice->isMorphing ())
		{
			if (m_comboBox != 0) clearWidget () ;
			return	;
		}

		if (m_comboBox == 0) setupWidget () ;
		setupDataProperties() ;
	}
}

/*  KBCtrlChoice							*/
/*  eventFilter	: Event filter on edit control				*/
/*  o		: QObject *	: Object being filtered			*/
/*  e		: QEvent *	: Event					*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBCtrlChoice::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (m_showing != KB::ShowAsData)
		return	KBControl::eventFilter (o, e) ;

	if
	(	(o != m_comboBox) &&
		(o != m_listBox ) &&
		(o != m_comboBox->lineEdit())
	)
		return	KBControl::eventFilter (o, e) ;


	if (o == m_listBox)
		if (m_listBox->isVisible())
			if ((e->type() == QEvent::KeyPress) || (e->type() == QEvent::KeyRelease))
				return	false	;

	if (e->type() == QEvent::FocusOut)
	{
		/* If morphing, the widget must be cleared. Do not do	*/
		/* this immediately, else we get focus problems.	*/
		if (m_choice->isMorphing ()) startMorphTimer () ;

		return	KBControl::eventFilter (o, e) ;
	}

	if (e->type() == QEvent::FocusIn)
	{
		stopMorphTimer () ;
		return	KBControl::eventFilter (o, e) ;
	}

	return	KBControl::eventFilter (o, e) ;
}

bool	KBCtrlChoice::isNavigationKey
	(	QKeyEvent	*k
	)
{
	return	KBNavigator::isNavigationKey (k) ;
}

/*  KBCtrlChoice							*/
/*  setCurrentItem							*/
/*		: Set current item by index				*/
/*  item	: uint		: Item index				*/
/*  (returns)	: void		:					*/

void	KBCtrlChoice::setCurrentItem
	(	uint		item
	)
{
	if (m_comboBox != 0)
		m_comboBox->setCurrentItem (item) ;

	KBControl::setValue  (getValue()) ;
}

/*  KBCtrlChoice							*/
/*  currentItem	: Get current item index				*/
/*  (returns)	: uint		: Item index				*/

uint	KBCtrlChoice::currentItem ()
{
	if (m_comboBox == 0)
		return	0 ;

	return	m_comboBox->currentItem	() ;
}

/*  KBCtrlChoice							*/
/*  setReadOnly	: Explicitely set readonly state			*/
/*  readOnly	: bool		: Required state			*/
/*  (returns)	: void		:					*/

void	KBCtrlChoice::setReadOnly
	(	bool		readOnly,
		const QColor	&fgColor,
		const QColor	&bgColor
	)
{
	DPRINTF
	((	"KBCtrlChoice::setReadOnly: %p: %d\n",
		(void *)m_comboBox,
		readOnly
	))	;
	if (m_comboBox != 0) m_comboBox->setReadOnly (readOnly) ;
	KBControl::setReadOnly (readOnly, fgColor, bgColor) ;
}

bool	KBCtrlChoice::isReadOnly ()
{
	return	m_comboBox == 0 ? false : m_comboBox->isReadOnly () ;
}
