/***************************************************************************
    file	         : kb_qtfield.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	<qcursor.h>
#include	<qvalidator.h>
#include	<qtimer.h>

#if		__KB_AUTOMOC
#include 	"kb_qtfield.moc"
#else
#include 	"kb_qtfield.h"
#endif

#include	"kb_field.h"
#include	"kb_writer.h"
#include	"kb_block.h"
#include	"kb_display.h"
#include	"kb_docroot.h"
#include	"kb_attrimage.h"
#include	"kb_helper.h"

#include	"rk_pushbutton.h"
#include	"rk_lineedit.h"


#define		__DEBUG_QTFIELD	(0)
#if		__DEBUG_QTFIELD
#include	"kb_dprintf.h"
#endif


/*  KBCtrlField								*/
/*  KBCtrlField	: Constructor for line edit wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  field	: KBField *	: Parent field				*/
/*  drow	: uint		: Row number in field			*/
/*  (returns)	: KBCtrlField	:					*/

KBCtrlField::KBCtrlField
	(	KBDisplay	*display,
		KBField		*field,
		uint		drow
	)
	:
	KBControl
	(	display,
		field,
		drow
	),
	m_field		(field)
{
	DPRINTF
	((	"KBCtrlField::KBCtrlField: %s: creating row %d\n",
		m_field->getName().latin1(),
		drow
	))	;
	m_lEdit		= 0 ;
	m_inputMask	= 0 ;
	m_bHelp		= 0 ;
}

/*  KBCtrlField								*/
/*  ~KBCtrlField: Desctructor for field class				*/
/*  (returns)	:		:					*/

KBCtrlField::~KBCtrlField ()
{
	DPRINTF
	((	"KBCtrlField::~KBCtrlField: %s: row %d: w=%p\n",
		m_field->getName().latin1(),
		(void *)getWidget()
	))	;

	if (getWidget()) DELOBJ(m_bHelp) ;
}

/*  KBCtrlField								*/
/*  setupWidget	: Set up widget						*/
/*  (returns)	: void		:					*/

void	KBCtrlField::setupWidget ()
{
	DPRINTF
	((
		"KBCtrlField::setupWidget: %s: display=%p\n",
		 m_field->getName().latin1(),
		(void *)getDisplay()
	))	;

	if (getDisplay() == 0)
		return	;

	m_lEdit		= new RKLineEdit     (getDisplay()->getDisplayWidget()) ;
	m_inputMask	= new KBMaskedInput  (m_lEdit) ;
	m_bHelp		= 0 ;
	m_helper	= QString::null ;

	m_lEdit->setSizePolicy	(QSizePolicy (QSizePolicy::Preferred, QSizePolicy::Preferred)) ;

	connect
	(	m_inputMask,
		SIGNAL	(textChangedTo(const QString &)),
		SLOT	(userChange   ())
	)	;
	connect
	(	m_lEdit,
		SIGNAL	(returnPressed()),
		SLOT	(returnPressed())
	)	;

	DPRINTF
	((	"KBCtrlField::setupWidget: %s: m_lEdit=%p\n",
		 m_field->getName().latin1(),
		(void *)m_lEdit
	))	;

	KBControl::setupWidget (m_lEdit) ;
}

/*  KBCtrlField								*/
/*  clearWidget	: Clear down widget					*/
/*  (returns)	: void		:					*/

void	KBCtrlField::clearWidget ()
{
	DPRINTF
	((	"KBCtrlField::clearWidget: %s\n",
		 m_field->getName().latin1()
	))	;

	KBControl::setValue (getValue()) ;

	m_lEdit	    = 0 ;
	m_inputMask = 0 ;

	KBControl::clearWidget () ;
}

/*  KBCtrlField								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlField::showAs
	(	KB::ShowAs	mode
	)
{
	DPRINTF
	((	"KBCtrlField::showAs: %s: mode=%d, drow=%d\n",
		 m_field->getName().latin1(),
		mode,
		m_drow
	))	;

	KBControl::showAs (mode) ;

	{
		KBInSetValueLock lock (*this) ;
		setupProperties  () ;
	}

	if (m_layoutItem)
		m_layoutItem->setHelper (0) ;
	DELOBJ	(m_bHelp) ;
}

/*  KBCtrlField								*/
/*  setValue	: Set field value					*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlField::setValue
	(	const KBValue	&value
	)
{
	DPRINTF
	((	"KBCtrlField::setValue: %s: %s: [%s]->[%s]\n",
		 m_field->getName().latin1(),
		(cchar *)value.getType()->getDescrip(),
		(cchar *)value.getRawText(),
		(cchar *)value.getText(m_field->getFormat())
	))	;

	if (m_lEdit != 0)
	{
		KBInSetValueLock lock (*this) ;
		m_lEdit->setText (value.getText(m_field->getFormat())) ;
	}

	KBControl::setValue (value) ;
}

/*  KBCtrlField								*/
/*  getValue	: Get field value					*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCtrlField::getValue ()
{
	if (m_lEdit != 0)
	{
		QString	text	= m_lEdit->text() ;

		/* If the control is empty and the initial value was	*/
		/* null (as opposed to an empty string), then return a	*/
		/* null string.						*/
		if (text.isEmpty() && getIniValue().isNull())
			return	KBValue (m_field->getFieldType()) ;

		/* If the "deformat" option is set then we try to undo	*/
		/* the formatting using the same string used to do the	*/
		/* formatting.						*/
		if (m_field->getDeFormat ())
			return	KBValue
				(	text,
					m_field->getFieldType(),
					m_field->getFormat   ()
				)	;


		return	KBValue (text, m_field->getFieldType()) ;
	}

	return	KBControl::getValue () ;
}

/*  KBCtrlField								*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBCtrlField::clearValue
	(	bool	query
	)
{
	if (m_lEdit != 0)
	{
		DPRINTF
		((	"KBCtrlField::clearValue: %s: drow=%d\n",
			m_field->getName().latin1(),
			m_drow
		))	;

		KBInSetValueLock lock  (*this) ;
		m_lEdit	    ->setText  ("")    ;
		if (m_showing == KB::ShowAsData)
			m_layoutItem->setValid (isValid (false)) ;
	}

	KBControl::clearValue (query) ;
}

/*  KBCtrlField								*/
/*  changed	: Get value changed status				*/
/*  (returns)	: bool		: Value changed				*/

bool	KBCtrlField::changed ()
{
	KBValue	curValue = getValue   () ;
	KBValue	iniValue = getIniValue() ;

	/* Special case: This handles the situation where the inital	*/
	/* value is null, otherwise we get empty != null.		*/
	if (curValue.isEmpty() && iniValue.isEmpty())
		return	false	;


	/* If there is a format then format the initial value so that	*/
	/* we can compare apples with apples rather than apples with	*/
	/* oranges.							*/
	QString	format	= m_field->getFormat() ;
	if (!format.isEmpty ())
	{
		iniValue = iniValue.getText (format, 0) ;

		DPRINTF
		((	"KBCtrlField::changed: %s: trimmed [%s][%s]\n",
			m_field->getName   ().latin1(),
			curValue.getRawText().latin1(),
			iniValue.getRawText().latin1()
		))	;
	}

	DPRINTF
	((	"KBCtrlField::changed: %s: %d: [%s][%s]\n",
		m_field->getName   ().latin1(),
		m_drow,
		curValue.getRawText().latin1(),
		iniValue.getRawText().latin1()
	))	;

	return	curValue != iniValue ;
}

/*  KBCtrlField								*/
/*  loadImage	: Load a named image					*/
/*  image	: const QString & : Image name				*/
/*  (returns)	: QPixmap	  : Image or null on error		*/

QPixmap	KBCtrlField::loadImage
	(	const QString	&image
	)
{
	if (!image.isEmpty())
	{
		QStringList	iparts	= QStringList::split ('.', image) ;

		DPRINTF
		((	"KBCtrlField::loadImage: %s: [%s][%s]\n",
			 m_field->getName().latin1(),
			(cchar *)iparts[0],
			(cchar *)iparts[1]
		))	;

		KBError		error	;
		QPixmap		pixmap	= KBAttrImage::pixmapFromLocation
					  (	m_field->getDocRoot(),
						iparts[0],
						iparts[1],
						error
					  )	;

		if (pixmap.isNull()) error.DISPLAY() ;
		return	pixmap	;
	}

	return	QPixmap	() ;
}

/*  KBCtrlField	:							*/
/*  eventFilter	: Event filter on edit control				*/
/*  o		: QObject *	: Object being filtered			*/
/*  e		: QEvent *	: Event					*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBCtrlField::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (m_showing != KB::ShowAsData)
		return	KBControl::eventFilter (o, e) ;

	if (o != m_lEdit)
		return	KBControl::eventFilter (o, e) ;


	if (e->type() == QEvent::FocusIn)
	{
		QFocusEvent *fe = (QFocusEvent *)e ;

		/* Get the focus-caret setting; if this is default then	*/
		/* select all the text if focus is arriving on account	*/
		/* of something like tabbing.				*/
		KBField::FocusCaret fc = m_field->getFocusCaret () ;
		if (fc == KBField::Default)
			if (fe->reason() == QFocusEvent::Tab)
				fc = KBField::SelectAll ;

		int l = m_lEdit->text().length () ;
		switch (fc)
		{
			case KBField::AtStart	:
				setSelection (0, 0) ;
				break	;

			case KBField::AtEnd	:
				setSelection (l, 0) ;
				break	;

			case KBField::SelectAll :
				setSelection (0, l) ;
				break	;

			default	:
				break	;
		}

		DPRINTF
		((	"KBCtrlField::eventFilter: %s: sd=%d bh=%p h=[%s]\n",
			 m_field->getName().latin1(),
			m_showing == KB::ShowAsData,
			(void *)m_bHelp,
			m_field->getHelper().latin1()
		))	;

		/* See if we need to create a helper button. It would	*/
		/* be nice to do this in ::showAs, but that seems to	*/
		/* cause a crash in QScrollView.			*/
		if ((m_showing == KB::ShowAsData) && !m_bHelp && !m_field->getHelper().isEmpty())
		{
			QStringList hbits = QStringList::split (';', m_field->getHelper ()) ;
			while (hbits.size() < 4) hbits.append (QString::null) ;

			if (KBHelperReg::helperExists (hbits[0]))
			{
				DPRINTF
				((	"KBCtrlField::eventFilter: %s: create helper: %s: [%s][%s][%s]\n",
					 m_field->getName().latin1(),
					m_field->getHelper().latin1(),
					hbits[0].latin1(),
					hbits[1].latin1(),
					hbits[2].latin1()
				))	;

				m_helper   = hbits[0] ;
				m_bHelp	   = new RKPushButton(getDisplay()->getDisplayWidget()) ;

				m_normal   = loadImage (hbits[1]) ;
				m_rollover = loadImage (hbits[2]) ;

				if (m_normal.isNull())
					m_bHelp->setText   ("..")	;					
				else	m_bHelp->setPixmap (m_normal)	;

				m_bHelp->setFixedWidth	(m_lEdit->height()) ;
				m_bHelp->setFixedHeight	(m_lEdit->height()) ;

				connect
				(	m_bHelp,
					SIGNAL(clicked    ()),
					SLOT  (helpClicked())
				)	;

				m_layoutItem->setHelper (m_bHelp) ;
			}
			else	m_helper   = QString::null ;
		}

		DPRINTF
		((	"KBCtrlField::eventFilter: %s: bh=%p\n",
			 m_field->getName().latin1(),
			(void *)m_bHelp
		))	;

		if (m_bHelp)
			m_layoutItem->showHelper (true) ;

		return	KBControl::eventFilter (o, e) ;
	}

	if (e->type() == QEvent::FocusOut)
	{
		switch (m_field->getMapCase())
		{
			case KBField::MapUpper	:
				m_lEdit->setText (m_lEdit->text().upper()) ;
				break	;

			case KBField::MapLower	:
				m_lEdit->setText (m_lEdit->text().lower()) ;
				break	;

			default	:
				break	;
		}

		/* On focus out, hide the helper button if one is on	*/
		/* display.						*/
		if (m_bHelp) m_layoutItem->showHelper (false) ;

		/* If morphing, the widget must be cleared. Do not do	*/
		/* this immediately, else we get focus problems.	*/
		if (m_field->isMorphing ()) startMorphTimer () ;

		return	KBControl::eventFilter (o, e) ;
	}

	if ((e->type() == QEvent::KeyPress) || (e->type() == QEvent::KeyRelease))
	{
		/* Normally Enter/Return triggers field navigation, but	*/
		/* if there is on "onreturn" event skip navigation	*/
		/* processing.						*/
		int	key = ((QKeyEvent *)e)->key() ;

		DPRINTF
		((	"KBCtrlField::eventFilter: %s: keypress: %04x r=%d or=%d\n",
			 m_field->getName().latin1(),
			key,
			(key == Qt::Key_Enter) || (key == Qt::Key_Return),
			!m_field->getAttrVal("onreturn").isEmpty()
		))	;

		if ((key == Qt::Key_Enter) || (key == Qt::Key_Return))
			if (!m_field->getAttrVal("onreturn").isEmpty())
				return	false	;
	}


	return	KBControl::eventFilter (o, e) ;
}

/*  KBCtrlField								*/
/*  helpClicked	: Helper button clicked					*/
/*  (returns)	: void		:					*/

void	KBCtrlField::helpClicked ()
{
	KBValue	value1	= KBValue
			  (	m_lEdit->text(),
				m_field->getFieldType(),
				m_field->getDeFormat () ?
					m_field->getFormat() :
					QString::null
			  )	;

	QString	resval	= KBHelperDlg::run
			  (	m_helper,
				value1  .getRawText(),
				m_field->getDocRoot()->getDocLocation()
			  )	;
	if (resval.isNull()) return ;


	KBValue	value2	= KBValue (resval, m_field->getFieldType()) ;

	DPRINTF
	((	"KBCtrlField::helpClicked: resval=[%s] [%s]->[%s]\n",
		resval.latin1(),
		m_field->getFormat().latin1(),
		value2.getText(m_field->getFormat()).latin1()
	))	;

	m_lEdit->setText    (value2.getText(m_field->getFormat ())) ;
	m_field->onHelper   (m_field->getBlock()->getCurDRow() + m_drow, m_lEdit->text()) ;
}

/*  KBCtrlField								*/
/*  returnPressed: Used presses return/enter				*/
/*  (returns)	 : void		:					*/

void	KBCtrlField::returnPressed ()
{
	DPRINTF
	((	"KBCtrlField::returnPressed: %s: %d %d [%s]\n",
		 m_field->getName().latin1(),
		m_inSetValue,
		m_showing == KB::ShowAsData,
		m_lEdit->text().latin1()
	))	;
	/* NOTE: See the code in ::eventFilter ref. return/enter being	*/
	/*	 normally a navigation event.				*/
	if (!m_inSetValue && (m_showing == KB::ShowAsData))
		m_field->returnPressed
		(	m_field->getBlock()->getCurDRow() + m_drow,
			m_lEdit->text()
		)	;
}



/*  KBCtrlField								*/
/*  write	: Write field 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  extra	: int &		: Return extra space			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlField::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		KBWriterItem *wi = new KBWriterText
				   (	writer,
					rect,
					m_field->getPalette(true),
					m_field->getFont   (true),
					value.isNull () ?
						QString::null :
						value.getText(m_field->getFormat()),
					m_field->getAlign  ()|Qt::AlignVCenter,
					fSubs
				)	;
		wi->setParent   (m_field, m_field->getBlock()->getCurQRow()) ;
		writerSetFrame  (wi, 0, 0) ;

		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

/*  KBCtrlField								*/
/*  setFocus	: Set focus into widget					*/
/*  (returns)	: void		:					*/

void	KBCtrlField::setFocus ()
{
	DPRINTF
	((	"KBCtrlField::setFocus: %s: m_lEdit=%p\n",
		 m_field->getName().latin1(),
		(void *)m_lEdit
	))	;

	/* If the edit widget does not exist then we are currently	*/
	/* morphed, so restore the widget.				*/
	if (m_lEdit == 0)
	{
		setupWidget	   () ;
		setupDataProperties() ;
		setValue	   (KBControl::getValue()) ;

		m_lEdit->show	   () ;
		m_lEdit->setEnabled(isEnabled()) ;
	}

	KBControl::setFocus () ;
}


/*  KBCtrlField								*/
/*  morphText	: Get text for morphed control				*/
/*  align	: int &		: Return alignment			*/
/*  (returns)	: QString	: Text					*/

QString	KBCtrlField::morphText
	(	int		&align
	)
{
	align = m_field->getAlign   () ;
	return	m_curVal.getRawText () ;
}

/*  KBCtrlField								*/
/*  setupDataProperties							*/
/*		: Set up widget for data mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlField::setupDataProperties ()
{
	DPRINTF
	((	"KBCtrlField::setupDataProperties: %s: %d: ro=%d\n",
		m_field->getName().latin1(),
		m_drow,
		m_field->isReadOnly()
	))	;

	if (m_lEdit != 0)
	{
		m_lEdit	    ->setText		(QString::null)	   ;
		m_lEdit	    ->setCursor		(Qt::ibeamCursor)  ;
		m_lEdit	    ->setEchoMode	(m_field->isPasswd  () ? RKLineEdit::Password : RKLineEdit::Normal) ;
		m_lEdit	    ->setReadOnly	(m_field->isReadOnly())	;
		m_lEdit	    ->setAlignment	(m_field->getAlign  ()) ;
		m_inputMask ->setEnabled	(true)		   ;
		m_inputMask ->setInputMask	(m_field->getAttrVal("mask")) ;
		m_layoutItem->setValidatorMode	(m_field) ;

		if (m_field->getAttrVal("frame").isEmpty())
			m_lEdit->setFrame (true) ;
		else	ctrlSetFrame	  (m_lEdit, 0, 0) ;
	}
}

/*  KBCtrlField								*/
/*  setupProperties							*/
/*		: Set properties after design change or mode switch	*/
/*  (returns)	: void		:					*/

void	KBCtrlField::setupProperties ()
{
#if	! __KB_RUNTIME
	/* For design view, ensure that we have a widget (if switching	*/
	/* from morphed data view), then set up.			*/
	if (m_showing == KB::ShowAsDesign)
	{
		DPRINTF
		((	"KBCtrlField::setupProperties: %s: m_lEdit=%p drow=%d\n",
			 m_field->getName().latin1(),
			(void *)m_lEdit,
			m_drow
		))	;

		if (m_lEdit == 0) setupWidget () ;

		m_lEdit->setCursor    (Qt::arrowCursor)	   ;
		m_lEdit->setEchoMode  (RKLineEdit::Normal) ;
		m_lEdit->setReadOnly  (true)		   ;
		m_lEdit->setAlignment (Qt::AlignAuto|Qt::AlignVCenter) ;

		if (m_field->getAttrVal("frame").isEmpty())
			m_lEdit->setFrame (true) ;
		else	ctrlSetFrame	  (m_lEdit, 0, 0) ;

		if (m_drow == 0)
			m_lEdit->setText (m_field->getName())	;
		else	m_lEdit->setText (QString::null)	;

		m_layoutItem->setValidatorMode (m_field)	;
		return	    ;
	}
#endif

	if (m_showing == KB::ShowAsData)
	{
		/* Data view, if morphing then ensure that the widget	*/
		/* is morphed out. If not morphing, then make sure we	*/
		/* have the widget, and set up appropriate properties.	*/
		if (m_field->isMorphing ())
		{
			if (m_lEdit != 0) clearWidget () ;
			return	;
		}

		if (m_lEdit == 0) setupWidget () ;
		setupDataProperties() ;
	}
}

/*  KBCtrlField								*/
/*  sizeHint	: Get size hint						*/
/*  (returns)	: QSize		: Size hint				*/

QSize	KBCtrlField::sizeHint () const
{
	/* See comments in header file ...				*/
	return	m_lEdit->sizeHint() ;
}

/*  KBCtrlField								*/
/*  sizePolicy	: Get size policy					*/
/*  (returns)	: QSizePolicy	: Size policy				*/

QSizePolicy
	KBCtrlField::sizePolicy () const
{
	/* See comments in header file ...				*/
	return	QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed) ;
}

/*  KBCtrlField								*/
/*  setReadOnly	: Explicitely set readonly state			*/
/*  readOnly	: bool		: Required state			*/
/*  (returns)	: void		:					*/

void	KBCtrlField::setReadOnly
	(	bool		readOnly,
		const QColor	&fgColor,
		const QColor	&bgColor
	)
{
	if (m_lEdit != 0) m_lEdit->setReadOnly (readOnly) ;
	KBControl::setReadOnly (readOnly, fgColor, bgColor) ;
}

bool	KBCtrlField::isReadOnly ()
{
	return	m_lEdit == 0 ? false : m_lEdit->isReadOnly() ;
}

/*  KBCtrlField								*/
/*  setSelection: Set text selection					*/
/*  start	: int		: Selection start point			*/
/*  length	: int		: Selection length			*/
/*  (returns)	: void		:					*/

void	KBCtrlField::setSelection
	(	int		start,
		int		length
	)
{
	/* Typically we want to use "setSelection" inside the "onEnter"	*/
	/* event, but in that case we get called too early (before	*/
	/* focus is actually in the control). So, defer the setting on	*/
	/* a zero-period timer.						*/
	m_selStart	= start	 ;
	m_selLength	= length ;
	QTimer::singleShot (0, this, SLOT(slotSetSelection())) ;
}

/*  KBCtrlField								*/
/*  slotSetSelection							*/
/*		: Actually set text selection				*/
/*  (returns)	: void		:					*/

void	KBCtrlField::slotSetSelection ()
{
	DPRINTF
	((	"KBCtrlField::slotSetSelection: %s: %d,%d: %p\n",
		m_field->getName().latin1(),
		m_selStart,
		m_selLength,
		(void *)m_lEdit
	))	;

	if (m_lEdit != 0) m_lEdit->setSelection (m_selStart, m_selLength) ;
}
