/***************************************************************************
    file	         : kb_qtgraphic.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qimage.h>

#include	"kb_graphic.h"
#include	"kb_display.h"
#include	"kb_writer.h"
#include	"kb_ascii.h"


extern	const	QPixmap	&scalePixmap	(const QPixmap &, const QRect &, uint) ;


/*  KBCtrlGraphic							*/
/*  KBCtrlGraphic: Constructor for embeded graphic wrapper		*/
/*  display	 : KBDisplay *	: Parent display			*/
/*  graphic	 : KBGraphic 	: Parent graphic			*/
/*  (returns)	 : KBGraphic	:					*/

KBCtrlGraphic::KBCtrlGraphic
	(	KBDisplay	*display,
		KBGraphic	*graphic
	)
	:
	KBControl
	(	display,
		graphic
	),
	m_graphic (graphic)
{
	m_label	= new QLabel (display->getDisplayWidget()) ;
	setupWidget (m_label) ;
}

/*  KBCtrlGraphic							*/
/*  ~KBCtrlGraphic: Destructor for embedded graphic			*/
/*  (returns)	 :		:					*/

KBCtrlGraphic::~KBCtrlGraphic ()
{
}

/*  KBCtrlGraphic							*/
/*  setValue	: Set graphic text					*/
/*  value	: const KBValue &: Image data				*/
/*  (returns)	: void		 :					*/

void	KBCtrlGraphic::setValue
	(	const KBValue	&value
	)
{
	QPixmap	pm ;

	pm.loadFromData
	(	(const unsigned char *)value.dataPtr(),
		value.dataLength()
	)	;

	QRect	rect	= m_graphic->geometry() ;
	int	style	;
	int	width	;
	if (getFrameSettings (style, width, 0, 0))
	{	rect.setWidth  (rect.width () - 2 * width) ;
		rect.setHeight (rect.height() - 2 * width) ;
	}

	m_label->setPixmap 
	(	scalePixmap
		(	pm,
			rect,
			m_graphic->autoSize()
	)	)	;

	KBControl::setValue (value) ;
}

/*  KBCtrlGraphic							*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: bool		:					*/

void	KBCtrlGraphic::setupProperties ()
{
	ctrlSetFrame   (m_label, 0, 0) ;
}

/*  KBCtrlGraphic							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlGraphic::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;
	setupProperties   ()	 ;
}

/*  KBCtrlGraphic							*/
/*  write	: Write field 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlGraphic::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		if (m_label->pixmap() != 0)
		{
			KBWriterPixmap *wi = new KBWriterPixmap
						 (	writer,
							rect,
							*m_label ->pixmap  (),
							m_graphic->autoSize()
						 )	;
			wi->setParent   (m_graphic, 0)  ;
			writerSetFrame  (wi, 0, 0) ;
		}

		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

