/***************************************************************************
    file	         : kb_qtgrid.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#if		__KB_AUTOMOC
#include 	"kb_qtgrid.moc"
#else
#include 	"kb_qtgrid.h"
#endif

#include	"kb_block.h"
#include	"kb_display.h"
#include	"kb_writer.h"
#include	"kb_grid.h"
#include	"kb_navigator.h"
#include	"kb_ascii.h"


#define		__DEBUG_QTGRID		(0)
#if		__DEBUG_QTGRID
#include	"kb_dprintf.h"
#endif


/*  KBCtrlGrid								*/
/*  KBCtrlGrid	: Constructor for grid header wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  grid	: KBGrid 	: Parent grid header			*/
/*  (returns)	: KBCtrlGrid	:					*/

KBCtrlGrid::KBCtrlGrid
	(	KBDisplay		*display,
		KBGrid			*grid,
		QPtrList<KBItem>	&visItems
	)
	:
	KBControl
	(	display,
		grid
	),
	m_grid	  	(grid),
	m_visItems	(visItems)
{
	m_header = new QHeader (display->getDisplayWidget()) ;
	setupWidget (m_header, KBLayoutItem::RTTIGrid) ;

	connect
	(	m_header,
		SIGNAL	(sizeChange (int,int,int)),
		SLOT	(sizeChange (int,int,int))
	)	;
	connect
	(	m_header,
		SIGNAL	(indexChange(int,int,int)),
		SLOT	(indexChange(int,int,int))
	)	;
	connect
	(	m_header,
		SIGNAL	(clicked      (int)),
		m_grid,
		SLOT	(columnClicked(int))
	)	;

	m_header->setTracking      (false) ;
	m_header->setClickEnabled  (true ) ;
}

/*  KBCtrlGrid								*/
/*  ~KBCtrlGrid: Destructor for grid header widget			*/
/*  (returns)	 :		:					*/

KBCtrlGrid::~KBCtrlGrid ()
{
}

/*  KBCtrlGrid								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlGrid::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;
	m_header->setClickEnabled(!m_grid->noSort()) ;
}

/*  KBCtrlGrid								*/
/*  clearLabels	: Clear all labels and sections				*/
/*  (returns)	: void		:					*/

void	KBCtrlGrid::clearLabels ()
{
	while (m_header->count() > 0) m_header->removeLabel (0) ;
}

/*  rlGrid								*/
/*  appendLabel	: Append a label and section				*/
/*  label	: const QString & : Label text				*/
/*  width	: int		  : Section width			*/
/*  dynamic	: bool		  : Section is dynamic			*/
/*  (returns)	: void		  :					*/

void	KBCtrlGrid::appendLabel
	(	const QString	&label,
		int		width,
		bool		dynamic
	)
{
	int sn = m_header->addLabel (label, width) ;
	m_header->setResizeEnabled  (dynamic,  sn) ;
}

/*  KBCtrlGrid								*/
/*  write	: Write grid header 					*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlGrid::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

/*  KBCtrlGrid								*/
/*  adjustItems	: Adjust item positions and sizes			*/
/*  section	: int		: Staring section number		*/
/*  (returns)	: void		:					*/

void	KBCtrlGrid::adjustItems
	(	int	section
	)
{
	int	left	= ctrlGeometry().x() ;
	int	gwidth	= m_header->width () ;
	int	awidth	= 0 ;
	
	int	index	= m_header->mapToIndex (section) ;

	DPRINTF
	((	"KBCtrlGrid::adjustItems: %d -> %d: gw=%d\n",
		section,
		index,
		gwidth
	))	;
	if (index < 0) return ;

//	/* If we are embedded in a top-level block then we will resize	*/
//	/* the scroller contents to the grid width (and a bit ...:)	*/
//	if (getDisplay()->getParent() == 0) gwidth = 0x7fffffff ;


	/* Adjust the items corresponding to the altered section and	*/
	/* all to its right (ie., greater indexes).			*/
	while (index < (int)m_visItems.count())
	{
		int	section	= m_header->mapToSection(index  ) ;
		int	pos	= m_header->sectionPos  (section) ;
		int	size	= m_header->sectionSize (section) ;
		KBItem	*item	= m_visItems.at  (index) ;
		QRect	ig	= item->geometry () ;

		/* Prevent the right-hand edge of the section going	*/
		/* outside the maximum width of the header widget width	*/
		if (pos + size > gwidth)
			if ((size = gwidth - pos) < 0)
				size = 1 ; 

		if (index == (int)m_visItems.count() - 1)
		{
			size	= gwidth - pos	;
			if (size <= 0) size = 1 ;
			m_header->resizeSection (section, size) ;
		}

		DPRINTF
		((	"KBCtrlGrid::adjustItems: %3d [%-12s] -> (%d,%d)\n",
			index,
			(cchar *)item->getAttrVal("name"),
			left + pos,
			size
		))	;
		item->setCtrlGeometry (QRect(left + pos, ig.y(), size, ig.height())) ;

		awidth	= pos + size ;
		index  += 1	     ;
	}


	DPRINTF
	((	"KBCtrlGrid::adjustItems: awidth=%d left=%d\n",
		awidth,
		left
	))	;
//	KBDisplay *display = m_grid->getDisplay () ;
//	if (display != 0) display->redoLayout (true) ;
//	if (awidth > 0)
//		getDisplay()->resizeContents (QSize (awidth + left + 8, 0)) ;


//	if (awidth > 0)
//		setCtrlGeometry
//		(	QRect
//			(	QPoint(m_grid->rowMarkOffset(), 0),
//				QSize (awidth + 0 + 8, m_header->size().height())
//		)	)	;
}

/*  KBCtrlGrid								*/
/*  eventFilter	: Event filter on edit control				*/
/*  o		: QObject *	: Object being filtered			*/
/*  e		: QEvent *	: Event					*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBCtrlGrid::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (m_showing != KB::ShowAsData)
		return	KBControl::eventFilter (o, e) ;

	if (o != m_header)
		return	KBControl::eventFilter (o, e) ;
#if	0
	if (e->type() == QEvent::Resize)
	{
		DPRINTF
		((	"KBCtrlGrid::eventFilter: resize: s=%s\n",
			_TEXT(((QResizeEvent *)e)->size())
		))	;
		adjustItems (0)) ;
	}
#endif
	return	KBControl::eventFilter (o, e) ;
}

/*  KBCtrlGrid								*/
/*  sizeChange	: Handle size change from grid header			*/
/*  section	: int		: Section number			*/
/*  oldSize	: int		: Original size				*/
/*  newSize	: int		: New size				*/
/*  (returns)	: void		:					*/

void	KBCtrlGrid::sizeChange
	(	int	section,
		int	,
		int	
	)
{
	adjustItems (section) ;
}

/*  KBCtrlGrid								*/
/*  indexChange	: Handle reordering from grid header			*/
/*  section	: int		: Section number			*/
/*  fromIndex	: int		: Index moved from ...			*/
/*  toIndex	: int		: ... and to				*/
/*  (returns)	: void		:					*/

void	KBCtrlGrid::indexChange
	(	int	,
		int	fromIndex,
		int	toIndex
	)
{
	/* Determine the section number for the left-hand most position	*/
	/* and adjust the items from this one onwards. We also move	*/
	/* the item to keep the visible item list in display order.	*/
	KBItem	*item	= m_visItems.take (fromIndex) ;
	int	lower	= fromIndex < toIndex ? fromIndex : toIndex ;

	m_visItems.insert (toIndex > fromIndex ? toIndex - 1 : toIndex, item) ;
	adjustItems  	  (m_header->mapToSection (lower)) ;

	for (uint index = 0 ; index < m_visItems.count() ; index += 1)
	{
		KBItem	*item	= m_visItems.at (index)  ;

		if (item->getTabOrder () != 0)
			item->setTabOrder (index + 1) ;

		DPRINTF
		((	"KBCtrlGrid::indexChange: %3d: %s\n",
			index,
			(cchar *)item->getAttrVal("name")
		))	;
	}

	KBNavigator *navi = m_grid->getNavigator() ;
	if (navi != 0) navi->setupTabOrder() ;
}

void	KBCtrlGrid::setColumnWidth
	(	int		idx,
		uint		width
	)
{
	int	sect	= m_header->mapToSection(idx) ;
	m_header->resizeSection (sect, width) ;
	adjustItems   (sect) ;
}

void	KBCtrlGrid::setSortIndicator
	(	int		col,
		bool		asc
	)
{
	m_header->setSortIndicator (col, asc) ;
}
