/***************************************************************************
    file	         : kb_qtlabel.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qobjectlist.h>
#include	<qaccel.h>

#if		__KB_AUTOMOC
#include	"kb_qtlabel.moc"
#else
#include	"kb_qtlabel.h"
#endif

#include	"kb_label.h"
#include	"kb_display.h"
#include	"kb_writer.h"
#include	"kb_attralign.h"
#include	"kb_form.h"


#define		__DEBUG_QTLABEL		(0)
#if		__DEBUG_QTLABEL
#include	"kb_dprintf.h"
#endif



/*  KBCtrlLabel								*/
/*  KBCtrlLabel	: Constructor for label wrapper				*/
/*  display	: KBDisplay *	: Parent display			*/
/*  label	: KBLabel 	: Parent label				*/
/*  (returns)	: KBLabel	:					*/

KBCtrlLabel::KBCtrlLabel
	(	KBDisplay	*display,
		KBLabel		*label
	)
	:
	KBControl
	(	display,
		label
	),
	m_label	 	(label)
{
	m_qtLabel = new QLabel (display->getDisplayWidget()) ;
	setupWidget (m_qtLabel) ;

	/* Check if display widget emits the "backgroundChanged()"	*/
	/* signal. This can be removed after the KBDispXXXXX merges.	*/
	QWidget *dw = display->getDisplayWidget() ;
	if (dw->metaObject()->signalNames().find("backgroundChanged()") >= 0)
		connect
		(	dw,
			SIGNAL	(backgroundChanged()),
			SLOT	(backgroundChanged())
		)	;
}

/*  KBCtrlLabel								*/
/*  ~KBCtrlLabel: Destructor for label widget				*/
/*  (returns)	 :		:					*/

KBCtrlLabel::~KBCtrlLabel ()
{
}

/*  KBCtrlLabel								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlLabel::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;
	setupProperties	  ()	 ;
}

/*  KBLabel								*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: bool		:					*/

void	KBCtrlLabel::setupProperties ()
{
	int	align	= m_label->getAlign()	;

	if (align == RICHTEXT)
	{
		m_qtLabel->setAlignment  (Qt::AlignLeft) ;
		m_qtLabel->setTextFormat (Qt::RichText ) ;
	}
	else
	{	m_qtLabel->setTextFormat (Qt::PlainText) ;
		m_qtLabel->setAlignment  (Qt::ShowPrefix|align) ;
	}

	m_qtLabel->setText (m_label->getAttrVal("text")) ;
	ctrlSetFrame	   (m_qtLabel, 0, 0) ;
}

/*  KBCtrlLabel								*/
/*  setValue	: Set label text					*/
/*  value	: const KBValue &: Value for text			*/
/*  (returns)	: void		 :					*/

void	KBCtrlLabel::setValue
	(	const KBValue	&value
	)
{
	QString	text	= value.getRawText() ;
	m_qtLabel->setText (text) ;

	/* Now for a load of mucking about. We need to handle buddying	*/
	/* ourselves, but the QLabel class does not expose the right	*/
	/* methods, so here goes .... if there is no shortcut key then	*/
	/* nothing to do.						*/
	int	ak	= QAccel::shortcutKey (text) ;
	if (ak == 0) return ;


	/* If there is an accelerator key then we should be able to	*/
	/* find the QAccel object that the label creates. We need this	*/
	/* since we must clear it and handle the accelerator ourselves.	*/
	const QObjectList *cl = m_qtLabel->children() ;
	if (cl == 0) return ;

	QObjectListIt	iter (*cl) ;
	QAccel		*accel = 0 ;
	QObject		*obj	   ;

	while ((obj = iter.current()) != 0)
	{
		if (obj->isA("QAccel"))
		{
			accel = (QAccel *)obj ;
			break	;
		}
		*++iter ;
	}

	if (accel == 0) return	;
	accel->clear()	;

	/* Get the name of the buddy (and exit if not set), then locate	*/
	/* it. If not found or the buddy is not an item then we do	*/
	/* nothing.							*/
	QString	 bname	= m_label->getAttrVal("buddy") ;
	if (bname.isEmpty ()) return ;

	KBNode *buddy	= m_label->getParent()->getNamedNode (bname) ;
	if ((buddy == 0) || (buddy->isItem() == 0))
	{
		DPRINTF
		((	"KBCtrlLabel::setValue: no buddy or not item [%s][%p]\n",
			(cchar *)bname,
			(void  *)buddy
		))	;
		return	;
	}

	/* Got it! The accelerator is installed by the form object,	*/
	/* which handles stuff line current row (since if there are	*/
	/* multiple rows displayed then the buddy is really one of the	*/
	/* controls for the buddy item).				*/
	m_label->getRoot()->isForm()->addAccelerator(ak, buddy->isItem()) ;
}

/*  KBCtrlLabel								*/
/*  getValue	: Get button text					*/
/*  (returns)	: KBValue	: Button text				*/

KBValue	KBCtrlLabel::getValue ()
{
	return	KBValue(m_qtLabel->text()) ;
}

void	KBCtrlLabel::backgroundChanged ()
{
	DPRINTF
	((	"KBCtrlLabel::backgroundChanged: called\n"
	))	;

	const QPixmap *pm = m_qtLabel->parentWidget()->paletteBackgroundPixmap() ;
	if (pm != 0)
	{	m_qtLabel->setPaletteBackgroundPixmap (*pm) ;
		m_qtLabel->setBackgroundOrigin	      (QWidget::ParentOrigin) ;
	}
}

/*  KBCtrlLabel								*/
/*  eventFilter	: Filter events						*/
/*  o		: QObject *	: Originating object			*/
/*  e		: QEvent *	: Event to be filtered			*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBCtrlLabel::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (m_showing == KB::ShowAsData)
		if (e->type() == QEvent::MouseButtonRelease)
		{
			DPRINTF
			((	"KBCtrlLabel::mouseReleaseEvent: called\n"
			))	;
			m_label->clicked () ;
		}

	return	KBControl::eventFilter (o, e) ;
}

/*  KBCtrlLabel								*/
/*  write	: Write field 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlLabel::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		KBWriterText *wi = new KBWriterText
				   (	writer,
					rect,
					m_label  ->getPalette(true),
					m_label  ->getFont   (true),
					m_qtLabel->text	     (),
					m_label  ->getAlign  (),
					false
				   ) 	;

		wi->setParent   (m_label, 0) ;
		writerSetFrame  (wi, 0, 0) ;

		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

