/***************************************************************************
    file	         : kb_qtlink.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>
#include	<qlistbox.h>
#include	<qtimer.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_writer.h"
#include	"kb_block.h"
#include	"kb_link.h"
#include	"kb_display.h"
#include	"kb_navigator.h"

#include	"kb_multilistbox.h"

#if		__KB_AUTOMOC
#include 	"kb_qtlink.moc"
#else
#include 	"kb_qtlink.h"
#endif


#define		__DEBUG_LINK	(0)
#if		__DEBUG_LINK
#include	"kb_dprintf.h"
#endif


class	KBCtrlLinkComboBox : public RKComboBox
{
public	:

	inline	KBCtrlLinkComboBox
		(	QWidget		*parent
		)
		:
		RKComboBox (parent)
	{
	}

	friend	class	KBCtrlLink ;
}	;


/*  KBCtrlLink								*/
/*  KBCtrlLink	: Constructor for combo box wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  link	: KBLink *	: Parent link				*/
/*  drow	: uint		: Row number in field			*/
/*  (returns)	: KBCtrlLink	:					*/

KBCtrlLink::KBCtrlLink
	(	KBDisplay	*display,
		KBLink		*link,
		uint		drow
	)
	:
	KBControl
	(	display,
		link,
		drow
	),
	m_link		(link)
{
	m_comboBox	= 0	;
	m_listBox	= 0	;

	m_deferFocus	= 0	;
	m_deferMouse	= 0	;
	m_keyset	= 0	;
	m_valset	= 0	;
	m_inSetValue	= false	;
}

/*  KBCtrlLink								*/
/*  ~KBCtrlLink	: Desctructor for link class				*/
/*  (returns)	:		:					*/

KBCtrlLink::~KBCtrlLink ()
{
	DELOBJ	(m_deferFocus)	;
	DELOBJ	(m_deferMouse)	;
	DELOBJ	(m_keyset    )	;
	DELOBJ	(m_valset    )	;
}


/*  KBCtrlLink								*/
/*  setupWidget	: Set up widget						*/
/*  (returns)	: void		:					*/

void	KBCtrlLink::setupWidget ()
{
	DPRINTF
	((
		"KBCtrlLink::setupWidget: called\n"
	))	;
	if (getDisplay() == 0)
		return	;

	m_comboBox = new KBCtrlLinkComboBox (getDisplay()->getDisplayWidget()) ;
	m_listBox  = new KBMultiListBox
			 (	m_comboBox,
				false,
				m_link->getAttrVal("showcols").toInt()
			 )	;
	m_comboBox->setListBox (m_listBox ) ;

	KBControl::setupWidget (m_comboBox) ;

	connect
	(	m_comboBox,
		SIGNAL	(activated	(int)),
		SLOT	(deferUserChange())
	)	;

//	m_comboBox->show () ;
}

/*  KBCtrlLink								*/
/*  clearWidget	: Clear down widget					*/
/*  (returns)	: void		:					*/

void	KBCtrlLink::clearWidget ()
{
	DPRINTF
	((	"KBCtrlLink::clearWidget: called\n"
	))	;

	KBControl::setValue (getValue()) ;

	m_comboBox = 0	;
	m_listBox  = 0	;

	KBControl::clearWidget () ;
}

/*  KBCtrlLink								*/
/*  setData	: Load values into combo				*/
/*  valset	: const KBLTValueSet &	: List of values		*/
/*  (returns)	: void			:				*/

void	KBCtrlLink::loadDataValues
	(	const KBLTValueSet	&valset
	)
{
	if (m_comboBox != 0)
	{
		uint exprCount = m_link->exprCount () ;

		m_comboBox->clear () ;

		for (uint idx = 0 ; idx < valset.count() ; idx += 1)
			m_listBox->insertEntry (*valset.at(idx), exprCount) ;

		m_listBox->calcGeometry () ;
	}
}

/*  KBCtrlLink								*/
/*  loadControls: Load contol values					*/
/*  keyset	: const QStringList  & : Keys				*/
/*  valset	: const KBLTValueSet & : Values				*/
/*  (returns)	: void		       :				*/

void	KBCtrlLink::loadControl
	(	const QStringList	&keyset,
		const KBLTValueSet	&valset
	)
{
	KBInSetValueLock lock (*this) ;

	DELOBJ		(m_keyset) ;
	DELOBJ		(m_valset) ;

	/* If either a user filter or sorting expression in set	*/
	/* the we do an explicit load for this control only,	*/
	/* providing our own keyset (which is retained).	*/
	if (!m_userFilter.isEmpty() || !m_userSorting.isEmpty())
	{
		m_valset = new KBLTValueSet ;
		m_keyset = new QStringList  ;

		m_link->loadValues
		(	m_userFilter,
			m_userSorting,
			*m_keyset,
			*m_valset
		)	;

		DPRINTF
		((	"KBCtrlLink::reload: done query: m_keyset=[%p] curval=[%s]\n",
			(void  *)m_keyset,
			getValue().getRawText().latin1()
		))	;

		loadDataValues (*m_valset) ;
		return	;
	}

	loadDataValues	(valset) ;
}

/*  KBCtrlLink								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlLink::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;

	m_userSorting	= QString::null ;
	m_userFilter	= QString::null ;

	DELOBJ (m_keyset) ;
	DELOBJ (m_valset) ;

	{
		KBInSetValueLock lock (*this) ;
		setupProperties  () ;
	}
}

/*  KBCtrlLink								*/
/*  setValue	: Set value						*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlLink::setValue
	(	const KBValue	&value
	)
{
	DPRINTF
	((	"KBCtrlLink::setValue: m_keySet=[%p] [%s]\n",
		(void  *)m_keyset,
		(cchar *)value.getRawText()
	))	;

	if (m_comboBox != 0)
	{
		KBInSetValueLock lock (*this) ;
		m_comboBox->setCurrentItem (m_link->valueToItem (value, m_keyset)) ;
	}

	KBControl::setValue (value) ;
}

/*  KBCtrlLink								*/
/*  getValue	: Get value						*/
/*  (returns)	: KBValue 	: Value					*/

KBValue	KBCtrlLink::getValue ()
{
	if (m_comboBox != 0)
		return	m_link->itemToValue (m_comboBox->currentItem(), m_keyset) ;

	return	KBControl::getValue () ;
}

/*  KBCtrlLink								*/
/*  clearValue	: Get value						*/
/*  query	: bool		: Clear for query			*/
/*  (returns)	: KBValue 	: Value					*/

void	KBCtrlLink::clearValue
	(	bool	query
	)
{
	DPRINTF
	((	"KBCtrlLink::clearValue: query=%d\n",
		query
	))	;

	if (m_comboBox != 0)
	{
		KBInSetValueLock lock    (*this) ;
		m_comboBox  ->setCurrentItem (0) ;
		if (m_showing == KB::ShowAsData)
			m_layoutItem->setValid (isValid (false)) ;
	}

	KBControl::clearValue (query) ;
}

/*  KBCtrlLink								*/
/*  isEmpty	: See if control is empty				*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlLink::isEmpty ()
{
	if (m_comboBox != 0)
		return	m_comboBox->currentItem() == 0 ;

	return	m_curVal.isEmpty() ;
}

/*  KBCtrlLink								*/
/*  passFocus	: Actually pass focus into the control			*/
/*  (returns)	: void		:					*/

void	KBCtrlLink::passFocus ()
{
	DPRINTF
	((	"KBCtrlLink::passFocus: [%p] [%p]\n",
		(void *)m_deferFocus,
		(void *)m_deferMouse
	))	;
	if (m_comboBox != 0)
	{
		if (m_deferFocus != 0)
		{
			QFocusEvent::setReason	    (QFocusEvent::Mouse) ;
			m_comboBox->focusInEvent    (m_deferFocus) ;
			DELOBJ	(m_deferFocus) ;
		}

		if (m_deferMouse != 0)
		{
			m_comboBox->mousePressEvent (m_deferMouse) ;
			DELOBJ	(m_deferMouse) ;
		}
	}
}


/*  KBCtrlLink								*/
/*  write	: Write link 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlLink::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		new KBWriterText
		(	writer,
			rect,
			m_link->getPalette(true),
			m_link->getFont   (true),
			value.getRawText  (),
			Qt::AlignLeft|Qt::AlignVCenter,
			false
		)	;

		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

/*  KBCtrlLink								*/
/*  setFocus	: Set focus into widget					*/
/*  (returns)	: void		:					*/

void	KBCtrlLink::setFocus ()
{
	/* If the edit widget does not exist then we are currently	*/
	/* morphed, so restore the widget.				*/
	if (m_comboBox == 0)
	{
		setupWidget	   () ;
		setupDataProperties() ;
		setValue	   (KBControl::getValue()) ;
		reload		   () ;
	}

	KBControl::setFocus () ;
}

/*  KBCtrlLink								*/
/*  morphText	: Get text for morphed control				*/
/*  align	: int &		: Return alignment			*/
/*  (returns)	: QString	: Text					*/

QString	KBCtrlLink::morphText
	(	int		&align
	)
{
	return	m_link->valueToText (m_curVal) ;
}

/*  KBCtrlLink								*/
/*  setupDataProperties							*/
/*		: Set up widget for data mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlLink::setupDataProperties ()
{
	if (m_listBox)
	{
		m_listBox   ->calcGeometry	() ;
		m_listBox   ->setShowCols 	(m_link->getAttrVal("showcols").toUInt()) ;
		m_comboBox  ->setReadOnly	(m_link->isReadOnly()) ;
		m_layoutItem->setValidatorMode	(m_link) ;
	}
}

/*  KBCtrlLink								*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: void		:					*/

void	KBCtrlLink::setupProperties ()
{
#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		if (m_comboBox == 0) setupWidget () ;

		m_comboBox->setReadOnly (false) ;
		m_comboBox->clear            () ;
		if (m_drow == 0) m_listBox->insertEntry (m_link->getName()) ;

		m_layoutItem->setValidatorMode	(m_link) ;
		return	;
	}
#endif

	if (m_showing == KB::ShowAsData)
	{
		/* Data view, if morphing then ensure that the widget	*/
		/* is morphed out. If not morphing, then make sure we	*/
		/* have the widget, and set up appropriate properties.	*/
		if (m_link->isMorphing ())
		{
			if (m_comboBox != 0) clearWidget () ;
			return	;
		}

		if (m_comboBox == 0) setupWidget () ;
		setupDataProperties() ;
	}
}

/*  KBCtrlLink							*/
/*  setUserFilter						*/
/*		: Set user filter for link query		*/
/*  filter	: const QString & : Filter expression		*/
/*  (returns)	: void		  :				*/

void	KBCtrlLink::setUserFilter
	(	const QString	&filter
	)
{
	m_userFilter	= filter ;
}

/*  KBCtrlLink							*/
/*  setUserSorting						*/
/*		: Set user sorting for link query		*/
/*  sorting	: const QString & : Sorting expression		*/
/*  (returns)	: void		  :				*/

void	KBCtrlLink::setUserSorting
	(	const QString	&sorting
	)
{
	m_userSorting	= sorting ;
}

/*  KBCtrlLink							*/
/*  reload	: Reload values list				*/
/*  (returns)	: void		  :				*/

void	KBCtrlLink::reload ()
{
	/* Note the current value. We will restore this after	*/
	/* the control has been updated.			*/
	KBValue	curVal	= m_curVal ;

	DELOBJ  (m_keyset) ;
	DELOBJ  (m_valset) ;

	/* If either a user filter or sorting expression in set	*/
	/* the we do an explicit load for this control only,	*/
	/* providing our own keyset (which is retained).	*/
	if (!m_userFilter.isEmpty() || !m_userSorting.isEmpty())
	{
		m_valset = new KBLTValueSet ;
		m_keyset = new QStringList  ;

		m_link->loadValues
		(	m_userFilter,
			m_userSorting,
			*m_keyset,
			*m_valset
		)	;

		DPRINTF
		((	"KBCtrlLink::reload: done query: m_keyset=[%p] curval=[%s]\n",
			(void  *)m_keyset,
			(cchar *)curVal.getRawText()
		))	;

		loadDataValues (*m_valset) ;
	}
	else	m_link->doRefresh (m_drow) ;

	setValue (curVal)    ;
}

/*  KBCtrlLink								*/
/*  getExtra	: Return "extra" value for current row			*/
/*  which	: bool		: Display or extra			*/
/*  index	: uint		: Extra value index			*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCtrlLink::getExtra
	(	bool		which,
		uint		index
	)
{
	return	m_link->itemToExtra
		(	m_comboBox->currentItem(),
			which,
			index,
			m_valset
		)	;
}

/*  KBCtrlLink								*/
/*  setCurrentItem							*/
/*		: Set current item by index				*/
/*  item	: uint		: Item index				*/
/*  (returns)	: void		:					*/

void	KBCtrlLink::setCurrentItem
	(	uint		item
	)
{
	if (m_comboBox != 0)
		m_comboBox->setCurrentItem (item) ;

	KBControl::setValue (m_link->itemToValue (item, m_keyset)) ;
}

/*  KBCtrlLink								*/
/*  currentItem	: Get current item index				*/
/*  (returns)	: uint		: Item index				*/

uint	KBCtrlLink::currentItem ()
{
	if (m_comboBox == 0)
		return	0 ;

	return	m_comboBox->currentItem	() ;
}



bool	KBCtrlLink::isNavigationKey
	(	QKeyEvent	*k
	)
{
	return	KBNavigator::isNavigationKey (k) ;
}

/*  KBCtrlLink								*/
/*  eventFilter	: Event filter on edit control				*/
/*  o		: QObject *	: Object being filtered			*/
/*  e		: QEvent *	: Event					*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBCtrlLink::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (m_showing != KB::ShowAsData)
		return	KBControl::eventFilter (o, e) ;

	if
	(	(o != m_comboBox) &&
		(o != m_listBox )
	)
		return	KBControl::eventFilter (o, e) ;

	if (o == m_listBox)
		if (m_listBox->isVisible())
			if ((e->type() == QEvent::KeyPress) || (e->type() == QEvent::KeyRelease))
				return	false	;

	if (e->type() == QEvent::FocusOut)
	{
		/* If morphing, the widget must be cleared. Do not do	*/
		/* this immediately, else we get focus problems.	*/
		if (m_link->isMorphing ()) startMorphTimer () ;

		return	KBControl::eventFilter (o, e) ;
	}

	/* If in data mode and the link is dynamic then on focus entry	*/
	/* we have to to refresh the control. This is nasty since	*/
	/* changing the combo values in the middle of a mouse-focus-in	*/
	/* seems (perhaps not suprisingly) to do nasty things!		*/
	if (e->type() == QEvent::FocusIn)
	{
		stopMorphTimer () ;

		if (m_link->dynamic())
		{
			QFocusEvent *fe	   = (QFocusEvent *)e ;
			KBValue	    curVal ;

			switch (fe->reason())
			{
				case QFocusEvent::Tab	   :
				case QFocusEvent::Shortcut :
				case QFocusEvent::Other    :
					/* These cases are easy, the	*/
					/* one below is not.		*/
					curVal	= getValue() ;
					m_link->doRefresh (m_drow) ;
					setValue (curVal)    ;
					break ;

				case QFocusEvent::Mouse    :
					/* If focus arrives on account	*/
					/* of the mouse then we have	*/
					/* to defer then focus in and	*/
					/* mouse button events or we	*/
					/* get an escape-only lockup.	*/
					curVal	= getValue() ;
					m_link->doRefresh (m_drow) ;
					setValue (curVal)    ;

					m_deferFocus = new QFocusEvent(QEvent::FocusIn) ;
					QTimer::singleShot (250, this, SLOT(passFocus())) ;
					return	true	;

				default	:
					break	;
			}
		}
	}

	/* If this is a mouse press and focus is being deferred then	*/
	/* we have to defer the mouse press as well.			*/
	if ((e->type() == QEvent::MouseButtonPress) && (m_deferFocus != 0))
	{
		QMouseEvent *me = (QMouseEvent *)e ;
		m_deferMouse = new QMouseEvent (me->type(), me->pos(), me->globalPos(), me->button(), me->state()) ;
		return	true ;
	}


	return	KBControl::eventFilter (o, e) ;
}

/*  KBCtrlLink								*/
/*  setReadOnly	: Explicitely set readonly state			*/
/*  readOnly	: bool		: Required state			*/
/*  (returns)	: void		:					*/

void	KBCtrlLink::setReadOnly
	(	bool		readOnly,
		const QColor	&fgColor,
		const QColor	&bgColor
	)
{
	if (m_comboBox != 0) m_comboBox->setReadOnly (readOnly) ;
	KBControl::setReadOnly (readOnly, fgColor, bgColor) ;
}

bool	KBCtrlLink::isReadOnly ()
{
	return	m_comboBox == 0 ? false : m_comboBox->isReadOnly() ;
}
/*  KBCtrlLink								*/
/*  getNumValues: Get number of values showing				*/
/*  nvals	: uint &	: Return number of values		*/
/*  (returns)	: bool		: True if local values			*/

bool	KBCtrlLink::getNumValues
	(	uint		&nvals
	)
{
	if (m_keyset != 0)
	{
		nvals	= m_keyset->count() ;
		return	true ;
	}

	return	false	;
}

/*  KBCtrlLink								*/
/*  getDisplayList							*/
/*		: Get list of displayed text values at row		*/
/*  list	: QStringList &	: List of values on return		*/
/*  (returns)	: bool		: True if local values			*/

bool	KBCtrlLink::getDisplayList
	(	QStringList	&list
	)
{
	if (m_valset != 0)
	{
		for (uint idx = 0 ; idx < m_valset->count() ; idx += 1)
			list.append ((*m_valset)[idx][0]) ;
		return	true ;
	}

	return	false	;
}

/*  KBCtrlLink								*/
/*  setCurrentText							*/
/*		: Set current item by text value			*/
/*  text	: const QString & : Text in question			*/
/*  (returns)	: bool		  : True of found			*/

bool	KBCtrlLink::setCurrentDisplay
	(	const QString	&text
	)
{
	DPRINTF
	((	"KBCtrlLink::setCurrentDisplay: %s: lb=%p\n",
		text.latin1(),
		m_listBox
	))	;

	if ((m_comboBox != 0) && (m_listBox != 0))
	{
		/* Note that if the item is found then we need to set	*/
		/* the current item in the combobox, since the change	*/
		/* to the listbox is not reflected there.		*/
		int idx = m_listBox->setCurrentText (text) ;

		if (idx >= 0)
		{	m_comboBox->setCurrentItem (idx) ;
			return	true ;
		}

		return	false	;
	}

	return	false	;
}
