/***************************************************************************
    file	         : kb_qtlistbox.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_qtlistbox.moc"
#else
#include	"kb_qtlistbox.h"
#endif

#include	"kb_listbox.h"
#include	"kb_display.h"
#include	"kb_block.h"

#include	"rk_listbox.h"


#define		__DEBUG_QTLISTBOX	(0)
#if		__DEBUG_QTLISTBOX
#include	"kb_dprintf.h"
#endif


/*  KBCtrlListBox							*/
/*  KBCtrlListBox							*/
/*		: Constructor for list box wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  listbox	: KBListBox *	: Parent listbox item			*/
/*  drow	: uint		: Row number in field			*/
/*  (returns)	: KBCtrlListBox	:					*/

KBCtrlListBox::KBCtrlListBox
	(	KBDisplay	*display,
		KBListBox	*listbox,
		uint		drow
	)
	:
	KBControl
	(	display,
		listbox,
		drow
	),
	m_listbox	(listbox)
{
	m_qtListBox = new RKListBox (display->getDisplayWidget()) ;
	setupWidget (m_qtListBox) ;

	connect
	(	m_qtListBox,
		SIGNAL	(highlighted	(int)),
		SLOT	(deferUserChange())
	)	;

	m_inSetValue = false ;
}

/*  KBCtrlListBox							*/
/*  ~KBCtrlListBox							*/
/*		: Destructor for list box class				*/
/*  (returns)	:		:					*/

KBCtrlListBox::~KBCtrlListBox ()
{
}

/*  KBCtrlListBox							*/
/*  setValues	: Set value set						*/
/*  values	: const QStringList & : Values				*/
/*  (returns)	: void		      :					*/

void	KBCtrlListBox::setValues
	(	const QStringList	&values
	)
{
	KBInSetValueLock lock (*this) ;
	m_qtListBox->clear	      () ;
	m_qtListBox->insertStringList (values)	;
}

/*  KBCtrlListBox							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlListBox::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;

	{
		KBInSetValueLock lock (*this) ;
		setupProperties () ;
	}
}

/*  KBCtrlListBox							*/
/*  setValue	: Set value						*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlListBox::setValue
	(	const KBValue	&value
	)
{
	QString	v	  = value.getRawText() ;
	int	slot	  = m_listbox->getValues().findIndex(v) ;

	if (slot < 0)
	{
		for (int p = v.length() - 1 ; p >= 0 ; p -= 1)
			if (v.at(p) != QChar(' '))
			{
				v = v.left (p + 1) ;
				break	;
			}

		slot	  = m_listbox->getValues().findIndex(v) ;

		fprintf
		(	stderr,
			"KBCtrlListBox::setValue: retry [%s]->[%s]->[%d]\n",
			(cchar *)value.getRawText(),
			(cchar *)v,
			slot
		)	;
	}

	{
		KBInSetValueLock lock (*this) ;
		m_qtListBox->setCurrentItem (slot < 0 ? 0 : slot) ;
	}

	KBControl::setValue (value) 	;
}

/*  KBCtrlListBox							*/
/*  getValue	: Get value						*/
/*  (returns)	: KBValue 	: Value					*/

KBValue	KBCtrlListBox::getValue ()
{
	/* If the control is empty and the initial value was null (as	*/
	/* opposed to an empty string), then return a null string.	*/
	if (m_qtListBox->currentText().isEmpty() && getIniValue().isNull())
		return	KBValue (m_listbox->getFieldType()) ;

	return	KBValue (m_qtListBox->currentText(), m_listbox->getFieldType()) ;
}

/*  KBCtrlListBox							*/
/*  clearValue	: Clear value						*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: KBValue 	: Value					*/

void	KBCtrlListBox::clearValue
	(	bool	
	)
{
	{
		KBInSetValueLock lock   (*this) ;
		m_qtListBox ->setCurrentItem(0) ;
		if (m_showing == KB::ShowAsData)
			m_layoutItem->setValid (isValid (false)) ;
	}
}

/*  KBCtrlListBox							*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: void		:					*/

void	KBCtrlListBox::setupProperties ()
{
	m_layoutItem->setValidatorMode (m_listbox) ;
	m_qtListBox ->clear            () ;

#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		m_qtListBox->setReadOnly (true) ;
		if (m_drow == 0) m_qtListBox->insertItem (m_listbox->getName()) ;
		return	;
	}
#endif

	m_qtListBox->setReadOnly	(m_listbox->isReadOnly()) ;
	m_qtListBox->insertStringList	(m_listbox->getValues ()) ;
}


void	KBCtrlListBox::setCurrentItem
	(	uint		item
	)
{
	m_qtListBox->setCurrentItem(item) ;
	KBControl::setValue  (getValue()) ;
}

uint	KBCtrlListBox::currentItem ()
{
	int	citem = m_qtListBox->currentItem  () ;
	return	citem < 0 ? 0 : citem ;
}

/*  KBCtrlListBox							*/
/*  setReadOnly	: Explicitely set readonly state			*/
/*  readOnly	: bool		: Required state			*/
/*  (returns)	: void		:					*/

void	KBCtrlListBox::setReadOnly
	(	bool		readOnly,
		const QColor	&fgColor,
		const QColor	&bgColor
	)
{
	DPRINTF
	((	"KBCtrlListBox::setReadOnly: %p: %d\n",
		(void *)m_qtListBox,
		readOnly
	))	;
	if (m_qtListBox != 0) m_qtListBox->setReadOnly (readOnly) ;
	KBControl::setReadOnly (readOnly, fgColor, bgColor) ;
}

bool	KBCtrlListBox::isReadOnly ()
{
	return	m_qtListBox == 0 ? false : m_qtListBox->isReadOnly () ;
}
