/***************************************************************************
    file	         : kb_qtmemo.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	<qpopupmenu.h>
#include	<qtimer.h>
#include	<qstyle.h>

#if		__KB_AUTOMOC
#include 	"kb_qtmemo.moc"
#else
#include 	"kb_qtmemo.h"
#endif

#include	"kb_block.h"
#include	"kb_memo.h"
#include	"kb_display.h"
#include	"kb_writer.h"
#include	"kb_dialog.h"
#include	"kb_highlighter.h"
#include	"kb_file.h"

#include	"tk_filedialog.h"


#define		__DEBUG_QTMEMO	(0)
#if		__DEBUG_QTMEMO
#include	"kb_dprintf.h"
#endif


class	KBCtrlLinkTextEdit : public KBTextEdit
{
	KBCtrlMemo	*m_ctrlMemo	;

public	:

	inline	KBCtrlLinkTextEdit
		(	QWidget		*parent,
			KBCtrlMemo	*ctrlMemo
		)
		:
		KBTextEdit	(parent),
		m_ctrlMemo	(ctrlMemo)
	{
	}

	virtual	QPopupMenu *createPopupMenu (const QPoint &) ;

}	;

QPopupMenu
	*KBCtrlLinkTextEdit::createPopupMenu
	(	const QPoint	&pos
	)
{
	QPopupMenu *p = KBTextEdit::createPopupMenu (pos) ;
	m_ctrlMemo->extendPopupMenu (p) ;
	return	p  ;
}


/*  KBCtrlMemo								*/
/*  KBCtrlMemo	: Constructor for memo edit wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  memo	: KBMemo *	: Parent memo				*/
/*  drow	: uint		: Row number in memo			*/
/*  (returns)	: KBCtrlMemo	:					*/

KBCtrlMemo::KBCtrlMemo
	(	KBDisplay	*display,
		KBMemo		*memo,
		uint		drow
	)
	:
	KBControl
	(	display,
		memo,
		drow
	),
	m_memo		(memo)
{
	m_textEdit = new KBCtrlLinkTextEdit (display->getDisplayWidget(), this) ;
	setupWidget (m_textEdit) ;

	m_highlighter	= 0	;
	m_inSetValue	= false ;

	connect
	(	m_textEdit,
		SIGNAL	(textChanged()),
		SLOT	(userChange ())
	)	;
}

/*  KBCtrlMemo								*/
/*  ~KBCtrlMemo	: Desctructor for memo class				*/
/*  (returns)	:		:					*/

KBCtrlMemo::~KBCtrlMemo ()
{
	DELOBJ (m_highlighter) ;
}

/*  KBCtrlMemo								*/
/*  extendPopupMenu							*/
/*		: Extent popup menu for the memo control		*/
/*  p		: QPopupMenu *	: Menu					*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::extendPopupMenu
	(	QPopupMenu	*p
	)
{
	p->insertSeparator () ;
	if (!m_memo->isReadOnly())
		p->insertItem (TR("Insert from file"), this, SLOT(loadFromFile())) ;
	p->insertItem (TR("Save to file"), this, SLOT(saveToFile())) ;
}

/*  KBCtrlMemo								*/
/*  loadFromFile: Load control from a file				*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::loadFromFile ()
{
	DPRINTF
	((	"KBCtrlMemo::loadFromFile: called\n"
	))	;
	KBFileDialog fDlg 
		     (	".",
			QString::null,
			qApp->activeWindow(),
			"loadfile",
			true
		     )	;

	fDlg.setMode	  (KBFileDialog::ExistingFile) ;
	fDlg.setCaption   (TR("Load from file ...." )) ;
	if (!fDlg.exec ()) return ;

	QString	name	  = fDlg.selectedFile () ;
	if (name.isEmpty()) return ;

	KBFile	file(name)	;
	if (!file.open(IO_ReadOnly))
	{
		file.lastError().DISPLAY() ;
		return	;
	}

	m_textEdit->setText (file.readAll()) ;
}

/*  KBCtrlMemo								*/
/*  saveToFile	: Save contents to a file				*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::saveToFile ()
{
	DPRINTF
	((	"KBCtrlMemo::saveToFile: called\n"
	))	;
	KBFileDialog fDlg 
		     (	".",
			QString::null,
			qApp->activeWindow(),
			"loadfile",
			true
		     )	;

	fDlg.setMode	  (KBFileDialog::AnyFile  ) ;
	fDlg.setCaption   (TR("Save to file ....")) ;
	if (!fDlg.exec ()) return ;

	QString	name	  = fDlg.selectedFile () ;
	if (name.isEmpty()) return ;

	KBFile	file(name)	;
	if (!file.open(IO_WriteOnly|IO_Truncate))
	{
		file.lastError().DISPLAY() ;
		return	;
	}

	file.writeBlock (m_textEdit->text().local8Bit()) ;
}

/*  KBCtrlMemo								*/
/*  eventFilter	: Widget event filter routine				*/
/*  o		: QObject *	: Object receiving event		*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: False to continue processing		*/

bool	KBCtrlMemo::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (m_showing != KB::ShowAsData)
		return	KBControl::eventFilter (o, e) ;

	switch (e->type())
	{
		case QEvent::KeyPress	:
			return	false	;

		case QEvent::FocusIn	:
			/* Get the focus-caret setting; if this is	*/
			/* default then select all the text if focus	*/
			/* is arriving on account of something like	*/
			/* tabbing.					*/
			{
			QFocusEvent *fe = (QFocusEvent *)e ;

			KBField::FocusCaret fc = m_memo->getFocusCaret () ;
			if (fc == KBField::Default)
				if (fe->reason() == QFocusEvent::Other)
					fc = KBField::SelectAll ;

			if (fc != KBField::Default)
				setSelection (fc) ;
			}

			break	;

		case QEvent::FocusOut	:
			switch (m_memo->getMapCase())
			{
				case KBField::MapUpper	:
					m_textEdit->setText (m_textEdit->text().upper()) ;
					break	;

				case KBField::MapLower	:
					m_textEdit->setText (m_textEdit->text().lower()) ;
					break	;

				default	:
					break	;
			}

			break	;

		default	:
			break	;
	}

	if (KBControl::eventFilter (o, e))
		return	true	;

	return	m_textEdit->eventFilter (o, e) ;
}

/*  KBCtrlMemo								*/
/*  viewSetFocusOK							*/
/*		: Check if OK to set focus into the memo		*/
/*  (returns)	: bool		: True if OK				*/

bool	KBCtrlMemo::viewSetFocusOK()
{
	return	m_memo->moveFocusOK (m_drow) ;
}

/*  KBCtrlMemo								*/
/*  setPalette	: Set palette on widget					*/
/*  pal		: const QPalette *: Palette in question			*/
/*  (returns)	: void		  :					*/

void	KBCtrlMemo::setPalette
	(	const QPalette *pal
	)
{
	m_textEdit->setPalette (*pal) ;
}

/*  KBCtrlMemo								*/
/*  setFont	: Set font on widget					*/
/*  font	: const QFont *	: Font in question			*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::setFont
	(	const QFont *font
	)
{
	m_textEdit->setFont (*font) ;
}

/*  KBCtrlMemo								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;

	{
		KBInSetValueLock lock (*this) ;
		setupProperties  ()	 ;
	}
}


/*  KBCtrlMemo								*/
/*  setValue	: Set memo value					*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlMemo::setValue
	(	const KBValue	&value
	)
{
	{
		KBInSetValueLock lock (*this) ;
		m_textEdit->setText (value.getRawText()) ;
	}

	KBControl::setValue (value) ;
}

/*  KBCtrlMemo								*/
/*  getValue	: Get memo value					*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCtrlMemo::getValue ()
{
	QString	text	= m_textEdit->text () ;

	if (text.isEmpty() && getIniValue().isNull())
		return	KBValue (m_memo->getFieldType()) ;

	return	KBValue (text,   m_memo->getFieldType()) ;
}

/*  KBCtrlMemo								*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::clearValue
	(	bool	query
	)
{
	{
		KBInSetValueLock lock  (*this) ;
		m_textEdit->clear () ;
		if (m_showing == KB::ShowAsData)
			m_layoutItem->setValid (isValid (false)) ;
	}

	KBControl::clearValue (query) ;
}

/*  KBCtrlMemo								*/
/*  write	: Write memo 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Memo area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlMemo::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		KBWriterText *wi = new KBWriterText
				   (	writer,
					rect,
					m_memo->getPalette(true),
					m_memo->getFont   (true),
					value.getRawText  (),
					0x1001,
					fSubs
				   )	;

		wi->setParent   (m_memo, m_memo->getBlock()->getCurQRow()) ;
		writerSetFrame	(wi, 0, 0) ;

		extra	= wi->extra()	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

/*  KBCtrlMemo								*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::setupProperties ()
{
	RKTextEdit *textEdit = m_textEdit->textEdit() ;
	ctrlSetFrame
	(	textEdit,
		QFrame::StyledPanel|QFrame::Sunken,
		textEdit->style().pixelMetric(QStyle::PM_DefaultFrameWidth, textEdit)	
	)	;

	m_layoutItem->setValidatorMode (m_memo) ;
	m_textEdit  ->setText	       (QString::null) ;
	m_textEdit  ->setWrapPolicy    (RKTextEdit::AtWordBoundary) ;

#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		m_textEdit->setReadOnly	 (true)			;
		m_textEdit->setHighlight (QString::null)	;
		m_textEdit->setWordWrap	 (RKTextEdit::NoWrap)	;
		if (m_drow == 0) m_textEdit->setText (m_memo->getName()) ;
		return	;
	}
#endif

	m_textEdit->setHighlight (m_memo->getAttrVal("hilite")) ;
	m_textEdit->setReadOnly	 (m_memo->isReadOnly())		;

	if (m_memo->isWrapping())
		m_textEdit->setWordWrap	  (RKTextEdit::WidgetWidth) ;
	else	m_textEdit->setWordWrap	  (RKTextEdit::NoWrap	  ) ;

}

/*  KBCtrlMemo								*/
/*  setSelection: Set text selection					*/
/*  pending	: KBField::FocusCaret					*/
/*				: Required selection			*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::setSelection
	(	KBField::FocusCaret	pending
	)
{
	/* Typically we want to use "setSelection" inside the "onEnter"	*/
	/* event, but in that case we get called too early (before	*/
	/* focus is actually in the control). So, defer the setting on	*/
	/* a zero-period timer.						*/
	m_pendingCaret 	= pending ;
	QTimer::singleShot (0, this, SLOT(slotSetSelection())) ;
}

/*  KBCtrlField								*/
/*  slotSetSelection							*/
/*		: Actually set text selection				*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::slotSetSelection ()
{
	DPRINTF
	((	"KBCtrlMemo::slotSetSelection: %s: %d\n",
		m_memo->getName().latin1(),
		m_pendingCaret
	))	;

	switch (m_pendingCaret)
	{
		case KBField::AtStart   :
			m_textEdit->caretToStart () ;
			break	;

		case KBField::AtEnd     :
			m_textEdit->caretToEnd   () ;
			break	;

		case KBField::SelectAll :
			m_textEdit->selectAll    () ;
			break	;

		default	:
			break	;
	}
}

/*  KBCtrlMemo								*/
/*  setReadOnly	: Explicitely set readonly state			*/
/*  readOnly	: bool		: Required state			*/
/*  (returns)	: void		:					*/

void	KBCtrlMemo::setReadOnly
	(	bool		readOnly,
		const QColor	&fgColor,
		const QColor	&bgColor
	)
{
	if (m_textEdit != 0) m_textEdit->setReadOnly (readOnly) ;
	KBControl::setReadOnly (readOnly, fgColor, bgColor) ;
}

bool	KBCtrlMemo::isReadOnly ()
{
	return	m_textEdit == 0 ? false : m_textEdit->isReadOnly() ;
}

