/***************************************************************************
    file	         : kb_qtreplink.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_writer.h"
#include	"kb_link.h"
#include	"kb_block.h"
#include	"kb_display.h"
#include	"kb_qtreplink.h"



/*  KBCtrlRepLink							*/
/*  KBCtrlRepLink: Constructor for combo box wrapper			*/
/*  display	 : KBDisplay *	 : Parent display			*/
/*  link	 : KBLink *	 : Parent link				*/
/*  drow	 : uint		 : Row number in field			*/
/*  (returns)	 : KBCtrlRepLink :					*/

KBCtrlRepLink::KBCtrlRepLink
	(	KBDisplay	*display,
		KBLink		*link,
		uint		drow
	)
	:
	
	KBControl
	(	display,
		link,
		drow
	),
	m_link	   (link)
{
	m_lineEdit = new RKLineEdit (display->getDisplayWidget()) ;
	setupWidget (m_lineEdit) ;
}

/*  KBCtrlRepLink							*/
/*  ~KBCtrlRepLink							*/
/*		: Destructor for link class				*/
/*  (returns)	:		:					*/

KBCtrlRepLink::~KBCtrlRepLink ()
{
}

void	KBCtrlRepLink::setupProperties ()
{
	m_lineEdit->setText (QString::null) ;

#if	__KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		if (m_drow == 0) m_lineEdit->setText (m_link->getName()) ;
		return	;
	}
#endif
}

/*  KBCtrlRepLink							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlRepLink::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;
	setupProperties	  ()	 ;
}

/*  KBCtrlRepLink							*/
/*  setValue	: Set value						*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlRepLink::setValue
	(	const KBValue	&value
	)
{
	m_lineEdit->setText (m_link->valueToText (value)) ;
	KBControl::setValue (value) ;
}

/*  KBCtrlRepLink							*/
/*  getValue	: Get value						*/
/*  (returns)	: KBValue 	: Value					*/

KBValue	KBCtrlRepLink::getValue ()
{
	return	KBControl::getValue () ;
}

/*  KBCtrlRepLink							*/
/*  clearValue	: Get value						*/
/*  query	: bool		: Clear for query			*/
/*  (returns)	: KBValue 	: Value					*/

void	KBCtrlRepLink::clearValue
	(	bool	query
	)
{
	m_lineEdit->setText   ("")    ;
	KBControl::clearValue (query) ;
}

/*  KBCtrlRepLink							*/
/*  write	: Write link 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlRepLink::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		KBWriterItem *wi = new KBWriterText
				   (	writer,
					rect,
					m_link->getPalette(true),
					m_link->getFont   (true),
					value.getRawText(),
					Qt::AlignLeft|Qt::AlignVCenter,
					false
				)	;
		wi->setParent (m_link, m_link->getBlock()->getCurQRow()) ;

		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}
