/***************************************************************************
    file	         : kb_qtrichtext.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include 	"kb_qtrichtext.moc"
#else
#include 	"kb_qtrichtext.h"
#endif

#include	"kb_richtext.h"
#include	"kb_block.h"
#include	"kb_display.h"
#include	"kb_docroot.h"
#include	"kb_writer.h"
#include	"kb_mimefactory.h"


#include	<qtoolbutton.h>
#include	<qtooltip.h>
#include	<qcursor.h>
#include	<qstyle.h>
#include	<qobjectlist.h>

#include	"rk_textbrowser.h"
#include	"tk_fontdialog.h"
#include	"tk_colordialog.h"
#include	"tk_icons.h"

#define		__DEBUG_QTRICHTEXT	(0)
#if		__DEBUG_QTRICHTEXT
#include	"kb_dprintf.h"
#endif


KBCtrlRichTextWrapper::KBCtrlRichTextWrapper
	(	QWidget			*parent,
		KBCtrlRichText		*richText
	)
	:
	RKTextBrowser	(parent),
	m_richText	(richText)
{
}

void	KBCtrlRichTextWrapper::emitLinkClicked
	(	const QString	&link
	)
{
	m_richText->emitLinkClicked (link) ;
}

void	KBCtrlRichTextWrapper::slotToggleBold ()
{
	setBold (!bold()) ;
}

void	KBCtrlRichTextWrapper::slotToggleItalic ()
{
	setItalic (!italic()) ;
}

void	KBCtrlRichTextWrapper::slotToggleUnderline ()
{
	setUnderline (!underline()) ;
}

void	KBCtrlRichTextWrapper::slotFontSizeUp ()
{
	setPointSize (pointSize() + 1) ;
}

void	KBCtrlRichTextWrapper::slotFontSizeDown ()
{
	if (pointSize() > 1)
		setPointSize (pointSize() - 1) ;
}

void	KBCtrlRichTextWrapper::slotFont ()
{
	TKFontDialog  fDlg (0, TR("Font"), false, true) ;
	fDlg.setFont (currentFont(), false) ;
	if (fDlg.exec()) setCurrentFont (fDlg.font()) ;
}

void	KBCtrlRichTextWrapper::slotColor ()
{
	TKColorDialog cDlg (0, TR("Colour"), true) ;
	cDlg.setColor (color()) ;
	if (cDlg.exec()) setColor (cDlg.color()) ;
}

void	KBCtrlRichTextWrapper::slotTextLeft ()
{
	setAlignment (Qt::AlignLeft) ;
}

void	KBCtrlRichTextWrapper::slotTextRight ()
{
	setAlignment (Qt::AlignRight) ;
}

void	KBCtrlRichTextWrapper::slotTextCenter ()
{
	setAlignment (Qt::AlignCenter) ;
}



/*  ------------------------------------------------------------------	*/

KBCtrlRichTextContainer::KBCtrlRichTextContainer
	(	QWidget			*parent,
		KBCtrlRichText		*richText
	)
	:
	RKVBox		(parent),
	m_richText	(richText)
{
	m_toolBar	= new RKHBox		    (this) ;
	m_wrapper	= new KBCtrlRichTextWrapper (this, richText) ;
	m_toolBarInited	= false	;

	m_toolBar->hide () ;
}

KBCtrlRichTextWrapper *KBCtrlRichTextContainer::wrapper ()
{
	return	m_wrapper	;
}

/*  setupToolBar: Set up tool bar					*/
/*  large	: bool		  : Use large icons			*/
/*  tools	: const QString & : Which tools to show			*/

void	KBCtrlRichTextContainer::setupToolBar
	(	bool			large,
		const QString		&tools
	)
{
	DPRINTF
	((	"KBCtrlRichTextContainer::setupToolBar: l=%d t=%s\n",
		large,
		tools.latin1()
	))	;

	/* Initialise the toolbar on demand. Usually it will not be	*/
	/* needed so this saves unwanted work.				*/
	if (!m_toolBarInited)
	{
		struct	ToolBarItem
		{	const char	*m_name		;
			const char	*m_icon		;
			const char	*m_tip		;
			const char	*m_slot		;
		}	;

		static	ToolBarItem	toolBarItems[]	=
		{
		{	"bold",		"text_bold",	"Bold",			SLOT(slotToggleBold     ())	},
		{	"italic",	"text_italic",	"Italic",		SLOT(slotToggleItalic   ())	},
		{	"under",	"text_under",	"Underline",		SLOT(slotToggleUnderline())	},
		{	"font",		"font",		"Set font",		SLOT(slotFont		())	},
		{	"font-up",	"fontsizeup",	"Increase font size",	SLOT(slotFontSizeUp	())	},
		{	"font-down",	"fontsizedown",	"Decrease font size",	SLOT(slotFontSizeDown	())	},
		{	"color",	"color_line",	"Set font color",	SLOT(slotColor		())	},
		{	"text-left",	"text_left",	"Left align text",	SLOT(slotTextLeft	())	},
		{	"text-right",	"text_right",	"Right align text",	SLOT(slotTextRight	())	},
		{	"text-center",	"text_center",	"Center text",		SLOT(slotTextCenter	())	},
		{	0,		0,		0,			0				}
		}	;
		
		m_toolBarInited = true	;

		QPixmap (*getIcon)(const QString &) ;
		if (large)
			getIcon	= getBarIcon	;
		else	getIcon	= getSmallIcon	;

		for (const ToolBarItem *tbi = &toolBarItems[0] ; tbi->m_icon != 0 ; tbi += 1)
		{
			QPixmap   icon = (*getIcon) (tbi->m_icon) ;
			QToolButton *b = new QToolButton (m_toolBar, tbi->m_name) ;
			b->setIconSet   (icon) ;
			b->setFixedSize (icon.width(),icon.height()) ;
			b->setCursor	(QCursor::ArrowCursor) ;
			QToolTip::add	(b, TR(tbi->m_tip)) ;

			connect	(b, SIGNAL(clicked()), m_wrapper, tbi->m_slot)  ;
		}

		m_toolBar->addFiller () ;
	}

	/* Decide which tools need to be displayed. If none are enabled	*/
	/* then the toolbar is hidden.					*/
	QStringList	  enable	= QStringList::split (',', tools) ;
	const QObjectList *children	= m_toolBar->children() ;
	bool		  any		= false ;

	for
	(	QObjectListIt iter (*children) ;
		iter.current() != 0 ;
		++iter
	)
		if (iter.current()->isWidgetType())
			if (enable.contains(iter.current()->name()))
			{	((QWidget *)iter.current())->show () ;
				any	= true ;
			}
			else	((QWidget *)iter.current())->hide () ;

	DPRINTF
	((	"KBCtrlRichTextContainer::setupToolBar: any=%d\n",
		any
	))	;

	if (any)
		m_toolBar->show() ;
	else	m_toolBar->hide() ;
}

void	KBCtrlRichTextContainer::setPalette
	(	const QPalette		&palette
	)
{
	m_wrapper->setPalette	(palette) ;
}

/*  setFont	: Set the font						*/
/*  font	: const QFont &	: Font in question			*/
/*  (returns)	: void		 :					*/

void	KBCtrlRichTextContainer::setFont
	(	const QFont	&font
	)
{
	m_wrapper->setFont	(font) ;
}


/*  ------------------------------------------------------------------	*/

/*  KBCtrlRichText							*/
/*  KBCtrlRichText							*/
/*		: Constructor for line edit wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  field	: KBField *	: Parent field				*/
/*  drow	: uint		: Row number in field			*/
/*  (returns)	: KBCtrlRichText:					*/

KBCtrlRichText::KBCtrlRichText
	(	KBDisplay	*display,
		KBRichText	*richText,
		uint		drow
	)
	:
	KBControl
	(	display,
		richText,
		drow
	),
	m_richText	(richText)
{
	m_container	= new KBCtrlRichTextContainer
			  (	display->getDisplayWidget(),
				this
			  )	;
	m_wrapper	= m_container->wrapper () ;

	m_wrapper->setMimeSourceFactory (m_richText->getDocRoot()->mimeSource()) ;
	m_wrapper->setTextFormat	(Qt::RichText) ;

	connect
	(	m_wrapper,
		SIGNAL	(textChanged	()),
		SLOT	(userChange	())
	)	;

	setupWidget (m_container) ;
}

/*  KBCtrlRichText							*/
/*  ~KBCtrlRichText							*/
/*		: Destructor for field class				*/
/*  (returns)	:		:					*/

KBCtrlRichText::~KBCtrlRichText ()
{
}

/*  KBCtrlRichText							*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: bool		:					*/

void	KBCtrlRichText::setupProperties ()
{
	ctrlSetFrame
	(	m_wrapper,
		QFrame::StyledPanel|QFrame::Sunken,
		m_wrapper->style().pixelMetric(QStyle::PM_DefaultFrameWidth, m_wrapper)	
	)	;
	m_wrapper  ->setText	 (QString::null) ;
	m_wrapper  ->setCursor	 (QCursor(Qt::ibeamCursor)) ;
	m_container->setupToolBar(true, m_richText->getAttrVal("toolbar")) ;

#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		if (m_drow == 0) m_wrapper->setText (m_richText->getName()) ;
		m_wrapper->setReadOnly (true) ;
		return	;
	}
#endif

	m_wrapper->setReadOnly (m_richText->isReadOnly()) ;
}

/*  KBCtrlRichText							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlRichText::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;
	setupProperties	  ()	 ;
}

/*  KBCtrlRichText							*/
/*  eventFilter	: Widget event filter routine				*/
/*  o		: QObject *	: Object receiving event		*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: False to continue processing		*/

bool	KBCtrlRichText::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (m_showing != KB::ShowAsData)
		return	KBControl::eventFilter (o, e) ;

	switch (e->type())
	{
		case QEvent::KeyPress	:
			if ((((QKeyEvent *)e)->state() & Qt::ControlButton) == 0)
				return	false	;

			switch (((QKeyEvent *)e)->key())
			{
				case Qt::Key_B	:
					m_wrapper->slotToggleBold     () ;
					return	true	;

				case Qt::Key_I	:
					m_wrapper->slotToggleItalic   () ;
					return	true	;

				case Qt::Key_U	:
					m_wrapper->slotToggleUnderline() ;
					return	true	;
			}

			return	false	;

		default	:
			break	;
	}

	if (KBControl::eventFilter (o, e))
		return	true	;

	return	m_wrapper->eventFilter (o, e) ;
}

/*  KBCtrlRichText							*/
/*  setValue	: Set field value					*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlRichText::setValue
	(	const KBValue	&value
	)
{
	DPRINTF
	((	"KBCtrlRichText::setValue: %d: [%s]\n",
		m_drow,
		value.getRawText().latin1()
	))	;

	{
		KBInSetValueLock lock (*this) ;
		m_wrapper->setText  (value.getRawText()) ;
	}

	KBControl::setValue (value) ;
}

/*  KBCtrlRichText							*/
/*  getValue	: Get field value					*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCtrlRichText::getValue ()
{
	/* If the control is empty and the initial value was null (as	*/
	/* opposed to an empty string), then return a null string.	*/
	if (m_wrapper->text().isEmpty() && getIniValue().isNull())
		return	KBValue (m_richText->getFieldType()) ;

	return	KBValue (m_wrapper->text(), m_richText->getFieldType()) ;
}

/*  KBCtrlRichText							*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBCtrlRichText::clearValue
	(	bool	query
	)
{
	{
		KBInSetValueLock lock (*this) ;
		m_wrapper->setText    (QString::null) ;
	}

	KBControl::clearValue (query) ;
}

/*  KBCtrlRichText							*/
/*  write	: Write field 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlRichText::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		KBWriterText *w = new KBWriterText
				  (	writer,
					rect,
					m_richText->getPalette(true),
					m_richText->getFont   (true),
					value.getRawText    (),
					0x1001,
					fSubs
				  )	;
		extra	= w->extra()	;
		return	true ;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

/*  KBCtrlRichText							*/
/*  emitLinkClicked							*/
/*		: Link click method					*/
/*  link	: const QString & : Link				*/
/*  (returns)	: void		  :					*/

void	KBCtrlRichText::emitLinkClicked
	(	const QString	&link
	)
{
	/* Link clicking is handled locally via an event function, so	*/
	/* this is a nasty hack. In order that we can display text that	*/
	/* contains embedded URLs and *not* have QTextBrowser follow	*/
	/* them automatically, we override this method (even though it	*/
	/* is declared as private!).					*/
	DPRINTF
	((	"KBCtrlRichText::emitLinkClicked: [%d][%s] focus=[%d]\n",
		m_drow,
		link.latin1(),
		m_wrapper->hasFocus()
	))	;

	m_richText->linkClicked (m_richText->getBlock()->getCurDRow() + m_drow, link) ;
}

