/***************************************************************************
    file	         : kb_qtrowmark.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<ctype.h>

#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>


#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_rowmark.h"
#include	"kb_display.h"
#include	"kb_writer.h"
#include	"kb_qtrowmark.h"
#include	"kb_queryset.h"
#include	"kb_formblock.h"

#include	"tk_icons.h"



/*  KBCtrlRowMark							*/
/*  KBCtrlRowMark: Constructor for marker widget			*/
/*  display	 : KBDisplay *	: Parent display			*/
/*  rowmark	 : KBRowMark *	: Parent row marker			*/
/*  drow	 : uint		: Row number in marker			*/
/*  (returns)	 : KBCtrlRowMark:					*/

KBCtrlRowMark::KBCtrlRowMark
	(	KBDisplay	 *display,
		KBRowMark	 *rowmark,
		uint	 	 drow
	)
	:
	KBControl
	(	display,
		rowmark,
		drow
	),
	m_rowmark (rowmark)
{
	m_current = false	 ;
	m_inBlock = false	 ;
	m_state	  = KB::RSInSync ;

	m_label = new QLabel (display->getDisplayWidget ()) ;
	m_label->setAlignment (Qt::AlignRight|Qt::AlignTop) ;
	setupWidget (m_label)	 ;

	m_showRow = m_rowmark->getShowRow() ;
}

/*  KBCtrlRowMark							*/
/*  ~KBCtrlRowMark: Destructor for marker widget			*/
/*  (returns)	  :		:					*/

KBCtrlRowMark::~KBCtrlRowMark ()
{
}

/*  KBCtrlRowMark							*/
/*  eventFilter	: Widget event filter					*/
/*  o		: QObject *	: Object originating event		*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: False to continue processing		*/

bool	KBCtrlRowMark::eventFilter
	(	QObject	*o,
		QEvent	*e
	)
{
	if (e->type() == QEvent::MouseButtonPress)
		if (m_current && (m_showing == KB::ShowAsData))
		{
			QMouseEvent *me      = (QMouseEvent *)e ;
			uint	    qrow     = m_rowmark->getBlock()->getCurDRow() + m_drow ;
			KB::MarkOp  op	     ;

			if	(me->state() & QMouseEvent::ControlButton)
				op = KB::MarkOpToggle ;
			else if	(me->state() & QMouseEvent::ShiftButton  )
				op = KB::MarkOpRange  ;
			else
				op = KB::MarkOpSet    ;

			m_rowmark->setRowMarked (qrow, op) ;
		}

	if (e->type() == QEvent::MouseButtonRelease)
		if (m_current && (m_showing == KB::ShowAsData))
		{
			m_rowmark->doSingleClick (m_rowmark->getBlock()->getCurDRow() + m_drow) ;
			return	true	;
		}

	return	KBControl::eventFilter (o, e) ;
}

/*  KBCtrlRowMark							*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: bool		:					*/

void	KBCtrlRowMark::setupProperties ()
{
	ctrlSetFrame (m_label, 0, 0) ;
}

/*  KBCtrlRowMark							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlRowMark::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;

	if (mode == KB::ShowAsData)
	{
		m_showRow = m_rowmark->getShowRow() ;
		m_rowNo	  = 0x7fffffff	;
		return	  ;
	}

	m_rowNo	= 0 ;
	m_label->setText (m_showRow ? QString("%1").arg(m_drow) : QString::null) ;
}

/*  KBCtrlRowMark							*/
/*  showIcon	: Show icon appropriate to the state			*/
/*  (returns)	: void		:					*/

void	KBCtrlRowMark::showIcon ()
{
	if (m_current)
	{
		if (m_inBlock)
			m_label->setPixmap (getSmallIcon("current")) ;
		else	m_label->setPixmap (getSmallIcon("elsewhere")) ;

		return	;
	}

	switch (m_state)
	{
		case KB::RSDeleted  :
			m_label->setPixmap (getSmallIcon("removed" )) ;
			break	;

		case KB::RSChanged  :
			m_label->setPixmap (getSmallIcon("changed" )) ;
			break	;

		case KB::RSInserted :
			m_label->setPixmap (getSmallIcon("inserted")) ;
			break	;

		default	:
			m_label->setText
			(	m_showRow ?
					QString("%1").arg(m_rowNo + 1) :
					QString("")
			)	;
			break	;
	}
}

/*  KBCtrlRowMark							*/
/*  setState	: Set rowmark state					*/
/*  state	: KB::RState	: New state				*/
/*  (returns)	: void		:					*/

void	KBCtrlRowMark::setState
	(	KB::RState	state
	)
{
	if (state   != m_state)
	{	m_state	  = state   ;
		showIcon  () ;
	}
}

void	KBCtrlRowMark::setCurrent
	(	bool		current
	)
{
	if (current != m_current)
	{	m_current = current ;
		showIcon  ()	;
	}
}

void	KBCtrlRowMark::setInBlock
	(	bool		inBlock
	)
{
	if (inBlock != m_inBlock)
	{	m_inBlock = inBlock ;
		showIcon  ()	;
	}
}

void	KBCtrlRowMark::setRowNumber
	(	uint		rowNo
	)
{
	if (rowNo   != m_rowNo)
	{	m_rowNo	  = rowNo  ;
		showIcon  ()	;
	}
}

/*  KBCtrlRowMark							*/
/*  clearValue	: Clear value						*/
/*  query	: bool		: Clear for query			*/
/*  (returns)	: void		:					*/

void	KBCtrlRowMark::clearValue
	(	bool
	)
{
	m_state	= KB::RSInSync ;
	showIcon ()    ;
}

/*  KBCtrlRowMark							*/
/*  write	: Write row mark 					*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlRowMark::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}
