/***************************************************************************
    file	         : kb_qtspinbox.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qspinbox.h>
#include	<qlineedit.h>
#include	<qobjectlist.h>


#if		__KB_AUTOMOC
#include 	"kb_qtspinbox.moc"
#else
#include 	"kb_qtspinbox.h"
#endif

#include	"kb_spinbox.h"
#include	"kb_writer.h"
#include	"kb_block.h"
#include	"kb_display.h"
#include	"kb_docroot.h"
#include	"kb_helper.h"



#define		__DEBUG_QTSPINBOX	(0)
#if		__DEBUG_QTSPINBOX
#include	"kb_dprintf.h"
#endif


class	KBCtrlSpinBoxWrapper : public QSpinBox
{
public	:

	inline	KBCtrlSpinBoxWrapper
		(	QWidget		*parent
		)
		:
		QSpinBox (parent)
	{
	}

	friend	class	KBCtrlSpinBox ;
}	;



/*  KBCtrlSpinBox							*/
/*  KBCtrlSpinBox: Constructor for spinbox wrapper			*/
/*  display	 : KBDisplay *	 : Parent display			*/
/*  spinbox	 : KBSpinBox *	 : Parent spinbox			*/
/*  drow	 : uint		 : Row number in field			*/
/*  (returns)	 : KBCtrlSpinBox :					*/

KBCtrlSpinBox::KBCtrlSpinBox
	(	KBDisplay	*display,
		KBSpinBox	*spinbox,
		uint		drow
	)
	:
	KBControl
	(	display,
		spinbox,
		drow
	),
	m_spinbox	(spinbox)
{
	DPRINTF
	((	"KBCtrlSpinBox::KBCtrlSpinBox: creating row %d\n",
		drow
	))	;
	m_qtSpinBox = new KBCtrlSpinBoxWrapper (display->getDisplayWidget()) ;
	setupWidget (m_qtSpinBox) ;

	m_inSetValue	= false	;
	m_isNull	= false	;

	connect
	(	m_qtSpinBox,
		SIGNAL	(valueChanged	(const QString &)),
		SLOT	(userChange	())
	)	;
	connect
	(	m_qtSpinBox->editor(),
		SIGNAL	(textChanged	(const QString &)),
		SLOT	(userChange	())
	)	;

	m_editor  = m_qtSpinBox->editor() ;
	m_spinner = 0	     ;

	QObjectList *ol = m_qtSpinBox->queryList ("QSpinWidget") ;
	if (ol != 0)
	{	if (ol->count() > 0) m_spinner = (QWidget *)ol->at(0) ;
		delete	ol ;
	}

	DPRINTF
	((	"KBCtrlSpinBox::KBCtrlSpinBox: m_editor=%p m_spinner=%p\n",
		(void *)m_editor,
		(void *)m_spinner
	))	;

	m_editor ->installEventFilter (this) ;
	m_spinner->installEventFilter (this) ;
	RKApplication::self()->installMousePressFilter (m_editor,  this) ;
	RKApplication::self()->installMousePressFilter (m_spinner, this) ;
}

/*  KBCtrlSpinBox							*/
/*  ~KBCtrlSpinBox: Destructor for spinbox class			*/
/*  (returns)	  :		:					*/

KBCtrlSpinBox::~KBCtrlSpinBox ()
{
}


/*  KBCtrlSpinBox							*/
/*  mapValueToText: Map spin box value to text				*/
/*  value	  : int		: Value					*/
/*  (returns)	  : QString	: Text to display			*/

QString	KBCtrlSpinBox::mapValueToText
	(	int		value
	)
{
	/* If in design view then show the control name for the first	*/
	/* rown and blank thereafter.					*/
	if (m_showing == KB::ShowAsDesign)
		return	m_drow == 0 ? m_spinbox->getName() : QString::null ;

	/* Otherwise, show null as empty text, and all non-null values	*/
	/* as normal.							*/
	return	m_isNull ? QString::null : m_qtSpinBox->mapValueToText (value) ;
}


/*  KBCtrlSpinBox							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlSpinBox::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;

	{
		KBInSetValueLock lock (*this) ;
		setupProperties  ()	 ;
	}
}

void	KBCtrlSpinBox::setupProperties ()
{
	m_layoutItem->setValidatorMode	(m_spinbox)	;
	m_editor    ->setText		(QString::null) ;

#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		if (m_drow == 0) m_editor->setText (m_spinbox->getName()) ;
		return	;
	}
#endif
	m_qtSpinBox->setValue (0) ;
}

/*  KBCtrlSpinBox							*/
/*  setValue	: Set spinbox value					*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlSpinBox::setValue
	(	const KBValue	&value
	)
{
	DPRINTF
	((	"KBCtrlSpinBox::setValue: %d: %s\n",
		m_drow,
		value.getRawText().latin1()
	))	;

	if (m_qtSpinBox != 0)
	{
		KBInSetValueLock lock  (*this) ;
		m_isNull = value.isNull() ;
		m_qtSpinBox->setValue (value.getRawText().toInt()) ;
	}

	KBControl::setValue (value) ;
}

/*  KBCtrlSpinBox							*/
/*  getValue	: Get spinbox value					*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCtrlSpinBox::getValue ()
{
	/* If the control value is null and the initial value was null	*/
	/* (as opposed to an empty string), then return a null.		*/
	if (m_isNull && getIniValue().isNull())
		return	KBValue (m_spinbox->getFieldType()) ;

	return	KBValue (m_qtSpinBox->value(), m_spinbox->getFieldType()) ;
}

/*  KBCtrlSpinBox							*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBCtrlSpinBox::clearValue
	(	bool	query
	)
{
	{
		KBInSetValueLock lock (*this) ;
		m_isNull = true		 ;
		m_qtSpinBox->setValue(0) ;
	}

	KBControl::clearValue (query) ;
}

/*  KBCtrlSpinBox							*/
/*  isEmpty	: See if control is empty				*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlSpinBox::isEmpty ()
{
	return	m_isNull ;
}

/*  KBCtrlSpinBox							*/
/*  isValid	: See if control is valid				*/
/*  allowNull	: bool		: True to ignore not-null check		*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlSpinBox::isValid
	(	bool		allowNull
	)
{
	return	!m_isNull || allowNull || m_spinbox->allowNull() ;
}

/*  KBCtrlSpinBox							*/
/*  userChange	: User has changed value				*/
/*  (returns)	: bool		  :					*/

bool	KBCtrlSpinBox::userChange ()
{
	DPRINTF
	((	"KBCtrlSpinBox::userChange: called: value=[%s]\n",
		m_qtSpinBox->text().latin1()
	))	;

	if (KBControl::userChange())
	{
		m_isNull = m_qtSpinBox->text().isEmpty() ;
		return	true ;
	}

	return	false	;
}

/*  KBCtrlSpinBox							*/
/*  write	: Write spinbox 					*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: spinbox area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlSpinBox::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

/*  KBCtrlSpinBox							*/
/*  morphText	: Get text for morphed control				*/
/*  align	: int &		: Return alignment			*/
/*  (returns)	: QString	: Text					*/

QString	KBCtrlSpinBox::morphText
	(	int		&align
	)
{
	return	QString::number(m_qtSpinBox->value()) ;
}

