/***************************************************************************
    file	         : kb_qttabber.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<stdarg.h>

#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>

#include	"kb_classes.h"
#include	"kb_attr.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_event.h"
#include	"kb_tabber.h"
#include	"kb_display.h"
#include	"kb_writer.h"
#include	"kb_qttabber.h"
#include	"kb_ascii.h"


class	KBCtrlTabBar : public QTabBar
{
	KBTabberBar	*m_tabberBar	;

public	:

	inline	KBCtrlTabBar
		(	QWidget		*parent,
			KBTabberBar	*tabberBar
		)
		:
		QTabBar		(parent),
		m_tabberBar	(tabberBar)
	{
	}

	virtual	void	layoutTabs	()	;
		
}	;

void	KBCtrlTabBar::layoutTabs ()
{
	QTabBar::layoutTabs() ;
	if (!m_tabberBar->wideTabs()) return ;

	uint	nTabs	= count() ;
	if (nTabs == 0) return	  ;

	uint	tWidth	= width()/nTabs ;
	for (uint idx = 0 ; idx < nTabs ; idx += 1)
	{
		QTab	*tab	= tabAt(idx)	;
		QRect	rect	= tab->rect()	;

		rect.setLeft	(tWidth * idx)	;
		rect.setWidth	(tWidth)	;

//		fprintf
//		(	stderr,
//			"KBCtrlTabBar::layoutTabs: %d: %s  *****\n",
//			idx,
//			_TEXT(rect)
//		)	;

		tab->setRect	(rect)	;
	}
}

/*  ------------------------------------------------------------------	*/


/*  KBCtrlTabberBar							*/
/*  KBCtrlTabberBar							*/
/*		: Constructor for tab bar wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  atbber	: KBTabber *	: Parent tabber				*/
/*  design	: bool		: Design mode				*/
/*  (returns)	: KBCtrlTabber	:					*/

KBCtrlTabberBar::KBCtrlTabberBar
	(	KBDisplay	*display,
		KBTabberBar	*tabberBar
	)
	:
	KBControl
	(	display,
		tabberBar
	),
	m_tabberBar	(tabberBar)
{
	m_qtTabBar = new KBCtrlTabBar (display->getDisplayWidget(), tabberBar) ;
	setupWidget (m_qtTabBar) ;

	QTabBar::connect
	(	m_qtTabBar,
		SIGNAL(   selected(int)),
		m_tabberBar,
 		SLOT  (tabSelected(int))
	)	;
}

/*  KBCtrlTabberBar							*/
/*  ~KBCtrlTabberBar							*/
/*		: Destructor for tab bar widget				*/
/*  (returns)	:		:					*/

KBCtrlTabberBar::~KBCtrlTabberBar ()
{
}

/*  KBCtrlTabberBar							*/
/*  addTab	: Add a new tab						*/
/*  label	: const QString & : Tab text				*/
/*  (returns)	: int		  : Tab identifier			*/

int	KBCtrlTabberBar::addTab
	(	const QString	&label
	)
{
	return	m_qtTabBar->addTab (new QTab (label)) ;
}

/*  KBCtrlTabberBar							*/
/*  setTabText	: Set tab text						*/
/*  id		: int		  : Tab identififer			*/
/*  text	: const QString & : Text				*/
/*  (returns)	: void							*/

void	KBCtrlTabberBar::setTabText
	(	int		id,
		const QString	&text
	)
{
	QTab *t = m_qtTabBar->tab (id) ;
	if (t != 0) t->setText  (text) ;
}

/*  KBCtrlTabberBar							*/
/*  getTabRect	: Get tab area for specified page			*/
/*  id		: int		: Page identifier			*/
/*  (returns)	: QRect		: Area					*/

QRect	KBCtrlTabberBar::getTabRect
	(	int		id
	)
{
	fprintf
	(	stderr,
		"KBCtrlTabberBar::getTabRect: WIDTH=%d\n",
		m_qtTabBar->width()
	)	;

	QTab	*t = m_qtTabBar->tab (id) ;
	return	t == 0 ? QRect() : t->rect() ;
}

/*  KBCtrlTabberBar							*/
/*  removeTab	: Remove a tab						*/
/*  id		: int		: Tab identififer			*/
/*  (returns)	: void							*/

void	KBCtrlTabberBar::removeTab
	(	int		id
	)
{
	QTab *t = m_qtTabBar->tab (id) ;
	if (t != 0) m_qtTabBar->removeTab (t) ;
}


/*  KBCtrlTabberBar							*/
/*  write	: Write field 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlTabberBar::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		extra	= 0  ;
		return	true ;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

int	KBCtrlTabberBar::getNumTabs ()
{
	return	m_qtTabBar->count () ;
}

int	KBCtrlTabberBar::getCurrentTab ()
{
	return	m_qtTabBar->currentTab () ;
}

void	KBCtrlTabberBar::setCurrentTab
	(	int		id
	)
{
	m_qtTabBar->setCurrentTab (id) ;
}

void	KBCtrlTabberBar::setTabEnabled
	(	int		id,
		bool		enabled
	)
{
	m_qtTabBar->setTabEnabled (id, enabled) ;
}

