/***************************************************************************
    file	         : kb_qttree.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<time.h>

#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>
#include	<qlistbox.h>
#include	<qtimer.h>
#include	<qptrstack.h>
#include	<qheader.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_writer.h"
#include	"kb_block.h"
#include	"kb_tree.h"
#include	"kb_display.h"

#include	"kb_multilistbox.h"

#if		__KB_AUTOMOC
#include 	"kb_qttree.moc"
#else
#include 	"kb_qttree.h"
#endif


#define		__DEBUG_QTTREE	(0)
#if		__DEBUG_QTTREE
#include	"kb_dprintf.h"
#endif


class	KBCtrlTreeListView : public RKListView
{
public	:

	inline	KBCtrlTreeListView
		(	QWidget		*parent
		)
		:
		RKListView (parent)
	{
	}

	friend	class	KBCtrlTree ;
}	;



/*  KBCtrlTreeItem							*/
/*  --------------							*/
/*  Class used for entries in the tree ...				*/

class	KBCtrlTreeItem : public QListViewItem
{
	KBCtrlTree		*m_ctrlTree	;
	const KBLTValueSet	&m_valset	;

	/* The index value is the index into the value set for this	*/
	/* item, with negative values for branch nodes that do not map	*/
	/* to actual values.						*/
	int			m_index		;

	/* The from and use values specify the range of entries in the	*/
	/* value set that are covered by this item. The use value will	*/
	/* always be unity for leaf nodes.				*/
	int			m_from		;
	int			m_use		;

public	:

	KBCtrlTreeItem
	(	RKListView		*parent,
		QListViewItem		*after,
		KBCtrlTree		*ctrlTree,
		const KBLTValueSet	&valset,
		int			index,
		int			from,
		int			use,
		int			data,
		int			width
	)	;
	KBCtrlTreeItem
	(	QListViewItem		*parent,
		QListViewItem		*after,
		KBCtrlTree		*ctrlTree,
		const KBLTValueSet	&valset,
		int			index,
		int			from,
		int			use,
		int			data,
		int			width
	)	;


	bool			containsIndex	(int)	;
	const	QStringList	&entry		() 	;
	virtual	void		setOpen		(bool)	;

	inline	int	index	()
	{
		return	m_index	;
	}
}	;


/*  KBCtrlTreeItem							*/
/*  KBCtrlTreeItem							*/
/*		: Constructor for tree tree item			*/
/*  parent	: RKListView *	  : Parent list view			*/
/*  after	: QListViewItem * : Insert after			*/
/*  ctrlTree	: KBCtrlTree *	  : Control object			*/
/*  valset	: KBLTValueSet &  : Data value set			*/
/*  index	: int		  : Corresponding entry in value set	*/
/*  from	: int		  : Index of first entry in value set	*/
/*  use		: int		  : Number of value set entries covered	*/
/*  data	: int		  : Index for first display data value	*/
/*  width	: int		  : Number of values displayed		*/
/*  (returns)	: KBCtrlTreeItem  :					*/

KBCtrlTreeItem::KBCtrlTreeItem
	(	RKListView		*parent,
		QListViewItem		*after,
		KBCtrlTree		*ctrlTree,
		const KBLTValueSet	&valset,
		int			index,
		int			from,
		int			use,
		int			data,
		int			width
	)
	:
	QListViewItem	(parent, after),
	m_ctrlTree	(ctrlTree),
	m_valset	(valset),
	m_index		(index),
	m_from		(from),
	m_use		(use)
{
	DPRINTF
	((	"KBCtrlTreeItem::KBCtrlTreeItem: top    [%d:%d] at %d\n",
		m_index,
		m_use,
		time(0)
	))	;

	const QStringList &e = entry() ;

	for (int col = 0 ; col < width ; col += 1)
		if (data + col < (int)e.count())
			setText	(col, e[data + col]) ;
		else	setText	(col, QString::null) ;

	setExpandable (m_ctrlTree != 0) ;
}

/*  KBCtrlTreeItem							*/
/*  KBCtrlTreeItem							*/
/*		: Constructor for tree tree item			*/
/*  parent	: QListViewItem * : Parent list view item		*/
/*  after	: QListViewItem * : Insert after			*/
/*  ctrlTree	: KBCtrlTree *	  : Control object			*/
/*  valset	: KBLTValueSet &  : Data value set			*/
/*  index	: int		  : Corresponding entry in value set	*/
/*  from	: int		  : Index of first entry in value set	*/
/*  use		: int		  : Number of value set entries covered	*/
/*  data	: int		  : Index for first display data value	*/
/*  width	: int		  : Number of values displayed		*/
/*  (returns)	: KBCtrlTreeItem  :					*/

KBCtrlTreeItem::KBCtrlTreeItem
	(	QListViewItem		*parent,
		QListViewItem		*after,
		KBCtrlTree		*ctrlTree,
		const KBLTValueSet	&valset,
		int			index,
		int			from,
		int			use,
		int			data,
		int			width
	)
	:
	QListViewItem	(parent, after),
	m_ctrlTree	(ctrlTree),
	m_valset	(valset),
	m_index		(index),
	m_from		(from),
	m_use		(use)
{
	DPRINTF
	((	"KBCtrlTreeItem::KBCtrlTreeItem: child [%d,%d]\n",
		m_index,
		m_use
	))	;
	const QStringList &e = entry() ;

	for (int col = 0 ; col < width ; col += 1)
		setText	(col, e[data + col]) ;

	setExpandable (m_ctrlTree != 0) ;
}


/*  KBCtrlTreeItem							*/
/*  containsIndex: See if specified index is within this item		*/
/*  idx		 : int		: Required index			*/
/*  (returns)	 : bool		: True if so				*/

bool	KBCtrlTreeItem::containsIndex
	(	int		idx
	)
{
	DPRINTF
	((	"KBCtrlTreeItem::containsIndex: %d: (%d,%d)\n",
		idx,
		m_from,
		m_from + m_use
	))	;
	return	(m_from <= idx) && (idx < m_from + m_use) ;

}

/*  KBCtrlTreeItem							*/
/*  entry	: Get data values entry for this item			*/
/*  (returns)	: QStringList &	: Values				*/

const QStringList &KBCtrlTreeItem::entry ()
{
	return	m_valset[m_from] ;
}

/*  KBCtrlTreeItem							*/
/*  setOpen	: Open or close item					*/
/*  open	: bool		: True to open				*/
/*  (returns)	: void		:					*/

void	KBCtrlTreeItem::setOpen
	(	bool		open
	)
{
	if (open)
		/* If the control tree is set (will not be for known	*/
		/* leaf nodes) and there are no children then create	*/
		/* the children. We can then clear the tree pointer	*/
		/* (saves trying repeatedly if no children are		*/
		/* forthcoming).					*/
		if ((m_ctrlTree != 0) && (firstChild() == 0))
		{
			m_ctrlTree->loadDataTree (this, m_valset, m_from, m_use, depth() + 1) ;
			m_ctrlTree = 0 ;
		}

	QListViewItem::setOpen (open) ;
}


/*  ------------------------------------------------------------------	*/

/*  KBCtrlTree								*/
/*  KBCtrlTree	: Constructor for combo box wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  tree	: KBTree *	: Parent tree				*/
/*  drow	: uint		: Row number in field			*/
/*  (returns)	: KBCtrlTree	:					*/

KBCtrlTree::KBCtrlTree
	(	KBDisplay	*display,
		KBTree		*tree,
		uint		drow
	)
	:
	KBControl
	(	display,
		tree,
		drow
	),
	m_tree		(tree)
{
	m_listView = new KBCtrlTreeListView (display->getDisplayWidget()) ;
	m_listView->setSorting		(-1)	;
	m_listView->setRootIsDecorated	(true)	;
	m_listView->header()->hide()		;

	setupWidget (m_listView)		;

	connect
	(	m_listView,
		SIGNAL	(currentChanged	(QListViewItem *)),
		SLOT	(deferUserChange())
	)	;

	m_deferFocus	= 0	;
	m_deferMouse	= 0	;
	m_keyset	= 0	;
	m_valset	= 0	;
	m_inSetValue	= false	;
}

/*  KBCtrlTree								*/
/*  ~KBCtrlTree	: Desctructor for tree class				*/
/*  (returns)	:		:					*/

KBCtrlTree::~KBCtrlTree ()
{
	DELOBJ	(m_deferFocus)	;
	DELOBJ	(m_deferMouse)	;

	DELOBJ  (m_keyset) ;
	DELOBJ  (m_valset) ;
}


/*  KBCtrlTree								*/
/*  loadDataTree: Load data as tree					*/
/*  parent	: KBCtrlTreeItem * : Parent item or null for top level	*/
/*  valset	: 		   : List of values			*/
/*  fromRow	: uint		   : Load data starting at row ...	*/
/*  forRows	: uint		   : ... for rows			*/
/*  offset	: uint		   : Grouping offset			*/
/*  (returns)	: void		   :					*/

void	KBCtrlTree::loadDataTree
	(	KBCtrlTreeItem		*parent,
		const KBLTValueSet	&valset,
		uint			fromRow,
		uint			forRows,
		uint			offset
	)
{
	KBTree::TreeType treeType	= m_tree->treeType    () ;
	uint		 exprCount	= m_tree->exprCount   () ;
	uint		 extraCount	= m_tree->extraCount  () ;
	uint		 groupCount	= m_tree->groupCount  () ;
	uint		 groupOffset	= exprCount + extraCount ;
	KBCtrlTreeItem	 *last		= 0 ;

	if (parent == 0)
		if (!m_tree->noblank())
		{
			last	= new KBCtrlTreeItem
				      (		m_listView,
						last,
						0,
						valset,
						0,
						0,
						1,
						0,
						0
				      )	;

			fromRow	+= 1	;
			forRows	-= 1	;
		}

	if (parent != 0)
		if (treeType == KBTree::Shallow)
		{
			fromRow	+= 1	;
			forRows	-= 1	;
		}

	if (offset >= groupCount)
	{
		DPRINTF
		((	"KBCtrlTree::loadDataTree: data: for=%d\n",
			forRows
		))	;

		for (uint idx = 0 ; idx < forRows ; idx += 1, fromRow += 1)
			if ((treeType != KBTree::Shallow) || !valset[fromRow][offset].isEmpty())
				last = new KBCtrlTreeItem
					   (	parent,
						last,
						0,
						valset,
						fromRow,
						fromRow,
						1,
						offset,
						exprCount - offset
				   	   )	;

		return	;
	}

	KBLTValueSet::const_iterator iter = valset.at(fromRow) ;

	while (forRows > 0)
	{
		const QString	&grpval	= (*iter)[groupOffset + offset] ;
		uint		useRows	= 1  ;
		int		index	= -1 ;

		iter++	;

		while (useRows < forRows)
			if ((*iter)[groupOffset + offset] == grpval)
			{
				useRows	+= 1 ;
				iter	++   ;
			}
			else	break	;

		DPRINTF
		((	"KBCtrlTree::loadDataTree: nest: from=%d for=%d grpval=[%s]\n",
			fromRow,
			useRows,
			(cchar *)grpval
		))	;

		if (treeType == KBTree::Shallow)
			index	= fromRow ;

		if ((treeType != KBTree::Shallow) || !valset[fromRow][offset].isEmpty())
		{
			if (parent == 0)
				last	= new KBCtrlTreeItem
					      (		m_listView,
							last,
							this,
							valset,
							index,
							fromRow,
							useRows,
							offset,
							1
					      )	;
			else	last	= new KBCtrlTreeItem
					      (		parent,
							last,
							this,
							valset,
							index,
							fromRow,
							useRows,
							offset,
							1
				      	)	;
		}

		fromRow	+= useRows ;
		forRows	-= useRows ;
	}

}


/*  KBCtrlTree								*/
/*  loadDataFlat: Load data as flattened tree				*/
/*  valset	: 		: List of values			*/
/*  (returns)	: void		:					*/

void	KBCtrlTree::loadDataFlat
	(	const KBLTValueSet	&valset
	)
{
	uint	exprCount	= m_tree->exprCount   () ;
	uint	extraCount	= m_tree->extraCount  () ;
	uint	groupCount	= m_tree->groupCount  () ;
	uint	groupOffset	= exprCount + extraCount ;

	DPRINTF
	((	"KBCtrlTree::loadDataFlat: ec=%d gc=%d\n",
		exprCount,
		groupCount
	))	;
	/* The strategy is to insert an item into the list view for	*/
	/* each value. These are structured such that we get one top-	*/
	/* level item for each distinct first group value (strictly,	*/
	/* one for each change in first group value); under each there	*/
	/* is a second-level item for each distinct second group value	*/
	/* (with the same first group value) and so on.	We use a	*/
	/* stack to keep track of the last listview item at each level	*/
	/* (we don't actually need a stack, just a vector, but the	*/
	/* stack is easy to manipulate).				*/

	/* Prime the system with the first entry, setting all stack	*/
	/* entries to point to this first item. The add all following	*/
	/* entries.							*/
	QPtrStack<KBCtrlTreeItem> stack	;
	KBCtrlTreeItem *null	= new KBCtrlTreeItem (m_listView,    0, 0, valset, -1, 0, 1, 0, exprCount) ;
	KBCtrlTreeItem *item	= new KBCtrlTreeItem (m_listView, null, 0, valset,  1, 1, 1, 0, exprCount) ;

	while (stack.count() < groupCount)
		stack.push (item) ;


	for (uint idx = 2 ; idx < valset.count() ; idx += 1)
	{
		const QStringList &sl = valset[idx] ;

		/* Work back down the stack to pop'ing from it while	*/
		/* the group value differs from the group value for the	*/
		/* entry to be inserted....				*/
		for (int gc2 = groupCount - 1 ; gc2 >= 0 ; gc2 -= 1)
			if (stack.top()->entry()[groupOffset + gc2] != sl[groupOffset + gc2])
				stack.pop() ;
			else	break	;

		/* ....so that the stack is left with items that match	*/
		/* the group terms of the current entry (think of this	*/
		/* as prefix matching). We can then append this entry,	*/
		/* with the special case that if the stack is empty	*/
		/* then we append at the top level.			*/
		if (stack.count() > 0)
		{
			QListViewItem *child = stack.top()->firstChild() ;
			while ((child != 0) && (child->nextSibling() != 0))
				child = child->nextSibling() ;

			item	= new KBCtrlTreeItem (stack.top(), child, 0, valset, idx, idx, 1, 0, exprCount) ;
		}
		else
		{
			QListViewItem *child = m_listView->firstChild() ;
			while ((child != 0) && (child->nextSibling() != 0))
				child = child->nextSibling() ;

			item	= new KBCtrlTreeItem (m_listView,  child, 0, valset, idx, idx, 1, 0, exprCount) ;
		}

		/* Top up the stack with the new listview item.		*/
		while (stack.count() < groupCount)
			stack.push (item) ;
	}
}

/*  KBCtrlTree								*/
/*  loadDataValues							*/
/*		: Load values into control				*/
/*  valset	: 		: List of values			*/
/*  (returns)	: void		:					*/

void	KBCtrlTree::loadDataValues
	(	const KBLTValueSet	&valset
	)
{
	uint exprCount = m_tree->exprCount () ;
	if (exprCount == 0) return ;

	m_listView->clear () ;

	while (m_listView->columns() > (int)exprCount) m_listView->removeColumn (0) ;
	while (m_listView->columns() < (int)exprCount) m_listView->addColumn    (QString::null) ;

	if (valset.count() > 1)
		switch (m_tree->treeType())
		{
			case KBTree::Expanded :
			case KBTree::Shallow  :
				loadDataTree (0, valset, 0, valset.count(), 0) ;
				break	;

			default	:
				loadDataFlat (valset) ;
				break	;
		}
}

/*  KBCtrlTree								*/
/*  loadControls: Load contol values					*/
/*  keyset	: const QStringList  & : Keys				*/
/*  valset	: const KBLTValueSet & : Values				*/
/*  (returns)	: void		       :				*/

void	KBCtrlTree::loadControl
	(	const QStringList	&keyset,
		const KBLTValueSet	&valset
	)
{
	KBInSetValueLock lock (*this) ;

	DELOBJ		(m_keyset) ;
	loadDataValues	(valset	 ) ;
}

/*  KBCtrlTree								*/
/*  userChange	: User changes value					*/
/*  (returns)	: void		  :					*/

bool	KBCtrlTree::userChange ()
{
	/* Ignore if either no item is selected or the item has a	*/
	/* negative index (in which case it is a branch node).		*/
	KBCtrlTreeItem *item = (KBCtrlTreeItem *)m_listView->currentItem() ;

	DPRINTF
	((	"KBCtrlTree::userChange: item=%p index=%d\n",
		(void *)item,
		item == 0 ? 0 : item->index()
	))	;

	if ((item == 0) || (item->index() < 0))
		return	false	;

	if (KBControl::userChange ())
	{
		if (m_tree->clickOpen ())
		{
			QListViewItem  *item  = m_listView->currentItem() ;
			if (item != 0) item->setOpen (true) ;
		}
		return	true ;
	}

	return	false	;
}


/*  KBCtrlTree								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlTree::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;

	{
		KBInSetValueLock lock (*this) ;
		setupProperties () ;
	}
}

/*  KBCtrlTree								*/
/*  setCurrent	: Set current item					*/
/*  idx		: uint		   : Index required			*/
/*  child	: KBCtrlTreeItem * : Scan position in tree		*/
/*  (returns)	: bool		   : Item found				*/

bool	KBCtrlTree::setCurrent
	(	int		idx,
		KBCtrlTreeItem	*child
	)
{
	/* This is a depth-first recursive descend through the tree	*/
	/* looking for the item with the correct index. Once found we	*/
	/* unwind by returning true.					*/
	while (child != 0)
	{
		DPRINTF
		((	"KBCtrlTree::setCurrent: %d: depth=%d: has %d\n",
			idx,
			child->depth(),
			child->index()
		))	;

		if (child->index() == idx)
		{
//			m_listView->setCurrentItem    (child) ;
			m_listView->setSelected       (child, true) ;
			m_listView->ensureItemVisible (child) ;
			return	true ;
		}

		/* If the child has no further children and it is not a	*/
		/* leaf then see if it contains the required index, and	*/
		/* if so expand that branch.				*/
		DPRINTF
		((	"KBCtrlTree::setCurrent: %d: depth=%d: has %d fp=%p\n",
			idx,
			child->depth(),
			child->index(),
			(void *)child->firstChild()
		))	;

		if (child->firstChild() == 0)
			if (child->containsIndex(idx))
				child->setOpen (true) ;


		if (setCurrent (idx, (KBCtrlTreeItem *)child->firstChild()))
		{
			return	true ;
		}

		child  = (KBCtrlTreeItem *)child->nextSibling() ;
	}

	return	false	;
}

/*  KBCtrlTree								*/
/*  closeAllTree: Close all tree branches				*/
/*  item	: QListViewItem * : Branch to close			*/
/*  (returns)	: void		  :					*/

void	KBCtrlTree::closeAllTree
	(	QListViewItem	*item
	)
{
	while (item != 0)
	{
		item->setOpen (false) ;
		closeAllTree  (item->firstChild()) ;
		item = item->nextSibling() ;
	}
}

/*  KBCtrlTree								*/
/*  setValue	: Set value						*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlTree::setValue
	(	const KBValue	&value
	)
{
	if (m_listView != 0)
	{
		KBInSetValueLock lock (*this) ;

		if (m_tree->setClose ())
			closeAllTree (m_listView->firstChild()) ;

		/* Set the current item, with a fallback on the first	*/
		/* (null) item if we cannot find the value.		*/
		if (!setCurrent
			(	m_tree->valueToItem (value, m_keyset),
				(KBCtrlTreeItem *)m_listView->firstChild()
			))
		{
			m_listView->clearSelection   () ;
			m_listView->setCurrentItem   (m_listView->firstChild()) ;
			m_listView->ensureItemVisible(m_listView->firstChild()) ;
		}
	}

	KBControl::setValue (value) ;
}

/*  KBCtrlTree								*/
/*  getValue	: Get value						*/
/*  (returns)	: KBValue 	: Value					*/

KBValue	KBCtrlTree::getValue ()
{
	KBCtrlTreeItem *item = (KBCtrlTreeItem *)m_listView->currentItem() ;

	DPRINTF
	((	"KBCtrlTree::getValue: item=[%p] idx=[%d] value=[%s]\n",
		(void *)item,
		item == 0 ? 0  : item->index(),
		item == 0 ? "" : item->index() < 0 ? "" : (cchar *)m_tree->itemToValue(item->index(), m_keyset).getRawText()
	))	;
	return	item 	      == 0 ? KBValue() :
		item->index() <  0 ? KBValue() :
		m_tree->itemToValue (item->index(), m_keyset) ;
}

/*  KBCtrlTree								*/
/*  clearValue	: Get value						*/
/*  query	: bool		: Clear for query			*/
/*  (returns)	: KBValue 	: Value					*/

void	KBCtrlTree::clearValue
	(	bool	query
	)
{
	if (m_listView != 0)
	{
		KBInSetValueLock lock   (*this) ;
		m_listView->clearSelection   () ;
		m_listView->setCurrentItem   (m_listView->firstChild()) ;
		m_listView->ensureItemVisible(m_listView->firstChild()) ;
		if (m_showing == KB::ShowAsData)
			m_layoutItem->setValid (isValid (false)) ;
	}

	KBControl::clearValue (query) ;
}

/*  KBCtrlTree								*/
/*  isEmpty	: See if control is empty				*/
/*  (returns)	: bool		: Empty					*/

bool	KBCtrlTree::isEmpty ()
{
	KBCtrlTreeItem *item = (KBCtrlTreeItem *)m_listView->currentItem() ;

	DPRINTF
	((	"KBCtrlTree::isEmpty: item=[%p] idx=[%d]\n",
		(void *)item,
		item == 0 ? 0 : item->index()
	))	;
	return	(item == 0) || (item->index() <= 0) ;
}

/*  KBCtrlTree								*/
/*  passFocus	: Actually pass focus into the control			*/
/*  (returns)	: void		:					*/

void	KBCtrlTree::passFocus ()
{
	DPRINTF
	((	"KBCtrlTree::passFocus: [%p] [%p]\n",
		(void *)m_deferFocus,
		(void *)m_deferMouse
	))	;
	if (m_deferFocus != 0)
	{
		QFocusEvent::setReason	    (QFocusEvent::Mouse) ;
		m_listView->focusInEvent    (m_deferFocus) ;
		DELOBJ	(m_deferFocus) ;
	}

	if (m_deferMouse != 0)
	{
		m_listView->mousePressEvent (m_deferMouse) ;
		DELOBJ	(m_deferMouse) ;
	}
}

/*  KBCtrlTree								*/
/*  write	: Write link 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlTree::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		new KBWriterText
		(	writer,
			rect,
			m_tree->getPalette(true),
			m_tree->getFont   (true),
			value.getRawText  (),
			Qt::AlignLeft|Qt::AlignVCenter,
			false
		)	;

		extra	= 0	;
		return	true	;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

/*  KBCtrlTree								*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: void		:					*/

void	KBCtrlTree::setupProperties ()
{
	m_layoutItem->setValidatorMode (m_tree) ;
	m_listView  ->clear () ;

#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		if (m_drow == 0) new QListViewItem (m_listView, m_tree->getName()) ;
		return	;
	}
#endif
}

/*  KBCtrlTree								*/
/*  setUserFilter							*/
/*		: Set user filter for link query			*/
/*  filter	: const QString & : Filter expression			*/
/*  (returns)	: void		  :					*/

void	KBCtrlTree::setUserFilter
	(	const QString	&filter
	)
{
	m_userFilter	= filter ;
}

/*  KBCtrlTree								*/
/*  setUserSorting							*/
/*		: Set user sorting for link query			*/
/*  sorting	: const QString & : Sorting expression			*/
/*  (returns)	: void		  :					*/

void	KBCtrlTree::setUserSorting
	(	const QString	&sorting
	)
{
	m_userSorting	= sorting ;
}

/*  KBCtrlTree								*/
/*  reload	: Reload values list					*/
/*  (returns)	: void		  :					*/

void	KBCtrlTree::reload ()
{
	/* Note the current value. We will restore this after the	*/
	/* control has been updated.					*/
	KBValue	curVal	= m_curVal ;

	DELOBJ	(m_keyset) ;
	DELOBJ	(m_valset) ;

	/* If either a user filter or sorting expression in set the we	*/
	/* do an explicit load for this control only, providing our	*/
	/* own keyset (which is retained).				*/
	if (!m_userFilter.isEmpty() || !m_userSorting.isEmpty())
	{
		m_keyset = new QStringList  ;
		m_valset = new KBLTValueSet ;

		m_tree->loadValues
		(	m_userFilter,
			m_userSorting,
			*m_keyset,
			*m_valset
		)	;

		for (uint row = 0 ; row < m_valset->count() ; row += 1)
		{	fprintf	(stderr, "%s:\n", (*m_keyset)[row].latin1()) ;
			const QStringList &v = (*m_valset)[row] ;
			for (uint col = 0 ; col < v.count() ; col += 1)
				fprintf	(stderr, "    %4d: %s\n", col, v[col].latin1()) ;
		}

		loadDataValues (*m_valset) ;		
	}
	else	m_tree->doRefresh (m_drow) ;

	setValue (curVal)    ;
}

/*  KBCtrlTree								*/
/*  getExtra	: Return "extra" value for current row			*/
/*  which	: bool		: Display or extra			*/
/*  index	: uint		: Extra value index			*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCtrlTree::getExtra
	(	bool		which,
		uint		index
	)
{
	KBCtrlTreeItem	*item	= (KBCtrlTreeItem *)m_listView->currentItem() ;

	if ((item == 0) || (item->index() < 0))
		return	KBValue	() ;

	return	m_tree->itemToExtra
		(	item->index(),
			which,
			index,
			m_valset
		)	;
}

/*  KBCtrlTree								*/
/*  eventFilter	: Event filter on edit control				*/
/*  o		: QObject *	: Object being filtered			*/
/*  e		: QEvent *	: Event					*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBCtrlTree::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (m_showing != KB::ShowAsData)
		return	KBControl::eventFilter (o, e) ;

	if (o != m_listView->viewport())
		return	KBControl::eventFilter (o, e) ;


	/* If in data mode and the link is dynamic then on focus entry	*/
	/* we have to to refresh the control. This is nasty since	*/
	/* changing the combo values in the middle of a mouse-focus-in	*/
	/* seems (perhaps not suprisingly) to do nasty things!		*/
	if ((e->type() == QEvent::FocusIn) && m_tree->dynamic())
	{
		QFocusEvent *fe	   = (QFocusEvent *)e ;
		KBValue	    curVal ;

		switch (fe->reason())
		{
			case QFocusEvent::Tab	   :
			case QFocusEvent::Shortcut :
			case QFocusEvent::Other    :
				/* These cases are easy, the one below	*/
				/* is not.				*/
				curVal	= getValue() ;
				m_tree->doRefresh (m_drow) ;
				setValue (curVal)    ;
				break ;

			case QFocusEvent::Mouse    :
				/* If focus arrives on account of the	*/
				/* mouse then we have to defer then	*/
				/* focus in and mouse button events or	*/
				/* we get an escape-only lockup.	*/
				curVal	= getValue() ;
				m_tree->doRefresh (m_drow) ;
				setValue (curVal)    ;

				m_deferFocus = new QFocusEvent(QEvent::FocusIn) ;
				QTimer::singleShot (250, this, SLOT(passFocus())) ;
				return	true	;

			default	:
				break	;
		}
	}

	/* If this is a mouse press and focus is being deferred then	*/
	/* we have to defer the mouse press as well.			*/
	if ((e->type() == QEvent::MouseButtonPress) && (m_deferFocus != 0))
	{
		QMouseEvent *me = (QMouseEvent *)e ;
		m_deferMouse = new QMouseEvent (me->type(), me->pos(), me->globalPos(), me->button(), me->state()) ;
		return	true ;
	}


	return	KBControl::eventFilter (o, e) ;
}

bool	KBCtrlTree::getNumValues
	(	uint		&nvals
	)
{
	if (m_keyset != 0)
	{
		nvals	= m_keyset->count() ;
		return	true ;
	}

	return	false	;
}

/*  KBCtrlTree							*/
/*  getDisplayList						*/
/*		: Get list of displayed text values at row	*/
/*  list	: QStringList &	: List of values on return	*/
/*  (returns)	: bool		: True if local values		*/

bool	KBCtrlTree::getDisplayList
	(	QStringList	&list
	)
{
	if (m_keyset != 0)
	{
		for (uint idx = 0 ; idx < m_keyset->count() ; idx += 1)
			list.append ((*m_keyset)[idx]) ;
		return	true ;
	}

	return	false	;
}
