/***************************************************************************
    file	         : kb_qttree.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#ifndef	_KB_QTTREE_H
#define	_KB_QTTREE_H


#include	<qstringlist.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_ctrl.h"

#include	"libkbase_exports.h"

#include	"rk_listview.h"


class	KBCtrlTreeItem		;
class	KBTree			;
class	KBCtrlTreeListView	;

/*  KBCtrlTree								*/
/*  ----------								*/
/*  This class provides a tree-link control where displayed values can	*/
/*  be grouped up into a tree for compact and organised display.	*/

class	LIBKBASE_API	KBCtrlTree : public KBControl
{
	Q_OBJECT

	KBTree			*m_tree		;
	KBCtrlTreeListView	*m_listView	;

	QString			m_userFilter	;
	QString			m_userSorting	;
	QStringList		*m_keyset	;
	KBLTValueSet		*m_valset	;

	QFocusEvent		*m_deferFocus	;
	QMouseEvent		*m_deferMouse	;

	bool			m_inSetValue	;

	void			loadDataTree
				(	KBCtrlTreeItem		*parent,
					const KBLTValueSet	&valset,
					uint			fromRow,
					uint			forRows,
					uint			offset
				)	;
	void			loadDataFlat
				(	const KBLTValueSet	&valset
				)	;
	void			loadDataValues
				(	const KBLTValueSet	&valset
				)	;

	void			closeAllTree		(QListViewItem       *) ;
	bool			setCurrent		(int, KBCtrlTreeItem *) ;

private slots :

	virtual bool		userChange		() ;
	void			passFocus		() ;

protected :

	virtual bool		eventFilter		(QObject *, QEvent *) ;

public	:

	virtual	void		setupProperties	() ;
	virtual	void		loadControl	(const QStringList &, const KBLTValueSet &) ;
	virtual	void		showAs		(KB::ShowAs) ;
	virtual	void		setValue   	(const KBValue &) ;
	virtual	KBValue		getValue   	() ;
	virtual	void		clearValue	(bool) ;
	virtual	bool		isEmpty		() ;
	virtual	bool		write		(KBWriter *, QRect, const KBValue &, bool, int &) ;

	void			setUserFilter	(const QString &) ;
	void			setUserSorting	(const QString &) ;
	void			reload		() ;
	KBValue			getExtra	(bool, uint) ;
	bool			getDisplayList	(QStringList   &) ;
	bool			getNumValues	(uint &) ;

	friend	class	KBCtrlTreeItem	;

	KBCtrlTree 				(KBDisplay *, KBTree *, uint) ;
	virtual	~KBCtrlTree 			() ;
}	;

#endif	// _KB_QTTREE_H
