/***************************************************************************
    file	         : kb_querychooser.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"rk_combobox.h"
#include	"rk_vbox.h"
#include	"rk_gridbox.h"
#include	"rk_pushbutton.h"

#if		__KB_AUTOMOC
#include 	"kb_querychooser.moc"
#else
#include 	"kb_querychooser.h"
#endif

#include	"kb_serverinfo.h"
#include	"kb_dbdociter.h"


/*  KBQueryChooser							*/
/*  KBQueryChooser							*/
/*		: Constructor for query chooser				*/
/*  location	: KBLocation &	 : Database location			*/
/*  cbServer	: RKComboBox *	 : Server selector			*/
/*  cbQuery	: RKComboBox *	 : Query selector			*/
/*  (returns)	: KBQueryChooser :					*/

KBQueryChooser::KBQueryChooser
	(	KBLocation	&location,
		RKComboBox	*cbServer,
		RKComboBox	*cbQuery
	)
	:
	m_location	(location),
	m_cbServer	(cbServer),
	m_cbQuery 	(cbQuery)
{
	if (!m_location.dbInfo()->findServer(KBLocation::m_pFile)->dbType().isEmpty())
		cbServer->insertItem (KBLocation::m_pFile) ;

	QPtrListIterator<KBServerInfo> *svIter = location.dbInfo()->getServerIter () ;
	KBServerInfo		    *svInfo ;

	while ((svInfo = svIter->current()) != 0)
	{
		cbServer->insertItem (svInfo->serverName()) ;
		(*svIter) += 1 ;
	}

	delete	svIter	;

	connect
	(	m_cbServer,
		SIGNAL(activated     (const QString &)),
		SLOT  (serverSelected(const QString &))
	)	;
	connect
	(	m_cbQuery,
		SIGNAL(activated     (const QString &)),
		SLOT  (querySelected (const QString &))
	)	;

	serverSelected (m_cbServer->text(0)) ;
}

/*  KBQueryChooser							*/
/*  serverSelected							*/
/*		: User selects a database server			*/
/*  svName	: const QString & : Server name				*/
/*  (returns)	: void		  :					*/

void	KBQueryChooser::serverSelected
	(	const QString	&svName
	)
{
	m_cbQuery->clear     ()	;
	emit	serverChanged()	;

	if (svName == "")
		return	;

	KBError		error	;
	KBDBDocIter	docIter	;
	if (!docIter.init
		(	m_location.dbInfo(),
			svName,
			"query",
			"qry",
			error,
			false
	   ))
	{
		error.DISPLAY() ;
		return	;
	}

	QString	name	;
	QString	stamp	;

	m_cbQuery->insertItem ("") ;

	while (docIter.getNextDoc (name, stamp))
	{
		m_cbQuery->insertItem (name) ;
	}

	emit	serverChanged	() ;
}

/*  KBQueryChooser							*/
/*  querySelected							*/
/*		: User selects a table					*/
/*  qryName	: const QString & : Query name				*/
/*  (returns)	: void		  :					*/

void	KBQueryChooser::querySelected
	(	const QString	&
	)
{
	emit	queryChanged	() ;
}

/*  KBQueryChooser							*/
/*  setServer	: Set specifier server					*/
/*  svrName	: const QString & : Server name				*/
/*  (returns)	: bool		  : Server name valid			*/

bool	KBQueryChooser::setServer
	(	const QString	&svrName
	)
{
	bool	rc	= false ;

	m_cbServer->setCurrentItem (0) ;

	for (int idx = 0 ; idx < m_cbServer->count() ; idx += 1)
		if (m_cbServer->text(idx) == svrName)
		{	m_cbServer->setCurrentItem (idx) ;
			serverSelected	(svrName) ;
			rc	= true	;
			break	;
		}

	emit serverChanged () ;
	return	rc ;
}

/*  KBQueryChooser							*/
/*  setQuery	: Set specifier query					*/
/*  qryName	: const QString & : Query name				*/
/*  (returns)	: bool		  : Query name valid			*/

bool	KBQueryChooser::setQuery
	(	const QString	&qryName
	)
{
	bool	rc	= false ;

	m_cbQuery->setCurrentItem (0) ;

	for (int idx = 0 ; idx < m_cbQuery->count() ; idx += 1)
		if (m_cbQuery->text(idx) == qryName)
		{	m_cbQuery->setCurrentItem (idx) ;
			querySelected	(qryName) ;
			rc	= true	;
			break	;
		}

	emit queryChanged () ;
	return	rc ;
}


/*  ------------------------------------------------------------------ */

/*  KBQueryChooserDlg							*/
/*  KBQueryChooserDlg							*/
/*		: Simple server/query selection dialog			*/
/*  location	: KBLocation &	    : Database location			*/
/*  (returns)	: KBQueryChooserDlg :					*/

KBQueryChooserDlg::KBQueryChooserDlg
	(	KBLocation	&location
	)
	:
	KBDialog	("Pick server and query", true),
	m_location 	(location)
{
	RKVBox    *box  = new RKVBox (this) ;
	box->setTracking () ;

	RKGridBox *grid = new RKGridBox  (2, box) ;

	new QLabel (TR("Server"), grid) ;
	m_cbServer = new RKComboBox (grid) ;

	new QLabel (TR("Query" ), grid) ;
	m_cbQuery  = new RKComboBox (grid) ;

	addOKCancel (box, &m_bOK) ;
	m_bOK->setEnabled (false) ;

	m_chooser = new KBQueryChooser (m_location, m_cbServer, m_cbQuery) ;
	connect  (m_chooser, SIGNAL(changed()), SLOT(changed())) ;
}

KBQueryChooserDlg::~KBQueryChooserDlg ()
{
	delete	m_chooser ;
}

/*  KBQueryChooserDlg							*/
/*  changed	: Selection changed					*/
/*  (returns)	: void		:					*/

void	KBQueryChooserDlg::changed ()
{
	m_bOK->setEnabled
	(	!m_cbServer->currentText().isEmpty() &&
		!m_cbQuery ->currentText().isEmpty()
	)	;
}

QString	KBQueryChooserDlg::getServer ()
{
	return	m_cbServer->currentText() ;
}

QString	KBQueryChooserDlg::getQuery ()
{
	return	m_cbQuery ->currentText() ;
}
