/***************************************************************************
    file	         : kb_recorder.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_item.h"
#include	"kb_tabber.h"
#include	"kb_recorder.h"
#include	"kb_macro.h"

#include	"tk_messagebox.h"

#define		__DEBUG_RECORDER	(1)
#if		__DEBUG_RECORDER
#include	"kb_dprintf.h"
#endif


#include	<qevent.h>


KBRecorder::KBRecorder ()
{
	m_docRoot	= 0 ;
	m_macro		= 0 ;
}

bool	KBRecorder::start
	(	KBDocRoot	*docRoot,
		KBDBInfo	*dbInfo,
		const QString	&server
	)
{
	DPRINTF
	((	"KBRecorder::start: macro=%p\n",
		(void *)m_macro
	))	;

	if (m_macro != 0)
	{
		if (TKMessageBox::questionYesNo
			(	0,
				TR("Test recording in progress: cancel?"),
				TR("Start test recording")
			)
			!= TKMessageBox::Yes
		   )
			return	false	;

		delete	m_macro	;
	}

	m_docRoot = docRoot ;
	m_macro	  = new KBMacroExec (dbInfo, server, "tests") ;
	m_macro->setName ("recording") ;
	return	true ;
}

bool	KBRecorder::isRecording
	(	KBDocRoot	*docRoot
	)
{
	return	m_docRoot == docRoot ;
}

KBMacroExec
	*KBRecorder::recording ()
{
	KBMacroExec *macro = m_macro ;
	m_docRoot = 0	;
	m_macro   = 0	;
	return	macro	;
}


void	KBRecorder::verifyValue
	(	KBItem		*item,
		uint		drow,
		const KBValue	&value
	)
{
	DPRINTF
	((	"KBRecorder::verifyValue: p=[%s] n=[%s] dr=%d v=[%s]\n",
		item->getPath(false).latin1(),
		item->getName()     .latin1(),
		drow,
		value.getRawText  ().latin1()
	))	;

	if (m_macro != 0)
	{
		QStringList	args	;
		KBError		error	;

		args.append	(item->getPath(false))	;
		args.append	(item->getName())	;
		args.append	(QString::number(drow)) ;
		args.append	(QString("%1:%2").arg(value.getType()->getIType()).arg(value.getRawText())) ;

		if (!m_macro->append ("VerifyValue", args, QString::null, error))
			error.DISPLAY() ;
	}
}

void	KBRecorder::verifyRegexp
	(	KBItem		*item,
		uint		drow,
		const QString	&regexp
	)
{
	DPRINTF
	((	"KBRecorder::verifyRegexp: p=[%s] n=[%s] dr=%d r=[%s]\n",
		item->getPath(false).latin1(),
		item->getName()     .latin1(),
		drow,
		regexp.latin1()
	))	;

	if (m_macro != 0)
	{
		QStringList	args	;
		KBError		error	;

		args.append	(item->getPath(false))	;
		args.append	(item->getName())	;
		args.append	(QString::number(drow)) ;
		args.append	(regexp)		;

		if (!m_macro->append ("VerifyRegexp", args, QString::null, error))
			error.DISPLAY() ;
	}
}

void	KBRecorder::verifyText
	(	KBObject	*object,
		uint		drow,
		const QString	&text
	)
{
	DPRINTF
	((	"KBRecorder::verifyText: p=[%s] n=[%s] dr=%d t=[%s]\n",
		object->getPath(false).latin1(),
		object->getName()     .latin1(),
		drow,
		text.latin1()
	))	;

	if (m_macro != 0)
	{
		QStringList	args	;
		KBError		error	;

		args.append	(object->getPath(false));
		args.append	(object->getName())	;
		args.append	(QString::number(drow)) ;
		args.append	(text)	;

		if (!m_macro->append ("VerifyText", args, QString::null, error))
			error.DISPLAY() ;
	}
}

void	KBRecorder::verifyChoices
	(	KBItem		*item,
		uint		drow,
		const QString	&choices
	)
{
	DPRINTF
	((	"KBRecorder::verifyChoices: p=[%s] n=[%s] dr=%d v=[%s]\n",
		item->getPath(false).latin1(),
		item->getName()     .latin1(),
		drow,
		choices.latin1()
	))	;

	if (m_macro != 0)
	{
		QStringList	args	;
		KBError		error	;

		args.append	(item->getPath(false))	;
		args.append	(item->getName())	;
		args.append	(QString::number(drow)) ;
		args.append	(choices)	;

		if (!m_macro->append ("VerifyChoices", args, QString::null, error))
			error.DISPLAY() ;
	}
}

void	KBRecorder::verifyState
	(	KBObject	*object,
		uint		drow,
		bool		enabled,
		bool		visible
	)
{
	DPRINTF
	((	"KBRecorder::verifyState: p=[%s] n=[%s] dr=%d e=%d v=%d\n",
		object->getPath(false).latin1(),
		object->getName()     .latin1(),
		drow,
		enabled,
		visible
	))	;

	if (m_macro != 0)
	{
		QStringList	args	;
		KBError		error	;

		args.append	(object->getPath(false));
		args.append	(object->getName())	;
		args.append	(QString::number(drow)) ;
		args.append	(enabled ? "1" : "0")	;
		args.append	(visible ? "1" : "0")	;

		if (!m_macro->append ("VerifyState", args, QString::null, error))
			error.DISPLAY() ;
	}
}

void	KBRecorder::verifyStack
	(	KBObject	*object,
		const QString	&page
	)
{
	DPRINTF
	((	"KBRecorder::verifyStack: p=[%s] n=[%s] dr=%d p=[%s]\n",
		object->getPath(false).latin1(),
		object->getName()     .latin1(),
		page.latin1()
	))	;

	if (m_macro != 0)
	{
		QStringList	args	;
		KBError		error	;

		args.append	(object->getPath(false));
		args.append	(object->getName())	;
		args.append	(page	) ;

		if (!m_macro->append ("VerifyStack", args, QString::null, error))
			error.DISPLAY() ;
	}
}

void	KBRecorder::updateValue
	(	KBItem		*item,
		uint		drow,
		const KBValue	&value
	)
{
	DPRINTF
	((	"KBRecorder::updateValue: p=[%s] n=[%s] dr=%d v=[%s]\n",
		item->getPath(false).latin1(),
		item->getName()     .latin1(),
		drow,
		value.getRawText()  .latin1()
	))	;

	if (m_macro != 0)
	{
		QStringList	args	;
		KBError		error	;

		args.append	(item->getPath(false))	;
		args.append	(item->getName())	;
		args.append	(QString::number(drow)) ;
		args.append	(QString("%1:%2").arg(value.getType()->getIType()).arg(value.getRawText())) ;

		if (!m_macro->append ("UpdateValue", args, QString::null, error))
			error.DISPLAY() ;
	}
}

void	KBRecorder::keyNavigation
	(	KBItem		*item,
		uint		drow,
		QKeyEvent	*key
	)
{
	DPRINTF
	((	"KBRecorder::keyNavigation: p=[%s] n=[%s] dr=%d k=%d s=%04x\n",
		item->getPath(false).latin1(),
		item->getName()     .latin1(),
		drow,
		key->key  (),
		key->state()
	))	;

	if (m_macro != 0)
	{
		QStringList	args	;
		KBError		error	;

		args.append	(item->getPath(false))	;
		args.append	(item->getName())	;
		args.append	(QString::number(drow)) ;
		args.append
		(	QString("%1:%2:%3")
				.arg(key->key  ())
				.arg(key->ascii())
				.arg(key->state())
		)	;

		if (!m_macro->append ("KeyNavigation", args, QString::null, error))
			error.DISPLAY() ;
	}
}

void	KBRecorder::mouseNavigation
	(	KBItem		*item,
		uint		drow
	)
{
	DPRINTF
	((	"KBRecorder::mouseNavigation: p=[%s] n=[%s] dr=%d\n",
		item->getPath(false).latin1(),
		item->getName()     .latin1(),
		drow
	))	;

	if (m_macro != 0)
	{
		QStringList	args	;
		KBError		error	;

		args.append	(item->getPath(false))	;
		args.append	(item->getName())	;
		args.append	(QString::number(drow)) ;

		if (!m_macro->append ("MouseNavigation", args, QString::null, error))
			error.DISPLAY() ;
	}
}


void	KBRecorder::mouseClick
	(	KBObject	*object,
		uint		drow,
		const QString	&link
	)
{
	DPRINTF
	((	"KBRecorder::mouseClick: p=[%s] n=[%s] dr=%d l=%s\n",
		object->getPath(false).latin1(),
		object->getName()     .latin1(),
		drow,
		link.latin1()
	))	;

	if (m_macro != 0)
	{
		QStringList	args	;
		KBError		error	;

		args.append	(object->getPath(false));
		args.append	(object->getName())	;
		args.append	(QString::number(drow)) ;
		args.append	(link)	;

		if (!m_macro->append ("MouseClick", args, QString::null, error))
			error.DISPLAY() ;
	}
}

void	KBRecorder::popupResult
	(	KBRecorder::PopupType	type,
		bool			rc,
		const QString		&value
	)
{
	DPRINTF
	((	"KBRecorder::PopupType: t=[%d] rc=[%d] v=[%s]\n",
		(int)type,
		rc,
		value.latin1()
	))	;

	if (m_macro != 0)
	{
		QStringList	args	;
		KBError		error	;

		args.append	(QString::number((int)type)) ;
		args.append	(QString::number(rc)) ;
		args.append	(value)	;

		if (!m_macro->append ("PopupResult", args, QString::null, error))
			error.DISPLAY() ;
	}
}

void	KBRecorder::raisePage
	(	KBTabber	*tabber,
		const QString	&page
	)
{
	DPRINTF
	((	"KBRecorder::raisePage::PopupType: p=[%s]\n",
		page.latin1()
	))	;

	if (m_macro != 0)
	{
		QStringList	args	;
		KBError		error	;

		args.append	(tabber->getPath(false));
		args.append	(tabber->getName())	;
		args.append	(page)	;

		if (!m_macro->append ("RaisePage", args, QString::null, error))
			error.DISPLAY() ;
	}
}


KBRecorder
	*KBRecorder::self ()
{
	static	KBRecorder *recorder ;
	if (recorder == 0) recorder = new KBRecorder () ;
	return	recorder   ;
}
