/***************************************************************************
    file	         : kb_recordnav.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qlabel.h>
#include	<qvalidator.h>

#if		__KB_AUTOMOC
#include	"kb_recordnav.moc"
#else
#include	"kb_recordnav.h"
#endif

#include	"tk_icons.h"
#include	"rk_pushbutton.h"
#include	"rk_lineedit.h"


static	uint	recordNavHeight	= 16 ;


/*  KBRecordNav								*/
/*  KBRecordNav	: Record navigation tool constructor			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  d		: uint		: Dimension				*/
/*  (returns)	: KBRecordNav	:					*/

KBRecordNav::KBRecordNav
	(	QWidget		*parent,
		uint		d
	)
	:
	RKHBox		(parent)
{
	recordNavHeight	= d ;

	m_bFirst	= new RKPushButton (this) ;
	m_bPrevious	= new RKPushButton (this) ;
	m_lRecord	= new QLabel	   (this) ;
	m_eRecordNo	= new RKLineEdit   (this) ;
	m_lRecordOf	= new QLabel	   (this) ;
	m_bNext		= new RKPushButton (this) ;
	m_bLast		= new RKPushButton (this) ;
	m_bAdd		= new RKPushButton (this) ;

	m_bFirst	->setIconSet	(QIconSet (getSmallIcon ("mini_first"))) ;
	m_bPrevious	->setIconSet	(QIconSet (getSmallIcon ("mini_prev" ))) ;
	m_bNext		->setIconSet	(QIconSet (getSmallIcon ("mini_next" ))) ;
	m_bLast		->setIconSet	(QIconSet (getSmallIcon ("mini_last" ))) ;
	m_bAdd		->setIconSet	(QIconSet (getSmallIcon ("mini_add"  ))) ;

	m_bFirst	->setFixedSize	(( 3 * d) / 2, d) ;
	m_bPrevious	->setFixedSize	(( 3 * d) / 2, d) ;
	m_bNext		->setFixedSize	(( 3 * d) / 2, d) ;
	m_bLast		->setFixedSize	(( 3 * d) / 2, d) ;
	m_bAdd		->setFixedSize	(( 3 * d) / 2, d) ;


	m_bPrevious	->setAutoRepeat (true) ;
	m_bNext		->setAutoRepeat (true) ;
	m_lRecord	->setText	(TR("Record")) ;

	m_eRecordNo	->setFixedSize	(d * 3, d) ;
	m_lRecordOf	->setFixedSize	(d * 4, d) ;

	m_eRecordNo	->setFrameStyle	(QFrame::Box|QFrame::Plain) ;
	m_eRecordNo	->setLineWidth	(1) ;
	m_lRecordOf	->setFrameStyle	(QFrame::Box|QFrame::Plain) ;
	m_lRecordOf	->setLineWidth	(1) ;

	connect	(m_bFirst,	SIGNAL(clicked()), SLOT(slotClickFirst	 ())) ;
	connect	(m_bPrevious,	SIGNAL(clicked()), SLOT(slotClickPrevious())) ;
	connect	(m_bNext,	SIGNAL(clicked()), SLOT(slotClickNext	 ())) ;
	connect	(m_bLast,	SIGNAL(clicked()), SLOT(slotClickLast	 ())) ;
	connect	(m_bAdd,	SIGNAL(clicked()), SLOT(slotClickAdd	 ())) ;

	connect
	(	m_eRecordNo,
		SIGNAL	(returnPressed	  ()),
		SLOT	(slotReturnPressed())
	)	;

	m_eRecordNo->setValidator (new QIntValidator (1, 0x7fffffff, m_eRecordNo)) ;

	setFixedSize (sizeHint()) ;
}

/*  KBRecordNav								*/
/*  slotClickFirst							*/
/*		: User clicks first-record button			*/
/*  (returns)	: void		:					*/

void	KBRecordNav::slotClickFirst ()
{
	emit	operation (KB::First, 0) ;
}

/*  KBRecordNav								*/
/*  slotClickPrevious							*/
/*		: User clicks previous-record button			*/
/*  (returns)	: void		:					*/

void	KBRecordNav::slotClickPrevious ()
{
	emit	operation (KB::Previous, 0) ;
}

/*  KBRecordNav								*/
/*  slotClickNext							*/
/*		: User clicks next-record button			*/
/*  (returns)	: void		:					*/

void	KBRecordNav::slotClickNext ()
{
	emit	operation (KB::Next, 0) ;
}

/*  KBRecordNav								*/
/*  slotClickLast							*/
/*		: User clicks last-record button			*/
/*  (returns)	: void		:					*/

void	KBRecordNav::slotClickLast ()
{
	emit	operation (KB::Last, 0) ;
}

/*  KBRecordNav								*/
/*  slotClickAdd							*/
/*		: User clicks add-record button				*/
/*  (returns)	: void		:					*/

void	KBRecordNav::slotClickAdd ()
{
	emit	operation (KB::Add, 0) ;
}

/*  KBRecordNav								*/
/*  slotReturnPressed							*/
/*		: User hits return on record number			*/
/*  (returns)	: void		:					*/

void	KBRecordNav::slotReturnPressed ()
{
	int	qrow	= m_eRecordNo->text().toInt() ;
	if (qrow > 0) qrow -= 1 ;
	emit	operation (KB::GotoQRow, qrow) ;
}

/*  KBRecordNav								*/
/*  setRecord	: Set current record					*/
/*  curQRow	: uint		: Current query row			*/
/*  totalRows	: uint		: Total number of rows			*/
/*  (returns)	: void		:					*/

void	KBRecordNav::setRecord
	(	uint		curQRow,
		uint		totalRows
	)
{
	m_lRecordOf->setText (QString(TR("of %1")).arg(totalRows)) ;

	if (curQRow < totalRows)
		m_eRecordNo->setText (QString::number(curQRow + 1)) ;
	else	m_eRecordNo->clear   () ;

	m_bFirst	->setEnabled (curQRow > 0) ;
	m_bPrevious	->setEnabled (curQRow > 0) ;
	m_bNext		->setEnabled (curQRow <  totalRows) ;
	m_bLast		->setEnabled (curQRow != totalRows - 1) ;
}


uint	KBRecordNav::getHeight ()
{
	return	recordNavHeight	;
}
