/***************************************************************************
    file	         : kb_report.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_report.moc"
#else
#include	"kb_report.h"
#endif

#include	"kb_script.h"
#include	"kb_import.h"
#include	"kb_param.h"
#include	"kb_dispscroller.h"
#include	"kb_sizer.h"
#include	"kb_writer.h"
#include	"kb_framer.h"
#include	"kb_popupmenu.h"
#include	"kb_slot.h"
#include	"kb_reportinitdlg.h"
#include	"kb_dcopobject.h"

#define		__DEBUG_REPORT	(0)
#if		__DEBUG_REPORT
#include	"kb_dprintf.h"
#endif


static	cchar	*makeDCOPName
	(	const QString	&name
	)
{
	return	qstrdup (QString("rekall-report/" + name).latin1()) ;
}

/*  ------------------------------------------------------------------	*/

/*  KBReport								*/
/*  KBReport	: Constructor for report root class for extant report	*/
/*  location	: KBLocation &	 : Report location			*/
/*  aList	: const QDict<QString> &				*/
/*				 : List of attributes			*/
/*  (returns)	: KBReport	 :					*/

KBReport::KBReport
	(	const KBLocation	&location,
		const QDict<QString>	&aList
	)
	:
	KBReportBlock	(0,     aList,       "KBReport"),
	KBLayout	(this),
	m_language	(this,  "language",  	aList     ),
	m_caption	(this,  "caption",   	aList     ),
	m_modal		(this,  "modal",     	aList     ),
	m_printer	(this,  "printer",   	aList     ),
	m_printdlg	(this,  "printdlg",  	aList     ),
	m_margins	(this,		     	aList     ),
	m_vpage		(this,		     	aList     ),
	m_onAuth	(this,  "onauth",	aList	  ),
	m_onLoad	(this,  "onload",	aList	  ),
	m_onOpened	(this,  "onopened",	aList	  ),
	m_onUnload	(this,  "onunload",	aList	  ),
	m_local		(this,  "local",	aList	  ),
	m_uuid		(this,	"uuid",		aList	  ),
	m_docRoot	(this,  m_children,  	location  )
{
	m_root		= this	;
	m_display	= 0     ;
	m_writer	= 0	;
	m_dcopIface	= new KBDCOPObject (this, makeDCOPName (m_name.getValue())) ;

	m_geom.set	(KBAttrGeom::FMFixed, KBAttrGeom::FMFixed) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;
	m_geom.set	(KBAttrGeom::MgmtStatic, 0, 0) ;
}

#if	! __KB_RUNTIME

/*  KBReport								*/
/*  KBReport	: Constructor for report root class for new report	*/
/*  location	: KBLocation &	 : Report location			*/
/*  aList	: const QDict<QString> &				*/
/*				 : List of attributes			*/
/*  _ok		: bool &	 : Success				*/
/*  (returns)	: KBReport	 :					*/

KBReport::KBReport
	(	const KBLocation	&location,
		const QDict<QString>	&aList,
		bool			&_ok
	)
	:
	KBReportBlock	(0,	  aList,   "KBReport"),
	KBLayout	(this),
	m_language	(this,  "language",	aList     ),
	m_caption	(this,  "caption",	aList     ),
	m_modal		(this,  "modal",	aList     ),
	m_printer	(this,  "printer",	aList     ),
	m_printdlg	(this,  "printdlg",	aList     ),
	m_margins	(this,			aList     ),
	m_vpage		(this,			aList     ),
	m_onAuth	(this,  "onauth",	aList	  ),
	m_onLoad	(this,  "onload",	aList	  ),
	m_onOpened	(this,  "onopened",	aList	  ),
	m_onUnload	(this,  "onunload",	aList	  ),
	m_local		(this,  "local",	aList	  ),
	m_uuid		(this,	"uuid",		aList	  ),
	m_docRoot	(this,  m_children,	location  )
{
	m_root		= this ;
	m_display	= 0    ;
	m_writer	= 0    ;
	m_dcopIface	= new KBDCOPObject (this, makeDCOPName (m_name.getValue())) ;

	m_geom.set	(KBAttrGeom::FMFixed, KBAttrGeom::FMFixed) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;
	m_geom.set	(KBAttrGeom::MgmtStatic, 0, 0) ;


	if (!reportPropDlg (0))
	{	_ok	= false ;
		return	;
	}

	if (!KBBlock::propertyDlg (0))
	{	_ok	= false ;
		return	;
	}

	addFramers ()	;
	KBLayout::setChanged ()	;
	_ok	= true	;
}
#endif

/*  KBReport								*/
/*  ~KBReport	: Destructor for report root class			*/
/*  (returns)	:		:					*/

KBReport::~KBReport ()
{
	showMonitor ((RKListView *)0) ;
}

/*  KBReport								*/
/*  printNode	: Output node XML					*/
/*  text	: QString &	: Resulting text			*/
/*  indent	: int		: Indent level of this node		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBReport::printNode
	(	QString		&text,
		int		indent,
		bool		flat
	)
{
	extern	QString	kbXMLEncoding () ;
	QString		nodeText ;

	/* Implement this specially, since we need to output the XML	*/
	/* header. Also output script stuff and paramaters near the	*/
	/* front.							*/
	text	+= QString
		   (	"<?xml version=\"1.0\" encoding=\"%1\"?>\n"
		   )
		   .arg	(kbXMLEncoding()) ;

	text	+= QString("%1<%2").arg("", indent).arg(getElement()) ;
	for (uint idx1 = 0 ; idx1 < m_attribs.count () ; idx1 += 1)
		m_attribs.at(idx1)->printAttr(text, nodeText, indent + 2, flat) ;
	text	+= ">\n" ;

	CITER	(Script, s, s->printNode (text, indent + 2, flat))
	CITER	(Param,  p, p->printNode (text, indent + 2, flat))

	LITER
	(	KBNode,
		m_children,
		child,
		{
			if (child->isScript ()) continue ;
			if (child->isParam  ()) continue ;
			child->printNode (text, indent + 2, flat) ;
		}
	)

	for (uint idx2 = 0 ; idx2 < m_slotList.count () ; idx2 += 1)
		m_slotList.at(idx2)->printNode (text, indent + 2, flat) ;

	text	+= nodeText ;
	text	+= QString("%1</%2>\n").arg("", indent).arg(getElement()) ;
}


/*  KBReport								*/
/*  requery	: Re-execute top-level query				*/
/*  (returns)	: bool		  : Success				*/

bool	KBReport::requery ()
{
	if (!KBReportBlock::requery ()) return false ;
	if (!KBReportBlock::showData()) return false ;

	/* Success .....						*/
	return	true	;
}

/*  KBReport								*/
/*  resize	: Handle resize in design mode				*/
/*  size	: QSize		: Raw size				*/
/*  (returns)	: void		:					*/

void	KBReport::resize
	(	QSize	size
	)
{
#if	! __KB_RUNTIME
	if (showingDesign())
	{
		KBReportBlock::setGeometry (QRect(QPoint(0, 0), size)) ;
		KBLayout     ::setChanged () ;
	}
#endif
}

/*  KBReport								*/
/*  showData	: Show report in data mode				*/
/*  parent	: QWidget *	  : Parent widget			*/
/*  writer	: KBWriter *	  : Writer used for output		*/
/*  pDict	: QDict<QString>& : Paramater dictionary		*/
/*  key		: const KBValue	& : Parent key				*/
/*  size	: QSize &	  : Return required size		*/
/*  printing	: bool		  : Actually printing			*/
/*  (returns)	: KB::ShowRC	  : Startup return code			*/

KB::ShowRC
	KBReport::showData
	(	QWidget			*parent,
		KBWriter		*writer,
		const QDict<QString>	&pDict,
		const KBValue		&key,
		QSize			&size,
		bool			printing
	)
{
	/* In the event of any error while starting up, we switch to	*/
	/* design mode, and indicate this to the called via the return	*/
	/* code.							*/
	KBScriptError	*eRc	;
	KB::ShowRC 	sRc	;
	KBError	   	error 	;
	KBValue		resval	;

	m_writer = writer ;

	DPRINTF
	((	"KBReport::showData: p=[%s] d=[%d] p=[%d]\n",
		(cchar *)m_printer.getValue (),
		m_printdlg.getBoolValue(),
		printing
	))	;

	if (!m_writer->setup
		(	m_printer .getValue    (),
			m_printdlg.getBoolValue(),
			m_margins.m_lMargin,
			m_margins.m_rMargin,
			m_margins.m_tMargin,
			m_margins.m_bMargin,
			printing
		))
		return	KB::ShowRCCancel ;

	if (m_vpage.m_enabled)
		if (!writer->setupVirtual
			(	m_vpage.m_colWidth,
				m_vpage.m_rowHeight,
				m_vpage.m_colGap,
				m_vpage.m_rowGap,
				m_vpage.m_borders,
				m_vpage.m_skip
			))
			return	KB::ShowRCCancel ;			

	writer->setReport () ;

	m_parentKey = key ;
	DPRINTF
	((	"KBReport::showData: key=[%s]\n",
		(cchar *)m_parentKey.getRawText()
	))	;

	/* Reset the document root object (which, for instance, clears	*/
	/* an extant scripting interface).				*/
	m_docRoot.reset () ;


	/* We can now set the parameter dictionary (which may invoke	*/
	/* scripts for default values). This may return indicating an	*/
	/* error or that the user cancelled.				*/
	sRc = m_docRoot.setParamDict (pDict, error) ;
	if (sRc != KB::ShowRCOK)
	{
		if (sRc == KB::ShowRCCancel)
			return	sRc ;

		setError (error) 	;
		goto	 showError	;
	}


	/* Next run the block setup. This may pick up parameter values	*/
	/* that are now set.						*/
	if (!blockSetup ())
		goto	showError	;


	/* Then build the GUI and sort out the geometry. The returned	*/
	/* size is the size needed by the display. Actually, in report	*/
	/* data view there is no real GUI, but the call stitches stuff	*/
	/* together, and the user may later switch to design view.	*/
	if (m_display == 0)
	{
		m_display = KBDisplay::newTopDisplay (parent, this, 0, 0, true) ;
		buildTopDisplay (m_display) ;
	}

	KBReportBlock::showAs (KB::ShowAsData) ;
	size	= m_writer->getSize () ;

	/* Just about there. Add all items to the queries, and then	*/
	/* run the query; this is the bit that will actually generate	*/
	/* the report.							*/
	if (!addAllItems())
	{
		setError
		(	KBError::Error,
			TR
			(	"Report contains blocks which retrieve no values"
			),
			TR
			(	"At least one field in each block should "
				"have a non-empty display expression"
			),
			__ERRLOCN
		)	;
		goto	showError ;
	}

	/* Scripting is up and ready if required, and blocks know	*/
	/* what is going on, so start the form off by executing the	*/
	/* onLoad event, if any.					*/
	eRc	= m_onLoad.execute (resval, 0, 0) ;
	if (eRc != 0)
	{
		KBScriptError::processError (eRc, KBScriptError::Form) ;
		goto	showError ;
	}

	if (!requery ())
		goto	showError ;

	eRc	= m_onOpened.execute (resval, 0, 0) ;
	if (eRc != 0)
	{
		KBScriptError::processError (eRc, KBScriptError::Form) ;
		goto	showError ;
	}

	m_writer->showPage (0) ;
	return	KB::ShowRCData ;

	showError :

#if	! __KB_RUNTIME

		if (showDesign (parent, size) == KB::ShowRCDesign)
			return	KB::ShowRCDesign  ;
#endif
		return	KB::ShowRCError	  ;
}

/*  KBReport								*/
/*  finish	: End rerport operation					*/
/*  (returns)	: void		:					*/

void	KBReport::finish ()
{
	KBValue		resval	;
	KBScriptError	*rc	= m_onUnload.execute (resval, 0, 0) ;

	if (rc != 0)
		KBScriptError::processError (rc, KBScriptError::Form) ;
}

/*  KBReport								*/
/*  showDesign	: Show report in design mode				*/
/*  parent	: QWidget *	  : Parent widget			*/
/*  size	: QSize &	  : Return required size		*/
/*  (returns)	: KB::ShowRC	  : Startup return code			*/

KB::ShowRC
	KBReport::showDesign
	(	QWidget		*parent,
		QSize		&size
	)
{
#if	__KB_RUNTIME

	return	KB::ShowRCError ;

#else

	/* Next run the block setup. This links blocks to queries and	*/
	/* such.							*/
	if (!blockSetup ())
		return	KB::ShowRCError  ;

	/* Then build the GUI and sort out the geometry. The returned	*/
	/* size is the size needed by the design display.		*/
	if (m_display == 0)
	{
		m_display = KBDisplay::newTopDisplay (parent, this, 0, 0, true) ;
		buildTopDisplay (m_display) ;
	}

#if	! __KB_RUNTIME
	KBLayout     ::clear      (true)  ;
	KBLayout     ::initSizer  ()	  ;
#endif
	KBReportBlock::showAs     (KB::ShowAsDesign) ;

	size	= geometry().size() ;
	size	+= QSize (m_margins.m_lMargin + m_margins.m_rMargin, 0) ;
	size	+= QSize (100, 200) ;

	m_display->resizeContents     (QSize(1600, 1600)) ;
	m_display->setDisplayGeometry (geometry()) ;

	return	KB::ShowRCDesign ;
#endif
}

/*  KBReport								*/
/*  objType	: Get object type for this node				*/
/*  (returns)	: KB::ObjType	: Object type				*/

KB::ObjType KBReport::objType ()
{
	return	KB::ObjReport	;
}

/*  KBReport								*/
/*  getBlockVal	: Get block value pointer				*/
/*  (returns)	: KBValue *	: Value pointer or null at top level	*/

KBValue	*KBReport::getBlockVal ()
{
	if (!m_cexpr.getValue().isEmpty() && !m_parentKey.isNull())
	{
		DPRINTF
		((	"KBReport::getBlockVal: returns [%s]\n",
			(cchar *)m_parentKey.getRawText()
		))	;
		return	&m_parentKey ;
	}

//	return	KBReportBlock::getBlockVal() ;
	return	0 ;
}

/*  KBReport								*/
/*  margins	: Get margin settings					*/
/*  left, ...	: int &, ...	: Return margins			*/
/*  (returns)	: void		:					*/

void	KBReport::margins
	(	int		&left,
		int		&right,
		int		&top,
		int		&bottom
	)
{
//	left	= m_lMargin.getIntValue() ;
//	right	= m_rMargin.getIntValue() ;
//	top	= m_tMargin.getIntValue() ;
//	bottom	= m_bMargin.getIntValue() ;
	left	= m_margins.m_lMargin ;
	right	= m_margins.m_rMargin ;
	top	= m_margins.m_tMargin ;
	bottom	= m_margins.m_bMargin ;
}

/*  KBReport								*/
/*  margins	: Get margin settings					*/
/*  left, ...	: int &, ...	: Return margins			*/
/*  (returns)	: void		:					*/

void	KBReport::margins
	(	int		&left,
		int		&top
	)
{
//	left	= m_lMargin.getIntValue() ;
//	top	= m_tMargin.getIntValue() ;
	left	= m_margins.m_lMargin ;
	top	= m_margins.m_tMargin ;
}

#if	! __KB_RUNTIME

/*  KBReport								*/
/*  reportPropDlg							*/
/*		: Run report-level property dialog			*/
/*  iniAttr	: cchar	*	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBReport::reportPropDlg
	(	cchar		*iniAttr
	)
{
	QPtrList<KBModule>	scripts	;
	QPtrList<KBModule>	imports	;
	QPtrList<KBParam >	params  ;

	bool	       creating = false ;

	if (getBlkType() == KBBlock::BTUnknown)
	{
		bool		ok ;
		KBReportInitDlg	riDlg (ok) ;
		if (!ok || !riDlg.exec()) return false ;

		m_blkType = riDlg.toplevel() ;
		m_language.setValue (riDlg.language  ()) ;

		creating  = true ;
	}

	KBAttrStr	mattr	(this, "modlist",      "", KAF_SYNTHETIC) ;
	KBAttrStr	iattr	(this, "implist",      "", KAF_SYNTHETIC) ;
	KBAttrStr	pattr	(this, "paramlist",    "", KAF_SYNTHETIC) ;

	CITER
	(	Script,
		s,
		scripts.append (s)
	)
	CITER
	(	Import,
		i,
		imports.append (i)
	)
	CITER
	(	Param,
		p,
		params .append (p)
	)

	if (!::reportPropDlg (this, "Report", m_attribs, scripts, imports, params))
		return false ;

	if (creating)
	{
		bool	rc = false ;

		switch (getBlkType())
		{
			case KBBlock::BTNull	:
			case KBBlock::BTTable	:
			case KBBlock::BTSQL	:
			case KBBlock::BTQuery	:
				rc	= setBlkType (getBlkType()) ;
				break	;

			default	:
				break	;
		}

		if (!rc) return false ;
	}

	if (getContainer() != 0)
		getContainer()->getDisplayWidget()->repaint() ;

	KBLayout::setChanged ()	;
	return	true	;
}

/*  KBReport								*/
/*  contextMenuType							*/
/*		: Return type text for context menu			*/
/*  (returns)	: QString	: Text					*/

QString	KBReport::contextMenuType ()
{
	return	TR("Form") ;
}
#endif
