/***************************************************************************
    file	         : kb_reportblock.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include 	"kb_reportblock.moc"

#include	"kb_framer.h"
#include	"kb_report.h"
#include	"kb_writer.h"
#include	"kb_layout.h"
#include	"kb_qrybase.h"
#include	"kb_display.h"
#include	"kb_nodereg.h"
#include	"kb_popupmenu.h"
#include	"kb_blockevents.h"
#include	"kb_attrdict.h"
#include	"kb_designpopup.h"
#include	"kb_component.h"
#include	"kb_report.h"


#define		__DEBUG_REPORTBLOCK	(0)
#if		__DEBUG_REPORTBLOCK
#include	"kb_dprintf.h"
#endif


/*  KBYObject								*/
/*  KBYObject	: Constructor for Y-order sorting object		*/
/*  object	: KBObject *	: Underlying object			*/
/*  (returns)	: KBYObject	:					*/

KBYObject::KBYObject
	(	KBObject *object
	)
	:
	object	(object)
{
	needs	= 0 ;
	y	= object->geometry().y() ;
}


/*  ------------------------------------------------------------------  */

/*  KBYSort								*/
/*  compareItems: Compare function for Y-order sorted			*/
/*  o1		: QCollection::Item : First KBYObject			*/
/*  o2		: QCollection::Item : Second KBYObject			*/
/*  (returns)	: int		    : Ordering value			*/

int	KBYSort::compareItems
	(	QCollection::Item	o1,
		QCollection::Item	o2
	)
{
	return	((KBYObject *)o1)->y - ((KBYObject *)o2)->y ;
}


/*  ------------------------------------------------------------------  */

/*  KBReportBlock							*/
/*  KBReportBlock: Constructor for extant mreport block element		*/
/*  parent	 : KBNode *	: Parent element			*/
/*  aList	 : const QDict<QString> &				*/
/*				: List of attributes			*/
/*  element	 : cchar *	: Element name				*/
/*		 : bool *	:					*/
/*  (returns)	 : KBReportBlock:					*/

KBReportBlock::KBReportBlock
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		cchar			*element,
		bool			*
	)
	:
	KBBlock	(parent, aList,		element),
	pThrow	(this,	 "pthrow",	aList)
{
	ySort.setAutoDelete (true) ;

	m_geom.set
	(	0,
		parent == 0 ? 0 : KBAttrGeom::XYNoChange,
		parent == 0 ? KBAttrGeom::XYNoChange : 0,
		KBAttrGeom::XYNoChange
	)	;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMFixed) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideW|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;

	/* The number of rows is set to one since reports have a	*/
	/* nominal single row per block, which is iterated over for	*/
	/* each output record.						*/
	m_numRows	= 1	;
}

/*  KBReportBlock							*/
/*  KBReportBlock: Constructor for new report block element		*/
/*  parent	 : KBObject *	 : Parent element			*/
/*  rect	 : const QRect & : Size and position			*/
/*  blkType	 : BlkType	 : Initial block type			*/
/*  ok		 : bool &	 : Success				*/
/*  element	 : cchar *	 : Element name				*/
/*  (returns)	 : KBReportBlock :					*/

KBReportBlock::KBReportBlock
	(	KBObject	*parent,
		const QRect	&rect,
		BlkType		_blkType,
		bool		&ok,
		cchar		*element
	)
	:
	KBBlock	(parent, rect,     _blkType, ok, element),
	pThrow	(this,   "pthrow", false)
{
	if (ok) addFramers  () ;
	ySort.setAutoDelete (true) ;

	m_geom.set
	(	0,
		parent == 0 ? 0 : KBAttrGeom::XYNoChange,
		parent == 0 ? KBAttrGeom::XYNoChange : 0,
		KBAttrGeom::XYNoChange
	)	;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMFixed) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideW|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;

	m_numRows	= 1	;

#if	! __KB_RUNTIME
	if (ok)
		if (!propertyDlg(0))
		{	ok	= false	;
			return	;
		}
#endif
}

/*  KBReportBlock							*/
/*  KBReportBlock: Constructor for extant report block element		*/
/*  _parent	: KBNode *	  : Parent element			*/
/*  _block	: KBReportBlock * : Extant block			*/
/*  (returns)	: KBReportBlock	  :					*/

KBReportBlock::KBReportBlock
	(	KBNode		*parent,
		KBReportBlock	*block
	)
	:
	KBBlock	(parent, 	    block),
	pThrow	(this,    "pthrow", block)
{
	ySort.setAutoDelete (true) ;

	m_geom.set
	(	0,
		parent == 0 ? 0 : KBAttrGeom::XYNoChange,
		parent == 0 ? KBAttrGeom::XYNoChange : 0,
		KBAttrGeom::XYNoChange
	)	;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMFixed) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideW|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;

	m_numRows	= 1	;
}

/*  KBReportBlock							*/
/*  ~KBReportBlock: Destructor for form block element			*/
/*  (returns)	: void		:					*/

KBReportBlock::~KBReportBlock ()
{
}

/*  KBReportBlock							*/
/*  addFramers	: Add header and footer					*/
/*  (returns)	: void		:					*/

void	KBReportBlock::addFramers ()
{
	QRect	r = geometry () ;

	if (r.height() > 120)
	{
		KBAttrDict hDict ;
		KBAttrDict fDict ;

		hDict.addValue ("x",	0		) ;
		hDict.addValue ("y",	0		) ;
		hDict.addValue ("w",	r.width ()	) ;
		hDict.addValue ("h",	40		) ;
		hDict.addValue ("name", "header"	) ;

		fDict.addValue ("x",	0		) ;
		fDict.addValue ("y",	r.height() - 40	) ;
		fDict.addValue ("w",	r.width ()	) ;
		fDict.addValue ("h",	40		) ;
		fDict.addValue ("name", "footer"	) ;

		m_header = new KBHeader (this, hDict, "KBBlockHeader") ;
		m_footer = new KBFooter (this, fDict, "KBBlockFooter") ;
	}
}

/*  KBReportBlock							*/
/*  ySortObjects: Sort embedded objects into Y order			*/
/*  (returns)	: void		:					*/

void	KBReportBlock::ySortObjects ()
{
	ySort.clear () ;

	/* We need to group controls into sets, divided up by nested	*/
	/* blocks, so that we can output in the appropriate order. To	*/
	/* start, order all objects in increasing Y-order.		*/
	CITER
	(	Object,
		o,
		if ((o->isHidden() == 0) && (o->isFramer()) == 0)
			ySort.inSort (new KBYObject (o))
	)

	/* We now need to find out how much space (depth) is used by	*/
	/* each set, so that we can generate page throws. We find the	*/
	/* depth for each set, and note it against the first control	*/
	/* in that set.							*/
	KBYObject *unknown	= ySort.at(0) ;
	uint	  deltaP	= m_header == 0 ? 0 : m_header->height() ;

	for (uint idx = 0 ; idx < ySort.count() ; idx += 1)
	{
		KBYObject	*yObj	= ySort.at(idx) ;
		KBReportBlock	*repBlk	= yObj->object->isReportBlock() ;

		if (repBlk != 0)
		{
			unknown->needs = yObj->y - deltaP  ;
			deltaP	       = yObj->y + repBlk->geometry().height() ;
			unknown	       = idx + 1 >= ySort.count() ?
							 0 : ySort.at(idx + 1) ;
		}
	}

	if (unknown != 0)
	{
		int	h = geometry().height() ;

		if (m_footer == 0)
			unknown->needs = h - deltaP ;
		else	unknown->needs = h - deltaP - m_footer->height() ;
	}

//	LITER (KBYObject, ySort, o)
//		fprintf (stderr, "[%d][%d]\n", o->y, o->needs) ;
}

/*  KBReportBlock							*/
/*  showData	 : Display values from query in fields			*/
/*  (returns)	 : bool		: Success				*/

bool	KBReportBlock::showData ()
{
	uint	 qRows	 = m_query->getNumRows (m_qryLvl) ;
	KBWriter *writer = getReport ()->getWriter () ;


	/* Sort the block objects into Y-order, this is the order that	*/
	/* they will be output so that we move down the page and handle	*/
	/* nested containers correctly.					*/
	ySortObjects () ;


	/* Unlike a form, where all control are prepared once when the	*/
	/* form is executed, in a report we need to prepare all		*/
	/* controls at and below a block each time the block data is	*/
	/* shows. This will, for instance, reset all summary controls.	*/
	prepare	() ;


	/* The header is only output here if there are no data rows,	*/
	/* otherwise it gets output inside the loop (see comment there)	*/
	/* Its not obvious that we get called with no rows, but lets be	*/
	/* safe.							*/
	if (qRows == 0)
		if (m_header != 0)
		{
			m_header->writeData () ;
			writer  ->setOffset (false, QPoint (0, m_header->height())) ;
		}


	/* If there is a footer then reserver space for it. Page throws	*/
	/* will occur when the footer reservation would otherwise be	*/
	/* encroached upon.						*/
	if (m_footer != 0)
		writer->reserve (m_footer->height()) ;


	for (m_curQRow = 0, m_curDRow = 0 ; m_curQRow < qRows ; m_curQRow += 1)
	{
		int	extra	= 0 ;
		uint	delta	= m_header == 0 ? 0 : m_header->height() ;
		bool	evRc	;

		/* Load the current row from the query. This will also	*/
		/* load any header and/or footer fields.		*/

		/* TRICKY CODE ALERT:					*/
		/* We have to load before deciding whether a page throw	*/
		/* is needed, since some controls have heights that are	*/
		/* data depenand. But, this means that summary controls	*/
		/* that appear in a footer will get an "extra" row of	*/
		/* data. Because of this, printing in the footer passes	*/
		/* a "prior" flag; see KBSummary::getReportValue().	*/
		m_query->setCurrentRow (m_qryLvl, m_curQRow) ;
		m_query->loadItems     (m_qryLvl, m_curQRow) ;

		KBValue	arg ((int)m_curQRow) ;
		if (!eventHook (m_events->onCurrent, 1, &arg, evRc)) return false ;


		/* If this is the first row and there is a header then	*/
		/* generate it now. It is deferred to this point so	*/
		/* that the first query row has been loaded, so that	*/
		/* data values can appear in the header (importantly 	*/
		/* including those with "=....." display expressions).	*/
		if ((m_curQRow == 0) && (m_header != 0))
		{
			m_header->writeData () ;
			writer  ->setOffset (false, QPoint (0, m_header->height())) ;
		}

		/* Now loop through all the objects to be displayed,	*/
		/* outputing their values ....				*/
		LITER
		(	KBYObject,
			ySort,
			yObj,

			KBObject	*obj	= yObj->object ;
			KBReportBlock	*repBlk	= obj ->isReportBlock() ;

			/* At each nested block, advance down by the	*/
			/* appropriate amount and recurse into the	*/
			/* block.					*/
			if (repBlk != 0)
			{
				writer->setOffset
				(	false,
					QPoint (0, yObj->y - delta)
				)	;

				if (!repBlk->requery ())
				{	setError (repBlk->lastError()) ;
					return   false ;
				}

				if (!repBlk->showData ())
				{	setError (repBlk->lastError()) ;
					return   false ;
				}

				delta	 = yObj->y + repBlk->geometry().height() ;
				continue ;
			}

			/* See if there is enough space for the next	*/
			/* set of controls. If not then finish this	*/
			/* page and page throw. 			*/
			if (!writer->spaceAvailable (yObj->needs))
			{

				/* Check to prevent looping if we run	*/
				/* out of space (ie., we get squeezed	*/
				/* out by headers and footers)		*/
				if (writer->pageIsEmpty())
				{
					setError
					(	KBError::Error,
						TR("Insufficient space on page"),
						QString(TR("Object %1: needs %2"))
							.arg(yObj->object->getName())
							.arg(yObj->needs),
						__ERRLOCN
					)	;
					return	false	;
				}

				finishPage	(true)  ;
				writer->newPage ()	;

				m_curDRow = m_curQRow 	;

				/* Start a new page since we know that	*/
				/* there must be at least one control	*/
				/* to output.				*/
				startPage ()		;
			}

			/* OK, enough space, so output. As we do so,	*/
			/* keep track of the maximum extra extension	*/
			/* resulting from expanding controls (like	*/
			/* memos and rich text).			*/
			int e ;
			if (!obj->write (writer, QPoint(0, -delta), m_curDRow == m_curQRow, e))
				return	false	;

			e += obj->geometry().height() ;
			if (e > extra) extra = e ;
		)

		/* Here we finish with one row of data, so advance down	*/
		/* the page for the space occupied by the last set of	*/
		/* controls. If there has been extra space for expended	*/
		/* controls beyond the nominal advances then allow for	*/
		/* that.						*/
		int h = geometry().height() - delta ;
		if (m_footer != 0) h -= m_footer->height() ;

		if (extra > h)
		{
			DPRINTF
			((	"KBReportBlock::showData: extra %d -> %d\n",
				h,
				extra
			))	;

			h = extra ;
		}
			
		writer->setOffset (false, QPoint(0, h)) ;

		/* If set to page-per-row then finish this page. We	*/
		/* also start a new page if thete are more rows to data	*/
		/* to come.						*/
		if (pThrow.getIntValue() == 2)
		{
			finishPage 	(true)	;
			writer->newPage	()	;
			m_curDRow = m_curQRow	;

			if (m_curQRow < qRows - 1) startPage() ;
		}
	}

	/* If set to page-per-group then finish the page now and start	*/
	/* a new one ....						*/
	if (pThrow.getIntValue() == 1)
	{
		finishPage 	(true) ;
		writer->newPage	() ;
		return	true	;
	}

	/* ... otherwise if there is a footer then output that. Note	*/
	/* that if this is the topmost block then set the offset to	*/
	/* get the footer at the bottom on the page.			*/
	if (m_footer != 0)
	{
		if (getParent() == 0)
			writer->setOffset (true, QPoint(0, writer->reserve (0))) ;

		m_footer->writeData (false) ;
		writer  ->setOffset (false, QPoint(0, m_footer->height())) ;

		/* In case we are not the topmost block, "unreserve"	*/
		/* the footer space.					*/
		writer  ->reserve   (- m_footer->height()) ;
	}

	return	true ;
}

/*  KBReportBlock							*/
/*  startPage	: Start a new output page				*/
/*  (returns)	: void		:					*/

void	KBReportBlock::startPage ()
{
	KBWriter *writer = getReport()->getWriter() ;

	if (getBlock() != 0)
		getBlock()->isReportBlock()->startPage () ;

	if (m_header != 0)
	{
		m_header->writeData () ;
		writer  ->setOffset (false, QPoint (0, m_header->height())) ;
	}

	if (m_footer != 0) writer->reserve (m_footer->height()) ;
}

/*  KBReportBlock							*/
/*  finishPage	: Finish a page of output				*/
/*  goDown	: bool		: Move down to reservation point	*/
/*  (returns)	: void		:					*/

void	KBReportBlock::finishPage
	(	bool	goDown
	)
{
	KBWriter *writer = getReport()->getWriter() ;

	if (goDown)
		writer  ->setOffset (true,  QPoint(0, writer->reserve (0))) ;

	if (m_footer != 0)
	{
		/* Tricky code alert. Write *prior* values to the	*/
		/* footer for applicable controls.			*/
		m_footer->writeData (true) ;
		writer  ->setOffset (false, QPoint(0, m_footer->height())) ;
	}

	if (getBlock() != 0)
		getBlock()->isReportBlock()->finishPage(false) ;
}

/*  KBReportBlock							*/
/*  getValue	: Get value						*/
/*  qrow	: uint		: Query row				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBReportBlock::getValue
	(	uint	qrow
	)
{
	return	m_query->getField (m_qryLvl, qrow, m_qryIdx.qryIdx()) ;
}

/*  KBBlock								*/
/*  setValue	: Set value display row					*/
/*  qrow	: uint		 : Query row				*/
/*  value	: const KBValue &: Value to set				*/
/*  (returns)	: bool		 : Success				*/

bool	KBReportBlock::setValue
	(	uint		,
		const KBValue	&
	)
{
//	if (mValues[0].value.getRawText() != value.getRawText())
//	{	mValues[0].value = value ;
//		mValues[0].delta = true	 ;
//	}

	return	true	;
}

/*  KBReportBlock							*/
/*  newNode	: Add a new node					*/
/*  id		: int		: Actually NodeSpec pointer for node	*/
/*  (returns)	: KBNode *	: The new node				*/

KBNode	*KBReportBlock::newNode
	(	int	id
	)
{
#if	! __KB_RUNTIME

	extern	KBNode	*makeCtrlFromWizard
		(	KBNode		*parent,
			KBQryBase	*query,
			NodeSpec	*spec,
			KBAttrDict	&aDict,
			bool		&cancel
		)	;

	NodeSpec	*spec	= idToNodeSpec (id) ;
	QRect		cRect	= autoCtrlRect (false) ;
	KBAttrDict	aDict	(0) ;
	bool		ok	;
	KBNode		*node	;

	aDict.addValue ("x", 	    cRect.x     ()) ;
	aDict.addValue ("y", 	    cRect.y     ()) ;
	aDict.addValue ("w", 	    cRect.width ()) ;
	aDict.addValue ("h", 	    cRect.height()) ;
	aDict.addValue ("align",    Qt::AlignLeft ) ;

	bool	cancel	;
	node	= makeCtrlFromWizard (this, getQuery(), spec, aDict, cancel) ;

	if (node == 0)
		if (!cancel)
		{
			node	= spec->m_nodeFunc (this, aDict, &ok) ;
			if (!ok) return 0 ;
		}
		else	return	0 ;

	KBObject *obj ;
	if ((obj = node->isObject()) != 0)
	{
		obj->buildDisplay (m_blkDisp) ;
		if (obj->isItem() != 0)
			obj->isItem()->setupControls () ;
	}

	node->showAs (KB::ShowAsDesign) ;
	getLayout()->setChanged() ;

	if (obj != 0)
		getLayout()->addSizer (obj->getSizer(), true) ;

	return	node	;
#else
	return	0	;
#endif
}


/*  KBReportBlock							*/
/*  replicate	: Replicate the block					*/
/*  parent	: KBNode *	: Parent node				*/
/*  (returns)	: KBNode *	: Replicant				*/

KBNode	*KBReportBlock::replicate
	(	KBNode		*parent
	)
{
	KBReportBlock *rep = new KBReportBlock (parent, this) ;
	KBNode	      *res = replicateBelow    (rep) ;

	rep->findQuery	() ;
	return	res	;
}

/*  KBReportBlock							*/
/*  docPropDlg	: Show report property dialog				*/
/*  (returns)	: void		:					*/

void	KBReportBlock::docPropDlg ()
{
#if	! __KB_RUNTIME
	if (getReport   () != 0)
	{	getReport   ()->reportPropDlg (0) ;
		return	;
	}
	if (getComponent() != 0)
	{	getComponent()->compPropDlg   (0) ;
		return	;
	}
#endif
}

/*  KBReportBlock							*/
/*  newTableBlock: Add a new table block				*/
/*  (returns)	 : void		:					*/

void	KBReportBlock::newTableBlock ()
{
#if	! __KB_RUNTIME
	newReportBlock (m_blkDisp, KBBlock::BTTable) ;
#endif
}

/*  KBReportBlock								*/
/*  newSQLBlock	: Add a new SQL block					*/
/*  (returns)	: void		:					*/

void	KBReportBlock::newSQLBlock ()
{
#if	! __KB_RUNTIME
	newReportBlock (m_blkDisp, KBBlock::BTSQL) ;
#endif
}

/*  KBReportBlock							*/
/*  newQueryBlock: Add a new query block				*/
/*  (returns)	 : void		:					*/

void	KBReportBlock::newQueryBlock ()
{
#if	! __KB_RUNTIME
	newReportBlock (m_blkDisp, KBBlock::BTQuery) ;
#endif
}

/*  KBReportBlock							*/
/*  addHeader	: Add a header						*/
/*  (returns)	: void		:					*/

void	KBReportBlock::addHeader ()
{
#if	! __KB_RUNTIME
	KBAttrDict ad	;
	ad.addValue ("h", 20) ;

	bool	   ok 	;
	KBHeader   *h 	= new KBHeader (this, ad, "KBBlockHeader", &ok) ;

	if (!ok)
	{
		delete	h ;
		return	  ;
	}

	h->buildDisplay	(m_blkDisp) ;
	h->setGeometry	(h->geometry()) ;
	h->showAs	(KB::ShowAsDesign) ;
	h->getContainer	()->show() ;

	getLayout ()->setChanged() ;
#endif
}

/*  KBReportBlock							*/
/*  addFooter	: Add a footer						*/
/*  (returns)	: void		:					*/

void	KBReportBlock::addFooter ()
{
#if	! __KB_RUNTIME
	KBAttrDict ad	;
	ad.addValue ("h", 20) ;

	bool	   ok 	;
	KBFooter   *f 	= new KBFooter (this, ad, "KBBlockFooter", &ok) ;

	if (!ok)
	{
		delete	f ;
		return	  ;
	}

	f->buildDisplay	(m_blkDisp) ;
	f->setGeometry	(f->geometry()) ;
	f->showAs	(KB::ShowAsDesign) ;
	f->getContainer	()->show() ;

	getLayout ()->setChanged() ;
#endif
}


/*  KBReportBlock							*/
/*  makeNewPopup: Make a new-thingy popup menu for a report		*/
/*  parent	: KBPopupMenu *	: Parent popup menu			*/
/*  rect	: QRect		: New object area			*/
/*  (returns)	: void		:					*/

KBPopupMenu
	*KBReportBlock::makeNewPopup
	(	KBPopupMenu	*parent,
		QRect		rect
	)
{
#if	!__KB_RUNTIME
	setCtrlRect (rect) ;
	return	makeReportNewPopup (parent, rect, this, &m_bState) ;
#else
	return	0 ;
#endif
}

/*  KBReportBlock							*/
/*  setGeometry	: Block moves in its parent				*/
/*  rect	: const QRect &	: New position				*/
/*  (returns)	: void		:					*/

void	KBReportBlock::setGeometry
	(	const QRect	&rect
	)
{
	KBBlock::setGeometry (rect) ;
}

/*  KBReportBlock							*/
/*  rowsInBlock	: Get number of rows					*/
/*  (returns)	: uint		: Number of rows			*/

uint	KBReportBlock::rowsInBlock ()
{
	/* This is only used in design mode, when the value is aways	*/
	/* unity.							*/
	return	1	;
}

NEWNODENAMED	(ReportBlock, (cchar *)0, 0, KNF_REPORT, ReportBlock   )
NEWNODENAMED	(ReportBlock, (cchar *)0, 0, KNF_REPORT, ReportSubBlock)
