/***************************************************************************
    file	         : kb_reportdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qprinter.h>
#include	<qwidgetstack.h>

#if		__KB_AUTOMOC
#include	"kb_reportdlg.moc"
#else
#include	"kb_reportdlg.h"
#endif

#include	"kb_report.h"
#include	"kb_param.h"
#include	"kb_dbdociter.h"

#include	"tk_messagebox.h"
#include	"tk_print.h"
#include	"rk_combobox.h"


static	IntChoice	choiceResolution[] =
{
	{	TKPrinter::ResScreen,		"Screen (Y-axis)"	},
	{	TKPrinter::ResPrinter,		"Printer (75 dpi)"	},
	{	TKPrinter::ResHigh,		"High (600 dpi)"	},
	{	-1,				0			}
}	;



/*  KBReportPropDlg							*/
/*  KBReportPropDlg: Constructor for root properties dialog		*/
/*  root	 : KBReport *	   : Root node				*/
/*  caption	 : cchar *	   : Caption text			*/
/*  attribs	 : QPtrList<KBAttr  >&:Attributes list			*/
/*  scripts	 : QPtrList<KBModule>&: List of script modules		*/
/*  imports	 : QPtrList<KBModule>&: List of import modules		*/
/*  params	 : QPtrList<KBParam >&: List of parameters modules		*/
/*  (returns)	 : KBReportPropDlg :					*/

KBReportPropDlg::KBReportPropDlg
	(	KBReport		*root,
		cchar			*caption,
		QPtrList<KBAttr  >	&attribs,
		QPtrList<KBModule>	&scripts,
		QPtrList<KBModule>	&imports,
		QPtrList<KBParam >	&params 
	)
	:
	KBPropDlg (root, caption, attribs),
	root	  (root)
{
	m_scriptDlg = new KBScriptDlg (m_propStack, root, scripts, false) ;
	m_importDlg = new KBImportDlg (m_propStack, root, imports) ;
	m_paramDlg  = new KBParamDlg  (m_propStack, root, params ) ;

	m_scriptDlg->hide() ;
	m_importDlg->hide() ;
	m_paramDlg ->hide() ;
}

/*  KBReportPropDlg							*/
/*  ~KBReportPropDlg: Constructor for root properties dialog		*/
/*  (returns)	  :		:					*/

KBReportPropDlg::~KBReportPropDlg ()
{
}

/*  KBReportPropDlg							*/
/*  hideProperty : Check whether to hide attribute property		*/
/*  attr	 : KBAttr *	: Attribute in question			*/
/*  (returns)	 : bool		: True to hide				*/

bool	KBReportPropDlg::hideProperty
	(	KBAttr	*attr
	)
{
	/* Since KBReport is derived from KBBlock there are a whole lot	*/
	/* of properties which we must ignore; so check just for those	*/
	/* of our own.							*/
	if (qstrcmp (attr->getOwnerName(), "KBReport") == 0)
		return	false	;

	return	true	;
}


/*  KBReportPropDlg							*/
/*  getAttrItem	: Get attribute item for attribute			*/
/*  attr	: KBAttr *	: Attribute				*/
/*  (returns)	: KBAttrItem *	: Associated attribute item		*/

KBAttrItem
	*KBReportPropDlg::getAttrItem
	(	KBAttr	*attr
	)
{
	if (attr->getName() == "resolution")
		return	new KBAttrIntChoice (attr, choiceResolution) ;

	return	KBPropDlg::getAttrItem (attr) ;
}

/*  KBReportPropDlg							*/
/*  showProperty: Show property						*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Property on show			*/

bool	KBReportPropDlg::showProperty
	(	KBAttrItem	*item
	)
{
	QString	aName	= item->attr()->getName () ;

	/* The script module list is handled entirely locally, with	*/
	/* none of the default KBPropDlg controls.			*/
	if (aName == "modlist")
	{
		setUserWidget (m_scriptDlg) ;
		return	true	 ;
	}
	if (aName == "implist")
	{
		setUserWidget (m_importDlg) ;
		return	true	 ;
	}
	if (aName == "paramlist")
	{
		setUserWidget (m_paramDlg ) ;
		return	true	 ;
	}

	/* Block type options are shown using the KBPropDlg combobox,	*/
	/* initialised with the set of possibilities. Also, the first	*/
	/* time round, make a note of the current report setting.		*/
	if (aName == "blktype")
	{
		m_comboBox->clear	() ;
		m_comboBox->insertItem  (TR("Menu-only block" )) ;
		m_comboBox->insertItem  (TR("Table block"     )) ;
		m_comboBox->insertItem  (TR("SQL block"       )) ;
		m_comboBox->insertItem  (TR("Query block"     )) ;

		QString	value = item->value() ;
		if	(value == "null" ) m_comboBox->setCurrentItem (0) ;
		else if (value == "sql"  ) m_comboBox->setCurrentItem (2) ;
		else if (value == "query") m_comboBox->setCurrentItem (3) ;
		else			   m_comboBox->setCurrentItem (1) ;

		m_comboBox->show () ;

		if (m_ibtype.isNull()) m_ibtype = value ;
		return	true	 ;
	}

	if (aName == "printer")
	{
		KBDBDocIter docIter	;
		KBError	    error	;

		if (!docIter.init
			(	root->getDocRoot()->getDocLocation().dbInfo(),
				root->getDocRoot()->getDocLocation().server(),
				"print",
				"prn",
				error
			))
		{
			error.DISPLAY() ;
			return	false	;
		}

		QString	name	;
		QString	stamp	;
		int	aIdx	= -1	;

		m_comboBox->insertItem ("");

		while (docIter.getNextDoc (name, stamp))
		{
			if (name == item->value()) aIdx = m_comboBox->count() ;
			m_comboBox->insertItem (name) ;
		}

		if (aIdx >= 0) m_comboBox->setCurrentItem (aIdx) ;
		m_comboBox->show () ;
		return	true	;
	}

	if (aName == "resolution")
	{
		showChoices (item, choiceResolution, item->value()) ;
		return	true ;
	}

	return	KBPropDlg::showProperty (item) ;
}

/*  KBReportPropDlg								*/
/*  saveProperty: Check and save attribute				*/
/*  item	: KBAttrItem *	: Associated item			*/
/*  (returns)	: bool		: Attribute OK				*/

bool	KBReportPropDlg::saveProperty
	(	KBAttrItem	*item
	)
{
	QString	aName	= item->attr()->getName() ;

	/* As for showing the script module list, saving the results is	*/
	/* handled locally.						*/
	if (aName == "modlist"  )
	{
		setProperty  ("modlist",   m_scriptDlg->getText()) ;
		return	true ;
	}
	if (aName == "implist"  )
	{
		setProperty  ("implist",   m_importDlg->getText()) ;
		return	true ;
	}
	if (aName == "paramlist")
	{
		setProperty  ("paramlist", m_paramDlg ->getText()) ;
		return	true ;
	}

	/* Similarly handle the block type here ...			*/
	if (aName == "blktype")
	{
		static	cchar	*vals[] =
		{	"null",
			"table",
			"sql",
			"query"
		}	;

		/* Get the selected type. If this is a change from the	*/
		/* current block type then warn the user that a change	*/
		/* will trash the report contents.			*/
		cchar	*newTyp	= vals[m_comboBox->currentItem()] ;

		if (!m_ibtype.isEmpty() && (newTyp != m_ibtype))
			if (TKMessageBox::questionYesNo
			   	(	0,
					TR("Changing block type will reset report contents; continue?"),
					TR("Change block type")
				)
				!= TKMessageBox::Yes) return false ;

		setProperty (item, newTyp) ;
		return	true ;
	}

	if (aName == "printer")
	{
		setProperty (item, m_comboBox->currentText()) ;
		return	true ;
	}

	if (aName == "resolution")
	{
		saveChoices (item, choiceResolution) ;
		return	true	;
	}

	return	KBPropDlg::saveProperty (item) ;
}

/*  KBReportPropDlg:							*/
/*  clickOK	 : Handle OK button click				*/
/*  (returns)	 : void		:					*/

void	KBReportPropDlg::clickOK ()
{
	m_scriptDlg->fixUp () ;
	m_importDlg->fixUp () ;
	m_paramDlg ->fixUp () ;
	KBPropDlg::clickOK () ;
}

/*  KBReportPropDlg:							*/
/*  preExec	  : Called immediately before KBDialog::exec		*/
/*  (returns)	  : void	:					*/

void	KBReportPropDlg::preExec ()
{
	/* Trap this and load the notional hidden attribute. We have to	*/
	/* do this here since it is not until "KBPropDlg::exec" has	*/
	/* been called that "setProperty" will work.			*/
	setProperty  ("modlist",   m_scriptDlg->getText()) ;
	setProperty  ("implist",   m_importDlg->getText()) ;
	setProperty  ("paramlist", m_paramDlg ->getText()) ;
}


/*  reportPropDlg: Run the root properties dialog box			*/
/*  report	: KBReport *	  : Report node				*/
/*  caption	: cchar *	  : Caption text			*/
/*  attribs	: QPtrList<KBAttr  >&:Attributes list			*/
/*  scripts	: QPtrList<KBModule>&: List of script modules		*/
/*  imports	: QPtrList<KBModule>&: List of import modules		*/
/*  params	: QPtrList<KBParam >&: List of parameters			*/
/*  (returns)	: bool		  : Success				*/

bool	reportPropDlg
	(	KBReport	*report,
		cchar		*caption,
		QPtrList<KBAttr  >	&attribs,
		QPtrList<KBModule>	&scripts,
		QPtrList<KBModule>	&imports,
		QPtrList<KBParam >	&params
	)
{
	KBReportPropDlg rDlg (report, caption, attribs, scripts, imports, params) ;
	return	rDlg.exec  () ;
}
