/***************************************************************************
    file	         : kb_richtext.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<ctype.h>

#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>
#include	<qcursor.h>
#include	<qurl.h>


#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_sizer.h"
#include	"kb_display.h"
#include	"kb_ctrl.h"
#include	"kb_formblock.h"

#if		__KB_AUTOMOC
#include	"kb_richtext.moc"
#else
#include	"kb_richtext.h"
#endif

#include	"kb_qtrichtext.h"
#include	"kb_nodereg.h"
#include	"kb_writer.h"
#include	"kb_layout.h"

#if		! __KB_RUNTIME
#include	"kb_itemdlg.h"
#include	"kb_recorder.h"
#endif

#define		__DEBUG_RICHTEXT	(0)
#if		__DEBUG_RICHTEXT
#include	"kb_dprintf.h"
#endif


/*  KBRichText								*/
/*  KBRichText	: Constructor for QT rich text field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBNode	:					*/

KBRichText::KBRichText
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBItem		(parent, "KBRichText",	"expr",  aList),
	m_fgcolor	(this,   "fgcolor",	aList),
	m_bgcolor	(this,   "bgcolor",	aList),
	m_font		(this,	 "font",	aList),
	m_frame		(this,	 "frame", 	aList, KAF_FORM),
	m_supress	(this,	 "supress",	aList, KAF_REPORT),
	m_toolbar	(this,	 "toolbar", 	aList, KAF_FORM),
	m_onLink	(this,	 "onlink",	aList, KAF_FORM|KAF_EVCS)
{
#if	! __KB_RUNTIME
	if (ok != 0)
	{
		KBItemPropDlg rtDlg (this, "Rich text", m_attribs) ;
		if (!rtDlg.exec())
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif
	if (isReportDoc())
		m_report = getParent()->getReport () ;
	else	m_report = 0 ;
}

/*  KBRichText								*/
/*  KBRichText	: Constructor for simple form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  richText	: KBRichText *	: Extant field				*/
/*  (returns)	: KBRichText	:					*/

KBRichText::KBRichText
	(	KBNode			*parent,
		KBRichText		*richText
	)
	:
	KBItem		(parent,"expr",		richText),
	m_fgcolor	(this,   "fgcolor",	richText),
	m_bgcolor	(this,   "bgcolor",	richText),
	m_font		(this,	 "font",	richText),
	m_frame		(this,	 "frame", 	richText),
	m_supress	(this,	 "supress",	richText, KAF_REPORT),
	m_toolbar	(this,	 "toolbar", 	richText, KAF_FORM  ),
	m_onLink	(this,	 "onlink",	richText, KAF_FORM|KAF_EVCS)
{
	if (isReportDoc())
		m_report = getParent()->getReport () ;
	else	m_report = 0 ;
}

/*  KBRichText								*/
/*  ~KBRichText	: Destructor for simple form field			*/
/*  (returns)	:		:					*/

KBRichText::~KBRichText ()
{
}

/*  KBRichText								*/
/*  makeCtrl	: Make a field control					*/
/*  morph	: bool		: Get morped control			*/
/*  (returns)	: KBControl *	: New control				*/

KBControl
	*KBRichText::makeCtrl
	(	uint		drow
	)
{
	return	new KBCtrlRichText (getDisplay(), this, drow) ;
}

/*  KBRichText								*/
/*  getReportValue							*/
/*		: Get value for report writing				*/
/*  first	: bool		: First output flag			*/
/*  prior	: bool		: Return prior value flag		*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBRichText::getReportValue
	(	bool		first,
		bool
	)
{
	QString	text	;

	if (first || (m_curVal != m_lastVal) || !m_supress.getBoolValue())
	{
		m_lastVal = m_curVal ;
		return	    m_curVal ;
	}

	return	KBValue() ;
}

/*  KBRichText								*/
/*  replicate	: Replicate this field					*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New field node			*/

KBNode	*KBRichText::replicate
	(	KBNode	*parent
	)
{
	return	new KBRichText (parent, this) ;
}

/*  KBRichText								*/
/*  linkClicked	: Link click method					*/
/*  qrow	: uint		  : Query row				*/
/*  link	: const QString & : Link				*/
/*  (returns)	: void		  :					*/

void	KBRichText::linkClicked
	(	uint		qrow,
		const QString	&link
	)
{
#if	! __KB_RUNTIME
	KBRecorder *recorder = KBRecorder::self() ;
	if ((recorder != 0) && recorder->isRecording(getDocRoot()))
		recorder->mouseClick (this, qrow - getBlock()->getCurDRow(), link) ;
#endif
	/* This is probably overkill, but we decode the link as a full	*/
	/* URL and pass the component values to the event function.	*/
	QUrl	url	(link)	;

	KBValue	args[9]	;
	bool	evRc	;

	args[0] = (int)qrow	 ;
	args[1] = url.protocol() ;
	args[2] = url.host    () ;
	args[3] = url.port    () ;
	args[4] = url.path    () ;
	args[5] = url.fileName() ;
	args[6] = url.query   () ;
	args[7] = url.user    () ;
	args[8] = url.password() ;

	DPRINTF
	((	"KBRichText::linkClicked: [%d][%d]\n%s\n",
		qrow,
		link.latin1(),
		m_onLink.getValue().latin1()
	))	;

	eventHook (m_onLink, 9, args, evRc) ;
}

#if	! __KB_RUNTIME

/*  KBRichText								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar	*	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBRichText::propertyDlg
	(	cchar		*iniAttr
	)
{
	KBItemPropDlg rtDlg (this, "Rich text", m_attribs, iniAttr) ;
	if (rtDlg.exec())
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}
#endif


NEWNODE(RichText, __TR("New &Rich Text"), 0, KNF_FORM|KNF_REPORT|KNF_BLOCK|KNF_DATA)
