/***************************************************************************
    file	         : kb_rowcoldialog.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qlabel.h>
#include	<qspinbox.h>
#include	<qframe.h>
#include	<qlayout.h>
#include	<qpainter.h>
#include	<qtextview.h>

#include	"kb_attrgeom.h"
#include	"kb_object.h"
#include	"kb_display.h"
#include	"kb_geometry.h"
#include	"kb_docroot.h"
#include	"kb_sidepanel.h"

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_combobox.h"
#include	"rk_lineedit.h"

#if		__KB_AUTOMOC
#include	"kb_rowcoldialog.moc"
#else
#include	"kb_rowcoldialog.h"
#endif

static	cchar	blurb[] =

	"This dialog is used to set the spacing (minimum width or height) "
	"and stretch factors for rows and columns in the grid layout. Use "
	"the rows and columns combo boxes to select rows or columns to be "
	"changed, and the alter the value as needed. The changes will     "
	"appear in the design view. If you cancel the changes, the design "
	"will revert to the original values"
	;

class	KBRowColPicker : public QWidget
{
	KBRowColDialog		*m_dialog	;
	uint			m_rows		;
	uint			m_cols		;
	uint			m_atRow		;
	uint			m_atCol		;

	static const int	m_lm		;
	static const int	m_tm		;
	static const int	m_cg		;
	static const int	m_rg		;

protected :

	virtual	void	paintEvent	(QPaintEvent *) ;
	virtual	void	mousePressEvent	(QMouseEvent *) ;

public	:

	KBRowColPicker	(QWidget *, KBRowColDialog *, uint, uint) ;

	void		setRowCol	(uint, uint)	;
}	;


const int	KBRowColPicker::m_lm	= 5	;
const int	KBRowColPicker::m_tm	= 5	;
const int	KBRowColPicker::m_cg	= 5	;
const int	KBRowColPicker::m_rg	= 5	;

KBRowColPicker::KBRowColPicker
	(	QWidget		*parent,
		KBRowColDialog	*dialog,
		uint		rows,
		uint		cols
	)
	:
	QWidget		(parent),
	m_dialog	(dialog),
	m_rows		(rows),
	m_cols		(cols)
{
	if (m_rows == 0) m_rows = 1 ;
	if (m_cols == 0) m_cols = 1 ;

	m_atRow	= 0	;
	m_atCol	= 0	;
	setMinimumSize (200, 200) ;
}

void	KBRowColPicker::paintEvent
	(	QPaintEvent	*e
	)
{
	QPainter p (this) ;

	p.fillRect
	(	0,
		0,
		width (),
		height(),
		QBrush(Qt::gray)
	)	;

	int	  w	= width () ;
	int	  h	= height() ;

	const int cw	= (w - (m_lm * 2) - (m_cg * (m_cols - 1))) / m_cols ;
	const int rh	= (h - (m_tm * 2) - (m_rg * (m_rows - 1))) / m_rows ;

	p.setPen   (Qt::lightGray) ;

	for (uint r = 0 ; r < m_rows ; r += 1)
		for (uint c = 0 ; c < m_cols ; c += 1)
		{
			QRect	rect
			(	m_lm + c * (cw + m_cg),
				m_tm + r * (rh + m_rg),
				cw,
				rh
			)	;

			p.fillRect
			(	rect,
				(r == m_atRow) && (c == m_atCol) ?
						QBrush(Qt::gray ) :
						QBrush(Qt::white)
			)	;
			p.drawRect (rect) ;
		}
}

void	KBRowColPicker::mousePressEvent
	(	QMouseEvent	*m
	)
{
	int	  x	= m->x  () ;
	int	  y	= m->y  () ;

	int	  w	= width () ;
	int	  h	= height() ;

	const int cw	= (w - (m_lm * 2) - (m_cg * (m_cols - 1))) / m_cols ;
	const int rh	= (h - (m_tm * 2) - (m_rg * (m_rows - 1))) / m_rows ;

	for (uint r = 0 ; r < m_rows ; r += 1)
		for (uint c = 0 ; c < m_cols ; c += 1)
			if (QRect(m_lm + c * (cw + m_cg), m_tm + r * (rh + m_rg), cw, rh).contains(x, y))
				m_dialog->showRowCol (r, c) ;
}

void	KBRowColPicker::setRowCol
	(	uint		row,
		uint		col
	)
{
	m_atRow	= row	;
	m_atCol	= col	;
	update	()	;
}

/*  KBRowColDialog							*/
/*  KBRowColDialog							*/
/*		: Constructor for row/column geomtry dialog		*/
/*  geom	: KBAttrGeom &	 : Geometry				*/
/*  object	: KBObject *	 : Owning container			*/
/*  irow	: uint		 : Initial row				*/
/*  icol	: uint		 : Initial column			*/
/*  (returns)	: KBRowColDialog :					*/

KBRowColDialog::KBRowColDialog
	(	KBAttrGeom	&geom,
		KBObject	*object,
		uint		irow,
		uint		icol
	)
	:
	KBDialog	(TR("Grid setup"), true, "kbrowcoldialog"),
	m_geom		(geom),
	m_object	(object),
	m_display	(object->getContainer()),
	m_rowSetup	(m_geom.rowSetup()),
	m_colSetup	(m_geom.colSetup())
{
	RKVBox	  *layTop	= new RKVBox	(this)	  ;
	layTop->setTracking () ;

	RKHBox	  *layMain	= new RKHBox	(layTop)  ;
	new KBSidePanel (layMain, TR("Grid setup"))	  ;

	QTextView *tv		= new QTextView	(layMain) ;
	tv->setText 	   (TR(blurb)) ;
	tv->adjustSize	   () ;
	tv->setMinimumSize (tv->sizeHint()) ;

	RKVBox	  *laySetup	= new RKVBox	(layMain) ;
	
	RKGridBox *layGrid	= new RKGridBox (3, laySetup) ;

	new QWidget (layGrid) ;
	new QLabel  (TR("Row"),    layGrid) ;
	new QLabel  (TR("Column"), layGrid) ;

	new QWidget (layGrid) ;
	m_rowSelect	= new RKComboBox (layGrid) ;
	m_colSelect	= new RKComboBox (layGrid) ;

	for (uint row = 0 ; row < m_geom.numRows(true) ; row += 1)
		m_rowSelect->insertItem (QString::number(row)) ;
	for (uint col = 0 ; col < m_geom.numCols(true) ; col += 1)
		m_colSelect->insertItem (QString::number(col)) ;

	new QLabel  (TR("Spacing"), layGrid) ;
	m_rowSpacing = new QSpinBox (0, 5000, 1, layGrid) ;
	m_colSpacing = new QSpinBox (0, 5000, 1, layGrid) ;
	
	new QLabel  (TR("Stretch"), layGrid) ;
	m_rowStretch = new QSpinBox (0, 5000, 1, layGrid) ;
	m_colStretch = new QSpinBox (0, 5000, 1, layGrid) ;

	laySetup->addFiller () ;

	m_picker = new KBRowColPicker (layMain, this, m_geom.numRows(true), m_geom.numCols(true)) ;

	addOKCancel (layTop)   ;

	m_row	= -1	;
	m_col	= -1	;
	m_lock	= false	;

	rowChanged (0)	;
	colChanged (0)	;

	connect
	(	m_rowSelect,
		SIGNAL	(activated   (int)),
		SLOT	(rowChanged  (int))
	)	;
	connect
	(	m_colSelect,
		SIGNAL	(activated   (int)),
		SLOT	(colChanged  (int))
	)	;
	connect
	(	m_rowSpacing,
		SIGNAL	(valueChanged(int)),
		SLOT	(settingChanged())
	)	;
	connect
	(	m_rowStretch,
		SIGNAL	(valueChanged(int)),
		SLOT	(settingChanged())
	)	;
	connect
	(	m_colSpacing,
		SIGNAL	(valueChanged(int)),
		SLOT	(settingChanged())
	)	;
	connect
	(	m_colStretch,
		SIGNAL	(valueChanged(int)),
		SLOT	(settingChanged())
	)	;

	showRowCol (irow, icol) ;
}

/*  KBRowColDialog							*/
/*  ~KBRowColDialog							*/
/*		: Destructor for row/column geomtry dialog		*/
/*  (returns)	:		:					*/

KBRowColDialog::~KBRowColDialog ()
{
}

void	KBRowColDialog::showRowCol
	(	uint		row,
		uint		col
	)
{
	m_rowSelect->setCurrentItem (row) ;
	m_colSelect->setCurrentItem (col) ;

	rowChanged (row) ;
	colChanged (col) ;
}

/*  KBRowColDialog							*/
/*  settingChanged							*/
/*		: User has changed a spacing or stretch setting		*/
/*  (returns)	: void		:					*/

void	KBRowColDialog::settingChanged ()
{
	if (!m_lock)
	{
		fprintf
		(	stderr,
			"KBRowColDialog::settingChanged: called ....\n"
		)	;

		/* Bring the grid settings up to date, then pass these	*/
		/* over to the display so that the changes are		*/
		/* reflected in the design view.			*/
		m_rowSetup[m_row].m_spacing = m_rowSpacing->value() ;
		m_rowSetup[m_row].m_stretch = m_rowStretch->value() ;
		m_colSetup[m_col].m_spacing = m_colSpacing->value() ;
		m_colSetup[m_col].m_stretch = m_colStretch->value() ;
		m_display->setRowColSetup  (m_rowSetup, m_colSetup) ;
		m_display->redoLayout	   (true) ;
		m_object ->getRoot()->getDocRoot()->doLayoutChanged() ;
	}
}

/*  KBRowColDialog							*/
/*  rowChanged	: User changes the row section				*/
/*  row		: int		: New selection				*/
/*  (returns)	: void		:					*/

void	KBRowColDialog::rowChanged
	(	int		row
	)
{
	fprintf
	(	stderr,
		"KBRowColDialog::rowChanged: row=%d m_row=%d\n",
		row,
		m_row
	)	;

	/* If there was a prior current row then save the current	*/
	/* stretch and spacing settings from the spinboxes, before	*/
	/* changing them for the newly selected row.			*/
	if (m_row >= 0)
	{
		m_rowSetup[m_row].m_spacing = m_rowSpacing->value() ;
		m_rowSetup[m_row].m_stretch = m_rowStretch->value() ;
	}

	m_lock	= true	;
	m_rowSpacing->setValue (m_rowSetup[row].m_spacing) ;
	m_rowStretch->setValue (m_rowSetup[row].m_stretch) ;
	m_lock	= false ;
	m_row	= row	;

	m_picker->setRowCol (m_row, m_col) ;
}

/*  KBRowColDialog							*/
/*  colChanged	: User changes the column section			*/
/*  col		: int		: New selection				*/
/*  (returns)	: void		:					*/

void	KBRowColDialog::colChanged
	(	int		col
	)
{
	/* This is the same as the above for the row setting.		*/
	if (m_col >= 0)
	{
		m_colSetup[m_col].m_spacing = m_colSpacing->value() ;
		m_colSetup[m_col].m_stretch = m_colStretch->value() ;
	}

	m_lock	= true	;
	m_colSpacing->setValue (m_colSetup[col].m_spacing) ;
	m_colStretch->setValue (m_colSetup[col].m_stretch) ;
	m_lock	= false	;
	m_col	= col	;

	m_picker->setRowCol (m_row, m_col) ;
}

/*  KBRowColDialog							*/
/*  accept	: User accepts						*/
/*  (returns)	: void		:					*/

void	KBRowColDialog::accept()
{
	/* Copy the local values back to the geometry attribute before	*/
	/* signalling success.						*/
	for (uint row = 0 ; row < m_rowSetup.count() ; row += 1)
		m_geom.setRowSetup
		(	row,
			m_rowSetup[row].m_spacing,
			m_rowSetup[row].m_stretch
		)	;
	for (uint col = 0 ; col < m_colSetup.count() ; col += 1)
		m_geom.setColSetup
		(	col,
			m_colSetup[col].m_spacing,
			m_colSetup[col].m_stretch
		)	;

	KBDialog::accept () ;
}
