/***************************************************************************
    file	         : kb_script.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	<qdict.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_database.h"
#include	"kb_form.h"
#include	"kb_report.h"
#include	"kb_location.h"
#include	"kb_callback.h"
#include	"kb_notifier.h"
#include	"kb_docroot.h"
#include	"kb_event.h"
#include	"kb_slot.h"
#include	"kb_errordlg.h"

#if		__KB_AUTOMOC
#include 	"kb_script.moc"
#else
#include 	"kb_script.h"
#endif

#include	"kb_location.h"
#include	"kb_callback.h"
#include	"kb_appptr.h"

#include	"kb_libloader.h"

#include	"rk_listbox.h"
#include	"tk_messagebox.h"


#define		__DEBUG_SCRIPT	(0)
#if		__DEBUG_SCRIPT
#include	"kb_dprintf.h"
#endif

class	KBScriptFactory
{
public	:

	KBFactory	*m_factory	;
	QString		m_options	;
	uint		m_code		;

	inline	KBScriptFactory
		(	KBFactory	*factory,
			const QString	&options,
			uint		code
		)
		:
		m_factory	(factory),
		m_options	(options),
		m_code		(code)
	{
	}
}	;

static	QDict<KBScriptFactory>	factories	;
static	QDict<QString>		identStrings	;


/*  KBScriptIF								*/
/*  registerScript							*/
/*		: Register scripting language and interpreter factory	*/
/*  language	: const QString & : Language				*/
/*  factory	: KBFactory *	  : Interpreter factory			*/
/*  options	: const QString & : Options				*/
/*  (returns)	: void		  :					*/

void	KBScriptIF::registerScript
	(	const QString	&language,
		KBFactory	*factory,
		const QString	&options
	)
{
	factories.insert
	(	language,
		new KBScriptFactory (factory, options, factories.count())
	)	;

	DPRINTF
	((	"KBScriptIF::registerScript: [%s]->[%d]\n",
		(cchar *)language,
		languageToCode(language)
	))	;
}

/*  KBScriptIF								*/
/*  languageToCode							*/
/*  language	: const QString & : Language				*/
/*  (returns)	: uint		  : Code				*/

uint	KBScriptIF::languageToCode
	(	const QString	&language
	)
{
	/* Each language as it is registered is assigned a code number	*/
	/* from zero upwards.						*/
	return	factories.find(language)->m_code ;
}

/*  KBScriptIF								*/
/*  languageCount: Get count of registered languages			*/
/*  (returns)	 : uint		: Count of languages			*/

uint	KBScriptIF::languageCount ()
{
	return	factories.count () ;
}


/*  ------------------------------------------------------------------  */

/*  KBScript								*/
/*  KBScript	: Constructor for script node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: Attribute list			*/
/*		: bool *	:					*/
/*  (returns)	: KBScript	:					*/

KBScript::KBScript
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*
	)
	:
	KBModule	(parent, "KBScript",	aList),
	m_l2		(this,	 "l2",		aList, KAF_FORM)
{
}

/*  KBScript								*/
/*  KBScript	: Constructor for script node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  name	: cchar *	: Module name				*/
/*  l2		: bool		: Second language script		*/
/*  (returns)	: KBScript	:					*/

KBScript::KBScript
	(	KBNode		*parent,
		cchar		*name,
		bool		l2
	)
	:
	KBModule	(parent, "KBScript",	name),
	m_l2		(this,	 "l2",		l2,	KAF_FORM)
{
}

/*  KBScript								*/
/*  KBScript	: Constructor for script node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  entant	: KBScript *	: Extant node				*/
/*  (returns)	: KBScript	:					*/

KBScript::KBScript
	(	KBNode		*parent,
		KBScript	*extant
	)
	:
	KBModule	(parent, "KBScript",	extant),
	m_l2		(this,	 "l2",		extant, KAF_FORM)
{
}


/*  KBScript								*/
/*  ~KBScript	: Destructor for script node				*/
/*  (returns)	:		:					*/

KBScript::~KBScript ()
{
}

/*  KBScript								*/
/*  isL2	: Check if this is a second-language script		*/
/*  (returns)	: bool		: True of so				*/

bool	KBScript::isL2 ()
{
	return	m_l2.getBoolValue() ;
}

/*  ------------------------------------------------------------------  */

/*  KBScriptIF								*/
/*  KBScriptIF	: Constructor for script base class			*/
/*  (returns)	: KBScriptIF	:					*/

KBScriptIF::KBScriptIF ()
{
}

/*  KBScriptIF								*/
/*  ~KBScriptIF	: Destructor for script base class			*/
/*  (returns)	:		:					*/

KBScriptIF::~KBScriptIF ()
{
}

/*  KBScriptIF								*/
/*  editorInit	: Initialise the editor colouring			*/
/*  editWin	: KBTextEdit *	: Editor				*/
/*  (returns)	: void		:					*/

void	KBScriptIF::editorInit
	(	KBTextEdit	*
	)
{
	/* Default does nothing.					*/
}

/*  KBScriptIF								*/
/*  showDebug	: Show debugger if possible				*/
/*  action	: TKToggleAction * : Toggle button			*/
/*  (returns)	: KBDebug *	   : Debugger widget			*/

KBDebug	*KBScriptIF::showDebug
	(	TKToggleAction	*
	)
{
	/* Default does nothing.					*/
	return	0 ;
}

/*  KBScriptIF								*/
/*  debugScript	: Load script into debugger				*/
/*  location	: KBLocation &	: Script loaation			*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBScriptIF::debugScript
	(	KBLocation	&,
		KBError		&pError
	)
{
	pError	= KBError
		  (	KBError::Error,
			TR("Scripts cannot be loaded directly into debugger"),
			QString::null,
			__ERRLOCN
		  )	;
	return	false	;
}

/*  ------------------------------------------------------------------	*/

struct	LocationStackItem
{
	const KBLocation	*m_location	;
	KBNode			*m_node		;
}	;

static	QValueList<LocationStackItem>	*locationStack	;

/*  KBScriptIF								*/
/*  pushLocation: Push location and/or node on to location stack	*/
/*  location	: const KBLocation *	: Location			*/
/*  node	: KBNode *		: Node				*/
/*  (returns)	: void			:				*/

void	KBScriptIF::pushLocation
	(	const KBLocation	*location,
		KBNode			*node
	)
{
	DPRINTF
	((	"KBScriptIF::pushLocation: l=%p n=%p\n",
		(void *)location,
		(void *)node
	))	;

	if (locationStack == 0)
		locationStack = new QValueList<LocationStackItem> ;

	/* Special case, if the location is not set but there is a	*/
	/* prior location, then use that.				*/
	if (location == 0)
		if (locationStack->count() > 0)
			location = locationStack->last().m_location ;

	LocationStackItem lsi 		;
	lsi.m_location	= location	;
	lsi.m_node	= node		;
	locationStack->push_back (lsi)	;
}

/*  KBScriptIF								*/
/*  popLocation	: Pop most recent location from stack			*/
/*  (returns)	: void		:					*/

void	KBScriptIF::popLocation ()
{
	DPRINTF
	((	"KBScriptIF::popLocation\n"
	))	;
	if (locationStack != 0)
		if (locationStack->count() > 0)
			locationStack->pop_back() ;
}

/*  KBScriptIF								*/
/*  topLocation	: Get top-most location on stack			*/
/*  (returns)	: const KBLocation *	: Location or null if none	*/

const KBLocation
	*KBScriptIF::topLocation ()
{
	if (locationStack != 0)
		if (locationStack->count() > 0)
			return	locationStack->last().m_location ;
	return	0	;
}

/*  KBScriptIF								*/
/*  topLocationNode : Get top-most node on stack			*/
/*  (returns)	    : const KBLocation * : Node or null if none		*/

KBNode	*KBScriptIF::topLocationNode ()
{
	if (locationStack != 0)
		if (locationStack->count() > 0)
			return	locationStack->last().m_node ;
	return	0	;
}




void	KBScriptIF::addIdentString
	(	const QString	&lang,
		const QString	&ident
	)
{
	identStrings.setAutoDelete (true) ;
	identStrings.replace	   (lang, new QString(ident)) ;
}

QString	KBScriptIF::getIdentStrings ()
{
	QString			res  ;
	QDictIterator<QString>	iter (identStrings) ;

	while (iter.current() != 0)
	{
		res	+= QString("<tr><td>Script</td><td><b>%1</b></td><td><nobr>%2</nobr></td></tr>")
				  .arg( iter.currentKey())
				  .arg(*iter.current   ()) ;

		iter	+= 1 ;
	}

	return	res	;
}

/*  ------------------------------------------------------------------  */

/*  LinkKBScript: Link to database scripting language			*/
/*  name	: const QString & : Scripting language name		*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: KBScriptIF *	  : Script interface object or null	*/

KBScriptIF
	*LinkKBScript
	(	const QString	&name,
		KBError		&pError
	)
{
	KBScriptFactory *factory = factories.find (name) ;
	if (factory == 0)
	{
		pError	= KBError
			  (	KBError::Fault,
				QString (TR("Cannot locate script library for \"%1\"")).arg(name),
				"",
				__ERRLOCN
			  )	;
		return	0	;
	}

	KBScriptIF *iface = (KBScriptIF*)0L;
	iface =	(KBScriptIF *)factory->m_factory->create
					(	0,
						"scriptiface",
						factory->m_options
					)	;
	if (iface == 0)
	{
		pError	= KBError
			  (	KBError::Fault,
				QString ("Cannot create script interface for \"%1\"").arg(name), 
				"",
				__ERRLOCN
			  )	;
		return	0	;
	}

	iface->setApp  (KBNotifier::self(), KBAppPtr::getCallback()) ;
	KBScriptIF::addIdentString (name, iface->ident()) ;

	return	iface	;
}

/*  KBScript								*/
/*  replicate	: Replicate this parameter				*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New choice node			*/

KBNode	*KBScript::replicate
	(	KBNode	*_parent
	)
{
	return	new KBScript (_parent, this) ;
}


NEWNODE(Script, (cchar *)0, 0, KNF_FORM|KNF_REPORT)


/*  ------------------------------------------------------------------  */

/*  KBScriptItem							*/
/*  ~KBScriptItem: Destructor for script items				*/
/*  (returns)	 :		:					*/

KBScriptItem::~KBScriptItem ()
{
}

/*  KBScriptItem							*/
/*  makeModule	: Instantiate a script module				*/
/*  parent	: KBNode *	  : Parent node				*/
/*  name	: const QString & : Script name				*/
/*  (returns)	: KBModule *	  : New script module			*/

KBModule*KBScriptItem::makeModule
	(	KBNode		*parent,
		const QString	&name
	)
{
	return	new KBScript (parent, name, m_l2) ;
}

/*  ------------------------------------------------------------------  */

/*  KBScriptDlg								*/
/*  KBScriptDlg	: Constructor for script list dialog			*/
/*  parent	: QWidget *		: Parent widget			*/
/*  node	: KBNode *		: Parent node			*/
/*  modules	: QPtrList<KBModule> &	: List of scripts		*/
/*  l2		: bool			: Second language		*/
/*  (returns)	: KBScriptDlg	        :				*/

KBScriptDlg::KBScriptDlg
	(	QWidget			*parent,
		KBNode			*node,
		QPtrList<KBModule>	&modules,
		bool			l2
	)
	:
	KBModuleDlg
	(	parent,
		node,
		false,
		node->getRoot()->getAttrVal (l2 ? "language2" : "language")
	),
	m_l2		(l2)
{
	setModules 	(modules) ;
}

/*  KBScriptDlg								*/
/*  ~KBScriptDlg: Constructor for script list dialog			*/
/*  (returns)	:		:					*/

KBScriptDlg::~KBScriptDlg()
{
}

/*  KBScriptDlg								*/
/*  makeModuleItem: Make item for script				*/
/*  listBox	  : RKListBox *	  : Parent list box			*/
/*  module	  : KBModule *	  : Actually the script			*/
/*  (returns)	  : KBModuleItem  : Script item				*/

KBModuleItem
	*KBScriptDlg::makeModuleItem
	(	RKListBox	*listBox,
		KBModule	*module
	)
{
	return	new KBScriptItem (listBox, module->isScript()) ;
}

/*  KBScriptDlg								*/
/*  makeModuleItem: Make item for script				*/
/*  listBox	  : RKListBox *	    : Parent list box			*/
/*  name	  : const QString & : Script name			*/
/*  (returns)	  : KBModuleItem    : Script item			*/

KBModuleItem
	*KBScriptDlg::makeModuleItem
	(	RKListBox	*listBox,
		const QString	&name
	)
{
	return	new KBScriptItem (listBox, name, m_l2) ;
}


/*  ------------------------------------------------------------------  */

KBScriptObject::KBScriptObject ()
{
}

KBScriptObject::~KBScriptObject ()
{
	DPRINTF
	((	"KBScriptObject::~KBScriptObject\n"
	))	;
}

/*  ------------------------------------------------------------------  */

/*  KBScriptError							*/
/*  KBScriptError: Constructor for script error object			*/
/*  (returns)	 : KBScriptError	:				*/

KBScriptError::KBScriptError ()
	:
	m_type		(Abort),
	m_event		(0),
	m_slot		(0),
	m_expr		(0)
{
	/* This constructor is used when the script error is an abort,	*/
	/* that is, when there was an earlier error. Abort "errors" are	*/
	/* never displayed.						*/
	DPRINTF
	((	"KBScriptError::KBScriptError: ABORT\n"
	))	;

	m_node		= 0	 ;
	m_handle	= Normal ;
}

/*  KBScriptError							*/
/*  KBScriptError: Constructor for script error object			*/
/*  error	 : const KBError &	: Error details			*/
/*  event	 : KBEvent *		: Event throwing error		*/
/*  (returns)	 : KBScriptError	:				*/

KBScriptError::KBScriptError
	(	const KBError	&error,
		KBEvent		*event
	)
	:
	m_type		(Event),
	m_error		(error),
	m_event		(event),
	m_slot		(0),
	m_expr		(0)
{
	DPRINTF
	((	"KBScriptError::KBScriptError: EVENT\n"
	))	;

	m_node		= m_event->getOwner() ;
	m_handle	= Normal ;
}

/*  KBScriptError							*/
/*  KBScriptError: Constructor for script error object			*/
/*  error	 : const KBError &	: Error details			*/
/*  slot	 : KBSlot *		: Slot throwing error		*/
/*  (returns)	 : KBScriptError	:				*/

KBScriptError::KBScriptError
	(	const KBError	&error,
		KBSlot		*slot
	)
	:
	m_type		(Slot ),
	m_error		(error),
	m_event		(0),
	m_slot		(slot ),
	m_expr		(0)
{
	DPRINTF
	((	"KBScriptError::KBScriptError: SLOT\n"
	))	;

	m_node		= m_slot->parent () ;
	m_handle	= Normal ;
}

KBScriptError::KBScriptError
	(	const KBError	&error,
		KBAttrExpr	*expr
	)
	:
	m_type		(Slot ),
	m_error		(error),
	m_event		(0),
	m_slot		(0),
	m_expr		(expr)
{
	DPRINTF
	((	"KBScriptError::KBScriptError: EXPR\n"
	))	;

	m_node		= expr->getOwner() ;
	m_handle	= Normal ;
}

KBScriptError::KBScriptError
	(	const KBError	&error,
		KBNode		*node,
		KBLocation	&location,
		const QString	&errText,
		int		errLno,
		bool		debugged
	)
	:
	m_type		(Location),
	m_error		(error	 ),
	m_event		(0),
	m_slot		(0),
	m_location	(location),
	m_expr		(0),
	m_errText	(errText ),
	m_errLno	(errLno	 ),
	m_debugged	(debugged)
{
	DPRINTF
	((	"KBScriptError::KBScriptError: LOCATION\n"
	))	;
	m_node		= node	 ;
	m_handle	= Normal ;
}

KBScriptError::KBScriptError
	(	const KBError	&error
	)
	:
	m_type		(Error),
	m_error		(error),
	m_event		(0),
	m_slot		(0),
	m_expr		(0)
{
	DPRINTF
	((	"KBScriptError::KBScriptError: ERROR\n"
	))	;

	m_node		= 0	 ;
	m_handle	= Normal ;
}

KBScriptError::KBScriptError
	(	const KBError	&error,
		KBNode		*node,
		KBMacroExec	*macro
	)
	:
	m_type		(Macro),
	m_error		(error),
	m_event		(0),
	m_slot		(0),
	m_expr		(0)
{
	DPRINTF
	((	"KBScriptError::KBScriptError: MACRO\n"
	))	;

	m_node		= node   ;
	m_handle	= Normal ;
}


/*  KBScriptError							*/
/*  processError: Process the error					*/
/*  name	: const QString & : Property name			*/
/*  (returns)	: void		  :					*/

void	KBScriptError::processError
	(	const QString	&name
	)
{
#if	! __KB_RUNTIME
	DPRINTF
	((	"KBScriptError::processError: [%s]\n",
		name.latin1()
	))	;

	/* Check Form's and Report's as special cases, as these have	*/
	/* distinct property dialogs.					*/
	if ((m_handle == Form  ) && (m_node->isForm  () != 0))
	{
		m_node->isForm  ()->KBForm  ::formPropDlg  (name) ;
		return	;
	}

	if ((m_handle == Report) && (m_node->isReport() != 0))
	{
		m_node->isReport()->KBReport::reportPropDlg(name) ;
		return	;
	}

	/* Hidden's are a bit special since it does not have its own	*/
	/* property dialog ...						*/
	if ((m_handle == Hidden) && (m_node->isItem  () != 0))
	{
		m_node->isItem  ()->KBItem  ::propertyDlg  ("hidden") ;
		return	;
	}

	/* Default is to call the "propertyDlg" method on the node.	*/
	/* This is virtual so we get the right dialog.			*/
	m_node->propertyDlg (name) ;
#endif
}

/*  KBScriptError							*/
/*  processError: Process the error					*/
/*  (returns)	: void		:					*/

void	KBScriptError::processError ()
{
#if	! __KB_RUNTIME
	if (m_type == None )
		return	;

	if (m_type == Abort)
		return	;

	if (m_node != 0)
		m_node->getDocRoot ()->doExecError() ;


	switch (m_type)
	{
		case Event	:
			/* EVENT					*/
			/* We get a hint for which property dialog to	*/
			/* display since we need to handle the special	*/
			/* cases.					*/
			processError (m_event->getName ()) ;
			break	;

		case Slot 	:
			/* SLOT						*/
			/* Similarly use the hint, except that form and	*/
			/* report level errors are processed at the	*/
			/* block dialog level (which is where the slots	*/
			/* will be seen).				*/
			processError (QString("slots:%1").arg(m_slot->name())) ;
			break	;

		case Location	:
			if (!m_debugged)
			{	KBError	error	;
				if (!KBAppPtr::getCallback()->editScript (m_location, m_errText, "", m_errLno, error))
					error.DISPLAY () ;
			}
			break	;

		case Macro	:
			break	;

		case Error	:
			break	;

		default	:
			break	;
	}
#endif
}

/*  KBScriptError							*/
/*  processError: Process the error					*/
/*  error	: KBScriptError * : Error				*/
/*  handle	: Handle	  : Error handling mode			*/
/*  (returns)	: void		  :					*/

void	KBScriptError::processError
	(	KBScriptError	*error,
		Handle		handle
	)
{
	/* Set the error handling mode and display the actual error. If	*/
	/* error blocking is on, this may simply add the error to those	*/
	/* outstanding.							*/
	error->setHandler (handle) ;
	error->error().DISPLAY  () ;

	/* Check whether to process the error now; if an error blocker	*/
	/* is extant then the error will be queued until the error	*/
	/* blocker terminates.						*/
	if (KBErrorBlock::processError (error))
	{
		error->processError () ;
		delete	error	;
	}
}

/*  ------------------------------------------------------------------	*/


/*  KBScriptCode							*/
/*  KBScriptCode: Constructor for inline code base class		*/
/*  owner	: KBNode *	: Node owning event, slot, etc.		*/
/*  inherit	: KBEvent *	: Event inheritance			*/
/*  (returns)	: KBScriptCode	:					*/

KBScriptCode::KBScriptCode
	(	KBNode		*owner,
		KBEvent		*inherit
	)
	:
	m_owner	   (owner),
	m_inherit  (inherit)
{
}


/*  KBScriptCode							*/
/*  ~KBScriptCode: Destructor for inline code base class		*/
/*  (returns)	 :		:					*/

KBScriptCode::~KBScriptCode ()
{
}

/*  KBScriptCode							*/
/*  owner	: Get owning node					*/
/*  (returns)	: KBNode *	: Owning node				*/

KBNode	*KBScriptCode::owner () const
{
	return	m_owner	;
}

/*  KBScriptCode							*/
/*  setErrorMessage							*/
/*		: Store error message after execution error		*/
/*  errorMsg	: const QSTring & : Error message			*/
/*  (returns)	: void		  :					*/

void	KBScriptCode::setErrorMessage
	(	const QString	&errorMsg
	)
{
	m_errorMsg = errorMsg ;
}

/*  KBScriptCode							*/
/*  errorMessage: Retrieve last error message				*/
/*  (returns)	: QString	: Error message				*/

QString	KBScriptCode::errorMessage ()
{
	fprintf
	(	stderr,
		"KBScriptCode::errorMessage: %p: %12.12s...\n",
		(void *)this,
		m_errorMsg.latin1()
	)	;
	return	m_errorMsg ;
}



/*  ------------------------------------------------------------------	*/

KBScriptTestResult::KBScriptTestResult
	(	const QString			&location,
		uint				lineNumber,
		const QString			&comment,
		KBScriptTestResult::TestResult	result,
		const QString			&message,
		const QString			&language,
		const QString			&path
	)
	:
	m_location	(location),
	m_lineNumber	(lineNumber),
	m_comment	(comment),
	m_result	(result),
	m_message	(message),
	m_language	(language),
	m_path		(path)
{
	DPRINTF
	((	"KBScriptTestResult::KBScriptTestResult: l=[%s/%d] c=[%s]\n",
		location.latin1(),
		lineNumber,
		comment .latin1()
	))	;
}

KBScriptTestResult::KBScriptTestResult ()
	:
	m_lineNumber	(0),
	m_result	(KBScriptTestResult::testOK)
{
}
