/***************************************************************************
    file	         : kb_script.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		_KB_SCRIPT_H
#define		_KB_SCRIPT_H

#include	<qvaluelist.h>
#include	<qptrlist.h>
#include	<qdict.h>

#include	"kb_module.h"
#include	"kb_moduleitem.h"
#include	"kb_location.h"

#include	"libkbase_exports.h"


class	KBDebug			;
class	TKToggleAction		;
class	KBTextEdit		;
class	RKListBox		;


/*  KBScriptTestResult							*/
/*  ------------------							*/
/*  Class used to store results from tests when in test suite mode.	*/
/*  Results are accumulated in a list which can be retrieved for	*/
/*  display.								*/

class	LIBKBASE_API	KBScriptTestResult
{
public	:

	enum	TestResult
	{	testOK		= 0,
		testFailed	= 1
	}	;

	QString		m_location	;
	uint		m_lineNumber	;
	QString		m_comment	;
	TestResult	m_result	;	
	QString		m_message	;
	QString		m_language	;
	QString		m_path		;

	KBScriptTestResult	() ;
	KBScriptTestResult
		(	const QString	&,
			uint		,
			const QString	&,
			TestResult	,
			const QString	&,
			const QString	&,
			const QString	&
		)	;
}	;

typedef	QValueList<KBScriptTestResult>	KBScriptTestResultList	;



/*  KBScript								*/
/*  ------------							*/
/*  This node specifies information about database script modules used	*/
/*  by reports and forms.						*/

class 	LIBKBASE_API	KBScript : public KBModule
{
	KBAttrBool	m_l2	;

public	:

	KBScript	(KBNode *, const QDict<QString> &, bool * = 0) ;
	KBScript	(KBNode *, cchar    *, bool) ;
	KBScript	(KBNode *, KBScript *) ;
	virtual 	~KBScript () ;

	virtual	KBNode		*replicate (KBNode *) ;

	virtual	KBScript	*isScript  () { return this ; }

	enum	ExeRC
	{
		ExeError,	/* Script error, caller to handle	*/
		ExeFail,	/* Script error, debugger handles	*/
		ExeAbort,	/* Aborted for nested error		*/
		ExeTrue,	/* Script returned true			*/
		ExeFalse,	/* Script returned false		*/
		ExeTest		/* Script returned test failure		*/
	}	;

	bool		isL2	() ;
}	;


/*  KBScriptItem							*/
/*  ------------							*/
/*  Wrapper class round KBModuleItem which manufactures KBScript	*/
/*  instances.								*/

class 	LIBKBASE_API	KBScriptItem : public KBModuleItem
{
	bool		m_l2	;

protected :

	virtual	KBModule *makeModule (KBNode *, const QString &) ;

public	:

	inline	KBScriptItem
		(	RKListBox	*listBox,
			KBScript	*script
		)
		:
		KBModuleItem	(listBox, script)
	{
		m_l2	= script->isL2() ;
	}

	inline	KBScriptItem
		(	RKListBox	*listBox,
			const QString	&name,
			bool		l2
		)
		:
		KBModuleItem	(listBox, name),
		m_l2		(l2)
	{
	}

	virtual	~KBScriptItem () ;
}	;


/*  KBScriptDlg								*/
/*  -----------								*/
/*  This wraps the module dialog to implement the virtual methods that	*/
/*  instantiate module items (here as script items).			*/

class 	LIBKBASE_API	KBScriptDlg : public KBModuleDlg
{
	bool		m_l2	;

protected :

	virtual	KBModuleItem	*makeModuleItem (RKListBox *, KBModule 	    *) ;
	virtual	KBModuleItem	*makeModuleItem (RKListBox *, const QString &) ;

public	:

	KBScriptDlg (QWidget *, KBNode *, QPtrList<KBModule> &, bool) ;
	virtual ~KBScriptDlg () ;
}	;


/*  KBScriptCode							*/
/*  ------------							*/
/*  This class is a base class for chunks of script code which are	*/
/*  manipulated in-store (eg., embedded in forms or held in a database)	*/

class 	LIBKBASE_API	KBScriptCode
{
protected	:

	KBNode		*m_owner	;
	KBEvent		*m_inherit	;
	QString		m_errorMsg	;

public		:

	KBScriptCode			(KBNode *, KBEvent *) ;
	virtual	~KBScriptCode 		()	 ;

	KBNode		*owner		() const ;

	virtual	KBScript::ExeRC	execute
		(	uint		,
			const KBValue	*,
			KBValue		&
		)	= 0 ;
	virtual	KBScript::ExeRC	execute
		(	KBNode		*,
			const QString	&,
			uint		,
			const KBValue	*,
			KBValue		&
		)	= 0 ;

	virtual	void	setBreakpoints
		(	const QValueList<int>	&
		)	= 0 ;

	void		setErrorMessage	(const QString &) ;
	QString		errorMessage	()		  ;
}	;


/*  KBScriptObject							*/
/*  --------------							*/
/*  Instances of this class can be attached to a KBObject by a script	*/
/*  interpreter to provide object persistence. These instances are	*/
/*  destroyed on data/design transitions and on KBObject deletion.	*/

class	LIBKBASE_API	KBScriptObject
{
public	:

	KBScriptObject() ;
	virtual	~KBScriptObject	() ;
}	;


/*  KBScriptIF								*/
/*  ----------								*/
/*  An instance of this object is instantiated by each form in order	*/
/*  to access a scripting language. It contains methods to load and to	*/
/*  unload modules, and to executen a script function.			*/

class 	LIBKBASE_API	KBScriptIF : public QObject
{	
	Q_OBJECT

public	:

	enum	TestMode
	{	testOff		= 0,
		testSuite	= 1
	}	;


	virtual	void	setApp  (KBNotifier *, KBCallback *) = 0 ;

	virtual	bool	compile
		(	KBLocation		&,
			KBError			&
		)	= 0 ;

	virtual	bool	load
		(	KBLocation		&,
			KBError			&
		)	= 0 ;

	virtual	KBScriptCode   *compileFunc
		(	KBNode			*owner,
			const QString		&script,
			const QString		&path,
			const QString		&fname,
			const QStringList	&imports,
			KBEvent			*inherit,
			KBError			&error
		)	= 0 ;

	virtual	KBScriptCode   *compileExpr
		(	KBNode			*owner,
			const QString		&expr,
			const QString		&path,
			const QStringList	&imports,
			KBError			&error
		)	= 0 ;

	virtual	KBScript::ExeRC	execute
		(	const QStringList	&,
			const QString		&,
			KBNode			*,
			uint,
			const KBValue		*,
			KBValue 		&
		)	= 0 ;

	virtual	QString	   	ident		() = 0	;
	virtual	bool		isClientSide	() = 0	;
	virtual	KBLocation	lastError	(QString &, uint &, QString &) = 0 ;

	/* Deleting and renaming script files is done through the	*/
	/* interface so that we can handle cases such as EL, where	*/
	/* there may be a compiled code file as well as the source file	*/
	virtual	bool		unlink		(KBLocation   &, KBError &) = 0 ;
	virtual	bool		rename		(KBLocation   &, const QString &, KBError &) = 0 ;

	virtual	void		editorInit	(KBTextEdit     *) ;
	virtual	KBDebug		*showDebug	(TKToggleAction *) ;
	virtual	bool		debugScript	(KBLocation     &, KBError &) ;


	static	void		addIdentString	(const QString &, const QString &) ;
	static	QString		getIdentStrings	() ;


	/* Register script						*/
	/* ---------------						*/
	/* Called to register a scripting langage. Specifies the	*/
	/* langauge code, the factory for the language interface, and	*/
	/* options.							*/
	static	void		registerScript
		        	(       const QString   &language,
        			        KBFactory       *factory,
                			const QString   &options
        			)	;

	/* Language codes						*/
	/* --------------						*/
	/* Langages are mapped to small integers (0,1,...n) which are	*/
	/* then used to quickly identify which language is in use.	*/
	static	uint		languageToCode
				(	const QString	 &language
				) 	;
	static	uint		languageCount	()	;


	/* Location stack						*/
	/* --------------						*/
	/* As control weaves back and forth between the Rekall API and	*/
	/* the scripting languages, the most location and node which	*/
	/* are invoking a script are maintained. These are used when:	*/
	/* 	* importing python modules from the database		*/
	/*	* checking for test recording and replay		*/
	static	void		pushLocation
				(	const KBLocation *location,
					KBNode		 *node
				)	;
	static	void		popLocation	()	;

	static	const KBLocation*topLocation	()	;
	static	KBNode		*topLocationNode()	;


	KBScriptIF 		() ;
	virtual	~KBScriptIF	() ;
}	;


/*  KBScriptError							*/
/*  -------------							*/
/*  This class is used to store script error information, and to fire	*/
/*  up the appropriate handling code.					*/

class	KBMacroExec	;
class	KBAttrExpr	;

class	LIBKBASE_API	KBScriptError
{
public	:

	enum	Handle
	{	Normal,
		Form,
		Report,
		Block,
		Hidden
	}	;

private	:

	enum	Type
	{	None,
		Event,
		Slot,
		Location,
		Macro,
		Error,
		Abort
	}	;


	Type		m_type		;	
	KBError		m_error		;
	KBEvent		*m_event	;
	KBSlot		*m_slot		;
	KBLocation	m_location	;
	KBAttrExpr	*m_expr		;

	KBNode		*m_node		;
	QString		m_errText	;
	int		m_errLno	;
	bool		m_debugged	;
	Handle		m_handle	;

	void		processError	(const QString &) ;

public	:

	KBScriptError	() ;
	KBScriptError	(const KBError &) ;
	KBScriptError	(const KBError &, KBEvent     *) ;
	KBScriptError	(const KBError &, KBSlot      *) ;
	KBScriptError	(const KBError &, KBNode      *, KBLocation  &, const QString &, int, bool) ;
	KBScriptError	(const KBError &, KBNode      *, KBMacroExec *) ;
	KBScriptError	(const KBError &, KBAttrExpr  *) ;

	inline	void	setHandler
		(	Handle	handle
		)
	{
		m_handle = handle ;
	}

	inline	const	KBError	&error	()
	{
		return	m_error	;
	}

	void		processError	() ;
	static	void	processError	(KBScriptError *, Handle) ;

	friend	class	KBErrorsTest	;
}	;


/*  ------------------------------------------------------------------	*/

extern LIBKBASE_API KBScriptIF *LinkKBScript (const QString &, KBError &) ;

#endif	// _KB_SCRIPT_H
