/***************************************************************************
    file	         : kb_sizer.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qcursor.h>
#include	<qobjectlist.h>

#if		__KB_AUTOMOC
#include	"kb_sizer.moc"
#else
#include 	"kb_sizer.h"
#endif

#include	"kb_framer.h"
#include	"kb_ctrl.h"
#include	"kb_layout.h"
#include	"kb_display.h"
#include	"kb_options.h"
#include	"kb_report.h"
#include	"kb_writer.h"
#include	"kb_popupmenu.h"
#include	"kb_ascii.h"


#define		__DEBUG_SIZER	(0)
#if		__DEBUG_SIZER
#include	"kb_dprintf.h"
#endif


static	const	QColor	sbColIdle	(63,	63,	63  ) ;
static	const	QColor	sbColLeader	(0,	0,	255 ) ;
static	const	QColor	sbColFollower	(0,	255,	0   ) ;
static	const	QColor	sbColActive	(255,   0,	0   ) ;

static	QCursor		cMove		(Qt::SizeAllCursor  ) ;
static	QCursor		cSizeVer	(Qt::SizeVerCursor  ) ;
static	QCursor		cSizeHor	(Qt::SizeHorCursor  ) ;
static	QCursor		cSizeFDiag	(Qt::SizeFDiagCursor) ;
LIBKBASE_API QCursor 	cNoCursor	(Qt::BlankCursor    ) ;

static	KBSizerInfoSet	defaultInfo	=
{
	SZF_LEFT|SZF_RIGHT|SZF_DBL,

	{	0,	SZF_X|SZF_Y|SZF_MOVE,	&cMove		},
	{	0,	SZF_X,			&cSizeHor	},
	{	0,	SZF_Y,			&cSizeVer	},
	{	0,	SZF_X|SZF_Y,		&cSizeFDiag	}
}	;

static	KBSizerInfoSet	dynamicInfo	=
{
	SZF_LEFT|SZF_RIGHT|SZF_DBL,

	{	0,	0,			&cNoCursor	},
	{	0,	0,			&cNoCursor	},
	{	0,	0,			&cNoCursor	},
	{	0,	0,			&cNoCursor	}
}	;


void	KBSizer::defaultInfoSet
	(	KBSizerInfoSet	&infoSet
	)
{
	infoSet	= defaultInfo	;
}

/*  KBSizerBlob	: Constructor for sizer tool marker			*/
/*  parent	: QWidget *	: Parent display widget			*/
/*  object	: KBObject *	: Object being manipulated		*/
/*  flags	: uint		: Sizing flags				*/
/*  cursor	: QCursor *	: Cursor to be used			*/
/*  (returns)	: KBSizerBlob	:					*/

KBSizerBlob::KBSizerBlob
	(	QWidget		*parent,
		KBObject	*object,
		uint		flags,
		QCursor		*cursor
	)
	:
	QWidget		(parent),
	m_object	(object),
	m_flags		(flags)
{
	setGeometry (0, 0, SZ_BLOBSIZE, SZ_BLOBSIZE) ;
	setPalette  (QPalette (sbColIdle)) ;

	if ((cursor != 0) && (cursor != &cNoCursor))
		setCursor (*cursor) ;

	show	    () ;
}

/*  KBSizerBlob								*/
/*  mousePressEvent							*/
/*		: Handle mouse press to start tracking			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBSizerBlob::mousePressEvent
	(	QMouseEvent 	*e
	)
{
	DPRINTF
	((	"KBSizerBlob::mousePressEvent:\n"
	))	;

	if ((e->button() == Qt::LeftButton) && (m_flags != 0))
		m_object->getSizer()->trackStart (e, this) ;
}

/*  KBSizerBlob								*/
/*  mouseMoveEvent							*/
/*		: Handle mouse movement during tracking			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBSizerBlob::mouseMoveEvent
	(	QMouseEvent 	*e
	)
{
	DPRINTF
	((	"KBSizerBlob::mouseMoveEvent:\n"
	))	;

	m_object->getSizer()->trackMove (e) ;
}

/*  KBSizerBlob								*/
/*  mouseReleaseEvent							*/
/*		: Handle mouse release during tracking			*/
/*  event	: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBSizerBlob::mouseReleaseEvent
	(	QMouseEvent 	*
	)
{
	DPRINTF
	((	"KBSizerBlob::mouseReleaseEvent:\n"
	))	;

	m_object->getSizer()->trackDone () ;
}

/*  KBSizerBlob								*/
/*  move	: Move sizer blob					*/
/*  x		: int		: X-position				*/
/*  y		: int		: Y-position				*/
/*  (returns)	: void		:					*/

void	KBSizerBlob::move
	(	int		x,
		int		y
	)
{
	/* If the object is a report then adjust the position by the	*/
	/* margin settings.						*/
	KBReport *report ;
	if  ((report = m_object->isReport()) != 0)
	{
		int	left	;
		int	top	;

		report->margins (left, top) ;

		x	+= (int)(left * pixelsPerMM()) ;
		y	+= (int)(top  * pixelsPerMM()) ;
	}

	QWidget::move (x, y)  ;
}


/*  ------------------------------------------------------------------	*/

/*  KBSizer								*/
/*  KBSizer	: Constructor for resize/positioning control		*/
/*  object	: KBObject *	   : Object being worked upon		*/
/*  parent	: KBDisplay *	   : Parent display			*/
/*  widget	: QWidget *	   : Actual widget			*/
/*  blobInfo	: KBSizerInfoSet * : Sizer blob options			*/
/*  (returns)	: KBSizer	   :					*/

KBSizer::KBSizer
	(	KBObject	*object,
		KBDisplay	*display,
		QWidget		*widget,
		KBSizerInfoSet	*blobInfo
	)
	:
	m_object	(object),
	m_display	(display),
	m_widget	(widget),
	m_ctrl		(0),
	m_trap		(SZF_LEFT|SZF_RIGHT|SZF_DBL)
{
	init	(blobInfo) ;
}	

/*  KBSizer								*/
/*  KBSizer	: Constructor for resize/positioning control		*/
/*  object	: KBObject *	   : Object being worked upon		*/
/*  parent	: KBDisplay *	   : Parent display			*/
/*  ctrl	: KBControl *	   : Associated control			*/
/*  blobInfo	: KBSizerInfoSet * : Sizer blob options			*/
/*  (returns)	: KBSizer	   :					*/

KBSizer::KBSizer
	(	KBObject	*object,
		KBDisplay	*display,
		KBControl	*ctrl,
		KBSizerInfoSet	*blobInfo
	)
	:
	m_object	(object),
	m_display	(display),
	m_widget	(ctrl->getWidget()),
	m_ctrl		(ctrl),
	m_trap		(SZF_LEFT|SZF_RIGHT|SZF_DBL)
{
	init	(blobInfo) ;
}

/*  KBSizer								*/
/*  init	: Initialise routine					*/
/*  blobInfo	: KBSizerInfoSet * : Sizer blob options			*/
/*  (returns)	: KBSizer	   :					*/

void	KBSizer::init
	(	KBSizerInfoSet	*blobInfo
	)
{
	/* Note if our parent object is a container, in which case it	*/
	/* will do its own mouse handling.				*/
	m_contain =
	     (m_object->isFramer    () != 0) ||
	     (m_object->isBlock     () != 0) ||
	     (m_object->isComponent () != 0) ;


	m_widget->installEventFilter (this)  ;
	QObjectList *l = m_widget->queryList ("QWidget", 0, false, true) ;
	if (l != 0)
		for
		(	QObjectListIt iter (*l) ;
			iter.current() != 0	;
			++iter
		)
			iter.current()->installEventFilter (this) ;

	if (blobInfo == 0)
	{
		if (m_object->parentIsDynamic())
			blobInfo = &dynamicInfo ;
		else	blobInfo = &defaultInfo ;
	}

	KBObject *otl = blobInfo->m_tl.m_proxy == 0 ? m_object : blobInfo->m_tl.m_proxy ;
	KBObject *otr = blobInfo->m_tr.m_proxy == 0 ? m_object : blobInfo->m_tr.m_proxy ;
	KBObject *obl = blobInfo->m_bl.m_proxy == 0 ? m_object : blobInfo->m_bl.m_proxy ;
	KBObject *obr = blobInfo->m_br.m_proxy == 0 ? m_object : blobInfo->m_br.m_proxy ;

	m_btl	= new KBSizerBlob (m_widget, otl, blobInfo->m_tl.m_flags, blobInfo->m_tl.m_cursor) ;
	m_btr	= new KBSizerBlob (m_widget, otr, blobInfo->m_tr.m_flags, blobInfo->m_tr.m_cursor) ;
	m_bbl	= new KBSizerBlob (m_widget, obl, blobInfo->m_bl.m_flags, blobInfo->m_bl.m_cursor) ;
	m_bbr	= new KBSizerBlob (m_widget, obr, blobInfo->m_br.m_flags, blobInfo->m_br.m_cursor) ;
	setBlobs () ;

	m_altered	= false ;
	m_tracking	= false ;
	m_trap		= blobInfo->m_trap ;
}

/*  KBSizer								*/
/*  ~KBSizer	: Destructor for sizer object				*/
/*  (returns)	:		:					*/

KBSizer::~KBSizer ()
{
	DPRINTF
	((	"KBSizer::~KBSizer: delete %p/%p/%p/%p\n",
		(void *)m_btl,
		(void *)m_btr,
		(void *)m_bbl,
		(void *)m_bbr
	))	;

	if (m_object->getLayout() != 0)
		m_object->getLayout()->dropSizer (this) ;

	DELOBJ	(m_btl)	;
	DELOBJ	(m_btr)	;
	DELOBJ	(m_bbl)	;
	DELOBJ	(m_bbr)	;
}

void	KBSizer::hide ()
{
	m_btl->hide () ;
	m_btr->hide () ;
	m_bbl->hide () ;
	m_bbr->hide () ;
}

void	KBSizer::show ()
{
	m_btl->show () ;
	m_btr->show () ;
	m_bbl->show () ;
	m_bbr->show () ;
}

/*  KBSizer								*/
/*  trackStart	: Start tracking movement for resize			*/
/*  e		: QMouseEvent *	: Originating mouse event		*/
/*  blob	: KBSizerBlob *	: Blob originating request		*/
/*  (returns)	: void		:					*/

void	KBSizer::trackStart
	(	QMouseEvent	*e,
		KBSizerBlob	*blob
	)
{
	/* Ingore if already tracking, left button not pressed, or the	*/
	/* blob is marked out.						*/
	if (m_tracking)
		return	;
	if (e->button() != Qt::LeftButton)
		return	;
	if ((blob != 0) && (blob->getFlags() == 0))
		return	;

	if ((m_track = blob) == 0)
		m_track = m_btl ;

	m_tracking 	= true	  	 ;
	m_altered	= false	  	 ;
	m_startMX	= e->globalX ()  ;
	m_startMY	= e->globalY ()  ;

	m_minSize	= m_object->minimumSize() ;
	m_limit		= m_object->getLayout  ()->addSizer (this, (e->state() & Qt::ShiftButton) != 0) ;

	DPRINTF
	((	"KBSizer::trackStart: from (%d,%d) limit (%d,%d,%d,%d)\n",
		m_startMX,
		m_startMY,
		m_limit.left,
		m_limit.right,
		m_limit.up,
		m_limit.down
	))	;

#if	! __KB_PLAT_MACX
	/* Grab the mouse so that all events go to the selected blob.	*/
	/* HACK	:							*/
	/*	This seems not to work properly on OS/X so see the	*/
	/*	additional code in KBSizer::eventFilter()		*/
	m_track->grabMouse () ;
#endif
}

/*  KBSizer								*/
/*  trackMove	: Movement tracker					*/
/*  e		: QMouseEvent *	: Associated mouse event		*/
/*  (returns)	: void		:					*/

void	KBSizer::trackMove
	(	QMouseEvent	*e
	)
{
	DPRINTF
	((	"KBSizer::trackMove: m_tracking=%d\n",
		m_tracking
	))	;

	/* If tracking then move pass the movement to the active sizer	*/
	/* with the distance the most has moved since starting ...	*/
	if (m_tracking)
	{
		int	dx	= e->globalX() - m_startMX ;
		int	dy	= e->globalY() - m_startMY ;
		bool	move	= (m_track->getFlags() & SZF_MOVE) != 0 ;

		if (move)
		{
			if (dx < m_limit.left ) dx = m_limit.left  ;
			if (dx > m_limit.right) dx = m_limit.right ;
			if (dy < m_limit.up   ) dy = m_limit.up    ;
			if (dy > m_limit.down ) dy = m_limit.down  ;
		}

		if ((m_track->getFlags() & SZF_X) == 0) dx = 0 ;
		if ((m_track->getFlags() & SZF_Y) == 0) dy = 0 ;

		m_object->getLayout()->trackMove (dx, dy, move) ;
	}
}

/*  KBSizer								*/
/*  trackDone	: Signal end of tracking				*/
/*  (returns)	: void		:					*/

void	KBSizer::trackDone ()
{
	DPRINTF
	((	"KBSizer::trackDone: m_tracking=%d\n",
		m_tracking
	))	;

	/* Pass this up to the document root node, which is		*/
	/* responsible for passing information to all selected		*/
	/* controls.							*/
	if (m_tracking)
	{
		m_tracking = false ;
		m_track ->releaseMouse () ;
		m_object->getLayout()->releaseSizer () ;
	}
}

/*  KBSizer								*/
/*  doDesignPopup: Show design popup unless disabled			*/
/*  e		 : QMouseEvent * : Invoking event			*/
/*  (returns)	 : bool		 : Popup shown				*/

bool	KBSizer::doDesignPopup
	(	QMouseEvent	*
	)
{
	if ((m_trap & SZF_RIGHT) != 0)
	{
		KBPopupMenu *popup ;
		m_object->getLayout   ()->addSizer (this, false) ;
		popup = m_object->designPopup (0, QRect()) ;
		popup->exec (QCursor::pos()) ;
		delete	popup	;
		return	true	;
	}

	return	false	;
}

/*  KBSizer								*/
/*  eventFilter	: Event filter for associated widget			*/
/*  o		: QObject *	: Actually pointer at the widget	*/
/*  e		: QEvent *	: Event to process			*/
/*  (returns)	: bool		: False to continue processing		*/

bool	KBSizer::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if ((e->type() != QEvent::Timer) && (e->type() != QEvent::Paint))
	{
	DPRINTF
	((	"KBSizer::eventFilter: [%s] [%s] m_contain=%d\n",
		o->className(),
		_TEXT(e->type()),
		m_contain
	))	;
	}

	if (e->type() == QEvent::Resize)
	{
		if (o == m_widget)
		{
			setBlobs ()	;
			return	false	;
		}
	}

	/* If the parent object is a container then it does its own	*/
	/* mouse handling, so we return to let event processing		*/
	/* continue there.						*/
	if (m_contain)
		return	false	;


	/* Check for mouse presses. This starts tracking (left button)	*/
	/* or gives a design popup (right button)			*/
	if (e->type() == QEvent::MouseButtonPress)
	{
		QMouseEvent *me = (QMouseEvent *)e ;

		/* A left-button mouse press starts either tracking to	*/
		/* move a control, or marking to sweep an aread in a	*/
		/* block, except if the parent is dynamic, in which	*/
		/* canse movement is not applicable.			*/
		if (me->button() == QMouseEvent::LeftButton)
		{
			if (m_object->parentIsDynamic ())
			{
				m_object->getLayout ()->addSizer
				(	this,
					(me->state() & Qt::ShiftButton) != 0
				)	;
				return	true	;
			}

			if ((m_trap & SZF_LEFT) != 0)
			{
				trackStart ((QMouseEvent *)e, 0) ;
				return	true	;
			}
		}

		return	false	;
	}

	/* Double-click brings up the property dialog for the object	*/
	/* associated with the control.					*/
	if (e->type() == QEvent::MouseButtonDblClick)
	{
		if ((m_trap & SZF_DBL) != 0)
		{
			if (((QMouseEvent *)e)->button() == QMouseEvent::LeftButton)
			{	m_object->propertyDlg () ;
				return	true	;
			}
		}

		return	false	;
	}

	if (e->type() == QEvent::ContextMenu)
		return	doDesignPopup ((QMouseEvent *)e) ;

#if	__KB_PLAT_MACX
	/* HACK	:							*/
	/*	We do not grab the mouse under OS/X (see comments	*/
	/*	above) so check for mode movement and button release	*/
	/*	here and, if tracking is underway, do the right thing.	*/
	if (e->type() == QEvent::MouseMove)
		if (m_tracking)
		{
			trackMove ((QMouseEvent *)e) ;
			return	true ;
		}

	if (e->type() == QEvent::MouseButtonRelease)
		if (m_tracking)
		{
			trackDone () ;
			return	true ;
		}
#endif

	/* Anything else is handled by the widget itself ...		*/
	return	false ;
}

/*  KBSizer								*/
/*  getPosition	: Get control position					*/
/*  (returns)	: QRect		: Position				*/

QRect	KBSizer::getPosition ()
{
	DPRINTF
	((	"KBSizer::getPosition: obj=%s ctl=%s\n",
		_TEXT(m_object->geometry()),
		_TEXT(m_widget->geometry())
	))	;
	/* If the object has no parent then it is virtualised on a	*/
	/* scroll view, so get the geometry from the object itself.	*/
	if (m_object->getParent() == 0)
	{
		DPRINTF
		((	"KBSizer::getPosition: top: %s\n",
			_TEXT(m_object->geometry())
		))	;
		return	m_object->geometry() ;
	}


	/* If there is an associated control, then get the geometry	*/
	/* from that. This encompases any validator flag or helper	*/
	/* button.							*/
	if (m_ctrl != 0)
	{
		QRect	r = m_ctrl->ctrlGeometry() ;
		return	m_object->attrGeom().convGeometry(r.x(), r.y(), r.width(), r.height(), false) ;
	}


	/* Otherwise, get the geometry from the widget. In this case it	*/
	/* may need converting from viewport coordinates to logical	*/
	/* values, and adjusting for report margins.			*/
	QRect	rect  = m_widget->geometry() ;
	m_display->cvtViewToCtrl (rect) ;

	KBReport *report ;
	if  ((report = m_object->getParent()->isReport()) != 0)
	{
		int	left	;
		int	top	;

		report->margins (left, top) ;

		rect.moveBy
		(	-(int)(left * pixelsPerMM()),
			-(int)(top  * pixelsPerMM())
		)	;
	}

	return	rect	;
}

/*  KBSizer								*/
/*  setBlobs	: Position sizer blobs					*/
/*  (returns)	: void		:					*/

void	KBSizer::setBlobs ()
{
	if (m_ctrl != 0)
	{
		m_ctrl->setSizerBlobs(m_btl, m_btr, m_bbl, m_bbr) ;
		return	;
	}

	QRect	rect	= getPosition() ;
	int	x	;
	int	y	;
	int	w	;
	int	h	;

	/* If there is no parent display (ie., the control is not	*/
	/* itself embedded in a display) then this must be the top	*/
	/* level block ....						*/
	if (m_object->getParent() == 0)
	{
		m_display->cvtCtrlToView (rect) ;
		x = rect.left   () ;
		y = rect.top    () ;
		w = rect.width  () ;
		h = rect.height () ;
	}
	else
	{	x = 0 ;
		y = 0 ;
		w = rect.width  () ;
		h = rect.height () ;
	}

	DPRINTF
	((	"KBSizer::setBlobs: (%d,%d,%d,%d)\n",
		x, y, w, h
	))	;
	/* This routine is typically called after the control		*/
	/* properties (which include position and size) have been	*/
	/* manually edited, and the control resized and/or repositioned	*/
	m_btl->move (x, y) ;
	m_btr->move (x + w - SZ_BLOBSIZE, 0) ;
	m_bbl->move (x, y + h - SZ_BLOBSIZE) ;
	m_bbr->move (x + w - SZ_BLOBSIZE, y + h - SZ_BLOBSIZE) ;
}

/*  KBSizer								*/
/*  setGeometry	: Set position and size of the control			*/
/*  (returns)	: void		:					*/

void	KBSizer::setGeometry
	(	QRect	pRect
	)
{
	m_widget->resize (pRect.size()) ;
}
		
/*  KBSizer								*/
/*  doResize	: Handle control resize or move				*/
/*  dx		: int		: X-change				*/
/*  dy		: int		: Y-change				*/
/*  move	: bool		: Action is move rather than resize	*/
/*  (returns)	: void		:					*/

void	KBSizer::doResize
	(	int	dx,
		int	dy,
		bool	move
	)
{
	DPRINTF
	((	"KBSizer::doResize: (%d,%d) m=%d\n",
		dx,
		dy,
		move
	))	;

	if ((dx != 0) || (dy != 0))
		m_altered = true ;

	if (move)
	{
		m_object->setGeometry
		(	QRect
			(	m_startCX + dx,
				m_startCY + dy,
				m_startW,
				m_startH
			)
		)	;
		return	;
	}

	KBFramer *framer ;
	if ((framer = m_object->isFooter()) != 0) dy = -dy ;
			

	/* If changing size then check that we are not going below the	*/
	/* minimum size.						*/
	int	nw = m_startW + dx ;
	int	nh = m_startH + dy ;

	if (nw < m_minSize.width ()) nw = m_minSize.width () ;
	if (nh < m_minSize.height()) nh = m_minSize.height() ;

	DPRINTF
	((	"KBSizer::doResize: call obj->setGeom(%d,%d,%d,%d)\n",
		m_startCX,
		m_startCY,
		nw,
		nh
	))	;
	m_object->setGeometry
	(	QRect
		(	m_startCX,
			m_startCY,
			nw,
			nh
		)
	)	;
}

/*  KBSizer								*/
/*  setState	: Set sizer state					*/
/*  state	: KBSizerState	: New state				*/
/*  (returns)	: void		:					*/

void	KBSizer::setState
	(	KBSizerState	state
	)
{
	QColor	color	= state == sbActive   ? sbColActive   :
			  state == sbLeader   ? sbColLeader   :
			  state == sbFollower ? sbColFollower : sbColIdle ;

	m_btl->setPalette (color) ;
	m_btr->setPalette (color) ;
	m_bbl->setPalette (color) ;
	m_bbr->setPalette (color) ;

	/* Note the initial control position and size so that we can	*/
	/* calculate new positions and sizes if this is the start of a	*/
	/* move or resize.						*/
	QRect area	= getPosition() ;
	m_startCX	= area.left  () ;
	m_startCY	= area.top   () ;
	m_startW 	= area.width () ;
	m_startH	= area.height() ;
}

/*  KBSizer								*/
/*  accept	: Save values back to associated object			*/
/*  doSnap	: bool		: Forciply snap to grid			*/
/*  (returns)	: void		:					*/

void	KBSizer::accept
	(	bool	doSnap
	)
{
	extern	void	snapRect (QRect &) ;
	QRect	cRect	= getPosition ()   ;
	if (doSnap || (m_altered && KBOptions::snappingOn()))
		snapRect (cRect) ;


	m_tracking	= false	;
	m_object->setGeometry (cRect) ;
	m_object->setChanged  ()  ;

	m_altered	= false ;
}

/*  KBSizer								*/
/*  snapToGrid	: Snap position to grid					*/
/*  (returns)	: bool		:					*/

void	KBSizer::snapToGrid ()
{
	accept	(true)	;
}
