/***************************************************************************
    file	         : kb_sizer.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_SIZER_H
#define		__KB_SIZER_H

#if		! __KB_RUNTIME

#include 	<qcursor.h>
#include 	<qwidget.h>
#include 	<qguardedptr.h>

#include	"kb_classes.h"
#include	"kb_limit.h"

#include	"libkbase_exports.h"

class	KBDisplay	;
class	KBSizer		;
class	KBSizerBlob	;
class	KBDispWidget	;
class	KBDispScroller	;

struct	LIBKBASE_API	KBSizerInfo	
{
	KBObject	*m_proxy  ;	/* Proxy object for sizer blob	*/
	uint		m_flags	  ;	/* Resize/movement flags	*/
	QCursor		*m_cursor ;	/* Cursor when in blob		*/
}	;

struct	LIBKBASE_API	KBSizerInfoSet
{
	uint		m_trap	;	/* Mouse event trap flags	*/
	KBSizerInfo	m_tl	;	/* Top-left blob info		*/
	KBSizerInfo	m_tr	;	/* Top-right ...		*/
	KBSizerInfo	m_bl	;	/* Bottom-left ...		*/
	KBSizerInfo	m_br	;	/* Bottom-right			*/
}	;

#define	SZF_X		0x0001		/* Size/move in X direction	*/
#define	SZF_Y		0x0002		/* Size/move in Y direction	*/
#define	SZF_MOVE	0x0010		/* Move flag			*/

#define	SZF_LEFT	0x0100		/* Handle left mouse clicks	*/
#define	SZF_RIGHT	0x0200		/* Handle left mouse clicks	*/
#define	SZF_DBL		0x0400		/* Handle left mouse clicks	*/

#define	SZ_BLOBSIZE	6




/*  KBSizerBlob								*/
/*  -----------								*/
/*  This class is used for the marker blobs for resizing and moving	*/
/*  controls.								*/

class	LIBKBASE_API	KBSizerBlob : public QWidget
{
	Q_OBJECT

	KBObject	*m_object	;
	QCursor		m_cursor	;
	uint		m_flags		;
	
public	:

	KBSizerBlob (QWidget *, KBObject*, uint, QCursor *) ;

	virtual	void		move		  (int, int) ;
	virtual	void		mousePressEvent   (QMouseEvent *) ;
	virtual	void		mouseMoveEvent	  (QMouseEvent *) ;
	virtual	void		mouseReleaseEvent (QMouseEvent *) ;

	inline	QCursor		&getCursor 	  () { return m_cursor ; }
	inline	uint		getFlags	  () { return m_flags  ; }
}	;


typedef	QGuardedPtr<KBSizerBlob> KBBlobPtr ;


/*  KBSizer								*/
/*  -------								*/
/*  The KBSizer class is used to allow positioning and sizing of	*/
/*  form controls.							*/

class	LIBKBASE_API	KBSizer : public QObject
{
	Q_OBJECT

	KBObject	*m_object   ; 	/* Object being worked upon	*/
	KBDisplay	*m_display  ;	/* Parent display		*/
	QWidget		*m_widget   ;	/* Actual widget		*/
	KBControl	*m_ctrl	    ;	/* Associated control if any	*/

	KBBlobPtr	m_btl	    ;	/* Top-left marker blob		*/
	KBBlobPtr	m_btr	    ;	/* Top-right marker blob	*/
	KBBlobPtr	m_bbl	    ;	/* Bottom-left marker blob	*/
	KBBlobPtr	m_bbr	    ;	/* Bottom-right marker blob	*/
	KBSizerBlob	*m_track    ;	/* Tracking blob		*/
	uint		m_trap	    ;	/* Mouse event trap flags	*/
	bool		m_contain   ;	/* Object is container-like	*/

	int		m_startMX   ;
	int		m_startMY   ;
	int		m_startCX   ;
	int		m_startCY   ;
	int		m_startW    ;
	int		m_startH    ;

	bool		m_tracking  ;
	bool		m_altered   ;
	QSize		m_minSize   ;	/* Minimum size of object	*/
	KBLimit		m_limit	    ;	/* Movement limits		*/

	void	init	(KBSizerInfoSet *) ;

protected :

	bool	eventFilter (QObject *, QEvent *) ;

public	:

	KBSizer
	(	KBObject	*,
		KBDisplay	*,
		QWidget		*,
		KBSizerInfoSet 	* = 0
	)	;

	KBSizer
	(	KBObject	*,
		KBDisplay	*,
		KBControl	*,
		KBSizerInfoSet	* = 0
	)	;

	~KBSizer () ;

	void			trackStart	(QMouseEvent *, KBSizerBlob *) ;
	void			trackMove 	(QMouseEvent *) ;
	void			trackDone 	() ;

	void			setBlobs	() ;
	void			setGeometry	(QRect) ;
	void			doResize	(int, int, bool) ;

	void			hide		() ;
	void			show		() ;
	bool			doDesignPopup	(QMouseEvent *) ;

	enum	KBSizerState
	{	sbLeader,
		sbFollower,
		sbActive,
		sbIdle
	}	;

	void			setState	(KBSizerState) ;
	void			accept		(bool)	;
	void			snapToGrid	()	;
	QRect			getPosition	()	;

	inline	KBObject	*getObject	() { return m_object   ; }
	inline	bool		isTracking	() { return m_tracking ; }

	static	void		defaultInfoSet	(KBSizerInfoSet &) ;
}	;


#endif		// ! __KB_RUNTIME
#endif		// __KB_SIZER_H
