/***************************************************************************
    file	         : kb_skin.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_dom.h"
#include	"kb_skin.h"

#define		__DEBUG_SKIN	(0)
#if		__DEBUG_SKIN
#include	"kb_dprintf.h"
#endif


/*  KBSkinElement							*/
/*  KBSkinElement							*/
/*		: Constructor for a skin element			*/
/*  name	: const QString & : Element name			*/
/*  fgColor	: const QString & : Foreground colour			*/
/*  bgColor	: const QString & : Background colour			*/
/*  font	: const QString & : Font				*/
/*  (returns)	: KBSkinElement	  : Element				*/

KBSkinElement::KBSkinElement
	(	const QString	&name,
		const QString	&fgColor,
		const QString	&bgColor,
		const QString	&font
	)
	:
	m_name		(name),
	m_fgColor	(fgColor),
	m_bgColor	(bgColor),
	m_font		(font)
{
}

/*  KBSkinElement							*/
/*  KBSkinElement							*/
/*		: Constructor for a skin element			*/
/*  elem	: const QDomElement & : Definition			*/
/*  (returns)	: KBSkinElement	      : Element				*/

KBSkinElement::KBSkinElement
	(	const QDomElement	&elem
	)
	:
	m_name		(elem.attribute("name"   )),
	m_fgColor	(elem.attribute("fgcolor")),
	m_bgColor	(elem.attribute("bgcolor")),
	m_font		(elem.attribute("font"   ))
{
}

/*  KBSkinElement							*/
/*  name	: Return element name					*/
/*  (return)	: const QString & : Element name			*/

const	QString &KBSkinElement::name    () const
{
	return	m_name	  ;
}

/*  KBSkinElement							*/
/*  fgColor	: Return element foreground colour			*/
/*  (return)	: const QString & : Element foreground colour		*/

const	QString &KBSkinElement::fgColor () const
{
	return	m_fgColor ;
}

/*  KBSkinElement							*/
/*  bgColor	: Return element background colour			*/
/*  (return)	: const QString & : Element background colour		*/

const	QString &KBSkinElement::bgColor () const
{
	return	m_bgColor ;
}

/*  KBSkinElement							*/
/*  font	: Return element font					*/
/*  (return)	: const QString & : Element font			*/

const	QString &KBSkinElement::font    () const
{
	return	m_font	  ;
}

/*  KBSkinElement							*/
/*  save	: Save element definition				*/
/*  elem	: QDomElement &	: Parent element			*/
/*  (returns)	: void		:					*/

void	KBSkinElement::save
	(	QDomElement	&elem
	)
	const
{
	QDomElement se = elem.ownerDocument().createElement("element") ;
	se.setAttribute  ("name",    m_name   ) ;
	se.setAttribute  ("fgcolor", m_fgColor) ;
	se.setAttribute  ("bgcolor", m_bgColor) ;
	se.setAttribute  ("font",    m_font   ) ;
	elem.appendChild (se) ;
}


/*  ------------------------------------------------------------------	*/

/*  KBSkin								*/
/*  KBSkin	: Constructor for skin object				*/
/*  (returns)	: KBSkin	:					*/

KBSkin::KBSkin ()
{
	m_elements.setAutoDelete (true) ;
}

/*  KBSkin								*/
/*  KBSkin	: Constructor for skin object				*/
/*  skin	: const QDomelement & : Definition			*/
/*  (returns)	: KBSkin	      :					*/

KBSkin::KBSkin
	(	const QDomElement	&skin
	)
{
	DPRINTF	(("KBSkin::KBSkin: QDomElement: called\n")) ;

	m_name	= skin.attribute("name") ;
	m_elements.setAutoDelete(true  ) ;

	DOMITER_BEGIN(skin,"element",elem)
	{
		m_elements.insert
		(	elem.attribute    ("name"),
			new KBSkinElement (elem  )
		)	;
	}
	DOMITER_END
}

/*  KBSkinElement							*/
/*  fgColor	: Return foreground colour of named element		*/
/*  name	: const QString & : Element name			*/
/*  (return)	: const QString & : Element foreground colour		*/

const	QString	&KBSkin::fgColor
	(	const QString	&elem
	)
	const
{
	KBSkinElement *se = m_elements.find(elem) ;
	return	se == 0 ? QString::null : se->fgColor() ;
}

/*  KBSkin								*/
/*  bgColor	: Return background colour of named element		*/
/*  name	: const QString & : Element name			*/
/*  (return)	: const QString & : Element background colour		*/

const	QString	&KBSkin::bgColor
	(	const QString	&elem
	)
	const
{
	KBSkinElement *se = m_elements.find(elem) ;
	return	se == 0 ? QString::null : se->bgColor() ;
}

/*  KBSkin								*/
/*  font	: Returnfont of named element				*/
/*  name	: const QString & : Element name			*/
/*  (return)	: const QString & : Element font			*/

const	QString	&KBSkin::font
	(	const QString	&elem
	)
	const
{
	KBSkinElement *se = m_elements.find(elem) ;
	return	se == 0 ? QString::null : se->font() ;
}

/*  KBSkin								*/
/*  save	: Save  definition					*/
/*  elem	: QDomElement &	: Main elemen	t			*/
/*  (returns)	: void		:					*/

void	KBSkin::save
	(	QDomElement	&elem
	)
	const
{
	elem.setAttribute ("name", m_name) ;

	for
	(	QDictIterator<KBSkinElement> iter(m_elements) ;
		iter.current() != 0 ;
		++iter
	)
		iter.current()->save (elem) ;
}

/*  KBSkin								*/
/*  setName	: Set skin name						*/
/*  name	: const QString & : Name				*/
/*  (returns)	: void		  :					*/

void	KBSkin::setName
	(	const QString	&name
	)
{
	m_name	= name	;
}

/*  KBSkin								*/
/*  add		: Add an element					*/
/*  name	: const QString & : Name				*/
/*  fgColor	: const QString & : Foreground colour			*/
/*  bgColor	: const QString & : Background colour			*/
/*  font	: const QString & : Font				*/
/*  (returns)	: void		  :					*/

void	KBSkin::add
	(	const QString	&name,
		const QString	&fgColor,
		const QString	&bgColor,
		const QString	&font
	)
{
	m_elements.replace
	(	name,
		new KBSkinElement (name, fgColor, bgColor, font)
	)	;
}

/*  KBSkin								*/
/*  elements	: Access elements dictionary				*/
/*  (returns)	: const QDict<KBSkinElement> & : Elements		*/

const QDict<KBSkinElement>
	&KBSkin::elements () const
{
	DPRINTF
	((	"KBSkin::elements: has %d entries\n",
		m_elements.count()
	))	;

	return	m_elements ;
}

