/***************************************************************************
    file	         : kb_skindlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qcolor.h>
#include	<qtable.h>
#include	<qfontmetrics.h>
#include	<qpainter.h>
#include	<qcursor.h>

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_pushbutton.h"

#include	"kb_classes.h"
#include	"kb_font.h"
#include	"kb_popupmenu.h"
#include	"kb_skin.h"
#include	"kb_prompt.h"
#include	"kb_notifier.h"
#include	"kb_dprintf.h"

#include	"kb_skindlg.moc"

#include	"tk_colordialog.h"
#include	"tk_fontdialog.h"
#include	"tk_messagebox.h"

#define		__DEBUG_SKINDLG	(0)
#if		__DEBUG_SKINDLG
#include	"kb_dprintf.h"
#endif


/*  hexColorToName							*/
/*		: Map hex colour value to n ame				*/
/*  hex		: const QString & : Hex value				*/
/*  (return)	: QString	  : Name or hex value if no name	*/

static	QString	hexColorToName
	(	const QString	&hex
	)
{
	static	QDict<QString>	*hexColorMap ;

	/* Construct a lookup dictionary on demand. Note that we do not	*/
	/* map names like "blue1" or "green3", ie., those that end in a	*/
	/* digit).							*/
	if (hexColorMap == 0)
	{
		hexColorMap = new QDict<QString> ;

		QStringList names = QColor::colorNames() ;
		for (uint idx = 0 ; idx < names.count() ; idx += 1)
		{	const QString &name = names[idx] ;
			if (!name.at(name.length()-1).isDigit())
				hexColorMap->insert (QColor(name).name().mid(1).lower(), new QString(name)) ;
		}
	}

	if (hex.isEmpty())
		return	hex ;

	QString	*name	= hexColorMap->find(hex.mid(2).lower()) ;
	return	name == 0 ? hex : *name ;
}


/*  KBSkinColorItem							*/
/*  ---------------							*/
/*  QTableItem-derived class used to display colours as names where a	*/
/*  name can be mapped.							*/

class	KBSkinColorItem : public QTableItem
{
	QString		m_hex		;
	QString		m_name		;

public	:

	KBSkinColorItem	(QTable *, const QString &)	;

	QString		hex		()	const	;
	virtual	QString	text		()	const	;

	virtual	void	setText		(const QString &) ;
}	;


/*  KBSkinColorItem							*/
/*  KBSkinColorItem							*/
/*		: Constructor for table colour item			*/
/*  parent	: QTable *	   : Parent table			*/
/*  text	: const QString &  : Colour text			*/
/*  (returns)	: KBSkinColourItem :					*/

KBSkinColorItem::KBSkinColorItem
	(	QTable		*parent,
		const QString	&text
	)
	:
	QTableItem	(parent, QTableItem::Never)
{
	setText		(text)	;
}

/*  KBSkinColorItem							*/
/*  hex		: Return item value as colour hex value			*/
/*  (returns)	: QString	: Hex value				*/

QString	KBSkinColorItem::hex  () const
{
	return	m_hex	;
}

/*  KBSkinColorItem							*/
/*  text	: Return displayed text					*/
/*  (returns)	: QString	: Displayed text			*/

QString	KBSkinColorItem::text () const
{
	return	m_name	;
}

/*  KBSkinColorItem							*/
/*  setText	: Set colour text					*/
/*  text	: const QString & : Colour text				*/
/*  (returns)	: void		  :					*/

void	KBSkinColorItem::setText
	(	const QString	&text
	)
{
	m_hex		= text	;
	m_name		= hexColorToName(text) ;
}


/*  ------------------------------------------------------------------	*/

/*  KBSkinTable								*/
/*  -----------								*/
/*  Wrapper around QTable used for skin editing table			*/

class	KBSkinTable : public QTable
{
	int		m_defHeight	;

public	:

	KBSkinTable	(QWidget *)	;

	void		setRow		(const QString &, const QString &, const QString &, const QString &, int) ;
	void		addRow		(const QString &, const QString &, const QString &, const QString &) ;
	void		fixRowHeight	(int)	;
	void		syncSettings	()	;
	virtual void	paintCell	(QPainter *, int, int, const QRect &, bool, const QColorGroup &) ;
}	;


/*  KBSkinTable								*/
/*  KBSkinTable	: Constructor for skin dialog table			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  (returns)	: KBSkinTable	:					*/

KBSkinTable::KBSkinTable
	(	QWidget		*parent
	)
	:
	QTable	(parent)
{
	setNumCols	(5)	;
	setFocusStyle	(QTable::SpreadSheet) ;
	setSelectionMode(QTable::Single	    ) ;

	horizontalHeader()->setLabel	(0, TR("Element"   )) ;
	horizontalHeader()->setLabel	(1, TR("Foreground")) ;
	horizontalHeader()->setLabel	(2, TR("Background")) ;
	horizontalHeader()->setLabel	(3, TR("Font"	   )) ;
	horizontalHeader()->setLabel	(4, TR("Sample"	   )) ;

	m_defHeight = -1 ;
}

/*  KBSkinTable								*/
/*  setRow	: Set values in row					*/
/*  name	: const QString & : Name				*/
/*  fgColor	: const QString & : Foreground colour			*/
/*  bgColor	: const QString & : Background colour			*/
/*  font	: const QString & : Font				*/
/*  row		: int		  : Row number				*/
/*  (returns)	: void		  :					*/

void	KBSkinTable::setRow
	(	const QString	&name,
		const QString	&fgcolor,
		const QString	&bgcolor,
		const QString	&font,
		int		row
	)
{
	QTableItem	*iName	= new QTableItem	(this, QTableItem::WhenCurrent, name    ) ;
	QTableItem	*iFG	= new KBSkinColorItem	(this,			     	fgcolor ) ;
	QTableItem	*iBG	= new KBSkinColorItem	(this,			     	bgcolor ) ;
	QTableItem	*iFont	= new QTableItem	(this, QTableItem::Never,    	font    ) ;
	QTableItem	*iSample= new QTableItem	(this, QTableItem::Never,    	"Rekall") ;

	setItem (row, 0, iName  )	;
	setItem (row, 1, iFG    )	;
	setItem (row, 2, iBG    )	;
	setItem (row, 3, iFont  )	;
	setItem (row, 4, iSample)	;

	fixRowHeight (row)		;
}

/*  KBSkinTable								*/
/*  addRow	: Add row to table					*/
/*  name	: const QString & : Name				*/
/*  fgColor	: const QString & : Foreground colour			*/
/*  bgColor	: const QString & : Background colour			*/
/*  font	: const QString & : Font				*/
/*  (returns)	: void		  :					*/

void	KBSkinTable::addRow
	(	const QString	&name,
		const QString	&fgcolor,
		const QString	&bgcolor,
		const QString	&font
	)
{
	int	row	= numRows() ;
	setNumRows (row + 1)	    ;

	DPRINTF
	((	"KBSkinTable::addRow: [%s]\n",
		name.latin1()
	))	;

	setRow	(name, fgcolor, bgcolor, font, row) ;
}

/*  KBSkinTable								*/
/*  fixRowHeight: Fix row height after font is changed			*/
/*  row		: int		: Row nmumber				*/
/*  (returns)	: void		:					*/

void	KBSkinTable::fixRowHeight
	(	int		row
	)
{
	QString	fontStr	= text (row, 3)	;

	if (m_defHeight < 0)
		m_defHeight = rowHeight (0) ;

	if (!fontStr.isEmpty())
	{
		QFontMetrics fm (KBFont::specToFont (fontStr)) ;
		setRowHeight (row, fm.height()) ;
		return	;
	}

	setRowHeight (row, m_defHeight) ;
}

/*  KBSkinTable								*/
/*  syncSettings: Sync current cell into table				*/
/*  (returns)	: void		:					*/

void	KBSkinTable::syncSettings ()
{
	int	crow	   = currentRow   () ;
	int	ccol	   = currentColumn() ;

	if ((crow >= 0) && (ccol >= 0))
		setCellContentFromEditor (crow, ccol) ;
}

/*  KBSkinTable								*/
/*  paintCell	: Paint table cell					*/
/*  p		: QPainter *	: Painter being used			*/
/*  row		: int		: Row nmumber				*/
/*  col		: int		: Column nmumbe				*/
/*  cr		: const QRect &	: Area being painted			*/
/*  selected	: bool		: True of cell is selected		*/
/*  cg		: const QColorGroup &					*/
/*				: Colour group for paint		*/
/*  (returns)	: void		:					*/

void	KBSkinTable::paintCell
	(	QPainter		*p,
		int			row,
		int			col,
		const QRect		&cr,
		bool			selected,
		const QColorGroup	&cg
	)
{
	if (col != 4)
	{
		QTable::paintCell (p, row, col, cr, selected, cg) ;
		return	;
	}

	QTableItem	*fg	= item(row, 1) ;
	QTableItem	*bg	= item(row, 2) ;
	QTableItem	*font	= item(row, 3) ;

	if ((fg == 0) || (bg == 0) || (font == 0))
	{
		QTable::paintCell (p, row, col, cr, selected, cg) ;
		return	;
	}

	QColorGroup	useCG	= cg		;
	QString		fgStr	= ((KBSkinColorItem *)fg)->hex() ;
	QString		bgStr	= ((KBSkinColorItem *)bg)->hex() ;
	QString		fontStr	= font->text()	;
	QFont		oldFont	= p   ->font()	;

	if (!fgStr.isEmpty())
		useCG.setColor (QColorGroup::Text, QColor(QRgb(fgStr.toInt(0,0)))) ;
	if (!bgStr.isEmpty())
		useCG.setColor (QColorGroup::Base, QColor(QRgb(bgStr.toInt(0,0)))) ;

	if (!fontStr.isEmpty())
		p->setFont (KBFont::specToFont (fontStr)) ;

	QTable::paintCell (p, row, col, cr, selected, useCG) ;
	p->setFont (oldFont) ;
}

/*  ------------------------------------------------------------------	*/

/*  KBSkinDlg								*/
/*  KBSkinDlg	: Constructor for skin dialog class			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  allowSave	: bool		: Allow save button			*/
/*  allowSaveAs	: bool		: Allow save as button			*/
/*  (returns)	: KBSkinDlg	:					*/

KBSkinDlg::KBSkinDlg
	(	QWidget			*parent,
		const KBLocation	&location,
		bool			allowSave,
		bool			allowSaveAs
	)
	:
	KBDialog	(TR("Skin"), true, "skindlg"),
	m_location	(location)
{
	RKVBox	*layMain	= new RKVBox (this) ;
	layMain->setTracking ()	;

	m_table	= new KBSkinTable (layMain) ;

	RKHBox	*layButt	= new RKHBox (layMain) ;
	layButt->addFiller () ;

	RKPushButton *s  = new RKPushButton (TR("Save"),	layButt) ;
	RKPushButton *sa = new RKPushButton (TR("Save As ..."), layButt) ;
	RKPushButton *d  = new RKPushButton (TR("Done"),	layButt) ;

	if (!m_location.name().isEmpty())
		loadFromLocation () ;

	connect
	(	s,
		SIGNAL	(clicked()),
		SLOT	(save   ())
	)	;
	connect
	(	sa,
		SIGNAL	(clicked()),
		SLOT	(saveAs ())
	)	;
	connect
	(	d,
		SIGNAL	(clicked()),
		SLOT	(accept ())
	)	;

	if (!allowSave  )
		s ->setEnabled (false) ;
	if (!allowSaveAs)
		sa->setEnabled (false) ;

	/* The context menu is displayed for normal context menu events	*/
	/* and also for double clicks.					*/
	connect
	(	m_table,
		SIGNAL	(contextMenuRequested (int, int, const QPoint &)),
		SLOT	(contextMenu	      (int, int))
	)	;
	connect
	(	m_table,
		SIGNAL	(doubleClicked	      (int, int, int, const QPoint &)),
		SLOT	(contextMenu	      (int, int))
	)	;
	connect
	(	m_table,
		SIGNAL	(valueChanged	      (int, int)),
		SLOT	(fixupRows	      ())
	)	;
}

void	KBSkinDlg::load
	(	const KBSkin	&skin
	)
{
	for
	(	QDictIterator<KBSkinElement> iter(skin.elements()) ;
		iter.current() != 0 ;
		++iter
	)
	{
		const KBSkinElement *elem = iter.current() ;

		m_table->addRow
		(	elem->name   (),
			elem->fgColor(),
			elem->bgColor(),
			elem->font   ()
		)	;
	}
}


void	KBSkinDlg::loadFromLocation ()
{
	KBError		error	;
	QDomDocument	sXML	= m_location.contentsAsDom (TR("Skin"), error) ;
	if (sXML.isNull())
	{	error.DISPLAY()	;
		return	;
	}
	QDomElement	root	= sXML.documentElement() ;
	KBSkin		skin	(root)	;
	load	(skin)	;
#if	0
	for
	(	QDictIterator<KBSkinElement> iter(skin.elements()) ;
		iter.current() != 0 ;
		++iter
	)
	{
		const KBSkinElement *elem = iter.current() ;

		m_table->addRow
		(	elem->name   (),
			elem->fgColor(),
			elem->bgColor(),
			elem->font   ()
		)	;
	}
#endif
}

void	KBSkinDlg::save
	(	KBSkin		&skin
	)
{
	skin.setName (m_location.name()) ;

	for (int row = 0 ; row < m_table->numRows() ; row += 1)
		if (!m_table->text(row,0).isEmpty())
			skin.add
			(	m_table->text(row,0),
				((KBSkinColorItem *)m_table->item(row,1))->hex(),
				((KBSkinColorItem *)m_table->item(row,2))->hex(),
				m_table->text(row,3)
			)	;
}

void	KBSkinDlg::saveToLocation ()
{
	KBSkin	 skin  ;
	save	(skin) ;

#if	0
	skin.setName (m_location.name()) ;

	for (int row = 0 ; row < m_table->numRows() ; row += 1)
		if (!m_table->text(row,0).isEmpty())
			skin.add
			(	m_table->text(row,0),
				((KBSkinColorItem *)m_table->item(row,1))->hex(),
				((KBSkinColorItem *)m_table->item(row,2))->hex(),
				m_table->text(row,3)
			)	;
#endif

	KBDomDocument	sXML	("skin") ;
	QDomElement	root	= sXML.documentElement() ;
	skin   .save (root) ;

	KBError		error	;
	if (!m_location.save
		(	QString::null,
			QString::null,
			sXML.toString(),
			error
		))
			error.DISPLAY() ;

	KBNotifier::self()->nSkinChanged (m_location) ;
}

/*  KBSkinDlg								*/
/*  validate	: Validate current settings				*/
/*  (returns)	: bool		: Settings OK				*/

bool	KBSkinDlg::validate ()
{
	bool	anyUnset   = false ;
	bool	anyUnnamed = false ;

	m_table->syncSettings ()   ;

	for (int row = 0 ; row < m_table->numRows() ; row += 1)
	{
		if (m_table->text(row,0).isEmpty())
		{
			anyUnnamed |= 
				!m_table->text(row,1).isEmpty() ||
				!m_table->text(row,2).isEmpty() ||
				!m_table->text(row,3).isEmpty() ;
			continue ;
		}

		anyUnset |= 
			m_table->text(row,1).isEmpty() &&
			m_table->text(row,2).isEmpty() &&
			m_table->text(row,3).isEmpty() ;
	}

	if (anyUnnamed)
		if (TKMessageBox::questionYesNo
			(	0,
				TR("Some entries are unnamed and will be dropped: save anyway ...?"),
				TR("Skin: Save anyway ...")
			)
			!= TKMessageBox::Yes ) return false ;

	if (anyUnset)
		if (TKMessageBox::questionYesNo
			(	0,
				TR("Some entries have no settings: save anyway ...?"),
				TR("Skin: Save anyway ...")
			)
			!= TKMessageBox::Yes ) return false ;

	return	true ;
}

/*  KBSkinDlg								*/
/*  save	: Terminate dialog with ::Save return code		*/
/*  (returns)	: void		:					*/

void	KBSkinDlg::save ()
{
	if (validate())
		saveToLocation () ;
}

/*  KBSkinDlg								*/
/*  save	: Terminate dialog with ::SaveAs return code		*/
/*  (returns)	: void		:					*/

void	KBSkinDlg::saveAs ()
{
	if (validate())
	{
		QString	name 	;
		QString	locn	= m_location.server() ;

		if (!doPromptSave
			(	TR("Save skin as ..."),
				TR("Enter skin name" ),
				name,
				locn,
				m_location.dbInfo(),
				true
			))
			return	;

		m_location.setServer	(locn)	;
		m_location.setName	(name)	;
		saveToLocation		()	;
	}
}

/*  KBSkinDlg								*/
/*  contextMenu	: Display context menu					*/
/*  row		: int		: Row number				*/
/*  col		: int		: Column number				*/
/*  (returns)	: void		:					*/

void	KBSkinDlg::contextMenu
	(	int		row,
		int		col
	)
{
	/* The menu always shows the same entries, but they are		*/
	/* enabled or disabled according to circumstance.		*/
	m_row	= row	;
	m_col	= col	;

	KBPopupMenu	popup (this, 0) ;
	popup.setTitle	 (m_table->horizontalHeader()->label(col)) ;
	popup.insertEntry
	(	col == 0,
		TR("&Edit"  ),
		this,
		SLOT(edit  ())
	)	;
	popup.insertEntry
	(	false,
		TR("&Clear" ),
		this,
		SLOT(clear ())
	)	;
	popup.insertEntry
	(	false,
		TR("&Insert"),
		this,
		SLOT(insert())
	)	;
	popup.insertEntry
	(	row >= m_table->numRows() - 1,
		TR("&Remove"),
		this,
		SLOT(remove())
	)	;
	popup.exec	 (QCursor::pos()) ;
}

/*  KBSkinDlg	:							*/
/*  edit	: Edit selected cell					*/
/*  (returns)	: void		:					*/

void	KBSkinDlg::edit ()
{
	switch (m_col)
	{
		case 1	:
		case 2	:
			{
			TKColorDialog cDialog (this, TR("Colour"), true) ;
			cDialog.setColor (QColor (((KBSkinColorItem *)m_table->item(m_row, m_col))->hex().toInt(0,0))) ;
			if (cDialog.exec ())
			{	QString	res	;
				res.sprintf	      ("0x%06x", cDialog.color().rgb() & 0x00ffffff) ;
				m_table->item	      (m_row, m_col)->setText(res) ;
				m_table->updateCell   (m_row, 4) ;
			}
			break	;
			}

		case 3	:
			{
			TKFontDialog  fDialog (this, TR("Font"), false, true) ;
			fDialog.setFont (KBFont::specToFont (m_table->text(m_row, m_col))) ;
			if (fDialog.exec())
			{
				m_table->setText      (m_row, m_col, KBFont::fontToSpec (fDialog.font())) ;
				m_table->fixRowHeight (m_row) ;
				m_table->updateCell   (m_row, 4) ;
			}
			break	;
			}

		default	:
			break	;
	}
}

/*  KBSkinDlg	:							*/
/*  clear	: Clear selected cell					*/
/*  (returns)	: void		:					*/

void	KBSkinDlg::clear ()
{
	switch (m_col)
	{
		case 0	:
			m_table->setText (m_row, m_col, QString::null)	;
			break	;

		case 1	:
		case 2	:
			m_table->item         (m_row, m_col)->setText (QString::null) ;
			m_table->updateCell   (m_row, 4) ;
			break	;

		case 3	:
			m_table->setText      (m_row, m_col, QString::null)	;
			m_table->fixRowHeight (m_row) ;
			m_table->updateCell   (m_row, 4) ;
			break	;

		default	:
			break	;
	}
}

/*  KBSkinDlg	:							*/
/*  insert	: Insert a row						*/
/*  (returns)	: void		:					*/

void	KBSkinDlg::insert ()
{
	m_table->insertRows (m_row) ;
	m_table->setRow	    ("", "", "", "", m_row) ;
}

/*  KBSkinDlg	:							*/
/*  remove	: Remove selected row					*/
/*  (returns)	: void		:					*/

void	KBSkinDlg::remove ()
{
	m_table->removeRow  (m_row) ;
}

/*  KBSkinDlg	:							*/
/*  fixupRows	: Fix up rows to maintain final empty row		*/
/*  (returns)	: void		:					*/

void	KBSkinDlg::fixupRows ()
{
	int	lastRow	= m_table->numRows() - 1 ;

	if (lastRow < 0)
	{
		m_table->addRow	("", "", "", "") ;
		return	;
	}

	if
	(	!m_table->item(lastRow, 0)->text().isEmpty() ||
		!m_table->item(lastRow, 1)->text().isEmpty() ||
		!m_table->item(lastRow, 2)->text().isEmpty() ||
		!m_table->item(lastRow, 3)->text().isEmpty()
	)
	{
		m_table->addRow	("", "", "", "") ;
		return	;
	}
}

int	KBSkinDlg::exec ()
{
	fixupRows () ;
	return	KBDialog::exec () ;
}

const KBLocation
	&KBSkinDlg::location ()
{
	return	m_location ;
}
