/***************************************************************************
    file	         : kb_slot.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"kb_slot.moc"
#else
#include	"kb_slot.h"
#endif

#include	"kb_object.h"
#include	"kb_docroot.h"
#include	"kb_script.h"
#include	"kb_appptr.h"
#include	"kb_callback.h"
#include	"kb_emitter.h"
#include	"kb_slotnotifier.h"
#include	"kb_event.h"


#define		__DEBUG_SLOT	(0)
#if		__DEBUG_SLOT
#include	"kb_dprintf.h"
#endif


#include	<qlistview.h>

/*  KBSlotLink								*/
/*  KBSlotLink	: Constructor for a slot linkage			*/
/*  (returns)	: KBSlotLink	:					*/

KBSlotLink::KBSlotLink ()
{
}

/*  KBSlotLink								*/
/*  KBSlotLink	: Constructor for a slot linkage			*/
/*  name	: const QString & : Link name				*/
/*  target	: const QString & : Target object			*/
/*  event	: const QString & : Event on target object		*/
/*  enabled	: bool		  : Slot enabled			*/
/*  (returns)	: KBSlotLink	  :					*/

KBSlotLink::KBSlotLink
	(	const QString	&name,
		const QString	&target,
		const QString	&event,
		bool		enabled
	)
	:
	m_name	 (name),
	m_target (target),
	m_event	 (event),
	m_enabled(enabled)
{
}


/*  ------------------------------------------------------------------  */

/*  KBSlot								*/
/*  KBSlot	: Constructor for slot					*/
/*  parent	: KBNode *	  : Parent node				*/
/*  name	: const QString & : Slot name				*/
/*  l2		: bool		  : Second language			*/
/*  (returns)	: KBSlot	  :					*/

KBSlot::KBSlot
	(	KBNode		*parent,
		const QString	&name,
		bool		l2
	)
	:
	m_parent(parent),
	m_name	(name),
	m_l2	(l2)
{
	if (m_parent != 0) m_parent->addSlot (this) ;

	m_disable = false ;
	m_script  = 0	  ;
}

/*  KBSlot								*/
/*  KBSlot	: Constructor for slot					*/
/*  parent	: KBNode *	: Parent object				*/
/*  extant	: KBSlot *	: Extant slot to be copied		*/
/*  (returns)	: KBSlot	:					*/

KBSlot::KBSlot
	(	KBNode		*parent,
		KBSlot		*extant
	)
	:
	m_parent(parent)
{
	if (m_parent != 0) m_parent->addSlot (this) ;

	m_linkage = extant->m_linkage ;
	m_name	  = extant->m_name    ;
	m_code	  = extant->m_code    ;
	m_l2	  = extant->m_l2      ;

	m_disable = false ;
	m_script  = 0	  ;
}

/*  KBSlot								*/
/*  setParent	: Set slot's parent object				*/
/*  parent	: KBNode *	: Parent				*/
/*  (returns)	: void		:					*/

void	KBSlot::setParent
	(	KBNode		*parent
	)
{
	/* This method is typically used when the user OK's on a	*/
	/* property dialog, and updated slots are returned to the	*/
	/* parent.							*/
	if ((m_parent = parent) != 0) m_parent->addSlot (this) ;
}

/*  KBSlot								*/
/*  addLinkage	: Add a linkage element					*/
/*  name	: const QString & : Link name				*/
/*  target	: const QString & : Target object			*/
/*  event	: const QString & : Event on target object		*/
/*  enabled	: bool		  : Slot enabled			*/
/*  (returns)	: void		  :					*/

void	KBSlot::addLinkage
	(	const QString	&name,
		const QString	&target,
		const QString	&event,
		bool		enabled
	)
{
	m_linkage.append (KBSlotLink (name, target, event, enabled)) ;
}

/*  KBSlot								*/
/*  setCode	: Set code associated with slot				*/
/*  code	: const QString & : Code				*/
/*  append	: bool		  : Append code else replace		*/
/*  (returns)	: void		  :					*/

void	KBSlot::setCode
	(	const QString	&code,
		bool		append
	)
{
	if (append)
	{
		m_code	+= code	;
		return	;
	}

	m_code	= code	;
}

/*  KBSlot								*/
/*  tidy	: Tidy up after loading					*/
/*  (returns)	: void		:					*/

void	KBSlot::tidy ()
{
	/* This is just so we can output the code with a leading 	*/
	/* newline in the XML document.					*/
	m_code	= m_code.stripWhiteSpace() + "\n" ;
}

/*  KBSlot								*/
/*  printNode	: Print as node in XML document				*/
/*  text	: QString &	: Text buffer				*/
/*  indent	: int		: Indent level				*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		  :					*/

void	KBSlot::printNode
	(	QString		&text,
		int		indent,
		bool		flat
	)
{
	text	+= QString("%1<slot").arg("", indent) ;
	KBAttr::addAttrText (text, "name", m_name) ;
	KBAttr::addAttrText (text, "l2",   m_l2 ? "1" : "0") ;
	text	+= ">\n" ;

	for (uint idx = 0 ; idx < m_linkage.count() ; idx += 1)
	{
		const KBSlotLink &link = m_linkage[idx] ;

		text	+= QString("%1<slotlink").arg("", indent + 2) ;

		KBAttr::addAttrText (text, "name",   link.name  (), true) ;
		KBAttr::addAttrText (text, "target", link.target(), true) ;
		KBAttr::addAttrText (text, "event",  link.event (), true) ;

		text	+= QString(" enabled=\"%1\"").arg(link.enabled()) ;
		text	+= "/>\n" ;
	}

	text	+= QString("%1<slotcode>\n" ).arg("", indent + 2) ;
	text	+= KBAttr::escapeText (m_code, false) ;
	text	+= QString("%1</slotcode>\n").arg("", indent + 2) ;

	text	+= QString("%1</slot>\n").arg("", indent) ;
}

/*  KBSlot								*/
/*  connectLinks: Connect links to enitters				*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		:					*/

bool	KBSlot::connectLinks
	(	KBError		&pError
	)
{
	DELOBJ(m_script)  ;
	m_disable = false ;

	/* Disconnect the slot from the slot notified mechanism. In all	*/
	/* liklihood it will not be attached but this is the easy way.	*/
	KBSlotNotifier::self()->remove (this) ;

	for (uint idx = 0 ; idx < m_linkage.count() ; idx += 1)
	{
		const KBSlotLink &link = m_linkage[idx] ;

		if (!link.enabled())
		{
			DPRINTF
			((	"KBSlot::connectLinks: [%s:%s] DISABLED\n",
				m_name	   .latin1(),
				link.name().latin1()
			))	 ;
			continue ;
		}

		/* Special case is the target is "getNotifier()" in	*/
		/* which case connect the slot to the slot notifier.	*/
		/* The link event is used as the notifier event name.	*/
		if (link.target() == "getNotifier()")
		{
			KBSlotNotifier::self()->add (m_parent, this, link.event()) ;
			continue ;
		}

		KBNode	*target  = m_parent->getNamedNode
					(	link.target(),
				   		pError,
				   		false
					)	;
		if (target  == 0)
		{
			pError	= KBError
				  (	KBError::Error,
					TR("Target object not found for slot link"),
					QString(TR("Source: %1.%2\nTarget: %3.%4"))
						.arg(m_parent->getPath())
						.arg(link.name  ())
						.arg(link.target())
						.arg(link.event ()),
					__ERRLOCN
				  )	;
			return	false ;
		}

		KBEmitter  *emitter = target->getEmitter (link.event()) ;
		if (emitter == 0)
		{
			pError	= KBError
				  (	KBError::Error,
					TR("Target event not found for slot link"),
					QString(TR("Source: %1.%2\nTarget: %3.%4"))
						.arg(m_parent->getPath())
						.arg(link.name  ())
						.arg(link.target())
						.arg(link.event ()),
					__ERRLOCN
				  )	;
			return	false	;
		}

		if (!connect
			(	emitter,
				SIGNAL(eventSignal(KBObject *, const QString &, uint, KBValue *, KBScriptError *&, int)),
				SLOT  (eventSignal(KBObject *, const QString &, uint, KBValue *, KBScriptError *&, int))
			)
		   )
		{
			pError	= KBError
				  (	KBError::Error,
					TR("Failed to connect to event emitter"),
				  	QString("%1[%2]")
						.arg(link.target())
						.arg(link.event ()),
					__ERRLOCN
				  )	;
			return	false	;
		}

		DPRINTF
		((	"KBSlot::connectLink: [%s] [%s/%s]\n",
			m_name		 .latin1(),
			target->getName().latin1(),
			link.event()	 .latin1()
		))	;
	}

	return	true	;
}

/*  KBSlot								*/
/*  eventSignal	: Event handler for the slot				*/
/*  source	: KBObject *	   : Originating object			*/
/*  event	: const QString &  : Originators event name		*/
/*  argc	: uint		   : Number of arguments		*/
/*  argv	: KBValue *	   : Vector of arguments		*/
/*  resval	: KBValue &	   : Return script result		*/
/*  rc		: KBScriptError *& : Error or null if OK		*/
/*  language	: int		   : 1->L1 2->L2 0->Any			*/
/*  (returns)	: void		   :					*/

void	KBSlot::eventSignal
	(	KBObject	*source,
		const QString	&event,
		uint		argc,
		KBValue		*argv,
		KBValue		&resval,
		KBScriptError	*&eRC,
		int		language
	)
{
	/* NOTE: The "eRC" return is *only* set on error, having been	*/
	/* initialised by the caller to value "KB::ScriptOK". This can	*/
	/* then be detected so that event/slot execution can be aborted	*/
	/* as soon as any slot throws an error.				*/
	if (eRC != 0)
		return	;

	DPRINTF
	((	"KBSlot::eventSignal: [%s][%s]->[%s][%s] [%d/%p] l=%d\n%s\n",
		(cchar *)source  ->getAttrVal("name"),
		(cchar *)event,
		(cchar *)m_parent->getAttrVal("name"),
		(cchar *)m_name,
		argc,
		(void  *)argv,
		(cchar *)m_code,
		language
	))	;

	if (!m_parent->showingData())
	{
		DPRINTF
		((	"KBSlot::eventSignal: parent not showing data"
		))	;
		return	;
	}

	switch (language)
	{
		case 1 :
			if ( m_l2) return ;
			break	;

		case 2 :
			if (!m_l2) return ;
			break	;

		default	:
			break	;
	}

	KBDocRoot	*root	  = m_parent->getDocRoot    ()	;
	KBScriptIF	*scrIface = m_l2 ?
					root->loadScripting2 (eRC) :
					root->loadScripting  (eRC) ;
	KBCallback	*cb	  = KBAppPtr::getCallback()	;

	if (cb != 0)
		cb->logEvent
		(	"Slot",
			m_parent->className (),
			m_parent->getAttrVal("name"),
			m_name,
			argc,
			argv
		)	;

	if (scrIface == 0)
		return	;

	/* If the event is flagged as disabled then there has been an	*/
	/* earlier error ...						*/
	if (m_disable)
	{
		eRC	= new KBScriptError
			  (	KBError
				(	KBError::Error,
					TR("Slot has been disabled due to earlier error"),
					QString(TR("Trying to execute slot %1.%2"))
						.arg(m_parent->getAttrVal("name"))
						.arg(m_name),
					__ERRLOCN
				),
				this
			  )	;
		return	;
	}

	/* Now on to explicit events, where we compile the code and	*/
	/* keep a copy. First see if the code has already been compiled	*/
	/* and if not then so so now.					*/
	if (m_script == 0)
	{
		KBError	error	;
		QString	ident	= QString("%1.%2")
					.arg(m_parent->getPath())
					.arg(name())
					;

		DPRINTF
		((	"KBSlot::eventSignal: compiling code .......\n"
		))	;

		if ((m_script = scrIface->compileFunc
				(	m_parent,
					KBAttr::substitute (m_code, m_parent->getDocRoot()),
					ident,
					"slotFunc",
					m_parent->getDocRoot()->getImports(),
					0,
					error
				) ) == 0)
		{
			/* The slot is disabled ...			*/
			m_disable = true ;
			eRC	  = new KBScriptError (error, this) ;
			return	;
		}
	}

	/* Code was or has been compiled successfully, in which case	*/
	/* execute it. Check for an execution time error ....		*/
	KBScript::ExeRC	exerc = m_script->execute
			        (	source,
					event,
					argc,
					argv,
					resval
			        )	;

	switch (exerc)
	{
		case KBScript::ExeAbort	:
		case KBScript::ExeTest	:
			eRC	= new KBScriptError () ;
			return	;


		case KBScript::ExeError	:
		case KBScript::ExeFail	:
			{
			QString	   errMsg   ;
			uint	   errLno   ;
			QString	   errText  ;
			KBLocation errLocn  = scrIface->lastError (errMsg, errLno, errText) ;

			m_disable = true ;

			KBEvent::Origin origin = KBEvent::errorOrigin
						 (	m_parent->getRoot(),
							errLocn
						 )	;

			if (origin == KBEvent::OriginScript)
			{
				eRC   = new KBScriptError
					(	KBError
						(	KBError::Error,
							errMsg,
							errText,
							__ERRLOCN
						),
						m_parent,
						errLocn,
						errText,
						errLno,
						exerc == KBScript::ExeFail
					)	;
				return	;
			}

			if (origin == KBEvent::OriginLocal )
			{
				eRC   = new KBScriptError
					(	KBError
						(	KBError::Error,
							errMsg,
							errText,
							__ERRLOCN
						),
						m_parent->getRoot()->getAttr("local")->isEvent()
					)	;
				return	;
			}

			eRC   = new KBScriptError
				(	KBError
					(	KBError::Error,
						errMsg,
						errText,
						__ERRLOCN
					),
					this
				)	;
			return	;
			}

		default	:
			break	;
	}

	/* All OK. As noted above, the "eRC" value is *not* set under	*/
	/* this circumstance.						*/
}

/*  KBSlot								*/
/*  eventSignal	: Event handler for the slot				*/
/*  source	: KBObject *	   : Originating object			*/
/*  event	: const QString &  : Originators event name		*/
/*  argc	: uint		   : Number of arguments		*/
/*  argv	: KBValue *	   : Vector of arguments		*/
/*  resval	: KBValue &	   : Return script result		*/
/*  rc		: KBScriptError *& : Error or null if OK		*/
/*  language	: int		   : 1->L1 2->L2 0->Any			*/
/*  (returns)	: void		   :					*/

void	KBSlot::eventSignal
	(	KBObject	*source,
		const QString	&event,
		uint		argc,
		KBValue		*argv,
		KBScriptError	*&rc,
		int		language
	)
{
	/* This is the Qt-level slot method. Pass a dummy result value	*/
	/* to the real routine since no results are returned when slots	*/
	/* are triggered from events.					*/
	KBValue	resval	;
	eventSignal	(source, event, argc, argv, resval, rc, language) ;
}

void	KBSlot::displayLinks
	(	QListViewItem	*parent
	)
{
	while (parent->firstChild() != 0)
		delete	parent->firstChild() ;

	for (uint idx = 0 ; idx < m_linkage.count() ; idx += 1)
		new QListViewItem
		(	parent,
			QString::null,
			m_linkage[idx].name  (),
			m_linkage[idx].target(),
			m_linkage[idx].event ()
		)	;
}
