/***************************************************************************
    file	         : kb_slotdlg.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_SLOTDLG_H
#define	__KB_SLOTDLG_H


#include	"kb_classes.h"
#include	"kb_dialog.h"
#include	"kb_slot.h"

#include	"rk_vbox.h"
#include	"rk_hbox.h"

class	KBSlotLinkItem	;
class	RKComboBox	;
class	RKLineEdit	;
class	RKPushButton	;
class	QListView	;
class	QCheckBox	;
class	KBTextEdit	;


/*  KBSlotBaseDlg							*/
/*  -------------							*/
/*  This dialog is used to manipulate a particular slot. This is the	*/
/*  base dialog; the KBSlotDlg class wraps it as an actual dialog.	*/

class	LIBKBASE_API	KBSlotBaseDlg : public RKVBox
{
	Q_OBJECT

	KBSlot		*m_slot		;
	KBNode		*m_node		;

	RKLineEdit	*m_eSlotName	;
	RKComboBox	*m_cLanguage	;
	RKComboBox	*m_cLinkPicker	;
	RKLineEdit	*m_eLinkName	;
	RKLineEdit	*m_eLinkTarget	;
	RKLineEdit	*m_eLinkEvent	;
	QCheckBox	*m_cLinkEnabled	;
	RKPushButton	*m_bFind	;
	RKPushButton	*m_bAddLink	;
	RKPushButton	*m_bEditLink	;
	RKPushButton	*m_bDismissLink	;
	RKPushButton	*m_bSaveLink	;
	RKPushButton	*m_bDropLink	;
	KBTextEdit	*m_editor	;

	RKPushButton	*m_bOK		;
	RKPushButton	*m_bCancel	;

	KBSlotLinkItem	*m_currentItem	;
	bool		m_changed	;

	QString		m_language	;
	QString		m_language2	;

signals	:

	void		slotChanged	()	;

protected slots :

	void		switchLanguage	()	;
	void		clickFind	()	;
	void		clickAdd	()	;
	void		clickEdit	()	;
	void		clickSave	()	;
	void		clickDismiss	()	;
	void		clickDrop	()	;
	void		linkSelected	(int)	;

public	:

	KBSlotBaseDlg	(QWidget *, KBSlot *, KBNode *)	;

	bool		doVerify	()	;
	bool		doOK		()	;
	bool		doCancel	()	;

	void		setOKButton	(RKPushButton *) ;
	void		setCancelButton	(RKPushButton *) ;
}	;


/*  KBSlotDlg								*/
/*  ---------								*/
/*  Actual slot dialog, wrapped around KBSlotBaseDlg which does all the	*/
/*  real work.								*/

class	LIBKBASE_API	KBSlotDlg : public KBDialog
{
	Q_OBJECT

	KBSlotBaseDlg	*m_slotDlg	;
	RKPushButton	*m_bVerify	;
	RKPushButton	*m_bOK		;
	RKPushButton	*m_bCancel	;

protected slots :

	void		clickVerify	()	;
	void		clickOK		()	;
	void		clickCancel	()	;

public	:

	KBSlotDlg	(KBSlot	*, KBNode *)	;
}	;



/*  KBSlotListDlg							*/
/*  -------------							*/
/*  This dialog (which is actually derived from an RKHBox and is	*/
/*  intended to be embedded in a real dialog) is used to select amongst	*/
/*  a list of slots.							*/

class	LIBKBASE_API	KBSlotListDlg : public RKHBox
{
	Q_OBJECT

	KBNode		*m_node		;

	QListView	*m_lvSlotList	;
	RKPushButton	*m_bAddSlot	;
	RKPushButton	*m_bEditSlot	;
	RKPushButton	*m_bDropSlot	;

	QString		m_openSlot	;

protected slots :

	void	clickAddSlot	()	;
	void	clickEditSlot	()	;
	void	clickDropSlot	()	;
	void	highlighted	()	;
	void	slotOpenSlot	()	;

public	:

	KBSlotListDlg	(QWidget *, QPtrList<KBSlot> &, KBNode *) ;

	void	openSlot	(const QString	  &)	;
	void	save		(QPtrList<KBSlot> &)	;
}	;


#endif	// __KB_SLOTDLG_H
