/***************************************************************************
    file	         : kb_spinbox.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_sizer.h"
#include	"kb_display.h"
#include	"kb_ctrl.h"
#include	"kb_formblock.h"

#if		__KB_AUTOMOC
#include	"kb_spinbox.moc"
#else
#include	"kb_spinbox.h"
#endif

#include	"kb_qtspinbox.h"
#include	"kb_nodereg.h"
#include	"kb_writer.h"

#if		! __KB_RUNTIME
#include	"kb_itemdlg.h"
#endif


/*  KBSpinBox								*/
/*  KBSpinBox	: Constructor for simple form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBNode	:					*/

KBSpinBox::KBSpinBox
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBItem		(parent, "KBSpinBox",	"expr",  aList),
	m_fgcolor	(this,   "fgcolor",	aList),
	m_bgcolor	(this,   "bgcolor",	aList),
	m_nullOK	(this,   "nullok",	aList, KAF_FORM  ),
	m_morph		(this,   "morph",	aList, KAF_FORM  ),
	m_onChange	(this,   "onchange",	aList, KAF_FORM|KAF_EVCS)
{
#if	! __KB_RUNTIME
	if (ok != 0)
	{
		KBItemPropDlg iDlg (this, "Spinbox", m_attribs) ;
		if (!iDlg.exec())
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}

#endif
}

/*  KBSpinBox								*/
/*  KBSpinBox	: Constructor for simple form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  spinbox	: KBSpinBox *	: Extant field				*/
/*  (returns)	: KBSpinBox	:					*/

KBSpinBox::KBSpinBox
	(	KBNode		*parent,
		KBSpinBox	*spinbox
	)
	:
	KBItem		(parent, "expr",	spinbox),
	m_fgcolor	(this,   "fgcolor",	spinbox),
	m_bgcolor	(this,   "bgcolor",	spinbox),
	m_nullOK	(this,   "nullok",	spinbox, KAF_FORM  ),
	m_morph		(this,   "morph",	spinbox, KAF_FORM  ),
	m_onChange	(this,   "onchange",	spinbox, KAF_FORM|KAF_EVCS)
{
}

/*  KBSpinBox								*/
/*  ~KBSpinBox	: Destructor for simple form field			*/
/*  (returns)	:		:					*/

KBSpinBox::~KBSpinBox ()
{
}

/*  KBSpinBox								*/
/*  canBeInvalid: Check if control can be invalid			*/
/*  (returns)	: bool		: True if so				*/

bool	KBSpinBox::canBeInvalid ()
{
	return	! m_nullOK.getBoolValue() ;
}

/*  KBSpinBox								*/
/*  makeCtrl	: Make a field control					*/
/*  morph	: bool		: Get morped control			*/
/*  (returns)	: KBControl *	: New control				*/

KBControl
	*KBSpinBox::makeCtrl
	(	uint		drow
	)
{
	return	new KBCtrlSpinBox (getDisplay(), this, drow) ;
}

/*  KBSpinBox								*/
/*  isMorphed	: Test if field is morphed				*/
/*  (returns)	: bool		: Morphed				*/

bool	KBSpinBox::isMorphing ()
{
	return	m_morph.getBoolValue() ;
}

/*  KBSpinBox								*/
/*  replicate	: Replicate this field					*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New field node			*/

KBNode	*KBSpinBox::replicate
	(	KBNode		*parent
	)
{
	return	new KBSpinBox (parent, this) ;
}

#if	! __KB_RUNTIME
/*  KBSpinBox								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar *	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBSpinBox::propertyDlg
	(	cchar	*iniAttr
	)
{
	if (KBItemPropDlg(this, "Spinbox", m_attribs, iniAttr).exec())
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}
#endif

NEWNODE(SpinBox, __TR("New &Spinbox"), 0, KNF_FORM|KNF_BLOCK|KNF_DATA)
