/***************************************************************************
    file	         : kb_stack.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	<qcursor.h>
#include	<qlabel.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_display.h"
#include	"kb_sizer.h"
#include	"kb_popupmenu.h"
#include	"kb_designpopup.h"
#include	"kb_nodereg.h"
#include	"kb_formcopier.h"
#include	"kb_layout.h"
#include	"kb_attrdict.h"
#include	"kb_writer.h"
#include	"kb_ascii.h"

#if		! __KB_RUNTIME
#include	"kb_framerdlg.h"
#include	"kb_recorder.h"
#endif

#if		__KB_AUTOMOC
#include 	"kb_stack.moc"
#else
#include 	"kb_stack.h"
#endif

#include	"tk_icons.h"


/*  KBStackPage								*/
/*  KBStackPage	: Constructor for Stack page node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  element	: cchar *	: Element name				*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBFramer	:					*/

KBStackPage::KBStackPage
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		cchar			*element,
		bool			*ok
	)
	:
	KBFramer 	(parent, aList, element, ok)
{
	m_geom.set	(0, 0, 0, 0) ;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMStretch) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideW|KBAttrGeom::HideH|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;

#if	! __KB_RUNTIME
	if (ok != 0)
	{
		if (!::framerPropDlg (this, m_attribs, 0))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif
}

/*  KBStackPage								*/
/*  KBStackPage	: Consructor for Stack page node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  Stackpage	: KBStackPage *: Extant page				*/
/*  (returns)	: KBStackPage	:					*/

KBStackPage::KBStackPage
	(	KBNode	  	*parent,
		KBStackPage 	*page
	)
	:
	KBFramer	  (parent, page)
{
	m_geom.set	(0, 0, 0, 0) ;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMStretch) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideW|KBAttrGeom::HideH|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;
}

/*  KBStackPage								*/
/*  ~KBStackPage: Destructor for Stack page node			*/
/*  (returns)	:		:					*/

KBStackPage::~KBStackPage ()
{
}

#if	! __KB_RUNTIME

/*  KBStackPage								*/
/*  designPopup	: Handle design menu request				*/
/*  parent	: KBPopupMenu *	: Possible parent menu			*/
/*  cell	: QRect		: Position information			*/
/*  (returns)	: KBPopupMenu *	: Popup menu				*/

KBPopupMenu
	*KBStackPage::designPopup
	(	KBPopupMenu	*parent,
		QRect		cell
	)
{
	KBPopupMenu  *popupMain	= new KBPopupMenu (parent, &m_bState) ;
	KBPopupMenu  *popupEdit	= 0 ;

	if (parent == 0)
		popupEdit = makeContainerEditPopup
				(	popupMain,
					this,
					TR("stack page"),
					false
				)	;

	makeContainerMainPopup
	(	popupMain,
		this,
		TR("Stack page"),
		!isDynamic() || (objectInCell (cell) == 0) ?
			makeNewPopup (popupMain, cell) :
			0,
		popupEdit
	)	;

	KBStack	*stack	= getParent()->isStack() ;
	if (stack != 0) stack->raiserMenu (popupMain) ;

	return	popupMain ;
}

/*  KBStackPage								*/
/*  makeRecordPopup							*/
/*		: Construct recording popup menu			*/
/*  popup	: KBPopupMenu *	: Popup menu to add to			*/
/*  drow	: uint		: Display row number			*/
/*  parents	: bool		: Add submenus for parents		*/
/*  (returns)	: void		:					*/

void	KBStackPage::makeRecordPopup
	(	KBPopupMenu		*popup,
		uint			drow,
		bool			parents
	)
{
}
#endif

/*  KBStackPage								*/
/*  replicate	: Replicate this Stack page				*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New framer node			*/

KBNode	*KBStackPage::replicate
	(	KBNode	*parent
	)
{
	return	replicateBelow (new KBStackPage (parent, this)) ;
}


/*  KBStackPage								*/
/*  setGeometry	: Set position in parent				*/
/*  rect	: const QRect &	: Position				*/
/*  (returns)	: void		:					*/

void	KBStackPage::setGeometry
	(	const QRect	&
	)
{
	/* Totally ignore this. The page geometry is completely locked	*/
	/* to the parent.						*/
}

/*  KBStackPage								*/
/*  geometry	: Get page geometry					*/
/*  (returns)	: QRect		: Geometry				*/

QRect	KBStackPage::geometry ()
{
	m_geom.set (0, 0, 0, 0) ;
	return	KBFramer::geometry() ;
}

/*  KBStackPage								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBStackPage::showAs
	(	KB::ShowAs	mode
	)
{
#if	! __KB_RUNTIME
	if ((mode == KB::ShowAsDesign) && (getSizer() == 0))
	{
		static	QCursor	cSizeVer  (SizeVerCursor) ;
		extern	QCursor	cNoCursor ;

		KBSizerInfoSet	infoSet	;
		KBSizer::defaultInfoSet (infoSet) ;

		infoSet.m_bl.m_proxy  = parentObject () ;
		infoSet.m_br.m_proxy  = parentObject () ;
		infoSet.m_tl.m_flags  = 0		;
		infoSet.m_tl.m_cursor = &cNoCursor	;
		infoSet.m_tr.m_flags  = 0		;
		infoSet.m_tr.m_cursor = &cNoCursor	;

		setSizer
		(	new KBSizer
			(	this,
				getDisplay(),
				m_frmDisp->getDisplayWidget(),
				&infoSet
		)	)	;
	}
#endif
	KBFramer::showAs (mode) ;
}

void	KBStackPage::setCurrent ()
{
	if (getParent()->isStack() != 0)
		getParent()->isStack()->setCurrentPage (this) ;
}


/*  ------------------------------------------------------------------	*/

/*  KBStack								*/
/*  KBStack	: Constructor for stack node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  element	: cchar *	: Element name				*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBStack	:					*/

KBStack::KBStack
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBFramer	(parent, aList, "KBStack", ok),
	m_initPage	(this,	 "initpage", aList)
{
#if	! __KB_RUNTIME
	if (ok != 0)
	{
		if (!::framerPropDlg (this, m_attribs, 0))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif

	m_currentPage = 0	;
}

/*  KBStack								*/
/*  KBStack	: Consructor for Stack node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  stack	: KBStack *	: Extant stack				*/
/*  (returns)	: KBStack	:					*/

KBStack::KBStack
	(	KBNode	  *parent,
		KBStack  *stack
	)
	:
	KBFramer	(parent, stack),
	m_initPage	(this,	 "initpage", stack)
{
	m_currentPage = 0	;
}

/*  KBStack								*/
/*  KBStack	: Desructor for stack node				*/
/*  (returns)	:		:					*/

KBStack::~KBStack ()
{
}

/*  KBStack								*/
/*  replicate	: Replicate this stack					*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New Stack node			*/

KBNode	*KBStack::replicate
	(	KBNode	*parent
	)
{
	return	replicateBelow (new KBStack (parent, this)) ;
}

/*  KBStack								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBStack::showAs
	(	KB::ShowAs	mode
	)
{
	KBFramer::showAs (mode) ;

	KBStackPage	*defPage = 0 ;
	QString		initPage = m_initPage.getValue() ;

	CITER
	(	StackPage,
		page,

		if (!initPage.isEmpty() && (page->getName() == initPage))
		{
			defPage	= page	;
			break	;
		}

		if (defPage == 0)
			defPage	= page	;
	)

	if (defPage != 0) setCurrentPage (defPage) ;
}

/*  KBStack								*/
/*  setCurrentPage							*/
/*		: Set current page					*/
/*  page	: int		: Page, actually KBStackPage *		*/
/*  (returns)	: void		:					*/

void	KBStack::setCurrentPage
	(	int		page
	)
{
	setCurrentPage ((KBStackPage *)page) ;
}

/*  KBStack								*/
/*  setCurrentPage							*/
/*		: Set current page					*/
/*  page	: KBStackPage *	: Page					*/
/*  (returns)	: void		:					*/

void	KBStack::setCurrentPage
	(	KBStackPage	*page
	)
{
//	fprintf
//	(	stderr,
//		"KBStack::pageSelected: page %p selected\n",
//		(void *)page
//	)	;

	CITER
	(	StackPage,
		f,
		if (f == page) f->setVisible (true ) ;
	)
	CITER
	(	StackPage,
		f,
		if (f != page) f->setVisible (false) ;
	)

	m_currentPage = page ;
}

/*  KBStack								*/
/*  newPage	: Add new page						*/
/*  (returns)	: void		:					*/

void	KBStack::newPage ()
{
#if	! __KB_RUNTIME
	KBAttrDict aDict ;
	bool	   ok	 ;

	KBStackPage *page = new KBStackPage
			     (		this,
					aDict,
					"KBStackPage",
					&ok
			      )	;

	if (!ok) return	;

	page->buildDisplay (m_frmDisp) ;
	page->showAs       (KB::ShowAsDesign) ;
	page->getContainer ()->show() ;

	getLayout()->setChanged() ;

	setCurrentPage (page) ;
#endif
}

/*  KBStack								*/
/*  raiserMenu	: Construct menu to raise a stack page			*/
/*  parent	: KBPopupMenu *	: Parent menu				*/
/*  (returns)	: KBPopupMenu * : Raiser menu or null if no pages	*/

KBPopupMenu
	*KBStack::raiserMenu
	(	KBPopupMenu	*raiser
	)
{
//	KBPopupMenu *raiser = new KBPopupMenu (parent) ;
	uint	    count   = 1 ;

	CITER
	(	StackPage,
		p,

		QString	text	= QString ("%1: %2").arg(count).arg(p->getName()) ;
		bool	cp	= false	;

		if (m_currentPage == 0)
			cp = count == 1 ;
		else	cp = m_currentPage == p ;

		if (count == 1)
			raiser->setTitle (TR("Raise page")) ;

		count += 1 ;

		if (cp)
			raiser->insertItem
			(	getSmallIcon("selected"),
				text,
				this,
				SLOT	(setCurrentPage(int)),
				0,
				(long)p
			)	;
		else
			raiser->insertItem
			(	text,
				this,
				SLOT	(setCurrentPage(int)),
				0,
				(long)p
			)	;

	)

//	if (count == 0)
//	{
//		delete	raiser	;
//		return	0	;
//	}

	return	raiser	;
}

#if	! __KB_RUNTIME

/*  KBStack								*/
/*  designPopup	: Handle design menu request				*/
/*  parent	: KBPopupMenu *	: Possible parent popup			*/
/*  cell	: QRect		: Position information			*/
/*  (returns)	: KBPopupMenu *	: Popup menu				*/

KBPopupMenu
	*KBStack::designPopup
	(	KBPopupMenu	*parent,
		QRect		cell
	)
{
	static Qt::ButtonState	bState ;

	KBPopupMenu *popupMain	= new KBPopupMenu (parent, &bState) ;

	KBNode	    *cpyNode	= 0 	;
	bool	    noIns	= !KBFormCopier::self()->anyCopied(cpyNode) ;
	bool	    noPage	= true	;

	if (cpyNode != 0) noIns = cpyNode->isStackPage() == 0 ;

	CITER (StackPage, t, noPage = false ; break)

	popupMain->setTitle    (this) ;

	KBPopupMenu *popupEdit	= new KBPopupMenu (popupMain) ;

	popupEdit->insertEntry
	(	false,
		getSmallIcon("editcut"),
		TR("C&ut"),
	 	this,
		SLOT(cutObj   	())
	)	;
	popupEdit->insertEntry
	(	false,
		getSmallIcon("editcopy"),
		TR("&Copy"),
		this,
		SLOT(copyObj  	())
	)	;
	popupEdit->insertEntry
	(	true, //noIns, 
		TR("&Paste page"),
		this,
		SLOT(pasteObjects())
	)	;
	popupEdit->insertEntry
	(	false,
		getSmallIcon("editdel"),
		TR("&Delete"),
		this,
		SLOT(deleteObj	())
	)	;

	popupMain->insertItem
	(	getSmallIcon("newtab"),
		TR("&New Page"),
	        this,
		SLOT(newPage    ())
	)	;
	popupMain->insertItem
	(	getSmallIcon("properties"),
		TR("Stack properties"),
		this,
		SLOT(propertyDlg())
	)	;

	raiserMenu (popupMain) ;

	if ((parent == 0) && (parentObject() != 0))
		makeAncestorPopup (popupMain, this) ;

	setCtrlRect (cell) ;
	return	popupMain  ;
}

/*  KBStack								*/
/*  makeRecordPopup							*/
/*		: Construct recording popup menu			*/
/*  popup	: KBPopupMenu *	: Popup menu to add to			*/
/*  drow	: uint		: Display row number			*/
/*  parents	: bool		: Add submenus for parents		*/
/*  (returns)	: void		:					*/

void	KBStack::makeRecordPopup
	(	KBPopupMenu		*popup,
		uint			drow,
		bool			parents
	)
{
	popup->insertItem
	(	TR("Verify page"),
		this,
		SLOT(recordVerifyStack())
	)	;
}

#endif

/*  KBStack								*/
/*  recordVerifyStack							*/
/*		: Record state for verification				*/
/*  (returns)	: void		:					*/

void	KBStack::recordVerifyStack ()
{
#if	! __KB_RUNTIME
	KBRecorder *recorder = KBRecorder::self() ;
	if ((recorder != 0) && recorder->isRecording(getDocRoot()))
		recorder->verifyStack
		(	this,
			m_currentPage == 0 ? "" : m_currentPage->getName()
		)	;
#endif
}

/*  KBStack								*/
/*  currentPage	: Get current page					*/
/*  (returns)	: KBStagePage *	: Page					*/

KBStackPage
	*KBStack::currentPage ()
{
	return	m_currentPage	;
}

/*  KBStack								*/
/*  write	: Write Stack 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  offset	: QPoint	: Offset into writer			*/
/*  first	: bool		: First control flag			*/
/*  extra	: int &		: Return extra space			*/
/*  prior	: bool		: Write prior value flag		*/
/*  (returns)	: bool		: Success				*/

bool	KBStack::write
	(	KBWriter	*writer,
		QPoint		offset,
		bool		first,
		int		&extra,
		bool
	)
{
	QString	bgcolor	;
	bgcolor.sprintf	("0x%06x", m_frmDisp->getDisplayWidget()->backgroundColor().rgb() & 0x00ffffff) ;

	new KBWriterBG  (writer, geometry(offset), bgcolor) ;

#if	! __KB_RUNTIME
	if (showingDesign())
		new KBWriterBox (writer, geometry(offset)) ;
#endif

	return	true	;
}

NEWNODENAMED(StackPage, (cchar *)0, 0, KNF_FORM,   StackPage)
NEWNODE	    (Stack, __TR("New &Stack control"), 0, KNF_FORM|KNF_BLOCK|KNF_STATIC)
