/***************************************************************************
    file	         : kb_stack.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_STACK_H
#define	__KB_STACK_H

#include	"kb_classes.h"
#include	"kb_framer.h"

#include	"libkbase_exports.h"


class	KBStack		;
class	KBStackPage	;
class	KBPopupMenu	;

/*  KBStackPage								*/
/*  ------------							*/
/*  This class is used for a page under a stack. It is essentially a	*/
/*  container, except that its geometry is locked to its parent stack.	*/

class	LIBKBASE_API	KBStackPage : public KBFramer
{
	Q_OBJECT

public	:

	KBStackPage (KBNode *, const QDict<QString> &, cchar *, bool * = 0) ;
	KBStackPage (KBNode *, KBStackPage *) ;
	virtual	 ~KBStackPage () ;

	virtual	KBStackPage	*isStackPage	() { return this ; }

#if	! __KB_RUNTIME
	virtual	KBPopupMenu	*designPopup	(KBPopupMenu *, QRect)	;
	virtual	void		makeRecordPopup	(KBPopupMenu *, uint, bool) ;
#endif
	virtual	void		showAs		(KB::ShowAs)    ;
	virtual	void		setGeometry	(const QRect &)	;
	virtual	QRect		geometry	() 		;
	virtual	KBNode		*replicate	(KBNode *)	;

	void			setCurrent	()		;
}	;

/*  KBStack								*/
/*  -------								*/
/*  Rekall equivalent of a QWidgetStack.				*/


class	LIBKBASE_API	KBStack : public KBFramer
{
	Q_OBJECT

	KBAttrStr		m_initPage	;
	KBStackPage		*m_currentPage	;


protected slots :

	void			newPage			()	;
	void			setCurrentPage		(int)	;
	void			recordVerifyStack	()	;


public	:

	KBStack	(KBNode *, const QDict<QString> &, bool * = 0) ;
	KBStack	(KBNode *, KBStack *) ;
	virtual	 ~KBStack () ;

	virtual		KBStack	*isStack	() { return this ; }

	virtual		KBNode		*replicate	(KBNode *)	;
	virtual		void		showAs		(KB::ShowAs)	;
	virtual		bool		write		(KBWriter *, QPoint, bool, int &, bool = false) ;

#if	! __KB_RUNTIME
	virtual		void		makeRecordPopup	(KBPopupMenu *, uint, bool) ;
	virtual		KBPopupMenu	*designPopup	(KBPopupMenu *, QRect)	;
#endif
	void				pageSelected	(KBStackPage	*) ;
	void				setCurrentPage	(KBStackPage    *) ;

	KBPopupMenu			*raiserMenu	(KBPopupMenu *) ;

	KBStackPage			*currentPage	()	;
}	;

#endif	// __KB_STACK_H
