/***************************************************************************
    file	         : kb_summary.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qapplication.h>
#include	<qptrlist.h>
#include	<qstring.h>
#include	<qcursor.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_sizer.h"
#include	"kb_display.h"
#include	"kb_ctrl.h"

#if		__KB_AUTOMOC
#include	"kb_summary.moc"
#else
#include	"kb_summary.h"
#endif

#include	"kb_qtsummary.h"
#include	"kb_nodereg.h"

#include	"kb_writer.h"
#include	"kb_layout.h"
#include	"kb_script.h"


void	KBSummary::sumUnknown
	(	const KBValue	&
	)
{
}

void	KBSummary::sumSumInt
	(	const KBValue	&value
	)
{
	if (m_nVals == 0) m_intVal = 0 ;
	m_intValPrior = m_intVal ;
	m_intVal	+= value.getRawText().toInt() ;
}

void	KBSummary::sumSumDouble
	(	const KBValue	&value
	)
{
	if (m_nVals == 0) m_dblVal = 0 ;
	m_dblValPrior = m_dblVal ;
	m_dblVal	+= value.getRawText().toDouble() ;
}

void	KBSummary::sumMaxInt
	(	const KBValue	&value
	)
{
	int	v = value.getRawText().toInt() ;
	m_intValPrior = m_intVal ;
	if ((m_nVals == 0) || (v > m_intVal)) m_intVal = v ;
}

void	KBSummary::sumMaxDouble
	(	const KBValue	&value
	)
{
	double	v = value.getRawText().toDouble() ;
	m_dblValPrior = m_dblVal ;
	if ((m_nVals == 0) || (v > m_dblVal)) m_dblVal = v ;
}

void	KBSummary::sumMinInt
	(	const KBValue	&value
	)
{
	int	v = value.getRawText().toInt() ;
	m_intValPrior = m_intVal ;
	if ((m_nVals == 0) || (v < m_intVal)) m_intVal = v ;
}

void	KBSummary::sumMinDouble
	(	const KBValue	&value
	)
{
	double	v = value.getRawText().toDouble() ;
	m_dblValPrior = m_dblVal ;
	if ((m_nVals == 0) || (v < m_dblVal)) m_dblVal = v ;
}

void	KBSummary::sumMinString
	(	const KBValue	&value
	)
{
	QString	v = value.getRawText() ;
	m_strValPrior = m_strVal ;
	if ((m_nVals == 0) || (v < m_strVal)) m_strVal = v ;
}

void	KBSummary::sumMaxString
	(	const KBValue	&value
	)
{
	QString	v = value.getRawText() ;
	m_strValPrior = m_strVal ;
	if ((m_nVals == 0) || (v > m_strVal)) m_strVal = v ;
}

void	KBSummary::sumMinDateTime
	(	const KBValue	&value
	)
{
	const KBDateTime *dv = value   .getDateTime() ;
	if (dv == 0)
		return	;
	const KBDateTime *dc = m_datVal.getDateTime() ;
	if (dc == 0)
	{	m_datVal = value ;
		return	 ;
	}

	QDateTime   qv = dv->getDateTime() ;
	QDateTime   qc = dc->getDateTime() ;

	m_datValPrior = m_datVal ;
	if ((m_nVals == 0) || (qv < qc)) m_datVal = value ;
}

void	KBSummary::sumMaxDateTime
	(	const KBValue	&value
	)
{
	const KBDateTime *dv = value   .getDateTime() ;
	if (dv == 0)
		return	;
	const KBDateTime *dc = m_datVal.getDateTime() ;
	if (dc == 0)
	{	m_datVal = value ;
		return	 ;
	}

	QDateTime   qv = dv->getDateTime() ;
	QDateTime   qc = dc->getDateTime() ;

	m_datValPrior = m_datVal ;
	if ((m_nVals == 0) || (qv > qc)) m_datVal = value ;
}


/*  KBSummary								*/
/*  KBSummary	: Constructor for simple form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBNode	:					*/

KBSummary::KBSummary
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBItem		(parent, "KBSummary",	"expr",  aList),
	m_fgcolor	(this,   "fgcolor",	aList),
	m_bgcolor	(this,   "bgcolor",	aList),
	m_font		(this,	 "font",	aList),
	m_format	(this,	 "format",	aList),
	m_align		(this,	 "align",	aList),
	m_summary	(this,	 "summary",	aList, KAF_REQD|KAF_GRPDATA),
	m_reset		(this,	 "reset",	aList)
{
	m_nVals	= 0 ;

#if	! __KB_RUNTIME
	if (ok != 0)
	{
		if (!summaryPropDlg (this, "Summary", m_attribs))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true		;
	}
#endif
	if (getParent() != 0) m_report = getParent()->getReport () ;
}

/*  KBSummary								*/
/*  KBSummary	: Constructor for simple form field			*/
/*  parent	: KBNode *	: Parent node				*/
/*  summary	: KBSummary *	: Extant summary control		*/
/*  (returns)	: KBSummary	:					*/

KBSummary::KBSummary
	(	KBNode		*parent,
		KBSummary	*summary
	)
	:
	KBItem		(parent,  "expr",	summary),
	m_fgcolor	(this,    "fgcolor",	summary),
	m_bgcolor	(this,    "bgcolor",	summary),
	m_font		(this,	  "font",	summary),
	m_format	(this,    "format", 	summary),
	m_align		(this,	  "align",	summary),
	m_summary	(this,	  "summary",	summary, KAF_REQD|KAF_GRPDATA),
	m_reset		(this,	  "reset",	summary)
{
	m_nVals	= 0 ;
	if (getParent() != 0) m_report = getParent()->getReport () ;
}

/*  KBSummary								*/
/*  ~KBSummary	: Destructor for simple form field			*/
/*  (returns)	:		:					*/

KBSummary::~KBSummary ()
{
}

/*  KBSummary								*/
/*  makeCtrl	: Make a field control					*/
/*  drow	: uint		: Display row number			*/
/*  (returns)	: KBControl *	: New control				*/

KBControl
	*KBSummary::makeCtrl
	(	uint		drow
	)
{
	return	new KBCtrlSummary (getDisplay(), this, drow) ;
}

/*  KBSummary								*/
/*  replicate	: Replicate this field					*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New field node			*/

KBNode	*KBSummary::replicate
	(	KBNode		*parent
	)
{
	return	new KBSummary (parent, this) ;
}

/*  KBSummary								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBSummary::showAs
	(	KB::ShowAs	mode
	)
{
	m_nVals	= 0 ;
	KBItem::showAs (mode) ;
}

/*  KBSummary								*/
/*  setFieldType: Set field type from query				*/
/*  _type	: KBType *	: Query field type			*/
/*  (returns)	: void		  :					*/


void	KBSummary::setFieldType
	(	KBType	*type
	)
{
	KBItem::setFieldType (type) ;
	m_sumFunc = &KBSummary::sumUnknown ;

	switch (type->getIType())
	{
		case KB::ITFixed	:
			if	(m_summary.getValue() == "Total"  ) m_sumFunc = &KBSummary::sumSumInt    ;
			else if	(m_summary.getValue() == "Minimum") m_sumFunc = &KBSummary::sumMinInt    ;
			else if	(m_summary.getValue() == "Maximum") m_sumFunc = &KBSummary::sumMaxInt    ;
			break	;

		case KB::ITFloat	:
			if	(m_summary.getValue() == "Total"  ) m_sumFunc = &KBSummary::sumSumDouble ;
			else if	(m_summary.getValue() == "Minimum") m_sumFunc = &KBSummary::sumMinDouble ;
			else if	(m_summary.getValue() == "Maximum") m_sumFunc = &KBSummary::sumMaxDouble ;
			break	;

		case KB::ITString	:
			if	(m_summary.getValue() == "Minimum") m_sumFunc = &KBSummary::sumMinString ;
			else if	(m_summary.getValue() == "Maximum") m_sumFunc = &KBSummary::sumMaxString ;
			break	;

		case KB::ITDate		:
		case KB::ITTime		:
		case KB::ITDateTime	:
			if	(m_summary.getValue() == "Minimum") m_sumFunc = &KBSummary::sumMinDateTime ;
			else if	(m_summary.getValue() == "Maximum") m_sumFunc = &KBSummary::sumMaxDateTime ;
			break	;

		default	:
				break	;
	}

	if (m_sumFunc == &KBSummary::sumUnknown)
		KBError::EWarning
		(	"Summary field type error",
			"Invalid combination for summary and field type",
			__ERRLOCN
		)	;
}

/*  KBSummary								*/
/*  prepare	: Prepare item prior prior to display			*/
/*  (returns)	: void		:					*/

void	KBSummary::prepare ()
{
	KBObject::prepare () ;
	m_nVals	= 0 ;
}

/*  KBSummary								*/
/*  setValue	: Set value						*/
/*  qrow	: uint		  : Current query row			*/
/*  value	: const KBValue & : Value				*/
/*  (returns)	: bool		  : Success				*/

bool	KBSummary::setValue
	(	uint		,
		const KBValue	&value
	)
{
	KBScriptError	*eRC 	;
	bool		subs	;
	(this->*m_sumFunc) (m_expr.evaluate (value, eRC, subs)) ;

	if (eRC != 0)
	{
		KBScriptError::processError (eRC, KBScriptError::Normal) ;
		return	false	;
	}

	m_nVals  += 1  ;
	return	true ;
}

/*  KBSummary								*/
/*  getReportValue							*/
/*		: Get value for report writing				*/
/*  first	: bool		: First output flag			*/
/*  prior	: bool		: Return prior value flag		*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBSummary::getReportValue
	(	bool		,
		bool		prior
	)
{
	KBValue	value	;

	/* TRICKY CODE ALERT						*/
	/* See also "KBReportBlock::showData()". The "prior" flag is	*/
	/* set when printing in a footer and returns the prior value	*/
	/* rather than the current value. Used since a row of data must	*/
	/* be loaded before the report writer can decide whether a page	*/
	/* throw (and hence a footer) is needed.			*/
	if (m_sumFunc != &KBSummary::sumUnknown)
		switch (m_type->getIType())
		{
			case KB::ITFixed  	:
				value	= prior ? m_intValPrior : m_intVal ;
				break	;

			case KB::ITFloat  	:
				value	= prior ? m_dblValPrior : m_dblVal ;
				break	;

			case KB::ITString 	:
				value	= prior ? m_strValPrior : m_strVal ;
				break	;

			case KB::ITDate		:
			case KB::ITTime		:
			case KB::ITDateTime	:
				value	= prior ? m_datValPrior : m_datVal ;
				break	;

			default	:
				value	= "#Error"	;
				break	;
		}
		else	value	= "#Error" ;

	
	if (m_reset.getBoolValue()) m_nVals = 0 ;
	return	value	;
}

#if	! __KB_RUNTIME
/*  KBSummary								*/
/*  propertyDlg	: Show property dialog					*/
/*  iniAttr	: cchar *	: Initial attribute			*/
/*  (returns)	: bool		: Success				*/

bool	KBSummary::propertyDlg
	(	cchar	*iniAttr
	)
{
	if (::summaryPropDlg (this, "Summary", m_attribs, iniAttr))
	{
		setupProperties () ;
		return	true ;
	}

	return	false	;
}
#endif

NEWNODE(Summary, __TR("New &Summary"), 0, KNF_REPORT|KNF_BLOCK|KNF_FRAMER|KNF_DATA)
