/***************************************************************************
    file	         : kb_summary.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_SUMMARY_H
#define		__KB_SUMMARY_H

#include	"kb_classes.h"
#include	"kb_item.h"
#include	"kb_attralign.h"

#include	"libkbase_exports.h"



/*  KBSummary								*/
/* --- ------								*/
/*  This class is used to represent a KBSummary element in the XML of a	*/
/*  form description.							*/

class	LIBKBASE_API	KBSummary : public KBItem
{
	Q_OBJECT

	KBAttrStr	m_fgcolor	;	/* Foreground (colour	*/
	KBAttrStr	m_bgcolor	;	/* Background colour	*/
	KBAttrStr	m_font		;	/* Font specification	*/
	KBAttrStr	m_format	;	/* Format string	*/
	KBAttrAlign	m_align		;	/* Alignment		*/
	KBAttrStr	m_summary	;	/* Summary to be done	*/
	KBAttrBool	m_reset		;	/* Page reset flag	*/

	KBReport	*m_report	;	/* Embedding report	*/

	int		m_intValPrior	;
	double		m_dblValPrior	;
	QString		m_strValPrior	;
	KBValue		m_datValPrior	;

	int		m_intVal	;
	double		m_dblVal	;
	QString		m_strVal	;
	KBValue		m_datVal	;

	uint		m_nVals	;	/* Number of values		*/

	void		sumUnknown		(const KBValue &) ;
	void		sumSumInt		(const KBValue &) ;
	void		sumSumDouble		(const KBValue &) ;
	void		sumMaxInt		(const KBValue &) ;
	void		sumMaxDouble		(const KBValue &) ;
	void		sumMinInt		(const KBValue &) ;
	void		sumMinDouble		(const KBValue &) ;
	void		sumMinString		(const KBValue &) ;
	void		sumMaxString		(const KBValue &) ;
	void		sumMinDateTime		(const KBValue &) ;
	void		sumMaxDateTime		(const KBValue &) ;
	void		(KBSummary::*m_sumFunc)	(const KBValue &) ;

protected :

	virtual	KBControl 	*makeCtrl	(uint)	   ;
	virtual	void		setFieldType	(KBType *) ;

public	:

	KBSummary	(KBNode  *, const QDict<QString> &, bool * = 0) ;
	KBSummary  (KBNode  *, KBSummary *) ;
virtual~KBSummary	() ;

#if	! __KB_RUNTIME
	virtual	bool	 	propertyDlg	(cchar  *) ;
#endif
	virtual	KBNode		*replicate	(KBNode *) ;
	virtual	void		showAs		(KB::ShowAs) ;
	virtual	bool		setValue 	(uint, const KBValue &) ;
	virtual	KBValue		getReportValue	(bool, bool = false) ;
	virtual	void		prepare		() ;

	inline	QString	getFormat ()
	{
		return	m_format.getValue() ;
	}
	inline	int	getAlign  ()
	{
		return	m_align.getIntValue() ;
	}
}	;


#if	! __KB_RUNTIME
extern	LIBKBASE_API	bool	summaryPropDlg
			(	KBSummary		*,
				cchar			*,
				QPtrList<KBAttr>	&,
				cchar			* = 0
			)	;
#endif
#endif	// __KB_SUMMARY_H
