/***************************************************************************
    file	         : kb_tabber.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#include	<qlabel.h>
#include	<qcursor.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_display.h"
#include	"kb_sizer.h"
#include	"kb_popupmenu.h"
#include	"kb_designpopup.h"
#include	"kb_nodereg.h"
#include	"kb_formcopier.h"
#include	"kb_layout.h"
#include	"kb_qttabber.h"
#include	"kb_attrdict.h"
#include	"kb_writer.h"
#include	"kb_ascii.h"
#include	"kb_slot.h"

#if		! __KB_RUNTIME
#include	"kb_tabpagedlg.h"
#include	"kb_framerdlg.h"
#include	"kb_recorder.h"
#endif

#if		__KB_AUTOMOC
#include 	"kb_tabber.moc"
#else
#include 	"kb_tabber.h"
#endif

#include	"tk_icons.h"


#define		__DEBUG_TABBER	(0)
#if		__DEBUG_TABBER
#include	"kb_dprintf.h"
#endif


/*  KBTabberTab								*/
/*  -----------								*/
/*  Helper class used to record tab id/page pointer pairs, so that we	*/
/*  can map back and forth. The KBTabberBar class keeps a list of these	*/
/*  for all tabs.							*/

class	LIBKBASE_API KBTabberTab
{
public	:

	int		id	;
	KBTabberPage	*page	;

	inline	KBTabberTab
		(	int		id,
			KBTabberPage	*page
		)
		:
		id	(id),
		page	(page)
	{
	}
}	;

/*  ------------------------------------------------------------------	*/

static	int	tabBarHeight ()
{
	static	int	_tabBarHeight	;

	if (_tabBarHeight == 0)
	{
		QTabBar *tabBar = new QTabBar (0)     ;
		QTab	*tabTab	= new QTab    ("Tab") ;
		tabBar->addTab(tabTab) ;
		_tabBarHeight	= tabBar->sizeHint().height() ;
		delete	tabBar	;

		DPRINTF
		((	"tabBarHeight: height=%d\n",
			_tabBarHeight
		))	;
	}

	return	_tabBarHeight ;
}

/*  ------------------------------------------------------------------	*/

/*  KBTabberBar								*/
/*  KBTabberBar	: Consructor for tabber tab bar node			*/
/*  tabber	: KBTabber *	: Parent tabber				*/
/*  (returns)	: KBTabberBar	:					*/

KBTabberBar::KBTabberBar
	(	KBTabber  *tabber
	)
	:
	KBObject	(tabber, "KBTabberBar", QRect(0, 0, 0, 0)),
	m_tabber	(tabber)
{
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMFixed) ;
	m_geom.set	(0, 0, 0, m_tabber->tabBarHeight()) ;
	m_tabBar = 0 ;
	m_tabList.setAutoDelete (true) ;
}


/*  KBTabberBar								*/
/*  KBTabberBar	: Consructor for tabber tab bar node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  tabber	: KBTabberBar *	: Extant tabber	bar			*/
/*  (returns)	: KBTabberBar	:					*/

KBTabberBar::KBTabberBar
	(	KBNode	  	*parent,
		KBTabberBar	*tabber
	)
	:
	KBObject	(parent, tabber),
	m_tabber	(parent == 0 ? 0 : parent->isTabber())
{
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMFixed) ;
	m_geom.set	(0, 0, 0, m_tabber->tabBarHeight()) ;
	m_tabBar = 0 ;
	m_tabList.setAutoDelete (true) ;
}

/*  KBTabberBar								*/
/*  ~KBTabberBar: Desructor for tabber tab bar node			*/
/*  (returns)	:		:					*/

KBTabberBar::~KBTabberBar ()
{
}

/*  KBTabberBar								*/
/*  buildDisplay: Build GUI for this node				*/
/*  display	: KBDisplay *	: Owning display			*/
/*  (returns)	: void		:					*/

void	KBTabberBar::buildDisplay
	(	KBDisplay	*display
	)
{
	KBObject::buildDisplay (display) ;

	if (m_tabBar == 0)
	{
		m_tabBar = new KBCtrlTabberBar (display, this) ;
		setControl (m_tabBar) ;
		display->insertWidget
		(	new KBLayoutItem
			(	m_tabBar->getWidget(),
				display,
				m_geom,
				KBAttrGeom::MgmtStatic
			)
		)	;
	}

	setupProperties	() ;
}

/*  KBTabberBar								*/
/*  setupProperties							*/
/*		: Set properties for tabber bar				*/
/*  (returns)	: void		:					*/

void	KBTabberBar::setupProperties ()
{
	DPRINTF
	((	"KBTabberBar::setupProperties: tbh=%d\n",
		m_tabber->tabBarHeight()
	))	;
	setGeometry (m_geom.convGeometry (0, 0, 0, m_tabber->tabBarHeight(), false)) ;
	KBObject::setupProperties () ;
}


bool	KBTabberBar::wideTabs ()
{
	return	m_tabber->wideTabs ()	;
}

/*  KBTabberBar								*/
/*  replicate	: Replicate this tabber bar				*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New tabber node			*/

KBNode	*KBTabberBar::replicate
	(	KBNode	*_parent
	)
{
	return	new KBTabberBar (_parent, this) ;
}


/*  KBTabberBar								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBTabberBar::showAs
	(	KB::ShowAs	mode
	)
{
#if	! __KB_RUNTIME
	if ((mode == KB::ShowAsDesign) && (getSizer() == 0))
	{
		extern	QCursor	cNoCursor	  ;

		KBSizerInfoSet	infoSet		  ;
		KBSizer::defaultInfoSet (infoSet) ;

		infoSet.m_trap	      = SZF_RIGHT|SZF_DBL ;
		infoSet.m_tl.m_proxy  = parentObject () ;
		infoSet.m_tr.m_proxy  = parentObject () ;
		infoSet.m_bl.m_flags  = 0		;
		infoSet.m_bl.m_cursor = &cNoCursor	;
		infoSet.m_br.m_flags  = 0		;
		infoSet.m_br.m_cursor = &cNoCursor	;

		setSizer
		(	new KBSizer
			(	this,
				getDisplay(),
				m_tabBar->getWidget(),
				&infoSet
		)	)	;
	}

	if ((mode == KB::ShowAsData) && (getSizer() != 0))
		setSizer(0) ;
#endif

	for (int idx = 0 ; idx < m_tabBar->getNumTabs() ; idx += 1)
		m_tabBar->setTabEnabled (m_tabList.at(idx)->id, true) ;

	/* Pass the request on down to the base class, which will in	*/
	/* turn recurse down to any children.				*/
	KBObject::showAs (mode) ;
}

#if	! __KB_RUNTIME

/*  KBTabberBar								*/
/*  designPopup	: Handle design menu request				*/
/*  parent	: KBPopupMenu *	: Possible parent menu			*/
/*  cell	: QRect		: Cell for dynamic layouts		*/
/*  (returns)	: KBPopupMenu *	: Popup menu				*/

KBPopupMenu
	*KBTabberBar::designPopup
	(	KBPopupMenu	*parent,
		QRect		cell
	)
{
	return	m_tabber->designPopup (parent, cell) ;
}

void	KBTabberBar::makeRecordPopup
	(	KBPopupMenu	*popup,
		uint		drow,
		bool		parents
	)
{
	m_tabber->makeRecordPopup (popup, drow, parents) ;
}

#endif

/*  KBTabberBar								*/
/*  getNumTabs	: Get the number of tabs				*/
/*  (returns)	: uint		: Number of tabs			*/

uint	KBTabberBar::getNumTabs ()
{
	return	m_tabBar->getNumTabs () ;
}

/*  KBTabberBar								*/
/*  getCurrentTab: Get current tab					*/
/*  page	 : KBTabberPage *& : Current page			*/
/*  (returns)	 : int		   : Tab identifier or negative if none	*/

int	KBTabberBar::getCurrentTab
	(	KBTabberPage	*&page
	)
{
	int	id = m_tabBar->getCurrentTab () ;

	LITER
	(	KBTabberTab,
		m_tabList,
		t,

		if (t->id == id)
		{	page	= t->page	;
			return	id		;
		}
	)

	page	= 0 ;
	return	-1  ;
}

/*  KBTabberBar								*/
/*  setCurrentTab: Set current by bay page				*/
/*  page	 : KBTabberPage & : Required page			*/
/*  (returns)	 : void		  :					*/

void	KBTabberBar::setCurrentTab
	(	KBTabberPage	*page
	)
{
	LITER
	(	KBTabberTab,
		m_tabList,
		t,

		if (t->page == page)
		{	m_tabBar->setCurrentTab (t->id) ;
			return	;
		}
	)
}

/*  KBTabberBar								*/
/*  tabSelected	: User selects a tab					*/
/*  id		: int		: Identifier for selected tab		*/
/*  (returns)	: void		:					*/

void	KBTabberBar::tabSelected
	(	int		id
	)
{
	DPRINTF
	((	"KBTabberBar::tabSelected: user selects tab %d\n",
		id
	))	;

	LITER
	(	KBTabberTab,
		m_tabList,
		t,

		if (t->id == id)
		{	m_tabber->tabSelected (t->page) ;
			return	;
		}
	)
}

/*  KBTabberBar								*/
/*  addTab	: Add a new tab						*/
/*  label	: const QString & : Text for tab			*/
/*  page	: KBTabberPage *  : Associated tab page			*/
/*  adding	: bool		  : New tab is being added		*/
/*  (returns)	: int		  : New tab identifier			*/

int	KBTabberBar::addTab
	(	const QString	&label,
		KBTabberPage	*page,
		bool		adding
	)
{
	int	id = m_tabBar->addTab (label) ;
	if (id < 0) return -1 ;

	m_tabList.append (new KBTabberTab (id, page)) ;

	if (adding)
	{
		m_tabBar->getWidget()->adjustSize () ;
		m_tabBar->setCurrentTab (id)	;
		setGeometry (geometry())	;
	}

	m_geom.set (0, 0, 0, m_tabBar->getWidget()->sizeHint().height()) ;
	return	id	;
}

void	KBTabberBar::pagesInOrder
	(	QPtrList<KBTabberPage>	&pages
	)
{
	LITER
	(	KBTabberTab,
		m_tabList,
		t,

		pages.append (t->page)
	)
}

/*  KBTabberBar								*/
/*  setTabText	: Set text for specified tab				*/
/*  label	: const QString & : Text for tab			*/
/*  page	: KBTabberPage *  : Associated tab page			*/
/*  (returns)	: int		  : New tab identifier			*/

void	KBTabberBar::setTabText
	(	const QString	&label,
		KBTabberPage	*page
	)
{
	LITER
	(	KBTabberTab,
		m_tabList,
		t,

		if (t->page == page)
		{	m_tabBar->setTabText  (t->id, label) ;
			m_tabBar->getWidget()->repaint(true) ;
			return	;
		}
	)
}

/*  KBTabberBar								*/
/*  removeTab	: Remove tab corresponding to page			*/
/*  page	: KBNode *	: Page in question			*/
/*  (returns)	: void		:					*/

void	KBTabberBar::removeTab
	(	KBNode		*page
	)
{
	LITER
	(	KBTabberTab,
		m_tabList,
		t,

		if (t->page == page)
		{	m_tabBar->removeTab (t->id) ;
			m_tabList.remove    (t)	  ;
			break	 ;
		}
	)

	m_tabBar->getWidget()->repaint (true) ;

	if (m_tabList.count() > 0)
	{
		m_tabBar->setCurrentTab (m_tabList.at(0)->id  ) ;
		m_tabber->tabSelected   (m_tabList.at(0)->page) ;
	}
}

/*  KBTabber								*/
/*  setPageEnabled							*/
/*		: Enable/disable a page					*/
/*  page	: KBTabberPage *: Page in question			*/
/*  enabled	: bool		: True to enable			*/
/*  (returns)	: void		:					*/

void	KBTabberBar::setPageEnabled
	(	KBTabberPage	*page,
		bool		enabled
	)
{
	if (m_tabBar != 0)
		LITER
		(	KBTabberTab,
			m_tabList,
			tab,

			if (tab->page == page)
			{	m_tabBar->setTabEnabled (tab->id, enabled) ;
				break	;
			}
		)
}

/*  KBTabberBar								*/
/*  getTabRect	: Get tab area for specified page			*/
/*  page	: KBTabberPage * : Page in question			*/
/*  (returns)	: QRect		 : Area					*/

QRect	KBTabberBar::getTabRect
	(	KBTabberPage	*page
	)
{
	LITER
	(	KBTabberTab,
		m_tabList,
		t,

		if (t->page == page)
			return	m_tabBar->getTabRect (t->id) ;
	)

	return	QRect()	;
}

/*  KBTabber								*/
/*  printPages	: Output node XML					*/
/*  text	: QString &	: Result string				*/
/*  indent	: int		: Indent level of this node		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBTabberBar::printPages
	(	QString		&text,
		int		indent,
		bool		flat
	)
{
	LITER
	(	KBTabberTab,
		m_tabList,
		tab,
		tab->page->printNode (text, indent + 2, flat)
	)
}

/*  KBTabberBar								*/
/*  currentPage	: Get current page					*/
/*  (returns)	: KBTabberPage *	: Page				*/

KBTabberPage
	*KBTabberBar::currentPage ()
{
	int	id = m_tabBar->getCurrentTab () ;

	LITER
	(	KBTabberTab,
		m_tabList,
		t,

		if (t->id == id) return t->page	;
	)

	return	0 ;
}

/*  ------------------------------------------------------------------  */


#if	! __KB_RUNTIME
static	cchar	*imageLegends[] =
{
	"Normal",
	"Rollover",
	"Active",
	"Disable",
	0
}	;
#endif

/*  KBTabberPage							*/
/*  KBTabberPage: Constructor for tabber page node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  element	: cchar *	: Element name				*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBFramer	:					*/

KBTabberPage::KBTabberPage
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		cchar			*element,
		bool			*ok
	)
	:
	KBFramer 	(parent, aList, element, ok),
	m_tabText	(this,   "tabtext",	aList, KAF_FORM),
	m_image		(this,	 "image",	aList, KAF_GRPWEB)
{
	m_geom.set	(0, tabBarHeight(), 0, 0) ;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMStretch) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideW|KBAttrGeom::HideH|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;

#if	! __KB_RUNTIME
	m_image.setNumImages (4);
	m_image.setLegends   (imageLegends) ;

	if (ok != 0)
	{
		if (!::framerPropDlg (this, m_attribs, 0))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif
}

/*  KBTabberPage							*/
/*  KBTabberPage: Consructor for tabber page node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  tabberpage	: KBTabberPage *: Extant page				*/
/*  (returns)	: KBTabberPage	:					*/

KBTabberPage::KBTabberPage
	(	KBNode	  	*parent,
		KBTabberPage 	*page
	)
	:
	KBFramer	(parent, page),
	m_tabText	(this,   "tabtext", 	page, KAF_FORM),
	m_image		(this,	 "image",	page, KAF_GRPWEB)
{
#if	! __KB_RUNTIME
	m_image.setNumImages (4);
	m_image.setLegends   (imageLegends) ;
#endif

	m_geom.set	(0, tabBarHeight(), 0, 0) ;
	m_geom.set	(KBAttrGeom::FMStretch, KBAttrGeom::FMStretch) ;
	m_geom.setMask	(KBAttrGeom::HideX|KBAttrGeom::HideY|KBAttrGeom::HideW|KBAttrGeom::HideH|KBAttrGeom::HideXMode|KBAttrGeom::HideYMode) ;
}

/*  KBTabberPage							*/
/*  ~KBTabberPage: Destructor for tabber page node			*/
/*  (returns)	:		:					*/

KBTabberPage::~KBTabberPage ()
{
}

/*  KBTabberPage							*/
/*  setupProperties							*/
/*		: Set properties for tabber page			*/
/*  (returns)	: void		:					*/

void	KBTabberPage::setupProperties ()
{
	DPRINTF
	((	"KBTabberPage::setupProperties: tbh=%d\n",
		tabBarHeight()
	))	;
	setGeometry (m_geom.convGeometry (0, tabBarHeight(), 0, 0, false)) ;
	KBFramer::setupProperties () ;
}

uint	KBTabberPage::tabBarHeight ()
{
	KBNode	*parent	= getParent() ;
	if ((parent != 0) && (parent->isTabber() != 0))
		return	parent->isTabber()->tabBarHeight() ;

	return	::tabBarHeight() ;
}

/*  KBTabberPage							*/
/*  replicate	: Replicate this tabber page				*/
/*  parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New framer node			*/

KBNode	*KBTabberPage::replicate
	(	KBNode	*parent
	)
{
	return	replicateBelow (new KBTabberPage (parent, this)) ;
}


/*  KBTabberPage							*/
/*  setGeometry	: Set position in parent				*/
/*  rect	: const QRect &	: Position				*/
/*  (returns)	: void		:					*/

void	KBTabberPage::setGeometry
	(	const QRect	&rect
	)
{
	KBFramer::setGeometry (rect) ;
}

/*  KBTabberPage							*/
/*  geometry	: Get page geometry					*/
/*  (returns)	: QRect		: Geometry				*/

QRect	KBTabberPage::geometry ()
{
	return	KBFramer::geometry() ;
}

/*  KBtabberPage							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBTabberPage::showAs
	(	KB::ShowAs	mode
	)
{
#if	! __KB_RUNTIME
	if ((mode == KB::ShowAsDesign) && (getSizer() == 0))
	{
		static	QCursor	cSizeVer  (SizeVerCursor) ;
		extern	QCursor	cNoCursor ;

		KBSizerInfoSet	infoSet	;
		KBSizer::defaultInfoSet (infoSet) ;

		infoSet.m_bl.m_proxy  = parentObject () ;
		infoSet.m_br.m_proxy  = parentObject () ;
		infoSet.m_tl.m_flags  = 0		;
		infoSet.m_tl.m_cursor = &cNoCursor	;
		infoSet.m_tr.m_flags  = 0		;
		infoSet.m_tr.m_cursor = &cNoCursor	;

		setSizer
		(	new KBSizer
			(	this,
				getDisplay(),
				m_frmDisp->getDisplayWidget(),
				&infoSet
		)	)	;
	}
#endif
	KBFramer::showAs (mode) ;
}

/*  KBTabberPage							*/
/*  propertyDlg	: Show tabber page property dialog			*/
/*  iniattr	: cchar *	: Initial attribute			*/ 
/*  (returns)	: bool		: Success				*/

bool	KBTabberPage::propertyDlg
	(	cchar		*iniAttr
	)
{
	DPRINTF
	((	"KBTabberPage::propertyDlg: called\n"
	))	;

	if (KBFramer::propertyDlg (iniAttr))
	{
		getParent()->isTabber()->setTabText (this, m_tabText.getValue()) ;
		return	true	;
	}

	return	false	;
}

/*  KBTabberPage							*/
/*  setEnabled	: Set page enabled/disabled				*/
/*  enabled	: bool		: Set enabled				*/
/*  (returns)	: void		:					*/

void	KBTabberPage::setEnabled
	(	bool	enabled
	)
{
	if (getParent()->isTabber() != 0)
		getParent()->isTabber()->setPageEnabled (this, enabled) ;

	if (m_frmDisp)
		m_frmDisp->getDisplayWidget()->setEnabled (enabled) ;
}

/*  KBTabberPage							*/
/*  setCurrent	: Set this page as current page				*/
/*  (returns)	: void		:					*/

void	KBTabberPage::setCurrent ()
{
	if (getParent()->isTabber() != 0)
		getParent()->isTabber()->setCurrentPage (this) ;
}

/*  KBTabberPage							*/
/*  printNode	: Output node XML					*/
/*  text	: QString &	: Result string				*/
/*  indent	: int		: Indent level of this node		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBTabberPage::printNode
	(	QString		&text,
		int		indent,
		bool		flat
	)
{
	KBAttrUInt *tw	= 0 ;

	if (flat)
	{
		uint	w = getParent()->isTabber()->getTabRect(this).width() ;
		tw	= new KBAttrUInt (this, "tabwidth", w) ;
	}
	
	KBFramer::printNode (text, indent, flat) ;
	if (tw != 0) delete tw	;
}

#if	! __KB_RUNTIME

/*  KBTabberPage							*/
/*  makeRecordPopup							*/
/*		: Construct recording popup menu			*/
/*  popup	: KBPopupMenu *	: Popup menu to add to			*/
/*  drow	: uint		: Display row number			*/
/*  parents	: bool		: Add submenus for parents		*/
/*  (returns)	: void		:					*/

void	KBTabberPage::makeRecordPopup
	(	KBPopupMenu		*popup,
		uint			drow,
		bool			parents
	)
{
}

#endif

/*  ------------------------------------------------------------------  */

/*  KBTabber								*/
/*  KBTabber	: Constructor for Tabber node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: List of attributes			*/
/*  element	: cchar *	: Element name				*/
/*  ok		: bool *	: Success				*/
/*  (returns)	: KBTabber	:					*/

KBTabber::KBTabber
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*ok
	)
	:
	KBFramer	(parent, aList, 	"KBTabber",	ok),
	m_initPage	(this,	 "initpage",	aList),
	m_wideTabs	(this,	 "widetabs",	aList),
	m_forceHeight	(this,	 "forceheight",	aList),
	m_onTabSelect	(this,   "ontabselect",	aList, KAF_EVCS)
{
	m_tabberBar = new KBTabberBar (this) ;

#if	! __KB_RUNTIME
	if (ok != 0)
	{
		if (!::framerPropDlg (this, m_attribs, 0))
		{	delete	this	;
			*ok	= false	;
			return	;
		}

		*ok	= true	;
	}
#endif
}

/*  KBTabber								*/
/*  KBTabber	: Consructor for Tabber node				*/
/*  parent	: KBNode *	: Parent node				*/
/*  tabber	: KBTabber *	: Extant Tabber				*/
/*  (returns)	: KBTabber	:					*/

KBTabber::KBTabber
	(	KBNode	  *parent,
		KBTabber  *tabber
	)
	:
	KBFramer	(parent, tabber),
	m_initPage	(this,	 "initpage",	tabber),
	m_wideTabs	(this,	 "widetabs",	tabber),
	m_forceHeight	(this,	 "forceheight",	tabber),
	m_onTabSelect	(this,   "ontabselect",	tabber, KAF_EVCS)
{
	m_tabberBar = new KBTabberBar (this) ;
}

/*  KBTabber								*/
/*  KBTabber	: Desructor for tabber node				*/
/*  (returns)	:		:					*/

KBTabber::~KBTabber ()
{
}

/*  KBTabber								*/
/*  recordVerifyTabber							*/
/*		: Record state for verification				*/
/*  (returns)	: void		:					*/

void	KBTabber::recordVerifyTabber ()
{
#if	! __KB_RUNTIME
	KBRecorder *recorder = KBRecorder::self() ;
	if ((recorder != 0) && recorder->isRecording(getDocRoot()))
	{
		KBTabberPage *page ;
		m_tabberBar->getCurrentTab(page) ;

		recorder->verifyStack
		(	this,
			page == 0 ? "" : page->getName()
		)	;
	}
#endif
}

#if	! __KB_RUNTIME

/*  KBTabber								*/
/*  makeRecordPopup							*/
/*		: Construct recording popup menu			*/
/*  popup	: KBPopupMenu *	: Popup menu to add to			*/
/*  drow	: uint		: Display row number			*/
/*  parents	: bool		: Add submenus for parents		*/
/*  (returns)	: void		:					*/

void	KBTabber::makeRecordPopup
	(	KBPopupMenu		*popup,
		uint			drow,
		bool			parents
	)
{
	popup->insertItem
	(	TR("Verify page"),
		this,
		SLOT(recordVerifyTabber())
	)	;

	KBFramer::makeRecordPopup (popup, drow, parents) ;
}

/*  KBTabber								*/
/*  propertyDlg	: Show tabber page property dialog			*/
/*  iniattr	: cchar *	: Initial attribute			*/ 
/*  (returns)	: bool		: Success				*/

bool	KBTabber::propertyDlg
	(	cchar		*iniAttr
	)
{
	if (!KBFramer::propertyDlg (iniAttr))
		return false ;

	CITER
	(	Object,
		o,
		o->setupProperties ()
	)	;

	return	true	;
}
#endif

/*  KBTabber								*/
/*  tabBarHeight: Return height of the tab bar				*/
/*  (returns)	: int		:					*/

int	KBTabber::tabBarHeight ()
{
	uint	height	;

	if ((height = m_forceHeight.getIntValue()) > 0)
		return	height	;

	return	::tabBarHeight () ;
}

bool	KBTabber::wideTabs ()
{
	return	m_wideTabs.getBoolValue() ;
}

/*  KBTabber								*/
/*  replicate	: Replicate this Tabber					*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New tabber node			*/

KBNode	*KBTabber::replicate
	(	KBNode	*_parent
	)
{
	return	replicateBelow (new KBTabber (_parent, this)) ;
}

/*  KBTabber								*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBTabber::showAs
	(	KB::ShowAs	mode
	)
{
	if (m_tabberBar->getNumTabs() == 0)
		CITER
		(	TabberPage,
			page,
			m_tabberBar->addTab (page->getAttrVal("tabtext"), page) ;
		)

	KBFramer::showAs (mode) ;

	KBTabberPage	*defPage = 0 ;
	QString		initPage = m_initPage.getValue() ;

	CITER
	(	TabberPage,
		page,

		if (!initPage.isEmpty() && (page->getName() == initPage))
		{
			defPage	= page	;
			break	;
		}

		if (defPage == 0)
			defPage	= page	;
	)

	if (defPage != 0)
	{ 
		tabSelected (defPage) ;
		m_tabberBar->setCurrentTab (defPage) ;
	}
}

/*  KBTabber								*/
/*  printNode	: Output node XML					*/
/*  text	: QString &	: Result string				*/
/*  indent	: int		: Indent level of this node		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBTabber::printNode
	(	QString		&text,
		int		indent,
		bool		flat
	)
{
	QString		nodeText ;

	text	+= QString("%1<%2").arg("", indent).arg(getElement()) ;

	for (uint idx = 0 ; idx < m_attribs.count () ; idx += 1)
		m_attribs.at(idx)->printAttr(text, nodeText, indent + 2, flat) ;

	if (flat)
		text	+= QString(" barheight=\"%1\"").arg(tabBarHeight()) ;

	if ((m_children.count() == 0) && (m_slotList.count() == 0))
	{
		text	+= "/>\n" ;
		return	;
	}

	text	+= ">\n" ;

	for (uint idx2 = 0 ; idx2 < m_slotList.count () ; idx2 += 1)
		m_slotList.at(idx2)->printNode (text, indent + 2, flat) ;

	/* The child printing is handled via the tabber bar, since the	*/
	/* nodes must be output in the order they appear, which may not	*/
	/* be the original order.					*/
	m_tabberBar->printPages (text, indent, flat) ;

	text	+= nodeText ;
	text	+= QString("%1</%2>\n").arg("", indent).arg(getElement()) ;
}

/*  KBTabber								*/
/*  tabSelected	: User selects a tab					*/
/*  (returns)	: void		:					*/

void	KBTabber::tabSelected
	(	KBTabberPage	*page
	)
{
	DPRINTF
	((	"KBTabber::tabSelected: page %p selected\n",
		(void *)page
	))	;

	CITER
	(	TabberPage,
		f,
		if (f == page) f->setVisible (true ) ;
	)
	CITER
	(	TabberPage,
		f,
		if (f != page) f->setVisible (false) ;
	)

	if (showingData())
	{
#if	! __KB_RUNTIME
		KBRecorder *recorder = KBRecorder::self() ;
		if ((recorder != 0) && recorder->isRecording(getDocRoot()))
			recorder->raisePage
			(	this,
				page->getName()
			)	;
#endif

		bool	evRc	;
		KBValue	name	(page->getName()) ;
		eventHook (m_onTabSelect, 1, &name, evRc) ;
	}
}

/*  KBTabber								*/
/*  newPage	: Add new page						*/
/*  (returns)	: void		:					*/

void	KBTabber::newPage ()
{
#if	! __KB_RUNTIME
	KBAttrDict aDict ;
	bool	   ok	 ;

	aDict.addValue
	(	"tabtext",
		QString("Page %1").arg(m_tabberBar->getNumTabs() + 1)
	)	;

	KBTabberPage *page = new KBTabberPage
			     (		this,
					aDict,
					"KBTabberPage",
					&ok
			      )	;

	if (!ok) return	;

	page->buildDisplay (m_frmDisp) ;
	page->showAs       (KB::ShowAsDesign) ;
	page->getContainer ()->show() ;

	getLayout()->setChanged() ;

	m_tabberBar->addTab (page->getAttrVal("tabtext"), page, true) ;
	tabSelected (page) ;

#endif
}

/*  KBTabber								*/
/*  setPageOrder: Handle reordering of pages				*/
/*  (returns)	: void		:					*/

void	KBTabber::setPageOrder ()
{
#if	! __KB_RUNTIME

	QPtrList<KBTabberPage> pageList ;
	m_tabberBar->pagesInOrder (pageList) ;

	if (!KBTabPageDlg(pageList).exec()) return ;

	CITER
	(	TabberPage,
		page,
		m_tabberBar->removeTab (page)
	)

	int	tabOrder = 1 ;
	LITER
	(	KBTabberPage,
		pageList,
		page,

		page->setTabOrder   (tabOrder) ;
		m_tabberBar->addTab (page->getAttrVal("tabtext"), page) ;
		tabOrder += 1 ;
	)

	
	getLayout()->setChanged() ;
#endif
}

#if	! __KB_RUNTIME

/*  KBTabber								*/
/*  designPopup	: Handle design menu request				*/
/*  parent	: KBPopupMenu *	: Possible parent popup			*/
/*  cell	: QRect		: Position information			*/
/*  (returns)	: KBPopupMenu *	: Popup menu				*/

KBPopupMenu
	*KBTabber::designPopup
	(	KBPopupMenu	*parent,
		QRect		cell
	)
{
	static Qt::ButtonState	bState ;

	KBPopupMenu *popupMain	= new KBPopupMenu (parent, &bState) ;

	KBNode	    *cpyNode	= 0 	;
	bool	    noIns	= !KBFormCopier::self()->anyCopied(cpyNode) ;
	bool	    noPage	= true	;

	if (cpyNode != 0) noIns = cpyNode->isTabberPage() == 0 ;

	CITER (TabberPage, t, noPage = false ; break)

	popupMain->setTitle    (this) ;

	if (parent == 0)
	{
		KBPopupMenu *popupEdit	= new KBPopupMenu (popupMain) ;

		popupEdit->insertEntry
		(	false,
			getSmallIcon("editcut"),
			TR("C&ut"),
			this,
			SLOT(cutObj   	())
		)	;
		popupEdit->insertEntry
		(	false,
			getSmallIcon("editcopy"),
			TR("&Copy"),
			this,
			SLOT(copyObj  	 ())
		)	;
		popupEdit->insertEntry
		(	noIns,
			TR("&Paste page"),
			this,
			SLOT(pasteObjects())
		)	;
		popupEdit->insertEntry
		(	false,
			getSmallIcon("editdelete"),
			TR("&Delete"),
			this,
			SLOT(deleteObj	 ())
		)	;

		popupMain->insertItem  (TR("&Edit"), popupEdit) ;
	}

	popupMain->insertItem 
	(	getSmallIcon("newtab"),
		TR("&New Page"),
	 	this,
		SLOT(newPage()),
		0, 0
	)	;
	popupMain->insertEntry
	(	noPage,
		TR("Set page order"),
		this,
		SLOT(setPageOrder ())
	)	;
	popupMain->insertItem 
	(	getSmallIcon("properties"),
		TR("Tabber properties"),
		this,
		SLOT(propertyDlg  ())
	)	;

	if ((parent == 0) && (parentObject() != 0))
		makeAncestorPopup (popupMain, this) ;

	popupMain->insertSeparator() ;
	popupMain->insertItem
	(	getSmallIcon("info"),
		TR("&Information"),
		this,
		SLOT(whatsThis	  ())
	)	;

	setCtrlRect (cell) ;
	return	popupMain ;
}
#endif

/*  KBTabber								*/
/*  setTabText	: Set tab text for specified page			*/
/*  page	: KBTabberPage * : Page in question			*/
/*  tabText	: QString	 : Tab text				*/
/*  (returns)	: void		 :					*/

void	KBTabber::setTabText
	(	KBTabberPage	*page,
		QString		tabText
	)
{
	m_tabberBar->setTabText (tabText, page) ;
}

QRect	KBTabber::getTabRect
	(	KBTabberPage	*page
	)
{
	return	m_tabberBar->getTabRect (page) ;
}

/*  KBTabber								*/
/*  remChild	: Child node being removed				*/
/*  node	: KBNode *	: Node in question			*/
/*  (returns)	: void		:					*/

void	KBTabber::remChild
	(	KBNode		*node
	)
{
	/* Intercept this method so that we can remove the table which	*/
	/* corresponds to the page					*/
	m_tabberBar->removeTab (node) ;
	KBNode::remChild (node) ;
}

void	KBTabber::setPageEnabled
	(	KBTabberPage	*page,
		bool		enabled
	)
{
	m_tabberBar->setPageEnabled (page, enabled) ;
}

void	KBTabber::setCurrentPage
	(	KBTabberPage	*page
	)
{
	m_tabberBar->setCurrentTab (page) ;
}

/*  KBTabber								*/
/*  write	: Write tabber 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  offset	: QPoint	: Offset into writer			*/
/*  first	: bool		: First control flag			*/
/*  extra	: int &		: Return extra space			*/
/*  prior	: bool		: Write prior value flag		*/
/*  (returns)	: bool		: Success				*/

bool	KBTabber::write
	(	KBWriter	*writer,
		QPoint		offset,
		bool		first,
		int		&extra,
		bool
	)
{
	QString	bgcolor	;
	bgcolor.sprintf	("0x%06x", m_frmDisp->getDisplayWidget()->backgroundColor().rgb() & 0x00ffffff) ;

	new KBWriterBG  (writer, geometry(offset), bgcolor) ;

#if	! __KB_RUNTIME
	if (showingDesign())
		new KBWriterBox (writer, geometry(offset)) ;
#endif

	QPoint		s	= writer->setOffset (false, geometry().topLeft()) ;
	KBTabberPage	*page	;

	if (m_tabberBar->getCurrentTab (page) >= 0)
		page->write  (writer, offset, first, extra) ;

	m_tabberBar->write     (writer, offset, first, extra) ;
	writer     ->setOffset (true, s)  ;

	return	true	;
}

KBTabberPage
	*KBTabber::currentPage ()
{
	return	m_tabberBar->currentPage () ;

}


NEWNODE(Tabber, __TR("New &Tab control"), 0, KNF_FORM|KNF_BLOCK|KNF_STATIC)
