/***************************************************************************
    file	         : kb_tabber.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_TABBER_H
#define	__KB_TABBER_H

#include	"kb_classes.h"
#include	"kb_framer.h"
#include	"kb_qttabber.h"

#include	"libkbase_exports.h"

class	KBTabber	;
class	KBTabberBar	;
class	KBTabberTab	;


/*  KBTabberBar								*/
/*  -----------								*/
/*  This class is the KBObject-level representation of the tab bar in	*/
/*  a tabbed form widget. Note that it never appears in the XML form	*/
/*  definition; instances are created as needed by the KBTabber class.	*/

class	LIBKBASE_API	KBTabberBar : public KBObject
{
	Q_OBJECT

	KBTabber		*m_tabber	;
	KBCtrlTabberBar		*m_tabBar	;
	QPtrList<KBTabberTab>	m_tabList	;

public	:

	virtual	KBNode		*replicate	(KBNode    *)	;

	virtual	void		showAs		(KB::ShowAs )	;
	virtual	void		buildDisplay	(KBDisplay *)	;
	virtual	void		setupProperties	()		;

#if	! __KB_RUNTIME
	virtual	KBPopupMenu	*designPopup	(KBPopupMenu *, QRect) ;
	virtual	void		makeRecordPopup	(KBPopupMenu *, uint, bool) ;
#endif

	uint			getNumTabs	()			;
	int			getCurrentTab	(KBTabberPage  *&)	;
	void			setCurrentTab	(KBTabberPage   *)	;
	void			setPageEnabled	(KBTabberPage   *, bool);

	void			pagesInOrder	(QPtrList<KBTabberPage> &)	;

	int			addTab		(const QString &, KBTabberPage *, bool = false) ;
	void			setTabText	(const QString &, KBTabberPage *) ;
	void			removeTab	(KBNode *)		;
	void			printPages  	(QString &, int, bool) 	;
	QRect			getTabRect	(KBTabberPage *)	;

	bool			wideTabs	()	;
	KBTabberPage		*currentPage	()	;

public	slots :

	void			tabSelected	(int)	;

public	:

	KBTabberBar 			(KBTabber *) ;
	KBTabberBar 			(KBNode *, KBTabberBar *) ;
	virtual	 ~KBTabberBar 		() ;
}	;


/*  KBTabberPage							*/
/*  ------------							*/
/*  This class is used for a page under a tabber. It is essentially a	*/
/*  container, except that its geometry is locked to its parent tabber.	*/

class	LIBKBASE_API	KBTabberPage : public KBFramer
{
	Q_OBJECT

protected :

	KBAttrStr		m_tabText	;
	KBAttrImage		m_image		;

public	:

	KBTabberPage (KBNode *, const QDict<QString> &, cchar *, bool * = 0) ;
	KBTabberPage (KBNode *, KBTabberPage *) ;
	virtual	 ~KBTabberPage () ;

	virtual	KBTabberPage	*isTabberPage	() { return this ; }

#if	! __KB_RUNTIME
	virtual	void		makeRecordPopup	(KBPopupMenu *, uint, bool) ;
#endif

	virtual	void		printNode  	(QString &, int, bool)	;
	virtual	void		showAs		(KB::ShowAs)    ;
	virtual	void		setGeometry	(const QRect &)	;
	virtual	QRect		geometry	() 		;
	virtual	KBNode		*replicate	(KBNode *)	;

	virtual	void		setEnabled	(bool)		;
	void			setCurrent	()		;
	uint			tabBarHeight	() 		;
	virtual	void		setupProperties	()		;

	virtual	bool		propertyDlg	(cchar *) 	;
}	;

/*  KBTabber								*/
/*  This class is used for tabbed form widgets. It is a container,	*/
/*  which (implicitely) contains a tabbar, plus a further number of	*/
/*  containers which are the tabbed pages.				*/

class	LIBKBASE_API	KBTabber : public KBFramer
{
	Q_OBJECT

	KBAttrStr		m_initPage	;
	KBAttrBool		m_wideTabs	;
	KBAttrUInt		m_forceHeight	;
	KBEvent			m_onTabSelect	;
	KBTabberBar		*m_tabberBar	;

protected slots :

	void			newPage			() ;
	void			setPageOrder		() ;
	void			recordVerifyTabber	() ;

public	:

	KBTabber (KBNode *, const QDict<QString> &, bool * = 0) ;
	KBTabber (KBNode *, KBTabber *) ;
	virtual	 ~KBTabber () ;

	virtual	KBTabber	*isTabber	() { return this ; }

	virtual	KBNode		*replicate	(KBNode *)	;
	virtual	void		printNode	(QString &, int, bool)	;
	virtual	void		showAs		(KB::ShowAs)	;
	virtual	bool		write		(KBWriter *, QPoint, bool, int &, bool = false) ;

#if	! __KB_RUNTIME
	virtual	KBPopupMenu	*designPopup	(KBPopupMenu *, QRect)	;
	virtual	void		makeRecordPopup	(KBPopupMenu *, uint, bool) ;
	virtual	bool		propertyDlg	(cchar    *) 	;
#endif
	virtual	void		remChild   	(KBNode   *)	;

	bool			wideTabs	() ;
	int			tabBarHeight	() ;

	void			tabSelected	(KBTabberPage	*) ;
	void			setTabText	(KBTabberPage	*, QString) ;
	void			setPageEnabled	(KBTabberPage	*, bool   ) ;
	void			setCurrentPage	(KBTabberPage   *) ;
	QRect			getTabRect	(KBTabberPage   *) ;

	KBTabberPage		*currentPage	() ;
}	;

#endif	// __KB_TABBER_H
