/***************************************************************************
    file	         : kb_tablechooser.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"rk_combobox.h"
#include	"rk_vbox.h"
#include	"rk_gridbox.h"
#include	"rk_pushbutton.h"

#if		__KB_AUTOMOC
#include 	"kb_tablechooser.moc"
#else
#include 	"kb_tablechooser.h"
#endif

#include	"kb_serverinfo.h"


/*  KBTableChooser							*/
/*  KBTableChooser							*/
/*		: Constructor for table chooser				*/
/*  location	: KBLocation &	 : Database location			*/
/*  cbServer	: RKComboBox *	 : Server selector			*/
/*  cbTable	: RKComboBox *	 : Table selector			*/
/*  (returns)	: KBTableChooser :					*/

KBTableChooser::KBTableChooser
	(	KBLocation	&location,
		RKComboBox	*cbServer,
		RKComboBox	*cbTable
	)
	:
	m_location	(location),
	m_cbServer	(cbServer),
	m_cbTable	(cbTable)
{
	if (!m_location.dbInfo()->findServer(KBLocation::m_pFile)->dbType().isEmpty())
		m_cbServer->insertItem (KBLocation::m_pFile) ;

	QPtrListIterator<KBServerInfo> *svIter = location.dbInfo()->getServerIter () ;
	KBServerInfo		    *svInfo ;

	while ((svInfo = svIter->current()) != 0)
	{
		m_cbServer->insertItem (svInfo->serverName()) ;
		(*svIter) += 1 ;
	}

	delete	svIter	;

	connect
	(	m_cbServer,
		SIGNAL(activated     (const QString &)),
		SLOT  (serverSelected(const QString &))
	)	;
	connect
	(	m_cbTable,
		SIGNAL(activated     (const QString &)),
		SLOT  (tableSelected (const QString &))
	)	;

	serverSelected (m_cbServer->text(0)) ;
}

/*  KBTableChooser							*/
/*  serverSelected							*/
/*		: User selects a database server			*/
/*  svName	: const QString & : Server name				*/
/*  (returns)	: void		  :					*/

void	KBTableChooser::serverSelected
	(	const QString	&svName
	)
{
	KBDBLink		dbLink	;
	KBTableDetailsList	tabList	;

	m_cbTable->clear     ()	;
	emit	serverChanged()	;

	if (svName == "")
		return	;

	if (!dbLink.connect (m_location, svName))
	{	dbLink.lastError().DISPLAY() ;
		return	;
	}
	if (!dbLink.listTables (tabList))
	{	dbLink.lastError().DISPLAY() ;
		return	;
	}

	m_cbTable->insertItem ("") ;

	for (uint idx = 0 ; idx < tabList.count() ; idx += 1)
		m_cbTable->insertItem (tabList[idx].m_name) ;

	emit	serverChanged	() ;
}

/*  KBTableChooser							*/
/*  tableSelected							*/
/*		: User selects a table					*/
/*  tabName	: const QString & : Table name				*/
/*  (returns)	: void		  :					*/

void	KBTableChooser::tableSelected
	(	const QString	&
	)
{
	emit	tableChanged	() ;
}

/*  KBTableChooser							*/
/*  setServer	: Set specifier server					*/
/*  svrName	: const QString & : Server name				*/
/*  (returns)	: bool		  : Server name valid			*/

bool	KBTableChooser::setServer
	(	const QString	&svrName
	)
{
	bool	rc	= false ;

	m_cbServer->setCurrentItem (0) ;

	for (int idx = 0 ; idx < m_cbServer->count() ; idx += 1)
		if (m_cbServer->text(idx) == svrName)
		{	m_cbServer->setCurrentItem (idx) ;
			serverSelected	(svrName) ;
			rc	= true	;
			break	;
		}

	emit serverChanged () ;
	return	rc ;
}

/*  KBTableChooser							*/
/*  setTable	: Set specifier table					*/
/*  tabName	: const QString & : Table name				*/
/*  (returns)	: bool		  : Table name valid			*/

bool	KBTableChooser::setTable
	(	const QString	&tabName
	)
{
	bool	rc	= false ;

	m_cbTable->setCurrentItem (0) ;

	for (int idx = 0 ; idx < m_cbTable->count() ; idx += 1)
		if (m_cbTable->text(idx) == tabName)
		{	m_cbTable->setCurrentItem (idx) ;
			tableSelected	(tabName) ;
			rc	= true	;
			break	;
		}

	emit tableChanged () ;
	return	rc ;
}


/*  ------------------------------------------------------------------ */

/*  KBTableChooserDlg							*/
/*  KBTableChooserDlg							*/
/*		: Simple server/table selection dialog			*/
/*  location	: KBLocation &	    : Database location			*/
/*  (returns)	: KBTableChooserDlg :					*/

KBTableChooserDlg::KBTableChooserDlg
	(	KBLocation	&location
	)
	:
	KBDialog	("Pick server and table", true),
	m_location	(location)
{
	RKVBox    *box  = new RKVBox (this) ;
	box->setTracking () ;

	RKGridBox *grid = new RKGridBox  (2, box) ;

	new QLabel (TR("Server"), grid) ;
	m_cbServer = new RKComboBox (grid) ;

	new QLabel (TR("Table" ), grid) ;
	m_cbTable  = new RKComboBox (grid) ;

	addOKCancel (box, &m_bOK) ;
	m_bOK->setEnabled (false) ;

	m_chooser = new KBTableChooser (m_location, m_cbServer, m_cbTable) ;
	connect  (m_chooser, SIGNAL(changed()), SLOT(changed())) ;
}

KBTableChooserDlg::~KBTableChooserDlg ()
{
	delete	m_chooser ;
}

/*  KBTableChooserDlg							*/
/*  changed	: Selection changed					*/
/*  (returns)	: void		:					*/

void	KBTableChooserDlg::changed ()
{
	m_bOK->setEnabled
	(	!m_cbServer->currentText().isEmpty() &&
		!m_cbTable ->currentText().isEmpty()
	)	;
}


QString	KBTableChooserDlg::getServer ()
{
	return	m_cbServer->currentText() ;
}

QString	KBTableChooserDlg::getTable ()
{
	return m_cbTable ->currentText() ;
}
