/***************************************************************************
    file	         : kb_taborddlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qspinbox.h>
#include	<qlabel.h>


#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_item.h"
#include	"kb_manual.h"
#include	"kb_sidepanel.h"

#if		__KB_AUTOMOC
#include	"kb_taborddlg.moc"
#else
#include	"kb_taborddlg.h"
#endif

#include	"rk_hbox.h"
#include	"rk_vbox.h"
#include	"rk_gridbox.h"
#include	"rk_listbox.h"
#include	"rk_pushbutton.h"


#define		__DEBUG_TABORDDLG	(1)
#if		__DEBUG_TABORDDLG
#include	"kb_dprintf.h"
#endif


/*  KBTabOrderObject							*/
/*  objectInRange							*/
/*		: Test whether item has coordinate in object range	*/
/*  Object	: KBObject *	: Object in question			*/
/*  axis	: char		: Specifies "x" or "y"			*/
/*  tol		: uint		: Alignment tolerance			*/
/*  (returns)	: bool		: True if in range within telerance	*/

bool	KBTabOrderObject::objectInRange
	(	KBObject	*object,
		cchar		axis,
		uint		tol
	)
{
	QRect	r	= object->geometry() ;
	uint	v	= axis == 'x' ? r.x() : r.y() ;

	/* Note that the following calculation is ordered so that we	*/
	/* work with unsigned values.					*/
	if ((m_minval <= v + tol) && (v <= m_maxval + tol))
	{	if (v < m_minval) m_minval = v ;
		if (v > m_maxval) m_maxval = v ;
		return	true	;
	}

	return	false	;
}


/*  ------------------------------------------------------------------  */

/*  KBTabListObject							*/
/*  comparedTo	: Determin ordering of object with respect to another	*/
/*  other	: KBTabListObject * : Other object			*/
/*  (returns)	: int		    : Sorting value for QT sort methods	*/

int	KBTabListObject::comparedTo
	(	KBTabListObject	*other
	)
{
	/* If the ordering is by the current tab order then simply	*/
	/* compare the current ordering values.				*/
	if (*m_order == byTabOrder)
		return	m_object->getTabOrder() - other->m_object->getTabOrder() ;

	int	tv1	;
	int	tv2	;
	int	ov1	;
	int	ov2	;

	if (*m_order == byColumn)
	{	tv1	= 	 m_orderx->getOrderVal () ;
		tv2	= 	 m_ordery->getOrderVal () ;
		ov1	= other->m_orderx->getOrderVal () ;
		ov2	= other->m_ordery->getOrderVal () ;
	}
	else
	{	tv1	= 	 m_ordery->getOrderVal () ;
		tv2	= 	 m_orderx->getOrderVal () ;
		ov1	= other->m_ordery->getOrderVal () ;
		ov2	= other->m_orderx->getOrderVal () ;
	}

	return	(tv1 == ov1) ? (tv2 - ov2) : (tv1 - ov1) ;	
}



/*  KBTabListObject							*/
/*  coalesce	: Coalesce into orderings				*/
/*  x		: QPtrList<KBTabOrderObject>				*/
/*				 : Ordering list for x-axis		*/
/*  y		: QPtrList<KBTabOrderObject>				*/
/*				 : Ordering list for y-axis		*/
/*  tol		: uint		 : Alignment tolerance			*/
/*  (returns)	: KBTabListObject:					*/

void	KBTabListObject::coalesce
	(	QPtrList<KBTabOrderObject>	&x,
		QPtrList<KBTabOrderObject>	&y,
		uint			tol
	)
{
	m_orderx	= 0 ;
	m_ordery	= 0 ;

	/* Scan through the x ordering list to see if we have an	*/
	/* overlap with this object. If so then use this overlap, if	*/
	/* not append a new overlap element.				*/
	LITER
	(	KBTabOrderObject,
		x,
		p1,

		if (p1->objectInRange (m_object, 'x', tol))
		{	m_orderx = p1 ;
			break	 ;
		}
	)

	if (m_orderx == 0)
	{	m_orderx = new KBTabOrderObject (m_object->geometry().x()) ;
		x.append (m_orderx) ;
	}

	/* Similarly repeat for the y ordering ...			*/
	LITER
	(	KBTabOrderObject,
		y,
		p2,

		if (p2->objectInRange (m_object, 'y', tol))
		{	m_ordery = p2 ;
			break	 ;
		}
	)

	if (m_ordery == 0)
	{	m_ordery = new KBTabOrderObject (m_object->geometry().y()) ;
		y.append (m_ordery) ;
	}
}

/*  KBTabListObject							*/
/*  KBTabListObject							*/
/*		: Constructor for tab list sorting object		*/
/*  item	: KBObject *	  : Associated object			*/
/*  order	: Order *	  : Ordering type pointer		*/
/*  (returns)	: KBTabListObject :					*/

KBTabListObject::KBTabListObject
	(	KBObject	*object,
		Order		*order
	)
	:
	m_object	(object),
	m_order	(order)
{
	/* This version is used on entry to the tab order dialog, where	*/
	/* we are ordering by the existing tab order.			*/
	m_tabbed = object->getTabOrder() > 0 ;
	m_orderx = 0 ;
	m_ordery = 0 ;
}



/*  ------------------------------------------------------------------  */

/*  KBTabList								*/
/*  compareItems: Comparisom function for tab ordering			*/
/*  _item1	: QCollection::Object : First item			*/
/*  _item2	: QCollection::Object : Second item			*/
/*  (returns)	: int		    L Orderinf value			*/

int	KBTabList::compareItems
	(	QCollection::Item	_item1,
		QCollection::Item	_item2
	)
{
	KBTabListObject	*item1	= (KBTabListObject *)_item1 ;
	KBTabListObject	*item2	= (KBTabListObject *)_item2 ;

	return	item1->comparedTo (item2) ;
}



/*  ------------------------------------------------------------------  */

/*  KBTabListBoxObject							*/
/*  KBTabListBoxObject : Constructor for displayed list entry		*/
/*  listBox	: RKListBox *		: Parent list view		*/
/*  object	: KBTabListObject *	: Associated tab list object	*/
/*  (returns)	: KBTabListBoxObject	:				*/

KBTabListBoxObject::KBTabListBoxObject
	(	RKListBox	*listBox,
		KBTabListObject	*object
	)
	:
	QListBoxText	(listBox),
	m_object	(object)
{
	QString	name	= m_object->getObject()->getName() ;

	if (name.isEmpty())
		name	= QString(TR("Control %1")).arg(listBox->count()) ;

	setText	(name) ;
}



/*  ------------------------------------------------------------------  */

/*  KBTabOrderDlg							*/
/*  KBTabOrderDlg: Constructor for tab order dialog			*/
/*  block	 : KBBlock *	     : Parent block			*/
/*  objects	 : QPtrList<KBObject> & : List of tab-ordered objects 	*/
/*  (returns)	 : KBTabOrderDlg     :					*/

KBTabOrderDlg::KBTabOrderDlg
	(	KBBlock			*block,
		QPtrList<KBObject>	&objects
	)
	:
	KBDialog 	(TR("Tab ordering"), true),
	m_block	  	(block),
	m_objects	(objects)
{
	RKVBox	*layTop	   = new RKVBox (this) ;
	layTop->setTracking () ;

	RKHBox	*layMain   = new RKHBox (layTop) ;

	new KBSidePanel (layMain, TR("Tab ordering")) ;
	m_xObjects = new RKListBox (layMain) ;

	RKVBox	*layAddRem = new RKVBox (layMain) ;

	m_lObjects = new KBDragBox (layMain) ;

	m_bAdd		= new RKPushButton (TR("&Add >>"   ), layAddRem) ;
	m_bRemove	= new RKPushButton (TR("<< &Remove"), layAddRem) ;

	layAddRem->addFiller()  ;

	RKGridBox *layRight = new RKGridBox (2, layMain) ;

	m_bMoveUp	= new RKPushButton (TR("Move &Up"  ), layRight) ;
	m_bByColumn	= new RKPushButton (TR("By Co&lumn"), layRight) ;
	m_bMoveDown	= new RKPushButton (TR("Move &Down"), layRight) ;
	m_bByRow	= new RKPushButton (TR("By &Row"),    layRight) ;

	new QLabel  (TR("Tolerance"), layRight) ;
	m_sTolerance	= new QSpinBox	  (1, 100, 1, layRight) ;

	layRight    ->addFillerRow     () ;

	m_sTolerance->setValue         (5)   ;
	m_xObjects  ->setMinimumWidth  (150) ;
	m_lObjects  ->setMinimumWidth  (150) ;

	connect (m_bByColumn,	SIGNAL(clicked ()), 	SLOT(clickByColumn ())) ;
	connect (m_bByRow,	SIGNAL(clicked ()), 	SLOT(clickByRow    ())) ;

	m_xObjects  ->setSelectionMode (RKListBox::Single) ;
	m_lObjects  ->setSelectionMode (RKListBox::Single) ;

	m_bMoveUp   ->setEnabled (false) ;
	m_bMoveDown ->setEnabled (false) ;

	connect (m_bAdd,	SIGNAL(clicked ()),     SLOT(clickAdd      ())) ;
	connect (m_bRemove,	SIGNAL(clicked ()),     SLOT(clickRemove   ())) ;
	connect (m_bMoveUp,	SIGNAL(clicked ()),     SLOT(clickMoveUp   ())) ;
	connect (m_bMoveDown,	SIGNAL(clicked ()),     SLOT(clickMoveDown ())) ;

	connect	(m_lObjects, 	SIGNAL(highlighted  (int)),	       SLOT(highlighted(int))) ;
	connect (m_xObjects,	SIGNAL(doubleClicked(QListBoxItem *)), SLOT(clickAdd   ()   )) ;
	connect (m_lObjects,	SIGNAL(doubleClicked(QListBoxItem *)), SLOT(clickRemove()   )) ;


	m_tabList.setAutoDelete  (true) ;
	m_orderx .setAutoDelete  (true) ;
	m_ordery .setAutoDelete  (true) ;

	bool	hasL2	= !block->getRoot()->getAttrVal("language2").isEmpty() ;

	DPRINTF
	((	"KBTabOrderDlg::KBTabOrderDlg: hasL2=%d\n",
		hasL2
	))	;

	/* Add the items to our list and sort them by current tab order	*/
	/* and then load them into the display list box. Hiddens are	*/
	/* always skipped; blocks are skipped unless there is a client	*/
	/* side language.						*/
	LITER
	(	KBObject,
		m_objects,
		obj,

		if (obj->isHidden() != 0)
			continue ;

		if (obj->isBlock () != 0)
			if (!hasL2)
				continue ;

		m_tabList.append (new KBTabListObject (obj, &m_order)) ;
	)

	m_order = byTabOrder ;
	m_tabList.sort () ;
	loadListBox    () ;

	addOKCancel (layTop, 0, 0, KBHELPKEY("Chap5Navigation")) ;
}

/*  KBTabOrderDlg							*/
/*  loadListBox	: Load list box with item (control) names		*/
/*  (returns)	: void		:					*/

void	KBTabOrderDlg::loadListBox ()
{
	m_lObjects->clear () ;
	m_xObjects->clear () ;

	LITER
	(	KBTabListObject,
		m_tabList,
		object,

		if (object->getTabbed ())
			new KBTabListBoxObject (m_lObjects, object) ;
		else	new KBTabListBoxObject (m_xObjects, object) ;
	)
}

/*  KBTabOrderDlg							*/
/*  accept	: Handle click of the OK button				*/
/*  (returns)	: void		:					*/

void	KBTabOrderDlg::accept ()
{
	LITER
	(	KBObject,
		m_objects,
		obj,
		obj->setTabOrder (0)
	)

	for (uint idx = 0 ; idx < m_lObjects->count () ; idx += 1)
		((KBTabListBoxObject *)m_lObjects->item(idx))->getObject()->getObject()->setTabOrder (idx + 1) ;

	done	(true) ;
}

/*  KBTabOrderDlg							*/
/*  positionSort: Sort items on basis of position			*/
/*  _order	: Order		: Required ordering			*/
/*  (returns)	: void		:					*/

void	KBTabOrderDlg::positionSort
	(	Order	_order
	)
{
	m_orderx .clear () ;
	m_ordery .clear () ;

	LITER
	(	KBTabListObject,
		m_tabList,
		obj,
		obj->coalesce (m_orderx, m_ordery, m_sTolerance->value())
	)

	m_order	= _order  ;
	m_tabList.sort () ;
	loadListBox    () ;
}

/*  KBTabOrderDlg							*/
/*  clickAdd	: Request to add an untabbed item to the tab list	*/
/*  (returns)	: void		:					*/

void	KBTabOrderDlg::clickAdd  ()
{
	int		xidx	;
	int		lidx	;
	QListBoxItem	*item	;

	if ((xidx = m_xObjects->currentItem()) <  0) return ;
	if ((item = m_xObjects->item   (xidx)) == 0) return ;

	m_xObjects->takeItem (item) ;

	if ((lidx  = m_lObjects->currentItem()) < 0) lidx = m_lObjects->count() ;
	m_lObjects->insertItem     (item, lidx + 1) ;
	m_lObjects->setCurrentItem (item) ;

	((KBTabListBoxObject *)item)->getObject()->setTabbed (true) ;

	if ((uint)xidx < m_xObjects->count()) m_xObjects->setCurrentItem (xidx) ;
}

/*  KBTabOrderDlg							*/
/*  clickRemove	: Request to remove a tabbed item from the tab list	*/
/*  (returns)	: void		:					*/

void	KBTabOrderDlg::clickRemove  ()
{
	int		lidx	;
	int		xidx	;
	QListBoxItem	*item	;

	if ((lidx = m_lObjects->currentItem()) <  0) return ;
	if ((item = m_lObjects->item   (lidx)) == 0) return ;

	m_lObjects->takeItem (item) ;

	if ((xidx  = m_xObjects->currentItem()) < 0) xidx = m_xObjects->count() ;
	m_xObjects->insertItem     (item, xidx + 1) ;
	m_xObjects->setCurrentItem (item) ;

	((KBTabListBoxObject *)item)->getObject()->setTabbed (false) ;

	if ((uint)lidx < m_lObjects->count()) m_lObjects->setCurrentItem (lidx) ;
}

/*  KBTabOrderDlg							*/
/*  clickByColumn: Handle click of the by-column button			*/
/*  (returns)	 : void		:					*/

void	KBTabOrderDlg::clickByColumn ()
{
	positionSort (byColumn)  ;
}

/*  KBTabOrderDlg							*/
/*  clickByRow	 : Handle click of the by-row button			*/
/*  (returns)	 : void		:					*/

void	KBTabOrderDlg::clickByRow ()
{
	positionSort (byRow) ;
}

/*  KBTabOrderDlg							*/
/*  clickMoveUp	 : Handle click of the move-up button			*/
/*  (returns)	 : void		:					*/

void	KBTabOrderDlg::clickMoveUp ()
{
	int		idx	;
	QListBoxItem	*cur	;

	if ((idx = m_lObjects->currentItem()) <= 0) return ;
	if ((cur = m_lObjects->item    (idx)) == 0) return ;

	m_lObjects->takeItem       (cur) ;
	m_lObjects->insertItem     (cur, idx - 1) ;
	m_lObjects->setCurrentItem (cur) ;
}

/*  KBTabOrderDlg							*/
/*  clickMoveDown: Handle click of the move-down button			*/
/*  (returns)	 : void		:					*/

void	KBTabOrderDlg::clickMoveDown ()
{
	int		idx	;
	QListBoxItem	*cur	;

	if ((idx = m_lObjects->currentItem())  < 0) return ;
	if ((cur = m_lObjects->item    (idx)) == 0) return ;
	if (idx >= (int)m_lObjects->count() - 1)    return ;

	m_lObjects->takeItem       (cur) ;
	m_lObjects->insertItem     (cur, idx + 1) ;
	m_lObjects->setCurrentItem (cur) ;
}

/*  KBTabOrderDlg							*/
/*  highlighted	: Used clicks on an entry in the list box		*/
/*  idx		: int		: Highlighted item index		*/
/*  (returns)	: void		:					*/

void	KBTabOrderDlg::highlighted
	(	int	idx
	)
{
	m_bMoveUp  ->setEnabled (idx > 0) ;
	m_bMoveDown->setEnabled (idx < (int)m_lObjects->count() - 1) ;
}


/*  tabOrderDlg	: Run the tab order dialog				*/
/*  block	: KBBlock *	     : Parent block			*/
/*  objects	: <QPtrList<KBObject> & : List of tab-ordered objects	*/
/*  (returns)	: bool		     : Success				*/

bool	tabOrderDlg
	(	KBBlock		*block,
		QPtrList<KBObject>	&objects
	)
{
	KBTabOrderDlg tDlg (block, objects) ;
	return	tDlg.exec  () ;
}
