/***************************************************************************
    file	         : kb_test.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_test.h"
#include	"kb_node.h"
#include	"kb_macro.h"
#include	"kb_docroot.h"
#include	"kb_prompt.h"

#include	"tk_messagebox.h"


#include	"kb_script.h"
#include	"kb_test.h"
#include	"kb_macro.h"


/*  KBTest								*/
/*  KBTest	: Construcutor for a test object			*/
/*  owner	: KBNode *	: Owning node				*/
/*  name	: cchar *	: Test name				*/
/*  (returns)	: KBTest	:					*/

KBTest::KBTest
	(	KBNode		*owner,
		cchar		*name
	)
	:
	KBEvent	(owner, name, "", KAF_EVCS|KAF_LOOSE)
{
	if (owner != 0) owner->addTest (this) ;
}

/*  KBTest								*/
/*  KBTest	: Construcutor for a test object			*/
/*  owner	: KBNode *	: Owning node				*/
/*  entant	: KBTest *	: Extant test				*/
/*  (returns)	: KBTest	:					*/

KBTest::KBTest
	(	KBNode		*owner,
		KBTest		*extant
	)
	:
	KBEvent	(owner, extant->getName(), "", KAF_EVCS|KAF_LOOSE)
{
	if (owner != 0) owner->addTest (this) ;

	setValue  (extant->getValue ()) ;
	setValue2 (extant->getValue2()) ;
	setComment(extant->comment  ()) ;

	KBMacroExec *macro = extant->getMacro() ;
	if (macro != 0)
	{	KBMacroExec *m = new KBMacroExec (macro) ;
		m->setName    (extant->getName()) ;
		m->setComment (extant->comment()) ;
		setMacro  (m) ;
	}
}

/*  KBTest								*/
/*  ~KBTest	: Destrucutor for a test object				*/
/*  (returns)	:		:					*/

KBTest::~KBTest ()
{
}

void	KBTest::setName
	(	const QString	&name
	)
{
	m_name	  = name    ;
}

/*  KBTest								*/
/*  setOwner	: Set tests's owner object				*/
/*  owner	: KBNode *	: Owner					*/
/*  (returns)	: void		:					*/

void	KBTest::setOwner
	(	KBNode		*owner
	)
{
	/* This method is typically used when the user OK's on a	*/
	/* property dialog, and updated tests are returned to the	*/
	/* owner.							*/
	m_owner = owner ;
	m_owner->addTest (this) ;
}

/*  KBTest								*/
/*  printAttr	: Print attribute					*/
/*  attrText	: QString &	: Result text for attribute output	*/
/*  nodeText	: QString &	: Result text for node output		*/
/*  indent	: int		: Indent depth for node output		*/
/*  flat	: bool		: Generate flatened version		*/
/*  (returns)	: void		:					*/

void	KBTest::printAttr
	(	QString		&attrText,
		QString		&nodeText,
		int		indent,
		bool		flat
	)
{
	DPRINTF
	((	"KBTest::printAttr: %s: v=%s v2=%s m=%p\n",
		m_name.latin1(),
		getValue ().isEmpty() ? "no" : "yes",
		getValue2().isEmpty() ? "no" : "yes",
		(void *)getMacro()
	))	;

	/* See if this is a macro test, in which case save as a macro.	*/
	/* The 'test="1"' attribute signals to the parser code that the	*/
	/* macro is to be dumped into a test, rather than associated	*/
	/* with an object event.					*/
	if (getMacro() != 0)
	{
		nodeText += QString("%1<macro name=\"%2\" comment=\"%3\" test=\"1\">\n")
				   .arg("", indent)
				   .arg(m_name)
				   .arg(KBAttr::escapeText (comment(), true))
				   ;

		getMacro()->save (nodeText, indent + 2) ;

		nodeText += QString("%1</macro>\n")
				   .arg("", indent) ;
		return	;
	}

	if (!getValue ().isEmpty() || !comment().isEmpty())
	{
		nodeText += QString("%1<test name=\"%2\" comment=\"%3\">\n")
				.arg("", indent)
				.arg(m_name)
				.arg(KBAttr::escapeText (comment(), true))
				;
		nodeText += KBAttr::escapeText (getValue (), false) ;
		nodeText += QString("%1</test>\n" ).arg("", indent) ;
	}
	if (!getValue2().isEmpty() && ((m_nFlags & KAF_EVCS) != 0))
	{
		nodeText += QString("%1<test2 name=\"%2\">\n")
				.arg("", indent)
				.arg(m_name)
				;
		nodeText += KBAttr::escapeText (getValue2(), false) ;
		nodeText += QString("%1</test2>\n").arg("", indent) ;
	}
}

/*  ------------------------------------------------------------------	*/


static	KBScriptIF::TestMode	testMode	= KBScriptIF::testOff	;
static	KBTest::ErrorOpt	testErrorOpt	= KBTest::ErrorOK	;
static	bool			testObjectFixed	= false			;
static	KBScriptTestResultList	*testResults	;

/*  KBTest								*/
/*  setTestMode	: Set testing mode					*/
/*  testMode	: KBScriptIF::TestMode	: Testing mode			*/
/*  (returns)	: void			:				*/

void	KBTest::setTestMode
	(	KBScriptIF::TestMode	tm
	)
{
	DPRINTF
	((	"KBTest::setTestMode: mode=%d\n",
		(int)tm
	))	;

	/* Allocate the test results list on the first call, then node	*/
	/* the mode and clear any prior test results.			*/
	if (testResults == 0)
		testResults = new KBScriptTestResultList ;

	testMode = tm	;
	testResults->clear () ;
}

/*  KBTest								*/
/*  getTestMode	: Return current test mode				*/
/*  (returns)	: KBScriptIF::TestMode	: Test mode			*/

KBScriptIF::TestMode KBTest::getTestMode ()
{
	DPRINTF
	((	"KBScriptIF::TestMode: mode=%d\n",
		(int)testMode
	))	;

	return	testMode ;
}

/*  KBTest								*/
/*  setErrorOpt	: Set current error handling option			*/
/*  errorOpt	: ErrorOpt	: Option				*/
/*  (returns)	: void		:					*/

void	KBTest::setErrorOpt
	(	ErrorOpt	errorOpt
	)
{
	DPRINTF
	((	"KBTest::setErrorOpt: set=%d\n",
		(int)errorOpt
	))	;
	testErrorOpt	= errorOpt	;
}

/*  KBTest								*/
/*  getErrorOpt	: Get current error handling option			*/
/*  (returns)	: ErrorOpt	: Option				*/

KBTest::ErrorOpt
	KBTest::getErrorOpt ()
{
	DPRINTF
	((	"KBTest::getErrorOpt: get=%d\n",
		(int)testErrorOpt
	))	;
	return	testErrorOpt	;
}

void	KBTest::setObjectFixed
	(	bool		fixed
	)
{
	testObjectFixed	= fixed	;
}

bool	KBTest::getObjectFixed ()
{
	return	testObjectFixed	;
}


/*  KBTest								*/
/*  appendTestResult							*/
/*		: Append result to current results list			*/
/*  result	: const KBScriptTestResult & : Test result		*/
/*  (returns)	: void			     :				*/

void	KBTest::appendTestResult
	(	const KBScriptTestResult &result
	)
{
	if (testResults != 0)
		testResults->append (result) ;
}

/*  KBTest								*/
/*  getTestResults							*/
/*		: Get current set of results				*/
/*  (returns)	: const KBScriptTestResultList * : Results		*/

const KBScriptTestResultList *KBTest::getTestResults ()
{
	return	testResults ;
}


/*  ------------------------------------------------------------------	*/

/*  KBTest								*/
/*  execute	: Execute event						*/
/*  resval	: KBValue &	  : Return value 			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: KBValue *	  : Argument vector			*/
/*  defval	: bool		  : Default execution return code	*/
/*  (returns)	: KBScriptError * : Error or null if OK			*/

KBScriptError
	*KBTest::execute
	(	KBValue		&resval,
		uint		argc,
		KBValue		*argv,
		bool		defval
	)
{
#if	0
	KBMacroExec	*macro	= getMacro() ;
	KBTest		*orig	= m_owner->getDocRoot()->enterTest (this) ;
	KBScriptError	*rc	= 0 ;
 
	if ((macro != 0) && (macro->getMacroSet() == "tests"))
	{
		DPRINTF
		((	"KBTest::execute: detected test macro\n"
		))	;

		rc = macro->execute (m_owner->getRoot()) ;
		if (rc == 0) 
			if (defval)
				resval.setTrue	() ;
			else	resval.setFalse	() ;
	}
	else	rc	= KBEvent::execute (resval, argc, argv, defval) ;

	m_owner->getDocRoot()->enterTest (orig) ;
	return	rc	;
#endif
	return	KBEvent::execute (resval, argc, argv, defval) ;
}

#if	! __KB_RUNTIME

/*  KBTest								*/
/*  executeTest	: Execute event as a test				*/
/*  resval	: KBValue  &	  : Return value 			*/
/*  errorOpt	: ErrorOpt &	  : Error option			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: KBValue *	  : Argument vector			*/
/*  defval	: bool		  : Default execution return code	*/
/*  (returns)	: KBScriptError * : Error or null if OK			*/

KBScriptError
	*KBTest::executeTest
	(	KBValue			&resval,
		KBTest::ErrorOpt	&errorOpt,
		uint			argc,
		KBValue			*argv,
		bool			defval
	)
{
	/* This method is called when running test suites. The error	*/
	/* option is stored before running the test. On return, if	*/
	/* the test produced an error, the error option is retrieved,	*/
	/* as it may have been changed in the test error dialog.	*/
	KBMacroExec	*macro	= getMacro() ;
	KBTest		*orig	= m_owner->getDocRoot()->enterTest (this) ;
	KBScriptError	*rc	= 0 ;

	DPRINTF
	((	"KBTest::execute: macro=%p set=[%s]\n",
		(void *)macro,
		macro == 0 ? "" : macro->getMacroSet().latin1()
	))	;
 
	KBTest::setErrorOpt (errorOpt) ;

	if ((macro != 0) && (macro->getMacroSet() == "tests"))
	{
		DPRINTF
		((	"KBTest::execute: detected test macro: errorOpt=%d\n",
			errorOpt
		))	;


		if ((rc = macro->execute (m_owner->getRoot())) == 0)
			if (defval)
				resval.setTrue	() ;
			else	resval.setFalse	() ;
	}
	else	rc	= execute (resval, argc, argv, defval) ;

	if (rc == 0)
		errorOpt = ErrorOK ;
	else	errorOpt = KBTest::getErrorOpt () ;

	m_owner->getDocRoot()->enterTest (orig) ;
	return	rc	;
}

/*  KBTest								*/
/*  testPopupResult							*/
/*		: Check for popup test result				*/
/*  type	: KBRecorder::PopupType					*/
/*				: Expected type				*/
/*  rc		: int &		: Return popup return code		*/
/*  value	: QString &	: Return popup return value		*/
/*  (returns)	: PopupResult	: NotTesting, Expected or NotExpected	*/

KBTest::PopupResult
	KBTest::testPopupResult
	(	KBRecorder::PopupType	type,
		int			&rc,
		QString			&value
	)
{
	KBNode	  *topNode = KBScriptIF::topLocationNode  () ;
	if (topNode == 0) return NotTesting ;

	KBTest	  *curTest = topNode->getDocRoot()->runningTest() ;
	if (curTest == 0) return NotTesting

	DPRINTF
	((	"KBTest::testPopupResult: topNode=%p curTest=%p macro=%p\n",
		topNode,
		curTest,
		curTest->getMacro()
	))	;

	KBMacroExec *macro = curTest->getMacro() ;
	if (macro   == 0) return NotTesting ;
	if (macro->getMacroSet() != "tests") return NotTesting ;

	return	macro->nextPopupResult (type, rc, value) ? Expected : NotExpected ;
}

/*  KBTest								*/
/*  recordPopupResult							*/
/*		: Record popup result for later test/replay		*/
/*  type	: KBRecorder::PopupType					*/
/*				: Expected type				*/
/*  rc		: int &		: Return popup return code		*/
/*  value	: QString &	: Return popup return value		*/
/*  (returns)	: bool		: True if result available		*/

void	KBTest::recordPopupResult
	(	KBRecorder::PopupType	type,
		int			rc,
		const QString		&value
	)
{
	DPRINTF
	((	"KBTest::recordPopupResult: called: tn=%p rec=%p isr=%d\n",
		KBScriptIF::topLocationNode(),
		KBRecorder::self(),
		(KBScriptIF::topLocationNode() != 0) && (KBRecorder::self() != 0) ?
			KBRecorder::self()->isRecording(KBScriptIF::topLocationNode()->getDocRoot()) : false
	))	;

	/* If the node which executed the script leading to this query	*/
	/* is available, and recording is enabled for the associated	*/
	/* document, then record the result.				*/
	KBNode	*topNode = KBScriptIF::topLocationNode  () ;
	if (topNode == 0) return ;

	KBRecorder *recorder = KBRecorder::self() ;
	if ((recorder == 0) || !recorder->isRecording(topNode->getDocRoot()))
		return	;

	recorder->popupResult (type, rc, value) ;
}

#endif

/*  KBTest								*/
/*  messageBox	: Message box display with test recording hook		*/
/*  message	: const QString & : Popup message			*/
/*  caption	: const QString & : Popup caption			*/
/*  (returns)	: bool		  : False to throw test exception	*/

bool	KBTest::messageBox
	(	const QString		&message,
		const QString		&caption
	)
{
#if	! __KB_RUNTIME
	/* Check for an extant test replay, if found and matched then	*/
	/* return the result.						*/
	int	trc	;
	QString	value	;
	switch (testPopupResult (KBRecorder::PopupMessage, trc, value))
	{
		case Expected	: return true 	;
		case NotExpected: return false	;
		default		: break		;
	}
#endif

	TKMessageBox::information (0, message, caption) ;

#if	! __KB_RUNTIME
	recordPopupResult (KBRecorder::PopupMessage, false, QString::null) ;
#endif

	return	true	;
}

/*  KBTest								*/
/*  queryBox	: Query box display with test recording hook		*/
/*  message	: const QString & : Popup message			*/
/*  caption	: const QString & : Popup caption			*/
/*  rc		: bool &	  : Standard OK/Cancel response		*/
/*  (returns)	: bool		  : False to throw test exception	*/

bool	KBTest::queryBox
	(	const QString	&message,
		const QString	&caption,
		bool		&rc
	)
{
#if	! __KB_RUNTIME
	/* Check for an extant test replay, if found and matched then	*/
	/* return the result.						*/
	int	trc	;
	QString	value	;
	switch (testPopupResult (KBRecorder::PopupQuery, trc, value))
	{
		case Expected	: rc = trc ; return true  ;
		case NotExpected: rc = trc ; return false ;
		default		: break	 ;
	}

#endif

	rc	 = TKMessageBox::questionYesNo
			(	0,
				message,
				caption
			)
			== TKMessageBox::Yes ;

#if	! __KB_RUNTIME
	recordPopupResult (KBRecorder::PopupQuery, rc, QString::null) ;
#endif
	return	true	;
}

bool	KBTest::promptBox
	(	const QString		&message,
		const QString		&caption,
		bool			&rc,
		QString			&value
	)
{
#if	! __KB_RUNTIME
	/* Check for an extant test replay, if found and matched then	*/
	/* return the result.						*/
	int	trc	;
	switch (testPopupResult (KBRecorder::PopupPrompt, trc, value))
	{
		case Expected	: rc = trc ; return true  ;
		case NotExpected: rc = trc ; return false ;
		default		: break	 ;
	}
#endif

	rc	 = doPrompt (caption, message,  value) ;

#if	! __KB_RUNTIME
	recordPopupResult (KBRecorder::PopupPrompt, rc, value) ;
#endif
	return	true	;
}

bool	KBTest::choiceBox
	(	const QString		&message,
		const QString		&caption,
		const QStringList	&choices,
		bool			&rc,
		QString			&value
	)
{
#if	! __KB_RUNTIME
	/* Check for an extant test replay, if found and matched then	*/
	/* return the result.						*/
	int	trc	;
	switch (testPopupResult (KBRecorder::PopupChoice, trc, value))
	{
		case Expected	: rc = trc ; return true  ;
		case NotExpected: rc = trc ; return false ;
		default		: break	 ;
	}
#endif

	KBChoiceDlg dDlg (caption, message, choices, value) ;
	rc = dDlg.exec() ;

#if	! __KB_RUNTIME
	recordPopupResult (KBRecorder::PopupChoice, rc, value) ;
#endif
	return	true	;
}
