/***************************************************************************
    file	         : kb_test.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_TEST_H
#define		__KB_TEST_H

#include	"kb_event.h"
#include	"kb_script.h"
#include	"kb_recorder.h"

#include	"libkbase_exports.h"


/*  KBTest								*/
/*  ------								*/
/*  Specialised event for testing ...					*/

class	LIBKBASE_API	KBTest : public KBEvent
{
public	:

	enum	ErrorOpt
	{	ErrorAbortAll		=  0,
		ErrorAbortSuite		=  1,
		ErrorAbortTest		=  2,
		ErrorPauseTest		=  3,
		ErrorUpdateTest		=  4,
		ErrorOK			= -1,
		ErrorAutoUpdate		= -2
	}	;

private	:

	enum	PopupResult
	{	NotTesting	= 0,
		Expected	= 1,
		NotExpected	= 2
	}	;

#if	! __KB_RUNTIME
	static	PopupResult	testPopupResult
				(	KBRecorder::PopupType	type,
					int			&rc,
					QString			&value
				)	;
	static	void		recordPopupResult
				(	KBRecorder::PopupType	type,
					int			rc,
					const QString		&value
				)	;
#endif

public	:

	KBTest	(KBNode *, cchar  *) ;
	KBTest	(KBNode *, KBTest *) ;
       ~KBTest	() ;

	void			setName		(const QString &) ;
	void			setOwner	(KBNode *) ;

	virtual	void		printAttr 	(QString &, QString &, int, bool) ;
	virtual KBScriptError	*execute	(KBValue &, uint = 0, KBValue * = 0, bool = false) ;

#if	! __KB_RUNTIME
	KBScriptError		*executeTest	(KBValue &, ErrorOpt &, uint = 0, KBValue * = 0, bool = false) ;
#endif
	static	void				setTestMode	(KBScriptIF::TestMode	   ) ;
	static	KBScriptIF::TestMode		getTestMode	() ;
	static	void				setErrorOpt	(ErrorOpt) ;
	static	ErrorOpt			getErrorOpt	() ;
	static	void				setObjectFixed	(bool    ) ;
	static	bool				getObjectFixed	() ;

	static	void				appendTestResult(const KBScriptTestResult &) ;
	static	const KBScriptTestResultList	*getTestResults	() ;


	/* Popups							*/
	/* ------							*/
	/* Simple popups. These are provided primarily so the that	*/
	/* test record/playback mechanism can automate popup display	*/
	/* and responses.						*/
	static	bool		messageBox
				(	const QString		&message,
					const QString		&caption
				)	;
	static	bool		queryBox
				(	const QString		&message,
					const QString		&caption,
					bool			&rc
				)	;
	static	bool		promptBox
				(	const QString		&message,
					const QString		&caption,
					bool			&rc,
					QString			&value
				)	;
	static	bool		choiceBox
				(	const QString		&message,
					const QString		&caption,
					const QStringList	&choices,
					bool			&rc,
					QString			&value
				)	;
}	;


#endif		// __KB_TEST_H
