/***************************************************************************
    file	         : kb_testdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qtimer.h>
#include	<qsplitter.h>

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_listbox.h"
#include	"rk_pushbutton.h"
#include	"rk_textedit.h"

#include	"kb_eventdlg.h"
#include	"kb_test.h"
#include	"kb_manual.h"
#include	"kb_sidepanel.h"
#include	"kb_docroot.h"

#if		__KB_AUTOMOC
#include	"kb_testdlg.moc"
#else
#include	"kb_testdlg.h"
#endif

#include	"tk_messagebox.h"

#define		__DEBUG__TESTDLG	(0)
#if		__DEBUG__TESTDLG
#include	"kb_dprintf.h"
#endif


/*  KBTestItem								*/
/*  ----------								*/
/*  This class is used for entries in the test list dialogs list of	*/
/*  tests, and embeds a pointer at the test.				*/

class	KBTestItem : public QListBoxText
{
	KBTest		*m_test	;

public	:

	KBTestItem	(RKListBox *, KBTest *) ;
	~KBTestItem	()		;

	friend	class	KBTestListDlg	;
}	;


/*  KBTestItem								*/
/*  KBTestItem	: Constructor for list-of-tests dialog list entry	*/
/*  listBox	: RKListBox *	  : Parent list box			*/
/*  test	: KBTest *	  : Test				*/
/*  (returns)	: KBTestItem	  :					*/

KBTestItem::KBTestItem
	(	RKListBox	*listBox,
		KBTest		*test
	)
	:
	QListBoxText	(listBox, test->getName())
{
	m_test	= test	;
}

KBTestItem::~KBTestItem ()
{
}

/*  ------------------------------------------------------------------	*/

/*  KBTestListDlg							*/
/*  KBTestListDlg: Constructor for test list dialog			*/
/*  parent	 : QWidget *	     : Parent widget			*/
/*  test	 : KBTest *	     : Test to be edited		*/
/*  node	 : KBNode *	     : Parent node			*/
/*  (returns)	 : KBTestListDlg     :					*/

KBTestListDlg::KBTestListDlg
	(	QWidget			*parent,
		const QPtrList<KBTest>	&testList,
		KBNode			*node
	)
	:
	RKHBox		(parent),
	m_testList	(testList),
	m_node		(node)
{
	m_lbTestList	 = new RKListBox (this) ;

	RKVBox	*layButt = new RKVBox    (this) ;
	m_bAddTest	 = new RKPushButton (TR("Add" ), layButt) ;
	m_bEditTest	 = new RKPushButton (TR("Edit"), layButt) ;
	m_bDropTest	 = new RKPushButton (TR("Drop"), layButt) ;
	layButt->addFiller () ;

	connect
	(	m_lbTestList,
		SIGNAL(highlighted(int)),
		SLOT  (highlighted())
	)	;
	connect
	(	m_lbTestList,
		SIGNAL(doubleClicked(QListBoxItem *)),
		SLOT  (clickEditTest())
	)	;
	connect
	(	m_lbTestList,
		SIGNAL(returnPressed(QListBoxItem *)),
		SLOT  (clickEditTest())
	)	;

	connect	(m_bAddTest,   SIGNAL(clicked()), SLOT(clickAddTest ())) ;
	connect	(m_bEditTest,  SIGNAL(clicked()), SLOT(clickEditTest())) ;
	connect	(m_bDropTest,  SIGNAL(clicked()), SLOT(clickDropTest())) ;


	/* Add an entry to the list box for each test, storing the	*/
	/* test pointer into it.					*/
	LITER
	(	KBTest,
		testList,
		test,
		new KBTestItem (m_lbTestList, test)
	)

	m_bEditTest->setEnabled (m_lbTestList->currentItem() >= 0) ;
	m_bDropTest->setEnabled (m_lbTestList->currentItem() >= 0) ;
}

/*  KBTestListDlg							*/
/*  addTest	 : Add a new test					*/
/*  (returns)	 : void		:					*/

void	KBTestListDlg::clickAddTest ()
{
	KBTest	  	*test	= new KBTest (0, "UnnamedTest") ;

	KBTestDlg testDlg (test, m_testList, m_node) ;
	if (testDlg.exec())
	{
		new KBTestItem (m_lbTestList, test) ;
		m_lbTestList->update     () ;
		m_bEditTest ->setEnabled (true) ;
		m_bDropTest ->setEnabled (true) ;
		return	;
	}

	delete	test	;
}

/*  KBTestListDlg							*/
/*  editTest	 : Edit the current test if any				*/
/*  (returns)	 : void		:					*/

void	KBTestListDlg::clickEditTest ()
{
	int idx = m_lbTestList->currentItem() ;
	if (idx < 0) return ;

	KBTestItem *item = (KBTestItem *)m_lbTestList->item (idx) ;
	KBTestDlg  testDlg (item->m_test, m_testList, m_node) ;

	if (testDlg.exec())
	{
		item->setText (item->m_test->getName()) ;
		m_lbTestList->update  () ;
	}
}

/*  KBTestListDlg							*/
/*  dropTest	 : Drop the current test if any				*/
/*  (returns)	 : void		:					*/

void	KBTestListDlg::clickDropTest ()
{
	int idx = m_lbTestList->currentItem() ;
	if (idx < 0) return ;

	KBTestItem *test = (KBTestItem *)m_lbTestList->item(idx) ;
	delete	test->m_test  ;
	delete	test ;
}

/*  KBTestListDlg							*/
/*  highlighted	: User highliggts slot in list				*/
/*  (returns)	 : void		:					*/

void	KBTestListDlg::highlighted ()
{
	m_bEditTest->setEnabled (m_lbTestList->currentItem() >= 0) ;
	m_bDropTest->setEnabled (m_lbTestList->currentItem() >= 0) ;
}

void	KBTestListDlg::slotOpenTest ()
{
	for (uint idx = 0 ; idx < m_lbTestList->count() ; idx += 1)
		if (m_lbTestList->text(idx) == m_openTest)
		{
			m_lbTestList->setCurrentItem (idx) ;
			clickEditTest () ;
		}

	m_openTest = QString::null ;
}

void	KBTestListDlg::openTest
	(	const QString	&test
	)
{
	DPRINTF
	((	"KBTestListDlg::openTest: [%s]\n",
		test.latin1()
	))	;

	m_openTest = test ;
	QTimer::singleShot (0, this, SLOT(slotOpenTest())) ;
}

/*  KBTestListDlg							*/
/*  save	: Save 		     : Save results back into test list	*/
/*  testList	: QPtrList<KBTest> & : List of tests			*/
/*  (returns)	: void		     :					*/

void	KBTestListDlg::save
	(	QPtrList<KBTest>	&testList
	)
{
	testList.clear() ;
	for (uint idx = 0 ; idx < m_lbTestList->count() ; idx += 1)
		testList.append (((KBTestItem *)m_lbTestList->item(idx))->m_test) ;
}


/*  ------------------------------------------------------------------	*/

/*  KBTestDlg								*/
/*  KBTestDlg	: Constructor for test script dialog			*/
/*  test	: KBTest *	     : Test to be edited		*/
/*  testList	: QPtrList<KBTest> & : List of tests			*/
/*  parent	: KBNode *	     : Parent node			*/
/*  (returns)	: KBTestDlg	     :					*/

KBTestDlg::KBTestDlg
	(	KBTest			*test,
		const QPtrList<KBTest>	&testList,
		KBNode			*parent
	)
	:
	KBDialog	(TR("Test"), true, "kbtestdlg"),
	m_testList	(testList),
	m_test		(test),
	m_parent	(parent)
{
	QString	l	= m_parent->getRoot ()->getAttrVal("language" ) ;
	QString	l2	= m_parent->getRoot ()->getAttrVal("language2") ;

	RKVBox	*layMain = new RKVBox (this) ;
	layMain->setTracking () ;

	RKHBox	*layData = new RKHBox (layMain) ;
	new KBSidePanel (layData, caption()) ;

	RKVBox	*layEdit = new RKVBox	  (layData) ;
	m_name		 = new RKLineEdit (layEdit) ;

	QSplitter *split = new QSplitter  (Qt::Vertical, layEdit) ;

	m_eventDlg	 = new KBEventBaseDlg
			       (	split,
			       		m_parent->getDocRoot()->getDocLocation(),
					l,
					l2,
					QString::null,
					QString::null,
					"tests",
					true
			       )	;
	m_eventDlg->setEventNode    (parent) ;

	m_comment	= new RKTextEdit (split) ;

	layMain   ->setStretchFactor(layData, 1) ;
	m_name    ->setText	    (m_test->getName()) ;

	DPRINTF
	((	"KBTestDlg::KBTestDlg: v=%s v2=%s m=%p\n",
		m_test->getValue ().isEmpty() ? "no": "yes",
		m_test->getValue2().isEmpty() ? "no": "yes",
		(void *)m_test->getMacro()
	))	;

	/* Display either a macro if this is a macro test or a normal	*/
	/* script test. The "m_showing" flag is set so that when the	*/
	/* dialog is OK'd, we definitevely know what we have.		*/
	if (m_test->getMacro() != 0)
	{
		m_eventDlg->init (m_test->getMacro()) ;
		m_showing = KBOptions::ScriptTypeAlwaysMacro ;
	}
	else
	{
		m_eventDlg->init (m_test->getValue(), m_test->getValue2(), "eventFunc") ;
		m_showing = KBOptions::ScriptTypeAlwaysScript ;
	}

	/* Set up the buttons. Note that the verify button is disabled	*/
	/* for macro tests, whence it makes no sense.			*/
	RKHBox	*layButt = new RKHBox (layMain) ;

	new KBManualPushButton	(layButt, "Chap14Tests") ;
	layButt->addFiller  () ;
	RKPushButton *bVerify	= new RKPushButton (TR("Verify"), layButt) ;
	m_bOK			= new RKPushButton (layButt, "ok"    ) ;
				  new RKPushButton (layButt, "cancel") ;

	bVerify->setEnabled (m_showing == KBOptions::ScriptTypeAlwaysScript) ;

	connect
	(	bVerify,
		SIGNAL	(clicked	()),
		SLOT	(clickVerify	())
	)	;
	connect
	(	m_name,
		SIGNAL	(textChanged	(const QString &)),
		SLOT	(nameChanged	())
	)	;

	nameChanged ()	;
	m_comment->setText (m_test->comment()) ;
}

/*  KBTestDlg								*/
/*  nameChanged	: User changes the name					*/
/*  (returns)	: void		:					*/

void	KBTestDlg::nameChanged ()
{
	m_bOK->setEnabled (!m_name->text().isEmpty()) ;
}

/*  KBTestDlg								*/
/*  clickVerify	: User asks to verify code				*/
/*  (returns)	: void		:					*/

void	KBTestDlg::clickVerify ()
{
	m_eventDlg->verify (m_parent) ;
}

/*  KBTestDlg								*/
/*  accept	: User request to save test				*/
/*  (returns)	: void		:					*/

void	KBTestDlg::accept ()
{
	for
	(	QPtrListIterator<KBTest> iter(m_testList) ;
		iter.current() != 0 ;
		iter += 1
	)
	{
		const KBTest *test = iter.current() ;
		if (test != m_test)
			if (test->getName() == m_name->text())
			{
				TKMessageBox::sorry
				(	0,
					TR("Duplicate test name, please choose another name"),
					TR("Duplicate test name")
				)	;
				return	;
			}
	}

	m_test->setName	  (m_name   ->text()) ;
	m_test->setComment(m_comment->text()) ;

	if (m_showing == KBOptions::ScriptTypeAlwaysMacro)
	{
		KBError		error	;
		KBMacroExec	*macro	= m_eventDlg->macro (error, m_parent) ;

		if (macro != 0)
			m_test->setMacro (macro) ;
		else	error.DISPLAY() ;
	}
	else
	{
		m_test->setValue  (m_eventDlg->value ()) ;
		m_test->setValue2 (m_eventDlg->value2()) ;
	}

	done	(1)	;
}
