/***************************************************************************
    file	         : kb_testerrordlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"kb_classes.h"
#include	"kb_script.h"
#include	"kb_sidepanel.h"

#if		__KB_AUTOMOC
#include	"kb_testerrordlg.moc"
#else
#include	"kb_testerrordlg.h"
#endif


#include	"rk_textbrowser.h"
#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"
#include	"rk_vbox.h"
#include	"rk_hbox.h"


/*  KBTestErrorDlg							*/
/*  KBTestErrorDlg							*/
/*		: Constructor for test error dialog			*/
/*  str		; const KBScriptTestResult &				*/
/*					: Error details			*/
/*  errorOpt	: KBTest::ErrorOpt	: Error handling option		*/
/*  canFix	: bool			: Error can be fixed		*/
/*  (returns)	: KBTestErrorDlg	:				*/

KBTestErrorDlg::KBTestErrorDlg
	(	const KBScriptTestResult	&str,
		KBTest::ErrorOpt		errorOpt,
		bool				canFix
	)
	:
	KBDialog	(TR("Test Error"), true, "testerrordlg")
{
	RKVBox	*layTop	   = new RKVBox (this) ;
	layTop->setTracking () ;

	RKHBox	*layMain   = new RKHBox (layTop ) ;
	new KBSidePanel (layMain, TR("Test Error")) ;

	RKVBox	*layInfo   = new RKVBox (layMain) ;

	RKTextBrowser *message = new RKTextBrowser (layInfo) ;
	message->setText (str.m_message) ;

	if (!str.m_path.isEmpty())
	{
		RKLineEdit    *path    = new RKLineEdit    (layInfo) ;
		path->setText     (str.m_path) ;
		path->setReadOnly (true) ;
	}

	if (!str.m_comment.isEmpty())
	{
		RKTextBrowser *comment = new RKTextBrowser (layInfo) ;
		comment->setText (str.m_comment) ;
	}

	RKHBox	*buttons = new RKHBox (layTop) ;
	buttons->addFiller () ;


	RKPushButton *bAbortAll   = new RKPushButton(TR("Abort All"    ), buttons) ;
	RKPushButton *bAbortSuite = new RKPushButton(TR("Abort Suite"  ), buttons) ;
	RKPushButton *bAbortTest  = new RKPushButton(TR("Abort Test"   ), buttons) ;
	RKPushButton *bContinue   = new RKPushButton(TR("Continue Test"), buttons) ;

	connect	(bAbortAll,   SIGNAL(clicked()), SLOT(slotAbortAll  ())) ;
	connect	(bAbortSuite, SIGNAL(clicked()), SLOT(slotAbortSuite())) ;
	connect	(bAbortTest,  SIGNAL(clicked()), SLOT(slotAbortTest ())) ;
	connect	(bContinue,   SIGNAL(clicked()), SLOT(slotContinue  ())) ;

	if (errorOpt == KBTest::ErrorUpdateTest)
	{
		RKPushButton *bUpdate = new RKPushButton(TR("Update Test"), buttons) ;
		connect (bUpdate, SIGNAL(clicked()), SLOT(slotUpdate())) ;
		bUpdate->setEnabled (canFix) ;
	}
}

/*  KBTestErrorDlg							*/
/*  slotUpdate	: User clicks update button				*/
/*  (returns)	: void		:					*/

void	KBTestErrorDlg::slotUpdate ()
{
	done	((int)KBTest::ErrorUpdateTest) ;
}

/*  KBTestErrorDlg							*/
/*  slotContinue: User clicks continue button				*/
/*  (returns)	: void		:					*/

void	KBTestErrorDlg::slotContinue ()
{
	done	((int)KBTest::ErrorPauseTest ) ;
}

/*  KBTestErrorDlg							*/
/*  slotAbortAll: User clicks abort-all button				*/
/*  (returns)	: void		:					*/

void	KBTestErrorDlg::slotAbortAll ()
{
	done	((int)KBTest::ErrorAbortAll  ) ;
}

/*  KBTestErrorDlg							*/
/*  slotAbortSuite							*/
/*		: User clicks abort-suite button				*/
/*  (returns)	: void		:					*/

void	KBTestErrorDlg::slotAbortSuite ()
{
	done	((int)KBTest::ErrorAbortSuite) ;
}

/*  KBTestErrorDlg							*/
/*  slotAbortTest							*/
/*		: User clicks abort-test button				*/
/*  (returns)	: void		:					*/

void	KBTestErrorDlg::slotAbortTest ()
{
	done	((int)KBTest::ErrorAbortTest ) ;
}

/*  KBTestErrorDlg							*/
/*  reject	: Dialog rejected					*/
/*  (returns)	: void		:					*/

void	KBTestErrorDlg::reject()
{
	/* Override since "cancel" does not return a zero result.	*/
	done	((int)KBTest::ErrorAbortTest ) ;
}

