/***************************************************************************
    file	         : kb_testsuite.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_attr.h"

#include	"kb_form.h"
#include	"kb_docroot.h"
#include	"kb_test.h"
#include	"kb_testsuite.h"
#include	"kb_nodereg.h"
#include	"kb_macro.h"
#include	"kb_testsuiteresultsdlg.h"

#include	"tk_messagebox.h"

#define		__DEBUG_TESTSUITE	(0)
#if		__DEBUG_TESTSUITE
#include	"kb_dprintf.h"
#endif


/*  KBTestSuite								*/
/*  KBTestSuite	: Constructor for test suite node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: Attribute list			*/
/*		: bool *	:					*/
/*  (returns)	: KBTestSuite	:					*/

KBTestSuite::KBTestSuite
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*
	)
	:
	KBNode		(parent, "KBTestSuite",	aList),
	m_transaction	(this,	 "transaction",	aList),
	m_maxErrors	(this,	 "maxErrors",	aList),
	m_initialise	(this,	 "initialise",	aList),
	m_setup		(this,	 "setup",	aList),
	m_teardown	(this,	 "teardown",	aList),
	m_reset		(this,	 "reset",	aList),
	m_testList	(this,	 "testList",	aList)
{
}

/*  KBTestSuite								*/
/*  KBTestSuite	: Constructor for test suite node			*/
/*  parent	: KBNode *	  : Parent node				*/
/*  extant	: KBTestSuite *	  : Extant test suite			*/
/*  (returns)	: KBTestSuite	  :					*/

KBTestSuite::KBTestSuite
	(	KBNode		*parent,
		KBTestSuite		*extant
	)
	:
	KBNode		(parent, "KBTestSuite"),
	m_transaction	(this,	 "transaction",	extant),
	m_maxErrors	(this,	 "maxErrors",	extant),
	m_initialise	(this,	 "initialise",	extant),
	m_setup		(this,	 "setup",	extant),
	m_teardown	(this,	 "teardown",	extant),
	m_reset		(this,	 "reset",	extant),
	m_testList	(this,	 "testList",	extant)
{
}

/*  KBTestSuite								*/
/*  KBTestSuite	: Destructor for test suite node			*/
/*  (returns)	:		:					*/

KBTestSuite::~KBTestSuite ()
{
}

/*  KBTestSuite								*/
/*  replicate	: Replicate this test suite				*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New choice node			*/

KBNode	*KBTestSuite::replicate
	(	KBNode	*_parent
	)
{
	return	new KBTestSuite (_parent, this) ;
}

#if	! __KB_RUNTIME

bool	KBTestSuite::transaction()
{
	return	m_transaction.getBoolValue () ;
}

uint	KBTestSuite::maxErrors	()
{
	return	m_maxErrors.getIntValue () ;
}

QString	KBTestSuite::initialise ()
{
	return	m_initialise.getValue () ;
}


QString	KBTestSuite::setup ()
{
	return	m_setup.getValue () ;
}

QString	KBTestSuite::teardown ()
{
	return	m_teardown.getValue () ;
}

QString	KBTestSuite::reset ()
{
	return	m_reset.getValue () ;
}

QString	KBTestSuite::testList ()
{
	return	m_testList.getValue () ;
}

void	KBTestSuite::setTransaction
	(	bool		tranaction
	)
{
	m_transaction.setValue (tranaction) ;
}

void	KBTestSuite::setMaxErrors
	(	uint		maxErrors
	)
{
	m_maxErrors.setValue (maxErrors) ;
}

void	KBTestSuite::setTestList
	(	const QString	&listList
	)
{
	m_testList.setValue (listList) ;
}

void	KBTestSuite::setInitialise
	(	const QString	&initialise
	)
{
	m_initialise.setValue (initialise) ;
}

void	KBTestSuite::setSetup
	(	const QString	&setup
	)
{
	m_setup.setValue (setup) ;
}

void	KBTestSuite::setTeardown
	(	const QString	&teardown
	)
{
	m_teardown.setValue (teardown) ;
}

void	KBTestSuite::setReset
	(	const QString	&reset
	)
{
	m_reset.setValue (reset) ;
}

KBTest	*KBTestSuite::findTest
	(	const QString		&name
	)
{
	QStringList  bits  = QStringList::split(".", name, true) ;
	QString	     cname = bits[0] ;
	int	     sep   = cname.find ('/') ;

	if (sep >= 0) cname = cname.mid (sep + 1) ;

	KBNode	     *node = getParent()->getNamedNode (cname) ;
	if (node == 0)
	{
		TKMessageBox::sorry
		(	0,
			QString	(TR("Test suite %1: test '%2' not found"))
				.arg(getName())
				.arg(name),
			TR("Test suite: missing test")
		)	;
		return	0 ;
	}

	KBTest	    *test = 0 ;
	LITER
	(	KBTest,
		node->getTests(),
		t,

		if (t->getName() == bits[1])
		{	test	= t ;
			break	;
		}
	)
	if (test == 0)
	{
		TKMessageBox::sorry
		(	0,
			QString	(TR("Test suite %1: test '%2' not found"))
				.arg(getName())
				.arg(name),
			TR("Test suite: missing test")
		)	;
		return	0 ;
	}

	return	test	;
}

KBTest::ErrorOpt
	KBTestSuite::executeTest
	(	KBTest::ErrorOpt	errorOpt,
		KBTest			*test
	)
{
	DPRINTF
	((	"KBTestSuite::executeTest: %p: [%s]\n",
		(void *)test,
		test == 0 ? "" : test->getName().latin1()
	))	;

	if (test != 0)
	{
		KBValue		resval	;
		KBScriptError	*rc	= test->executeTest (resval, errorOpt, 0, 0, false) ;
		if (rc != 0)
		{
			delete	rc	;
			return	errorOpt;
		}
	}

	return	KBTest::ErrorOK	;
}


void	KBTestSuite::executeStart
	(	KBDocRoot	*docRoot
	)
{
	KBTest::setTestMode (KBScriptIF::testSuite) ;
}

void	KBTestSuite::executeFinish
	(	KBDocRoot	*docRoot
	)
{
	KBTest::setTestMode (KBScriptIF::testOff  ) ;
}

KBTest::ErrorOpt
	KBTestSuite::executeTests
	(	KBTest::ErrorOpt	errorOpt,
		bool			reload
	)
{

	QStringList tests	= QStringList::split (",", m_testList.getValue()) ;
	KBTest	    *setup	= 0 ;
	KBTest	    *teardown	= 0 ;

/**/	reload	= false	;

	if (!m_setup   .getValue().isEmpty())
		if ((setup    = findTest ("." + m_setup   .getValue())) == 0)
			return	KBTest::ErrorOK	;
	if (!m_teardown.getValue().isEmpty())
		if ((teardown = findTest ("." + m_teardown.getValue())) == 0)
			return	KBTest::ErrorOK	;

	for (uint idx = 0 ; idx < tests.count() ; idx += 1)
	{
		KBTest	*test	= findTest (tests[idx]) ;
		if (test == 0) break	;

		if (reload)
		{
			QDict<QString>	dummy	;
			QSize		size	;
			KBForm		*form	= test->getOwner()->getForm() ;

			if (form != 0)
				form->showData (0, dummy, KBValue(), size) ;
		}

		KBTest::ErrorOpt rc ;

		switch (rc = executeTest (errorOpt, setup   ))
		{
			case KBTest::ErrorAbortAll	:
			case KBTest::ErrorAbortSuite	: return   rc ;
			case KBTest::ErrorAbortTest	: continue ;
			default				: break	   ;
		}

		switch (rc = executeTest (errorOpt, test    ))
		{
			case KBTest::ErrorAbortAll	:
			case KBTest::ErrorAbortSuite	: return   rc ;
			case KBTest::ErrorAbortTest	: continue ;
			default				: break	   ;
		}

		switch (rc = executeTest (errorOpt, teardown))
		{
			case KBTest::ErrorAbortAll	:
			case KBTest::ErrorAbortSuite	: return   rc ;
			case KBTest::ErrorAbortTest	: continue ;
			default				: break	   ;
		}

		KBTest::appendTestResult
			(	KBScriptTestResult
                                        (	QString("::%1").arg(test->getName()),
						0,
						test->comment(),
						KBScriptTestResult::testOK,
						QString::null,
						"",
						QString::null
			)	)	;
	}

	return	KBTest::ErrorOK	;
}

bool	KBTestSuite::executeResults
	(	KBDocRoot		*docRoot,
		KBTestSuiteResultsDlg	&tsrDlg
	)
{
	return	tsrDlg.addResults (KBTest::getTestResults ()) ;
}

void	KBTestSuite::execute
	(	KBTest::ErrorOpt	errorOpt,
		bool			reload
	)
{
	KBTestSuiteResultsDlg tsrDlg ;

	executeStart	(getParent()->isDocRoot()) ;
	executeTests	(errorOpt, reload)	   ;
	executeResults	(getParent()->isDocRoot(), tsrDlg) ;
	executeFinish	(getParent()->isDocRoot()) ;
	
	tsrDlg.exec	()	 ;
}

#endif

NEWNODE(TestSuite, (cchar *)0, 0, KNF_FORM)
