/***************************************************************************
    file	         : kb_testsuitedlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qcheckbox.h>
#include	<qspinbox.h>
#include	<qlabel.h>


#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_combobox.h"
#include	"rk_listbox.h"
#include	"rk_listview.h"

#if		__KB_AUTOMOC
#include	"kb_testsuitedlg.moc"
#else
#include 	"kb_testsuitedlg.h"
#endif

#include	"kb_node.h"
#include	"kb_form.h"
#include	"kb_test.h"
#include	"kb_testsuite.h"

#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"


/*  KBTestSuiteListItem							*/
/*  -------------------							*/
/*  Entry for the test suite list, stores the transaction, errors and	*/
/*  test list, as well as the test suite name, which is displayed in	*/
/*  the list box.							*/

class	KBTestSuiteListItem : public QListBoxText
{
	QString		m_name		;
	bool		m_transaction	;
	uint		m_maxErrors	;
	QString		m_initialise	;
	QString		m_setup		;
	QString		m_teardown	;
	QString		m_reset		;
	QString		m_testList	;

public	:

	KBTestSuiteListItem
		(	RKListBox	*, 
			const QString	&,
			bool		,
			uint		,
			const QString	&,
			const QString	&,
			const QString	&,
			const QString	&,
			const QString	&
		)	;

	void	update
		(	const QString	&,
			bool		,
			uint		,
			const QString	&,
			const QString	&,
			const QString	&,
			const QString	&,
			const QString	&
		)	;

	inline	QString	name		()	{ return m_name		; }
	inline	bool	transaction	()	{ return m_transaction	; }
	inline	uint	maxErrors	()	{ return m_maxErrors	; }
	inline	QString	initialise	()	{ return m_initialise	; }
	inline	QString	setup		()	{ return m_setup	; }
	inline	QString	teardown	()	{ return m_teardown	; }
	inline	QString	reset		()	{ return m_reset	; }
	inline	QString	testList	()	{ return m_testList	; }
}	;


/*  KBTestSuiteListItem							*/
/*  KBTestSuiteListItem							*/
/*		: Constructor for test suite list item			*/
/*  parent	: RKListBox *	 	: Parent list box		*/
/*  name	: const QString &	: Test suite name		*/
/*  transaction	: bool		 	: Run test suite in transaction	*/
/*  maxErrors	: uint		 	: Maximum errors allowed	*/
/*  initialise	: const QString &	: Test initialise		*/
/*  setup	: const QString &	: Test setup			*/
/*  teardown	: const QString &	: Test teardown			*/
/*  reset	: const QString &	: Test reset			*/
/*  testList	: const QString &	: Comma-separated test list	*/
/*  (returns)	: KBTestSuiteListItem	:				*/

KBTestSuiteListItem::KBTestSuiteListItem
	(	RKListBox	*parent,
		const QString	&name,
		bool		transaction,
		uint		maxErrors,
		const QString	&initialise,
		const QString	&setup,
		const QString	&teardown,
		const QString	&reset,
		const QString	&testList
	)
	:
	QListBoxText	(parent, name),
	m_name		(name),
	m_transaction	(transaction),
	m_maxErrors	(maxErrors),
	m_initialise	(initialise),
	m_setup		(setup),
	m_teardown	(teardown),
	m_reset		(reset),
	m_testList	(testList)
{
}

/*  KBTestSuiteListItem							*/
/*  update	: Update item, used after editing			*/
/*  name	: const QString &	: Test suite name		*/
/*  transaction	: bool		 	: Run test suite in transaction	*/
/*  maxErrors	: uint		 	: Maximum errors allowed	*/
/*  initialise	: const QString &	: Test initialise		*/
/*  setup	: const QString &	: Test setup			*/
/*  teardown	: const QString &	: Test teardown			*/
/*  reset	: const QString &	: Test reset			*/
/*  testList	: const QString &	: Comma-separated test list	*/
/*  (returns)	: void			:				*/

void	KBTestSuiteListItem::update
	(	const QString	&name,
		bool		transaction,
		uint		maxErrors,
		const QString	&initialise,
		const QString	&setup,
		const QString	&teardown,
		const QString	&reset,
		const QString	&testList
	)
{
	m_name		= name		;
	m_transaction	= transaction	;
	m_maxErrors	= maxErrors	;
	m_initialise	= initialise	;
	m_setup		= setup		;
	m_teardown	= teardown	;
	m_reset		= reset		;
	m_testList	= testList	;

	setText	(name)	;
}


/*  ------------------------------------------------------------------	*/

/*  KBTestSuiteList							*/
/*  KBTestSuiteList							*/
/*		 : Test suite list, used as component in prop. dialogs	*/
/*  parent	 : QWidget *		   : Parent widget		*/
/*  form	 : KBForm *		   : Parent form		*/
/*  testSuiteList: QPtrList<KBTestSuite> & : List of test suites	*/
/*  (returns)	 : KBTestSuiteList	   :				*/

KBTestSuiteList::KBTestSuiteList
	(	QWidget			*parent,
		KBForm			*form,
		QPtrList<KBTestSuite>	&testSuiteList
	)
	:
	RKHBox	(parent),
	m_root	(form)
{
	m_testSuiteList	= new RKListBox (this)	;

	RKVBox	*btns	= new RKVBox	(this)	;
	m_bAdd		= new RKPushButton (TR("Add"),    btns) ;
	m_bEdit		= new RKPushButton (TR("Edit"),   btns) ;
	m_bRemove	= new RKPushButton (TR("Remove"), btns) ;
	btns->addFiller () ;

	connect	(m_bAdd,    SIGNAL(clicked()), SLOT(clickAdd   ())) ;
	connect	(m_bEdit,   SIGNAL(clicked()), SLOT(clickEdit  ())) ;
	connect	(m_bRemove, SIGNAL(clicked()), SLOT(clickRemove())) ;

	connect
	(	m_testSuiteList,
		SIGNAL	(doubleClicked	(QListBoxItem *)),
		SLOT	(clickEdit	())
	)	;

	LITER
	(	KBTestSuite,
		testSuiteList,
		t,

		new KBTestSuiteListItem
			(	m_testSuiteList,
				t->getName    (),
				t->transaction(),
				t->maxErrors  (),
				t->initialise (),
				t->setup      (),
				t->teardown   (),
				t->reset      (),
				t->testList   ()
			)
	)	;
}

/*  KBTestSuiteList							*/
/*  clickAdd	: Request to add a new test suite			*/
/*  (returns)	: void		:					*/

void	KBTestSuiteList::clickAdd ()
{
	KBTestSuiteDlg	tsDlg
			(	m_root,
				QString::null,
				false,
				0,
				QString::null,
				QString::null,
				QString::null,
				QString::null,
				QString::null
			)	;

	if (tsDlg.exec())
		new KBTestSuiteListItem
			(	m_testSuiteList,
				tsDlg.name    	 (),
				tsDlg.transaction(),
				tsDlg.maxErrors  (),
				tsDlg.initialise (),
				tsDlg.setup      (),
				tsDlg.teardown   (),
				tsDlg.reset      (),
				tsDlg.testList   ()
			)	;
}

/*  KBTestSuiteList							*/
/*  clickEdit	: Request to edit an extant test suite			*/
/*  (returns)	: void		:					*/

void	KBTestSuiteList::clickEdit ()
{
	KBTestSuiteListItem *curItem = (KBTestSuiteListItem *)m_testSuiteList->selectedItem() ;
	if (curItem == 0) return ;

	KBTestSuiteDlg	tsDlg
			(	m_root,
				curItem->name	    (),
				curItem->transaction(),
				curItem->maxErrors  (),
				curItem->initialise (),
				curItem->setup	    (),
				curItem->teardown   (),
				curItem->reset      (),
				curItem->testList   ()
			)	;

	if (tsDlg.exec())
		curItem->update
			(	tsDlg.name    	 (),
				tsDlg.transaction(),
				tsDlg.maxErrors  (),
				tsDlg.initialise (),
				tsDlg.setup      (),
				tsDlg.teardown   (),
				tsDlg.reset      (),
				tsDlg.testList   ()
			)	;
}

/*  KBTestSuiteList							*/
/*  clickEdit	: Request to remove an extant test suite		*/
/*  (returns)	: void		:					*/

void	KBTestSuiteList::clickRemove ()
{
	QListBoxItem *curItem = m_testSuiteList->selectedItem() ;
	if (curItem != 0) delete curItem ;
}

/*  KBTestSuiteList							*/
/*  fixUp	: Fix up test suites when user OK's property dialog	*/
/*  extant	: QPtrList<KBTestSuite> & : Extant test suites		*/
/*  (returns)	: void		          :				*/

void	KBTestSuiteList::fixUp
	(	QPtrList<KBTestSuite>	&extant
	)
{
	for (uint idx1 = 0 ; idx1 < extant.count() ; idx1 += 1)
		delete	extant.at(idx1) ;

	for (uint idx2 = 0 ; idx2 < m_testSuiteList->count() ; idx2 += 1)
	{
		KBTestSuiteListItem  *item  = (KBTestSuiteListItem *)m_testSuiteList->item(idx2) ;
		QDict<QString>	     dummy  ;
		dummy.setAutoDelete  (true) ;
		dummy.insert 	     ("name", new QString(item->name())) ;

		KBTestSuite	     *suite = new KBTestSuite (m_root, dummy) ;

		suite->setTransaction (item->transaction()) ;
		suite->setMaxErrors   (item->maxErrors  ()) ;
		suite->setInitialise  (item->initialise ()) ;
		suite->setSetup       (item->setup      ()) ;
		suite->setTeardown    (item->teardown   ()) ;
		suite->setReset       (item->reset      ()) ;
		suite->setTestList    (item->testList   ()) ;
	}
}

/*  KBTestSuiteList							*/
/*  getText	: Get text for properties list				*/
/*  (returns)	: QString	:					*/

QString	KBTestSuiteList::getText ()
{
	QStringList suites ;
	for (uint idx = 0 ; idx < m_testSuiteList->count() ; idx += 1)
		suites.append (m_testSuiteList->text(idx)) ;
	return  suites.join (", ") ;
}

/*  ------------------------------------------------------------------	*/

/*  KBTestSuiteDlgItem							*/
/*  ------------------							*/
/*  The test suite dialog shows a tree of controls and tests. This	*/
/*  class is used for entries, and stores the node and test.		*/

class	KBTestSuiteDlgItem : public QListViewItem
{
	KBNode		*m_node	;
	KBTest		*m_test	;

public	:

	KBTestSuiteDlgItem	(RKListView    *, KBNode *) ;
	KBTestSuiteDlgItem	(QListViewItem *, KBNode *) ;
	KBTestSuiteDlgItem	(QListViewItem *, KBNode *, KBTest *) ;

	inline	KBNode		*node	() { return m_node ; }
	inline	KBTest		*test	() { return m_test ; }
}	;


KBTestSuiteDlgItem::KBTestSuiteDlgItem
	(	RKListView	*parent,
		KBNode		*node
	)
	:
	QListViewItem	(parent, node->getName()),
	m_node		(node),
	m_test		(0)
{
}

KBTestSuiteDlgItem::KBTestSuiteDlgItem
	(	QListViewItem	*parent,
		KBNode		*node
	)
	:
	QListViewItem	(parent, node->getName()),
	m_node		(node),
	m_test		(0)
{
}

KBTestSuiteDlgItem::KBTestSuiteDlgItem
	(	QListViewItem	*parent,
		KBNode		*node,
		KBTest		*test
	)
	:
	QListViewItem	(parent, QString::null, test->getName()),
	m_node		(node),
	m_test		(test)
{
}

/*  ------------------------------------------------------------------	*/

/*  KBTestSuiteDlg							*/
/*  KBTestSuiteDlg							*/
/*		: Constructor for test suite dialog			*/
/*  (returns)	: KBTestSuiteDlg :					*/

KBTestSuiteDlg::KBTestSuiteDlg
	(	KBForm		*form,
		const QString	&name,
		bool		transaction,
		uint		maxErrors,
		const QString	&initialise,
		const QString	&setup,
		const QString	&teardown,
		const QString	&reset,
		const QString	&testList
	)
	:
	KBDialog	(TR("Test Suite"), true, "kbtestsuitedlg"),
	m_root		(form)
{
	RKVBox	*lay	= new RKVBox	   (this) ;
	lay->setTracking () ;

	RKGridBox *grid	= new RKGridBox	 (2, lay) ;
	new QLabel  (TR("Test suite name"), grid) ;
	m_name		= new RKLineEdit   (grid) ;
	new QLabel  (TR("Initialise"	 ), grid) ;
	m_initialise	= new RKComboBox   (grid) ;
	new QLabel  (TR("Setup"		 ), grid) ;
	m_setup		= new RKComboBox   (grid) ;
	new QLabel  (TR("Tear down"	 ), grid) ;
	m_teardown	= new RKComboBox   (grid) ;
	new QLabel  (TR("Reset"		 ), grid) ;
	m_reset		= new RKComboBox   (grid) ;

	m_tests		= new RKListBox    (lay ) ;

	connect
	(	m_name,
		SIGNAL	(textChanged (const QString &)),
		SLOT	(setButtons  ())
	)	;
	connect
	(	m_tests,
		SIGNAL	(highlighted (QListBoxItem  *)),
		SLOT	(setButtons  ())
	)	;

	RKHBox	*btns	= new RKHBox       (lay ) ;
	m_bAdd		= new RKPushButton (TR("Add"),	  btns) ;
	m_bRemove	= new RKPushButton (TR("Remove"), btns) ;
	m_bUp		= new RKPushButton (TR("Up"), 	  btns) ;
	m_bDown		= new RKPushButton (TR("Down"),   btns) ;

	RKPushButton *e	= new RKPushButton (TR("Expand"), btns) ;

	connect	(m_bAdd,    SIGNAL(clicked()), SLOT(clickAdd   ())) ;
	connect	(m_bRemove, SIGNAL(clicked()), SLOT(clickRemove())) ;
	connect	(m_bUp,     SIGNAL(clicked()), SLOT(clickUp    ())) ;
	connect	(m_bDown,   SIGNAL(clicked()), SLOT(clickDown  ())) ;
	connect	(e,	    SIGNAL(clicked()), SLOT(clickExpand())) ;

	m_allTests	= new RKListView   (lay ) ;
	m_allTests->addColumn (TR("Object"), 100) ;
	m_allTests->addColumn (TR("Test"  ), 100) ;
	m_allTests->setRootIsDecorated (true) ;
	m_allTests->setColumnWidthMode (0, QListView::Maximum) ;
	m_allTests->setColumnWidthMode (1, QListView::Maximum) ;
	m_allTests->setResizeMode      (QListView::LastColumn) ;

	connect
	(	m_allTests,
		SIGNAL(selectionChanged	(QListViewItem *)),
		SLOT  (selectionChanged	(QListViewItem *))
	)	;
	connect
	(	m_allTests,
		SIGNAL(doubleClicked	(QListViewItem *)),
		SLOT  (clickAdd		())
	)	;
	connect
	(	m_allTests,
		SIGNAL(returnPressed	(QListViewItem *)),
		SLOT  (clickAdd		())
	)	;

	findAllTests	(m_root, (QListViewItem *)0) ;

	RKHBox	*opts	= new RKHBox       (lay ) ;
	m_transaction	= new QCheckBox	   (TR("Run in transaction"), opts) ;
	opts->addFiller () ;
	new QLabel	(TR("Max. errors"), opts) ;
	m_maxErrors	= new QSpinBox	   (opts) ;

	addOKCancel (lay, &m_bOK) ;

	m_name		->setText	   (name) ;

	QStringList testNames ;
	LITER
	(	KBTest,
		form->getTests(),
		t,
		testNames.append(t->getName())
	)

	testNames.sort   () ;
	testNames.prepend(QString::null) ;

	m_initialise	->insertStringList (testNames) ;
	m_setup     	->insertStringList (testNames) ;
	m_teardown  	->insertStringList (testNames) ;
	m_reset     	->insertStringList (testNames) ;

	m_initialise	->setCurrentByText (initialise ) ;
	m_setup		->setCurrentByText (setup      ) ;
	m_teardown	->setCurrentByText (teardown   ) ;
	m_reset		->setCurrentByText (reset      ) ;

	m_tests		->insertStringList (QStringList::split (",", testList)) ;
	m_transaction	->setChecked	   (transaction) ;
	m_maxErrors	->setValue	   (maxErrors  ) ;

	setButtons ()	;
}

void	KBTestSuiteDlg::expandAll
	(	QListViewItem	*item
	)
{
	if (item == 0) return ;

	item->setOpen (true) ;

	for
	(	QListViewItem *child = item->firstChild() ;
		child != 0 ;
		child  = child->nextSibling()
	)
		expandAll (child) ;
}

/*  KBTestSuiteDlg							*/
/*  findAllTests: Recursive tree scanned to find all tests		*/
/*  node	: KBNode *	  : Node to scan			*/
/*  parent	: QListViewItem * : Parent in tree or null at top	*/
/*  (returns)	: void		  :					*/

void	KBTestSuiteDlg::findAllTests
	(	KBNode		*node,
		QListViewItem	*parent
	)
{
	const QPtrList<KBTest>  &tests	  = node->getTests    () ;
	const QPtrList<KBNode>  &children = node->getChildren () ;

	if ((tests.count() == 0) && (children.count() == 0))
		return	;

	KBTestSuiteDlgItem *item =
		parent == 0 ?
			new KBTestSuiteDlgItem	(m_allTests, node) :
			new KBTestSuiteDlgItem	(parent,     node) ;


	LITER
	(	KBTest,
		tests,
		t,

		new KBTestSuiteDlgItem (item, node, t) ;
	)

	LITER
	(	KBNode,
		children,
		n,

		findAllTests (n, item)
	)
}

/*  KBTestSuiteDlg							*/
/*  clickAdd	: User clicks add button				*/
/*  (returns)	: void		:					*/

void	KBTestSuiteDlg::clickAdd ()
{
	KBTestSuiteDlgItem *curItem = (KBTestSuiteDlgItem *)m_allTests->currentItem() ;
	if ((curItem != 0) && (curItem->test() != 0))
	{
		m_tests->insertItem
		(	curItem->node()->getPath(false) + "." + curItem->test()->getName()
		)	;
		setButtons () ;
	}

}

/*  KBTestSuiteDlg							*/
/*  clickRemove	: User clicks remove button				*/
/*  (returns)	: void		:					*/

void	KBTestSuiteDlg::clickRemove ()
{
	QListBoxItem *curItem = m_tests->selectedItem() ;
	if (curItem != 0) delete curItem ;
	setButtons () ;
}

/*  KBTestSuiteDlg							*/
/*  clickUp	: User clicks move-up button				*/
/*  (returns)	: void		:					*/

void	KBTestSuiteDlg::clickUp ()
{
	int	dCur	= m_tests->currentItem () ;

	if (dCur > 0)
	{
		QListBoxItem *item = m_tests->item (dCur);

		m_tests->takeItem	(item)		 ;
		m_tests->insertItem	(item, dCur - 1) ;
		m_tests->setCurrentItem	(dCur - 1)	 ;
		setButtons		()		 ;
	}
}

/*  KBTestSuiteDlg							*/
/*  clickDown	: User clicks move-down button				*/
/*  (returns)	: void		:					*/

void	KBTestSuiteDlg::clickDown ()
{
	int	dCur	= m_tests->currentItem () ;

	if (dCur < (int)m_tests->count() - 1)
	{
		QListBoxItem *item = m_tests->item (dCur);

		m_tests->takeItem	(item)		 ;
		m_tests->insertItem	(item, dCur + 1) ;
		m_tests->setCurrentItem	(dCur + 1)	 ;
		setButtons		()		 ;
	}
}

void	KBTestSuiteDlg::clickExpand ()
{
	expandAll (m_allTests->firstChild()) ;
}

/*  KBTestSuiteDlg							*/
/*  setButtons	: Set buttons state following change			*/
/*  (returns)	: void		:					*/

void	KBTestSuiteDlg::setButtons ()
{
	/* This is really inefficient. No need to do this so often but	*/
	/* it keeps the code simple.					*/
	QListBoxItem *item = m_tests->selectedItem() ;

	m_bRemove->setEnabled (item != 0) ;
	m_bUp	 ->setEnabled (m_tests->currentItem() > 0) ;
	m_bDown	 ->setEnabled (m_tests->currentItem() < (int)m_tests->count() - 1) ;

	m_bOK	 ->setEnabled ((m_tests->count() > 0) && !m_name->text().isEmpty()) ;
}

/*  KBTestSuiteDlg							*/
/*  selectionChanged							*/
/*		: User changes selection				*/
/*  item	: QListViewItem *	: Newly selected item		*/

void	KBTestSuiteDlg::selectionChanged
	(	QListViewItem 	*item
	)
{
	if ((item != 0) && (((KBTestSuiteDlgItem *)item)->test() != 0))
		m_bAdd->setEnabled (true ) ;
	else	m_bAdd->setEnabled (false) ;
}

/*  KBTestSuiteDlg							*/
/*  name	: Get suite name					*/
/*  (returns)	: QString	: Suite name				*/

QString	KBTestSuiteDlg::name ()
{
	return	m_name->text ()	;
}

/*  KBTestSuiteDlg							*/
/*  transaction	: Get transaction flag					*/
/*  (returns)	: bool		: Enable transactions			*/

bool	KBTestSuiteDlg::transaction ()
{
	return	m_transaction->isChecked () ;
}

/*  KBTestSuiteDlg							*/
/*  maxErrors	: Get maximum allowed errors				*/
/*  (returns)	: uint		: Maximum errors			*/

uint	KBTestSuiteDlg::maxErrors ()
{
	return	m_maxErrors->value () ;
}

QString	KBTestSuiteDlg::setup ()
{
	return	m_setup->currentText ()	;
}

QString	KBTestSuiteDlg::initialise ()
{
	return	m_initialise->currentText () ;
}

QString	KBTestSuiteDlg::teardown ()
{
	return	m_teardown->currentText () ;
}

QString	KBTestSuiteDlg::reset ()
{
	return	m_reset->currentText () ;
}

/*  KBTestSuiteDlg							*/
/*  testList	: Get test list as comma-separated strings		*/
/*  (returns)	: QString	: Test list				*/

QString	KBTestSuiteDlg::testList ()
{
	QStringList tests ;
	for (uint idx = 0 ; idx < m_tests->count() ; idx += 1)
		tests.append (m_tests->text(idx)) ;
	return  tests.join (",") ;
}

