/***************************************************************************
    file	         : kb_testsuiteresultsdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_sidepanel.h"

#include	"rk_vbox.h"
#include	"rk_listview.h"
#include	"rk_pushbutton.h"
#include	"rk_textbrowser.h"
#include	"tk_icons.h"

#if		__KB_AUTOMOC
#include	"kb_testsuiteresultsdlg.moc"
#else
#include	"kb_testsuiteresultsdlg.h"
#endif

#include	<qtable.h>
#include	<qheader.h>
#include	<qsplitter.h>


static	QString	mapScriptTestResult
	(	KBScriptTestResult::TestResult	result
	)
{
	switch (result)
	{
		case KBScriptTestResult::testOK	    :
			return	TR("OK") ;

		case KBScriptTestResult::testFailed :
			return	TR("Failed") ;

		default	:
			break	;
	}

	return	QString(TR("Unknown: %1").arg((int)result)) ;
}

/*  KBTestSuiteResultsDlg						*/
/*  KBTestSuiteResultsDlg						*/
/*		: Constructor for test suite results dialog		*/
/*  (returns)	: KBTestSuiteResultsDlg	:				*/

KBTestSuiteResultsDlg::KBTestSuiteResultsDlg ()
	:
	KBDialog (TR("Test Suite Results"), "testsuiteresultsdlg")
{
	RKVBox	*lay	= new RKVBox (this) ;
	lay->setTracking () ;

	RKHBox	*top	= new RKHBox (lay ) ;
	new KBSidePanel (top, caption()) ;
	QSplitter *res	= new QSplitter (Qt::Vertical, top) ;

	top->setStretchFactor (res, 1) ;

	m_results	= new QTable	(res)	;
	m_results->setNumCols		(10)	;
	m_results->setLeftMargin	(0)	;
	m_results->setColumnWidth	(0, 25)	;
	m_results->setColumnStretchable	(7, true) ;
	m_results->hideColumn		(1)	;
	m_results->hideColumn		(2)	;
	m_results->hideColumn		(8)	;
	m_results->hideColumn		(9)	;
	m_results->setReadOnly		(true)	;

	QHeader	*hh	= m_results->horizontalHeader () ;

	hh->setLabel	(0, TR("OK"  	 )) ;
	hh->setLabel	(1, TR("Object"  )) ;
	hh->setLabel	(2, TR("Suite"   )) ;
	hh->setLabel	(3, TR("Test"    )) ;
	hh->setLabel	(4, TR("Line"    )) ;
	hh->setLabel	(5, TR("Lang."   )) ;
	hh->setLabel	(6, TR("Result"  )) ;
	hh->setLabel	(7, TR("Message" )) ;

	connect
	(	m_results,
		SIGNAL	(currentChanged	(int, int)),
		SLOT	(showDetails	(int))
	)	;

	m_details	= new RKTextBrowser (res) ;
	m_info		= new RKTextBrowser (res) ;
	m_details->setTextFormat (RKTextBrowser::PlainText) ;
	m_info   ->setTextFormat (RKTextBrowser::PlainText) ;

	RKHBox	*btns	= new RKHBox (lay) ;
	btns->addFiller  () ;
	new RKPushButton (btns, "ok") ;

	m_errorCount	= 0 ;
}

/*  KBTestSuiteResultsDlg						*/
/*  setObject	: Set object name for subsequent results		*/
/*  object	: const QString & : Object name				*/
/*  (returns)	: void		  :					*/

void	KBTestSuiteResultsDlg::setObject
	(	const QString		&object
	)
{
	m_object = object ;
}

/*  KBTestSuiteResultsDlg						*/
/*  setSuite	: Set suite name for subsequent results			*/
/*  suite	: const QString & : Suite name				*/
/*  (returns)	: void		  :					*/

void	KBTestSuiteResultsDlg::setSuite
	(	const QString		&suite
	)
{
	m_suite  = suite  ;
}

/*  KBTestSuiteResultsDlg						*/
/*  addResults	: Add results to display				*/
/*  result	: const KBScriptTestResult *				*/
/*				: Result to be added			*/
/*  (returns)	: bool		: Result OK				*/

bool	KBTestSuiteResultsDlg::addResults
	(	const KBScriptTestResult	*result
	)
{
	QStringList bits = QStringList::split (':', result->m_location, true) ;
	QString	    msg	 = result->m_message ;

	if (msg.length() > 16) msg = msg.left(15) + " ..." ;

	DPRINTF
	((	"KBTestSuiteResultsDlg::addResults: o=[%s] s=[%s]\n",
		m_object.latin1(),
		m_suite .latin1()
	))	;

	int nr = m_results->numRows () ;
	m_results->setNumRows (nr + 1) ;

	m_results->setPixmap
	(	nr,
		0,
		result->m_result == KBScriptTestResult::testOK ?
			getBarIcon("ok") :
			getBarIcon("cancel")
	)	;

	m_results->setText  (nr, 1, m_object		) ;
	m_results->setText  (nr, 2, m_suite		) ;
	m_results->setText  (nr, 3, bits[2]		) ;
	m_results->setText  (nr, 4, result->m_result == KBScriptTestResult::testOK ? QString::null : QString::number (result->m_lineNumber)) ;
	m_results->setText  (nr, 5, result->m_language	) ;
	m_results->setText  (nr, 6, mapScriptTestResult(result->m_result    )) ;
	m_results->setText  (nr, 7, msg			) ;
	m_results->setText  (nr, 8, result->m_message	) ;
	m_results->setText  (nr, 9, result->m_comment	) ;

	if (!m_object.isEmpty()) m_results->showColumn (1) ;
	if (!m_suite .isEmpty()) m_results->showColumn (2) ;

	if (result->m_result != KBScriptTestResult::testOK)
	{
		m_errorCount += 1 ;
		return	false	  ;
	}

	return	true	;
}

/*  KBTestSuiteResultsDlg						*/
/*  addResults	: Add results to display				*/
/*  results	: const KBScriptTestResultList *			*/
/*				: List of results, may be null		*/
/*  (returns)	: bool		: All results OK			*/

bool	KBTestSuiteResultsDlg::addResults
	(	const KBScriptTestResultList	*results
	)
{
	bool	allOK	= true ;

	if (results != 0)
		for (uint idx = 0 ; idx < results->count() ; idx += 1)
			if (!addResults (&(*results)[idx]))
				allOK	= false ;

	return	allOK	;
}

void	KBTestSuiteResultsDlg::showDetails
	(	int		row
	)
{
	if (row < m_results->numRows())
	{
		m_details->setText (m_results->text (row, 8)) ;
		m_info   ->setText (m_results->text (row, 9)) ;
	}
}


uint	KBTestSuiteResultsDlg::resultCount ()
{
	return	m_results->numRows () ;
}

uint	KBTestSuiteResultsDlg::errorCount ()
{
	return	m_errorCount ;
}
