/***************************************************************************
    file	         : kb_textedit.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qtextedit.h>
#include	<qlayout.h>
#include	<qpainter.h>
#include	<qpopupmenu.h>
#include	<qtoolbutton.h>
#include	<qtooltip.h>
#include	<qcheckbox.h>
#include	<qregexp.h>


#include	"kb_highlighter.h"
#include	"kb_keymapper.h"
#include	"kb_font.h"
#include	"kb_ascii.h"

#include	"kb_textedit.moc"

#ifndef		__TEST
#include	"kb_locator.h"
#endif

#include	"tk_icons.h"
#include	"rk_vbox.h"
#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"

#define		__DEBUG_TEXTEDIT	(0)
#if		__DEBUG_TEXTEDIT
#include	"kb_dprintf.h"
#endif


/*  ------------------------------------------------------------------	*/


static	QString	*lastFind	;
static	QString	*lastReplace	;
static	QString	*lastNew	;
static	QColor	*textMarkers	;

/*  ------------------------------------------------------------------	*/

/*  KBTextEditWrapper							*/
/*  KBTextEditWrapper							*/
/*		: Constructor for text edit wrapper			*/
/*  parent	: QWidget *	    : Parent widget			*/
/*  textEdit	: KBTextEdit *	    : Associated text edit		*/
/*  (returns)	: KBTextEditWrapper :					*/

KBTextEditWrapper::KBTextEditWrapper
	(	QWidget		*parent,
		KBTextEdit	*textEdit
	)
	:
	RKTextEdit	(parent),
	m_textEdit	(textEdit)
{
	setTextFormat	(Qt::PlainText) ;
	viewport()->installEventFilter (this) ;

	m_skeleton	= 0 ;

	connect
	(	this,
		SIGNAL	(textChanged	()),
		SLOT	(slotTextChanged())
	)	;
}

/*  KBTextEditWrapper							*/
/*  ~KBTextEditWrapper							*/
/*		: Descructor for text editor wrapper			*/
/*  (returns)	:		:					*/

KBTextEditWrapper::~KBTextEditWrapper ()
{
}

/*  KBTextEditWrapper							*/
/*  showSkeleton: Show/remove skeleton button				*/
/*  show	: bool		: True to show				*/
/*  (returns)	: void		:					*/

void	KBTextEditWrapper::showSkeleton
	(	bool		show
	)
{
	if (show)
	{
		if (m_skeleton == 0)
		{
			QPixmap	icon = getSmallIcon ("rekall") ;
			m_skeleton   = new QToolButton  (this) ;
			m_skeleton->setIconSet  (icon) ;
			m_skeleton->setGeometry (10,10, icon.width(), icon.height()) ;
			m_skeleton->show ()	 ;

			connect
			(	m_skeleton,
				SIGNAL	(clicked	()),
				m_textEdit,
				SIGNAL	(skeletonClicked())
			)	;

			QToolTip::add
			(	m_skeleton,
				TR("Click to insert skeleton event code")
			)	;
		}

		return	;
	}

	DELOBJ	(m_skeleton) ;
}

/*  KBTextEditWrapper							*/
/*  slotTextChanged							*/
/*		: User changes text					*/
/*  (returns)	: void		:					*/

void	KBTextEditWrapper::slotTextChanged ()
{
	if (m_skeleton != 0)
	{
		if (text().isEmpty())
			m_skeleton->show () ;
		else	m_skeleton->hide () ;
	}
}


/*  KBTextEditWrapper							*/
/*  setText	: Set text						*/
/*  text	: const QString & : Text				*/
/*  (returns)	: void		  :					*/

void	KBTextEditWrapper::setText
	(	const QString	&text
	)
{
	RKTextEdit::setText (text) ;
	slotTextChanged	    ()	   ;
}

/*  KBTextEditWrapper							*/
/*  createPopopMenu							*/
/*		: Create context popup menu				*/
/*  pos		: const QPoint & : Cursor position			*/
/*  (returns)	: QPopupMenu *	 : Menu					*/

QPopupMenu
	*KBTextEditWrapper::createPopupMenu
	(	const QPoint	&pos
	)
{
	/* Farmed off to the parent ...					*/
	return	m_textEdit->createPopupMenu (pos) ;
}


/*  KBTextEditWrapper							*/
/*  drawContents: Redraw editor contents				*/
/*  p		: QPainter *	: Painter on which to draw		*/
/*  clipx, ...	: int, ...	: Clipping rectangle			*/
/*  (returns)	: void		:					*/

void	KBTextEditWrapper::drawContents
	(	QPainter	*p,
		int		clipx,
		int		clipy,
		int		clipw,
		int		cliph
	)
{
	RKTextEdit::drawContents (p, clipx, clipy, clipw, cliph) ;
	m_textEdit->updateLabels  () ;
	m_textEdit->updateMarkers () ;
}


/*  KBTextEditWrapper							*/
/*  eventFilter	: Event filter routine					*/
/*  o		: QObject *	: Target object				*/
/*  e		: QEvent *	: Event to process			*/
/*  (returns)	: bool		: True if consumed			*/

bool	KBTextEditWrapper::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	DPRINTF
	((	"KBTextEdit::eventFilter: type=%d\n",
		e->type()
	))	;
	if (o == viewport())
		switch (e->type())
		{
			case QEvent::MouseButtonPress	:
			case QEvent::MouseButtonRelease	:
			case QEvent::MouseButtonDblClick:
				DPRINTF
				((	"KBTextEditWrapper::eventFilter: MOUSE EVENT\n"
				))	;
				if (m_textEdit->processEvent (e)) return true ;
				break	;

			default	:
				break	;
		}

	return	RKTextEdit::eventFilter (o, e) ;
}

/*  KBTextEditWrapper							*/
/*  event	: Event handler routine					*/
/*  e		: QEvent *	: Event to process			*/
/*  (returns)	: bool		: True if consumed			*/

bool	KBTextEditWrapper::event
	(	QEvent		*e
	)
{
	DPRINTF
	((	"KBTextEdit::event: type=%d\n",
		e->type()
	))	;

	switch (e->type())
	{
		case QEvent::FocusOut		:
		case QEvent::FocusIn		:
			/* Pass these events to the key mapper which	*/
			/* may consume them.				*/
			DPRINTF
			((	"KBTextEditWrapper::event: FOCUS EVENT\n"
			))	;
			if (m_textEdit->processEvent (e)) return true ;
			break	;

		case QEvent::KeyPress		:
		case QEvent::AccelOverride	:
			/* Although these are used by the key mapper,	*/
			/* pass them from "::keyPressEvent", else we	*/
			/* seem to get three events for each key :(	*/
			DPRINTF
			((	"KBTextEditWrapper::event: KEY/ACCEL EVENT\n"
			))	;
			break	;

		default	:
			break	;
	}

	return	RKTextEdit::event (e) ;
}

/*  KBTextEditWrapper							*/
/*  keyPressEvent							*/
/*		: Key press event handler routine			*/
/*  e		: QKeyEvent *	: Event to process			*/
/*  (returns)	: bool		: True if consumed			*/

void	KBTextEditWrapper::keyPressEvent
	(	QKeyEvent	*k
	)
{
	DPRINTF
	((	"KBTextEdit::keyPressEvent: key=%d\n",
		k->key()
	))	;
	if (!m_textEdit->processEvent (k))
		if (k->key() != Qt::Key_Escape)
			RKTextEdit::keyPressEvent (k) ;
}

QPoint	KBTextEditWrapper::textCursorPoint ()
{
	int	para	;
	int	pos 	;
	getCursorPosition (&para, &pos) ;

        QPoint	point	= paragraphRect(para).bottomLeft() ;

	while (charAt(point, 0) < pos)
		point.rx() += 10 ;

	return	mapToGlobal (contentsToViewport(point));
}

/*  ------------------------------------------------------------------	*/

/*  KBTextEdit								*/
/*  KBTextEdit	: Constructor for text editor				*/
/*  parent	: QWidget *	: Parent widget				*/
/*  (returns)	: KBTextEdit	:					*/

KBTextEdit::KBTextEdit
	(	QWidget		*parent
	)
	:
	RKHBox		(parent)
{
	if (lastFind == 0)
	{
		lastFind	= new QString	;
		lastReplace	= new QString	;
		lastNew		= new QString	;

		textMarkers	= new QColor[4]	;
		textMarkers[0]	= Qt::blue	;
		textMarkers[1]	= Qt::red	;
		textMarkers[2]	= Qt::yellow	;
		textMarkers[3]	= Qt::darkYellow;

	}

	m_highlighter	= 0 ;
	m_mapper	= 0 ;

	m_labels	= new QFrame		(this) ;
	m_markers	= new QFrame		(this) ;

	RKVBox	*vb	= new RKVBox		(this) ;

	m_textedit	= new KBTextEditWrapper (vb, this) ;
	m_finder	= new RKHBox		(vb  ) ;
	m_replacer	= new RKHBox		(vb  ) ;

	m_findPrev	= new RKPushButton	("<<", m_finder) ;
	m_findText	= new RKLineEdit	(m_finder) ;
	m_findNext	= new RKPushButton	(">>", m_finder) ;
	m_matchCase	= new QCheckBox		(TR("Match case"), m_finder) ;

	m_replText	= new RKLineEdit	(m_replacer) ;
	m_replNew	= new RKLineEdit	(m_replacer) ;
	m_replReplace	= new RKPushButton	(TR("Replace"), m_replacer) ;
	m_replAll	= new RKPushButton	(TR("All"    ), m_replacer) ;
	m_replSkip	= new RKPushButton	(">>",   	m_replacer) ;
	m_replRegexp	= new QCheckBox		(TR("RexExp"),  m_replacer) ;

	connect
	(	m_findText,
		SIGNAL	(textChanged	(const QString &)),
		SLOT	(findTextChanged())
	)	;
	connect
	(	m_findNext,
		SIGNAL	(clicked	()),
		SLOT	(findClickNext	())
	)	;
	connect
	(	m_findPrev,
		SIGNAL	(clicked	()),
		SLOT	(findClickPrev	())
	)	;
	connect
	(	m_matchCase,
		SIGNAL	(toggled	(bool)),
		SLOT	(findCaseChanged())
	)	;

	connect
	(	m_replText,
		SIGNAL	(textChanged	(const QString &)),
		SLOT	(replTextChanged())
	)	;
	connect
	(	m_replReplace,
		SIGNAL	(clicked	 ()),
		SLOT	(replClickReplace())
	)	;
	connect
	(	m_replAll,
		SIGNAL	(clicked	()),
		SLOT	(replClickAll   ())
	)	;
	connect
	(	m_replSkip,
		SIGNAL	(clicked	()),
		SLOT	(replClickSkip	())
	)	;

	m_finder  ->hide () ;
	m_replacer->hide () ;

	m_mapper	= new KBTextEditMapper	(this) ;

	m_labels ->setFrameStyle 	(QFrame::LineEditPanel|QFrame::Sunken) ;
	m_labels ->setFixedWidth 	(0) ;
	m_labels ->setBackgroundMode	(m_textedit->backgroundMode()) ;
	m_labels ->hide	() ;

	m_markers->setFrameStyle	(QFrame::LineEditPanel|QFrame::Sunken) ;
	m_markers->setFixedWidth	(0) ;
	m_markers->setBackgroundMode	(m_textedit->backgroundMode()) ;
	m_markers->hide	() ;

	m_font		= m_textedit->font() ;
	m_spacing	= QFontMetrics(m_font).lineSpacing() ;

	m_labels 	->installEventFilter (this) ;
	m_markers	->installEventFilter (this) ;
	m_findText	->installEventFilter (this) ;
	m_replText	->installEventFilter (this) ;
	m_replNew	->installEventFilter (this) ;

	m_findText->setText (*lastFind	 ) ;
	m_replText->setText (*lastReplace) ;
	m_replNew ->setText (*lastNew	 ) ;

	connect
	(	m_textedit,
		SIGNAL	(textChanged	()),
		SLOT	(slotTextChanged())
	)	;

	setFocusProxy (m_textedit) ;
}

/*  KBTextEdit								*/
/*  ~KBTextEdit	: Destructor for text editor				*/
/*  (returns)	:		:					*/

KBTextEdit::~KBTextEdit ()
{
	if (m_highlighter != 0)
		delete m_highlighter ;
}

void	KBTextEdit::slotTextChanged ()
{
	DPRINTF
	((	"KBTextEdit::slotTextChanged: emitting textChanged()\n"
	))	;
	emit	textChanged() ;
}

/*  KBTextEdit								*/
/*  createPopupMenu							*/
/*		: Create popup menu					*/
/*  pos		: const QPoint & : Menu display position		*/
/*  (returns)	: QPopupMenu *	 : Menu					*/

QPopupMenu
	*KBTextEdit::createPopupMenu
	(	const QPoint	&pos
	)
{
	return	m_textedit->RKTextEdit::createPopupMenu (pos) ;
}

/*  KBTextEdit								*/
/*  setHighlight: Set highlight scheme					*/
/*  name	: const QString &name : Scheme name			*/
/*  (returns)	: void		      :					*/

void	KBTextEdit::setHighlight
	(	const QString	&name
	)
{
	DPRINTF
	((	"KBTextEdit::setHighlight: [%s]\n",
		name.latin1()
	))	;

	DELOBJ	(m_highlighter) ;

	if (!name.isEmpty())
		m_highlighter = new KBSyntaxHighlighter
				(	this,
					name,
					m_font
				)	;
}

/*  KBTextEdit								*/
/*  setFont	: Set the font						*/
/*  font	: const QFont &	: Font in question			*/
/*  (returns)	: void		 :					*/

void	KBTextEdit::setFont
	(	const QFont	&font
	)
{
	DPRINTF
	((	"KBTextEdit::setFont: font [%s]\n",
		(const char *)KBFont::fontToSpec(font)
	))	;

	m_font	  = font ;
	m_spacing = QFontMetrics(font).lineSpacing() ;

	m_textedit->setFont (m_font) ;
	m_labels  ->setFont (m_font) ;
	m_markers ->setFont (m_font) ;

	if (m_highlighter != 0) m_highlighter->setFont (m_font) ;
}

/*  KBTextEdit								*/
/*  reparent	: Reparent the widget					*/
/*  parent	: QWidget *	 : New parent widget			*/
/*  f		: WFlags	 : New widget flags			*/
/*  p		: const QPoint & : New position in parent		*/
/*  showIt	: bool		 : Call show after reparenting		*/
/*  (returns)	: void		 :					*/

void	KBTextEdit::reparent
	(	QWidget		*parent,
		WFlags 		f,
		const QPoint	&p,
		bool		showIt
	)
{
	DPRINTF
	((	"KBTextEdit::reparent: called .......\n"
	))	;

	/* This method is overridden in order to preserve the font over	*/
	/* a reparent (for instance, when being inserted int a tab	*/
	/* or stack widget.						*/
	QFont font = m_font ;
	QWidget::reparent (parent, f, p, showIt) ;
	setFont (font) ;
}


void	KBTextEdit::showLineNumbers
	(	bool		show,
		int		width
	)
{
	DPRINTF
	((	"KBTextEdit::showLineNumbers: width=%d mw=%d fw=%d total=%d\n",
		width,
		QFontMetrics(m_font).maxWidth(),
		m_labels ->frameWidth(),
		QFontMetrics(m_font).maxWidth() * width + m_labels ->frameWidth() * 2
	))	;

	m_labels ->setFixedWidth
	(	QFontMetrics(m_font).maxWidth() * width +
		m_labels ->frameWidth() * 2
	)	;

	if (show)
		m_labels ->show() ;
	else	m_labels ->hide() ;
}

void	KBTextEdit::showMarkers
	(	bool		show,
		int		width
	)
{
	m_markers->setFixedWidth
	(	QFontMetrics(m_font).maxWidth() * width +
		m_markers->frameWidth() * 2
	)	;

	if (show)
		m_markers->show() ;
	else	m_markers->hide() ;
}

/*  KBTextEdit								*/
/*  setMark	: Set line marker					*/
/*  lno		: unsigned int	: Line number				*/
/*  mark	: unsigned int	: Mark value				*/
/*  (returns)	: void		:					*/

void	KBTextEdit::setMark
	(	unsigned int	lno,
		unsigned int	mark
	)
{
	DPRINTF
	((	"KBTextEdit::setMark: %d->%d\n",
		lno,
		mark
	))	;

	while (m_marks.count() <= lno) m_marks.append(0) ;
	m_marks[lno] = mark ;
	updateMarkers ()    ;
}

/*  KBTextEdit								*/
/*  getMark	: Get line marker					*/
/*  lno		: unsigned int	: Line number				*/
/*  (returns)	: unsigned int	: Mark value				*/

int	KBTextEdit::getMark
	(	unsigned int	lno
	)
{
	return	lno >= m_marks.count() ? 0 : m_marks[lno] ;
}

void	KBTextEdit::updateLabels ()
{
	if (m_labels->isShown())
	{
		int	cy	= m_textedit->contentsY    () ;
		int	ch	= m_textedit->visibleHeight() ;
		int	fw	= m_labels  ->frameWidth   () ;

		int	lfrom	= cy / m_spacing ;
		int	lto	= (cy + ch) / m_spacing  ;
		int	lat	= lfrom * m_spacing - cy ;

		DPRINTF
		((	"KBTextEdit::updateLabels : cy=%d ch=%d lf=%d lt=%d\n",
			cy,
			ch,
			lfrom,
			lto
		))	;

		QPainter painter   (m_labels) ;
		painter .eraseRect
		(	fw,
			fw,
			m_labels ->width () - 2 * fw,
			m_labels ->height() - 2 * fw
		)	;

		while (lfrom <= lto)
		{
			if (lfrom > 0)
				painter.drawText
				(	m_labels->frameWidth(),
					lat,
					QString().sprintf("%5d", lfrom)
				)	;

			lfrom	+= 1 ;
			lat	+= m_spacing ;
		}
	}
}

void	KBTextEdit::updateMarkers ()
{
	if (m_markers->isShown())
	{
		int	cy	= m_textedit->contentsY    () ;
		int	ch	= m_textedit->visibleHeight() ;
		int	mw	= m_markers ->width	   () ;
		int	fw	= m_markers ->frameWidth   () ;
		int	diam	= mw - (fw + 2) * 2  ;

		uint	lfrom	= cy / m_spacing ;
		uint	lto	= (cy + ch) / m_spacing  ;
		uint	lat	= lfrom * m_spacing - cy + fw ;

		DPRINTF
		((	"KBTextEdit::updateMarkers: cy=%d ch=%d lf=%d lt=%d\n",
			cy,
			ch,
			lfrom,
			lto
		))	;
		QPainter painter   (m_markers) ;
		painter .eraseRect
		(	fw,
			fw,
			m_markers->width () - 2 * fw,
			m_markers->height() - 2 * fw
		)	;

		while (lfrom <= lto)
		{
			int	mark	= getMark (lfrom) ;

			DPRINTF
			((	"KBTextEdit::updateMarkers: lfrom=%d mark=%d\n",
				lfrom,
				mark
			))	;
			if (mark != 0)
			{
				painter.setBrush (textMarkers[mark & 0x03]) ;
				
				painter.drawEllipse
				(	fw  + 2,
					lat + m_spacing - diam - 2,
					diam,
					diam
				)	;
			}

			lfrom	+= 1 ;
			lat	+= m_spacing ;
		}
	}
}

/*  KBTextEdit								*/
/*  processEvent: Process event from editor				*/
/*  e		: QEvent *	: Event to process			*/
/*  (returns)	: bool		: True if consumed			*/

bool	KBTextEdit::processEvent
	(	QEvent		*e
	)
{
	return	m_mapper->processEvent (e) ;
}

/*  KBTextEdit								*/
/*  eventFilter	: Event filtering					*/
/*  o		: QObject *	: Receiving object			*/
/*  e		: QEvent *	: Event received by object		*/
/*  (returns)	: bool		: True if event consumed		*/

bool	KBTextEdit::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if ((o == m_labels) || (o == m_markers))
	{
		switch (e->type())
		{
			case QEvent::Paint		:
				if (o == m_labels )
				{	updateLabels () ;
					return	false	;
				}
				if (o == m_markers)
				{	updateMarkers() ;
					return	false	;
				}
				break	;

			case QEvent::MouseButtonPress	:
			case QEvent::MouseButtonRelease	:
			case QEvent::MouseButtonDblClick:
			case QEvent::ContextMenu 	:
				if (o == m_markers)
				{
					emit	clickMarkers
						(	e,
							(m_textedit->contentsY() + ((QMouseEvent *)e)->y()) / m_spacing
						)	;
					return	true	;
				}
				break	;

			default	:
				break	;
		}

		return	QWidget::eventFilter (o, e) ;
	}

	if ((o == m_findText) || (o == m_replText) || (o == m_replNew))
	{
		switch (e->type())
		{
			case QEvent::KeyPress	:
				if (((QKeyEvent *)e)->key() == Qt::Key_Escape)
				{
					showFindText    (false) ;
					showReplaceText (false) ;
					return	true ;
				}

				break	;

			default	:
				break	;
		}

		return	QWidget::eventFilter (o, e) ;
	}

	return	QWidget::eventFilter (o, e) ;
}

/*  KBTextEdit								*/
/*  textEdit	: Get underlying text edit control			*/
/*  (returns)	: RKTextEdit *	: Edit control				*/

RKTextEdit *KBTextEdit::textEdit ()
{
	return	m_textedit	;
}

/*  KBTextEdit								*/
/*  currentPara	: Get current paragraph					*/
/*  (returns)	: int		: Paragraph number			*/

int	KBTextEdit::currentPara ()
{
	int	p	;
	int	i	;
	m_textedit->getCursorPosition (&p, &i) ;
	return	p	;
}

/*  KBTextEdit								*/
/*  currentColumn: Get current column					*/
/*  (returns)	 : int		: Column number				*/

int	KBTextEdit::currentColumn ()
{
	int	p	;
	int	i	;
	m_textedit->getCursorPosition (&p, &i) ;
	return	i	;
}

int	KBTextEdit::lineHeight ()
{
	return	m_spacing ;
}

QString	KBTextEdit::currentText ()
{
	return	m_textedit->text (currentPara()) ;
}


void	KBTextEdit::setWordWrap
	(	RKTextEdit::WordWrap	wrap
	)
{
	m_textedit->setWordWrap (wrap) ;
}

void	KBTextEdit::setWrapPolicy
	(	RKTextEdit::WrapPolicy	policy
	)
{
	m_textedit->setWrapPolicy (policy) ;
}

void	KBTextEdit::setReadOnly
	(	bool		readOnly
	)
{
	m_textedit->setReadOnly (readOnly) ;
}

bool	KBTextEdit::isReadOnly ()
{
	return	m_textedit->isReadOnly () ;
}

void	KBTextEdit::setText
	(	const QString	&text
	)
{
	m_textedit->setText (text) ;
}

void	KBTextEdit::insert
	(	const QString	&text,
		uint		flags
	)
{
	m_textedit->insert (text, flags) ;
}

int	KBTextEdit::find
	(	const QString 	&text,
		bool		cs,
		bool		wo
	)
{
	if (m_textedit->find (text, cs, wo))
	{
		m_textedit->ensureCursorVisible() ;
		return	true	;
	}

	return	false	;
}

void	KBTextEdit::removeSelectedText ()
{
	DPRINTF
	((	"KBTextEdit::removeSelectedText: [%s]\n",
		m_textedit->selectedText().latin1()
	))	;
	m_textedit->removeSelectedText () ;
}

QString	KBTextEdit::text ()
{
	return	m_textedit->text () ;
}

void	KBTextEdit::moveCursor
	(	RKTextEdit::CursorAction &action,
		bool			 flag
	)
{
	m_textedit->moveCursor (action, flag) ;
}

void	KBTextEdit::delLine ()
{
	m_textedit->setSelection (currentPara(), 0, currentPara() + 1, 0) ;
	m_textedit->copy() ;
	m_textedit->del () ;
}

void	KBTextEdit::copy ()
{
	m_textedit->copy () ;
}

void	KBTextEdit::del ()
{
	m_textedit->copy() ;
	m_textedit->del () ;
}

void	KBTextEdit::paste ()
{
	m_textedit->paste () ;
}

void	KBTextEdit::undo ()
{
	m_textedit->undo () ;
}

void	KBTextEdit::redo ()
{
	m_textedit->redo () ;
}

void	KBTextEdit::clear ()
{
	m_textedit->clear () ;
}

bool	KBTextEdit::isModified ()
{
	return	m_textedit->isModified () ;
}

void	KBTextEdit::setModified
	(	bool		mod
	)
{
	m_textedit->setModified (mod) ;
}

void	KBTextEdit::setCursorPosition
	(	int		para,
		int		index
	)
{
	m_textedit->setCursorPosition (para, index) ;
}

void	KBTextEdit::getCursorPosition
	(	int		*para,
		int		*index
	)
{
	m_textedit->getCursorPosition (para, index) ;
}

QWidget	*KBTextEdit::viewport ()
{
	return	m_textedit->viewport () ;
}

void	KBTextEdit::setMimeSourceFactory
	(	QMimeSourceFactory	*sf
	)
{
	m_textedit->setMimeSourceFactory (sf) ;
}

QPoint	KBTextEdit::textCursorPoint ()
{
	return	m_textedit->textCursorPoint () ;
}

void	KBTextEdit::setSelection
	(	int		paraFrom,
		int		indexFrom,
		int		paraTo,
		int		indexTo,
		int		selNum
	)
{
	m_textedit->setSelection (paraFrom, indexFrom, paraTo, indexTo, selNum) ;
}

void	KBTextEdit::showSkeleton
	(	bool		show
	)
{
	m_textedit->showSkeleton (show) ;
}

void	KBTextEdit::showFindText
	(	bool		show
	)
{
	if (show)
	{
		m_finder  ->show    () ;
		m_findText->setFocus() ;
		return	;
	}

	m_finder  ->hide    () ;
	m_textedit->setFocus() ;
}

void	KBTextEdit::showReplaceText
	(	bool		show
	)
{
	if (show)
	{
		m_replacer->show    () ;
		m_replText->setFocus() ;
		return	;
	}

	m_replacer->hide    () ;
	m_textedit->setFocus() ;
}

/*  KBTextEdit								*/
/*  locateText	: Text search routine					*/
/*  ctrl	: RKLineEdit *	: Control containing text		*/
/*  advance	: int		: Forward/backward/text changed		*/
/*  (returns)	: bool		: Text found				*/

bool	KBTextEdit::locateText
	(	RKLineEdit	*ctrl,
		int		advance
	)
{
	QString	text	= ctrl->text() ;

	DPRINTF
	((	"KBTextEdit::locateText: [%d][%s]\n",
		advance,
		text.latin1()
	))	;

	/* If the editor has selected text then we will start the	*/
	/* search from the beginning of the selected text. The mucking	*/
	/* about is paranoia to make sure we have the start of the	*/
	/* selected text.						*/
	int	paraSet,  indexSet  ;

	if (m_textedit->hasSelectedText ())
	{
		int	paraFrom, indexFrom ;
		int	paraTo,   indexTo   ;

		m_textedit->getSelection
		(	&paraFrom,
			&indexFrom,
			&paraTo,
			&indexTo
		)	;

		if 	(paraTo < paraFrom)
		{	paraSet	 = paraTo	;
			indexSet = indexTo	;
		}
		else if	(paraFrom < paraTo)
		{	paraSet	 = paraFrom	;
			indexSet = indexFrom	;
		}
		else
		{	paraSet	 = paraFrom	;
			indexSet = indexFrom < indexTo ? indexFrom : indexTo ;
		}

		DPRINTF
		((	"KBTextEdit::findTextChanged: (%d,%d)-(%d,%d) -> (%d,%d)\n",
			paraFrom, indexFrom,
			paraTo,   indexTo,
			paraSet,  indexSet
		))	;

	}
	else	m_textedit->getCursorPosition (&paraSet, &indexSet) ;

	/* Advance is +1 for find next, -1 for find previous, and zero	*/
	/* for incremental searches.					*/
	switch (advance)
	{
		case +1 :
			indexSet += 1 ;
			if (indexSet >= m_textedit->paragraphLength (paraSet))
			{	indexSet = 0 ;
				paraSet += 1 ;
			}
			break	;

		case -1 :
			if (indexSet == 0)
			{	if (paraSet == 0) return false ;
				paraSet	 -= 1 ;
				indexSet  = m_textedit->paragraphLength(paraSet) ;
			}
			else	indexSet -= 1 ;

		default	:
			break	;
	}

	DPRINTF
	((	"KBTextEdit::locateText: from (%d,%d)\n",
		paraSet,
		indexSet
	))	;

	/* Assume the search works. If not then we will set the search	*/
	/* text colour to red below.					*/
	ctrl->setPaletteForegroundColor (Qt::black) ;


	if (advance >= 0)
		if (m_textedit->find
			(	text,
				m_matchCase->isChecked(),
				false,
				true,
				&paraSet,
				&indexSet
			))
			return	true	;

	if (advance  < 0)
		if (m_textedit->find
			(	text,
				m_matchCase->isChecked(),
				false,
				false,
				&paraSet,
				&indexSet
			))
			return	true	;
		
	ctrl->setPaletteForegroundColor (Qt::red) ;
	return	false	;
}

/*  KBTextEdit								*/
/*  findTextChanged							*/
/*		: User changes search text				*/
/*  (returns)	: void		:					*/

void	KBTextEdit::findTextChanged ()
{
	locateText (m_findText,  0) ;

	*lastFind = m_findText->text() ;
}

/*  KBTextEdit								*/
/*  findClickNext: User clicks find next				*/
/*  (returns)	 : void		:					*/

void	KBTextEdit::findClickNext ()
{
	locateText (m_findText, +1) ;

	*lastFind = m_findText->text() ;
}

/*  KBTextEdit								*/
/*  findClickPrev: User clicks find previous				*/
/*  (returns)	 : void		:					*/

void	KBTextEdit::findClickPrev ()
{
	locateText (m_findText, -1) ;

	*lastFind = m_findText->text() ;
}

/*  KBTextEdit								*/
/*  findCaseChanged							*/
/*		: Case match change					*/
/*  (returns)	: void		:					*/

void	KBTextEdit::findCaseChanged ()
{
	/* Not used at present ...					*/
}

/*  KBTextEdit								*/
/*  replTextChanged							*/
/*		: User changes replace text				*/
/*  (returns)	: void		:					*/

void	KBTextEdit::replTextChanged ()
{
	locateText (m_replText,  0) ;

	*lastReplace = m_replText->text() ;
}

/*  KBTextEdit								*/
/*  replClickReplace							*/
/*		: User clicks replace					*/
/*  (returns)	: void		:					*/

void	KBTextEdit::replClickReplace ()
{
	m_textedit->insert (m_replNew->text()) ;
	locateText (m_replText,  0) ;

	*lastReplace = m_replText->text() ;
	*lastNew     = m_replNew ->text() ;
}

/*  KBTextEdit								*/
/*  replClickAll: User clicks replace all				*/
/*  (returns)	: void		:					*/

void	KBTextEdit::replClickAll ()
{
	QString	replText = m_replText->text() ;
	QString	replNew  = m_replNew ->text() ;

	/* If regular expression replace is on then we just work on the	*/
	/* entire text in one go; no selections, no stepping.		*/
	if (m_replRegexp->isOn())
	{
		QRegExp	re	(replText, false)    ;
		QString	srce	= m_textedit->text() ;
		QString	dest	;

		int	offset1	= 0 ;
		int	offset2	;
		int	rcap	;

		while ((offset2 = re.search (srce, offset1)) >= 0)
		{
			dest	+= srce.mid (offset1, offset2 - offset1) ;

			for (uint idx = 0 ; idx < replNew.length() ; idx += 1)
			{
				if (replNew.at(idx) != '\\')
				{	dest += replNew.at(idx) ;
					continue ;
				}
				if (!replNew.at(idx+1).isDigit())
				{	dest += replNew.at(idx) ;
					continue ;
				}
				if ((rcap = (char)(QChar)replNew.at(idx+1) - '0') > re.numCaptures())
				{	dest += replNew.at(idx) ;
					continue ;
				}

				dest += re.cap(rcap) ;
				idx  += 1 ;
			}

			offset1	 = offset2 + re.cap().length() ;
		}

		dest	+= srce.mid (offset1)	;
		m_textedit->setText (dest)	;

		*lastReplace = replText		;
		*lastNew     = replNew		;
		return	;
	}

	do
	{
		m_textedit->insert (m_replNew->text()) ;
	}
	while	(locateText (m_replText, +1)) ;

	*lastReplace = replText		;
	*lastNew     = replNew		;
}

/*  KBTextEdit								*/
/*  replClickSkip: User clicks replace skip				*/
/*  (returns)	 : void		:					*/

void	KBTextEdit::replClickSkip ()
{
	locateText (m_replText, +1) ;

	*lastReplace = m_replText->text() ;
	*lastNew     = m_replNew ->text() ;
}

void	KBTextEdit::caretToEnd ()
{
	int	p = m_textedit->paragraphs () - 1   ;
	int	l = m_textedit->paragraphLength (p) ;

	m_textedit->setSelection (p, l, p, l) ;	
}

void	KBTextEdit::caretToStart ()
{
	m_textedit->setSelection (0, 0, 0, 0) ;
}

void	KBTextEdit::selectAll ()
{
	int	p = m_textedit->paragraphs	()  ;
	int	l = m_textedit->paragraphLength (p) ;

	m_textedit->setSelection (0, 0, p, l) ;	
}



#ifdef	__TEST

#include	<qapplication.h>
#include	<qtextedit.h>
#include	<qfontmetrics.h>

int	main
	(	int		argc,
		char		*argv[]
	)
{
	QApplication		app (argc, argv) ;

	new KBHLHighlighter ("cpp.hilight") ;
	new KBHLHighlighter ("python.hilight") ;

	KBTextEdit		*textEdit	= new KBTextEdit (0) ;
	QFont 			font		("courier") ;
	textEdit->setFont	  (font) ;
	textEdit->setTabStopWidth (QFontMetrics (font).maxWidth() * 8) ;
	textEdit->setWordWrap	  (RKTextEdit::NoWrap) ;
	textEdit->setHighlight	  (argc < 2 ? "CPP" : argv[1]) ;

	if (argc > 2)
	{
		QFile	f (argv[2]) ;
		f.open (IO_ReadOnly);
		textEdit->setText (f.readAll()) ;
	}

	textEdit->show		  () ;
//	textEdit->showMarkers     (true) ;
	textEdit->showLineNumbers (true) ;

	return	app.exec () ;
}

#endif
