/***************************************************************************
    file	         : kb_textedit.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_TEXTEDIT_H
#define	__KB_TEXTEDIT_H


#include 	"libkbase_exports.h"
#include	"rk_textedit.h"
#include	"rk_hbox.h"

class	KBSyntaxHighlighter	;
class	KBTextEditMapper	;
class	QEvent			;
class	KBTextEdit		;
class	QMimeSourceFactory	;
class	QToolButton		;
class	RKLineEdit		;
class	RKPushButton		;
class	RKPushButton		;
class	QCheckBox		;


/*  KBTextEditWrapper							*/
/*  ----------------							*/
/*  This is a wrapper around QTextEdit used to pass events up to our	*/
/*  KBTextEdit class.							*/

class	KBTextEditWrapper : public RKTextEdit
{
	Q_OBJECT

	KBTextEdit		*m_textEdit	;
	QToolButton		*m_skeleton	;

protected slots :

	void			slotTextChanged	() ;

protected :

	virtual	QPopupMenu	*createPopupMenu(const QPoint &) ;

public	:

	KBTextEditWrapper	(QWidget *, KBTextEdit *)	;
	~KBTextEditWrapper	() ;

	virtual	bool		eventFilter	(QObject      *, QEvent *) ;
	virtual	bool		event		(QEvent	      *) ;
	virtual	void		keyPressEvent	(QKeyEvent    *) ;
	virtual	void		drawContents	(QPainter *, int, int, int, int) ;

	QPoint			textCursorPoint	() ;
	void			setText		(const QString &) ;
	void			showSkeleton	(bool)	;

	friend	class		KBTextEdit	;
}	;


/*  KBTextEdit								*/
/*  ----------								*/
/*  This is a really a wrapper around QTextEdit and some markers which	*/
/*  handles highlighting and keymapping.				*/

class	LIBKBASE_API KBTextEdit : public RKHBox
{
	Q_OBJECT

	KBTextEditWrapper	*m_textedit	;
	QFrame			*m_labels	;
	QFrame			*m_markers	;
	RKHBox			*m_finder	;
	RKHBox			*m_replacer	;

	RKLineEdit		*m_findText	;
	RKPushButton		*m_findNext	;
	RKPushButton		*m_findPrev	;
	QCheckBox		*m_matchCase	;

	RKLineEdit		*m_replText	;
	RKLineEdit		*m_replNew	;
	RKPushButton		*m_replReplace	;
	RKPushButton		*m_replAll	;
	RKPushButton		*m_replSkip	;
	QCheckBox		*m_replRegexp	;

	KBSyntaxHighlighter	*m_highlighter	;
	KBTextEditMapper	*m_mapper	;
	QFont			m_font		;
	int			m_spacing	;
	QValueList<int>		m_marks		;

	bool		locateText	(RKLineEdit *, int) ;
	void		updateLabels	()		;
	void		updateMarkers	() 		;
	bool		processEvent	(QEvent *)	;

signals	:

	void		clickMarkers	(QEvent *, int)	;
	void		textChanged	()		;
	void		skeletonClicked	()		;

protected slots :

	void		slotTextChanged	() 		;

	void		findTextChanged	()		;
	void		findClickNext	()		;
	void		findClickPrev	()		;
	void		findCaseChanged	()		;

	void		replTextChanged	()		;
	void		replClickReplace()		;
	void		replClickAll	()		;
	void		replClickSkip	()		;

public	:

	KBTextEdit			(QWidget *)	;
	~KBTextEdit			() ;

	virtual	void	reparent	(QWidget *, WFlags, const QPoint &, bool) ;

	virtual	bool	eventFilter	(QObject *, QEvent *) ;
	void		setHighlight	(const QString &) ;
	void		setFont		(const QFont   &) ;

	void		showLineNumbers	(bool, int = 5)	;
	void		showMarkers	(bool, int = 5)	;

	void		setMark		(unsigned int, unsigned int)	 ;
	int		getMark		(unsigned int)			 ;

	int		lineHeight	() ;
	int		currentPara	() ;
	int		currentColumn	() ;
	QString		currentText	() ;
	RKTextEdit	*textEdit	() ;

	/* The remaining methods simply map through to the underlying	*/
	/* RKTextEdit methods ....					*/

	void		setWordWrap		(RKTextEdit::WordWrap  ) ;
	void		setWrapPolicy		(RKTextEdit::WrapPolicy) ;
	void		setReadOnly		(bool)	;
	bool		isReadOnly		()	;

	void		setText			(const QString &) ;
	void		insert			(const QString &, uint = QTextEdit::CheckNewLines|QTextEdit::RemoveSelected) ;
	int		find			(const QString &, bool, bool) ;
	void		removeSelectedText	() ;
	QString		text			() ;
	void		moveCursor		(RKTextEdit::CursorAction &, bool) ;
	void		delLine			() ;
	void		copy			() ;
	void		del			() ;
	void		paste			() ;
	void		undo			() ;
	void		redo			() ;
	void		clear			() ;
	QWidget		*viewport		() ;
	bool		isModified		() ;
	void		setModified		(bool) ;
	void		setCursorPosition	(int,   int  ) ;
	void		getCursorPosition	(int *, int *) ;
	void		setMimeSourceFactory	(QMimeSourceFactory *) ;
	QPoint		textCursorPoint		() ;
	void		setSelection		(int, int, int, int, int = 0) ;
	void		showSkeleton		(bool)	;

	void		caretToEnd		()	;
	void		caretToStart		()	;
	void		selectAll		()	;

	void		showFindText		(bool)	;
	void		showReplaceText		(bool)	;

	virtual	QPopupMenu	*createPopupMenu(const QPoint &) ;

	inline	KBTextEditMapper *mapper	()
	{
		return	m_mapper	;
	}

	friend	class	KBTextEditWrapper  ;
}	;

#endif	// __KB_TEXTEDIT_H
