/***************************************************************************
    file	         : kb_toolbox.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qlayout.h>
#include	<qtoolbutton.h>
#include	<qtooltip.h>
#include	<qlabel.h>

#include	"kb_node.h"
#include	"kb_nodereg.h"
#include	"kb_options.h"
#include	"kb_popupmenu.h"

#if		__KB_AUTOMOC
#include	"kb_toolbox.moc"
#else
#include	"kb_toolbox.h"
#endif

#include	"tk_icons.h"
#include	"tk_part.h"

#define		__DEBUG_TOOLBOX	(0)
#if		__DEBUG_TOOLBOX
#include	"kb_dprintf.h"
#endif


static	NodeSpec noControl =
{
	"pointer",
	"Select area",
	0,
	0,
	0,
	0,
	-1
}	;

static	NodeSpec usePopup =
{
	"popupmenu",
	"Use popup menus",
	0,
	0,
	0,
	0,
	-1
}	;

static	NodeSpec useWizard =
{
	"wizard",
	"Use design wizards",
	0,
	0,
	0,
	0,
	-1
}	;

static	NodeSpec menuBlock =
{
	"menuBlock",
	"Menu Block",
	0,
	0,
	0,
	0,
	-1
}	;

static	NodeSpec tableBlock =
{
	"tableBlock",
	"Table Block",
	0,
	0,
	0,
	0,
	-1
}	;

static	NodeSpec queryBlock =
{
	"queryBlock",
	"Query Block",
	0,
	0,
	0,
	0,
	-1
}	;

static	NodeSpec sqlBlock =
{
	"sqlBlock",
	"SQL Block",
	0,
	0,
	0,
	0,
	-1
}	;

static	NodeSpec container =
{
	"container",
	"Container",
	0,
	0,
	0,
	0,
	-1
}	;

static	NodeSpec pasteComp =
{
	"pasteComp",
	"Paste Component",
	0,
	0,
	0,
	0,
	-1
}	;

static	NodeSpec linkComp  =
{
	"linkComp",
	"Link Component",
	0,
	0,
	0,
	0,
	-1
}	;

#define	MAXCOLS	5


static	QGroupBox *newGroupBox
	(	RKVBox		*parent,
		const QString	&title
	)
{
	QGroupBox   *box = new QGroupBox  (4, Qt::Horizontal, title, parent) ;
	box   ->setMargin	(0)	;
	box   ->setFlat		(true)	;
	box   ->setInsideSpacing(0)	;
	box   ->setInsideMargin (0)	;
	return	box ;
}


/*  ------------------------------------------------------------------	*/

/*  KBToolBoxToolSet							*/
/*  KBToolBoxToolSet							*/
/*		: Constructor for toolbox tool set			*/
/*  parent	: KBtoolBoxWidget   * : Parent tool boc widget		*/
/*  type	: int		      : Type mask for this tool set	*/
/*  nodeSpecs	: KBToolBoxSpecList * : List of specs. for buttons	*/
/*  (returns)	: KBToolBoxToolSet    :					*/

KBToolBoxToolSet::KBToolBoxToolSet
	(	KBToolBoxWidget		*parent,
		int			type,
		KBToolBoxSpecList	*nodeSpecs
	)
	:
	RKVBox	(parent),
	m_type	(type)
{
	RKVBox	    *layout = new RKVBox (this) ;
	layout->setTracking () ;

	QGroupBox   *box ;

	setMargin (4) ;
	layout->setMargin (0) ;
	layout->setSpacing(0) ;

	/* First section contains the no-control button (ie., select	*/
	/* this to allow area operations), and the use-wizard toggle	*/
	/* button, and the use-popup-menu button.			*/
	box = newGroupBox  (layout, TR("Actions")) ;
	m_noCtrlButton	= addButton (&::noControl, box) ;
	m_popupButton	= addButton (&::usePopup,  box) ;
	m_wizButton	= addButton (&::useWizard, box) ;

	m_wizButton->setOn (KBOptions::getUseWizards())	;

	/* Now add tool buttons corresponding to the "block" elements	*/
	/* (which will comprise TableBlock, QueryBlock, and so forth)	*/
	box = newGroupBox  (layout, TR("Blocks" )) ;
	for (NodeSpec *spec0 = nodeSpecs->first() ; spec0 != 0 ; spec0 = nodeSpecs->next())
		if (spec0->m_flags == 0)
			addButton (spec0, box) ;


	/* Next the static controls (such as buttons and labels),	*/
	/* followed by the data controls (like fields and combos).	*/
	box = newGroupBox  (layout, TR("Static controls")) ;
	for (NodeSpec *spec1 = nodeSpecs->first() ; spec1 != 0 ; spec1 = nodeSpecs->next())
		if ((spec1->m_flags != 0) && ((spec1->m_flags & KNF_DATA) == 0))
			addButton (spec1, box) ;

	box = newGroupBox  (layout, TR("Data controls")) ;
	for (NodeSpec *spec2 = nodeSpecs->first() ; spec2 != 0 ; spec2 = nodeSpecs->next())
		if ((spec2->m_flags != 0) && ((spec2->m_flags & KNF_DATA) != 0))
			addButton (spec2, box) ;


	/* Finally, add two buttons for component pasting and linking	*/
	/* respectively.						*/
	box = newGroupBox  (layout, TR("Components")) ;
	addButton (&pasteComp, box) ;
	addButton (&linkComp,  box) ;

	m_currButton	= 0 ;
	m_currSpec	= 0 ;
}

/*  KBToolBoxToolSet							*/
/*  ~KBToolBoxToolSet							*/
/*		: Destructor for toolbox tool set			*/
/*  (returns)	:		:					*/

KBToolBoxToolSet::~KBToolBoxToolSet ()
{
}

/*  KBToolBoxToolSet							*/
/*  addButton	: Add a button						*/
/*  spec	: NodeSpec *	: Specification for button		*/
/*  group	: QGroupBox *	: Group box to which to add button	*/
/*  (returns0	: QToolButton *	: The button				*/

QToolButton
	*KBToolBoxToolSet::addButton
	(	NodeSpec	*spec,
		QGroupBox	*group
	)
{
	QToolButton *tb = new QToolButton (group) ;

	tb->setToggleButton (true) ;
	tb->setIconSet	    (QIconSet (getBarIcon (spec->m_nodeName))) ;

	QToolTip::add (tb, QString(spec->m_msgText).remove('&')) ;

	if (spec->m_popupFunc != 0)
	{
		static Qt::ButtonState bs ;
		NodeSpec    *dflt ;
		KBPopupMenu *p    = (*spec->m_popupFunc) (tb, this, &bs, &dflt) ;
		tb->setPopup	  (p) ;
		tb->setPopupDelay (0) ;
		m_specMap.insert  (tb, dflt) ;
	}
	else
	{
		m_specMap.insert  (tb, spec) ;
	}

	connect (tb, SIGNAL(pressed()), SLOT(slotPressed())) ;
	return	tb ;
}

/*  KBToolBoxToolSet							*/
/*  slotPressed	: Handle user pressing button				*/
/*  (returns)	: void		:					*/

void	KBToolBoxToolSet::slotPressed ()
{
	QToolButton *button = (QToolButton *)sender() ;

	/* Ignore the use-wizard button, this is a simple on/off toggle	*/
	/* that does not interact with the other buttons.		*/
	if (button == m_wizButton)
		return	;

	/* Note which button has been pressed, then iterate through	*/
	/* the other buttons, raising them, except that we skip the	*/
	/* use-wizard button.						*/
	m_currButton = button ;

	for
	(	QMapIterator<QToolButton *,NodeSpec *> iter = m_specMap.begin() ;
		iter != m_specMap.end() ;
		iter ++
	)
	{
		if (iter.key() == m_wizButton) continue ;

		if (iter.key() == sender())
		{
			m_currSpec = iter.data () ;
		}
		else	iter.key()->setOn (false) ;
	}

	/* If the user has pressed the no-control button then there is	*/
	/* no selected tool, so clear the current button and spec	*/
	/* settings.							*/
	if (m_currSpec == &noControl)
	{	m_currButton = 0 ;
		m_currSpec   = 0 ;
	}
}

/*  KBToolBoxToolSet							*/
/*  newNode	: New node request from popup menu			*/
/*  id		: int		: Actually NodeSpec pointer for node	*/
/*  (returns)	: void		:					*/

void	KBToolBoxToolSet::newNode
	(	int		id
	)
{
	DPRINTF
	((	"KBToolBoxToolSet::newNode: id=%08x\n",
		id
	))	;

	for
	(	QMapIterator<QToolButton *,NodeSpec *> iter = m_specMap.begin() ;
		iter != m_specMap.end() ;
		iter ++
	)
	{
		if (iter.key() != m_wizButton)
			iter.key()->setOn (false) ;
	}

	m_currSpec = idToNodeSpec(id) ;
}

/*  KBToolBoxToolSet							*/
/*  currentSpec	: Return the current specification 			*/
/*  reset	: bool		: True to reset buttons			*/
/*  (returns)	: NodeSpec *	: Current specification or null		*/

NodeSpec*KBToolBoxToolSet::currentSpec
	(	bool		reset
	)
{
	NodeSpec *spec = m_currSpec ;

	if (reset)
	{
		if (m_currButton != 0)
		{
			m_currButton->setOn (false) ;
			m_currButton = 0 ;
		}

		m_currSpec = 0   ;
		m_noCtrlButton->setOn (true) ;
	}

	return	  spec ;
}

/*  KBToolBoxToolSet							*/
/*  useWizard	: Check use-wizard state				*/
/*  (returns)	: bool		: Using wizards				*/

bool	KBToolBoxToolSet::useWizard ()
{
	return	m_wizButton->isOn () ;
}


/*  ------------------------------------------------------------------	*/


KBToolBoxWidget::KBToolBoxWidget
	(	QIntDict<KBToolBoxSpecList>	&specSets
	)
	:
	QWidgetStack
	(	0,
		"kbtoolboxwidget",
		WStyle_Customize|WStyle_Title|WStyle_NormalBorder|WStyle_Tool|WStyle_StaysOnTop
	)
{
	QIntDictIterator<KBToolBoxSpecList> iter (specSets) ;

	while (iter.current() != 0)
	{
		m_toolSets.insert
		(	iter.currentKey(),
			new KBToolBoxToolSet (this, iter.currentKey(), iter.current())
		)	;
		++iter	;
	}

	setCaption	("ToolBox")	;

	DPRINTF
	((	"KBToolBoxWidget::KBToolBoxWidget: focus policy %d\n",
		(int)focusPolicy()
	))	;
}

/*  KBToolBoxWidget							*/
/*  raiseToolSet: Raise toolset and prepare to be shown			*/
/*  type	: int		: Required toolset			*/
/*  (returns)	: bool		: Success				*/

bool	KBToolBoxWidget::raiseToolSet
	(	int		type
	)
{
	KBToolBoxToolSet *ts = m_toolSets.find (type) ;
	if (ts == 0) return false ;

	raiseWidget (ts) ;
	return	true ;
}

NodeSpec*KBToolBoxWidget::currentSpec
	(	bool		reset
	)
{
	return	((KBToolBoxToolSet *)visibleWidget())->currentSpec (reset) ;
}

bool	KBToolBoxWidget::useWizard ()
{
	return	((KBToolBoxToolSet *)visibleWidget())->useWizard () ;
}

/*  ------------------------------------------------------------------	*/

/*  KBToolBox								*/
/*  KBToolBox	: Constructor for toolbox interface class		*/
/*  (returns)	: KBToolBox	: Toolbox interface object		*/

KBToolBox::KBToolBox ()
{
	m_toolBox	= 0 ;
	m_suspendCount	= 0 ;

	/* Add the local specifications for the block tools (including	*/
	/* the container). This is needed since these will not be	*/
	/* passed to us by the node registration system.		*/
	appendNode (KNF_FORM,	&menuBlock ) ;
	appendNode (KNF_FORM,	&tableBlock) ;
	appendNode (KNF_FORM,	&queryBlock) ;
	appendNode (KNF_FORM,	&sqlBlock  ) ;
	appendNode (KNF_FORM,	&container ) ;

	appendNode (KNF_REPORT,	&menuBlock ) ;
	appendNode (KNF_REPORT,	&tableBlock) ;
	appendNode (KNF_REPORT,	&queryBlock) ;
	appendNode (KNF_REPORT,	&sqlBlock  ) ;

	if (TKPartManager::self() != 0)
		connect
		(	TKPartManager::self(),
			SIGNAL	(activePartActivated(TKPart *)),
			SLOT	(activePartActivated(TKPart *))
		)	;
}

KBToolBox::~KBToolBox ()
{
}

void	KBToolBox::suspendToolBox ()
{
	fprintf
	(	stderr,
		"KBToolBox::suspendToolBox: count=%d tb=%p\n",
		m_suspendCount,
		(void *)m_toolBox
	)	;

	if (m_suspendCount == 0)
		if (m_toolBox != 0)
		{
			m_lastHiddenAt = m_toolBox->pos() ;
			m_toolBox->hide () ;
		}

	m_suspendCount += 1 ;
}

void	KBToolBox::resumeToolBox ()
{
	fprintf
	(	stderr,
		"KBToolBox::resumeToolBox: count=%d tb=%p\n",
		m_suspendCount,
		(void *)m_toolBox
	)	;

	if (m_suspendCount <= 1)
	{
		if (m_toolBox != 0)
		{
			if (!m_lastHiddenAt.isNull()) m_toolBox->move (m_lastHiddenAt) ;
			m_toolBox->show () ;
		}

		m_suspendCount = 0 ;
		return	;
	}

	m_suspendCount -= 1 ;
}

/*  KBToolBox								*/
/*  activePartActivated							*/
/*		: Part is activated					*/
/*  part	: TKPart *	: Part in question			*/
/*  (returns)	: void		:					*/

void	KBToolBox::activePartActivated
	(	TKPart		*part
	)
{
	DPRINTF
	((	"KBToolBox::activePartActivated: called [%p]\n",
		(void *)part
	))	;

	if (m_toolBox != 0)
	{
		/* If the part is listed in the part map then raise the	*/
		/* appopriate toolbox tool set and ensure that the	*/
		/* toolbox is showing.					*/
		if (m_partMap.find(part) != m_partMap.end())
		{	m_toolBox->raiseToolSet (m_partMap[part]) ;
			m_toolBox->show () ;
			return	;
		}

		/* This should not happen, but if so hide the toolbox	*/
		/* to ensure that we do not show the wrong tools.	*/
		m_toolBox->hide () ;
	}
}

/*  KBToolBox								*/
/*  partDestroyed: Part which used the toolbox is destroyed		*/		
/*  part	 : QObject *	: Part in question			*/
/*  (returns)	: void		:					*/

void	KBToolBox::partDestroyed
	(	QObject		*part
	)
{
	m_partMap.remove ((TKPart *)part) ;

	if (m_toolBox != 0)
	{
		m_lastHiddenAt = m_toolBox->geometry().topLeft() ;
		m_toolBox->hide() ;
	}
}

/*  KBToolBox								*/
/*  appendNode	: Add a node specification				*/
/*  mask	: int		: Specification mask, see below		*/
/*  spec	: NodeSpec *	: Node specification			*/
/*  (returns)	: void		:					*/

void	KBToolBox::appendNode
	(	int		mask,
		const NodeSpec	*spec
	)
{
	DPRINTF
	((	"KBToolBox::appendNode: %08x: %s\n",
		mask,
		spec->m_nodeName
	))	;

	/* The mask is the document type mask from the node		*/
	/* registration	system, and indicates whether the specification	*/
	/* is for a form or report, etc. We maintain separate lists of	*/
	/* specifications for each, stored in a dictionary keyed on the	*/
	/* mask. This allows us to construct a toolbox appropriate to a	*/
	/* form or report, etc.						*/
	KBToolBoxSpecList *specList = m_specLists.find (mask) ;
	if (specList == 0)
	{	specList = new KBToolBoxSpecList    ;
		m_specLists.insert (mask, specList) ;
	}

	specList->append (spec) ;
	DELOBJ	(m_toolBox)	;
}

/*  KBToolBox								*/
/*  showToolBox	: Get appropriate toolbox displayed			*/
/*  part	: TKPart *	: Invoking part				*/
/*  mask	: int		: Required tool bask			*/
/*  (returns)	: void		:					*/

void	KBToolBox::showToolBox
	(	TKPart		*part,
		int		mask
	)
{
	DPRINTF
	((	"KBToolBox::showToolBox: p=%p %08x\n",
		(void *)part,
		mask
	))	;


	/* Ensure that all form and report nodes are registered, else	*/
	/* we can get partly populated toolboxes if the user creates a	*/
	/* new form or report before opening an extant one.		*/
	extern	void	FrmLoadNodeFuncs () ; FrmLoadNodeFuncs () ;
	extern	void	RepLoadNodeFuncs () ; RepLoadNodeFuncs () ;
	extern	void	QryLoadNodeFuncs () ; QryLoadNodeFuncs () ;

	if (m_toolBox == 0)
		m_toolBox = new KBToolBoxWidget (m_specLists) ;

	/* Look for the specification list for the required tool mask.	*/
	/* If there is none then hide any showing toolbox (which must	*/
	/* be a wrong one) and return.					*/
	if (!m_toolBox->raiseToolSet (mask))
	{
		m_lastHiddenAt = m_toolBox->geometry().topLeft() ;
		m_toolBox->hide () ;
		return	;
	}

	m_toolBox->show ()	;
	m_toolBox->setFixedSize (m_toolBox->sizeHint()) ;

	if (m_partMap.find (part) == m_partMap.end())
	{
		m_partMap.insert (part, mask)	;

		connect
		(	part,
			SIGNAL	(destroyed	(QObject *)),
			SLOT	(partDestroyed	(QObject *))
		)	;
	}
}

/*  KBToolBox								*/
/*  hideToolBox	: Hide current toolbox if any				*/
/*  part	: TKPart *	: Invoking part				*/
/*  (returns)	: void		:					*/

void	KBToolBox::hideToolBox
	(	TKPart		*part
	)
{
	if (m_toolBox != 0)
	{
		m_lastHiddenAt = m_toolBox->pos() ;
		m_toolBox->hide () ;
	}

	m_partMap.remove (part) ;
}

/*  KBToolBox								*/
/*  toggleToolBox: Toggle current toolbox if any			*/
/*  part	: TKPart *	: Invoking part				*/
/*  mask	: int		: Required tool bask			*/
/*  (returns)	 : void		:					*/

void	KBToolBox::toggleToolBox
	(	TKPart		*part,
		int		mask
	)
{
	if (m_toolBox != 0)
		if (m_toolBox->isVisible())
		{
			m_toolBox->hide () ;
			return	;
		}

	showToolBox (part, mask) ;
}

/*  KBToolBox								*/
/*  showing	: Test if toolbox is on display				*/
/*  (returns)	: bool		: On display				*/

bool	KBToolBox::showing()
{
	return	(m_toolBox != 0) && m_toolBox->isVisible() ;
}


/*  KBToolBox								*/
/*  currentSpec	: Get currently selected specification			*/
/*  reset	: bool		: True to reset buttons			*/
/*  (returns)	: NodeSpec *	: Current specification or null		*/

NodeSpec*KBToolBox::currentSpec
	(	bool		reset
	)
{
	return	m_toolBox != 0 ? m_toolBox->currentSpec (reset) : 0 ;
}

/*  KBToolBox								*/
/*  useWizard	: See if wizard usage is enabled			*/
/*  (returns)	: bool		: True to use wizard			*/

bool	KBToolBox::useWizard ()
{
	return	m_toolBox != 0 ? m_toolBox->useWizard () : false ;
}

/*  KBToolBox								*/
/*  self	: Instantiate toolbox interface				*/
/*  (returns)	: KBtoolBox *	: Tool box				*/

KBToolBox *KBToolBox::self ()
{
	static	KBToolBox *toolBox ;
	if (toolBox == 0) toolBox = new KBToolBox ;
	return	toolBox	;
}


/*  ------------------------------------------------------------------  */

/*  The remaining KBToolBox methods are used to check whether a		*/
/*  particular specification is one of the special cases ...		*/

bool	KBToolBox::specIsMenuBlock
	(	NodeSpec	*spec
	)
{
	return	spec == &menuBlock ;
}

bool	KBToolBox::specIsTableBlock
	(	NodeSpec	*spec
	)
{
	return	spec == &tableBlock ;
}

bool	KBToolBox::specIsQueryBlock
	(	NodeSpec	*spec
	)
{
	return	spec == &queryBlock ;
}

bool	KBToolBox::specIsSQLBlock
	(	NodeSpec	*spec
	)
{
	return	spec == &sqlBlock ;
}

bool	KBToolBox::specIsContainer
	(	NodeSpec	*spec
	)
{
	return	spec == &container ;
}

bool	KBToolBox::specIsPasteComp
	(	NodeSpec	*spec
	)
{
	return	spec == &pasteComp ;
}

bool	KBToolBox::specIsLinkComp
	(	NodeSpec	*spec
	)
{
	return	spec == &linkComp ;
}

bool	KBToolBox::specIsUsePopup
	(	NodeSpec	*spec
	)
{
	return	spec == &usePopup ;
}

bool	KBToolBox::useWizard
	(	Qt::ButtonState	bState
	)
{
	if (KBToolBox::self()->showing())
		return	KBToolBox::self()->useWizard() ;

	return	(bState & Qt::ControlButton) != 0 ;
}

