/***************************************************************************
    file	         : kb_toolbox.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_TOOLBOX_H
#define	__KB_TOOLBOX_H

#include	<qwidget.h>
#include	<qmap.h>
#include	<qintdict.h>
#include	<qguardedptr.h>
#include	<qbuttongroup.h>
#include	<qwidgetstack.h>

#include 	"libkbase_exports.h"
#include	"rk_vbox.h"

class	NodeSpec				;
class	QToolButton				;

struct	KBToolBoxControl			;
class	KBToolBoxWidget				;
class	KBToolBox				;

class	TKPart					;
typedef	QPtrList<NodeSpec>	KBToolBoxSpecList	;


class	LIBKBASE_API	KBToolBoxToolSet : public RKVBox
{
	Q_OBJECT

	int				m_type		;
	QMap<QToolButton *,NodeSpec *>	m_specMap	;
	QToolButton			*m_noCtrlButton	;
	QToolButton			*m_wizButton	;
	QToolButton			*m_popupButton	;
	QToolButton			*m_currButton	;
	NodeSpec			*m_currSpec	;

	QToolButton	*addButton	(NodeSpec *, QGroupBox *) ;

private slots :

	void		slotPressed	()		;
	void		newNode		(int)		;

public	:

	KBToolBoxToolSet (KBToolBoxWidget *, int, KBToolBoxSpecList *) ;
	virtual ~KBToolBoxToolSet	()		;

	NodeSpec	*currentSpec	(bool = true)	;
	bool		useWizard	()		;
	int		type		()		;
}	;


/*  KBToolBoxWidget							*/
/*  ---------------							*/
/*  This is the actual toolbox display widget, as opposed to the	*/
/*  KBToolBox class (below) which is the interface to the toolbox.	*/

class	LIBKBASE_API	KBToolBoxWidget  : public QWidgetStack
{
	QIntDict<KBToolBoxToolSet>	m_toolSets	;

public	:

	KBToolBoxWidget		(QIntDict<KBToolBoxSpecList> &)	;

	bool		raiseToolSet	(int)	;
	NodeSpec	*currentSpec	(bool)	;
	bool		useWizard	()	;
}	;




/*  KBToolBox								*/
/*  ---------								*/
/*  Class providing an interface to the Rekall design toolbox. This	*/
/*  should *always* be accessed via the KBToolBox::self() method, which	*/
/*  instantiates a single instance of the class (hence, the constructor	*/
/*  is private).							*/

class	LIBKBASE_API	KBToolBox : public QObject
{
	Q_OBJECT

	QMap	   <QObject *,int>	m_partMap	;
	QIntDict   <KBToolBoxSpecList>	m_specLists	;
	KBToolBoxWidget			*m_toolBox	;
	QPoint				m_lastHiddenAt	;
	int				m_suspendCount	;

	KBToolBox	()	;

protected slots :

	void		activePartActivated	(TKPart  *)	;
	void		partDestroyed		(QObject *)	;

public	:

	virtual	~KBToolBox	()	;

	void			showToolBox	(TKPart *, int)	;
	void			hideToolBox	(TKPart *)	;
	void			suspendToolBox	()	;
	void			resumeToolBox	()	;
	void			toggleToolBox	(TKPart *, int)	;
	bool			showing		()	;

	NodeSpec		*currentSpec	(bool = true) ;
	bool			useWizard	()	;

	void			appendNode	(int, const NodeSpec *) ;

	static	KBToolBox	*self		()	;
	static	bool		specIsMenuBlock	(NodeSpec *)	;
	static	bool		specIsTableBlock(NodeSpec *)	;
	static	bool		specIsQueryBlock(NodeSpec *)	;
	static	bool		specIsSQLBlock  (NodeSpec *)	;
	static	bool		specIsContainer (NodeSpec *)	;
	static	bool		specIsPasteComp (NodeSpec *)	;
	static	bool		specIsLinkComp  (NodeSpec *)	;
	static	bool		specIsUsePopup	(NodeSpec *)	;

	static	bool		useWizard 	(Qt::ButtonState) ;
}	;


#endif	// __KB_TOOLBOX_H
