/***************************************************************************
    file	         : kb_tree.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_TREE_H
#define		__KB_TREE_H

#include	"libkbase_exports.h"

#include	<qdict.h>
#include	<qstringlist.h>

#include	"kb_classes.h"
#include	"kb_linktree.h"

/*  KBTree								*/
/*  ----------								*/
/*  This class is used to represent a tree element. This is vartually	*/
/*  the same as a link, differing in the display.			*/

class	LIBKBASE_API	KBTree : public KBLinkTree
{
	Q_OBJECT

	KBAttrStr		m_group		;
	KBAttrBool		m_clickOpen	;
	KBAttrBool		m_setClose	;
	KBAttrInt		m_treeType	;
	uint			m_groupCount	;

protected :

	virtual	uint		addDummyItems 	()	;
	virtual	KBControl	*makeCtrl    	(uint)	;
	virtual	void		loadControl	(uint, const QStringList &, const KBLTValueSet &) ;

public	:

	enum	TreeType
	{	Expanded	= 0,
		Flatenned	= 1,
		Shallow		= 2
	}	;

#if	! __KB_RUNTIME
	virtual	bool		propertyDlg  	(cchar  *) ;
#endif
	virtual	KBNode		*replicate	(KBNode *) ;

	uint			groupCount	()	;
	bool			clickOpen	()	;
	bool			setClose	()	;
	TreeType		treeType	()	;

	virtual void		setUserFilter	(uint, const QString	&) ;
	virtual void		setUserSorting	(uint, const QString	&) ;
	virtual	void		reload		(uint)	;
	virtual	KBValue		getExtra	(uint, bool, uint) ;
	virtual QStringList	getDisplayList	(uint) 	;
	virtual	uint		getNumValues	(uint)	;

	KBTree			(KBNode  *, const QDict<QString> &, bool * = 0) ;
	KBTree			(KBNode  *, KBTree *) ;
	virtual	~KBTree		()		 ;

	static	KBPopupMenu	*makeTreePopup
				(	QWidget		*,
					QObject		*,
					Qt::ButtonState	*,
					NodeSpec	**
				)	;
}	;

#if	! __KB_RUNTIME
extern	LIBKBASE_API	bool	treePropDlg
			(	KBTree 			*,
				cchar			*,
				QPtrList<KBAttr>	&,
				cchar			* = 0
			)	;
#endif
#endif		// __KB_TREE_H
