/***************************************************************************
    file	         : kb_widgettree.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	<stdio.h>
#include	<qobjectlist.h>
#include	<qwidget.h>
#include	<qlayout.h>

#include 	"libkbase_exports.h"
#include	"kb_ascii.h"


QString	LIBKBASE_API
	textWidgetTree
	(	QWidget		*widget,
		uint		indent,
		int		maxDepth,
		int		flags		= 0
	)
{
	if (widget == 0) return QString::null ;

	QString	text	;

	text	+= QString().sprintf
			(	"%*s%s (%s) %s",
				indent,
				"",
				widget->className(),
				widget->name("noname"),
				widget->isVisible() ? "visible" : "hidden "
			)	;

	if ((flags & 0x0001) != 0)
		text	+=  QString().sprintf
			 	(	" (%d,%d)(%d,%d)",
					widget->x(),
					widget->y(),
					widget->width (),
					widget->height()
				)	;

	if ((flags & 0x0002) != 0)
		text	+= QString().sprintf
				(	" @%p",
					(void *)widget
				)	;

	text	+= "\n"	;

	if (maxDepth == 0) return text	;

	QObjectList *children = widget->queryList("QWidget", 0, true, false) ;
	if (children == 0) return text	;

	QObjectListIt iter (*children) ;
	QObject	      *obj ;

	while ((obj = iter.current()) != 0)
	{	*++iter ;
		text	+= textWidgetTree ((QWidget *)obj, indent + 2, maxDepth - 1, flags) ;
	}

	delete	children ;
	return	text	 ;
}


void	LIBKBASE_API
	printWidgetTree
	(	QWidget		*widget,
		uint		indent,
		int		maxDepth,
		int		flags		= 0
	)
{
	fprintf (stderr, "%s", textWidgetTree(widget, indent, maxDepth, flags).latin1()) ;
}


void	LIBKBASE_API	printLayoutTree (QLayout *, uint, int)	;
void	LIBKBASE_API	printLayoutTree (QWidget *, uint, int)	;

void	LIBKBASE_API
	printLayoutTree
	(	QLayout		*layout,
		uint		indent,
		int		maxDepth
	)
{
	if (layout == 0)
	{
		fprintf	(stderr, "%*snull layout\n", indent, "") ;
		return ;
	}

	fprintf
	(	stderr,
		"%*slayout:%s (%p) %s\n",
		indent,
		"",
		layout->mainWidget()->className(),
		(void *)layout,
		_TEXT(layout->sizeHint())
	)	;

	if (maxDepth == 0) return ;

	QLayoutIterator li	= layout->iterator() ;
	QLayoutItem	*child	;
	while ((child = li.current()) != 0)
	{
		if (child->layout())
			printLayoutTree (child->layout(), indent + 2, maxDepth - 1) ;
		if (child->widget())
			printLayoutTree (child->widget(), indent + 2, maxDepth - 1) ;
		++li	;
	}
}

void	LIBKBASE_API
	printLayoutTree
	(	QWidget		*widget,
		uint		indent,
		int		maxDepth
	)
{
	if (widget == 0)
	{
		fprintf	(stderr, "%*snull widget\n", indent, "") ;
		return ;
	}

	fprintf
	(	stderr,
		"%*swidget:%s (%p) %s\n",
		indent,
		"",
		widget->className(),
		(void *)widget,
		_TEXT(widget->sizeHint())
	)	;

	printLayoutTree (widget->layout(), indent + 2, maxDepth - 2) ;
}
