/***************************************************************************
    file	         : kb_wizard.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<errno.h>
#include	<stdlib.h>

#include	<qfile.h>
#include	<qlabel.h>
#include	<qtextbrowser.h>
#include	<qwidgetstack.h>

#include	"kb_wizard.h"
#include	"kb_sidepanel.h"
#include	"kb_ascii.h"
#include	"kb_mimefactory.h"


#include	"std.h"
#include	"eli.h"
#include	"interp.h"
#include	"syn.h"
#include	"code.h"

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_pushbutton.h"
#include	"rk_combobox.h"


#define		__DEBUG_WIZARD	(0)
#if		__DEBUG_WIZARD
#include	"kb_dprintf.h"
#endif


extern	METHSET	wiz_combo_methset	;
extern	METHSET	wiz_ctrl_methset 	;
extern	METHSET	wiz_page_methset 	;
extern	METHSET	wiz_wiz_methset  	;

ELTAG	wiz_combo_TAG	=
{
	131,
	0,
	&wiz_combo_methset,
	"wizardCombo"
}	;

ELTAG	wiz_ctrl_TAG	=
{
	128,
	0,
	&wiz_ctrl_methset,
	"wizardCtrl"
}	;

ELTAG	wiz_page_TAG	=
{
	129,
	0,
	&wiz_page_methset,
	"wizardPage"
}	;

ELTAG	wiz_wiz_TAG	=
{
	130,
	0,
	&wiz_wiz_methset,
	"wizard"
}	;

static	VALUE	if_comboIndex
	(	VALUE	*argv
	)
{
	KBWizardComboBox*ctrl	= (KBWizardComboBox *)argv[0].val.user ;
	int		index	= ctrl->comboBox()->currentItem()	;

	return	index	;
}

static	VALUE	if_comboValueList
	(	VALUE	*argv
	)
{
	KBWizardComboBox*ctrl	= (KBWizardComboBox *)argv[0].val.user ;
	RKComboBox	*combo	= ctrl->comboBox() ;
	VEC	    	*rv	= new VEC(combo->count()) ;

	for (int idx = 0 ; idx < combo->count() ; idx += 1)
		rv->vals[idx] = new STRING(combo->text(idx)) ;

	return	rv   ;
}

static	VALUE	if_ctrlName
	(	VALUE	*argv
	)
{
	KBWizardCtrl	*ctrl	= (KBWizardCtrl *)argv[0].val.user ;
	return	new STRING((cchar *)ctrl->name()) ;
}

static	VALUE	if_ctrlValue
	(	VALUE	*argv
	)
{
	KBWizardCtrl	*ctrl	= (KBWizardCtrl *)argv[0].val.user ;
	QString		value	= ctrl->value() ;

	if (value.isNull()) value = ""  ;
	return	new STRING((cchar *)value) ;
}

static	VALUE	if_ctrlSetValue
	(	VALUE	*argv
	)
{
	KBWizardCtrl	*ctrl	= (KBWizardCtrl *)argv[0].val.user ;

	ctrl->setValue	(argv[1].val.str->text) ;

	return	0 ;
}

static	VALUE	if_ctrlSetValueList
	(	VALUE	*argv
	)
{
	KBWizardCtrl	*ctrl	= (KBWizardCtrl *)argv[0].val.user ;
	VEC		*vec	= argv[1].val.vec ;
	QStringList	values	;

	for (int idx = 0 ; idx < vec->size ; idx += 1)
	{
		VALUE	&val	= vec->vals[idx] ;
		if	(val.tag == &tagSTR ) values.append (val.val.str->text) ;
		else if (val.tag == &tagNUM ) values.append (QString::number(val.val.num)) ;
		else if (val.tag == &tagDBL ) values.append (QString::number(val.val.dbl)) ;
		else			      values.append (QString::null) ;		
	}

	ctrl->setValueList (values) ;
	return	0 ;
}

static	VALUE	if_ctrlAttr
	(	VALUE	*argv
	)
{
	KBWizardCtrl	*ctrl	= (KBWizardCtrl *)argv[0].val.user ;
	cchar		*attr	= argv[1].val.str->text ;
	QVariant	value	= ctrl->attribute(attr) ;

	if (!value.isValid())
		return	0 ;

	switch (value.type())
	{
		case QVariant::Int  :
			return	(int)value.asInt () ;

		case QVariant::UInt :
			return	(int)value.asUInt() ;

		case QVariant::StringList :
		{
			QStringList strings = value.toStringList() ;
			VEC	    *rv	    = new VEC(strings.count()) ;

			for (uint idx = 0 ; idx < strings.count() ; idx += 1)
				rv->vals[idx] = new STRING((cchar *)strings[idx]) ;

			return	rv   ;
		}

		default	:
			break	;
	}

	QString	text = value.toString() ;
	if (text.isNull()) text = ""    ;
	return	new STRING((cchar *)text) ;
}

static	VALUE	if_ctrlOK
	(	VALUE	*argv
	)
{
	KBWizardCtrl	*ctrl	= (KBWizardCtrl *)argv[0].val.user ;
	return	ctrl->ok() ;
}

static	VALUE	if_ctrlPage
	(	VALUE	*argv
	)
{
	KBWizardCtrl	*ctrl	= (KBWizardCtrl *)argv[0].val.user ;
	KBWizardPage	*page	= ctrl->page() ;

	return	VALUE ((void *)page, &wiz_page_TAG) ;
}

static	VALUE	if_pageName
	(	VALUE	*argv
	)
{
	KBWizardPage	*page	= (KBWizardPage *)argv[0].val.user ;
	return	new STRING((cchar *)page->name()) ;
}

static	VALUE	if_pageCtrl
	(	VALUE	*argv
	)
{
	KBWizardPage	*page	= (KBWizardPage *)argv[0].val.user ;
	cchar		*name	= argv[1].val.str->text ;

	KBWizardCtrl	*ctrl	= page->findCtrl (name) ;
	if (ctrl == 0)
	{
		DPRINTF
		((	"if_pageCtrl: not found: [%s]\n",
			name
		))	;

		return	0 ;
	}

	ELTAG	*tag	= ctrl->eltag  () ;
	if (tag == 0) tag = &wiz_ctrl_TAG ;
	return	VALUE ((void *)ctrl, tag) ;
}

static	VALUE	if_pageWizard
	(	VALUE	*argv
	)
{
	KBWizardPage	*page	= (KBWizardPage *)argv[0].val.user ;
	KBWizard	*wizard	= page->wizard() ;

	if (wizard == 0)
	{
		DPRINTF
		((	"if_pageWizard: no wizard\n"
		))	;

		return	0 ;
	}

	return	VALUE ((void *)wizard, &wiz_wiz_TAG) ;
}

static	VALUE	if_wizardPage
	(	VALUE	*argv
	)
{
	KBWizard	*wizard	= (KBWizard *)argv[0].val.user ;
	cchar		*name	= argv[1].val.str->text ;

	KBWizardPage	*page	= wizard->findPage (name) ;
	if (page == 0)
	{
		DPRINTF
		((	"if_wizardPage: not found: [%s]\n",
			name
		))	;

		return	0 ;
	}

	return	VALUE ((void *)page, &wiz_page_TAG) ;
}

static	VALUE	if_wizardCtrlValue
	(	VALUE	*argv
	)
{
	KBWizard	*wizard	= (KBWizard *)argv[0].val.user ;
	cchar		*page	= argv[1].val.str->text ;
	cchar		*ctrl	= argv[2].val.str->text ;

	return	new STRING((cchar *)wizard->ctrlValue(page, ctrl)) ;
}

static	VALUE	if_wizardCookie
	(	VALUE	*argv
	)
{
	KBWizard	*wizard	= (KBWizard *)argv[0].val.user ;
	cchar		*name	= argv[1].val.str->text ;
	const KBValue	*value	= wizard->cookie (name) ;

	DPRINTF
	((	"if_wizardCookie: lookup [%s] gives [%p]\n",
		name,
		(void *)value
	))	;

	if (value == 0)
		return argv[2] ;

	switch	(value->getType()->getIType())
	{
		case KB::ITBool	  :
			DPRINTF
			((	"if_wizardCookie: bool  : %d\n",
				(int)value->isTrue()
			))	;
			return	(int)value->isTrue() ;

		case KB::ITFixed  :
			DPRINTF
			((	"if_wizardCookie: fixed : %d\n",
				value->getRawText().toInt   ()
			))	;
			return	value->getRawText().toInt   () ;

		case KB::ITFloat  :
			DPRINTF
			((	"if_wizardCookie: float : %f\n",
				value->getRawText().toDouble()
			))	;
			return	value->getRawText().toDouble() ;

		case KB::ITString :
			DPRINTF
			((	"if_wizardCookie: string: %f\n",
				(cchar *)value->getRawText()
			))	;
			return	new STRING((cchar *)value->getRawText()) ;

		default		:
			DPRINTF
			((	"if_wizardCookie::execute: unexpected ttype [%d]\n",
				value->getType()->getIType()
			))	;
			break	;
	}

	return	argv[2]	;
}

MC	wiz_combo_methods[] =
{
	{	"index",
		{
			0,
		},
		if_comboIndex
	},
	{	"valueList",
		{
			0,
		},
		if_comboValueList
	},
	{	0,
		{
			0
		},
		0
	}
}	;

MC	wiz_ctrl_methods[] =
{
	{	"name",
		{
	 		0
		},
		if_ctrlName
	},
	{	"value",
		{
	 		0
		},
		if_ctrlValue
	},
	{	"attr",
		{
			&tagSTR,
			0
		},
		if_ctrlAttr
	},
	{	"setValue",
		{
			&tagSTR,
			0
		},
		if_ctrlSetValue
	},
	{	"setValueList",
		{
			&tagVEC,
			0
		},
		if_ctrlSetValueList
	},
	{	"page",
		{
			0
		},
		if_ctrlPage
	},
	{	"ok",
		{
			0
		},
		if_ctrlOK
	},
	{
		0,
		{
			0
		},
		0
	}
}	;

MC	wiz_page_methods[] =
{
	{	"name",
		{
	 		0
		},
		if_pageName
	},
	{	"ctrl",
		{
			&tagSTR,
	 		0
		},
		if_pageCtrl
	},
	{	"wizard",
		{
	 		0
		},
		if_pageWizard
	},
	{
		0,
		{
			0
		},
		0
	}
}	;

MC	wiz_wiz_methods[] =
{
	{	"page",
		{
			&tagSTR,
	 		0
		},
		if_wizardPage
	},
	{	"ctrlValue",
		{
			&tagSTR,
			&tagSTR,
	 		0
		},
		if_wizardCtrlValue
	},
	{	"cookie",
		{
			&tagSTR,
			&tagANY,
			0
		},
		if_wizardCookie
	},
	{
		0,
		{
			0
		},
		0
	}
}	;

#ifdef	__ALLOW
ELTAG	*wiz_combo_allowed[] =
{
	&wiz_combo_TAG,
	0
}	;

ELTAG	*wiz_ctrl_allowed[] =
{
	&wiz_combo_TAG,
	&wiz_ctrl_TAG,
	0
}	;

ELTAG	*wiz_page_allowed[] =
{
	&wiz_page_TAG,
	0
}	;

ELTAG	*wiz_wiz_allowed[] =
{
	&wiz_wiz_TAG,
	0
}	;
#endif

METHSET	wiz_combo_methset = 
{
	&wiz_ctrl_methset,
#ifdef	__ALLOW
	wiz_combo_allowed,
#endif
	wiz_combo_methods
}	;

METHSET	wiz_ctrl_methset = 
{
	0,
#ifdef	__ALLOW
	wiz_ctrl_allowed,
#endif
	wiz_ctrl_methods
}	;

METHSET	wiz_page_methset = 
{
	0,
#ifdef	__ALLOW
	wiz_page_allowed,
#endif
	wiz_page_methods
}	;

METHSET	wiz_wiz_methset = 
{
	0,
#ifdef	__ALLOW
	wiz_wiz_allowed,
#endif
	wiz_wiz_methods
}	;


/*  ------------------------------------------------------------------	*/

/*  KBWizardCtrlMaker							*/
/*  -----------------							*/
/*  Simple class to wrap up a wizard control name and its factory	*/
/*  function.								*/

class	KBWizardCtrlMaker
{
public	:

	cchar		*m_name	;
	MKWIZARDCTRL	m_fn	;

	inline	KBWizardCtrlMaker
		(	cchar 		*name,
			MKWIZARDCTRL	fn
		)
		:
		m_name	(name),
		m_fn	(fn)
	{
	}
}	;

/*  KBWizardCtrlMaker							*/
/*  -----------------							*/
/*  Simple class to wrap up a wizard name and its factory function.	*/

class	KBWizardMaker
{
public	:

	cchar		*m_name	;
	MKWIZARD	m_fn	;

	inline	KBWizardMaker
		(	cchar		*name,
			MKWIZARD	fn
		)
		:
		m_name	(name),
		m_fn	(fn)
	{
	}
}	;

/*  Wizard control and wizard makers (factories) are stored in a pair	*/
/*  of dictionaries.							*/

static	QDict<KBWizardCtrlMaker>	*wizardCtrlMaker	;
static	QDict<KBWizardMaker>		*wizardMaker		;

/*  ------------------------------------------------------------------	*/


void	KBWizardCtrlReg::registerCtrl
	(	cchar		*name,
		MKWIZARDCTRL	fn
	)
{
	DPRINTF
	((	"KBWizardCtrlReg::registerCtrl: [%s]\n",
		name
	))	;

	if (wizardCtrlMaker == 0)
		wizardCtrlMaker = new QDict<KBWizardCtrlMaker> ;

	wizardCtrlMaker->insert (name, new KBWizardCtrlMaker (name, fn)) ;
}


KBWizardCtrl
	*KBWizardCtrlReg::makeWizardCtrl
	(	const QString		&name,
		KBWizardPage		*page,
		const QDomElement	&elem
	)
{
	KBWizardCtrlMaker *maker = wizardCtrlMaker->find (name) ;

	DPRINTF
	((	"KBWizardCtrlReg::makeWizardCtrl: [%s]->[%p]\n",
		(cchar *)name,
		(void  *)maker
	))	;


	return	maker == 0 ? 0 : (*maker->m_fn) (page, elem) ;
}

/*  ------------------------------------------------------------------  */

void	KBWizardReg::registerWizard
	(	cchar		*name,
		MKWIZARD	fn
	)
{
	DPRINTF
	((	"KBWizardReg::registerWizard: [%s]\n",
		name
	))	;

	if (wizardMaker == 0)
		wizardMaker = new QDict<KBWizardMaker> ;

	wizardMaker->insert (name, new KBWizardMaker (name, fn)) ;
}


KBWizard
	*KBWizardReg::makeWizard
	(	const QString		&name,
		KBLocation		&location,
		const QString		&server
	)
{
	KBWizardMaker *maker = wizardMaker->find (name) ;

	DPRINTF
	((	"KBWizardReg::makeWizard: [%s]->[%p\n",
		(cchar *)name,
		(void  *)maker
	))	;


	return	maker == 0 ? 0 : (*maker->m_fn) (location, server) ;
}

/*  ------------------------------------------------------------------  */

/*  KBWizard								*/
/*  KBWizard	: Constructor for rekall wizard class			*/
/*  dbInfo	: KBDBInfo *		: Database information		*/
/*  server	: const QString &	: Server name			*/
/*  (returns)	: KBWizard		:				*/

KBWizard::KBWizard
	(	KBDBInfo	*dbInfo,
		const QString	&server
	)
	:
	KBDialog	("", true, "kbwizard"),
	m_dbInfo	(dbInfo),
	m_server	(server)
{
	RKVBox		*layMain = new RKVBox	(this) ;
	layMain->setTracking () ;

	m_title		= new QLabel	(layMain) ;
	m_title->setTextFormat (Qt::RichText) ;

	RKHBox		*layData = new RKHBox	 (layMain) ;

	m_sidePanel	= new KBSidePanel  	 (layData) ;
	m_blurb		= new QTextBrowser   	 (layData) ;
	m_stack		= new QWidgetStack	 (layData) ;

	m_mimeSource	= new KBMimeSourceFactory  (0) ;
	m_blurb->setTextFormat		(Qt::RichText) ;
	m_blurb->setMimeSourceFactory	(m_mimeSource) ;


	/* HACK ALERT: We want to stop the blurb area gobbling up	*/
	/* space as the user widens the dialog, so fix a maximum width	*/
	/* based on the font. We then also have to stop the side panel	*/
	/* widening up at all.						*/
	m_blurb->setMaximumWidth
	(	QFontMetrics(m_blurb->font())
			.width ("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456")
	)	;
	layData->setStretchFactor (m_sidePanel, 0) ;
	layData->setStretchFactor (m_blurb,     1) ;
	layData->setStretchFactor (m_stack,	1) ;

	QFrame		*hbar1	 = new QFrame (layMain) ;
	hbar1  ->setFrameStyle  (QFrame::Sunken|QFrame::HLine) ;
	hbar1  ->setFixedHeight (12)	;

	RKHBox		*layButt = new RKHBox (layMain) ;
	layButt->addFiller () ;
	m_bPrevious	= new RKPushButton (TR("Previous"), layButt) ;
	m_bNext		= new RKPushButton (TR("Next"	 ), layButt) ;
	m_bFinish	= new RKPushButton (TR("Finish"  ), layButt) ;
	m_bCancel	= new RKPushButton (TR("Cancel"  ), layButt) ;

	m_bNext->setDefault (true) ;

	connect	(m_bPrevious,	SIGNAL(clicked()), SLOT(clickPrevious())) ;
	connect	(m_bNext,	SIGNAL(clicked()), SLOT(clickNext    ())) ;
	connect	(m_bFinish,	SIGNAL(clicked()), SLOT(clickFinish  ())) ;
	connect	(m_bCancel,	SIGNAL(clicked()), SLOT(clickCancel  ())) ;

	m_curPage = 0 ;

	m_blurb->setMinimumWidth  (180) ;
	m_blurb->setMinimumHeight (250) ;
//	m_stack->setMinimumWidth  (320) ;
}

/*  KBWizard								*/
/*  showPage	: Show specified page					*/
/*  idx		: uint		 : Page index				*/
/*  page	: KBWizardPage * : The page				*/
/*  push	: bool		 : Push page onto history		*/
/*  next	: bool		 : Movement is "next"			*/
/*  (returns)	: void		 :					*/

void	KBWizard::showPage
	(	uint		idx,
		KBWizardPage	*page,
		bool		push,
		bool		next
	)
{
	DPRINTF
	((	"KBWizard::showPage: %d,%p: p=%d n=%d\n",
		idx,
		(void *)page,
		push,
		next
	))	;

	/* NOTE: Arguments are redunant but we need both, and the 	*/
	/*	 public methods below derive one from the other.	*/
	if (push)
		m_history.push(m_pages.at(m_curPage)) ;

	page->enterPage (next) ;

	m_stack	   ->raiseWidget (page) ;
	m_bPrevious->setEnabled  (m_history.count() > 0) ;

	const QString	&ft = page->finishText() ;
	m_bFinish  ->setText (ft.isEmpty() ? TR("Finish") : ft) ;

	m_title    ->setText	("<qt><b>" + page->title() + "</b></qt>") ;
	m_blurb    ->setText	("<qt>" + page->blurb() + "</qt>") ;

	m_curPage  = idx 	;
	page->pageShown (next)	;

	ctrlChanged (page, 0)	;
}

/*  KBWizard								*/
/*  showPage	: Show specified page					*/
/*  page	: KBWizardPage * : The page				*/
/*  push	: bool		 : Push page onto history		*/
/*  next	: bool		 : Movement is "next"			*/
/*  (returns)	: void		 :					*/

void	KBWizard::showPage
	(	KBWizardPage	*page,
		bool		push,
		bool		next
	)
{
	for (uint idx = 0 ; idx < m_pages.count() ; idx += 1)
		if (m_pages.at(idx) == page)
		{	showPage (idx, page, push, next) ;
			return	 ;
		}
}

/*  KBWizard								*/
/*  showPage	: Show specified page					*/
/*  idx		: uint		 : Page index				*/
/*  push	: bool		 : Push page onto history		*/
/*  next	: bool		 : Movement is "next"			*/
/*  (returns)	: void		 :					*/

void	KBWizard::showPage
	(	uint		idx,
		bool		push,
		bool		next
	)
{
	if (idx < m_pages.count())
	{	showPage (idx, m_pages.at(idx), push, next) ;
		return	 ;
	}
}

/*  KBWizard								*/
/*  settings	: Get settings of all controls				*/
/*  dict	: QDict<QString> & : Results dictionary			*/
/*  changed	: bool		   : Return only changed values		*/
/*  (returns)	: void		   :					*/

void	KBWizard::settings
	(	QDict<QString>	&dict,
		bool		changed
	)
{
	for (uint idx = 0 ; idx < m_pages.count() ; idx += 1)
		m_pages.at(idx)->settings(dict, changed) ;
}

/*  KBWizard								*/
/*  clickPrevious: User clicks previous page button			*/
/*  (returns)	 :		:					*/

void	KBWizard::clickPrevious ()
{
	if (m_history.count() > 0)
		showPage (m_history.pop(), false, false) ;
}

/*  KBWizard								*/
/*  clickNext	: User clicks next page button				*/
/*  (returns)	:		:					*/

void	KBWizard::clickNext ()
{
	QString	next = m_pages.at(m_curPage)->nextPage() ;
	for (uint idx = 0 ; idx < m_pages.count() ; idx += 1)
		if (m_pages.at(idx)->name() == next)
		{	showPage (idx, true, true) ;
			return	 ;
		}

	if (m_curPage < m_pages.count() - 1)
		showPage (m_curPage + 1, true, true) ;
}

/*  KBWizard								*/
/*  clickFinish	: User clicks finish button				*/
/*  (returns)	:		:					*/

void	KBWizard::clickFinish ()
{
	done	(1) ;
}

/*  KBWizard								*/
/*  clickCancel	: User clicks cancel					*/
/*  (returns)	:		:					*/

void	KBWizard::clickCancel ()
{
	done	(0) ;
}


KBWizardPage
	*KBWizard::addNewPage
	(	const QDomElement	&pageElem
	)
{
	int		pageNo	= m_pages.count	  ()	 ;
	KBWizardPage	*page	= new KBWizardPage(this, m_stack, pageElem.attribute("name")) ;

	page->init (pageElem) ;

	m_stack->addWidget (page, pageNo) ;
	m_pages .append	   (page) ;

	return	page ;
}

/*  KBWizard								*/
/*  init	: Initialise wizard					*/
/*  doc		: QDomDocument & : Specification document		*/
/*  (returns)	: bool		 : Success				*/

bool	KBWizard::init
	(	const QDomDocument	&doc
	)
{
	m_elem = doc.documentElement() ;

	for (QDomNode node = m_elem.firstChild () ;
			    !node  .isNull     () ;
		      node = node  .nextSibling() )
	{
		QDomElement elem = node.toElement () ;
		if (elem.isNull()) continue ;

		/* Caption						*/
		/* Specifies the dialog caption text			*/
		if (elem.nodeName() == "caption")
		{
			QString	caption = elem.text().stripWhiteSpace() ;
			setCaption	      (caption) ;
			m_sidePanel->setExtra (caption) ;
			continue   ;
		}

		/* Page							*/
		/* Specifies a wizard page. The details are contined	*/
		/* within this element.					*/
		if (elem.nodeName() == "page")
		{
			addNewPage (elem) ;
			continue   ;
		}

		/* Text							*/
		/* This is the text that will be parsed or otherwise	*/
		/* inserted into the document, after substitution.	*/
		if (elem.nodeName() == "text")
		{
			m_text	= elem.text() ;
			continue  ;
		}
	}

	return	true	  ;
}

/*  KBWizard								*/
/*  init	: Initialise wizard					*/
/*  wName	: const QString & : Specification file name		*/
/*  (returns)	: bool		  : Success				*/

bool	KBWizard::init
	(	const QString	&wName
	)
{
	QFile	wFile	(wName) ;

	if (!wFile.open (IO_ReadOnly))
	{
		m_error	= KBError
			  (	KBError::Error,
				QString	(TR("Cannot open \"%1\"")).arg(wName),
				strerror(errno),
				__ERRLOCN
			  )	;
		return	false	;
	}

	QDomDocument	doc	;
	if (!doc.setContent (&wFile))
	{
		m_error	= KBError
			  (	KBError::Error,
				QString	(TR("Cannot parse \"%1\"")).arg(wName),
				QString::null,
				__ERRLOCN
			  )	;
		return	false	;
	}

	return	init (doc)	;
}

/*  KBWizard								*/
/*  execute	: Execute wizard					*/
/*  startPage	: int		: Starting page number			*/
/*  (returns)	: int		: Dialog exit code			*/

int	KBWizard::execute
	(	int		startPage
	)
{
	extern	void	printLayoutTree (QWidget *, uint, int) ;
	DPRINTF
	((	"KBWizard::exec: stack %s\n",
		_TEXT(m_stack->sizeHint())
	))	;
#if	__DEBUG_WIZARD
	for (uint idx = 0 ; idx < m_pages.count() ; idx += 1)
		printLayoutTree (m_pages.at(idx), 0, 10) ;
#endif
	showPage (startPage >= 0 ? startPage : m_curPage, false, true) ;

	int	rc  = KBDialog::exec () ;
	save   (rc) ;
	return	rc  ;
}

int	KBWizard::execute ()
{
	return	execute	(0) ;
}


/*  KBWizard								*/
/*  findPage	: Find named page					*/
/*  name	: const QString & : Name of page			*/
/*  (returns)	: KBWizardPage *  : Page or null if not found		*/

KBWizardPage
	*KBWizard::findPage
	(	QString const	&name
	)
{
	for (uint idx = 0 ; idx < m_pages.count() ; idx += 1)
		if (m_pages.at(idx)->name() == name)
			return	m_pages.at(idx)	;

	return	0 ;
}

/*  KBWizard								*/
/*  findCtrl	: Find named control					*/
/*  page	: const QString & : Name of page			*/
/*  name	: const QString & : Name of control			*/
/*  klass	: const char *	  : Required class			*/
/*  (returns)	: KBWizardCtrl *  : Control or null if not found	*/

KBWizardCtrl
	*KBWizard::findCtrl
	(	QString const	&page,
		QString const	&name,
		const char	*klass
	)
{
	for (uint idx = 0 ; idx < m_pages.count() ; idx += 1)
		if ((page == "*") || (page == m_pages.at(idx)->name()))
		{
			KBWizardCtrl *ctrl = m_pages.at(idx)->findCtrl (name, klass) ;

			if (ctrl !=   0) return ctrl ;
			if (page != "*") return 0    ;
		}

	return	0 ;
}

/*  KBWizard								*/
/*  ctrlValue	: Get control value					*/
/*  page	: const QString & : Name of page			*/
/*  name	: const QString & : Name of control			*/
/*  (returns)	: QString	  : Value or null if no control		*/

QString	KBWizard::ctrlValue
	(	const QString 	&page,
		const QString	&name
	)
{
	KBWizardCtrl *ctrl = findCtrl (page, name) ;
	return	ctrl == 0 ? QString::null : ctrl->value() ;
}

/*  KBWizard								*/
/*  ctrlAttribute							*/
/*		: Get control attribute					*/
/*  page	: const QString & : Name of page			*/
/*  name	: const QString & : Name of control			*/
/*  attr	: const QString & : Name of attribute			*/
/*  (returns)	: QVariant	  : Value or null if no control/attr	*/

QVariant
	KBWizard::ctrlAttribute
	(	const QString 	&page,
		const QString	&name,
		const QString	&attr
	)
{
	KBWizardCtrl *ctrl = findCtrl (page, name) ;
	return	ctrl == 0 ? QVariant() : ctrl->attribute(attr) ;
}

/*  KBWizard								*/
/*  ctrlChanged	: Called when a control is changed			*/
/*  page	: KBWizardPage * : Page containing control		*/
/*  ctrl	: KBWizardCtrl * : Control in question			*/
/*  (returns)	: void		 :					*/

void	KBWizard::ctrlChanged
	(	KBWizardPage	*page,
		KBWizardCtrl	*ctrl
	)
{
	DPRINTF
	((	"KBWizard::ctrlChanged: p=[%s] c=[%s] f=[%d]\n",
		(cchar *)page->name(),
		ctrl == 0 ? "" : (cchar *)ctrl->name(),
		page->finish()
	))	;

	/* If the page values are acceptable then enable the next and	*/
	/* finish buttons according to the finish value:		*/
	/*	0	- Next only					*/
	/*	1	- Finish only					*/
	/*	2	- Next and finish both allowed			*/
	if (page->ok())
	{
		uint	finish	= page->finish() ;

		m_bNext  ->setEnabled ((finish == 0) || (finish == 2)) ;
		m_bFinish->setEnabled ((finish == 1) || (finish == 2)) ;

		if	(m_bFinish->isEnabled()) m_bFinish->setDefault (true) ;
		else if (m_bNext  ->isEnabled()) m_bNext  ->setDefault (true) ;

		return	;
	}

	/* Contents not acceptable, neither of the next or finish	*/
	/* buttons are enabled.						*/
	m_bNext  ->setEnabled (false) ;
	m_bFinish->setEnabled (false) ;
}

/*  KBWizard								*/
/*  save	: Save any required settings				*/
/*  rc		: int		: Dialog exit code			*/
/*  (returns)	: void		:					*/

void	KBWizard::save
	(	int		rc
	)
{
	for (uint idx = 0 ; idx < m_pages.count() ; idx += 1)
		m_pages.at(idx)->save(rc) ;
}

/*  KBWizard								*/
/*  currentPageName							*/
/*		: Return name of current page				*/
/*  (returns)	: QString	: Page name				*/

QString	KBWizard::currentPageName ()
{
	return	m_pages.at(m_curPage)->name() ;
}

/*  KBWizard								*/
/*  results	: Retrieve wizard results				*/
/*  (returns)	: QValueList<QVariant>					*/
/*				: Results				*/

QValueList<QVariant> KBWizard::results ()
{
	QValueList<QVariant>	rv ;

	CBUFF	*code	= KBWizard::compile (m_elem, "results", "wizard", 0) ;
	if (code == 0)
		return	rv ;

	VALUE argv = VALUE ((void *)this, &wiz_wiz_TAG) ;

	if (!el_loadtext (code))
	{
		DPRINTF
		((	"KBWizard::results: load error\n"
		))	;

		free	((void *)code) ;
		return	rv	;
	}

	free	((void *)code)	;

	VALUE	res	= el_vexec ("", "f", 1, &argv) ;

	if (res.tag->tag != V_VEC)
	{
		DPRINTF
		((	"KBWizard::results: unexpected tag [%d]\n",
			res.tag->tag
		))	;
		return	rv	;
	}

	/* Results are returned as a list of QVariants, so that the	*/
	/* called can get both type and value information.		*/
	for (int idx = 0 ; idx < res.val.vec->size ; idx += 1)
	{
		VALUE	&val = res.val.vec->vals[idx] ;

		switch (val.tag->tag)
		{
			case V_NUM	:
				rv.append (QVariant(val.val.num)) ;
				break	;

			case V_DBL	:
				rv.append (QVariant(val.val.dbl)) ;
				break	;

			case V_STR	:
				rv.append (QString (val.val.str->text)) ;
				break	;

			default		:
				DPRINTF
				((	"KBWizard::results: unexpected value tag [%d]\n",
					val.tag->tag
				))	;
				rv.append (QVariant(QString::null)) ;
				break	;
		}
	}

	return	rv	;
}

/*  KBWizard								*/
/*  setCookie	: Set a cookie value					*/
/*  key		: const QString & : Key (the cookie name)		*/
/*  cookie	: const KBValue & : Value				*/
/*  (returns)	: void		  :					*/

void	KBWizard::setCookie
	(	const QString	&key,
		const KBValue	&cookie
	)
{
	/* Wizard cookies are used to pass in values that will be	*/
	/* needed when the wizard executes (either bu the wizard	*/
	/* controls or from the wizard scripts) without requiring the	*/
	/* wizard code to know about them explicitely.			*/
	m_cookies.insert (key, cookie) ;
}

/*  KBWizard								*/
/*  cookie	: Get cookie value					*/
/*  key		: const QString & : Key (the cookie name)		*/
/*  (returns)	: const KBValue * : Value or null if not found		*/

const KBValue
	*KBWizard::cookie
	(	const QString	&key
	)
{
	DPRINTF
	((	"KBWizard::cookie: lookup [%s] gives [%d]\n",
		(cchar *)key,
		m_cookies.find(key) == m_cookies.end()
	))	;

	if (m_cookies.find(key) == m_cookies.end())
		return	0	;

	return	&m_cookies[key]	;
}

