/***************************************************************************
    file	         : kb_stockdb.cpp
    copyright            : (C) 1999,2000,2001,2002,2003,2004 by Mike Richardson
			   (C) 2001,2002,2003,2004 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qlabel.h>
#include	<qtextbrowser.h>
#include	<qwidgetstack.h>
#include	<qprogressbar.h>

#include	"kb_classes.h"
#include	"kb_database.h"
#include	"kb_locator.h"


#ifdef		__KB_AUTOMOC
#include	"kb_stockdb.moc"
#else
#include	"kb_stockdb.h"
#endif

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_combobox.h"
#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"
#include	"tk_config.h"


#define		SDB_EMPTY	TR("Empty database" )
#define		SDB_STOCK	TR("Stock databases")


/*  KBWizardCtrlStockDB							*/
/*  KBWizardCtrlStockDB							*/
/*		: Constructor for stock database wizard control		*/
/*  page	: KBWizardPage *	: Parent page			*/
/*  elem	: const QDomElement &	: Definition element		*/
/*  (returns)	: KBWizardCtrlStockDB	:				*/

KBWizardCtrlStockDB::KBWizardCtrlStockDB
	(	KBWizardPage		*page,
		const QDomElement	&elem
	)
	:
	KBWizardCtrl	(page, elem.attribute("name"))
{
	RKVBox	*layMain = new RKVBox	   (page) ;

	setWide		(true)	  ;
	setWidget	(layMain) ;

	RKHBox	*layLocn = new RKHBox	   (layMain) ;
	m_location	 = new RKComboBox  (layLocn) ;
	m_bFetch	 = new RKPushButton(TR("Fetch"), layLocn) ;

	layLocn   ->setStretchFactor (m_location, 1)    ;
	m_location->setFullWidth     (1000)		;

	m_progress 	= new QWidgetStack (layMain)	;
	m_progressText	= new RKLineEdit   (m_progress) ;
	m_progressBar	= new QProgressBar (m_progress)	;

	m_progress->addWidget	(m_progressText) ;
	m_progress->addWidget	(m_progressBar ) ;
	m_progress->raiseWidget	(m_progressText) ;

	m_progressText->setReadOnly (true) ;

	m_details	 = new QTextBrowser(layMain) ;
	m_databases	 = new RKComboBox  (layMain) ;
	layMain->setStretchFactor (m_details, 1) ;

	m_location->setEditable		(true) ;
	m_location->insertItem		(SDB_EMPTY) ;
	m_location->insertItem		(SDB_STOCK) ;

	TKConfig    *config = TKConfig::getConfig() ;
	config->setGroup ("Wizard StockDB") ;
	m_location->insertStringList (config->readListEntry("specs"), 2) ;
	m_location->setMaximumWidth  (m_location->sizeHint().width()) ;

	fprintf
	(	stderr,
		"KBWizardCtrlStockDB::KBWizardCtrlStockDB: append: %s\n",
		(cchar *)config->readListEntry("specs").join(",")
	)	;

	connect
	(	m_bFetch,
		SIGNAL	(clicked	()),
		SLOT	(slotFetch	())
	)	;
	connect
	(	m_location,
		SIGNAL	(activated	(int)),
		SLOT	(slotFetch	())
	)	;
	connect
	(	m_location->lineEdit	(),
		SIGNAL	(returnPressed	()),
		SLOT	(slotFetch	())
	)	;
	connect
	(	m_databases,
		SIGNAL	(activated	(int)),
		SLOT	(slotDatabase	())
	)	;
}

/*  KBWizardCtrlStockDB							*/
/*  ~KBWizardCtrlStockDB						*/
/*		: Destructor for stock database wizard control		*/
/*  (returns)	:		:					*/

KBWizardCtrlStockDB::~KBWizardCtrlStockDB ()
{
	TKConfig *config = TKConfig::getConfig() ;
	config->setGroup ("Wizard StockDB") ;

	QStringList	specs	;
	for (int idx = 2 ; (idx < m_location->count()) && (idx < 10) ; idx += 1)
		specs.append (m_location->text(idx)) ;

	config->writeEntry ("specs", specs) ;

	fprintf
	(	stderr,
		"KBWizardCtrlStockDB::~KBWizardCtrlStockDB: saved: %s\n",
		(cchar *)specs.join(",")
	)	;
}

/*  KBWizardCtrlStockDB							*/
/*  setProgress	: Set installation progress				*/
/*  msg		: const QString & : Progress message			*/
/*  (returns)	: void		  :					*/

void	KBWizardCtrlStockDB::setProgress
	(	const QString	&msg
	)
{
	/* This method sets the message and raises the text progress	*/
	/* widget into view.						*/
	m_progressText->setText    (msg) ;
	m_progress    ->raiseWidget(m_progressText) ;
}

/*  KBWizardCtrlStockDB							*/
/*  setProgress	: Set installation progress				*/
/*  progress	: int		: Download progress			*/
/*  total	: int		: Total to download			*/
/*  (returns)	: void		:					*/

void	KBWizardCtrlStockDB::setProgress
	(	int		progress,
		int		total
	)
{
	/* This method sets the progress bar and raises the text	*/
	/* progress widget into view.					*/
	m_progressBar->setProgress (progress, total) ;
	m_progress   ->raiseWidget (m_progressText ) ;
}

void	KBWizardCtrlStockDB::setError
	(	const QString	&error,
		const QString	&details
	)
{
	setProgress	      (error)	;
	m_details->setText    (details) ;

	m_bFetch->setText (TR("Fetch")) ;
	m_location->setEnabled   (true) ;
	ctrlChanged ()	;
}

void	KBWizardCtrlStockDB::setError
	(	const KBError	&pError
	)
{
	setError (pError.getMessage(), pError.getDetails()) ;
}

void	KBWizardCtrlStockDB::downloadData
	(	const QByteArray	&data
	)
{
	m_specification.append (data) ;
}

void	KBWizardCtrlStockDB::downloadComplete ()
{
	m_bFetch->setText (TR("Fetch")) ;
	m_location->setEnabled   (true) ;
	ctrlChanged ()	;

	if (!m_dbSpec.loadText(m_specification))
	{
		m_dbSpec.lastError().DISPLAY() ;
		return	;
	}

	m_databases->insertStringList (m_dbSpec.databases()) ;
	slotDatabase () ;
}

/*  KBWizardCtrlStockDB							*/
/*  slotFetch	: User clicks fetch button				*/
/*  (returns)	: void		:					*/

void	KBWizardCtrlStockDB::slotFetch ()
{
	if (KBDownloader::active())
	{
		KBDownloader::halt()		;
		m_bFetch->setText (TR("Fetch")) ;
		m_location->setEnabled   (true) ;
		ctrlChanged ()	;
		return	;
	}

	QString	location = m_location->currentText() ;

	m_details  ->clear () ;
	m_databases->clear () ;
	m_specification	= QString::null ;

	/* If the user selects the empty database entry then they 	*/
	/* presuabmly don't want a stock database so nothing to do.	*/
	if (m_location->currentText() == SDB_EMPTY)
		return	;


	/* If the user selects the stock database entry then retrieve	*/
	/* the local stock database manifest.				*/
	if (m_location->currentText() == SDB_STOCK)
	{
		QString	manifest = locateFile ("appdata", "stock/databases/manifest.xml") ;
		if (manifest.isNull())
		{
			KBError::EError
			(	QString(TR("Cannot locate stock database manifest")),
				QString::null,
				__ERRLOCN
			)	;
			return	;
		}

		if (!m_dbSpec.loadFile(manifest))
		{
			m_dbSpec.lastError().DISPLAY() ;
			return	;
		}

		m_databases->insertStringList (m_dbSpec.databases()) ;
		slotDatabase () ;
		return	;
	}

	for (int idx = 1 ; idx < m_location->count() ; idx += 1)
		if (m_location->text(idx) == location)
		{	m_location->removeItem (idx) ;
			break	;
		}

	m_location->insertItem     (location, 2) ;
	m_location->setCurrentItem (2) ;


	/* Check for HTTP protocol and if so fire up the	*/
	/* downloader to retrieve the manifest.			*/
	if (location.left(7).lower() == "http://")
	{
		QString	error	= KBDownloader::exec
				  (	location,
					"database manifest"
				  )	;
		if (!error.isEmpty())
			KBError::EError
			(	TR("Error downloading database manifest"),
				error,
				__ERRLOCN
			)	;

		m_bFetch  ->setText    (TR("Stop")) ;
		m_location->setEnabled (false) ;
		ctrlChanged ()	;
		return	;
	}

	if (location.left(6).lower() == "ftp://")
	{
		return	;
	}

	if (!m_dbSpec.loadFile(location))
	{
		m_dbSpec.lastError().DISPLAY() ;
		return	;
	}

	m_databases->insertStringList (m_dbSpec.databases()) ;
	slotDatabase () ;
}

/*  KBWizardCtrlStockDB							*/
/*  slotDatabase: User changes database selection			*/
/*  (returns)	: void		:					*/

void	KBWizardCtrlStockDB::slotDatabase ()
{
	if (m_databases->count() == 0)
	{
		m_details->clear() ;
		return	;
	}

	QString	    database = m_databases->currentText() ;
	QString	    descrip  = m_dbSpec.description (database) ;
	QStringList validFor = m_dbSpec.validForDBs (database) ;


	KBDriverDetailsList	drvList	;
	KBError			error	;
	if (KBServer::listDrivers (drvList, error))
		for (uint v = 0 ; v < validFor.count() ; v += 1)
			for (uint d = 0 ; d < drvList.count() ; d += 1)
				if (validFor[v] == drvList[d].tag())
				{	validFor[v] = drvList[d].comment() ;
					break	;
				}

	descrip	= descrip
		+ TR("<p><b>Usable with:</b><ul><li>")
		+ validFor.join ("</li><li>")
		+ "</li></ul>"
		;

	m_details->setText (descrip) ;
}


/*  KBWizardCtrlStockDB							*/
/*  ok		: Check if settings are acceptable			*/
/*  (returns)	: bool		: Acceptable				*/

bool	KBWizardCtrlStockDB::ok ()
{
	return	! KBDownloader::active() ;
}

/*  KBWizardCtrlStockDB							*/
/*  setValue	: Set data value					*/
/*  value	: const QString & : Value to set			*/
/*  (returns)	: void		  :					*/

void	KBWizardCtrlStockDB::setValue
	(	const QString	&
	)
{
}

/*  KBWizardCtrlStockDB							*/
/*  value	: Get value						*/
/*  (returns)	: QString	: Value					*/

QString	KBWizardCtrlStockDB::value ()
{
	return	QString::null	;
}

/*  KBWizardCtrlStockDB							*/
/*  attribute	: Get attribute value					*/
/*  attr	: const QString & : Attribute to retrieve		*/
/*  (returns)	: QVariant	  :					*/

QVariant
	KBWizardCtrlStockDB::attribute
	(	const QString	&attr
	)
{
	QString	name = m_databases->currentText() ;

	/* First threee cases are easy; the database name, which	*/
	/* servers it is valid for, and whethers forms, etc., are	*/
	/* stored as files.						*/
	if (attr == "name"    ) return	name ;
	if (attr == "validfor") return	m_dbSpec.validForDBs (name) ;
	if (attr == "asfiles" ) return	m_dbSpec.asfiles     (name) ;

	/* Next case is the URL. We need to do some stuff here to	*/
	/* handle relative URLs.					*/
	if (attr == "url")
	{
		QString	url = m_dbSpec.url (m_databases->currentText()) ;

		if (m_location->currentText() == SDB_STOCK)
			return	locateFile ("appdata", "stock/databases/" + url) ;

		if (QUrl::isRelativeUrl (url))
			url = QUrl(m_location->currentText(), url, true).toString() ;

		if (url.left(5).lower() == "file:")
			url = url.mid(5) ;

		return	url ;
	}

	return	QVariant() ;
}

WIZARDCTRLREG(StockDB)
