/***************************************************************************
    file	         : kb_wizdbtype.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qtextbrowser.h>

#include	"rk_combobox.h"

#include	"kb_classes.h"
#include	"kb_serverinfo.h"

#if		__KB_AUTOMOC
#include	"kb_wizdbtype.moc"
#else
#include	"kb_wizdbtype.h"
#endif

class	KBDBTypeDriver : public QListBoxText, public KBDriverDetails
{
public	:

	inline	KBDBTypeDriver
		(	RKComboBox		*parent,
			const KBDriverDetails	&details
		)
		:
		QListBoxText	(parent->listBox(), details.comment()),
		KBDriverDetails	(details)
	{
	}
	inline	KBDBTypeDriver
		(	RKComboBox		*parent
		)
		:
		QListBoxText	(parent->listBox(), QString::null),
		KBDriverDetails	()
	{
	}
}	;



/*  KBWizardCtrlDBType							*/
/*  KBWizardCtrlDBType							*/
/*		: Constructor for database type wizard control		*/
/*  page	: KBWizardPage *      : Parent page			*/
/*  elem	: const QDomElement & : Definition			*/
/*  (returns)	: KBWizardCtrlDBType  :					*/

KBWizardCtrlDBType::KBWizardCtrlDBType
	(	KBWizardPage		*page,
		const QDomElement	&elem
	)
	:
	KBWizardCtrl	(page, elem.attribute("name"))
{
	setWidget (m_dbType = new RKComboBox (false, page)) ;
	m_dbType->setListBox (new QListBox (m_dbType)) ;

	page->setInfoCtrl (m_info = new QTextBrowser (page)) ;

	loadTypes () ;

	connect
	(	m_dbType,
		SIGNAL(activated      (int)),
		SLOT  (ctrlChanged    ())
	)	;
	connect
	(	m_dbType,
		SIGNAL(activated      (int)),
		SLOT  (slotTypeChanged(int))
	)	;
}

/*  KBWizardCtrlDBType							*/
/*  ~KBWizardCtrlDBType							*/
/*		: Destructor for database type wizard control		*/
/*  (returns)	:		:					*/

KBWizardCtrlDBType::~KBWizardCtrlDBType ()
{
}

/*  KBWizardCtrlDBType							*/
/*  setValue	: Set current value					*/
/*  value	: const QString & : Required value			*/
/*  (returns)	: void		  :					*/

void	KBWizardCtrlDBType::setValue
	(	const QString	&value
	)
{
	for (int idx = 0 ; idx < m_dbType->count() ; idx += 1)
		if (m_dbType->text(idx) == value)
		{	m_dbType->setCurrentItem (idx) ;
			break	;
		}
}

/*  KBWizardCtrlDBType							*/
/*  value	: Get value						*/
/*  (returns)	: QString	: Value					*/

QString	KBWizardCtrlDBType::value ()
{
	KBDBTypeDriver *t = (KBDBTypeDriver *)m_dbType->listBox()->item(m_dbType->currentItem()) ;
	return	t == 0 ? QString::null : t->tag () ;
}

/*  KBWizardCtrlDBType							*/
/*  loadTypes	: Load available database types				*/
/*  (returns)	: void		:					*/

void	KBWizardCtrlDBType::loadTypes ()
{
	/* Note the currently selected driver, used below to preserve	*/
	/* selections.							*/
	QString	ctag	= value () ;

	m_dbType->clear() ;

	/* Load the driver list the first time round (unless there are	*/
	/* no drivers at all in which case it might get retried!).	*/
	if (m_drvList.count() == 0)
	{	KBError	error	;
		if (!KBServer::listDrivers (m_drvList, error))
		{	error.DISPLAY() ;
			return	;
		}
	}

	/* Insert a null first entry, then add an entry for each driver	*/
	/* If the acceptable-driver list is not empty, only insert	*/
	/* drivers that are on the acceptable list.			*/
	new KBDBTypeDriver (m_dbType) ;
	for (uint idx = 0 ; idx < m_drvList.count() ; idx += 1)
		if ((m_accept.count() == 0) || (m_accept.find (m_drvList[idx].tag()) != m_accept.end()))
			new KBDBTypeDriver
			(	m_dbType,
				m_drvList[idx]
			)	;


	/* Select the first entry by default, then see if the prior	*/
	/* set driver is still present, in which case choose that one.	*/
	m_dbType->setCurrentItem (0) ;
	for (int slot = 0 ; slot < m_dbType->count() ; slot += 1)
		if (((KBDBTypeDriver *)m_dbType->listBox()->item(slot))->tag() == ctag)
		{	m_dbType->setCurrentItem (slot) ;
			break	;
		}

	/* Special case. If no driver is selected and there is only	*/
	/* one driver, then select that one. This makes for sensible	*/
	/* behaviour when drivers are filtered for stock databases.	*/
	if (m_dbType->currentItem() == 0)
		if (m_dbType->count() == 2)
			m_dbType->setCurrentItem (1) ;


	slotTypeChanged	(m_dbType->currentItem()) ;
}

/*  KBWizardCtrlDBType							*/
/*  setValueList: Set list of values					*/
/*  values	: const QStringList & : Value list			*/
/*  (returns)	: void		      :					*/

void	KBWizardCtrlDBType::setValueList
	(	const QStringList	&values
	)
{
	/* This is interpreted as a filter on the available drivers.	*/
	m_accept = values ;
	loadTypes ()	  ;
}

/*  KBWizardCtrlDBType							*/
/*  attribute	: Get selected driver attribute				*/
/*  attr	: const QString & : Attribute required			*/
/*  (returns)	: QVariant	  : Value				*/

QVariant KBWizardCtrlDBType::attribute
	 (	const QString 	&attr
	 )
{
	KBDBTypeDriver *t = (KBDBTypeDriver *)m_dbType->listBox()->item(m_dbType->currentItem()) ;

	if (attr == "flags")
		return	QVariant (t->flags()) ;

	return	QVariant() ;
}

/*  KBWizardCtrlDBType							*/
/*  slotTypeChanged							*/
/*		: Selected driver changes				*/
/*  idx		: int		: New selection				*/
/*  (returns)	: void		:					*/

void	KBWizardCtrlDBType::slotTypeChanged
	(	int		idx
	)
{
	if (m_dbType->count() <= 1)
	{
		m_info->setText
		(	TR
			(	"There are no available database drivers. If you	"
				"are trying to load a stock database this probably	"
				"means that you do not have a driver that the stock	"
				"database can be used with.				"
			)
		)	;

		return	;
	}

	KBDBTypeDriver *t = (KBDBTypeDriver *)m_dbType->listBox()->item(idx) ;
	m_info->setText (t->info()) ;
}

WIZARDCTRLREG(DBType)
