/***************************************************************************
    file	         : kb_wizexpr.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"rk_combobox.h"

#include	"kb_classes.h"
#include	"kb_database.h"
#include	"kb_dblink.h"
#include	"kb_location.h"
#include	"kb_qrybase.h"
#include	"kb_qryquery.h"

#include	"kb_wizsource.h"

#if		__KB_AUTOMOC
#include	"kb_wizexpr.moc"
#else
#include	"kb_wizexpr.h"
#endif


/*  KBWizardCtrlExpr							*/
/*  KBWizardCtrlExpr							*/
/*		: Constructor for data expression wizard control	*/
/*  page	: KBWizardPage *	 : Parent page			*/
/*  elem	: QDomElement  &	 : Definition element		*/
/*  (returns)	: KBWizardCtrlExpr :					*/

KBWizardCtrlExpr::KBWizardCtrlExpr
	(	KBWizardPage		*page,
		const QDomElement	&elem
	)
	:
	KBWizardComboBox
	(	page,
		elem.attribute("name"),
		QStringList(),
		QString::null,
		elem.attribute("editable", "0").toInt()
	),
	m_sourceName	(elem.attribute("source")),
	m_allowEmpty	(elem.attribute("allowempty").toInt())
{
	comboBox()->setInsertionPolicy (RKComboBox::NoInsertion) ;
}

/*  KBWizardCtrlExpr							*/
/*  ~KBWizardCtrlExpr							*/
/*		: Destructor for data source wizard control		*/
/*  (returns)	:		:					*/

KBWizardCtrlExpr::~KBWizardCtrlExpr ()
{
}

void	KBWizardCtrlExpr::pageShown
	(	bool		next
	)
{
	/* Do nothing if moving backwards. The field list will not	*/
	/* have changed and this preserves the user selection.		*/
	if (!next) return ;

	comboBox()->clear () ;

	/* If the source name is empty then use the query which has	*/
	/* beem lodged with the wizard under the "query" cookie.	*/
	if (m_sourceName.isEmpty())
	{
		const KBValue	*cookie	= page()->wizard()->cookie("exprquery") ;
		if (cookie == 0) return	;

		KBNode		*node	= cookie->getNode() ;
		if (node   == 0) return ;

		KBQryBase	*query	= node->isQryBase() ;
		if (query  == 0) return ;


		QPtrList<KBFieldSpec>	fldList	;
		int			pKey	;

		fldList.setAutoDelete (true) ;

		if (query->getFieldList  (0, fldList, pKey))
		{
			for (uint idx = 0 ; idx < fldList.count() ; idx += 1)
				comboBox()->insertItem (fldList.at(idx)->m_name) ;
		}
		else	query->lastError().DISPLAY() ;

		return	;
	}

	/* If the source name is set then get the source (table or	*/
	/* query) from the named source control. This is rather like	*/
	/* the "Fields" wizard control.					*/
	KBWizard	*wizard	= page()->wizard() ;
	QStringList	source	= QStringList::split ('.', m_sourceName) ;
	KBWizardCtrl	*ctrl	= wizard->findCtrl   (source[0], source[1]) ;

	if (ctrl == 0)
	{
		KBError::EError
		(	TR("Cannot locate source control"),
			QString("Path '%1'").arg(m_sourceName),
			__ERRLOCN
		)	;
		return	;
	}

	KBWizardCtrlDataSource::Type type =
		(KBWizardCtrlDataSource::Type)ctrl->attribute("type").toUInt() ;

	/* First case is the source is a table, in which case query the	*/
	/* database to get a list of the columns in the table. Add	*/
	/* these to the source list.					*/
	if (type == KBWizardCtrlDataSource::Table)
	{
		KBTableSpec 	tabSpec (ctrl->value()) ;
		KBDBLink    	dbLink  ;

		if (!dbLink.connect (wizard->dbInfo(), wizard->server()))
		{	dbLink.lastError().DISPLAY() ;
			return	;
		}

		if (!dbLink.listFields (tabSpec))
		{	dbLink.lastError().DISPLAY() ;
			return	;
		}

		if (m_allowEmpty)
			comboBox()->insertItem("") ;

		LITER
		(	KBFieldSpec,
			tabSpec.m_fldList,
			fldSpec,

			comboBox()->insertItem(fldSpec->m_name) ;
		)

		return	;
	}

	/* Second case is a query, in which case we need to load the	*/
	/* query definition and get the field list from that.		*/
	if (type == KBWizardCtrlDataSource::Query)
	{
		KBLocation	locn
				(	wizard->dbInfo(),
					"query",
					wizard->server(),
					ctrl  ->value ()
				)	;

		KBDummyRoot	dummy (locn) ;

		KBQryQuery *qry	 = new KBQryQuery (&dummy) ;

		if (!qry->loadQueryDef (locn))
		{
			qry->lastError().DISPLAY() ;
			return	;
		}

		QPtrList<KBFieldSpec> fldList ;
		int		   pKey	   ;
		fldList.setAutoDelete (true) ;

		if (!qry->getFieldList (0, fldList, pKey))
		{
			qry->lastError().DISPLAY() ;
			return	;
		}

		if (m_allowEmpty)
			comboBox()->insertItem("") ;

		LITER
		(	KBFieldSpec,
			fldList,
			fldSpec,

			comboBox()->insertItem (fldSpec->m_name) ;
		)

		return	;
	}

}


/*  KBWizardCtrlExpr							*/
/*  setValue	: Set data value					*/
/*  value	: const QString & : Value to set			*/
/*  (returns)	: void		  :					*/

void	KBWizardCtrlExpr::setValue
	(	const QString	&
	)
{
}

/*  KBWizardCtrlExpr							*/
/*  value	: Get value						*/
/*  (returns)	: QString	: Value					*/

QString	KBWizardCtrlExpr::value ()
{
	return	comboBox()->currentText() ;
//	return	m_expr->currentText() ;
}

WIZARDCTRLREG(Expr)
