/***************************************************************************
    file	         : kb_wizorder.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"kb_classes.h"
#include	"kb_database.h"
#include	"kb_dblink.h"
#include	"kb_location.h"
#include	"kb_qryquery.h"


#include	"kb_wizsource.h"
#include	"kb_wizfields.h"

#if		__KB_AUTOMOC
#include	"kb_wizorder.moc"
#else
#include	"kb_wizorder.h"
#endif

#include	"rk_vbox.h"
#include	"rk_gridbox.h"
#include	"rk_listbox.h"
#include	"rk_pushbutton.h"


KBWizOrderSpecList::KBWizOrderSpecList
	(	RKListBox	*source,
		RKListBox	*dest,
		RKPushButton	*bAdd,
		RKPushButton	*bAddAll,
		RKPushButton	*bRemove,
		RKPushButton	*bUp,
		RKPushButton	*bDown,
		bool		multi
	)
	:
	KBListBoxPair (source, dest, bAdd, bAddAll, bRemove, bUp, bDown, multi)
{
}

QListBoxItem
	*KBWizOrderSpecList::replicate
	(	QListBoxItem	*item
	)
{
	return	new QListBoxText (item->text()) ;
}


/*  ------------------------------------------------------------------  */

/*  KBWizardCtrlOrder							*/
/*  KBWizardCtrlOrder							*/
/*		: Constructor for ordering wizard control		*/
/*  page	: KBWizardPage *	 : Parent page			*/
/*  elem	: QDomElement  &	 : Definition element		*/
/*  (returns)	: KBWizardCtrlOrder :					*/

KBWizardCtrlOrder::KBWizardCtrlOrder
	(	KBWizardPage		*page,
		const QDomElement	&elem
	)
	:
	KBWizardCtrl	(page, elem.attribute("name")),
	m_sourceName	(elem.attribute  ("source"))
{
	RKVBox		*layMain = new RKVBox (page) ;

	/* The "source" attribute should have the form "page.ctrl"	*/
	/* and names the control (which should actually be of class	*/
	/* "KBWizardCtrlFields") that specifies the fields for the	*/
	/* order.							*/

	setWide		(true)    ;
	setWidget	(layMain) ;

	m_source	= new RKListBox	   (layMain) ;

	RKGridBox	*layGrid = new RKGridBox (3, layMain) ;

	m_bAdd		= new RKPushButton (layGrid) ;
	m_bAddAll	= new RKPushButton (layGrid) ;
	m_bMoveUp	= new RKPushButton (layGrid) ;
	m_bRemove	= new RKPushButton (layGrid) ;
	new QWidget (layGrid) ;
	m_bMoveDown	= new RKPushButton (layGrid) ;

	KBDialog::sameSize (m_bAdd, m_bAddAll, m_bMoveUp, m_bRemove, m_bMoveDown, 0) ;

	m_dest		= new RKListBox	   (layMain) ;

	m_lbPair = new	KBWizOrderSpecList
			(	m_source,
				m_dest,
				m_bAdd,
				m_bAddAll,
				m_bRemove,
				m_bMoveUp,
				m_bMoveDown,
				false
			)	;

	connect
	(	m_lbPair,
		SIGNAL	(destChanged(bool)),
		SLOT	(ctrlChanged())
	)	;
}

/*  KBWizardCtrlOrder							*/
/*  ~KBWizardCtrlOrder							*/
/*		: Destructor for data source wizard control		*/
/*  (returns)	:		:					*/

KBWizardCtrlOrder::~KBWizardCtrlOrder ()
{
	delete	m_lbPair ;
}

/*  KBWizardCtrlOrder							*/
/*  pageShown	: Page containing control is shown			*/
/*  next	: bool		: Moving to next page			*/
/*  (returns)	: void		:					*/

void	KBWizardCtrlOrder::pageShown
	(	bool		next
	)
{
	/* Do nothing if moving backwards. The field list will not	*/
	/* have changed and this preserves the user selection.		*/
	if (!next) return ;

	KBWizard	*wizard	= page()->wizard() ;

	QStringList	source	= QStringList::split ('.', m_sourceName) ;
	KBWizardCtrl	*ctrl	= wizard->findCtrl   (source[0], source[1]) ;

	m_source->clear() ;
	m_dest  ->clear() ;
	m_lbPair->setButtonState() ;

	if (ctrl == 0)
	{
		KBError::EError
		(	TR("Cannot locate source control"),
			QString("Path '%1'").arg(m_sourceName),
			__ERRLOCN
		)	;
		return	;
	}

	QPtrList<KBFieldSpec>	fields	;
	((KBWizardCtrlFields *)ctrl)->fields (fields) ;

	LITER
	(	KBFieldSpec,
		fields,
		f,

		new QListBoxText (m_source, f->m_name) ;
	)

	/* Initialise the button state. Do it this way so that the	*/
	/* code is all in one place, even though the initial settings	*/
	/* should always be the same.					*/
	m_lbPair->setButtonState() ;
}

/*  KBWizardCtrlOrder							*/
/*  setValue	: Set data value					*/
/*  value	: const QString & : Value to set			*/
/*  (returns)	: void		  :					*/

void	KBWizardCtrlOrder::setValue
	(	const QString	&
	)
{
	/* Not used!							*/
}

/*  KBWizardCtrlOrder							*/
/*  value	: Get value						*/
/*  (returns)	: QString	: Value					*/

QString	KBWizardCtrlOrder::value ()
{
	return	QString::null	;
}

/*  KBWizardCtrlOrder							*/
/*  ok		: Check if settings are acceptable			*/
/*  (returns)	: bool		: Acceptable				*/

bool	KBWizardCtrlOrder::ok ()
{
	return	true	;
}

/*  KBWizardCtrlOrder							*/
/*  attribute	: Get attribute value					*/
/*  attr	: const QString & : Attribute to retrieve		*/
/*  (returns)	: QVariant	  :					*/

QVariant
	KBWizardCtrlOrder::attribute
	(	const QString	&attr
	)
{
	if (attr == "order")
	{
		QStringList order	;
		for (uint idx = 0 ; idx < m_dest->count() ; idx += 1)
			order.append (m_dest->text(idx)) ;
		return	QVariant (order) ;
	}

	return	QVariant() ;
}


WIZARDCTRLREG(Order)
