/***************************************************************************
    file	         : kb_wizscript.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qtextbrowser.h>
#include	<qlistbox.h>

#include	"rk_combobox.h"

#include	"kb_classes.h"
#include	"kb_locator.h"
#include	"kb_desktop.h"

#if		__KB_AUTOMOC
#include	"kb_wizscript.moc"
#else
#include	"kb_wizscript.h"
#endif

class	KBScriptInfo : public QListBoxText
{
public	:
	QString		m_tag		;
	QString		m_comment	;

	inline	KBScriptInfo
		(	RKComboBox		*parent,
			QListBoxItem		*after,
			const QString		&tag,
			const QString		&name,
			const QString		&comment
		)
		:
		QListBoxText	(parent->listBox(), name, after),
		m_tag		(tag),
		m_comment	(comment)
	{
	}
}	;


/*  KBWizardCtrlScript							*/
/*  KBWizardCtrlScript							*/
/*		: Constructor for script language wizard control		*/
/*  page	: KBWizardPage *      : Parent page			*/
/*  elem	: const QDomElement & : Definition			*/
/*  (returns)	: KBWizardCtrlScript  :					*/

KBWizardCtrlScript::KBWizardCtrlScript
	(	KBWizardPage		*page,
		const QDomElement	&elem
	)
	:
	KBWizardCtrl	(page, elem.attribute("name"))
{
	setWidget (m_script = new RKComboBox (false, page)) ;
	m_script->setListBox (new QListBox (m_script)) ;

	page->setInfoCtrl (m_info = new QTextBrowser (page)) ;

	loadLanguages () ;

	connect
	(	m_script,
		SIGNAL(activated      (int)),
		SLOT  (ctrlChanged    ())
	)	;
	connect
	(	m_script,
		SIGNAL(activated      (int)),
		SLOT  (slotLangChanged(int))
	)	;
}

/*  KBWizardCtrlScript							*/
/*  ~KBWizardCtrlScript							*/
/*		: Destructor for script language wizard control		*/
/*  (returns)	:		:					*/

KBWizardCtrlScript::~KBWizardCtrlScript ()
{
}

/*  KBWizardCtrlScript							*/
/*  setValue	: Set current value					*/
/*  value	: const QString & : Required value			*/
/*  (returns)	: void		  :					*/

void	KBWizardCtrlScript::setValue
	(	const QString	&value
	)
{
	for (int idx = 0 ; idx < m_script->count() ; idx += 1)
		if (m_script->text(idx) == value)
		{	m_script->setCurrentItem (idx) ;
			break	;
		}
}

/*  KBWizardCtrlScript							*/
/*  value	: Get value						*/
/*  (returns)	: QString	: Value					*/

QString	KBWizardCtrlScript::value ()
{
	KBScriptInfo *s = (KBScriptInfo *)m_script->listBox()->item(m_script->currentItem()) ;
	return	s == 0 ? QString::null : s->m_tag ;
}

/*  KBWizardCtrlScript							*/
/*  loadLanguages: Load available languages				*/
/*  (returns)	 : void		:					*/

void	KBWizardCtrlScript::loadLanguages ()
{
	/* Note the currently selected language, used below to preserve	*/
	/* selections.							*/
	QString		ctag	= value () ;
	KBScriptInfo	*last	= 0 ;

	QString	 	dtDir	= locateDir
				  (	"appdata",
					"services/rekall_dummy.desktop"
				  )	;
	QPtrList<KBDesktop> dtDefs	  ;
	KBDesktop::scan (dtDir + "/services", "rekall_", dtDefs) ;

	m_script->clear() ;

	for (uint idx = 0 ; idx < dtDefs.count() ; idx += 1)
	{
		KBDesktop *desktop = dtDefs.at(idx) ;
		if (desktop->property("ServiceTypes") != "Rekall/Script")
			continue ;

		QString	tag     = desktop->property("X-KDE-RekallPart-Language"   ) ;
		QString	name    = desktop->property("X-KDE-RekallPart-DisplayName") ;
		QString	comment = desktop->property("Comment") ;

		if (comment.isEmpty()) comment = name ;

		/* HACK: Get python at the front of the list. A more	*/
		/*	 general solution would be to give KBDesktop an	*/
		/*	 ordering function.				*/
		last	= new KBScriptInfo
				(	m_script,
					tag == "py" ? 0 : last,
					tag,
					name,
					comment
				)	;
	}

	m_script->setCurrentItem (0) ;
	for (int slot = 0 ; slot < m_script->count() ; slot += 1)
		if (((KBScriptInfo *)m_script->listBox()->item(slot))->m_tag == ctag)
		{	m_script->setCurrentItem (slot) ;
			break	;
		}


	slotLangChanged	(m_script->currentItem()) ;
}


/*  KBWizardCtrlScript							*/
/*  slotLangChanged							*/
/*		: Selected language changes				*/
/*  idx		: int		: New selection				*/
/*  (returns)	: void		:					*/

void	KBWizardCtrlScript::slotLangChanged
	(	int		idx
	)
{
	if (m_script->count() == 0)
	{
		m_info->setText
		(	TR
			(	"There are no available scripting languages. This	"
				"is most likely an installation error. Check that at	"
				"least one scripting language is installed.		"
			)
		)	;

		return	;
	}

	KBScriptInfo *s = (KBScriptInfo *)m_script->listBox()->item(idx) ;
	m_info->setText (s->m_comment) ;
}

WIZARDCTRLREG(Script)
