/***************************************************************************
    file	         : kb_filewidget.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdlib.h>

#include	<qapplication.h>
#include	<qdom.h>

#include	<qgroupbox.h>
#include	<qlabel.h>
#include	<qlistbox.h>
#include	<qlabel.h>
#include	<qwidgetstack.h>
#include	<qspinbox.h>

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_combobox.h"
#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_dialog.h"
#include	"kb_location.h"

#include	"kb_dbinfo.h"
#include	"kb_dblink.h"
#include	"kb_serverinfo.h"
#include	"kb_fieldchooser.h"

#if		__KB_AUTOMOC
#include	"kb_filewidget.moc"
#else
#include 	"kb_filewidget.h"
#endif

#include	"tk_icons.h"
#include	"tk_filedialog.h"



/*  KBFileWidget							*/
/*  KBFileWidget: Constructor for file copier widget			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  part	: QObject *	: Parent copier part			*/
/*  srce	: bool		: File is source			*/
/*  location	: KBLocation &	: Database location			*/
/*  (returns)	: KBTableWidget	:					*/

KBFileWidget::KBFileWidget
	(	QWidget		*parent,
		QObject		*part,
		bool		srce,
		KBLocation	&location
	)
	:
	RKVBox		(parent),
	KBCopyFile	(srce, location),
	m_part		(part),
	m_srce		(srce),
	m_location	(location)
{
	m_cbFixedDelim		= new RKComboBox	(this) ;

	RKHBox	*layHdr		= new RKHBox		(this) ;
	m_cbHeader		= new QCheckBox		(TR("File has column headers"), layHdr) ;

	QLabel	*shl		= new QLabel		(TR("Skip lines"), layHdr) ;
	m_spHeader		= new QSpinBox		(layHdr ) ;

	shl->setAlignment (Qt::Right) ;
	if (!srce)
	{	shl	  ->setEnabled (false) ;
		m_spHeader->setEnabled (false) ;
	}

	m_spHeader->setRange (0, 999999) ;

	m_typeStack		= new QWidgetStack	(this) ;
	m_gbFixed		= new QGroupBox		(1, Qt::Horizontal, TR("Fixed"),     m_typeStack) ;
	m_gbDelim		= new QGroupBox		(2, Qt::Horizontal, TR("Delimited"), m_typeStack) ;
	m_typeStack->addWidget  (m_gbDelim) ;
	m_typeStack->addWidget  (m_gbFixed) ;

				  new QLabel 		(TR("Delimiter"), m_gbDelim) ;
	m_cbDelim		= new RKComboBox	(m_gbDelim) ;
				  new QLabel		(TR("Qualifier"), m_gbDelim) ;
	m_cbQualif		= new RKComboBox	(m_gbDelim) ;
	QLabel	*l3		= new QLabel		(TR("Copy list"), m_gbDelim) ;
	m_leCopyList		= new RKLineEdit	(m_gbDelim) ;

	l3          ->setEnabled (m_srce) ;
	m_leCopyList->setEnabled (m_srce) ;

	m_fixedCols		= new KBEditListView	(true, m_gbFixed) ;
	RKHBox	*layFixed	= new RKHBox		(m_gbFixed) ;
	m_bTable		= new RKPushButton	(layFixed) ;
	m_bSample		= new RKPushButton	(layFixed) ;

	m_gbFile		= new QGroupBox		(3, Qt::Horizontal, TR("File/Error"), this) ;
	m_eFile			= new RKLineEdit	(m_gbFile) ;
	m_bBrowse		= new RKPushButton	(m_gbFile) ;
	m_cbErrOpt		= new RKComboBox	(m_gbFile) ;

	KBDialog::setupLayout  (this) ;

	m_cbFixedDelim	->insertItem (TR("Delimited")) ;
	m_cbFixedDelim	->insertItem (TR("Fixed"    )) ;

	m_cbDelim	->insertItem ("," ) ;
	m_cbDelim	->insertItem (";" ) ;
	m_cbDelim	->insertItem (":" ) ;
	m_cbDelim	->insertItem ("|" ) ;
	m_cbDelim	->insertItem ("<tab>") ;
	m_cbDelim	->setEditable(true) ;

	m_cbQualif    	->insertItem (""  ) ;
	m_cbQualif    	->insertItem ("\"") ;
	m_cbQualif    	->insertItem ("'" ) ;
	m_cbQualif    	->setEditable(true) ;

	m_cbErrOpt    	->insertItem (m_srce ? TR("Ignore excess") : TR("Pad with nulls")) ;
	m_cbErrOpt    	->insertItem (TR("Skip line" )) ;
	m_cbErrOpt    	->insertItem (TR("Abort copy")) ;

	connect
	(	m_cbFixedDelim,
		SIGNAL(activated     (int)),
		SLOT  (fixedSelected (int))
	)	;

	connect (m_cbDelim,    SIGNAL(textChanged (const QString &)), part, SLOT(setChanged())) ;
	connect (m_cbQualif,   SIGNAL(textChanged (const QString &)), part, SLOT(setChanged())) ;
	connect (m_leCopyList, SIGNAL(textChanged (const QString &)), part, SLOT(setChanged())) ;
	connect (m_cbHeader,   SIGNAL(toggled     (bool)), 	       part, SLOT(setChanged())) ;
	connect (m_spHeader,   SIGNAL(valueChanged(int)), 	       part, SLOT(setChanged())) ;


	m_fixedCols->addColumn (TR("Index" )) ;
	m_fixedCols->addColumn (TR("Name"  ), 170) ;
	m_fixedCols->addColumn (TR("Offset")) ;
	m_fixedCols->addColumn (TR("Width" )) ;
	m_fixedCols->addColumn (TR("Strip" )) ;

	m_bBrowse  ->setPixmap (getSmallIcon("browse")) ;
	m_bTable   ->setText   (TR("Set from table")) ;
	m_bSample  ->setText   (TR("Set from sample")) ;

	new KBEditListViewItem (m_fixedCols, "0") ;

	connect	(m_fixedCols,  SIGNAL(changed (uint,uint)), part, SLOT(setChanged ())) ;
	connect	(m_fixedCols,  SIGNAL(inserted(uint)),      part, SLOT(setChanged ())) ;
	connect	(m_fixedCols,  SIGNAL(deleted (uint)),      part, SLOT(setChanged ())) ;
	connect	(this, 	       SIGNAL(changed ()),          part, SLOT(setChanged ())) ;

	connect	(m_bBrowse,    SIGNAL(clicked()), SLOT(clickBrowse())) ;
	connect	(m_bTable,     SIGNAL(clicked()), SLOT(clickTable ())) ;
	connect	(m_bSample,    SIGNAL(clicked()), SLOT(clickSample())) ;

	m_gbFile   ->setFixedHeight (m_gbFile->sizeHint().height()) ;

	m_cbDelim ->lineEdit()->setMaxLength (1) ;
	m_cbQualif->lineEdit()->setMaxLength (1) ;

	m_fixedCols->setEditType (4, KBEditListView::EdCheckBox) ;
	m_typeStack->raiseWidget (m_gbDelim) ;
}

/*  KBFileWidget							*/
/*  saveall	: Save all settings to copy object			*/
/*  (returns)	: void		:					*/

void	KBFileWidget::saveall ()
{
	reset	  () ;

	setWhich   (m_cbFixedDelim->currentItem	()) ;
	setErrOpt  (m_cbErrOpt    ->currentItem	()) ;
	setDelim   (m_cbDelim     ->currentText	()) ;
	setQualif  (m_cbQualif    ->currentText	()) ;
	setCopyList(m_leCopyList  ->text	()) ;
	setFile	   (m_eFile       ->text	()) ;
	setHeader  (m_cbHeader    ->isChecked	(), m_spHeader->value()) ;

	for (QListViewItem *item  = m_fixedCols->firstChild() ;
			    item != 0 ;
			    item  = item->nextSibling())
		if (!item->text(2).isEmpty() && !item->text(3).isEmpty())
			addField
			(	item->text(1),
				item->text(2).toUInt(),
				item->text(3).toUInt(),
				item->text(4) == "Yes"
			) ;
}

/*  KBFileWidget							*/
/*  error	: KBError &	 : Error return				*/
/*  (returns)	: bool		 : Success				*/

bool	KBFileWidget::valid
	(	KBError		&error
	)
{
	saveall	() ;
	return	KBCopyFile::valid (error) ;
}

/*  KBFileWidget							*/
/*  init	: Initialise empty copier				*/
/*  error	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBFileWidget::init
	(	KBError		&error
	)
{
	return	KBCopyFile::init (error) ;
}

/*  KBFileWidget								*/
/*  set		: Set copier from definition				*/
/*  copy	: QDomElement & : Definition parent			*/
/*  error	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBFileWidget::set
	(	QDomElement	&copy,
		KBError		&error
	)
{
	if (!KBCopyFile::set (copy, error)) return false ;

	m_cbFixedDelim->setCurrentItem (getWhich() == 0 ? 0 : 1) ;

	m_cbDelim   ->lineEdit()->setText (getDelim   ()) ;
	m_cbQualif  ->lineEdit()->setText (getQualif  ()) ;
	m_leCopyList->setText		  (getCopyList()) ;
	m_cbErrOpt  ->setCurrentItem      (getErrOpt  ()) ;

	uint		skip	;
	m_cbHeader  ->setChecked	  (getHeader  (skip)) ;
	m_spHeader  ->setValue		  (skip) ;

	uint		idx	;
	QString		name	;
	uint		offset	;
	uint		width	;
	bool		strip	;
	QListViewItem	*last	= 0 ;

	m_fixedCols->clear () ;

	for (idx = 0 ; getField (idx, name, offset, width, strip) ; idx += 1)
		last	= new KBEditListViewItem
			  (	m_fixedCols,
				last,
				QString("%1").arg(idx),
				name,
				QString("%1").arg(offset),
				QString("%1").arg(width ),
				strip ? "Yes" : "No"
			  )	;
	new	KBEditListViewItem (m_fixedCols, last, QString("%1").arg(idx)) ;

	m_eFile->setText (getFile ()) ;
	fixedSelected 	 (getWhich() == 0 ? 0 : 1) ;

	return	true	;
}

/*  KBFileWidget							*/
/*  def		: Get file copy definition				*/
/*  copy	: QDomElement &	: Element to which to attach		*/
/*  (returns)	: void		:					*/

void	KBFileWidget::def
	(	QDomElement	&copy
	)
{
	saveall	() ;
	return	KBCopyFile::def (copy) ;
}

/*  KBFileWidget							*/
/*  fixedSelected: User changes fixed/delimited setting			*/
/*  index	 : int		: Selected item				*/
/*  (returns)	 : void		:					*/

void	KBFileWidget::fixedSelected
	(	int	index
	)
{
//	fprintf
//	(	stderr,
//		"KBFileWidget::fixedSelected: index=%d\n",
//		index
//	)	;

	if (index == 0)
		m_typeStack->raiseWidget (m_gbDelim) ;
	else	m_typeStack->raiseWidget (m_gbFixed) ;

	emit changed () ;
}

/*  KBFileWidget							*/
/*  clickBrowse	: User clicks browse button				*/
/*  (returns)	: void		:					*/

void	KBFileWidget::clickBrowse ()
{
	QString	name	;

	if (m_srce)
		name	= KBFileDialog::getOpenFileName
			  (	QString::null,
			  	"*.txt|Text files\n*.csv|CSV files\n*.*|All files",
			  	qApp->activeWindow(),
			  	TR("Source File")
			  )	;
	else	name	= KBFileDialog::getSaveFileName
			  (	QString::null,
			  	"*.txt|Text files\n*.csv|CSV files\n*.*|All files",
			  	qApp->activeWindow(),
			  	TR("Destination File")
			  )	;

	if (!name.isEmpty())
	{	m_eFile->setText (name) ;
		emit changed  ()     ;
	}
}

/*  KBFileWidget							*/
/*  clickTable	: User clicks table button				*/
/*  (returns)	: void		:					*/

void	KBFileWidget::clickTable ()
{

	KBFieldChooserDlg fDlg (m_location, true, !m_srce) ;

	if (!fDlg.exec()) return ;

	QString		svrName	= fDlg.getServer () ;
	QString		tabName	= fDlg.getObject () ;
	QStringList	fields	= fDlg.getFields () ;

	KBTableSpec 	tabSpec (tabName) ;
	KBDBLink    	dbLink  ;

	if (!dbLink.connect (m_location, svrName))
	{	dbLink.lastError().DISPLAY() ;
		return	;
	}

	if (!dbLink.listFields (tabSpec))
	{	dbLink.lastError().DISPLAY() ;
		return	;
	}

	QListViewItem	*last	= 0 ;
	uint		colno 	= 0 ;
	uint		offset	= 0 ;

	m_fixedCols->clear () ;

	for (uint idx = 0 ; idx < fields.count() ; idx += 1)
	{
		QString	field	= *fields.at(idx) ;

		LITER
		(
			KBFieldSpec,
			tabSpec.m_fldList,
			fldSpec,

			if (fldSpec->m_name == field)
			{
				last	= new KBEditListViewItem
					  (	m_fixedCols,
						last,
						QString("%1").arg(colno ),
						fldSpec->m_name,
						QString("%1").arg(offset),
						QString("%1").arg(fldSpec->m_length)
					  )	;

				colno	+= 1 ;
				offset	+= fldSpec->m_length ;
			}
		)
	}

	new KBEditListViewItem (m_fixedCols, last) ;
}

/*  KBFileWidget							*/
/*  clickSample	: User clicks sample button				*/
/*  (returns)	: void		:					*/

void	KBFileWidget::clickSample ()
{
}
