/***************************************************************************
    file	         : kb_querywidget.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qdom.h>
#include	<qgroupbox.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"

#include	"kb_fieldchooser.h"
#include	"kb_dbinfo.h"
#include	"kb_dblink.h"
#include	"kb_serverinfo.h"

#if		__KB_AUTOMOC
#include	"kb_querywidget.moc"
#else
#include 	"kb_querywidget.h"
#endif

#include	"tk_icons.h"

#include	"rk_hbox.h"
#include	"rk_vbox.h"
#include	"rk_gridbox.h"
#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"
#include	"rk_combobox.h"
#include	"rk_listbox.h"


/*  KBQueryWidget								*/
/*  KBQueryWidget: Constructor for table copier widget			*/
/*  parent	 : QWidget *	: Parent widget				*/
/*  part	 : QObject *	: Parent copier part			*/
/*  srce	 : bool		: Table is source			*/
/*  location	 : KBLocation &	: Database location			*/
/*  (returns)	 : KBQueryWidget:					*/

KBQueryWidget::KBQueryWidget
	(	QWidget		*parent,
		QObject		*part,
		bool		srce,
		KBLocation	&location
	)
	:
	RKVBox		(parent),
	KBCopyQuery	(srce, location),
	m_part		(part),
	m_srce		(srce),
	m_location	(location)
{
	RKHBox	*layQuery	= new RKHBox	 (this  ) ;
	m_cbServer		= new RKComboBox (layQuery ) ;
	m_cbQuery		= new RKComboBox (layQuery ) ;

	RKHBox	*laySelect	= new RKHBox	 (this  ) ;
	m_lbFields		= new RKListBox	 (laySelect) ;
	RKVBox	*layButton	= new RKVBox	 (laySelect) ;
	m_lbSelect		= new RKListBox	 (laySelect) ;

	m_bAdd			= new RKPushButton (layButton) ;
	m_bAddAll		= new RKPushButton (layButton) ;
	m_bRemove		= new RKPushButton (layButton) ;
	m_bUp			= new RKPushButton (layButton) ;
	m_bDown			= new RKPushButton (layButton) ;
	layButton->addFiller()	 ;

	m_fldChooser = new KBFieldChooser
		(	location,
			m_cbServer, m_cbQuery,  m_lbFields, m_lbSelect,
			m_bAdd,	    m_bAddAll,  m_bRemove,  m_bUp,      m_bDown,
			false,
			true
		)	;

	RKHBox	*layExpr	= new RKHBox	   (this) ;
	new QLabel (TR("Expression"), layExpr) ;
	m_eExpr			= new RKLineEdit   (layExpr) ;
	RKPushButton *bExpr	= new RKPushButton (layExpr) ;

	bExpr->setPixmap (getSmallIcon("insert")) ;
	connect	(bExpr, SIGNAL(clicked()), SLOT(clickExpr())) ; 

	QGroupBox *layWhere	= new QGroupBox (2, Qt::Horizontal, srce ? TR("Where/Order") : TR("Operation"), this) ;

	new QLabel    (TR("Where"),    layWhere) ;
	m_eWhere = new RKLineEdit (layWhere) ;
	new QLabel    (TR("Order By"), layWhere) ;
	m_eOrder = new RKLineEdit (layWhere) ;

	connect
	(	m_eWhere,
		SIGNAL	(textChanged(const QString &)),
		part,
		SLOT	(setChanged())
	)	;
	connect
	(	m_eOrder,
		SIGNAL	(textChanged(const QString &)),
		part,
		SLOT	(setChanged())
	)	;


	m_lbFields->setMinimumWidth (120) ;
	m_lbSelect->setMinimumWidth (120) ;

	KBDialog::setupLayout  (this  ) ;

	connect	(this,	       SIGNAL(changed	   ()),	    part, SLOT(setChanged ())) ;
	connect	(m_fldChooser, SIGNAL(fieldsChanged()),     part, SLOT(setChanged ())) ;
	connect	(m_fldChooser, SIGNAL(selectChanged(bool)), part, SLOT(setChanged ())) ;
}

/*  KBQueryWidget							*/
/*  KBQueryWidget: Destructor for query copier widget			*/
/*  (returns)	 :		:					*/

KBQueryWidget::~KBQueryWidget ()
{
	delete	m_fldChooser ;
}


/*  KBQueryWidget							*/
/*  clickExpr	: User clicks add-expression button			*/
/*  (returns)	: void		:					*/

void	KBQueryWidget::clickExpr ()
{
	QString	expr	;

	if (!(expr = m_eExpr->text()).isEmpty())
	{
		int	sCur	= m_lbSelect->currentItem () + 1 ;
		m_lbSelect->insertItem     (expr, sCur) ;
		m_lbSelect->setCurrentItem (sCur) ;
		emit changed () ;
	}
}


/*  KBQueryWidget							*/
/*  saveall	: Save all settings					*/
/*  (returns)	: void		:					*/

void	KBQueryWidget::saveall ()
{
	reset	() ;
	setServer (m_cbServer->currentText()) ;
	setQuery  (m_cbQuery ->currentText()) ;

	for (uint idx = 0 ; idx < m_lbSelect->count() ; idx += 1)
		addField (m_lbSelect->text (idx)) ;

//	setWhere  (m_eWhere->text()) ;
//	setOrder  (m_eOrder->text()) ;
}

/*  KBQueryWidget							*/
/*  error	: KBError &	 : Error return				*/
/*  (returns)	: bool		 : Success				*/

bool	KBQueryWidget::valid
	(	KBError		&error
	)
{
	saveall () ;
	return	KBCopyQuery::valid (error) ;
}

/*  KBQueryWidget							*/
/*  init	: Initialise empty copier				*/
/*  error	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBQueryWidget::init
	(	KBError		&error
	)
{
	return	KBCopyQuery::init (error) ;
}

/*  KBQueryWidget								*/
/*  set		: Set copier from definition				*/
/*  copy	: QDomElement & : Definition parent			*/
/*  error	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBQueryWidget::set
	(	QDomElement	&copy,
		KBError		&error
	)
{
	if (!KBCopyQuery::set (copy, error)) return false ;

	QStringList &fl = getFields () ;

	m_lbSelect->clear () ;
	m_lbFields->clear () ;
	
	if (!m_fldChooser->setServer (getServer ())) return true ;
	if (!m_fldChooser->setObject (getQuery  ())) return true ;

	for (uint idx1 = 0 ; idx1 < fl.count() ; idx1 += 1)
		m_lbSelect->insertItem (*fl.at(idx1)) ;

	m_eWhere->setEnabled(false) ;
	m_eOrder->setEnabled(false) ;
//	m_eWhere->setText (getWhere ()) ;
//	m_eOrder->setText (getOrder ()) ;
	return	true	;
}

/*  KBQueryWidget							*/
/*  def		: Get table copy definition				*/
/*  copy	: QDomElement &	: Element to which to attach		*/
/*  (returns)	: void		:					*/

void	KBQueryWidget::def
	(	QDomElement	&copy
	)
{
	saveall () ;
	return	KBCopyQuery::def (copy) ;
}
