/***************************************************************************
    file	         : kb_sqlwidget.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qdom.h>
#include	<qtextedit.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"
#include	"kb_dialog.h"

#include	"kb_dbinfo.h"
#include	"kb_dblink.h"
#include	"kb_serverinfo.h"

#if		__KB_AUTOMOC
#include	"kb_sqlwidget.moc"
#else
#include 	"kb_sqlwidget.h"
#endif


#include	"rk_vbox.h"
#include	"rk_combobox.h"


/*  KBSQLWidget								*/
/*  KBSQLWidget	: Constructor for SQL copier widget			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  part	: QObject *	: Parent copier part			*/
/*  srce	: bool		: Source, else error			*/
/*  location	: KBLocation &	: Database location			*/
/*  (returns)	: KBSQLWidget	:					*/

KBSQLWidget::KBSQLWidget
	(	QWidget		*parent,
		QObject		*part,
		bool		srce,
		KBLocation	&location
	)
	:
	RKVBox		(parent),
	KBCopySQL	(srce, location),
	m_part		(part),
	m_srce		(srce),
	m_location	(location)
{
	m_cbServer		= new RKComboBox(this) ;
	m_mlSQL			= new QTextEdit	(this) ;

	m_mlSQL->setTextFormat (QTextEdit::PlainText) ;
	KBDialog::setupLayout  (this) ;


	KBServerInfo *self  = location.dbInfo()->findServer (location.server()) ;
	if (!self ->dbType().isEmpty())
		m_cbServer->insertItem ("Self") ;

	KBServerInfo *files = location.dbInfo()->findServer (KBLocation::m_pFile) ;
	if (!files->dbType().isEmpty())
		m_cbServer->insertItem (KBLocation::m_pFile) ;

	QPtrListIterator<KBServerInfo> *svIter = location.dbInfo()->getServerIter () ;
	KBServerInfo		    *svInfo ;

	while ((svInfo = svIter->current()) != 0)
	{
		m_cbServer->insertItem (svInfo->serverName()) ;
		(*svIter) += 1 ;
	}

	delete	svIter	;

	connect	(m_cbServer, SIGNAL(activated  (int)), part, SLOT(setChanged())) ;
	connect	(m_mlSQL,    SIGNAL(textChanged   ()), part, SLOT(setChanged())) ;
}

/*  KBSQLWidget								*/
/*  saveall	: Save all settings					*/
/*  (returns)	: void		:					*/

void	KBSQLWidget::saveall ()
{
	setServer (m_cbServer->currentText()) ;
	setSQL    (m_mlSQL->text()) ;
}

/*  KBSQLWidget								*/
/*  error	: KBError &	 : Error return				*/
/*  (returns)	: bool		 : Success				*/

bool	KBSQLWidget::valid
	(	KBError		&error
	)
{
	saveall () ;
	return	KBCopySQL::valid (error) ;
}

/*  KBSQLWidget								*/
/*  init	: Initialise empty copier				*/
/*  error	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBSQLWidget::init
	(	KBError		&error
	)
{
	return	KBCopySQL::init (error) ;
}

/*  KBSQLWidget								*/
/*  set		: Set copier from definition				*/
/*  copy	: QDomElement & : Definition parent			*/
/*  error	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBSQLWidget::set
	(	QDomElement	&copy,
		KBError		&error
	)
{
	if (!KBCopySQL::set (copy, error)) return false ;

	for (int idx = 0 ; idx < m_cbServer->count() ; idx += 1)
		if (m_cbServer->text(idx) == getServer())
		{	m_cbServer->setCurrentItem (idx) ;
			break	;
		}

	m_mlSQL->setText (getSQL()) ;
	return	true	;

}

/*  KBSQLWidget							*/
/*  def		: Get table copy definition				*/
/*  copy	: QDomElement &	: Element to which to attach		*/
/*  (returns)	: void		:					*/

void	KBSQLWidget::def
	(	QDomElement	&copy
	)
{
	saveall () ;
	return	KBCopySQL::def (copy) ;
}
