/***************************************************************************
    file	         : kb_tablewidget.cpp
    copyright            : (c)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qdom.h>
#include	<qgroupbox.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"

#include	"kb_fieldchooser.h"
#include	"kb_dbinfo.h"
#include	"kb_dblink.h"
#include	"kb_serverinfo.h"

#if		__KB_AUTOMOC
#include	"kb_tablewidget.moc"
#else
#include 	"kb_tablewidget.h"
#endif

#include	"tk_icons.h"

#include	"rk_hbox.h"
#include	"rk_vbox.h"
#include	"rk_gridbox.h"
#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"
#include	"rk_combobox.h"
#include	"rk_listbox.h"


/*  KBTableWidget							*/
/*  KBTableWidget: Constructor for table copier widget			*/
/*  parent	 : QWidget *	: Parent widget				*/
/*  part	 : QObject *	: Parent copier part			*/
/*  srce	 : bool		: Table is source			*/
/*  location	 : KBLocation &	: Database location			*/
/*  (returns)	 : KBTableWidget:					*/

KBTableWidget::KBTableWidget
	(	QWidget		*parent,
		QObject		*part,
		bool		srce,
		KBLocation	&location
	)
	:
	RKVBox		(parent),
	KBCopyTable	(srce, location),
	m_part		(part),
	m_srce		(srce)
{
	RKHBox	*layTable	= new RKHBox	 (this  ) ;
	m_cbServer		= new RKComboBox (layTable ) ;
	m_cbTable		= new RKComboBox (layTable ) ;

	RKHBox	*laySelect	= new RKHBox	 (this  ) ;
	m_lbFields		= new RKListBox	 (laySelect) ;
	RKVBox	*layButton	= new RKVBox	 (laySelect) ;
	m_lbSelect		= new RKListBox	 (laySelect) ;

	m_bAdd			= new RKPushButton (layButton) ;
	m_bAddAll		= new RKPushButton (layButton) ;

	if (!m_srce)
	{
		m_bAddAuto = new RKPushButton (TR("Add Auto"), layButton) ;
		connect (m_bAddAuto, SIGNAL(clicked()), SLOT(slotAddAuto())) ;
	}

	m_bRemove		= new RKPushButton (layButton) ;
	m_bUp			= new RKPushButton (layButton) ;
	m_bDown			= new RKPushButton (layButton) ;
	layButton->addFiller()	 ;

	m_fldChooser = new KBFieldChooser
		(	location,
			m_cbServer, m_cbTable,  m_lbFields, m_lbSelect,
			m_bAdd,	    m_bAddAll,  m_bRemove,  m_bUp,      m_bDown,
			true,
			true
		)	;


	if (m_srce)
	{
		RKHBox	*layExpr	= new RKHBox	   (this) ;
		new QLabel (TR("Expression"), layExpr) ;
		m_eExpr			= new RKLineEdit   (layExpr) ;
		RKPushButton *bExpr	= new RKPushButton (layExpr) ;

		bExpr->setPixmap (getSmallIcon("insert")) ;
		connect	(bExpr, SIGNAL(clicked()), SLOT(clickExpr())) ; 

		QGroupBox *layWhere	= new QGroupBox (2, Qt::Horizontal, srce ? TR("Where/Order") : TR("Operation"), this) ;

		new QLabel    (TR("Where"),    layWhere) ;
		m_eWhere = new RKLineEdit (layWhere) ;
		new QLabel    (TR("Order By"), layWhere) ;
		m_eOrder = new RKLineEdit (layWhere) ;

		connect
		(	m_eWhere,
			SIGNAL	(textChanged(const QString &)),
			part,
			SLOT	(setChanged())
		)	;
		connect
		(	m_eOrder,
			SIGNAL	(textChanged(const QString &)),
			part,
			SLOT	(setChanged())
		)	;
	}

	else
	{
		QGroupBox *layOper	= new QGroupBox (2, Qt::Horizontal, srce ? TR("Where/Order") : TR("Operation"), this) ;

		m_cbOption   = new RKComboBox  (layOper) ;
		m_cbOptField = new RKComboBox  (layOper) ;

		m_cbOption->insertItem (TR("Append"       )) ;
		m_cbOption->insertItem (TR("Replace"	  )) ;
		m_cbOption->insertItem (TR("Update"       )) ;
		m_cbOption->insertItem (TR("Update/insert")) ;
		m_cbOption->insertItem (TR("Insert new"   )) ;
		m_cbOption->insertItem (TR("Compare"      )) ;

		connect	(m_cbOption,   SIGNAL(activated(int)), this, SLOT(optSelected(int))) ;
		connect	(m_cbOptField, SIGNAL(activated(int)), part, SLOT(setChanged (   ))) ;

		m_cbOptField->setEnabled (false) ;
	}

	m_lbFields->setMinimumWidth (120) ;
	m_lbSelect->setMinimumWidth (120) ;

	KBDialog::setupLayout  (this) ;

	connect	(this,	       SIGNAL(changed	  ()),	    part, SLOT(   setChanged ())) ;
	connect	(m_fldChooser, SIGNAL(fieldsChanged()),     part, SLOT(   setChanged ())) ;
	connect	(m_fldChooser, SIGNAL(selectChanged(bool)), part, SLOT(   setChanged ())) ;
	connect	(m_fldChooser, SIGNAL(selectChanged(bool)), this, SLOT(selectChanged (bool))) ;
}

/*  KBTableWidget							*/
/*  KBTableWidget: Destructor for table copier widget			*/
/*  (returns)	 :		:					*/

KBTableWidget::~KBTableWidget ()
{
	delete	m_fldChooser ;
}

/*  KBTableWidget							*/
/*  selectChanged: Handle change in selected fields			*/
/*  delta	 : bool		: True if contents changed		*/
/*  (returns)	 : void		:					*/

void	KBTableWidget::selectChanged
	(	bool	delta
	)
{
	if (delta && !m_srce)
	{
		QString	curVal	= m_cbOptField->currentText() ;
		int	idx1	= -1 ;

		m_cbOptField->clear () ;
		for (uint idx2 = 0 ; idx2 < m_lbSelect->count() ; idx2 += 1)
		{	QString	field	= m_lbSelect->text(idx2) ;
			m_cbOptField->insertItem (field) ;
			if (field == curVal) idx1 = m_cbOptField->count() - 1 ;
		}

		if (idx1 >= 0) m_cbOptField->setCurrentItem (idx1) ;
	}
}

/*  KBTableWidget							*/
/*  slotAddAuto	: Add an auto field					*/
/*  (returns)	: void		:					*/

void	KBTableWidget::slotAddAuto ()
{
	int cur = m_lbSelect->currentItem()  ;
	m_lbSelect->insertItem     ("<Auto>", cur + 1) ;
	m_lbSelect->setCurrentItem (cur + 1) ;
	m_bRemove ->setEnabled	   (true) ;
}

/*  KBTableWidget							*/
/*  clickExpr	: User clicks add-expression button			*/
/*  (returns)	: void		:					*/

void	KBTableWidget::clickExpr ()
{
	QString	expr	;

	if (m_srce && !(expr = m_eExpr->text()).isEmpty())
	{
		int	sCur	= m_lbSelect->currentItem () + 1 ;
		m_lbSelect->insertItem     (expr, sCur) ;
		m_lbSelect->setCurrentItem (sCur) ;
		emit changed () ;
	}
}


/*  KBTableWidget							*/
/*  optSelected	: User selects a destination option			*/
/*  idx		: int		: Option index				*/
/*  (returns)	: void		:					*/

void	KBTableWidget::optSelected
	(	int	idx
	)
{
	if (!m_srce)
	{	m_cbOptField->setEnabled (idx >= 2) ;
		emit	changed	 () ;
	}
}


/*  KBTableWidget							*/
/*  saveall	: Save all settings					*/
/*  (returns)	: void		:					*/

void	KBTableWidget::saveall ()
{
	reset	() ;
	setServer (m_cbServer->currentText()) ;
	setTable  (m_cbTable ->currentText()) ;

	for (uint idx = 0 ; idx < m_lbSelect->count() ; idx += 1)
		addField (m_lbSelect->text (idx)) ;

	if (m_srce)
	{	setWhere  (m_eWhere->text()) ;
		setOrder  (m_eOrder->text()) ;
	}
	else	setOption (m_cbOption->currentItem(), m_cbOptField->currentText()) ;
}

/*  KBTableWidget							*/
/*  error	: KBError &	 : Error return				*/
/*  (returns)	: bool		 : Success				*/

bool	KBTableWidget::valid
	(	KBError		&error
	)
{
	saveall () ;
	return	KBCopyTable::valid (error) ;
}

/*  KBTableWidget							*/
/*  init	: Initialise empty copier				*/
/*  error	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBTableWidget::init
	(	KBError		&error
	)
{
	return	KBCopyTable::init (error) ;
}

/*  KBTableWidget								*/
/*  set		: Set copier from definition				*/
/*  copy	: QDomElement & : Definition parent			*/
/*  error	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBTableWidget::set
	(	QDomElement	&copy,
		KBError		&error
	)
{
	if (!KBCopyTable::set (copy, error)) return false ;

	QStringList &fl = getFields () ;

	m_lbSelect->clear () ;
	m_lbFields->clear () ;

	if (!m_fldChooser->setServer (getServer ())) return true ;
	if (!m_fldChooser->setObject (getTable  ())) return true ;

	for (uint idx1 = 0 ; idx1 < fl.count() ; idx1 += 1)
		m_lbSelect->insertItem (*fl.at(idx1)) ;
	
	if (m_srce)
	{
		m_eWhere->setText (getWhere ()) ;
		m_eOrder->setText (getOrder ()) ;
		return	true ;
	}


	m_cbOption->setCurrentItem (getOption()) ;

	for (uint idx2 = 0 ; idx2 < fl.count() ; idx2 += 1)
		m_cbOptField->insertItem (*fl.at(idx2)) ;

	for (int idx3 = 0 ; idx3 < m_cbOptField->count() ; idx3 += 1)
		if (m_cbOptField->text(idx3) == getOptField())
		{	m_cbOptField->setCurrentItem (idx3) ;
			break	;
		}

	m_cbOptField->setEnabled (m_cbOption->currentItem() >= 2) ;
	return	true	;

}

/*  KBTableWidget							*/
/*  def		: Get table copy definition				*/
/*  copy	: QDomElement &	: Element to which to attach		*/
/*  (returns)	: void		:					*/

void	KBTableWidget::def
	(	QDomElement	&copy
	)
{
	saveall () ;
	return	KBCopyTable::def (copy) ;
}
