/***************************************************************************
    file	         : kb_editor.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<stdio.h>
#include	<qsplitter.h>
#include 	<qtextbrowser.h>


#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_dbinfo.h"
#include	"kb_gui.h"
#include	"kb_script.h"
#include	"kb_viewer.h"
#include	"kb_dialog.h"
#include	"kb_options.h"
#include	"kb_font.h"

#if		__KB_AUTOMOC
#include	"kb_editor.moc"
#else
#include 	"kb_editor.h"
#endif

#include	"kb_textedit.h"

#include	"tk_icons.h"
#include	"tk_messagebox.h"

#include	"rk_listbox.h"

/*  KBEditor	: 							*/
/*  KBEditor	: Constructor for simple script editor			*/
/*  parent	: KBObjBase *	: Parent object				*/
/*  embed	: QWidget *	: Embedding widget			*/
/*  (returns)	: KBEditor	:					*/

KBEditor::KBEditor
	(	KBObjBase	*parent,
		QWidget		*embed
	)
	:
	KBViewer   (parent, embed, WStyle_NormalBorder),
	m_splitter (new QSplitter (m_partWidget))
{
	m_splitter->setOrientation (QSplitter::Vertical) ;

	m_editWin   = new KBTextEdit  (m_splitter) ;

	m_editWin->setWordWrap     (QTextEdit::NoWrap) ;
	m_editWin->setFont         (KBFont::specToFont(KBOptions::getScriptFont())) ;
	m_editWin->showLineNumbers (true) ;

	connect
	(	m_editWin,
		SIGNAL	(textChanged()),
		SLOT	(modified   ())
	)	;

	m_errorWin  = new QTextBrowser(m_splitter) ;
	m_gui	    = 0	 ;

	m_splitter->show () ;
	m_editWin ->show () ;
	m_errorWin->show () ;

	if (getLocation().extension().isEmpty())
	{
		KBError::EError
		(	QString (TR("Script file \"%1\" does not have an extension")).arg(getLocation().name()),
			TR("Will not be able to compile"),
			__ERRLOCN
		)	;
		m_scrIface = 0 ;
	}
	else
	{
		KBError	error	;
		m_scrIface = LinkKBScript (getLocation().extension(), error) ;
		if (m_scrIface == 0) error.DISPLAY () ;
	}

	if (m_scrIface != 0)
		m_scrIface->editorInit (m_editWin) ;

	TKConfig *config = TKConfig::getConfig () ;
	config->setGroup ("Editor Options") ;
	m_size	 = config->readSizeEntry ("Geometry" ) ;
	m_ewind	 = config->readNumEntry  ("EditDepth") ;

	if (m_size.isEmpty())
		m_size = 	  QSize (500, 400)  ;
	else	m_size.boundedTo (QSize (300, 200)) ;

	m_topWidget = m_splitter	;

	setGUI (m_gui = new KBaseGUI (this, this, "rekallui_editor.gui")) ;
	if (m_scrIface == 0) m_gui->setEnabled ("KB_compile", false) ;

	m_partWidget->resize	     (m_size.width(), m_size.height(), true, false) ;
	m_partWidget->setMinimumSize (300, 200) ;
	m_partWidget->setIcon	     (getSmallIcon ("shellscript")) ;
	m_partWidget->show	     ()  ;

	QValueList<int> sizes ;

	if (m_ewind == 0)
	{	sizes.append  (m_size.height() - 45) ;
		sizes.append  (45)    ;
	}
	else
	{	sizes.append  (m_ewind) ;
		sizes.append  (m_size.height() - m_ewind) ;
	}
	m_splitter->setGeometry (0, 0, m_size.width(), m_size.height()) ;
	m_splitter->setSizes    (sizes) ;

	connect	(m_editWin, SIGNAL(modified()), SLOT(modified())) ;
}

/*  KBEditor	: 							*/
/*  ~KBEditor	: Destructor for simple script editor			*/
/*  (returns)	:		:					*/

KBEditor::~KBEditor ()
{
	TKConfig *config = TKConfig::getConfig () ;

	config->setGroup   ("Editor Options"    ) ;
	config->writeEntry ("Geometry",  m_size ) ;
	config->writeEntry ("EditDepth", m_ewind) ;
	config->sync	   () ;
}

/*  KBEditor								*/
/*  startup	: Startup editor					*/
/*  create	: bool		 : New script creation			*/
/*  (returns)	: bool		 : Success				*/

bool	KBEditor::startup
	(	const QString	&errText,
		uint		lno,
		bool		create
	)
{
	if (!create) loadFiles (errText, lno) ;

	setCaption  (getLocation().title()) ;
	return	true	;
}


/*  KBEditor	: 							*/
/*  modified	: Handle scintilla modification notice			*/
/*  (returns)	: void		  :					*/

void	KBEditor::modified ()
{
	m_gui->setEnabled ("KB_saveDoc", m_editWin->isModified()) ;
}

/*  KBEditor	: 							*/
/*  gotoLine	: Go to line number					*/
/*  lno		: uint		  : Line number				*/
/*  (returns)	: void		  :					*/

void	KBEditor::gotoLine
	(	uint	lno
	)
{
	m_editWin->setCursorPosition (lno, 0) ;
	m_editWin->setFocus () ;
}

/*  KBEditor	: 							*/
/*  loadFiles	: Load files into edit and error windows		*/
/*  eText	: const QString & : Script errors			*/
/*  lno		: uint		  : Initial line number			*/
/*  (returns)	: void		  :					*/

void	KBEditor::loadFiles
	(	const QString	&eText,
		uint		lno
	)
{
	QString	doc	;
	KBError	error	;

	doc = getLocation().contents (error) ;
	if (doc.isNull())
	{	error.DISPLAY () ;
		return	;
	}
	
	m_editWin ->setText (doc  ) ;
	m_errorWin->setText (eText) ;

	gotoLine (lno) ;
}

bool	KBEditor::queryClose ()
{
	if (m_editWin->isModified())
		if (TKMessageBox::questionYesNo
		   	(	0,
				QString (TR("Script file \"%1\" has been changed: close anyway?")).arg(getLocation().name()),
				TR("Edit script file")
			)
			!= TKMessageBox::Yes) return false ;

	m_size	= m_partWidget->size () ;
	m_ewind	= m_splitter  ->sizes() [0] ;
	return	true	;
}

/*  KBEditor	: 							*/
/*  saveDocument: Handle file save					*/
/*  (returns)	: void		:					*/

void	KBEditor::saveDocument ()
{
	if (m_objBase->saveDocument())
	{
		m_gui     ->setEnabled  ("KB_saveDoc", false) ;
		m_editWin ->setModified (false) ;
		setCaption (getLocation().title()) ;
	}
}

/*  KBEditor	   : 							*/
/*  saveDocumentAs : Handle file save-as				*/
/*  (returns)	   : void	:					*/

void	KBEditor::saveDocumentAs ()
{
	if (m_objBase->saveDocumentAs())
	{
		m_gui     ->setEnabled  ("KB_saveDoc", false) ;
		m_editWin ->setModified (false) ;
		setCaption (getLocation().title()) ;
	}
}

QString	KBEditor::def ()
{
	return	m_editWin->text().stripWhiteSpace() + "\n" ;
}

/*  KBEditor	: 							*/
/*  doCompile	: Handle file compile					*/
/*  (returns)	: void		: Success				*/

void	KBEditor::doCompile ()
{
	if (m_editWin->isModified())
		if (m_objBase->saveDocument())
		{	m_gui     ->setEnabled  ("KB_saveDoc", false) ;
			m_editWin ->setModified (false) ;
		}

	if (m_scrIface == 0) return ;

	KBError	error	;

	if (!m_scrIface->compile (getLocation(), error))
	{
		QString	eMsg    ;
		uint	eLine   ;
		QString	eText   ;
		m_scrIface->lastError (eMsg, eLine, eText) ;

		error.DISPLAY () ;
		reloadScript  (eMsg, eLine) ;
		return	;
	}

	m_errorWin->clear () ;
}

/*  KBEditor	: 							*/
/*  reloadFile	: Reload files						*/
/*  errText	: const QString & : Script error text			*/
/*  lno		: uint		  : Initial line number			*/
/*  (returns)	: void		  :					*/

void	KBEditor::reloadScript
	(	const QString	&errText,
		uint		lno
	)
{
	if (m_editWin->isModified())
		if (TKMessageBox::questionYesNo
		   	(	0,
				QString (TR("Script file \"%1\" has been changed: reload anyway?")).arg(getLocation().name()),
				TR("Edit script file")
			)
			!= TKMessageBox::Yes) return ;

	loadFiles   (errText, lno) ;
}

/*  KBEditor	: 							*/
/*  getChanged	: Report whether there are unsaved changes		*/
/*  both	: bool		: True for both data and design change	*/
/*  (returns)	: cchar *	: Changed message or null if none	*/

cchar	*KBEditor::getChanged
	(	bool
	)
{
	return	m_editWin->isModified () ? "text" : 0 ;
}

void	KBEditor::showAs
	(	KB::ShowAs
	)
{
}
