/***************************************************************************
    file	         : kb_formlist.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_dbinfo.h"
#include	"kb_listitem.h"
#include	"kb_viewer.h"

#include	"kb_nodereg.h"
#include	"kb_parse.h"
#include	"kb_appptr.h"
#include	"kb_callback.h"
#include	"kb_dbdociter.h"
#include	"kb_file.h"
#include	"kb_errordlg.h"
#include 	"kb_popupmenu.h"
#include	"kb_formbase.h"
#include	"kb_formviewer.h"
#include	"kb_display.h"
#include	"kb_script.h"
#include	"kb_options.h"

#if		__KB_AUTOMOC
#include	"kb_formlist.moc"
#else
#include	"kb_formlist.h"
#endif

#if		! __KB_RUNTIME
#include	"kb_wizform.h"
#include	"kb_parse.h"
#include	"kb_test.h"
#include	"kb_testsuite.h"
#include	"kb_testsuiteresultsdlg.h"
#include	"kb_formtransaction.h"
#endif

#include	"tk_messagebox.h"
#include	"rk_vbox.h"
#include	"tk_filedialog.h"
#include	"tk_icons.h"


#include 	<qcursor.h>

#define		__DEBUG_FORMLIST	(0)
#if		__DEBUG_FORMLIST
#include	"kb_dprintf.h"
#endif


/*  KBFormList								*/
/*  KBFormList	: Constructor for form list widget			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  dbInfo	: KBDBInfo *	: Database information object		*/
/*  (returns)	: KBFormList	:					*/

KBFormList::KBFormList
	(	QWidget		*parent,
		KBDBInfo	*dbInfo
	)
	:
	KBFileList
	(	parent,
		dbInfo,
		"Create new form",
		"form",
		"form",
		"Form name"
	),
	m_objectMenu	(0, &m_buttonState)
{
	enableWizard () ;
	enableWeb    () ;
}

/*  KBFormList								*/
/*  ~KBFormList	: Destructor for form list widget			*/
/*  (returns)	:		:					*/

KBFormList::~KBFormList ()
{
}

/*  KBFormList								*/
/*  showServerMenu : Show popup menu for server entries			*/
/*  (returns)	   : void	:					*/

void	KBFormList::showServerMenu
	(	int
	)
{
	KBPopupMenu pop (0) ;

	if ((m_curItem != 0) && (m_curItem->type() == KBListItem::Server))
		pop.setTitle   (QString(TR("Forms: %1")).arg(m_curItem->text(0))) ;
	else	pop.setTitle   (TR("Forms")) ;

	pop.insertItem
	(	getSmallIcon("reload"),
		TR("&Reload form list"),
		this,
		SLOT(reloadServer ())
	)	;
#if	! __KB_RUNTIME

	KBServerInfo::TestsEnable testsEnable = KBServerInfo::TestsOff ;
	if ((m_curItem != 0) && (m_curItem->type() == KBListItem::Server))
		testsEnable = m_dbInfo->findServer(m_curItem->text(0))->showTests() ;

	pop.insertItem
	(	getSmallIcon("editcopy"),
		TR("&Copy to server ..."),
		this,
		SLOT(copyToServer ())
	)	;
	pop.insertItem
	(	getSmallIcon("filesave"),
		TR("Save all to &files ..."),
		this,
		SLOT(saveAllToFile())
	)	;
	pop.insertItem
	(	getSmallIcon("www"),
		TR("Save all to &web ..."),
		this,
		SLOT(saveAllToWeb ())
	)	;

	switch (testsEnable)
	{
		case KBServerInfo::TestsSuite  :
		case KBServerInfo::TestsRecord :
			pop.setTitle (TR("Test Suites")) ;
			pop.insertItem
			(	TR("Execute tests ..."),
				this,
				SLOT(slotExecuteTestAll())
			)	;
			break	;

		default	:
			break	;
	}
#endif
	pop.exec       (QCursor::pos()) ;
	return	;
}

#if	! __KB_RUNTIME

/*  KBFormList								*/
/*  listAllSuites: List all test suites					*/
/*  server	 : const QString & : Server name			*/
/*  name	 : const QString & : Form name				*/
/*  (returns)	 : QStringPairList : Suite names (comments pending)	*/

QStringPairList
	KBFormList::listAllSuites
	(	const QString		&server,
		const QString		&name
	)
{
	KBError		error	;
	KBForm		*form	;
	QByteArray	xml	;
	QStringPairList	suites	;

	KBLocation location
		   (	m_dbInfo,
			"form",
			server,
			name
		   )	;

	if (!location.contents (xml, error))
		return	suites	;

	if ((form = KBOpenFormText (location, xml, error)) == 0)
		return	suites	;

	DPRINTF
	((	"KBFormList::listAllSuites: %s.%s: form parsed\n",
		server.latin1(),
		name  .latin1()
	))	;

	TITER
	(	TestSuite,
		form->getChildren(),
		t,

		QStringPair    sp  ;
		sp.m_first  = t->getName () ;
		sp.m_second = QString::null ; 
		suites.append (sp) ;
	)

	return	suites	;
}

/*  KBFormList								*/
/*  executeTestSuite							*/
/*		: Execute a test suite					*/
/*  location	: KBLocation &		: Form to be tested		*/
/*  suiteName	: const QString & 	: Test suite name		*/
/*  pause	: bool		  	: Pause at end of test		*/
/*  tsrDlg	: KBTestSuiteResultsDlg	: Results dialog		*/
/*  (returns)	: KBScriptTestResult *	: Returned if error		*/

KBScriptTestResult
	*KBFormList::executeTestSuite
	(	KBLocation		&location,
		const QString		&suiteName,
		bool			pause,
		KBTest::ErrorOpt	errorOpt,
		KBTestSuiteResultsDlg	&tsrDlg
	)
{
	XPRINTF
	((	"KBFormList::executeTestSuite: fl=[%s] sn=[%s]\n",
		location.ident().latin1(),
		suiteName.latin1()
	))	;

	KBError		error		;
	KBFormBase	formBase (0)	;

	KBTest::setObjectFixed (false)	;

	/* Load the form. This, and the call below to KBFormBase::show	*/
	/* essentially duplicates the normal execution of a form.	*/
	if (!formBase.build (location, false, error))
		return	new KBScriptTestResult
				(	TR("Form Test Error"),
					0,
					QString::null,
					KBScriptTestResult::testFailed,
					QString(TR("Error loading form: %1: %2"))
						.arg(error.getMessage())
						.arg(error.getDetails()),
					"",
					QString::null
				)	;

	KBForm		*form	= formBase.getTopNode()->isForm() ;

	/* Locate the test suite. There really should not be an error	*/
	/* here so popup up a message box as well as returning the	*/
	/* error to the caller.						*/
	KBTestSuite *suite	= 0 ;
	TITER
	(	TestSuite,
		form->getChildren(),
		t,

		if (t->getName() == suiteName)
		{
			suite	= t ;
			break	;
		}
	)

	if (suite == 0)
	{
		TKMessageBox::sorry
		(	0,
			QString(TR("Internal error: test suite '%1' not found")).arg(suiteName),
			TR("Execute test suite")
		)	;
		return	new KBScriptTestResult
				(	TR("Form Test Error"),
					0,
					QString::null,
					KBScriptTestResult::testFailed,
					TR("Test suite not found"),
					"",
					QString::null
				)	;
	}

	/* Scan for initialise and reset tests, recording errors if	*/
	/* either is expected but not found.				*/
	KBTest *initialise = 0 ;
	KBTest *reset	   = 0 ;

	LITER
	(	KBTest,
		form->getTests(),
		t,

		if (t->getName() == suite->initialise())
			initialise = t	;
		if (t->getName() == suite->reset    ())
			reset      = t	;
	)

	KBFormTransaction ft (form->getDocRoot(), suite->transaction()) ;
	ft.begin () ;

	if (!suite->initialise().isEmpty())
		if (initialise == 0)
			return	new KBScriptTestResult
					(	TR("Form Test Error"),
						0,
						initialise->comment(),
						KBScriptTestResult::testFailed,
						QString(TR("Test suite initialiser not found: %1"))
							.arg(suite->initialise()),
						"",
						QString::null
					)	;
	if (!suite->reset().isEmpty())
		if (reset      == 0)
			return	new KBScriptTestResult
					(	TR("Form Test Error"),
						0,
						reset->comment(),
						KBScriptTestResult::testFailed,
						QString(TR("Test suite reset not found: %1"))
							.arg(suite->reset()),
						"",
						QString::null
					)	;

	/* Execute the initialiser if found. Note that this is done	*/
	/* before the form is actually started, so its limited as to	*/
	/* what it can do.						*/
	if (initialise != 0)
	{
		bool evRc ;
		form->KBObject::eventHook (*initialise, 0, 0, evRc) ;
	}


	QDict<QString>	pDict	;
	KB::ShowRC	orc	= formBase.show (KB::ShowAsData, pDict, 0, error, KBValue(), 0) ;
	if (orc != KB::ShowRCOK)
		return	new KBScriptTestResult
				(	TR("Form Test Error"),
					0,
					QString::null,
					KBScriptTestResult::testFailed,
					QString(TR("Error opening form: %1: %2"))
						.arg(error.getMessage())
						.arg(error.getDetails()),
					"",
					QString::null
				)	;

	/* Pump the events so that all the resizes get done, else we	*/
	/* may not see all the controls correctly sized.		*/
	qApp->processEvents () ;


	/* Rock and roll!!!! Run the tests. The test execution is	*/
	/* wrapped to calls to setup and clear down test mode, and to	*/
	/* transfer error results to the test suite result dialog.	*/
	KBTestSuite::executeStart   (form->isDocRoot()) ;
	KBTest::ErrorOpt rc = suite->executeTests (errorOpt, true) ;
	bool 		 ok = KBTestSuite::executeResults (form->isDocRoot(), tsrDlg) ;
	KBTestSuite::executeFinish  (form->isDocRoot()) ;

	KBScriptTestResult tsr
				(	QString::null,
					0,
					QString::null,
					ok ?
						KBScriptTestResult::testOK    :
						KBScriptTestResult::testFailed,
					QString::null,
					"",
					QString::null
				)	;
	tsrDlg.addResults  (&tsr)	;

	/* On completion pause if requested so that the user can see	*/
	/* the form in its final state.					*/
	if (pause)
		TKMessageBox::information
		(	0,
			TR("Test suite completed"),
			TR("Execute test suite"  )
		)	;


	if (reset != 0)
	{
		bool evRc ;
		form->KBObject::eventHook (*reset, 0, 0, evRc) ;
	}

	ft.rollback ()	;

	if (rc == KBTest::ErrorAbortAll)
		return	new KBScriptTestResult
				(	TR("Form Test Error"),
					0,
					QString::null,
					KBScriptTestResult::testFailed,
					TR("Aborted all tests"),
					"",
					QString::null
				)	;

	if (KBTest::getObjectFixed())
	{
		XPRINTF
		((	"KBFormList::slotExecuteTestAll: FIXED\n"
		))	;

		formBase.saveDocumentAs () ;
	}

	return	0	;
}
#endif


/*  KBFormList								*/
/*  slotExecuteInServer							*/
/*		: Execute a form in a specified server			*/
/*  id		: int		: Menu identifier for server		*/
/*  (returns)	: void		:					*/

void	KBFormList::slotExecuteInServer
	(	int		id
	)
{
#if	! __KB_RUNTIME
	KBLocation	location ;
	KBError		error 	 ;
	QDict<QString>	empty	 ;
	KBCallback	*cb	 = KBAppPtr::getCallback() ;

	if (!itemToLocation (m_curItem, location))
		return ;

	XPRINTF
	((	"KBFormList::slotExecuteInServer: data=[%s]\n",
		m_objectMenu.text(id).latin1()
	))	;

	location.setDataServer (m_objectMenu.text(id)) ;
	if (cb->openObject (0, location, KB::ShowAsData, empty, error) == KB::ShowRCError)
		error.DISPLAY () ;
#endif
}

/*  KBFormList								*/
/*  slotExecuteTestSuite						*/
/*		: Execute a test suite					*/
/*  id		: int		: Menu identifier for suite		*/
/*  (returns)	: void		:					*/

void	KBFormList::slotExecuteTestSuite
	(	int		id
	)
{
#if	! __KB_RUNTIME
	KBLocation location	;
	if (!itemToLocation (m_curItem, location))
		return	;

	KBTestSuiteResultsDlg	  tsrDlg ;

	tsrDlg.setSuite (m_objectMenu.text(id)) ;

	KBScriptTestResult *rc = executeTestSuite
				 (	location,
					m_objectMenu.text(id),
					(m_buttonState & Qt::ControlButton) != 0,
					KBTest::ErrorAbortTest,
					tsrDlg
				 )	;

	if (rc != 0)
	{	tsrDlg.addResults(rc)	;
		delete rc ;
	}

	tsrDlg.exec ()	;
#endif
}

/*  KBFormList								*/
/*  slotExecuteAllSuites						*/
/*		: Execute all test suites for the current form		*/
/*  (returns)	: void		:					*/

void	KBFormList::slotExecuteAllSuites ()
{
#if	! __KB_RUNTIME
	KBLocation	location	;
	if (!itemToLocation (m_curItem, location))
		return	;

	KBTestSuiteResultsDlg	tsrDlg	;
	QStringPairList		suites	= listAllSuites
					  (	m_curItem->parent()->text(0),
						m_curItem          ->text(0)
					  )	;

	for (uint idx = 0 ; idx < suites.count() ; idx += 1)
	{
		XPRINTF
		((	"KBFormList::slotExecuteAllSuites: %d: %s\n",
			idx,
			suites[idx].m_first.latin1()
		))	;

		tsrDlg.setSuite (suites[idx].m_first) ;

		KBScriptTestResult *rc = executeTestSuite
					 (	location,
						suites[idx].m_first,
						(m_buttonState & Qt::ControlButton) != 0,
						KBTest::ErrorAbortTest,
						tsrDlg
					 )	;
		if (rc != 0)
		{	tsrDlg.addResults(rc)	;
			delete rc ;
			break	  ;
		}
	}

	tsrDlg.exec ()	;
#endif
}

/*  KBFormList								*/
/*  slotExecuteTestAll							*/
/*		: Execute all (or selection thereof) test suites	*/
/*  (returns)	: void		:					*/

void	KBFormList::slotExecuteTestAll ()
{
#if	! __KB_RUNTIME
	KBError		error	;
	bool		anyTest	= false	;
	QString		server	= m_curItem->text (0) ;
	KBDBDocIter	docIter ;

	if (KBOptions::getCacheInTests())
		KBLocation::setCacheAll (true)	;

	if (!docIter.init
		(	m_dbInfo,
			server,
			m_tabType,
			KBLocation::extnForType (m_dbInfo, m_tabType, getDocExtension()),
			error,
			false
		))
	{
		KBLocation::setCacheAll (false)	;
		error.DISPLAY() ;
		return	;
	}

	KBTestAllDlg taDlg (m_dbInfo, server) ;
	QString	     name  ;
	QString	     stamp ;

	while (docIter.getNextDoc (name, stamp))
	{
		QStringPairList suites = listAllSuites (server, name) ;
		if (suites.count() > 0)
		{
			taDlg.addSuites (name, suites) ;
			anyTest	= true	;
		}
	}


	if (!anyTest)
		TKMessageBox::information
		(	0,
			TR("No test suites found"),
			TR("Execute Tests")
		)	;

	if (!taDlg.exec())
	{
		KBLocation::setCacheAll (false)	;
		return ;
	}

	QStringPairList		selected = taDlg.selected() ;
	KBTest::ErrorOpt	errorOpt = taDlg.errorOpt() ;
	KBTestSuiteResultsDlg	tsrDlg	;

	for (uint idx = 0 ; idx < selected.count() ; idx += 1)
	{
		XPRINTF
		((	"KBFormList::slotExecuteTestAll: [%s][%s]\n",
			selected[idx].m_first .latin1(),
			selected[idx].m_second.latin1()
		))	;

		tsrDlg.setObject (selected[idx].m_first ) ;
		tsrDlg.setSuite  (selected[idx].m_second) ;

		KBLocation location
			   (	m_dbInfo,
				"form",
				m_curItem->text(0),
				selected[idx].m_first
		   	   )	;
		location.setDataServer (taDlg.server()) ;


		KBScriptTestResult *rc = executeTestSuite
					 (	location,
						selected[idx].m_second,
						false,
						errorOpt,
						tsrDlg
					 )	;
		if (rc != 0)
		{	tsrDlg.addResults(rc)	;
			delete rc ;
			break	  ;
		}
	}

	KBLocation::setCacheAll (false)	;
	tsrDlg.exec ()	;
#endif
}

#if	! __KB_RUNTIME

/*  KBFormList								*/
/*  addTestMenu	: Add tests entries to menu				*/
/*  popup	: KBPopupMenu &	: Popup menu to which to add		*/
/*  (returns)	: void		:					*/

void	KBFormList::addTestMenu
	(	KBPopupMenu	&popup
	)
{
	/* Nothing to do it we are not on a form item in the list, or	*/
	/* test suites are not enabled.					*/
	if ((m_curItem == 0) || (m_curItem->type() != KBListItem::Object))
		return	;

	switch (m_dbInfo->findServer(m_curItem->parent()->text(0))->showTests())
	{
		case KBServerInfo::TestsSuite  :
		case KBServerInfo::TestsRecord :
			break	;

		default	:
			return	;
	}

	QPtrListIterator<KBServerInfo>	*svIter = m_dbInfo->getServerIter() ;
	KBServerInfo			*svInfo ;
	if (svIter->count() > 0)
	{
		popup.setTitle(TR("Data view from server")) ;

		while ((svInfo = svIter->current()) != 0)
		{	popup.insertItem
			(	svInfo->serverName(),
				this,
				SLOT(slotExecuteInServer(int))
			)	;
			(*svIter) += 1 ;
		}
	}
	delete	svIter	;

	QStringPairList	suites	= listAllSuites
				  (	m_curItem->parent()->text(0),
					m_curItem          ->text(0)
				  )	;
	if (suites.count() == 0)
		return	;

	/* Got some tests suites, so add a title into the menu and then	*/
	/* add an entry for each test suite.				*/
	DPRINTF
	((	"KBFormList::addTestMenu: form has test suites\n"
	))	;

	popup.setTitle (TR("Test Suites")) ;

	popup.insertItem
	(	TR("All Suites"),
		this,
		SLOT(slotExecuteAllSuites())
	)	;

	for (uint idx = 0 ; idx < suites.count() ; idx += 1)
		popup.insertItem
		(	suites[idx].m_first,
			this,
			SLOT(slotExecuteTestSuite(int))
		)	;
}
#endif

/*  KBFormList								*/
/*  showObjectMenu : Show popup menu for server entries			*/
/*  (returns)	   : void	:					*/

void	KBFormList::showObjectMenu
	(	int		state
	)
{
	m_objectMenu.clear ()	;

	if ((m_curItem != 0) && (m_curItem->type() == KBListItem::Object))
		m_objectMenu.setTitle   (QString(TR("Forms: %1")).arg(m_curItem->text(0))) ;
	else	m_objectMenu.setTitle   (TR("Form")) ;

	m_objectMenu.insertItem
	(	getSmallIcon("dataview"),
		TR("&Data view"),
		this,
		SLOT(showAsData   ())
	)	;
#if	! __KB_RUNTIME
	m_objectMenu.insertItem
	(	getSmallIcon("designview"),
		TR("D&esign view"),
		this,
		SLOT(showAsDesign ())
	)	;
	m_objectMenu.insertItem
	(	TR("&Rename form"),
		this,
		SLOT(renameForm   ())
	)	;
	m_objectMenu.insertItem
	(	getSmallIcon("editdelete"),
		TR("De&lete form"),
		this,
		SLOT(deleteForm   ())
	)	;
	m_objectMenu.insertItem
	(	getSmallIcon("filesave"),
		TR("&Save to file"),
		this,
		SLOT(saveObjToFile())
	)	;
	m_objectMenu.insertItem
	(	getSmallIcon("www"),
		TR("Save to &web"),
		this,
		SLOT(saveObjToWeb ())
	)	;

	if ((state & Qt::ControlButton) != 0)
		addTestMenu (m_objectMenu) ;
#endif

	m_objectMenu.exec   (QCursor::pos()) ;
}

#if	! __KB_RUNTIME

/*  KBFormList								*/
/*  createByWizard							*/
/*		: Handle request to create object using a wizard	*/
/*  item	: KBListItem *	: Selected item				*/
/*  (returns)	: void		:					*/

void	KBFormList::createByWizard
	(	KBListItem	*item
	)
{
	QString		server	= item->parent()->text(0) ;
	KBLocation	dummy	(m_dbInfo, "form", server, "") ;

	KBWizardForm	wizForm (dummy, server)	;
	if (!wizForm.exec()) return ;

	KB::ShowAs	showAs	;
	QString		name	;
	QString		form	= wizForm.create (name, showAs)	;

	if (form.isEmpty ()) return ;

	KBLocation location	(m_dbInfo, "form", server, name) ;
	KBError	   error	;

	if (!location.save
		(	QString::null,
			QString::null,
			form,
			error
		))
	{
		error.DISPLAY()	;
		return	;	;
	}

	QDict<QString>	empty	 ;
	KBCallback	*cb	= KBAppPtr::getCallback() ;
	if (cb->openObject (0, location, showAs, empty, error) == KB::ShowRCError)
		error.DISPLAY () ;
}
#endif

/*  KBFormList								*/
/*  renameForm	: Rename a form						*/
/*  (returns)	: void		:					*/

void	KBFormList::renameForm ()
{
#if	! __KB_RUNTIME
	rename	(m_curItem) ;
#endif
}

/*  KBFormList								*/
/*  deleteForm	: Delete a form						*/
/*  (returns)	: void		:					*/

void	KBFormList::deleteForm ()
{
#if	! __KB_RUNTIME
	delobj	(m_curItem) ;
#endif
}

#if	! __KB_RUNTIME

/*  KBFormList								*/
/*  saveObjToWeb: Save form for web access				*/
/*  location	: KBLocation &	  : Form location			*/
/*  dir		: const QString & : Target directory			*/
/*  actRC	: ActRC		  : Continutation code			*/
/*  (returns)	: ActRC		  : Continutation code			*/

KBMessageBoxYNAC::ActRC
	KBFormList::saveObjToWeb
	(	const KBLocation	&location,
		const QString		&dir,
		KBMessageBoxYNAC::ActRC	actRC
	)
{
	DPRINTF
	((	"KBFormList::saveObjToWeb: [%s]->[%s]\n",
		(cchar *)location.name(),
		(cchar *)dir
	))	;

	KBError	   error ;
	QByteArray doc   ;
	KBForm	   *form ;

	if (!location.contents (doc, error))
	{
		error.DISPLAY() ;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	if ((form = KBOpenFormText (location, doc, error)) == 0)
	{
		error.DISPLAY() ;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	QWidget	display	;
	RKVBox	layout	(&display) ;
	layout.setTracking () ;

	QSize	size	;
	if (form->showDesign (&layout, size) != KB::ShowRCDesign)
	{
		delete	form	;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	display.resize (size) ;
	display.show() ;

	/* Pump the events so that all the resizes get done, else we	*/
	/* may not see all the controls correctly sized.		*/
	qApp->processEvents() ;

	
	QString	text   ;

	{
		KBErrorBlock eb (KBErrorBlock::Accrue) ;
		form->printNode(text, 0, true) ;
	}

	if (text.isNull())
	{
		TKMessageBox::sorry
		(	0,
			TR("Save to web"),
			TR("Document is null, not saving")
		)	;
		delete	form	;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	QString	name	= dir + "/" + location.name() + ".rkl.frm" ;

	/* Check what to do if the file already exists. This depends	*/
	/* on the action return code passed in.				*/
	if (QFileInfo(name).exists())
		switch (actRC)
		{
			case KBMessageBoxYNAC::ActYes	:
			case KBMessageBoxYNAC::ActNo	:
				/* Already proceeding to the next	*/
				/* object, so repeat the options.	*/
				actRC = KBMessageBoxYNAC::query
			   		(	0,
						QString (TR("%1 already exists: overwrite?")).arg(name),
						TR("Save to file ....")
					)	;
				if (actRC == KBMessageBoxYNAC::ActCancel) return actRC ;
				if (actRC == KBMessageBoxYNAC::ActNo    ) return actRC ;
				break	;

			case KBMessageBoxYNAC::ActSingle:
				/* This is a single file case, so its a	*/
				/* simple yes/no decision.		*/
				if (TKMessageBox::questionYesNo
				   	(	0,
						QString (TR("%1 already exists: overwrite?")).arg(name),
						TR("Save to file ....")
					)
					!= TKMessageBox::Yes)
					return	KBMessageBoxYNAC::ActCancel ;
				break	;

			default	:
				/* Presumably allowing overwrite of all	*/
				/* objects, so just proceed.		*/
				break	;
		}


	KBFile	file	(name) ;
	if (!file.open (IO_WriteOnly|IO_Truncate))
	{
		file.lastError().DISPLAY() ;
		delete	form	;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	QCString data = text.utf8() ;
	file.writeBlock (data.data(), data.length()) ;

	delete	form	;
	return	actRC	;
}
#endif
