/***************************************************************************
    file	         : kb_formtransaction.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"

#include	"kb_formtransaction.h"

#include	"kb_location.h"
#include	"kb_database.h"
#include	"kb_docroot.h"


#define		__DEBUG_FORMTRANSACTION		(1)
#if		__DEBUG_FORMTRANSACTION
#include	"kb_dprintf.h"
#endif


KBFormTransaction::KBFormTransaction
	(	KBDocRoot	*docRoot,
		bool		usetrans
	)
	:
	m_docRoot	(docRoot),
	m_usetrans	(usetrans)
{
	m_cookie = 0	 ;
	m_active = false ;
}

KBFormTransaction::~KBFormTransaction ()
{
	if (m_active) rollback () ;
}

bool	KBFormTransaction::begin ()
{
	const KBLocation &location = m_docRoot->getDataLocation() ;

	DPRINTF
	((	"KBFormTransaction::begin: m_usetrans=%d\n",
		m_usetrans
	))	;

	m_cookie = 0	;
	if (!m_dbLink.connect (location, location.server()))
	{
		m_lError = m_dbLink.lastError() ;
		return	 false ;
	}

	if (m_usetrans)
		if (!m_dbLink.transaction (KBServer::BeginTransaction, &m_cookie))
		{
			m_lError = m_dbLink.lastError() ;
			return	 false ;
		}

	m_active = true  ;
	return	true	 ;
}

bool	KBFormTransaction::rollback ()
{
	DPRINTF
	((	"KBFormTransaction::rollback: m_usetrans=%d\n",
		m_usetrans
	))	;

	m_active = false ;

	if (m_usetrans)
		if (!m_dbLink.transaction (KBServer::RollbackTransaction, &m_cookie))
		{
			m_lError = m_dbLink.lastError() ;
			return	 false ;
		}

	return	true	;
}

const KBError &KBFormTransaction::lastError ()
{
	return	m_lError ;
}
