/***************************************************************************
    file	         : kb_formviewer.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qapplication.h>
#include	<qwidget.h>
#include	<qguardedptr.h>
#include	<qstatusbar.h>
#include	<qpopupmenu.h>

#include	"tk_icons.h"
#include	"tk_messagebox.h"

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_database.h"
#include	"kb_gui.h"
#include	"kb_dbinfo.h"
#include	"kb_dblink.h"
#include	"kb_display.h"
#include	"kb_form.h"
#include	"kb_testsuite.h"
#include	"kb_options.h"
#include	"kb_string.h"
#include	"kb_nodereg.h"
#include	"kb_parse.h"
#include	"kb_docroot.h"
#include	"kb_writer.h"
#include	"kb_viewer.h"
#include	"kb_ascii.h"
#include	"kb_progress.h"
#include	"kb_fixedlabel.h"
#include	"kb_filelist.h"
#include	"kb_file.h"
#include	"kb_errordlg.h"
#include	"kb_pluginaction.h"
#include	"kb_recorder.h"
#include	"kb_macro.h"
#include	"kb_testsavedlg.h"

#if		__KB_AUTOMOC
#include	"kb_formviewer.moc"
#else
#include 	"kb_formviewer.h"
#endif

#include	"kb_formbase.h"
#include	"kb_objtreeviewer.h"

#if		! __KB_RUNTIME
#include	"kb_test.h"
#include	"kb_toolbox.h"
#include	"kb_formtransaction.h"
#endif


#define		__DEBUG_FORMVIEWER	(0)
#if		__DEBUG_FORMVIEWER
#include	"kb_dprintf.h"
#endif



static	QLabel	*menuTitle
	(	QPopupMenu	*menu,
		const QString	&title
	)
{
	static	QFont	*lfont	;
	if (lfont == 0)
	{
		lfont = new QFont (QApplication::font()) ;
		lfont->setWeight  (QFont::Bold) ;
	}

	QLabel	*l = new QLabel   (title, menu) ;
	l->setFont	(*lfont) ;
	l->setLineWidth (2) ;
	l->setFrameStyle(QFrame::Panel|QFrame::Raised) ;
	return	l  ;
}

/*  KBFormViewer							*/
/*  KBFormViewer: Constructor for form viewer object for new form	*/
/*  KBObjBase	: QObject	   : Parent object			*/
/*  embed	: QWidget *	   : Embedding widget			*/
/*  pDict	: QDict<QString> & : Parameter dictionary		*/
/*  modal	: bool		   : Run form modally			*/
/*  (returns)	: KBFormViewer	   :					*/

KBFormViewer::KBFormViewer
	(	KBFormBase		*parent,
		QWidget			*embed,
		const QDict<QString>	&pDict,
		bool			modal
	)
	:
	KBViewer (parent, embed, WStyle_NormalBorder, modal),
	m_pDict	 (pDict)
{
	m_formBase	= parent;
	m_showing	= KB::ShowAsUnknown ;
	m_docRoot	= 0	;

	m_executing	= false	;
	m_running	= false	;
	m_closePending	= false	;
	m_closeRC	= 0	;

	m_objTree	= 0	;
	m_statusBar	= m_partWidget->statusBar () ;

	m_dataGUI	= new KBNavGUI (this, this, "rekallui_form_data.gui"  ) ;
#if	! __KB_RUNTIME
	m_designGUI	= new KBaseGUI (this, this, "rekallui_form_design.gui") ;
	m_recordTrans	= 0	;
#endif

	switch (getLocation().getServerInfo()->showTests())
	{
		case KBServerInfo::TestsSuite  :
		case KBServerInfo::TestsRecord :
			m_testMenu	= new TKActionMenu ("Tests", m_dataGUI, 0) ;
			m_dataGUI->addAction ("KB_tests", m_testMenu) ;

			connect
			(	m_testMenu->popupMenu(),
				SIGNAL	(aboutToShow	()),
				SLOT	(setupTestMenu	())
			)	;

			break	;

		default	:
			m_testMenu	= 0 ;
			break	;
	}
}

/*  KBFormViewer							*/
/*  ~KBFormViewer: Destructor for form viewer				*/
/*  (returns)	 : void		:					*/

KBFormViewer::~KBFormViewer ()
{
	if ((m_showing == KB::ShowAsData) && m_docRoot)
		m_docRoot->finish () ;

#if	! __KB_RUNTIME
	DELOBJ	(m_recordTrans)	;
	DELOBJ	(m_objTree    ) ;
#endif
}

/*  KBFormViewer							*/
/*  setupWidget	: Common top-level setup stuff				*/
/*  size	: const QSize &	: Top-level size			*/
/*  (returns)	: void		:					*/

void	KBFormViewer::setupWidget
	(	const QSize	&size
	)
{
	m_executing = m_showing == KB::ShowAsData ;

	setCaption    		(m_docRoot->getAttrVal("caption" )) ;
	m_partWidget->setIcon	(getSmallIcon ("form")) ;

	QSize	asize	= m_partWidget->resize
			  (	size.width (),
				size.height(),
				!m_executing || !m_docRoot->hideMenuToolBars (),
				 m_executing &&  m_docRoot->hasStatusBar     ()
			  )	;

	DPRINTF
	((	"KBFormViewer::setupWidget: %s %s\n",
		_TEXT( size),
		_TEXT(asize)
	))	;

	if (m_executing)
	{
		bool hmt = m_docRoot->hideMenuToolBars ()   ;
		bool ems = m_docRoot->enforceMinSize   ()   ;
		m_partWidget->showMenuToolBars (!hmt, !hmt) ;
		m_partWidget->setMinimumSize   (ems ? asize : QSize(100,100)) ;
	}
	else
	{
		m_partWidget->showMenuToolBars (true, true) ;
		m_partWidget->setMinimumSize   (QSize(100,100)) ;
	}

	if (m_statusBar != 0)
		if (m_executing && m_docRoot->hasStatusBar ())
			m_statusBar->show() ;
		else	m_statusBar->hide() ;

}

/*  KBFormViewer							*/
/*  slotStartRecording							*/
/*		: Start a test recording				*/
/*  (returns)	: void		:					*/

void	KBFormViewer::slotStartRecording ()
{
#if	! __KB_RUNTIME
	KBRecorder::self()->start
	(	m_docRoot->getDocRoot(),
		getLocation().dbInfo(),
		getLocation().server()
	)	;
#endif
}

/*  KBFormViewer							*/
/*  slotStartRecordingTrans						*/
/*		: Start a test recording in a transaction		*/
/*  (returns)	: void		:					*/

void	KBFormViewer::slotStartRecordingTrans ()
{
#if	! __KB_RUNTIME
	/* As above but the recording will be run in a transaction.	*/
	/* The initial delete should be null but lets be paranoid.	*/
	DELOBJ(m_recordTrans) ;
	m_recordTrans = new KBFormTransaction (m_docRoot->getDocRoot(), true) ;
	m_recordTrans->begin() ;

	KBRecorder::self()->start
	(	m_docRoot->getDocRoot(),
		getLocation().dbInfo(),
		getLocation().server()
	)	;
#endif
}

/*  KBFormViewer							*/
/*  slotSaveRecording							*/
/*		: Save a test recording					*/
/*  (returns)	: void		:					*/

void	KBFormViewer::slotSaveRecording ()
{
#if	! __KB_RUNTIME
	if (KBRecorder::self()->isRecording(m_docRoot->getDocRoot()))
	{
		KBTestSaveDlg tsDlg (m_docRoot) ;
		if (!tsDlg.exec())
			return	;

		/* Might have been running recording in a transaction	*/
		/* so kill it.						*/
		DELOBJ	(m_recordTrans) ;

		KBMacroExec *macro = KBRecorder::self()->recording() ;
		KBTest	    *test  = new KBTest (m_docRoot, tsDlg.name()) ;
		test     ->setMacro   (macro) ;
		test	 ->setComment (tsDlg.comment()) ;

		if (tsDlg.andSave())
			m_objBase->saveDocument () ;

		return	;
	}

	TKMessageBox::sorry
	(	0,
		TR("No recording in progress for this form"),
		TR("Save test recording")
	)	;
#endif
}

/*  KBFormViewer							*/
/*  slotCancelRecording							*/
/*		: Cancel a test recording				*/
/*  (returns)	: void		:					*/

void	KBFormViewer::slotCancelRecording ()
{
#if	! __KB_RUNTIME
	if (KBRecorder::self()->isRecording(m_docRoot->getDocRoot()))
	{
		KBMacroExec *macro = KBRecorder::self()->recording() ;
		if (macro) delete macro ;

		/* Might have been running recording in a transaction	*/
		/* so kill it.						*/
		DELOBJ	(m_recordTrans) ;
	}
#endif
}

/*  KBFormViewer							*/
/*  setupTestMenu: Dynamically set up test menu				*/
/*  (returns)	 : void		:					*/

void	KBFormViewer::setupTestMenu ()
{
#if	! __KB_RUNTIME

	DPRINTF
	((	"KBFormViewer::setupTestMenu: %p: Called .....\n",
		m_testMenu
	))	;

	if (m_testMenu != 0)
	{

		QPopupMenu *menu	= m_testMenu->popupMenu() ;
		bool	   recording	= KBRecorder::self()->isRecording(m_docRoot->getDocRoot()) ;

		menu->clear () ;

		if (getLocation().getServerInfo()->showTests() == KBServerInfo::TestsRecord)
		{
			menu->insertItem(menuTitle (menu, TR("Test recording"))) ;

			int id1 = menu->insertItem
	 			(	TR("Start recording"),
	 				this,
					SLOT(slotStartRecording ())
				)	;
			int id2 = menu->insertItem
	 			(	TR("Start recording in transaction"),
	 				this,
					SLOT(slotStartRecordingTrans ())
				)	;
			int id3 = menu->insertItem
				(	TR("Save recording"),
					this,
					SLOT(slotSaveRecording  ())
				)	;
			int id4 = menu->insertItem
				(	TR("Cancel recording"),
					this,
					SLOT(slotCancelRecording())
				)	;
			menu->setItemEnabled (id1, !recording) ;
			menu->setItemEnabled (id2, !recording) ;
			menu->setItemEnabled (id3,  recording) ;
			menu->setItemEnabled (id4,  recording) ;
		}

		QStringList suiteNames ;
		TITER
		(	TestSuite,
			m_docRoot->getChildren(),
			t,
			suiteNames.append (t->getName())
		)
		if (suiteNames.count() > 0)
		{
			suiteNames.sort() ;
			menu->insertItem(menuTitle (menu, TR("Form test suites"))) ;
			for (uint idx = 0 ; idx < suiteNames.count() ; idx += 1)
				menu->insertItem
				(	suiteNames[idx],
					this,
					SLOT(slotExecuteTestSuite(int))
				)	;
		}

		QStringList testNames ;
		LITER
		(	KBTest,
			m_docRoot->getTests(),
			t,
			testNames.append (t->getName())
		)
		if (testNames.count() > 0)
		{
			testNames.sort() ;
			menu->insertItem(menuTitle (menu, TR("Form level tests"))) ;
			for (uint idx = 0 ; idx < testNames.count() ; idx += 1)
				menu->insertItem
				(	testNames[idx],
					this,
					SLOT(slotExecuteTest(int))
				)	;
		}
	}
#endif
}

/*  KBFormViewer							*/
/*  startup	: Startup form viewer object				*/
/*  docRoot	: KBForm *	: Form object				*/
/*  showAs	: KB::ShowAs	: Initial display mode			*/
/*  key		: const KBValue & : Parent key				*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: KB::ShowRC	: Startup success			*/

KB::ShowRC
	KBFormViewer::startup
	(	KBForm		*docRoot,
		KB::ShowAs	showAs,
		const KBValue	&key,
		KBError		&pError
	)
{
	KB::ShowRC	rc1		;
	KB::ShowRC	rc2		;
	QSize		size		;
	KBaseGUI 	*gui		;

	m_docRoot	= docRoot	;
	m_showing	= showAs	;
	m_parentKey	= key		;

	if (m_statusBar != 0)
	{
		QStringList  labels    ;
		labels.append (TR("Record locked")) ;
		labels.append (TR("Unlocked"	 )) ;

		QLabel	      *message	= new QLabel	    (m_statusBar) ;
		KBFixedLabel  *locking  = new KBFixedLabel  (labels, m_statusBar) ;
		KBProgressBox *progress	= new KBProgressBox (m_statusBar, TR("Record"), QString::null, false) ;
		m_statusBar->addWidget	(message,  1, true) ;
		m_statusBar->addWidget	(locking,  0, true) ;
		m_statusBar->addWidget  (progress, 0, true) ;

		m_docRoot->getDocRoot()->setStatusBar (message, locking, progress) ;
	}

	connect
	(	m_docRoot->getDocRoot(),
		SIGNAL	(requestClose(int)),
		SLOT	(requestClose(int))
	)	;

	{
	KBErrorBlock	eb(KBErrorBlock::AccrueDistinct) ;

	/* Show the GUI in the expected state. This is needed to that	*/
	/* the initial call back to "focusAtRow" works correctly. If	*/
	/* the form does not come up as requested, the GUI will be	*/
	/* reset.							*/

	/* Show the document, either in design or data view. If the	*/
	/* user cancels (which should never happen for design view)	*/
	/* then immediately return a false result.			*/
#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		setGUI		  (gui = m_designGUI) ;
		m_docRoot->setGUI (gui) ;
		rc1	= m_docRoot->showDesign (m_partWidget, size) ;
	}
	else
#endif
	{
		m_dataGUI->setStatusBar (m_docRoot->hasStatusBar()) ;
		setGUI		  (gui = m_dataGUI) ;
		m_docRoot->setGUI (gui) ;
		rc1	= m_docRoot->showData   (m_partWidget, m_pDict, m_parentKey, size) ;
	}


	switch (rc1)
	{
		case KB::ShowRCCancel :
			/* User cancelled, return this at once to the	*/
			/* caller.					*/
			return	KB::ShowRCCancel;

		case KB::ShowRCData   :
			/* Form now shown in data mode. This will be	*/
			/* passed back as being OK.			*/
			break	;

#if	! __KB_RUNTIME
		case KB::ShowRCDesign :
			/* If the user wanted data view then show the	*/
			/* error, but anyways passes back as OK.	*/
			if (m_showing == KB::ShowAsData)
			{	m_docRoot->lastError().DISPLAY() ;
				m_showing = KB::ShowAsDesign ;
			}
			break	;
#endif
		default	:
			/* The only other case should be an error ...	*/
			pError	= m_docRoot->lastError() ;
			return	KB::ShowRCError	;
	}

	/* Note the top-most widget in the form itself and show it,	*/
	/* size the GUI building will have left it hidden.		*/
	m_topWidget  = m_docRoot->getDisplay()->getTopWidget () ;
	m_topWidget ->show () ;


#if	! __KB_RUNTIME
	if (m_showing != showAs)
	{
		KBaseGUI *g = m_showing == KB::ShowAsDesign ? m_designGUI : m_dataGUI ;
		setGUI		  (g) ;
		m_docRoot->setGUI (g) ;
	}
#endif

	/* Connect to the execution error signal. The "executing" flag	*/
	/* is then set so that we will ignore this signal unless we	*/
	/* are executing the form (ie., we are in data view.		*/
	connect
	(	m_docRoot->getDocRoot(),
		SIGNAL	(execError()),
		SLOT	(execError())
	)	;
	connect
	(	m_docRoot,
		SIGNAL	(focusAtRow(bool, uint, uint, bool)),
		SLOT	(focusAtRow(bool, uint, uint, bool))
	)	;
	connect
	(	m_docRoot->getDocRoot(),
		SIGNAL	(layoutChanged()),
		SLOT	(layoutChanged())
	)	;

	setupWidget (size) ;

	m_docRoot->setOpener(m_formBase->opener()) ;
	m_docRoot->isLayout ()->setChanged (false) ;
	}

	/* Finally show the part; it should be shown modally or not	*/
	/* depending on the initial modal setting.			*/
	rc2 = m_partWidget->show  (false, KB::ShowAuto) ;

#if	! __KB_RUNTIME
	if ((rc2 == KB::ShowRCOK) && (m_showing == KB::ShowAsDesign))
		if (KBOptions::getUseToolbox())
			KBToolBox::self()->showToolBox(this, KNF_FORM) ;
#endif

	/* Set the running state and, if running, see if there is a	*/
	/* pending close.						*/
	m_running = (rc1 == KB::ShowRCData) && (m_showing == KB::ShowAsData) ;
	if (m_running && m_closePending) requestClose (m_closeRC) ;

//	m_docRoot->setOpener(m_formBase->opener()) ;
//	m_docRoot->isLayout ()->setChanged (false) ;
	return rc2 ;
}

/*  KBFormViewer							*/
/*  showAs	: Show document in specified mode			*/
/*  _showAs	: KB::ShowAs	: Specified mode			*/
/*  forced	: bool		: Forced switch				*/
/*  (returns)	: void		:					*/

void	KBFormViewer::showAs
	(	KB::ShowAs	_showAs,
		bool		forced
	)
{
	KBErrorBlock	eb(KBErrorBlock::AccrueDistinct) ;

#if	! __KB_RUNTIME
	slotCancelRecording () ;

	if ((_showAs == KB::ShowAsDesign) || (_showAs == KB::ShowAsData))
	{
		cchar		*cmsg	;
		KB::ShowRC	rc	;
		QSize		size	;
		QStringList	changed	;

		if (_showAs == m_showing)
			return ;

		if (!forced)
			if ((cmsg = getChanged (false, changed)) != 0)
			{
				QString	text	;

				text	= QString(TR("<qt>Form %1 changed: switch mode anyway?<br/><ul><li>")).arg(cmsg) ;
				text   += changed.join("</li><li>") ;
				text   += "</li></ul></qt>" ;

				if (TKMessageBox::questionYesNo
					(	0,
						text,
						"Switch mode"
					)
					!= TKMessageBox::Yes
				   )
					return	;
			}

		m_showing = _showAs	;

		/* As for ::startup, show the GUI in the expected	*/
		/* state.						*/
		m_dataGUI->setStatusBar (m_docRoot->hasStatusBar()) ;
		KBaseGUI *g = m_showing == KB::ShowAsDesign ? m_designGUI : m_dataGUI ;
		setGUI		  (g) ;
		m_docRoot->setGUI (g) ;


		/* Clear the execution flags so that any execution	*/
		/* errors that occur as we (possibly) enter data view	*/
		/* are not picked up here. At this stage we reply on	*/
		/* the return codes from showDesign/showData.		*/
		m_executing	= false	;
		m_running	= false	;
		m_closePending	= false	;
		m_closeRC	= 0	;

		if (m_showing == KB::ShowAsDesign)
			rc = m_docRoot->showDesign (m_partWidget, size) ;
		else	rc = m_docRoot->showData   (m_partWidget, m_pDict, m_parentKey, size) ;

		switch (rc)
		{
			case KB::ShowRCCancel :
				/* User cancelled. Must have been a	*/
				/* switch to data mode, so stay in	*/
				/* design.				*/
				m_showing = KB::ShowAsDesign ;
				break	;

			case KB::ShowRCData   :
				/* Form now shown in data mode.		*/
				break	;

			case KB::ShowRCDesign :
				/* If the user wanted data view then	*/
				/* show the error, but otherwise OK.	*/
				if (m_showing == KB::ShowAsData)
				{	m_docRoot->lastError().DISPLAY() ;
					m_showing = KB::ShowAsDesign ;
				}
				break	;

			default	:
				/* The only other case should be an	*/
				/* error, so assume design mode.	*/
				m_docRoot->lastError().DISPLAY() ;
				m_showing = KB::ShowAsDesign ;
				break	;
		}


		m_topWidget  = m_docRoot->getDisplay()->getTopWidget () ;
		m_topWidget ->show () ;

		if (m_showing != _showAs)
		{
			KBaseGUI *g = m_showing == KB::ShowAsDesign ? m_designGUI : m_dataGUI ;
			setGUI		  (g) ;
			m_docRoot->setGUI (g) ;
		}

		setupWidget (size) ;
		m_docRoot->setOpener(m_formBase->opener()) ;

		m_partWidget->show (true, KB::ShowAuto) ;

		if (m_showing == KB::ShowAsDesign)
		{
			if (KBOptions::getUseToolbox())
				KBToolBox::self()->showToolBox(this, KNF_FORM) ;
		}
		else
			KBToolBox::self()->hideToolBox(this) ;

//		m_docRoot->setOpener(m_formBase->opener()) ;

		/* Set the running state and, if running, see if there	*/
		/* is a pending close.					*/
		m_running = (rc == KB::ShowRCData) && (m_showing == KB::ShowAsData) ;
		if (m_running && m_closePending) requestClose (m_closeRC) ;

		return	;
	}
#endif

	if (_showAs == KB::ShowAsPrint)
	{
		KBWriter *writer = new KBWriter (0, getLocation()) ;

		if (!writer->setup
			(	QString::null,
				true,
				KBOptions::getLeftMargin  (),
				KBOptions::getRightMargin (),
				KBOptions::getTopMargin   (),
				KBOptions::getBottomMargin(),
				true
		   )	)
		{
			delete	writer ;
			return	;
		}

		int extra ;
		m_docRoot->write    (writer, QPoint(0,0), false, extra) ;
		writer   ->printDoc (QString::null, -1) ;

		delete	writer ;
		return	;
	}
}

/*  KBFormViewer							*/
/*  showAs	: Show document in specified mode			*/
/*  _showAs	: KB::ShowAs	: Specified mode			*/
/*  (returns)	: void		:					*/

void	KBFormViewer::showAs
	(	KB::ShowAs	_showAs
	)
{
	showAs (_showAs, false) ;
}

/*  KBFormViewer							*/
/*  saveDocumentAs: Save document under specified name			*/
/*  (returns)	  : void	:					*/

void	KBFormViewer::saveDocumentAs ()
{
#if	! __KB_RUNTIME
	/* This is only meaningful in design mode ....			*/
	if (m_showing == KB::ShowAsDesign)
		if (m_objBase->saveDocumentAs ())
		{	m_docRoot->isLayout()->setChanged (false) ;
			setCaption (m_docRoot->getAttrVal("caption")) ;
		}
#endif
}

/*  KBFormViewer							*/
/*  saveDocument: Save document						*/
/*  (returns)	: void		:					*/

void	KBFormViewer::saveDocument ()
{
#if	! __KB_RUNTIME
	/* If we are in design mode then this is interpreted as saving	*/
	/* the form design.						*/
	if (m_showing == KB::ShowAsDesign)
	{
		if (m_objBase->saveDocument ())
		{	m_docRoot->isLayout ()->setChanged  (false) ;
			setCaption (m_docRoot->getAttrVal("caption")) ;
		}

		return	;
	}
#endif
	/* Otherwise, save the values displayed in the form ...		*/
	m_docRoot->formAction (KB::Save) ;
}


/*  KBFormViewer							*/
/*  queryClose	: Query whether to close				*/
/*  (returns)	: bool		: True to allow close			*/

bool	KBFormViewer::queryClose ()
{
	cchar		*cmsg	;
	QStringList	changed	;

	if ((cmsg = getChanged (true, changed)) != 0)
	{
		QString	text	;

		text	= QString(TR("<qt>Form %1 changed: close anyway?<br/><ul><li>")).arg(cmsg) ;
		text   += changed.join("</li><li>") ;
		text   += "</li></ul></qt>" ;

		if (TKMessageBox::questionYesNo
			(	0,
				text,
				"Close form"
			)
			!= TKMessageBox::Yes
		   )
			return	false	;
	}

	if (m_showing == KB::ShowAsData)
		if (!m_docRoot->queryClose ())
			return	false	;

	return	true	;
}


/*  KBFormViewer							*/
/*  doCtrlAlign	: Handle control alignment request			*/
/*  align	: int		: Aligment operation			*/
/*  (returns)	: void		:					*/

void	KBFormViewer::doCtrlAlign
	(	int	align
	)
{
#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
		m_docRoot->doCtrlAlign ((KB::CtrlAlign)align) ;
#endif
}

void	KBFormViewer::toggleToolBox ()
{
#if	! __KB_RUNTIME
	KBToolBox::self()->toggleToolBox (this, KNF_FORM) ;
#endif
}

/*  KBFormViewer							*/
/*  showObjTree	: Toggle the object tree for the form			*/
/*  (returns)	: void		:					*/

void	KBFormViewer::showObjTree ()
{
#if	! __KB_RUNTIME
	if (m_objTree == 0)
	{
		m_objTree = new KBObjTreeViewer
			  	(	0,//m_objBase,
					m_embed,
					getLocation(),
					m_docRoot,
					m_docRoot
				)	;

		connect
		(	m_objTree, SIGNAL(destroyed	   ()),
			this,	   SLOT  (objTreeViewerDead())
		)	;

		m_designGUI->setChecked ("KB_showObjTree", true) ;
		m_dataGUI  ->setChecked ("KB_showObjTree", true) ;
	}
	else
	{	DELOBJ	 (m_objTree) ;
		objTreeViewerDead () ;
	}
#endif
}

/*  KBFormViewer							*/
/*  doMultiProp	: Invoke multiple-object properties dialog		*/
/*  (returns)	: void		:					*/

void	KBFormViewer::doMultiProp ()
{
#if	! __KB_RUNTIME
	if (m_docRoot) m_docRoot->KBLayout::doMultiProp () ;
#endif
}

/*  KBFormViewer							*/
/*  doProperties: Invoke object properties dialog			*/
/*  (returns)	: void		:					*/

void	KBFormViewer::doProperties ()
{
#if	! __KB_RUNTIME
	if (m_docRoot) m_docRoot->KBLayout::doSingleProp () ;
#endif
}


/*  KBFormViewer							*/
/*  objTreeViewerDead							*/
/*		: Note death of object tree display			*/
/*  (returns)	: void		:					*/

void	KBFormViewer::objTreeViewerDead ()
{
#if	! __KB_RUNTIME
	m_objTree	= 0 ;
	m_designGUI->setChecked ("KB_showObjTree", false) ;
	m_dataGUI  ->setChecked ("KB_showObjTree", false) ;
#endif
}

/*  KBFormViewer							*/
/*  dbaseAction	: Handle database action request			*/
/*  action	: int		: Database action			*/
/*  (returns)	: void		:					*/

void	KBFormViewer::dbaseAction
	(	int	action
	)
{
#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsData)
#endif
		if (!m_docRoot->formAction ((KB::Action)action))
			m_docRoot->lastError().DISPLAY() ;
}

/*  KBFormViewer							*/
/*  reload	: Reload table data					*/
/*  (returns)	: void		:					*/

void	KBFormViewer::reload ()
{
#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsData)
#endif
		if (!m_docRoot->requery ())
			m_docRoot->lastError().DISPLAY() ;
}

/*  KBFormViewer							*/
/*  doCut	: Handle cut request					*/
/*  (returns)	: void		:					*/

void	KBFormViewer::doCut ()
{
#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign) m_docRoot->doCut () ;
#endif
}

/*  KBFormViewer							*/
/*  doCopy	: Handle copy request					*/
/*  (returns)	: void		:					*/

void	KBFormViewer::doCopy ()
{
#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign) m_docRoot->doCopy () ;
#endif
}

/*  KBFormViewer							*/
/*  doSaveComponent							*/
/*		: Handle request to save objects as component		*/
/*  (returns)	: void		:					*/

void	KBFormViewer::doSaveComponent ()
{
#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
		m_docRoot->doSaveComponent (getLocation()) ;
#endif
}

/*  KBFormViewer							*/
/*  doPaste	: Handle patse request					*/
/*  (returns)	: void		:					*/

void	KBFormViewer::doPaste ()
{
#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign) m_docRoot->doPaste () ;
#endif
}

/*  KBFormViewer							*/
/*  getChanged	: Report whether there are unsaved changes		*/
/*  both	: bool		: True for both data and design change	*/
/*  changed	: QStringList & : Changed objects			*/
/*  (returns)	: cchar *	: Changed message or null if none	*/

cchar	*KBFormViewer::getChanged
	(	bool		both,
		QStringList	&changed
	)
{
	return	m_docRoot->getChanged (both, changed) ;
}

/*  KBFormViewer							*/
/*  snapToGrid	: Snap selected controls to the grid			*/
/*  (returns)	: void		:					*/

void	KBFormViewer::snapToGrid ()
{
#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign) m_docRoot->snapToGrid () ;
#endif
}

/*  KBFormViewer							*/
/*  execError	: Handle script execution error				*/
/*  (returns)	: void		:					*/

void	KBFormViewer::execError ()
{
#if	! __KB_RUNTIME
	/* The form emits a signal if there is an execution-time error,	*/
	/* typically a script error. If the form is being executed	*/
	/* (ie., we are not inside ::startup nor ::showAs) then flip	*/
	/* to design mode.						*/
	if (m_executing) showAs (KB::ShowAsDesign, true) ;
#endif
}

/*  KBFormViewer							*/
/*  requestClose: Handler slot for close requests from the form		*/
/*  rc		: int		: Exit code				*/
/*  (returns)	: void		:					*/

void	KBFormViewer::requestClose
	(	int	rc
	)
{
	DPRINTF
	((	"KBFormViewer::requestClose: rc=%d running=%d\n",
		rc,
		m_running
	))	;

	if (m_partWidget->isModal()) m_partWidget->exitModal() ;

	/* If the form is not yet in the "running" state, ie., if the	*/
	/* form startup has not completed) the flag the request. It	*/
	/* cannot be safely closed at this stage.			*/
	if (!m_running)
	{
		m_closePending	= true	;
		m_closeRC	= rc	;
		return		;
	}

	/* Form is running. We queue an event so that the request is	*/
	/* handled outside if the form code itself (ie., we are not	*/
	/* somewhere down a stack).					*/
	m_objBase->setRC (rc) ;
	qApp->postEvent  (this, new QEvent(QEvent::User)) ;
}

/*  KBFormViewer							*/
/*  event	: Local event filter					*/
/*  e		: QEvent *	: Event to handle			*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBFormViewer::event
	(	QEvent		*e
	)
{
	/* The special case is the user event, used to signal a close	*/
	/* request.							*/
	if (e->type() == QEvent::User)
	{
		slotClose () ;
		return	true ;
	}

	return	false	;
}


/*  KBFormViewer							*/
/*  setFocusAtRow: Set focus row information for current block		*/
/*  hasData	 : bool		: Block has data			*/
/*  curRow	 : uint		: Current row number			*/
/*  numRows	 : uint		: Number of rows			*/
/*  inQuery	 : bool		: In query				*/
/*  (returns)	 : void		:					*/

void	KBFormViewer::focusAtRow
	(	bool		hasData,
		uint		curRow,
		uint		numRows,
		bool		inQuery
	)
{
	if (m_showing == KB::ShowAsData)
		m_dataGUI->setFocusAtRow (hasData, curRow, numRows, inQuery) ;
}

void	KBFormViewer::layoutChanged ()
{
}

void	KBFormViewer::saveToWeb ()
{
	saveToWeb (m_docRoot, KBMessageBoxYNAC::ActSingle) ;
}

/*  KBFormViewer							*/
/*  slotExecuteTestSuite						*/
/*		: User requests test suite				*/
/*  id		: int		: Identifier from popup menu		*/
/*  (returns)	: void		:					*/

void	KBFormViewer::slotExecuteTestSuite
	(	int		id
	)
{
#if	! __KB_RUNTIME
	if (m_testMenu == 0)
		return	;

	QString		name	= m_testMenu->popupMenu()->text(id) ;
	KBTestSuite	*suite	= 0 ;

	TITER
	(	TestSuite,
		m_docRoot->getChildren(),
		t,

		if (t->getName() == name)
		{	suite	= t ;
			break	;
		}
	)

	if (suite == 0)
	{
		TKMessageBox::sorry
		(	0,
			QString(TR("Internal error: test '%1' not found")).arg(name),
			TR("Execute test")
		)	;
		return	;
	}

	KBFormTransaction ft (m_docRoot->getDocRoot(), suite->transaction()) ;

	ft.begin       () ;
	suite->execute (KBTest::ErrorAbortTest, true) ;
	ft.rollback    () ;
#endif
}

/*  KBFormViewer							*/
/*  slotExecuteTest							*/
/*		: User requests test					*/
/*  id		: int		: Identifier from popup menu		*/
/*  (returns)	: void		:					*/

void	KBFormViewer::slotExecuteTest
	(	int		id
	)
{
#if	! __KB_RUNTIME
	if (m_testMenu != 0)
	{
		QString	name	= m_testMenu->popupMenu()->text(id) ;

		LITER
		(	KBTest,
			m_docRoot->getTests(),
			t,

			if (t->getName() == name)
			{
				bool evRc ;
				m_docRoot->KBObject::eventHook (*t, 0, 0, evRc) ;
				return	  ;
			}
		)

		TKMessageBox::sorry
		(	0,
			QString(TR("Internal error: test '%1' not found")).arg(name),
			TR("Execute test")
		)	;
	}
#endif
}

KBMessageBoxYNAC::ActRC
	KBFormViewer::saveToWeb
	(	KBForm			*form,
		KBMessageBoxYNAC::ActRC	actRC
	)
{
	QString		 text	 ;
	QStringList	 queries ;

	const KBLocation &locn	 = form->getDocRoot()->getDocLocation() ;
	QString		 dir	 = KBFileList::getWebDir (locn)		;
	QString		 name	 = dir + "/" + locn.name() + ".rkl.frm" ;

	/* Wrap with the error blocker to pick up warnings. If there	*/
	/* were any then force a query even if the user has earlier	*/
	/* answered Yes-To-All.						*/
	{
		KBErrorBlock eb (KBErrorBlock::Accrue) ;
		form->printNode(text, 0, true) ;

		if (eb.errorCount() > 0)
		{	queries.append (QString (TR("%1 has possible errors: save?")).arg(locn.name())) ;
			actRC = KBMessageBoxYNAC::ActNo ;
		}
	}

	/* Paranoia check that there is a document! Very unlikely so we	*/
	/* refuse to save. Return KBMessageBoxYNAC::ActNo so that the	*/
	/* used will be queried even having earlier answered Yes-To-All	*/
	if (text.isNull())
	{
		TKMessageBox::sorry
		(	0,
			TR("Document is null, not saving"),
			TR("Save to web")
		)	;
		delete	form	;
		return	KBMessageBoxYNAC::ActNo ;
	}


	/* If the destination file exists then add to the list of	*/
	/* queries. Will not get shown if the user earlier answered	*/
	/* Yes-To-All (and no errors above).				*/
	if (QFileInfo(name).exists())
		queries.append (QString (TR("%1 already exists: overwrite?")).arg(name)) ;



	/* Check what to do if there were any errors. This depends on	*/
	/* the action return code passed in.				*/
	if (queries.count() > 0)
	{
		QString	qtxt	= queries.join("\n") ;

		switch (actRC)
		{
			case KBMessageBoxYNAC::ActYes	:
			case KBMessageBoxYNAC::ActNo	:
				/* Already proceeding to the next	*/
				/* object, so repeat the options.	*/
				actRC = KBMessageBoxYNAC::query
			   		(	0,
						qtxt,
						TR("Save to file ....")
					)	;
				if (actRC == KBMessageBoxYNAC::ActCancel) return actRC ;
				if (actRC == KBMessageBoxYNAC::ActNo    ) return actRC ;
				break	;

			case KBMessageBoxYNAC::ActSingle:
				/* This is a single file case, so its a	*/
				/* simple yes/no decision.		*/
				if (TKMessageBox::questionYesNo
				   	(	0,
						qtxt,
						TR("Save to file ....")
					)
					!= TKMessageBox::Yes)
					return	KBMessageBoxYNAC::ActCancel ;
				break	;

			default	:
				/* Presumably allowing overwrite of all	*/
				/* objects, so just proceed.		*/
				break	;
		}
	}

	KBFile	file	(name) ;
	if (!file.open (IO_WriteOnly|IO_Truncate))
	{
		file.lastError().DISPLAY() ;
		return	KBMessageBoxYNAC::ActCancel ;
	}

	QCString data = text.utf8() ;
	file.writeBlock (data.data(), data.length()) ;

	return	actRC	;
}


void	KBFormViewer::getPluginActionInfo
	(	KBPluginActionInfo	&info
	)
{
	KBViewer::getPluginActionInfo (info) ;
	info.m_form = m_formBase->getTopNode()->isForm() ;
}

