/***************************************************************************
    file	         : kb_formviewer.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef  _KB_FORMVIEWER_H
#define  _KB_FORMVIEWER_H

#include	"kb_classes.h"
#include	"kb_attrdict.h"
#include	"kb_viewer.h"
#include	"kb_navgui.h"
#include	"kb_docroot.h"
#include	"kb_progress.h"
#include	"kb_prompt.h"


class	KBFormBase		;
class	KBObjTreeViewer		;
class	QStatusBar		;
class	KBFormTransaction	;

/*  KBFormViewer							*/
/*  ------------							*/
/*  This class is the MDI child wrapper for form viewing, in either	*/
/*  data or design mode.						*/

class	KBFormViewer : public KBViewer
{
	Q_OBJECT

	KBFormBase		*m_formBase	;
	KBAttrDict		m_pDict		;

	QGuardedPtr<KBForm  >	m_docRoot 	;
	KB::ShowAs		m_showing	;

	/* TRICKY CODE ALERT!						*/
	/* These member variables are used to ensure sane behaviour if	*/
	/* a script closes the form during form startup (eg., in the	*/
	/* form's "onopened" event). See comments in the ::requestClose	*/
	/* slot for details.						*/
	bool			m_executing	;
	bool			m_running	;
	bool			m_closePending	;
	int			m_closeRC	;

	KBNavGUI		*m_dataGUI	;
	TKActionMenu		*m_testMenu	;
	KBObjTreeViewer		*m_objTree	;
	KBValue			m_parentKey	;

	QStatusBar		*m_statusBar	;

#if	! __KB_RUNTIME
	KBaseGUI		*m_designGUI	;
	KBFormTransaction	*m_recordTrans	;
#endif

	void			setupWidget		(const QSize &)	;

protected :

	virtual	void		getPluginActionInfo	(KBPluginActionInfo &) ;

protected slots	:

	void			execError		()	;
	void			requestClose		(int)	;
	void			objTreeViewerDead	()	;
	void			focusAtRow		(bool, uint, uint, bool) ;
	void			layoutChanged		()	;
	void			saveToWeb		()	;
	void			setupTestMenu		()	;
	void			slotStartRecording	()	;
	void			slotStartRecordingTrans	()	;
	void			slotSaveRecording	()	;
	void			slotCancelRecording	()	;
	void			slotExecuteTestSuite	(int)	;
	void			slotExecuteTest		(int)	;

public	:

	KBFormViewer (KBFormBase *, QWidget *, const QDict<QString> &, bool = false) ;
virtual~KBFormViewer () ;

	virtual	KBFormViewer	*isFormViewer	() { return this  ; }

	virtual	cchar		*getChanged	(bool, QStringList &) ;
	virtual	bool		queryClose	()	;
	virtual	bool		event		(QEvent *) ;

	KB::ShowRC		startup		(KBForm *, KB::ShowAs, const KBValue &, KBError &) ;

	void			showAs		(KB::ShowAs, bool) ;

public	slots	:

	virtual	void	showAs		(KB::ShowAs)	;

	void		saveDocument	()     		;
	void		saveDocumentAs	()     		;

	void		dbaseAction	(int)		;
	void		reload	        ()		;
	void		doCtrlAlign	(int)		;
	void		doCut		()		;
	void		doCopy		()		;
	void		doSaveComponent	()		;
	void		doPaste		()		;
	void		snapToGrid	()		;

	void		toggleToolBox	()		;
	void		showObjTree	()		;
	void		doMultiProp	()		;
	void		doProperties	()		;

	static	KBMessageBoxYNAC::ActRC
			saveToWeb	(KBForm *, KBMessageBoxYNAC::ActRC) ;
}	;

extern	KBForm	*KBOpenForm  (KBDBInfo *, const QString &, QDict<QString> *, KBError &) ;

#endif // _KB_FORMVIEWER_H
