/***************************************************************************
    file	         : kb_testalldlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_sidepanel.h"
#include	"kb_dbinfo.h"
#include	"kb_serverinfo.h"

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_listview.h"
#include	"rk_combobox.h"
#include	"rk_checkbox.h"


#if		__KB_AUTOMOC
#include	"kb_testalldlg.moc"
#else
#include	"kb_testalldlg.h"
#endif


/*  KBTestAllDlg							*/
/*  KBTestAllDlg							*/
/*		: Constructor for test-all prompt dialog		*/
/*  (returns)	: KBPromptDlg	:					*/

KBTestAllDlg::KBTestAllDlg
	(	KBDBInfo	*dbInfo,
		const QString	&server
	)
	:
	KBDialog	(TR("Execute Tests"), "testalldlg")
{
	RKVBox	  *layMain	= new RKVBox (this) ;
	layMain->setTracking () ;

	RKHBox	  *layDlg	= new RKHBox (layMain) ;
	new KBSidePanel (layDlg, caption()) ;

	RKVBox	  *layPick	= new RKVBox (layDlg ) ;

	m_suites  = new RKListView (layPick) ;
	m_last	  = 0	;

	m_suites->addColumn	     (TR("Form/Suite"),   -1) ;
	m_suites->setResizeMode	     (RKListView::LastColumn) ;
	m_suites->setRootIsDecorated (true) ;
	m_suites->setMinimumWidth    (550 ) ;

	RKGridBox  *layServer	= new RKGridBox (2,  layPick) ;
	new QLabel(TR("Test against"),   layServer) ;
	m_server   = new RKComboBox     (layServer) ;
	new QLabel(TR("Error handling"), layServer) ;
	m_errorOpt = new RKComboBox     (layServer) ;

	QPtrListIterator<KBServerInfo> *svIter = dbInfo->getServerIter() ;
	KBServerInfo		       *svInfo ;
	while ((svInfo = svIter->current()) != 0)
	{	m_server->insertItem (svInfo->serverName()) ;
		(*svIter) += 1 ;
	}
	m_server->setCurrentByText  (server) ;
	delete	svIter	;

	/* Note that the values set here depend on the valies of the	*/
	/* KBTest::ErrorOpt enum corresponding to the item indexes	*/
	m_errorOpt->insertItem	(TR("Abort all"        )) ;
	m_errorOpt->insertItem	(TR("Abort suite"      )) ;
	m_errorOpt->insertItem	(TR("Abort test"       )) ;
	m_errorOpt->insertItem	(TR("Prompt on error"  )) ;
	m_errorOpt->insertItem	(TR("Prompt for update")) ;

	addOKCancel	(layMain) ;
}

/*  KBTestAllDlg							*/
/*  ~KBTestAllDlg: Destructor for test all dialog			*/
/*  (returns)	 : void		:					*/

KBTestAllDlg::~KBTestAllDlg ()
{
}

void	KBTestAllDlg::addSuites
	(	const QString		&form,
		const QStringPairList	&suites
	)
{
	m_last	= new QCheckListItem
			(	m_suites,
				form, 	
				QCheckListItem::CheckBoxController
			)	;
	m_last->setOn	(true)	;

	for (uint idx = 0 ; idx < suites.count() ; idx += 1)
	{
		QCheckListItem *s = new QCheckListItem
					(	m_last,
						suites[idx].m_first,
						QCheckListItem::CheckBoxController
					)	;
		s->setOn (true) ;
	}
}

QStringPairList KBTestAllDlg::selected ()
{
	QStringPairList	seln	;

	for
	(	QListViewItem *form = m_suites->firstChild() ;
		form != 0 ;
		form  = form->nextSibling()
	)
	{
		for
		(	QListViewItem *suite = form->firstChild() ;
			suite != 0 ;
			suite  = suite->nextSibling()
		)
		{
			XPRINTF
			((	"KBTestAllDlg::selected: [%s][%s]: %d/%d\n",
				form ->text (0).latin1(),
				suite->text (0).latin1(),
				((QCheckListItem *)suite)->isOn (),
				((QCheckListItem *)suite)->state()
			))	;

			if (!((QCheckListItem *)suite)->state() != QCheckListItem::Off)
				continue ;

			QStringPair  sp  ;
			sp.m_first  = form ->text(0) ;
			sp.m_second = suite->text(0) ;
			seln.append (sp) ;   

		}
	}

	return	seln	;
}

QString	KBTestAllDlg::server ()
{
	return	m_server->currentText () ;
}

KBTest::ErrorOpt
	KBTestAllDlg::errorOpt ()
{
	return	(KBTest::ErrorOpt)m_errorOpt->currentItem () ;
}
