/***************************************************************************
    file	         : kb_testsavedlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_node.h"
#include	"kb_sidepanel.h"
#include	"kb_test.h"

#include	<qlabel.h>

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_checkbox.h"
#include	"rk_lineedit.h"
#include	"rk_textedit.h"
#include	"rk_textbrowser.h"

#if		__KB_AUTOMOC
#include	"kb_testsavedlg.moc"
#else
#include	"kb_testsavedlg.h"
#endif


static	char	*message =
		__TR
		(	"<p>"
			"Enter a name for the test recording. If the "
			"<i>save form</i> option is checked, the test "
			"will be saved; if not, you will need to switch "
			"to design view and save the form definition."
			"</p>"
		)	;
static	char	*attr_exists =
		__TR
		(	"<p><b>Attribute exists with this name.</b></p>"
		)	;
static	char	*test_exists =
		__TR
		(	"<p><b>Test exists with this name.</b></p>"
		)	;


/*  KBTestSaveDlg							*/
/*  KBTestSaveDlg							*/
/*		: Constructor for test-save prompt dialog		*/
/*  node	: KBNode *	: Node into which test will be saved	*/
/*  (returns)	: KBPromptDlg	:					*/

KBTestSaveDlg::KBTestSaveDlg
	(	KBNode		*node
	)
	:
	KBDialog	(TR("Save Test Recording"), true),
	m_node		(node)
{
	RKVBox	  *layMain	= new RKVBox (this) ;
	layMain->setTracking () ;

	RKHBox	  *layDlg	= new RKHBox (layMain) ;
	new KBSidePanel (layDlg, caption()) ;

	m_message	= new RKTextBrowser (layDlg) ;
	m_message->setText	   (TR(message)) ;
	m_message->setMaximumWidth (250) ;

	RKVBox	  *layData	= new RKVBox (layDlg) ;
	RKGridBox *layGrid	= new RKGridBox (2, layData) ;

	new QLabel 	(TR("Test name"),   layGrid) ;
	m_name		= new RKLineEdit   (layGrid) ;
	new QWidget	(layGrid) ;
	m_andSave	= new RKCheckBox   (layGrid) ;
	m_andSave->setText (TR("Save test to form")) ;

	new QLabel 	(TR("Comment:"),    layData) ;
	m_comment	= new RKTextEdit   (layData) ;

	layData->setStretchFactor (m_comment, 1) ;

	addOKCancel	(layMain) ;
	setMinimumWidth	(500)	  ;
}

/*  KBTestSaveDlg							*/
/*  ~KBTestSaveDlg: Destructor for prompt dialog			*/
/*  (returns)	    : void		:				*/

KBTestSaveDlg::~KBTestSaveDlg ()
{
}

void	KBTestSaveDlg::accept ()
{
	if (name().isEmpty())
		return	;

	if (m_node->getAttr(name()) != 0)
	{
		m_message->setText (TR(message) + TR(attr_exists)) ;
		return	;
	}

	LITER
	(	KBTest,
		m_node->getTests(),
		t,
		if (t->getName() == name())
		{
			m_message->setText (TR(message) + TR(test_exists)) ;
			return	;
		}
	)

	done (true) ;
}

QString	KBTestSaveDlg::name ()
{
	return	m_name->text()	;
}

bool	KBTestSaveDlg::andSave ()
{
	return	m_andSave->isChecked() ;
}

QString	KBTestSaveDlg::comment ()
{
	return	m_comment->text() ;
}
