/***************************************************************************
    file	         : kb_qryjoindlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_sidepanel.h"
#include	"kb_manual.h"

#if		__KB_AUTOMOC
#include	"kb_qryjoindlg.moc"
#else
#include 	"kb_qryjoindlg.h"
#endif

#include	"kb_tablealias.h"

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_gridbox.h"
#include	"rk_lineedit.h"
#include	"rk_textedit.h"
#include	"rk_combobox.h"
#include	"rk_pushbutton.h"
#include	"rk_tabwidget.h"
#include	"tk_messagebox.h"

#include	<qlabel.h>

#define		__DEBUG_QRYJOINDLG	(0)
#if		__DEBUG_QRYJOINDLG
#include	"kb_dprintf.h"
#endif


/*  KBQryJoinDlg							*/
/*  KBQryJoinDlg  : Constructor for query join dialog			*/
/*  tblChild 	  : KBTableAlias  *	: Child table			*/
/*  fldChildName  : const QString &	: Child field name		*/
/*  tblParent	  : KBTableAlias  *	: Parent table			*/
/*  fldParentName : const QString &	: Parent field name		*/
/*  joinType	  : const QString &	: Join type			*/
/*  joinExpr	  : const QString &	: Arbitrary join expression	*/
/*  useExpr	  : bool		: Using expression		*/
/*  canDelete	  : bool	   	: Deletion enabled		*/
/*  (returns)	  : KBQryJoinDlg	:				*/

KBQryJoinDlg::KBQryJoinDlg
	(	KBTableAlias	*tblChild,
		const QString	&fldChildName,
		KBTableAlias	*tblParent,
		const QString	&fldParentName,
		const QString	&joinType,
		const QString	&joinExpr,
		bool		useExpr,
		bool		canDelete
	)
	:
	KBDialog	("Join properties", true),
	m_tblChild	(tblChild),
	m_tblParent	(tblParent)
{
	RKVBox	    *layMain	= new RKVBox	  (this) ;
	layMain->setTracking () ;

	RKHBox	    *layTop	= new RKHBox	  (layMain)   ;
	new KBSidePanel (layTop, TR("Table join conditions")) ;

	m_tabber		= new RKTabWidget (layTop )     ;
	m_link			= new RKGridBox   (2, m_tabber) ;

				  new QLabel	  (TR("Many ..."),       m_link) ;
				  new QLabel      (TR("... to one"),     m_link) ;
	RKLineEdit *leTblParent	= new RKLineEdit  (tblParent->caption(), m_link) ;
	RKLineEdit *leTblChild	= new RKLineEdit  (tblChild ->caption(), m_link) ;
	RKLineEdit *leFldParent	= new RKLineEdit  (fldParentName,	 m_link) ;
	RKLineEdit *leFldChild	= new RKLineEdit  (fldChildName, 	 m_link) ;

	leTblChild   ->setReadOnly(true) ;
	leTblParent  ->setReadOnly(true) ;
	leFldChild   ->setReadOnly(true) ;
	leFldParent  ->setReadOnly(true) ;


	m_link->addFillerRow ()	;

	m_joinExpr		= new RKTextEdit (m_tabber) ;
	m_joinExpr->setText (joinExpr) ;

	m_tabber->addTab	(m_link,     TR("Many-to-one")) ;
	m_tabber->addTab	(m_joinExpr, TR("Join expression")) ;


	RKHBox	   *layJoin	= new RKHBox    (layMain) ;
	new QLabel (TR("Join type"  ), layJoin) ;
	m_joinType		= new RKComboBox(layJoin) ;

	m_joinType ->insertItem	(TR("Inner"      )) ;
	m_joinType ->insertItem	(TR("Left Outer" )) ;
	m_joinType ->insertItem	(TR("Right Outer")) ;

	RKHBox	  *layButt	= new RKHBox    (layMain) ;

	new KBManualPushButton	(layButt, "Chap6Joins") ;
	layButt    ->addFiller 	() ;
	new RKPushButton 	(layButt, "ok") ;
	new RKPushButton 	(layButt, "cancel") ;

	if (canDelete)
	{
		RKPushButton *bDelete	= new RKPushButton (TR("Delete"), layButt) ;
		connect	(bDelete, SIGNAL(clicked()), SLOT(clickDelete())) ;
	}

	if	(joinType == "left" ) m_joinType->setCurrentItem (1) ;
	else if	(joinType == "right") m_joinType->setCurrentItem (2) ;
	else			      m_joinType->setCurrentItem (0) ;

	m_oldPalette = tblChild->palette()   ;

	QPalette pal = m_oldPalette ;
	pal.setColor (QColorGroup::Base,       QColor(192,192,192)) ;
	pal.setColor (QColorGroup::Background, QColor(192,192,192)) ;
 
	leTblChild ->setPalette  (pal) ;
	leTblParent->setPalette  (pal) ;
	leFldChild ->setPalette  (pal) ;
	leFldParent->setPalette  (pal) ;
	m_tblChild ->setPalette  (pal) ;
	m_tblParent->setPalette  (pal) ;

	m_doDel	 = false ;

	if (useExpr)
		m_tabber->showPage (m_joinExpr) ;
	else	m_tabber->showPage (m_link    ) ;
}

void	KBQryJoinDlg::done
	(	int		rc
	)
{
	m_tblChild ->setPalette  (m_oldPalette) ;
	m_tblParent->setPalette  (m_oldPalette) ;
	KBDialog::done (rc) ;
}

/*  KBQryJoinDlg							*/
/*  accept	: User accepts						*/
/*  (returns)	: void		:					*/

void	KBQryJoinDlg::accept ()
{
	/* If the user is accepting an arbitrary join expression then	*/
	/* it must not be empty.					*/
	if (m_tabber->currentPage() == m_joinExpr)
		if (m_joinExpr->text().isEmpty())
		{
			TKMessageBox::sorry
			(	0,
				TR("Please specify a join expression")
			)	;
			return	;
		}

	KBDialog::accept () ;
}

void	KBQryJoinDlg::clickDelete ()
{
	m_doDel	= true	;
	done	(1)	;
}

bool	KBQryJoinDlg::getResults
	(	QString		&joinType,
		QString		&joinExpr,
		bool		&useExpr
	)
{
	switch (m_joinType->currentItem())
	{
		case 1  : joinType = "left"  ; break ;
		case 2  : joinType = "right" ; break ;
		default : joinType = ""	  ; break ;
	}

	joinExpr = m_joinExpr->text() ;
	useExpr	 = m_tabber->currentPage() == m_joinExpr ;
	return	 m_doDel	;
}
