/***************************************************************************
    file	         : kb_querydlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include 	<qpainter.h>
#include 	<qcursor.h>
#include 	<qtooltip.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_gui.h"

#include	"kb_dbinfo.h"
#include	"kb_dblink.h"
#include	"kb_node.h"
#include	"kb_query.h"
#include	"kb_table.h"
#include	"kb_qryexpr.h"
#include	"kb_prompt.h"
#include	"kb_ascii.h"

#if		__KB_AUTOMOC
#include	"kb_querydlg.moc"
#else
#include 	"kb_querydlg.h"
#endif

#include	"kb_primarydlg.h"
#include	"kb_qryjoindlg.h"
#include	"kb_select.h"

#include	"tk_messagebox.h"

#define		__DEBUG_QUERYDLG	(0)
#if		__DEBUG_QUERYDLG
#include	"kb_dprintf.h"
#endif


static	cchar	*exprUsage[] =
{	"",
	"Sort Ascending",
	"Sort Descending",
	"Where",
	"Group By",
	"Having",
	0
}	;


static	int	usageToCode
	(	const QString	&usage
	)
{
	for (uint idx = 0 ; exprUsage[idx] != 0 ; idx += 1)
		if (exprUsage[idx] == usage) return idx ;

	return	0 ;
}


/*  ------------------------------------------------------------------  */

class	KBQueryDlgTip : public QToolTip
{
	KBQueryDlg	*m_queryDlg	;

public	:

	KBQueryDlgTip			(KBQueryDlg *, QWidget *) ;
	virtual	~KBQueryDlgTip		()		 ;

	virtual	void	maybeTip	(const QPoint &) ;
}	;

KBQueryDlgTip::KBQueryDlgTip
	(	KBQueryDlg	*queryDlg,
		QWidget		*monitor
	)
	:
	QToolTip	(monitor),
	m_queryDlg	(queryDlg)
{
}

KBQueryDlgTip::~KBQueryDlgTip ()
{
	remove	(parentWidget()) ;
}

void	KBQueryDlgTip::maybeTip
	(	const QPoint 	&point
	)
{
	QRect	rect	;
	QString	text	= m_queryDlg->tip (point, rect) ;

	if (!text.isEmpty())
		tip
		(	rect,
			QString("<qt>%1</qt>").arg(text)
		)	;
}

/*  ------------------------------------------------------------------  */

/*  KBQryExprs								*/
/*  KBQryExprs	: Constructor for query expression list			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  (returns)	: KBQryExprs	:					*/

KBQueryExprs::KBQueryExprs
	(	QWidget	*parent
	)
	:
	KBEditListView (false, parent)
{
}

/*  KBQryExprs								*/
/*  fillCombo	: Fill combo box					*/
/*  comboBox	: QComboBox &	  : Combo box in question		*/
/*  		: uint		  : Column number			*/
/*  curVal	: const QString & : Current text value			*/
/*  (returns)	: void		  :					*/

void	KBQueryExprs::fillCombo
	(	RKComboBox	&comboBox,
		uint		,
		const QString	&curVal
	)
{
	uint	at	= 0 ;

	comboBox.clear	() ;

	for (uint idx = 0 ; exprUsage[idx] != 0 ; idx += 1)
	{
		comboBox.insertItem (exprUsage[idx]) ;
		if (exprUsage[idx] == curVal) at = idx ;
	}

	comboBox.setCurrentItem (at) ;
}



/*  ------------------------------------------------------------------  */

/*  KBQueryDlg								*/
/*  KBQueryDlg	: Constructor for query design dialog			*/
/*  parent	: QWidget *	: Parent widget				*/
/*  location	: KBLocation &	: Location information			*/
/*  qryRoot	: KBQuery *	: Document root				*/
/*  designGUI	: KBaseGUI *	: Design GUI				*/
/*  (returns)	: KBQueryDlg	:					*/

KBQueryDlg::KBQueryDlg
	(	QWidget		*parent,
		KBLocation	&location,
		KBQuery		*qryRoot,
		KBaseGUI	*designGUI
	)
	:
	KBQueryDlgBase	(parent),
	m_parent	(parent),
	m_location	(location),
	m_qryRoot	(qryRoot),
	m_designGUI	(designGUI),
	m_tblPicker	(this),
	m_hLayMain	(&m_tblPicker),
	m_vLayPick	(&m_hLayMain),
	m_serverList	(&m_tblPicker),
	m_allTables	(&m_tblPicker),
	m_qryDisplay	(&m_tblPicker),
	m_qrySpace	(&m_qryDisplay, this),

	m_expressions	(this),
	m_SQLView	(this)
{
	new KBQueryDlgTip (this, &m_qrySpace) ;

	m_vLayPick   .addWidget   (&m_serverList) ;
	m_vLayPick   .addWidget   (&m_allTables ) ;
	m_hLayMain   .addWidget   (&m_qryDisplay, 1) ;

	m_expressions.addColumn		(TR("Usage"),      100) ;
	m_expressions.addColumn		(TR("Expression"), 500) ;
	m_expressions.addColumn		(TR("Alias"),      100) ;
	m_expressions.setEditType 	(0, KBEditListView::EdComboBox) ;

	m_qryDisplay .show        () ;

	m_serverList .setFixedWidth  (160) ;
	m_allTables  .setFixedWidth  (160) ;

	KBServerInfo *self  = m_location.dbInfo()->findServer (m_location.server()) ;
	if (!self ->dbType().isEmpty())
		m_serverList.insertItem ("Self") ;

	KBServerInfo *files = m_location.dbInfo()->findServer (KBLocation::m_pFile) ;
	if (!files->dbType().isEmpty())
		m_serverList.insertItem (KBLocation::m_pFile) ;

	QPtrListIterator<KBServerInfo> *svIter = m_location.dbInfo()->getServerIter() ;
	KBServerInfo		       *svInfo ;

	while ((svInfo = svIter->current ()) != 0)
	{
		m_serverList.insertItem (svInfo->serverName()) ;
		(*svIter) += 1 ;
	}

	delete	svInfo	;

	m_popup = new QPopupMenu (this) ;
	m_popup->insertItem ("Cancel")  ;
	m_popup->insertItem ("Delete",    this, SLOT(clickDropTable())) ;
	m_popup->insertItem ("Set Alias", this, SLOT(setAlias      ())) ;
	m_popup->insertItem ("Set Key",   this, SLOT(setKey        ())) ;

	connect
	(	&m_serverList,
		SIGNAL	(activated     	(int)),
		SLOT	(serverSelected	(int))
	)	;
	connect
	(	&m_allTables, 
		SIGNAL	(selected	(int)),
		SLOT	(clickAddTable  ())
	)	;

	connect
	(	&m_qryDisplay,
		SIGNAL	(resized	(KBResizeFrame *, QSize)),
		SLOT	(displayResize	(KBResizeFrame *, QSize))
	)	;
	connect
	(	&m_qrySpace,
		SIGNAL	(windowActivated(QWidget *)),
		SLOT	(tableSelected  (QWidget *))
	)	;

	connect
	(	&m_expressions,
		SIGNAL	(changed    	(uint,uint)),
		SLOT	(exprChanged	(uint,uint))
	)	;
	connect
	(	&m_expressions,
		SIGNAL	(inserted	(uint)),
		SLOT	(exprChanged	())
	)	;
	connect
	(	&m_expressions,
		SIGNAL	(deleted	(uint)),
		SLOT	(exprChanged	())
	)	;
	connect
	(	&m_exprTimer,
		SIGNAL	(timeout	()),
		SLOT	(updateExpr	())
	)	;

	m_qryDisplay.setFrameStyle(QFrame::Box|QFrame::Plain) ;
	m_qrySpace.move	  (2, 2) ;

	m_tabList.setAutoDelete (true) ;
	buildDisplay  ()    ;

	m_trackTable	= 0	;
	m_curTable	= 0	;

	loadSQL	      ()  ;

	QValueList<int> sl		;
	sl.append	(250)		;
	sl.append	(100)		;
	sl.append	(200)		;
	resize		(sizeHint())	;
	setSizes	(sl)		;
	setResizeMode	(&m_SQLView, QSplitter::KeepSize) ;

	qApp->installEventFilter (this) ;
}

/*  KBQueryDlg								*/
/*  ~KBQueryDlg	: Destructor for query design dialog			*/
/*  (returns)	:		:					*/

KBQueryDlg::~KBQueryDlg ()
{
}

/*  KBQueryDlg								*/
/*  setChanged	: Set changed on update					*/
/*  (returns)	: void		 :					*/

void	KBQueryDlg::setChanged ()
{
	m_designGUI->setEnabled (KBaseGUI::GRDelta, true) ;
	m_qryRoot  ->setChanged (true) ;
}

/*  KBQueryDlg								*/
/*  sizeHint	: Return preferred size					*/
/*  (returns)	: QSize		: Preferred size			*/

QSize	KBQueryDlg::sizeHint ()
{
	return	QSize (600, 400) ;
}

/*  KBQueryDlg								*/
/*  resizeEvent	: Handle resize event					*/
/*  e		: QResizeEvent * : The event				*/
/*  (returns)	: void		 :					*/

void	KBQueryDlg::resizeEvent
	(	QResizeEvent	*e
	)
{
	QSplitter  ::resizeEvent (e) ;
}

QValueList<int>
	KBQueryDlg::exprSizes ()
{
	QValueList<int>	sizes	;
	sizes.append (m_expressions.columnWidth (0)) ;
	sizes.append (m_expressions.columnWidth (1)) ;
	sizes.append (m_expressions.columnWidth (2)) ;
	return		sizes	;
}

void	KBQueryDlg::setExprSizes
	(	QValueList<int>	sizes
	)
{
	if (sizes[0] > 16) m_expressions.setColumnWidth (0, QMIN(sizes[0],  200)) ;
	if (sizes[1] > 16) m_expressions.setColumnWidth (1, QMIN(sizes[1], 1200)) ;
	if (sizes[2] > 16) m_expressions.setColumnWidth (2, QMIN(sizes[2],  300)) ;
}

/*  KBQueryDlg								*/
/*  displayResize: Handle resize of display widget			*/
/*  frame	 : KBResizeFrame * : Frame				*/
/*  s		 : QSize	   : New size				*/
/*  (returns)	 : void		   :					*/

void	KBQueryDlg::displayResize
	(	KBResizeFrame	*,
		QSize		s
	)
{
	m_qrySpace.resize (s - QSize(4,4)) ;
}

/*  KBQueryDlg								*/
/*  buildDisplay: Build list view and graphic displays			*/
/*  (returns)	: void		:					*/

void	KBQueryDlg::buildDisplay ()
{
	/* Get the server name from the query and look through the list	*/
	/* of known servers to verify that it is there and to show the	*/
	/* appropriate one ...						*/
	QString			svName	   ;	
	QPtrList<KBTable  >	qryTabList ;
	QPtrList<KBQryExpr>	exprList   ;
	KBEditListViewItem	*last	   = 0 ;

	m_qryRoot->getQueryInfo (svName, qryTabList, exprList) ;

	m_serverIndex	= -1 	   ;

	for (int idx = 0 ; idx < m_serverList.count() ; idx += 1)
		if (m_serverList.text(idx) == svName)
		{	m_serverIndex = idx ;
			break ;
		}

	if ((m_serverIndex > 0) || (svName == "Self"))
	{
		/* Found the server, so load the table and join stuff.	*/
		/* Iterate over the tables and add them, then add any	*/
		/* expressions.						*/
		m_serverList.setCurrentItem (m_serverIndex) ;
		serverConnect () ;

		LITER
		(	KBTable,
			qryTabList,
			table,
			m_tabList.append (new KBTableAlias (this, table))
		)

		LITER
		(	KBQryExpr,
			exprList,
			expr,

			KBQryExpr::Usage usage = expr->getUsage() ;
			QString		 alias = expr->getAlias() ;

			if ((uint)usage >= sizeof(exprUsage)/sizeof(exprUsage[0]) - 1)
				usage = KBQryExpr::AsExprOnly ;

			if (usage != KBQryExpr::AsExprOnly) alias = "" ;

			last  = new KBEditListViewItem
				(	&m_expressions,
					last,
					exprUsage[usage],
					expr->getExpr (),
					alias
				)
		)
	}
	else if (!svName.isEmpty())
	{
		/* Server not found. In this case grumble, and clear	*/
		/* the tables from the query object, since thay are not	*/
		/* referred to from any KBTableAlias objects.		*/
		TKMessageBox::sorry
		(	0,
			QString (TR("Server \"%1\" is not in this database")).arg(svName),
			TR("Query server error")
		)	;
		m_serverIndex = 0 ;

		LITER (KBTable, qryTabList, t, delete t)

		m_serverList.setCurrentItem (0) ;
	}

	serverSelected   (m_serverList.currentItem()) ;

	new KBEditListViewItem  (&m_expressions, last, "") ;
	m_tblPicker .show() ;
	m_qrySpace  .show() ;
}

/*  KBQueryDlg								*/
/*  serverConnect: Connect to selected server				*/
/*  (returns)	 : void		:					*/

void	KBQueryDlg::serverConnect ()
{
	m_dbLink.disconnect () ;
	m_allTables.clear   () ;

	if (!m_dbLink.connect (m_location, m_serverList.currentText()))
	{
		m_dbLink.lastError().DISPLAY () ;
		return	;
	}

	KBTableDetailsList tabList	;

	if (!m_dbLink.listTables (tabList))
	{
		m_dbLink.lastError().DISPLAY() ;
		return	;
	}

	for (uint idx = 0 ; idx < tabList.count() ; idx += 1)
		m_allTables.insertItem (tabList[idx].m_name) ;
}

/*  KBQueryDlg								*/
/*  nameIsFree	: See if name is free for use				*/
/*  name	: const QString & : Name to be checked			*/
/*  checkAll	: bool		  : Check against all			*/
/*  (returns)	: bool		  : Name is free			*/

bool	KBQueryDlg::nameIsFree
	(	const QString	&name,
		bool		checkAll
	)
{
	LITER
	(	KBTableAlias,
		m_tabList,
		table,

		/* If the current search item is the item being edited	*/
		/* then maybe skip it, otherwise see if the name	*/
		/* conflicts with the name by which the table is	*/
		/* identified in the SQL statement.			*/
		if (checkAll || (table != m_curTable))
			if (table->getTable()->getQueryName () == name)
				return	false	;
	)
	return	true	;
}

/*  KBQueryDlg								*/
/*  setAlias	: Set table alias					*/
/*  (returns)	: void		:					*/

void	KBQueryDlg::setAlias ()
{
	if (m_curTable == 0) return ;

	QString	table	= m_curTable->getTable()->getTable() ;
	QString	alias	= m_curTable->getTable()->getAlias() ;

	if (!doPrompt
		(	TR("Alias"),
			QString(TR("Enter alias for table %1")).arg(table),
			alias
	   )	)	return ;

	/* If the first alias is blank then verify that this is the	*/
	/* only entry for that table. On the other hand, if it is not	*/
	/* blank then it must be different to all other tables and	*/
	/* aliases.							*/
	if (alias.isEmpty())
	{
		if (!nameIsFree (table, false))
		{
			TKMessageBox::sorry
			(	0,
				QString (TR("Table \"%1\" appears more than once: an alias is required"))
					.arg(table),
				TR("Alias required")
			)	;
			return	;
		}
	}
	else
	{
		if (!nameIsFree (alias, false))
		{
			TKMessageBox::sorry
			(	0,
				QString (TR("Alias \"%1\" already used as a table or alias name"))
					.arg(alias),
				TR("Unique alias required")
			)	;
			return	;
		}
	}

	/* Update the alias, except that if the alias is the same as	*/
	/* the table name then clear it.				*/
	if (alias != table)
		m_curTable->setAlias (alias) ;
	else	m_curTable->setAlias (""   ) ;

	loadSQL	   () ;
	setChanged () ;
}

/*  KBQueryDlg								*/
/*  setKey	: Set table key						*/
/*  (returns)	: void		:					*/

void	KBQueryDlg::setKey ()
{
	if (m_curTable == 0)  return ;

	QString		    column ;
	QStringList	    names  ;
	KBTable::UniqueType utype  ;
	QString		    uexpr  ;
	KBTableSpec	    tabSpec(m_curTable->getTable()->getTable()) ;

	if (!m_dbLink.listFields (tabSpec))
	{
		m_dbLink.lastError().DISPLAY() ;
		return	;
	}

	utype = m_curTable->getTable()->getUnique (names, uexpr) ;

	KBQryPrimaryDlg pDlg (tabSpec, names, utype, uexpr) ;
	if (!pDlg.exec()) return ;

	utype  = pDlg.retrieve (column, uexpr) ;
	m_curTable->setPrimary (column, utype) ;
	setChanged () ;
}

/*  KBQueryDlg								*/
/*  serverSelected: Handle server selection				*/
/*  index	  : int		: Server list index			*/
/*  (returns)	  : void	:					*/

void	KBQueryDlg::serverSelected
	(	int	index
	)
{
	if (index == m_serverIndex)
		return ;

	if (m_tabList.count() > 0)
		switch (TKMessageBox::questionYesNoCancel
			   	(	0,
					TR
					(	"This will probably invalidate all tables and relationships\n"
						"Do you really want to do this?"
					),
					0,
					TR	("Yes: clear query"),
					TR	("Yes: leave query")
				))
		{
			case TKMessageBox::Yes :
				LITER
				(	KBTableAlias,
					m_tabList,
					t,
					delete	t->getTable()
				)
				m_tabList.clear ()    ;
				break	;

			case TKMessageBox::No  :
				break	;

			default	:
				m_serverList.setCurrentItem (m_serverIndex) ;
				return	;
		}

	m_serverIndex = index ;
	serverConnect () ;

	m_qryRoot->setServer (m_serverList.currentText()) ;
	loadSQL	      () ;
	repaintLinks  () ;
	setChanged    () ;
}

/*  KBQueryDlg								*/
/*  getUniqeueAlias: Get unique alias for a table			*/
/*  name	   : const QString & : Table name			*/
/*  (returns)	   : QString	     : Unique alias			*/

QString	KBQueryDlg::getUniqueAlias
	(	const QString	&name
	)
{
	uint		mIdx	= 0 ;
	QString		alias	;

	if (nameIsFree  (name, true)) return "" ;

	for (;;)
	{
		mIdx   += 1	;
		alias	= QString("%1_%2").arg(name).arg(mIdx) ;
		if (nameIsFree (alias, true)) return alias ;
	}
}

/*  KBQueryDlg								*/
/*  findTable	: Find table alias by widget				*/
/*  widget	: QWidget *	 : Widget to seach for			*/
/*  (returns)	: KBTableAlias * : Table alias or null if not found	*/

KBTableAlias
	*KBQueryDlg::findTable
	(	QWidget		*widget
	)
{
	LITER
	(	KBTableAlias,
		m_tabList,
		table,
		if ((QWidget *)table == widget) return table ;
	)

	return	0 ;
}

/*  KBQueryDlg								*/
/*  findTable	: Find table alias by global position			*/
/*  p		: QPoint	 : Global position			*/
/*  field	: QString &	 : Field under point if found		*/
/*  (returns)	: KBTableAlias * : Table alias or null if not found	*/

KBTableAlias
	*KBQueryDlg::findTable
	(	QPoint		p,
		QString		&field
	)
{
	LITER
	(	KBTableAlias,
		m_tabList,
		table,
		if (table->hit (p, field)) return table ;
	)

	return	0 ;
}

/*  KBQueryDlg								*/
/*  getPrimary	: Get primary key for named table			*/
/*  tabName	: const QString & : Table name				*/
/*  colName	: QString &	  : Return column name			*/
/*  (returns)	: UniqueType	  : Uniqueness type			*/

KBTable::UniqueType
	KBQueryDlg::getPrimary
	(	const QString	&tabName,
		QString		&colName
	)
{
	KBTableSpec	tabSpec	(tabName) ;
	KBFieldSpec	*fSpec	;

	if (!m_dbLink.listFields (tabSpec))
	{
		m_dbLink.lastError().DISPLAY() ;
		colName	= QString::null	     ;
		return	KBTable::AnySingle   ;
	}


	fSpec	= tabSpec.findPrimary() ;
	if (fSpec != 0)
	{	colName	= fSpec->m_name	     ;
		return	KBTable::PrimaryKey  ;
	}

	fSpec	= tabSpec.findUnique () ;
	if (fSpec != 0)
	{	colName	= fSpec->m_name	     ;
		return	KBTable::AnyUnique   ;
	}

	colName	= QString::null	     ;
	return	KBTable::AnySingle   ;
}

/*  KBQueryDlg								*/
/*  clickAddTable: Handle request to add a table			*/
/*  (returns)	 : void		:					*/

void	KBQueryDlg::clickAddTable ()
{
	if (m_allTables.currentItem () < 0) return ;

	QString	tName	= m_allTables.text (m_allTables.currentItem()) ;
	QString	aName	= getUniqueAlias   (tName) ;

	QString		    uName ;
	KBTable::UniqueType uType = getPrimary (tName, uName) ;

	/* Construct a new table object and set the primary key if	*/
	/* there is one.						*/
	KBTable	*table	= new KBTable  (m_qryRoot, tName, aName, "", "", "", "", "", "", "") ;
	table->setPrimary (uName, uType) ;

	m_tabList.append  (m_curTable = new KBTableAlias (this, table)) ;
	loadSQL    () ;
	setChanged () ;
}

/*  KBQueryDlg	  :							*/
/*  clickDropTable: Request to drop a table				*/
/*  (returns)	  : void	:					*/

void	KBQueryDlg::clickDropTable ()
{
	if (m_curTable != 0)
	{
		KBTableAlias *t = m_curTable ;
		m_curTable	= 0	     ;
		delete	t->getTable()	     ;
		m_tabList.remove (t)	     ;
		
		loadSQL 	() ;
		repaintLinks	() ;
		setChanged	() ;
	}
}

/*  KBQueryDlg	 :							*/
/*  tableSelected: User selects table in graphic view			*/
/*  widget	 : QWidget *	: Table					*/
/*  (returns)	 : void		:					*/

void	KBQueryDlg::tableSelected
	(	QWidget		*widget
	)
{
	if (widget != 0) m_curTable = findTable (widget) ;
}

/*  KBQueryDlg								*/
/*  loadSQL	: Load SQL into SQL view window				*/
/*  (returns)	: void		:					*/

void	KBQueryDlg::loadSQL ()
{

	/* NOTE: This is done mostly by interrogating the KBQuery	*/
	/* object and its KBTable children, rather than by extracting	*/
	/* data directly from the local list. It is done this way as a	*/
	/* check that changes at the dialog box level are being		*/
	/* correctly propogated to the table nodes.			*/
	KBSelect		select	 ;
	QString			svName   ;
	QPtrList<KBTable>	qryList	 ;
	QPtrList<KBTable>	tabList	 ;
	QPtrList<KBQryExpr>	exprList ;
	KBError			error	 ;

	m_qryRoot->getQueryInfo (svName, qryList, exprList) ;


	/* No tables, no SQL ...					*/
	if (qryList.count() == 0)
	{
		m_SQLView.setText ("") ;
		return	;
	}

	/* Block up into a flat query (exactly as will be done in data	*/
	/* view. This has the side effect of setting the table join	*/
	/* expressions.							*/
	if (!KBTable::blockUp (qryList, QString::null, tabList, error))
	{
		error.DISPLAY () ;
		m_SQLView.setText ("") ;
		return	;
	}

	LITER
	(	KBQryExpr,
		exprList,
		expr,

		switch (expr->getUsage())
		{
			case KBQryExpr::AsExprOnly :
				select.appendExpr  (expr->getExpr(), expr->getAlias()) ;
				break	;

			case KBQryExpr::AsWhere	   :
				select.appendWhere (expr->getExpr()) ;
				break	;

			case KBQryExpr::AsSortAsc  :
				select.appendOrder (expr->getExpr() + " asc" ) ;
				break	;

			case KBQryExpr::AsSortDesc :
				select.appendOrder (expr->getExpr() + " desc") ;
				break	;

			case KBQryExpr::AsGroup	   :
				select.appendGroup (expr->getExpr()) ;
				break	;

			case KBQryExpr::AsHaving   :
				select.appendHaving(expr->getExpr()) ;
				break	;

			default	:
				break	;
		}
	)

	LITER
	(	KBTable,
		tabList,
		table,
		table->addToSelect (select, true) ;
	)

	m_SQLView.setText (select.getPrettyText(true, &m_dbLink)) ;
}


/*  KBQueryDlg	:							*/
/*  exprChanged	: User has added or deleted an expression		*/
/*  (returns)	: void		:					*/

void	KBQueryDlg::exprChanged ()
{
	updateExprs (false) ;
	setChanged  () ;
}

/*  KBQueryDlg	:							*/
/*  exprChanged	: User has added or deleted an expression		*/
/*  rowNo	: uint		: Row number				*/
/*  colNo	: uint		: Column number				*/
/*  (returns)	: void		:					*/

void	KBQueryDlg::exprChanged
	(	uint		rowNo,
		uint		
	)
{
	QListViewItem	*item = m_expressions.firstChild() ;
	while ((item != 0) && (rowNo > 0))
	{	item	= item->nextSibling() ;
		rowNo  -= 1 ;
	}

	if ((item != 0) && (item->text(0) != ""))
		item->setText (2, "") ;

	updateExprs (false) ;
	setChanged  () ;

	m_exprTimer.start (500) ;
}

/*  KBQueryDlg								*/
/*  updateExprs	: Update expressions to match dialog			*/
/*  checkCommas	: bool		: Check for spurious commas		*/
/*  (returns)	: bool		: All expressions OK			*/

bool	KBQueryDlg::updateExprs
	(	bool	checkCommas
	)
{
	QString			svName	   ;	
	QPtrList<KBTable  >	qryTabList ;
	QPtrList<KBQryExpr>	exprList   ;
	bool			rc	   = true  ;
	bool			commas	   = false ;

	m_exprTimer.stop() ;

	m_qryRoot->getQueryInfo (svName, qryTabList, exprList) ;
	exprList.setAutoDelete(true) ;
	exprList.clear	      ()     ;


	for (QListViewItem *lvi  = m_expressions.firstChild () ;
			    lvi != 0 ;
			    lvi  = lvi->nextSibling())
	{
		if (lvi->text(1).isEmpty())
			continue ;

		if	(lvi->text(1).stripWhiteSpace() == "*")
		{	lvi->setText (1, "*") ;
			lvi->setText (2,  "") ;
		}
		else if (checkCommas && !KBSelect::singleExpression (lvi->text(1)))
		{
			if (!commas)
			{
				KBError::EError
				(	QString (TR("Expressions may not contain multiple columns, please edit")),
					lvi->text(1),
					__ERRLOCN
				)	;
				commas	= true	;
				rc	= false	;
			}
		}

		new KBQryExpr
		(	m_qryRoot,
			lvi->text(1),
			lvi->text(2),
			(KBQryExpr::Usage)usageToCode(lvi->text(0))
		)	;
	}

	loadSQL () ;
	return	rc ;
}

/*  KBQueryDlg								*/
/*  updateExprs	: Update expressions to match dialog			*/
/*  (returns)	: bool		: All expressions OK			*/

bool	KBQueryDlg::updateExprs ()
{
	return	updateExprs (false) ;
}

/*  KBQueryDlg								*/
/*  saveDocument: Save current document state				*/
/*  (returns)	: bool		: Success				*/

bool	KBQueryDlg::saveDocument ()
{
	m_exprTimer  .stop	() ;
	m_expressions.cancelEdit() ;

	if (!updateExprs (true)) return false ;

	/* Log the geometries of the table listboxes. Note that we have	*/
	/* to get the position via the parent, since they are embedded	*/
	/* in a workspace, while the size comes from list listboxes	*/
	/* themselves.							*/
	LITER
	(	KBTableAlias,
		m_tabList,
		table,
		table->getTable()->setGeometry
		(
			QRect
			(	table->geometry().topLeft(),
				table->size()
		)	)	;
	)

	return	true	;
}


/*  KBQueryDlg								*/
/*  repaintLinks: Repaint all relationship links			*/
/*  p		: QPintEvent *	: Paint event				*/
/*  (returns)	: void		:					*/

void	KBQueryDlg::repaintLinks
	(	QPaintEvent	*
	)
{
	QPainter painter (m_qrySpace.spaceWidget()) ;

	for
	(	QPtrListIterator<KBTableAlias> iter (m_tabList) ;
		iter.current() != 0 ;
		iter += 1
	)
	{
		KBTableAlias	*table	= iter.current   () ;
		QString		pid	= table->getTable()->getParent() ;
		KBTableAlias	*parent = 0 ;

		table->setLinkArea (QRect(), 0)	;
		if (pid.isEmpty()) continue	;

		LITER
		(	KBTableAlias,
			m_tabList,
			table,

			if (table->getTable()->getIdent() == pid)
			{	parent	= table ;
				break	;
			}
		)

		if (parent == 0) continue ;

		QString	field	= table->getTable()->getField () ;
		QString	field2	= table->getTable()->getField2() ;

		if (field.isEmpty() || field2.isEmpty())
			continue ;

		QRect	pr	= parent->geometry() ;
		QRect	tr	= table ->geometry() ;
		bool	ps	= false	;
		bool	ts	= false	;

		if	(pr.right() <= tr. left())
		{	ps	= true	;
			ts	= false	;
		}
		else if	(pr.left () >= tr.right())
		{	ps	= false	;
			ts	= true	;
		}

		bool	pkp	;
		bool	tkp	;
		QPoint	pp1	= parent->getPosition (field2, ps, pkp) ;
		QPoint	tp1	= table ->getPosition (field,  ts, tkp) ;

		QRect	hit	= QRect(pp1, tp1).normalize() ;
		if (hit.width () < 16)
		{	hit.moveLeft  (hit.left() - (16 - hit.width ()) / 2) ;
			hit.setWidth  (16) ;
		}
		if (hit.height() < 16)
		{	hit.moveTop   (hit.top () - (16 - hit.height()) / 2) ;
			hit.setHeight (16) ;
		}
		table->setLinkArea (hit, parent) ;

		QPoint	pp2	= ps ? pp1 + QPoint(12, 0) : pp1 - QPoint(12, 0) ;
		QPoint	tp2	= ts ? tp1 + QPoint(12, 0) : tp1 - QPoint(12, 0) ;
		static	QPen	 pen1 (black, 1) ;
		static	QPen	 pen3 (black, 3) ;

		painter.setPen   (pen3	  ) ;
		painter.drawLine (pp1, pp2) ;
		painter.drawLine (tp1, tp2) ;
		painter.setPen   (pen1	  ) ;
		painter.drawLine (pp2, tp2) ;

		if (!pkp)
		{	painter.drawLine (pp1 + QPoint(0, 6), pp2) ;
			painter.drawLine (pp1 - QPoint(0, 6), pp2) ;
		}
		if (!tkp)
		{	painter.drawLine (tp1 + QPoint(0, 6), tp2) ;
			painter.drawLine (tp1 - QPoint(0, 6), tp2) ;
		}
	}
}

/*  KBQueryDlg								*/
/*  repaintLinks: Repaint all relationship links on timeout		*/
/*  (returns)	: void		:					*/

void	KBQueryDlg::repaintLinks ()
{
	m_qrySpace.spaceWidget()->update() ;
}

/*  KBQueryDlg								*/
/*  linkProperties							*/
/*		: Display link properties dialog (maybe ...)		*/
/*  mp		: QPoint	: Mouse position			*/
/*  (returns)	: void		:					*/

void	KBQueryDlg::linkProperties
	(	QPoint		mp
	)
{
	KBTableAlias	*owner	= 0	    ;
	KBTableAlias	*other	= 0	    ;
	int		best	= 0x7ffffff ;


	/* Search to see if the mouse position is reasonably close to	*/
	/* being over a link. The main loop iterates over all tables	*/
	/* that are linked to a parent.					*/
	LITER
	(	KBTableAlias,
		m_tabList,
		table,

		QString		pid	= table->getTable()->getParent() ;
		KBTableAlias	*parent = 0 ;

		if (pid.isEmpty()) continue ;

//		QPainter painter (m_qrySpace.spaceWidget()) ;
//		painter.setPen	 (Qt::red) ;
//		painter.drawRect (table->linkArea()) ;

		parent	= table->linkAreaHit (mp, best) ;
		if (parent != 0)
		{	owner	= table	 ;
			other	= parent ;
		}
	)

	/* No match, give up, otherwise run the join dialog based on	*/
	/* the tables found.						*/
	if (owner == 0) return ;

	DPRINTF
	((	"KBQueryDlg::linkProperties: pointers: owner=%p other=%p\n",
		(void  *)owner->getTable(),
		(void  *)other->getTable()
	))	;
	DPRINTF
	((	"KBQueryDlg::linkProperties: names:    owner=%s other=%s\n",
		(cchar *)owner->getTable()->getTable(),
		(cchar *)other->getTable()->getTable()
	))	;

	KBQryJoinDlg jDlg
		     (	owner,
			owner->getTable()->getField (),
			other,
			owner->getTable()->getField2(),
			owner->getTable()->getJType (),
			owner->getTable()->getJExpr (),
			owner->getTable()->useJExpr (),
			true
		     )  ;

	if (!jDlg.exec ()) return ;

	QString	jtype	;
	QString	jexpr	;
	bool	uexpr	;
	bool	doDel	= jDlg.getResults (jtype, jexpr, uexpr) ;

	if (doDel)
	{
		KBTable	*table	= owner->getTable() ;
		table->setParent ("") ;
		table->setField  ("") ;
		table->setField2 ("") ;
		table->setJType  ("") ;
		table->setJExpr  ("") ;
	}
	else
	{
		owner->getTable()->setJType  (jtype) ;
		owner->getTable()->setJExpr  (jexpr) ;
		owner->getTable()->setUseExpr(uexpr) ;
	}

	loadSQL		() ;
	repaintLinks	() ;
	setChanged	() ;
}

/*  KBQueryDlg								*/
/*  hasAncestor	: See if a table has a specified ancestor		*/
/*  table	: KBTable *	: Table in question			*/
/*  ancestor	: KBTable *	: Putative ancestor			*/
/*  (returns)	: bool		: True if ancestor really is		*/

bool	KBQueryDlg::hasAncestor
	(	KBTable		*table,
		KBTable		*ancestor
	)
{
	KBTable	*parent	= 0	;
	QString	pid		= table->getParent() ;

	if (pid.isEmpty()) return false ;

	LITER
	(	KBTableAlias,
		m_tabList,
		t,

		if (t->getTable()->getIdent() == pid)
		{
			parent	= t->getTable() ;
			break	;
		}
	)

	return	parent == 0	   ? false :
		parent == ancestor ? true  : hasAncestor (parent, ancestor) ;
}

/*  KBQueryDlg								*/
/*  eventFilter	: Event filter method					*/
/*  o		: QObject *	: Object being filtered			*/
/*  e		: QEvent  *	: Event					*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBQueryDlg::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (e->type() != QEvent::ContextMenu)
		return	false	;
	
	if (qstrcmp ("qt_ws_titlebar", o->name()) != 0)
		return	false	;

	for (uint idx = 0 ; idx < m_tabList.count() ; idx += 1)
		if (m_tabList.at(idx)->parent() == o->parent())
		{
			showContextMenu (m_tabList.at(idx)) ;
			return	true	;
		}

	return	false	;
}

/*  KBQueryDlg								*/
/*  startLinking: Start table linking operation				*/
/*  table	: KBTableAlias * : Table from which tracking starts	*/
/*  field	: QString	 : Field in table			*/
/*  (returns)	: void		 :					*/

void	KBQueryDlg::startLinking
	(	KBTableAlias	*table,
		QString		field
	)
{
	m_trackTable = table ;
	m_trackField = field ;
	grabMouse  (QCursor(PointingHandCursor)) ;
}

/*  KBQueryDlg								*/
/*  showContextMenu							*/
/*		: Show context menu for a table				*/
/*  table	: KBTableAlias * : Table from which tracking starts	*/
/*  (returns)	: void		 :					*/

void	KBQueryDlg::showContextMenu
	(	KBTableAlias	*table
	)
{
	m_curTable = table ;
	m_popup->popup (QCursor::pos()) ;
}

/*  KBQueryDlg								*/
/*  mouseReleaseEvent							*/
/*		: Handle release of mouse button			*/
/*  e		: QMouseEvent *	: Mouse event				*/
/*  (returns)	: void		:					*/

void	KBQueryDlg::mouseReleaseEvent
	(	QMouseEvent	*e
	)
{
	/* If not tracking then just do the default ....		*/
	if (m_trackTable == 0)
	{
		QWidget::mouseReleaseEvent (e) ;
		return	;
	}

	releaseMouse () ;

	/* See if the mouse has been released over a field in a table	*/
	/* list box, except in the case if being released over the	*/
	/* table where we started ...					*/
	QString		toField   ;
	KBTableAlias	*toTable  = findTable (e->globalPos(), toField) ;

	if ((toTable != 0) && (toTable != m_trackTable))
	{
		DPRINTF
		((	"KBQueryDlg::mouseReleaseEvent: [%s][%s][%s] -> [%s][%s][%s]\n",
			(cchar *)m_trackTable->getTable()->getTable(),
			(cchar *)m_trackField,
			(cchar *)m_trackTable->getKeyField(),
			(cchar *)toTable->getTable()->getTable(),
			(cchar *)toField,
			(cchar *)toTable->getKeyField()
		))	;

		KBTableAlias	*pTable	= 0	;
		KBTableAlias	*cTable	= 0	;
		QString		pField	;
		QString		cField	;

		/* First check if one end or other is a primary key, in	*/
		/* which case the table ordering is fixed so that the	*/
		/* "child" table is at the primary key end (and if	*/
		/* both ends are primary, then we use the table at	*/
		/* which the user started.				*/
		/*							*/
		/* NOTE: The "parent" table refers to the parent/child	*/
		/*	 relation as stored; the "parent" will be the	*/
		/*	 non-primary-key table, ie., the one which has	*/
		/*	 the link information stored.			*/
		if	(m_trackTable->getKeyField() == m_trackField)
		{
			pTable	= toTable	;
			pField	= toField	;
			cTable	= m_trackTable	;
			cField	= m_trackField	;
		}
		else if	(toTable->getKeyField() == toField)
		{
			pTable	= m_trackTable	;
			pField	= m_trackField	;
			cTable	= toTable	;
			cField	= toField	;
		}
		/* If neither end is a primary key then ignore ...	*/
		else	return	;

		m_trackTable = 0	;

		if (pTable == 0)
			return	;

		if (hasAncestor (pTable->getTable(), cTable->getTable()))
		{
			TKMessageBox::sorry
			(	0,
				TR("Cannot create a relationship loop"),
				TR("Query Error")
			)	;
			return	;
		}

		cTable->getTable()->setParent (pTable->getTable()->getIdent()) ;
		cTable->getTable()->setField  (cField) ;
		cTable->getTable()->setField2 (pField) ;

		loadSQL      () ;
		repaintLinks () ;
		setChanged   () ;
		return		;
	}

	/* See if the mouse has been released over an expression field	*/
	/* in the expressions list ....					*/
	QPoint			vpPos	= m_expressions.viewport()->mapFromGlobal(e->globalPos()) ;
	KBEditListViewItem	*item	= (KBEditListViewItem *)m_expressions.itemAt(vpPos) ;

	if ((item != 0) && (m_trackTable != 0))
	{
		item->putText
		(	1,
			QString	("%1 %2.%3")
				.arg(item->text(1))
				.arg(m_trackTable->getTable()->getQueryName())
				.arg(m_trackField)
		)	;
	
		if (item->nextSibling() == 0)
			new KBEditListViewItem (&m_expressions, item, "") ;


		m_trackTable = 0 ;

		updateExprs (false) ;
		setChanged  ()  ;

		return   ;
	}

	m_trackTable = 0 ;
}

/*  KBQueryDlg								*/
/*  getQrySpace	: Get the query space widget				*/
/*  (returns)	: QWidget *	: Widget				*/

QWidget	*KBQueryDlg::getQrySpace ()
{
	return	m_qrySpace.spaceWidget () ;
}

/*  KBQueryDlg								*/
/*  getDBLink	: Get associated database link				*/
/*  (returns)	: KBDBLink &	:					*/

KBDBLink &KBQueryDlg::getDBLink	()
{
	return	m_dbLink ;
}

/*  KBQueryDlg								*/
/*  tip		: Get possible "tool" tip				*/
/*  point	: const QPoint & : Mouse position			*/
/*  rect	: QRect &	 : Return rea for which tip remains	*/
/*  (returns)	: QString	 : Tip text or null if none		*/

QString	KBQueryDlg::tip
	(	const QPoint	&point,
		QRect		&rect
	)
{
	KBTableAlias	*owner	= 0	    ;
	KBTableAlias	*other	= 0	    ;
	int		best	= 0x7ffffff ;

	/* Scan the tables to see if the mouse is sufficiently over a	*/
	/* link. If not then nothing will be shown.			*/
	LITER
	(	KBTableAlias,
		m_tabList,
		table,

		QString	 pid	= table->getTable()->getParent() ;
		if (pid.isEmpty()) continue ;

		KBTableAlias *parent = table->linkAreaHit (point, best) ;
		if (parent != 0)
		{
			owner	= table	 ;
			other	= parent ;
			break	;
		}
	)

	if (owner == 0)
		return	QString::null ;

	/* Get the underlying tables and construct some text that	*/
	/* specifies the tables (and possible alias) along with their	*/
	/* join condition, in a way that looks like a fragment of an	*/
	/* SQL select query.						*/
	KBTable	    *tabOwner	= owner->getTable() ;
	KBTable	    *tabOther	= other->getTable() ;
	cchar	    *sep	= "<br/><b>where</b>" ;
	QStringList text	;

	text.append (other->caption()) ;

	if	(tabOwner->getJType() == "left" )
	{	text.append ("<br/><b>left outer join</b>") ;
		sep	= "<br/><b>on</b>" ;
	}
	else if (tabOwner->getJType() == "right")
	{	text.append ("<br/><b>left outer join</b>") ;
		sep	= "<br/><b>on</b>" ;
	}
	else	text.append (",") ;

	text.append (owner->caption()) ;
	text.append (sep) ;

	if (!tabOwner->useJExpr())
	{
		text.append (QString("%1.%2").arg(tabOwner->getQueryName()).arg(tabOwner->getField ())) ;
		text.append ("=") ;
		text.append (QString("%1.%2").arg(tabOther->getQueryName()).arg(tabOwner->getField2())) ;

	}
	else	text.append (tabOwner->getJExpr()) ;

	/* Return the area outside of which the tip should disappear,	*/
	/* and the text.						*/
	rect	= owner->linkArea() ;
	return	text.join(" ") ;
}
