/***************************************************************************
    file	         : kb_querydlg.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_QUERYDLG_H
#define	_KB_QUERYDLG_H


#include	<qsplitter.h>

#include	<qlistview.h>
#include	<qlineedit.h>
#include	<qcombobox.h>
#include	<qlabel.h>
#include	<qtextview.h>
#include	<qpushbutton.h>

#include	<qlayout.h>
#include	<qpopupmenu.h>
#include	<qtimer.h>


#include	"kb_editlistview.h"
#include	"kb_tablealias.h"
#include	"kb_resizewidget.h"
#include	"kb_location.h"
#include	"kb_dblink.h"


#include	"kb_qryspace_std.h"
#include	"kb_querydlg_std.h"

#include	"rk_listbox.h"


class	KBQueryDlg	;
class	KBQueryExprs	;
class	RKComboBox	;

class	KBQueryExprs : public KBEditListView
{
public	:

	KBQueryExprs	(QWidget *)	;

	virtual	void	fillCombo   (RKComboBox &, uint, const QString &) ; 
}	;


/*  KBQueryDlg								*/
/*  ----------								*/
/*  Design dialog for KBase queries. Actually, it isn't a dialog at all	*/
/*  in the Qt sense, but it looks like one to the user.			*/

class	KBQueryDlg : public KBQueryDlgBase
{
	Q_OBJECT

	QWidget			*m_parent		;

	KBLocation		&m_location		;
	KBQuery			*m_qryRoot		;
	KBaseGUI		*m_designGUI		;
	QWidget			m_tblPicker		;
	QHBoxLayout		m_hLayMain		;
	QVBoxLayout		m_vLayPick		;
	QComboBox		m_serverList		;
	RKListBox		m_allTables		;

	KBResizeFrame		m_qryDisplay		;
	KBQueryspace		m_qrySpace		;

	KBQueryExprs		m_expressions		;
	QTextView		m_SQLView		;

	QTimer			m_exprTimer		;
	QPopupMenu		*m_popup		;

	int			m_serverIndex		;
	int			m_tableIndex		;
	KBDBLink		m_dbLink		;
	KBTableAlias		*m_curTable		;

	QPtrList<KBTableAlias>	m_tabList		;
	KBTableAlias		*m_trackTable		;
	QString			m_trackField		;

	void			serverConnect		() ;
	void			buildDisplay		() ;

	QString			getUniqueAlias		(const QString   &) ;
	KBTable::UniqueType	getPrimary		(const QString   &, QString &) ;

	void			loadSQL			() ;

	bool			hasAncestor		(KBTable *, KBTable *) ;
	bool			nameIsFree		(const QString   &, bool) ;
	bool			updateExprs		(bool) ;

	KBTableAlias		*findTable		(QWidget         *) ;
	KBTableAlias		*findTable		(QPoint, QString &) ;

protected :

	virtual	void		resizeEvent		(QResizeEvent  *)	;
	virtual	void		mouseReleaseEvent	(QMouseEvent   *)	;
	virtual bool		eventFilter		(QObject *, QEvent *) ;


private	slots :

	void			displayResize		(KBResizeFrame *, QSize) ;
	void			serverSelected		(int  ) ;
	void			tableSelected		(QWidget       *) ;

	void			clickAddTable		() ;
	void			clickDropTable		() ;

	void			setAlias		() ;
	void			setKey			() ;

	void			exprChanged		(uint, uint) ;
	void			exprChanged		() ;
	bool			updateExprs		() ;

public	slots	:

	void			repaintLinks		() ;

public	:

	KBQueryDlg (QWidget *, KBLocation &, KBQuery *, KBaseGUI *) ;
       ~KBQueryDlg () ;

	QSize			sizeHint		()	;
	void			repaintLinks		(QPaintEvent *) ;
	void			linkProperties		(QPoint);

	void			showContextMenu		(KBTableAlias *) ;
	void			startLinking		(KBTableAlias *, QString) ;

	bool			saveDocument		()	;
	QValueList<int>		exprSizes		()	;
	void			setExprSizes		(QValueList<int>) ;
	void			setChanged		()	;

	QString			tip			(const QPoint &, QRect &) ;
	QWidget			*getQrySpace		()	;
	KBDBLink		&getDBLink		()	;
}	;

#define	TABITER(list,name)	\
	for (KBTableAlias *name = (KBTableAlias *)list .firstChild () ;	\
							    name != 0 ;	\
			  name  = (KBTableAlias *)name->nextSibling() )


#endif	//_KB_QUERYDLG_H
