/***************************************************************************
    file	         : kb_tablealias.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qimage.h>
#include	<qpainter.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"

#include	"kb_dbinfo.h"
#include	"kb_dblink.h"
#include	"kb_node.h"
#include	"kb_query.h"
#include	"kb_table.h"
#include	"kb_ascii.h"

#include	"kb_querydlg.h"

#if		__KB_AUTOMOC
#include	"kb_tablealias.moc"
#else
#include	"kb_tablealias.h"
#endif

#include	"tk_icons.h"
#include	"rk_vbox.h"


#define		__DEBUG_TABLEALIAS	(0)
#if		__DEBUG_TABLEALIAS
#include	"kb_dprintf.h"
#endif


/*  KBTableListBoxItem							*/
/*  paint	: Handle paint operation on field entry			*/
/*  painter	: QPainter *	: Paint object				*/
/*  (returns)	: void		:					*/

void	KBTableListBoxItem::paint
	(	QPainter	*painter
	)
{
	static	QImage	*kImg	;

	/* First time round load the key marker image ...		*/
	if (kImg == 0)
		kImg = new QImage  (getSmallIcon("key").convertToImage()) ;

	/* ... and draw it if this is the preferred key fieldd.		*/
	if (m_pKey)
		painter->drawImage (2, 0, *kImg, 0, 0, kImg->width(), kImg->height(), 0) ;

	/* Move the painter origin to miss the key marker, and then	*/
	/* display the text as normal.					*/
	painter->translate  (20, 0) ;
	QListBoxText::paint (painter) ;
}

/*  ------------------------------------------------------------------  */

/*  KBTableAlias							*/
/*  KBTableAlias: Constructor for table alias/join object		*/
/*  qryDlg	: KBQueryDlg *	: Owning query dialog			*/
/*  table	: KBTable *	: Real table node			*/
/*  (returns)	: KBTableAlias	:					*/

KBTableAlias::KBTableAlias
	(	KBQueryDlg	*qryDlg,
		KBTable		*table
	)
	:
	QWidget		(qryDlg->getQrySpace(), 0, WStyle_Customize|WStyle_NormalBorder|WStyle_Title),
	m_qryDlg	(qryDlg),
	m_table		(table)
{
	RKVBox	*layMain = new RKVBox (this)	;
	layMain->setTracking () ;

	m_fieldList = new RKListBox (layMain)	;

	setLegend () ;

	m_fieldList->setSelectionMode (RKListBox::Single) ;

	QRect	tg	= table->geometry ()  	;
	if (tg.width()  == 0) tg.setWidth (140) ;
	if (tg.height() == 0) tg.setHeight(180) ;
	setGeometry	 (tg) ;


	/* Populate the list box with column names. The key column, if	*/
	/* any, is marked with a key icon. Note that the column is	*/
	/* picked up from the table object definition. This works	*/
	/* since when a table is added, the uniqueness will have been	*/
	/* set at that point; thereafter the user can change it.	*/
	QStringList		uNames	;
	QString			uExpr	;
	KBTableSpec		tabSpec	( m_table->getTable  ()) ;
	m_table->getUnique	(uNames, uExpr);

	if (!qryDlg->getDBLink().listFields (tabSpec))
	{
		qryDlg->getDBLink().lastError().DISPLAY() ;
		return	;
	}

	for (uint idx = 0 ; idx < tabSpec.m_fldList.count() ; idx += 1)
	{
		KBFieldSpec	*fSpec	= tabSpec.m_fldList.at(idx) ;
		bool		isPKey	= fSpec->m_name == uNames[0] ;

		new KBTableListBoxItem (m_fieldList, fSpec->m_name, isPKey) ;

		if (isPKey) m_keyField = fSpec->m_name ;
	}

	/* Connect the listbox scrolling signal to the main dialog. It	*/
	/* will use this to redraw the links. Also pick up mouse	*/
	/* mouse presses, which lead to context menus.			*/
	connect
	(	m_fieldList,
		SIGNAL	(contentsMoving (int, int)),
		m_qryDlg,
		SLOT  	(repaintLinks   ())
	)	;
	connect
	(	m_fieldList,
		SIGNAL	(mouseButtonPressed (int, QListBoxItem *, const QPoint &)),
		SLOT  	(mouseButtonPressed (int, QListBoxItem *, const QPoint &))
	)	;
	connect
	(	m_fieldList,
		SIGNAL	(contextMenuRequested(QListBoxItem *, const QPoint &)),
		SLOT	(contextMenuRequested(QListBoxItem *, const QPoint &))
	)	;

	m_linkParent = 0 ;

	/* The following order is important, so that this widget is	*/
	/* reparented by the workspace before we install the event	*/
	/* filter.							*/
	show	() ;
	qApp->processEvents () ;
	parentWidget()->installEventFilter (this) ;
}

/*  KBTableAlias							*/
/*  eventFilter	: Filter events						*/
/*  o		: QObject *	: Target object				*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBTableAlias::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (e->type() == QEvent::Move)
	{
		DPRINTF
		((	"KBTableAlias::event: move  : %s->%s\n",
			_TEXT(((QMoveEvent *)e)->oldPos()),
			_TEXT(((QMoveEvent *)e)->pos   ())	
		))	;

		m_qryDlg->repaintLinks () ;
		m_qryDlg->setChanged   () ;
		return	false	;
	}

	if (e->type() == QEvent::Resize)
	{
		DPRINTF
		((	"KBTableAlias::event: resize: %s->%s\n",
			_TEXT(((QResizeEvent *)e)->oldSize()),
			_TEXT(((QResizeEvent *)e)->size   ())	
		))	;

		m_qryDlg->repaintLinks () ;
		m_qryDlg->setChanged   () ;
		return	false	;
	}

	return	false	;
}


void	KBTableAlias::mouseButtonPressed
	(	int		button,
		QListBoxItem	*item,
		const QPoint	&
	)
{
	if (button == Qt::LeftButton)
	{
		if (item != 0)
			m_qryDlg->startLinking (this, item->text()) ;
		return	;
	}
}

void	KBTableAlias::contextMenuRequested
	(	QListBoxItem	*,
		const QPoint	&
	)
{
	m_qryDlg->showContextMenu (this) ;
}

/*  KBTableAlias							*/
/*  setLegend	: Set legend						*/
/*  (returns)	: void		  :					*/

void	KBTableAlias::setLegend ()
{
	QString	caption	;

	if (m_table->getAlias().isEmpty())
		caption	= m_table->getTable() ;
	else	caption	= QString("%1 as %2")
				 .arg(m_table->getTable())
				 .arg(m_table->getAlias()) ;

	setCaption 	  (caption) ;
}

/*  KBTableAlias							*/
/*  setAlias	: Set table alias					*/
/*  alias	: const QString & : Alias				*/
/*  (returns)	: void		  :					*/

void	KBTableAlias::setAlias
	(	const QString	&alias
	)
{
	m_table->setAlias (alias) ;
	setLegend	  ()	  ;
}

/*  KBTableAlias							*/
/*  setPrimary	: Set table primary field				*/
/*  primary	: const QString & : Primary				*/
/*  utype	: UniqueType	  : Uniqueness type			*/
/*  (returns)	: void		  :					*/

void	KBTableAlias::setPrimary
	(	const QString		&primary,
		KBTable::UniqueType	utype
	)
{
	m_keyField = primary ;

	for (uint idx = 0 ; idx < m_fieldList->count() ; idx += 1)
	{
		KBTableListBoxItem *tlbi = (KBTableListBoxItem *)m_fieldList->item(idx) ;
		tlbi->setPKey (tlbi->text() == primary) ;
	}

	m_table->setPrimary (primary, utype) ;
	m_fieldList->triggerUpdate (true) ;
}

/*  KBTableAlias							*/
/*  reparent	: Handle reparenting					*/
/*  parent	: QWidget *	 : New parent				*/
/*  flags	: WFlags	 : New flags				*/
/*  p		: const QPoint & : New position				*/
/*  showIt	: bool		 : Show after reparenting		*/
/*  (returns)	: void		 :					*/

void	KBTableAlias::reparent
	(	QWidget		*parent,
		WFlags		flags,
		const QPoint	&p,
		bool		showIt
	)
{
	QWidget::reparent (parent, flags, p, showIt) ;
//	parentWidget()->installEventFilter (this) ;
//	parent->installEventFilter (m_qryDlg) ;
}

/*  KBTableAlias							*/
/*  getPosition	: Get position for specified field			*/
/*  field	: const QString & : Field name				*/
/*  onright	: bool		  : True to return RHS position		*/
/*  ispk	: bool &	  : Field is preferred key		*/
/*  (returns)	: QPoint	  : Position				*/

QPoint	KBTableAlias::getPosition
	(	const QString	&field,
		bool		onright,
		bool		&ispk
	)
{
	int	slot	= -1 ;
	QPoint	pp	;
	QRect	fr	= m_fieldList->geometry () ;

	/* Scan through the list box to locate the specified field. It	*/
	/* really should be there ...					*/
	for (uint idx = 0 ; idx < m_fieldList->count() ; idx += 1)
		if (m_fieldList->text(idx) == field)
		{	KBTableListBoxItem *tlbi = (KBTableListBoxItem *)m_fieldList->item(idx) ;
			slot	= idx ;
			ispk	= tlbi->isPkey() ;
			break	;
		}

	/* ... but if not then return the origin.			*/
	if	(slot < 0)
	{
		pp	= QPoint (0, 0) ;
		ispk	= false		;
	}
	/* If found but not visible then return either the top or the	*/
	/* bottom of the list box, depending on which way it is hidden.	*/
	else if (!m_fieldList->itemVisible(slot))
	{
		pp	= QPoint
			  (	0,
				slot < m_fieldList->topItem() ? 0 : fr.height()
			  )	;
	}
	/* If visible then return the position mid-way down the area	*/
	/* that it occupies.						*/
	else
	{
		QRect	ir = m_fieldList->itemRect (m_fieldList->item(slot)) ;
		int	y  = ir.top() + ir.height()/2 ;

		if (y > m_fieldList->height()) y = m_fieldList->height() ;
		if (y < 0		    ) y = 0 ;

		pp	= QPoint (0, y) ;
	}

	/* The result is the position in the workspace coordinates, so	*/
	/* offset for the widget than the workspace wraps with.		*/
	QRect	lbr	= QRect
			  (	parentWidget()->x() + x(),
			        parentWidget()->y() + y(),
				m_fieldList->width (),
				m_fieldList->height()
			  )	;

	if (onright)
		pp	= QPoint (lbr.right(), pp.y() + lbr.top()) ;
	else	pp	= QPoint (lbr.left (), pp.y() + lbr.top()) ;

	return	pp	;			
}

/*  KBTableAlias							*/
/*  hit		: See if position hits a list box item			*/
/*  p		: QPoint	: Position in global coordinates	*/
/*  field	: QString &	: Field name if hit			*/
/*  (returns)	: bool		: Hit					*/

bool	KBTableAlias::hit
	(	QPoint		p,
		QString		&field
	)
{
	QListBoxItem	*item	;

	if ((item = m_fieldList->itemAt (m_fieldList->mapFromGlobal(p))) != 0)
	{	field	= item->text () ;
		return	true	;
	}

	return	false	;
}

/*  KBTableAlias							*/
/*  geometry	: Get geometry of workspace wrapping widget		*/
/*  (returns)	: QRect		: Geometry				*/

QRect	KBTableAlias::geometry ()
{
	return	parentWidget()->geometry() ;
}

/*  KBTableAlias							*/
/*  getTable	: Get underlying table					*/
/*  (returns)	: KBTable *	: Table					*/

KBTable	*KBTableAlias::getTable ()
{
	return	m_table	;
}

/*  KBTableAlias							*/
/*  getKeyField	: Get key field for table				*/
/*  (returns)	: const QString & : Key field				*/

const QString
	&KBTableAlias::getKeyField ()
{
	return	m_keyField ;
}

void	KBTableAlias::setLinkArea
	(	const QRect	&linkArea,
		KBTableAlias	*linkParent
	)
{
	m_linkArea   = linkArea   ;
	m_linkParent = linkParent ;
}

KBTableAlias
	*KBTableAlias::linkAreaHit
	(	const QPoint	&mp,
		int		&best
	)
{
	int	ml	= (m_linkArea.center() - mp).manhattanLength() ;
	if (m_linkArea.contains(mp) && (ml < best))
	{
		best	= ml ;
		return	m_linkParent ;
	}
	return	0 ;
}

const QRect
	&KBTableAlias::linkArea ()
{
	return	m_linkArea ;
}

