/***************************************************************************
    file	         : kb_tablealias.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_TABLEALIAS_H
#define	__KB_TABLEALIAS_H


class	KBTableListBox	;
class	KBTableAlias	;
class	KBQueryDlg	;
class	KBTable		;

#include	"kb_classes.h"
#include	"kb_database.h"
#include	"kb_node.h"
#include	"kb_table.h"
#include	"kb_dialog.h"

#include	"rk_listbox.h"


/*  KBTableListBoxItem							*/
/*  ------------------							*/
/*  The list of fields is displayed with a marker for the preferred	*/
/*  key. This class is used to handle this.				*/

class	KBTableListBoxItem : public QListBoxText
{
	bool	m_pKey	;

protected :

	virtual	void	paint	(QPainter *) ;

public	:

	inline	KBTableListBoxItem
		(	RKListBox	*lbox,
			const QString	&text,
			bool		pKey
		)
		:
		QListBoxText	(lbox, text),
		m_pKey		(pKey)
	{
	}

	inline	bool	isPkey	()
	{
		return	m_pKey	;
	}
	inline	void	setPKey
		(	bool	pKey
		)
	{
		m_pKey	= pKey	;
	}
}	;

/*  KBTableAlias							*/
/*  ------------							*/
/*  This class is used for the items in the tables list. Basically it	*/
/*  is a list box with various extra information.			*/

class	KBTableAlias : public QWidget
{
	Q_OBJECT

	RKListBox	*m_fieldList	;
	KBQueryDlg	*m_qryDlg	;
	KBTable		*m_table	;
	QString		m_keyField	;

	QRect		m_linkArea	;
	KBTableAlias	*m_linkParent	;

	void		setLegend	() ;

	virtual	bool	eventFilter	(QObject *, QEvent *)	;

protected slots :

	void	mouseButtonPressed	(int, QListBoxItem *, const QPoint &) ;
	void	contextMenuRequested	(     QListBoxItem *, const QPoint &) ;

public	:

	KBTableAlias (KBQueryDlg *, KBTable *) ;

	virtual	void	reparent	(QWidget  *, WFlags, const QPoint &, bool) ;

	QRect		geometry	() ;
	void		setAlias	(const QString &) ;
	void		setPrimary	(const QString &, KBTable::UniqueType) ;

	QPoint		getPosition	(const QString &, bool, bool &) ;
	bool		hit		(QPoint, QString &)	;

	KBTable 	*getTable 	()	;
	const QString	&getKeyField	()	;

	void		setLinkArea	(const QRect  &, KBTableAlias *) ;
	KBTableAlias	*linkAreaHit	(const QPoint &, int &) ;
	const QRect	&linkArea	()	;
}	;


#define	C_TABLE		0
#define	C_ALIAS		1
#define	C_PRIMARY	2
#define	C_FIELD		3
#define	C_FIELD2	4

#endif	// __KB_TABLEALIAS_H
