/***************************************************************************
    file	         : kb_scriptall.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qlistview.h>
#include	<qsplitter.h>
#include	<qgroupbox.h>
#include	<qregexp.h>
#include	<qtoolbutton.h>

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_checkbox.h"
#include	"rk_pushbutton.h"
#include	"rk_tabwidget.h"
#include	"tk_icons.h"
#include	"tk_messagebox.h"
#include	"tk_config.h"

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_form.h"
#include	"kb_report.h"
#include	"kb_docroot.h"
#include	"kb_layout.h"
#include	"kb_component.h"
#include	"kb_sidepanel.h"

#include	"kb_event.h"
#include	"kb_eventdlg.h"
#include	"kb_slot.h"
#include	"kb_slotdlg.h"

#include	"kb_options.h"
#include	"kb_toolbox.h"


#if		__KB_AUTOMOC
#include	"kb_scriptall.moc"
#else
#include	"kb_scriptall.h"
#endif



/*  KBScriptAllEditor							*/
/*  KBScriptAllEditor							*/
/*		: Constructor for editor wrapper			*/
/*  tabber	: RKTabWidget *		: Parent tabber			*/
/*  item	: KBScriptAllItem *	: Associated script item	*/
/*  language	: const QString &	: Main scripting language	*/
/*  language2	: const QString &	: Second scripting language	*/
/*  (returns)	: KBEventBaseDlgWrap	:				*/

KBScriptAllEditor::KBScriptAllEditor
	(	RKTabWidget	*tabber,
		KBScriptAllItem	*item,
		const QString	&language,
		const QString	&language2
	)
	:
	RKVBox		(tabber),
	m_tabber	(tabber),
	m_item		(item)	
{
	KBEvent	*event	= m_item->event() ;
	KBSlot	*slot	= m_item->slot () ;

	setMargin (5)	;

	m_eventDlg	= 0	;
	m_slotDlg	= 0	;
	m_changed	= false	;

	if	(event != 0)
	{
		m_eventDlg	= new KBEventBaseDlg
					(	this,
						m_item->node()->getDocRoot()->getDocLocation(),
						language,
						language2,
						QString::null,
						QString::null,
						"standard",
						(event->getFlags() & KAF_EVCS) != 0
					)	;

		m_eventDlg->setEventNode (item ->node ()) ;
		m_eventDlg->init	 (event->getValue(), event->getValue2(), "eventFunc") ;

		connect
		(	m_eventDlg,
			SIGNAL	(textChanged	()),
			SLOT	(scriptChanged	())
		)	;
	}
	else if (slot  != 0)
	{
		m_slotDlg	= new KBSlotBaseDlg (this, slot, item->node()) ;

		connect
		(	m_slotDlg,
			SIGNAL	(slotChanged	()),
			SLOT	(scriptChanged	())
		)	;
	}
	else	return	;

	m_tabber  ->insertTab	(this, legend()) ;
	m_tabber  ->showPage	(this) ;
	m_item    ->setEditor	(this) ;
	m_changed = false ;

	show () ;
}

/*  KBScriptAllEditor							*/
/*  legend	: Get legend for tab					*/
/*  suffix	: const QString & : Suffix to add to legend		*/
/*  (returns)	: QString	  : Suffix				*/

QString	KBScriptAllEditor::legend
	(	const QString	&suffix
	)
{
	KBEvent	*event	= m_item->event() ;
	KBSlot	*slot	= m_item->slot () ;

	if (event != 0)
		return	m_item->node()->getName() + "." + event->getName() + suffix ;

	if (slot  != 0)
		return	m_item->node()->getName() + "." + slot  ->name  () + suffix ;

	return	QString::null	;
}

/*  KBScriptAllEditor							*/
/*  scriptChanged: User makes a change					*/
/*  (returns)	 : void		:					*/

void	KBScriptAllEditor::scriptChanged ()
{
	if (!m_changed)
	{
		m_tabber->setTabLabel (this, legend("*")) ;
		m_changed = true ;
		return	;		
	}
}

/*  KBScriptAllEditor							*/
/*  changed	: Return changed state					*/
/*  (returns)	: bool		: True if changed			*/

bool	KBScriptAllEditor::changed ()
{
	return	m_changed ;
}

/*  KBScriptAllEditor							*/
/*  save	: Save if changed					*/
/*  (returns)	: void		:					*/

void	KBScriptAllEditor::save ()
{
	if (m_changed)
	{
		if (m_eventDlg != 0)
		{
			m_tabber->setTabLabel (this, legend()) ;
			m_item->event()->setValue (m_eventDlg->value ()) ;
			m_item->event()->setValue2(m_eventDlg->value2()) ;
		}

		if (m_slotDlg  != 0)
		{
			if (!m_slotDlg->doOK ())
				return	;
			m_tabber->setTabLabel (this, legend()) ;
		}

		m_changed = false ;
		return	;
	}
}

/*  KBScriptAllEditor							*/
/*  item	: Get associated script list item			*/
/*  (returns)	: KBScriptAllItem *	: List item			*/

KBScriptAllItem
	*KBScriptAllEditor::item ()
{
	return	m_item	;
}

/*  ------------------------------------------------------------------	*/

/*  KBScriptAllItem							*/
/*  KBScriptAllItem							*/
/*		: Constructor for node entry				*/
/*  dialog	: KBScriptAllDlg * : Parent dialog			*/
/*  parent	: QListView *	   : Parent list view			*/
/*  node	: KBNode *	   : Node containing script		*/
/*  (returns)	: KBScriptAllItem :					*/

KBScriptAllItem::KBScriptAllItem
	(	KBScriptAllDlg	*dialog,
		QListView	*parent,
		KBNode		*node
	)
	:
	QListViewItem	(parent, node->getName(), node->getElement().mid(2)),
	m_dialog	(dialog)
{
	m_node		= node	;
	m_event		= 0	;
	m_slot		= 0	;
	m_editor	= 0	;
	m_matched	= false	;

	setExpandable (true)	;
}

/*  KBScriptAllItem							*/
/*  KBScriptAllItem							*/
/*		: Constructor for node entry				*/
/*  dialog	: KBScriptAllDlg * : Parent dialog			*/
/*  parent	: QListViewItem *  : Parent list view item		*/
/*  after	: QListViewItem *  : Insert after item			*/
/*  node	: KBNode *	   : Node containing script		*/
/*  (returns)	: KBScriptAllItem :					*/

KBScriptAllItem::KBScriptAllItem
	(	KBScriptAllDlg	*dialog,
		QListViewItem	*parent,
		QListViewItem	*after,
		KBNode		*node
	)
	:
	QListViewItem	(parent, after, node->getName(), node->getElement().mid(2)),
	m_dialog	(dialog)
{
	m_node		= node	;
	m_event		= 0	;
	m_slot		= 0	;
	m_editor	= 0	;
	m_matched	= false	;

	setExpandable (true)	;
}

/*  KBScriptAllItem							*/
/*  KBScriptAllItem							*/
/*		: Constructor for event	entry				*/
/*  dialog	: KBScriptAllDlg * : Parent dialog			*/
/*  parent	: QListViewItem *  : Parent list view item		*/
/*  after	: QListViewItem *  : Insert after item			*/
/*  node	: KBNode *	   : Node containing script		*/
/*  event	: KEvent *	   : Event containing script		*/
/*  (returns)	: KBScriptAllItem :					*/

KBScriptAllItem::KBScriptAllItem
	(	KBScriptAllDlg	*dialog,
		QListViewItem	*parent,
		QListViewItem	*after,
		KBNode		*node,
		KBEvent		*event
	)
	:
	QListViewItem	(parent, after, event->getName(), TR("Event")),
	m_dialog	(dialog)
{
	m_node		= node	;
	m_event		= event	;
	m_slot		= 0	;
	m_editor	= 0	;
	m_matched	= false	;

	setExpandable (false)	;
}

/*  KBScriptAllItem							*/
/*  KBScriptAllItem							*/
/*		: Constructor for slot entry				*/
/*  dialog	: KBScriptAllDlg * : Parent dialog			*/
/*  parent	: QListViewItem *  : Parent list view			*/
/*  after	: QListViewItem *  : Insert after item			*/
/*  node	: KBNode *	   : Node containing script		*/
/*  slot	: KBSlot *	   : Slot containing script		*/
/*  (returns)	: KBScriptAllItem :					*/

KBScriptAllItem::KBScriptAllItem
	(	KBScriptAllDlg	*dialog,
		QListViewItem	*parent,
		QListViewItem	*after,
		KBNode		*node,
		KBSlot		*slot
	)
	:
	QListViewItem	(parent, after, slot->name(), TR("Slot")),
	m_dialog	(dialog)
{
	m_node		= node	;
	m_event		= 0	;
	m_slot		= slot	;
	m_editor	= 0	;
	m_matched	= false	;

	setExpandable (false)	;
}

/*  KBScriptAllItem							*/
/*  setOpen	: Open or close the branch				*/
/*  open	: bool		: True to open				*/
/*  (returns)	: void		:					*/

void	KBScriptAllItem::setOpen
	(	bool		open
	)
{
	if (open && (m_event == 0) && (m_slot == 0))
		if (firstChild() == 0)
			m_dialog->loadScripts (this) ;

	QListViewItem::setOpen (open) ;
}

void	KBScriptAllItem::paintCell
	(	QPainter	  *p,
		const QColorGroup &cg1,
		int		  column,
		int		  width,
		int		  align
	)
{
	if (m_matched)
	{
		QColorGroup cg2 (cg1) ;
		cg2.setColor (QColorGroup::Text, Qt::red) ;
		QListViewItem::paintCell (p, cg2, column, width, align) ;
	}
	else	QListViewItem::paintCell (p, cg1, column, width, align) ;
}

void	KBScriptAllItem::setMatched
	(	bool	matched
	)
{
	m_matched = matched ;
	repaint	()	    ;
}


/*  ------------------------------------------------------------------	*/

/*  KBScriptAllDlg							*/
/*  KBScriptAllDlg							*/
/*		: Constructor for script search dialog			*/
/*  root	: KBObject *	  : Root object				*/
/*  (returns)	: KBScriptAllDlg :					*/

KBScriptAllDlg::KBScriptAllDlg
	(	KBObject	*root
	)
	:
	KBDialog (TR("Scripts"), true, "scriptall")
{
	RKVBox	*layTop	  = new RKVBox	  (this) ;
	layTop->setTracking ()	;

	RKHBox	*layMain  = new RKHBox	  (layTop) ;
	new KBSidePanel (layMain, TR("Script Display"));

	m_splitter	  = new QSplitter (QSplitter::Horizontal, layMain) ;
	RKVBox	*layLeft  = new RKVBox    (m_splitter) ;

	m_scriptList	  = new QListView (layLeft)    ;
	m_scriptList->addColumn		(TR("Name"  )) ;
	m_scriptList->addColumn		(TR("Type"  )) ;
	m_scriptList->setColumnWidthMode(0, QListView::Maximum) ;
	m_scriptList->setRootIsDecorated(true) ;
	m_scriptList->setSorting	(-1  ) ;
	m_scriptList->setResizeMode	(QListView::LastColumn) ;

	QGroupBox *layGrid= new QGroupBox  (2, Qt::Horizontal, TR("Search for"), layLeft) ;

	new QLabel (TR("Search for"),  layGrid) ; 
	m_usePattern	  = new RKLineEdit(layGrid) ;
	m_useRegexp	  = new RKCheckBox(layGrid) ;
	m_useRegexp->setText (TR("Reg.Exp.")) ;
	RKPushButton *s	  = new RKPushButton (TR("Search"), layGrid) ;
	connect (s, SIGNAL(clicked()), SLOT(slotSearch())) ;

	layLeft->setStretchFactor (m_scriptList, 1) ;

	m_tabber = new RKTabWidget   (m_splitter) ;
	connect
	(	m_tabber,
		SIGNAL	(currentChanged (QWidget *)),
		SLOT	(slotPageChanged(QWidget *))
	)	;

	QToolButton *ct = new QToolButton (m_tabber) ;
	ct->setIconSet  (getSmallIcon ("fileclose")) ;
	ct->hide	() ;
	m_tabber->setCornerWidget (ct) ;
	connect	(ct, SIGNAL(clicked()), SLOT(slotCloseTab())) ;

	RKHBox	  *layButt = new RKHBox	(layTop)  ;
	layButt->addFiller ()	;

	m_edit			= new RKPushButton (TR("Edit"	 ), layButt) ;
	m_save			= new RKPushButton (TR("Save"  	 ), layButt) ;
	m_saveAll		= new RKPushButton (TR("Save All"), layButt) ;
	RKPushButton	*finish	= new RKPushButton (TR("Finish"	 ), layButt) ;

	connect (m_edit,	SIGNAL(clicked()), SLOT(slotEdit   ())) ;
	connect (m_save,	SIGNAL(clicked()), SLOT(slotSave   ())) ;
	connect (m_saveAll,	SIGNAL(clicked()), SLOT(slotSaveAll())) ;
	connect (finish,	SIGNAL(clicked()), SLOT(reject     ())) ;

	KBDialog::sameSize (m_edit, m_save, m_saveAll, finish, 0) ;
	connect
	(	m_scriptList,
		SIGNAL	(doubleClicked	(QListViewItem *, const QPoint &, int)),
		SLOT	(slotEdit	())
	)	;
	connect
	(	m_scriptList,
		SIGNAL	(currentChanged (QListViewItem *)),
		SLOT	(slotChanged	())
	)	;

	m_edit->setEnabled (false ) ;

	layTop->setStretchFactor (layMain, 1) ;

	m_regexp	= 0	;
	m_root		= root	;
	m_language	= root->getRoot()->getAttrVal ("language" ) ;
	m_language2	= root->getRoot()->getAttrVal ("language2") ;

	new KBScriptAllItem  (this, m_scriptList, m_root) ;
}

/*  KBScriptAllDlg							*/
/*  ~KBScriptAllDlg							*/
/*		: Destructor for script searcher			*/
/*  (returns)	:		:					*/

KBScriptAllDlg::~KBScriptAllDlg ()
{
	TKConfig *config = TKConfig::getConfig() ;
	config->setGroup   ("scriptall") ;
	config->writeEntry ("splitter", m_splitter->sizes()) ;

	DELOBJ	(m_regexp) ;
}


void	KBScriptAllDlg::show ()
{
	KBDialog::show	() ;

	TKConfig *config = TKConfig::getConfig() ;
	config->setGroup   ("scriptall") ;

	QValueList<int> sizes = config->readIntListEntry ("splitter") ;
	if (sizes.count() < 2)
	{	sizes.clear() ;
		sizes.append (m_scriptList->columnWidth(0) + m_scriptList->columnWidth(1) + 40) ;
		sizes.append (m_splitter->width() - sizes[0]) ;
	}

	m_splitter->setSizes (sizes) ;
}

int	KBScriptAllDlg::exec ()
{
	bool	suspend	= KBOptions::getSuspendToolbox() ;

	if (suspend) KBToolBox::self()->suspendToolBox() ;
	bool	rc = KBDialog::exec () ;
	if (suspend) KBToolBox::self()->resumeToolBox () ;
	return	rc ;
}


/*  KBScriptAllDlg							*/
/*  save	: Save changes on specified page			*/
/*  widget	: QWidget *	: Page					*/
/*  (returns)	: void		:					*/

void	KBScriptAllDlg::save
	(	QWidget		*widget
	)
{
	if (widget->isA("KBScriptAllEditor"))
	{
		((KBScriptAllEditor *)widget)->save() ;
		m_root->getRoot()->isLayout()->setChanged() ;
	}
}

/*  KBScriptAllDlg							*/
/*  slotSave	: Save changes on current page				*/
/*  (returns)	: void		:					*/

void	KBScriptAllDlg::slotSave ()
{
	save (m_tabber->currentPage()) ;
}

/*  KBScriptAllDlg							*/
/*  slotSaveAll	: Save all changes					*/
/*  (returns)	: void		:					*/

void	KBScriptAllDlg::slotSaveAll ()
{
	for (int idx = 0 ; idx < m_tabber->count() ; idx += 1)
		save (m_tabber->page (idx)) ;
}

/*  KBScriptAllDlg							*/
/*  loadScripts	: Scan for scripts and child nodes			*/
/*  item	: KBScriptAllItem * : Item from which to load		*/
/*  (returns)	: void		    :					*/

void	KBScriptAllDlg::loadScripts
	(	KBScriptAllItem		*item
	)
{
	QListViewItem	*last	= 0 ;

	/* Scan through all attributes looking for those that are	*/
	/* events. For each such, look in both the first and second	*/
	/* language code. Subsequently scan slots, which only have one	*/
	/* chuck of code.						*/
	KBNode			*node	  = item->node	      () ;
	const QPtrList<KBAttr> 	&attrList = node->getAttribs  () ;
	LITER
	(	KBAttr,
		attrList,
		a,

		KBEvent *event ;
		if ((event = a->isEvent()) == 0) continue ;

		if (!event->getValue().isEmpty() || !event->getValue2().isEmpty())
			last	= new KBScriptAllItem (this, item, last, node, event) ;
	)

	const QPtrList<KBSlot>	&slotList = node->getSlots    () ;
	LITER
	(	KBSlot,
		slotList,
		slot,

		if (!slot->code().isEmpty())
			last	= new KBScriptAllItem (this, item, last, node, slot ) ;
	)

	/* Recurse down into any children, except that we do not go	*/
	/* below component links.					*/
	if (node->isCompLink() == 0)
	{
		const QPtrList<KBNode>	&children = node->getChildren () ;
		LITER
		(	KBNode,
			children,
			n,
			last = new KBScriptAllItem  (this, item, last, n) ;
		)
	}
}

/*  KBScriptAllDlg							*/
/*  matchScript	: Check if script matches				*/
/*  code	: const QString & : Script				*/
/*  (returns)	: bool		  : Match found				*/

bool	KBScriptAllDlg::matchScript
	(	const QString	&code
	)
{
	/* Empty code is never matched; while an empty search pattern	*/
	/* always maches, so we can find all non-empty scripts.		*/
	if (code.isEmpty())
		return	false	;

	if (m_usePattern->text().isEmpty())
		return	true	;

	if (m_regexp == 0)
		return	code.find (m_usePattern->text(), 0, false) >= 0 ;

	return	code.find (*m_regexp) >= 0  ;
}

/*  KBScriptAllDlg							*/
/*  scanScripts	: Scan scripts from item				*/
/*  item	: KBScriptAllItem * : Item in question			*/
/*  (returns)	: void		    :					*/

void	KBScriptAllDlg::scanScripts
	(	KBScriptAllItem		*item
	)
{
	item->setOpen (true) ;

	for
	(	QListViewItem *c = item->firstChild() ;
		c != 0 ;
		c  = c->nextSibling()
	)
	{
		KBScriptAllItem	*child	= (KBScriptAllItem *)c	;
		scanScripts     (child) ;

		KBEvent		*event	= child->event ()	;
		KBSlot		*slot	= child->slot  ()	;

		if (event != 0)
		{
			if (matchScript(event->getValue ()))
			{
				child->setMatched (true) ;
				continue ;
			}
			if (matchScript(event->getValue2()))
			{
				child->setMatched (true) ;
				continue ;
			}
		}

		if (slot  != 0)
		{
			if (matchScript(slot->code()))
			{
				child->setMatched (true) ;
				continue ;
			}
		}

		child->setMatched (false) ;
	}
}

/*  KBScriptAllDlg							*/
/*  slotPageChanged							*/
/*		: Tab page has changed					*/
/*  widget	: QWidget *	: Newly current page			*/
/*  (returns)	: void		:					*/

void	KBScriptAllDlg::slotPageChanged
	(	QWidget		*widget
	)
{
	if (widget != 0)
		if (widget->isA("KBScriptAllEditor"))
		{
			KBScriptAllItem *item = ((KBScriptAllEditor *)widget)->item() ;
			m_scriptList->setCurrentItem    (item) ;
			m_scriptList->ensureItemVisible (item) ;
		}
}

/*  KBScriptAllDlg							*/
/*  slotChanged	: Selection changed					*/
/*  (returns)	: void		:					*/

void	KBScriptAllDlg::slotChanged ()
{
	m_edit->setEnabled (m_scriptList->currentItem() != 0) ;
}

/*  KBScriptAllDlg							*/
/*  slotSearch	: Search through scripts for specified text		*/
/*  (returns)	: void		:					*/

void	KBScriptAllDlg::slotSearch ()
{
	DELOBJ (m_regexp)	;
	if (m_useRegexp->isChecked())
		m_regexp = new QRegExp (m_usePattern->text(), false) ;

	scanScripts ((KBScriptAllItem *)m_scriptList->firstChild()) ;
}

/*  KBScriptAllDlg							*/
/*  slotEdit	: Edit selected event or slot				*/
/*  (returns)	: void		:					*/

void	KBScriptAllDlg::slotEdit ()
{
	KBScriptAllItem *si = (KBScriptAllItem *)m_scriptList->currentItem() ;
	if (si  == 0) return ;

	KBObject	*obj = si->node()->isObject() ;
	if (obj == 0) return ;

	if (si->editor() != 0)
	{
		m_tabber->showPage (si->editor()) ;
		return	;
	}

	if (si->event() != 0)
	{
		new KBScriptAllEditor (m_tabber, si, m_language, m_language2) ;
		m_tabber->cornerWidget()->show() ;
		return	;
	}

	if (si->slot () != 0)
	{
		new KBScriptAllEditor (m_tabber, si, m_language, m_language2) ;
		m_tabber->cornerWidget()->show() ;
		return	;
	}
}

/*  KBScriptAllDlg							*/
/*  slotCloseTab: Close current tab					*/
/*  (returns)	: void		:					*/

void	KBScriptAllDlg::slotCloseTab ()
{
	QWidget	*widget	= m_tabber->currentPage() ;
	if ((widget == 0) || ! widget->isA("KBScriptAllEditor"))
		return	;

	KBScriptAllEditor *editor = (KBScriptAllEditor *)widget ;
	if (editor->changed())
		switch (TKMessageBox::questionYesNoCancel
				(	this,
					TR("Event or slot has changed"),
					QString::null,
					TR("Save changes"),
					TR("Close anyway")
				))
		{
			case TKMessageBox::Ok	:
				editor->save	() ;
				break	;	;

			case TKMessageBox::No	:
				break	;

			default	:
				return	;
		}

	editor->item()->setEditor(0)	;
	delete	editor	;	

	if (m_tabber->count() == 0)
		m_tabber->cornerWidget()->hide() ;
}

/*  ------------------------------------------------------------------	*/

/*  KBScriptAll								*/
/*  KBScriptAll: Constructor for script search object			*/
/*  (returns)	: KBScriptAll	:					*/

KBScriptAll::KBScriptAll ()
{
}

/*  KBScriptAll								*/
/*  execute	: Execute searcher					*/
/*  info	: KBPluginActionInfo &	: General plugin information	*/
/*  (returns)	: bool			: Success			*/

bool	KBScriptAll::execute
	(	KBPluginActionInfo	&info
	)
{
	fprintf
	(	stderr,
		"KBScriptAll::execute: %s: called: form=%p\n",
		info.m_location.ident().latin1(),
		info.m_form
	)	;

	KBScriptAllDlg *dlg = 0 ;

	if	(info.m_form	  != 0) dlg = new KBScriptAllDlg (info.m_form     ) ;
	else if	(info.m_report	  != 0) dlg = new KBScriptAllDlg (info.m_report   ) ;
	else if	(info.m_component != 0) dlg = new KBScriptAllDlg (info.m_component) ;

	if (dlg != 0)
	{
		dlg->exec () ;
		delete	dlg  ;
	}

	return	true	;
}


/*  ------------------------------------------------------------------	*/


KBFACTORYIMPL
(	KBScriptAllFactory,
	plugin_scriptall
)


QObject	*KBScriptAllFactory::create
	(	QObject		  *,
		cchar		  *,
		cchar		  *name,
		const QStringList &
	)
{
	fprintf
	(	stderr,
		"KBScriptAllFactory::create: %s: CALLED\n",
		name	
	)	;

	return	new KBScriptAll ;
}

QString	KBScriptAllFactory::ident ()
{
	return	QString("ScriptDisplay: %1%2 %3")
			.arg(__KB_BUILD_RELEASE)
			.arg(__KB_BUILD_BUILD  )
			.arg(__KB_BUILD_TIME   )
			;
}
