/***************************************************************************
    file	         : kb_scriptall.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#ifndef	__KB_SCRIPTALL_H
#define	__KB_SCRIPTALL_H

#include 	"libscriptall_exports.h"

#include	"kb_libloader.h"
#include	"kb_pluginaction.h"
#include	"kb_dialog.h"

#include	"rk_vbox.h"


class	KBEvent		;
class	KBSlot		;
class	QListView	;
class	QSplitter	;
class	QRegExp		;
class	QPainter	;
class	QColorGroup	;

class	RKLineEdit	;
class	RKCheckBox	;
class	RKTabWidget	;

class	KBScriptAllItem	;
class	KBScriptAllDlg	;

class	KBEventBaseDlg	;
class	KBSlotBaseDlg	;


/*  KBScriptAllEditor							*/
/*  -----------------							*/
/*  Wrapper around KBEventBaseDlg/KBSlotBaseDlg, used so that we can	*/
/*  access the "KBScriptAllItem" item with which it is associated.	*/

class	LIBSCRIPTALL_API  KBScriptAllEditor : public RKVBox
{
	Q_OBJECT

	RKTabWidget		*m_tabber	;
	KBScriptAllItem		*m_item		;
	KBEventBaseDlg		*m_eventDlg	;
	KBSlotBaseDlg		*m_slotDlg	;
	bool			m_changed	;

	QString			legend		(const QString & = QString::null) ;

protected slots :

	void			scriptChanged	() ;

public	:

	KBScriptAllEditor
			(	RKTabWidget	*tabber,
				KBScriptAllItem	*item,
				const QString	&language,
				const QString	&language2
			)	;

	bool			changed	()	;
	void			save	()	;
	KBScriptAllItem		*item	()	;
}	;


/*  KBScriptAllItem							*/
/*  ---------------							*/
/*  Wrapper around QListViewItem for script list items. Contains	*/
/*  pointers to the associated node, and event or slot as appropriate.	*/
/*  When an editor is opened on an event or a slot then the "m_editor"	*/
/*  member is set.							*/

class	LIBSCRIPTALL_API KBScriptAllItem : public QListViewItem
{
	KBScriptAllDlg	*m_dialog	;
	KBNode		*m_node		;
	KBEvent		*m_event	;
	KBSlot		*m_slot		;
	QWidget		*m_editor	;
	bool		m_matched	;

public	:

	KBScriptAllItem	(KBScriptAllDlg *, QListView	 *, KBNode *) ;
	KBScriptAllItem	(KBScriptAllDlg *, QListViewItem *, QListViewItem *, KBNode *) ;
	KBScriptAllItem	(KBScriptAllDlg *, QListViewItem *, QListViewItem *, KBNode *, KBEvent *) ;
	KBScriptAllItem	(KBScriptAllDlg *, QListViewItem *, QListViewItem *, KBNode *, KBSlot  *) ;

	void		setMatched	(bool)	;
	virtual	void	paintCell	(QPainter *, const QColorGroup &, int, int, int) ;

	inline	KBNode	*node	()
	{
		return	m_node	;
	}
	inline	KBEvent	*event	()
	{
		return	m_event	;
	}
	inline	KBSlot	*slot	()
	{
		return	m_slot	;
	}
	inline	QWidget	*editor	()
	{
		return	m_editor ;
	}
	inline	void	setEditor
		(	QWidget	*editor
		)
	{
		m_editor = editor   ;
	}
	
	virtual	void	setOpen	(bool)	;
}	;

/*  KBScriptAllDlg							*/
/*  --------------							*/
/*  All-script tool dialog.						*/

class	LIBSCRIPTALL_API KBScriptAllDlg : public KBDialog
{
	Q_OBJECT

	KBObject	*m_root		;
	QSplitter	*m_splitter	;
	QListView	*m_scriptList	;
	RKTabWidget	*m_tabber	;

	RKLineEdit	*m_usePattern	;
	RKCheckBox	*m_useRegexp	;
	QRegExp		*m_regexp	;

	RKPushButton	*m_save		;
	RKPushButton	*m_saveAll	;
	RKPushButton	*m_edit		;
	QString		m_language	;
	QString		m_language2	;

	void		save		(QWidget	 *)	;
	bool		matchScript	(const QString   &)	;
	void		scanScripts	(KBScriptAllItem *)	;

protected slots	:

	void		slotSearch	()		;
	void		slotEdit	()		;
	void		slotCloseTab	()		;
	void		slotChanged	()		;
	void		slotSave	()		;
	void		slotSaveAll	()		;
	void		slotPageChanged	(QWidget *)	;

public	:

	KBScriptAllDlg	(KBObject *)	;
	~KBScriptAllDlg	()		;

	virtual	void	show		() ;
	int		exec		() ;
	void		loadScripts	(KBScriptAllItem *) ;
}	;


/*  ------------------------------------------------------------------	*/

KBFACTORY(KBScriptAllFactory,"scriptall")


class	LIBSCRIPTALL_API KBScriptAll : public KBPluginActionBase
{
public	:

	KBScriptAll	()	;

	virtual	bool		execute (KBPluginActionInfo &)	;
}	;


#endif	// __KB_SCRIPTALL_H
