/***************************************************************************
    file	         : rk_application.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"rk_application.moc"
#else
#include	"rk_application.h"
#endif

#include	"rk_dcop.h"


/*  RKNotifyFilter							*/
/*  RKNotifyFilter							*/
/*		: Constructor for notify filter helper			*/
/*  parent	: QObject *	 : Parent object			*/
/*  name	: const char *	 : Object name				*/
/*  (returns)	: RKNotifyFilter : Filter helper			*/

RKNotifyFilter::RKNotifyFilter
	(	QObject		*parent,
		const char	*name
	)
	:
	QObject	(parent, name)
{
}

/*  RKNotifyFilter							*/
/*  notifyFilter: Filter method						*/
/*  o		: QObject *	: Object to receive event		*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: True if event consumed		*/

bool	RKNotifyFilter::notifyFilter
	(	QObject		*,
		QEvent		*
	)
{
	/* Dummy. Derived classes will override this method as needed.	*/
	return	false	;
}

/*  ------------------------------------------------------------------	*/

static	RKApplication	*rkApp	;


/*  RKApplication							*/
/*  RKApplication							*/
/*		: Constructor for application class			*/
/*  (returns)	: RKApplication	:					*/

RKApplication::RKApplication
	(	int		argc,
		char		*argv[]
	)
	:
	QApplication	(argc, argv)
{
	/* Set global pointer. We should only ever be called once.	*/
	rkApp	= this	;
}

/*  RKApplication							*/
/*  self	: Return instance					*/
/*  (returns)	: RKApplication * : Instance				*/

RKApplication
	*RKApplication::self ()
{
	/* Note, this is *not* a factory function. The one and only	*/
	/* instance should already have been created.			*/
	return	rkApp	;
}

/*  RKApplication							*/
/*  installMousePressFilter						*/
/*		: Install filter for mouse press events			*/
/*  o		: QObject *	   : Object to be filtered		*/
/*  f		: RKNotifyFilter * : Filter handler object		*/
/*  (returns)	: void		   :					*/

void	RKApplication::installMousePressFilter
	(	QObject		*o,
		RKNotifyFilter	*f
	)
{
	/* Sanity check, then map the object to the filter handler.	*/
	/* Connect the destroyed signal so that we can automatically	*/
	/* clean up.							*/
	if (o != 0)
	{
		m_mousePressFilters.insert (o, f) ;

		connect
		(	o,
			SIGNAL	(destroyed	(QObject *)),
			SLOT	(slotDestroyed	(QObject *))
		)	;
	}
}

/*  RKApplication							*/
/*  removeMousePressFilter						*/
/*		: Remove filter for mouse press events			*/
/*  o		: QObject *	   : Object to be filtered		*/
/*  (returns)	: void		   :					*/

void	RKApplication::removeMousePressFilter
	(	QObject		*o
	)
{
	if (o != 0)
	{
		m_mousePressFilters.remove (o) ;

		disconnect
		(	o,
			SIGNAL	(destroyed	(QObject *)),
			this,
			SLOT	(slotDestroyed	(QObject *))
		)	;
	}
}

/*  RKApplication							*/
/*  slotDestroyed: Handle filtered object destruction			*/
/*  o		 : QObject *	: Object in question			*/
/*  (returns)	 : void		:					*/

void	RKApplication::slotDestroyed
	(	QObject		*o
	)
{
	m_mousePressFilters.remove (o) ;
}

/*  RKApplication							*/
/*  notify	: Notification handler					*/
/*  o		: QObject *	: Object to receive event		*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: True if event consumed		*/

bool	RKApplication::notify
	(	QObject		*o,
		QEvent		*e
	)
{
	switch (e->type())
	{
		/* Most button press is picked up so that we can stop	*/
		/* focus changes (when user tries to exit a field with	*/
		/* invalid data) for classes (like QSpinBox) where we	*/
		/* cannot work via ::setFocus(). We have to do this	*/
		/* in ::notify since ::setFocus() is called *before*	*/
		/* any application-level event filters are called.	*/
		case QEvent::MouseButtonPress :
			{
			RKNotifyFilter	*f = m_mousePressFilters[o] ;
			if (f != 0)
			{
				fprintf
				(	stderr,
					"RKApplication::notify: hit: [%s] %p\n",
					o->className(),
					o
				)	;

				if (f->notifyFilter (o, e))
					return	true	;
			}
			}
			break	;

		default	:
			break	;
	}

	return	QApplication::notify (o, e) ;
}
