/*  -*- C++ -*-
    This file is part of the KDE libraries
    Copyright (C) 1997 Tim D. Gilman (tdgilman@best.org)
              (C) 1998-2001 Mirko Boehm (mirko@kde.org)
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <qdatetime.h>
#include <qstring.h>
#include <qpen.h>
#include <qpainter.h>
#include <qdialog.h>
#include <qdict.h>
#include <assert.h>
#include <qapplication.h>
#include <qpopupmenu.h>

#if	 __KB_AUTOMOC
#include "rk_dategridview.moc"
#else
#include "rk_dategridview.h"
#endif

RKDateGridView::RKDateGridView( QWidget* parent, QDate aDate, const char* name, WFlags f )
	: QGridView( parent, name, f )
{
	m_PopupMenuEnabled = false;
	m_UseCustomColors = false;	
	setFontSize( 10 );
  
	if ( ! aDate.isValid())
    {
		aDate = QDate::currentDate();
    }
  
	setFocusPolicy( QWidget::StrongFocus );
	setNumRows( 7 ); 
	setNumCols( 7 ); 
	setHScrollBarMode( AlwaysOff );
	setVScrollBarMode( AlwaysOff );
	//viewport()->setEraseColor();
	setDate( aDate ); 
	//initAccels();
}

RKDateGridView::RKDateGridView( QWidget* parent, const char* name, WFlags f )
	: QGridView( parent, name, f )
{
	setFontSize( 10 );
	setFocusPolicy( QWidget::StrongFocus );
	setNumRows( 7 ); 
	setNumCols( 7 ); 
	setHScrollBarMode( AlwaysOff );
	setVScrollBarMode( AlwaysOff );
	//viewport()->setEraseColor(KGlobalSettings::baseColor());
	setDate( QDate::currentDate() ); 
	//initAccels();
}

RKDateGridView::~RKDateGridView()
{
}


int RKDateGridView::posFromDate( const QDate& aDate )
{
	//const KCalendarSystem* calendar = KGlobal::locale()->calendar();
	//const int firstWeekDay = KGlobal::locale()->weekStartDay();
	const int firstWeekDay = 1;
	//int pos = calendar->day( dt );
	int pos = aDate.day();
	int offset = ( m_FirstDay - firstWeekDay + 7 ) % 7;
	// make sure at least one day of the previous month is visible.
	// adjust this <1 if more days should be forced visible:
	if ( offset < 1 ) 
		offset += 7;
  
	return pos + offset;
}

QDate RKDateGridView::dateFromPos( int pos )
{
	QDate dateInCell( m_Date.year(), m_Date.month(), 1 );
	//const KCalendarSystem * calendar = KGlobal::locale()->calendar();
	//calendar->setYMD(pCellDate, calendar->year(date), calendar->month(date), 1);
	
	int firstWeekDay = 1;//KGlobal::locale()->weekStartDay();
	int offset = ( m_FirstDay - firstWeekDay + 7 ) % 7;
	// make sure at least one day of the previous month is visible.
	// adjust this <1 if more days should be forced visible:
	if ( offset < 1 ) 
		offset += 7;

	dateInCell = dateInCell.addDays( pos - offset );
	return dateInCell;
}

void RKDateGridView::paintCell( QPainter* pPainter, int row, int col )
{
  	QRect rect;
	QString text;
	QPen pen;
	int w = cellWidth();
	int h = cellHeight();
	QFont font = QApplication::font();
  

	if ( row == 0 )
    { 
		font.setBold( true );
		pPainter->setFont( font );
		bool normalDay = true;
		int firstWeekDay = 1;//KGlobal::locale()->weekStartDay();
		int dayNum = ( col + firstWeekDay < 8 ) ? col + firstWeekDay : col + firstWeekDay - 7;
      
		if ( dayNum == 6 || dayNum == 7 )
			normalDay = false;

		QBrush brushInvertTitle( colorGroup().base() );
		QColor titleColor( isEnabled() ? ( palette().active().highlight() ) : ( palette().inactive().highlight() ) );
		QColor textColor( isEnabled() ? ( palette().active().highlightedText() ) : ( palette().inactive().highlightedText() ) );
      
		if ( ! normalDay )
        {
			pPainter->setPen( textColor );
			pPainter->setBrush( textColor );
			pPainter->drawRect( 0, 0, w, h );
			pPainter->setPen( titleColor );
        } 
		else 
		{
			pPainter->setPen( titleColor );
			pPainter->setBrush( titleColor );
			pPainter->drawRect( 0, 0, w, h );
			pPainter->setPen( textColor );
        }
      
		pPainter->drawText( 0, 0, w, h - 1, AlignCenter, QDate::shortDayName( dayNum ), -1, &rect );
		pPainter->setPen( colorGroup().text() );
		pPainter->moveTo( 0, h - 1 );
		pPainter->lineTo( w - 1, h - 1 );
      
	} 
	else 
	{
		bool paintRect = true;
		pPainter->setFont( font );
		int pos = 7 * ( row - 1 ) + col;

		QDate cellDate = dateFromPos( pos );
		// First day of month
//		text = QDate::shortDayName( cellDate.day() );
		text = QString("%1").arg( cellDate.day() );
      
		if ( cellDate.month() != m_Date.month() )
        { 
			pPainter->setPen( colorGroup().mid() );
        } 
		else 
		{ 
			if ( m_UseCustomColors )
			{
				DatePaintingMode* mode = m_CustomPaintingModes[ cellDate.toString() ];
            
				if ( mode )
				{
					if ( mode->bgMode != NoBgMode )
					{
						QBrush oldBrush = pPainter->brush();
						pPainter->setBrush( mode->bgColor );
                
						switch ( mode->bgMode )
						{
							case( CircleMode ): 
								
								pPainter->drawEllipse( 0, 0, w, h);
								break;
							
							case( RectangleMode ): 
								
								pPainter->drawRect( 0, 0, w, h);
								break;

							case( NoBgMode ): 
							default: 
								
								break;
						}
                
						pPainter->setBrush( oldBrush );
						paintRect = false;
					}
              
					pPainter->setPen( mode->fgColor );
				} 
				else
					pPainter->setPen( colorGroup().text() );
			} 
			else 
				pPainter->setPen( colorGroup().text() );
        }

		pen = pPainter->pen();
      
		int firstWeekDay = 1;
		int offset = m_FirstDay - firstWeekDay;
      
		if ( offset < 1 )
			offset += 7;

		int d = m_Date.day();
           
		if ( ( offset + d ) == ( pos + 1 ) )
        {
			if ( isEnabled() )
			{
				pPainter->setPen( colorGroup().highlight());
				pPainter->setBrush(colorGroup().highlight());
			}
			else 
			{
				pPainter->setPen( colorGroup().text() );
				pPainter->setBrush( colorGroup().text() );
			}
           
			pen = colorGroup().highlightedText();
		} 
		else 
		{
			pPainter->setBrush( paletteBackgroundColor() );
			pPainter->setPen( paletteBackgroundColor() );
        }

		if ( cellDate == QDate::currentDate() )
		{
			pPainter->setPen( colorGroup().text() );
		}

		if ( paintRect ) 
			pPainter->drawRect( 0, 0, w, h );

		pPainter->setPen( pen );
		pPainter->drawText( 0, 0, w, h, AlignCenter, text, -1, &rect );
    }
  
	if ( rect.width() > m_MaxCell.width() ) 
		m_MaxCell.setWidth( rect.width() );
  
	if ( rect.height() > m_MaxCell.height() ) 
		m_MaxCell.setHeight( rect.height() );
}

void RKDateGridView::nextMonth()
{
	setDate( m_Date.addMonths( 1 ) );
}

void RKDateGridView::previousMonth()
{
	setDate( m_Date.addMonths( -1 ) );
}

void RKDateGridView::beginningOfMonth()
{
	setDate( m_Date.addDays( 1 - m_Date.day() ) );
}

void RKDateGridView::endOfMonth()
{
	setDate( m_Date.addDays( m_Date.daysInMonth() - m_Date.day() ) );
}

void RKDateGridView::beginningOfWeek()
{
	setDate( m_Date.addDays( 1 - m_Date.dayOfWeek() ) );
}

void RKDateGridView::endOfWeek()
{
	setDate( m_Date.addDays( 7 - m_Date.dayOfWeek() ) );
}    

void RKDateGridView::keyPressEvent( QKeyEvent* pEvent )
{
 	if ( pEvent->key() == Key_Up )
    	setDate( m_Date.addDays( -7 ) );
    
	if ( pEvent->key() == Key_Down )
		setDate( m_Date.addDays( 7 ) );

	if ( pEvent->key() == Key_Left )
		setDate( m_Date.addDays( -1 ) );
    
	if ( pEvent->key() ==  Key_Right )
		setDate( m_Date.addDays( 1 ) );

	if ( pEvent->key() ==  Key_Minus )
		setDate( m_Date.addDays( -1 ) );

	if ( pEvent->key() ==  Key_Plus )
		setDate( m_Date.addDays( 1 ) );
    
	if ( pEvent->key() ==  Key_N )
    	setDate( QDate::currentDate() );

	if ( pEvent->key() == Key_Home )
		beginningOfMonth();

	if ( pEvent->key() == Key_End )
		endOfMonth();
    
	if ( pEvent->key() == Key_Return || pEvent->key() == Key_Enter )
		emit gridClicked();

	if ( pEvent->key() == Key_N && ( pEvent->state() & ControlButton ) )
		nextMonth();

	if ( pEvent->key() == Key_P && ( pEvent->state() & ControlButton ) )
		previousMonth();

	if ( pEvent->key() == Key_Home && ( pEvent->state() & ControlButton ) )
		beginningOfWeek();

	if ( pEvent->key() == Key_End && ( pEvent->state() & ControlButton ) )
		endOfWeek();

	if ( ! pEvent->state() ) 
	{
		QApplication::beep();
	}
}

void RKDateGridView::viewportResizeEvent( QResizeEvent* pEvent )
{
	QGridView::viewportResizeEvent( pEvent );

	setCellWidth( viewport()->width() / 7 );
	setCellHeight( viewport()->height() / 7 );
}

void RKDateGridView::setFontSize( int size )
{
	int count;
	QFontMetrics metrics( fontMetrics() );
	QRect rect;

	m_FontSize = size;
  
	m_MaxCell.setWidth( 0 );
	m_MaxCell.setHeight( 0 );
  
	for ( count = 0; count < 7; ++count )
    {
		rect = metrics.boundingRect( QDate::shortDayName( count + 1 ) );
		m_MaxCell.setWidth( QMAX( m_MaxCell.width(), rect.width() ) );
		m_MaxCell.setHeight( QMAX( m_MaxCell.height(), rect.height() ) );
    }
  
	rect = metrics.boundingRect( QString::fromLatin1( "88" ) );
	m_MaxCell.setWidth( QMAX( m_MaxCell.width() + 2, rect.width() ) );
	m_MaxCell.setHeight( QMAX( m_MaxCell.height() + 4, rect.height() ) );
}

void RKDateGridView::wheelEvent( QWheelEvent* pEvent )
{
    setDate( m_Date.addMonths( -(int)( pEvent->delta() / 120 ) ) );
    pEvent->accept();
}

void RKDateGridView::contentsMousePressEvent( QMouseEvent* pEvent )
{
	if ( pEvent->type() != QEvent::MouseButtonPress )
    { 
      return;
    }
  
	if( ! isEnabled() )
    {
		QApplication::beep();
		return;
    }

    int row; 
	int col; 
	int pos; 
	int temp;
	QPoint mouseCoord;
  
	mouseCoord = pEvent->pos();
	row = rowAt( mouseCoord.y() );
	col = columnAt( mouseCoord.x() );

	if ( row < 1 || col < 0 )
    {	// the user clicked on the frame of the table
		return;
    }

	// Rows and columns are zero indexed.  The (row - 1) below is to avoid counting
	// the row with the days of the week in the calculation.

	// old selected date:

	temp = posFromDate( m_Date );

	// new position and date
	pos = ( 7 * ( row - 1 ) ) + col; 
	QDate clickedDate = dateFromPos( pos );

	// set the new date. If it is in the previous or next month, the month will
	// automatically be changed, no need to do that manually...

	setDate( clickedDate );

	// call updateCell on the old and new selection. If setDate switched to a different 
	// month, these cells will be painted twice, but that's no problem.

	updateCell( temp / 7 + 1, temp % 7 );
	updateCell( row, col );

	emit gridClicked();

	if (  pEvent->button() == Qt::RightButton && m_PopupMenuEnabled )
	{
        QPopupMenu* pMenu = new QPopupMenu();
        
        emit aboutToShowContextMenu( pMenu, clickedDate );
        pMenu->popup( pEvent->globalPos() );
	}
}

bool RKDateGridView::setDate(const QDate& date )
{
	bool changed = false;
	QDate temp;
  
	if( ! date.isValid() )
    {
		qDebug( "RKDateGridView::setDate: refusing to set invalid date." );
		return false;
    }
  
	if( m_Date != date )
    {
		emit( dateChanged( m_Date, date ) );
		m_Date = date;
		emit(dateChanged( m_Date ) );
		changed = true;
    }
  
	//const KCalendarSystem * calendar = KGlobal::locale()->calendar();

	temp.setYMD( m_Date.year(), m_Date.month(), 1 );
	//temp.setYMD(date.year(), date.month(), 1);
	//kdDebug() << "firstDayInWeek: " << temp.toString() << endl;
	m_FirstDay = temp.dayOfWeek();
	m_NumDays = m_Date.daysInMonth();

	temp = temp.addMonths( -1 );
	m_NumDaysPrevMonth = temp.daysInMonth();
  
	if ( changed )
    {
      repaintContents( false );
    }
  
	return true;
}

const QDate& RKDateGridView::date() const
{
	return m_Date;
}

void RKDateGridView::focusInEvent( QFocusEvent *pEvent )
{
    QGridView::focusInEvent( pEvent );
}

void RKDateGridView::focusOutEvent( QFocusEvent *pEvent )
{
    QGridView::focusOutEvent( pEvent );
}

QSize RKDateGridView::sizeHint() const
{
	if ( m_MaxCell.height() > 0 && m_MaxCell.width() >0 )
    {
		return QSize( m_MaxCell.width() * numCols() + 2 * frameWidth(),
             ( m_MaxCell.height() + 2 ) * numRows() + 2 * frameWidth() );
    } 
	else 
	{
		qDebug( "RKDateGridView::sizeHint: obscure failure" );
		return QSize( -1, -1 );
    }
}

void RKDateGridView::setPopupMenuEnabled( bool enable )
{
	m_PopupMenuEnabled = enable;
}

bool RKDateGridView::popupMenuEnabled() const
{
   return m_PopupMenuEnabled;
}

void RKDateGridView::setCustomDatePainting(const QDate& date, const QColor& fgColor,
	BackgroundMode bgMode, const QColor& bgColor )
{
    if ( ! fgColor.isValid() )
    {
        unsetCustomDatePainting( date );
        return;
    }

    DatePaintingMode* pMode = new DatePaintingMode;
    pMode->bgMode = bgMode;
    pMode->fgColor = fgColor;
    pMode->bgColor = bgColor;

    m_CustomPaintingModes.replace( date.toString(), pMode );
    m_UseCustomColors = true;
    update();
}

void RKDateGridView::unsetCustomDatePainting( const QDate &date )
{
    m_CustomPaintingModes.remove( date.toString() );
}

