/*  -*- C++ -*-
    This file is part of the KDE libraries
    Copyright (C) 1997 Tim D. Gilman (tdgilman@best.org)
              (C) 1998-2001 Mirko Boehm (mirko@kde.org)
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.

	Modified by John Dean, July 2005

	RKDateGrid is a 100% pure Qt port of KDateTable, which is part of the KDELibs.
	Class, function and variable names have been changed to make them consistent
	with those found in Rekall
*/
#ifndef RK_DATEGRIDVIEW_H
#define RK_DATEGRIDVIEW_H

#include <qvalidator.h>
#include <qgridview.h>
#include <qlineedit.h>
#include <qdatetime.h>
#include <qcolor.h>
#include <qdict.h>

#include "libtkkde_exports.h"

class QPopupMenu;

class LIBTKKDE_API RKDateGridView : public QGridView
{
    Q_OBJECT

public:
    
    RKDateGridView( QWidget* parent = 0, QDate date = QDate::currentDate(),
	       const char* name = 0, WFlags f = 0);

    RKDateGridView( QWidget* parent, const char* name, WFlags f = 0 );
    ~RKDateGridView();

    virtual QSize sizeHint() const;
    void setFontSize( int size );
    bool setDate( const QDate& );
    const QDate& date() const;

	void setPopupMenuEnabled( bool enable );
    bool popupMenuEnabled() const;

    enum BackgroundMode { NoBgMode = 0, RectangleMode, CircleMode };

    void setCustomDatePainting( const QDate& date, const QColor& fgColor, 
		BackgroundMode bgMode = NoBgMode, const QColor& bgColor = QColor() );
    
    void unsetCustomDatePainting( const QDate &date );

protected:
   
    int posFromDate( const QDate &date ); 
    QDate dateFromPos( int pos );

    virtual void paintCell( QPainter*, int, int );
    virtual void viewportResizeEvent( QResizeEvent* );
    
    virtual void contentsMousePressEvent( QMouseEvent* );
    virtual void wheelEvent( QWheelEvent* pEvent );
    virtual void keyPressEvent( QKeyEvent* pEvent );
    virtual void focusInEvent( QFocusEvent* pEvent );
    virtual void focusOutEvent( QFocusEvent* pEvent );

    int			m_FontSize;
    QDate		m_Date;
    int			m_FirstDay;
    int			m_NumDays;
    int			m_NumDaysPrevMonth;
    QRect		m_MaxCell;

signals:
    
    void dateChanged( QDate );
    void dateChanged( const QDate& cur, const QDate& old );
    void gridClicked();
    void aboutToShowContextMenu( QPopupMenu* pMenu, const QDate& date );

private:
    
	bool		m_PopupMenuEnabled;
	bool		m_UseCustomColors;

	struct DatePaintingMode
	{
		QColor fgColor;
		QColor bgColor;
		BackgroundMode bgMode;
	};
   
	QDict<DatePaintingMode> m_CustomPaintingModes;

	void nextMonth();
	void previousMonth();
	void beginningOfMonth();
	void endOfMonth();
	void beginningOfWeek();
	void endOfWeek();
};


#endif // RK_DATEGRIDVIEW_H
