/* 
    This file is part of the KDE libraries
    Copyright (C) 1997 Tim D. Gilman (tdgilman@best.org)
              (C) 1998-2001 Mirko Boehm (mirko@kde.org)
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
	
	Modified by John Dean, July 2005 
	
	RKDatePicker is a 100% pure Qt port of KDatePicker, which 
	is part of the KDELibs. Class, function and variable names have been changed 
	to make them consistent with those found in Rekall
*/

#ifndef KDATEPICKER_H
#define KDATEPICKER_H

#include <qdatetime.h>
#include <qframe.h>
#include <qcombobox.h>
#include <qlayout.h>

#include "libtkkde_exports.h"

class QLineEdit;
class QToolButton;
class RKDateValidator;
class RKDateGridView;

class LIBTKKDE_API RKDatePicker: public QFrame
{
	Q_OBJECT

public:
 
	RKDatePicker( QWidget* parent = 0, QDate = QDate::currentDate(),
	      const char* name = 0 );
    RKDatePicker( QWidget* parent, QDate, const char* name, WFlags f );
	RKDatePicker( QWidget* parent, const char* name );

    virtual ~RKDatePicker();

    QSize sizeHint() const;

	bool setDate( const QDate& );
    const QDate& date() const;
	void setEnabled( bool );

	RKDateGridView* dateGridView() const { return m_pDateGridView; };

	void setFontSize(int);
	int fontSize() const { return m_FontSize; }
  
	void setCloseButton( bool enable );
	bool hasCloseButton() const;

protected:

	virtual bool eventFilter( QObject* pObject, QEvent* pEvent );
	virtual void resizeEvent( QResizeEvent* );

	QToolButton*		m_pYearForwardToolButton;
	QToolButton*		m_pYearBackwardToolButton;
	QToolButton*		m_pMonthForwardToolButton;
	QToolButton*		m_pMonthBackwardToolButton;
	QToolButton*		m_pSelectMonthToolButton;
	QToolButton*		m_pSelectYearToolButton;
	
	QLineEdit*			m_pDateLineEdit;
	
	RKDateValidator*	m_pDateValidator;

	RKDateGridView*		m_pDateGridView;

	QSize				m_MaxMonthRect;

protected slots:
  
	void slotDateChanged( QDate );
	void slotGridViewClicked();
	void slotMonthForwardClicked();
	void slotMonthBackwardClicked();
	void slotYearForwardClicked();
	void slotYearBackwardClicked();
	//void slotSelectWeekClicked();
	void slotSelectMonthClicked();
	void slotSelectYearClicked();
	void slotLineEditEnterPressed();
	void slotTodayButtonClicked();
	void slotSelectedWeekClicked( int );

signals:
	
	void dateChanged( QDate );
	void dateSelected( QDate );
	void dateEntered( QDate );
	void gridViewClicked();

private:
  
	void init( const QDate& date );
	void fillWeeksCombo( const QDate& date );
	QString yearString( const QDate& date, bool bShort ) const;
	QString monthString( const QDate& date, bool bShort ) const;
	QString dayString( const QDate& date, bool bShort ) const;

    QToolButton*		m_pCloseToolButton;
    QComboBox*			m_pSelectWeekComboBox;
    QToolButton*		m_pTodayToolButton;
    QBoxLayout*			m_pNavigationLayout;

	int					m_FontSize;
};

#endif //  KDATEPICKER_H
