/***************************************************************************
    file	         : rk_dialog.cpp
    copyright            : (C) 1999,2000,2001,2002,2003,2004 by Mike Richardson
			   (C) 2001,2002,2003,2004 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<stdarg.h>
#include	<qobjectlist.h>
#include	<qapplication.h>

#include	"tk_config.h"
#include	"tk_icons.h"
#include	"rk_hbox.h"
#include	"rk_pushbutton.h"
#include	"rk_modalfilter.h"

#if		__KB_AUTOMOC
#include 	"rk_dialog.moc"
#else
#include 	"rk_dialog.h"
#endif



static	char	configGroup[]	= "Dialog Sizes" ;
static	int	nest		;


/*  RKDialog								*/
/*  RKDialog	: Constructor for kbase dialog base class		*/
/*  parent	: QWidget *	: Parent widget				*/
/*  name	: cchar *	: Dialog name				*/
/*  flags	: WFlags	: Flags					*/
/*  modal	: bool		: Modal flag				*/
/*  (returns)	: RKDialog	:					*/

RKDialog::RKDialog
	(	QWidget		*parent,
		const char	*name,
		bool		modal,
		WFlags		flags
	)
	:
	QDialog	(RKDialog::activeWindow (parent), name, modal, flags),
	m_name	(name)
{
	setIcon	(getSmallIcon ("rekall")) ;

	m_showMaxed  = false ;

	if (name != 0)
	{
		TKConfig *config = TKConfig::getConfig() ;
		config->setGroup (configGroup) ;

		m_lastSize	 = config->readSizeEntry (name, 0) ;
	}
}

RKDialog::~RKDialog ()
{
}

/*  RKDialog								*/
/*  getSpacing	: Get layout spacing					*/
/*  layout	: QLayout *	: Requesting layout			*/
/*  (returns)	: int		: Spacing				*/

int	RKDialog::getSpacing
	(	QLayout	*
	)
{
	return	4 ;
}

/*  RKDialog								*/
/*  getMargin	: Get layout margin					*/
/*  layout	: QLayout *	: Requesting layout			*/
/*  (returns)	: int		: Spacing				*/

int	RKDialog::getMargin
	(	QLayout	*
	)
{
	return	4 ;
}

/*  RKDialog								*/
/*  show	: Trap show to set layout				*/
/*  (returns)	: void		:					*/

void	RKDialog::show ()
{
	QDialog ::show () ;
}

/*  RKDialog								*/
/*  resizeEvent	: Trap resize events					*/
/*  e		: QResizeEvent * : The event				*/
/*  (returns)	: void		 :					*/

void	RKDialog::resizeEvent
	(	QResizeEvent	*e
	)
{
	m_lastSize = e->size()  ;
	QDialog::resizeEvent(e) ;
}

int	RKDialog::exec ()
{
	RKModalFilter::self()->push(0)	;
	int	rc = QDialog::exec ()	;
	RKModalFilter::self()->pop ( )	;
	return	rc ;
}




/*  [ STATIC METHODS ] -----------------------------------------------  */

/*  RKDialog								*/
/*  resizeLayout: Resize layout						*/
/*  widget	: QWidget *	: Parent widget for layout		*/
/*  margin	: int		: Margin				*/
/*  spacing	: int		: Spacing				*/
/*  (returns)	: void		:					*/

void	RKDialog::resizeLayout
	(	QWidget		*widget,
		int		margin,
		int		spacing
	)
{
//	fprintf
//	(	stderr,
//		"RKDialog::resizeLayout:%*.s w=%s l=%p\n",
//		nest,
//		"",
//		widget->className(),
//		(void *)widget->layout()
//	)	;

	if (widget->layout() != 0)
	{
		QApplication::sendPostedEvents (widget, QEvent::ChildInserted) ;

		resizeLayout
		(	widget->layout(),
			widget->name()[0] == '_' ? 0 : margin,
			spacing
		)	;
		return	;
	}

	QObjectList   *wlist = widget->queryList ("QWidget", 0, false, false) ;
	if (wlist != 0)
	{
		QObjectListIt	iter (*wlist) ;
		QObject		*obj ;
		while ((obj = iter.current()) != 0)
		{
			resizeLayout ((QWidget *)obj, margin, spacing) ;
			iter += 1 ;
		}
	}
}

/*  RKDialog								*/
/*  resizeLayout: Resize layout						*/
/*  layout	: QLayout *	: Layout to set				*/
/*  margin	: int		: Margin				*/
/*  spacing	: int		: Spacing				*/
/*  (returns)	: void		:					*/

void	RKDialog::resizeLayout
	(	QLayout		*layout,
		int		margin,
		int		spacing
	)
{
	if (layout != 0)
	{
		nest	+= 2 ;

		if (margin  < 0) margin  = 4 ;
		if (spacing < 0) spacing = 4 ;

		layout->setMargin  (margin ) ;
		layout->setSpacing (spacing) ;

		QLayoutIterator iter = layout->iterator() ;
		QLayoutItem	*lay ;

		while ((lay = iter.current()) != 0)
		{
//			fprintf
//			(	stderr,
//				"_KBDialog::setupLayout:%*.s l=%p l=%p w=%p\n",
//				nest,
//				"",
//				(void *)lay,
//				(void *)lay->layout(),
//				(void *)lay->widget()
//			)	;

			if (lay->layout() != 0)
				resizeLayout (lay->layout(), margin, spacing) ;
			if (lay->widget() != 0)
				resizeLayout (lay->widget(), margin, spacing) ;

			++iter	;
		}

		nest	-= 2	;
	}
}

/*  ------------------------------------------------------------------	*/

/*  RKDialog								*/
/*  activeWindow: Get active window for parenting			*/
/*  widget	: QWidget *	: If set, specific widget to use	*/
/*  (returns)	: QWidget *	: Widget to be used			*/

QWidget	*RKDialog::activeWindow
	(	QWidget		*widget
	)
{
	if (widget == 0)
	{
		qApp->processEvents () ;
		widget = qApp->activeWindow() ;
	}
		
	return	widget ;
}
