/***************************************************************************
    file	         : rk_gridbox.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qapplication.h>
#include 	<qlayout.h>

#if		__KB_AUTOMOC
#include	"rk_gridbox.moc"
#else
#include	"rk_gridbox.h"
#endif

class	RKGridLayout : public QGridLayout
{
	bool		m_noMargin	;

public	:

	RKGridLayout	(QWidget *, int, int, bool) ;

	void		setNoMargin	(bool)	;
	virtual	void	setMargin	(int) ;
	virtual	void	addItem		(QLayoutItem *)	;
}	;

RKGridLayout::RKGridLayout
	(	QWidget		*parent,
		int		nRows,
		int		nCols,
		bool		noMargin
	)
	:
	QGridLayout	(parent, nRows, nCols),
	m_noMargin	(noMargin)
{
}

void	RKGridLayout::setNoMargin
	(	bool		noMargin
	)
{
	m_noMargin = noMargin ;
}

void	RKGridLayout::setMargin
	(	int		margin
	)
{
	QGridLayout::setMargin (m_noMargin ? 0 : margin) ;
}


void	RKGridLayout::addItem
	(	QLayoutItem	*item
	)
{
	if (qstrcmp (item->widget()->className(), "KBSizerBlob") != 0)
		QGridLayout::addItem (item) ;
}

RKGridBox::RKGridBox
	(	int		nCols,
		QWidget		*parent,
		const char	*name,
		WFlags		flags
	)
	:
	RKBox	(parent, name, flags)
{
	m_layout = new RKGridLayout (this, -1, nCols, isNestedBox()) ;
	m_layout->setAutoAdd (true) ;

	setMargin (0) ;
}


void	RKGridBox::setSpacing
	(	int		space
	)
{
	m_layout->setSpacing (space)	;
}

void	RKGridBox::setMargin
	(	int		margin
	)
{
	m_layout->setMargin (margin)	;
}

void	RKGridBox::frameChanged ()
{
	m_layout->setMargin (frameWidth()) ;
}


QSize	RKGridBox::sizeHint() const
{
	QApplication::sendPostedEvents
	(	(QWidget *)this,
		QEvent::ChildInserted
	)	;
	return	QFrame::sizeHint() ;
}

void	RKGridBox::addFillerRow ()
{
	QApplication::sendPostedEvents
	(	(QWidget *)this,
		QEvent::ChildInserted
	)	;

	int nr = m_layout->numRows() ;
	for (int c = 0 ; c < m_layout->numCols() ; c += 1)
		new QWidget (this) ;
	m_layout->setRowStretch (nr, 1) ;
}

void	RKGridBox::setRowStretch
	(	int		row,
		int		stretch
	)
	
{
	m_layout->setRowStretch (row, stretch) ;
}

void	RKGridBox::setColStretch
	(	int		col,
		int		stretch
	)
{
	m_layout->setColStretch (col, stretch) ;
}

void	RKGridBox::setNoMargin
	(	bool		noMargin
	)
{
	m_layout->setNoMargin (noMargin) ;
}
