/***************************************************************************
    file	         : rk_modalfilter.h
    copyright            : (C) 1999,2000,2001,2002,2003,2004 by Mike Richardson
			   (C) 2001,2002,2003,2004 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<stdio.h>

#include	<qwidget.h>
#include	<qevent.h>
#include	<qapplication.h>

#if		__KB_AUTOMOC
#include	"rk_modalfilter.moc"
#else
#include	"rk_modalfilter.h"
#endif


RKMFFilter::RKMFFilter ()
{
}

RKMFFilter::RKMFFilter
	(	QWidget		*modalTo
	)
	:
	m_modalTo (modalTo),
	m_dropped (false)
{
}

/*  ------------------------------------------------------------------	*/

/*  RKModalFilter							*/
/*  RKModalFilter: Filter for pretend modal widgets			*/
/*  (returns)	 : RKModalFilter :					*/

RKModalFilter::RKModalFilter ()
{
}

/*  RKModalFilter							*/
/*  widget	: QWidget *	: Push modal widget onto stack		*/
/*  (returns)	: void		:					*/

void	RKModalFilter::push
	(	QWidget		*widget
	)
{
//	fprintf
//	(	stderr,
//		"RKModalFilter::push: for %p\n",
//		(void *)modalTo
//	)	;

	/* Clear current filter if any. We may or may not need to	*/
	/* reinstall below, but removing anyways easier.		*/
	qApp->removeEventFilter  (this) ;

	m_stack.insert (m_stack.begin(), RKMFFilter (widget)) ;

	/* The widget may be zero, in which case we suspend modal	*/
	/* filtering.							*/
	if (m_stack[0].m_modalTo != 0)
		qApp->installEventFilter (this) ;

}

/*  RKModalFilter							*/
/*  pop		: Pop topmost element from stack			*/
/*  (returns)	: void		:					*/

void	RKModalFilter::pop ()
{
//	fprintf
//	(	stderr,
//		"RKModalFilter::pop: for %p\n",
//		(void *)m_stack[0].m_modalTo
//	)	;

	qApp->removeEventFilter  (this) ;

	m_stack.remove (m_stack.begin()) ;

	if ((m_stack.count() > 0) && (m_stack[0].m_modalTo != 0))
		qApp->installEventFilter (this) ;
}

/*  RKModalFilter							*/
/*  (returns)	: bool		:					*/

bool	RKModalFilter::anyDropped ()
{
	/* Returns true if the topmost entry is marked as having 	*/
	/* dropped any events						*/
	if (m_stack.count() > 0)
		if (m_stack[0].m_dropped)
		{	m_stack[0].m_dropped = false ;
			return	true	;
		}

	return	false	;
}
		

/*  RKModalFilter							*/
/*  eventFilter	: Event filter method					*/
/*  o		: QObject *	: Object to which event is destined	*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		:					*/

bool	RKModalFilter::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	/* Should not be called if stack is empty, but check for the	*/
	/* sake of paranoia.						*/
	if (m_stack.count() == 0)
		return	false	;

	/* If the top-most entry has a null widget pointer then modal	*/
	/* filtering is disabled and we let the event proceed.		*/
	QWidget *modalTo = m_stack[0].m_modalTo ;
	if (modalTo == 0)
		return	false	;

	/* If the target object is a widget then track up to see if the	*/
	/* current modal widget is a direct ancestor of the target; if	*/
	/* so then all events are allowed to proceed.			*/
	if (o->isWidgetType()) 
		for (QWidget *w = (QWidget *)o ; w != 0 ; w = w->parentWidget())
			if (w == modalTo)
				return	false	;


	/* If the event is a user event which does something, then it	*/
	/* is consumed. Otherwise (for instance, painting and geometry	*/
	/* management) it is allowed to proceed.			*/
	switch (e->type())
	{
		case QEvent::MouseButtonPress    :
		case QEvent::MouseButtonRelease  :
		case QEvent::MouseButtonDblClick :
		case QEvent::ContextMenu	 :
		case QEvent::KeyPress		 :
		case QEvent::KeyRelease		 :
		case QEvent::Wheel		 :
		case QEvent::FocusIn		 :
		case QEvent::FocusOut		 :
			m_stack[0].m_dropped = true ;
			return	true	 ;

		default	:
			break	;
	}

	return	false	;
}


/*  RKModalFilter							*/
/*  self	: Get modal filter object				*/
/*  (returns)	: RKModalFilter * :					*/

RKModalFilter
	*RKModalFilter::self ()
{
	static	RKModalFilter	*filter = 0 ;
	if (filter == 0) filter = new RKModalFilter ;
	return	filter	;
}
