#include <qstring.h>
#include <qpen.h>
#include <qpainter.h>
#include <qapplication.h>

#if	__KB_AUTOMOC
#include "rk_monthselector.moc"
#else
#include "rk_monthselector.h"
#endif

RKMonthSelector::RKMonthSelector( const QDate& date, QWidget* parent, const char* name )
  : QGridView( parent, name ), m_Result( 0 ) 
{
	QRect rect;
	QFont font;

	m_Day = date.day();
	m_Month = date.month();
	m_Year = date.year();
	m_ActiveCol = -1;
	m_ActiveRow = -1;
	font = QApplication::font();
	setFont( font );
	setHScrollBarMode( AlwaysOff );
	setVScrollBarMode( AlwaysOff );
	setFrameStyle( QFrame::NoFrame );
	setNumCols( 3 );

	setNumRows( 4 );
	  
	QFontMetrics metrics(font);
  
	QDate tempDate( date.year(), 1, 1 );

	for( int i = 0; i < 12; ++i )
    {
		QString str = QDate::shortMonthName( tempDate.month() );
      
		if ( str.isNull() ) 
			break;
			
		rect = metrics.boundingRect( str );

		if ( max.width() < rect.width() )
			max.setWidth( rect.width() );

		if ( max.height() < rect.height() ) 
			max.setHeight( rect.height() );

		tempDate.addMonths( 1 );
    }
}

RKMonthSelector::~RKMonthSelector()
{
}

QSize RKMonthSelector::sizeHint() const
{
	return QSize( ( max.width() + 6 ) * numCols() + 2 * frameWidth(),
		( max.height() + 6 ) * numRows() + 2 * frameWidth() );
}

int RKMonthSelector::result() const
{
  return m_Result;
}

void RKMonthSelector::setupPainter( QPainter* pPainter )
{
	pPainter->setPen( paletteBackgroundColor() );
}

void RKMonthSelector::viewportResizeEvent( QResizeEvent* )
{
	setCellWidth( width() / numCols() );
	setCellHeight( height() / numRows() );
}

void RKMonthSelector::paintCell( QPainter* pPainter, int row, int col )
{
	int index;
	QString text;

	index = 3 * row + col + 1;

	text = QDate::shortMonthName( m_Month );
		
	pPainter->drawText( 0, 0, cellWidth(), cellHeight(), AlignCenter, text );
  
	if ( m_ActiveCol == col && m_ActiveRow == row )
		pPainter->drawRect( 0, 0, cellWidth(), cellHeight() );
}

void RKMonthSelector::contentsMousePressEvent( QMouseEvent* pEvent )
{
	if ( ! isEnabled() || pEvent->button() != LeftButton )
    {
		QApplication::beep();
		return;
    }

	int row;
	int col;
	QPoint mouseCoord;

	mouseCoord = pEvent->pos();
	row = rowAt( mouseCoord.y() );
	col = columnAt( mouseCoord.x() );

	if ( row < 0 || col < 0 )
    { 
		m_ActiveCol = -1;
		m_ActiveRow = -1;
    } 
	else 
	{
		m_ActiveCol = col;
		m_ActiveRow = row;
		updateCell( row, col );
	}
}

void RKMonthSelector::contentsMouseMoveEvent( QMouseEvent* pEvent )
{
	if ( pEvent->state() & LeftButton )
    {
		int row;
		int col;
		QPoint mouseCoord;

		mouseCoord = pEvent->pos();
		row = rowAt( mouseCoord.y() );
		col = columnAt( mouseCoord.x() );
      
		int tmpRow = -1; 
		int tmpCol = -1;

		if ( row < 0 || col < 0 )
        { 
			if ( m_ActiveCol > -1 )
            {
				tmpRow = m_ActiveRow;
				tmpCol = m_ActiveCol;
            }
          
			m_ActiveCol = -1;
			m_ActiveRow = -1;
        } 
		else 
		{
			bool differentCell = ( m_ActiveRow != row || m_ActiveCol != col );
          
			if ( m_ActiveCol > -1 && differentCell )
            {
				tmpRow = m_ActiveRow;
				tmpCol = m_ActiveCol;
            }
          
			if ( differentCell )
            {
				m_ActiveRow = row;
				m_ActiveCol = col;
				updateCell( row, col );
            }
        }
      
		if ( tmpRow > -1 ) 
			updateCell( tmpRow, tmpCol );
    }
}

void RKMonthSelector::contentsMouseReleaseEvent( QMouseEvent* pEvent )
{
	if( ! isEnabled() )
    {
		return;
    }
  
	int row; 
	int col; 
	int pos;
	QPoint mouseCoord;
  
	mouseCoord = pEvent->pos();
	row = rowAt( mouseCoord.y() );
	col = columnAt( mouseCoord.x() );
  
	if ( row < 0 || col < 0 )
    { 
		emit( shutDown( 0 ) );
    }

	pos = 3 * row + col + 1;
	m_Result = pos;
  
	emit( shutDown( 1 ) );
}
