/***************************************************************************
    file	         : tk_icons.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qstring.h>
#include	<qimage.h>
#include	<qpixmap.h>
#include	<qfileinfo.h>
#include	<qdict.h>

#include 	"libtkkde_exports.h"



static	QDict<QPixmap>	pixMapDict		;
extern	const QString	&getAppdataDir	()	;

/*  findIconPath: Locate path to icon					*/
/*  size	: const char *	  : Icon size required			*/
/*  name	: const QString & : Icon name				*/
/*  (returns)	: QString	  : Path or null if not found		*/

static	QString	findIconPath
	(	const char	*size,
		const QString	&name
	)
{
	static	const	char	*paths[] =
	{
		"icons/hicolor/%1/actions/%1.png",
		"icons/hicolor/%1/apps/%1.png",
		0
	}	;

	for (const char **pp = &paths[0] ; *pp != 0 ; pp += 1)
	{
		QString	  path = getAppdataDir() + '/' + QString(*pp).arg(size).arg(name) ;

		QFileInfo file(path)	;
		if (file.exists ())
		{
			return path ;
		}
	}

	fprintf
	(	stderr,
		"findIconPath: %s %s: ***** not found *****\n",
		size,
		name.latin1()
	)	;

	return	QString::null ;
}

/*  loadPixmapPNG: Load image from PNG file				*/
/*  path	 : const QString & : Path to image file			*/
/*  (returns)	 : QPixMap	   : Image				*/

static	QPixmap	loadPixmapPNG
	(	const QString	&path
	)
{
	QPixmap	*cached	= pixMapDict.find (path) ;
	if (cached != 0) return *cached ;

	QPixmap	pixmap	;
	pixmap.convertFromImage (QImage(path));
	pixMapDict.insert (path, new QPixmap (pixmap)) ;
	return	pixmap	;
}

#if	0

/*  loadPixmapXPM: Load image from XMP definition			*/
/*  xpm		 : const char ** : Definition				*/
/*  (returns)	 : QPixMap	 : Image				*/

static	QPixmap	loadPixmapXPM
	(	const char	**xpm
	)
{
	QPixmap	pixmap	;
	pixmap.convertFromImage (QImage(xpm)) ;
	return	pixmap	;
}

#endif


LIBTKKDE_API
	QPixmap	getDesktopIcon
	(	const QString	&name
	)
{
	QString	path	= findIconPath ("48x48", name) ;
	return	path.isEmpty() ? QPixmap () : loadPixmapPNG (path) ;
}

LIBTKKDE_API
	QPixmap	getBarIcon
	(	const QString	&name
	)
{
	QString	path	= findIconPath ("22x22", name) ;
	return	path.isEmpty() ? QPixmap () : loadPixmapPNG (path) ;
}

LIBTKKDE_API
	QPixmap	getSmallIcon
	(	const QString	&name
	)
{
	QString	path	= findIconPath ("16x16", name) ;
	return	path.isEmpty() ? QPixmap () : loadPixmapPNG (path) ;
}
