/***************************************************************************
    file	         : tk_part.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__TK_PART_H
#define	__TK_PART_H

#if		defined(_WIN32) && !defined(__MINGW32__)
#pragma		warning(push)
#pragma 	warning(disable : 4251)
#endif

#include	<qwidget.h>

#include	"tk_xmlguispec.h"
#include 	"libtkkde_exports.h"

class	TKPart		;
class	TKPartManager	;


/*  TKPartManager							*/
/*  -------------							*/
/*  This class is the part manager for the QT3 only builds of Rekall.	*/
/*  It is basically a minima; implementation of KDE's part manager, and	*/
/*  provides sufficient functionality in common.			*/

class 	LIBTKKDE_API TKPartManager : public QObject
{
	Q_OBJECT

	QWidget			*m_topWidget	;
	QPtrList<TKPart>	m_partList	;
	TKPart			*m_activePart	;

	TKPartManager		(QWidget *, const char * = 0) ;
	TKPart			*partForWidget	(QWidget *)   ;

protected :

	virtual	bool	eventFilter	(QObject *, QEvent *) ;

public slots :

	void		partDestroyed	() ;
	void		partActivated	(TKPart	*) ;

public	:


	virtual	void	addPart		(TKPart *, bool = true) ;
	virtual	void	setActivePart	(TKPart *) ;

	inline	const	QPtrList<TKPart>	*parts ()
	{
		return	&m_partList	;
	}
	inline	TKPart	*activePart ()
	{
		return	m_activePart	;
	}

	/* Static interface. The main program should call create() to	*/
	/* instantiate a single instance, thereafter self() should be	*/
	/* used to access said instance.				*/
	static	TKPartManager	*create	(QWidget *, const char * = 0) ;
	static	TKPartManager	*self	() ;

signals :

	void	activePartChanged 	(TKPart *) ;
	void	activePartActivated	(TKPart *) ;
}	;


/*  TKPart								*/
/*  ------								*/
/*  Equivalent to KPart in KDE. Implements enough common functionality	*/
/*  for Rekall.								*/

class	LIBTKKDE_API TKPart : public QObject, public TKXMLGUISpec
{
	Q_OBJECT

	QWidget			*m_widget	;
	TKPartManager		*m_manager	;

protected slots :

	virtual	void	slotWidgetDestroyed ()	;

public	:

	TKPart	(QObject * = 0, const char * = 0) ;
	virtual	~TKPart	() ;

	virtual	void	setWidget	(QWidget *) ;

	inline	QWidget	*widget ()
	{
		return	m_widget ;
	}
	inline	void	setManager
		(	TKPartManager	*manager
		)
	{
		m_manager = manager ;
	}
	inline	TKPartManager *manager ()
	{
		return	m_manager ;
	}
}	;

#if		defined(_WIN32) && !defined(__MINGW32__)
#pragma		warning(pop)
#endif

#endif	// __TK_PART_H
