/***************************************************************************
    file	         : rk_dcopbase.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"rk_dcopbase.moc"
#else
#include	"rk_dcopbase.h"
#endif

#include	<stdlib.h>

#include	<qsocket.h>
#include	<qsocketdevice.h>
#include	<qsocketnotifier.h>
#include	<qhostaddress.h>
#include	<qdatastream.h>



/*  RKDCopConnection							*/
/*  RKDCopConnection							*/
/*		: Constructor for RKDCOP connection object		*/
/*  listener	: RKDCopListener * : Parent listener			*/
/*  socket	: int		   : Socket to caller			*/
/*  (returns)	: RKDCopConnection :					*/

RKDCopConnection::RKDCopConnection
	(	RKDCopListener	*listener,
		int		socket
	)
	:
	QSocket		(listener),
	m_listener	(listener)
{
	setSocket	(socket) ;

	connect
	(	this,
		SIGNAL	(readyRead	()),
		SLOT	(slotReadyRead	())
	)	;
}

/*  RKDCopConnection							*/
/*  slotReadyRead: Data available on socket				*/
/*  (returns)	 : void		:					*/

void	RKDCopConnection::slotReadyRead ()
{
	uint	avail	= bytesAvailable () ;
	uint	now	= m_data.size	 () ;

//	fprintf
//	(	stderr,
//		"RKDCopConnection::slotReadyRead: called for %d\n",
//		avail
//	)	;

	m_data.resize (now + avail) ;
	readBlock     (m_data.data() + now, avail) ;

	int	zero	= m_data.find (0) ;
	if (zero < 0)
	{
//		fprintf
//		(	stderr,
//			"RKDCopConnection::slotReadyRead: awaiting header\n"
//		)	;
		return	;
	}

	uint	len	= atoi (m_data.data()) ;
	if (m_data.size() < zero + 1 + len)
	{
//		fprintf
//		(	stderr,
//			"RKDCopConnection::slotReadyRead: awaiting %d\n",
//			zero + 1 + len - m_data.size()
//		)	;
		return	;
	}

	uint	p1	= m_data.find('|',      0) ;
	uint	p2	= m_data.find('|', p1 + 1) ;

	m_data.data()[p1++]	= 0	;
	m_data.data()[p2++]	= 0	;

//	fprintf
//	(	stderr,
//		"RKDCopConnection::slotReadyRead: target=[%s] fn=[%s]\n",
//		m_data.data() + p1,
//		m_data.data() + p2
//	)	;

	QByteArray data	;
	data.duplicate	(m_data.data() + zero + 1, m_data.size() - zero - 1) ;

	QCString   replyType	;
	QByteArray replyData	;

	bool	   rc	= m_listener->process
			  (	m_data.data() + p1,
				m_data.data() + p2,
				data,
				replyType,
				replyData
			  )	;

//	fprintf
//	(	stderr,
//		"RKDCopConnection::slotReadyRead: rc=%d rt=[%s] rds=%d\n",
//		rc,
//		replyType.data(),
//		replyData.size()
//	)	;

	QString	h  ;
	if (rc)
		h.sprintf  ("%d|%s", replyData.size(), replyType.data()) ;
	else	h.sprintf  ("%d|%s", replyData.size(), "#Error") ;

	writeBlock (h.latin1(), qstrlen(h.latin1()) + 1) ;
	writeBlock (replyData.data(), replyData.size()) ;
}

/*  ------------------------------------------------------------------	*/

class	RKAppDCOPObject : public RKDCOPBase
{
	virtual		bool	process
				(	const QCString		&,
					const QByteArray	&,
					QCString		&,
					QByteArray		&
				)	;
	QCStringList		functions  () ;
	QCStringList		interfaces () ;

public	:

	RKAppDCOPObject	(const char *) ;
}	;


RKAppDCOPObject::RKAppDCOPObject
	(	const char	*name
	)
	:
	RKDCOPBase	(name)
{
}

bool	RKAppDCOPObject::process
	(	const QCString		&func,
		const QByteArray	&,
		QCString		&replyType,
		QByteArray		&
	)
{
	if (func == "quit()")
	{
		qApp->quit()	;
		replyType	= "void" ;
		return	true	;
	}

	return	true	;
}

/*  RKAppDCOPObject							*/
/*  interfaces	: Return interface list					*/
/*  (returns)	: QCStringList	: Interface list			*/

QCStringList
	RKAppDCOPObject::interfaces()
{
	return	QCStringList() ;
}

/*  RKAppDCOPObject							*/
/*  functions	: Return function list					*/
/*  (returns)	: QCStringList	: Function list				*/

QCStringList
	RKAppDCOPObject::functions ()
{
	return	QCStringList() ;
}

/*  ------------------------------------------------------------------	*/


static	uint	rkDCOPPort	= 0	;
static	bool	rkDCOPGlobal	= false	;


RKDCopListener::RKDCopListener ()
{
	QHostAddress addr ;

	m_notify = 0 ;
	m_sd	 = new QSocketDevice () ;
	m_sd->setAddressReusable (true) ;

	if (rkDCOPGlobal)
		addr.setAddress (  "0.0.0.0") ;
	else	addr.setAddress ("127.0.0.1") ;

	if (!m_sd->bind (addr, rkDCOPPort))
	{
		fprintf
		(	stderr,
			"RKDCopListener::RKDCopListener: cannot bind\n"
		)	;
		return	;
	}

	if (!m_sd->listen (10))
	{
		fprintf
		(	stderr,
			"RKDCopListener::RKDCopListener: cannot listen\n"
		)	;
		return	;
	}

	m_notify = new QSocketNotifier (m_sd->socket(), QSocketNotifier::Read, this) ;
	connect
	(	m_notify,
		SIGNAL	(activated	(int)),
		SLOT	(slotCommsCalled())
	)	;
}

void	RKDCopListener::slotCommsCalled ()
{
//	fprintf
//	(	stderr,
//		"RKDCopListener::slotCommsCalled: m_sd=%p\n",
//		(void *)m_sd
//	)	;

	if (m_sd == 0)
		return	;

	int	socket	= m_sd->accept() ;
	if (socket == -1)
	{
		fprintf
		(	stderr,
			"RKDCopListener::slotCommsCalled: failed to accept\n"
		)	;
		return	;
	}


	RKDCopConnection *comms = new RKDCopConnection (this, socket) ;
	connect
	(	comms,
		SIGNAL	(connectionClosed()),
		SLOT	(slotCommsClosed ())
	)	;
}

void	RKDCopListener::slotCommsClosed ()
{
//	fprintf
//	(	stderr,
//		"RKDCopListener::slotCommsClosed: called\n"
//	)	;
	delete	sender	() ;
}

bool	RKDCopListener::process
	(	const char		*target,
		const QCString		&function,
		const QByteArray	&data,
		QCString		&replyType,
		QByteArray		&replyData
	)
{
	RKDCOPBase *object = m_objects.find (target) ;
	if (object != 0)
	{
//		fprintf
//		(	stderr,
//			"RKDCopListener::process: %s: found object\n",
//			target
//		)	;

		return	object->process (function, data, replyType, replyData) ;
	}

	return	false	;
}

void	RKDCopListener::attach
	(	const char 	*name,
		RKDCOPBase	*object
	)
{
//	fprintf
//	(	stderr,
//		"RKDCopListener::attach: adding [%s]\n",
//		name
//	)	;

	m_objects.insert (name, object) ;
}

void	RKDCopListener::detach
	(	const char	*name
	)
{
	m_objects.remove (name) ;
}

RKDCopListener
	*RKDCopListener::self ()
{
	static	RKDCopListener *listener ;

	if (listener == 0)
		listener = new RKDCopListener () ;
	
	return	listener ;
}

/*  ------------------------------------------------------------------	*/

/*  RKDCOPBase								*/
/*  RKDCOPBase	: Constructor for RKDCOP handler base class		*/
/*  name	: const char *	: Object name				*/
/*  (returns)	: RKDCOPBase	:					*/

RKDCOPBase::RKDCOPBase
	(	const char	*name
	)
{
	/* Save the name. Then attach to the listener object if RKDCOP	*/
	/* is enabled (the port is non-zero).				*/
	m_name	= qstrdup (name) ;
	if (rkDCOPPort > 0)
		RKDCopListener::self()->attach (m_name, this) ;
}

/*  RKDCOPBase								*/
/*  ~RKDCOPBase	: Destructor for RKDCOP handler base class		*/
/*  (returns)	:		:					*/

RKDCOPBase::~RKDCOPBase ()
{
	/* Detach will silently do nothing if we are not connected, by	*/
	/* call anyway just in case RKDCOP was initially enabled, then	*/
	/* later disabled.						*/
	RKDCopListener::self()->detach (m_name) ;
	delete	[] m_name ;
}


QCString
	RKDCOPBase::normalizeFunctionSignature
	(	const QCString	&func
	)
{
	return	func	;
}

/*  RKDCOPBase								*/
/*  setAccess	: RKDCOP access control function			*/
/*  port	: uint		: Port number				*/
/*  global	: bool		: Allow network-wide access		*/
/*  (returns)	: void		:					*/

void	RKDCOPBase::setAccess
	(	uint		port,
		bool		global
	)
{
	/* Setting the port to zero disabled RKDCOP, setting to one is	*/
	/* mapped to the default port.					*/
	rkDCOPPort	= port	== 0 ? 0 : port == 1 ? 6990 : port ;
	rkDCOPGlobal	= global ;

	if (rkDCOPPort > 0)
		new RKAppDCOPObject ("MainApplication-Interface") ;
}

bool	RKDCOPBase::process
	(	const QCString		&,
		const QByteArray	&,
		QCString		&,
		QByteArray		&
	)
{
	return	false	;
}

QCStringList RKDCOPBase::functions  ()
{
	return	QCStringList() ;
}

QCStringList RKDCOPBase::interfaces ()
{
	return	QCStringList() ;
}

	
