/***************************************************************************
    file	         : rk_dcopbase.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#ifndef		__RK_DCOPBASE_H
#define		__RK_DCOPBASE_H

#include	<qcstring.h>
#include	<qvaluelist.h>
#include	<qasciidict.h>

#include	<qcstring.h>
#include	<qobject.h>
#include	<qsocket.h>

#include 	"librkcop_exports.h"


typedef	QValueList<QCString>	QCStringList	;

class	QSocketDevice		;
class	QSocketNotifier		;
class	RKDCopConnection	;
class	RKDCopListener		;
class	RKDCOPBase		;


/*  RKDCopConnection							*/
/*  ----------------							*/
/*  RKDCOP connection object. This is a wrapper around a QSocket, which	*/
/*  handles request and result data processing.				*/

class	RKDCopConnection : public QSocket
{
	Q_OBJECT

	RKDCopListener		*m_listener	;
	QByteArray		m_data		;

protected slots	:

	void			slotReadyRead	() ;

public	:

	RKDCopConnection	(RKDCopListener *, int)	;
}	;


/*  RKDCopListener							*/
/*  --------------							*/
/*  This is a singleton class which is responsible for listening for	*/
/*  RKDCOP requests and creating instances of the RKDCopConnection	*/
/*  class to handle each such request.					*/

class	RKDCopListener : public QObject
{
	Q_OBJECT

	QSocketDevice		*m_sd		;
	QSocketNotifier		*m_notify	;
	QAsciiDict<RKDCOPBase>m_objects	;

protected slots :

	void			slotCommsCalled	() ;
	void			slotCommsClosed	() ;

public	:

	RKDCopListener	()	;

	void			attach		(const char *, RKDCOPBase *) ;
	void			detach		(const char *)	;

	bool			process
				(	const char		*,
					const QCString		&,
					const QByteArray	&,
					QCString		&,
					QByteArray		&
				)	;

	static	RKDCopListener	*self	() ;
}	;

/*  RKDCOPBase								*/
/*  ----------								*/
/*  RKDCOP object, used to field requests for a specified object. This	*/
/*  is Rekall-specific coded, running over TCP/IP. It mostly emulates	*/
/*  the KDE DCOP implementation.					*/

class	LIBRKCOP_API	RKDCOPBase
{
	const char	*m_name		;

public	:

	RKDCOPBase		(const char *)	;
	virtual	~RKDCOPBase	()		;

	virtual		bool	process
				(	const QCString		&,
					const QByteArray	&,
					QCString		&,
					QByteArray		&
				)	;
	virtual	QCStringList	functions	() ;
	virtual	QCStringList	interfaces	() ;

	static	void	 setAccess		   (uint, bool) ;

	static	QCString normalizeFunctionSignature(const QCString &func) ;
}	;

#endif		// __RK_DCOPBASE_H
