#include	<Python.h>

#include	<qvaluelist.h>
#include	<qcstring.h>
#include	<qvariant.h>


static	PyObject *rkdcopError	;

static	PyObject *rkdcopRequest
	(	 PyObject	*self,
		 PyObject	*args
	)
{
	if (PySequence_Size(args) < 3)
	{
		PyErr_SetString (rkdcopError, "Insufficient arguments") ;
		return	0	;
	}

	PyObject *host   = PySequence_GetItem (args, 0) ;
	PyObject *target = PySequence_GetItem (args, 1) ;
	PyObject *func   = PySequence_GetItem (args, 2) ;

	if (!PyString_Check (host ))
	{
		PyErr_SetString (rkdcopError, "Host argument not a string") ;
		return	0	;
	}
	if (!PyString_Check (target))
	{
		PyErr_SetString (rkdcopError, "Target argument not a string") ;
		return	0	;
	}
	if (!PyString_Check (func  ))
	{
		PyErr_SetString (rkdcopError, "Function argument not a string") ;
		return	0	;
	}

	QValueList<QCString> argv ;
	for (int idx = 3 ; idx < PySequence_Size(args) ; idx += 1)
	{
		PyObject *arg   = PySequence_GetItem (args, idx) ;
		PyObject *str	= PyObject_Str       (arg)	 ;

		argv.append (PyString_AsString(str)) ;
		Py_DECREF(str)	;
	}

	extern	QString	rkDCOPCall
		(	const char		*host,
			const char		*target,
			const char		*func,
			QValueList<QCString>	&args,
			QVariant		&resv
		)	;

	QVariant rv	;
	QString	 rc	= rkDCOPCall
			  (	PyString_AsString(host),
				PyString_AsString(target),
				PyString_AsString(func),
				argv,
				rv
			  )	;

	switch (rv.type())
	{
		case QVariant::StringList :
		{
			QStringList sl = rv.toStringList() ;
			PyObject    *t = PyTuple_New	(sl.count()) ;
			for (uint idx = 0 ; idx < sl.count() ; idx += 1)
				PyTuple_SetItem (t, idx, PyString_FromString(sl[idx].latin1())) ;
			return	t ;
		}

		case QVariant::String	  :
			return	PyString_FromString (rv.toString().latin1()) ;

		case QVariant::Int	  :
			return	PyInt_FromLong 	    (rv.toInt()) ;

		default	:
			break	;
	}

	PyErr_SetString (rkdcopError, "Unknown rkdcop result type") ;
	return	0	;
}

static	PyMethodDef rkdcopMethods[] =
{
{	"request", 	rkdcopRequest,	METH_VARARGS,	"Make a RKDCOP request"	},
{	0,		0,		0,		0			}
}	;


PyMODINIT_FUNC	initrkdcop ()
{
	rkdcopError = PyErr_NewException("RKDCOP.error", 0, 0) ;
	(void) Py_InitModule ("rkdcop", rkdcopMethods) ;
}
