/***************************************************************************
    file	         : browser.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"browser.moc"
#else
#include	"browser.h"
#endif

#include	"viewer.h"

#include	<qmessagebox.h>
#include	<qprocess.h>
#include	<qfileinfo.h>
#include	<qtoolbutton.h>

#include	<stdlib.h>

SOHelpPage::SOHelpPage
	(	QWidget		*parent,
		SOHelpViewer	*viewer,
		const char 	*name
	)
	:
	QTextBrowser	(parent, name),
	m_viewer	(viewer),
	m_noScroll	(false)
{
	extern	QString	mwHelpDir ;

	QMimeSourceFactory *cf = new QMimeSourceFactory  () ;

	cf->addFilePath	     (QDir::convertSeparators(mwHelpDir)) ;
	cf->setExtensionType ("html",  "text/html;charset=UTF-8") ;
	cf->setExtensionType ("gif",   "image/gif" ) ;
	cf->setExtensionType ("png",   "image/png" ) ;
	cf->setExtensionType ("jpg",   "image/jpeg") ;
	cf->setExtensionType ("jpeg",  "image/jpeg") ;

	setMimeSourceFactory (cf) ;

	setFrameStyle	     (QFrame::Panel | QFrame::Sunken) ;

	m_current	= 0  ;
}

QString	SOHelpPage::findWebBrowser ()
{
	QStringList path 	= QStringList::split (':', getenv("PATH")) ;
	QStringList browsers	;

	browsers.append	("konqueror"	) ;
	browsers.append	("mozilla"	) ;
	browsers.append	("galeon"	) ;
	browsers.append	("opera"	) ;
	browsers.append	("netscape"	) ;
	browsers.append	("firebird"	) ;

	QStringList::iterator	browserIter	;
	QStringList::iterator	pathIter	;

	for (browserIter = browsers.begin() ; browserIter != browsers.end() ; browserIter++)
		for (pathIter = path.begin() ; pathIter != path.end() ; pathIter++)
		{
			QString	file	= (*pathIter) + "/" + (*browserIter) ;
			if (QFileInfo(file).isExecutable()) return file ;
		}

	return	QString::null	;
}

void	SOHelpPage::setSource
	(	const QString	&link
	)
{
	fprintf
	(	stderr,
		"SOHelpPage::setSource: [%s]\n",
		link.latin1()
	)	;

	QTextBrowser::setSource (link) ;
}

QString	SOHelpPage::showHelp
	(	const QString	&source,
		const QString	&name,
		const QString	&title
	)
{
	fprintf
	(	stderr,
		"SOHelpPage::setSource: [%s][%s]\n",
		source.latin1(),
		name  .latin1()
	)	;

	if (source.isEmpty())
		return	QString::null	;


	if (source.left(7) == "http://")
	{
		QString	webBrowser = findWebBrowser () ;

		if (webBrowser.isEmpty())
		{
			QMessageBox::information
			(	m_viewer,
				tr("Help"),
				tr("Cannot find a web browser")
			)	;
		}

		QProcess *proc = new QProcess() ;

		proc->addArgument(webBrowser) ;
		proc->addArgument(source) ;
		proc->launch	 ("" )	;
		return	QString::null	;
	}

	m_current = m_sources.count() ;

	m_sources.append (source) ;
	m_names  .append (name  ) ;

	if (title.length() > 12)
		m_titles .append (title.left(10) + " ...") ;
	else	m_titles .append (title ) ;

	QTextBrowser::setSource  (source + "#" + name) ;
	return	m_titles[m_current] ;
}


void	SOHelpPage::blockScrolling
	(	bool		noScroll
	)
{
	m_noScroll	= noScroll ;
}

void	SOHelpPage::ensureCursorVisible ()
{
	if (!m_noScroll)
		QTextBrowser::ensureCursorVisible () ;
}

QString	SOHelpPage::prevpage ()
{
	fprintf
	(	stderr,
		"SOHelpPage::prevpage: at %d of %d\n",
		m_current,
		m_sources.count()
	)	;

	if (m_current > 0)
	{
		m_current -= 1 ;
		QTextBrowser::setSource  (m_sources[m_current] + "#" + m_names[m_current]) ;
		return	m_titles[m_current] ;
	}

	return	m_titles[0] ;
}

QString	SOHelpPage::nextpage ()
{
	fprintf
	(	stderr,
		"SOHelpPage::nextpage: at %d of %d\n",
		m_current,
		m_sources.count()
	)	;


	if (m_current + 1 < m_sources.count())
	{
		m_current += 1 ;
		QTextBrowser::setSource  (m_sources[m_current] + "#" + m_names[m_current]) ;
		return	m_titles[m_current] ;
	}

	return	m_titles[m_current] ;
}

/*  -------------------------------------------------------------------	*/

SOHelpBrowser::SOHelpBrowser
	(	QWidget		*parent,
		SOHelpViewer	*viewer,
		const char 	*name
	)
	:
	QTabWidget	(parent, name),
	m_viewer	(viewer)
{
	m_currentPage	= new SOHelpPage (this, viewer) ;
	addTab (m_currentPage, "") ;

	QToolButton *newtab = new QToolButton (this)	;
	newtab->setIconSet (QIconSet(QPixmap::fromMimeSource("tabopen.png"))) ;
	setCornerWidget (newtab, Qt::TopLeft )		;

	QToolButton *deltab = new QToolButton (this)	;
	deltab->setIconSet (QIconSet(QPixmap::fromMimeSource("tabclose.png"))) ;
	setCornerWidget (deltab, Qt::TopRight)		;

	connect	(newtab, SIGNAL(clicked()), SLOT (openpage ())) ;
	connect	(deltab, SIGNAL(clicked()), SLOT (closepage())) ;

	connect
	(	this,
		SIGNAL	(currentChanged(QWidget *)),
		SLOT	(pagechanged())
	)	;	
}

void	SOHelpBrowser::openpage ()
{
	m_currentPage	= new SOHelpPage (this, m_viewer) ;
	addTab (m_currentPage, "") ;
	showPage (m_currentPage) ;
}

void	SOHelpBrowser::closepage ()
{
	if (count() > 1)
	{
		delete m_currentPage ;
		m_currentPage = (SOHelpPage *)currentPage() ;
	}
}

void	SOHelpBrowser::pagechanged ()
{
	m_currentPage = (SOHelpPage *)currentPage() ;
}

void	SOHelpBrowser::showHelp
	(	const QString	&source,
		const QString	&name,
		const QString	&title
	)
{
	QString l = m_currentPage->showHelp (source, name, title) ;
	if (!l.isNull())
		setTabLabel (m_currentPage, l) ;
}


void	SOHelpBrowser::blockScrolling
	(	bool		noScroll
	)
{
	m_currentPage->blockScrolling (noScroll) ;
}

QString	SOHelpBrowser::source ()
{
	return	m_currentPage->source() ;
}

void	SOHelpBrowser::sync ()
{
	m_currentPage->sync () ;
}

void	SOHelpBrowser::setCursor
	(	const QCursor	&cursor
	)
{
	m_currentPage->setCursor (cursor) ;
}

void	SOHelpBrowser::setColor
	(	const QColor	&color
	)
{
	m_currentPage->setColor (color) ;
}

void	SOHelpBrowser::setUpdatesEnabled
	(	bool		enabled
	)
{
	m_currentPage->viewport()->setUpdatesEnabled (enabled) ;
}

void	SOHelpBrowser::updateContents ()
{
	m_currentPage->updateContents () ;
}

void	SOHelpBrowser::setCursorPosition
	(	int		para,
		int		index
	)
{
	m_currentPage->setCursorPosition (para, index) ;
}

void	SOHelpBrowser::reload ()
{
	m_currentPage->reload () ;
}

bool	SOHelpBrowser::find
	(	const QString	&expr,
		bool		cs,
		bool		wo,
		bool		forward,
		int		*para,
		int		*index
	)
{
	return	m_currentPage->find (expr, cs, wo, forward, para, index) ;
}

void	SOHelpBrowser::prevpage ()
{
	QString l = m_currentPage->prevpage() ;
	if (!l.isNull())
		setTabLabel (m_currentPage, l) ;
}

void	SOHelpBrowser::nextpage ()
{
	QString l = m_currentPage->nextpage() ;
	if (!l.isNull())
		setTabLabel (m_currentPage, l) ;
}
