/***************************************************************************
    file	         : chooser.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include	"chooser.moc"
#else
#include	"chooser.h"
#endif

#include	"browser.h"
#include	"parser.h"
#include	"viewer.h"

#include	<qfile.h>
#include	<qdir.h>
#include	<qtextstream.h>
#include	<qapplication.h>
#include	<qmessagebox.h>
#include	<qlayout.h>
#include	<qpushbutton.h>
#include	<qlabel.h>
#include	<qframe.h>
#include	<qtabwidget.h>
#include	<qlineedit.h>
#include	<qtextbrowser.h>
#include	<qregexp.h>
#include	<qmime.h>
#include	<qheader.h>
#include	<qstatusbar.h>
#include	<qptrstack.h>
#include	<qptrlist.h>

#include	<stdlib.h>
#include	<limits.h>



static	QListViewItem	*lastChild
	(	QListViewItem	*child
	)
{
	while ((child != 0) && (child->nextSibling() != 0))
		child	= child->nextSibling() ;
	return	child	;
}



ContentsItem::ContentsItem
	(	QListView	*parent,
		const QString	&title,
		const QString	&link
	)
	:
	QListViewItem	(parent, lastChild (parent->firstChild()), title),
	m_link		(link)
{
}

ContentsItem::ContentsItem
	(	QListViewItem	*parent,
		const QString	&title,
		const QString	&link
	)
	:
	QListViewItem	(parent, lastChild (parent->firstChild()), title),
	m_link		(link)
{
}



SOHelpChooser::SOHelpChooser
	(	QWidget		*parent,
		SOHelpViewer	*viewer,
		SOHelpBrowser	*browser
	)
	:
	QWidget		(parent),
	m_viewer	(viewer),
	m_browser	(browser)
{
	QVBoxLayout	*topLayout = new QVBoxLayout(this, 0, 6) ;

	m_tabber       = new QTabWidget (this) ;

	m_contentsList = new QListView  (m_tabber) ;
	m_contentsList->addColumn	  (QString::null) ;
	m_contentsList->setRootIsDecorated(true)   ;
	m_contentsList->setSorting	  (-1)	   ;
	m_contentsList->viewport()->installEventFilter(this)   ;

	m_contentsList->header()->hide() ;
	m_contentsList->header()->setStretchEnabled(true) ;
	m_tabber      ->insertTab (m_contentsList, tr("Con&tents")) ;

	m_indexWidget   = new QWidget	  (m_tabber) ;
	QVBoxLayout *l1	= new QVBoxLayout (m_indexWidget) ;
	QHBoxLayout *l2 = new QHBoxLayout (l1) ;

	QLabel	    *t1 = new QLabel	  (tr("Find:"), m_indexWidget) ;
	m_indexEdit	= new QLineEdit	  (m_indexWidget) ;
	m_indexList	= new QListView	  (m_indexWidget) ;
	m_indexList   ->addColumn	  (QString::null) ;
	m_indexList   ->setRootIsDecorated(true)   ;
	m_indexList   ->setSorting	  (-1)	   ;

	m_indexList   ->header()->hide() ;
	m_indexList   ->header()->setStretchEnabled(true) ;

	l2->addWidget (t1) ;
	l2->addWidget (m_indexEdit) ;
	l1->addWidget (m_indexList) ;
	t1->setBuddy  (m_indexEdit) ;

	m_indexList->installEventFilter	  (this) ;

	m_tabber->insertTab (m_indexWidget, tr("&Index")) ;

	QWidget	    *searchTab	  = new QWidget	    (m_tabber) ;
	QGridLayout *searchLayout = new QGridLayout (searchTab, 1, 1, 5, 6) ; 

	QSpacerItem *s1		  = new QSpacerItem (20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed) ;
	searchLayout->addItem (s1, 3, 0) ;

	QLabel	    *l3	= new QLabel	(tr("Searching f&or:"  ), searchTab) ;
	QLabel	    *l4 = new QLabel	(tr("Found &Documents:"), searchTab) ;

	m_searchEdit	= new QLineEdit (searchTab) ;
	m_searchList	= new QListBox  (searchTab) ;

	searchLayout->addWidget (l3, 	       0, 0) ;
	searchLayout->addWidget (m_searchEdit, 1, 0) ;
	searchLayout->addWidget (m_searchList, 5, 0) ;
	searchLayout->addWidget (l4,	       4, 0) ;

	QHBoxLayout *l5 = new QHBoxLayout (0,  0,  6); 
	QSpacerItem *s2 = new QSpacerItem (61, 21, QSizePolicy::Expanding, QSizePolicy::Minimum) ;
	l5->addItem (s2) ;

	QPushButton *searchButton = new QPushButton (tr("&Search"), searchTab) ;
	l5->addWidget (searchButton) ;

	searchLayout->addLayout (l5, 2, 0) ;
	m_tabber    ->insertTab (searchTab, tr("Se&arch")) ;
	topLayout   ->addWidget (m_tabber) ;

	setTabOrder (m_tabber,       m_contentsList) ;
	setTabOrder (m_contentsList, m_indexEdit   ) ;
	setTabOrder (m_indexEdit,    m_indexList   ) ;
	setTabOrder (m_indexList,    m_searchEdit  ) ;
	setTabOrder (m_searchEdit,   searchButton  ) ;
	setTabOrder ( searchButton,  m_searchList  ) ;

	l3->setBuddy(m_searchEdit) ;
	l4->setBuddy(m_searchList) ;

	connect
	(	m_searchEdit,
		SIGNAL	(returnPressed	()),
		searchButton,
		SLOT  	(animateClick	())
	)	;
	connect
	(	searchButton,
		SIGNAL	(clicked	()),
		SLOT  	(startSearch	())
	)	;

	connect
	(	m_contentsList,
		SIGNAL	(clicked		(QListViewItem *)),
		SLOT	(showContentsTopic	(QListViewItem *)) 
	)	;
//	connect
//	(	m_contentsList,
//		SIGNAL	(currentChanged		(QListViewItem *)),
//		SLOT	(showContentsTopic	(QListViewItem *)) 
//	)	;
//	connect
//	(	m_contentsList,
//		SIGNAL	(returnPressed		(QListViewItem *)),
//		SLOT	(showContentsTopic	(QListViewItem *)) 
//	)	;
	connect
	(	m_indexEdit,
		SIGNAL	(returnPressed		()),
		SLOT	(showIndexTopic		()) 
	)	;
	connect
	(	m_indexEdit,
		SIGNAL	(textChanged		(const QString &)),
		SLOT	(scanIndex		(const QString &)) 
	)	;

	connect
	(	m_indexList,
		SIGNAL	(clicked		(QListViewItem *)),
		SLOT	(showContentsTopic	(QListViewItem *)) 
	)	;
	connect
	(	m_indexList,
		SIGNAL	(currentChanged		(QListViewItem *)),
		SLOT	(showContentsTopic	(QListViewItem *)) 
	)	;
	connect
	(	m_indexList,
		SIGNAL	(returnPressed		(QListViewItem *)),
		SLOT	(showContentsTopic	(QListViewItem *)) 
	)	;

	connect
	(	m_searchList,
		SIGNAL	(selected		(int)),
		SLOT	(showSearch		(int))
	)	;
	connect
	(	m_searchList,
		SIGNAL	(highlighted		(int)),
		SLOT	(showSearch		(int))
	)	;

	m_display = false ;
}

void	SOHelpChooser::initialise ()
{
	extern	QString	mwHelpDir ;

	fprintf
	(	stderr,
		"SOHelpChooser::initialise: mwHelpDir=%s\n",
		mwHelpDir.latin1()
	)	;

#if	0
	QMimeSourceFactory *cf = new QMimeSourceFactory  () ;

	cf->addFilePath	     (QDir::convertSeparators(mwHelpDir)) ;
	cf->setExtensionType ("html",  "text/html;charset=UTF-8") ;
	cf->setExtensionType ("gif",   "image/gif" ) ;
	cf->setExtensionType ("png",   "image/png" ) ;
	cf->setExtensionType ("jpg",   "image/jpeg") ;
	cf->setExtensionType ("jpeg",  "image/jpeg") ;

	m_browser->setMimeSourceFactory	(cf) ;
#endif

	loadHelp () ;
	m_viewer->statusBar()->message( tr( "Done" ), 3000 );
}

/*  loadHelp	: Load help information					*/
/*  (returns)	: void		:					*/

void	SOHelpChooser::loadHelp ()
{
	extern QStringList 		xmlFiles	;
	QStringList::iterator 		it		;
	QAsciiDict<SOHelpIndex>		dictionary	;

	/* Scan the XML file list. Each is parsed and the contents and	*/
	/* keywords extracted. Unlike Qt Assistant, we do this every	*/
	/* time, since the volume of help is quite small.		*/
	for (it = xmlFiles.begin() ; it != xmlFiles.end() ; it++)
	{
		fprintf
		(	stderr,
			"SOHelpChooser::loadHelp: [%s]\n",
			(*it).latin1()
		)	;

		QFile	file	(*it)	;
		if (!file.exists()) continue ;

		QXmlInputSource 	source  (file)	;
		QXmlSimpleReader	reader		;
		SOHelpParser		*handler	;
		ContentsItem		*book		= 0 ;

		if (xmlFiles.count() > 1)
		{
			book 	= new ContentsItem
				  	(	m_contentsList,
						"BOOK",
						QString::null
					)	;

			handler	= new SOHelpParser
					(	book,
						m_indexList,
						dictionary
					)	;
		}
		else	handler	= new SOHelpParser
					(	m_contentsList,
						m_indexList,
						dictionary
					)	;

		reader.setContentHandler (handler) ;
		reader.setErrorHandler   (handler) ;

		bool ok = reader.parse	 (source) ;
		file.close() 	;

		if (book != 0)
			if (handler->bookTitle().isEmpty())
				book->setText (0, "Untitled Book") ;
			else	book->setText (0, handler->bookTitle()) ;

		delete	handler	;

		if (!ok)
		{
			fprintf
			(	stderr,
				"SOHelpChooser::buildContentList: %s: parse error\n",
				(*it).latin1()
			)	;
			continue;
		}
	}

	extern	QString	mwHelpDir ;
	m_searchIndex = new SOSearcher (mwHelpDir, dictionary) ;

	m_indexList->sort	   () ;
	qApp->processEvents() ;
}


void	SOHelpChooser::showIndexTopic
	(	QListViewItem		*item
	)
{
	if (item != 0)
	{
		m_indexEdit->blockSignals (true) 		;
		m_indexEdit->setText	  (item->text(0))	;
		m_indexEdit->blockSignals (false)		;

		extern	QString	mwHelpDir ;
		m_viewer->showLink (mwHelpDir + ((ContentsItem *)item)->link(), item->text(0)) ;
	}
}

void	SOHelpChooser::showIndexTopic ()
{
	showIndexTopic (m_indexList->currentItem()) ;
}

void	SOHelpChooser::scanIndex
	(	const QString	&scanFor
	)
{
	QListViewItem	*i	= m_indexList->firstChild() ;
	QString		sfl	= scanFor.lower() ;

//	fprintf
//	(	stderr,
//		"SOHelpChooser::scanIndex: [%s]\n",
//		sfl.latin1()
//	)	;

	while (i != 0)
	{
		QString entry = i->text(0) ;

		if (entry.length() >= sfl.length())
			if (entry.left(sfl.length()).lower() == sfl)
			{
				m_indexList->setCurrentItem (i) ;
				m_indexList->setSelected    (i, true) ;
				break	;
			}
		i	= i->nextSibling() ;
	}
}

bool	SOHelpChooser::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if ((o == 0) || (e == 0))
		return	true	;

	if ((o == m_contentsList->viewport()) && (e->type() == QEvent::MouseButtonPress))
	{
		QMouseEvent *m = (QMouseEvent *)e ;
		m_display = (m->state() & (Qt::ShiftButton|Qt::ControlButton)) == (Qt::ShiftButton|Qt::ControlButton) ;
	}

	if ((o == m_indexList) && (e->type() == QEvent::KeyPress))
	{
		QKeyEvent 	*k	= (QKeyEvent *)e ;
		QListViewItem	*i	= m_indexList->currentItem () ;
		bool	  	u	= false	;

		switch (k->key())
		{

			case Key_Up	:
				{
					QListViewItem *p = m_indexList->firstChild() ;

					while ((p != 0) && (p->nextSibling() != i))
						p = p->nextSibling() ;

					if (p != 0)
					{
						m_indexList->setCurrentItem (p) ;
						m_indexList->setSelected    (p, true) ;
						u = true	;
					}
				}
				break	;


			case Key_Down	:
				if ((i = i->nextSibling()) != 0)
				{
					m_indexList->setCurrentItem (i) ;
					m_indexList->setSelected    (i, true) ;
					u = true ;
				}
				break	;

			case Key_Next	:
			case Key_Prior	:
				QApplication::sendEvent (m_indexList, e) ;
				u = true ;
				break	;

			default	:
				return	QWidget::eventFilter (o, e) ;
		}

		if (u)
		{
			fprintf
			(	stderr,
				"SOHelpChooser::eventFilter: set [%s]\n",
				i->text(0).latin1()
			)	;
			m_indexEdit->blockSignals	(true );
			m_indexEdit->setText		(i->text(0)) ;
			m_indexEdit->blockSignals	(false);
		}

		return	true	;
	}

	return	QWidget::eventFilter (o, e) ;
}



void	SOHelpChooser::showContentsTopic
	(	QListViewItem		*item
	)
{
	extern	QString	mwHelpDir ;
	if (item == 0) return ;
	QString	link	= ((ContentsItem *)item)->link() ;
	if (link.isEmpty()) return ;

	if (m_display)
	{	m_viewer->statusBar()->message(link) ;
		fprintf	(stderr, "showContentsTopic: %s\n", link.latin1()) ;
	}
	else	m_viewer->statusBar()->clear  () ;
	m_viewer->showLink (mwHelpDir + link, item->text(0)) ;
}


void	SOHelpChooser::startSearch ()
{
	/* Pick up the text the user has entered, and initially		*/
	/* sanitise it bu converting single-quotes and back-ticks to	*/
	/* double-quotes. Makes life easier below.			*/
	QString	string	= m_searchEdit->text() ;
	string	= string.replace ("\'", "\"")  ;
	string	= string.replace ("`",  "\"")  ;

	QString buf	= string ;

	string	= string.replace ("-", " " );
	string	= string.replace (QRegExp ("\\s[A-Za-z0-9]?\\s"), " ") ;

	m_searchTerms = QStringList::split (" ", string) ;

	/* Convert all the words into lower case and strip out any	*/
	/* double-quotes (this screws up if people embed double-quotes	*/
	/* within a word but we can ignore that).			*/
	for
	(	QStringList::iterator
			iter  = m_searchTerms.begin() ;
			iter != m_searchTerms.end  () ;
			++iter
	)
	{
		(*iter) = (*iter).lower   () ;
		(*iter) = (*iter).replace ( "\"", "" ) ;
	}

	QStringList	quoted ;

	/* Next look for quoted sequences. The main part of the search	*/
	/* will use all the words, including those in quoted sequences,	*/
	/* in order to minimise the number of documents that need to be	*/
	/* scanned for the quoted sequences.				*/
	if (string.contains ('\"'))
	{
		if ((string.contains('\"')) % 2 != 0)
		{
			QMessageBox::warning
			(	this,
				tr("Help"),
				tr("Missing quote marks")
			)	;
			return	;
		}

		int	begin	= 0 ;
		int	end	= 0 ;
		QString s	;

		begin = string.find ('\"') ;
		while (begin >= 0)
		{
			begin	 += 1 ;
			end	  = string.find ('\"', begin) ;

			QString	s = string.mid  (begin, end - begin) ;
			s 	  = s.lower () ;
			s 	  = s.simplifyWhiteSpace () ;

			quoted.append (s) ;

			begin	  = string.find ('\"', end + 1) ;
		}
	}


	/* We can now to the search. Pass the list of search terms	*/
	/* (which is used first in the search) and then the list of	*/
	/* quoted sequences, which will be applied to the matched	*/
	/* documents.							*/
	setCursor (waitCursor) ;
	m_searchDocs = m_searchIndex->search (m_searchTerms, quoted) ;

	m_viewer->statusBar()->message
	(	QString("%1 matching documents found").arg(m_searchDocs.count()),
		5000
	)	;

	setCursor (arrowCursor) ;

	m_searchList->clear() ;

	for
	(	QStringList::iterator iter  = m_searchDocs.begin() ;
				      iter != m_searchDocs.end  () ;
				    ++iter
	)
	{
		m_searchList->insertItem (m_searchIndex->getDocumentTitle (*iter)) ;
	}

	bool	inQuotes = false ;
	QString s	 = ""	 ;
	m_searchTerms.clear ()	 ;

	for (int i = 0 ; i < (int)buf.length(); i += 1)
	{
		if (buf[i] == '\"')
		{
			inQuotes = !inQuotes ;
			s	 = s.simplifyWhiteSpace() ;
			if (!s.isEmpty()) m_searchTerms.append (s) ;
			s	 = "" ;
			continue ;
		}

		if ((buf[i] == ' ') && !inQuotes)
		{
			s = s.simplifyWhiteSpace() ;
			if (!s.isEmpty()) m_searchTerms.append (s) ;
			s = ""   ;
			continue ;
		}

		s += buf[i] ;
	}

	if (!s.isEmpty()) m_searchTerms.append(s) ;
}

/*  SOHelpChooser							*/
/*  showSearch	: Show a search result					*/
/*  page	: int		: Search page number			*/
/*  (returns)	: void		:					*/

void	SOHelpChooser::showSearch
	(	int		page
	)
{
	extern	QString	mwHelpDir ;
	QString	path	= mwHelpDir + m_searchDocs[page] ;

	m_browser->blockScrolling (true) ;
	m_browser->setCursor	  (waitCursor)  ;

	if (m_browser->source() == path)
		m_browser->reload   ()	   ;
	else	m_viewer ->showLink (path, m_searchList->currentText()) ;

	m_browser->sync		  () ;
	m_browser->setCursor	  (arrowCursor) ;
	m_browser->setUpdatesEnabled (false) ;

	int minPara	= INT_MAX ;
	int minIndex	= INT_MAX ;
	

	for
	(	QStringList::ConstIterator
				iter  = m_searchTerms.begin() ;
				iter != m_searchTerms.end  () ;
				++iter
	)
	{
		int	para 	= 0 ;
		int	index	= 0 ;
		bool	found	= m_browser->find (*iter, false, true, true, &para, &index) ;

		while (found)
		{
			if (para < minPara)
			{
				minPara	 = para	 ;
				minIndex = index ;
			}

			m_browser->setColor (QColor("red")) ;
			found = m_browser->find (*iter, false, true, true) ;
		}
	}

	m_browser->blockScrolling    (false) ;
	m_browser->setUpdatesEnabled (true ) ;
	m_browser->setCursorPosition (minPara, minIndex) ;
	m_browser->updateContents    () ;
}

void	SOHelpChooser::showContents ()
{
	m_tabber->showPage (m_contentsList) ;
}

void	SOHelpChooser::showIndex ()
{
	m_tabber->showPage (m_indexWidget ) ;
}

bool	SOHelpChooser::setHelpKey
	(	const QString	&link,
		ContentsItem	*parent
	)
{
	fprintf
	(	stderr,
		"SOHelpChooser::setHelpKey: %s\n",
		link.latin1()
	)	;

	for
	(	ContentsItem *item = (ContentsItem *)parent->firstChild() ;
		item != 0 ;
		item  = (ContentsItem *)item->nextSibling()
	)
	{
		if (item->link() == link)
		{
			m_contentsList->ensureItemVisible (item) ;
			m_contentsList->setCurrentItem    (item) ;
			return	true	;
		}

		if (setHelpKey (link, item))
			return	true	;
	}

	return	false	;
}

void	SOHelpChooser::setHelpKey
	(	const QString	&link
	)
{
	for
	(	ContentsItem *item = (ContentsItem *)m_contentsList->firstChild() ;
		item != 0 ;
		item  = (ContentsItem *)item->nextSibling()
	)
	{
		if (item->link() == link)
		{
			m_contentsList->ensureItemVisible (item) ;
			m_contentsList->setCurrentItem    (item) ;
			return	;
		}

		if (setHelpKey (link, item))
			return	;
	}
}
