/***************************************************************************
    file	         : index.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"index.h"

#include	<qfile.h>
#include	<qstringlist.h>
#include	<qasciidict.h>
#include	<qregexp.h>



SOSearcher::SOSearcher
	(	const QString			&hd,
		const QAsciiDict<SOHelpIndex>	&dict
	)
	:
	m_helpDir	(hd),
	m_dict		(dict)
{
}


QStringList
	SOSearcher::search
	(	const QStringList	&words,
		const QStringList	&phrases
	)
{
	QStringList	results	 ;
	SOHelpTermList	termList ;
	termList.setAutoDelete (true) ;

	/* First stage is to search for words (including wildcards).	*/
	/* Since we are implictely doing an AND then if a word is not	*/
	/* found then return.						*/
	for (QStringList::ConstIterator it = words.begin() ; it != words.end() ; ++it)
	{

		/* Where there is a wildcard, do a wildcard match on	*/
		/* each word and accumulate the distinct documents over	*/
		/* all matches.						*/
		if ((*it).contains('*'))
		{
			QRegExp				regexp	(*it, false, true) ;
			QValueList<SOHelpDocument>	docList	;
			QDict<void>			docMap	;

			QAsciiDictIterator<SOHelpIndex>	iter(m_dict) ;
			for ( ; iter.current() ; ++iter)
			{
				if (regexp.search (iter.currentKey()) < 0)
					continue ;

				QValueList<SOHelpDocument>::iterator scan = iter.current()->m_documents.begin() ;
				while (scan != iter.current()->m_documents.end())
				{
					if (docMap.find (*(*scan).m_document) == 0)
					{	docMap .insert (*(*scan).m_document, (void *)1) ;
						docList.append (*scan) ;
					}
					++scan	;
				}
			}

			termList.append (new SOHelpTerm (docList)) ;

			continue;
		}

		/* No wildcard is easy, just look up in the dictionary	*/
		/* and append to the list of found.			*/
		SOHelpIndex	*e ;
		if ((e = m_dict[(*it).latin1()]) != 0)
		{
			termList.append (new SOHelpTerm (e->m_documents)) ;
			continue ;
		}
	
		return	QStringList() ;
	}

	if (!termList.count())
		return	QStringList() ;

	/* Next stage is to merge the document lists to contain just	*/
	/* those documents that contain all the words. Start with the	*/
	/* document list for the first matched term, then check against	*/
	/* each following term.						*/
	QValueList<SOHelpDocument> docList = termList.first()->m_documents ;
	termList.removeFirst () ;

	for (SOHelpTerm *term = termList.first() ; term != 0 ; term = termList.next())
	{

		QValueList<SOHelpDocument>		docs	= term->m_documents ;
		QValueList<SOHelpDocument>::iterator	scan	= docList.begin()   ;

		while (scan != docList.end())
		{
			bool	found	= false ;

			QValueList<SOHelpDocument>::ConstIterator iter ;
			for (iter = docs.begin() ; iter != docs.end() ; ++iter)
				if ((*scan).m_document == (*iter).m_document)
				{
					(*scan).m_count += (*scan).m_count ;
					found	= true ;
					break	;
				}

			if (!found)
			{	scan	= docList.remove (scan) ;
				continue;
			}

			++scan	;
		}
	}

	qHeapSort (docList) ;

	/* If there are no search phrases then we can just return the	*/
	/* current document list.					*/
	if (phrases.isEmpty())
	{
		QValueList<SOHelpDocument>::iterator iter ;
		for (iter = docList.begin() ; iter != docList.end() ; ++iter)
			results.append (*(*iter).m_document) ;
		return	results ;
	}

	QValueList<SOHelpDocument>::iterator iter ;
	for (iter = docList.begin() ; iter != docList.end() ; ++iter)
	{
		const QString &fileName =  *(*iter).m_document ;
		if (findPhrases (phrases, fileName))
			results << fileName ;
	}

	return	results ;
}

QString	SOSearcher::getDocumentTitle
	(	const QString	&fileName
	)
{
	{
		QString	*title	= m_titleMap.find (fileName) ;
		if (title != 0) return *title ;
	}

	QFile		file	(m_helpDir + fileName) ;
	if (!file.open (IO_ReadOnly)) return fileName  ;

	QTextStream	stream	(&file) ;
	QString		text	= stream.read () ;

	int		start	;
	int		end	;

	if ((start = text.find ("<title>",  0,	       false)) < 0)
		return	fileName ;
	if ((end   = text.find ("</title>", start + 7, false)) < 0)
		return	fileName ;

	QString	title	= text.mid (start + 7, end - start - 7) ;
	m_titleMap.insert (fileName, new QString(title)) ;
	return	title	;
}

bool	SOSearcher::findPhrases
	(	const QStringList	&patterns,
		const QString		&htmlName
	)
{
	int	dot	= htmlName.findRev ('.')	;
	QString	wlName	= htmlName.left(dot) + ".wl"	;

	static	QDict<QValueList<int> >	wordLists	;
	QValueList<int>			*wordList	;

	if ((wordList = wordLists.find(wlName)) == 0)
	{
		wordList = new QValueList<int>		;
		wordLists.insert (wlName, wordList)	;

		QFile wlFile (m_helpDir + wlName)	;
		if (!wlFile.open (IO_ReadOnly))
		{
			fprintf
			(	stderr,
				"SOSearcher::findPhrases: %s: cannot open\n",
				wlName.latin1()
			)	;
			return	false	;
		}

		QTextStream wlStream(&wlFile) ;
		QString	    line ;

		while (!(line = wlStream.readLine()).isNull())
		{
			wordList->append (line.toInt()) ;
		}
	}


	for
	(	QStringList::ConstIterator pattIter = patterns.begin() ; 
		pattIter != patterns.end() ;
		++pattIter
	)
	{
		QStringList	wAsText = QStringList::split(' ', *pattIter) ;
		QValueList<int>	wAsId	;

		for
		(	QStringList::ConstIterator wordIter = wAsText.begin() ; 
			wordIter != wAsText.end() ;
			++wordIter
		)
		{
			SOHelpIndex	*e ;
			if ((e = m_dict[(*wordIter).latin1()]) != 0)
			{
//				fprintf
//				(	stderr,
//					"SOSearcher::findPhrases: [%s]->[%d]\n",
//					(*wordIter).latin1(),
//					e->m_id
//				)	;
				wAsId.append (e->m_id) ;
			}
		}

		for
		(	int offset = 0 ;
			offset < (int)wordList->count() - (int)wAsId.count() ;
			offset += 1
		)
		{
			bool ok = true ;
			for (int wn = 0 ; wn < (int)wAsId.count() ; wn += 1)
				if ((*wordList)[offset + wn] != wAsId[wn])
				{	ok	= false	;
					break	;
				}

			if (ok)
			{
				fprintf
				(	stderr,
					"SOSearcher::findPhrases: at [%d] in [%s]\n",
					offset,
					htmlName.latin1()
				)	;
				return	true	;
			}
		}
	}

	return	false	;
}

