/***************************************************************************
    file	         : index.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__SOINDEX_H
#define 	__SOINDEX_H

#include 	<qstringlist.h>
#include 	<qasciidict.h>
#include	<qdict.h>


/*  SOHelpDocument							*/
/*  --------------							*/
/*  Class used to store basic document information when searching for	*/
/*  keywords and suchlike. Contains the document name and a count of	*/
/*  the number of times it is referenced. Comparisom operators are	*/
/*  implemented for list sorting based on the reference count.		*/

struct	SOHelpDocument
{
	const QString	*m_document	;
	int		m_count		;

	SOHelpDocument
		(	const QString	*document,
			int		count
		)
		:
		m_document	(document),
		m_count		(count)
	{
	}

	SOHelpDocument ()
		:
		m_document	(0),
		m_count		(0)
	{
	}

	bool	operator ==
		(	const SOHelpDocument	&doc
		)
		const
	{
		return m_document == doc.m_document ;
	}

	bool	operator <
		(	const SOHelpDocument	&doc
		)
		const
	{
		return	m_count > doc.m_count	;
	}
	bool	operator <=
		(	const SOHelpDocument	&doc
		)
		const
	{
		return	m_count >= doc.m_count	;
	}
	bool	operator >
		(	const SOHelpDocument	&doc
		)
		const
	{
		return	m_count < doc.m_count	;
	}
	bool	operator >=
		(	const SOHelpDocument	&doc
		)
		const
	{
		return	m_count <= doc.m_count	;
	}
}	;


/*  SOHelpIndex								*/
/*  -----------								*/

class	SOHelpIndex
{
public	:

	QValueList<SOHelpDocument>	m_documents	;
	int				m_id		;

	inline	SOHelpIndex
		(	int	id
		)
		:
		m_id	(id)
	{
	}

        void	append
		(	const	QString	*document
		)
	{
		m_documents.append (SOHelpDocument (document, 1)) ;
	}
}	;

/*  SOSearcher								*/
/*  ----------								*/
/*  This class is loosely modelled on the Index class in QtAssistant	*/
/*  and encapsulates the keywork search function.			*/

class	SOSearcher
{
	QString			m_helpDir	;
	QAsciiDict<SOHelpIndex>	m_dict		;

	bool		findPhrases
			(	const QStringList	&,
				const QString		&
			)	;

	QDict<QString>	m_titleMap	;

public	:

	SOSearcher
	(	const QString			&helpDir,
		const QAsciiDict<SOHelpIndex>	&dict
	)	;

	QStringList	search
			(	const QStringList	&,
				const QStringList	&
			)	;

	QString	getDocumentTitle
			(	const QString		&
			)	;
}	;

class	SOHelpTerm
{
public	:
	QValueList<SOHelpDocument>	m_documents	;

	SOHelpTerm
		(	const QValueList<SOHelpDocument> &docs
		)
		:
		m_documents	(docs)
	{
	}
}	;

typedef	QPtrList<SOHelpTerm>	SOHelpTermList	;

#endif		// __SOINDEX_H

