/***************************************************************************
    file	         : main_qt3.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<stdio.h>

#include	<qapplication.h>
#include	<qstringlist.h>
#include	<qsettings.h>
#include	<qdir.h>
#include	<qguardedptr.h>

#include	"viewer.h"
#include	"tk_config.h"

#include	"images.cpp"

#ifdef		_WIN32
#include	<windows.h>
#include	<wincon.h>
#endif


QStringList	xmlFiles	;
QString		mwHelpDir	;
QString		mwHelpKey	;
uint		mwHelpPort	;

int	main
	(	int	argc,
		char	**argv
	)
{
	QApplication	app (argc, argv) ;

	argc	-= 1	;
	argv	+= 1	;

	while (argc > 0)
	{
		if (qstrcmp (argv[0], "--helpdir") == 0)
		{
			if (argc > 1)
			{
				mwHelpDir = argv[1] ;
				argc	 -= 2 ;
				argv	 += 2 ;
				continue ;
			}
		}

		if (qstrcmp (argv[0], "--helpkey") == 0)
		{
			if (argc > 1)
			{
				mwHelpKey = argv[1] ;
				argc	 -= 2 ;
				argv	 += 2 ;
				continue ;
			}
		}

		if (qstrcmp (argv[0], "--port") == 0)
		{
			if (argc > 1)
			{
				mwHelpPort= QString(argv[1]).toInt() ;
				argc	 -= 2 ;
				argv	 += 2 ;
				continue ;
			}
		}

		argv	+= 1	;
		argc	-= 1	;
	}

	TKConfig *config = TKConfig::getConfig () ;
	config->setApplication	("rekallManual" ) ;

	QDir		dir	(mwHelpDir) ;
	QStringList	xml	= dir.entryList("*.xml", QDir::Files, QDir::Name) ;

	for (unsigned int idx = 0 ; idx < xml.count() ; idx += 1)
		xmlFiles.append (dir.absFilePath(xml[idx])) ;

	QMimeSourceFactory::defaultFactory()->addFactory (factory) ;

	QGuardedPtr<SOHelpViewer> mw  = new SOHelpViewer
					(	mwHelpPort,
						0,
						"Rekall Manual",
						Qt::WDestructiveClose
					)	;


	if (config->readBoolEntry ("Maximized", false))
		mw->showMaximized() ;
	else	mw->show	 () ;

	qApp->processEvents	 () ;

	if (!mw) return 0  ;

	if (!mwHelpKey.isEmpty())
		mw->setHelpKey (mwHelpKey) ;

	app.connect (&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit())) ;

#ifdef _WIN32
	if ( getenv( "MWCONSOLE" ) == QString( "YES" ) )
		if ( AllocConsole() )
		{
			freopen( "CON", "w", stdout );
			freopen( "CON", "w", stderr );
			fprintf( stderr, "Starting Rekall Manual ......\n" );
		}
#endif
	fprintf
	(	stderr,
		"FTHelp: cwd     =%s\n"
		"        helpdir =%s\n"
		"        helpkey =%s\n"
		"        helpport=%d\n",
		QDir::current().absPath().latin1(),
		mwHelpDir.latin1(),
		mwHelpKey.latin1(),
		mwHelpPort
	)	;

	return	app.exec() ;
}
