/***************************************************************************
    file	         : parser.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"parser.h"
#include	"chooser.h"
#include	"index.h"



SOHelpParser::SOHelpParser
	(	QListView		*listContents,
		QListView		*listIndex,
		QAsciiDict<SOHelpIndex>	&dictionary
	)
	:
	QXmlDefaultHandler	(),
	m_contentsList		(listContents),
	m_contentsPart		(0),
	m_indexList		(listIndex),
	m_dictionary		(dictionary)
{
	SOHelpParser::startDocument()	;
}

SOHelpParser::SOHelpParser
	(	QListViewItem		*partContents,
		QListView		*listIndex,
		QAsciiDict<SOHelpIndex>	&dictionary
	)
	:
	QXmlDefaultHandler	(),
	m_contentsList		(0),
	m_contentsPart		(partContents),
	m_indexList		(listIndex),
	m_dictionary		(dictionary)
{
	SOHelpParser::startDocument()	;
}

bool	SOHelpParser::startDocument()
{
	m_currentKeyword = 0		;
	m_errors	 = ""		;

	return	true	;
}

bool	SOHelpParser::startElement
	(	const QString		&,
		const QString		&,
		const QString		&qname,
		const QXmlAttributes	&attr
	)
{
//	fprintf
//	(	stderr,
//		"SOHelpParser::startElement: [%s]\n",
//		qname.latin1()
//	)	;

	if (qname == "help")
		return	true	;

	if (qname == "contents")
	{
		m_bookTitle	= attr.value("title") ;
		return	true	;
	}

	if (qname == "section")
	{
		if (m_currentContents.count() == 0)
		{
			ContentsItem *item ;

			if (m_contentsList != 0)
				item	= new ContentsItem
					  (	m_contentsList,
						attr.value("title"),
						attr.value("ref")
					  )	;
			else	item	= new ContentsItem
					  (	m_contentsPart,
						attr.value("title"),
						attr.value("ref")
					  )	;

			m_currentContents.push (item) ;
			return	true	;
		}

		m_currentContents.push
		(
			new ContentsItem
			(	
				m_currentContents.top(),
				attr.value("title"),
				attr.value("ref")
			)
		)	;
		return	true	;
	}

	if (qname == "index")
		return	true	;

	if (qname == "item")
	{
		if (m_currentIndex.count() == 0)
		{
			m_currentIndex.push
			(
				new ContentsItem
				(	m_indexList,
					attr.value("title"),
					attr.value("ref")
				)
			)	;
			return	true	;
		}

		m_currentIndex.push
		(
			new ContentsItem
			(	
				m_currentIndex.top(),
				attr.value("title"),
				attr.value("ref")
			)
		)	;
		return	true	;
	}

	if (qname == "keyfiles")
		return	true	;

	if (qname == "keyfile")
	{
		m_keywordFiles.append (new QString(attr.value("ref"))) ;
		return	true	;
	}

	if (qname == "keywords")
		return	true	;

	if (qname == "keyword")
	{
		m_currentKeyword = new SOHelpIndex(attr.value("id").toInt()) ;
		m_dictionary.insert (attr.value("word"), m_currentKeyword) ;
		return	true	;
	}


	if (qname == "keyoccurs")
	{
		if (m_currentKeyword != 0)
		{
			m_currentKeyword->append
			(	m_keywordFiles.at(attr.value("id").toInt())
			)	;
			return	true	;
		}
	}

	return	false	;
}

bool	SOHelpParser::endElement
	(	const QString		&,
		const QString		&,
		const QString		&qname
	)
{
//	fprintf
//	(	stderr,
//		"SOHelpParser::endElement: [%s]\n",
//		qname.latin1()
//	)	;

	if (qname == "help")
		return	true	;

	if (qname == "contents")
		return	true	;

	if (qname == "section")
	{
		m_currentContents.pop ();
		return	true	;
	}

	if (qname == "index")
		return	true	;

	if (qname == "item")
	{
		m_currentIndex   .pop ();
		return	true	;
	}

	if (qname == "keyfiles")
		return	true	;

	if (qname == "keyfile")
		return	true	;

	if (qname == "keywords")
		return	true	;

	if (qname == "keyword")
	{
		m_currentKeyword = 0 ;
		return	true	;
	}

	if (qname == "keyoccurs")
		return	true	;

	return	false	;
}

bool	SOHelpParser::characters
	(	const QString		&text
	)
{
	return	true	;
}

bool	SOHelpParser::fatalError
	(	const QXmlParseException	&exception
	)
{
	m_errors += 
		QString("Fatal error parsing '%1': at line %2, column %3\n")
			.arg(exception.message	   ())
			.arg(exception.lineNumber  ())
			.arg(exception.columnNumber())
		;

	return	QXmlDefaultHandler::fatalError (exception) ;
}
