/***************************************************************************
    file	         : viewer.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<stdlib.h>


#include	<qdir.h>
#include	<qapplication.h>
#include	<qfileinfo.h>
#include	<qmessagebox.h>
#include	<qsettings.h>
#include	<qmenubar.h>
#include	<qstatusbar.h>
#include	<qaction.h>
#include	<qpopupmenu.h>
#include	<qtoolbar.h>
#include	<qpixmap.h>
#include	<qobjectlist.h>
#include	<qtimer.h>
#include	<qdragobject.h>
#include	<qsplitter.h>

#include	"chooser.h"
#include	"browser.h"

#if		__KB_AUTOMOC
#include	"viewer.moc"
#else
#include	"viewer.h"
#endif

#include	"tk_config.h"

#if		QT_VERSION >= 330
#define		DESCRTEXT
#else
#define		DESCRTEXT		"",
#endif

SOHelpViewer::SOHelpViewer
	(	uint		port,
		QWidget		*parent,
		const char	*name,
		WFlags		flags
	)
	:
	QMainWindow (parent, name, flags)
{
	QAction	*exit	= new QAction
			  (	DESCRTEXT
				QIconSet(),
				tr("E&xit"),
			  	0,
				this
			  )	;

	QAction	*prev	= new QAction
			  (	DESCRTEXT
				QIconSet(QPixmap::fromMimeSource("previous.png")),
				tr("&Previous"),
			  	tr("Alt+Left"),
				this
			  )	;

	QAction	*next	= new QAction
			  (	DESCRTEXT
				QIconSet(QPixmap::fromMimeSource("next.png")),
				tr("&Next"),
				tr("Alt+Right"),
				this
			  )	;

	QAction	*about	= new QAction
			  (	DESCRTEXT
				QIconSet(),
				tr("About"),
				0,
				this
			  )	;

	QToolBar	*tb		= new QToolBar   (QString::null, this, DockTop) ;
	QMenuBar	*mb		= menuBar() ;
	QPopupMenu	*fileMenu	= new QPopupMenu (this) ;
	QPopupMenu	*helpMenu	= new QPopupMenu (this) ;

	prev	->addTo (tb) ;
	next	->addTo (tb) ;

	exit	->addTo (fileMenu) ;
	about	->addTo (helpMenu) ;

	mb	->insertItem (tr("&File"), fileMenu, 0) ;
	mb	->insertItem (tr("&Help"), helpMenu, 1) ;

	m_splitter = new QSplitter (this) ;
	setCentralWidget  (m_splitter) ;

	m_browser = new SOHelpBrowser (m_splitter, this) ;
//	m_browser->setFrameStyle(QFrame::Panel | QFrame::Sunken);

	m_chooser = new SOHelpChooser (m_splitter, this, m_browser);
	m_splitter->moveToFirst (m_chooser) ;

	connect
	(	about,
		SIGNAL	(activated()),
		SLOT	(about	  ())
	)	;
	connect
	(	prev,
		SIGNAL	(activated()),
		m_browser,
		SLOT	(prevpage ())
	)	;
	connect
	(	next,
		SIGNAL	(activated()),
		m_browser,
		SLOT	(nextpage ())
	)	;
	connect
	(	exit,
		SIGNAL	(activated()),
		qApp,
		SLOT	(closeAllWindows())
	)	;
	connect
	(	m_browser,
		SIGNAL	(highlighted(const QString &)),
		statusBar(),
		SLOT	(message    (const QString &))
	)	;

	setCaption (tr("Rekall Manual")) ;


	TKConfig *config = TKConfig::getConfig() ;

	if (!config->readBoolEntry ("Maximized", false))
	{
		QRect	r ;
		int	x = config->readNumEntry("X", r.x     ());
		int	y = config->readNumEntry("Y", r.y     ());
		int	w = config->readNumEntry("W", r.width ());
		int	h = config->readNumEntry("H", r.height());

		if (x < 0) x = 0 ;
		if (y < 0) y = 0 ;

		r.setX	   (x) ;
		r.setY	   (y) ;
		r.setWidth (w) ;
		r.setHeight(h) ;

		setGeometry(r) ;
	}

	QValueList<int> sizes = config->readIntListEntry ("Split") ;
	if ((sizes.count() > 0) && (sizes[0] > 50))
		m_splitter->setSizes (sizes) ;

	m_chooser->initialise () ;

	if (port > 0)
	{
		m_socket.connectToHost ("127.0.0.1", port) ;
		connect	(&m_socket,   SIGNAL(connectionClosed()), SLOT(slotLinkClosed ())) ;
		connect	(&m_socket,   SIGNAL(readyRead	     ()), SLOT(slotReadyRead  ())) ;
	}
}

SOHelpViewer::~SOHelpViewer ()
{
}

bool	SOHelpViewer::close
	(	bool		andDel
	)
{
	saveSettings();
	return	QMainWindow::close (andDel) ;
}

void	SOHelpViewer::about ()
{
	QString about_text = "Rekall Manual ........." ;
	QMessageBox mb (this) ;
	mb.setText	(about_text);
	mb.setCaption	(tr("Rekall")) ;
	mb.setIcon	(QMessageBox::NoIcon)  ;
	mb.exec		() ;
}

void	SOHelpViewer::showLink
	(	const QString	&link,
		const QString	&title
	)
{
//	fprintf
//	(	stderr,
//		"SOHelpViewer::showLink: [%s]\n",
//		link.latin1()
//	)	;

	QStringList	bits = QStringList::split('#', link) ;
	QFileInfo	fi(bits[0]) ;

	if (fi.exists()) m_browser->showHelp (bits[0], bits[1], title) ;
	m_browser->setFocus () ;
}

void	SOHelpViewer::setHelpKey
	(	const QString	&key
	)
{
	fprintf
	(	stderr,
		"SOHelpViewer::setHelpKey: %s\n",
		key.latin1()
	)	;
	
	if (key == "$Contents")
	{
		m_chooser->showContents () ;
		return	;
	}
	if (key == "$Index"   )
	{
		m_chooser->showIndex    () ;
		return	;
	}

	QStringList bits = QStringList::split ('#',  key) ;	
	m_chooser->setHelpKey(bits[0] + ".html") ;
	m_browser->showHelp  (bits[0] + ".html", bits[1], "Help") ;
}


void	SOHelpViewer::saveSettings()
{
	TKConfig *config = TKConfig::getConfig() ;

	config->writeEntry	("X", 	      x		 ()) ;
	config->writeEntry	("Y", 	      y		 ()) ;
	config->writeEntry	("W", 	      width 	 ()) ;
	config->writeEntry	("H", 	      height	 ()) ;
	config->writeEntry	("Split",     m_splitter->sizes()) ;
	config->writeEntry	("Maximized", isMaximized()) ;
	config->sync		() ;
}


void	SOHelpViewer::slotLinkClosed ()
{
	exit	(0) ;
}

void	SOHelpViewer::slotReadyRead ()
{
	QString	line	= m_socket.readLine() ;

	fprintf
	(	stderr,
		"SOHelpViewer::slotReadyRead: got key [%s]\n",
		line.latin1()
	)	;

	setHelpKey (QDir::convertSeparators(line.stripWhiteSpace())) ;
}

