/***************************************************************************
    file	         : kb_blockproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>


#include	"kb_classes.h"
#include	"kb_formblock.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_itemproxy.h"
#include	"kb_blockproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"


#define		__DEBUG_BLOCKPROXY	(0)
#if		__DEBUG_BLOCKPROXY
#include	"kb_dprintf.h"
#endif


/*  KBBlockProxy							*/
/*  KBBlockProxy: Constructor for rekall block proxy			*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  block	: KBBlock *		: Rekall block			*/
/*  (returns)	: KBBlockProxy		:				*/

KBBlockProxy::KBBlockProxy
	(	KBKJSInterpreter	*interp,
		KBBlock			*block
	)
	:
	KBItemProxy	(interp, block),
	m_block		(block)
{
}

/*  KBBlockProxy							*/
/* ~KBBlockProxy	: Destructor for rekall block proxy		*/
/*  (returns)	:		:					*/

KBBlockProxy::~KBBlockProxy ()
{
}

/*  KBBlockProxy							*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBBlockProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"getNumRows",		MethodImp::id_getNumRows,	0	},
	{	"getQueryRow",		MethodImp::id_getQueryRow,	0	},
	{	"overLimit",		MethodImp::id_overLimit,	0	},
	{	"getRowValue",		MethodImp::id_getRowValue,	0	},
	{	"setRowValue",		MethodImp::id_setRowValue,	0	},
	{	"setUserFilter",	MethodImp::id_setUserFilter,	0	},
	{	"setUserSorting",	MethodImp::id_setUserSorting,	0	},
	{	0,			0,				0	}
	}	;

	KBItemProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBBlockProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBBlockProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBBlockProxy::get: [%s][%s] cs=%d\n",
		(cchar *)m_object->getName(),
		(cchar *)p,
		m_interp->clientSide()
	))	;

	return	KBItemProxy::get (exec, property) ;
}

/*  KBBlockProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBBlockProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KJS::ObjectImp::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBBlockProxy							*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  spec	: KBMethodSpec * : SPecification			*/
/*  object	: KBBlockProxy * : Associated proxy			*/
/*  (return)	: MethodImp	 :					*/

KBBlockProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBBlockProxy	*block
	)
	:
	KBItemProxy::MethodImp(spec, block),
	m_block		      (block)
{
}


/*  MethodImp								*/
/*  callBase	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBBlockProxy::MethodImp::callBase
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBFormBlock	*fblk	= m_block->m_block->isFormBlock() ;

	if (fblk == 0)
		return	KBItemProxy::MethodImp::callBase (exec, object, args) ;

	switch (m_methodSpec->m_id)
	{
		case id_getNumRows	:
			return	KJS::Number (fblk->getNumRows()) ;

		case id_getQueryRow	:
			return	KJS::Number (fblk->getCurQRow()) ;

		case id_overLimit	:
			return	KJS::Number (fblk->overLimit ()) ;

		case id_getRowValue	:
			return	KBObjectProxy::fromKBValue
				(	exec,
					fblk->getRowValue
					(	kjsStringArg (exec, args, 0),
						kjsNumberArg (exec, args, 0)
					)
				)	;

		case id_setRowValue	:
			fblk->setRowValue
			(	kjsStringArg (exec, args, 0),
				kjsNumberArg (exec, args, 1),
				kjsNumberArg (exec, args, 2)
			)	;
			return	KJS::Number(0) ;

		case id_setUserFilter	:
			fblk->setUserFilter  (kjsStringArg (exec, args, 0)) ;
			break	;

		case id_setUserSorting	:
			fblk->setUserSorting (kjsStringArg (exec, args, 0)) ;
			break	;

		default	:
			break	;
	}

	return	KBItemProxy::MethodImp::callBase (exec, object, args) ;
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBBlockProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;

	return	callBase (exec, object, args) ;
}


KBObjectProxy
	*makeBlockProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBBlockProxy (interp, (KBBlock *)node) ;
}

static	const char	*blockAlias[] =
{	
//	"KBFormBlock",
//	"KBReportBlock",
//	"KBFormSubBlock",
//	"KBReportSubBlock",
	0
}	;

void	registerKBBlock	()
{
	registerClass	("KBBlock", makeBlockProxy, blockAlias) ;
}

