/***************************************************************************
    file	         : kb_buttonproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_classes.h"
#include	"kb_button.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_buttonproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_BUTTONPROXY	(0)
#if		__DEBUG_BUTTONPROXY
#include	"kb_dprintf.h"
#endif


/*  KBButtonProxy							*/
/*  KBButtonProxy: Constructor for rekall button proxy			*/
/*  interp	 : KJS::Interpreter *	: Intepreter object		*/
/*  button	 : KBButton *		: Rekall button			*/
/*  (returns)	 : KBOButtonProxy	:				*/

KBButtonProxy::KBButtonProxy
	(	KBKJSInterpreter	*interp,
		KBButton		*button
	)
	:
	KBObjectProxy	(interp, button),
	m_button	(button)
{
}

/*  KBButtonProxy							*/
/* ~KBButtonProxy: Destructor for rekall button proxy			*/
/*  (returns)	 :		:					*/

KBButtonProxy::~KBButtonProxy ()
{
}

/*  KBButtonProxy							*/
/*  addBindings	: Add proxy bindings to specified button		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBButtonProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"setText",	MethodImp::id_setText,		0	},
	{	"getText",	MethodImp::id_getText,		0	},
	{	"isOn",		MethodImp::id_isOn,		0	},
	{	"setOn",	MethodImp::id_setOn,		0	},
	{	"setPixmaps",	MethodImp::id_setPixmaps,	0	},
	{	"setToolTip",	MethodImp::id_setToolTip,	0	},
	{	0,		0,				0	}
	}	;

	KBObjectProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBButtonProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBButtonProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBButtonProxy::get: [%s] cs=%d\n",
		(cchar *)p,
		m_interp->clientSide()
	))	;

	return	KBObjectProxy::get (exec, property) ;
}

/*  KBButtonProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBButtonProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBObjectProxy::put (exec, property, value, attr) ;
}




/*  ------------------------------------------------------------------  */

KBButtonProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBButtonProxy	*button
	)
	:
	KBObjectProxy::MethodImp(spec, button),
	m_button		(button)
{
}

KJS::Value
	KBButtonProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBButton *button = m_button->m_button	;

	DPRINTF
	((	"KBButtonProxy::MethodImp::call: [%s] for [%d]\n",
		(cchar *)button->getName(),
		m_methodSpec->m_id
	))	;

	switch (m_methodSpec->m_id)
	{
		case id_getText	:
			return	KJS::String(button->getText()) ;

		case id_setText	:
			button->setText  (kjsStringArg (exec, args, 0)) ;
			return	KJS::Null() ;

		case id_isOn	:
			return	KJS::Boolean(button->isOn()) ;

		case id_setOn	:
			button->setOn     (kjsBooleanArg (exec, args, 0)) ;
			return	KJS::Null () ;

		case id_setPixmaps :
			button->setPixmaps(kjsStringArg (exec, args, 0), kjsStringArg (exec, args, 1)) ;
			return	KJS::Null () ;

		case id_setToolTip :
			button->setToolTip(kjsStringArg (exec, args, 0)) ;
			return	KJS::Null () ;

		default	:
			break	;
	}

	return	KBObjectProxy::MethodImp::call (exec, object, args) ;
}



KBObjectProxy
	*makeButtonProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBButtonProxy (interp, (KBButton *)node) ;
}

void	registerKBButton	()
{
	registerClass	("KBButton", makeButtonProxy) ;
}

