/***************************************************************************
    file	         : kb_checkproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_classes.h"
#include	"kb_check.h"
#include	"kb_block.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_itemproxy.h"
#include	"kb_checkproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_CHECKPROXY	(0)
#if		__DEBUG_CHECKPROXY
#include	"kb_dprintf.h"
#endif


/*  KBCheckProxy							*/
/*  KBCheckProxy: Constructor for rekall check proxy			*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  check	: KBCheck *		: Rekall check			*/
/*  (returns)	: KBCheckProxy		:				*/

KBCheckProxy::KBCheckProxy
	(	KBKJSInterpreter	*interp,
		KBCheck			*check
	)
	:
	KBItemProxy	(interp, check),
	m_check	(check)
{
}

/*  KBCheckProxy							*/
/* ~KBCheckProxy: Destructor for rekall check proxy			*/
/*  (returns)	:		:					*/

KBCheckProxy::~KBCheckProxy ()
{
}

/*  KBCheckProxy							*/
/*  addBindings	: Add proxy bindings to specified check			*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBCheckProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"isChecked",		MethodImp::id_isChecked,	0	},
	{	"setChecked",		MethodImp::id_setChecked,	0	},
	{	0,			0,				0	}
	}	;

	KBItemProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBCheckProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBCheckProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBCheckProxy::get: [%s] cs=%d\n",
		(cchar *)p,
		m_interp->clientSide()
	))	;

	return	KBItemProxy::get (exec, property) ;
}

/*  KBCheckProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBCheckProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBItemProxy::put (exec, property, value, attr) ;
}




/*  ------------------------------------------------------------------  */

KBCheckProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBCheckProxy	*check
	)
	:
	KBItemProxy::MethodImp		(spec, check),
	m_check			(check)
{
}

/*  MethodImp								*/
/*  callBase	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBCheckProxy::MethodImp::callBase
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBCheck *check = m_check->m_check ;
	int	row    = getCurQRow(kjsNumberArg (exec, args, 0, -1)) ;

	DPRINTF
	((	"KBCheckProxy::MethodImp::call: [%s] for [%d]\n",
		(cchar *)check->getName(),
		m_methodSpec->m_id
	))	;

	switch (m_methodSpec->m_id)
	{
		case id_isChecked	:
			return	KJS::Number(check->isChecked(row)) ;

		case id_setChecked	:
			check->setChecked(row, args.at(1).toInteger(exec)) ;
			return	KJS::Null() ;
		default	:
			break	;
	}

	return	KBItemProxy::MethodImp::callBase (exec, object, args) ;
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBCheckProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;

	return	callBase (exec, object, args) ;
}



KBObjectProxy
	*makeCheckProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBCheckProxy (interp, (KBCheck *)node) ;
}

void	registerKBCheck	()
{
	registerClass	("KBCheck", makeCheckProxy) ;
}

