/***************************************************************************
    file	         : kb_cookiehar.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_classes.h"
#include	"kb_prompt.h"
#include	"kb_cookiejar.h"
#include	"kb_cookiejar.lut"
#include	"kb_kjsscript.h"
#include	"kb_cookie.h"


#include	<qcstring.h>
#include	<qasciidict.h>

#define		__DEBUG_COOKIEJAR	(0)
#if		__DEBUG_COOKIEJAR
#include	"kb_dprintf.h"
#endif


using	namespace	KJS	;

extern	void	displayAllProperties
		(	KJS::Interpreter	*,
			KJS::Object		&
		)	;

const	ClassInfo	RekallCookieJarObjectImp::info =
			{
				"RekallCookieJar",
				0,
				&RekallCookieJarTable,
				0
			}	;

#if	0
@begin RekallCookieJarTable 21
  getCookie     RekallCookieJarObjectImp::getCookie		DontEnum|Function 2
  setCookie     RekallCookieJarObjectImp::setCookie		DontEnum|Function 1
  clear	        RekallCookieJarObjectImp::clear			DontEnum|Function 1
  jar	        RekallCookieJarObjectImp::jar			DontEnum|Function 0
@end
#endif


/*  ------------------------------------------------------------------	*/

RekallCookieJarObjectImp::RekallCookieJarObjectImp
	(	ExecState		*,
		ObjectPrototypeImp	*objProto
	)
	:
	ObjectImp (Object(objProto))
{
	DPRINTF
	((	"RekallCookieJarObjectImp::RekallCookieJarObjectImp: called\n"
	))	;
}


Value	RekallCookieJarObjectImp::get
	(	ExecState	 *exec,
		const Identifier &propertyName
	)
	const
{
	DPRINTF
	((	"RekallCookieJarObjectImp::get\n"
	))	;

	return	lookupGet<RekallCookieJarFunctionImp, RekallCookieJarObjectImp, ObjectImp>
			 (	exec,
				propertyName,
				&RekallCookieJarTable,
				this
			 )	;
}


Value	RekallCookieJarObjectImp::getValueProperty
	(	ExecState	*,
		int		token
	)
	const
{
	switch (token)
	{
		default	:
			break	;
	}

	return	Number(-1)	;
}


RekallCookieJarFunctionImp::RekallCookieJarFunctionImp
	(	ExecState	*exec,
		int		id,
		int		length
	)
	:
	InternalFunctionImp
	(
		(FunctionPrototypeImp*)(exec->interpreter()->builtinFunctionPrototype().imp())
	),
	m_id	(id)
{
	DPRINTF
	((	"RekallCookieJarFunctionImp::RekallCookieJarFunctionImp: called\n"
	))	;

	Value	protect (this) ;
	put	(exec, "length", Number(length), DontDelete|ReadOnly|DontEnum) ;
}

bool	RekallCookieJarFunctionImp::implementsCall () const
{
	return	true	;
}

Value	RekallCookieJarFunctionImp::call
	(	ExecState	*exec,
		Object		&,
		const List	&args
	)
{
	DPRINTF
	((	"RekallCookieJarFunctionImp::call: m_id=%d\n",
		m_id
	))	;

	switch (m_id)
	{
		case RekallCookieJarObjectImp::setCookie:

			KBCookieJar::self()->setCookie
			(	kjsStringArg(exec, args, 0).latin1(),
				kjsStringArg(exec, args, 1).latin1()
			)	;
			return	Number (0) ;


		case RekallCookieJarObjectImp::getCookie:
			{
			cchar	*value	= KBCookieJar::self()->getCookie
					  (	kjsStringArg(exec, args, 0).latin1()
					  )	;
			if (value != 0)
				return	KJS::String (QString(value)) ;
			}
			return	KJS::Null () ;

		case RekallCookieJarObjectImp::clear	:
			KBCookieJar::self()->clear ()	;
			return	KJS::Null () 		;

		case RekallCookieJarObjectImp::jar	:
			{
			const QAsciiDict<QCString> &jar = KBCookieJar::self()->jar() ;
			KJS::ObjectImp	*proxy = new KJS::ObjectImp () ;
			KJS::Object 	object (proxy) 	  ;

			QAsciiDictIterator<QCString> iter (jar) ;
			QCString *res	;
			while ((res = iter.current()) != 0)
			{
				QCString	v = *res ;
				if (v.isNull()) v = ""	 ;

				object.put
				(	exec,
					KJS::Identifier(iter.currentKey()),
					KJS::String    (QString(v))
				)	;

				iter += 1 ;
			}

			proxy->deref () ;
			return	object	;
			}

		default	:
			break	;
	}

	return	Number(-1) ;
}
