/***************************************************************************
    file	         : kb_eventsproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qvariant.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>
#include	<kjs/array_instance.h>


#include	"kb_classes.h"
#include	"kb_form.h"
#include	"kb_config.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_eventsproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_EVENTSPROXY	(0)
#if		__DEBUG_EVENTSPROXY
#include	"kb_dprintf.h"
#endif


/*  ------------------------------------------------------------------	*/

/*  KBEventsProxy							*/
/*  KBEventsProxy: Constructor for rekall event(s) proxy		*/
/*  interp	 : KBKJSInterpreter *	: Intepreter object		*/
/*  object	 : KBObject *		: Rekall object			*/
/*  (returns)	: KBEventsProxy		:				*/

KBEventsProxy::KBEventsProxy
	(	KBKJSInterpreter	*interp,
		KBObject		*object
	)
	:
	KJS::ObjectImp	(),
	m_interp	(interp),
	m_object	(object)
{
}

/*  KBEventsProxy							*/
/* ~KBEventsProxy: Destructor for rekall object proxy			*/
/*  (returns)	 :		:					*/

KBEventsProxy::~KBEventsProxy ()
{
}

/*  KBEventsProxy							*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBEventsProxy::addBindings
	(	KJS::ExecState	*,
		KJS::Object	&object
	)
{
	LITER
	(
		KBAttr,
		m_object->getAttribs(),
		attr,

		if (attr->isEvent() != 0)
		{
			DPRINTF
			((	"KBEventsProxy::addBindings: add for [%s]\n",
				(cchar *)attr->getName()
			))	;
			object.put
			(	m_interp->globalExec(),
				KJS::Identifier(attr->getName().latin1()),
				KJS::Object    (new MethodImp(attr->isEvent(), this))
			)	;
		}
	)
}

/*  KBEventsProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBEventsProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p	= property.qstring() ;

	if (p == "toString")
	{
		DPRINTF
		((	"KBEventsProxy::get: [%s]\n",
			(cchar *)p
		))	;

		return	KJS::String (QString("<%1 %2 Events>")
					.arg(m_object->getElement())
					.arg(m_object->getName   ())
				    )	;
	}

	return	KJS::ObjectImp::get (exec, property) ;
}

/*  KBEventsProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBEventsProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KJS::ObjectImp::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBEventsProxy							*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  event	: KBEvent *	 : Actual event				*/
/*  proxy	: KBEventsProxy * : Associated proxy			*/
/*  (return)	: MethodImp	 :					*/

KBEventsProxy::MethodImp::MethodImp
	(	KBEvent			*event,
		const KBEventsProxy	*proxy
	)
	:
	KJS::ObjectImp	(),
	m_event		(event),
	m_proxy		(proxy)
{
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBEventsProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&,
		const KJS::List		&args
	)
{
	KBValue	  *__args = new KBValue[args.size()] ;

	DPRINTF
	((	"KBEventsProxy::MethodImp::call: argc=%d\n",
		args.size()
	))	;

	for (int idx = 0 ; idx < args.size() ; idx += 1)
		__args[idx] = KBObjectProxy::fromKJSValue (exec, args[idx]) ;

	KBValue	      res ;
	KBScriptError *rc = m_event->doExecute (res, args.size(), __args, false) ;
	delete	 []__args ;

	if (rc != 0)
		KBScriptError::processError (rc, KBScriptError::Normal) ;

	return	KJS::Null() ;
}
