/***************************************************************************
    file	         : kb_fieldproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_classes.h"
#include	"kb_field.h"
#include	"kb_block.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_itemproxy.h"
#include	"kb_fieldproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_FIELDPROXY	(0)
#if		__DEBUG_FIELDPROXY
#include	"kb_dprintf.h"
#endif
			

/*  KBFieldProxy							*/
/*  KBFieldProxy: Constructor for rekall field proxy			*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  field	: KBField *		: Rekall field			*/
/*  (returns)	: KBFieldProxy		:				*/

KBFieldProxy::KBFieldProxy
	(	KBKJSInterpreter	*interp,
		KBField			*field
	)
	:
	KBItemProxy	(interp, field),
	m_field		(field)
{
}

/*  KBFieldProxy							*/
/* ~KBFieldProxy: Destructor for rekall field proxy			*/
/*  (returns)	:		:					*/

KBFieldProxy::~KBFieldProxy ()
{
}

/*  KBFieldProxy							*/
/*  addBindings	: Add proxy bindings to specified field			*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBFieldProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"setSelection",		MethodImp::id_setSelection,	0	},
	{	0,			0,				0	}
	}	;

	KBItemProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBFieldProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBFieldProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBFieldProxy::get: [%s] cs=%d\n",
		(cchar *)p,
		m_interp->clientSide()
	))	;

	return	KBItemProxy::get (exec, property) ;
}

/*  KBFieldProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBFieldProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBItemProxy::put (exec, property, value, attr) ;
}




/*  ------------------------------------------------------------------  */

KBFieldProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBFieldProxy	*field
	)
	:
	KBItemProxy::MethodImp		(spec, field),
	m_field				(field)
{
}

/*  MethodImp								*/
/*  callBase	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBFieldProxy::MethodImp::callBase
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBField *field = m_field->m_field ;

	DPRINTF
	((	"KBFieldProxy::MethodImp::call: [%s] for [%d]\n",
		(cchar *)field->getName(),
		m_methodSpec->m_id
	))	;

	switch (m_methodSpec->m_id)
	{
		case id_setSelection :
			field->setSelection
			(	getCurQRow (args.at(0).toInteger(exec)),
				args.at(1).toInteger(exec),
				args.at(2).toInteger(exec)
			)	;
			return	KJS::Null() ;

		default	:
			break	;
	}

	return	KBItemProxy::MethodImp::callBase (exec, object, args) ;
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBFieldProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;

	return	callBase (exec, object, args) ;
}


KBObjectProxy
	*makeFieldProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBFieldProxy (interp, (KBField *)node) ;
}

void	registerKBField	()
{
	registerClass	("KBField", makeFieldProxy) ;
}

