/***************************************************************************
    file	         : kb_fileproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<qfile.h>
#include	<qtextstream.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>
#include	<kjs/array_instance.h>


#include	"kb_classes.h"
#include	"kb_form.h"
#include	"kb_config.h"

#include	"kb_idbase.h"
#include	"kb_fileproxy.h"

#include	"kb_kjsscript.h"




/*  KBFileProxy								*/
/*  KBFileProxy: Constructor for rekall object proxy			*/
/*  interp	 : KBKJSInterpreter *	: Intepreter object		*/
/*  (returns)	 : KBFileProxy	:					*/

KBFileProxy::KBFileProxy
	(	KBKJSInterpreter	*interp
	)
	:
	KJS::ObjectImp	(),
	m_interp	(interp)
{
}

/*  KBFileProxy								*/
/* ~KBFileProxy: Destructor for rekall object proxy			*/
/*  (returns)	 :		:					*/

KBFileProxy::~KBFileProxy ()
{
}

/*  KBFileProxy								*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBFileProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	object.put
	(	exec,
		KJS::Identifier("open"),
		KJS::Object    (new MethodImp(MethodImp::id_open, this))
	)	;
	object.put
	(	exec,
		KJS::Identifier("read"),
		KJS::Object    (new MethodImp(MethodImp::id_read, this))
	)	;
	object.put
	(	exec,
		KJS::Identifier("readLine"),
		KJS::Object    (new MethodImp(MethodImp::id_readLine, this))
	)	;
	object.put
	(	exec,
		KJS::Identifier("write"),
		KJS::Object    (new MethodImp(MethodImp::id_write, this))
	)	;
	object.put
	(	exec,
		KJS::Identifier("close"),
		KJS::Object    (new MethodImp(MethodImp::id_close, this))
	)	;
}

/*  KBFileProxy								*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBFileProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	return	KJS::ObjectImp::get (exec, property) ;
}

/*  KBFileProxy								*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBFileProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KJS::ObjectImp::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBFileProxy								*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  id		: int *		  : Method identifier			*/
/*  file	: KBFileProxy *   : Associated proxy			*/
/*  (return)	: MethodImp	  :					*/

KBFileProxy::MethodImp::MethodImp
	(	int			id,
		KBFileProxy		*file
	)
	:
	KJS::ObjectImp	(),
	m_id		(id),
	m_file		(file)
{
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBFileProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&,
		const KJS::List		&args
	)
{
	int		rc	;
	QFile		&file	= m_file->m_file   ;
	QTextStream	&stream	= m_file->m_stream ;

	fprintf
	(	stderr,
		"KBFileProxy::MethodImp::call: id=%d\n",
		m_id
	)	;

	switch (m_id)
	{
		case id_open		:
			fprintf
			(	stderr,
				"KBFileProxy::MethodImp::call: open(%s,%s)\n",
				kjsStringArg (exec, args, 0).latin1(),
				kjsStringArg (exec, args, 1).latin1()
			)	;
			file.setName (kjsStringArg (exec, args, 0)) ;
			if (kjsStringArg(exec, args, 1).lower()[0] == 'w')
				rc	= file.open (IO_WriteOnly) ;
			else	rc	= file.open (IO_ReadOnly ) ;
			stream.setDevice   (&file) ;
			return	KJS::Number(rc)	   ;

		case id_read		:
			return	KJS::String(stream.read()) ;

		case id_readLine	:
			{
			QString	line	= stream.readLine() ;
			if (line.isNull()) return KJS::Null() ;
			return	KJS::String(line) ;
			}

		case id_write		:
			stream << kjsStringArg (exec, args, 0) ;
			return	KJS::Null() ;

		case id_close		:
			file.close ()	;
			return	KJS::Null() ;

		default	:
			break	;
	}

	return	KJS::Null()	;
}
