/***************************************************************************
    file	         : kb_formproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>


#include	"kb_classes.h"
#include	"kb_form.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_itemproxy.h"
#include	"kb_blockproxy.h"
#include	"kb_formblockproxy.h"
#include	"kb_formproxy.h"
#include	"kb_globalproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#include	"kb_appptr.h"
#include	"kb_callback.h"

#define		__DEBUG_FORMPROXY	(0)
#if		__DEBUG_FORMPROXY
#include	"kb_dprintf.h"
#endif


/*  KBKJSOpenInfo							*/
/*  -------------							*/
/*  Helper class. There is a bunch of common stuff needed to open a	*/
/*  form or report, gather this up here.				*/

class	KBKJSOpenInfo
{
public	:

	KBForm		*m_form		;
	QString		m_name		;
	QString		m_key		;
	QDict<QString>	m_dict		;
	QDict<QString>	m_results	;
	KBError		m_error		;

	KBKJSOpenInfo	(const char *, KJS::ExecState *, const KJS::List &, KBForm *) ;

	KB::ShowRC	exec	(KBLocation &) ;
}	;

/*  KBKJSOpenInfo							*/
/*  KBKJSOpenInfo							*/
/*		: Constructor for helper class				*/
/*  caller	: const char *	    : Called name for errors		*/
/*  exec	: KJS::ExecState *  : Execution state			*/
/*  args	: const KJS::List & : Argument state			*/
/*  form	: KBForm *	    : Invoking form			*/
/*  (returns)	: KBKJSOpenInfo	    : m_pyBase will be null on error	*/

KBKJSOpenInfo::KBKJSOpenInfo
	(	const char	*caller,
		KJS::ExecState	*exec,
		const KJS::List	&args,
		KBForm		*form
	)
{
	m_name	= kjsStringArg(exec, args, 0) ;
	m_key	= kjsStringArg(exec, args, 2) ;
	m_form	= form	;

	DPRINTF
	((	"KBKJSOpenInfo::KBKJSOpenInfo: al=%d name=[%s] key=[%s]\n",
		args.size(),
		(cchar *)m_name,
		(cchar *)m_key
	))	;

	if (args.size() >= 2)
	{
		KJS::Object params = KJS::Object::dynamicCast (args.at(1)) ;
		if (params.type() == KJS::ObjectType)
		{
			KJS::ReferenceList	   k = params.propList(exec) ;
			KJS::ReferenceListIterator i = k.begin() ;
			while (i != k.end())
			{
				KJS::Identifier ident = i->getPropertyName (exec) ;
				KJS::Value	value = i->getValue	   (exec) ;

				DPRINTF
				((	"KBKJSOpenInfo::KBKJSOpenInfo: [%s]->[%s]\n",
					(cchar *)ident.ustring ()    .qstring(),
					(cchar *)value.toString(exec).qstring()
				))	;

				m_dict.insert
				(	ident.ustring ().qstring(),
					new QString(value.toString(exec).qstring())
				)	;

				i++ ;
			}
		}
	}

	m_results.setAutoDelete (true) 	     ;
	m_dict   .setAutoDelete (true)	     ;
}

/*  KBKJSOpenInfo							*/
/*  exec	: Execute form or whatever				*/
/*  location	: KBLocation &	: Object to execute			*/
/*  (returns)	: KB::ShowRC	: Execution code			*/

KB::ShowRC
	KBKJSOpenInfo::exec
	(	KBLocation	&location
	)
{
	KBCallback *cb	= KBAppPtr::getCallback() ;

	if (cb == 0)
		return	KB::ShowRCError ;

	return	cb->openObject
			  (	m_form,
				location,
				KB::ShowAsData,
				m_dict,
				m_error,
				m_key,
				&m_results
			  )	;
}


/*  ------------------------------------------------------------------	*/

/*  KBFormProxy								*/
/*  KBFormProxy: Constructor for rekall form proxy			*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  form	: KBForm *		: Rekall form			*/
/*  (returns)	: KBFormProxy		:				*/

KBFormProxy::KBFormProxy
	(	KBKJSInterpreter	*interp,
		KBForm			*form
	)
	:
	KBFormBlockProxy	(interp, form),
	m_form			(form)
{
	m_global	= 0 ;
}

/*  KBFormProxy								*/
/* ~KBFormProxy	: Destructor for rekall form proxy			*/
/*  (returns)	:		:					*/

KBFormProxy::~KBFormProxy ()
{
	if (m_global  != 0)
	{	m_global->deref() ;
		delete	m_global  ;
	}
}

/*  KBFormProxy								*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBFormProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"openForm",		MethodImp::id_openForm,		"s|Os"	},
	{	"openReport",		MethodImp::id_openReport,	"s|Os"	},
	{	"close",		MethodImp::id_close,		"|n"	},
	{	"getOpener",		MethodImp::id_getOpener,	0	},
	{	"getParameter",		MethodImp::id_getParameter,	"s"	},
	{	"parameterDict",	MethodImp::id_parameterDict,	0	},
	{	"showMessage",		MethodImp::id_showMessage,	"s"	},
	{	"closeAll",		MethodImp::id_closeAll,		"|n"	},
	{	"getGlobalCtrl",	MethodImp::id_getGlobalCtrl,	0	},
	{	"getSkinFGColor",	MethodImp::id_getSkinFGColor,	"s|b"	},
	{	"getSkinBGColor",	MethodImp::id_getSkinBGColor,	"s|b"	},
	{	"getSkinFont",		MethodImp::id_getSkinFont,	"s"	},
	{	0,			0,				0	}
	}	;

	KBFormBlockProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBFormProxy								*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBFormProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBFormProxy::get: [%s][%s] cs=%d\n",
		(cchar *)m_object->getName(),
		(cchar *)p,
		m_interp->clientSide()
	))	;

	if (p == "__objects__")
	{
		if (m_global == 0)
		{	((KBFormProxy *)this)->m_global = new KBGlobalProxy (m_interp, m_form) ;
			m_global->ref() ;
		}

		KJS::Object    object (m_global) ;
		return	object ;
	}

	return	KBFormBlockProxy::get (exec, property) ;
}

/*  KBFormProxy								*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/


void	KBFormProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KJS::ObjectImp::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBFormProxy								*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  spec	: KBMethodSpec * : Specification			*/
/*  object	: KBFormProxy *  : Associated proxy			*/
/*  (return)	: MethodImp	 :					*/

KBFormProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBFormProxy 	*form
	)
	:
	KBFormBlockProxy::MethodImp	(spec, form),
	m_form				(form)
{
}


static	QString	hashColor
	(	const QString		&color,
		bool			hash
	)
{
	if (hash)
		if (color.left(2).lower() == "0x")
			return	"#" + color.mid(2) ;

	return	color	;
}

/*  MethodImp								*/
/*  callBase	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBFormProxy::MethodImp::callBase
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBForm		*form	= m_form->m_form ;

	DPRINTF
	((	"KBFormProxy::MethodImp::call: id=%d name=%s\n",
		m_methodSpec->m_id,
		m_methodSpec->m_method
	))	;

	if (m_methodSpec->m_id == id_openForm)
	{
		KBKJSOpenInfo info ("KBForm.openForm", exec, args, form) ;
		KB::ShowRC rc	;

		KBDocRoot  *docRoot = form->getDocRoot() ;
		KBLocation location 
		(	docRoot->getDBInfo  (),
			"form",
			docRoot->getDocLocation().server(),
			info.m_name
		)	;
		location.setDataServer (docRoot->getDocLocation().dataServer()) ;

		rc	= info.exec (location) ;

		if (rc == KB::ShowRCError)
		{	info.m_error.DISPLAY()  ;
			return	KJS::Number (0) ;
		}

		// CONVERT RESULTS DICTIONARY
		return	KJS::Null()	;
	}

	if (m_methodSpec->m_id == id_openReport)
	{
		KBKJSOpenInfo info ("KBForm.openReport", exec, args, form) ;
		KB::ShowRC rc	;

		KBDocRoot  *docRoot = form->getDocRoot() ;
		KBLocation location 
		(	docRoot->getDBInfo  (),
			"report",
			docRoot->getDocLocation().server(),
			info.m_name
		)	;
		location.setDataServer (docRoot->getDocLocation().dataServer()) ;

		rc	= info.exec (location) ;

		if (rc == KB::ShowRCError)
		{	info.m_error.DISPLAY()  ;
			return	KJS::Number (0) ;
		}

		return	KJS::Number(0)	;
	}

	if (m_methodSpec->m_id == id_close)
	{
		form->getDocRoot()->doRequestClose(kjsNumberArg(exec, args, 0)) ;
		return	KJS::Number(0)	;
	}
	if (m_methodSpec->m_id == id_closeAll)
	{
		form->getDocRoot()->doRequestClose(kjsNumberArg(exec, args, 0)) ;
		return	KJS::Number(0)	;
	}

	if (m_methodSpec->m_id == id_getOpener)
	{
		return	fromKBNode (exec, form->opener()) ;
	}

	if (m_methodSpec->m_id == id_getParameter)
	{
		return	KJS::String (form->getDocRoot()->getParamValue (kjsStringArg(exec, args, 0))) ;
	}

	if (m_methodSpec->m_id == id_parameterDict)
	{
		const KBAttrDict &kbDict = form->getDocRoot()->parameterDict() ;

		KJS::ObjectImp	*proxy = new KJS::ObjectImp () ;
		KJS::Object 	object (proxy) 	  ;

		QDictIterator<QString> iter (kbDict) ;
		QString *res	;
		while ((res = iter.current()) != 0)
		{
			QString	v = *res ;
			if (v.isNull()) v = ""	 ;

			object.put
			(	exec,
				KJS::Identifier(iter.currentKey().latin1()),
				KJS::String    (QString(v))
			)	;

			iter += 1 ;
		}

		proxy->deref () ;
		return	object	;
	}

	if (m_methodSpec->m_id == id_showMessage)
	{
		form->getDocRoot()->doSetMessage(kjsStringArg(exec, args, 0)) ;
		return	KJS::Number(0)	;
	}
	
	if (m_methodSpec->m_id == id_getGlobalCtrl)
	{
		return	fromKBNode
			(	exec,
				form->getGlobalNode 
				(	kjsStringArg (exec, args, 0)
				)
			)	;
	}

	if (m_methodSpec->m_id == id_getSkinFGColor)
	{
		QString	color	= form->getDocRoot()->skinFGColor (kjsStringArg (exec, args, 0)) ;
		if (color.isEmpty()) return KJS::Null() ;

		return	KJS::String(hashColor(color, kjsNumberArg(exec, args, 1, 0))) ;
	}

	if (m_methodSpec->m_id == id_getSkinBGColor)
	{
		QString	color	= form->getDocRoot()->skinBGColor (kjsStringArg (exec, args, 0)) ;
		if (color.isEmpty()) return KJS::Null() ;

		return	KJS::String(hashColor(color, kjsNumberArg(exec, args, 1, 0))) ;
	}

	if (m_methodSpec->m_id == id_getSkinFont)
	{
		QString	font	= form->getDocRoot()->skinFont (kjsStringArg (exec, args, 0)) ;
		if (font.isEmpty()) return KJS::Null() ;

		return	KJS::String(font) ;
	}


	return	KBFormBlockProxy::MethodImp::callBase (exec, object, args) ;
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBFormProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;

	return	callBase (exec, object, args) ;
}


KBObjectProxy
	*makeFormProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBFormProxy (interp, (KBForm *)node) ;
}

void	registerKBForm	()
{
	registerClass	("KBForm", makeFormProxy) ;
}

