/***************************************************************************
    file	         : kb_framerproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>


#include	"kb_classes.h"
#include	"kb_framer.h"
#include	"kb_display.h"
#include	"kb_docroot.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_framerproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_FRAMERPROXY	(0)
#if		__DEBUG_FRAMERPROXY
#include	"kb_dprintf.h"
#endif


/*  KBFramerProxy							*/
/*  KBFramerProxy: Constructor for rekall framer proxy			*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  framer	: KBFramer *		: Rekall framer			*/
/*  (returns)	: KBFramerProxy		:				*/

KBFramerProxy::KBFramerProxy
	(	KBKJSInterpreter	*interp,
		KBFramer		*framer
	)
	:
	KBObjectProxy	(interp, framer),
	m_framer		(framer)
{
}

/*  KBFramerProxy							*/
/* ~KBFramerProxy	: Destructor for rekall framer proxy		*/
/*  (returns)	:		:					*/

KBFramerProxy::~KBFramerProxy ()
{
}

/*  KBFramerProxy							*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBFramerProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"getNumRows",		MethodImp::id_getNumRows,	0	},
	{	"getQueryRow",		MethodImp::id_getQueryRow,	0	},
	{	"getRowValue",		MethodImp::id_getRowValue,	0	},
	{	"setRowValue",		MethodImp::id_setRowValue,	0	},
	{	"setBackgroundGraphic",	MethodImp::id_setBackgroundGraphic,
									0	},
	{	0,			0,				0	}
	}	;

	KBObjectProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBFramerProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBFramerProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBFramerProxy::get: [%s][%s] cs=%d\n",
		(cchar *)m_object->getName(),
		(cchar *)p,
		m_interp->clientSide()
	))	;

	return	KBObjectProxy::get (exec, property) ;
}

/*  KBFramerProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBFramerProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KJS::ObjectImp::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBFramerProxy							*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  spec	: KBMethodSpec * : SPecification			*/
/*  object	: KBFramerProxy * : Associated proxy			*/
/*  (return)	: MethodImp	 :					*/

KBFramerProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBFramerProxy	*framer
	)
	:
	KBObjectProxy::MethodImp(spec, framer),
	m_framer		      (framer)
{
}


/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBFramerProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBFramer *framer = m_framer->m_framer ;

	switch (m_methodSpec->m_id)
	{
		case id_getNumRows	:
			return	KJS::Number (framer->getNumRows()) ;

		case id_getQueryRow	:
			return	KJS::Number (framer->getCurQRow()) ;

		case id_getRowValue	:
			return	KBObjectProxy::fromKBValue
				(	exec,
					framer->getRowValue
					(	kjsStringArg (exec, args, 0),
						kjsNumberArg (exec, args, 1)
					)
				)	;

		case id_setRowValue	:
			framer->setRowValue
			(	kjsStringArg (exec, args, 0),
				kjsNumberArg (exec, args, 1),
				kjsNumberArg (exec, args, 2)
			)	;
			return	KJS::Number(0) ;

		case id_setBackgroundGraphic :
		{
			QStringList	iparts	= QStringList::split
						  (	'.',
							kjsStringArg (exec, args, 0)
						  )	;
			int		scale	= kjsNumberArg (exec, args, 1) ;

			KBLocation location
			(	framer->getDocRoot()->getDBInfo(),
				"graphic",
				framer->getDocRoot()->getDocLocation().server(),
				iparts[0],
				iparts[1]
			)	;

			KBError	   error	;
			QByteArray image	;

			if (!location.contents (image, error))
			{
				error.DISPLAY()		;
				return	KJS::Number(0)	;
			}


			framer->getContainer()->setBackgroundPixmap
			(	QPixmap(image),
				scale
			)	;

			return	KJS::Number(1) ;
		}

		default	:
			break	;
	}

	return	KBObjectProxy::MethodImp::call (exec, object, args) ;
}


KBObjectProxy
	*makeFramerProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBFramerProxy (interp, (KBFramer *)node) ;
}

static	const char	*framerAlias[] =
{	
	"KBHeader",
	"KBFooter",
	"KBContainer",
	0
}	;

void	registerKBFramer	()
{
	registerClass	("KBFramer", makeFramerProxy, framerAlias) ;
}

