/***************************************************************************
    file	         : kb_globalproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>


#include	"kb_classes.h"
#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_globalproxy.h"
#include	"kb_kjsregister.h"
#include	"kb_kjsscript.h"
#include	"kb_form.h"

#define		__DEBUG_GLOBALPROXY	(0)
#if		__DEBUG_GLOBALPROXY
#include	"kb_dprintf.h"
#endif



/*  KBGlobalProxy							*/
/*  KBGlobalProxy							*/
/*		: Constructor for rekall global objects proxy		*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  form	: KBForm *		: Rekall form			*/
/*  (returns)	: KBGlobalProxy		:				*/

KBGlobalProxy::KBGlobalProxy
	(	KBKJSInterpreter	*interp,
		KBForm			*form
	)
	:
	KJS::ObjectImp	(),
	m_interp	(interp),
	m_form		(form)
{
	DPRINTF
	((	"KBGlobalProxy::KBGlobalProxy: called\n"
	))	;
}

/*  KBGlobalProxy							*/
/* ~KBGlobalProxy							*/
/*		: Destructor for rekall global objects proxy		*/
/*  (returns)	:		:					*/

KBGlobalProxy::~KBGlobalProxy ()
{
}

/*  KBGlobalProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBGlobalProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBGlobalProxy::get: [%s] cs=%d\n",
		(cchar *)p,
		m_interp->clientSide()
	))	;

	KBNode	*node = m_form->getGlobalNode (p) ;
	if (node != 0)
	{
		KBObjectProxy *proxy = makeProxy (m_interp, node) ;
		KJS::Object   object (proxy) ;

		proxy->deref	     ()	     ;
		proxy->addBindings   (exec, object) ;

		return	object	;
	}

	return	KJS::ObjectImp::get (exec, property) ;
}
