/***************************************************************************
    file	         : kb_itemproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>


#include	"kb_classes.h"
#include	"kb_item.h"
#include	"kb_block.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_itemproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_ITEMPROXY	(0)
#if		__DEBUG_ITEMPROXY
#include	"kb_dprintf.h"
#endif



/*  KBItemProxy								*/
/*  KBItemProxy	: Constructor for rekall item proxy			*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  object	: KBItem *		: Rekall object			*/
/*  (returns)	: KBItemProxy		:				*/

KBItemProxy::KBItemProxy
	(	KBKJSInterpreter	*interp,
		KBItem			*item
	)
	:
	KBObjectProxy	(interp, item),
	m_item		(item)
{
}

/*  KBItemProxy								*/
/* ~KBItemProxy	: Destructor for rekall object proxy			*/
/*  (returns)	:		:					*/

KBItemProxy::~KBItemProxy ()
{
}

/*  KBItemProxy								*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBItemProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"setValue",	MethodImp::id_setValue,		0	},
	{	"getValue",	MethodImp::id_getValue,		0	},
	{	"setEnabled",	MethodImp::id_setEnabled,	0	},
	{	"setVisible",	MethodImp::id_setVisible,	0	},
	{	"isEnabled",	MethodImp::id_isEnabled,	0	},
	{	"isVisible",	MethodImp::id_isVisible,	0	},
	{	"getRowValue",	MethodImp::id_getRowValue,	0	},
	{	"errorText",	MethodImp::id_errorText,	0	},
	{	"clearValue",	MethodImp::id_clearValue,	0	},
	{	"isValid",	MethodImp::id_isValid,		0	},
	{	"setReadOnly",	MethodImp::id_setReadOnly,	0	},
	{	"isReadOnly",	MethodImp::id_isReadOnly,	0	},
	{	0,		0,				0	}
	}	;

	KBObjectProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBItemProxy								*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBItemProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBItemProxy::get: [%s][%s] cs=%d\n",
		(cchar *)m_object->getName(),
		(cchar *)p,
		m_interp->clientSide()
	))	;

	return	KBObjectProxy::get (exec, property) ;
}

/*  KBItemProxy								*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBItemProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBObjectProxy::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBItemProxy								*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  spec	: KBMethodSpec * : Specification			*/
/*  object	: KBItemProxy *  : Associated proxy			*/
/*  (return)	: MethodImp	 :					*/

KBItemProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBItemProxy	*item
	)
	:
	KBObjectProxy::MethodImp(spec, item),
	m_item			(item)
{
}


/*  MethodImp								*/
/*  getCurQRow	: Get current query row					*/
/*  row		: int		: Specified row				*/
/*  (returns)	: int		: Row					*/

int	KBItemProxy::MethodImp::getCurQRow
	(	int		row
	)
{
	/* If the row is negative then get the block current row, with	*/
	/* the special case if we call this on the topmost block.	*/
	return	row 		   	   >= 0 ? row :
		m_item->m_item->getBlock() == 0 ? 0   :
					   m_item->m_item->getBlock()->getCurQRow() ;
}

/*  MethodImp								*/
/*  callBase	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBItemProxy::MethodImp::callBase
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBItem	*item	= m_item->m_item ;
	int	row	= getCurQRow(kjsNumberArg (exec, args, 0, -1)) ;

	DPRINTF
	((	"KBItemProxy::MethodImp::call: [%s] for [%d] row=[%d]\n",
		(cchar *)item->getName(),
		m_methodSpec->m_id,
		row
	))	;

	switch (m_methodSpec->m_id)
	{
		case id_setValue	:
			if (args.size() < 2)
				item->setValue (row, KBValue()) ;
			else if (args.at(1).type() == KJS::NullType)
				item->setValue (row, KBValue()) ;
			else	item->setValue (row, KBValue(kjsStringArg (exec, args, 1))) ;
			return	KJS::Null() ;

		case id_getValue	:
			return	KJS::String(item->getValue(row).getRawText()) ;

		case id_clearValue	:
			item->clearValue(row, false) ;
			return	KJS::Null() ;

		case id_setEnabled	:
			item->setEnabled     (row, kjsBooleanArg (exec, args, 1)) ;
			return	KJS::Null() ;

		case id_setVisible	:
			item->setVisible     (row, kjsBooleanArg (exec, args, 1)) ;
			return	KJS::Null() ;

		case id_setReadOnly	:
		{
			QColor	fgColor	;
			QColor	bgColor	;

			QString	fg	= kjsStringArg (exec, args, 2) ;
			QString	bg	= kjsStringArg (exec, args, 3) ;
			if (!fg.isEmpty()) fgColor.setNamedColor  (fg) ;
			if (!bg.isEmpty()) bgColor.setNamedColor  (bg) ;

			item->setCtrlReadOnly
				(	row,
					kjsBooleanArg  (exec, args, 1),
					fgColor,
					bgColor
				)	;
			return	KJS::Null() ;
		}

		case id_isEnabled	:
			return	KJS::Number(item->isEnabled     (row)) ;

		case id_isVisible	:
			return	KJS::Number(item->isVisible     (row)) ;

		case id_isReadOnly	:
			return	KJS::Number(item->isCtrlReadOnly(row)) ;

		case id_getRowValue	:
			return	KBObjectProxy::fromKBValue
				(	exec,
					item->getRowValue(row)
				)	;
		case id_errorText	:
			return	KJS::String(item->errorText()) ;

		case id_isValid		:
			return	KJS::Number(item->isValid(row, false)) ;

		default	:
			break	;
	}

	return	KBObjectProxy::MethodImp::callBase (exec, object, args) ;
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBItemProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;

	return	callBase (exec, object, args) ;
}


KBObjectProxy
	*makeItemProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBItemProxy (interp, (KBItem *)node) ;
}

void	registerKBItem	()
{
	registerClass	("KBItem", makeItemProxy) ;
}

