/***************************************************************************
    file	         : kb_kjsscript.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<math.h>

#include	<qintdict.h>

#include	"kb_classes.h"
#include	"kb_node.h"
#include	"kb_textedit.h"
#include	"kb_file.h"
#include	"kb_locator.h"
#include	"kb_docroot.h"

#include	"kb_kjsscript.moc"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_rekallmain.h"
#include	"kb_cookiejar.h"
#include	"kb_kjstest.h"
#include	"kb_kjdebug.h"
#include	"kb_kjsregister.h"
#include	"kb_kjsfactory.h"

#include	"kb_libloader_kde.h"


#define		__DEBUG_KJSSCRIPT	(0)
#if		__DEBUG_KJSSCRIPT
#include	"kb_dprintf.h"
#endif


#define	EXPRFUNC	"exprFunc"
#define	EXPRWRAP	"function " EXPRFUNC " () { return %1 ; }"



#define	__DISPLAYALLPROPERTIES(i,o)

#ifdef	__DISPLAYALLPROPERTIES
void	displayAllProperties
	(	KJS::ExecState		*exec,
		KJS::Object		&object
	)
{
	KJS::ReferenceList	   allProps = object.propList(exec, true) ;
	KJS::ReferenceListIterator iter	    = allProps.begin() ;

	DPRINTF
	((	"Identifier: %s: ic=%d: properties\n",
		object.className().qstring().latin1(),
		object.implementsCall()
	))	;

    	while (iter != allProps.end())
	{
		KJS::Reference  ref	= iter++ ;
		KJS::Identifier ident	= ref.getPropertyName(exec) ;
		KJS::Value	value	= object.get (exec, ident)  ;

		DPRINTF
		((	"Identifier: [%s] type=%d [%s]\n",
			ident.qstring().latin1(),
			value.type(),
			value.toString(exec).qstring().latin1()
		))	;
	}
}

void	displayAllProperties
	(	KJS::Interpreter	*interpreter,
		KJS::Object		&object
	)
{
	displayAllProperties (interpreter->globalExec(), object) ;
}

#endif

/*  kjsStringArg: Return argument as a string				*/
/*  exec	: ExecState *	  : Execution state			*/
/*  args	: const List &	  : Argument list			*/
/*  index	: int		  : Index into argument list		*/
/*  defval	: const QString & : Default value			*/
/*  (returns)	: QString	  : Argument as string			*/

QString	LIBKJSSCRIPT_API
	kjsStringArg
	(	ExecState	*exec,
		const List	&args,
		int		index,
		const QString	&defval
	)
{
	if (index >= args.size())
		return	defval	;

	const KJS::Value &v = args.at(index) ;
	switch (v.type())
	{
		case KJS::ObjectType	:
			/* KJS::Value::toString() throws an exception	*/
			/* on objects (type 6) so treat this		*/
			/* specially. Not a usual case but might be	*/
			/* used for debugging.				*/
			{
			KJS::Object o = KJS::Object::dynamicCast(v) ;
			if (o.implementsCall())
				return	"[function]" ;
			}
			return	"[object]" ;


		case KJS::BooleanType	:
			/* This case is handled since we want True to	*/
			/* come back as "1" and False as "0" rather	*/
			/* than the JS symbols "true" and "false."	*/
			return	v.toBoolean(exec) ? "1" : "0" ;

		default	:
			break	;
	}

	return	v.toString(exec).qstring() ;
}

/*  kjsBooleanArg							*/
/*		: Return argument as a boolean				*/
/*  exec	: ExecState *	: Execution state			*/
/*  args	: const List &	: Argument list				*/
/*  index	: int		: Index into argument list		*/
/*  defval	: bool		: Default value				*/
/*  (returns)	: bool		: Argument as boolean			*/

bool	LIBKJSSCRIPT_API
	kjsBooleanArg
	(	ExecState	*exec,
		const List	&args,
		int		index,
		bool		defval
	)
{
	if (index >= args.size())
		return	defval	;

	const KJS::Value &v = args.at(index) ;
	switch (v.type())
	{
		case KJS::ObjectType	:
			return	false	;

		default	:
			break	;
	}

	return	v.toBoolean(exec) ;
}

/*  kjsNumberArg: Return argument as a number				*/
/*  exec	: ExecState *	: Execution state			*/
/*  args	: const List &	: Argument list				*/
/*  index	: int		: Index into argument list		*/
/*  defval	: int		: Default value				*/
/*  (returns)	: int		: Argument as number			*/

int	LIBKJSSCRIPT_API
	kjsNumberArg
	(	ExecState	*exec,
		const List	&args,
		int		index,
		int		defval
	)
{
	if (index >= args.size())
		return	defval	;

	const KJS::Value &v = args.at(index) ;
	switch (v.type())
	{
		case KJS::ObjectType	:
			return	0	;

		default	:
			break	;
	}

	return	(int)v.toNumber(exec) ;
}


namespace KJS
{
	UString::UString
		(	const QString	&text
		)
	{
		uint	length	= text.length ()    ;
		UChar	*data	= new UChar[length] ;

		memcpy	(data, text.unicode(), length * sizeof(UChar)) ;
		rep	= UString::Rep::create(data, length) ;
	}

	QString	UString::qstring () const
	{
		return	QString	((QChar *)data(), size()) ;
	}
	QString	Identifier::qstring () const
	{
		return	QString	((QChar *)data(), size()) ;
	}
}


/*  ------------------------------------------------------------------  */


static	QIntDict<KBKJSScriptCode>	scriptCodeMap	;

static	QString				errMsg		;
static	uint				errLno		;
static	QString				errText		;
static	int				errSourceID	;

/*  ------------------------------------------------------------------  */

/*  KBKJSScriptObject							*/
/*  KBKJSScriptObject							*/
/*		: Constructor for persistent script object		*/
/*  kjsObject	: KBObjectProxy *   : Associated KJS objkect object	*/
/*  (returns)	: KBKJSScriptObject :					*/

KBKJSScriptObject::KBKJSScriptObject
	(	KBObjectProxy	*kjsObject
	)
	:
	m_kjsObject (kjsObject)
{
	m_kjsObject->ref () ;

	DPRINTF
	((	"KBKJSScriptObject::KBKJSScriptObject: called for [%p] r=[%d]\n",
		(void *)m_kjsObject,
		m_kjsObject->refcount
	))	;
}

/*  KBKJSScriptObject							*/
/*  KBKJSScriptObject							*/
/*		: Destructor for persistent script object		*/
/*  (returns)	:		:					*/

KBKJSScriptObject::~KBKJSScriptObject ()
{
	DPRINTF
	((	"KBKJSScriptObject::~KBKJSScriptObject: called for [%p] r=[%d]\n",
		(void *)m_kjsObject,
		m_kjsObject->refcount
	))	;
	if (m_kjsObject->deref ())
		delete	m_kjsObject ;
}

/*  KBKJSScriptObject							*/
/*  object	: Retrieve python object				*/
/*  (returns)	: PyObject *	: Python object				*/

KBObjectProxy
	*KBKJSScriptObject::object ()
{
	DPRINTF
	((	"KBKJSScriptObject::object: called for [%p] r=[%d]\n",
		(void *)m_kjsObject,
		m_kjsObject->refcount
	))	;
	/* Increment the reference count. The called must decrement it	*/
	/* later when the called no longer needs it.			*/
	m_kjsObject->ref ()	;
	return	m_kjsObject	;
}



/*  ------------------------------------------------------------------  */

/*  KBKJSScriptCode							*/
/*  KBKJSScriptCode							*/
/*		: Constructor for inline script object			*/
/*  interpreter	: KBKJSInterpreter *	: KJS interpreter object	*/
/*  script	: const QString &	: Script code			*/
/*  fname	: KJS::Identifier &	: Entry function name		*/
/*  location	: KBLocation &		: Nominal location		*/
/*  ok		: bool &		: Return success		*/
/*  (returns)	: KBKJSScriptCode	: Inline script object		*/

KBKJSScriptCode::KBKJSScriptCode
	(	KBKJSInterpreter	*interpreter,
		const QString		&script,
		KBNode			*owner,
		KBEvent			*inherit,
		const QString		&fname,
		KBLocation		&location,
		bool			&ok
	)
	:
	KBScriptCode	(owner, inherit),
	m_interpreter	(interpreter),
	m_location	(location)
{
	/* Evaluate the code. This should result in the named function	*/
	/* as a property of the interpreters global object. Check first	*/
	/* that the execution is successful.				*/
	KJS::Completion compVal	= m_interpreter->evaluate
				  (	script,
					m_interpreter->globalObject()
				  )	;

	DPRINTF
	((	"KBKJSScriptCode::KBKJSScriptCode: got ct=%d val=%d\n%s\n",
		(int)compVal.complType(),
		(int)compVal.value().isValid(),
		(cchar *)script
	))	;

	switch (compVal.complType())
	{
		case KJS::Normal	:
		case KJS::ReturnValue	:
			ok	= true	;
			break	;

		case KJS::Break		:
		case KJS::Continue	:
		case KJS::Throw		:
			ok	= false	;
			return	;

		default	:
			ok	= false	;
			return	;
	}

	if (!ok)
	{
		DPRINTF
		((	"KBKJSScriptCode::KBKJSScriptCode: unexpected return code: ct=%d\n",
			(int)compVal.complType()
		))	;
		m_error	= KBError
			  (	KBError::Error,
				TR("Error compiling script code"),
				QString::null,
				__ERRLOCN
			  )	;
		return	  ;
	}

	__DISPLAYALLPROPERTIES (m_interpreter, m_interpreter->globalObject()) ;

	/* Retrieve the function; checking that we get a valid value,	*/
	/* that it is an object, and that it implements "call" (if not	*/
	/* then it is not a function).					*/
	if (!fname.isEmpty())
	{
		KJS::Value value = m_interpreter->globalObject().get
					(	m_interpreter->globalExec(),
						KJS::Identifier(fname.latin1())
					)	;
		if (!value.isValid())
		{
			DPRINTF
			((	"KBKJSScriptCode::KBKJSScriptCode: no function [%s]\n",
				fname.latin1()
			))	;
			m_error	= KBError
				  (	KBError::Error,
					TR("Script code lacks entry function"),
					QString(TR("Expecting '%1'")).arg(fname),
					__ERRLOCN
				  )	;
			ok	= false	;
			return	;
		}

		DPRINTF
		((	"KBKJSScriptCode::KBKJSScriptCode: %s: got vt=%d\n",
			fname.latin1(),
			(int)value.type()
		))	;

		if (value.type() != KJS::ObjectType)
		{
			DPRINTF
			((	"KBKJSScriptCode::KBKJSScriptCode: not object type\n"
			))	;
			m_error	= KBError
				  (	KBError::Error,
					TR("Script code lacks entry function"),
					QString(TR("Expecting '%1'")).arg(fname),
					__ERRLOCN
				  )	;
			ok	= false	;
			return	;
		}

		m_kjsFunc = value.toObject(m_interpreter->globalExec()) ;
		if (!m_kjsFunc.implementsCall())
		{
			DPRINTF
			((	"KBKJSScriptCode::KBKJSScriptCode: call not implemented\n"
			))	;
			m_error	= KBError
				  (	KBError::Error,
					TR("Entry function is not callable"),
					QString(TR("Expecting '%1'")).arg(fname),
					__ERRLOCN
				  )	;
			ok	= false	;
			return	;
		}
	}

	m_sourceID = KBKJSDebugger::self()->lastSourceID() ;
	scriptCodeMap.insert (m_sourceID, this) ;

	ok	= true	;
}

/*  KBKJSScriptCode							*/
/*  ~KBKJSScriptCode							*/
/*		: Destructor for inline script object			*/
/*  (returns)	:		:					*/

KBKJSScriptCode::~KBKJSScriptCode ()
{
	scriptCodeMap.remove (m_sourceID) ;
}


/*  KBKJSScriptCode							*/
/*  execFunc	: Execute function					*/
/*  node	: KBNode *	  : Invoking rekall object		*/
/*  source	: KBNode *	  : Originator for slots		*/
/*  event	: const QString & : Event for slots			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: const KBValue * : Argument vector			*/
/*  resval	: KBValue *	  : Function return value		*/
/*  (return)	: KBScript::ExeRC : Execution return code		*/

KBScript::ExeRC
	KBKJSScriptCode::execFunc
	(	KBNode		*source,
		const QString	&event,
		uint		argc,
		const KBValue	*argv,
		KBValue		&resval
	)
{
	extern	void	kjsTestClearTestException () ;
	extern	bool	kjsTestHadTestException	  () ;

	KJS::List   	args  	;
	KJS::ExecState	*exec	= m_interpreter->globalExec() ;
	KBObjectProxy	*proxy	= makeProxy (m_interpreter, m_owner) ;

	if (proxy == 0)
	{
		KBError::EFault
		(	QString(TR("Failed to locate KJS class for %1")).arg(m_owner->getElement()),
			QString::null,
			__ERRLOCN
		)	;
		return	KBScript::ExeError ;
	}

	KJS::Object object (proxy) 	  ;
	proxy->addBindings (exec, object) ;

	if (source != 0)
	{
		args.append (KBObjectProxy::fromKBValue (exec, source)) ;
		args.append (KBObjectProxy::fromKBValue (exec, event )) ;
	}

	/* Convert the KBValue argument values to KJS values and stack	*/
	/* them up in the argument list object.				*/
	for (uint idx = 0 ; idx < argc ; idx += 1)
		args.append (KBObjectProxy::fromKBValue (exec, argv[idx])) ;


	/* Invoke the function (clearing any outstanding test		*/
	/* exception first). The object is passed so that the function	*/
	/* looks like a method call with 'this' referring to the	*/
	/* object.							*/
	kjsTestClearTestException () ;

	KBScriptIF::pushLocation (&m_owner->getDocRoot()->getDocLocation(), m_owner) ;
	KJS::Value 	value	= m_kjsFunc.call(exec, object, args) ;
	KBScriptIF::popLocation  () ;


	/* Decrement the reference count and delete if requested. I	*/
	/* *think* that this is right; actually, it will probably be	*/
	/* deleted only when the KJS::Object object disappears.		*/
	if (proxy->deref ()) delete proxy ;


	if (exec->hadException())
	{
		DPRINTF
		((	"KBKJSScriptCode::execFunc: Exception occurred\n"

		))	;
		KBKJSDebugger *d = KBKJSDebugger::self() ;

		errSourceID	= d->lastSourceID     () ;
		errLno		= d->lastLine         () ;

		errText		= QString("Line %1: %2")
					.arg(errLno)
					.arg(d->exceptionMessage ())
					;
		errMsg		= QString("KJS Error: %2")
					.arg(d->exceptionName())
					;

		exec->clearException ()	;

		return	kjsTestHadTestException () ?
				KBScript::ExeTest  :
				KBScript::ExeError ;
	}

	/* The value returned is converted back to a KBValue ..		*/
	switch (value.type())
	{
		case KJS::UnspecifiedType :
		case KJS::UndefinedType	  :
			DPRINTF
			((	"KBKJSScriptCode::execFunc: returned unspec/undef\n"
			))	;
			resval	= KBValue() ;
			break	;

		case KJS::NullType	  :
			DPRINTF
			((	"KBKJSScriptCode::execFunc: returned null\n"
			))	;
			resval	= KBValue() ;
			break	;

		case KJS::BooleanType	:
			DPRINTF
			((	"KBKJSScriptCode::execFunc: returned bool %d\n",
				value.toBoolean(exec)
			))	;
			resval	= KBValue
				  (	value.toBoolean(exec),
					&_kbBool
				  )	;
			break	;

		case KJS::StringType	:
			DPRINTF
			((	"KBKJSScriptCode::execFunc: returned string [%s]\n",
				(cchar *)value.toString (exec).qstring()
			))	;
			resval	= KBValue
				  (	value.toString (exec).qstring(),
					&_kbString
				  )	;
			break	;

		case KJS::NumberType	:
		{
			double	h ;
			double	d = value.toNumber (exec) ;
			double	f = modf (d, &h) ;

			DPRINTF
			((	"KBKJSScriptCode::execFunc: returned number [%f]\n",
				d
			))	;
			if (f == 0.0)
				resval	= KBValue ((int)h, &_kbFixed) ;
			else	resval	= KBValue (d,	   &_kbFloat) ;

			break	;
		}

		default	:
			DPRINTF
			((	"KBKJSScriptCode::execFunc: returned unknown\n"
			))	;
			resval	= KBValue() ;
			break	;
	}

	return	KBScript::ExeTrue ;
}

/*  KBKJSScriptCode							*/
/*  execute	: Execute function					*/
/*  node	: KBNode *	  : Invoking rekall object		*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: const KBValue * : Argument vector			*/
/*  resval	: KBValue *	  : Function return value		*/
/*  (return)	: KBScript::ExeRC : Execution return code		*/

KBScript::ExeRC
	KBKJSScriptCode::execute
	(	uint		argc,
		const KBValue	*argv,
		KBValue		&resval
	)
{
	return	execFunc (0, QString::null, argc, argv, resval) ;
}

KBScript::ExeRC
	KBKJSScriptCode::execute
	(	KBNode		*source,
		const QString	&event,
		uint		argc,
		const KBValue	*argv,
		KBValue		&resval
	)
{
	return	execFunc (source, event, argc, argv, resval) ;
}

void	KBKJSScriptCode::setBreakpoints
	(	const QValueList<int>	&
	)
{
}

KBKJSScriptCode
	*KBKJSScriptCode::lookup
	(	int		id
	)
{
	return	scriptCodeMap.find (id) ;
}


/*  ------------------------------------------------------------------  */




KJS::Value
	KBKJSScriptIF::compileText
	(	KBLocation		&,
		const QString		&,
		QString			&,
		QString			&,
		KBError			&
	)
{
	return	KJS::Value() ;
}


KBScriptCode
	*KBKJSScriptIF::compileInline
	(	KBNode			*,
		const QString		&,
		const QString		&,
		const QStringList	&,
		KBError			&
	)
{
	return	0 ;
}

KJS::Value
	KBKJSScriptIF::findFunction
	(	const QStringList	&,
		const QString		&
	)
{
	return	KJS::Value() ;
}


/*  KBKJSScriptIF							*/
/*  KBKJSScriptIF: Constructor for KJS scripting interface		*/
/*  options	 : const char *	 : Options				*/
/*  (returns)	 : KBKJSScriptIF :					*/

KBKJSScriptIF::KBKJSScriptIF
	(	const char	*options
	)
{
	m_clientSide	= qstrcmp (options, "client") == 0 ;
	m_languageCode	= languageToCode (m_clientSide ? "kjs_cs" : "kjs") ;
	m_interpreter	= new KBKJSInterpreter (m_languageCode, m_clientSide) ;

	DPRINTF
	((	"KBKJSScriptIF::KBKJSScriptIF: m_languageCode=%d cs=%d\n",
		m_languageCode,
		m_clientSide
	))	;

	KJS::ExecState *exec = m_interpreter->globalExec  	    () ;
	KJS::Object    glob  = m_interpreter->globalObject	    () ;
	KJS::Object    objp  = m_interpreter->builtinObjectPrototype() ;

	glob.put
	(
		exec,
		"RekallMain",
		KJS::Object(new RekallMainObjectImp      (exec, static_cast<ObjectPrototypeImp *>(objp.imp()))),
		DontEnum
	)	;

	glob.put
	(
		exec,
		"RekallCookieJar",
		KJS::Object(new RekallCookieJarObjectImp (exec, static_cast<ObjectPrototypeImp *>(objp.imp()))),
		DontEnum
	)	;

	glob.put
	(
		exec,
		"RekallTest",
		KJS::Object(new RekallTestObjectImp      (exec, static_cast<ObjectPrototypeImp *>(objp.imp()))),
		DontEnum
	)	;

	registerClasses () ;

	KBKJSDebugger::self()->attach(m_interpreter) ;

	QString	path    = locateFile ("appdata", QString("script/kjs/support.kjs")) ;
	if (path.isEmpty())
	{
		KBError::EError
		(	TR("Unable to locate kjs/support.kjs"),
			QString::null,
			__ERRLOCN
		)	;
		return	;
	}

	KBFile	support	(path) ;
	if (!support.open(IO_ReadOnly))
	{
		support.lastError().DISPLAY() ;
		return	;
	}

	QString		script	= support.readAll() ;
	KJS::Completion compVal	= m_interpreter->evaluate
				  (	script,
					m_interpreter->globalObject()
				  )	;

	DPRINTF
	((	"KBKJSScriptIF::KBKJSScriptIF:: got ct=%d val=%d\n",
		(int)compVal.complType(),
		(int)compVal.value().isValid()
	))	;
}

KBKJSScriptIF::~KBKJSScriptIF ()
{
	delete	m_interpreter	;
}



void	KBKJSScriptIF::setApp 
	(	KBNotifier	*,
		KBCallback	*
	)
{
}


bool	KBKJSScriptIF::compile
	(	KBLocation		&,
		KBError			&
	)
{
	return	false	;
}

/*  KBKJSScriptIF							*/
/*  compileFunc	: Compile an inline function				*/
/*  script	: const QString & : Function text			*/
/*  fname	: const QString & : Function name			*/
/*		: QString & 	  :					*/
/*		: QString & 	  :					*/
/*		: QStringList &   :					*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: KBScriptCode *  : Inl;ine script object		*/

KBScriptCode
	*KBKJSScriptIF::compileFunc
	(	KBNode			*owner,
		const QString		&script,
		const QString		&path,
		const QString		&fname,
		const QStringList	&,
		KBEvent			*inherit,
		KBError			&pError
	)
{
	DPRINTF
	((	"KBKJSScriptIF::compileFunc: code\n%s\n",
		(cchar *)script
	))	;

	int	errLine	;
	UString errMsg	;

	if (!m_interpreter->checkSyntax (script, &errLine, &errMsg))
	{
		pError	= KBError
			  (	KBError::Error,
				QString(TR("Syntax error at line %1")).arg(errLine),
				errMsg.qstring(),
				__ERRLOCN
			  )	;
		return	0 ;
	}

	KBLocation	inlineLoc
			(	0,
				"script",
				KBLocation::m_pInline,
				path + ":" + fname,
				script
			)	;

	bool	ok ;
	KBKJSScriptCode *code	= new KBKJSScriptCode
					(	m_interpreter,
						script,
						owner,
						inherit,
						fname,
						inlineLoc,
						ok
					)	;

	if (!ok)
	{	pError	= code->m_error	;
		delete	code	;
		return	0	;
	}

	return	code	;
}

KBScriptCode
	*KBKJSScriptIF::compileExpr
	(	KBNode			*owner,
		const QString		&script,
		const QString		&,
		const QStringList	&,
		KBError			&pError
	)
{
	if (!m_interpreter->checkSyntax (script))
	{
		pError	= KBError
			  (	KBError::Error,
				"Syntax error",
				script,
				__ERRLOCN
			  )	;
		return	0 ;
	}

	KBLocation	inlineLoc
			(	0,
				"expr",
				KBLocation::m_pInline,
				EXPRFUNC,
				script
			)	;

	bool	ok ;
	KBKJSScriptCode *code	= new KBKJSScriptCode
				  (	m_interpreter,
					QString(EXPRWRAP).arg(script),
					owner,
					0,
					EXPRFUNC,
					inlineLoc,
					ok
				  )	;

	if (!ok)
	{	delete	code	;
		return	0	;
	}

	return	code	;
}

/*  KBKJSScriptIF							*/
/*  load	: Load a module into the interpreter instance		*/
/*  location	: KBLocation &	: Database location			*/
/*  eText	: QString &	: Error text				*/
/*  ePatt	: QString &	: Error pattern				*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		:					*/

bool	KBKJSScriptIF::load
	(	KBLocation		&location,
		KBError			&pError
	)
{
	QString	text	= location.contents  (pError) ;
	if (text.isNull())
	{
		return	false	;
	}

	KJS::Completion compVal	= m_interpreter->evaluate
				  (	text,
					m_interpreter->globalObject()
				  )	;

	DPRINTF
	((	"KBKJSScriptIF::load: got ct=%d val=%d\n%s\n",
		(int)compVal.complType(),
		(int)compVal.value().isValid(),
		(cchar *)text
	))	;
	switch (compVal.complType())
	{
		case KJS::Normal	:
		case KJS::ReturnValue	:
			return	true	;

		case KJS::Break		:
		case KJS::Continue	:
		case KJS::Throw		:
			return	false	;

		default	:
			break	;
	}

	DPRINTF
	((	"KBKJSScriptIF::load: unexpected return code\n"
	))	;
	return	false	;
}

/*  KBKJSScriptIF							*/
/*  execute	: Execute a KJS script					*/
/*  scrList	: const QStringList &					*/
/*				  : List of script modules to search	*/
/*  fname	: const QString & : Function name			*/
/*  node	: KBNode *	  : Invoking node			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: const KBValue * : Argument vector			*/
/*  resval	: KBValue &	  : Result value			*/
/*  (returns)	: KBScript::ExeRC : Result				*/

KBScript::ExeRC
	KBKJSScriptIF::execute
	(	const QStringList	&,
		const QString		&,
		KBNode			*,
		uint,
		const KBValue		*,
		KBValue 		&
	)
{
	/* This method is used for the "#name" script shortcut and is	*/
	/* not implemented in the KJS scripting module.			*/
	return	KBScript::ExeError ;
}

KBLocation
	KBKJSScriptIF::lastError
	(	QString		&_errMsg,
		uint		&_errLno,
		QString 	&_errText
	)
{
	_errMsg	 = errMsg  ;
	_errLno	 = errLno  ;
	_errText = errText ;

	/* If the error source is empty then it must be a failure	*/
	/* to locate the error. In this case we have nothing to go on.	*/
	if (errSourceID == 0)
		return	KBLocation
			(	0,
				"script",
				KBLocation::m_pInline,
				"[unknown]",
				"kjs"
			) ;

	/* If the error was in inline code then return just enough to	*/
	/* correspond to this. The original caller should know where	*/
	/* it is.							*/
	if (scriptCodeMap.find (errSourceID) != 0)
		return	KBLocation
			(	0,
				"script",
				KBLocation::m_pInline,
				KBLocation::m_pInline,
				"kjs"
			) ;


	/* Should not get here!						*/
	fprintf
	(	kbDPrintfGetStream(),
		"KBKJSScriptIF::exeError failed for [%d]\n",
		errSourceID
	)	;
	return	KBLocation () ;
}

bool	KBKJSScriptIF::unlink
	(	KBLocation	&,
		KBError		&
	)
{
	return	false	;
}

bool	KBKJSScriptIF::rename
	(	KBLocation	&,
		const QString	&,
		KBError		&
	)
{
	return	false	;
}

void	KBKJSScriptIF::editorInit
	(	KBTextEdit	*editor
	)
{
	editor->setHighlight ("kjs") ;
}

KBDebug	*KBKJSScriptIF::showDebug
	(	TKToggleAction *
	)
{
	return	0 ;
}

bool	KBKJSScriptIF::debugScript
	(	KBLocation  	&,
		KBError 	&
	)
{
	return	false	;
}

bool	KBKJSScriptIF::isClientSide ()
{
	return	m_clientSide	;
}

QString	KBKJSScriptIF::ident ()
{
	return	QString("KJS: %1%2 %3")
			.arg(__KB_BUILD_RELEASE)
			.arg(__KB_BUILD_BUILD  )
			.arg(__KB_BUILD_TIME   ) ;
}

KBFACTORYIMPL
(	KBKJSScriptFactory,
	script_kjs
)

/*  KBPYScriptFactory							*/
/*		: Get an instance of the appropriate interpreter	*/
/*  parent	: QObject *	: Parent object				*/
/*  (returns)	: QObject *	: Interpreter instance			*/

QObject	*KBKJSScriptFactory::create
	(	QObject		  *,
		cchar		  *,
		cchar		  *options,
		const QStringList &
	)
{
	return	new KBKJSScriptIF (options) ;
}

QString	KBKJSScriptFactory::ident ()
{
	return	QString("KJS: %1%2 %3")
			.arg(__KB_BUILD_RELEASE)
			.arg(__KB_BUILD_BUILD  )
			.arg(__KB_BUILD_TIME   ) ;
}
