/***************************************************************************
    file	         : kb_kjsscript.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_KJSSCRIPT_H
#define	_KB_KJSSCRIPT_H


#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_classes.h"
#include	"kb_script.h"

#include	"libkjsscript_exports.h"


class	KBKJSInterpreter	;
class	KBObjectProxy		;


/*  KBKJSScriptCode							*/
/*  --------------							*/
/*  KJS-specific class for precompiled inline code.			*/

class	KBKJSScriptCode : public KBScriptCode
{
	KBScript::ExeRC	execFunc
		(	KBNode		*,
			const QString	&,
			uint		,
			const KBValue	*,
			KBValue		&
		)	;

public	:

	KBKJSInterpreter	*m_interpreter	;
	KJS::Object		m_kjsFunc	;
	KBLocation		m_location	;
	int			m_sourceID	;
	KBError			m_error		;

	KBKJSScriptCode
	(	KBKJSInterpreter	*,
		const QString		&,
		KBNode			*,
		KBEvent			*,
		const QString		&,
		KBLocation		&,
		bool			&
	)	;

	virtual	~KBKJSScriptCode	() ;

	virtual	KBScript::ExeRC	execute
		(	uint		,
			const KBValue	*,
			KBValue		&
		)	;

	virtual	KBScript::ExeRC	execute
		(	KBNode		*,
			const QString	&,
			uint		,
			const KBValue	*,
			KBValue		&
		)	;

	virtual	void	setBreakpoints
		(	const QValueList<int>	&
		)	;

	static	KBKJSScriptCode	*lookup	(int)	;
}	;


/*  KBKJSScriptObject							*/
/*  -----------------							*/
/*  Local derived version of KBScriptObject, used to implement object	*/
/*  persistence.							*/

class	KBKJSScriptObject : public KBScriptObject
{
	KBObjectProxy	*m_kjsObject  ;

public	:

	KBKJSScriptObject(KBObjectProxy *) ;
	virtual	~KBKJSScriptObject () ;

	KBObjectProxy	*object	   () ;
}	;

/*  KBKJSInterpreter							*/
/*  ----------------							*/
/*  We need to know whether the scripts are running server-side only,	*/
/*  in which case we can fully use properties, or will be run		*/
/*  client-side, in which case the use of properties is restricted to	*/
/*  match the client-side libraries. The easy and safe way to do this	*/
/*  is to wrap the KJS::Interpreter class with an extra boolean flag.	*/

class	KBKJSInterpreter : public KJS::Interpreter
{
	uint			m_languageCode	;
	bool			m_clientSide	;

public	:

	inline	KBKJSInterpreter
		(	uint	languageCode,
			bool	clientSide
		)
	{
		m_languageCode	= languageCode	;
		m_clientSide 	= clientSide	;
	}

	inline	uint	languageCode ()
	{
		return	m_languageCode	;
	}

	inline	bool	clientSide   ()
	{
		return	m_clientSide	;
	}
}	;


/*  KBKJSScriptIF							*/
/*  ------------							*/
/*  This is the script interface object for the KJS script module.	*/

class	KBKJSScriptIF : public KBScriptIF
{
	Q_OBJECT

	bool			m_clientSide	;
	uint			m_languageCode	;
	KBKJSInterpreter	*m_interpreter	;

	KJS::Value	compileText
			(	KBLocation		&,
				const QString		&,
				QString			&,
				QString			&,
				KBError			&
			) 	;

	KBScriptCode   *compileInline
			(	KBNode			*,
				const QString		&,
				const QString		&,
				const QStringList	&,
				KBError			&
			)	;

	KJS::Value	findFunction
			(	const QStringList	&,
				const QString		&
			)	;

public	:

	KBKJSScriptIF (const char *)	;
	virtual	~KBKJSScriptIF ()	;

	virtual	void	setApp
		(	KBNotifier		*,
			KBCallback		*
		)	;

	virtual	bool	compile
		(	KBLocation		&,
			KBError			&
		)	;

	virtual	bool	load
		(	KBLocation		&,
			KBError			&
		)	;

	virtual	KBScriptCode   *compileFunc
		(	KBNode			*,
			const QString		&,
			const QString		&,
			const QString		&,
			const QStringList	&,
			KBEvent			*,
			KBError			&
		)	;

	virtual	KBScriptCode   *compileExpr
		(	KBNode			*,
			const QString		&,
			const QString		&,
			const QStringList	&,
			KBError			&
		)	;

	virtual	KBScript::ExeRC	execute
		(	const QStringList	&,
			const QString		&,
			KBNode			*,
			uint,
			const KBValue		*,
			KBValue 		&
		)	;


	virtual	KBLocation	lastError	(QString &, uint &, QString &) ;

	virtual	bool		unlink		(KBLocation     &, KBError &) ;
	virtual	bool		rename		(KBLocation     &, const QString &, KBError &) ;

	virtual	void		editorInit	(KBTextEdit    *) ;
	virtual	KBDebug		*showDebug	(TKToggleAction *) ;
	virtual	bool		debugScript	(KBLocation   &, KBError 	   &) ;  

	virtual	QString		ident		() ;
	virtual	bool		isClientSide	() ;
}	;

extern	QString LIBKJSSCRIPT_API
		kjsStringArg	(KJS::ExecState *, const KJS::List &, int, const QString & = QString::null) ;
extern	int	LIBKJSSCRIPT_API
		kjsNumberArg	(KJS::ExecState *, const KJS::List &, int, int 		   = -1   ) ;
extern	bool	LIBKJSSCRIPT_API
		kjsBooleanArg	(KJS::ExecState *, const KJS::List &, int, bool		   = false) ;


#endif	// _KB_KJSSCRIPT_H
