/***************************************************************************
    file	         : kb_labelproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_classes.h"
#include	"kb_label.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_labelproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_LABELPROXY	(0)
#if		__DEBUG_LABELPROXY
#include	"kb_dprintf.h"
#endif


/*  KBLabelProxy							*/
/*  KBLabelProxy : Constructor for rekall label proxy			*/
/*  interp	 : KJS::Interpreter *	: Intepreter object		*/
/*  label	 : KBLabel *		: Rekall label			*/
/*  (returns)	 : KBLabelProxy		:				*/

KBLabelProxy::KBLabelProxy
	(	KBKJSInterpreter	*interp,
		KBLabel			*label
	)
	:
	KBObjectProxy	(interp, label),
	m_label		(label)
{
}

/*  KBLabelProxy							*/
/* ~KBLabelProxy: Destructor for rekall label proxy			*/
/*  (returns)	 :		:					*/

KBLabelProxy::~KBLabelProxy ()
{
}

/*  KBLabelProxy							*/
/*  addBindings	: Add proxy bindings to specified label		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBLabelProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"setText",	MethodImp::id_setText,		0	},
	{	"getText",	MethodImp::id_getText,		0	},
	{	0,		0,				0	}
	}	;

	KBObjectProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBLabelProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBLabelProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBLabelProxy::get: [%s] cs=%d\n",
		(cchar *)p,
		m_interp->clientSide()
	))	;

	return	KBObjectProxy::get (exec, property) ;
}

/*  KBLabelProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBLabelProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBObjectProxy::put (exec, property, value, attr) ;
}




/*  ------------------------------------------------------------------  */

KBLabelProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBLabelProxy	*label
	)
	:
	KBObjectProxy::MethodImp(spec, label),
	m_label			(label)
{
}

KJS::Value
	KBLabelProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBLabel *label = m_label->m_label	;

	DPRINTF
	((	"KBLabelProxy::MethodImp::call: [%s] for [%d]\n",
		(cchar *)label->getName(),
		m_methodSpec->m_id
	))	;

	switch (m_methodSpec->m_id)
	{
		case id_getText	:
			return	KJS::String(label->getText()) ;

		case id_setText	:
			label->setText  (kjsStringArg (exec, args, 0)) ;
			return	KJS::Null() ;

		default	:
			break	;
	}

	return	KBObjectProxy::MethodImp::call (exec, object, args) ;
}



KBObjectProxy
	*makeLabelProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBLabelProxy (interp, (KBLabel *)node) ;
}

void	registerKBLabel	()
{
	registerClass	("KBLabel", makeLabelProxy) ;
}

