/***************************************************************************
    file	         : kb_linkproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_classes.h"
#include	"kb_link.h"
#include	"kb_block.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_itemproxy.h"
#include	"kb_linktreeproxy.h"
#include	"kb_linkproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_LINKPROXY	(0)
#if		__DEBUG_LINKPROXY
#include	"kb_dprintf.h"
#endif


/*  KBLinkProxy								*/
/*  KBLinkProxy	: Constructor for rekall link proxy			*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  link	: KBLink *		: Rekall link			*/
/*  (returns)	: KBLinkProxy		:				*/

KBLinkProxy::KBLinkProxy
	(	KBKJSInterpreter	*interp,
		KBLink			*link
	)
	:
	KBLinkTreeProxy	(interp, link),
	m_link		(link)
{
}

/*  KBLinkProxy								*/
/* ~KBLinkProxy	: Destructor for rekall link proxy			*/
/*  (returns)	:		:					*/

KBLinkProxy::~KBLinkProxy ()
{
}

/*  KBLinkProxy								*/
/*  addBindings	: Add proxy bindings to specified link			*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBLinkProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"currentItem",		MethodImp::id_currentItem,	0	},
	{	"setCurrentItem",	MethodImp::id_setCurrentItem,	0	},
	{	"setCurrentDisplay",	MethodImp::id_setCurrentDisplay,0	},
	{	0,			0,				0	}
	}	;

	KBLinkTreeProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBLinkProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBLinkProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBLinkProxy::get: [%s] cs=%d\n",
		(cchar *)p,
		m_interp->clientSide()
	))	;

	return	KBLinkTreeProxy::get (exec, property) ;
}

/*  KBLinkProxy								*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBLinkProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBLinkTreeProxy::put (exec, property, value, attr) ;
}




/*  ------------------------------------------------------------------  */

KBLinkProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBLinkProxy	*link
	)
	:
	KBLinkTreeProxy::MethodImp	(spec, link),
	m_link				(link)
{
}

KJS::Value
	KBLinkProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBLink	*link	= m_link->m_link ;
	int	 row	= getCurQRow (args.at(0).toInteger(exec)) ;

	DPRINTF
	((	"KBLinkProxy::MethodImp::call: [%s] for [%d]\n",
		(cchar *)link->getName(),
		m_methodSpec->m_id
	))	;

	switch (m_methodSpec->m_id)
	{
		case id_currentItem	   :
			return	KJS::Number(link->currentItem(row)) ;

		case id_setCurrentItem	  :
			link->setCurrentItem    (row, args.at(1).toInteger(exec)) ;
			return	KJS::Null() ;

		case id_setCurrentDisplay :
			link->setCurrentDisplay (row, kjsStringArg(exec, args,1)) ;
	}

	return	KBLinkTreeProxy::MethodImp::call (exec, object, args) ;
}



KBObjectProxy
	*makeLinkProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBLinkProxy (interp, (KBLink *)node) ;
}

void	registerKBLink	()
{
	registerClass	("KBLink", makeLinkProxy) ;
}

