/***************************************************************************
    file	         : kb_listboxproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_classes.h"
#include	"kb_listbox.h"
#include	"kb_block.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_itemproxy.h"
#include	"kb_listboxproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_LISTBOXPROXY	(0)
#if		__DEBUG_LISTBOXPROXY
#include	"kb_dprintf.h"
#endif


/*  KBListBoxProxy							*/
/*  KBListBoxProxy							*/
/*		: Constructor for rekall choice proxy			*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  choice	: KBListBox *		: Rekall choice			*/
/*  (returns)	: KBListBoxProxy		:				*/

KBListBoxProxy::KBListBoxProxy
	(	KBKJSInterpreter	*interp,
		KBListBox		*listbox
	)
	:
	KBItemProxy	(interp, listbox),
	m_listbox	(listbox)
{
}

/*  KBListBoxProxy							*/
/* ~KBListBoxProxy: Destructor for rekall listbox proxy			*/
/*  (returns)	:		:					*/

KBListBoxProxy::~KBListBoxProxy ()
{
}

/*  KBListBoxProxy							*/
/*  addBindings	: Add proxy bindings to specified listbox		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBListBoxProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"currentItem",		MethodImp::id_currentItem,	0	},
	{	"setCurrentItem",	MethodImp::id_setCurrentItem,	0	},
	{	"getValues",		MethodImp::id_getValues,	0	},
	{	"setValues",		MethodImp::id_setValues,	0	},
	{	"getNumValues",		MethodImp::id_getNumValues,	0	},
	{	0,			0,				0	}
	}	;

	KBItemProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBListBoxProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBListBoxProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBListBoxProxy::get: [%s] cs=%d\n",
		(cchar *)p,
		m_interp->clientSide()
	))	;

	return	KBItemProxy::get (exec, property) ;
}

/*  KBListBoxProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBListBoxProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBItemProxy::put (exec, property, value, attr) ;
}




/*  ------------------------------------------------------------------  */

KBListBoxProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBListBoxProxy	*listbox
	)
	:
	KBItemProxy::MethodImp		(spec, listbox),
	m_listbox			(listbox)
{
}

/*  MethodImp								*/
/*  callBase	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBListBoxProxy::MethodImp::callBase
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBListBox *listbox = m_listbox->m_listbox ;
	int	 row	 ;

	DPRINTF
	((	"KBListBoxProxy::MethodImp::call: [%s] for [%d]\n",
		(cchar *)listbox->getName(),
		m_methodSpec->m_id
	))	;

	switch (m_methodSpec->m_id)
	{
		case id_currentItem	:
			row = getCurQRow (args.at(0).toInteger(exec)) ;
			return	KJS::Number(listbox->currentItem(row)) ;

		case id_setCurrentItem	:
			row = getCurQRow (args.at(0).toInteger(exec)) ;
			listbox->setCurrentItem(row, args.at(1).toInteger(exec)) ;
			return	KJS::Null() ;

		case id_getValues	:
		{
			const QStringList &values = listbox->getValues () ;
			bool		  noblank = listbox->getAttrVal("noblank") == "Yes" ;
			uint		  offset  = noblank ? 0 : 1 ;

			KJS::List	  list	  ;
			for (uint idx = offset ; idx < values.count() ; idx += 1)
				list.append (KJS::String(values[idx])) ;

			return	KJS::Object::dynamicCast
				(	exec->interpreter()->builtinArray().construct
					(	exec,
						list
				)	)	;
		}

		case id_setValues	:
		{
			KJS::Object list = KJS::Object::dynamicCast (args.at(0)) ;
			QStringList vals ;

			for (int idx = 0 ; ; idx += 1)
			{	KJS::Value v = list.get (exec, idx) ;
				if (v.type() <= KJS::NullType) break ;
				vals.append (v.toString(exec).qstring()) ;
			}

			listbox->setValues (vals);
			return	KJS::Null ()	;
		}

		case id_getNumValues	:
		{
			uint	nVals 	= listbox->getNumValues () ;
			bool	noblank	= listbox->getAttrVal("noblank") == "Yes" ;
			uint	offset	= noblank ? 0 : 1 ;
			return	KJS::Number (nVals - offset) ;
		}

		default	:
			break	;
	}

	return	KBItemProxy::MethodImp::callBase (exec, object, args) ;
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBListBoxProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;

	return	callBase (exec, object, args) ;
}


KBObjectProxy
	*makeListBoxProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBListBoxProxy (interp, (KBListBox *)node) ;
}

void	registerKBListBox	()
{
	registerClass	("KBListBox", makeListBoxProxy) ;
}

