/***************************************************************************
    file	         : kb_objectproxy.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#ifndef	__KB_OBJECTPROXY_H
#define	__KB_OBJECTPROXY_H

#include "libkjsscript_exports.h"


class	KBKJSInterpreter	;
class	KBEventsProxy		;
class	KBSlotsProxy		;


struct	KBMethodSpec
{	cchar		*m_method	;
	uint		m_id		;
	cchar		*m_argSpec	;
}	;

/*  KBObjectProxy							*/
/*  -------------							*/
/*  Proxy class for Rekall objects. All other proxy classes are derived	*/
/*  from this.								*/

class	KBObjectProxy : public KJS::ObjectImp
{
protected :

	KBKJSInterpreter	*m_interp	;
	KBObject		*m_object 	;
	KBEventsProxy		*m_events	;
	KBSlotsProxy		*m_slots	;

public	:

	KBObjectProxy	(KBKJSInterpreter *, KBObject *) ;
	~KBObjectProxy	() ;


	virtual	void		addBindings
				(	KJS::ExecState		*,
					KJS::Object		&
				)	;
	virtual	KJS::Value	get
				(	KJS::ExecState		*,
					const KJS::Identifier	&
				)
				const	;

	virtual	void		put
				(	KJS::ExecState		*,
					const KJS::Identifier	&,
					const KJS::Value	&,
					int			= KJS::None
				)	;

protected :

	inline	KBKJSInterpreter *interpreter () const
	{
		return	m_interp ;
	}

	/* We could create a separate KJS::ObjectImp-derived class for	*/
	/* every method, but that would be overkill. Instead, we create	*/
	/* instances of the KBObjectProxy::MethodImp class for each	*/
	/* method, passing a method code and optional argument		*/
	/* specification to each. We then derive from this class for	*/
	/* corresponding Rekall objects (eg., KBButton is a subclass of	*/
	/* KBObject so we will have KBButtonProxy::MethodImp as a	*/
	/* subclass of KBObjectProxy::MethodImp.			*/

	class MethodImp : public KJS::ObjectImp
	{

	protected :

		KBMethodSpec		*m_methodSpec	;
		const KBObjectProxy	*m_object	;

		virtual	KJS::Value	callBase
					(	KJS::ExecState	*,
						KJS::Object	&,
						const KJS::List &
					)	;

		KJS::Value		checkArgs
					(	KJS::ExecState		*,
						const KJS::List		&,
						cchar			* = 0
					)	;

		KJS::Value		fromKBNode
					(	KJS::ExecState		*,
						KBNode			*
					)	;

	public	:

		/* The constructor arguments are the method name and	*/
		/* code and the actual object proxy. The argument	*/
		/* specification is optional.				*/
		MethodImp	(KBMethodSpec *, const KBObjectProxy *) ;
		~MethodImp()	{}

		virtual	bool	implementsCall () const
		{
			return	true	;
		}

		virtual	KJS::Value call
			(	KJS::ExecState	*,
				KJS::Object	&,
				const KJS::List &
			)	;

		static	const uint	id_getAttr	= IDB_OBJECT + 1  ;
		static	const uint	id_getConfig	= IDB_OBJECT + 2  ;
		static	const uint	id_setAttr	= IDB_OBJECT + 3  ;
		static	const uint	id_resize	= IDB_OBJECT + 4  ;
		static	const uint	id_getNamedCtrl	= IDB_OBJECT + 5  ;
		static	const uint	id_setEnabled	= IDB_OBJECT + 6  ;
		static	const uint	id_setVisible	= IDB_OBJECT + 7  ;
		static	const uint	id_isEnabled	= IDB_OBJECT + 8  ;
		static	const uint	id_isVisible	= IDB_OBJECT + 9  ;
		static	const uint	id_getType	= IDB_OBJECT + 10 ;
		static	const uint	id_getName	= IDB_OBJECT + 11 ;
		static	const uint	id_width	= IDB_OBJECT + 12 ;
		static	const uint	id_height	= IDB_OBJECT + 13 ;
		static	const uint	id_getQTProperty= IDB_OBJECT + 14 ;
		static	const uint	id_setFont	= IDB_OBJECT + 15 ;
		static	const uint	id_getParent	= IDB_OBJECT + 16 ;
		static	const uint	id_getBlock	= IDB_OBJECT + 17 ;
		static	const uint	id_getForm	= IDB_OBJECT + 18 ;
		static	const uint	id_lastError	= IDB_OBJECT + 19 ;
		static	const uint	id_getControls	= IDB_OBJECT + 20 ;
		static	const uint	id_getRoot	= IDB_OBJECT + 21 ;
		static	const uint	id_setFGColor	= IDB_OBJECT + 22 ;
		static	const uint	id_setBGColor	= IDB_OBJECT + 23 ;
		static	const uint	id_getElement	= IDB_OBJECT + 24 ;
		static	const uint	id_notify	= IDB_OBJECT + 25 ;
		static	const uint	id_toString	= IDB_OBJECT + 98 ;
	}	;

	friend	class	MethodImp ;

public	:

	static	KJS::Value	fromKBValue
				(	KJS::ExecState		*,
					const KBValue		&
				)	;

	static	KBValue		fromKJSValue
				(	KJS::ExecState		*,
					const KJS::Value	&
				)	;
	static	KBObject	*toKBObject
				(	KJS::ExecState		*,
					const KJS::Value	&
				)	;

	LIBKJSSCRIPT_API
		static	bool	checkArgs
				(	KJS::ExecState		*,
					const KJS::List		&,
					cchar			*
				)	;
}	;

#endif	// __KB_OBJECTPROXY_H
