/***************************************************************************
    file	         : kb_rekallmain.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_classes.h"
#include	"kb_prompt.h"
#include	"kb_rekallmain.h"
#include	"kb_rekallmain.lut"
#include	"kb_kjsscript.h"
#include	"kb_fileproxy.h"
#include	"kb_object.h"
#include	"kb_docroot.h"
#include	"kb_helper.h"
#include	"kb_popupprompt.h"
#include	"kb_urlrequest.h"
#include	"kb_test.h"

#include	"kb_objectproxy.h"
#include	"kb_kjdebug.h"

#define		__DEBUG_REKALLMAIN	(0)
#if		__DEBUG_REKALLMAIN
#include	"kb_dprintf.h"
#endif


using	namespace	KJS	;



extern	void	displayAllProperties
		(	KJS::Interpreter	*,
			KJS::Object		&
		)	;

const	ClassInfo	RekallMainObjectImp::info =
			{
				"RekallMain",
				0,
				&rekallMainTable,
				0
			}	;

#if	0
@begin rekallMainTable 21
  messageBox      RekallMainObjectImp::messageBox	DontEnum|Function 1
  choiceBox       RekallMainObjectImp::choiceBox	DontEnum|Function 2
  queryBox        RekallMainObjectImp::queryBox		DontEnum|Function 2
  print           RekallMainObjectImp::print		DontEnum|Function 1
  file            RekallMainObjectImp::file		DontEnum|Function 0
  showPopupHelper RekallMainObjectImp::showPopupHelper	DontEnum|Function 5
  showPopupPrompt RekallMainObjectImp::showPopupPrompt	DontEnum|Function 5
  showPopupChoice RekallMainObjectImp::showPopupChoice	DontEnum|Function 5
  webMode	  RekallMainObjectImp::webMode		DontEnum|Function 0
  URLRequest	  RekallMainObjectImp::URLRequest	DontEnum|Function 0
@end
#endif


static	QStringList
		objectToStringList
		(	ExecState		*exec,
			const KJS::Object	&object
		)
{
	QStringList	cList	;

	for (int idx = 0 ; ; idx += 1)
	{	KJS::Value v = object.get (exec, idx) ;
		if (v.type() <= KJS::NullType) break ;
		cList.append (v.toString(exec).qstring()) ;
	}

	return	cList	;
}

static	Value	kbTestFailed
	(	ExecState	*exec,
		const QString	&what
	)
{
	KBKJSDebugger	*d 	= KBKJSDebugger::self () ;
	int	 	id	= d->lastSourceID     () ;
	uint		lineno	= d->lastLine         () ;

	KBKJSScriptCode *code	= KBKJSScriptCode::lookup (id) ;
	QString		name	;

	extern	void	kjsTestSetTestException	() ;
	if (code != 0)
		name	= code->m_location.ident() ;
	else	name	= ":Unknown:Unknown:"	   ;

	KBTest::appendTestResult
		(	KBScriptTestResult
			(	name,
				lineno,
				QString::null,
				KBScriptTestResult::testFailed,
				what,
				"kjs",
				QString::null
		)	)	;

	KJS::Value err	= KJS::Error::create
			  (	exec,
				KJS::GeneralError,
				QString	("Test suite failure")
			  )	;
	exec->setException	(err)	;
	kjsTestSetTestException	()	;
	return	err ;
}


/*  choiceBox	: Display user choice box				*/
/*  exec	: ExecState *	: Execution state			*/
/*  args	: const List &	: Argument list				*/
/*  (returns)	: Value		: Chosen value, null on cancel		*/

static	Value	choiceBox
	(	ExecState	*exec,
		const List	&args
	)
{
	if (!args.at(1).isA(KJS::ObjectType))
	{
		return	KJS::Number(-1) ;
	}

	QString		message	= kjsStringArg (exec, args, 0) ;
	QString		caption	= kjsStringArg (exec, args, 1) ;
	KJS::Object	options	= KJS::Object::dynamicCast (args.at(1)) ;
	QStringList	cList	= objectToStringList (exec, options) ;
	QString		resVal	;
	bool		rc	;

	if (!KBTest::choiceBox (caption, message, cList, rc, resVal))
		return	kbTestFailed (exec, TR("Choice box not expected")) ;

	if (!rc)
		return	KJS::Null() ;

	return	KJS::String(resVal) ;
}

/*  queryBox	: Display user query box				*/
/*  exec	: ExecState *	: Execution state			*/
/*  args	: const List &	: Argument list				*/
/*  (returns)	: Value		: True/False on OK/Cancel		*/

static	Value	queryBox
	(	ExecState	*exec,
		const List	&args
	)
{
	QString		message	= kjsStringArg (exec, args, 0) ;
	QString		caption	= kjsStringArg (exec, args, 1) ;
	bool		rc	;

	if (!KBTest::queryBox (message, caption, rc))
		return	kbTestFailed (exec, TR("Query box not expected")) ;

	return	KJS::Boolean(rc)	;
}

/*  popupHelper	: Show popup helper					*/
/*  exec	: ExecState *	: Execution state			*/
/*  args	: const List &	: Argument list				*/
/*  (returns)	: Value		: True if shown OK			*/

static	Value	popupHelper
	(	ExecState	*exec,
		const List	&args
	)
{
	QString	 helper	 = kjsStringArg (exec, args, 0) ;
	QString	 value	 = kjsStringArg (exec, args, 1) ;
	QString	 slot	 = kjsStringArg (exec, args, 3) ;
	QString	 format	 = kjsStringArg (exec, args, 4) ;
	KBObject *target = KBObjectProxy::toKBObject (exec, args.at(2)) ;

	if (target == 0)
	{
		KBError::EError
		(	TR("Target argument to showPopupHelper is not an object"),
			QString::null,
			__ERRLOCN
		)	;
		return	Number(0) ;
	}

	KBHelperPopup::run
		(	helper,
			value,
			target->getDocRoot()->getDocLocation(),
			target,
			slot,
			format
		) ;

	return	Number(1) ;
}

/*  popupPrompt	: Show popup (slot return) prompt			*/
/*  exec	: ExecState *	: Execution state			*/
/*  args	: const List &	: Argument list				*/
/*  (returns)	: Value		: True if shown OK			*/

static	Value	popupPrompt
	(	ExecState	*exec,
		const List	&args
	)
{
	KBObject *target = KBObjectProxy::toKBObject (exec, args.at(0)) ;
	QString	 slot	 = kjsStringArg (exec, args, 1) ;
	QString	 legend	 = kjsStringArg (exec, args, 2) ;
	QString	 caption = kjsStringArg (exec, args, 3) ;

	if (target == 0)
	{
		KBError::EError
		(	TR("Target argument to showPopupPrompt is not an object"),
			QString::null,
			__ERRLOCN
		)	;
		return	Number(0) ;
	}

	KBPopupPrompt *pp = new KBPopupPrompt
				(	caption,
					legend,
					QString::null,
					target,
					slot
				)	;
	if (!pp->ok())
	{	delete	pp	;
		return	Number(0) ;
	}

	pp->show ()	  ;
	return	Number(1) ;
}

/*  popupChoice	: Show popup (slot return) choice box			*/
/*  exec	: ExecState *	: Execution state			*/
/*  args	: const List &	: Argument list				*/
/*  (returns)	: Value		: True if shown OK			*/

static	Value	popupChoice
	(	ExecState	*exec,
		const List	&args
	)
{
	KBObject 	*target = KBObjectProxy::toKBObject (exec, args.at(0)) ;
	QString	 	slot	= kjsStringArg (exec, args, 1) ;
	QString	 	legend	= kjsStringArg (exec, args, 2) ;
	QString	 	caption = kjsStringArg (exec, args, 3) ;

	KJS::Object	options	= KJS::Object::dynamicCast (args.at(4)) ;
	QStringList	cList	= objectToStringList (exec, options) ;

	if (target == 0)
	{
		KBError::EError
		(	TR("Target argument to showPopupPrompt is not an object"),
			QString::null,
			__ERRLOCN
		)	;
		return	Number(0) ;
	}

	KBPopupChoice *pc = new KBPopupChoice
				(	caption,
					legend,
					cList,
					target,
					slot
				)	;
	if (!pc->ok())
	{	delete	pc	;
		return	Number(0) ;
	}

	pc->show ()	  ;
	return	Number(1) ;
}

static	Value	URLRequest
	(	ExecState	*exec,
		const List	&args
	)
{
	KBObject 	*target = KBObjectProxy::toKBObject (exec, args.at(0)) ;
	QString	 	slot	= kjsStringArg (exec, args, 1) ;
	QString	 	url	= kjsStringArg (exec, args, 2) ;
	QString	 	what	= kjsStringArg (exec, args, 3) ;

	if (what.isEmpty()) what = TR("document") ;

	KBURLRequest *r = new KBURLRequest (target, slot) ;
	r->exec (url, what) ;

	return	Number(1)   ;
}


/*  ------------------------------------------------------------------	*/

RekallMainObjectImp::RekallMainObjectImp
	(	ExecState		*,
		ObjectPrototypeImp	*objProto
	)
	:
	ObjectImp (Object(objProto))
{
	DPRINTF
	((	"RekallMainObjectImp::RekallMainObjectImp: called\n"
	))	;
}


Value	RekallMainObjectImp::get
	(	ExecState	 *exec,
		const Identifier &propertyName
	)
	const
{
	DPRINTF
	((	"RekallMainObjectImp::get\n"
	))	;

	return	lookupGet<RekallMainFunctionImp, RekallMainObjectImp, ObjectImp>
			 (	exec,
				propertyName,
				&rekallMainTable,
				this
			 )	;
}


Value	RekallMainObjectImp::getValueProperty
	(	ExecState	*,
		int		token
	)
	const
{
	switch (token)
	{
		default	:
			break	;
	}

	return	Number(-1)	;
}

RekallMainFunctionImp::RekallMainFunctionImp
	(	ExecState	*exec,
		int		id,
		int		length
	)
	:
	InternalFunctionImp
	(
		(FunctionPrototypeImp*)(exec->interpreter()->builtinFunctionPrototype().imp())
	),
	m_id	(id)
{
	DPRINTF
	((	"RekallMainFunctionImp::RekallMainFunctionImp: called\n"
	))	;

	Value	protect (this) ;
	put	(exec, "length", Number(length), DontDelete|ReadOnly|DontEnum) ;
}

bool	RekallMainFunctionImp::implementsCall () const
{
	return	true	;
}

Value	RekallMainFunctionImp::call
	(	ExecState	*exec,
		Object		&,
		const List	&args
	)
{
	DPRINTF
	((	"RekallMainFunctionImp::call: m_id=%d\n",
		m_id
	))	;

	switch (m_id)
	{
		case RekallMainObjectImp::messageBox :
			KBTest::messageBox
			(	kjsStringArg (exec, args, 0),
				kjsStringArg (exec, args, 1)
			)	;
			return	Number (0) ;


		case RekallMainObjectImp::queryBox   :
			return	::queryBox  (exec, args) ;


		case RekallMainObjectImp::choiceBox  :
			return	::choiceBox (exec, args) ;


		case RekallMainObjectImp::print	     :
			fprintf
			(	stdout,
				"%s\n",
				(cchar *)kjsStringArg (exec, args, 0)
			)	;
			return	Number (0) ;


		case RekallMainObjectImp::file       :
			{
			KBFileProxy *proxy = new KBFileProxy ((KBKJSInterpreter *)exec->interpreter()) ;
			KJS::Object object (proxy) ;

			proxy->addBindings (exec, object) ;
			return	object	;
			}

		case RekallMainObjectImp::showPopupHelper :
			return	::popupHelper (exec, args) ;

		case RekallMainObjectImp::showPopupPrompt :
			return	::popupPrompt (exec, args) ;

		case RekallMainObjectImp::showPopupChoice :
			return	::popupChoice (exec, args) ;

		case RekallMainObjectImp::URLRequest	  :
			return	::URLRequest  (exec, args) ;

		case RekallMainObjectImp::webMode :
			return	Number(0) ;

		default	:
			break	;
	}

	return	Number(-1) ;
}
