/***************************************************************************
    file	         : kb_choiceproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>

#include	"kb_classes.h"
#include	"kb_richtext.h"
#include	"kb_block.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_itemproxy.h"
#include	"kb_richtextproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_RICHTEXTPROXY	(0)
#if		__DEBUG_RICHTEXTPROXY
#include	"kb_dprintf.h"
#endif


/*  KBRichTextProxy							*/
/*  KBRichTextProxy							*/
/*		: Constructor for rekall rich text proxy		*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  choice	: KBRichText *		: Rekall choice			*/
/*  (returns)	: KBRichTextProxy	:				*/

KBRichTextProxy::KBRichTextProxy
	(	KBKJSInterpreter	*interp,
		KBRichText		*richtext
	)
	:
	KBItemProxy	(interp, richtext),
	m_richtext	(richtext)
{
}

/*  KBRichTextProxy							*/
/* ~KBRichTextProxy							*/
/*		: Destructor for rekall rich text proxy			*/
/*  (returns)	:		:					*/

KBRichTextProxy::~KBRichTextProxy ()
{
}

/*  KBRichTextProxy							*/
/*  addBindings	: Add proxy bindings to specified richtext		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBRichTextProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"makeSimpleLink",	MethodImp::id_makeSimpleLink,	0	},
	{	0,			0,				0	}
	}	;

	KBItemProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBRichTextProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBRichTextProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBRichTextProxy::get: [%s] cs=%d\n",
		(cchar *)p,
		m_interp->clientSide()
	))	;

	return	KBItemProxy::get (exec, property) ;
}

/*  KBRichTextProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBRichTextProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBItemProxy::put (exec, property, value, attr) ;
}




/*  ------------------------------------------------------------------  */

KBRichTextProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBRichTextProxy	*richtext
	)
	:
	KBItemProxy::MethodImp		(spec, richtext),
	m_richtext			(richtext)
{
}

/*  MethodImp								*/
/*  callBase	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBRichTextProxy::MethodImp::callBase
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	DPRINTF
	((	"KBRichTextProxy::MethodImp::call: [%s] for [%d]\n",
		(cchar *)m_richtext->m_richtext->getName(),
		m_methodSpec->m_id
	))	;

	switch (m_methodSpec->m_id)
	{
		case id_makeSimpleLink :
		{
			QString	 text	;
			text.sprintf
				(	"<a href='link:///%s'>%s</a>",
					kjsStringArg (exec, args, 1).latin1(),
					kjsStringArg (exec, args, 0).latin1()
				)	;
			return	KJS::String (text) ;
		}

		default	:
			break	;
	}

	return	KBItemProxy::MethodImp::callBase (exec, object, args) ;
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBRichTextProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;

	return	callBase (exec, object, args) ;
}


KBObjectProxy
	*makeRichtextProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBRichTextProxy (interp, (KBRichText *)node) ;
}

void	registerKBRichText	()
{
	registerClass	("KBRichText", makeRichtextProxy) ;
}

