/***************************************************************************
    file	         : kb_slotsproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qvariant.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>
#include	<kjs/array_instance.h>


#include	"kb_classes.h"
#include	"kb_form.h"
#include	"kb_config.h"
#include	"kb_slot.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_slotsproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"


#define		__DEBUG_SLOTSPROXY	(0)
#if		__DEBUG_SLOTSPROXY
#include	"kb_dprintf.h"
#endif


/*  ------------------------------------------------------------------	*/

/*  KBSlotsProxy							*/
/*  KBSlotsProxy: Constructor for rekall event(s) proxy			*/
/*  interp	 : KBKJSInterpreter *	: Intepreter object		*/
/*  object	 : KBObject *		: Rekall object			*/
/*  (returns)	: KBSlotsProxy		:				*/

KBSlotsProxy::KBSlotsProxy
	(	KBKJSInterpreter	*interp,
		KBObject		*object
	)
	:
	KJS::ObjectImp	(),
	m_interp	(interp),
	m_object	(object)
{
}

/*  KBSlotsProxy							*/
/* ~KBSlotsProxy: Destructor for rekall object proxy			*/
/*  (returns)	 :		:					*/

KBSlotsProxy::~KBSlotsProxy ()
{
}

/*  KBSlotsProxy							*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBSlotsProxy::addBindings
	(	KJS::ExecState	*,
		KJS::Object	&object
	)
{
	LITER
	(
		KBSlot,
		m_object->getSlots(),
		slot,

		DPRINTF
		((	"KBSlotsProxy::addBindings: add for [%s]\n",
			(cchar *)slot->name()
		))	;
		object.put
		(	m_interp->globalExec(),
			KJS::Identifier(slot->name().latin1()),
			KJS::Object    (new MethodImp(slot, this))
		)	;
	)
}

/*  KBSlotsProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBSlotsProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p	= property.qstring() ;

	if (p == "toString")
	{
		DPRINTF
		((	"KBSlotsProxy::get: [%s]\n",
			(cchar *)p
		))	;

		return	KJS::String (QString("<%1 %2 Slots>")
					.arg(m_object->getElement())
					.arg(m_object->getName   ())
				    )	;
	}

	return	KJS::ObjectImp::get (exec, property) ;
}

/*  KBSlotsProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBSlotsProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KJS::ObjectImp::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBSlotsProxy							*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  slot	: KBSlot *	 : Actual slot				*/
/*  proxy	: KBSlotsProxy * : Associated proxy			*/
/*  (return)	: MethodImp	 :					*/

KBSlotsProxy::MethodImp::MethodImp
	(	KBSlot			*slot,
		const KBSlotsProxy	*proxy
	)
	:
	KJS::ObjectImp	(),
	m_slot		(slot),
	m_proxy		(proxy)
{
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBSlotsProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&,
		const KJS::List		&args
	)
{
	if (args.size() < 2)
	{
		KBError	error
			(	KBError::Error,
				TR("Slot invoked with insufficient arguments"),
				QString(TR("Got %1, expected at least 2")).arg(args.size()),
				__ERRLOCN
			)	;
			
		KBScriptError::processError
			(	new KBScriptError (error, m_slot),
				KBScriptError::Normal
			)	;

		return	KJS::Null() ;
	}

	/* First argument should be the caller, followed by the event.	*/
	/* Other arguments are gathered up into a separate vector.	*/
	KBObject  *caller = KBObjectProxy::toKBObject (exec, args.at(0)) ;
	QString	  event	  = kjsStringArg (exec, args, 1) ;

	if (caller == 0)
	{
		KBError	error
			(	KBError::Error,
				TR("Slot invoked without caller as first argument"),
				QString::null,
				__ERRLOCN
			)	;
			
		KBScriptError::processError
			(	new KBScriptError (error, m_slot),
				KBScriptError::Normal
			)	;

		return	KJS::Null() ;
	}

	KBValue	  *__args = new KBValue[args.size() - 2] ;

	DPRINTF
	((	"KBSlotsProxy::MethodImp::call: args.size()=%d\n",
		args.size()
	))	;

	for (int idx = 2 ; idx < args.size() ; idx += 1)
		__args[idx-2] = KBObjectProxy::fromKJSValue (exec, args[idx]) ;

	KBScriptError	*rc	= 0 ;
	KBValue		resval	;
	m_slot->eventSignal
	(	caller,
		event,
		args.size() - 2,
		__args,
		resval,
		rc,
		0
	)	;
	delete	[] __args   ;

	if (rc != 0)
	{
		KBScriptError::processError (rc, KBScriptError::Normal) ;
		return	KJS::Null() ;
	}

	return	KBObjectProxy::fromKBValue (exec, resval) ;
}
