/***************************************************************************
    file	         : kb_stackproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qvariant.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>
#include	<kjs/array_instance.h>


#include	"kb_classes.h"
#include	"kb_stack.h"
#include	"kb_config.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_stackproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_STACKPROXY	(0)
#if		__DEBUG_STACKPROXY
#include	"kb_dprintf.h"
#endif


/*  ------------------------------------------------------------------	*/

/*  KBStackProxy							*/
/*  KBStackProxy							*/
/*		: Constructor for rekall stack proxy			*/
/*  interp	: KBKJSInterpreter *	: Intepreter object		*/
/*  stack	: KBStack *		: Rekall stack			*/
/*  (returns)	: KBStackProxy		:				*/

KBStackProxy::KBStackProxy
	(	KBKJSInterpreter	*interp,
		KBStack			*stack
	)
	:
	KBObjectProxy	(interp, stack),
	m_stack		(stack)
{
}

/*  KBStackProxy							*/
/* ~KBStackProxy							*/
/*		: Destructor for rekall object proxy			*/
/*  (returns)	:		:					*/

KBStackProxy::~KBStackProxy ()
{
}

/*  KBStackProxy							*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBStackProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods [] =
	{
	{	"currentPage",	MethodImp::id_currentPage,	0	},
	{	0,		0,				0	}
	}	;

	KBObjectProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBStackProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBStackProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	return	KBObjectProxy::get (exec, property) ;
}

/*  KBStackProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBStackProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBObjectProxy::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBStackProxy							*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  spec	: KBMethodSpec *     : Specification			*/
/*  stack	: KBStackProxy *     : Associated proxy			*/
/*  (return)	: MethodImp	     :					*/

KBStackProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBStackProxy	*stack
	)
	:
	KBObjectProxy::MethodImp(spec, stack),
	m_stack			(stack)
{
}

/*  MethodImp								*/
/*  callBase	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBStackProxy::MethodImp::callBase
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBStack *stack	= m_stack->m_stack ;

	DPRINTF
	((	"KBStackProxy::MethodImp::call: m_id=%d\n",
		m_methodSpec->m_id
	))	;

	switch (m_methodSpec->m_id)
	{
		case id_currentPage :
			return	fromKBNode (exec, stack->currentPage()) ;

		default	:
			break	;
	}

	return	KBObjectProxy::MethodImp::callBase (exec, object, args) ;
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBStackProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;

	return	callBase (exec, object, args) ;
}


KBObjectProxy
	*makeStackProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBStackProxy (interp, (KBStack *)node) ;
}

void	registerKBStack	()
{
	registerClass	("KBStack",	makeStackProxy) ;
}

