/***************************************************************************
    file	         : kb_tabberpageproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qvariant.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>
#include	<kjs/array_instance.h>


#include	"kb_classes.h"
#include	"kb_tabber.h"
#include	"kb_config.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_tabberpageproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_TABBERPAGEPROXY	(0)
#if		__DEBUG_TABBERPAGEPROXY
#include	"kb_dprintf.h"
#endif



/*  ------------------------------------------------------------------	*/

/*  KBTabberPageProxy							*/
/*  KBTabberPageProxy							*/
/*		: Constructor for rekall tabber page proxy		*/
/*  interp	: KBKJSInterpreter *	: Intepreter object		*/
/*  tabberPage	: KBTabberPage *	: Rekall tabber page		*/
/*  (returns)	: KBTabberPageProxy	:				*/

KBTabberPageProxy::KBTabberPageProxy
	(	KBKJSInterpreter	*interp,
		KBTabberPage		*tabberPage
	)
	:
	KBObjectProxy	(interp, tabberPage),
	m_tabberPage	(tabberPage)
{
}

/*  KBTabberPageProxy							*/
/* ~KBTabberPageProxy							*/
/*		: Destructor for rekall object proxy			*/
/*  (returns)	:		:					*/

KBTabberPageProxy::~KBTabberPageProxy ()
{
}

/*  KBTabberPageProxy							*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBTabberPageProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods [] =
	{
	{	"setCurrent",	MethodImp::id_setCurrent,	0	},
	{	0,		0,				0	}
	}	;

	KBObjectProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBTabberPageProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBTabberPageProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	return	KBObjectProxy::get (exec, property) ;
}

/*  KBTabberPageProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBTabberPageProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBObjectProxy::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBTabberPageProxy							*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  spec	: KBMethodSpec *     : Specification			*/
/*  tabberPage	: KBTabberPageProxy * : Associated proxy			*/
/*  (return)	: MethodImp	     :					*/

KBTabberPageProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBTabberPageProxy	*tabberPage
	)
	:
	KBObjectProxy::MethodImp(spec, tabberPage),
	m_tabberPage		(tabberPage)
{
}

/*  MethodImp								*/
/*  callBase	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBTabberPageProxy::MethodImp::callBase
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBTabberPage *page = m_tabberPage->m_tabberPage ;

	DPRINTF
	((	"KBTabberPageProxy::MethodImp::call: m_id=%d\n",
		m_methodSpec->m_id
	))	;

	switch (m_methodSpec->m_id)
	{
		case id_setCurrent	:
			DPRINTF
			((	"KBTabberPageProxy::MethodImp::call: setCurrent: [%s.%s]\n",
				(cchar *)page->getElement(),
				(cchar *)page->getName   ()
			))	;
			page->setCurrent () ;
			return	KJS::Null() ;

		default	:
			break	;
	}

	return	KBObjectProxy::MethodImp::callBase (exec, object, args) ;
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBTabberPageProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;

	return	callBase (exec, object, args) ;
}


KBObjectProxy
	*makeTabberPageProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBTabberPageProxy (interp, (KBTabberPage *)node) ;
}

void	registerKBTabberPage	()
{
	registerClass	("KBTabberPage", makeTabberPageProxy) ;
}

