/***************************************************************************
    file	         : kb_tabberproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qvariant.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>
#include	<kjs/array_instance.h>


#include	"kb_classes.h"
#include	"kb_tabber.h"
#include	"kb_config.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_tabberproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"

#define		__DEBUG_TABBERPROXY	(0)
#if		__DEBUG_TABBERPROXY
#include	"kb_dprintf.h"
#endif



/*  ------------------------------------------------------------------	*/

/*  KBTabberProxy							*/
/*  KBTabberProxy							*/
/*		: Constructor for rekall tabber proxy			*/
/*  interp	: KBKJSInterpreter *	: Intepreter object		*/
/*  tabber	: KBTabber *		: Rekall tabber			*/
/*  (returns)	: KBTabberProxy		:				*/

KBTabberProxy::KBTabberProxy
	(	KBKJSInterpreter	*interp,
		KBTabber		*tabber
	)
	:
	KBObjectProxy	(interp, tabber),
	m_tabber	(tabber)
{
}

/*  KBTabberProxy							*/
/* ~KBTabberProxy							*/
/*		: Destructor for rekall object proxy			*/
/*  (returns)	:		:					*/

KBTabberProxy::~KBTabberProxy ()
{
}

/*  KBTabberProxy							*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBTabberProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods [] =
	{
	{	"currentPage",	MethodImp::id_currentPage,	0	},
	{	0,		0,				0	}
	}	;

	KBObjectProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBTabberProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBTabberProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	return	KBObjectProxy::get (exec, property) ;
}

/*  KBTabberProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBTabberProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KBObjectProxy::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBTabberProxy							*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  spec	: KBMethodSpec *     : Specification			*/
/*  tabber	: KBTabberProxy *    : Associated proxy			*/
/*  (return)	: MethodImp	     :					*/

KBTabberProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBTabberProxy	*tabber
	)
	:
	KBObjectProxy::MethodImp(spec, tabber),
	m_tabber		(tabber)
{
}

/*  MethodImp								*/
/*  callBase	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBTabberProxy::MethodImp::callBase
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KBTabber *tabber	= m_tabber->m_tabber ;

	DPRINTF
	((	"KBTabberProxy::MethodImp::call: m_id=%d\n",
		m_methodSpec->m_id
	))	;

	switch (m_methodSpec->m_id)
	{
		case id_currentPage :
			return	fromKBNode (exec, tabber->currentPage()) ;

		default	:
			break	;
	}

	return	KBObjectProxy::MethodImp::callBase (exec, object, args) ;
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBTabberProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;

	return	callBase (exec, object, args) ;
}


KBObjectProxy
	*makeTabberProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBTabberProxy (interp, (KBTabber *)node) ;
}

void	registerKBTabber	()
{
	registerClass	("KBTabber",	makeTabberProxy) ;
}

