/**************************************************************************
    file	         : kb_cookiejar.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"kb_python.h"

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"
#include	"kb_pyscript.h"
#include	"kb_pybase.h"
#include	"kb_cookie.h"

#define		__DEBUG_COOKIEJAR	(0)
#if		__DEBUG_COOKIEJAR
#include	"kb_dprintf.h"
#endif


extern	QString 	getPythonString	(PyObject *) ;
extern	KBCallback	*kbCallback	;


/*  kbPYSetCookie							*/
/*		: Set a cookie value					*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPYSetCookie
	(	 PyObject	*,
		 PyObject	*args
	)
{
 	cchar	*cookie	;
 	cchar	*value	;

	if (!PyArg_ParseTuple (args, __C("ss"), &cookie, &value))
		return	0	;

	DPRINTF
	((	"kbPYSetCookie: [%s]->[%s]\n",
		cookie,
		value
	))	;

	KBCookieJar::self()->setCookie (cookie, value) ;
	return 	  PY_NONE  ;
}

/*  kbPYGetCookie							*/
/*		: Get a cookie value					*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPYGetCookie
	(	 PyObject	*,
		 PyObject	*args
	)
{
 	cchar	*cookie	;

	if (!PyArg_ParseTuple (args, __C("s"), &cookie))
		return	0 ;

	cchar	*value	= KBCookieJar::self()->getCookie (cookie) ;

	DPRINTF
	((	"kbPYGetCookie: [%s]->[%s]\n",
		cookie,
		value
	))	;

	if (value == 0) return PY_NONE ;
	return	PyString_FromString (value) ;
}

/*  kbPYClearCookies							*/
/*		: Clear cookie jar					*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPYClearCookies
	(	 PyObject	*,
		 PyObject	*args
	)
{
	if (!PyArg_ParseTuple (args, __C("")))
		return	0 ;

	KBCookieJar::self()->clear () ;
	return PY_NONE ;
}

/*  kbPYJar	: Get entire cookie jar as a dictionary			*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPYJar
	(	 PyObject	*,
		 PyObject	*args
	)
{
	if (!PyArg_ParseTuple (args, __C("")))
		return	0 ;

	DPRINTF
	((	"kbPYJar: CALLED\n"
	))	;

	const	QAsciiDict<QCString> &jar = KBCookieJar::self()->jar  () ;
	extern	PyObject *qtDictToPyDict  (const QAsciiDict<QCString> &) ;
	return	qtDictToPyDict (jar) ;
}

static	PyMethodDef	cookieFuncs[] =
{
{	__C("setCookie"),	kbPYSetCookie,		METH_VARARGS,	0	},
{	__C("getCookie"),	kbPYGetCookie,		METH_VARARGS,	0	},
{	__C("clear"),		kbPYClearCookies,	METH_VARARGS,	0	},
{	__C("jar"),		kbPYJar,		METH_VARARGS,	0	},
{	0,			0,			0,		0	}
}	;

void	initPYCookie ()
{
	Py_InitModule (__C("RekallCookieJar"), cookieFuncs) ;
}

