/***************************************************************************
    file	         : kb_pybase.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_block.h"
#include	"kb_event.h"
#include	"kb_slot.h"
#include	"kb_dblink.h"

#include	"kb_pybase.h"
#include	"kb_pyscript.h"
#include	"kb_pyvalue.h"

#include	"kb_locator.h"

#include	<qasciidict.h>
#include	<qfile.h>


#define		__DEBUG_PYBASE	(0)
#if		__DEBUG_PYBASE
#include	"kb_dprintf.h"
#endif


static	QAsciiDict<PyObject>	*classMap 	;


/*  makeEventClass							*/
/*		: Make python class used for __events__			*/
/*  node	: KBNode *	: Node which will use the class		*/
/*  (returns)	: PyObject *	: Python (class) object			*/

static	PyObject*makeEventClass
	(	KBNode		*node
	)
{
	static	PyObject *pyEventName = 0;
	static	PyObject *pyCallName  = 0;
	static	PyObject *evName      = 0;

	if (pyEventName == 0)
	{
		pyEventName  = PyString_FromString("Events") ;
		pyCallName   = PyString_FromString("call"  ) ;
		evName       = PyString_FromString("event" ) ;
		
		if (pyEventName == 0 || pyCallName == 0 || evName == 0)
		{
			Py_XDECREF(pyEventName);
			Py_XDECREF(pyCallName);
			Py_XDECREF(evName);
			return 0;
		}
	}

	PyObject *pyEventDict  = PyDict_New() ;
	
	if (pyEventDict == 0)
		return 0;
	
	if (-1 == PyDict_SetItemString (pyEventDict, "__module__", pyEventName))
	{
		Py_DECREF(pyEventDict);
		return 0;
	}

	LITER
	(
		KBAttr,
		node->getAttribs(),
		attr,

		if (attr->isEvent() == 0) continue ;

		/* Attribute is an event. Make a python instance of the	*/
		/* event, then extract the "call" method which we use	*/
		/* to do the work. This gives us a bound method call	*/
		/* which we insert into the disctionary.		*/
		PyKBBase *pybase = new PyKBBase (attr, PyKBBase::m_event) ;
		PyObject *obj    = PyKBBase::makePythonInstance ("KBEvent", pybase) ;
		PyObject *call   = PyObject_GetAttr    (obj, pyCallName)  ;
		PyObject *aname  = PyString_FromString (attr->getName())  ;
		
		if (obj == 0 || call == 0 || aname == 0)
		{
			Py_DECREF(pyEventDict);
			Py_XDECREF(obj);
			Py_XDECREF(call);
			Py_XDECREF(aname);
			return 0;
		}

		if (-1 == PyDict_SetItem
				(
					pyEventDict,
					aname,
					call
				)
		   )
		{
			Py_DECREF(pyEventDict);
			Py_DECREF(obj);
			Py_DECREF(call);
			Py_DECREF(aname);
			return 0;
		}

		Py_DECREF(obj  ) ;
		Py_DECREF(aname) ;
		Py_DECREF(call ) ;
	)

 	PyObject *pyEventClass	= PyClass_New (0, pyEventDict, evName) ;
	Py_DECREF (pyEventDict ) ;
	return	pyEventClass	;
}

/*  makeSlotsClass							*/
/*		: Make python class used for __slots__			*/
/*  node	: KBNode *	: Node which will use the class		*/
/*  (returns)	: PyObject *	: Python (class) object			*/

static	PyObject*makeSlotsClass
	(	KBNode		*node
	)
{
	static	PyObject *pySlotsName = 0;
	static	PyObject *pyCallName  = 0;
	static	PyObject *slName      = 0;

	if (pySlotsName == 0)
	{
		pySlotsName  = PyString_FromString("Slots" ) ;
		pyCallName   = PyString_FromString("call"  ) ;
		slName 	     = PyString_FromString("slots" ) ;
		
		if (pySlotsName == 0 || pyCallName == 0 || slName == 0)
		{
			Py_XDECREF (pySlotsName) ;
			Py_XDECREF (pyCallName ) ;
			Py_XDECREF (slName     ) ;
			return 0;
		}
	}

	PyObject *pySlotsDict  = PyDict_New() ;
	
	if (pySlotsDict == 0)
		return 0;
	
	if (-1 == PyDict_SetItemString (pySlotsDict, "__module__", pySlotsName))
	{
		Py_DECREF(pySlotsDict);
		return 0;
	}

	DPRINTF
	((	"makeSlotsClass: dict is [%p]\n",
		(void *)pySlotsDict
	))	;

	LITER
	(
		KBSlot,
		node->getSlots(),
		slot,

		PyKBBase *pybase = new PyKBBase (slot, PyKBBase::m_slot) ;
		PyObject *obj    = PyKBBase::makePythonInstance ("KBSlot", pybase) ;
		PyObject *call   = PyObject_GetAttr    (obj, pyCallName) ;
		PyObject *aname  = PyString_FromString (slot->name())	 ;

		if (obj == 0 || call == 0 || aname == 0)
		{
			Py_DECREF  (pySlotsDict) ;
			Py_XDECREF (obj  ) ;
			Py_XDECREF (call ) ;
			Py_XDECREF (aname) ;
			return	0  ;
		}

		DPRINTF
		((	"makeSlotsClass: obj=[%p] call=[%p] aname=[%p]\n",
			(void *)obj,
			(void *)call,
			(void *)aname
		))	;

		if (-1 == PyDict_SetItem
				(
					pySlotsDict,
					aname,
					call
				)
		   )
		{
			Py_DECREF  (pySlotsDict) ;
			Py_XDECREF (obj  ) ;
			Py_XDECREF (call ) ;
			Py_XDECREF (aname) ;
			return	0  ;
		}


		Py_DECREF(obj  ) ;
		Py_DECREF(aname) ;
		Py_DECREF(call ) ;
	)

	PyObject *pySlotsClass	= PyClass_New (0, pySlotsDict, slName) ;
	Py_DECREF (pySlotsDict ) ;

	DPRINTF
	((	"makeSlotsClass: slotsClass=[%p] ref=%d\n",
		(void *)pySlotsClass,
		pySlotsClass->ob_refcnt
	))	;

	return	pySlotsClass	 ;
}

/*  dtrPyKBBase	: Destructor callback for PyKBBase objects		*/
/*  pyKBBase	: void *	: Actually the PyKBBase instance	*/
/*  (returns)	: void		:					*/

static	void	dtrPyKBBase
	(	void		*pyKBBase
	)
{
	if (((PyKBBase *)pyKBBase)->m_magic != PyKBBase_MAGIC)
	{
		fprintf
		(	stderr,
			"dtrPyKBBase: ********** BAD MAGIC VALUE *******\n"
		)	;
		return	;
	}

	delete	(PyKBBase *)pyKBBase ;
}


/*  ------------------------------------------------------------------	*/

/*  PyKBBase								*/
/*  PyKBBase	: Constructor for PyKBBase object			*/
/*  kbObject	: void *	: Associated rekall object		*/
/*  type	: const char *	: Object type				*/
/*  (returns)	: PyKBBase	:					*/

PyKBBase::PyKBBase
	(	void		*kbObject,
		const char	*type
	)
{
	DPRINTF
	((	"PyKBBase::PyKBBase: type=%s\n",
		type
	))	;

	m_magic		= PyKBBase_MAGIC	;
	m_type		= type			;
	m_kbObject	= kbObject		;
	m_pyInstance	= 0			;
	m_inherit	= 0			;
}

/*  PyKBBase								*/
/*  ~PyKBBase	: Destructor for PyKBBase object			*/
/*  (returns)	: void		:					*/

PyKBBase::~PyKBBase ()
{
	DPRINTF
	((	"PyKBBase::~PyKBBase: type=%s\n",
		m_type
	))	;

	/* If the rekall object is a database link then it will have	*/
	/* been created when this object was constructed, so delete it.	*/
	/* All other objects exist independantly of this object.	*/
	if (m_type == PyKBBase::m_dblink)
		delete	(KBDBLink *)m_kbObject	;
}

/*  PyKBBase								*/
/*  loadClassExtension							*/
/*		: Load extension code for class				*/
/*  dirPath	: const QString & : Directory path or null for default	*/
/*  className	: const char *	  : Python class name			*/
/*  (returns)	: void		  :					*/

void	PyKBBase::loadClassExtension
	(	const QString	&dirPath,
		const char	*className
	)
{
	QString	path	;

	DPRINTF
	((	"PyKBBase::loadClassExtension: check [%s]\n",
		className
	))	;

	/* If the directory path is null then look for the script in	*/
	/* the default place. The python class name is systematically	*/
	/* modified to get the file name.				*/
	if (dirPath.isNull())
	{
		path	= locateFile ("appdata", QString("script/py/extend/ext_%2.py").arg(className)) ;
		if (path.isEmpty()) return ;

	}
	else	path += QString("%1/ext_%2.py").arg(dirPath).arg(className)  ;


	/* See if the file exists and execute the code therein if so.	*/
	/* Failures are silently ignored.				*/
	QFile	extend	(path) ;
	if (!extend.open(IO_ReadOnly)) return ;

	DPRINTF
	((	"PyKBBase::loadClassExtension: executing\n"
	))	;

	QString code	= extend.readAll() ;
	// xxx check, if an exception occurred
	PyRun_SimpleString(__C(code)) ;
}

/*  PyKBBase								*/
/*  makePythonClass							*/
/*		: Construct a python class definition object		*/
/*  pythonName	: const char *	: Python class name			*/
/*  module	: PyObject *	: Module into which to save definition	*/
/*  methods	: PyMethodDef *	: Method definitions			*/
/*  baseName	: const char *	: Rekall base class name		*/
/*  kbnames	: const char ** : Rekall class names			*/
/*  (returns)	: void		:					*/

void	PyKBBase::makePythonClass
	(	const char	*pythonName,
		PyObject	*module,
		PyMethodDef	*methods,
		const char	*baseName,
		const char	**kbaseNames
	)
{
	/* error check: not likely, but we may get an execption even in	*/
	/* one of the calls of this method. If we ignore such an 	*/
	/* exception in a later call, the exception might be cleared	*/
	if (PyErr_Occurred())
		return;

	PyObject	*dict		= PyDict_New()	;
	PyObject	*name		= PyString_FromString (pythonName) ;
	PyObject	*bases		= 0		;
	const char	*mainName	= *kbaseNames	;
	PyObject	*klass		; 

	if (dict == 0 || name == 0)
	{
		Py_XDECREF(dict);
		Py_XDECREF(name);
		return;
	}
	
	/* The rekall name to python class object map is dynamically	*/
	/* created on the first call.					*/
	if (classMap == 0)
		classMap = new QAsciiDict<PyObject> ;


	/* If there is a rekall base name specified then locate the	*/
	/* corresponding python class object and set this up. We only	*/
	/* ever have a single base class at most, never multiple	*/
	/* inheritance.							*/
	if (baseName != 0)
	{
		PyObject *base = classMap->find (baseName) ;

		if (base != 0)
		{
			bases	= PyTuple_New (1) ;
			if (bases == 0)
			{
				Py_DECREF(dict);
				Py_DECREF(name);
				return;
			}
			Py_XINCREF	 (base)  ;
			PyTuple_SET_ITEM (bases, 0, base) ;
		}
		else	DPRINTF
			((	"PyKBBase::makePythonClass: cannot find base class [%s] for [%s]\n",
				baseName,
				pythonName
			))	;
	}

	/* Create the python class, then associate the set of methods	*/
	/* with it.							*/
	klass	= PyClass_New (bases, dict, name) ;
	Py_XDECREF(bases);
	Py_DECREF(dict);
	Py_DECREF(name);
	if (klass == 0)
		return;

	for ( ; methods->ml_name != 0 ; methods += 1)
	{
		PyObject *func	= PyCFunction_New (methods, 0)	    ;
		PyObject *meth	= PyMethod_New    (func, 0, klass)  ;
		if (func == 0 || meth == 0)
		{
			Py_DECREF(klass);
			return;
		}

		if (-1 == PyDict_SetItemString (dict, methods->ml_name, meth))
		{
			Py_DECREF(klass);
			return;
		}
	}

	/* Insert the class object into the dictionary for the		*/
	/* specified module.						*/
	PyObject	*mdict	= PyModule_GetDict (module) ;
	if (-1 == PyDict_SetItemString (mdict, (char *)pythonName, klass))
	{
		Py_DECREF(klass);
		return;
	}

	/* Lastly, insert entries into the map for each rekall class	*/
	/* with which it is assocated (for some cases, there may be	*/
	/* more than one.						*/
	while (*kbaseNames != 0)
	{
		classMap->insert (*kbaseNames, klass) ;
		kbaseNames += 1	 ;
	}

	if (mainName != 0)
		loadClassExtension (QString::null, mainName) ;
}


/*  PyKBBase								*/
/*  makePythonInstance							*/
/*		: Construct a python instance for a rekall object	*/
/*  kbaseName	: const char *	: Rekall object name			*/
/*  pyBase	: PyKBBase *	: Wrapper				*/
/*  (returns)	: PyObject *	: Python object or null			*/

PyObject*PyKBBase::makePythonInstance
	(	const char	*kbaseName,
		PyKBBase	*pyBase
	)
{
	/* Lookup the class object for the specified object name. If	*/
	/* not found then nothing we can do.				*/
	PyObject *klass	 = classMap->find (kbaseName) ;
	if (klass == 0) return 0 ;

	/* Create a new python instance of the class, and store a	*/
	/* pointer to the wrapper object in the python instance via a	*/
	/* PyCObject in the instance dictionary.			*/
	PyObject *inst	 = PyInstance_New (klass, 0, 0)		;
	if (inst == 0)
		return 0 ;
		
	PyObject *dict   = ((PyInstanceObject *)inst)->in_dict	;
	PyObject *robj	 = PyCObject_FromVoidPtr(pyBase, dtrPyKBBase)	;
	if (robj == 0)
	{
		Py_DECREF(inst) ;
		return 0 ;
	}
	
	if (PyDict_SetItemString (dict, "__rekallObject", robj) == -1)
	{
		Py_DECREF(inst) ;
		Py_DECREF(robj) ;
		return 0 ;
	}

	/* The wrapper object contains a pointer to the python		*/
	/* instance object as well as to the rekall object.		*/
	pyBase->m_pyInstance = (PyInstanceObject *)inst		;
	Py_DECREF(robj) ;

	DPRINTF
	((	"PyKBBase::makePythonInstance: inst->ref=%d robj->ref=%d\n",
		inst->ob_refcnt,
		robj->ob_refcnt
	))	;

	return	inst	;
}

/*  PyKBBase								*/
/*  makePythonInstance							*/
/*		: Construct a python instance for a rekall object	*/
/*  kbNode	: KBNode *	: Rekall node				*/
/*  pyBase	: PyKBBase *	: Wrapper				*/
/*  (returns)	: PyObject *	: Python object or null			*/

PyObject*PyKBBase::makePythonInstance
	(	KBNode		*kbNode
	)
{
	uint	 lc	= KBPYScriptIF::getIface()->languageCode() ;
	KBObject *kbObj	= kbNode->isObject() ;

	if (kbObj != 0)
	{
		KBPYScriptObject *so = (KBPYScriptObject *)kbObj->scriptObject(lc) ;

		DPRINTF
		((	"PyKBBase::makePythonInstance: %s: so=%p\n",
			kbNode->getName().latin1(),
			(void *)so
		))	;

		if (so != 0) return so->object() ;
	}

	/* This method is used for rekall objects derived from KBNode.	*/
	/* First see if we have a python class that corresponds		*/
	/* directly to the rekall node					*/
	PyKBNode *pyNode = new PyKBNode (kbNode) ;
	PyObject *pyObj	 = makePythonInstance
			   (		kbNode->getElement(),
					pyNode
			   )	;

	/* If not then check if the node is a KBItem or a KBObject, in	*/
	/* which case try those. This allows us to pick up common	*/
	/* interface methods for rekall objects that do not have an	*/
	/* explicit python wrapper.					*/
	if (pyObj == 0 && !PyErr_Occurred())
		if (kbNode->isItem  () != 0)
			pyObj = makePythonInstance ("KBItem",   pyNode) ;

	if (pyObj == 0 && !PyErr_Occurred())
		if (kbNode->isObject() != 0)
			pyObj = makePythonInstance ("KBObject", pyNode) ;

	if (pyObj == 0)
	{
		/* Err .... should not get here. This is an error and	*/
		/* will be handled in the main script interface code.	*/
		if (!PyErr_Occurred())
		{
			QString msg = "can't locate Python class object for ";
			msg.append(kbNode->getElement());
			PyErr_SetString(PyKBRekallError, __C(msg));
		}
		delete	pyNode	;
		return	0	;
	}

	/* If we now have a python object, and the node is actually an	*/
	/* object, make the python object persistent by saving a	*/
	/* pointer to it.						*/
	if (kbObj != 0)
		kbObj->setScriptObject (lc, new KBPYScriptObject (pyObj)) ;

	return	pyObj	;
}

/*  PyKBBase								*/
/*  getPyBaseFromPyInst							*/
/*		: Recover wrapper from python instance			*/
/*  pyInst	: PyObject *	: Python instance object		*/
/*  type	: const char *	: Type to check for			*/
/*  error	: const char *&	: Error text return			*/
/*  (returns)	: PyKBBase *	: Wrapper or null on error		*/

PyKBBase*PyKBBase::getPyBaseFromPyInst
	(	PyObject	*pyInst,
		const char	*type,
		const char	*&error
	)
{
	/* There is a bunch of paranoia-induced error checking here.	*/
	/* Start by checking that we actually have an python instance	*/
	/* object and not something else.				*/
	if (!PyInstance_Check (pyInst))
	{
		error	= "getPyBaseFromPyInst: not an instance" ;
		return	0 ;
	}

	/* Get the instance dictionary and from that the PyCObject that	*/
	/* links to the wrapper ... making sure that it really is a	*/
	/* PyCObject!							*/
	PyObject *dict = ((PyInstanceObject *)pyInst)->in_dict  ;
	PyObject *robj = PyDict_GetItemString(dict, "__rekallObject") ;
	if (robj == 0)
	{
		error	= "getPyBaseFromPyInst: no rekall object" ;
		return	0 ;
	}

	if (!PyCObject_Check (robj))
	{
		error	= "getPyBaseFromPyInst: not a PyCObject" ;
		return	0 ;
	}

	/* Fish out the wrapper and check the magic marker. This is not	*/
	/* bomb-proof but should be good enough to trap development	*/
	/* errors. Then check the type tag, which makes sure we have	*/
	/* the correct type of object.					*/
	PyKBBase *pyBase = (PyKBBase *)PyCObject_AsVoidPtr(robj) ;
	if (pyBase->m_magic != PyKBBase_MAGIC)
	{
		error	= "getPyBaseFromPyInst: no magic marker" ;
		return	0 ;
	}
	if ((type != 0) && (pyBase->m_type != type))
	{
		static	QString	etext	;
		etext	= QString(TR("Unexpected rekall object type: expected %1, got %2"))
				.arg(type)
				.arg(pyBase->m_type)
				;
		error	= etext.latin1() ;
		return	0 ;
	}

	return	pyBase	;
}

/*  PyKBBase								*/
/*  parseTuple	: Common argument parsing				*/
/*  caller	: const char *	: Caller for error messages		*/
/*  type	: const char *	: Type to check for			*/
/*  args	: Pyobject *	: Arguments				*/
/*  format	: const char *	: Argument format			*/
/*  a1, ...	: void *, ...	: Return pointers			*/
/*  (returns	: PyKBBase *	: Callers wrapper			*/

PyKBBase*PyKBBase::parseTuple
	(	const char	*caller,
		const char	*type,
		PyObject	*args,
		const char	*format,
		void		*a1,
		void		*a2,
		void		*a3,
		void		*a4
	)
{
	const char *error	;
	PyObject   *pyInst	;

	DPRINTF
	((	"PyKBBase::parseTuple: [%s] [%p][%p][%p][%p][%p]\n",
		format,
		(void *)&pyInst,
		(void *)a1,
		(void *)a2,
		(void *)a3,
		(void *)a4
	))	;

	/* Standard python parsing. Note that the format *must* start	*/
	/* with "O" in order to get the caller object instance.		*/
	if (!PyArg_ParseTuple (args, (char *)format, &pyInst, a1, a2, a3, a4))	
	{
		return	0	;
	}

	PyKBBase *pyBase = PyKBBase::getPyBaseFromPyInst (pyInst, type, error) ;
	if (pyBase == 0)
	{
		DPRINTF
		((	"PyKBBase::parseTuple: %s: %s\n",
			caller,
			error
		))	;
		PyErr_SetString (PyExc_TypeError, error) ;
		return	0 ;
	}

	return	pyBase	;
}

cchar	*PyKBBase::decodeError
	(	const KBError		&error
	)
{
	static  QString	aQString ;
	const	QString	&details = error.getDetails() ;

	aQString = error.getMessage() ;

	if (!details.isEmpty())
	{	aQString += ": "    ;
		aQString += details ;
	}

	return	aQString ;
}


/*  PyKBBase								*/
/*  fromKBValue	 : Get python value from KBValue			*/
/*  value	 : const KBValue &: Value				*/
/*  convertExact : int = 1	  : If 0 return a string representation */
/*				  : else return a Python type "more	*/
/*				  : closely" related to KBValue::type	*/
/*  (returns)	: PyObject *	  : Python value			*/

PyObject*PyKBBase::fromKBValue
	(	const KBValue	&value,
		int   		convertExact	// default = 1
	)
{
	if (value.isNull())
		return	PY_NONE	;

	if (convertExact)
	{
		switch (value.getType()->getIType())
		{
			case KB::ITFixed	:
				return	PyLong_FromLong	   (value.getRawText().toInt   ()) ;

			case KB::ITFloat	:
				return	PyFloat_FromDouble (value.getRawText().toDouble()) ; 

			case KB::ITBool		:
				return	PyLong_FromLong    (value.isTrue() ? 1 : 0) ;

			case KB::ITBinary	:
				return	PyString_FromStringAndSize
					(	value.dataPtr   () ,
						value.dataLength()
					)	;

			case KB::ITNode		:
				if (value.getNode() == 0)
					return	PY_NONE	;
				return	makePythonInstance (value.getNode()) ;

			default	:
				break	;
		}

		return	kb_qStringToPyString (value.getRawText()) ;
	}
	
	/* For conversion into a string, just check for binary data,	*/
	/* otherwise return the raw string value.			*/
	if (value.getType()->getIType() == KB::ITBinary)
	{
		return	PyString_FromStringAndSize
			(	value.dataPtr   () ,
				value.dataLength()
			)	;
	}
	return  kb_qStringToPyString(value.getRawText()) ;
}

/*  PyKBBase								*/
/*  fromPyObject: Generate KBValue from a python object			*/
/*  pyObj	: PyObject *	: Python object				*/
/*  pyErr	: bool &	: Error occurred flag			*/
/*  kbType	: KBType *	: Default type				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	PyKBBase::fromPyObject
	(	PyObject	*pyObj,
		bool		&pyErr,
		KBType		*kbType
	)
{
	pyErr	= false;

	/* Simple cases first. Check for simple python types and if	*/
	/* found then return the corresponding KBValue. Note that the	*/
	/* "string" types are not simple here - see below!		*/
	if (pyObj == Py_None)
		return KBValue () ;

	if (PyInt_Check    (pyObj))
		return (int)PyInt_AsLong (pyObj) ;

	if (PyLong_Check   (pyObj))
		return (int)PyLong_AsLong(pyObj) ;

	if (PyFloat_Check  (pyObj))
		return PyFloat_AsDouble  (pyObj) ;

#if	(PY_MAJOR_VERSION > 2) || (PY_MINOR_VERSION >= 3)
	if (PyBool_Check   (pyObj))
		return KBValue (pyObj == Py_True, &_kbBool) ;
#endif

	/* If the python object is an instance then see if it maps to a	*/
	/* rekall object, in which case return that object in the value	*/
	/* (used to pass objects through slot arguments, for instance).	*/
	/* Any non-rekall-object instance is turned into a descriptive	*/
	/* string.							*/
	if (PyInstance_Check(pyObj))
	{
		DPRINTF
		((	"PyKBBase::fromPyObject: got instance\n"
		))	;

		cchar	 *error	 ;
		PyKBBase *pyBase = PyKBBase::getPyBaseFromPyInst
				   (		pyObj,
						PyKBBase::m_object,
						error
				   )	;

		if (pyBase)
		{
			DPRINTF
			((	"PyKBBase::fromPyObject: %s %p\n",
				pyBase->m_type,
				pyBase->m_kbObject
			))	;

			return	KBValue((KBNode *)pyBase->m_kbObject) ;
		}
		
		return	KBValue (kb_pyStringToQString(pyObj, pyErr)) ;

	}

	/* Next check for a python object which is already a KBValue	*/
	/* (ie, it has type pyKBValueType) in which case simply return	*/
	/* the value.							*/
	KBValue	*kbvp ;
	if ((kbvp = kb_pyKBValueCheck(pyObj)) != 0)
		return	*kbvp	;

	QString	text = kb_pyStringToQString(pyObj, pyErr) ;

	/* A type is passed in if the conversion is to be done prior to	*/
	/* assignment to a control, in which case we get the data type	*/
	/* for the control.						*/
	if (kbType != 0)
	{
		bool	ok	;
		if (kbType->getIType() == KB::ITUnknown)
			if	(QString(text).toInt   (&ok), ok) kbType = &_kbFixed  ;
			else if (QString(text).toDouble(&ok), ok) kbType = &_kbFloat  ;
			else					  kbType = &_kbString ;

		return	KBValue	(text, kbType) ;
	}

	return	KBValue	(text)	;
}

int	PyKBBase::getCurQRow
	(	KBItem		*item,
		int		row
	)
{
	return	row		 >= 0 ? row :
		item->getBlock() == 0 ? 0   :
					item->getBlock()->getCurQRow() ;
}


/*  ------------------------------------------------------------------	*/

/*  PyKBBase								*/
/*  getAttrMethod: Method called for __getattr__			*/
/*  name	 : const char *	: Attribute name			*/
/*  (returns)	 : PyObject *	: Result of null on error		*/

PyObject*PyKBBase::getAttrMethod
	(	const char 	*name
	)
{
	/* If the property did not exist then return failure. Set an	*/
	/* attribute error, giving the same behaviour as if this	*/
	/* method did not exist.					*/
	DPRINTF
	((	"PyKBBase::getAttrMethod: getting [%s]\n",
		name
	))	;

	PyObject *res = PyDict_GetItemString(m_pyInstance->in_dict, (char *)name);
	if (res == 0)
	{
		PyErr_SetString	(PyExc_AttributeError, name) ;
		return	0 ;
	}

	Py_INCREF(res) ;
	return	  res  ;
}

/*  PyKBBase								*/
/*  setAttrMethod: Method called for __setattr__			*/
/*  name	 : const char *	: Attribute name			*/
/*  value	 : PyObject *	: Attribute value			*/
/*  (returns)	 : PyObject *	: Result of null on error		*/

PyObject*PyKBBase::setAttrMethod
	(	const char	*name,
		PyObject 	*value
	)
{
	/* If control gets here then the attribute does not exist at	*/
	/* the python level and is not special, so treat it like a	*/
	/* normal instance assignment.					*/
	DPRINTF
	((	"PyKBBase::setAttrMethod: in_dict [%p] setting [%s]\n",
		(void *)m_pyInstance->in_dict,
		name
	))	;
	if (-1 == PyDict_SetItemString (m_pyInstance->in_dict, (char *)name, value))
		return 0;
	return	PY_NONE	;
}

/*  PyKBBase								*/
/*  reprMethod	: Method called for __repr__				*/
/*  (returns)	: PyObject *	: Result of null on error		*/

PyObject*PyKBBase::reprMethod ()
{
	/* This should not get called, the derived classes should do	*/
	/* something more useful!					*/
	return	PyString_FromFormat
		(	"<%s object at %p>",
			m_pyInstance->ob_type->tp_name,
			(void *)m_pyInstance
		)	;
}

/*  ------------------------------------------------------------------	*/

/*  PyKBNode								*/
/*  PyKBNode	: Constructor for KBNode wrapper			*/
/*  node	: KBNode *	: Node being wrapped			*/
/*  (returns)	: PyKBNode	:					*/

PyKBNode::PyKBNode
	(	KBNode	*node
	)
	:
	PyKBBase	(node, m_object),
	m_node		(node)
{
	m_eventClass	= 0 ;
	m_slotsClass	= 0 ;
}

/*  PyKBNode								*/
/*  ~PyKBNode	: Destructor for KBNode wrapper				*/
/*  (returns)	:		:					*/

PyKBNode::~PyKBNode ()
{
	Py_XDECREF (m_eventClass) ;
	Py_XDECREF (m_slotsClass) ;
}

/*  PyKBNode								*/
/*  getAttrMethod: Method called for __getattr__			*/
/*  name	 : const char *	: Attribute name			*/
/*  (returns)	 : PyObject *	: Result of null on error		*/

PyObject*PyKBNode::getAttrMethod
	(	const char 	*name
	)
{
	DPRINTF
	((	"PyKBNode::getAttrMethod: %s: called [%s]\n",
		m_node->getName().latin1(),
		name
	))	;

	/* Two special cases, the __events__ and __slots__ attributes,	*/
	/* which allow direct access to the nodes events and slots.	*/
	if (qstrcmp (name, "__events__") == 0)
	{
		if (m_eventClass == 0)
			m_eventClass = makeEventClass (m_node) ;

		Py_XINCREF(m_eventClass);
		return	   m_eventClass ;
	}
	if (qstrcmp (name, "__slots__" ) == 0)
	{
		if (m_slotsClass == 0)
			m_slotsClass = makeSlotsClass (m_node) ;

		Py_XINCREF(m_slotsClass);
		return	   m_slotsClass	;
	}

	/* See if the underlying rekall object knows about this		*/
	/* attribute ...						*/
	KBValue	value	 ;
	bool	exists	= m_node->getKBProperty (name, value) ;
	if (exists)
	{
		DPRINTF
		((	"PyKBNode::getAttrMethod: %s: got [%s]->[%s]\n",
			m_node->getName().latin1(),
			name,
			value.getRawText().latin1()
		))	;
		return	PyKBBase::fromKBValue(value) ;
	}

	/* ... else drop through to the base class.			*/
	return	PyKBBase::getAttrMethod (name) ;
}

/*  PyKBNode								*/
/*  setAttrMethod: Method called for __setattr__			*/
/*  name	 : const char *	: Attribute name			*/
/*  value	 : PyObject *	: Attribute value			*/
/*  (returns)	 : PyObject *	: Result of null on error		*/

PyObject*PyKBNode::setAttrMethod
	(	const char	*name,
		PyObject 	*value
	)
{
	DPRINTF
	((	"PyKBNode::setAttrMethod: called [%s]\n",
		name
	))	;

	/* Problem: Let's assume that PyKBBase::fromPyObject fails,	*/
	/* because "value" is something that can't be converted into a	*/
	/* KBValue object, for example a function object or similar.	*/
	/* Then, "pyErr" is set to true, but it might make sense to	*/
	/* store this object as a Python attribute. Hence we should	*/
	/* first check, if this node has a property of this name, and	*/
	/* only if such a property exist, we should call		*/
	/* PyKBBase::fromPyObject					*/
	/* Added by Mike: 09-Jun-06					*/
	if (m_node->hasKBProperty(name))
	{
		bool	pyErr	;
		KBValue kbVal	= PyKBBase::fromPyObject(value, pyErr);
		if (pyErr) return 0 ;

		if (m_node->setKBProperty (name, kbVal))
			return	PY_NONE ;
	}

	return	PyKBBase::setAttrMethod (name, value) ;
}

/*  PyKBNode								*/
/*  reprMethod	: Method called for __repr__				*/
/*  (returns)	: PyObject *	: Result of null on error		*/

PyObject*PyKBNode::reprMethod ()
{
	/* This should not get called, the derived classes should do	*/
	/* something more useful!					*/
	return	PyString_FromFormat
		(	"<%s '%s'>",
			(cchar *)((KBNode *)m_kbObject)->getElement(),
			(cchar *)((KBNode *)m_kbObject)->getName   ()
		)	;
}

/*  ------------------------------------------------------------------	*/

/*  PyKBBase_getattr							*/
/*		: Python __getattr__ method				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Result				*/

static	PyObject*PyKBBase___getattr__
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*pyInst	;
	char		*name	;

	if (!PyArg_ParseTuple
		(	args,
			"Os",
			&pyInst,
			&name
	        ))
		return	0 ;

	DPRINTF
	((	"PyKBBase_getattr: request for [%s]\n",
		name
	))	;

	const char	*error	;
	PyKBBase	*pyBase = PyKBBase::getPyBaseFromPyInst(pyInst, 0, error) ;
	if (pyBase == 0)
	{
		PyErr_SetString(PyExc_TypeError, error);
		return 0;
	}

	return	 pyBase->getAttrMethod (name) ;
}

/*  PyKBBase_setattr							*/
/*		: Python __setattr__ method				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Result				*/

static	PyObject*PyKBBase___setattr__
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*pyInst	;
	char		*name	;
	PyObject	*value	;

	if (!PyArg_ParseTuple
		(	args,
			"OsO",
			&pyInst,
			&name,
			&value
	        ))
		return	0 ;

	DPRINTF
	((	"PyKBBase_setattr: request for [%s] as [%p]\n",
		name,
		value
	))	;

	const char	*error	;
	PyKBBase	*pyBase = PyKBBase::getPyBaseFromPyInst(pyInst, 0, error) ;
	if (pyBase == 0)
	{
		PyErr_SetString(PyExc_TypeError, error);
		return 0;
	}

	return	 pyBase->setAttrMethod (name, value) ;
}

/*  PyKBBase_repr							*/
/*		: Python __repr__ method				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Result				*/

static	PyObject*PyKBBase___repr__
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*pyInst	;

	if (!PyArg_ParseTuple (args, "O", &pyInst)) return 0 ;

	DPRINTF
	((	"PyKBBase_repr: request on [%p]\n",
		(void *)pyInst
	))	;

	const char	*error	;
	PyKBBase	*pyBase = PyKBBase::getPyBaseFromPyInst(pyInst, 0, error) ;
	if (pyBase == 0)
	{
		PyErr_SetString(PyExc_TypeError, error);
		return 0;
	}

	return	 pyBase->reprMethod () ;
}

/*  PyKBBase_super							*/
/*		: Method used to invoke into a linked component		*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Result				*/

static	PyObject*PyKBBase_super
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*pyInst				;
	PyObject	*pyValue[MAX_SUPER_ARGS]	;
	KBValue		kbValue [MAX_SUPER_ARGS]	;
	bool		pyErr;

	for (unsigned int idx = 0 ; idx < MAX_SUPER_ARGS ; idx += 1)
		pyValue[idx] = 0	;

	if (!PyArg_ParseTuple
		(	args,
			"O|OOOO",
			&pyInst,
			&pyValue[0], &pyValue[1],
			&pyValue[2], &pyValue[3]
	        ))
		return	0 ;

	unsigned int	nArgs	;
	for (nArgs = 0 ; nArgs < MAX_SUPER_ARGS ; nArgs += 1)
	{
		PyObject *py = pyValue[nArgs] ;
		if (py == 0) break ;
		kbValue[nArgs] = PyKBBase::fromPyObject (py, pyErr) ;
		if (pyErr)
			return 0;
	}

	const char	*error	;
	PyKBBase	*pyBase = PyKBBase::getPyBaseFromPyInst (pyInst, PyKBBase::m_object, error) ;
	if (pyBase == 0)
	{
		DPRINTF
		((	"PyKBBase::super: %s\n",
			error
		))	;
		PyErr_SetString (PyExc_TypeError, error) ;
		return	0 ;
	}

	if (pyBase->m_inherit == 0)
	{
		DPRINTF
		((	"PyKBBase::super: called with no inheritance\n"
		))	;
		PyErr_SetString (PyExc_TypeError, "PyKBBase::super: called with no inheritance\n");
		return	0 ;
	}

	KBScriptError	*rc	;
	KBValue		resval	;

	API_CALL((KBNode *)pyBase->m_kbObject, "PyKBBase.super") ;
	rc	= pyBase->m_inherit->doExecute (resval, nArgs, kbValue, false) ;
	API_DONE ;

	if (rc != 0)
		KBScriptError::processError (rc, KBScriptError::Normal) ;

	return	PY_NONE ;
}

/*  ------------------------------------------------------------------	*/


static	PyMethodDef pyKBBase_methods[] =
{
	PY_METHOD	(__getattr__,	PyKBBase	),
	PY_METHOD	(__setattr__,	PyKBBase	),
	PY_METHOD	(__repr__,	PyKBBase	),
	PY_METHOD	(super,		PyKBBase	),
	PY_METHOD_END
}	;

static	const char *pyKBBase_names  [] =
{
	"KBBase",
	0
}	;


void	initPyKBBase
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBBase",
		module,
		pyKBBase_methods,
		0,
		pyKBBase_names
	)	;
}


char	*PyKBBase::m_wrapper	= "wrapper"	;
char	*PyKBBase::m_object	= "object"	;
char	*PyKBBase::m_dblink	= "dblink"	;
char	*PyKBBase::m_sql	= "sql"		;
char	*PyKBBase::m_event	= "event"	;
char	*PyKBBase::m_slot	= "slot"	;
