/***************************************************************************
    file	         : kb_pybase.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		__KB_PYBASE_H
#define		__KB_PYBASE_H


extern PyObject *PyKBRekallError	;
extern PyObject *PyKBRekallAbort	;
extern PyObject *PyKBRekallTest		;

#define	PyKBBase_MAGIC		(0x1324fdec)
#define	MAX_SUPER_ARGS		4
#define	MAX_CALL_ARGS		6
#define	PY_NONE			(Py_INCREF(Py_None), Py_None)
#define	INITPYMOD(name,mod)	extern void name (PyObject *) ; name (mod) ;
#define	PY_METHOD(name,obj)	{ #name, obj##_##name, METH_VARARGS, #obj #name }
#define	PY_METHFN(name,obj,fn)	{ #name, fn##_##name,  METH_VARARGS, #obj #name }
#define	PY_METHOD_END		{ 0, 0, 0, 0 }


/*  Script error handling						*/
/*  ---------------------						*/
/*  There is an error handling subtlety we need to deal with. Suppose	*/
/*  a button "onClick" event calls "block.reload()" and the block	*/
/*  "preQuery" event throws an error; then we want report an error in	*/
/*  the latter *and* not allow any further execution in the former when	*/
/*  control unwinds.							*/
/*									*/
/*  To handle this, we wrap all Python->Rekall calls (that might fire	*/
/*  another event) with AP_CALL/API_DONE macros. The former accesses	*/
/*  the document error flag; if this flag is set on return from the	*/
/*  Rekall call then an exception is immediately thrown. The exception	*/
/*  "PyKBRekallAbort" denotes this special case, and does not result in	*/
/*  a further error report. The additional check in API_CALL is likely	*/
/*  not needed (since an exception would have been generated at the	*/
/*  same time), but the red cars might be out to get us anyway.		*/

#define	API_CALL(n,t)	\
	const bool	&ef	= (n)->gotExecError()	;		\
	cchar		*__t	= (t)			;		\
	if (ef)								\
	{	PyErr_SetString (PyKBRekallAbort, __t)	;		\
		return 0				;		\
	}

#define	API_DONE	\
	if (ef)								\
	{	PyErr_SetString (PyKBRekallAbort, __t)	;		\
		return 0				;		\
	}



#ifdef	_SHOWREF
#define	SHOWREF(p,i)					\
		fprintf					\
		(	stderr,				\
			"Ref count %3d on %08x %s\n",	\
			(p)->ob_refcnt,			\
			(int)(p),			\
			i				\
		)	;
#else
#define	SHOWREF(p,i)
#endif	// _SHOWREF


class	QString		;
class	KBNode		;
class	KBItem		;
class	KBEvent		;
class	KBError		;
class	KBValue		;
class	KBPYScriptIF	;
class	KBType		;


/*  PyKBBase								*/
/*  --------								*/
/*  The python instance objects which map to rekall objects point to	*/
/*  objects derived from PyKBBase below; this points back to the python	*/
/*  instance and to the actual python object.				*/

class	PyKBBase
{
public	:

	unsigned int		m_magic			;
	const char		*m_type			;
	void			*m_kbObject		;
	PyInstanceObject	*m_pyInstance		;
	KBEvent			*m_inherit		;

	PyKBBase		(void *, const char *)	;
	virtual ~PyKBBase	()			;


	static	void	loadClassExtension
		(	const QString	&dirPath,
			const char	*className
		)	;

	static	void	makePythonClass
		(	const char	*pythonName,
			PyObject	*module,
			PyMethodDef	*methods,
			const char	*baseName,
			const char	**kbaseNames
		)	;

	static	PyObject*makePythonInstance
		(	const char	*kbaseName,
			PyKBBase	*pyBase
		)	;

	static	PyObject*makePythonInstance
		(	KBNode		*kbaseNode
		)	;

	static	PyKBBase*getPyBaseFromPyInst
		(	PyObject	*pyInst,
			const char	*type,
			const char	*&error
		)	;

	static	PyKBBase*parseTuple
		(	const char	*caller,
			const char	*type,
			PyObject	*args,
			const char	*format,
			void		*a1	= 0,
			void		*a2	= 0,
			void		*a3	= 0,
			void		*a4	= 0
		)	;

	static	const char
			*decodeError
		(	const KBError	&error
		)	;

	static	PyObject*fromKBValue
		(	const KBValue	&value,
			int		convertExact = 1
		)	;
	static	KBValue	fromPyObject
		(	PyObject	*pyObj,
			bool		&pyErr,
			KBType		*kbType	= 0
		)	;

	static	int	getCurQRow
		(	KBItem		*item,
			int		row
		)	;

	/* Virtual methods used for the getattr/setattr/repr functions.	*/
	/* These are overriden for specific cases.			*/
	virtual	PyObject	*getAttrMethod	(const char *)			;
	virtual	PyObject	*setAttrMethod	(const char *, PyObject *)	;
	virtual	PyObject	*reprMethod	()				;

	static	char	*m_wrapper	;
	static	char	*m_object	;
	static	char	*m_dblink	;
	static	char	*m_sql		;
	static	char	*m_event	;
	static	char	*m_slot		;
}	;

/*  PyKBNode								*/
/*  --------								*/
/*  Derived class used where the python instance is wrapping a rekall	*/
/*  KBNode-derived object.						*/

class	PyKBNode : public PyKBBase
{
public	:

	KBNode		*m_node		;
	PyObject	*m_eventClass	;
	PyObject	*m_slotsClass	;

	PyKBNode	  (KBNode *)	;
	virtual	~PyKBNode ()		;

	virtual	PyObject	*getAttrMethod	(const char *)			;
	virtual	PyObject	*setAttrMethod	(const char *, PyObject *)	;
	virtual	PyObject	*reprMethod	()				;
}	;

#endif		// __KB_PYBASE_H
