/***************************************************************************
    file	         : kb_pyblock.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_form.h"
#include	"kb_config.h"

#include	"kb_pybase.h"
#include	"kb_pyvalue.h"

/*  PyKBBlock_getNumRows						*/
/*		: Ger number of rows					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBBlock_getNumRows
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBBlock.getNumRows",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	int	 nRows	;
	KBBlock	 *block	= (KBBlock *)pyBase->m_kbObject ;

	API_CALL(block,"KBBlock.getNumRows") ;
	nRows	= block->getNumRows() ;
	API_DONE  ;

	return	PyInt_FromLong ((long)nRows) ;
}

/*  PyKBBlock_getQueryRow						*/
/*		: Get current row					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBBlock_getQueryRow
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBBlock.getQueryRow",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	int	 qRow	;
	KBBlock	 *block	= (KBBlock *)pyBase->m_kbObject ;

	API_CALL(block,"KBBlock.getNumRows") ;
	qRow	= block->getCurQRow() ;
	API_DONE  ;

	return	PyInt_FromLong ((long)qRow) ;
}

/*  PyKBBlock_getRowValue						*/
/*		: Get value from for for named column			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBBlock_getRowValue
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *name	 ;
	int	 qrow	 ;
	bool	 error	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBBlock.getRowValue",
					PyKBBase::m_object,
					args,
					"OOi",
					&name,
					&qrow
				) ;
	if (pyBase == 0) return 0 ;

	QString qName = kb_pyStringToQString(name, error);
	if (error) return 0;
	
	// xxx no check, if the current row or the column exists.
	/// we simply return Py_NONE, which is also returned for
	// a null value in an existing row/column
	// shouldn't we raise an error in the former cases?
		
	KBValue	 value	;
	KBBlock	 *block	= (KBBlock *)pyBase->m_kbObject ;

	API_CALL(block,"KBBlock.getNumRows")
	value	= block->getRowValue(qName, qrow) ;
	API_DONE  ;

	return PyKBBase::fromKBValue (value) ;
}

/*  PyKBBlock_setRowValue						*/
/*		: Set value for named column				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBBlock_setRowValue
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *name	 ;
	int	 qrow	 ;
	PyObject *value	 ;
	QString	 qName	 ;
	KBValue	 kValue	 ;
	bool	 error	 ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBBlock.setRowValue",
					PyKBBase::m_object,
					args,
					"OOiO",
					&name,
					&qrow,
					&value
				) ;
	if (pyBase == 0) return 0 ;
	
	qName = kb_pyStringToQString(name, error);
	if (error) return 0;
	
	kValue = PyKBBase::fromPyObject(value, error);
	if (error) return 0;
	
	KBBlock	 *block	= (KBBlock *)pyBase->m_kbObject ;

	API_CALL(block,"KBBlock.setRowValue") ;
	// xxx shouldn't we check, if row and column exist?
	block->setRowValue (qName, qrow, kValue) ;
	API_DONE  ;

	return	PY_NONE ;
}



/*  PyKBBlock_setUserFilter						*/
/*		: Set user filter on block				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBBlock_setUserFilter
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *filter ;
	QString	 qFilter ;
	bool	 error	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBBlock.setUserFilter",
					PyKBBase::m_object,
					args,
					"OO",
					&filter
				) ;
	if (pyBase == 0) return 0 ;
	
	qFilter = kb_pyStringToQString(filter, error);
	if (error) return 0;

	KBBlock	 *block	= (KBBlock *)pyBase->m_kbObject ;

	API_CALL(block,"KBBlock.setUserFilter") ;
	block->setUserFilter (qFilter) ;
	API_DONE ;

	return	PY_NONE ;
}

/*  PyKBBlock_setUserSorting						*/
/*		: Set user sorting on block				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBBlock_setUserSorting
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *order  ;
	QString	 qOrder	 ;
	bool	 error	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBBlock.setUserSorting",
					PyKBBase::m_object,
					args,
					"OO",
					&order
				) ;
	if (pyBase == 0) return 0 ;
	
	qOrder = kb_pyStringToQString(order, error);
	if (error) return 0 ;

	KBBlock	 *block	= (KBBlock *)pyBase->m_kbObject ;

	API_CALL(block,"KBBlock.setUserSorting") ;
	block->setUserSorting (qOrder) ;
	API_DONE ;

	return	PY_NONE ;
}

/*  PyKBBlock_overLimit							*/
/*		: Check if query ran over limit				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button on				*/

static	PyObject*PyKBBlock_overLimit
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"PyKBBlock.overLimit",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	uint	over	;
	KBBlock	*block	= (KBBlock *)pyBase->m_kbObject ;

	API_CALL(block,"PyKBBlock.overLimit") ;
	over	= block->overLimit() ;
	API_DONE  ;

	return	PyInt_FromLong ((long)over) ;
}

/*  ------------------------------------------------------------------	*/

static	PyMethodDef PyKBBlock_methods[] =
{
	PY_METHOD	(getNumRows,	PyKBBlock	),
	PY_METHOD	(getQueryRow,	PyKBBlock	),
	PY_METHOD	(getRowValue,	PyKBBlock	),
	PY_METHOD	(setRowValue,	PyKBBlock	),
	PY_METHOD	(setUserFilter,	PyKBBlock	),
	PY_METHOD	(setUserSorting,PyKBBlock	),
	PY_METHOD	(overLimit,	PyKBBlock	),
	PY_METHOD_END
}	;

static	const char *PyKBBlock_names  [] =
{
	"KBBlock",
	0
}	;

void	initPyKBBlock
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBBlock",
		module,
		PyKBBlock_methods,
		"KBItem",
		PyKBBlock_names
	)	;
}
