/***************************************************************************
    file	         : kb_pybutton.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_button.h"

#include	"kb_pybase.h"
#include	"kb_pyvalue.h"


/*  PyKBButton_getText							*/
/*		: Get button text					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button text				*/

static	PyObject*PyKBButton_getText
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBButton.getText",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	QString		text	;
	KBButton	*button	= (KBButton *)pyBase->m_kbObject ;

	API_CALL(button,"KBButton.getText") ;
	text = button->getText() ;
	API_DONE ;

	return	kb_qStringToPyString(text) ;
}

/*  PyKBButton_setText							*/
/*		: Set button text					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBButton_setText
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *text	 ;
	QString  qText	 ;
	bool	 error	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBButton.setText",
					PyKBBase::m_object,
					args,
					"OO",
					&text
				) ;
	if (pyBase == 0) return 0 ;
	
	qText = kb_pyStringToQString(text, error);
	if (error) return 0;

	KBButton *button = (KBButton *)pyBase->m_kbObject ;

	API_CALL(button,"KBButton.setText") ;
	button->setText (qText) ;
	API_DONE ;

	return	PY_NONE ;
}

/*  PyKBButton_isOn							*/
/*		: Test if button is on					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button on				*/

static	PyObject*PyKBButton_isOn
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBButton.isOn",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	bool	 on	 ;
	KBButton *button = (KBButton *)pyBase->m_kbObject ;

	API_CALL(button,"KBButton.isOn") ;
	on	= button->isOn() ;
	API_DONE ;

	/* Return value from long on! Hey, its just not cricket!	*/
	return	PyInt_FromLong ((long)on) ;
}

/*  PyKBButton_setOn							*/
/*		: Set button on or off					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBButton_setOn
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 on	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBButton.setText",
					PyKBBase::m_object,
					args,
					"Oi",
					&on
				) ;
	if (pyBase == 0) return 0 ;

	KBButton *button = (KBButton *)pyBase->m_kbObject ;

	API_CALL(button,"KBButton.setOn") ;
	button->setOn (on) ;
	API_DONE ;

	return	PY_NONE ;
}

/*  PyKBButton_setPixmaps						*/
/*		: Set button pixmaps					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBButton_setPixmaps
	(	PyObject	*,
		PyObject	*args
	)
{
	char	 *pm1	 ;
	char	 *pm2	 ;
	char	 *pm3	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBButton.setText",
					PyKBBase::m_object,
					args,
					"Oss|s",
					&pm1,
					&pm2,
					&pm3
				) ;
	if (pyBase == 0) return 0 ;

	KBButton *button = (KBButton *)pyBase->m_kbObject ;

	API_CALL(button,"KBButton.setOn") ;
	button->setPixmaps (pm1, pm2) ;
	API_DONE ;

	return	PY_NONE ;
}

/*  PyKBButton_setTollTip						*/
/*		: Set button tooltip					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBButton_setToolTip
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *text	 ;
	QString  qText	 ;
	bool	 error	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBButton.setToolTip",
					PyKBBase::m_object,
					args,
					"OO",
					&text
				) ;
	if (pyBase == 0) return 0 ;
	
	qText = kb_pyStringToQString(text, error);
	if (error) return 0 ;

	KBButton *button = (KBButton *)pyBase->m_kbObject ;

	API_CALL(button,"KBButton.setText") ;
	button->setToolTip (qText) ;
	API_DONE ;

	return	PY_NONE ;
}

/*  ------------------------------------------------------------------	*/

static	PyMethodDef pyKBButton_methods[] =
{
	PY_METHOD	(getText,	PyKBButton	),
	PY_METHOD	(setText,	PyKBButton	),
	PY_METHOD	(isOn,		PyKBButton	),
	PY_METHOD	(setOn,		PyKBButton	),
	PY_METHOD	(setPixmaps,	PyKBButton	),
	PY_METHOD	(setToolTip,	PyKBButton	),
	PY_METHOD_END
}	;
static	const char *pyKBButton_names  [] =
{
	"KBButton",
	0
}	;

void	initPyKBButton
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBButton",
		module,
		pyKBButton_methods,
		"KBObject",
		pyKBButton_names
	)	;
}
