/***************************************************************************
    file	         : kb_pychoice.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_choice.h"
#include	"kb_block.h"

#include	"kb_pybase.h"
#include	"kb_pyvalue.h"

/*  PyKBChoice_currentItem						*/
/*		: Get current item					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button text				*/

static	PyObject*PyKBChoice_currentItem
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 qRow	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBChoice.currentItem",
					PyKBBase::m_object,
					args,
					"Oi",
					&qRow
				) ;
	if (pyBase == 0) return 0 ;

	KBChoice *kbChoice = (KBChoice *)pyBase->m_kbObject ;
	qRow	= PyKBBase::getCurQRow (kbChoice, qRow) ;

	int	current	;

	API_CALL(kbChoice,"KBChoice.currentItem") ;
	current	= kbChoice->currentItem(qRow)	  ;
	API_DONE ;

	return	PyLong_FromLong (current) ;
}

/*  PyKBChoice_setCurrentItem						*/
/*		: Set current item					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button text				*/

static	PyObject*PyKBChoice_setCurrentItem
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 qRow	 ;
	int	 current ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBChoice.currentItem",
					PyKBBase::m_object,
					args,
					"Oii",
					&qRow,
					&current
				) ;
	if (pyBase == 0) return 0 ;

	KBChoice *kbChoice = (KBChoice *)pyBase->m_kbObject ;
	qRow	= PyKBBase::getCurQRow (kbChoice, qRow) ;

	API_CALL(kbChoice,"KBChoice.currentItem") ;
	kbChoice->setCurrentItem(qRow, current) ;
	API_DONE ;

	return	PY_NONE	;
}

/*  PyKBChoice_setValues						*/
/*		: Set selection values					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button text				*/

static	PyObject*PyKBChoice_setValues
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *atuple	;
	bool	 error		;

	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBChoice.setValues",
					PyKBBase::m_object,
					args,
					"OO",
					&atuple
				)	;
	if (pyBase == 0) return	0	;

	if (!PySequence_Check(atuple))
	{
		PyErr_SetString
		(	PyExc_TypeError, 
			"argument for choice.setValues must be a sequence"
		)	;
		return 0;
	}

	QStringList vals	;
	for (int idx = 0 ; idx < PyList_Size(atuple) ; idx += 1)
	{
		vals.append (kb_pyStringToQString(PyList_GET_ITEM (atuple, idx), error)) ;
		if (error) return 0 ;
	}

	KBChoice *kbChoice = (KBChoice *)pyBase->m_kbObject ;

	API_CALL(kbChoice,"KBChoice.setValues") ;
	kbChoice->setValues (vals) ;
	API_DONE ;

	return	PY_NONE	;
}

/*  PyKBChoice_getValues						*/
/*		: Set selection values					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button text				*/

static	PyObject*PyKBChoice_getValues
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBChoice.getValues",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	KBChoice *kbChoice = (KBChoice *)pyBase->m_kbObject ;

	API_CALL(kbChoice,"KBChoice.getValues") ;
	const QStringList &values = kbChoice->getValues () ;
	bool		  noblank = kbChoice->getAttrVal("noblank") == "Yes" ;
	uint		  offset  = noblank ? 0 : 1 ;
	API_DONE ;

//	fprintf
//	(	stderr,
//		"PyKBChoice_getValues: v.c=%d nob=%d off=%d\n",
//		values.count(),
//		noblank,
//		offset
//	)	;

	PyObject  *vList  = PyList_New(values.count() - offset) ;
	if (vList == 0) return 0;

	for (uint idx = offset ; idx < values.count() ; idx += 1)
	{	
		PyObject *item = kb_qStringToPyString(values[idx]);
		if (item == 0)
		{
			Py_DECREF(vList);
			return 0;
		}
		PyList_SET_ITEM (vList, idx - offset, item) ;
	}

	return	  vList  ;
}

/*  PyKBChoice_getNumValues						*/
/*		: Get number of values in link/tree			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBChoice_getNumValues
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBChoice.getNumValues",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;


	KBChoice *kbChoice = (KBChoice *)pyBase->m_kbObject ;

	API_CALL(kbChoice,"KBChoice.getNumValues") ;
	uint	nVals	= kbChoice->getNumValues () ;
	bool	noblank = kbChoice->getAttrVal("noblank") == "Yes" ;
	uint	offset  = noblank ? 0 : 1 ;
	API_DONE  ;

	return	PyInt_FromLong ((long)(nVals - offset)) ;
}

/*  ------------------------------------------------------------------	*/

static	PyMethodDef pyKBChoice_methods[] =
{
	PY_METHOD	(currentItem,		PyKBChoice	),
	PY_METHOD	(setCurrentItem,	PyKBChoice	),
	PY_METHOD	(setValues,		PyKBChoice	),
	PY_METHOD	(getValues,		PyKBChoice	),
	PY_METHOD	(getNumValues,		PyKBChoice	),
	PY_METHOD_END
}	;
static	const char *pyKBChoice_names  [] =
{
	"KBChoice",
	0
}	;

void	initPyKBChoice
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBChoice",
		module,
		pyKBChoice_methods,
		"KBItem",
		pyKBChoice_names
	)	;
}
