/***************************************************************************
    file	         : kb_pycontainer.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	<qpixmap.h>
#include	<qcstring.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_framer.h"
#include	"kb_display.h"
#include	"kb_docroot.h"

#include	"kb_pybase.h"
#include	"kb_pyscript.h"


/*  PyKBContainer_setBackgroundImage					*/
/*		: Set background image from image data			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBContainer_setBackgroundImage
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *pyImage ;
	int	 scale	  = 0 ;
	PyKBBase *pyBase  = PyKBBase::parseTuple
				(	"KBContainer.setBackgroundImage",
					PyKBBase::m_object,
					args,
					"OO|i",
					&pyImage,
					&scale
				) ;
	if (pyBase == 0) return 0 ;

	if (!PyString_Check (pyImage))
	{
		PyErr_SetString
		(	PyExc_TypeError,
			"setBackgroundImage: Expected image buffer"
		)	;
		return	0 ;
	}

	char	*buffer	;
	int	len	;
	PyString_AsStringAndSize (pyImage, &buffer, (Py_ssize_t *)&len) ;

	QByteArray ba	;
	ba.duplicate	(buffer, len) ;

	KBContainer	*cont	= (KBContainer *)pyBase->m_kbObject ;

	API_CALL(cont,"KBContainer.setBackgroundImage") ;
	cont->getContainer()->setBackgroundPixmap (QPixmap(ba), scale) ;
	API_DONE ;

	return	PY_NONE	;
}

/*  PyKBContainer_setBackgroundGraphic					*/
/*		: Set background image from database graphic		*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBContainer_setBackgroundGraphic
	(	PyObject	*,
		PyObject	*args
	)
{
	char	 *name	 ;
	int	 scale	 = 0 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBContainer.setBackgroundGraphic",
					PyKBBase::m_object,
					args,
					"Os|i",
					&name,
					&scale
				) ;
	if (pyBase == 0) return 0 ;


	QStringList	iparts	= QStringList::split ('.', name) ;
	KBContainer	*cont	= (KBContainer *)pyBase->m_kbObject ;

//	fprintf
//	(	stderr,
//		"PyKBContainer_setBackgroundGraphic: [%s][%s]\n",
//		iparts[0].latin1(),
//		iparts[1].latin1()
//	)	;

	KBLocation location
	(	cont->getDocRoot()->getDBInfo(),
		"graphic",
		cont->getDocRoot()->getDocLocation().server(),
		iparts[0],
		iparts[1]
	)	;

	KBError	   error	;
	QByteArray image	;

	if (!location.contents (image, error))
	{
		PyErr_SetString
		(	PyExc_TypeError,
			"setBackgroundGraphic: cannot load graphic"
		)	;
		return	0 ;
	}

	API_CALL(cont,"KBContainer.setBackgroundGraphic") ;

	cont->getContainer()->setBackgroundPixmap
	(	QPixmap(image),
		scale
	)	;

	API_DONE;

	return	PY_NONE	;
}

/*  ------------------------------------------------------------------	*/

static	PyMethodDef PyKBContainer_methods[] =
{
	PY_METHOD	(setBackgroundImage,	PyKBContainer	),
	PY_METHOD	(setBackgroundGraphic,	PyKBContainer	),
	PY_METHOD_END
}	;

static	const char *PyKBContainer_names  [] =
{
	"KBContainer",
	0
}	;

void	initPyKBContainer
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBContainer",
		module,
		PyKBContainer_methods,
		"KBFramer",
		PyKBContainer_names
	)	;
}
