/***************************************************************************
    file	         : kb_pydblink.cpp
    copyright            : (C) 1999,2000,2001,2002,2003,2004 by Mike Richardson
			   (C) 2001,2002,2003,2004 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_dblink.h"

#include	"kb_pybase.h"
#include	"kb_pyvalue.h"


/*  getQueryArguments							*/
/*		: Get arguments for query execution			*/
/*  alist	: PyObject *	: Argument list				*/
/*  nvals	: uint &	: Return number of arguments		*/
/*  values	: KBValue *&	: Return vector of arguments		*/
/*  (returns)	: bool		: Success				*/

bool	getQueryArguments
	(	PyObject	*alist,
		uint		&nvals,
		KBValue		*&values
	)
{
	extern	QString	getPythonString	(PyObject *) ;

//	fprintf
//	(	stderr,
//		"getQueryArguments: alist=%p\n",
//		(void *)alist
//	)	;

	if (alist == 0)
	{
		nvals	= 0	;
		values	= 0	;
		return	true	;
	}

	if (!PySequence_Check(alist))
	{
		fprintf
		(	stderr,
			"getQueryArguments: bad arg type: %s\n",
			(cchar *)getPythonString(PyObject_Type(alist))
		)	;
		PyErr_SetString (PyExc_TypeError, "query arguments must be a list or tuple") ;
		return	false	;
	}

	nvals	= PySequence_Size (alist) ;
	values	= new KBValue [nvals]     ;
//	fprintf
//	(	stderr,
//		"getQueryArguments: %d arguments\n",
//		nvals
//	)	;


	for (uint idx = 0 ; idx < nvals ; idx += 1)
	{
		PyObject *arg = PySequence_GetItem (alist, idx) ;
		bool     pyErr;

		Py_DECREF(arg);
		values[idx] = KBValue (PyKBBase::fromPyObject(arg, pyErr))   ;
		if (pyErr)
			return false;

//		fprintf
//		(	stderr,
//			"      %2d: [%s]\n",
//			idx,
//			(cchar *)values[idx].getRawText()
//		)	;
	}

	return	true	;
}

/*  PyKBDBLink_listTypes						*/
/*		: Test if object is enabled				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBDBLink_listTypes
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBDBLink.listTypes",
					PyKBBase::m_dblink,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	QString	types	;

	//Py_BEGIN_ALLOW_THREADS
	types	= ((KBDBLink *)pyBase->m_kbObject)->listTypes() ;
	//Py_END_ALLOW_THREADS

	return	PyString_FromString (types) ;
}

/*  PyKBDBLink_getDBType						*/
/*		: Get database type					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBDBLink_getDBType
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBDBLink.getDBType",
					PyKBBase::m_dblink,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	QString	type	;

	//Py_BEGIN_ALLOW_THREADS
	type	= ((KBDBLink *)pyBase->m_kbObject)->databaseType() ;
	//Py_END_ALLOW_THREADS

	return	PyString_FromString (type) ;
}

/*  PyKBDBLink_rekallPrefix						*/
/*		: Get rekall table prefix				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBDBLink_rekallPrefix
	(	PyObject	*,
		PyObject	*args
	)
{
	char	 *name	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBDBLink.getDBType",
					PyKBBase::m_dblink,
					args,
					"Os",
					&name
				) ;
	if (pyBase == 0) return 0 ;

	QString	prefix	;

	//Py_BEGIN_ALLOW_THREADS
	prefix	= ((KBDBLink *)pyBase->m_kbObject)->rekallPrefix (name) ;
	//Py_END_ALLOW_THREADS

	return	PyString_FromString (prefix) ;
}

/*  PyKBDBLink_placeHolder						*/
/*		: Get place holder for position				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBDBLink_placeHolder
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 slot	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBDBLink.placeHolder",
					PyKBBase::m_dblink,
					args,
					"Oi",
					&slot
				) ;
	if (pyBase == 0) return 0 ;

	QString	holder	;

	//Py_BEGIN_ALLOW_THREADS
	holder	= ((KBDBLink *)pyBase->m_kbObject)->placeHolder (slot) ;
	//Py_END_ALLOW_THREADS

	return	PyString_FromString (holder) ;
}

/*  PyKBDBLink_keepsCase						*/
/*		: See if database preserves case			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBDBLink_keepsCase
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBDBLink.keepsCase",
					PyKBBase::m_dblink,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	bool	keeps	;

	//Py_BEGIN_ALLOW_THREADS
	keeps	= ((KBDBLink *)pyBase->m_kbObject)->keepsCase() ;
	//Py_END_ALLOW_THREADS

	return	PyLong_FromLong (keeps) ;
}

/*  PyKBDBLink_fixCase							*/
/*		: Fix case of text string				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBDBLink_fixCase
	(	PyObject	*,
		PyObject	*args
	)
{
	char	 *text	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBDBLink.fixCase",
					PyKBBase::m_dblink,
					args,
					"Os",
					&text
				) ;
	if (pyBase == 0) return 0 ;

	QString	result	;

	//Py_BEGIN_ALLOW_THREADS
	result	= ((KBDBLink *)pyBase->m_kbObject)->rekallPrefix (text) ;
	//Py_END_ALLOW_THREADS

	return	PyString_FromString (result) ;
}

/*  PyKBDBLink_qrySelect						*/
/*		: Create a select query					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBDBLink_qrySelect
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*query	;
	QString		qQuery	;
	bool		error	;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBDBLink.qrySelect",
					PyKBBase::m_dblink,
					args,
					"OO",
					&query
				) ;
	if (pyBase == 0) return 0 ;

	qQuery = kb_pyStringToQString(query, error);
	if (error) return 0 ;
	
	KBSQLSelect *select ;

	//Py_BEGIN_ALLOW_THREADS
	select	= ((KBDBLink *)pyBase->m_kbObject)->qrySelect (true, qQuery) ;
	//Py_END_ALLOW_THREADS

	if (select == 0)
		return	PY_NONE	;

	return	PyKBBase::makePythonInstance
			(	"KBSQLSelect",
				new PyKBBase (select, PyKBBase::m_sql)
			)	;
}

/*  PyKBDBLink_qryInsert						*/
/*		: Create an insert query				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBDBLink_qryInsert
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*query	;
	PyObject	*table	;
	QString		qQuery	;
	QString		qTable	;
	bool		error	;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBDBLink.qryInsert",
					PyKBBase::m_dblink,
					args,
					"OOO",
					&query,
					&table
				) ;
	if (pyBase == 0) return 0 ;
	
	qQuery = kb_pyStringToQString(query, error);
	if (error) return 0;
	
	qTable = kb_pyStringToQString(table, error);
	if (error) return 0;

	KBSQLInsert *insert ;

	//Py_BEGIN_ALLOW_THREADS
	insert	= ((KBDBLink *)pyBase->m_kbObject)->qryInsert (true, qQuery, qTable) ;
	//Py_END_ALLOW_THREADS

	if (insert == 0)
		return	PY_NONE	;

	return	PyKBBase::makePythonInstance
			(	"KBSQLInsert",
				new PyKBBase (insert, PyKBBase::m_sql)
			)	;
}

/*  PyKBDBLink_qryUpdate						*/
/*		: Create an update query				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBDBLink_qryUpdate
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*query	;
	PyObject	*table	;
	QString		qQuery	;
	QString		qTable	;
	bool		error	;

	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBDBLink.qryUpdate",
					PyKBBase::m_dblink,
					args,
					"OOO",
					&query,
					&table
				) ;
	if (pyBase == 0) return 0 ;

	qQuery = kb_pyStringToQString(query, error);
	if (error) return 0;
	
	qTable = kb_pyStringToQString(table, error);
	if (error) return 0;

	KBSQLUpdate *update ;

	//Py_BEGIN_ALLOW_THREADS
	update	= ((KBDBLink *)pyBase->m_kbObject)->qryUpdate (true, qQuery, qTable) ;
	//Py_END_ALLOW_THREADS

	if (update == 0)
		return	PY_NONE	;

//	fprintf
//	(	stderr,
//		"PyKBDBLink_qryUpdate: returning object\n"
//	)	;

	return	PyKBBase::makePythonInstance
			(	"KBSQLUpdate",
				new PyKBBase (update, PyKBBase::m_sql)
			)	;
}

/*  PyKBDBLink_qryDelete						*/
/*		: Create a delete query					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBDBLink_qryDelete
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*query	;
	PyObject	*table	;
	QString		qQuery	;
	QString		qTable	;
	bool		error	;

	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBDBLink.qryDelete",
					PyKBBase::m_dblink,
					args,
					"OOO",
					&query,
					&table
				) ;
	if (pyBase == 0) return 0 ;

	qQuery = kb_pyStringToQString(query, error);
	if (error) return 0;
	
	qTable = kb_pyStringToQString(table, error);
	if (error) return 0;

	KBSQLDelete *delrec ;

	//Py_BEGIN_ALLOW_THREADS
	delrec	= ((KBDBLink *)pyBase->m_kbObject)->qryDelete (true, qQuery, qTable) ;
	//Py_END_ALLOW_THREADS

	if (delrec == 0)
		return	PY_NONE	;

	return	PyKBBase::makePythonInstance
			(	"KBSQLDelete",
				new PyKBBase (delrec, PyKBBase::m_sql)
			)	;
}

/*  PyKBDBLink_getColumnWidth						*/
/*		: Get current width for column				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBDBLink_getColumnWidth
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*tabName;
	PyObject	*colName;
	QString		qTabName;
	QString		qColName;
	bool		error;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBDBLink.getDBType",
					PyKBBase::m_dblink,
					args,
					"OOO",
					&tabName,
					&colName
				) ;
	if (pyBase == 0) return 0 ;

	qTabName = kb_pyStringToQString(tabName, error);
	if (error) return 0;
	
	qColName = kb_pyStringToQString(colName, error);
	if (error) return 0;
	
	int	width	;

	//Py_BEGIN_ALLOW_THREADS
	width	= ((KBDBLink *)pyBase->m_kbObject)->getColumnWidth (qTabName, qColName) ;
	//Py_END_ALLOW_THREADS

	return	PyLong_FromLong (width) ;
}

/*  PyKBDBLink_setColumnWidth						*/
/*		: Set current width for column				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBDBLink_setColumnWidth
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*tabName;
	PyObject	*colName;
	QString		qTabName;
	QString		qColName;
	bool		error;
	int	 width	 ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBDBLink.getDBType",
					PyKBBase::m_dblink,
					args,
					"OOOi",
					&tabName,
					&colName,
					&width
				) ;
	if (pyBase == 0) return 0 ;

	qTabName = kb_pyStringToQString(tabName, error);
	if (error) return 0;
	
	qColName = kb_pyStringToQString(colName, error);
	if (error) return 0;
	
	//Py_BEGIN_ALLOW_THREADS
	((KBDBLink *)pyBase->m_kbObject)->setColumnWidth (qTabName, qColName, width) ;
	//Py_END_ALLOW_THREADS

	return	PY_NONE	;
}

/*  ------------------------------------------------------------------	*/

static	PyMethodDef PyKBDBLink_methods[] =
{
	PY_METHOD	(listTypes,		PyKBDBLink	),
	PY_METHOD	(getDBType,		PyKBDBLink	),
	PY_METHOD	(rekallPrefix,		PyKBDBLink	),
	PY_METHOD	(placeHolder,		PyKBDBLink	),
	PY_METHOD	(keepsCase,		PyKBDBLink	),
	PY_METHOD	(fixCase,		PyKBDBLink	),
	PY_METHOD	(qrySelect,		PyKBDBLink	),
	PY_METHOD	(qryInsert,		PyKBDBLink	),
	PY_METHOD	(qryUpdate,		PyKBDBLink	),
	PY_METHOD	(qryDelete,		PyKBDBLink	),
	PY_METHOD	(setColumnWidth,	PyKBDBLink	),
	PY_METHOD	(getColumnWidth,	PyKBDBLink	),
	PY_METHOD_END
}	;

static	const char *PyKBDBLink_names  [] =
{
	"KBDBLink",
	0
}	;

void	initPyKBDBLink
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBDBLink",
		module,
		PyKBDBLink_methods,
		"KBBase",
		PyKBDBLink_names
	)	;
}
