/***************************************************************************
    file	         : kb_pyevent.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_node.h"
#include	"kb_script.h"
#include	"kb_event.h"

#include	"kb_pybase.h"

#define		__DEBUG_PYEVENT		(0)
#if		__DEBUG_PYEVENT
#include	"kb_dprintf.h"
#endif

static	PyObject*PyKBEvent_call
	(	PyObject	*,
		PyObject	*args
	)
{
	bool		pyErr			;
	PyObject	*pyInst			;
	PyObject	*pyValue[MAX_CALL_ARGS]	;
	memset	(pyValue, 0, sizeof(pyValue))	;

	if (!PyArg_ParseTuple
		(	args,
			"O|OOOOOO",
			&pyInst,
			&pyValue[0], &pyValue[1],
			&pyValue[2], &pyValue[3],
			&pyValue[4], &pyValue[5]
	        ))
		return	0 ;

	const char	*error	;
	PyKBBase	*pyBase = PyKBBase::getPyBaseFromPyInst
						(	pyInst,
							PyKBBase::m_event,
							error
						)	;
	if (pyBase == 0)
	{
		DPRINTF
		((	"PyKBEvent.call: %s\n",
			error
		))	;
		PyErr_SetString(PyExc_TypeError, error);
		return	0 ;
	}

	KBValue		kbValue [MAX_CALL_ARGS]	;
	unsigned int	nArgs	;
	for (nArgs = 0 ; nArgs < MAX_CALL_ARGS ; nArgs += 1)
	{
		PyObject *py = pyValue[nArgs] ;
		if (py == 0) break ;
		kbValue[nArgs] = PyKBBase::fromPyObject (py, pyErr) ;
		if (pyErr)
			return 0;
	}

	KBEvent		*event	= (KBEvent *)pyBase->m_kbObject ;
	KBValue		resval	;
	KBScriptError	*rc	;

	API_CALL(event->getOwner(), "PyKBEvent.call") ;
	rc = event->doExecute (resval, nArgs, kbValue, false) ;
	API_DONE ;

	if (rc != 0)
	{
		KBScriptError::processError (rc, KBScriptError::Normal) ;
		PyErr_SetString (PyKBRekallError, "Event error") ;
		return	0 ;
	}

	return	PyKBBase::fromKBValue (resval) ;
}



static	PyMethodDef pyKBEvent_methods[] =
{
{	"call", 	PyKBEvent_call,		METH_VARARGS,	"PyKBEvent.call"	},
{	0,		0,			0,		0			}
}	;
static	const char *pyKBEvent_names  [] =
{
	"KBEvent",
	0
}	;

void	initPyKBEvent
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBEvent",
		module,
		pyKBEvent_methods,
		0,
		pyKBEvent_names
	)	;
}
