/***************************************************************************
    file	         : kb_pyform.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_form.h"
#include	"kb_report.h"
#include	"kb_config.h"
#include	"kb_appptr.h"
#include	"kb_copyexec.h"
#include	"kb_callback.h"

#include	"kb_pybase.h"
#include	"kb_pyscript.h"
#include	"kb_pyvalue.h"


extern	bool	pyDictToQtDict	(PyObject *, QDict<QString> &) ;
extern	PyObject*qtDictToPyDict	(const       QDict<QString> &) ;


/*  KBPYOpenInfo							*/
/*  ----------								*/
/*  Helper class. There is a bunch of common stuff needed to open a	*/
/*  form or report, gather this up here.				*/

class	KBPYOpenInfo
{
public	:

	QString		m_name		;
	PyObject	*m_pyDict	;
	PyObject	*m_pyKey	;
	PyKBBase	*m_pyBase	;
	QDict<QString>	m_kbDict	;
	QDict<QString>	m_results	;
	KBValue		m_kbKey		;
	KBForm		*m_form		;
	KBError		m_error		;

	KBPYOpenInfo	(const char *, PyObject *, const char *) ;

	KB::ShowRC	exec	(KBLocation &) ;
}	;

/*  KBYOpenInfo								*/
/*  KBYOpenInfo	: Constructor for helper class				*/
/*  caller	: const char *	: Called name for errors		*/
/*  args	: PyObject *	: Python arguments			*/
/*  format	: const char *	: Argument format string		*/
/*  (returns)	: KBPYOpenInfo	: m_pyBase will be null on error	*/

KBPYOpenInfo::KBPYOpenInfo
	(	const char	*caller,
		PyObject	*args,
		const char	*format
	)
{
	bool pyErr 	= false	;
	PyObject	*pName	;
	
	m_pyDict	= 0	;
	m_pyKey		= 0	;
	m_pyBase	= PyKBBase::parseTuple
				(	caller,
					PyKBBase::m_object,
					args,
					format,
					&pName,
					&m_pyDict,
					&m_pyKey
				)	;
	if (m_pyBase == 0) return ;

	m_name = kb_pyStringToQString(pName, pyErr);
	if (pyErr) 
	{
		m_pyBase = 0;
		return;
	}
	
	if (m_pyDict != 0)
		if (!PyDict_Check(m_pyDict))
		{
			m_pyBase = 0 ;
			PyErr_SetString (PyExc_TypeError, "argument must be a dictionary") ;
			return	;
		}

	m_results.setAutoDelete (true) 	     ;
	m_kbDict .setAutoDelete (true)	     ;
	if (!pyDictToQtDict	(m_pyDict, m_kbDict))
	{
		m_pyBase = 0;
		return;
	}

	if (m_pyKey != 0) m_kbKey = PyKBBase::fromPyObject (m_pyKey, pyErr) ;
	if (pyErr)
	{
		m_pyBase = 0;
		return;
	}
	m_form	= (KBForm *)m_pyBase->m_kbObject ;
}

/*  KBPYOpenInfo							*/
/*  exec	: Execute form or whatever				*/
/*  location	: KBLocation &	: Object to execute			*/
/*  (returns)	: KB::ShowRC	: Execution code			*/

KB::ShowRC
	KBPYOpenInfo::exec
	(	KBLocation	&location
	)
{
	KBCallback *cb	= KBAppPtr::getCallback() ;

	if (cb == 0)
		return	KB::ShowRCError ;

	DPRINTF
	((	"KBPYOpenInfo::exec: [%s][%s]\n",
		location.    server().latin1(),
		location.dataServer().latin1()
	))	;

	return	cb->openObject
			  (	m_form,
				location,
				KB::ShowAsData,
				m_kbDict,
				m_error,
				m_kbKey,
				&m_results
			  )	;
}


/*  ------------------------------------------------------------------	*/

/*  PyKBForm_openForm							*/
/*		: Open a form by name					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_openForm
	(	PyObject	*,
		PyObject	*args
	)
{
	KBPYOpenInfo	info ("KBForm.openForm", args, "OO|OO") ;
	if (info.m_pyBase == 0) return 0 ;

	KBDocRoot  *docRoot = info.m_form->getDocRoot() ;
	KBLocation location 
	(	docRoot->getDBInfo  (),
		"form",
		docRoot->getDocLocation().server(),
		info.m_name
	)	;
	location.setDataServer (docRoot->getDocLocation().dataServer()) ;

	KB::ShowRC	rc	;

	API_CALL(info.m_form,"KBForm.openForm") ;
	rc	= info.exec (location) ;
	API_DONE;

	if (rc == KB::ShowRCError)
	{	info.m_error.DISPLAY() ;
		return	PyInt_FromLong (0) ;
	}

	return	qtDictToPyDict (info.m_results) ;
}

/*  PyKBForm_openReport							*/
/*		: Open a report by name					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_openReport
	(	PyObject	*,
		PyObject	*args
	)
{
	KBPYOpenInfo	info ("KBForm.openReport", args, "OO|OO") ;
	if (info.m_pyBase == 0) return 0 ;

	KB::ShowRC	rc	;

	KBDocRoot  *docRoot = info.m_form->getDocRoot() ;
	KBLocation location 
	(	docRoot->getDBInfo  (),
		"report",
		docRoot->getDocLocation().server(),
		info.m_name
	)	;
	location.setDataServer (docRoot->getDocLocation().dataServer()) ;

	API_CALL(info.m_form,"KBForm.openReport") ;
	rc	= info.exec (location) ;
	API_DONE;

	if (rc == KB::ShowRCError)
	{	info.m_error.DISPLAY() ;
		return	PyInt_FromLong (0) ;
	}

	return	PyInt_FromLong (1) ;
}

/*  PyKBForm_openTextForm						*/
/*		: Open a form by text definition			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_openTextForm
	(	PyObject	*,
		PyObject	*args
	)
{
	KBPYOpenInfo	info ("KBForm.openTextForm", args, "OO|OO") ;
	if (info.m_pyBase == 0) return 0 ;

	KB::ShowRC	rc	;

	/* Note that the "info.m_name" field is actually the text	*/
	/* definition.							*/
	KBDocRoot  *docRoot = info.m_form->getDocRoot() ;
	KBLocation location 
	(	docRoot->getDBInfo  (),
		"form",
		KBLocation::m_pInline,
		"xmlTextForm",
		info.m_name
	)	;
	location.setDataServer (docRoot->getDocLocation().dataServer()) ;

	API_CALL(info.m_form,"KBForm.openTextForm") ;
	rc	= info.exec (location) ;
	API_DONE;

	if (rc == KB::ShowRCError)
	{	info.m_error.DISPLAY() ;
		return	PyInt_FromLong (0) ;
	}

	return	qtDictToPyDict (info.m_results) ;
}

/*  PyKBForm_openTextReport						*/
/*		: Open a report by text definition			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_openTextReport
	(	PyObject	*,
		PyObject	*args
	)
{
	KBPYOpenInfo	info ("KBForm.openTextReport", args, "OO|OO") ;
	if (info.m_pyBase == 0) return 0 ;

	KB::ShowRC	rc	;

	KBDocRoot  *docRoot = info.m_form->getDocRoot() ;
	KBLocation location 
	(	docRoot->getDBInfo  (),
		"report",
		KBLocation::m_pInline,
		"xmlTextReport",
		info.m_name
	)	;
	location.setDataServer (docRoot->getDocLocation().dataServer()) ;

	API_CALL(info.m_form,"KBForm.openTextReport") ;
	rc	= info.exec (location) ;
	API_DONE;

	if (rc == KB::ShowRCError)
	{	info.m_error.DISPLAY() ;
		return	PyInt_FromLong (0) ;
	}

	return	PyInt_FromLong (1) ;
}

/*  PyKBForm_openTable							*/
/*		: Open a table by name					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_openTable
	(	PyObject	*,
		PyObject	*args
	)
{
	KBPYOpenInfo	info ("KBForm.openTable", args, "OO|O") ;
	if (info.m_pyBase == 0) return 0 ;

	KB::ShowRC	rc	;

	KBDocRoot  *docRoot = info.m_form->getDocRoot() ;
	KBLocation location 
	(	docRoot->getDBInfo  (),
		"table",
		docRoot->getDocLocation().server(),
		info.m_name
	)	;
	location.setDataServer (docRoot->getDocLocation().dataServer()) ;

	API_CALL(info.m_form,"KBForm.openTable") ;
	rc	= info.exec (location) ;
	API_DONE;

	if (rc == KB::ShowRCError)
	{	info.m_error.DISPLAY() ;
		return	PyInt_FromLong (0) ;
	}

	return	PyInt_FromLong (1) ;
}

/*  PyKBForm_openQuery							*/
/*		: Open a query by name					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_openQuery
	(	PyObject	*,
		PyObject	*args
	)
{
	KBPYOpenInfo	info ("KBForm.openQuery", args, "OO|O") ;
	if (info.m_pyBase == 0) return 0 ;

	KB::ShowRC	rc	;

	KBDocRoot  *docRoot = info.m_form->getDocRoot() ;
	KBLocation location 
	(	docRoot->getDBInfo  (),
		"query",
		docRoot->getDocLocation().server(),
		info.m_name
	)	;
	location.setDataServer (docRoot->getDocLocation().dataServer()) ;

	API_CALL(info.m_form,"KBForm.openQuery") ;
	rc	= info.exec (location) ;
	API_DONE;

	if (rc == KB::ShowRCError)
	{	info.m_error.DISPLAY() ;
		return	PyInt_FromLong (0) ;
	}

	return	PyInt_FromLong (1) ;
}

/*  PyKBForm_executeCopier						*/
/*		: Execute a copier					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_executeCopier
	(	PyObject	*,
		PyObject	*args
	)
{
	KBPYOpenInfo info ("KBForm.executeCopier", args, "OO|O") ;
	if (info.m_pyBase == 0) return 0 ;

	KBDocRoot  *docRoot = info.m_form->getDocRoot() ;
	KBLocation location 
	(	docRoot->getDBInfo  (),
		"copier",
		docRoot->getDocLocation().server(),
		info.m_name
	)	;
	location.setDataServer (docRoot->getDocLocation().dataServer()) ;

	API_CALL(info.m_form,"KBForm.executeCopier")

	QString	report	;
	int	nRows	= KBCopyExec::execDocument
			  (	location,
				report,
				info.m_error,
				info.m_kbDict
			  )	;
	API_DONE

	if (nRows < 0)
		info.m_form->setError (info.m_error) ;

	return	PyInt_FromLong (nRows) ;
}

/*  PyKBForm_serverSetting						*/
/*		: Open database connection to server			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_serverSetting
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*setting = 0 ;
	PyObject	*name	 = 0 ;
	bool	 	pyErr 	 = false ;
	PyKBBase *pyBase  = PyKBBase::parseTuple
				(	"KBForm.serverSetting",
					PyKBBase::m_object,
					args,
					"OO|O",
					&setting,
					&name
				) ;
	if (pyBase == 0) return 0 ;

	QString		 svrName;
	if (name)
		svrName = kb_pyStringToQString(name, pyErr) ;
	if (pyErr) return 0;
	
	QString		 qSetting = kb_pyStringToQString(setting, pyErr);
	if (pyErr) return 0;
	
	KBForm		 *form   = (KBForm *)pyBase->m_kbObject ;
	const KBLocation &locn   = form->getDocRoot()->getDocLocation() ;

	if (svrName.isEmpty()) svrName = locn.server() ;

	KBServerInfo  *svinfo = locn.dbInfo()->findServer (svrName) ;
	if (svinfo == 0) return PY_NONE ;

	KBError  error	 ;
	KBServer *server = svinfo->getServer (error) ;
	if (server == 0) return PY_NONE ;


	return	kb_qStringToPyString (server->setting(qSetting)) ;
}

/*  PyKBForm_findOpenForm						*/
/*		: Find extant open form					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_findOpenForm
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *name	  = 0 ;
	QString	  qName	      ;
	bool	  error       ;
	PyKBBase *pyBase  = PyKBBase::parseTuple
				(	"KBForm.findOpenForm",
					PyKBBase::m_object,
					args,
					"OO",
					&name
				) ;
	if (pyBase == 0) return 0 ;

	qName = kb_pyStringToQString(name, error);
	if (error) return 0;
	
	KBForm	*form	  = (KBForm *)pyBase->m_kbObject ;

	KBLocation location 
	(	form->getDocRoot()->getDBInfo  (),
		"form",
		form->getDocRoot()->getDocLocation().server(),
		qName
	)	;

	KBNode	*node = KBAppPtr::getCallback()->objectNode (location) ;
	if ((node == 0) || (node->isForm() == 0))
		return	PY_NONE	;

	return	PyKBBase::makePythonInstance (node) ;
}

/*  PyKBForm_close							*/
/*		: Close form						*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_close
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 rc	 = 0	;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBForm.close",
					PyKBBase::m_object,
					args,
					"O|i",
					&rc
				) ;
	if (pyBase == 0) return 0 ;

	KBForm	*form	  = (KBForm *)pyBase->m_kbObject ;

	API_CALL(form,"KBForm.close") ;
	form->getDocRoot()->doRequestClose(rc) ;
	API_DONE;

	return	PY_NONE	;
}

/*  PyKBForm_closeAll							*/
/*		: Close forms						*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_closeAll
	(	PyObject	*self,
		PyObject	*args
	)
{
	return	PyKBForm_closeAll (self, args) ;
}

/*  PyKBForm_getOpener							*/
/*		: Get opener if set					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_getOpener
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase  = PyKBBase::parseTuple
				(	"KBForm.getOpener",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	KBNode	*opener	= ((KBForm *)pyBase->m_kbObject)->opener() ;

	if (opener == 0)
		return	PY_NONE	;

	return	PyKBBase::makePythonInstance (opener) ;
}

/*  PyKBForm_getParameter						*/
/*		: Get parameter value					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button text				*/

static	PyObject*PyKBForm_getParameter
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*name	;
	QString		qName	;
	bool		error	;
	PyKBBase 	*pyBase	= PyKBBase::parseTuple
					(	"KBForm.getParameter",
						PyKBBase::m_object,
						args,
						"OO",
						&name
					) ;
	if (pyBase == 0) return 0 ;
	
	qName = kb_pyStringToQString(name, error);
	if (error) return 0;

	QString	param	;
	KBForm	*form	  = (KBForm *)pyBase->m_kbObject ;

	API_CALL(form,"KBForm.getParameter") ;
	param = form->isDocRoot()->getParamValue (qName) ;
	API_DONE;

	return	param.isNull () ?
			PY_NONE :
			kb_qStringToPyString (param);
}

/*  PyKBForm_parameterDict						*/
/*		: Get parameter value					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button text				*/

static	PyObject*PyKBForm_parameterDict
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase 	*pyBase	= PyKBBase::parseTuple
					(	"KBForm.parameterDict",
						PyKBBase::m_object,
						args,
						"O"
					) ;
	if (pyBase == 0) return 0 ;
	
	KBForm	 *form	  = (KBForm *)pyBase->m_kbObject ;
	PyObject *pyDict  ;

	API_CALL(form,"KBForm.getParameter") ;

	const KBAttrDict &kbDict = form->isDocRoot()->parameterDict() ;
	pyDict	= qtDictToPyDict (kbDict) ;

	API_DONE;

	return	pyDict ;
}

/*  PyKBForm_showMessage						*/
/*		: Show message in status bar				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button text				*/

static	PyObject*PyKBForm_showMessage
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*message ;
	QString		qMessage ;
	bool		error	 ;
	PyKBBase 	*pyBase	 = PyKBBase::parseTuple
					(	"KBForm.showMessage",
						PyKBBase::m_object,
						args,
						"OO",
						&message
					) ;
	if (pyBase == 0) return 0 ;
	
	qMessage = kb_pyStringToQString(message, error);
	if (error) return 0 ;

	KBForm	*form	  = (KBForm *)pyBase->m_kbObject ;

	API_CALL(form,"KBForm.showMessage") ;
	form->isDocRoot()->doSetMessage (qMessage) ;
	API_DONE;

	return	PY_NONE	;
}

/*  PyKBForm_getGlobalCtrl						*/
/*		: Get globally named control				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_getGlobalCtrl
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *name	 ;
	QString	 qName	 ;
	bool	 error	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBForm.getGlobalCtrl",
					PyKBBase::m_object,
					args,
					"OO",
					&name
				) ;
	if (pyBase == 0) return 0 ;
	
	qName = kb_pyStringToQString (name, error) ;
	if (error) return 0 ;

	KBForm	*kbForm	= (KBForm *)pyBase->m_kbObject ;
	KBNode	*ctrl	;

	API_CALL(kbForm,"KBForm.getGlobalCtrl") ;
	ctrl	= kbForm->getGlobalNode (qName) ;
	API_DONE ;

	if (ctrl == 0)
		return	PY_NONE ;

	return	PyKBBase::makePythonInstance (ctrl) ;
}

static	QString	hashColor
	(	const QString		&color,
		bool			hash
	)
{
	if (hash)
		if (color.left(2).lower() == "0x")
			return	"#" + color.mid(2) ;

	return	color	;
}

/*  PyKBForm_getSkinFGColor						*/
/*		: Get skin foreground colour				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_getSkinFGColor
	(	PyObject	*,
		PyObject	*args
	)
{
	const char *name	;
	int	   hash		= false ;
	PyKBBase   *pyBase	= PyKBBase::parseTuple
				(	"KBForm.getSkinFGColor",
					PyKBBase::m_object,
					args,
					"Os|i",
					&name,
					&hash
				) ;
	if (pyBase == 0) return 0 ;
	
	KBDocRoot *kbDocRoot	= ((KBForm *)pyBase->m_kbObject)->getDocRoot() ;
	QString	  color		;

	API_CALL(kbDocRoot,"KBForm.getSkinFGColor") ;
	color	= kbDocRoot->skinFGColor  (name)	;
	API_DONE ;

	return	color.isEmpty () ?
			PY_NONE :
			kb_qStringToPyString (hashColor(color, hash)) ;
}

/*  PyKBForm_getSkinBGColor						*/
/*		: Get skin foreground colour				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_getSkinBGColor
	(	PyObject	*,
		PyObject	*args
	)
{
	const char *name	;
	int	   hash		= false ;
	PyKBBase   *pyBase	= PyKBBase::parseTuple
				(	"KBForm.getSkinBGColor",
					PyKBBase::m_object,
					args,
					"Os|i",
					&name,
					&hash
				) ;
	if (pyBase == 0) return 0 ;
	
	KBDocRoot *kbDocRoot	= ((KBForm *)pyBase->m_kbObject)->getDocRoot() ;
	QString	  color		;

	API_CALL(kbDocRoot,"KBForm.getSkinBGColor") ;
	color	= kbDocRoot->skinBGColor  (name) ;
	API_DONE ;

	return	color.isEmpty () ?
			PY_NONE :
			kb_qStringToPyString (hashColor(color, hash)) ;
}

/*  PyKBForm_getSkinFont						*/
/*		: Get skin font						*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBForm_getSkinFont
	(	PyObject	*,
		PyObject	*args
	)
{
	const char *name	;
	PyKBBase   *pyBase	= PyKBBase::parseTuple
				(	"KBForm.getSkinFont",
					PyKBBase::m_object,
					args,
					"Os",
					&name
				) ;
	if (pyBase == 0) return 0 ;
	
	KBDocRoot *kbDocRoot	= ((KBForm *)pyBase->m_kbObject)->getDocRoot() ;
	QString	  font		;

	API_CALL(kbDocRoot,"KBForm.getFont")	;
	font	= kbDocRoot->skinFont (name)	;
	API_DONE ;

	return	font.isEmpty () ?
			PY_NONE :
			kb_qStringToPyString (font) ;
}

/*  ------------------------------------------------------------------	*/

static	PyMethodDef PyKBForm_methods[] =
{
	PY_METHOD	(openForm,		PyKBForm	),
	PY_METHOD	(openReport,		PyKBForm	),
	PY_METHOD	(openTextForm,		PyKBForm	),
	PY_METHOD	(openTextReport,	PyKBForm	),
	PY_METHOD	(openTable,		PyKBForm	),
	PY_METHOD	(openQuery,		PyKBForm	),
	PY_METHOD	(executeCopier,		PyKBForm	),
	PY_METHOD	(serverSetting,		PyKBForm	),
	PY_METHOD	(findOpenForm,		PyKBForm	),
	PY_METHOD	(close,			PyKBForm	),
	PY_METHOD	(closeAll,		PyKBForm	),
	PY_METHOD	(getOpener,		PyKBForm	),
	PY_METHOD	(getParameter,		PyKBForm	),
	PY_METHOD	(parameterDict,		PyKBForm	),
	PY_METHOD	(showMessage,		PyKBForm	),
	PY_METHOD	(getGlobalCtrl,		PyKBForm	),
	PY_METHOD	(getSkinFGColor,	PyKBForm	),
	PY_METHOD	(getSkinBGColor,	PyKBForm	),
	PY_METHOD	(getSkinFont,		PyKBForm	),
	PY_METHOD_END
}	;

static	const char *PyKBForm_names  [] =
{
	"KBForm",
	0
}	;

void	initPyKBForm
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBForm",
		module,
		PyKBForm_methods,
		"KBFormBlock",
		PyKBForm_names
	)	;
}
