/***************************************************************************
    file	         : kb_pyformblock.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_form.h"
#include	"kb_config.h"

#include	"kb_pybase.h"
#include	"kb_pyscript.h"

/*  PyKBFormBlock_gotoQueryRow						*/
/*		: Move to sepecified row				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBFormBlock_gotoQueryRow
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 qrow	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBFormBlock.gotoQueryRow",
					PyKBBase::m_object,
					args,
					"Oi",
					&qrow
				) ;
	if (pyBase == 0) return 0 ;

	bool		rc	;
	KBFormBlock	*fb	= (KBFormBlock *)pyBase->m_kbObject ;

	API_CALL(fb,"KBFormBlock.gotoQueryRow") ;
	rc	= fb->gotoQRow (qrow) ;
	API_DONE;

	if (!rc)
	{	fb->lastError().display() ;
		PyErr_SetString (PyExc_TypeError, fb->lastError().getMessage()) ;
		return	0 ;
	}

	return	PY_NONE ;
}

/*  PyKBFormBlock_gotoQueryRowByKey					*/
/*		: Move to sepecified row				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBFormBlock_gotoQueryRowByKey
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *key	 ;
	bool	 error	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBFormBlock.gotoQueryRowByKey",
					PyKBBase::m_object,
					args,
					"OO",
					&key
				) ;
	if (pyBase == 0) return 0 ;

	KBFormBlock	*fb	= (KBFormBlock *)pyBase->m_kbObject ;
	int		row	;

	API_CALL(fb,"KBFormBlock.gotoQueryRowByKey") ;
	row	= fb->gotoQRow (PyKBBase::fromPyObject(key, error)) ;
	API_DONE;

	if (error) return 0	;

	return	PyLong_FromLong (row) ;
}

static	PyObject*PyKBFormBlock_doAction
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 action	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"PyKBFormBlock.doAction",
					PyKBBase::m_object,
					args,
					"Oi",
					&action
				) ;

//	fprintf
//	(	stderr,
//		"PyKBFormBlock_doAction: action=%d pyBase=%p\n",
//		action,
//		(void *)pyBase
//	)	;

	if (pyBase == 0) return 0 ;

	KBFormBlock	*fb	= (KBFormBlock *)pyBase->m_kbObject ;
	bool		 rc	 ;

	API_CALL(fb,"PyKBFormBlock.doAction") ;
	rc	= fb->doAction ((KB::Action)action) ;
	API_DONE ;

	return	PyLong_FromLong (rc) ;
}

static	PyObject*PyKBFormBlock_action
	(	cchar		*caller,
		PyObject	*args,
		KB::Action	action
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	caller,
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	KBFormBlock	*fb	= (KBFormBlock *)pyBase->m_kbObject ;
	bool	 	rc	 ;

	API_CALL(fb,caller) ;
	rc	= fb->doAction (action) ;
	API_DONE;

	return	PyLong_FromLong (rc) ;
}

#define	BLOCK_DOACTION(method,action)						\
										\
	static	PyObject*PyKBFormBlock_##method					\
		(	PyObject	*,					\
			PyObject	*args					\
		)								\
	{									\
		return	PyKBFormBlock_action					\
				(	"KBFormBlock." #method,			\
					args,					\
					action					\
				)	;					\
	}

BLOCK_DOACTION	(firstRecord,		KB::First		)
BLOCK_DOACTION	(previousRecord,	KB::Previous		)
BLOCK_DOACTION	(nextRecord,		KB::Next		)
BLOCK_DOACTION	(lastRecord,		KB::Last		)
BLOCK_DOACTION	(addRecord,		KB::Add			)
BLOCK_DOACTION	(saveRecord,		KB::Save		)
BLOCK_DOACTION	(deleteRecord,		KB::Delete		)
BLOCK_DOACTION	(startQuery,		KB::Query		)
BLOCK_DOACTION	(executeQuery,		KB::Execute		)
BLOCK_DOACTION	(cancelQuery,		KB::Cancel		)
BLOCK_DOACTION	(reload,		KB::Reload		)
BLOCK_DOACTION	(saveAsNewRecord,	KB::SaveAsNew		)
BLOCK_DOACTION	(saveUnchecked,		KB::SaveUnchecked	)

/*  PyKBFormBlock_isInQuery						*/
/*		: Check if block is in query mode			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: In query				*/

static	PyObject*PyKBFormBlock_isInQuery
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBFormBlock.isInQuery",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	KBFormBlock	*fb	= (KBFormBlock *)pyBase->m_kbObject ;
	bool		inQ ;

	API_CALL(fb,"KBFormBlock.isInQuery") ;
	inQ	= fb->isInQuery() ;
	API_DONE;

	return	PyInt_FromLong ((long)inQ) ;
}

/*  PyKBFormBlock_invalidControls					*/
/*		: Check if all controls in block are valid		*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: In query				*/

static	PyObject*PyKBFormBlock_invalidControls
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 	;
	int	 required	;
	bool	 recurse	= false ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBFormBlock.invalidControls",
					PyKBBase::m_object,
					args,
					"Oi|ii",
					&row,
					&required,
					&recurse
				) ;
	if (pyBase == 0) return 0 ;

	KBFormBlock	 *fb	= (KBFormBlock *)pyBase->m_kbObject ;
	QPtrList<KBItem> items	;
	bool		 anySet	;
	
	API_CALL(fb,"KBFormBlock.invalidControls") ;
	anySet	= fb->invalidControls (row, items, recurse) ;
	API_DONE;

	DPRINTF
	((	"PyKBFormBlock_invalidControls: anySet=%d count=%d\n",
		anySet,
		items.count()
	))	;

	/* If no control set and invalid controls are not required on	*/
	/* empty rows, then clear the list of any items that are there	*/
	/* so that the caller sees an empty list on return.		*/
	if (!anySet && !required)
		items.clear () ;

	PyObject *iList	= PyList_New(items.count()) ;
	if (iList == 0) return 0 ;

	for (uint idx = 0 ; idx < items.count() ; idx += 1)
		PyList_SET_ITEM (iList, idx, PyKBBase::makePythonInstance (items.at(idx))) ;

	return	  iList  ;
}

/*  PyKBFormBlock_changedControls					*/
/*		: Get list of controls that have changed		*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: In query				*/

static	PyObject*PyKBFormBlock_changedControls
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 	;
	bool	 recurse	= false	;

	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBFormBlock.changedControls",
					PyKBBase::m_object,
					args,
					"Oi|i",
					&row,
					&recurse
				) ;
	if (pyBase == 0) return 0 ;

	KBFormBlock	 *fb	= (KBFormBlock *)pyBase->m_kbObject ;
	QPtrList<KBItem> items	;
	
	API_CALL(fb,"KBFormBlock.changedControls") ;
	fb->changedControls (row, items, recurse) ;
	API_DONE;

	PyObject *iList	= PyList_New(items.count()) ;
	if (iList == 0) return 0;

	for (uint idx = 0 ; idx < items.count() ; idx += 1)
		PyList_SET_ITEM (iList, idx, PyKBBase::makePythonInstance (items.at(idx))) ;

	return	  iList  ;
}

/*  ------------------------------------------------------------------	*/

static	PyMethodDef PyKBFormBlock_methods[] =
{
	PY_METHOD	(gotoQueryRow,		PyKBFormBlock	),
	PY_METHOD	(gotoQueryRowByKey,	PyKBFormBlock	),
	PY_METHOD	(doAction,		PyKBFormBlock	),
	PY_METHOD	(isInQuery,		PyKBFormBlock	),
	PY_METHOD	(firstRecord,		PyKBFormBlock	),
	PY_METHOD	(previousRecord,	PyKBFormBlock	),
	PY_METHOD	(nextRecord,		PyKBFormBlock	),
	PY_METHOD	(lastRecord,		PyKBFormBlock	),
	PY_METHOD	(addRecord,		PyKBFormBlock	),
	PY_METHOD	(saveRecord,		PyKBFormBlock	),
	PY_METHOD	(deleteRecord,		PyKBFormBlock	),
	PY_METHOD	(startQuery,		PyKBFormBlock	),
	PY_METHOD	(executeQuery,		PyKBFormBlock	),
	PY_METHOD	(cancelQuery,		PyKBFormBlock	),
	PY_METHOD	(reload,		PyKBFormBlock	),
	PY_METHOD	(saveAsNewRecord,	PyKBFormBlock	),
	PY_METHOD	(saveUnchecked,		PyKBFormBlock	),
	PY_METHOD	(invalidControls,	PyKBFormBlock	),
	PY_METHOD	(changedControls,	PyKBFormBlock	),
	PY_METHOD_END
}	;

static	const char *PyKBFormBlock_names  [] =
{
	"KBFormBlock",
	"KBFormSubBlock",
	0
}	;

void	initPyKBFormBlock
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBFormBlock",
		module,
		PyKBFormBlock_methods,
		"KBBlock",
		PyKBFormBlock_names
	)	;
}
