/***************************************************************************
    file	         : kb_pyframer.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_framer.h"
#include	"kb_config.h"

#include	"kb_pybase.h"
#include	"kb_pyscript.h"
#include	"kb_pyvalue.h"

/*  PyKBFramer_getNumRows						*/
/*		: Ger number of rows					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBFramer_getNumRows
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBFramer.getNumRows",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	int	 nRows	 ;
	KBFramer *framer = (KBFramer *)pyBase->m_kbObject ;

	API_CALL(framer,"KBFramer.getNumRows") ;
	nRows	= framer->getNumRows() ;
	API_DONE ;

	return	PyInt_FromLong ((long)nRows) ;
}

/*  PyKBFramer_getQueryRow						*/
/*		: Get current row					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBFramer_getQueryRow
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBFramer.getQueryRow",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	int	 qRow	 ;
	KBFramer *framer = (KBFramer *)pyBase->m_kbObject ;

	API_CALL(framer,"KBFramer.getQueryRow") ;
	qRow	= framer->getCurQRow() ;
	API_DONE ;

	return	PyInt_FromLong ((long)qRow) ;
}

/*  PyKBFramer_getRowValue						*/
/*		: Get value from for for named column			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBFramer_getRowValue
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *name	 ;
	int	 qrow	 ;
	bool	 error	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBFramer.getRowValue",
					PyKBBase::m_object,
					args,
					"OOi",
					&name,
					&qrow
				) ;
	if (pyBase == 0) return 0 ;

	QString qName = kb_pyStringToQString(name, error);
	if (error) return 0;
	
	// no check, if the current row or the column exists.
	// we simply return Py_NONE. which is also returned for a 
	// null value of an existing row/column. Shouldn't we
	// raise an error for non-existing rows/columns?
	KBFramer *framer = (KBFramer *)pyBase->m_kbObject ;
	KBValue	 value	 ;

	API_CALL(framer,"KBFramer.getRowValue")
	value	= framer->getRowValue(qName, qrow) ;
	API_DONE;
 
	return PyKBBase::fromKBValue (value) ;
}

/*  PyKBFramer_setRowValue						*/
/*		: Set value for named column				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBFramer_setRowValue
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *name	 ;
	int	 qrow	 ;
	PyObject *value	 ;
	QString	 qName	 ;
	KBValue	 kValue	 ;
	bool	 error	 ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBFramer.setRowValue",
					PyKBBase::m_object,
					args,
					"OOiO",
					&name,
					&qrow,
					&value
				) ;
	if (pyBase == 0) return 0 ;

	qName = kb_pyStringToQString(name, error);
	if (error) return 0;
	
	kValue = PyKBBase::fromPyObject(value, error);
	if (error) return 0;
	
	KBFramer *framer = (KBFramer *)pyBase->m_kbObject ;

	API_CALL(framer,"KBFramer.setRowValue") ;
	// xxx shouldn't we check, if row and value exist?
	framer->setRowValue (qName, qrow, kValue) ;
	API_DONE ;

	return	PY_NONE ;
}


/*  PyKBFramer_invalidControls						*/
/*		: Check if all controls in block are valid		*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: In query				*/

static	PyObject*PyKBFramer_invalidControls
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 	;
	int	 recurse = 1	;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBFramer.invalidControls",
					PyKBBase::m_object,
					args,
					"Oi|i",
					&row,
					&recurse
				) ;
	if (pyBase == 0) return 0 ;

	QPtrList<KBItem> items	  ;
	KBFramer 	 *framer  = (KBFramer *)pyBase->m_kbObject ;
	
	API_CALL(framer,"KBFramer.invalidControls") ;
	framer->invalidControls (row, items, recurse) ;
	API_DONE ;

	PyObject *iList	= PyList_New(items.count()) ;
	for (uint idx = 0 ; idx < items.count() ; idx += 1)
		PyList_SET_ITEM (iList, idx, PyKBBase::makePythonInstance (items.at(idx))) ;

	return	  iList  ;
}

/*  ------------------------------------------------------------------	*/

static	PyMethodDef PyKBFramer_methods[] =
{
	PY_METHOD	(getNumRows,		PyKBFramer	),
	PY_METHOD	(getQueryRow,		PyKBFramer	),
	PY_METHOD	(getRowValue,		PyKBFramer	),
	PY_METHOD	(setRowValue,		PyKBFramer	),
	PY_METHOD	(invalidControls,	PyKBFramer	),
	PY_METHOD_END
}	;

static	const char *PyKBFramer_names  [] =
{
	"KBFramer",
	"KBHeader",
	"KBFooter",
	0
}	;

void	initPyKBFramer
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBFramer",
		module,
		PyKBFramer_methods,
		"KBObject",
		PyKBFramer_names
	)	;
}
