/***************************************************************************
    file	         : kb_pygrid.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
			   (C) 2001,2002,2003,2004 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_grid.h"

#include	"kb_pybase.h"
#include	"kb_pyscript.h"


/*  PyKBGrid_setOrder							*/
/*		: Get item ordering					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button text				*/

static	PyObject*PyKBGrid_setOrder
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *atuple ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBGrid.setOrder",
					PyKBBase::m_object,
					args,
					"OO",
					&atuple
				) ;
	if (pyBase == 0) return 0 ;

	if (!PySequence_Check(atuple))
	{
		PyErr_SetString (PyExc_TypeError, "argument for setOrder must be a list or tuple") ;
	}
	
	QPtrList<KBItem>	ilist	;

	for (int idx = 0 ; idx < PySequence_Size (atuple) ; idx += 1)
	{
		cchar	 *error	;
		PyObject *wItem = PySequence_GetItem (atuple, idx) ;
		Py_DECREF(wItem);
		PyKBBase *bItem = PyKBBase::getPyBaseFromPyInst (wItem, PyKBBase::m_object, error) ;

		if (bItem == 0)
		{
			KBError::EError
			(	"PyKBGrid::setOrder",
				error,
				__ERRLOCN
			)	;
			return PyInt_FromLong ((long)0) ;
		}

		ilist.append ((KBItem *)bItem->m_kbObject) ;
	}

	KBGrid *grid = (KBGrid *)pyBase->m_kbObject ;

	API_CALL(grid,"KBGrid.setOrder") ;
	grid->setOrder   (ilist) ;
	API_DONE;

	return PyInt_FromLong(1) ;
}

/*  PyKBGrid_getItems							*/
/*		: Set button text					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBGrid_getItems
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBGrid.getItems",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	QPtrList<KBItem>	ilist	;
	PyObject	*pyinst	;
	PyObject	*pyitem	;

	((KBGrid *)pyBase->m_kbObject)->getItems (ilist) ;

	if ((pyinst = PyTuple_New (ilist.count())) == 0)
		return	0	;

	for (uint idx = 0 ; idx < ilist.count() ; idx += 1)
	{
		if ((pyitem = PyKBBase::makePythonInstance (ilist.at(idx))) == 0)
		{
			Py_DECREF (pyinst) ;
			return	0 ;
		}

		PyTuple_SET_ITEM  (pyinst, idx, pyitem) ;
	}

	return	  pyinst  ;
}

/*  PyKBGrid_setColumnWidth						*/
/*		: Set column width in grid				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button on				*/

static	PyObject*PyKBGrid_setColumnWidth
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *pyItem ;
	int	 width	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBGrid.setColumnWidth",
					PyKBBase::m_object,
					args,
					"OOi",
					&pyItem,
					&width
				) ;
	if (pyBase == 0) return 0 ;

	cchar	 *error	 ;
	PyKBBase *kbItem = PyKBBase::getPyBaseFromPyInst (pyItem, PyKBBase::m_object, error) ;
	if (kbItem == 0)
	{
		KBError::EError
		(	"PyKBGrid::setColumnWidth",
			error,
			__ERRLOCN
		)	;
		return PY_NONE	;
	}

	KBGrid *grid = (KBGrid *)pyBase->m_kbObject ;

	API_CALL(grid,"KBGrid.setColumnWidth") ;
	grid->setColumnWidth ((KBItem *)kbItem->m_kbObject, width) ;
	API_DONE;

	return	PY_NONE	;
}

/*  ------------------------------------------------------------------	*/

static	PyMethodDef pyKBGrid_methods[] =
{
	PY_METHOD	(setOrder,		PyKBGrid	),
	PY_METHOD	(getItems,		PyKBGrid	),
	PY_METHOD	(setColumnWidth,	PyKBGrid	),
	PY_METHOD_END
}	;
static	const char *pyKBGrid_names  [] =
{
	"KBGrid",
	0
}	;

void	initPyKBGrid
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBGrid",
		module,
		pyKBGrid_methods,
		"KBObject",
		pyKBGrid_names
	)	;
}
