/***************************************************************************
    file	         : kb_pyitem.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_form.h"
#include	"kb_config.h"

#include	"kb_pybase.h"
#include	"kb_pyvalue.h"

/*  PyKBItem_isReadOnly							*/
/*		: Test if item control is read-only			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBItem_isReadOnly
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.isReadOnly",
					PyKBBase::m_object,
					args,
					"Oi",
					&row
				) ;
	if (pyBase == 0) return 0 ;

	bool	rdonly	;
	KBItem	*item	= (KBItem *)pyBase->m_kbObject ;

	API_CALL(item,"KBItem.isReadOnly") ;
	rdonly	= item->isCtrlReadOnly(row) ;
	API_DONE ;

	return	PyInt_FromLong ((long)rdonly) ;
}

/*  PyKBItem_setReadOnly						*/
/*		: Set item readOnly/writeable				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBItem_setReadOnly
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	int	 rdonly  ;
	cchar	 *fg	 = 0 ;
	cchar	 *bg	 = 0 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.setReadOnly",
					PyKBBase::m_object,
					args,
					"Oii|ss",
					&row,
					&rdonly,
					&fg,
					&bg
				) ;
	if (pyBase == 0) return 0 ;

	KBItem	*item	= (KBItem *)pyBase->m_kbObject ;
	QColor	fgColor	;
	QColor	bgColor	;

	if ((fg != 0) && (fg[0] != 0)) fgColor.setNamedColor (fg) ;
	if ((bg != 0) && (bg[0] != 0)) fgColor.setNamedColor (bg) ;

	API_CALL(item,"KBItem.setReadOnly") ;
	item->setCtrlReadOnly (row, rdonly, fgColor, bgColor) ;
	API_DONE ;

	return	PY_NONE ;
}

/*  PyKBItem_isEnabled							*/
/*		: Test if item control is enabled			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBItem_isEnabled
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.isEnabled",
					PyKBBase::m_object,
					args,
					"Oi",
					&row
				) ;
	if (pyBase == 0) return 0 ;

	bool	enabled	;
	KBItem	*item	= (KBItem *)pyBase->m_kbObject ;

	API_CALL(item,"KBItem.isEnabled") ;
	enabled	= item->isEnabled(row) ;
	API_DONE ;

	return	PyInt_FromLong ((long)enabled) ;
}

/*  PyKBItem_setEnabled							*/
/*		: Set item (dis)enabled					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBItem_setEnabled
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	int	 enabled ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.setEnabled",
					PyKBBase::m_object,
					args,
					"Oii",
					&row,
					&enabled
				) ;
	if (pyBase == 0) return 0 ;

	KBItem	*item	= (KBItem *)pyBase->m_kbObject ;

	API_CALL(item,"KBItem.setEnabled") ;
	item->setEnabled (row, enabled) ;
	API_DONE ;

	return	PY_NONE ;
}

/*  PyKBItem_isVisible							*/
/*		: Test if item control is visibl			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBItem_isVisible
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.isVisible",
					PyKBBase::m_object,
					args,
					"Oi",
					&row
				) ;
	if (pyBase == 0) return 0 ;

	bool	visible	;
	KBItem	*item	= (KBItem *)pyBase->m_kbObject ;

	API_CALL(item,"KBItem.isVisible") ;
	visible	= item->isVisible(row) ;
	API_DONE ;

	return	PyInt_FromLong ((long)visible) ;
}

/*  PyKBItem_setVisible							*/
/*		: Set item (in)visible					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBItem_setVisible
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	int	 visible ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.setVisible",
					PyKBBase::m_object,
					args,
					"Oii",
					&row,
					&visible
				) ;
	if (pyBase == 0) return 0 ;

	KBItem	*item	= (KBItem *)pyBase->m_kbObject ;

	API_CALL(item,"KBItem.setVisible") ;
	item->setVisible (row, visible) ;
	API_DONE ;

	return	PY_NONE ;
}

/*  PyKBItem_setValue							*/
/*		: Set item value					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBItem_setValue
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 qRow	 ;
	PyObject *pyValue;
	bool	 error	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.setValue",
					PyKBBase::m_object,
					args,
					"OiO",
					&qRow,
					&pyValue
				) ;
	if (pyBase == 0) return 0 ;

	KBItem	*kbItem	= (KBItem *)pyBase->m_kbObject ;
	KBValue value	 = PyKBBase::fromPyObject (pyValue, error, kbItem->getFieldType()) ;

	API_CALL(kbItem,"KBItem.setValue") ;
	kbItem->setValue(PyKBBase::getCurQRow (kbItem, qRow), value) ;
	API_DONE;

	return	PY_NONE	;
}

/*  PyKBItem_getValue							*/
/*		: Get item value					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBItem_getValue
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 qRow	 ;
	int	 convertExact = 0;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.getValue",
					PyKBBase::m_object,
					args,
					"Oi|i",
					&qRow,
					&convertExact
				) ;
	if (pyBase == 0) return 0 ;

	KBItem	*kbItem	= (KBItem *)pyBase->m_kbObject ;
	KBValue	value	;

	API_CALL(kbItem,"KBItem.getValue") ;
	value	= kbItem->getValue(PyKBBase::getCurQRow (kbItem, qRow)) ;
	API_DONE;

	return	PyKBBase::fromKBValue(value, convertExact);
}

/*  PyKBItem_getRowValue						*/
/*		: Get item value from query				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBItem_getRowValue
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 qRow	 ;
	int	 convertExact = 0 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.getRowValue",
					PyKBBase::m_object,
					args,
					"Oi|i",
					&qRow,
					&convertExact
				) ;
	if (pyBase == 0) return 0 ;

	KBItem	*kbItem	 = (KBItem *)pyBase->m_kbObject ;
	KBValue	value	 ;

	API_CALL (kbItem,"KBItem.getRowValue")	;
	value	= kbItem->getRowValue(PyKBBase::getCurQRow (kbItem, qRow)) ;
	API_DONE ;

	return  PyKBBase::fromKBValue(value, convertExact) ;
}

/*  PyKBItem_isValid							*/
/*		: Test if item value is valid				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBItem_isValid
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.isValid",
					PyKBBase::m_object,
					args,
					"Oi",
					&row
				) ;
	if (pyBase == 0) return 0 ;

	bool	valid	;
	KBItem	*item	= (KBItem *)pyBase->m_kbObject ;

	API_CALL(item,"KBItem.isVisible") ;
	valid   = item->isValid(row, false) ;
	API_DONE ;

	return	PyInt_FromLong ((long)valid) ;
}

/*  PyKBItem_changed							*/
/*		: Test if item value has changed			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBItem_changed
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.changed",
					PyKBBase::m_object,
					args,
					"Oi",
					&row
				) ;
	if (pyBase == 0) return 0 ;

	bool	changed	;
	KBItem	*item	= (KBItem *)pyBase->m_kbObject ;

	API_CALL(item,"KBItem.changed") ;
	changed	 = item->changed  (row) ;
	API_DONE ;

	return	PyInt_FromLong ((long)changed) ;
}

/*  PyKBItem_errorText							*/
/*		: Get error text for the control				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBItem_errorText
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.errorText",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	QString	text	;
	KBItem	*item	= (KBItem *)pyBase->m_kbObject ;

	API_CALL(item,"KBItem.errorText") ;
	text	= item->errorText() ;
	API_DONE ;

	return	PyString_FromString((const char *)text) ;
}

/*  PyKBItem_setFGColor							*/
/*		: Set item foreground colour				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBItem_setFGColor
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	PyObject *colour ;
	QString	 qColour ;
	bool	 error	 ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.setFGColor",
					PyKBBase::m_object,
					args,
					"OiO",
					&row,
					&colour
				) ;
	if (pyBase == 0) return 0 ;

	qColour = kb_pyStringToQString(colour, error);
	if (error) return 0 ;

	KBItem *item = (KBItem *)pyBase->m_kbObject ;

	API_CALL(item,"KBItem.setFGColor") ;
	item->setCtrlFGColor (row, qColour) ;
	API_DONE ;

	return	PY_NONE	;
}

/*  PyKBItem_setBGColor							*/
/*		: Get item background colour				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBItem_setBGColor
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	PyObject *colour ;
	QString	 qColour ;
	bool	 error	 ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.setBGColor",
					PyKBBase::m_object,
					args,
					"OiO",
					&row,
					&colour
				) ;
	if (pyBase == 0) return 0 ;

	qColour = kb_pyStringToQString(colour, error) ;
	if (error) return 0 ;
	
	KBItem *item = (KBItem *)pyBase->m_kbObject ;

	API_CALL(item,"KBItem.setBGColor") ;
	item->setCtrlBGColor (row, qColour) ;
	API_DONE ;

	return	PY_NONE	;
}

/*  PyKBItem_getsetFont							*/
/*		: Get item font						*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBItem_setFont
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	PyObject *font   ;
	QString	 qFont	 ;
	bool	 error	 ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBItem.setFont",
					PyKBBase::m_object,
					args,
					"OiO",
					&row,
					&font
				) ;
	if (pyBase == 0) return 0 ;

	qFont = kb_pyStringToQString(font, error);
	if (error) return 0 ;
	
	KBItem *item = (KBItem *)pyBase->m_kbObject ;

	API_CALL(item,"KBItem.setFont") ;
	item->setCtrlFont (row, qFont) ;
	API_DONE ;

	return	PY_NONE	;
}

/*  ------------------------------------------------------------------	*/

static	PyMethodDef PyKBItem_methods[] =
{
	PY_METHOD	(isReadOnly,	PyKBItem	),
	PY_METHOD	(setReadOnly,	PyKBItem	),
	PY_METHOD	(isEnabled,	PyKBItem	),
	PY_METHOD	(setEnabled,	PyKBItem	),
	PY_METHOD	(isVisible,	PyKBItem	),
	PY_METHOD	(setVisible,	PyKBItem	),
	PY_METHOD	(setValue,	PyKBItem	),
	PY_METHOD	(getValue,	PyKBItem	),
	PY_METHOD	(getRowValue,	PyKBItem	),
	PY_METHOD	(isValid,	PyKBItem	),
	PY_METHOD	(changed,	PyKBItem	),
	PY_METHOD	(errorText,	PyKBItem	),
	PY_METHOD	(setFGColor,	PyKBItem	),
	PY_METHOD	(setBGColor,	PyKBItem	),
	PY_METHOD	(setFont,	PyKBItem	),
	PY_METHOD_END
}	;

static	const char *PyKBItem_names  [] =
{
	"KBItem",
	0
}	;

void	initPyKBItem
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBItem",
		module,
		PyKBItem_methods,
		"KBObject",
		PyKBItem_names
	)	;
}
