/***************************************************************************
    file	         : kb_pylinktree.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_linktree.h"
#include	"kb_block.h"

#include	"kb_pybase.h"
#include	"kb_pyvalue.h"


/*  ------------------------------------------------------------------	*/


/*  PyKBLinkTree_setUserFilter						*/
/*		: Set filter on link or tree				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBLinkTree_setUserFilter
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 qRow	 ;
	PyObject *filter ;
	QString	 qFilter ;
	bool	 error   ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBLinkTree.setUserFilter",
					PyKBBase::m_object,
					args,
					"OiO",
					&qRow,
					&filter
				) ;
	if (pyBase == 0) return 0 ;
	
	qFilter = kb_pyStringToQString(filter, error);
	if (error) return 0;

	KBLinkTree *kbLT = (KBLinkTree *)pyBase->m_kbObject ;

	API_CALL(kbLT,"KBLinkTree.setUserFilter") ;
	kbLT->setUserFilter (PyKBBase::getCurQRow (kbLT, qRow), qFilter) ;
	API_DONE;

	return	PY_NONE ;
}


/*  PyKBLinkTree_setUserSorting						*/
/*		: Set sorting on link or tree				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBLinkTree_setUserSorting
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 qRow	 ;
	PyObject *sorting;
	QString	 qSorting;
	bool	 error   ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBLinkTree.setUserSorting",
					PyKBBase::m_object,
					args,
					"OiO",
					&qRow,
					&sorting
				) ;
	if (pyBase == 0) return 0 ;
	
	qSorting = kb_pyStringToQString(sorting, error);
	if (error) return 0;

	KBLinkTree *kbLT = (KBLinkTree *)pyBase->m_kbObject ;

	API_CALL(kbLT,"KBLinkTree.setUserSorting") ;
	kbLT->setUserSorting (PyKBBase::getCurQRow (kbLT, qRow), qSorting) ;
	API_DONE;

	return	PY_NONE ;
}

/*  PyKBLinkTree_reload							*/
/*		: Reload values for row					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBLinkTree_reload
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 qRow	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBLinkTree.reload",
					PyKBBase::m_object,
					args,
					"Oi",
					&qRow
				) ;
	if (pyBase == 0) return 0 ;

	KBLinkTree *kbLT = (KBLinkTree *)pyBase->m_kbObject ;

	API_CALL(kbLT,"KBLinkTree.reload") ;
	kbLT->reload (PyKBBase::getCurQRow (kbLT, qRow)) ;
	API_DONE;

	return	PY_NONE ;
}

/*  PyKBLinkTree_getExtra						*/
/*		: Get extra value					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBLinkTree_getExtra
	(	PyObject	*,
		PyObject	*args,
		cchar		*ident,
		bool		which
	)
{
	int	 qRow	 ;
	int	 extra	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	ident,
					PyKBBase::m_object,
					args,
					"Oii",
					&qRow,
					&extra
				) ;
	if (pyBase == 0) return 0 ;

	KBLinkTree *kbLT = (KBLinkTree *)pyBase->m_kbObject ;
	QString	   value ;

	API_CALL(kbLT,ident) ;
	value	= kbLT->getExtra(PyKBBase::getCurQRow (kbLT, qRow), which, extra).getRawText() ;
	API_DONE;

	if (value.isNull()) return PY_NONE ;
	return	kb_qStringToPyString(value) ;
}

/*  PyKBLinkTree_getExtra						*/
/*		: Get extra value					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBLinkTree_getExtra
	(	PyObject	*self,
		PyObject	*args
	)
{
	return	PyKBLinkTree_getExtra (self, args, "KBLinkTree.getExtra", false) ;
}

/*  PyKBLinkTree_getDisplay						*/
/*		: Get display value					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBLinkTree_getDisplay
	(	PyObject	*self,
		PyObject	*args
	)
{
	return	PyKBLinkTree_getExtra (self, args, "KBLinkTree.getDisplay", true) ;
}



/*  PyKBLinkTree_getRowExtra						*/
/*		: Get row extra value					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBLinkTree_getRowExtra
	(	PyObject	*,
		PyObject	*args,
		cchar		*ident,
		bool		which
	)
{
	int	 qRow	 ;
	int	 extra	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	ident,
					PyKBBase::m_object,
					args,
					"Oii",
					&qRow,
					&extra
				) ;
	if (pyBase == 0) return 0 ;

	KBLinkTree *kbLT = (KBLinkTree *)pyBase->m_kbObject ;
	QString	   value ;

	API_CALL(kbLT,"ident") ;
	value	= kbLT->getRowExtra(PyKBBase::getCurQRow (kbLT, qRow), which, extra).getRawText() ;
	API_DONE;

	if (value.isNull()) return PY_NONE ;
	return kb_qStringToPyString(value) ;
}

/*  PyKBLinkTree_getRowExtra						*/
/*		: Get extra value					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBLinkTree_getRowExtra
	(	PyObject	*self,
		PyObject	*args
	)
{
	return	PyKBLinkTree_getRowExtra (self, args, "KBLinkTree.getRowExtra", false) ;
}

/*  PyKBLinkTree_getRowDisplay						*/
/*		: Get display value					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBLinkTree_getRowDisplay
	(	PyObject	*self,
		PyObject	*args
	)
{
	return	PyKBLinkTree_getRowExtra (self, args, "KBLinkTree.getRowDisplay", true) ;
}

/*  PyKBLinkTree_getNumValues						*/
/*		: Get number of values in link/tree			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBLinkTree_getNumValues
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 qrow	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBLinkTree.getNumValues",
					PyKBBase::m_object,
					args,
					"Oi",
					&qrow
				) ;
	if (pyBase == 0) return 0 ;

	KBLinkTree *kbLT = (KBLinkTree *)pyBase->m_kbObject ;

	API_CALL(kbLT,"KBLinkTree.getNumValues") ;
	uint	nVals	= (kbLT->getNumValues (qrow)) ;
	bool	noblank = kbLT->getAttrVal("noblank") == "Yes" ;
	uint	offset  = noblank ? 0 : 1 ;
	API_DONE  ;

	return	PyInt_FromLong ((long)(nVals - offset)) ;
}

/*  PyKBLinkTree_getDisplayList						*/
/*		: Get display values					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button text				*/

static	PyObject*PyKBLinkTree_getDisplayList
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 qRow	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBLinkTree.getDisplayList",
					PyKBBase::m_object,
					args,
					"Oi",
					&qRow
				) ;
	if (pyBase == 0) return 0 ;


	KBLinkTree *kbLT = (KBLinkTree *)pyBase->m_kbObject ;

	API_CALL(kbLT,"KBLinkTree.getValues") ;
	const QStringList &values = kbLT->getDisplayList (qRow) ;
	bool		  noblank = kbLT->getAttrVal("noblank") == "Yes" ;
	uint		  offset  = noblank ? 0 : 1 ;
	API_DONE ;

	extern	PyObject *qtStringListToPyList (const QStringList &, int) ;
	return	qtStringListToPyList (values, offset) ;
}




static	PyMethodDef pyKBLinkTree_methods[] =
{
	PY_METHOD	(setUserFilter,		PyKBLinkTree	),
	PY_METHOD	(setUserSorting,	PyKBLinkTree	),
	PY_METHOD	(reload,		PyKBLinkTree	),
	PY_METHOD	(getExtra,		PyKBLinkTree	),
	PY_METHOD	(getDisplay,		PyKBLinkTree	),
	PY_METHOD	(getRowExtra,		PyKBLinkTree	),
	PY_METHOD	(getRowDisplay,		PyKBLinkTree	),
	PY_METHOD	(getNumValues,		PyKBLinkTree	),
	PY_METHOD	(getDisplayList,	PyKBLinkTree	),
	PY_METHOD_END
}	;

static	const char *pyKBLinkTree_names  [] =
{
	"KBLinkTree",
	0
}	;

void	initPyKBLinkTree
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBLinkTree",
		module,
		pyKBLinkTree_methods,
		"KBItem",
		pyKBLinkTree_names
	)	;
}
