/***************************************************************************
    file	         : kb_pyobject.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_form.h"
#include	"kb_report.h"
#include	"kb_config.h"
#include	"kb_slot.h"
#include	"kb_slotnotifier.h"

#include	"kb_pybase.h"
#include	"kb_pyscript.h"
#include	"kb_pyvalue.h"


#define		__DEBUG_PYOBJECT	(0)
#if		__DEBUG_PYOBJECT
#include	"kb_dprintf.h"
#endif


/*  PyKBObject_getElement						*/
/*		: Get object element type				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Element type				*/

static	PyObject*PyKBObject_getElement
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getElement",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	QString	 element ;
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.getElement") ;
	element	= object->getElement () ;
	API_DONE ;

	if (element.isNull())
		return	PY_NONE	;

	return	kb_qStringToPyString(element) ;
}

/*  PyKBObject_getName							*/
/*		: Get object name					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Name					*/

static	PyObject*PyKBObject_getName
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getName",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	QString	 name	 ;
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.getName") ;
	name	= object->getName () ;
	API_DONE ;

	if (name.isNull())
		return	PY_NONE	;

	return	kb_qStringToPyString (name) ;
}

/*  PyKBObject_getType							*/
/*		: Get object type					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_getType
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getType",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	QString	 type	 ;
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.getType") ;
	type	= object->getElement() ;
	API_DONE ;

	if (type.isNull())
		return	PY_NONE	;

	return	kb_qStringToPyString (type) ;
}

/*  PyKBObject_getAttr							*/
/*		: Get object attribute					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_getAttr
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *attr	 ;
	QString	 qAttr	 ;
	bool	 error	 ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getAttr",
					PyKBBase::m_object,
					args,
					"OO",
					&attr
				) ;
	if (pyBase == 0) return 0 ;
	
	qAttr = kb_pyStringToQString(attr, error);
	if (error) return 0;

	KBObject *object = (KBObject *)pyBase->m_kbObject ;
	QString	 value	 ;

	API_CALL(object,"KBObject.getAttr") ;
	value	= object->getAttrVal (qAttr) ;
	API_DONE ;

	if (value.isNull())
		return	PY_NONE	;

	return	kb_qStringToPyString (value) ;
}

/*  PyKBObject_setAttr							*/
/*		: Set object attribute					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_setAttr
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *attr	 ;
	PyObject *value	 ;
	QString	 qAttr	 ;
	QString	 qValue  ;
	bool	 error	 ;
	int	 upd	 = 1 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.setAttr",
					PyKBBase::m_object,
					args,
					"OOO|i",
					&attr,
					&value,
					&upd
				) ;
	if (pyBase == 0) return 0 ;
	
	qAttr  = kb_pyStringToQString (attr,  error) ;
	if (error) return 0;
	
	qValue = kb_pyStringToQString (value, error) ;
	if (error) return 0;

	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"") ;
	object->setAttrVal (qAttr, qValue, true, upd) ;
	API_DONE ;

	return	PY_NONE	;
}

/*  PyKBObject_getConfig						*/
/*		: Get object configuration				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_getConfig
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *config ;
	QString  qConfig ;
	bool	 error	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getConfig",
					PyKBBase::m_object,
					args,
					"OO",
					&config
				) ;
	if (pyBase == 0) return 0 ;

	qConfig = kb_pyStringToQString(config, error);
	if (error) return 0;
	
	KBObject *object = (KBObject *)pyBase->m_kbObject ;
	QString	 value	 ;

	API_CALL(object,"KBObject.getConfig") ;

		TITER
		(	Config,
			((PyKBNode *)pyBase)->m_node->getChildren(),
			c,
			if (c->attrib() == qConfig)
			{	value = c->value() ;
				break	 ;
			}
		)

	API_DONE ;

	if (value.isNull())
		return	PY_NONE	;

	return	kb_qStringToPyString (value) ;
}

/*  PyKBObject_isEnabled						*/
/*		: Test if object is enabled				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button on				*/

static	PyObject*PyKBObject_isEnabled
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.isEnabled",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	bool	 enabled ;
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.isEnabled") ;
	enabled	= object->isEnabled() ;
	API_DONE ;

	return	PyInt_FromLong ((long)enabled) ;
}

/*  PyKBObject_setEnabled						*/
/*		: Set button (dis)enabled				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_setEnabled
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 enabled ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.setEnabled",
					PyKBBase::m_object,
					args,
					"Oi",
					&enabled
				) ;
	if (pyBase == 0) return 0 ;

	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.setEnabled") ;
	object->setEnabled (enabled) ;
	API_DONE ;

	return	PY_NONE ;
}

/*  PyKBObject_isVisible						*/
/*		: Test if object is visible				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Button on				*/

static	PyObject*PyKBObject_isVisible
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.isVisible",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	bool	 visible ;
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.isVisible") ;
	visible	= object->isVisible() ;
	API_DONE ;

	return	PyInt_FromLong ((long)visible) ;
}

/*  PyKBObject_setVisible						*/
/*		: Set button (in)visible				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_setVisible
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 visible ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.setVisible",
					PyKBBase::m_object,
					args,
					"Oi",
					&visible
				) ;
	if (pyBase == 0) return 0 ;

	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.setVisible") ;
	object->setVisible (visible) ;
	API_DONE ;

	return	PY_NONE ;
}

/*  PyKBObject_setFGColor						*/
/*		: Get object foreground colour				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_setFGColor
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *colour ;
	QString	 qColour ;
	bool	 error	 ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.setFGColor",
					PyKBBase::m_object,
					args,
					"OO",
					&colour
				) ;
	if (pyBase == 0) return 0 ;

	qColour = kb_pyStringToQString(colour, error);
	if (error) return 0 ;

	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.setFGColor") ;
	object->setCtrlFGColor (qColour) ;
	API_DONE ;

	return	PY_NONE	;
}

/*  PyKBObject_setBGColor						*/
/*		: Get object background colour				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_setBGColor
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *colour ;
	QString	 qColour ;
	bool	 error	 ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.setBGColor",
					PyKBBase::m_object,
					args,
					"OO",
					&colour
				) ;
	if (pyBase == 0) return 0 ;

	qColour = kb_pyStringToQString(colour, error) ;
	if (error) return 0 ;
	
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.setBGColor") ;
	object->setCtrlBGColor (qColour) ;
	API_DONE ;

	return	PY_NONE	;
}

/*  PyKBObject_getsetFont						*/
/*		: Get object font					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_setFont
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *font   ;
	QString	 qFont	 ;
	bool	 error	 ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.setFont",
					PyKBBase::m_object,
					args,
					"OO",
					&font
				) ;
	if (pyBase == 0) return 0 ;

	qFont = kb_pyStringToQString(font, error);
	if (error) return 0 ;
	
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.setFont") ;
	object->setCtrlFont (qFont) ;
	API_DONE ;

	return	PY_NONE	;
}

/*  PyKBObject_getQTProperty						*/
/*		: Get QT object property				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_getQTProperty
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *prop	 ;
	QString	 qProp	 ;
	bool	 error	 ;
	
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getQTProperty",
					PyKBBase::m_object,
					args,
					"OO",
					&prop
				) ;
	if (pyBase == 0) return 0 ;

	qProp = kb_pyStringToQString(prop, error);
	if (error) return 0;
	
	QVariant value	 ;
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.getQTProperty") ;
	value	 = object->property (qProp) ;
	API_DONE ;

	/* Pick out special cases. These are most likely not of any use	*/
	/* in a real application, but are useful for test scripts.	*/
	if (value.type() == QVariant::Rect)
	{
		QRect	 rect	= value.toRect() ;
		return	Py_BuildValue ("iiii", rect.x(), rect.y(), rect.height(), rect.width()) ;
	}

	/* Apart from special cases, return as a string where the cast	*/
	/* is possible and Python None otherwise.			*/
	QString	text	= value.toString() ;
	if (text.isNull()) return PY_NONE  ;

	return	kb_qStringToPyString (value.toString()) ;
}

/*  PyKBObject_update							*/
/*		: Update displayed object				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_update
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.update",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.update") ;
	object->setupProperties () ;
	API_DONE ;

	return	PY_NONE	;
}

/*  PyKBObject_width							*/
/*		: Get object width					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_width
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.width",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	KBObject *object = (KBObject *)pyBase->m_kbObject ;
	QRect	 rect	 ;

	API_CALL(object,"KBObject.width") ;
	object->ctrlGeometry(rect) ;
	API_DONE ;

	return	PyLong_FromLong (rect.width()) ;
}

/*  PyKBObject_height							*/
/*		: Get object height					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_height
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.height",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	KBObject *object = (KBObject *)pyBase->m_kbObject ;
	QRect	 rect	 ;

	API_CALL(object,"KBObject.height") ;
	object->ctrlGeometry(rect) ;
	API_DONE ;

	return	PyLong_FromLong (rect.height()) ;
}

/*  PyKBObject_resize							*/
/*		: Resize object						*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_resize
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 width	 ;
	int	 height	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.resize",
					PyKBBase::m_object,
					args,
					"Oii",
					&width,
					&height
				) ;
	if (pyBase == 0) return 0 ;

	KBObject *object = (KBObject *)pyBase->m_kbObject ;
	QRect	 g	 = object->geometry() ;

	API_CALL(object,"KBObject.resize") ;
	object->setGeometry(QRect(g.x(), g.y(), width, height)) ;
	API_DONE ;

	return	PY_NONE	;
}

/*  PyKBObject_getParent						*/
/*		: Get parent object					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_getParent
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getParent",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	KBNode	 *node	 ;
	PyObject *parent ;
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.getParent") ;
	node	= object->getParent() ;
	API_DONE ;

	if (node == 0)
		return	PY_NONE ;

	if ((parent = PyKBBase::makePythonInstance (node)) == 0)
		return	0	;

	return	parent	;
}

/*  PyKBObject_getBlock							*/
/*		: Get containing block					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_getBlock
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getBlock",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	KBBlock	 *block	 ;
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.getBlock") ;
	block	= object->getBlock() ;
	API_DONE ;

	if (block == 0)
		return	PY_NONE ;

	return	PyKBBase::makePythonInstance (block) ;
}

/*  PyKBObject_getForm							*/
/*		: Get containing form					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_getForm
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getForm",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	KBForm	 *form	 ;
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.getForm") ;
	form	= object->getForm() ;
	API_DONE ;

	if (form == 0)
		return	PY_NONE ;

	return	PyKBBase::makePythonInstance (form) ;
}

/*  PyKBObject_getReport						*/
/*		: Get containing report					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_getReport
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getReport",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	KBReport *report ;
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.getReport") ;
	report	= object->getReport() ;
	API_DONE ;

	if (report == 0)
		return	PY_NONE ;

	return	PyKBBase::makePythonInstance (report) ;
}

/*  PyKBObject_getRoot							*/
/*		: Get containing root node				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_getRoot
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getRoot",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	KBNode	 *root	 ;
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.getRoot") ;
	root	= object->getRoot() ;
	API_DONE ;

	if (root == 0)
		return	PY_NONE ;

	return	PyKBBase::makePythonInstance (root) ;
}

/*  PyKBObject_getNamedCtrl						*/
/*		: Get named control					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_getNamedCtrl
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *name	 ;
	QString	 qName	 ;
	bool	 error	 ;
	int	 reqd	 = 1 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getNamedCtrl",
					PyKBBase::m_object,
					args,
					"OO|i",
					&name,
					&reqd
				) ;
	if (pyBase == 0) return 0 ;
	
	qName = kb_pyStringToQString(name, error);
	if (error) return 0 ;

	KBObject *object = (KBObject *)pyBase->m_kbObject ;
	bool	 abort	 = false ;
	KBNode   *ctrl	 ;

	API_CALL(object,"KBObject.getNamedCtrl") ;
	ctrl	= object->getNamedNode (qName, reqd, reqd ? &abort : 0) ;
	API_DONE ;

	if (abort)
		return	0	;

	if (ctrl == 0)
		return	PY_NONE ;

	return	PyKBBase::makePythonInstance (ctrl) ;
}

/*  PyKBObject_getControls						*/
/*		: Get child (data) controls				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_getControls
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getControls",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	QPtrList<KBItem>	items	;
	KBObject 		*object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.getControls") ;

	TITER
	(	Item,
		object->getChildren(),
		i,

//		fprintf
//		(	stderr,
//			"PyKBObject_getControls: %s: isb=%d\n",
//			i->getName().latin1(),
//			i->isBlock()
//		)	;

		if (i->isBlock() == 0) items.append (i) 
	)

	API_DONE ;

	PyObject *iList	= PyList_New(items.count()) ;
	if (iList == 0) return 0;
	
	for (uint idx = 0 ; idx < items.count() ; idx += 1)
	{
		PyObject *item = PyKBBase::makePythonInstance (items.at(idx));
		if (item == 0)
		{
			Py_DECREF(iList);
			return 0;
		}
		PyList_SET_ITEM (iList, idx, item) ;
	}

	return	  iList  ;
}

/*  PyKBObject_getChildren						*/
/*		: Get all children					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_getChildren
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.getChildren",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	QPtrList<KBNode> nodes	 ;
	KBObject 	 *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.getChildren") ;

	LITER
	(	KBNode,
		object->getChildren(),
		n,

		if (n->isObject())
			nodes.append (n) 
	)

	API_DONE ;

	PyObject *iList	= PyList_New(nodes.count()) ;
	if (iList == 0) return 0;
	for (uint idx = 0 ; idx < nodes.count() ; idx += 1)
	{
		PyObject *item = PyKBBase::makePythonInstance (nodes.at(idx));
		if (item == 0)
		{
			Py_DECREF(iList);
			return 0;
		}
		PyList_SET_ITEM (iList, idx, item) ;
	}

	return	  iList  ;
}

/*  PyKBObject_lastError						*/
/*		: Get last error					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_lastError
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.lastError",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	cchar	 *error	 ;
	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.lastError") ;
	error	= PyKBBase::decodeError (object->lastError()) ;
	API_DONE ;

	if (error == 0) return PY_NONE ;
	return	PyString_FromString (error) ;
}

/*  PyKBObject_setTabOrder						*/
/*		: Set object tab ordering				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: None or zero on error			*/

static	PyObject*PyKBObject_setTabOrder
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 order	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.setTabOrder",
					PyKBBase::m_object,
					args,
					"Oi",
					&order
				) ;
	if (pyBase == 0) return 0 ;

	KBObject *object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.setTabOrder") ;
	object->setTabOrder (order) ;
	API_DONE ;

	return	PY_NONE ;
}

/*  PyKBObject_openServer						*/
/*		: Open database connection to server			*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: PyKBDBLink or zero on error		*/

static	PyObject*PyKBObject_openServer
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *name	 = 0 ;
	QString  svrName = QString::null;
	bool	 error;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBForm.openServer",
					PyKBBase::m_object,
					args,
					"O|O",
					&name
				) ;
	if (pyBase == 0) return 0 ;
	
	if (name)
	{
		svrName = kb_pyStringToQString(name, error);
		if (error) return 0;
	}

	KBDBLink 	 *dbLink  = new KBDBLink ;
	KBObject 	 *object  = (KBObject *)pyBase->m_kbObject ;
	const KBLocation &locn	  = object->getDocRoot()->getDataLocation() ;
	bool		 rc	  ;

	if (svrName.isNull() || svrName.isEmpty()) svrName = locn.server() ;

	DPRINTF
	((	"PyKBObject_openServer: dt=[%s] dl=[%s] dn=[%s] svr=[%s]\n",
		(cchar *)locn.type  (),
		(cchar *)locn.server(),
		(cchar *)locn.name  (),
		(cchar *)svrName
	))	;

	API_CALL(object,"KBForm.openServer") ;
	rc	= dbLink->connect (locn, svrName, true) ;
	API_DONE ;

	if (!rc)
	{	delete	dbLink	;
		return	PY_NONE	;
	}

	return	PyKBBase::makePythonInstance
			(	"KBDBLink",
				new PyKBBase (dbLink, PyKBBase::m_dblink)
			)	;
}

/*  PyKBObject___repr__							*/
/*		: Get __repr__						*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Element type				*/

static	PyObject*PyKBObject___repr__
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBObject.__repr__",
					PyKBBase::m_object,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;
	KBObject *object  = (KBObject *)pyBase->m_kbObject ;

	QString	repr	= QString ("[%1.%2 @ 0x%3:0x%4]")
				.arg(object->getElement())
				.arg(object->getName   ())
				.arg((ulong)(void *)object, 0, 16)
				.arg((ulong)(void *)pyBase->m_pyInstance, 0, 16)
				;

	return	kb_qStringToPyString (repr) ;
}

/*  PyKBObject_notify							*/
/*		: Notification method					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Element type				*/

static	PyObject*PyKBObject_notify
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject   	*pyInst			;
	cchar		*event			;

	PyObject	*pyValue[MAX_CALL_ARGS]	;
	memset	(pyValue, 0, sizeof(pyValue))	;

	if (!PyArg_ParseTuple
		(	args,
			"Os|OOOOOO",
			&pyInst,
			&event,
			&pyValue[0],
			&pyValue[1],
			&pyValue[2],
			&pyValue[3],
			&pyValue[4],
			&pyValue[5]
		))
	{
//		fprintf
//		(	stderr,
//			"PyKBObject_notify: parse args failed\n"
//		)	;
		return	0	;
	}

	const char *error	;
	PyKBBase   *pyBase	= PyKBBase::getPyBaseFromPyInst (pyInst, PyKBBase::m_object, error) ;
	if (pyBase == 0)
	{
		fprintf
		(	stderr,
			"PyKBObject_notify: %s\n",
			error
		)	;
		PyErr_SetString (PyExc_TypeError, error) ;
		return	0 ;
	}

	KBValue		kbValue [MAX_CALL_ARGS]	;
	bool	 	pyErr	;
	unsigned int	nArgs	;
	for (nArgs = 0 ; nArgs < MAX_CALL_ARGS ; nArgs += 1)
		if (pyValue[nArgs] != 0)
		{
			kbValue[nArgs]	= PyKBBase::fromPyObject (pyValue[nArgs], pyErr) ;
			if (pyErr) return 0 ;
		}
		else	break	;

	KBScriptError	*rc	= 0 ;
	KBObject 	*object = (KBObject *)pyBase->m_kbObject ;

	API_CALL(object,"KBObject.notif:") ;
	KBSlotNotifier::self()->fire (object, event, nArgs, kbValue, rc) ;
	API_DONE ;

	if (rc != 0)
		KBScriptError::processError (rc, KBScriptError::Normal) ;

	return	PY_NONE	;
}


/*  ------------------------------------------------------------------	*/

static	PyMethodDef pyKBObject_methods[] =
{
	PY_METHOD	(getElement,	PyKBObject	),
	PY_METHOD	(getName,	PyKBObject	),
	PY_METHOD	(getType,	PyKBObject	),
	PY_METHOD	(getAttr,	PyKBObject	),
	PY_METHOD	(setAttr,	PyKBObject	),
	PY_METHOD	(getConfig,	PyKBObject	),
	PY_METHOD	(isEnabled,	PyKBObject	),
	PY_METHOD	(setEnabled,	PyKBObject	),
	PY_METHOD	(isVisible,	PyKBObject	),
	PY_METHOD	(setVisible,	PyKBObject	),
	PY_METHOD	(setFGColor,	PyKBObject	),
	PY_METHOD	(setBGColor,	PyKBObject	),
	PY_METHOD	(setFont,	PyKBObject	),
	PY_METHOD	(getQTProperty,	PyKBObject	),
	PY_METHOD	(update,	PyKBObject	),
	PY_METHOD	(width,		PyKBObject	),
	PY_METHOD	(height,	PyKBObject	),
	PY_METHOD	(resize,	PyKBObject	),
	PY_METHOD	(getParent,	PyKBObject	),
	PY_METHOD	(getBlock,	PyKBObject	),
	PY_METHOD	(getForm,	PyKBObject	),
	PY_METHOD	(getReport,	PyKBObject	),
	PY_METHOD	(getRoot,	PyKBObject	),
	PY_METHOD	(getNamedCtrl,	PyKBObject	),
	PY_METHOD	(getControls,	PyKBObject	),
	PY_METHOD	(getChildren,	PyKBObject	),
	PY_METHOD	(lastError,	PyKBObject	),
	PY_METHOD	(setTabOrder,	PyKBObject	),
	PY_METHOD	(openServer,	PyKBObject	),
	PY_METHOD	(__repr__,	PyKBObject	),
	PY_METHOD	(notify,	PyKBObject	),
	PY_METHOD_END
}	;

static	const char *pyKBObject_names  [] =
{
	"KBObject",
	0
}	;

void	initPyKBObject
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBObject",
		module,
		pyKBObject_methods,
		"KBBase",
		pyKBObject_names
	)	;
}
