/***************************************************************************
    file	         : kb_pyscript.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<Python.h>


#if		__KB_AUTOMOC
#include	"kb_pyscript.moc"
#else
#include 	"kb_pyscript.h"
#endif



#include	<qglobal.h>
#include	<qdict.h>
#include	<qfile.h>
#include	<qtextstream.h>
#include 	<qmessagebox.h>
#include	<qsettings.h>
#include	<qdir.h>

#ifdef		Q_OS_LINUX
#include	<unistd.h>
#endif

#include	"kb_python.h"

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_dbinfo.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_node.h"
#include	"kb_block.h"
#include	"kb_script.h"
#include	"kb_slot.h"
#include	"kb_textedit.h"
#include	"kb_docroot.h"
#include	"kb_pyfactory.h"

#include	"kb_notifier.h"
#include	"kb_locator.h"
#include	"kb_options.h"

#include	"kb_pydebug.h"
#include	"kb_pybase.h"
#include	"kb_pyvalue.h"


#include	"tkc_pydebug.h"
#include	"tkc_pydebugwidget.h"


#define		__DEBUG_PYSCRIPT	(0)
#if		__DEBUG_PYSCRIPT
#include	"kb_dprintf.h"
#endif


#define	EFUNC	"__expr_%1"
#define	PYHDR	"import RekallDefs.py\n\n"
#undef	PYHDR
#define	PYHDR	""
#define	EWRAP	"def %1 (_ctrl) :\n\t return %2\n"
#define	FWRAP	"%1"


#ifdef	_NODECREF
#undef	Py_DECREF
#undef	Py_XDECREF
#define	Py_DECREF(x)
#define	Py_XDECREF(x)
#endif	// _NODECREF

#ifdef	Q_OS_WIN32
extern	__declspec(dllimport)PyObject	*PyExc_IOError	;
#endif
#ifdef	Q_OS_LINUX
extern	PyObject			*PyExc_IOError	;
#endif


/*  KBPYModule								*/
/*  ----------								*/
/*  Simple class used to record information about loaded python modules	*/
/*  Basically just contains the location and a time stamp used to	*/
/*  check for validity.							*/

class	KBPYModule
{
public	:

	KBLocation	m_location	;
	PyObject	*m_pyMod	;
	QString		m_stamp		;

	inline	KBPYModule
		(	KBLocation	&location,
			PyObject 	*pyMod,
			QString		stamp	= QString::null
		)
		:
		m_location (location),
		m_pyMod	   (pyMod),
		m_stamp	   (stamp)
	{
	}
}	;


	KBNotifier		*kbNotifier	;
	KBCallback		*kbCallback	;


static	QString	errorMsg	;	/* Execution error message	*/
static	QString	errorIdent	;	/* Execution error source	*/
static	uint	errorLno	;	/* Execution error line number	*/
static	QString	errorText	;	/* Execution traceback		*/

static	uint	exprSeq		;	/* Unique expression index	*/

static	QDict<KBPYModule>	loadMap		;
static	QDict<KBPYModule>	nameMap		;
static	QDict<KBPYScriptCode>	inlineMap	;

static	PyObject		*rekallMod	;
static	PyObject		*rekallMain	;
static	PyObject		*rekallLoader	;
#if	0
static	PyObject		*rekallTBModule	;
static	PyObject		*rekallTBFunc	;
#endif
static	QString			pyPath		;

static	TKCPyDebug		debugger	;
static	bool			debuggerOK	;
static	bool			errDebugged	;

static	KBPYScriptIF		*pyScriptIF	;

static	QString	getPythonError	(cchar *)	;



/*  setErrDebugged							*/
/*		: Set error debugged flag				*/
/*  (returns)	: void		:					*/

void	TKCPySetErrDebugged ()
{
	/* Normally, after an error occurs, the error passed back to	*/
	/* the caller will indicate that the caller should display the	*/
	/* script file which contains the error. However, when a	*/
	/* debugger is running (which displays the script itself),	*/
	/* calling this routine will stop the caller displaying it.	*/
	errDebugged = true ;
}

/*  compileAndLoad							*/
/*		: Compile and load a script				*/
/*  module	: TKCPyCookie *	: Module cookie				*/
/*  eText	: QString &	: Python error text return		*/
/*  eError	: QString &	: Error return				*/
/*  eDetails	: QString *	: Error details return			*/
/*  pyErr	: bool &	: Python error occurred			*/
/*  (returns)	: bool		: Success				*/

bool	TKCPyCompileAndLoad
	(	TKCPyCookie	*module,
		QString		&eError,
		QString		&eDetails,
		bool		&pyErr
	)
{
	/* This routine is used so that the debugger can have a script	*/
	/* compiled and loaded. It is done via a routine so that the	*/
	/* KBScriptIF interface does not have to be visible to the	*/
	/* debugger.							*/
	if (pyScriptIF == 0)
	{
		eError	 = "Cannot compile: no python interface loaded?" ;
		eDetails = QString::null ;
		return	 false	;
	}

	KBError	error	;

	if (!pyScriptIF->load
		(	((TKCPyRekallCookie *)module)->getLocation(),
			error,
			pyErr
		))
	{
		eError   = error.getMessage () ;
		eDetails = error.getDetails () ;
		return	 false ;
	}

	return	true ;
}

/*  TKCPyModuleToCookie							*/
/*		: Get module cookie given identification		*/
/*  ident	: const QString & : Identification			*/
/*  (returns)	: TKCPyCookie *	  : Module or null if not found		*/

TKCPyCookie
	*TKCPyModuleToCookie
	(	const QString	&ident
	)
{
	KBPYModule	*module	;
	KBPYScriptCode	*code	;

	if ((module = loadMap  .find(ident)) != 0)
		return	new TKCPyRekallCookie (module->m_location) ;

	if ((code   = inlineMap.find(ident)) != 0)
		return	new TKCPyRekallCookie (code  ->m_location) ;

	return	0	;
}	

/*  TKCPyCookieToModule							*/
/*		: Get module for specified module cookie		*/
/*  cookie	: TKCPyCookie *	: Module cookie				*/
/*  (returns)	: PyObject *	: Module or null if not found		*/

PyObject*TKCPyCookieToModule
	(	TKCPyCookie *cookie
	)
{
	const QString	&ident	= ((TKCPyRekallCookie *)cookie)->getLocation().ident() ;
	KBPYModule	*module	;

	if ((module = loadMap.find (ident)) != 0)
		return	module->m_pyMod	;

	return	0 ;
}	

/*  getPythonString							*/
/*		: Get string for python object				*/
/*  pyobj	: PyObject *	: Python object				*/
/*  returns	: QString	: String				*/

QString	getPythonString
	(	PyObject	*pyobj
	)
{
	if ((pyobj == 0) || (pyobj == Py_None))
		return	"<None>" ;

	if (PyString_Check (pyobj))
		return	PyString_AsString (pyobj) ;

	QString	 res	;
	PyObject *str	= PyObject_Str (pyobj) ;
	if (str == 0)
		return	"<No string representation>" ;

	DPRINTF
	((	"getPythonString (from Obj) %s\n", 
		PyString_AsString (str)
	))	;

	res	= PyString_AsString (str) ;
	Py_DECREF (str) ;
	return	res	;
}


/*  TKCPySetupEditor							*/
/*		: Set up editor for python				*/
/*  editor	: KBTextEdit *	: Editor				*/
/*  (returns)	: void		:					*/

void	TKCPySetupEditor
	(	KBTextEdit	*editor
	)
{
	editor->setHighlight ("py") ;
}

/*  getPythonError							*/
/*		: Get error details					*/
/*  deferr	: cchar *	: Default error text			*/
/*  (returns)	: QString	: Error details				*/

static	QString	getPythonError
	(	cchar	*deferr
	)
{
	PyObject *except ;
	PyObject *value  ;
	PyObject *traceb ;

	PyErr_Fetch		 (&except, &value, &traceb) ;
	if (except == 0) return deferr ;
	PyErr_NormalizeException (&except, &value, &traceb) ;

	Py_XDECREF(except) ;

	QString	 valtext = QString::null ;
	QString	 trbtext = QString::null ;
	if (value  != 0) valtext = getPythonString (value ) ;
	if (traceb != 0) trbtext = getPythonString (traceb) ;

	Py_XDECREF(value ) ;
	Py_XDECREF(traceb) ;

	return	QString("%1 %2").arg(valtext).arg(trbtext) ;
}

/*  saveCompileError							*/
/*		: Get compilation error details				*/
/*  location	: KBLocation &	: Location of failed script		*/
/*  deferr	: cchar *	: Default error text			*/
/*  (returns)	: QString	: Error message				*/

QString	saveCompileError
	(	KBLocation	&location,
		cchar		*deferr
	)
{
	QString	 eText	 ;

	PyObject *object ;
	PyObject *value  ;
	PyObject *traceb ;

	PyObject *emsg	 ;
	PyObject *detail ;
	PyObject *lnoN	 ;
	PyObject *lnoS	 ;


	errorMsg	= deferr 	;
	errorLno	= 0	 	;
	errorText	= QString::null	;

	PyErr_Fetch (&object, &value, &traceb) ;
	if (value == 0) return	errorMsg ;


	Py_XDECREF(object) ;
	Py_XDECREF(traceb) ;

	/* The error format of an error tuple seems to be		*/
	/*								*/
	/* 	( error_text, ( unknown, line_no, col_no, line_text ))	*/
	/*								*/
	/* If this is appears to be the case then format the error as	*/
	/*								*/
	/* 	source: line_no: error_text				*/
	/*								*/
	/* If not then convert the error object to its representation	*/
	/* as a string and use that.					*/
	if (!PyTuple_Check (value))
	{
		errorMsg = getPythonString (value) ;
		Py_XDECREF(value) ;
		return	errorMsg  ;
	}

	if (PyTuple_Size (value) != 2) 
	{
		errorMsg = getPythonString (value) ;
		Py_XDECREF(value) ;
		return	errorMsg  ;
	}

	emsg	= PyTuple_GetItem (value, 0) ;
	detail	= PyTuple_GetItem (value, 1) ;

	if
	(	!PyString_Check (emsg  ) ||
		!PyTuple_Check  (detail) || (PyTuple_Size (detail) < 4)
	)
	{
		errorMsg = getPythonString (value) ;
		Py_XDECREF(value) ;
		return	errorMsg  ;
	}

	lnoN	 = PyTuple_GetItem (detail, 1) ;
	lnoS	 = PyObject_Str    (lnoN     ) ;

	errorLno = atoi(PyString_AsString(lnoS)) - 1 ;
	errorMsg = QString("%1 : %2: %2")
			 .arg(location.isFile() ? location.path () : location.name())
			 .arg(errorLno)
			 .arg(PyString_AsString(emsg)) ;

	Py_XDECREF(value) ;
	Py_XDECREF(lnoS ) ;

	return	errorMsg  ;
}


#if	0
static	QString	getTraceBack
	(	PyObject	*except,
		PyObject	*value,
		PyObject	*traceb
	)
{
	if (rekallTBModule == 0)
		if ((rekallTBModule = PyImport_ImportModule (__C("RekallTraceBack"))) == 0)
		{
			fprintf
			(	stderr,
				"getTraceBack: RekallTraceBack: cannot load\n"
			)	;
			return	QString::null ;
		}

	if (rekallTBFunc  == 0)
	{
		PyObject *dict	  = PyModule_GetDict    (rekallTBModule) ;
		if ((rekallTBFunc = PyDict_GetItemString(dict, __C("traceBack"))) == 0)
		{
			fprintf
			(	stderr,
				"getTraceBack: RekallTraceBack: no function\n"
			)	;
			return	QString::null ;
		}
	}

	PyObject *args = PyTuple_New  (3) ;
	Py_INCREF (except) ; PyTuple_SetItem (args, 0, except) ;
	Py_INCREF (value ) ; PyTuple_SetItem (args, 1, value ) ;
	Py_INCREF (traceb) ; PyTuple_SetItem (args, 2, traceb) ;

	PyObject *res  = PyEval_CallObject(rekallTBFunc, args) ;
	Py_DECREF(args)	 ;

	if (res == 0)
	{
		fprintf
		(	stderr,
			"getTraceBack: RekallTraceBack: no traceback\n"
		)	;
		return	QString::null ;
	}

//	DPRINTF
//	((	"getTraceBack: RekallTraceBack:\n%s\n",
//		PyString_AsString (res)
//	))	;

	QString	r = PyString_AsString (res) ;
	Py_DECREF(res)	;
	return	r	;
}
#endif

/*  saveExecutionError							*/
/*		: Save execution error					*/
/*  (returns)	: PyObject *	: Exception object if any		*/

static	PyObject *saveExecutionError ()
{
	PyObject *except ;
	PyObject *value  ;
	PyObject *traceb ;

	errorMsg	= "Unknown execution error" ;
	errorLno	= 0  ;
	errorIdent	= "" ;
	errorText	= QString::null ;

	PyErr_Fetch  (&except, &value, &traceb) ;

	DPRINTF
	((	"KBPYScriptIF::saveExeError: except %p/%p\n",
		(void *)except,
		(void *)PyKBRekallAbort
	))	;

	if (except == 0) return 0 ;

	PyErr_NormalizeException (&except, &value, &traceb) ;
	if ((traceb == 0) || !PyTraceBack_Check (traceb))
	{
		errorMsg = QString("%1<br>%2")
				.arg(getPythonString (except))
				.arg(getPythonString (value )) ;
		return	except	;
	}


#if	(PY_MAJOR_VERSION < 2) || (PY_MINOR_VERSION <= 3)
	typedef struct _tracebackobject
	{
	        PyObject		HEAD		;
        	struct _tracebackobject *tb_next	;
        	PyFrameObject		*tb_frame	;
        	int			tb_lasti	;
        	int			tb_lineno	;
	}
	PyTracebackObject	;
#endif

//	errorText = getTraceBack (except, value, traceb);

	PyTracebackObject *tb = (PyTracebackObject *)traceb ;
	QStringList	  tbl ;

	while (tb != 0)
	{
		if (tbl.count() > 256)
			tbl.remove(tbl.at(0)) ;

		tbl.append
		(	QString	("<tr><td>%1</td><td>%2</td><td>%3</td></tr>")
				.arg(PyString_AsString(tb->tb_frame->f_code->co_filename))
				.arg(tb->tb_lineno)
				.arg(PyString_AsString(tb->tb_frame->f_code->co_name))
		)	;

		if (tb->tb_next == 0)
		{
			errorLno   = tb->tb_lineno - 1 ;
			errorIdent = PyString_AsString(tb->tb_frame->f_code->co_filename) ;
			break	;
		}

                tb = tb->tb_next ;
	}

//	if (errorText.isNull())
		errorText	= "<table>" + tbl.join("") + "</table>" ;

	if (PyClass_Check(except))
	{
		errorMsg  = QString("%1: %2")
				.arg(PyString_AsString (((PyClassObject *)except)->cl_name))
				.arg(getPythonString(value))
				;
	}
	else 
	{
		/* We probably have an old style string exception ...	*/
		PyObject *exctype = PyObject_Str(except) ;
		QString  qExctype ;
	
		if (exctype)
		{
			qExctype = PyString_AsString(exctype);
			Py_DECREF(exctype)	;
		}
		else	qExctype = "<unknown>"	;

		errorMsg = QString("%1: %2")
				.arg(qExctype)
				.arg(getPythonString(value))
				;
	}

	return	except ;
}

#if	defined(Q_OS_LINUX) || defined(Q_OS_MACX)
/*  addPathElement							*/
/*		: Add element to python path				*/
/*  elem	: const QString & : Element				*/
/*  (returns)	: QString	  : New python path			*/

static	QString	addPathElement
	(	const QString	&elem
	)
{
	PyObject *path	= PySys_GetObject (__C("path")) ;
	int	 plen	= PyList_Size	  (path) ;
	QString	 text	= elem ;

	for (int idx = 0 ; idx < plen ; idx += 1)
	{
		PyObject *item = PyList_GET_ITEM (path, idx) ;

		text	+= DELIM ;
		text 	+= getPythonString (item) ;
	}

	DPRINTF
	((	"Python path: [%s]\n",
		(cchar *)text
	))	;
	return	text ;
}
#endif

/*  execFunc	: Execute function					*/
/*  func	: PyObject *	  : Function object			*/
/*  node	: PyObject *	  : Invoking node			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: const KBValue * : Argument vector			*/
/*  resval	: KBValue &	  : Result value			*/
/*  source	: KBNode *	  : Source object			*/
/*  event	: const QString & : Source event			*/
/*  (returns)	: KBScript::ExeRC : Return code				*/

static	KBScript::ExeRC	execFunc
	(	KBNode			*location,
		PyObject		*func,
		PyObject		*node,
		uint			argc,
		const KBValue		*argv,
		KBValue			&resval,
		KBNode			*source		= 0,
		const QString		&event		= QString::null
	)
{
	DPRINTF
	((	"KBPYScriptCode::execFunc: called: source=[%p]\n",
		(void *)source
	))	;

	uint	 extra	  = source == 0 ? 0 : 2 ;

	PyObject	*argvec = PyTuple_New (argc + 1 + extra) ;
	PyTuple_SetItem (argvec, 0, node) ;

	SHOWREF	 (argvec, "argument list"  ) ;

	/* This function may be called with a source object and an	*/
	/* event, in which case these are pushed onto the argument list	*/
	/* first. This is used when invoking a slot (to pass the	*/
	/* originating object and event name).				*/
	if (source != 0)
	{
		PyObject *pyInst = PyKBBase::makePythonInstance (source) ;
		if (pyInst == 0) return KBScript::ExeError ;

		PyTuple_SetItem (argvec, 1,  pyInst) ;

		if (event.isEmpty())
			PyTuple_SetItem (argvec, 2,  PyString_FromString (""   )) ;
		else	PyTuple_SetItem (argvec, 2,  PyString_FromString (event)) ;
	}

	for (uint idx = 0 ; idx < argc ; idx += 1)
	{
		PyObject *pyobj	= PyKBBase::fromKBValue (argv[idx]) ;
		PyTuple_SetItem (argvec, idx + extra + 1, pyobj)    ;
	}

	/* Clear the error-debugged flag. The debugger will set this if	*/
	/* it handles an error in while executing the function.		*/
	errDebugged = false  ;

	KBScriptIF::pushLocation
	(	location == 0 ? 0 : &location->getDocRoot()->getDocLocation(),
		location
	)	;

	PyObject *res = PyEval_CallObject (func, argvec) ;

	KBScriptIF::popLocation  () ;

	SHOWREF	 (argvec, "argument list") ;
	Py_DECREF(argvec) ;

	if (res == 0)
	{
		DPRINTF
		((	"KBPYScriptCode::execFunc: exception thrown: ed=%d\n",
			errDebugged
		))	;

		PyObject	*except	= saveExecutionError () ;
		KBScript::ExeRC rc	= except == PyKBRekallAbort ? KBScript::ExeAbort :
					  except == PyKBRekallTest  ? KBScript::ExeTest  :
					  errDebugged		    ? KBScript::ExeFail  :
								      KBScript::ExeError ;
		errDebugged  = false ;
		return	rc ;
	}

	bool	rc	;

	if	(PyLong_Check   (res))
	{	long	r = PyLong_AsLong     (res) ;
		resval	  = KBValue ((int)r) ;
		rc	  = r != 0 ;
	}
	else if	(PyInt_Check    (res))
	{	int	i = PyInt_AsLong      (res) ;
		resval	  = KBValue ((int)i) ;
		rc	  = i != 0 ;
	}
	else if	(PyFloat_Check  (res))
	{	double  d = PyFloat_AsDouble  (res) ;
		resval	  = KBValue (d) ;
		rc	  = d != 0.0    ;
	}
	else if (PyString_Check (res))
	{	char   *s = PyString_AsString (res) ;
		resval	  = KBValue (s) ;
		rc	  = s[0] != 0	;
	}
	else
	{	resval	  = KBValue ()	;
		rc	  = true	;
	}

	SHOWREF	 (res, "result") ;
	Py_DECREF(res) ;

	DPRINTF
	((	"KBPYScriptCode::execFunc: rc=%d\n",
		rc
	))	;

	return	rc ? KBScript::ExeTrue : KBScript::ExeFalse ;
}

/*  execFunc	: Execute function					*/
/*  func	: PyObject *	  : Function object			*/
/*  node	: KBNode *	  : Invoking node			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: const KBValue * : Argument vector			*/
/*  resval	: KBValue &	  : Result value			*/
/*  source	: KBNode *	  : Source object			*/
/*  event	: const QString & : Source event			*/
/*  (returns)	: KBScript::ExeRC : Return code				*/

static	KBScript::ExeRC	execFunc
	(	KBNode			*location,
		PyObject		*func,
		KBNode			*node,
		uint			argc,
		const KBValue		*argv,
		KBValue			&resval,
		KBNode			*source	= 0,
		const QString		event	= QString::null
	)
{
	/* Create a python class instance for the sepecified node. This	*/
	/* can fail, for instance of the node type is unknown here.	*/
	PyObject *pyInst ;

	if (node == 0)
		pyInst	= PY_NONE ;
	else	pyInst	= PyKBBase::makePythonInstance (node) ;

	if (pyInst == 0)
		return KBScript::ExeError ;

	return	execFunc
		(	location,
			func,
			pyInst,
			argc,
			argv,
			resval,
			source,
			event
		)	;
}

/*  ------------------------------------------------------------------  */

/*  KBPYScriptObject							*/
/*  KBPYScriptObject							*/
/*		: Constructor for persistent script object		*/
/*  pyObject	: PyObject *	   : Associated python object		*/
/*  (returns)	: KBPYScriptObject :					*/

KBPYScriptObject::KBPYScriptObject
	(	PyObject	*pyObject
	)
	:
	m_pyObject (pyObject)
{
	/* Since we are keeping a pointer to the python object, we need	*/
	/* to increment its use count.					*/
	Py_XINCREF (pyObject) ;
}

/*  KBPYScriptObject							*/
/*  KBPYScriptObject							*/
/*		: Destructor for persistent script object		*/
/*  (returns)	:		:					*/

KBPYScriptObject::~KBPYScriptObject ()
{
	/* Decrement the use count. Most likely this will result in the	*/
	/* desctruction of the python object.				*/
	if (m_pyObject->ob_refcnt > 1)
	{
		static	int	showPyRefCnt = -1 ;
		if (showPyRefCnt < 0)
			showPyRefCnt = getenv("REKALL_SHOWPYREFCNT") == 0 ? 0 : 1 ;
		if (showPyRefCnt > 0)
			kbDPrintf
			(	"KBPYScriptObject::~KBPYScriptObject: count %d\n",
				m_pyObject->ob_refcnt
			)	;
	}

	Py_XDECREF (m_pyObject) ;
}

/*  KBPYScriptObject							*/
/*  object	: Retrieve python object				*/
/*  (returns)	: PyObject *	: Python object				*/

PyObject
	*KBPYScriptObject::object ()
{
	/* Increment the use count on the object before returning it	*/
	/* to the caller.						*/
	DPRINTF
	((	"KBPYScriptObject::object: count %d\n",
		m_pyObject->ob_refcnt
	))	;
	Py_XINCREF (m_pyObject) ;
	return	m_pyObject	;
}



/*  ------------------------------------------------------------------  */

/*  KBPYScriptCode							*/
/*  KBPYScriptCode: Constructor for compiled (inline) script functions	*/
/*  pyFunc	  : PyObject *	    : Python function			*/
/*  pyDict	  : PyObject *	    : Associated global dictionary	*/
/*  location	  : KBLocation &    : Nominal inline location		*/
/*  inherit	  : KBEvent *	    : Inherited event			*/
/*  (returns)	  : KBPYScriptCode  :					*/
	
KBPYScriptCode::KBPYScriptCode
	(	PyObject	*pyFunc,
		PyObject	*pyDict,
		KBNode		*owner,
		KBEvent		*inherit,
		KBLocation	&location
	)
	:
	KBScriptCode	(owner, inherit),
	m_pyFunc	(pyFunc),
	m_location	(location)
{
	inlineMap.insert (m_location.ident(), this) ;

	PyObject *pyInst  = PyKBBase::makePythonInstance (m_owner->getRoot()) ;
	Py_INCREF	(pyInst ) ;
	PyDict_SetItem	(pyDict, PyString_FromString ("root"), pyInst) ;
}

/*  KBPYScriptCode							*/
/*  ~KBPYScriptCode: Destructor for compiled script object		*/
/*  (returns)	   :			:				*/

KBPYScriptCode::~KBPYScriptCode ()
{
	TKCPyDebugWidget *debWidget ;
	if ((debWidget = TKCPyDebugWidget::widget()) != 0)
	{
		TKCPyRekallCookie cookie (m_location) ;
		debWidget->dropSource (&cookie) ;
	}


	inlineMap.remove (m_location.ident()) ;
	Py_XDECREF	 (m_pyFunc) ;
}


/*  KBPYScriptCode							*/
/*  execute	: Execute compiled code					*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: const KBValue * : Argument vector			*/
/*  resval	: KBValue &	  : Result value			*/
/*  (returns)	: KBScript::ExeRC : Success				*/

KBScript::ExeRC	KBPYScriptCode::execute
	(	uint		argc,
		const KBValue	*argv,
		KBValue		&resval
	)
{
	/* Create a python class instance for the sepecified node. This	*/
	/* can fail, for instance of the node type is unknown here.	*/
	PyObject   *pyInst ;

	if (m_owner == 0)
		pyInst	= PY_NONE ;
	else	pyInst	= PyKBBase::makePythonInstance (m_owner) ;

	if (pyInst == 0)
		return KBScript::ExeError ;

	cchar	 *error  ;
	PyKBBase *pyBase = 0 ;
	KBEvent	 *saved	 = 0 ;

	/* If invoked with a node then update the inheritance before	*/
	/* executing the function, and restore it after. This is used	*/
	/* when handling events in linked objects.			*/
	if (m_owner)
	{
		pyBase	= PyKBBase::getPyBaseFromPyInst
					(	pyInst,
						PyKBBase::m_object,
						error
					)	;

		DPRINTF
		((	"KBPYScriptCode::execute: saving  event: %p <- %p\n",
			(void *)pyBase->m_inherit,
			(void *)m_inherit
		))	;

		saved	  	  = pyBase->m_inherit	;
		pyBase->m_inherit = m_inherit		;
	}


	KBScript::ExeRC	rc = execFunc
			     (		m_owner,
					m_pyFunc,
					pyInst,
					argc,
					argv,
					resval,
					0,
					QString::null
			     )	;

	switch (rc)
	{
		case KBScript::ExeTrue  :
		case KBScript::ExeFalse :
			break	;

		default	:
			fprintf
			(	stderr,
				"KBPYScriptCode::execute: %p: setting error text: %12.12s...\n",
				(void *)this,
				errorText.latin1()
			)	;
			setErrorMessage (errorText) ;
			break	;
	}


	DPRINTF
	((	"KBPYScriptCode::execute: restore event: %p\n",
		(void *)saved
	))	;

	if (pyBase != 0)
		pyBase->m_inherit = saved ;

	return	rc ;
}

/*  KBPYScriptCode							*/
/*  execute	: Execute compiled code					*/
/*  node	: KBNode *	  : Invoking node			*/
/*  source	: KBNode *	  : Source node				*/
/*  event	: const QString & : Source event			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: const KBValue * : Argument vector			*/
/*  resval	: KBValue &	  : Result value			*/
/*  (returns)	: KBScript::ExeRC : Success				*/

KBScript::ExeRC	KBPYScriptCode::execute
	(	KBNode		*source,
		const QString	&event,
		uint		argc,
		const KBValue	*argv,
		KBValue		&resval
	)
{
	/* This method is used for event-slot script invocation.	*/
	return	execFunc
		(	m_owner,
			m_pyFunc,
			m_owner,
			argc,
			argv,
			resval,
			source,
			event
		)	;
}

void	KBPYScriptCode::setBreakpoints
	(	const QValueList<int>	&bptList
	)
{
	DPRINTF
	((	"KBPYScriptCode::setBreakpoints: %d breakpoints\n",
		bptList.count()
	))	;

//	TKCPyDebugWidget *debWidget ;
//	if ((debWidget = TKCPyDebugWidget::widget()) != 0)
//		debWidget->setBreakpoint
//		(	pyFunc,
//			bptList,
//			location.ident()
//		)	;
}


//  The Python encoding names don't match those used in the Rekall UI
//  while the case does not matter, Python does not like spaces in the
//  names.
//
static QDict<const char> *pythonEncodingName ;

/*  ------------------------------------------------------------------  */

/*  KBPYScriptIF							*/
/*  KBPYScriptIF: Constructor for Python script interface language	*/
/*  (returns)	: KBPYScriptIF	:					*/

KBPYScriptIF::KBPYScriptIF ()
{
	m_languageCode = languageToCode ("py") ;
	
	DPRINTF
	((	"KBPYScriptIF::KBPYScriptIF: m_languageCode=%d\n",
		m_languageCode
	))	;

	if (Py_IsInitialized())
	{
		pyScriptIF	= this	;
		return	;
	}

	pythonEncodingName = new QDict<const char> ;

	pythonEncodingName->insert("UTF8",	"utf8"		) ;
	pythonEncodingName->insert("BIG5", 	"big5"		) ;
	pythonEncodingName->insert("CP 874",	"cp874"		) ;
	pythonEncodingName->insert("CP 1250",	"cp1250"	) ;
	pythonEncodingName->insert("CP 1251",	"cp1251"	) ;
	pythonEncodingName->insert("CP 1252",	"cp1252"	) ;
	pythonEncodingName->insert("CP 1253",	"cp1253"	) ;
	pythonEncodingName->insert("CP 1254",	"cp1254"	) ;
	pythonEncodingName->insert("CP 1255",	"cp1255"	) ;
	pythonEncodingName->insert("CP 1256",	"cp1256"	) ;
	pythonEncodingName->insert("CP 1257",	"cp1257"	) ;
	pythonEncodingName->insert("CP 1258",	"cp1258"	) ;
	pythonEncodingName->insert("EUCJP",	"euc-jp"	) ;
	pythonEncodingName->insert("EUCKR",	"euc-kr"	) ;
	pythonEncodingName->insert("GBK",	"gbk"		) ;
	pythonEncodingName->insert("KOI8-R",	"koi8-r"	) ;
	pythonEncodingName->insert("KOI8-U",	"koi8-u"	) ;
	pythonEncodingName->insert("SHIFT-JIS",	"shift-jis"	) ;
	pythonEncodingName->insert("ISO8859-1",	"iso8859-1"	) ;
	pythonEncodingName->insert("ISO8859-2",	"iso8859-2"	) ;
	pythonEncodingName->insert("ISO8859-3",	"iso8859-3"	) ;
	pythonEncodingName->insert("ISO8859-4",	"iso8859-4"	) ;
	pythonEncodingName->insert("ISO8859-5",	"iso8859-5"	) ;
	pythonEncodingName->insert("ISO8859-6",	"iso8859-6"	) ;
	pythonEncodingName->insert("ISO8859-7",	"iso8859-7"	) ;
	pythonEncodingName->insert("ISO8859-8",	"iso8859-8"	) ;
	pythonEncodingName->insert("ISO8859-9",	"iso8859-9"	) ;
	pythonEncodingName->insert("ISO8859-10","iso8859-10"	) ;
	pythonEncodingName->insert("ISO8859-13","iso8859-13"	) ;
	pythonEncodingName->insert("ISO8859-14","iso8859-14"	) ;
	pythonEncodingName->insert("ISO8859-15","iso8859-15"	) ;
	pythonEncodingName->insert("TIS-620",	"tis-620"	) ;
 

//	PyExc_IOError	= 0  ;
	Py_Initialize	  () ;
	PyEval_InitThreads() ;

	QString	err	;
	if ((err = debugger.init()) != QString::null)
		KBError::EError
		(	TR("Python debugger initialisation error"),
			err,
			__ERRLOCN
		)	;
	else	debuggerOK = true ;

	QString	hdrdir	= locateDir ("appdata", "script/py/RekallMain.py") ;

	if (hdrdir.isEmpty())
		KBError::EFatal
		(	TR("Unable to locate main python script directory"),
			QString::null,
			__ERRLOCN
		)	;

#if	defined(Q_OS_LINUX) || defined(Q_OS_MACX)
	pyPath	= addPathElement (hdrdir += "script/py") ;
#endif

#ifdef	Q_OS_WIN32
	char tempStr[_MAX_PATH];
	ZeroMemory(tempStr, _MAX_PATH);
	HKEY hKey;
	unsigned long numBytes = sizeof(tempStr);
	unsigned long type;
	long retCode;

	QString	rootDir = getRootDir() ;

	retCode = RegOpenKeyExA(HKEY_LOCAL_MACHINE, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\App Paths\\Python.exe", 
		0, KEY_ALL_ACCESS, &hKey);

	if(retCode == ERROR_SUCCESS)
	{
		RegQueryValueExA(hKey, "", NULL, &type, (unsigned char*)tempStr, (unsigned long*)&numBytes);

		QString pathToPython(tempStr);
	
		int pos = pathToPython.findRev('\\');
	
		const QString instDir = pathToPython.left(pos);

		pyPath	= QDir::convertSeparators(instDir + ";");
		pyPath += QDir::convertSeparators(instDir + "\\Lib;");
		pyPath += QDir::convertSeparators(instDir + "\\DLLs;");
		pyPath += QDir::convertSeparators(rootDir + "\\data\\script\\py");

		DPRINTF
		((	"KBPYScriptIF::KBPYScriptIF: pyPath=[%s]\n",
			(cchar *)pyPath
		))	;
	}
	else
	{
		QString msgStr = "The Python interpreter can not be found.\n";
		msgStr += "Please visit http://www.python.org\n";
		msgStr += "and downlaod a copy of Python V2.2.x";

		QMessageBox::critical(0L, "Python Interpretor Error", msgStr);
		return;
	}

	RegCloseKey(hKey);
#endif

	PySys_SetPath (__C(strdup(pyPath.latin1()))) ;

	{
		KBPYModule *module ;
		KBLocation locn	   ;

		rekallMain 	= PyImport_ImportModule (__C("RekallMain")) ;
		if (rekallMain == 0)
		{
			KBError::EFault
			(	TR("Failed to load python module 'RekallMain'"),
				getPythonError ("Import module failed"),
				__ERRLOCN
			)	;
			return	;
		}

		locn	   	= KBLocation (0, "script", "builtin", "RekallMain") ;
		module	   	= new KBPYModule   (locn, rekallMain, "") ;
		nameMap.insert ("RekallMain", module) ;
	}
	{
		rekallLoader 	= PyImport_ImportModule (__C("RekallLoader")) ;
		if (rekallLoader == 0)
		{
			KBError::EFault
			(	TR("Failed to load python module 'RekallLoader'"),
				getPythonError ("Import module failed"),
				__ERRLOCN
			)	;
			return	;
		}
	}


	/* Register all interface classes into a module which we	*/
	/* construct for the purpose, called (appropriately) "Rekall".	*/
	rekallMod = Py_InitModule3 ("Rekall", 0, "Rekall") ;
	
	PyKBRekallError = PyErr_NewException("Rekall.error", 0, 0) ;
	PyKBRekallAbort = PyErr_NewException("Rekall.abort", 0, 0) ;
	PyKBRekallTest  = PyErr_NewException("Rekall.test",  0, 0) ;

	Py_INCREF(PyKBRekallError) ;
	Py_INCREF(PyKBRekallAbort) ;
	Py_INCREF(PyKBRekallTest ) ;

	PyModule_AddObject(rekallMod, "error", PyKBRekallError) ;
	PyModule_AddObject(rekallMod, "abort", PyKBRekallAbort) ;
	PyModule_AddObject(rekallMod, "test",  PyKBRekallTest ) ;
	
	INITPYMOD (initPyKBBase,	rekallMod) ;
	INITPYMOD (initPyKBObject,	rekallMod) ;
	INITPYMOD (initPyKBItem,	rekallMod) ;
	INITPYMOD (initPyKBBlock,	rekallMod) ;
	INITPYMOD (initPyKBFramer,	rekallMod) ;
	INITPYMOD (initPyKBContainer,	rekallMod) ;
	INITPYMOD (initPyKBTabber,	rekallMod) ;
	INITPYMOD (initPyKBStack,	rekallMod) ;
	INITPYMOD (initPyKBFormBlock,	rekallMod) ;
	INITPYMOD (initPyKBForm,	rekallMod) ;
	INITPYMOD (initPyKBButton,	rekallMod) ;
	INITPYMOD (initPyKBLabel,	rekallMod) ;
	INITPYMOD (initPyKBField,	rekallMod) ;
	INITPYMOD (initPyKBRichText,	rekallMod) ;
	INITPYMOD (initPyKBChoice,	rekallMod) ;
	INITPYMOD (initPyKBListBox,	rekallMod) ;
	INITPYMOD (initPyKBCheck,	rekallMod) ;
	INITPYMOD (initPyKBField,	rekallMod) ;
	INITPYMOD (initPyKBGrid,	rekallMod) ;
	INITPYMOD (initPyKBEvent,	rekallMod) ;
	INITPYMOD (initPyKBSlot,	rekallMod) ;
	INITPYMOD (initPyKBDBLink,	rekallMod) ;
	INITPYMOD (initPyKBSQLSelect,	rekallMod) ;
	INITPYMOD (initPyKBSQLInsert,	rekallMod) ;
	INITPYMOD (initPyKBSQLUpdate,	rekallMod) ;
	INITPYMOD (initPyKBSQLDelete,	rekallMod) ;
	INITPYMOD (initPyKBTabberPage,	rekallMod) ;
	INITPYMOD (initPyKBStackPage,	rekallMod) ;
	INITPYMOD (initPyKBLinkTree,	rekallMod) ;
	INITPYMOD (initPyKBLink,	rekallMod) ;
	INITPYMOD (initPyKBTree,	rekallMod) ;

	extern	void	initPYBasic 	() ;
	extern	void	initPYTest 	() ;
	extern	void	initPYCookie 	() ;
	extern	void	initPyValue     (PyObject*) ;

	initPYBasic	() ;
	initPYTest	() ;
	initPYCookie	() ;
	initPyValue	(rekallMain) ;

	pyScriptIF	= this	;
}

/*  KBPYScriptIF							*/
/*  ~KBPYScriptIF: Destructor for Python script interface language	*/

KBPYScriptIF::~KBPYScriptIF ()
{
	pyScriptIF	= 0	;
}

/*  KBPYScriptIF							*/
/*  getIface	: Get interface object if any				*/
/*  (returns)	: KBPYScriptIF * : Interface object or null		*/

KBPYScriptIF
	*KBPYScriptIF::getIface ()
{
	return	pyScriptIF	;
}

/*  KBPYScriptIF							*/
/*  setApp	: Set application pointers				*/
/*  _kbNotifer	: KBNotifier *	: Application notifier			*/
/*  _kbCallback	: KBCallback *	: Application callback			*/
/*  (returns)	: void		:					*/

void	KBPYScriptIF::setApp
	(	KBNotifier	*_kbNotifier,
		KBCallback	*_kbCallback
	)
{
	kbNotifier	= _kbNotifier ;
	kbCallback	= _kbCallback ;

	connect
	(	KBNotifier::self(),
		SIGNAL	(sOptionsChanged()),
		SLOT	(optionsChanged ())
	)	;
}

/*  KBPYScriptIF							*/
/*  optionsChanged							*/
/*		: Notification of user changing options			*/
/*  (returns)	: void		:					*/

void	KBPYScriptIF::optionsChanged ()
{
	DPRINTF
	((	"KBPYScriptIF::optionsChanged: CALLED\n"
	))	;

	/* Reset the codec used for string translation. Note that this	*/
	/* _will not_ reset compiled code, so forms must be closed and	*/
	/* reopened to fully handle this.				*/
	extern	void	resetCodec () ;
	resetCodec ()	;
}

/*  KBPYScriptIF							*/
/*  load	: Load a module into the interpreter instance		*/
/*  location	: KBLocation &	: Database location			*/
/*  eText	: QString &	: Error text				*/
/*  ePatt	: QString &	: Error pattern				*/
/*  pError	: KBError &	: Error return				*/
/*  pyErr	: bool &	: Error is python error			*/
/*  (returns)	: bool		:					*/

bool	KBPYScriptIF::load
	(	KBLocation	&location,
		KBError		&pError,
		bool		&pyErr
	)
{
	KBPYModule  *module 	;
	PyObject    *pyCode	;
	PyObject    *pyMod	;

	/* Error in getting the timestamp or in getting the script text	*/
	/* are not python errors; everything else is.			*/
	QString	    stamp	= location.timestamp (pError) ;
	if (stamp == QString::null)
	{	pyErr	= false	;
		return	false	;
	}

	/* First see if the actual module has been loaded, so that it	*/
	/* can be shared between multiple instances, and if present	*/
	/* that it has not become out-of-date.				*/
	if ((module = loadMap.find (location.ident())) != 0)
	{
		DPRINTF
		((	"KBScriptIF::load(%s) %s/%s\n",
			(cchar *)location.ident(),
			(cchar *)stamp,
			(cchar *)module->m_stamp
		))	;

		if (module->m_stamp == stamp)
		{	pyErr	= false	;
			return	true	;
		}
	}

	/* Get the module text (and check for errors), and then compile	*/
	/* it to a code object; check at this point for compilation	*/
	/* errors.							*/
	QString	   text	= location.contents  (pError) ;
	if (text.isNull())
	{	pyErr	= false	;
		return	false	;
	}

	if (location.dbInfo() != 0)
#ifdef	Q_OS_WIN32
		PySys_SetPath (__C(QString("%1;%2").arg(location.dbInfo()->getDBPath()).arg(pyPath))) ;
#else
		PySys_SetPath (__C(QString("%1:%2").arg(location.dbInfo()->getDBPath()).arg(pyPath)));
#endif
	else	PySys_SetPath (__C(pyPath)) ;

	TKCPyDebugWidget::doPushExcTrap() ;

	if ((pyCode = compileText
		      	(	location,
				text,
				pError
			)) == 0)
	{
		TKCPyDebugWidget::doPopExcTrap() ;
		pyErr	= true	;
		return	false	;
	}

	/* OK to load the module. The use of ExecCodeModuleEx is based	*/
	/* on a reading for the Python 2.1 code, but seems to work OK	*/
	/* with Python 1.5 as well.					*/
	QString	modName	= location.name() ;
	QString	impName	= location.name() ;

	if (modName.find('$') >= 0)
	{
		QStringList bits  = QStringList::split ('$', modName) ;
		modName	= bits[0] ;
		impName	= bits[1] ;

		XPRINTF
		((	"KBPYScriptIF::load: [%s]->[%s][%s]\n",
			location.name().latin1(),
			modName.latin1(),
			impName.latin1()
		))	;
	}


	if ((pyMod  = PyImport_ExecCodeModuleEx
			(	__C(modName), //__C(location.name()),
				pyCode,
				__C(location.ident())
			)) == 0)
	{
		pError	= KBError
			  (	KBError::Error,
			  	QString (TR("Error loading python module %1"))
					.arg(location.name()),
				getPythonError ("Failed to import module"),
				__ERRLOCN
			  )	;
		Py_DECREF (pyCode) ;

		TKCPyDebugWidget::doPopExcTrap() ;
		pyErr	  = true   ;
		return	  false	   ;
	}

	TKCPyDebugWidget::doPopExcTrap() ;

	/* Create a new module object, and insert (or replace) it in	*/
	/* the module dictionary.					*/
	module	= new KBPYModule (location, pyMod, stamp) ;
	loadMap.replace (location.ident(), module) ;

	QString	name	= location.name()	   ;
	int	lsep	= name.findRev ('/') 	   ;
	if (lsep >= 0) name = name.mid (lsep + 1)  ;
	nameMap.replace (name, module)		   ;

	DPRINTF
	((	"Namemap added [%s]\n",
		(cchar *)name
	))	;

	TKCPyDebugWidget *debWidget ;
	if ((debWidget = TKCPyDebugWidget::widget()) != 0)
		debWidget->codeLoaded () ;

	pyErr	= false	;
	return	true	;
}

/*  KBPYScriptIF							*/
/*  load	: Load a module into the interpreter instance		*/
/*  location	: KBLocation &	: Database location			*/
/*  eText	: QString &	: Error text				*/
/*  ePatt	: QString &	: Error pattern				*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		:					*/

bool	KBPYScriptIF::load
	(	KBLocation	&location,
		KBError		&pError
	)
{
	bool	pyErr	;
	return	load	(location, pError, pyErr) ;
}

/*  KBPyScriptIF							*/
/*  compileText	: Compile script text					*/
/*  location	: KBLocation &	  : Document location			*/
/*  script	: const QString & : Script text				*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: PyObject *	  : Compiled code or null on error	*/

PyObject*KBPYScriptIF::compileText
	(	KBLocation	&location,
		const QString	&script,
		KBError		&pError
	)
{
	DPRINTF
	((	"KBPYScriptIF::compileText: compile\n[%s]\n",
		(cchar *)script
	))	;

	PyErr_Clear () ;

	/* Compile the text. If this succeeds then pass back the code	*/
	/* object with no more ado.					*/

	/* The source code is stored in a QString instance, i.e., as	*/
	/* Unicode. Hence is seems to be straightforward to pass a	*/
	/* utf8 representation of the code to the Python interpreter.	*/
	/* The drawback of this idea is that it makes string 		*/
	/* comparisions difficult, if a different encoding is selected	*/
	/* in options/python settings/string encoding. So let's use	*/
	/* that encoding. See						*/
	/*	http://www.python.org/peps/pep-0263.html		*/
	/*	for the "header"					*/
	/* If we do this then check the encoding, see the comments in	*/
	/* "checkScriptEncoding" for the details.			*/
	QString	pyenc	 = KBOptions::getPythonEncoding() ;
	cchar	*encname = pyenc.isEmpty() ? 0 : (*pythonEncodingName)[pyenc] ;
	QString source	 ;

	if (encname != 0)
	{
		source	= QString("# -*- coding: %1 -*-\r\n%2")
					.arg(encname)
					.arg(script )
					;

		if (!checkScriptEncoding
			(	location,
				source,
				encname,
				pError
			))
			return	0 ;
	}
	else	source	= "#\r\n" + script ;


	QCString encsource = pyEncodeSourcecode (source) ;
	PyObject *pyobj    = Py_CompileString
				(	__C(encsource),
					__C(location.ident()),
					Py_file_input
				)	;

	if (pyobj != 0) return pyobj ;

	DPRINTF
	((	"KBPYScriptIF::compileText: Py_CompileString failed\n%s\n",
		encsource.data()
	))	;

	/* Got an error. If possible then get the error message, which	*/
	/* is passed back via the error object.				*/
	QString	eText	= saveCompileError
			  (	location,
			  	"Unknown python compilation error occurred"
			  )	;

	pError	= KBError
		  (	KBError::Error,
			TR("Error compiling python script"),
			eText,
			__ERRLOCN
		  )	;

	return	0	;
}


bool	KBPYScriptIF::importModule
	(	PyObject	*pyFDict,
		const QString	&name,
		KBError		&pError
	)
{
	PyObject *pyModule ;
	if ((pyModule = PyImport_ImportModule (__C(name))) == 0)
	{
		pError	= KBError
			  (	KBError::Error,
				QString(TR("Cannot import module '%1'")).arg(name),
				QString::null,
				__ERRLOCN
			  )	;
		DPRINTF
		((	"Inline: lacking  [%s]\n",
			name.latin1()
		))	;
		return	false	;
	}

	Py_INCREF	(pyModule ) ;
	PyDict_SetItem	(pyFDict, PyString_FromString(__C(name)), pyModule) ;

	DPRINTF
	((	"KBPYScriptIF::importModule: imported [%s]\n",
		name.latin1()
	))	;

	return	true	;
}

/*  KBPYScriptIF							*/
/*  compileInline: Compile inline script				*/
/*  script	 : const QString & : Script text			*/
/*  fname	 : const QString & : Entry function name		*/
/*  eText	 : QString &	   : Error text				*/
/*  ePat	 : QString &	   : Error pattern			*/
/*  iList	 : QStringList &   : Import list			*/
/*  inherit	 : KBEvent *	   : Inherited event			*/
/*  pError	 : KBError &	   : Error return			*/
/*  (returns)	 : KBScriptCode *  : Compiled script or null on error	*/

KBScriptCode
	*KBPYScriptIF::compileInline
	(	KBNode			*owner,
		const QString		&script,
		const QString		&path,
		const QString		&fname,
		const QStringList	&iList,
		KBEvent			*inherit,
		KBError			&pError
	)
{
	PyObject	*pyTDict   = PyDict_New() ;
	PyObject	*pyInline  = 0 ;
	PyObject	*pyCode    = 0 ;
	PyObject	*pyFunc    = 0 ;
	PyObject	*pyModule  ;

	KBLocation	inlineLoc
			(	0,
				"script",
				KBLocation::m_pInline,
				path + ":" + fname,
				script
			)	;

	/* First compile the text of the inline script. This should	*/
	/* yeild code which represents the script itself.		*/
	if ((pyInline = compileText (inlineLoc, script, pError)) == 0)
		goto	error	;

	DPRINTF
	((	"KBPYScriptIF::compileInline: evaluating code ...\n"
	))	;

	PyDict_SetItemString (pyTDict, "__builtins__", PyEval_GetBuiltins()) ;

	/* Next evaluate the code with a temporary dictionary as its	*/
	/* environment. Since it should do nothing other than define a	*/
	/* function, this should work.					*/

	KBScriptIF::pushLocation (&owner->getDocRoot()->getDocLocation(), owner) ;
	pyCode = PyEval_EvalCode ((PyCodeObject *)pyInline, pyTDict, pyTDict) ;
	KBScriptIF::popLocation  () ;

	if (pyCode == 0)
	{
		saveExecutionError () ;

		pError	= KBError
			  (	KBError::Error,
				TR("Error creating inline code function"),
				script,
				__ERRLOCN
			  )	;
		goto	error	;
	}

	DPRINTF
	((	"KBPYScriptIF::compileInline: locating function [%s]\n",
		(cchar *)fname
	))	;

	/* Now we should be able to locate the code of the function as	*/
	/* specified by the function name ...				*/
	if (!fname.isEmpty())
	{
		if ((pyFunc = PyDict_GetItemString (pyTDict, __C(fname))) == 0)
		{
			pError	= KBError
				  (	KBError::Error,
					TR("Failed to locate inline code function"),
					script,
					__ERRLOCN
				  )	;
			goto	error	;
		}

		Py_INCREF(pyFunc) ;
	}

	/* OK. We now need to locate each module in the imports list	*/
	/* and store it in the function's dictionary. This should be	*/
	/* like the code doing an import.				*/
	for (QStringList::ConstIterator it = iList.begin() ; it != iList.end() ; ++it)
	{
		QString	modName	= *it	;
		QString	impName	= *it	;

		/* Skip RekallMain. It is now imported by default, just	*/
		/* below this loop.					*/
		if (modName == "RekallMain")
			continue ;

		/* Skip RekallCookieJar, similarly imported.		*/
		/* below this loop.					*/
		if (modName == "RekallCookieJar")
			continue ;

		DPRINTF
		((	"KBPYScriptIF::compileInline: importing [%s] ...\n",
			(cchar *)*it
		))	;

		if (modName.find('$') >= 0)
		{
			QStringList bits  = QStringList::split ('$', modName) ;
			modName	= bits[0] ;
			impName	= bits[1] ;

			XPRINTF
			((	"KBPYScriptIF::compileInline: [%s]->[%s][%s]\n",
				(*it).latin1(),
				modName.latin1(),
				impName.latin1()
			))	;
		}

		if ((pyModule = PyImport_ImportModule (__C(modName))) == 0)
		{
			pError	= KBError
				  (	KBError::Error,
					QString	(TR("Cannot import module %1")).arg(modName),
					QString::null,
					__ERRLOCN
				  )	;

			DPRINTF
			((	"KBPYScriptIF::compileInline:: lacking  [%s]\n",
				(*it).latin1()
			))	;
			goto	error	;
		}

		Py_INCREF(pyModule) ;

		PyDict_SetItem
		(	pyTDict,
			PyString_FromString (__C(impName)),
			pyModule
		)	;

		DPRINTF
		((	"KBPYScriptIF::compileInline::: imported [%s]\n",
			(*it).latin1()
		))	;
	}

	DPRINTF
	((	"KBPYScriptIF::compileInline: importing RekallMain\n"
	))	;

	/* Always import "RekallMain", "RekallCookieJar" and		*/
	/* "RekallTest". Seriously barf if the former fails, warn just	*/
	/* once for eacho of the latter.				*/
	if (!importModule (pyTDict, "RekallCookieJar", pError))
	{
		static	bool	warned ;
		if (!warned) pError.DISPLAY() ;
		warned	= true	;
	}
	if (!importModule (pyTDict, "RekallTest",      pError))
	{
		static	bool	warned ;
		if (!warned) pError.DISPLAY() ;
		warned	= true	;
	}

	if (!importModule (pyTDict, "RekallMain",   pError))
		goto	error	;


	PyDict_SetItem
	(	pyTDict,
		PyString_FromString (__C("__builtins__")),
		PyEval_GetBuiltins()
	)	;

	DPRINTF
	((	"KBPYScriptIF::compileInline: building script code object\n"
	))	;

	return	new KBPYScriptCode (pyFunc, pyTDict, owner, inherit, inlineLoc) ;

	error	:
		Py_XDECREF (pyTDict ) ;
		Py_XDECREF (pyInline) ;
		Py_XDECREF (pyCode  ) ;
		Py_XDECREF (pyFunc  ) ;

		DPRINTF
		((	"KBPYScriptIF::compileInline: failed\n"
				 "----\n"
				 "%s"
				 "----\n",
			(cchar *)script
		))	;
		return	false	;
}

/*  KBPYScriptIF							*/
/*  compileFunc	: Compile inline script function			*/
/*  script	: const QString & : Script text				*/
/*  fname	: const QString & : Entry function name			*/
/*  eText	: QString &	  : Error text				*/
/*  ePat	: QString &	  : Error pattern			*/
/*  iList	: QStringList &	  : Import list				*/
/*  inherit	: KBEvent *	  : Inherited event			*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: KBScriptCode *  : Compiled script or null on error	*/

KBScriptCode *KBPYScriptIF::compileFunc
	(	KBNode			*owner,
		const QString		&script,
		const QString		&path,
		const QString		&fname,
		const QStringList	&iList,
		KBEvent			*inherit,
		KBError			&pError
	)
{
	return	compileInline
		(	owner,
			QString(FWRAP).arg(script),
			path,
			fname,
			iList,
			inherit,
			pError
		)	;
}

/*  KBPYScriptIF							*/
/*  compileExpr	: Compile inline script expression			*/
/*  script	: const QString & : Script text				*/
/*  eText	: QString &	  : Error text				*/
/*  ePat	: QString &	  : Error pattern			*/
/*  iList	: QStringList &	  : Import list				*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: KBScriptCode *  : Compiled script or null on error	*/

KBScriptCode *KBPYScriptIF::compileExpr
	(	KBNode			*owner,
		const QString		&script,
		const QString		&path,
		const QStringList	&iList,
		KBError			&pError
	)
{
	QString	fname	= QString(EFUNC).arg(exprSeq++) ;

	return	compileInline
		(	owner,
			QString(EWRAP).arg(fname).arg(script),
			path,
			fname,
			iList,
			0,
			pError
		)	;
}


/*  KBPYScriptIF							*/
/*  compile	: Compile a script 					*/
/*  location	: KBLocation &	: Script location			*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBPYScriptIF::compile
	(	KBLocation	&location,
		KBError		&pError
	)
{

	QString	doc = location.contents (pError) ;
	if (doc == QString::null) return false	 ;

	PyObject *pyobj = compileText (location, doc, pError) ;

	if (pyobj != 0)
	{
		/* Seems OK, so bin the compiled object and return	*/
		/* success.						*/
		Py_DECREF (pyobj) ;
		return	  true	  ;
	}

	return	false	;
}


/*  findFunction: Find a function					*/
/*  scrList	: const QStringList &					*/
/*				  : List of script modules to search	*/
/*  fName	: const QString & : Function name			*/
/*  (returns)	: PyObject *	  : Function object or null		*/

PyObject*KBPYScriptIF::findFunction
	(	const QStringList	&scrList,
		const QString		&fName
	)
{
	for (QStringList::ConstIterator it = scrList.begin() ; it != scrList.end() ; ++it)
	{
		KBPYModule  *module ;
		QString	    mName   = *it ;
		int	    lsep    = mName.findRev ('/') ;

		if (lsep >= 0) mName = mName.mid (lsep + 1)  ;

		DPRINTF
		((	"KBPYScriptIF::findFunction: Namemap search [%s]\n",
			(cchar *)(mName)
		))	;

		if ((module = nameMap.find (mName)) == 0)
		{
			errorIdent = "" ;
			errorLno   = 0  ;
			errorMsg   = QString("Module %1 not found for function %2")
					  .arg(mName)
					  .arg(fName) ;
			return	 0   ;
		}

		/* Get the module dictionary (which is claimed never	*/
		/* to fail) and then look for the function therein.	*/
		/* The latter will fail if the script is messed up.	*/
		PyObject *dict	= PyModule_GetDict    (module->m_pyMod ) ;
		PyObject *func	= PyDict_GetItemString(dict, __C(fName)) ;

		if (func != 0) return func ;
	}

	errorIdent = "" ;
	errorLno   = 0  ;
	errorMsg    = QString ("Script function %1 not found").arg(fName) ;
	return	 0    ;
}

/*  KBPYScriptIF							*/
/*  execute	: Execute a python script				*/
/*  scrList	: const QStringList &					*/
/*				  : List of script modules to search	*/
/*  fname	: const QString & : Function name			*/
/*  node	: KBNode *	  : Invoking node			*/
/*  argc	: uint		  : Argument count			*/
/*  argv	: const KBValue * : Argument vector			*/
/*  resval	: KBValue &	  : Result value			*/
/*  (returns)	: KBScript::ExeRC : Result				*/

KBScript::ExeRC	KBPYScriptIF::execute
	(	const QStringList	&scrList,
		const QString		&fname,
		KBNode			*node,
		uint			argc,
		const KBValue		*argv,
		KBValue			&resval
	)
{
	QString	_errMsg		;
	QString	_errIdent	;
	uint	_errLno		= 0 ;

	if (!scrList.isEmpty())
	{
		PyObject *func = findFunction (scrList, fname) ;

		DPRINTF
		((	"KBPYScriptIF::execute: [%s]->[%p]\n",
			(cchar *)fname,
			(void  *)func
		))	;

		if (func != 0)
		{	SHOWREF	  (func, "function") ;
			KBScript::ExeRC	rc = execFunc
						(	0,
							func,
							node,
							argc,
							argv,
							resval
						)	;
			return	  rc	 ;
		}

		_errMsg	  = errorMsg   ;
		_errIdent = errorIdent ;
		_errLno	  = errorLno   ;
	}

	PyObject *func = findFunction ("RekallMain", fname) ;

	if (func != 0)
	{
		SHOWREF	  (func, "function") ;
		KBScript::ExeRC	rc = execFunc
					(	0,
						func,
						node,
						argc,
						argv,
						resval
					)	;
		return	  rc	 ;
	}

	if (!scrList.isEmpty())
	{
		errorMsg	= _errMsg   ;
		errorIdent	= _errIdent ;
		errorLno	= _errLno   ;
	}

	return	  KBScript::ExeError ;
}

/*  KBPYScriptIF							*/
/*  deleteScript: Delete script file(s)					*/
/*  location	: KBLocation &	  : Document location			*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: bool		  : Success				*/

bool	KBPYScriptIF::unlink
	(	KBLocation	&location,
		KBError		&pError
	)
{
	QString	path	= location.dbInfo()->getDBPath() + "/" + location.name() ;

	if (QFile::exists(path + ".pyc") && (::unlink (path + ".pyc") != 0))
	{
		pError	= KBError
			  (	KBError::Error,
				QString (TR("Failed to delete script code %1.pyc")).arg(path),
				strerror(errno),
				__ERRLOCN
			  )	;
		return	false	;
	}

	if (::unlink (path + ".py") != 0)
	{
		pError	= KBError
			  (	KBError::Error,
			 	QString (TR("Failed to delete script %1")).arg(path),
			 	strerror(errno),
			 	__ERRLOCN
			  )	;
		return	false	;
	}

	return	true	;
}

/*  KBPYScriptIF							*/
/*  rename	: Rename script file(s)					*/
/*  location	: KBLocation &	  : Document location			*/
/*  newName	: const QString & : New script name			*/
/*  pError	: KBError &	  : Error return			*/
/*  (returns)	: bool		  : Success				*/

bool	KBPYScriptIF::rename
	(	KBLocation	&location,
		const QString	&newName,
		KBError		&pError
	)
{
	QString	oldPath	= location.dbInfo()->getDBPath() + "/" + location.name() ;
	QString	newPath	= location.dbInfo()->getDBPath() + "/" + newName ;

	if (QFile::exists(oldPath + ".pyc") && (::rename (oldPath + ".pyc", newPath + ".pyc") != 0))
	{
		pError	= KBError
			  (	KBError::Error,
				QString (TR("Failed to rename script code %1.pyc")).arg(oldPath),
				strerror(errno),
				__ERRLOCN
			  )	;
		return	false	;
	}

	if (::rename (oldPath + ".py", newPath + ".py") != 0)
	{
		pError	= KBError
			  (	KBError::Error,
			 	QString (TR("Failed to rename script %1")).arg(oldPath),
			 	strerror(errno),
			 	__ERRLOCN
			  )	;
		return	false	;
	}

	return	true	;
}

/*  KBPYScriptIF							*/
/*  exeError	: Retrieve execution error details			*/
/*  _errMsg	: QString &	: Return error message			*/
/*  _errLno	: uint &	: Return error line number		*/
/*  _errText	: QString &	: Error details text			*/
/*  (returns)	: KBLocation	: Return location			*/

KBLocation
	KBPYScriptIF::lastError
	(	QString	&_errMsg,
		uint	&_errLno,
		QString	&_errText
	)
{
	_errMsg	 = errorMsg  ;
	_errLno	 = errorLno  ;
	_errText = errorText ;

	DPRINTF
	((	"KBPYScriptIF::exeError: errIdent=[%s]\n",
		errorIdent.latin1()
	))	;

	/* If the error identifier is empty then it must be a failure	*/
	/* to locate the error. In this case we have nothing to go on.	*/
	if (errorIdent == "")
		return	KBLocation
			(	0,
				"script",
				KBLocation::m_pInline,
				"[unknown]",
				"py"
			) ;

	/* If the error was in inline code then return just enough to	*/
	/* correspond to this. The original caller should know where	*/
	/* it is.							*/
	if (inlineMap.find (errorIdent) != 0)
		return	KBLocation
			(	0,
				"script",
				KBLocation::m_pInline,
				KBLocation::m_pInline,
				"py"
			) ;

	/* OK, got a real looking identifier. It should exist in the	*/
	/* module map, in which case return the location for the map	*/
	/* entry.							*/
	KBPYModule *module ;
	if ((module = loadMap.find (errorIdent)) != 0)
	{
		XPRINTF
		((	"KBPYScriptIF::exeError -> [%s]\n",
			 module->m_location.ident().latin1()
		))	;
		return	module->m_location ;
	}

	/* Should not get here!						*/
	DPRINTF
	((	"KBPYScriptIF::exeError failed for [%s]\n",
		(cchar *)errorIdent
	))	;
	return	KBLocation () ;
}

/*  KBPyScriptIF:							*/
/*  editorInit	: Set up editor for python 				*/
/*  editor	: TextEdit *	: Editor				*/
/*  (returns)	: void		:					*/

void	KBPYScriptIF::editorInit
	(	KBTextEdit	*editor
	)
{
	TKCPySetupEditor (editor) ;
}

/*  KBPyScriptIF:							*/
/*  showDebug	: Show debugger						*/
/*  action	: TKToggleAction *: Display toggle action		*/
/*  (returns)	: KBDebug *	  : Debugger				*/

KBDebug	*KBPYScriptIF::showDebug
	(	TKToggleAction	*action
	)
{
	bool	ok	;
	KBDebug	*debug	= 0;
	debug = new KBPYDebug (action, ok) ;

	if (!ok)
	{	
		if(debug != 0)
			delete	debug	;

		debug = 0;
		return	0	;
	}

	return	debug	;
}


/*  KBPyScriptIF							*/
/*  debugScript	: Load script into debugger				*/
/*  location	: KBLocation &	: Script location			*/
/*  pError	: KBError &	: Error return				*/
/*  (returns)	: bool		: Success				*/

bool	KBPYScriptIF::debugScript
	(	KBLocation	&location,
		KBError		&pError
	)
{
	if (!debuggerOK)
	{
		pError	= KBError
			  (	KBError::Error,
				TR("Cannot load: debugger was not loaded"),
				QString::null,
				__ERRLOCN
			  )	;
		return	false	;
	}

	TKCPyDebugWidget *debWidget ;
	if ((debWidget = TKCPyDebugWidget::widget()) == 0)
	{
		pError	= KBError
			  (	KBError::Error,
				TR("Cannot load: debugger not open"),
				QString::null,
				__ERRLOCN
			  )	;
		return	false	;
	}

	bool	pyErr	;
	bool	rc	= load	(location, pError, pyErr) ;

	if (rc || pyErr)
	{
		if (pyErr) pError.DISPLAY() ;

		TKCPyRekallCookie module (location) ;
		debWidget->editModule (&module, "") ;
	}

	return	rc	;
}

bool	KBPYScriptIF::isClientSide ()
{
	return	false	;
}

QString	KBPYScriptIF::ident ()
{
	return	QString("Python: %1%2 %3 (Python %4.%5)")
			.arg(__KB_BUILD_RELEASE)
			.arg(__KB_BUILD_BUILD  )
			.arg(__KB_BUILD_TIME   )
			.arg(PY_MAJOR_VERSION  )
			.arg(PY_MINOR_VERSION  ) ;
}

KBFACTORYIMPL
(	KBPYScriptFactory,
	script_py
)

/*  KBPYScriptFactory							*/
/*		: Get an instance of the appropriate interpreter	*/
/*  parent	: QObject *	: Parent object				*/
/*  (returns)	: QObject *	: Interpreter instance			*/

QObject	*KBPYScriptFactory::create
	(	QObject		  *parent,
		cchar		  *,
		cchar		  *,
		const QStringList &
	)
{
	DPRINTF
	((	"KBPYScriptFactory::create: ***CALLLED***: parent=%p\n",
		(void *)parent
	))	;

	loadMap.setAutoDelete    (true) ;
	return	new KBPYScriptIF ()	;
}

QString	KBPYScriptFactory::ident ()
{
	return	QString("Python: %1%2 %3 (Python %4.%5)")
			.arg(__KB_BUILD_RELEASE)
			.arg(__KB_BUILD_BUILD  )
			.arg(__KB_BUILD_TIME   )
			.arg(PY_MAJOR_VERSION  )
			.arg(PY_MINOR_VERSION  ) ;
}

PyObject *PyKBRekallError	;
PyObject *PyKBRekallAbort	;
PyObject *PyKBRekallTest	;
