/***************************************************************************
    file	         : kb_pyscript.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	_KB_PYSCRIPT_H
#define	_KB_PYSCRIPT_H

#include	"kb_python.h"

#include	"kb_classes.h"
#include	"kb_script.h"


/*  KBPYScriptCode							*/
/*  --------------							*/
/*  Python-specific class for precompiled inline code.			*/

class	KBPYScriptCode : public KBScriptCode
{
public	:

	PyObject	*m_pyFunc	;
	KBLocation	m_location	;

	KBPYScriptCode	(PyObject *, PyObject *, KBNode *, KBEvent *, KBLocation &) ;
	virtual	~KBPYScriptCode	() ;

	virtual	KBScript::ExeRC	execute
		(	uint		,
			const KBValue	*,
			KBValue		&
		)	;

	virtual	KBScript::ExeRC	execute
		(	KBNode		*,
			const QString	&,
			uint		,
			const KBValue	*,
			KBValue		&
		)	;
	virtual	void	setBreakpoints
		(	const QValueList<int>	&
		)	;
}	;

/*  KBPYScriptObject							*/
/*  ----------------							*/
/*  Local derived version of KBScriptObject, used to implement object	*/
/*  persistence.							*/

class	KBPYScriptObject : public KBScriptObject
{
	PyObject	*m_pyObject  ;

public	:

	KBPYScriptObject(PyObject *) ;
	virtual	~KBPYScriptObject () ;

	PyObject *object () ;
}	;

/*  KBPYScriptIF							*/
/*  ------------							*/
/*  This is the script interface object for the Python script module.	*/

class	KBPYScriptIF : public KBScriptIF
{
	Q_OBJECT

	uint		m_languageCode		;

	bool		importModule
			(	PyObject		*,
				const QString		&,
				KBError			&
			)	;

	PyObject	*compileText
			(	KBLocation		&,
				const QString		&,
				KBError			&
			) 	;

	KBScriptCode   *compileInline
			(	KBNode			*,
				const QString		&,
				const QString		&,
				const QString		&,
				const QStringList	&,
				KBEvent			*,
				KBError			&
			)	;

	PyObject	*findFunction
			(	const QStringList	&,
				const QString		&
			)	;

protected slots :

	void	optionsChanged	() ;

public	:

	KBPYScriptIF () ;
	virtual	~KBPYScriptIF () ;

	bool	load
		(	KBLocation		&,
			KBError			&,
			bool			&
		)	;

	virtual	void		setApp  (KBNotifier *, KBCallback *) ;

	virtual	bool	compile
		(	KBLocation		&,
			KBError			&
		)	;

	virtual	bool	load
		(	KBLocation		&,
			KBError			&
		)	;

	virtual	KBScriptCode   *compileFunc
		(	KBNode			*,
			const QString		&,
			const QString		&,
			const QString		&,
			const QStringList	&,
			KBEvent			*,
			KBError			&
		)	;

	virtual	KBScriptCode   *compileExpr
		(	KBNode			*,
			const QString		&,
			const QString		&,
			const QStringList	&,
			KBError			&
		)	;

	virtual	KBScript::ExeRC	execute
		(	const QStringList	&,
			const QString		&,
			KBNode			*,
			uint,
			const KBValue		*,
			KBValue 		&
		)	;


	virtual	KBLocation	lastError	(QString &, uint &, QString &) ;

	virtual	bool		unlink		(KBLocation     &, KBError &) ;
	virtual	bool		rename		(KBLocation     &, const QString &, KBError &) ;

	virtual	void		editorInit	(KBTextEdit     *) ;
	virtual	KBDebug		*showDebug	(TKToggleAction *) ;
	virtual	bool		debugScript	(KBLocation   &, KBError 	   &) ;  

	virtual	QString		ident		() ;
	virtual	bool		isClientSide	() ;

	inline	uint		languageCode	()
	{
		return		m_languageCode	;
	}

	static	KBPYScriptIF 	*getIface	() ;
}	;


#define	__C(x)	((char *)(cchar *)x)
#define	MD(t)	{	__C(#t),	_rekall_##t,		METH_VARARGS,	0	},
#define	MN(t)	{	__C("new_"#t),	_rekall_new_##t,	0,		0	},
#define	ME	{	0,		0,			0,		0	}

#endif	// _KB_PYSCRIPT_H
