/***************************************************************************
    file	         : kb_pysqlinsert.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_dblink.h"

#include	"kb_pybase.h"
#include	"kb_pyvalue.h"



/*  PyKBSQLInsert_execute						*/
/*		: Executew query					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBSQLInsert_execute
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *alist  = 0 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBSQLInsert.execute",
					PyKBBase::m_sql,
					args,
					"OO",
					&alist
				) ;
	if (pyBase == 0) return 0 ;

	extern	bool	getQueryArguments (PyObject *, uint &, KBValue *&) ;
	uint	nvals	;
	KBValue	*values	;
	if (!getQueryArguments (alist, nvals, values))
		return	0 ;

	bool	rc	;

	//Py_BEGIN_ALLOW_THREADS
	rc	= ((KBSQLInsert *)pyBase->m_kbObject)->execute (nvals, values) ;
	//Py_END_ALLOW_THREADS

	if (values != 0) delete	[] values ;
	return	PyInt_FromLong (rc) ;
}

/*  PyKBSQLInsert_getNewKey						*/
/*		: Get newly inserted key				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBSQLInsert_getNewKey
	(	PyObject	*,
		PyObject	*args
	)
{
	cchar	 *kc	 = 0 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBSQLInsert.getNewKey",
					PyKBBase::m_sql,
					args,
					"O|s",
					&kc
				) ;
	if (pyBase == 0) return 0 ;

	QString	keyCol	;
	KBValue	newKey	;
	bool	rc	;

	if (kc != 0) keyCol = kc ;

	//Py_BEGIN_ALLOW_THREADS
	rc = ((KBSQLInsert *)pyBase->m_kbObject)->getNewKey (keyCol, newKey, false) ;
	//Py_END_ALLOW_THREADS

	if (!rc)
	{	PyErr_SetString
		(	PyExc_TypeError,
			((KBSQLInsert *)pyBase->m_kbObject)->lastError().getMessage() +
			": " +
			((KBSQLInsert *)pyBase->m_kbObject)->lastError().getDetails()
		)	;
		return	0 ;
	}

	return	PyKBBase::fromKBValue (newKey) ;
}

/*  PyKBSQLInsert_getNumRows						*/
/*		: Get number of rows returned				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBSQLInsert_getNumRows
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBSQLInsert.getNumRows",
					PyKBBase::m_sql,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	int	nRows	;

	//Py_BEGIN_ALLOW_THREADS
	nRows	= ((KBSQLInsert *)pyBase->m_kbObject)->getNumRows() ;
	//Py_END_ALLOW_THREADS

	return	PyLong_FromLong (nRows) ;
}

/*  PyKBSQLInsert_lastError						*/
/*		: Get last query error					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBSQLInsert_lastError
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBSQLInsert.lastError",
					PyKBBase::m_sql,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	QString	error	;

	//Py_BEGIN_ALLOW_THREADS
	error	= PyKBBase::decodeError (((KBSQLInsert *)pyBase->m_kbObject)->lastError()) ;
	//Py_END_ALLOW_THREADS

	return	kb_qStringToPyString (error) ;
}

/*  PyKBSQLInsert___repr__						*/
/*		: Get __repr__						*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Element type				*/

static	PyObject*PyKBSQLInsert___repr__
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBSQLInsert.__repr__",
					PyKBBase::m_sql,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;
	KBSQLInsert *obj  = (KBSQLInsert *)pyBase->m_kbObject ;

	QString	repr	= QString ("[SQLInsert @ 0x%1]")
				.arg((ulong)(void *)obj, 0, 16)
				;

	return	kb_qStringToPyString (repr) ;
}

/*  ------------------------------------------------------------------	*/

static	PyMethodDef PyKBSQLInsert_methods[] =
{
	PY_METHOD	(execute,	PyKBSQLInsert	),
	PY_METHOD	(getNumRows,	PyKBSQLInsert	),
	PY_METHOD	(getNewKey,	PyKBSQLInsert	),
	PY_METHOD	(lastError,	PyKBSQLInsert	),
	PY_METHOD	(__repr__,	PyKBSQLInsert	),
	PY_METHOD_END
}	;

static	const char *PyKBSQLInsert_names  [] =
{
	"KBSQLInsert",
	0
}	;

void	initPyKBSQLInsert
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBSQLInsert",
		module,
		PyKBSQLInsert_methods,
		"KBBase",
		PyKBSQLInsert_names
	)	;
}
