/***************************************************************************
    file	         : kb_pysqlselect.cpp
    copyright            : (C) 1999,2000,2001,2002,2003,2004 by Mike Richardson
			   (C) 2001,2002,2003,2004 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_dblink.h"

#include	"kb_pybase.h"
#include	"kb_pyvalue.h"


/*  PyKBSQLSelect_execute						*/
/*		: Executew query					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBSQLSelect_execute
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject *alist  = 0 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBSQLSelect.execute",
					PyKBBase::m_sql,
					args,
					"O|O",
					&alist
				) ;
	if (pyBase == 0) return 0 ;

	extern	bool	getQueryArguments (PyObject *, uint &, KBValue *&) ;
	uint	nvals	;
	KBValue	*values	;
	if (!getQueryArguments (alist, nvals, values))
		return	0 ;

	bool	rc	;

	//Py_BEGIN_ALLOW_THREADS
	rc	= ((KBSQLSelect *)pyBase->m_kbObject)->execute (nvals, values) ;
	//Py_END_ALLOW_THREADS

//	fprintf
//	(	stderr,
//		"PyKBSQLSelect_execute: rc=%d [%s]\n",
//		rc,
//		(cchar *)((KBSQLSelect *)pyBase->m_kbObject)->getRawQuery()
//	)	;

	if (values != 0) delete	[] values ;
	return	PyInt_FromLong (rc) ;
}

/*  PyKBSQLSelect_getNumRows						*/
/*		: Get number of rows returned				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBSQLSelect_getNumRows
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBSQLSelect.getNumRows",
					PyKBBase::m_sql,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

//	int	nRows	;
//
//	Py_BEGIN_ALLOW_THREADS
//	nRows	= ((KBSQLSelect *)pyBase->m_kbObject)->getNumRows() ;
//	Py_END_ALLOW_THREADS
//
//	return	PyLong_FromLong (nRows) ;

	return	PyLong_FromLong (0) ;
}

/*  PyKBSQLSelect_getNumFields						*/
/*		: Get number of fields returned				*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBSQLSelect_getNumFields
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBSQLSelect.getNumFields",
					PyKBBase::m_sql,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	int	nFields	;

	//Py_BEGIN_ALLOW_THREADS
	nFields	= ((KBSQLSelect *)pyBase->m_kbObject)->getNumFields() ;
	//Py_END_ALLOW_THREADS

	return	PyLong_FromLong (nFields) ;
}

/*  PyKBSQLSelect_rowExists						*/
/*		: Test if row exists					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBSQLSelect_rowExists
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBSQLSelect.rowExists",
					PyKBBase::m_sql,
					args,
					"Oi",
					&row
				) ;
	if (pyBase == 0) return 0 ;

	bool	rc	;

	//Py_BEGIN_ALLOW_THREADS
	rc	= ((KBSQLSelect *)pyBase->m_kbObject)->rowExists (row) ;
	//Py_END_ALLOW_THREADS

//	fprintf
//	(	stderr,
//		"PyKBSQLSelect_rowExists: row %d -> %d\n",
//		row,
//		rc
//	)	;

	return	PyLong_FromLong (rc) ;
}

/*  PyKBSQLSelect_getField						*/
/*		: Get field value					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBSQLSelect_getField
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 row	 ;
	int	 col	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBSQLSelect.getField",
					PyKBBase::m_sql,
					args,
					"Oii",
					&row,
					&col
				) ;
	if (pyBase == 0) return 0 ;

	KBValue	value	;

	//Py_BEGIN_ALLOW_THREADS
	value	= ((KBSQLSelect *)pyBase->m_kbObject)->getField(row, col) ;
	//Py_END_ALLOW_THREADS

//	fprintf
//	(	stderr,
//		"PyKBSQLSelect_getField: %d,%d -> [%s]\n",
//		row,
//		col,
//		(cchar *)value.getRawText()
//	)	;

	//xxxxxxx check binary data / text encoding
	
	return	PyKBBase::fromKBValue (value) ;
}

/*  PyKBSQLSelect_getFieldName						*/
/*		: Get field name					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBSQLSelect_getFieldName
	(	PyObject	*,
		PyObject	*args
	)
{
	int	 col	 ;
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBSQLSelect.getFieldName",
					PyKBBase::m_sql,
					args,
					"Oi",
					&col
				) ;
	if (pyBase == 0) return 0 ;

	QString	name	;

	//Py_BEGIN_ALLOW_THREADS
	name	= ((KBSQLSelect *)pyBase->m_kbObject)->getFieldName (col) ;
	//Py_END_ALLOW_THREADS

	return	kb_qStringToPyString (name) ;
}

/*  PyKBSQLSelect_lastError						*/
/*		: Get last query error					*/
/* 		: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: Item on				*/

static	PyObject*PyKBSQLSelect_lastError
	(	PyObject	*,
		PyObject	*args
	)
{
	PyKBBase *pyBase = PyKBBase::parseTuple
				(	"KBSQLSelect.lastError",
					PyKBBase::m_sql,
					args,
					"O"
				) ;
	if (pyBase == 0) return 0 ;

	QString	error	;

	//Py_BEGIN_ALLOW_THREADS
	error	= PyKBBase::decodeError (((KBSQLSelect *)pyBase->m_kbObject)->lastError()) ;
	//Py_END_ALLOW_THREADS

	return	kb_qStringToPyString (error) ;
}

/*  ------------------------------------------------------------------	*/

static	PyMethodDef PyKBSQLSelect_methods[] =
{
	PY_METHOD	(execute,	PyKBSQLSelect	),
	PY_METHOD	(getNumRows,	PyKBSQLSelect	),
	PY_METHOD	(getNumFields,	PyKBSQLSelect	),
	PY_METHOD	(rowExists,	PyKBSQLSelect	),
	PY_METHOD	(getField,	PyKBSQLSelect	),
	PY_METHOD	(getFieldName,	PyKBSQLSelect	),
	PY_METHOD	(lastError,	PyKBSQLSelect	),
	PY_METHOD_END
}	;

static	const char *PyKBSQLSelect_names  [] =
{
	"KBSQLSelect",
	0
}	;

void	initPyKBSQLSelect
	(	PyObject	*module
	)
{
	PyKBBase::makePythonClass
	(	"PyKBSQLSelect",
		module,
		PyKBSQLSelect_methods,
		"KBBase",
		PyKBSQLSelect_names
	)	;
}
