/***************************************************************************
    file	         : kb_pysupport.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<Python.h>

#include	<qstring.h>
#include	<qcstring.h>
#include	<qdict.h>
#include	<qasciidict.h>
#include	<qstringlist.h>

#include	"kb_classes.h"
#include	"kb_python.h"
#include	"kb_pyvalue.h"


#define		__DEBUG_PYSUPPORT	(0)
#if		__DEBUG_PYSUPPORT
#include	"kb_dprintf.h"
#endif


bool	pyDictToQtDict
	(	PyObject	*pyDict,
		QDict<QString>	&kbDict
	)
{
	PyObject	*key	;
	PyObject	*value	;
	int		pos	= 0 ;

	kbDict.setAutoDelete (true) ;

	if (pyDict != 0)
		while (PyDict_Next (pyDict, (Py_ssize_t *)&pos, &key, &value))
		{
			bool	error	;
			QString qKey	= kb_pyStringToQString (key,   error) ;
			if (error) return false ;

			QString qValue	= kb_pyStringToQString (value, error) ;
			if (error) return false ;

			kbDict.insert	(qKey, new QString(qValue)) ;
		}

	return	true ;
}

PyObject
	*qtDictToPyDict
	(	const QDict<QString>	&kbDict
	)
{
	PyObject	*pyDict	 = PyDict_New () ;
	PyObject	*pyKey	 = 0 ;
	PyObject	*pyValue = 0 ;
	if (pyDict == 0) return	0 ;


	QDictIterator<QString>	iter	(kbDict) ;
	QString			*res	;
	while ((res = iter.current()) != 0)
	{
		QString	v = *res ;
		if (v.isNull()) v = ""	;

		if ((pyKey   = kb_qStringToPyString (iter.currentKey())) == 0)
			goto	error	;

		if ((pyValue = kb_qStringToPyString (v)) == 0)
			goto	error	;

		if (PyDict_SetItem (pyDict, pyKey, pyValue) < 0)
			goto	error	;

		pyKey	 = 0 ;
		pyValue	 = 0 ;
		iter	+= 1 ;
	}

	return	pyDict	;

	error	:
		Py_DECREF (pyDict) ;
		if (pyKey   != 0) Py_DECREF(pyKey  ) ;
		if (pyValue != 0) Py_DECREF(pyValue) ;
		return	0 ;
}

PyObject
	*qtDictToPyDict
	(	const QAsciiDict<QCString>	&kbDict
	)
{
	PyObject	*pyDict	 = PyDict_New () ;
	PyObject	*pyKey	 = 0 ;
	PyObject	*pyValue = 0 ;

	DPRINTF
	((	"qtDictToPyDict[ascii]: pyDict=%p\n",
		(void *)pyDict
	))	;

	if (pyDict == 0) return	0 ;

	QAsciiDictIterator<QCString> iter (kbDict) ;
	QCString *res	;
	while ((res = iter.current()) != 0)
	{
		QCString	v = *res ;
		if (v.isNull()) v = ""	 ;

		DPRINTF
		((	"qtDictToPyDict[ascii]: [%s]->[%s]\n",
			iter.currentKey(),
			v.data()
		))	;

		if ((pyKey   = PyString_FromString (iter.currentKey())) == 0)
			goto	error	;

		if ((pyValue = PyString_FromString (v)) == 0)
			goto	error	;

		if (PyDict_SetItem (pyDict, pyKey, pyValue) < 0)
			goto	error	;

		pyKey	 = 0 ;
		pyValue	 = 0 ;
		iter	+= 1 ;
	}

	return	pyDict	;

	error	:
		Py_DECREF (pyDict) ;
		if (pyKey   != 0) Py_DECREF(pyKey  ) ;
		if (pyValue != 0) Py_DECREF(pyValue) ;
		return	0 ;
}

PyObject
	*qtStringListToPyList
	(	const QStringList	&slist,
		int			from
	)
{
	PyObject	*pyList	 = PyList_New (slist.count() - from) ;
	if (pyList == 0) return	0 ;

	for (uint idx = from ; idx < slist.count() ; idx += 1)
	{
		PyObject *item = kb_qStringToPyString(slist[idx]);
		if (item == 0)
		{
			Py_DECREF (pyList) ;
			return	  0	   ;
		}
		PyList_SET_ITEM (pyList, idx - 1, item) ;
	}

	return	  pyList  ;
}

