/***************************************************************************
    file	         : tkc_pydebugwidget.h
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__TKC_PYDEBUGWIDGET_H
#define	__TKC_PYDEBUGWIDGET_H


#include	<qsplitter.h>
#include	<qlayout.h>
#include	<qlistview.h>
#include	<qpushbutton.h>
#include	<qlabel.h>
#include	<qdialog.h>
#include	<qpopupmenu.h>
#include	<qwidgetlist.h>
#include	<qregexp.h>
#include	<qtabwidget.h>

#include	"kb_python.h"


#include	"tkc_labeller.h"
#include	"tkc_pyvalue.h"
#include	"tkc_pyvaluelist.h"
#include	"tkc_pyfunclist.h"
#include	"tkc_pytracelist.h"
#include	"tkc_pystacklist.h"

#include	"kb_classes.h"
#include	"kb_gui.h"
#include	"kb_partwidget.h"
#include	"kb_part.h"



class	TKCPyEditor		;
class	TKCPyDebugDialog	;
class 	QGridLayout		;
class	RKTabWidget		;
class	RKListBox		;
class	QTextBrowser		;

/*  TKCPyDebugWidget							*/
/*  ----------------							*/
/*  This is the widget that provides the GUI interface to the python	*/
/*  debugger. Normally, it is non-modal, however inside a breakpoint it	*/
/*  is embedded into a QDialog and becomes effectively modal		*/

class	TKCPyDebugWidget : public QWidget
{
	Q_OBJECT

	QWidget			*parent			;
	KBSDIMainWindow		*window			;

	QLabel			*m_pReason		;

	QSplitter		*m_pSplitMain		;
	QSplitter		*m_pSplitRight		;
	QWidget			*m_pRightSide		;
	RKTabWidget   		*m_pDockTabGroup	;

	TKCPyValueList		*m_pObjectView		;
	TKCPyFuncList		*m_pFuncView		;
	TKCPyValueList		*m_pTraceView		;
	TKCPyValueList		*m_pStackView		;

	RKTabWidget		*m_pEditors		;
	QTextBrowser		*m_pErrWin		;

	QVBoxLayout		*m_pLayMain		;
	QVBoxLayout		*m_pLayRight		;
	TKCPyDebugDialog	*inDialog		;

	QPtrList<TKCPyEditor>	editList		;

	TKCPyValueItem		*popItem		;
	TKCPyTraceItem		*traceItem		;

	bool			inModalLoop		;
	TKCPyTraceOpt		modalEndRC		;
	uint			aborting		;
	bool			m_excTrap		;
	bool			m_excTrapSaved		;

	QRegExp			errExp			;

	TKCPyEditor		*showObjectCode	   (PyObject *) ;
	TKCPyCookie		*getObjectModule   (PyObject *, uint &) ;

	void			setTraceMessage	   (const QString &) ;
	void			loadErrorText	   (const QString &) ;
	void			showEvent	   (QShowEvent	  *) ;
	void			showTrace	   (PyFrameObject *, const QString &, const QString &) ;
	PyObject		*getCode	   (PyObject      *) ;
	void			addBreakOptions	   (QPopupMenu &, PyObject *) ;

	TKCPyTraceOpt		showAsDialog	   (bool) ;

	static	TKCPyDebugWidget	*debWidget ;

signals :

	void	showingFile		(bool)	;
	void	fileChanged		(bool)	;
	void	enterTrap		(bool, bool, bool) ;
	void	exitTrap		()	;

private slots :

	void	showSource		()	;

	void	setBreakpoint		()	;
	void	setWatchpoint		()	;
	void	enableBreakpoint	()	;
	void	disableBreakpoint	()	;
	void	removeWatchpoint	()	;

	void	showContextMenu		(int, QListViewItem *, const QPoint &, int) ;


	void	editorChanged		(QWidget *) ;
	void	moduleChanged		()	;

public	:

	TKCPyDebugWidget (QWidget *, KBSDIMainWindow *) ;
       ~TKCPyDebugWidget () ;

	TKCPyTraceOpt	doFuncTrace 	(PyObject *, PyObject *, PyObject *, void *) ;
	TKCPyTraceOpt	doLineTrace	(PyObject *, PyObject *, PyObject *, void *) ;
	TKCPyTraceOpt	doProfTrace	(PyObject *, PyObject *, PyObject *, void *) ;
	TKCPyTraceOpt	doDebugHook	(PY_FRAME_TYPE *, const char *) ;

	void		dropSource	(TKCPyCookie *) ;
	void		init		(TKConfig *) ;
	void		save		(TKConfig *) ;
	void		codeLoaded	() ;
	void		clearBreakpoints(TKCPyCookie *) ;
	void		toggleBreakpoint(TKCPyCookie *, uint, TKCPyEditor *) ;
	void		toggleBreakpoint()	;

	void		pushExcTrap	()	;
	void		popExcTrap	()	;

	void		breakAction	(int)	;
	void		setExcSkipList	()	;
	bool		saveModule	()	;
	void		doCompile	()	;
	void		closeModule	()	;

	TKCPyEditor	*editModule	(TKCPyCookie *, const QString     &) ;

	void		trapExceptions	(bool)	;


	static	TKCPyDebugWidget	*widget () ;
	static	void	doPushExcTrap	()	;
	static	void	doPopExcTrap	()	;
}	;

#endif	// __TKC_PYDEBUGWIDGET_H
