/***************************************************************************
    file	         : tkc_pyeditor.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"tkc_pydebugbase.h"
#include	"tkc_pydebugwidget.h"

#if		__KB_AUTOMOC
#include	"tkc_pyeditor.moc"
#else
#include 	"tkc_pyeditor.h"
#endif

#include	"kb_options.h"
#include	"kb_font.h"
#include	"kb_dialog.h"

#include	"kb_locator.h"



/*  TKCPyEditor								*/
/*  TKCPyEditor	: Constructor for debug/editor widget			*/
/*  parent	: QWidget *	     : Parent widget			*/
/*  debug	: TKCPyDebugWidget * : Parent debugger			*/
/*  module	: TKCPyCookie *	     : Module				*/
/*  (returns)	: TKCPyDebugWidget   :					*/

TKCPyEditor::TKCPyEditor
	(	QWidget		 	*parent,
		TKCPyDebugWidget 	*debug,
		TKCPyCookie		*module
	)
	:
	KBTextEdit	(parent),
	debug		(debug),
	module		(module->replicate())
{
	fprintf
	(
		stderr,
		"TKCPyEditor::TKCPyEditor: font [%s]\n",
		(const char *)KBOptions::getScriptFont()
	)	;

	setWordWrap     (QTextEdit::NoWrap) ;
	setFont		(KBFont::specToFont(KBOptions::getScriptFont())) ;
	setHighlight	("py")	   ;
	showLineNumbers	(true)	   ;
	showMarkers	(true, 2)  ;

	connect
	(	this,
		SIGNAL(clickMarkers	(QEvent *, int)),
		SLOT  (slotClickMarkers (QEvent *, int))
	)	;
	curLineNo = 0 ;
}

/*  TKCPyEditor								*/
/*  TKCPyEditor	: Destructor for debug/editor widget			*/
/*  (returns)	:   		:					*/

TKCPyEditor::~TKCPyEditor ()
{
	/* Clear all line breaks associated with the module and delete	*/
	/* the cookie copy which we have.				*/
	debug->clearBreakpoints (module) ;

	if(module != 0)
		delete	module	;

	module = 0;
}

/*  TKCPyEditor								*/
/*  setCurrentLine: Set current line marker				*/
/*  lineNo	  : uint	: Line number				*/
/*  (returns)	  : void	:					*/

void	TKCPyEditor::setCurrentLine
	(	uint	lineNo
	)
{
	fprintf
	(	stderr,
		"TKCPyEditor::setCurrentLine (%u) (was %d)\n",
		lineNo,
		curLineNo
	)	;

	if (curLineNo != 0)
	{
		setMark (curLineNo - 1, getMark(curLineNo - 1) & ~0x2) ;
		curLineNo = 0 ;
	}


	if (lineNo != 0)
	{	
		setMark	(lineNo - 1, getMark(lineNo - 1) | 0x2 ) ;
		setCursorPosition (lineNo - 1, 0) ;

		curLineNo = lineNo ;
	}
}

/*  TKCPyEditor								*/
/*  getCurrentLine: Get current line marker				*/
/*  (returns)	  : uint	: Line number				*/

uint	TKCPyEditor::getCurrentLine ()
{
	return	currentPara () + 1 ;
}

/*  TKCPyEditor								*/
/*  setBreakpoint: Set a breakpoint marker				*/
/*  lineNo	 : uint		: Line number				*/
/*  (returns)	 : void		:					*/

void	TKCPyEditor::setBreakpoint
	(	uint	lineNo
	)
{
	setMark (lineNo - 1, getMark(lineNo - 1) | 0x1) ;

	int	idx	;
	if ((idx = bptList.findIndex (lineNo)) < 0) bptList.append (lineNo) ;
}

/*  TKCPyEditor								*/
/*  clearBreakpoint							*/
/*		: Set a breakpoint marker				*/
/*  lineNo	: uint		: Line number				*/
/*  (returns)	: void		:					*/

void	TKCPyEditor::clearBreakpoint
	(	uint	lineNo
	)
{
	setMark (lineNo - 1, getMark(lineNo - 1) & ~0x1) ;

	bptList.remove (lineNo) ;
}


/*  TKCPyEditor								*/
/*  showText	: Show specified text					*/
/*  _content	: const QString & : Module text				*/
/*  (returns)	: void		  :					*/

void	TKCPyEditor::showText
	(	const QString	&content
	)
{
	setText	    (content) ;
	setModified (false  ) ;

	QValueList<int>::Iterator iter ;
	for (iter = bptList.begin() ; iter != bptList.end() ; ++iter)
		setMark(*iter - 1, getMark (*iter - 1) | 0x1) ;
}

/*  TKCPyEditor								*/
/*  save	: Save document						*/
/*  eText	: QString &	: Error text				*/
/*  eDetails	: QString &	: Error details				*/
/*  (return)	: bool		: Success				*/

bool	TKCPyEditor::save
	(	QString		&eText,
		QString		&eDetails
	)
{
	return	module->put (text(), eText, eDetails) ;
}

/*  TKCPyEditor								*/
/*  slotClickMarkers							*/
/*		: Handle mouse click in markers				*/
/*  e		: QEvent *	: Actual event				*/
/*  lno		: int		: Line number				*/
/*  		: ButtonState	: Shift/ctrl/alt state			*/
/*  (returns)	: void		:					*/

void	TKCPyEditor::slotClickMarkers
	(	QEvent		*,
		int
	)
{
}

/*  TKCPyEditor								*/
/*  gotoLine	: Move cursor to specified line				*/
/*  lno		: uint		: Line number				*/
/*  (returns)	: void		:					*/

void	TKCPyEditor::gotoLine
	(	uint	lno
	)
{
	setCursorPosition (lno, 0) ;
	KBTextEdit::setFocus() ;
}
