/***************************************************************************
    file	         : testdriver.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qmap.h>
#include	<qstring.h>
#include	<qdict.h>
#include	<qintdict.h>
#include	<qregexp.h>
#include	<qdir.h>
#include	<qdatetime.h>

#include	<stdarg.h>
#include	<cstdlib>

#include	"std.h"
#include	"eli.h"
#include	"interp.h"
#include	"syn.h"
#include	"code.h"

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_databuffer.h"
#include	"kb_database.h"
#include	"kb_serverinfo.h"
#include	"kb_libloader.h"
#include	"kb_locator.h"
#include	"kb_dom.h"
#include	"kb_basequery.h"

#define		__DEBUG_TESTDRIVER	(0)
#if		__DEBUG_TESTDRIVER
#include	"kb_dprintf.h"
#endif

struct	TestDriverTypeMap
{	int		ident		;	/* TestDriver identifier	*/
	KB::IType	itype		;	/* Internal type	*/
	char		mtype	[16]	;	/* TestDriver type name	*/
	uint		flags		;	/* Various flags	*/
}	;



#define	FIELD_TYPE_UNSET	(-1)

static	TestDriverTypeMap	typeMap[] =
{
/*	ident			itype		mtype		flags		*/
{	0,			KB::ITFixed,	"Integer",	0			},
{	1,			KB::ITFloat,	"Float",	FF_LENGTH|FF_PREC	},

{	2,			KB::ITDate,	"Date",		0			},
{	3,		 	KB::ITTime,	"Time",		0			},
{	4,		 	KB::ITDateTime,	"DateTime",	0			},

{	5,			KB::ITString,	"Char",		FF_LENGTH		},
{	6,			KB::ITString,	"VarChar",	FF_LENGTH		},

{	7,			KB::ITBinary,	"Binary",	0			}
}	;




/*  KBTestDriverType							*/
/*  ----------------							*/

class	KBTestDriverType : public KBType
{
	TestDriverTypeMap	*m_typeInfo	;

public	:

	KBTestDriverType  (TestDriverTypeMap *, uint, uint, bool) ;

	virtual	bool    isValid      (const QString &, KBError  &, const QString & = QString::null) ;
	virtual	void	getQueryText (KBDataArray   *, KBShared *, KBDataBuffer  &) ;
}	;


#include	"testdriver.h"


/*  KBTestDriverQrySelect						*/
/*  ---------------------						*/
/*  Implementation class for select queries on the TestDriver database	*/

class	KBTestDriverQrySelect : public KBSQLSelect
{
	KBTestDriver		*m_server	;
	QString			m_query		;
	QString			m_table		;
	QStringList		m_fetch		;
	QValueList<QDomElement>	m_rows		;

	QString			m_exprSkel	;
	QStringList		m_columns	;
	QValueList<int>		m_colTypes	;

	uint			m_crow		;
	ulong			*m_lengths	;

public	:
	KBTestDriverQrySelect (KBTestDriver *, bool, const QString &) ;
virtual~KBTestDriverQrySelect () ;

	virtual	bool	execute		(uint, const KBValue * ) ;
	virtual	KBValue	getField	(uint, uint, KBValue::VTrans) ;
	virtual	QString	getFieldName	(uint) ;
}	;


/*  KBTestDriverQryUpdate						*/
/*  ---------------------						*/
/*  Implementation class for update queries on the TestDriver database	*/

class	KBTestDriverQryUpdate : public KBSQLUpdate
{
	KBTestDriver		*m_server	;
	QString			m_query		;
	QString			m_table		;

public	:
	KBTestDriverQryUpdate (KBTestDriver *, bool, const QString &, const QString &) ;
virtual~KBTestDriverQryUpdate () ;

	virtual	bool	execute	 (uint, const KBValue * ) ;

}	;

/*  KBTestDriverQryInsert						*/
/*  ---------------------						*/
/*  Implementation class for insert queries on the TestDriver database	*/

class	KBTestDriverQryInsert : public KBSQLInsert
{
	KBTestDriver		*m_server	;
	QString			m_query		;
	QString			m_table		;
	QString			m_autocol	;
	KBValue			m_newKey	;

public	:
	KBTestDriverQryInsert (KBTestDriver *, bool, const QString &, const QString &) ;
virtual~KBTestDriverQryInsert () ;

	virtual	bool	execute	  (uint, const KBValue *) ;
	virtual	bool	getNewKey (const QString &, KBValue &, bool) ;
}	;

/*  KBTestDriverQryDelete						*/
/*  ---------------------						*/
/*  Implementation class for delete queries on the TestDriver database	*/

class	KBTestDriverQryDelete : public KBSQLDelete
{
	KBTestDriver		*m_server	;
	QString			m_query		;
	QString			m_table		;

public	:
	KBTestDriverQryDelete (KBTestDriver *, bool, const QString &, const QString &) ;
virtual~KBTestDriverQryDelete () ;

	virtual	bool	execute	 (uint, const KBValue * ) ;
}	;


static	QIntDict<TestDriverTypeMap>	dIdentToType	;
static	QDict	<KBTestDriverType >	typesDict	;

/*  ------------------------------------------------------------------  */

static	bool	evaluate
	(	const QString		&table,
		const QString		&exprSkel,
		const QString		&fetch,
		const QStringList	&columns,
		const QValueList<int>	&colTypes,
		const QDomElement	&record,
		KBValue			&result
	)
{
	QString	expr	= QString(exprSkel).arg(fetch)  ;
	expr		= expr.replace(table + ".", "") ;
	expr		= expr.replace("=", "==")	;
	expr		= expr.replace("'", "\"")	;

	DPRINTF
	((	"evaluate:\n%s\n",
		expr.latin1()
	))	;


	CBUFF	   *cbuff  = el_compile (0, 0, 0, expr.latin1(), 0) ;
	if (cbuff == 0)
	{
		DPRINTF
		((	"KBTestDriverQrySelect::getField: compile error\n"
		))	;
		return	false	;
	}

	if (!el_loadtext (cbuff))
	{	DPRINTF
		((	"KBTestDriverQrySelect::getField: load error\n"
                ))	;
		free	(cbuff)	;
		return	false	;
	}
	free	(cbuff)	;

	VALUE	    *args  = new VALUE[columns.count()] ;
	for (uint idx = 0 ; idx < columns.count() ; idx += 1)
	{
		QString v = record.attribute(columns[idx]) ;

		switch (colTypes[idx])
		{
			case KB::ITFixed  :
				args[idx] = v.toInt   () ;
				break	;
			case KB::ITFloat  :
				args[idx] = v.toDouble() ;
				break	;
			case KB::ITString :
				args[idx] = new STRING(v.latin1()) ;
				break	;
			default		:
				args[idx] = new STRING(v.latin1()) ;
				break	;
		}
	}

        VALUE   res     = el_vexec ("", "f", columns.count(), args) ;
	delete	[] args	;

	QString	value	;
	KBType	*type	;

	switch	(res.tag->tag)
	{
		case V_NUM	:
			value	= QString::number (res.val.num) ;
			type	= &_kbFixed	;
			break	;

		case V_DBL	:
			value	= QString::number (res.val.dbl) ;
			type	= &_kbFloat	;
			break	;

		case V_STR	:
			value	= res.val.str->text ;
			type	= &_kbString	;
			break	;

		default		:
			DPRINTF
			((	"KBTestDriverQrySelect::getField: unexpected tag [%d]\n",
				res.tag->tag
			))	;
			value	= QString::null ;
			type	= &_kbString	;
			break	;
	}

	DPRINTF
	((	"KBTestDriverQrySelect::getField: got [%s]\n",
		value.latin1()
	))	;

	result	= value.isNull() ? KBValue(type) : KBValue(value, type) ;
	return	true	;
}

/*  ------------------------------------------------------------------  */

/*  KBTestDriverType							*/
/*  KBTestDriverType							*/
/*		: Constructor for TestDriver type object		*/
/*  typeInfo	: TestDriverTypeMap * : Type information		*/
/*  length	: uint		 : Underlying database length		*/
/*  prec	: uint		 : Underlying database precision	*/
/*  nullOK	: bool		 : True if null is OK			*/
/*  (returns)	: KBTestDriverType	 :				*/

KBTestDriverType::KBTestDriverType
	(	TestDriverTypeMap	*typeInfo,
		uint		  	length,
		uint			prec,
		bool	  		nullOK
	)
	:
	KBType
	(	"Test",
		typeInfo == 0 ? KB::ITUnknown : typeInfo->itype,
		length,
		prec,
		nullOK
	),
	m_typeInfo	(typeInfo)
{
}

/*  KBTestDriverType							*/
/*  isValid	: Test if value is valid for type			*/
/*  value	: const QString & : Value to check			*/
/*  pError	: KBValue &	  : Error return			*/
/*  where	: const QString & : Caller				*/
/*  (returns)	: bool		  : Valid				*/

bool	KBTestDriverType::isValid
	(	const QString	 &value,
		KBError		 &pError,
		const QString	&where
	)
{
	/* *** Specific type checking					*/
	return	KBType::isValid (value, pError, where) ;
}

/*  KBTestDriverType							*/
/*  getQueryText							*/
/*  value	: KBDataArray  * : Raw text of value to convert		*/
/*  d		: KBShared     * : Decoded representation		*/
/*  buffer	: KBDataBuffer & : Results buffer			*/
/*  (returns)	: void		 :					*/

void	KBTestDriverType::getQueryText
	(	KBDataArray	*value,
		KBShared	*d,
		KBDataBuffer	&buffer
	)
{
	/* **** Stuff like binary escaping				*/
	KBType::getQueryText (value, d, buffer) ;
}





/*  ------------------------------------------------------------------  */

/*  KBTestDriver							*/
/*  KBTestDriver: Constructor for TestDriver database connection class	*/
/*  (returns)	: KBServer	:					*/

KBTestDriver::KBTestDriver ()
	:
	KBServer ()
{
	m_mapExpressions	= false	;
	m_lastArgc		= 0	;
	m_lastArgv		= 0	;
	m_callback		= 0	;
}

KBTestDriver::KBTestDriver
	(	const QMap<QString,QString>	&options
	)
	:
	KBServer ()
{
	m_mapExpressions	= options["mapExpressions"].toInt() ;
	m_tabPrefix		= options["tablePrefix"   ] ;
	m_lastArgc		= 0	;
	m_lastArgv		= 0	;
	m_callback		= 0	;
}

/*  KBTestDriver							*/
/*  ~KBTestDriver: Destructor for TestDriver database connection class	*/
/*  (returns)	 :		:					*/

KBTestDriver::~KBTestDriver ()
{
	if (m_lastArgv != 0)
		delete	[] m_lastArgv ;
}

/*  KBTestDriver							*/
/*  doConnect	: Open connection to database				*/
/*  svInfo	: KBServerInfo *: Server information			*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriver::doConnect
	(	KBServerInfo	*
	)
{
	return	true	;
}

/*  KBTestDriver							*/
/*  qrySelect	: Open a select query					*/
/*  data	: bool		  : Query for data			*/
/*  select	: const QString&: Select query				*/
/*  (returns)	: KBSQLSelect * : Select query class or NULL on error	*/

KBSQLSelect
	*KBTestDriver::qrySelect
	(	bool		data,
		const QString	&select,
		bool
	)
{
	return	new KBTestDriverQrySelect (this, data, select) ;
}


/*  KBTestDriver							*/
/*  qryUpdate	: Open an update query					*/
/*  data	: bool		  : Query for data			*/
/*  update	: const QString & : Update query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KNQryUpdate *   : Update query class or NULL on error	*/

KBSQLUpdate
	*KBTestDriver::qryUpdate
	(	bool		data,
		const QString	&update,
		const QString	&tabName
	)
{
	return	new KBTestDriverQryUpdate (this, data, update, tabName) ;
}

/*  KBTestDriver							*/
/*  qryInsert	: Open an insert query					*/
/*  data	: bool		  : Query for data			*/
/*  insert	: const QString & : Insert query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KNQryInsert *   : Insert query class or NULL on error	*/

KBSQLInsert
	*KBTestDriver::qryInsert
	(	bool		data,
		const QString	&insert,
		const QString	&tabName
	)
{
	return	new KBTestDriverQryInsert (this, data, insert, tabName) ;
}

/*  KBTestDriver							*/
/*  qryDelete	: Open an delete query					*/
/*  data	: bool		  : Query for data			*/
/*  _delete	: const QString & : Delete query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KNQryDelete *   : Delete query class or NULL on error	*/

KBSQLDelete
	*KBTestDriver::qryDelete
	(	bool		data,
		const QString	&_delete,
		const QString	&tabName
	)
{
	return	new KBTestDriverQryDelete (this, data, _delete, tabName) ;
}

/*  KBTestDriver							*/
/*  command	: Execute arbitrary SQL					*/
/*  data	: bool		  : Querying for data			*/
/*  rawqry	: const QString & : Query text				*/
/*  nvals	: uint		  : Number of substitution values	*/
/*  values	: KBValue *	  : Substitution values			*/
/*  select	: KBSQLSelect **  : Return for result queries		*/
/*  (returns)	: bool		  : Success				*/

bool	KBTestDriver::command
	(	bool		,
		const QString	&,
		uint		,
		KBValue		*,
		KBSQLSelect	**
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"create",
			__ERRLOCN
		   )	;

	return	false	;
}

/*  KBTestDriver							*/
/*  listTypes	: Get list of types with information flags		*/
/*  (returns)	: QString	: List as bar-separated string		*/

QString	KBTestDriver::listTypes ()
{
	static	QString	typeList ;

	if (typeList.isNull())
	{
		typeList = "Primary Key,0|Foreign Key,0" ;

		for (uint idx = 0 ; idx < sizeof(typeMap)/sizeof(TestDriverTypeMap) ; idx += 1)
		{
			TestDriverTypeMap *m = &typeMap[idx] ;

			if ((m->flags & FF_NOCREATE) == 0)
				typeList += QString("|%1,%2").arg(m->mtype).arg(m->flags) ;
		}
	}

	return	typeList ;
}

/*  KBTestDriver							*/
/*  tableExists	: See if named table exists				*/
/*  table	: const QString & : Table name				*/
/*  exists	: bool &	  : True if table exists		*/
/*  (returns)	: bool		  : Success				*/

bool	KBTestDriver::tableExists
	(	const QString	&table,
		bool		&exists
	)
{

	const char *location	= getenv("TESTPATH") ;
        QString	   path		;

	if (location == 0)
		path	= locateDir("appdata", "test/dummy/TableA.xml") ;
	else	path	= location ;

	QString	name	= QString("%1/test/dummy/%2.xml").arg(path).arg(table) ;
	DPRINTF
	((	"KBTestDriver::tableExists: [%s]->[%s] pfx=[%s]\n",
		table      .latin1(),
		name       .latin1(),
		m_tabPrefix.latin1()
	))	;

	if (!m_tabPrefix.isEmpty())
		if (table.left(m_tabPrefix.length()) != m_tabPrefix)
		{	exists	= false	;
			return	true	;
		}

	exists = QFile(name).exists() ;
	return	 true	;
}

/*  KBTestDriver							*/
/*  doListTables: List tables in database				*/
/*  tabList	: KBTableDetailsList &					*/
/*				: Result list				*/
/*  type	: uint		: Type flags				*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriver::doListTables
	(	KBTableDetailsList	&tabList,
		uint
	)
{
	const char *location	= getenv("TESTPATH") ;
        QString	   path		;

	if (location == 0)
		path	= locateDir("appdata", "test/dummy/TableA.xml") ;
	else	path	= location ;

	DPRINTF
	((	"KBTestDriver::doListTables: l=[%s] p=[%s]\n",
		location,
		path.latin1()
	))	;

	QDir dir (path + "test/dummy", "*.xml") ;
	for  (uint idx = 0 ; idx < dir.count() ; idx += 1)
	{
		QString	name = dir[idx] ;
		name = name.left(name.length() - 4) ;

		DPRINTF
		((	"KBTestDriver::doListTables: t=[%s] p=[%s]\n",
			name       .latin1(),
			m_tabPrefix.latin1()
		))	;

		if (!m_tabPrefix.isEmpty())
			if (name.left(m_tabPrefix.length()) != m_tabPrefix)
				continue ;

		tabList.append (KBTableDetails (name, KB::IsTable, 0xff)) ;
	}

	return	true ;
}

/*  KBTestDriver							*/
/*  doListFields: List fields in table					*/
/*  tabSpec	: KBTableSpec &	: Table specification			*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriver::doListFields
	(	KBTableSpec	&tabSpec
	)
{
	KBDomDocument	doc	;
	if (!doc.loadFile (tabSpec.m_name + ".xml", "test/dummy", getenv("TESTPATH")))
	{
		m_lError = doc.lastError() ;
		return	false	;
	}

	uint	colno	= 0	;

	DOMITER_BEGIN(doc.documentElement(), "column", column)
	{
		QString	flagStr	= column.attribute("flags") ;
		uint	itype	= column.attribute("itype"    ).toInt() ;
		int	length	= column.attribute("length"   ).toInt() ;
		int	prec	= column.attribute("precision").toInt() ;

		uint	flags	= 0 ;
		for (uint idx = 0 ; idx < flagStr.length() ; idx += 1)
			switch (flagStr.at(idx).latin1())
			{
				case 'P' : flags |= KBFieldSpec::Primary  ;
					   flags |= KBFieldSpec::Unique   ; break ;
				case 'N' : flags |= KBFieldSpec::NotNull  ; break ;
				case 'U' : flags |= KBFieldSpec::Unique   ; break ;
				case 'S' : flags |= KBFieldSpec::Serial   ; break ;
				case 'I' : flags |= KBFieldSpec::Indexed  ; break ;
				case 'R' : flags |= KBFieldSpec::ReadOnly ; break ;
				case 'A' : flags |= KBFieldSpec::InsAvail ; break ;
				default	 : 				    break ;
			}

		KBFieldSpec *fSpec = new KBFieldSpec
				     (		colno,
						column.attribute("name"),
						column.attribute("mtype"),
						(KB::IType)itype,
						flags,
						length,
						prec
				     ) ;

		fSpec->m_dbType	   = new KBType
				     (		"Test",
						(KB::IType)itype,
						length,
						prec,
						(((flags & KBFieldSpec::NotNull) == 0) || ((flags & KBFieldSpec::Serial) != 0))
				     ) ;

		tabSpec.m_fldList.append (fSpec) ;

		if (((flags & KBFieldSpec::Primary) != 0) && ((flags & KBFieldSpec::Serial) != 0))
			tabSpec.m_prefKey = colno ;
			
		colno	+= 1 ;
	}
	DOMITER_END

	return	true ;
}

/*  KBTestDriver							*/
/*  doCreateTable: Create a new table					*/
/*  tabSpec	 : KBTableSpec &: Table specification			*/
/*  assoc	 : bool		: Create associated objects		*/
/*  best	 : bool		: Use best match			*/
/*  (returns)	 : bool		: Success				*/

bool	KBTestDriver::doCreateTable
	(	KBTableSpec	&,
		bool		,
		bool
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"create",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBTestDriver							*/
/*  doRenameTable: Rename a  table					*/
/*  oldName	 : cchar *	: Current table name			*/
/*  newName	 : cchar *	: New table name			*/
/*  assoc	 : bool		: Rename associated stuff - none here	*/
/*  (returns)	 : bool		: Success				*/

bool	KBTestDriver::doRenameTable
	(	cchar	*,
		cchar	*,
		bool
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"rename",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBTestDriver							*/
/*  doDropTable	: Drop a table						*/
/*  table	: cchar *	: Table name				*/
/*  assoc	: bool		: Drop associated stuff - none here	*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriver::doDropTable
	(	cchar	*,
		bool
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"drop",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBTestDriver							*/
/*  mapExpression: Map expressions with double-quotes			*/
/*  expr	 : const QString & : Expression				*/
/*  (returns)	 : QString	   : Mapped expression			*/

QString	KBTestDriver::mapExpression
	(	const QString	&expr
	)
{
	static	QString	spec ("_") ;

	return	m_mapExpressions ?
			doMapExpression (expr, "\"", "\"", spec) :
			expr ;
}

void	KBTestDriver::setCallback
	(	KBTestDriverCallBack	callBack
	)
{
//	fprintf
//	(	stderr,
//		"KBTestDriver::setCallback: %p: set %p\n",
//		(void *)this,
//		(void *)callBack
//	)	;

	m_callback = callBack ;
}

void	KBTestDriver::setLastSQL
	(	const QString		&lastSQL,
		const QString		&lastTable,
		int			lastArgc,
		const KBValue		*lastArgv
	)
{
	if (!lastSQL.isNull  ()) m_lastSQL   = lastSQL   ;
	if (!lastTable.isNull()) m_lastTable = lastTable ;
	m_lastArgc = lastArgc	;

	if ((lastArgv != 0) && (lastArgc > 0))
	{
		m_lastArgv = new KBValue[lastArgc] ;
		for (int idx = 0 ; idx < lastArgc ; idx += 1)
			m_lastArgv[idx] = lastArgv[idx] ;
	}
	else	m_lastArgv = 0 ;

//	fprintf
//	(	stderr,
//		"KBTestDriver::setLastSQL: %p: %s\n",
//		(void *)m_callback,
//		m_lastSQL.latin1()
//	)	;

	if (m_callback != 0)
		(*m_callback) (m_lastSQL, m_lastTable, m_lastArgc, m_lastArgv) ;

}

/*  KBTestDriver							*/
/*  getSyntax	: Get text for syntax element				*/
/*  result	: QString &	: Return resule string			*/
/*  syntax	: Syntax	: Element				*/
/*  ...		: ...		: Arguments				*/
/*  (returns)	: QString	: Text					*/

bool	KBTestDriver::getSyntax
	(	QString			&result,
		KBServer::Syntax	syntax,
		...
	)
{
	va_list	 ap ;
	va_start (ap, syntax) ;

	switch (syntax)
	{
		case Limit :
			{
				int	limit	= va_arg(ap, int) ;
				int	offset	= va_arg(ap, int) ;

				if (limit  < 0) limit  = 0x7fffffff ;
				if (offset < 0) offset = 0	    ;
 
				result	= QString(" limit %1 offset %2 ").arg(limit).arg(offset) ;
			}
			return	true	;

		default	:
			break	;
	}

	m_lError = KBError
		   (	KBError::Error,
			QString(TR("Driver does not support %1")).arg(syntaxToText(syntax)),
			QString::null,
			__ERRLOCN
		   )	;
	return	false	;
}



/*  ------------------------------------------------------------------- */

/*  KBTestDriverQrySelect						*/
/*  KBTestDriverQrySelect						*/
/*		: Constructor for select query object			*/
/*  server	: KBTestDriver *  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  query	: const QString & : Select query			*/
/*  (returns)	: KBTestDriverQrySelect :				*/

KBTestDriverQrySelect::KBTestDriverQrySelect
	(	KBTestDriver		*server,
		bool			data,
		const QString		&query
	)	
	:
	KBSQLSelect	(server, data, query),
	m_server	(server)
{
	m_query		= query	;
	m_nRows		= 0	;
	m_nFields	= 0	;
	m_crow	 	= 0	;
	m_lengths	= 0	;
	m_table		= QString::null	;
}

/*  KBTestDriverQrySelect						*/
/*  ~KBTestDriverQrySelect						*/
/*		: Destructor for select query object			*/
/*  (returns)	:		:					*/

KBTestDriverQrySelect::~KBTestDriverQrySelect ()
{
}

/*  KBTestDriverQrySelect						*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriverQrySelect::execute
	(	uint		nvals,
		const KBValue	*values
	)
{
	KBBaseSelect	select	;
	KBDataBuffer	exeSQL	;

	m_server->setLastSQL (m_query, QString::null, nvals, values) ;

	if (!m_server->subPlaceList
		(	m_query,
			nvals,
			values,
			exeSQL,
			m_codec,
			m_lError
		))
		return false	;

	DPRINTF
	((	"KBTestDriverQrySelect::execute: [%s]\n",
		QString(exeSQL.data()).latin1()
	))	;

	if (!select.parse (QString(exeSQL.data()), 0))
	{
		m_lError = select.lastError() ;
		return	 false	;
	}

	m_table	= select.tableList()[0].tableName() ;

	const QValueList<KBBaseQueryFetch> &fetchList = select.fetchList() ;
	for (uint idx = 0 ; idx < fetchList.count() ; idx += 1)
		m_fetch.append (fetchList[idx].expr()) ;

	bool	random	= select.distinct() ;

	KBDomDocument	doc	;
	if (!doc.loadFile (m_table + ".xml", "test/dummy", getenv("TESTPATH")))
	{
		m_lError = doc.lastError() ;
		return	false	;
	}

	QDomNodeList nl	= doc.documentElement().elementsByTagName("record") ;

	if (random)
	{
		static	bool	r_first	= true ;

		if (r_first)
		{
			QTime t	= QTime::currentTime() ;
			srand (t.hour() * 12 + t.minute() * 60 + t.second() * 60) ;
			r_first	= false ;
		}
	}

	m_nFields	= m_fetch.count() ;


	if (m_types == 0)
	{
		cchar	*sep	= "" ;
		m_exprSkel	= "global ntos;\nglobal ston;\npublic f (" ;
		DOMITER_BEGIN(doc.documentElement(),"column",column)
		{
			QString	c = column.attribute("name") ;
			m_exprSkel	+= sep	;
			m_exprSkel	+= c	;
			sep		 = ", "	;
			m_columns .append (c)	;
			m_colTypes.append (column.attribute("itype").toInt()) ;
		}
		DOMITER_END
		m_exprSkel += ")\n{\n\treturn %1 ;\n}\n"	;

		m_types	= new KBType *[m_nFields] ;
		for (uint idx = 0 ; idx < m_nFields ; idx += 1)
		{
			m_types[idx] = 0 ;

			DOMITER_BEGIN(doc.documentElement(),"column",column)
			{
				if (m_fetch[idx] == column.attribute("name"))
				{
					m_types[idx] = new KBType
							   (	"Test",
								(KB::IType)column.attribute("itype").toInt(),
								column.attribute("length"   ).toInt(),
								column.attribute("precision").toInt(),
								true
							   )	;
					break	;
				}
			}
			DOMITER_END

			if (m_types[idx] == 0)
				m_types[idx] = new KBType
						   (	"Test",
						   	KB::ITString,
						   	255,
						   	0,
						   	true
						   )	;
		}
	}

	for (uint idx = 0 ; idx < nl.count() ; idx += 1)
	{
		QDomElement record = nl.item(idx).toElement() ;

		if (record.isNull())
			continue ;

		if (random && (rand() % 2 != 0))
			continue ;

		KBValue	result 	;
		bool	skip	= false ;
		uint	place	= 0	;

		const QValueList<KBBaseQueryExpr> &whereList = select.whereList() ;
		for (uint idx = 0 ; idx < whereList.count() ; idx += 1)
			if (evaluate
				(	m_table,
					m_exprSkel,
					whereList[idx].expr(0, place),
					m_columns,
					m_colTypes,
					record,
					result
				))
				if (!result.isTrue())
				{
					skip	= true	;
					break	;
				}
		if (skip)
			continue ;

		m_rows.append (record) ;

		DPRINTF
		((	"KBTestDriverQrySelect::execute: l=%d r=%d\n",
			select.limit(),
			m_rows.count()
		))	;

		if (select.limit() >= 0)
			if ((int)m_rows.count() >= select.limit())
				break	;

	}

	m_nRows		= m_rows .count() ;

	m_server->printQuery (m_query, m_tag, 0, 0, true) ;
	return	true	;
}

/*  KBTestDriverQrySelect						*/
/*  getField	: Get a specified field from the query results		*/
/*  qrow	: uint		: Row number				*/
/*  qcol	: uint		: Column number				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBTestDriverQrySelect::getField
	(	uint		qrow,
		uint		qcol,
		KBValue::VTrans
	)
{
	/* First check that the request is within the range of rows	*/
	/* and fields returned by the query.				*/
	if ((int)qrow >= m_nRows  ) return KBValue() ;
	if (     qcol >= m_nFields) return KBValue() ;

	QDomElement record = m_rows[qrow] ;
	QString	    value  = record.attribute(m_fetch[qcol]) ;

	if (m_columns.findIndex(m_fetch[qcol]) >= 0)
		return	value.isNull() ? KBValue() : KBValue(value, m_types[qcol]) ;


	KBValue	    result ;

	if (evaluate
		(	m_table,
			m_exprSkel,
			m_fetch[qcol],
			m_columns,
			m_colTypes,
			record,
			result
		))
		return	result	;

	return	value.isNull() ? KBValue() : KBValue(value, m_types[qcol]) ;
}

/*  KBTestDriverQrySelect						*/
/*  getFieldname: Get a specified field name from the query results	*/
/*  qcol	: uint		: Column number				*/
/*  (returns)	: QString	: Name					*/

QString	KBTestDriverQrySelect::getFieldName
	(	uint	qcol
	)
{
	/* First check that the request is within the range of rows	*/
	/* and fields returned by the query.				*/
	if (qcol >= m_nFields) return QString() ;

	return	QString::null ;
}


/*  KBTestDriverQryUpdate						*/
/*  KBTestDriverQryUpdate						*/
/*		: Constructor for update query object			*/
/*  server	: KBTestDriver *  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  query	: const QString & : Update query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KBTestDriverQryUpdate:				*/

KBTestDriverQryUpdate::KBTestDriverQryUpdate
	(	KBTestDriver		*server,
		bool		data,
		const QString	&query,
		const QString	&tabName
	)	
	:
	KBSQLUpdate (server, data, query, tabName),
	m_server    (server)
{
	m_nRows	= 0 ;
	m_query	= query		;
	m_table	= tabName	;
}

/*  KBTestDriverQryUpdate						*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriverQryUpdate::execute
	(	uint		nvals,
		const KBValue	*values
	)
{
	m_server->setLastSQL (m_query, m_table, nvals, values) ;

	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"update",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBTestDriverQryUpdate						*/
/*  ~KBTestDriverQryUpdate						*/
/*		: Destructor for update query object			*/
/*  (returns)	:		:					*/

KBTestDriverQryUpdate::~KBTestDriverQryUpdate ()
{
}


/*  KBTestDriverQryInsert						*/
/*  KBTestDriverQryInsert						*/
/*		: Constructor for insert query object			*/
/*  server	: KBTestDriver *  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  tabName	: const QString & : Table being updated			*/
/*  query	: const QString & : Insert query			*/
/*  (returns)	: KBTestDriverQryInsert:				*/

KBTestDriverQryInsert::KBTestDriverQryInsert
	(	KBTestDriver		*server,
		bool			data,
		const QString		&query,
		const QString		&tabName
	)	
	:
	KBSQLInsert (server, data, query, tabName),
	m_server    (server)
{
	m_nRows	= 0 ;
	m_query	= query		;
	m_table	= tabName	;
}

/*  KBTestDriverQryInsert						*/
/*  ~KBTestDriverQryInsert						*/
/*		: Destructor for insert query object			*/
/*  (returns)	:		:					*/

KBTestDriverQryInsert::~KBTestDriverQryInsert ()
{
}

/*  KBTestDriverQryInsert						*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriverQryInsert::execute
	(	uint		nvals,
		const KBValue	*values
	)
{
	fprintf
	(	stderr,
		"KBTestDriverQryInsert::execute: called\n"
	)	;

	m_server->setLastSQL (m_query, m_table, nvals, values) ;

	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"insert",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBTestDriverQryInsert						*/
/*  getNewKey	: Get new insert key					*/
/*  primary	: const QString & : Key column name			*/
/*  _newKey	: KBValue &	  : New key				*/
/*  prior	: bool		  : Pre-insert call			*/
/*  (returns)	: bool		  : Success				*/

bool	KBTestDriverQryInsert::getNewKey
	(	const QString	&,
		KBValue		&key,
		bool		
	)
{
	fprintf
	(	stderr,
		"KBTestDriverQryInsert::getNewKey: called\n"
	)	;
	key	= 99999	;
	return	true	;
}

/*  KBTestDriverQryDelete						*/
/*  KBTestDriverQryDelete						*/
/*		: Constructor for delete query object			*/
/*  server	: KBTestDriver *  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  query	: const QString & : Delete query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KBTestDriverQryDelete:				*/

KBTestDriverQryDelete::KBTestDriverQryDelete
	(	KBTestDriver		*server,
		bool		data,
		const QString	&query,
		const QString	&tabName
	)	
	:
	KBSQLDelete (server, data, query, tabName),
	m_server    (server)
{
	m_nRows	= 0 ;
	m_query	= query		;
	m_table	= tabName	;
}

/*  KBTestDriverQryDelete						*/
/*  ~KBTestDriverQryDelete						*/
/*		: Destructor for delete query object			*/
/*  (returns)	:		:					*/

KBTestDriverQryDelete::~KBTestDriverQryDelete ()
{
}

/*  KBTestDriverQryDelete						*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriverQryDelete::execute
	(	uint		nvals,
		const KBValue	*values
	)
{
	m_server->setLastSQL (m_query, m_table, nvals, values) ;

	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"delete",
			__ERRLOCN
		   )	;
	return	false	;
}


KBFACTORY
(	KBTestDriverFactory,
	"driver_dummy"
)

KBFACTORYIMPL
(	KBTestDriverFactory,
	driver_dummy
)

QObject	*KBTestDriverFactory::create
	(	QObject		  *parent,
		cchar		  *object,
		cchar		  *,
		const QStringList &
	)
{
	extern	void	el_initialize (uint, uint, bool) ;
	el_initialize (1024 * 64, 4096, false) ;

	if (dIdentToType.count() == 0)
		for (uint idx = 0 ; idx < sizeof(typeMap)/sizeof(TestDriverTypeMap) ; idx += 1)
			if (typeMap[idx].ident != FIELD_TYPE_UNSET)
			{
				TestDriverTypeMap *m = &typeMap[idx] ;
				dIdentToType.insert (m->ident, m) ;
			}

	if ((parent != 0) && !parent->inherits ("QWidget"))
	{
		fprintf	(stderr, "KBTestDriverFactory: parent does not inherit QWidget\n") ;
		return	0  ;
	}

	if (strcmp (object, "driver"  ) == 0) return new KBTestDriver () ;

	if (strcmp (object, "advanced") == 0) return 0 ;

	return	0 ;
}

QString	KBTestDriverFactory::ident ()
{
	return	"Dummty test driver" ;
}


EXPORT_API
	KBServer*createTestDriver
	(	const QMap<QString,QString>	&options
	)
{
	return	new KBTestDriver (options) ;
}
