/***************************************************************************
    file	         : testdriver.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#ifndef		__TESTDRIVER_H
#define		__TESTDRIVER_H

#include	"kb_database.h"

#if		defined(__WIN32__)
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		EXPORT_API
#endif

class	KBServerInfo	;
class	KBTableSpec	;
class	KBSQLSelect	;
class	KBSQLUpdate	;
class	KBSQLInsert	;
class	KBSQLDelete	;
class	KBValue		;


typedef	void	(*KBTestDriverCallBack)
		(	const QString		&,
			const QString		&,
			int			,
			const KBValue		*
		)	;

/*  KBTestDriver							*/
/*  ------------							*/
/*  Implementation class for insertface to the TestDriver database	*/

class	EXPORT_API	KBTestDriver : public KBServer
{
	bool			m_mapExpressions	;
	QString			m_tabPrefix		;
	KBTestDriverCallBack	m_callback		;

	bool			doListTables 	(KBTableDetailsList &, uint) ;

	virtual	bool	 	doConnect    	(KBServerInfo  *) ;
	virtual	bool		doListFields 	(KBTableSpec   &) ;
	virtual	bool		doCreateTable	(KBTableSpec   &,  bool, bool = false) ;
	virtual	bool		doRenameTable	(cchar *, cchar *, bool) ;
	virtual	bool		doDropTable  	(cchar *,	   bool) ;

public	:

	KBTestDriver	() ;
	KBTestDriver	(const QMap<QString,QString> &) ;
virtual~KBTestDriver	() ;


	virtual	KBSQLSelect	*qrySelect 	(bool, const QString &, bool) ;
	virtual	KBSQLUpdate	*qryUpdate 	(bool, const QString &, const QString &) ;
	virtual	KBSQLInsert	*qryInsert 	(bool, const QString &, const QString &) ;
	virtual	KBSQLDelete	*qryDelete 	(bool, const QString &, const QString &) ;

	virtual	bool		command		(bool, const QString &, uint, KBValue *, KBSQLSelect ** = 0) ;

	virtual	QString		mapExpression	(const QString &) ;
	virtual	QString		listTypes	() ;
	virtual	bool		tableExists	(const QString &, bool &) ;

	virtual	bool		getSyntax	(QString &, KBServer::Syntax, ...) ;

	void			setLastSQL	(const QString &, const QString &, int, const KBValue *) ;
	void			setCallback	(KBTestDriverCallBack) ;

	QString			m_lastSQL	;
	QString			m_lastTable	;
	int			m_lastArgc	;
	KBValue			*m_lastArgv	;
}	;


#endif		// __TESTDRIVER_H
