/***************************************************************************
    file	         : forms.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>
#include	<errno.h>
#include	<unistd.h>

#include	<qfile.h>
#include	<qmainwindow.h>
#include	<qapplication.h>

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_display.h"
#include	"kb_form.h"



#if		__KB_KDE
#include	<kapp.h>
#include	<klocale.h>
#include	<kaboutdata.h>
#include	<kcmdlineargs.h>
#endif

#define		SKELL	"forms"
#define		SKELU	"Forms"

static	FILE	*errfile	;


int	main
	(	int	argc,
		char	*argv[]
	)
{
	cchar	*sstde	= getenv ("REKALL_SHOWSTDERR") ;
	cchar	*fname	= 0	;
	bool	exec	= false	;

	if ((sstde == 0) || (strcmp(sstde, "YES") != 0))
	{
		freopen	("/dev/null", "w", stderr) ;
		errfile	= fopen ("/dev/tty",  "w") ;
	}
	else	errfile	= stderr ;


	/* Local argument processing first, means we can do the same	*/
	/* for both KDE and QT-Only builds.				*/
	int	largc	= argc - 1 ;
	char	**largv	= argv + 1 ;

	while ((largc > 0) && (qstrncmp (largv[0], "--", 2) == 0))
	{
		if (qstrcmp  (largv[0], "--exec") == 0)
		{
			exec	= true	;
			largv	+= 1	;
			largc	-= 1	;
			continue	;
		}

		if (qstrncmp (largv[0], "--style=", 8) == 0)
		{
			largv	+= 1	;
			largc	-= 1	;
			continue	;
		}

		fprintf	(errfile, "usage: " SKELL " [--exec] [--style=style] formfile\n") ;
		exit	(1) ;
	}

	if (largc != 1)
	{
		fprintf	(errfile, "usage: " SKELL " [--exec] [--style=style] formfile\n") ;
		exit	(1) ;
	}

	fname	= largv[0]  ;


#if	__KB_KDE
	static	const	KCmdLineOptions	options[] =
	{
	{	"exec",			I18N_NOOP("Execute"		),	0	},
	{	"+[form]",		I18N_NOOP("Form"		),	0	},
	{	0,		0,						0	}
	}	;

//	char *dummy[2]	;
//
//	dummy[0] = (char *)"forms" ;
//	dummy[1] = 0	;

	KAboutData about ("rekall",
			  I18N_NOOP("Rekall" SKELU "Test"),
			  "0.0.0",
			  I18N_NOOP("Rekall: " SKELU " test harness"),
			  KAboutData::License_Unknown,
			  "(c) 2000,2001,2002 by theKompany.com",
			  0,
			  "http://www.rekall.a-i-s.co.uk",
			  "mike@quaking.demon.co.uk"
			 ) ;
	about.addAuthor  ("Mike Richardson", 0, "mike@quaking.demon.co.uk") ; 
	about.addAuthor  ("John Dean", 	     0, "john@rygannon.com") ; 

	KCmdLineArgs::init		(argc, argv, &about) ;
	KCmdLineArgs::addCmdLineOptions	(options) ;

	KApplication			app ;
	KCmdLineArgs::parsedArgs	()  ;

#endif
#if	__KB_TKC
	QApplication	app	(argc, argv)  ;
#endif

	extern	void    registerAllNodes() ;
	registerAllNodes   () ;

	KBError		error	;
	QFile	formDef	(fname)	;
	if (!formDef.open(IO_ReadOnly))
	{
		fprintf	(errfile, "forms: %s: cannot open\n", fname) ;
		exit	(1) ;
	}

	QByteArray formText = formDef.readAll() ;
	KBLocation formLocn
		   (	0,
			"form",
			KBLocation::m_pFile,
			fname
		   )	;
	KBForm	   *form	;

	extern	KBForm	*KBOpenFormText (const KBLocation &, QByteArray &, KBError &) ;

	if ((form = KBOpenFormText (formLocn, formText, error)) == 0)
	{
		error.DISPLAY() ;
		exit	(1)	;
	}

	QMainWindow	window	;

	QSize	size	;
	form->showDesign (&window, size) ;
	QWidget	*top	= form->getDisplay()->getTopWidget() ;

	window.setCentralWidget (top)	;

	window .show     ()		;
	window .resize   (size.width(), size.height()) ;

	top   ->show	 () ;
	top   ->resize	 (size.width(), size.height()) ;

	extern	void	printWidgetTree
		(	QWidget		*widget,
			uint		indent,
			FILE		*out
		)	;

	printWidgetTree (top, 0, stdout) ;

	if (exec)
	{
		QObject::connect
		(	&app,
			SIGNAL	(lastWindowClosed()),
			&app,
			SLOT	(quit())
		)	;
		app.exec() ;
	}

	return	0 ;
}
